Package: vctrs
Title: Vector Helpers
Version: 0.6.5
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Lionel", "Henry", , "lionel@posit.co", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("data.table team", role = "cph",
           comment = "Radix sort based on data.table's forder() and their contribution to R's order()"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Defines new notions of prototype and size that are used to
    provide tools for consistent and well-founded type-coercion and
    size-recycling, and are in turn connected to ideas of type- and
    size-stability useful for analysing function interfaces.
License: MIT + file LICENSE
URL: https://vctrs.r-lib.org/, https://github.com/r-lib/vctrs
BugReports: https://github.com/r-lib/vctrs/issues
Depends: R (>= 3.5.0)
Imports: cli (>= 3.4.0), glue, lifecycle (>= 1.0.3), rlang (>= 1.1.0)
Suggests: bit64, covr, crayon, dplyr (>= 0.8.5), generics, knitr,
        pillar (>= 1.4.4), pkgdown (>= 2.0.1), rmarkdown, testthat (>=
        3.0.0), tibble (>= 3.1.3), waldo (>= 0.2.0), withr, xml2,
        zeallot
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2023-12-01 16:27:12 UTC; davis
Author: Hadley Wickham [aut],
  Lionel Henry [aut],
  Davis Vaughan [aut, cre],
  data.table team [cph] (Radix sort based on data.table's forder() and
    their contribution to R's order()),
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2023-12-01 23:50:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-17 17:37:42 UTC; unix

Package: vctrs
Title: Vector Helpers
Version: 0.6.5
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Lionel", "Henry", , "lionel@posit.co", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("data.table team", role = "cph",
           comment = "Radix sort based on data.table's forder() and their contribution to R's order()"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Defines new notions of prototype and size that are used to
    provide tools for consistent and well-founded type-coercion and
    size-recycling, and are in turn connected to ideas of type- and
    size-stability useful for analysing function interfaces.
License: MIT + file LICENSE
URL: https://vctrs.r-lib.org/, https://github.com/r-lib/vctrs
BugReports: https://github.com/r-lib/vctrs/issues
Depends: R (>= 3.5.0)
Imports: cli (>= 3.4.0), glue, lifecycle (>= 1.0.3), rlang (>= 1.1.0)
Suggests: bit64, covr, crayon, dplyr (>= 0.8.5), generics, knitr,
        pillar (>= 1.4.4), pkgdown (>= 2.0.1), rmarkdown, testthat (>=
        3.0.0), tibble (>= 3.1.3), waldo (>= 0.2.0), withr, xml2,
        zeallot
VignetteBuilder: knitr
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Encoding: UTF-8
Language: en-GB
RoxygenNote: 7.2.3
NeedsCompilation: yes
Packaged: 2023-12-01 16:27:12 UTC; davis
Author: Hadley Wickham [aut],
  Lionel Henry [aut],
  Davis Vaughan [aut, cre],
  data.table team [cph] (Radix sort based on data.table's forder() and
    their contribution to R's order()),
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2023-12-01 23:50:02 UTC
Built: R 4.4.0; i386-portbld-midnightbsd4.0; 2026-05-11 17:18:44 UTC; unix

Package: vctrs
Title: Vector Helpers
Version: 0.7.3
Authors@R: c(
    person("Hadley", "Wickham", , "hadley@posit.co", role = "aut"),
    person("Lionel", "Henry", , "lionel@posit.co", role = "aut"),
    person("Davis", "Vaughan", , "davis@posit.co", role = c("aut", "cre")),
    person("data.table team", role = "cph",
           comment = "Radix sort based on data.table's forder() and their contribution to R's order()"),
    person("Posit Software, PBC", role = c("cph", "fnd"))
  )
Description: Defines new notions of prototype and size that are used to
    provide tools for consistent and well-founded type-coercion and
    size-recycling, and are in turn connected to ideas of type- and
    size-stability useful for analysing function interfaces.
License: MIT + file LICENSE
URL: https://vctrs.r-lib.org/, https://github.com/r-lib/vctrs
BugReports: https://github.com/r-lib/vctrs/issues
Depends: R (>= 4.0.0)
Imports: cli (>= 3.4.0), glue, lifecycle (>= 1.0.3), rlang (>= 1.1.7)
Suggests: bit64, covr, crayon, dplyr (>= 0.8.5), generics, knitr,
        pillar (>= 1.4.4), pkgdown (>= 2.0.1), rmarkdown, testthat (>=
        3.0.0), tibble (>= 3.1.3), waldo (>= 0.2.0), withr, xml2,
        zeallot
VignetteBuilder: knitr
Config/build/compilation-database: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Encoding: UTF-8
KeepSource: true
Language: en-GB
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-10 14:27:17 UTC; davis
Author: Hadley Wickham [aut],
  Lionel Henry [aut],
  Davis Vaughan [aut, cre],
  data.table team [cph] (Radix sort based on data.table's forder() and
    their contribution to R's order()),
  Posit Software, PBC [cph, fnd]
Maintainer: Davis Vaughan <davis@posit.co>
Repository: CRAN
Date/Publication: 2026-04-11 06:20:03 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-11 07:11:36 UTC; unix
Archs: vctrs.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method("!",vctrs_vctr)
S3method("!=",vctrs_vctr)
S3method("$",vctrs_list_of)
S3method("$",vctrs_rcrd)
S3method("$",vctrs_sclr)
S3method("$",vctrs_vctr)
S3method("$<-",vctrs_list_of)
S3method("$<-",vctrs_rcrd)
S3method("$<-",vctrs_sclr)
S3method("$<-",vctrs_vctr)
S3method("%%",vctrs_vctr)
S3method("%/%",vctrs_vctr)
S3method("&",vctrs_vctr)
S3method("*",vctrs_vctr)
S3method("+",vctrs_vctr)
S3method("-",vctrs_vctr)
S3method("/",vctrs_vctr)
S3method("<",vctrs_vctr)
S3method("<=",vctrs_vctr)
S3method("==",vctrs_vctr)
S3method(">",vctrs_vctr)
S3method(">=",vctrs_vctr)
S3method("[",vctrs_rcrd)
S3method("[",vctrs_sclr)
S3method("[",vctrs_unspecified)
S3method("[",vctrs_vctr)
S3method("[<-",vctrs_list_of)
S3method("[<-",vctrs_rcrd)
S3method("[<-",vctrs_sclr)
S3method("[<-",vctrs_vctr)
S3method("[[",vctrs_list_of)
S3method("[[",vctrs_rcrd)
S3method("[[",vctrs_sclr)
S3method("[[",vctrs_vctr)
S3method("[[<-",vctrs_list_of)
S3method("[[<-",vctrs_rcrd)
S3method("[[<-",vctrs_sclr)
S3method("[[<-",vctrs_vctr)
S3method("^",vctrs_vctr)
S3method("dim<-",vctrs_sclr)
S3method("dim<-",vctrs_vctr)
S3method("dimnames<-",vctrs_sclr)
S3method("dimnames<-",vctrs_vctr)
S3method("is.na<-",vctrs_sclr)
S3method("is.na<-",vctrs_vctr)
S3method("length<-",vctrs_rcrd)
S3method("length<-",vctrs_vctr)
S3method("levels<-",vctrs_sclr)
S3method("levels<-",vctrs_vctr)
S3method("names<-",vctrs_rcrd)
S3method("names<-",vctrs_sclr)
S3method("names<-",vctrs_vctr)
S3method("|",vctrs_vctr)
S3method(Complex,vctrs_sclr)
S3method(Math,vctrs_sclr)
S3method(Math,vctrs_vctr)
S3method(Ops,vctrs_sclr)
S3method(Summary,vctrs_sclr)
S3method(Summary,vctrs_vctr)
S3method(anyDuplicated,vctrs_sclr)
S3method(anyDuplicated,vctrs_vctr)
S3method(anyNA,vctrs_vctr)
S3method(as.Date,vctrs_sclr)
S3method(as.Date,vctrs_vctr)
S3method(as.POSIXct,vctrs_sclr)
S3method(as.POSIXct,vctrs_vctr)
S3method(as.POSIXlt,vctrs_vctr)
S3method(as.character,vctrs_list_of)
S3method(as.character,vctrs_sclr)
S3method(as.character,vctrs_vctr)
S3method(as.data.frame,vctrs_sclr)
S3method(as.data.frame,vctrs_vctr)
S3method(as.double,vctrs_sclr)
S3method(as.double,vctrs_vctr)
S3method(as.integer,vctrs_sclr)
S3method(as.integer,vctrs_vctr)
S3method(as.list,vctrs_list_of)
S3method(as.list,vctrs_sclr)
S3method(as.list,vctrs_vctr)
S3method(as.logical,vctrs_sclr)
S3method(as.logical,vctrs_vctr)
S3method(as_list_of,list)
S3method(as_list_of,vctrs_list_of)
S3method(c,vctrs_sclr)
S3method(c,vctrs_vctr)
S3method(can_fall_back,"vctrs:::common_class_fallback")
S3method(can_fall_back,data.frame)
S3method(can_fall_back,default)
S3method(can_fall_back,ts)
S3method(can_fall_back,vctrs_vctr)
S3method(cnd_body,vctrs_error_cast_lossy)
S3method(cnd_body,vctrs_error_combine_unmatched)
S3method(cnd_body,vctrs_error_incompatible_size)
S3method(cnd_body,vctrs_error_matches_incomplete)
S3method(cnd_body,vctrs_error_matches_multiple)
S3method(cnd_body,vctrs_error_matches_nothing)
S3method(cnd_body,vctrs_error_matches_relationship_many_to_one)
S3method(cnd_body,vctrs_error_matches_relationship_one_to_many)
S3method(cnd_body,vctrs_error_matches_relationship_one_to_one)
S3method(cnd_body,vctrs_error_matches_remaining)
S3method(cnd_body,vctrs_error_names_cannot_be_dot_dot)
S3method(cnd_body,vctrs_error_names_cannot_be_empty)
S3method(cnd_body,vctrs_error_names_must_be_unique)
S3method(cnd_body,vctrs_error_subscript_oob)
S3method(cnd_body,vctrs_error_subscript_type)
S3method(cnd_header,vctrs_error_cast_lossy)
S3method(cnd_header,vctrs_error_combine_unmatched)
S3method(cnd_header,vctrs_error_incompatible_size)
S3method(cnd_header,vctrs_error_matches_incomplete)
S3method(cnd_header,vctrs_error_matches_multiple)
S3method(cnd_header,vctrs_error_matches_nothing)
S3method(cnd_header,vctrs_error_matches_relationship_many_to_one)
S3method(cnd_header,vctrs_error_matches_relationship_one_to_many)
S3method(cnd_header,vctrs_error_matches_relationship_one_to_one)
S3method(cnd_header,vctrs_error_matches_remaining)
S3method(cnd_header,vctrs_error_names_cannot_be_dot_dot)
S3method(cnd_header,vctrs_error_names_cannot_be_empty)
S3method(cnd_header,vctrs_error_names_must_be_unique)
S3method(cnd_header,vctrs_error_subscript_oob)
S3method(cnd_header,vctrs_error_subscript_size)
S3method(cnd_header,vctrs_error_subscript_type)
S3method(diff,vctrs_vctr)
S3method(duplicated,vctrs_sclr)
S3method(duplicated,vctrs_vctr)
S3method(format,hidden)
S3method(format,vctrs_bytes)
S3method(format,vctrs_group_rle)
S3method(format,vctrs_list_of)
S3method(format,vctrs_rcrd)
S3method(format,vctrs_vctr)
S3method(is.finite,vctrs_vctr)
S3method(is.infinite,vctrs_vctr)
S3method(is.na,vctrs_vctr)
S3method(is.nan,vctrs_vctr)
S3method(length,vctrs_rcrd)
S3method(levels,vctrs_sclr)
S3method(levels,vctrs_vctr)
S3method(max,vctrs_vctr)
S3method(mean,vctrs_vctr)
S3method(median,vctrs_vctr)
S3method(min,vctrs_vctr)
S3method(na.exclude,vctrs_vctr)
S3method(na.fail,vctrs_vctr)
S3method(na.omit,vctrs_vctr)
S3method(names,vctrs_rcrd)
S3method(obj_print_data,default)
S3method(obj_print_data,vctrs_list_of)
S3method(obj_print_footer,default)
S3method(obj_print_header,default)
S3method(obj_print_header,vctrs_group_rle)
S3method(obj_str_data,default)
S3method(obj_str_data,vctrs_rcrd)
S3method(obj_str_footer,default)
S3method(obj_str_header,default)
S3method(print,vctrs_bytes)
S3method(print,vctrs_sclr)
S3method(print,vctrs_unspecified)
S3method(print,vctrs_vctr)
S3method(quantile,vctrs_vctr)
S3method(range,vctrs_vctr)
S3method(rep,vctrs_rcrd)
S3method(rep,vctrs_vctr)
S3method(str,vctrs_vctr)
S3method(summary,vctrs_sclr)
S3method(summary,vctrs_vctr)
S3method(t,vctrs_sclr)
S3method(t,vctrs_vctr)
S3method(unique,vctrs_sclr)
S3method(unique,vctrs_vctr)
S3method(vec_arith,Date)
S3method(vec_arith,POSIXct)
S3method(vec_arith,POSIXlt)
S3method(vec_arith,default)
S3method(vec_arith,difftime)
S3method(vec_arith,factor)
S3method(vec_arith,logical)
S3method(vec_arith,numeric)
S3method(vec_arith.Date,Date)
S3method(vec_arith.Date,POSIXct)
S3method(vec_arith.Date,POSIXlt)
S3method(vec_arith.Date,default)
S3method(vec_arith.Date,difftime)
S3method(vec_arith.Date,numeric)
S3method(vec_arith.POSIXct,Date)
S3method(vec_arith.POSIXct,POSIXct)
S3method(vec_arith.POSIXct,POSIXlt)
S3method(vec_arith.POSIXct,default)
S3method(vec_arith.POSIXct,difftime)
S3method(vec_arith.POSIXct,numeric)
S3method(vec_arith.POSIXlt,Date)
S3method(vec_arith.POSIXlt,POSIXct)
S3method(vec_arith.POSIXlt,POSIXlt)
S3method(vec_arith.POSIXlt,default)
S3method(vec_arith.POSIXlt,difftime)
S3method(vec_arith.POSIXlt,numeric)
S3method(vec_arith.difftime,Date)
S3method(vec_arith.difftime,MISSING)
S3method(vec_arith.difftime,POSIXct)
S3method(vec_arith.difftime,POSIXlt)
S3method(vec_arith.difftime,default)
S3method(vec_arith.difftime,difftime)
S3method(vec_arith.difftime,numeric)
S3method(vec_arith.logical,default)
S3method(vec_arith.logical,logical)
S3method(vec_arith.logical,numeric)
S3method(vec_arith.numeric,Date)
S3method(vec_arith.numeric,POSIXct)
S3method(vec_arith.numeric,POSIXlt)
S3method(vec_arith.numeric,default)
S3method(vec_arith.numeric,difftime)
S3method(vec_arith.numeric,logical)
S3method(vec_arith.numeric,numeric)
S3method(vec_cast,Date)
S3method(vec_cast,POSIXct)
S3method(vec_cast,POSIXlt)
S3method(vec_cast,character)
S3method(vec_cast,character.factor)
S3method(vec_cast,character.ordered)
S3method(vec_cast,complex)
S3method(vec_cast,data.frame)
S3method(vec_cast,data.frame.data.table)
S3method(vec_cast,data.table.data.frame)
S3method(vec_cast,data.table.data.table)
S3method(vec_cast,difftime)
S3method(vec_cast,double)
S3method(vec_cast,double.exclude)
S3method(vec_cast,double.omit)
S3method(vec_cast,exclude.double)
S3method(vec_cast,exclude.exclude)
S3method(vec_cast,exclude.integer)
S3method(vec_cast,factor)
S3method(vec_cast,factor.character)
S3method(vec_cast,factor.factor)
S3method(vec_cast,integer)
S3method(vec_cast,integer.exclude)
S3method(vec_cast,integer.omit)
S3method(vec_cast,integer64)
S3method(vec_cast,list)
S3method(vec_cast,list.vctrs_list_of)
S3method(vec_cast,logical)
S3method(vec_cast,omit.double)
S3method(vec_cast,omit.integer)
S3method(vec_cast,omit.omit)
S3method(vec_cast,ordered)
S3method(vec_cast,ordered.character)
S3method(vec_cast,ordered.ordered)
S3method(vec_cast,raw)
S3method(vec_cast,table.table)
S3method(vec_cast,vctrs_list_of)
S3method(vec_cast,vctrs_list_of.list)
S3method(vec_cast,vctrs_rcrd)
S3method(vec_cast,vctrs_rcrd.vctrs_rcrd)
S3method(vec_cast,vctrs_vctr)
S3method(vec_cast.Date,Date)
S3method(vec_cast.Date,POSIXct)
S3method(vec_cast.Date,POSIXlt)
S3method(vec_cast.POSIXct,Date)
S3method(vec_cast.POSIXct,POSIXct)
S3method(vec_cast.POSIXct,POSIXlt)
S3method(vec_cast.POSIXlt,Date)
S3method(vec_cast.POSIXlt,POSIXct)
S3method(vec_cast.POSIXlt,POSIXlt)
S3method(vec_cast.character,character)
S3method(vec_cast.complex,complex)
S3method(vec_cast.complex,double)
S3method(vec_cast.complex,integer)
S3method(vec_cast.complex,logical)
S3method(vec_cast.data.frame,data.frame)
S3method(vec_cast.difftime,difftime)
S3method(vec_cast.double,double)
S3method(vec_cast.double,integer)
S3method(vec_cast.double,integer64)
S3method(vec_cast.double,logical)
S3method(vec_cast.integer,double)
S3method(vec_cast.integer,integer)
S3method(vec_cast.integer,integer64)
S3method(vec_cast.integer,logical)
S3method(vec_cast.integer64,double)
S3method(vec_cast.integer64,integer)
S3method(vec_cast.integer64,integer64)
S3method(vec_cast.integer64,logical)
S3method(vec_cast.list,list)
S3method(vec_cast.logical,double)
S3method(vec_cast.logical,integer)
S3method(vec_cast.logical,integer64)
S3method(vec_cast.logical,logical)
S3method(vec_cast.raw,raw)
S3method(vec_cast.vctrs_list_of,vctrs_list_of)
S3method(vec_cbind_frame_ptype,default)
S3method(vec_cbind_frame_ptype,sf)
S3method(vec_math,Date)
S3method(vec_math,POSIXct)
S3method(vec_math,POSIXlt)
S3method(vec_math,default)
S3method(vec_math,factor)
S3method(vec_math,vctrs_rcrd)
S3method(vec_proxy,"vctrs:::common_class_fallback")
S3method(vec_proxy,AsIs)
S3method(vec_proxy,Date)
S3method(vec_proxy,POSIXct)
S3method(vec_proxy,POSIXlt)
S3method(vec_proxy,default)
S3method(vec_proxy,exclude)
S3method(vec_proxy,factor)
S3method(vec_proxy,numeric_version)
S3method(vec_proxy,omit)
S3method(vec_proxy,ordered)
S3method(vec_proxy,vctrs_list_of)
S3method(vec_proxy,vctrs_rcrd)
S3method(vec_proxy,vctrs_vctr)
S3method(vec_proxy_compare,AsIs)
S3method(vec_proxy_compare,POSIXlt)
S3method(vec_proxy_compare,array)
S3method(vec_proxy_compare,default)
S3method(vec_proxy_compare,list)
S3method(vec_proxy_compare,raw)
S3method(vec_proxy_equal,AsIs)
S3method(vec_proxy_equal,POSIXlt)
S3method(vec_proxy_equal,array)
S3method(vec_proxy_equal,default)
S3method(vec_proxy_equal,integer64)
S3method(vec_proxy_equal,numeric_version)
S3method(vec_proxy_order,AsIs)
S3method(vec_proxy_order,array)
S3method(vec_proxy_order,default)
S3method(vec_proxy_order,list)
S3method(vec_proxy_order,raw)
S3method(vec_ptype,POSIXlt)
S3method(vec_ptype2,AsIs)
S3method(vec_ptype2,Date)
S3method(vec_ptype2,POSIXct)
S3method(vec_ptype2,POSIXlt)
S3method(vec_ptype2,character)
S3method(vec_ptype2,character.factor)
S3method(vec_ptype2,character.ordered)
S3method(vec_ptype2,complex)
S3method(vec_ptype2,data.frame)
S3method(vec_ptype2,data.frame.data.table)
S3method(vec_ptype2,data.table.data.frame)
S3method(vec_ptype2,data.table.data.table)
S3method(vec_ptype2,difftime)
S3method(vec_ptype2,double)
S3method(vec_ptype2,double.exclude)
S3method(vec_ptype2,double.omit)
S3method(vec_ptype2,exclude.double)
S3method(vec_ptype2,exclude.exclude)
S3method(vec_ptype2,exclude.integer)
S3method(vec_ptype2,factor)
S3method(vec_ptype2,factor.character)
S3method(vec_ptype2,factor.factor)
S3method(vec_ptype2,factor.ordered)
S3method(vec_ptype2,integer)
S3method(vec_ptype2,integer.exclude)
S3method(vec_ptype2,integer.omit)
S3method(vec_ptype2,integer64)
S3method(vec_ptype2,list)
S3method(vec_ptype2,list.vctrs_list_of)
S3method(vec_ptype2,logical)
S3method(vec_ptype2,omit.double)
S3method(vec_ptype2,omit.integer)
S3method(vec_ptype2,omit.omit)
S3method(vec_ptype2,ordered)
S3method(vec_ptype2,ordered.character)
S3method(vec_ptype2,ordered.factor)
S3method(vec_ptype2,ordered.ordered)
S3method(vec_ptype2,raw)
S3method(vec_ptype2,table.table)
S3method(vec_ptype2,vctrs_list_of)
S3method(vec_ptype2,vctrs_list_of.list)
S3method(vec_ptype2.AsIs,AsIs)
S3method(vec_ptype2.Date,Date)
S3method(vec_ptype2.Date,POSIXct)
S3method(vec_ptype2.Date,POSIXlt)
S3method(vec_ptype2.POSIXct,Date)
S3method(vec_ptype2.POSIXct,POSIXct)
S3method(vec_ptype2.POSIXct,POSIXlt)
S3method(vec_ptype2.POSIXlt,Date)
S3method(vec_ptype2.POSIXlt,POSIXct)
S3method(vec_ptype2.POSIXlt,POSIXlt)
S3method(vec_ptype2.character,character)
S3method(vec_ptype2.complex,complex)
S3method(vec_ptype2.complex,double)
S3method(vec_ptype2.complex,integer)
S3method(vec_ptype2.data.frame,data.frame)
S3method(vec_ptype2.difftime,difftime)
S3method(vec_ptype2.double,complex)
S3method(vec_ptype2.double,double)
S3method(vec_ptype2.double,integer)
S3method(vec_ptype2.double,logical)
S3method(vec_ptype2.integer,complex)
S3method(vec_ptype2.integer,double)
S3method(vec_ptype2.integer,integer)
S3method(vec_ptype2.integer,integer64)
S3method(vec_ptype2.integer,logical)
S3method(vec_ptype2.integer64,integer)
S3method(vec_ptype2.integer64,integer64)
S3method(vec_ptype2.integer64,logical)
S3method(vec_ptype2.list,list)
S3method(vec_ptype2.logical,double)
S3method(vec_ptype2.logical,integer)
S3method(vec_ptype2.logical,integer64)
S3method(vec_ptype2.logical,logical)
S3method(vec_ptype2.raw,raw)
S3method(vec_ptype2.vctrs_list_of,vctrs_list_of)
S3method(vec_ptype_abbr,"NULL")
S3method(vec_ptype_abbr,AsIs)
S3method(vec_ptype_abbr,Date)
S3method(vec_ptype_abbr,POSIXct)
S3method(vec_ptype_abbr,POSIXlt)
S3method(vec_ptype_abbr,data.frame)
S3method(vec_ptype_abbr,data.table)
S3method(vec_ptype_abbr,default)
S3method(vec_ptype_abbr,difftime)
S3method(vec_ptype_abbr,factor)
S3method(vec_ptype_abbr,integer64)
S3method(vec_ptype_abbr,ordered)
S3method(vec_ptype_abbr,table)
S3method(vec_ptype_abbr,vctrs_list_of)
S3method(vec_ptype_abbr,vctrs_unspecified)
S3method(vec_ptype_finalise,default)
S3method(vec_ptype_full,"NULL")
S3method(vec_ptype_full,AsIs)
S3method(vec_ptype_full,Date)
S3method(vec_ptype_full,POSIXct)
S3method(vec_ptype_full,POSIXlt)
S3method(vec_ptype_full,default)
S3method(vec_ptype_full,difftime)
S3method(vec_ptype_full,factor)
S3method(vec_ptype_full,integer64)
S3method(vec_ptype_full,ordered)
S3method(vec_ptype_full,table)
S3method(vec_ptype_full,vctrs_list_of)
S3method(vec_restore,AsIs)
S3method(vec_restore,Date)
S3method(vec_restore,POSIXct)
S3method(vec_restore,POSIXlt)
S3method(vec_restore,data.frame)
S3method(vec_restore,default)
S3method(vec_restore,exclude)
S3method(vec_restore,factor)
S3method(vec_restore,omit)
S3method(vec_restore,ordered)
S3method(vec_restore,table)
S3method(vec_restore,vctrs_rcrd)
S3method(vec_restore,vctrs_vctr)
S3method(xtfrm,vctrs_sclr)
S3method(xtfrm,vctrs_vctr)
export("%0%")
export("field<-")
export("vec_slice<-")
export(MISSING)
export(allow_lossy_cast)
export(as_list_of)
export(data_frame)
export(df_cast)
export(df_list)
export(df_ptype2)
export(field)
export(fields)
export(is_list_of)
export(list_all_recyclable)
export(list_all_size)
export(list_all_vectors)
export(list_check_all_recyclable)
export(list_check_all_size)
export(list_check_all_vectors)
export(list_combine)
export(list_drop_empty)
export(list_of)
export(list_of_ptype)
export(list_of_size)
export(list_of_transpose)
export(list_sizes)
export(list_unchop)
export(maybe_lossy_cast)
export(n_fields)
export(new_data_frame)
export(new_date)
export(new_datetime)
export(new_duration)
export(new_factor)
export(new_list_of)
export(new_ordered)
export(new_rcrd)
export(new_vctr)
export(num_as_location)
export(num_as_location2)
export(obj_check_list)
export(obj_check_vector)
export(obj_is_list)
export(obj_is_vector)
export(obj_print)
export(obj_print_data)
export(obj_print_footer)
export(obj_print_header)
export(obj_str)
export(obj_str_data)
export(obj_str_footer)
export(obj_str_header)
export(s3_register)
export(stop_incompatible_cast)
export(stop_incompatible_op)
export(stop_incompatible_size)
export(stop_incompatible_type)
export(tib_cast)
export(tib_ptype2)
export(unspecified)
export(vec_any_missing)
export(vec_arith)
export(vec_arith.Date)
export(vec_arith.POSIXct)
export(vec_arith.POSIXlt)
export(vec_arith.difftime)
export(vec_arith.logical)
export(vec_arith.numeric)
export(vec_arith_base)
export(vec_as_index)
export(vec_as_location)
export(vec_as_location2)
export(vec_as_names)
export(vec_as_names_legacy)
export(vec_as_subscript)
export(vec_as_subscript2)
export(vec_assert)
export(vec_assign)
export(vec_c)
export(vec_case_when)
export(vec_cast)
export(vec_cast.Date)
export(vec_cast.POSIXct)
export(vec_cast.POSIXlt)
export(vec_cast.character)
export(vec_cast.complex)
export(vec_cast.data.frame)
export(vec_cast.difftime)
export(vec_cast.double)
export(vec_cast.factor)
export(vec_cast.integer)
export(vec_cast.integer64)
export(vec_cast.list)
export(vec_cast.logical)
export(vec_cast.ordered)
export(vec_cast.raw)
export(vec_cast.vctrs_list_of)
export(vec_cast_common)
export(vec_cbind)
export(vec_cbind_frame_ptype)
export(vec_check_list)
export(vec_check_recyclable)
export(vec_check_size)
export(vec_chop)
export(vec_compare)
export(vec_count)
export(vec_data)
export(vec_default_cast)
export(vec_default_ptype2)
export(vec_detect_complete)
export(vec_detect_missing)
export(vec_duplicate_any)
export(vec_duplicate_detect)
export(vec_duplicate_id)
export(vec_empty)
export(vec_equal)
export(vec_equal_na)
export(vec_expand_grid)
export(vec_fill_missing)
export(vec_group_id)
export(vec_group_loc)
export(vec_group_rle)
export(vec_identify_runs)
export(vec_if_else)
export(vec_in)
export(vec_init)
export(vec_init_along)
export(vec_interleave)
export(vec_is)
export(vec_is_empty)
export(vec_is_list)
export(vec_locate_matches)
export(vec_locate_sorted_groups)
export(vec_match)
export(vec_math)
export(vec_math_base)
export(vec_names)
export(vec_names2)
export(vec_order)
export(vec_pall)
export(vec_pany)
export(vec_proxy)
export(vec_proxy_compare)
export(vec_proxy_equal)
export(vec_proxy_order)
export(vec_ptype)
export(vec_ptype2)
export(vec_ptype2.AsIs)
export(vec_ptype2.Date)
export(vec_ptype2.POSIXct)
export(vec_ptype2.POSIXlt)
export(vec_ptype2.character)
export(vec_ptype2.complex)
export(vec_ptype2.data.frame)
export(vec_ptype2.difftime)
export(vec_ptype2.double)
export(vec_ptype2.factor)
export(vec_ptype2.integer)
export(vec_ptype2.integer64)
export(vec_ptype2.list)
export(vec_ptype2.logical)
export(vec_ptype2.ordered)
export(vec_ptype2.raw)
export(vec_ptype2.vctrs_list_of)
export(vec_ptype_abbr)
export(vec_ptype_common)
export(vec_ptype_finalise)
export(vec_ptype_full)
export(vec_ptype_show)
export(vec_rank)
export(vec_rbind)
export(vec_recode_values)
export(vec_recycle)
export(vec_recycle_common)
export(vec_rep)
export(vec_rep_each)
export(vec_repeat)
export(vec_replace_values)
export(vec_replace_when)
export(vec_restore)
export(vec_run_sizes)
export(vec_seq_along)
export(vec_set_difference)
export(vec_set_intersect)
export(vec_set_names)
export(vec_set_symmetric_difference)
export(vec_set_union)
export(vec_size)
export(vec_size_common)
export(vec_slice)
export(vec_sort)
export(vec_split)
export(vec_type)
export(vec_type2)
export(vec_type_common)
export(vec_unchop)
export(vec_unique)
export(vec_unique_count)
export(vec_unique_loc)
export(vec_unrep)
import(rlang)
importFrom(stats,median)
importFrom(stats,na.exclude)
importFrom(stats,na.fail)
importFrom(stats,na.omit)
importFrom(stats,quantile)
useDynLib(vctrs, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# vctrs <a href="https://vctrs.r-lib.org"><img src="man/figures/logo.png" align="right" height="138" /></a>

<!-- badges: start -->

[![Codecov test
coverage](https://codecov.io/gh/r-lib/vctrs/graph/badge.svg)](https://app.codecov.io/gh/r-lib/vctrs)
![Lifecycle:
maturing](https://img.shields.io/badge/lifecycle-maturing-blue.svg)
[![R-CMD-check](https://github.com/r-lib/vctrs/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/r-lib/vctrs/actions/workflows/R-CMD-check.yaml)
<!-- badges: end -->

There are three main goals to the vctrs package, each described in a
vignette:

- To propose `vec_size()` and `vec_ptype()` as alternatives to
  `length()` and `class()`; `vignette("type-size")`. These definitions
  are paired with a framework for size-recycling and type-coercion.
  `ptype` should evoke the notion of a prototype, i.e. the original or
  typical form of something.

- To define size- and type-stability as desirable function properties,
  use them to analyse existing base functions, and to propose better
  alternatives; `vignette("stability")`. This work has been particularly
  motivated by thinking about the ideal properties of `c()`, `ifelse()`,
  and `rbind()`.

- To provide a new `vctr` base class that makes it easy to create new S3
  vectors; `vignette("s3-vector")`. vctrs provides methods for many base
  generics in terms of a few new vctrs generics, making implementation
  considerably simpler and more robust.

vctrs is a developer-focussed package. Understanding and extending vctrs
requires some effort from developers, but should be invisible to most
users. It’s our hope that having an underlying theory will mean that
users can build up an accurate mental model without explicitly learning
the theory. vctrs will typically be used by other packages, making it
easy for them to provide new classes of S3 vectors that are supported
throughout the tidyverse (and beyond). For that reason, vctrs has few
dependencies.

## Installation

Install vctrs from CRAN with:

``` r
install.packages("vctrs")
```

Alternatively, if you need the development version, install it with:

``` r
# install.packages("pak")
pak::pak("r-lib/vctrs")
```

## Usage

``` r
library(vctrs)

# Sizes
vec_size_common(1, 1:10)
#> [1] 10
str(vec_recycle_common(1, 1:10))
#> List of 2
#>  $ : num [1:10] 1 1 1 1 1 1 1 1 1 1
#>  $ : int [1:10] 1 2 3 4 5 6 7 8 9 10

# Prototypes
vec_ptype_common(FALSE, 1L, 2.5)
#> numeric(0)
str(vec_cast_common(FALSE, 1L, 2.5))
#> List of 3
#>  $ : num 0
#>  $ : num 1
#>  $ : num 2.5
```

## Classes

| Class | Description |
| ------ | ------ |
| [VctrsDatabase](Class.VctrsDatabase.md) | A persistent vector database backed by an HNSW index. |

## Interfaces

| Interface | Description |
| ------ | ------ |
| [GetResult](Interface.GetResult.md) | A vector and its metadata, returned by `get()`. |
| [SearchResult](Interface.SearchResult.md) | A single search result. |
| [VctrsStats](Interface.VctrsStats.md) | Graph-level statistics returned by `stats()`. |

## Type Aliases

| Type Alias | Description |
| ------ | ------ |
| [WhereFilter](TypeAlias.WhereFilter.md) | Metadata filter for search queries. |

expect_lossy_cast <- function(expr) {
  cnd <- NULL

  out <- with_handlers(
    warning = calling(function(x) {
      cnd <<- x
      cnd_muffle(x)
    }),
    expr
  )
  expect_s3_class(cnd, "vctrs_warning_cast_lossy")

  out
}

testthat_import_from <- function(ns, names, env = caller_env()) {
  skip_if_not_installed(ns)
  import_from(ns, names, env = env)
}

shaped_int <- function(...) {
  array(NA_integer_, c(...))
}

set_rownames_recursively <- function(x, i = NULL) {
  n <- vec_size(x)
  stopifnot(n <= length(letters))

  for (j in seq_along(x)) {
    if (is.data.frame(x[[j]])) {
      x[[j]] <- set_rownames_recursively(x[[j]], i = i)
    }
  }

  row.names(x) <- paste0(letters[seq_len(n)], i)
  x
}

expect_waldo_equal <- function(type, act, exp, info, ...) {
  comp <- waldo::compare(act$val, exp$val, ..., x_arg = "actual", y_arg = "expected")
  expect(
    length(comp) == 0,
    sprintf(
      "`actual` (%s) not %s to `expected` (%s).\n\n%s",
      act$lab, type, exp$lab, paste0(comp, collapse = "\n\n")
    ),
    info = info
  )

  invisible(act$val)
}

expect_identical <- function(object, expected, info = NULL, label = NULL,
                             expected.label = NULL, ...) {
  act <- quasi_label(enquo(object), label, arg = "object")
  exp <- quasi_label(enquo(expected), expected.label, arg = "expected")

  expect_waldo_equal("identical", act, exp, info, ...)
}

expect_equal <- function(object, expected, ...,
                         tolerance = .Machine$double.eps ^ 0.5,
                         info = NULL, label = NULL,
                         expected.label = NULL) {
  act <- quasi_label(enquo(object), label, arg = "object")
  exp <- quasi_label(enquo(expected), expected.label, arg = "expected")

  expect_waldo_equal("equal", act, exp, info, ..., tolerance = tolerance)
}

raw2 <- function(...) {
  as.raw(list_unchop(list2(...), ptype = integer()))
}
cpl2 <- function(...) {
  # R 4.4.0 changed `as.complex(NA_real/integer/logical)` so that it always uses
  # a `0` in the imaginary slot. While this is reasonable, it is annoying for
  # comparison purposes in tests, where we typically propagate the `NA`. As of
  # rlang 1.1.1, `cpl()` inherits this behavior change so we have a custom version
  # here that works the same on all R versions.
  # https://github.com/wch/r-source/commit/1a2aea9ac3c216fea718f33f712764afc34f6ee8
  out <- list2(...)
  out <- as.complex(out)
  out[is.na(out)] <- complex(real = NA_real_, imaginary = NA_real_)
  out
}

test_that("logical/integer/numeric works", {
  expect_equal(vec_arith("+", TRUE, TRUE), 2L)
  expect_equal(vec_arith("+", TRUE, 1L), 2L)
  expect_equal(vec_arith("+", TRUE, 1), 2)
  expect_equal(vec_arith("+", 1L, TRUE), 2L)
  expect_equal(vec_arith("+", 1L, 1L), 2L)
  expect_equal(vec_arith("+", 1L, 1), 2)
  expect_equal(vec_arith("+", 1, TRUE), 2L)
  expect_equal(vec_arith("+", 1, 1L), 2L)
  expect_equal(vec_arith("+", 1, 1), 2)
})

test_that("default is error", {
  f <- new_vctr(1:10, class = "foo")

  expect_error(vec_arith("+", f, 1), class = "vctrs_error_incompatible_op")

  expect_error(vec_arith("+", TRUE, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", 1L, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", 1, f), class = "vctrs_error_incompatible_op")
})

test_that("can compact missing elements", {
  x <- list(NULL, 1, NULL)
  expect_identical(list_drop_empty(x), list(1))
})

test_that("can compact empty elements", {
  x <- list(1, NULL, integer(), NULL)
  expect_identical(list_drop_empty(x), list(1))
})

test_that("emptyness works with data frames", {
  x <- data_frame()
  y <- data_frame(.size = 2L)

  lst <- list(x, y)

  expect_identical(list_drop_empty(lst), list(y))
})

test_that("emptyness works with rcrd types", {
  x <- new_rcrd(list(foo = integer(), bar = numeric()))
  y <- new_rcrd(list(foo = 1L, bar = 1))

  lst <- list(x, y)

  expect_identical(list_drop_empty(lst), list(y))
})

test_that("works with empty lists", {
  expect_identical(list_drop_empty(list()), list())
})

test_that("retains list type", {
  x <- list_of(NULL, integer())
  expect_identical(list_drop_empty(x), list_of(.ptype = integer()))
})

test_that("validates `x`", {
  expect_error(list_drop_empty(1), "must be a list")
  expect_error(list_drop_empty(data_frame()), "must be a list")
})

test_that("ptype methods are descriptive", {
  f <- factor()
  o <- ordered(character())

  expect_equal(vec_ptype_abbr(f), "fct")
  expect_equal(vec_ptype_abbr(o), "ord")

  expect_equal(vec_ptype_full(f), "factor<>")
  expect_equal(vec_ptype_full(o), "ordered<>")
})

test_that("slicing factors uses a proxy to not go through `[.factor`", {
  x <- factor("x")
  y <- ordered("y")

  local_methods(
    `[.factor` = function(x, ...) abort("should not be called")
  )

  expect_identical(vec_slice(x, 1), x)
  expect_identical(vec_slice(y, 1), y)
})

test_that("`vec_c()` throws the right error with subclassed factors (#1015)", {
  skip("Factors now have a `c()` method")

  a <- subclass(factor("a"))
  b <- subclass(factor("b"))

  # We used to return a subclass
  expect_identical(vec_c(a, a), subclass(factor(c("a", "a"))))

  # We used to fail if attributes were incoPatible
  expect_error(vec_c(a, b), class = "vctrs_error_incompatible_type")
})

# Coercion ----------------------------------------------------------------

test_that("factor/character coercions are symmetric and unchanging", {
  types <- list(
    ordered(character()),
    factor(),
    character()
  )
  mat <- maxtype_mat(types)
  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("factors level are unioned", {
  # This is technically incorrect, but because of R's existing behaviour
  # anything else will cause substantial friction.
  fa <- factor(levels = "a")
  fb <- factor(levels = "b")

  expect_equal(vec_ptype_common(fa, fb), factor(levels = c("a", "b")))
  expect_equal(vec_ptype_common(fb, fa), factor(levels = c("b", "a")))
})

test_that("coercion errors with factors", {
  f <- factor(levels = "a")

  expect_error(vec_ptype_common(f, logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype_common(logical(), f), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2(<factor>, NA) is symmetric (#687)", {
  fct <- new_factor()
  expect_identical(
    vec_ptype2(fct, NA),
    vec_ptype2(NA, fct)
  )

  fct <- new_ordered()
  expect_identical(
    vec_ptype2(fct, NA),
    vec_ptype2(NA, fct)
  )
})

test_that("vec_ptype2(<integer64>, NA) is symmetric (#687)", {
  i64 <- bit64::integer64()
  expect_identical(
    vec_ptype2(i64, NA),
    vec_ptype2(NA, i64)
  )
})

test_that("vec_ptype2() errors with malformed factors", {
  x <- structure(1, class = "factor")
  y <- factor("x")

  expect_error(vec_ptype2(x, y, x_arg = "z"), "`z` is a corrupt factor")
  expect_error(vec_ptype2(y, x, y_arg = "z"), "`z` is a corrupt factor")
})

test_that("vec_ptype2() errors with malformed ordered factors", {
  x <- structure(1, class = c("ordered", "factor"))
  y <- as.ordered(factor("x"))

  expect_error(vec_ptype2(x, y, x_arg = "z"), "`z` is a corrupt ordered factor")
  expect_error(vec_ptype2(y, x, y_arg = "z"), "`z` is a corrupt ordered factor")
})

test_that("ordered factors with different levels are not compatible", {
  expect_error(
    vec_ptype2(ordered("a"), ordered("b")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(ordered("a"), ordered(c("a", "b"))),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), ordered("b")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), ordered(c("a", "b"))),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("factors and ordered factors are not compatible", {
  expect_error(
    vec_ptype2(factor("a"), ordered("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(ordered("a"), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(factor("a"), ordered("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
})


# Casting -----------------------------------------------------------------

test_that("safe casts work as expected", {
  fa <- factor("a")
  fab <- factor(c("a", "b"))

  expect_equal(vec_cast(NULL, fa), NULL)

  expect_equal(vec_cast(fa, fa), fa)
  expect_equal(vec_cast(fa, fab), fab[1])
  expect_equal(vec_cast("a", fab), fab[1])

  expect_equal(vec_cast("a", factor()), fa)
  expect_equal(vec_cast(fa, factor()), fa)

  # This used to be allowed
  expect_error(vec_cast(list("a", "b"), fab), class = "vctrs_error_incompatible_type")
})

test_that("can cast to character", {
  expect_equal(vec_cast(factor("X"), character()), "X")
  expect_equal(vec_cast(ordered("X"), character()), "X")
})

test_that("can cast NA and unspecified to factor", {
  expect_identical(vec_cast(NA, new_factor()), factor(NA))
  expect_identical(vec_cast(NA, new_ordered()), ordered(NA))
  expect_identical(vec_cast(unspecified(2), new_factor()), factor(c(NA, NA)))
  expect_identical(vec_cast(unspecified(2), new_ordered()), ordered(c(NA, NA)))
})

test_that("lossy factor casts fail", {
  fa <- factor("a")
  fb <- factor("b")

  expect_lossy(vec_cast(fa, fb), factor(NA, levels = "b"), x = fa, to = fb)
  expect_lossy(vec_cast("a", fb), factor(NA, levels = "b"), x = chr(), to = fb)
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(double(), factor("a")), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(factor("a"), logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(ordered("a"), logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(logical(), factor("a")), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(logical(), ordered("a")), class = "vctrs_error_incompatible_type")
})

test_that("orderedness of factor is preserved", {
  ord <- ordered(c("a", "b"), levels = c("b", "a"))
  expect_equal(vec_cast("a", ord), ordered("a", levels = c("b", "a")))
})

test_that("NA are not considered lossy in factor cast (#109)", {
  f <- factor(c("itsy", "bitsy", NA, "spider", "spider"))
  expect_warning(vec_cast(f, f[1]), NA)
})

test_that("Casting to a factor with explicit NA levels retains them", {
  f <- factor(c("x", NA), exclude = NULL)
  expect_identical(vec_cast(f, f), f)
  expect_identical(vec_cast(f, factor()), f)
})

test_that("characters can be cast to ordered", {
  expect_identical(vec_cast("a", ordered("a")), ordered("a"))
  expect_error(vec_cast(c("a", "b"), ordered("a")), class = "vctrs_error_cast_lossy")
})


# Proxy / restore ---------------------------------------------------------

test_that("subclassed factors / ordered factors can be restored (#1015)", {
  x <- subclass(factor("a"))
  proxy <- vec_proxy(x)
  expect_identical(vec_restore(proxy, x), x)

  y <- subclass(ordered("a"))
  proxy <- vec_proxy(y)
  expect_identical(vec_restore(proxy, y), y)
})

# Arithmetic and factor ---------------------------------------------------

test_that("factors don't support math or arthimetic", {
  f <- factor("x")
  expect_error(vec_math("sum", f), class = "vctrs_error_unsupported")
  expect_error(vec_arith("+", f, f), class = "vctrs_error_unsupported")
})

# Never run on CRAN, even if they have sf, because we don't regularly
# check these on CI and we don't want a change in sf to force a CRAN
# failure for vctrs.
skip_on_cran()

# Avoids adding `sf` to Suggests.
# These tests are only run on the devs' machines.
testthat_import_from("sf", c(
  "st_sf",
  "st_sfc",
  "st_point",
  "st_bbox",
  "st_precision",
  "st_crs",
  "st_linestring",
  "st_as_sf",
  "st_multipoint"
))

# Need recent version to work around restore bug for sfc lists and changes
# to `c.sfc()`
skip_if_not_installed("sf", "1.0-11")

test_that("sf has a ptype2 method", {
	sfc1 = st_sfc(st_point(1:2), st_point(3:4))
	sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

	sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
	sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

	out = vctrs::vec_ptype2(sf1, sf2)
	exp = st_sf(
		x = double(),
		y = character(),
		geo1 = sfc1[0],
		geo2 = sfc2[0],
		stringsAsFactors = FALSE
	)
	expect_identical(out, exp)

	out = vctrs::vec_ptype2(sf1, new_data_frame(sf2))
	expect_identical(out, exp)

	out = vctrs::vec_ptype2(new_data_frame(sf1), sf2)
	exp_rhs = st_sf(
		x = double(),
		y = character(),
		geo1 = sfc1[0],
		geo2 = sfc2[0],
		stringsAsFactors = FALSE,
		sf_column_name = "geo2"
	)
	expect_identical(out, exp_rhs)
})

test_that("sf has a cast method", {
	sfc1 = st_sfc(st_point(1:2), st_point(3:4))
	sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

	sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
	sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

	expect_error(
		vctrs::vec_cast(sf1, sf2),
		class = "vctrs_error_cast_lossy"
	)
	expect_error(
		vctrs::vec_cast(sf2, sf1),
		class = "vctrs_error_cast_lossy"
	)

	common = vec_ptype2(sf1, sf2)

	out = vctrs::vec_cast(sf1, common)
	exp = st_sf(
		x = c(1, 0),
		y = character(2)[NA],
		geo1 = sfc1,
		geo2 = sfc2[c(NA, NA) + 0L],
		stringsAsFactors = FALSE
	)
	expect_identical(out, exp)

	out = vctrs::vec_cast(new_data_frame(sf1), common)
	expect_identical(out, exp)

	out = vctrs::vec_cast(sf1, new_data_frame(common))
	expect_identical(out, new_data_frame(exp))

	out = vctrs::vec_cast(sf2, common)
	exp = st_sf(
		x = 0,
		y = "",
		geo1 = sfc1[NA + 0L],
		geo2 = sfc2,
		stringsAsFactors = FALSE
	)
	expect_identical(out, exp)
})

# https://github.com/r-lib/vctrs/issues/1136
test_that("can combine sf data frames", {
	testthat_import_from("dplyr", "bind_rows")

	sfc1 = st_sfc(st_point(1:2), st_point(3:4))
	sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

	sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
	sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

	exp = data_frame(
		x = c(1, 0, 0),
		geo1 = sfc1[c(1:2, NA)],
		y = c(NA, NA, ""),
		geo2 = sfc2[c(NA, NA, 1)]
	)
	expect_identical(vctrs::vec_rbind(sf1, sf2), exp)
	expect_identical(bind_rows(sf1, sf2), st_as_sf(exp))

	exp = data_frame(
		y = c("", NA, NA, ""),
		x = c(0, 1, 0, 0),
		geo2 = sfc2[c(1, NA, NA, 1)],
		geo1 = sfc1[c(NA, 1:2, NA)]
	)
	expect_identical(vctrs::vec_rbind(sf2, sf1, sf2), exp)
	expect_identical(bind_rows(sf2, sf1, sf2), st_as_sf(exp))
})

test_that("can combine sf and tibble", {
	sfc1 = st_sfc(st_point(1:2), st_point(3:4))
	sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

	sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
	sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

	out = vctrs::vec_rbind(sf2, data.frame(x = 1))
	exp = data_frame(
		y = c("", NA),
		x = c(0, 1),
		geo2 = sfc2[c(1L, NA)]
	)
	expect_identical(out, exp)

	out = vctrs::vec_rbind(sf2, tibble::tibble(x = 1))
	expect_identical(out, exp)

	out = vctrs::vec_rbind(tibble::tibble(x = 1), sf2)
	exp = data_frame(
		x = c(1, 0),
		y = c(NA, ""),
		geo2 = sfc2[c(NA, 1L)]
	)
	expect_identical(out, exp)
})

# https://github.com/r-spatial/sf/issues/1390
test_that("can combine sfc lists", {
  ls <- st_linestring(matrix(1:3, ncol = 3))

  sfc <- st_sfc(ls)
  expect_identical(vec_c(sfc, sfc), c(sfc, sfc))

  sf <- st_as_sf(data.frame(id = 1, geometry = sfc))

  # Currently returns a bare data frame because of the workaround for
  # the `c()` fallback sentinels
  expect_identical(vec_rbind(sf, sf), new_data_frame(rbind(sf, sf)))
  expect_identical(vec_rbind(sf, sf, sf), new_data_frame(rbind(sf, sf, sf)))
})

test_that("can combine sfc lists with unspecified chunks", {
  point <- st_point(1:2)
  out <- vec_c(c(NA, NA), st_sfc(point), NA)
  expect_identical(out, st_sfc(NA, NA, point, NA))

  multipoint <- st_multipoint(matrix(1:4, 2))
  x <- st_sfc(point)
  y <- st_sfc(multipoint, multipoint)
  out <- vec_rbind(
    data_frame(x = x),
    data_frame(y = y)
  )
  expect_identical(out, data_frame(
    x = st_sfc(point, NA, NA),
    y = st_sfc(NA, multipoint, multipoint)
  ))
})

test_that("`n_empty` attribute of `sfc` vectors is restored", {
	pt1 = st_sfc(st_point(c(NA_real_, NA_real_)))
	pt2 = st_sfc(st_point(0:1))

	x = c(pt1, pt2)
	expect_identical(attr(vctrs::vec_slice(x, 1), "n_empty"), 1L)
	expect_identical(attr(vctrs::vec_slice(x, 2), "n_empty"), 0L)

	combined = vctrs::vec_c(pt1, pt2, pt1)
	expect_length(combined, 3)
	expect_identical(attr(combined, "n_empty"), 2L)
})

test_that("bbox attributes of `sfc` vectors are restored", {
	pt1 = st_sfc(st_point(c(1L, 2L)))
	pt2 = st_sfc(st_point(c(10L, 20L)))

	x = c(pt1, pt2)
	expect_identical(st_bbox(vctrs::vec_slice(x, 1)), st_bbox(pt1))
	expect_identical(st_bbox(vctrs::vec_slice(x, 2)), st_bbox(pt2))

	combined = vctrs::vec_c(pt1, pt2)
	expect_identical(st_bbox(x), st_bbox(combined))
})

test_that("`precision` and `crs` attributes of `sfc` vectors are restored", {
	x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
	out = vctrs::vec_slice(x, 1)
	expect_identical(st_precision(x), st_precision(out))
	expect_identical(st_crs(x), st_crs(out))
})

test_that("`precision` attributes of `sfc` vectors are combined", {
	x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
	y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 3857)

	out = vctrs::vec_c(x, y)
	expect_identical(st_precision(x), st_precision(out))

	# These used to be errors before we fell back to c()
	y = st_sfc(st_point(c(0, 0)), precision = 1e-2, crs = 3857)
	expect_identical(vctrs::vec_c(x, y), c(x, y))
	# expect_error(vctrs::vec_c(x, y), "precisions not equal")
})

test_that("`crs` attributes of `sfc` vectors must be the same", {
  x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
  y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 3857)

  out = vctrs::vec_c(x, y)
  expect_identical(st_crs(x), st_crs(out))

  # Error on different `crs` comes from sf as of 1.0-10
  y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 4326)
  expect_snapshot(error = TRUE, {
    vctrs::vec_c(x, y)
  })
})

test_that("`vec_locate_matches()` works with `sfc` vectors", {
  x <- c(
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 1))),
    st_sfc(st_point(c(2, 1))),
    st_sfc(c(st_point(c(0, 1)), st_point(c(0, 1))))
  )

  y <- c(
    st_sfc(c(st_point(c(0, 1)), st_point(c(0, 1)))),
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 3))),
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 1)))
  )

  out <- vec_locate_matches(x, y)
  expect_identical(out$needles,  c(1L, 1L, 2L, 3L, 4L))
  expect_identical(out$haystack, c(2L, 4L, 5L, NA, 1L))
})

test_that("`vec_rbind()` doesn't leak common type fallbacks (#1331)", {
	sf = st_sf(id = 1:2, geo = st_sfc(st_point(c(1, 1)), st_point(c(2, 2))))

	expect_equal(
		vec_rbind(sf, sf),
		data_frame(id = rep(1:2, 2), geo = rep(sf$geo, 2))
	)

	expect_equal(
		vec_rbind(sf, sf, .names_to = "id"),
		data_frame(id = rep(1:2, each = 2), geo = rep(sf$geo, 2))
	)
})


# Local Variables:
# indent-tabs-mode: t
# ess-indent-offset: 4
# tab-width: 4
# End:

# ------------------------------------------------------------------------------
# Printing

test_that("I() wraps contents", {
  f <- factor()

  expect_equal(vec_ptype_abbr(I(f)), "I<fct>")
  expect_equal(vec_ptype_full(I(f)), "I<factor<>>")
})

test_that("AsIs class stripped from I()", {
  df <- data.frame(x = 1, y = 1:2)
  class(df) <- c("myclass", "data.frame")

  expect_equal(vec_ptype_full(I(df)), "I<myclass<\n  x: double\n  y: integer\n>>")
  expect_equal(vec_ptype_full(I(df[1])), "I<myclass<x:double>>")
  expect_equal(vec_ptype_full(I(df[0])), "I<myclass<>>")
})

# ------------------------------------------------------------------------------
# Proxy / restore

test_that("can slice AsIs class", {
  df <- data.frame(x = I(1:3), y = I(list(4, 5, 6)))
  expect_identical(vec_slice(df, 2:3), unrownames(df[2:3, ]))
})

test_that("equality proxy is forwarded correctly for atomic types (#1557)", {
  # We don't define any equality proxies for base atomic types, but we can fake it
  local_methods(vec_proxy_equal.integer = function(x, ...) "dispatched")
  asis <- I(1L)
  expect_identical(vec_proxy_equal(asis), "dispatched")
})

test_that("comparison proxy is forwarded correctly for atomic types (#1557)", {
  # vec_proxy_compare.raw() exists
  x <- raw()
  asis <- I(x)

  expect_identical(vec_proxy_compare(asis), vec_proxy_compare(x))
  expect_identical(vec_proxy_compare(asis), integer())
})

test_that("order proxy is forwarded correctly for atomic types (#1557)", {
  # vec_proxy_order.list() exists
  x <- list(2, 1, 2)
  asis <- I(x)

  expect_identical(vec_proxy_order(asis), vec_proxy_order(x))
  expect_identical(vec_proxy_order(asis), c(1L, 2L, 1L))
})

# ------------------------------------------------------------------------------
# Coercion

test_that("can take the common type of identical AsIs objects", {
  expect_identical(vec_ptype2(I(1), I(1)), I(numeric()))
})

test_that("AsIs objects throw ptype2 errors with their underlying types", {
  expect_snapshot({
    (expect_error(vec_ptype2(I(1), I("x")), class = "vctrs_error_incompatible_type"))
  })
})

test_that("AsIs always wraps the common type", {
  expect_identical(vec_ptype2(I(1L), 1), I(numeric()))
  expect_identical(vec_ptype2(1, I(1L)), I(numeric()))
})

# ------------------------------------------------------------------------------
# Casting

test_that("can cast one AsIs to another AsIs", {
  expect_identical(vec_cast(I(1), I(1)), I(1))
  expect_identical(vec_cast(I(1), I(1L)), I(1L))
})

test_that("AsIs objects throw cast errors with their underlying types", {
  expect_snapshot({
    (expect_error(vec_cast(I(1), I(factor("x"))), class = "vctrs_error_incompatible_type"))
  })
})

test_that("casting from an AsIs drops the AsIs class", {
  expect_identical(vec_cast(I(1), 1), 1)
})

test_that("casting to an AsIs adds the AsIs class", {
  expect_identical(vec_cast(1, I(1)), I(1))
})

# ------------------------------------------------------------------------------
# Misc

test_that("can `vec_c()` with only AsIs objects", {
  expect_identical(vec_c(I(1), I(2)), I(c(1, 2)))
  expect_identical(vec_c(I(1), I(2L)), I(c(1, 2)))
})

test_that("can `vec_c()` with AsIs objects mixed with other types", {
  expect_identical(vec_c(I(1L), 1), I(c(1, 1)))
})

# common shape ------------------------------------------------------------

test_that("vec_shape2() applies recycling rules", {
  expect_equal(vec_shape2(shaped_int(1, 5, 5), shaped_int(1)),       c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1),       shaped_int(1, 5, 5)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1, 1),    shaped_int(1, 5, 5)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1, 1, 1), shaped_int(1, 5, 5)), c(0L, 5L, 5L))

  expect_equal(vec_shape2(shaped_int(1, 1, 5), shaped_int(1, 5, 1)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1, 5, 1), shaped_int(1, 1, 5)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1, 1, 1), shaped_int(1, 5, 5)), c(0L, 5L, 5L))

  expect_equal(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 1, 1)), c(0L, 0L, 5L))
})

test_that("incompatible shapes throw errors", {
  expect_snapshot({
    (expect_error(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1)), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5)), class = "vctrs_error_incompatible_type"))
  })
})

test_that("can override error args", {
  expect_snapshot({
    (expect_error(
      vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1), x_arg = "foo", y_arg = "bar"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_shape2() evaluates arg lazily", {
  expect_silent(vec_shape2(shaped_int(1, 5, 5), shaped_int(1), x_arg = print("oof")))
  expect_silent(vec_shape2(shaped_int(1, 5, 5), shaped_int(1), y_arg = print("oof")))
})

# broadcasting -------------------------------------------------------------

test_that("can broadcast to higher dimension, but not lower", {
  expect_identical(shape_broadcast_(1, NULL), 1)
  expect_null(shape_broadcast_(NULL, 1))

  expect_equal(
    shape_broadcast_(1, shaped_int(0, 4)),
    array(1, c(1, 4))
  )
  expect_error(
    shape_broadcast_(shaped_int(1, 1, 1), shaped_int(4, 4)),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    shape_broadcast_(shaped_int(3, 2), shaped_int(3, 3)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("shape_broadcast_() applies recycling rules", {
  expect_equal(
    shape_broadcast_(array(1:4, c(1, 1, 4)), shaped_int(0, 4, 4))[1, , ],
    matrix(1:4, 4, 4, byrow = TRUE)
  )

  expect_equal(
    shape_broadcast_(array(1:4, c(1, 4, 1)), shaped_int(0, 4, 4))[1, , ],
    matrix(1:4, 4, 4)
  )

  expect_equal(
    shape_broadcast_(array(1L, c(1, 1)), shaped_int(1, 0)),
    matrix(integer(), nrow = 1)
  )

  expect_error(
    shape_broadcast_(array(1L, c(1, 2)), shaped_int(1, 0)),
    "Non-recyclable dimensions",
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    shape_broadcast_(array(1L, c(1, 0)), shaped_int(1, 1)),
    "Non-recyclable dimensions",
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can combine shaped native classes (#1290, #1329)", {
  x <- new_datetime(c(1, 1e6))
  dim(x) <- c(1, 2)
  out <- vec_c(x, x)

  expect_s3_class(out, c("POSIXct", "POSIXt"))
  expect_dim(out, c(2, 2))

  y <- new_datetime(1:3 + 0.0)
  dim(y) <- c(1, 3)

  expect_snapshot(error = TRUE, vec_c(x, y))

  d <- structure(Sys.Date(), dim = 1)
  expect_equal(
    vec_rbind(data.frame(d), data.frame(d)),
    data.frame(d = structure(rep(Sys.Date(), 2), dim = 2))
  )
})

test_that("factor casts support shape", {
  x <- factor(c("x", "y", "z"))
  dim(x) <- c(3, 1)
  dimnames(x) <- list(c("r1", "r2", "r3"), "c1")

  y <- factor(c("w", "x", "y", "z"))
  dim(y) <- c(2, 2)

  exp <- factor(
    c("x", "y", "z", "x", "y", "z"),
    levels = c("w", "x", "y", "z")
  )
  dim(exp) <- c(3, 2)
  dimnames(exp) <- list(c("r1", "r2", "r3"), c("c1", "c1"))

  expect_equal(vec_cast(x, y), exp)

  x <- factor(c("x", "y", "z"))
  dim(x) <- c(3, 1)
  y <- factor(c("x", "y", "z"))
  expect_snapshot(error = TRUE, vec_cast(x, y))
})

# Vectorised --------------------------------------------------------------

test_that("vec_hash() produces same hash for same values", {
  x <- vec_hash(1:3)
  y <- do.call(c, map(1:3, vec_hash))
  expect_identical(x, y)
})

test_that("F, T, and NA hash to different values", {
  x <- map(c(TRUE, FALSE, NA), vec_hash)
  expect_length(unique(x), 3)
})

test_that("vec_hash of double produces different values", {
  x <- vec_hash(c(1, 1, 2))
  expect_true(identical(x[1:4], x[5:8]))
  expect_false(identical(x[5:8], x[9:12]))
})

test_that("NA and NaN hash to different values", {
  x <- vec_hash(c(NA, NaN))
  expect_false(identical(x[1:4], x[5:8]))
})

test_that("same string hashes to same value", {
  x <- vec_hash(c("1", "1", "2"))
  expect_true(identical(x[1:4], x[5:8]))
  expect_false(identical(x[5:8], x[9:12]))
})

test_that("list hashes to values of individual values", {
  x <- vec_hash(list(1:3, letters))
  expect_identical(x[1:4], obj_hash(1:3))
  expect_identical(x[5:8], obj_hash(letters))

  x <- map(list(list(1:3), list(letters)), vec_hash)
  expect_identical(x[[1]], obj_hash(1:3))
  expect_identical(x[[2]], obj_hash(letters))
})

test_that("hash of data frame works down rows", {
  df <- data.frame(x = 1:3, y = 1:3)
  x <- vec_hash(df)
  expect_length(x, 4 * vec_size(df))
  expect_identical(x[1:4], vec_hash(df[1, ]))
})

test_that("hashes are consistent from run to run", {
  # no string, since we're currently hashing the address in string pool
  df <- list(
    lgl = c(TRUE, FALSE, NA),
    int = 1:100,
    dbl1 = as.double(1:100),
    dbl2 = seq(0, 1, length = 100)
  )
  hash <- lapply(df, vec_hash)

  # Big-endian results are byte-swapped, but otherwise equivalent.
  # Swap results so that there's no need to save results twice.
  if (.Platform$endian == "big") {
    hash <- lapply(
      hash,
      function(x) {
        writeBin(readBin(x, "int", 100, endian = "big"), x, endian = "little")
      }
    )
  }

  local_options(max.print = 99999)
  expect_snapshot(hash)
})

test_that("can hash list of non-vectors", {
  x <- list(quote(x), mean)

  expect_equal(
    vec_hash(x),
    c(obj_hash(x[[1]]), obj_hash(x[[2]]))
  )
})

test_that("can hash matrices", {
  x <- matrix(c(1, 1, 1, 2, 2, 1), c(3, 2))

  expect_identical(
    vec_hash(x),
    vec_hash(x)
  )

  x <- matrix(c(1, 2, 3, 4), c(2, 2))

  expect_identical(
    vec_hash(x),
    vec_hash(x)
  )

  expect_false(identical(
    vec_hash(x),
    vec_hash(c(1, 2))
  ))

  y <- matrix(c(1, 2, 3, 5), c(2, 2))

  expect_false(identical(
    vec_hash(x),
    vec_hash(y)
  ))
})

test_that("can hash NA", {
  expect_identical(
    vec_hash(NA),
    vec_hash(NA),
  )
})

test_that("can hash 1D arrays", {
  # 1D arrays are dispatched to `as.data.frame.vector()` which
  # currently does not strip dimensions. This caused an infinite
  # recursion.
  expect_length(vec_hash(array(1:2)), 8)
  expect_identical(vec_hash(array(1:2)), vec_hash(1:2))
})

test_that("can hash raw vectors", {
  expect_identical(vec_hash(0:255), vec_hash(as.raw(0:255)))
})

test_that("can hash complex vectors", {
  expect_identical(
    vec_hash(c(1, 2) + 0i),
    c(obj_hash(c(1, 0)), obj_hash(c(2, 0)))
  )
})

test_that("hash treats positive and negative 0 as equivalent (#637)", {
  expect_equal(vec_hash(-0), vec_hash(0))
})

test_that("can hash lists of expressions", {
  expect_equal(
    vec_hash(list(expression(x), expression(y))),
    c(obj_hash(expression(x)), obj_hash(expression(y)))
  )
})

test_that("vec_hash() uses recursive equality proxy", {
  x <- new_data_frame(list(x = foobar(1:3)))
  default <- vec_hash(x)

  local_methods(vec_proxy_equal.vctrs_foobar = function(...) c(0, 0, 0))
  overridden <- vec_hash(x)

  expect_false(identical(default, overridden))
})


# Object ------------------------------------------------------------------

test_that("equal objects hash to same value", {
  # just test function since they'll recurse through every other object type
  f1 <- function(x, y = NULL) x + y
  f2 <- function(x, y = NULL) x + y
  expect_false(identical(obj_hash(f1), obj_hash(f2)))
  expect_false(identical(vec_hash(data_frame(x = list(f1))), vec_hash(data_frame(x = list(f2)))))

  attr(f1, "srcref") <- NULL
  attr(f2, "srcref") <- NULL
  expect_equal(obj_hash(f1), obj_hash(f2))
  expect_equal(vec_hash(data_frame(x = list(f1))), vec_hash(data_frame(x = list(f2))))
})

test_that("expression vectors hash to the same value as lists of calls/names", {
  expect_equal(
    obj_hash(expression(x, y)),
    obj_hash(list(as.name("x"), as.name("y")))
  )

  expect_equal(
    obj_hash(expression(mean(), sd())),
    obj_hash(list(call("mean"), call("sd")))
  )
})

# ------------------------------------------------------------------------------
# vec_normalize_encoding()

test_that("can translate a character vector of various encodings (#553)", {
  x <- unlist(encodings(), use.names = FALSE)

  results <- vec_normalize_encoding(x)

  expect_equal_encoding(results, encodings()$utf8)
})

test_that("translates all encodings to UTF-8", {
  encs <- encodings()

  for (enc in encs) {
    expect_equal_encoding(vec_normalize_encoding(enc), encodings()$utf8)
  }
})

test_that("can translate a list containing character vectors with different encodings", {
  results <- vec_normalize_encoding(encodings())
  results <- unlist(results)

  expect_equal_encoding(results, encodings()$utf8)
})

test_that("translation fails purposefully with any bytes", {
  expect_error(
    vec_normalize_encoding(encoding_bytes()),
    "translating strings with \"bytes\" encoding"
  )
})

test_that("translation fails purposefully when mixing with bytes with other encodings", {
  for (enc in encodings()) {
    x <- c(encoding_bytes(), enc)
    expect_error(vec_normalize_encoding(x), "translating strings with \"bytes\" encoding")
  }
})

test_that("attributes are kept on translation (#599)", {
  encs <- encodings()

  x <- c(encs$utf8, encs$latin1)
  x <- structure(x, names = c("a", "b"), extra = 1)

  expect_equal(attributes(vec_normalize_encoding(x)), attributes(x))
})

test_that("translation is robust against scalar types contained in lists (#633)", {
  x <- list(a = z ~ y, b = z ~ z)
  expect_equal(vec_normalize_encoding(x), x)
})

test_that("translation can still occur even if a scalar type is in a list", {
  encs <- encodings()
  x <- list(a = z ~ y, b = encs$latin1)

  result <- vec_normalize_encoding(x)

  expect_equal_encoding(result$b, encs$utf8)
})

test_that("translation occurs inside scalars contained in a list", {
  encs <- encodings()

  scalar <- structure(list(x = encs$latin1), class = "scalar_list")
  lst <- list(scalar)

  result <- vec_normalize_encoding(lst)

  expect_equal_encoding(result[[1]]$x, encs$utf8)
})

test_that("translation treats data frames elements of lists as lists (#1233)", {
  encs <- encodings()

  field <- c(encs$utf8, encs$latin1)

  a <- new_rcrd(list(field = field))
  df <- data.frame(a = a, b = 1:2)
  x <- list(df)

  # Recursive proxy won't proxy list elements,
  # so the rcrd column in the data frame won't get proxied
  proxy <- vec_proxy_equal(x)

  result <- vec_normalize_encoding(proxy)

  expect_identical(result, x)

  result_field <- field(result[[1]]$a, "field")
  expect_field <- c(encs$utf8, encs$utf8)

  expect_equal_encoding(result_field, expect_field)
})

test_that("attributes are translated", {
  utf8 <- encodings()$utf8
  latin1 <- encodings()$latin1

  a <- structure(1, enc = utf8)
  b <- structure(1, enc = latin1)
  c <- structure(1, enc1 = utf8, enc2 = list(latin1), enc3 = latin1)
  x <- list(a, b, c)

  result <- vec_normalize_encoding(x)

  a_enc <- attr(result[[1]], "enc")
  b_enc <- attr(result[[2]], "enc")
  c_enc1 <- attr(result[[3]], "enc1")
  c_enc2 <- attr(result[[3]], "enc2")[[1]]
  c_enc3 <- attr(result[[3]], "enc3")

  expect_equal_encoding(a_enc, utf8)
  expect_equal_encoding(b_enc, utf8)
  expect_equal_encoding(c_enc1, utf8)
  expect_equal_encoding(c_enc2, utf8)
  expect_equal_encoding(c_enc3, utf8)

  expect <- list(
    structure(1, enc = utf8),
    structure(1, enc1 = utf8, enc2 = list(utf8), enc3 = utf8)
  )

  expect_identical(vec_unique(x), expect)
})

test_that("attributes are translated recursively", {
  utf8 <- encodings()$utf8
  latin1 <- encodings()$latin1

  nested <- structure(1, latin1 = latin1)
  x <- structure(2, nested = nested, foo = 1, latin1 = latin1)

  result <- vec_normalize_encoding(x)
  attrib <- attributes(result)
  attrib_nested <- attributes(attrib$nested)

  expect_equal_encoding(attrib$latin1, utf8)
  expect_equal_encoding(attrib_nested$latin1, utf8)
})

test_that("NAs aren't converted to 'NA' (#1291)", {
  utf8 <- c(NA, encodings()$utf8)
  latin1 <- c(NA, encodings()$latin1)

  result1 <- vec_normalize_encoding(utf8)
  result2 <- vec_normalize_encoding(latin1)

  expect_equal_encoding(result1, utf8)
  expect_equal_encoding(result2, utf8)

  expect_identical(result1[[1]], NA_character_)
  expect_identical(result2[[1]], NA_character_)
})

# Print -------------------------------------------------------------------

test_that("ptype print methods are descriptive", {
  tab1 <- new_table()
  tab2 <- new_table(dim = c(0L, 1L, 2L, 1L))

  expect_equal(vec_ptype_abbr(tab1), "table[1d]")
  expect_equal(vec_ptype_abbr(tab2), "table[,1,2,1]")

  expect_equal(vec_ptype_full(tab1), "table[1d]")
  expect_equal(vec_ptype_full(tab2), "table[,1,2,1]")
})

# Coercion ----------------------------------------------------------------

test_that("can find a common type among tables with identical dimensions", {
  tab1 <- new_table()
  tab2 <- new_table(1:2, dim = c(1L, 2L, 1L))

  expect_identical(vec_ptype2(tab1, tab1), new_table())
  expect_identical(vec_ptype2(tab2, tab2), new_table(dim = c(0L, 2L, 1L)))
})

test_that("size is not considered in the ptype", {
  x <- new_table(1:2, dim = 2L)
  y <- new_table(1:3, dim = 3L)

  expect_identical(vec_ptype2(x, y), new_table())
})

test_that("vec_ptype2() can broadcast table shapes", {
  x <- new_table(dim = c(0L, 1L))
  y <- new_table(dim = c(0L, 2L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L)))

  x <- new_table(dim = c(0L, 1L, 3L))
  y <- new_table(dim = c(0L, 2L, 1L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L, 3L)))
})

test_that("vec_ptype2() never propagates dimnames", {
  x <- new_table(dim = c(0L, 1L), dimnames = list(character(), "x1"))
  y <- new_table(dim = c(0L, 2L), dimnames = list(character(), c("y1", "y2")))

  expect_null(dimnames(vec_ptype2(x, x)))
  expect_null(dimnames(vec_ptype2(x, y)))
})

test_that("implicit axes are broadcast", {
  x <- new_table(dim = c(0L, 2L))
  y <- new_table(dim = c(0L, 1L, 3L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L, 3L)))
})

test_that("errors on non-broadcastable dimensions", {
  x <- new_table(dim = c(0L, 2L))
  y <- new_table(dim = c(0L, 3L))

  expect_error(vec_ptype2(x, y), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2() errors on non-tables", {
  expect_error(vec_ptype2(new_table(), 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(new_table(), 1L), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(new_table(), "1"), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(1, new_table()), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(1L, new_table()), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2("1", new_table()), class = "vctrs_error_incompatible_type")
})

test_that("common types have symmetry when mixed with unspecified input", {
  x <- new_table()

  expect_identical(vec_ptype2(x, NA), new_table())
  expect_identical(vec_ptype2(NA, x), new_table())

  x <- new_table(dim = c(0L, 2L))

  expect_identical(vec_ptype2(x, NA), new_table(dim = c(0L, 2L)))
  expect_identical(vec_ptype2(NA, x), new_table(dim = c(0L, 2L)))
})

test_that("`table` delegates coercion", {
  expect_identical(
    vec_ptype2(new_table(1), new_table(FALSE)),
    new_table(double())
  )
  expect_error(
    vec_ptype2(new_table(1), new_table("")),
    class = "vctrs_error_incompatible_type"
  )
})


# Casting -----------------------------------------------------------------

test_that("can cast to an identically shaped table", {
  x <- new_table(1:5, dim = 5L)
  y <- new_table(1:8, dim = c(2L, 2L, 2L))

  expect_identical(vec_cast(x, x), x)
  expect_identical(vec_cast(y, y), y)
})

test_that("vec_cast() can broadcast table shapes", {
  # We test only the dim here and not the class because on R 3.2
  # the `[.table` method did not exist and `shape_broadcast()`
  # gives back a matrix, not a table.

  x <- new_table(dim = c(0L, 1L))
  y <- new_table(dim = c(0L, 2L))

  expect_identical(dim(vec_cast(x, y)), c(0L, 2L))

  x <- new_table(dim = c(0L, 1L, 1L))
  y <- new_table(dim = c(0L, 2L, 3L))

  expect_identical(dim(vec_cast(x, y)), c(0L, 2L, 3L))
})

test_that("cannot decrease axis length", {
  x <- new_table(dim = c(0L, 3L))
  y <- new_table(dim = c(0L, 1L))

  expect_error(vec_cast(x, y), "Non-recyclable", class = "vctrs_error_incompatible_type")
})

test_that("cannot decrease dimensionality", {
  x <- new_table(dim = c(0L, 1L, 1L))
  y <- new_table(dim = c(0L, 1L))

  expect_snapshot({
    (expect_error(vec_cast(x, y), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_cast() errors on non-tables", {
  expect_error(vec_cast(new_table(), 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(new_table(), 1L), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(new_table(), "1"), class = "vctrs_error_incompatible_type")

  expect_error(vec_cast(1, new_table()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1L, new_table()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast("1", new_table()), class = "vctrs_error_incompatible_type")
})

test_that("can cast from, but not to, unspecified", {
  x <- new_table()

  expect_error(vec_cast(x, NA), class = "vctrs_error_incompatible_type")
  expect_identical(vec_cast(NA, x), new_table(NA_integer_, dim = 1L))

  x <- new_table(dim = c(0L, 2L))

  expect_error(vec_cast(x, NA), class = "vctrs_error_incompatible_type")
  expect_identical(vec_cast(NA, x), new_table(c(NA_integer_, NA_integer_), dim = c(1L, 2L)))
})

test_that("`table` delegates casting", {
  expect_identical(
    vec_cast(new_table(1), new_table(FALSE)),
    new_table(TRUE)
  )
  expect_error(
    vec_cast(new_table(1), new_table("")),
    class = "vctrs_error_incompatible_type"
  )
})

# Misc --------------------------------------------------------------------

test_that("`new_table()` validates input", {
  expect_error(new_table(1L, 1), "`dim` must be an integer vector")
  expect_error(new_table(1:2, 1L), "must match the length of `x`")
})

test_that("ptype is correct", {
  tab1 <- new_table(1L, dim = 1L)
  tab2 <- new_table(1:2, dim = c(1L, 2L, 1L))

  expect_identical(vec_ptype(tab1), new_table())
  expect_identical(vec_ptype(tab2), new_table(dim = c(0L, 2L, 1L)))
})

test_that("can use a table in `vec_c()`", {
  expect_identical(vec_c(new_table()), new_table())
  expect_identical(vec_c(new_table(), new_table()), new_table())

  x <- new_table(1:5, 5L)
  y <- new_table(1:4, dim = c(2L, 2L))

  expect_identical(vec_c(x, x), new_table(c(1:5, 1:5), dim = 10L))
  expect_identical(vec_c(y, y), new_table(c(1:2, 1:2, 3:4, 3:4), dim = c(4L, 2L)))
  expect_identical(vec_c(x, y), new_table(c(1:5, 1:2, 1:5, 3:4), dim = c(7L, 2L)))
})

test_that("names of the first dimension are kept in `vec_c()`", {
  x <- new_table(1:4, c(2L, 2L))
  dimnames(x) <- list(c("r1", "r2"), c("c1", "c2"))

  xx <- vec_c(x, x)

  expect_identical(dimnames(xx), list(c("r1", "r2", "r1", "r2"), NULL))
})

test_that("can use a table in `list_unchop()`", {
  x <- new_table(1:4, dim = c(2L, 2L))

  expect_identical(list_unchop(list(x)), x)
  expect_identical(list_unchop(list(x, x), indices = list(1:2, 4:3)), vec_slice(x, c(1:2, 2:1)))
})

test_that("can concatenate tables", {
  x <- table(1:2)

  out <- vec_c(x, x)
  exp <- new_table(rep(1L, 4), dimnames = list(c("1", "2", "1", "2")))
  expect_identical(out, exp)

  out <- vec_rbind(x, x)
  exp <- data_frame(`1` = new_table(c(1L, 1L)), `2` = new_table(c(1L, 1L)))
  expect_identical(out, exp)


  y <- table(list(1:2, 3:4))

  # FIXME
  out <- vec_c(y, y)
  exp <- new_table(
    matrix(int(1, 0, 1, 0, 0, 1, 0, 1), nrow = 4),
    dim = c(4L, 2L),
    dimnames = list(c("1", "2", "1", "2"), NULL)
  )
  expect_identical(out, exp)

  out <- vec_rbind(y, y)
  exp <- new_data_frame(list(
    `3` = int(1, 0, 1, 0),
    `4` = int(0, 1, 0, 1)
    ),
    row.names = c("1...1", "2...2", "1...3", "2...4")
  )
  expect_identical(out, exp)

  skip("FIXME: dimnames of matrices are not properly concatenated")
})

test_that("can concatenate tables of type double (#1190)", {
  x <- table(c(1, 2)) / 2

  out <- vec_c(x, x)
  exp <- new_table(c(0.5, 0.5, 0.5, 0.5), dimnames = list(c("1", "2", "1", "2")))
  expect_identical(out, exp)

  out <- vec_rbind(x, x)
  exp <- data_frame(`1` = new_table(c(0.5, 0.5)), `2` = new_table(c(0.5, 0.5)))
  expect_identical(out, exp)
})

expect_dim <- function(x, shape) {
  dim <- dim2(x)
  expect_equal(dim, !!shape)
}

expect_lossy <- function(expr, result, x = NULL, to = NULL) {
  expr <- enquo(expr)
  expect_error(eval_tidy(expr), class = "vctrs_error_cast_lossy")

  out <- allow_lossy_cast(eval_tidy(expr), x_ptype = x, to_ptype = to)
  expect_identical(!!out, !!result)
}

expect_args <- function(x, y, x_arg, y_arg) {
  err <- catch_cnd(vec_ptype2(x, y, x_arg = x_arg, y_arg = y_arg), classes = "vctrs_error_incompatible_type")
  expect_true(!is_null(err))

  expect_true(grepl(paste0("combine `", x_arg, "`"), err$message, fixed = TRUE))
  expect_true(grepl(paste0("and `", y_arg, "`"), err$message, fixed = TRUE))

  expect_identical(list(err$x_arg, err$y_arg), list(x_arg, y_arg))
}

# Work around deparsing of !! on old versions of R
as_label2 <- function(expr) {
  expr <- duplicate(expr, shallow = FALSE)

  label <- as_label(fix_bang(expr))
  label <- gsub("+++", "!!!", label, fixed = TRUE)
  label <- gsub("++", "!!", label, fixed = TRUE)
  label
}
fix_bang <- function(expr) {
  curr <- expr
  while (!is_null(curr)) {
    car <- node_car(curr)

    if (is_triple_bang(car)) {
      replace_triple_bang(car)
    } else if (is_double_bang(car)) {
      replace_double_bang(car)
    } else if (is_call(car)) {
      node_poke_car(curr, fix_bang(car))
    }

    curr <- node_cdr(curr)
  }

  expr
}
is_double_bang <- function(expr) {
  is_call(expr, "!") && is_call(node_cadr(expr), "!")
}
is_triple_bang <- function(expr) {
  is_double_bang(expr) && is_call(node_cadr(node_cadr(expr)), "!")
}
replace_double_bang <- function(expr) {
  node_poke_car(expr, sym("+"))
  node_poke_car(node_cadr(expr), sym("+"))
}
replace_triple_bang <- function(expr) {
  replace_double_bang(expr)
  node_poke_car(node_cadr(node_cadr(expr)), sym("+"))
}

expect_syntactic <- function(name, exp_syn_name) {
  expect_identical(
    syn_name <- make_syntactic(name),
    exp_syn_name
  )
  expect_identical(syn_name, make.names(syn_name))
}

expect_error_cnd <- function(object, class, message = NULL, ..., .fixed = TRUE) {
  cnd <- expect_error(object, regexp = message, class = class[[1]], fixed = .fixed)
  expect_true(inherits_all(cnd, class))

  exp_fields <- list2(...)
  expect_true(is_empty(setdiff(!!names(exp_fields), names(cnd))))
  expect_equal(cnd[names(exp_fields)], exp_fields)
}

scrub_internal_error_line_number <- function(x) {
  # Because it varies by OS
  sub(pattern = "at line [[:digit:]]+", replacement = "at line <scrubbed>", x = x)
}

test_that("has ok print method", {
  pf <- vec_ptype2(partial_frame(x = 1L), data.frame(y = 2))
  expect_snapshot(pf)
  expect_equal(vec_ptype_abbr(pf), "prtl")
})

test_that("order of variables comes from data", {
  pf <- partial_frame(y = 1, x = 2)
  df <- data.frame(x = 1, y = 2)

  expect_named(vec_ptype_common(pf, df), c("x", "y"))
  expect_named(vec_ptype_common(df, pf), c("x", "y"))
})

test_that("partial variables added to end if not in data", {
  pf <- partial_frame(y = 1)
  df <- data.frame(x = 1)
  expect_named(vec_ptype_common(pf, df), c("x", "y"))
  expect_named(vec_ptype_common(df, pf), c("x", "y"))
})

test_that("can assert partial frames based on column presence", {
  pf <- partial_frame(y = 1)

  expect_true(vec_is(data.frame(y = 2), pf))
  expect_false(vec_is(data.frame(x = 1), pf))

  expect_true(vec_is(data.frame(x = 1, y = 2), pf))
  expect_true(vec_is(data.frame(x = 1, y = 2, z = 3), pf))

  pf <- partial_frame(y = 1, z = 3)

  expect_false(vec_is(data.frame(y = 2), pf))
  expect_false(vec_is(data.frame(x = 1), pf))
  expect_false(vec_is(data.frame(x = 1, y = 2), pf))
  expect_true(vec_is(data.frame(x = 1, y = 2, z = 3), pf))
})

test_that("can assert partial frames based on column type", {
  pf <- partial_frame(y = 1)
  expect_false(vec_is(data.frame(y = "2"), pf))
})

test_that("incompatible data frames are an error", {
  df <- data.frame(y = 1)
  expect_error(vec_ptype2(df, partial_frame(y = chr())), class = "vctrs_error_incompatible_type")
  expect_error(new_partial_frame(df, data.frame(y = chr())), class = "vctrs_error_incompatible_type")
})

test_that("dispatch is symmetric with tibbles", {
  # Now causes infloop with the new double-dispatch mechanism because
  # of the way we call vec_ptype2() from is_same_type() for partial
  # types
  return(expect_true(TRUE))

  left <- vec_ptype2(partial_frame(x = 1), tibble::tibble(x = 1))
  right <- vec_ptype2(tibble::tibble(x = 1), partial_frame(x = 1))
  expect_identical(left, right)
})

test_that("can take the common type with partial frames", {
  exp <- data.frame(x = dbl(), y = chr(), a = chr())

  out <- vec_ptype_common(
    partial_frame(x = double(), a = character()),
    data.frame(x = 1L, y = "a")
  )
  expect_identical(out, exp)

  out <- vec_ptype_common(
    data.frame(x = 1L, y = "a"),
    partial_frame(x = double(), a = character())
  )
  expect_identical(out, data.frame(x = dbl(), y = chr(), a = chr()))
})

test_that("can rbind with a partial frame prototype", {
  out <- vec_rbind(
    data.frame(x = 1L, y = "a"),
    data.frame(x = FALSE, z = 10),
    .ptype = partial_frame(x = double(), a = character())
  )

  exp <- data.frame(
    x = dbl(1, 0),
    y = chr("a", NA),
    z = dbl(NA, 10),
    a = chr(NA, NA)
  )

  expect_identical(out, exp)
})

test_that("has ok print method", {
  partial <- partial_factor("x")
  expect_snapshot(partial)

  both <- vec_ptype2(partial, factor("y"))
  expect_snapshot(both)

  empty <- partial_factor()
  expect_snapshot(empty)

  learned <- vec_ptype2(empty, factor("y"))
  expect_snapshot(learned)

  expect_equal(vec_ptype_abbr(partial), "prtl_fctr")
})

test_that("order of levels comes from data", {
  pfctr <- partial_factor(c("y", "x"))
  fctr <- factor(levels = c("x", "y"))

  expect_equal(levels(vec_ptype_common(pfctr, fctr)), c("x", "y"))
  expect_equal(levels(vec_ptype_common(fctr, pfctr)), c("x", "y"))
})

test_that("partial levels added to end if not in data", {
  pfctr <- partial_factor("y")
  fctr <- factor(levels = "x")

  expect_equal(levels(vec_ptype_common(pfctr, fctr)), c("x", "y"))
  expect_equal(levels(vec_ptype_common(fctr, pfctr)), c("x", "y"))
})

test_that("can assert partial factors based on level presence", {
  pfctr <- partial_factor("y")

  expect_true(vec_is(factor("y"), pfctr))
  expect_false(vec_is(factor("x"), pfctr))

  expect_true(vec_is(factor(c("x", "y")), pfctr))

  pfctr <- partial_factor(c("y", "z"))

  expect_false(vec_is(factor("y"), pfctr))
  expect_true(vec_is(factor(c("y", "z")), pfctr))
  expect_true(vec_is(factor(c("x", "y", "z")), pfctr))
})

# TODO - why is this not working?
# test_that("can assert partial factor based on factor type", {
#   pfctr <- partial_factor()
#   expect_false(vec_is(1, pfctr))
# })

test_that("interleaving is working as expected", {
  expect_identical(
    vec_interleave(1:3, 4:6),
    c(1L, 4L, 2L, 5L, 3L, 6L)
  )
  expect_identical(
    vec_interleave(1:3, 4:6, 7:9),
    c(1L, 4L, 7L, 2L, 5L, 8L, 3L, 6L, 9L)
  )
})

test_that("data frames can be interleaved", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  y <- data_frame(x = 3:4, y = c("c", "d"))

  expect_identical(
    vec_interleave(x, y),
    vec_slice(vec_c(x, y), c(1, 3, 2, 4))
  )
})

test_that("works with `NULL` inputs", {
  expect_identical(
    vec_interleave(1:3, NULL, 4:6),
    vec_interleave(1:3, 4:6)
  )
})

test_that("allows for name repair", {
  x <- c(x = 1)

  expect_identical(
    vec_interleave(x, x),
    c(x = 1, x = 1)
  )

  expect_snapshot(vec_interleave(x, x, .name_repair = "unique"))
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_interleave(c(x = 1), c(x = 2), .name_repair = "unique_quiet")
    res_universal <- vec_interleave(c("if" = 1), c("in" = 2), .name_repair = "universal_quiet")
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in"))
})

test_that("works with name specs", {
  x <- c(x = 1)
  y <- 1

  expect_named(
    vec_interleave(x = x, y = y, .name_spec = "{outer}_{inner}"),
    c("x_x", "y")
  )
})

test_that("recycles inputs", {
  expect_identical(
    vec_interleave(1:3, NA),
    c(1L, NA, 2L, NA, 3L, NA)
  )
  expect_identical(
    vec_interleave(integer(), NA),
    integer()
  )
})

test_that("works with no inputs", {
  expect_identical(vec_interleave(), NULL)
})

test_that("works with length zero input", {
  expect_identical(vec_interleave(integer(), integer()), integer())
})

test_that("respects ptype", {
  expect_identical(vec_interleave(.ptype = character()), character())
  expect_identical(vec_interleave(1L, 2L, .ptype = numeric()), c(1, 2))
})

test_that("uses recycling errors", {
  expect_snapshot(error = TRUE, vec_interleave(1:2, 1:3))
})

test_that("errors if the result would be a long vector", {
  # Internal multiplication overflows `r_ssize` resulting in a different error
  skip_on_os("windows")

  expect_snapshot(
    error = TRUE,
    vec_interleave_indices(3L, 1e9L)
  )
})

test_that("can split empty vector", {
  out <- vec_split(integer(), character())

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, character())
  expect_equal(out$val, list())
})

test_that("split data frame with data frame", {
  df <- data.frame(x = c(1, 1, 2), y = c(1, 1, 1))
  out <- vec_split(df, df)

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, data.frame(x = c(1, 2), y = c(1, 1)))
  expect_equal(out$val, list(
    data.frame(x = c(1, 1), y = c(1, 1)),
    data.frame(x = 2, y = 1)
  ))
})

test_that("x and by must be same size", {
  expect_error(
    vec_split(1:3, 1:2),
    "same size"
  )
})

test_that("split takes the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)
  expect_identical(nrow(vec_split(1:3, x)), 2L)
})

test_that("split works with different encodings", {
  encs <- encodings()
  expect_identical(nrow(vec_split(1:3, encs)), 1L)
})

test_that("`key` and `value` retain names", {
  x <- c(a = 1, b = 2, c = 1, a = 1)
  split <- vec_split(x, x)
  expect_identical(split$key, c(a = 1, b = 2))
  expect_identical(split$val[[1]], c(a = 1, c = 1, a = 1))
  expect_identical(split$val[[2]], c(b = 2))
})

# ------------------------------------------------------------------------------
# vec_detect_missing()

test_that("can detect different types of NA", {
  expect_true(vec_detect_missing(NA))
  expect_true(vec_detect_missing(NA_integer_))
  expect_true(vec_detect_missing(NA_real_))
  expect_true(vec_detect_missing(NA_complex_))
  expect_true(vec_detect_missing(complex(real = NA, imaginary = 1)))
  expect_true(vec_detect_missing(NaN))
  expect_true(vec_detect_missing(NA_character_))
  expect_true(vec_detect_missing(list(NULL)))
})

test_that("can detect different types of NA in data frames", {
  # using multiple columns to prevent proxy unwrapping
  expect_true(vec_detect_missing(data.frame(x = NA, y = NA)))
  expect_true(vec_detect_missing(data.frame(x = NA_integer_, y = NA_integer_)))
  expect_true(vec_detect_missing(data.frame(x = NA_real_, y = NaN)))
  expect_true(vec_detect_missing(data.frame(x = NA_complex_, y = NA_complex_)))
  expect_true(vec_detect_missing(data.frame(x = complex(real = NA, imaginary = 1), y = complex(real = 1, imaginary = NA))))
  expect_true(vec_detect_missing(data.frame(x = NA_character_, y = NA_character_)))
  expect_true(vec_detect_missing(new_data_frame(list(x = list(NULL), y = list(NULL)))))
})

test_that("raw vectors can never be NA", {
  expect_false(vec_detect_missing(raw(1)))
  expect_false(vec_detect_missing(data.frame(x = raw(1), y = raw(1))))
})

test_that("vectorised over rows of a data frame", {
  df <- data.frame(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA))
  expect_equal(vec_detect_missing(df), c(FALSE, FALSE, FALSE, TRUE))
})

test_that("works recursively with data frame columns", {
  df <- data.frame(x = c(1, 1, NA, NA))
  df$df <- data.frame(y = c(NA, 1, 1, NA), z = c(1, NA, 1, NA))
  expect_equal(vec_detect_missing(df), c(FALSE, FALSE, FALSE, TRUE))
})

test_that("0 row, N col data frame always returns `logical()` (#1585)", {
  expect_identical(vec_detect_missing(data_frame()), logical())
  expect_identical(vec_detect_missing(data_frame(x = integer(), y = double())), logical())
})

test_that(">0 row, 0 col data frame always returns `TRUE` for each row (#1585)", {
  # `vec_detect_missing()` returns `TRUE` for each row because it (in theory) does
  # `all()` on each row, and since there are 0 columns we get
  # `all(logical()) == TRUE` for each row.
  expect_identical(
    vec_detect_missing(data_frame(.size = 2L)),
    c(TRUE, TRUE)
  )
})

test_that("works with `NULL` input (#1494)", {
  expect_identical(vec_detect_missing(NULL), logical())
})

# ------------------------------------------------------------------------------
# vec_any_missing()

test_that("can check for any missing with all base vector types", {
  expect_false(vec_any_missing(TRUE))
  expect_false(vec_any_missing(1L))
  expect_false(vec_any_missing(1))
  expect_false(vec_any_missing(complex(real = 1, imaginary = 1)))
  expect_false(vec_any_missing("1"))
  expect_false(vec_any_missing(list(1)))

  expect_true(vec_any_missing(c(TRUE, NA)))
  expect_true(vec_any_missing(c(1L, NA_integer_)))
  expect_true(vec_any_missing(c(1, NA_real_)))
  expect_true(vec_any_missing(complex(real = c(1, NA), imaginary = c(1, NA))))
  expect_true(vec_any_missing(c("1", NA_character_)))
  expect_true(vec_any_missing(list(1, NULL)))
})

test_that("raw vectors can never be missing", {
  expect_false(vec_any_missing(raw(1)))
  expect_false(vec_any_missing(data.frame(x = raw(1), y = raw(1))))
})

test_that("works with empty vectors", {
  # Like `any(logical())`
  expect_false(vec_any_missing(integer()))
})

test_that("correctly detects complex missingness", {
  expect_false(vec_any_missing(complex(real = 1, imaginary = 1)))
  expect_true(vec_any_missing(complex(real = 1, imaginary = NA)))
  expect_true(vec_any_missing(complex(real = NA, imaginary = 1)))
  expect_true(vec_any_missing(complex(real = NA, imaginary = NA)))
})

test_that("treats NaN as missing", {
  expect_true(vec_any_missing(NaN))
})

test_that("works with `NULL` input", {
  expect_false(vec_any_missing(NULL))
})

test_that("entire row of a data frame must be missing", {
  df <- data.frame(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA))
  expect_true(vec_any_missing(df))
  expect_false(vec_any_missing(df[-4,]))
})

test_that("works recursively with data frame columns", {
  df <- data.frame(x = c(1, 1, NA, NA))
  df$df <- data.frame(y = c(NA, 1, 1, NA), z = c(1, NA, 1, NA))
  expect_true(vec_any_missing(df))
  expect_false(vec_any_missing(df[-4,]))
})

test_that("0 row, N col data frame always returns `FALSE` (#1585)", {
  df <- data_frame()

  expect_false(vec_any_missing(df))
  expect_false(vec_any_missing(data_frame(x = integer(), y = double())))

  # This is consistent with `vec_detect_missing()` returning `logical()` for 0 row
  # data frames. Then `any(logical()) == FALSE` to get `vec_any_missing()`.
  expect_identical(
    vec_any_missing(df),
    any(vec_detect_missing(df))
  )
})

test_that(">0 row, 0 col data frame always returns `TRUE` (#1585)", {
  df <- data_frame(.size = 2L)

  expect_true(vec_any_missing(df))

  # This is consistent with `vec_detect_missing()` returning `TRUE` for each row
  # because it (in theory) does `all()` on each row, and since there are 0
  # columns we get `all(logical()) == TRUE` for each row.
  # Then `any(c(TRUE, TRUE)) == TRUE` to get `vec_any_missing()`.
  expect_identical(
    vec_any_missing(df),
    any(vec_detect_missing(df))
  )
})

test_that("can rank with different types of `ties`", {
  x <- c(2L, 5L, 1L, 1L, 2L)

  expect_identical(vec_rank(x, ties = "min"), rank(x, ties.method = "min"))
  expect_identical(vec_rank(x, ties = "max"), rank(x, ties.method = "max"))
  expect_identical(vec_rank(x, ties = "sequential"), rank(x, ties.method = "first"))
  expect_identical(vec_rank(x, ties = "dense"), c(2L, 3L, 1L, 1L, 2L))
})

test_that("can rank in descending order", {
  x <- c(2L, 5L, 1L, 1L, 2L)

  expect_identical(vec_rank(x, ties = "min", direction = "desc"), rank(-x, ties.method = "min"))
  expect_identical(vec_rank(x, ties = "max", direction = "desc"), rank(-x, ties.method = "max"))
  expect_identical(vec_rank(x, ties = "sequential", direction = "desc"), rank(-x, ties.method = "first"))
  expect_identical(vec_rank(x, ties = "dense", direction = "desc"), c(2L, 1L, 3L, 3L, 2L))
})

test_that("can rank incomplete values with `NA`", {
  x <- c(2, NA, 4, NaN, 4, 2, NA)

  expect_identical(vec_rank(x, ties = "min", incomplete = "na"), rank(x, ties.method = "min", na.last = "keep"))
  expect_identical(vec_rank(x, ties = "max", incomplete = "na"), rank(x, ties.method = "max", na.last = "keep"))
  expect_identical(vec_rank(x, ties = "sequential", incomplete = "na"), rank(x, ties.method = "first", na.last = "keep"))
  expect_identical(vec_rank(x, ties = "dense", incomplete = "na"), c(1L, NA, 2L, NA, 2L, 1L, NA))

  # NaN are treated as missing, regardless of whether or not they are distinct from NA_real_
  expect_identical(
    vec_rank(x, ties = "min", incomplete = "na", nan_distinct = TRUE),
    vec_rank(x, ties = "min", incomplete = "na", nan_distinct = FALSE)
  )
})

test_that("works correctly when `incomplete = 'na'` with no missing values", {
  x <- c(1, 2, 1, 5, 2)
  expect_identical(vec_rank(x, incomplete = "na"), rank(x, ties.method = "min"))
})

test_that("when ranking incomplete values, all NA (or NaN) values get the same rank", {
  # this is in contrast to rank(), which treats all NA (NaN) as different
  x <- c(1, NA, 3, NaN, NA, 1, NaN)

  expect_identical(vec_rank(x, na_value = "largest"), c(1L, 4L, 3L, 4L, 4L, 1L, 4L))
  expect_identical(vec_rank(x, na_value = "smallest"), c(5L, 1L, 7L, 1L, 1L, 5L, 1L))

  # If distinct, NaN are always ranked between real numbers and NA_real_
  expect_identical(vec_rank(x, na_value = "largest", nan_distinct = TRUE), c(1L, 6L, 3L, 4L, 6L, 1L, 4L))
  expect_identical(vec_rank(x, na_value = "smallest", nan_distinct = TRUE), c(5L, 1L, 7L, 3L, 1L, 5L, 3L))
})

test_that("ranks character vectors in the C locale", {
  x <- c("B", "b", "a")
  expect_identical(vec_rank(x), c(1L, 3L, 2L))
})

test_that("works with data frames", {
  df <- data_frame(
    x = c(1, 2, 1, 2, 2),
    y = c(2, 2, 1, 2, 5)
  )

  expect_identical(vec_rank(df, ties = "min"), c(2L, 3L, 1L, 3L, 5L))
  expect_identical(vec_rank(df, ties = "sequential"), c(2L, 3L, 1L, 4L, 5L))
})

test_that("works with data frames with 0 columns and >0 rows (#1863)", {
  # All rows are treated as being from the same group
  df <- data_frame(.size = 5)

  expect_identical(vec_rank(df, ties = "min"), c(1L, 1L, 1L, 1L, 1L))
  expect_identical(vec_rank(df, ties = "sequential"), c(1L, 2L, 3L, 4L, 5L))
  expect_identical(vec_rank(df, ties = "sequential", direction = "desc"), c(1L, 2L, 3L, 4L, 5L))
})

test_that("works with data frames with 0 columns and 0 rows (#1863)", {
  df <- data_frame(.size = 0)

  expect_identical(vec_rank(df, ties = "min"), integer())
  expect_identical(vec_rank(df, ties = "sequential"), integer())
})

test_that("can control the direction per column", {
  df <- data_frame(
    x = c(1, 2, 1, 2, 2),
    y = c(2, 2, 1, 2, 5)
  )

  df2 <- df
  df2$y <- -df2$y

  expect_identical(
    vec_rank(df, direction = c("asc", "desc")),
    vec_rank(df2, direction = "asc")
  )
})

test_that("incompleteness is respected in data frames and rcrds", {
  df <- data_frame(
    x = c(1, NA, NA, 1),
    y = c(NA, NA, 1, 1)
  )

  expect_identical(vec_rank(df, incomplete = "na"), c(NA, NA, NA, 1L))
  expect_identical(vec_rank(df, incomplete = "na", direction = "desc"), c(NA, NA, NA, 1L))

  x <- new_rcrd(list(
    x = c(1, 1, NA, NA, 1),
    y = c(1, NA, 1, NA, 1)
  ))

  expect_identical(vec_rank(x, incomplete = "na"), c(1L, NA, NA, NA, 1L))
})

test_that("can control `na_value` per column", {
  df <- data_frame(
    x = c(1, 1, NA, NA, NA),
    y = c(3, NA, NA, 2, 1)
  )

  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest")),
    c(2L, 1L, 3L, 5L, 4L)
  )
  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest"), direction = "desc"),
    c(4L, 5L, 3L, 1L, 2L)
  )

  # But `incomplete = "na"` overrules it
  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest"), incomplete = "na"),
    c(1L, NA, NA, NA, NA)
  )
  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest"), incomplete = "na", direction = "desc"),
    c(1L, NA, NA, NA, NA)
  )
})

test_that("`x` must be a vector", {
  expect_error(vec_rank(identity), class = "vctrs_error_scalar_type")
})

test_that("`ties` is validated", {
  expect_snapshot(error = TRUE, vec_rank(1, ties = "foo"))
  expect_snapshot(error = TRUE, vec_rank(1, ties = 1))
})

test_that("`incomplete` is validated", {
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = NA))
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = c(TRUE, FALSE)))
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = "foo"))
})

encodings <- function(bytes = FALSE) {
  string <- "\u00B0C"

  utf8 <- iconv(string, from = Encoding(string), to = "UTF-8")
  unknown <- iconv(string, from = Encoding(string), to = "", mark = FALSE)
  latin1 <- iconv(string, from = Encoding(string), to = "latin1")

  out <- list(utf8 = utf8, unknown = unknown, latin1 = latin1)

  if (bytes) {
    out <- list2(!!! out, bytes = encoding_bytes())
  }

  out
}

encoding_bytes <- function() {
  string <- "\u00B0C"

  unknown <- iconv(string, from = Encoding(string), to = "", mark = FALSE)

  bytes <- unknown
  Encoding(bytes) <- "bytes"

  bytes
}

expect_equal_encoding <- function(object, expected) {
  args <- vec_recycle_common(object, expected)
  expect_equal(Encoding(args[[1L]]), Encoding(args[[2L]]))
}

# Never run on CRAN, even if they have data.table, because we don't regularly
# check these on CI and we don't want a change in data.table to force a CRAN
# failure for vctrs.
skip_on_cran()

# Avoids adding `data.table` to Suggests.
# These tests are only run on the devs' machines.
testthat_import_from("data.table", "data.table")

test_that("common type of data.table and data.frame is data.table", {
  expect_identical(
    vec_ptype2(data.table(x = TRUE), data.table(y = 2)),
    data.table(x = lgl(), y = dbl())
  )
  expect_identical(
    vec_ptype2(data.table(x = TRUE), data.frame(y = 2)),
    data.table(x = lgl(), y = dbl())
  )
  expect_identical(
    vec_ptype2(data.frame(y = 2), data.table(x = TRUE)),
    data.table(y = dbl(), x = lgl())
  )

  expect_identical(
    vec_cast(data.table(y = 2), data.table(x = TRUE, y = 1L)),
    data.table(x = NA, y = 2L)
  )
  expect_identical(
    vec_cast(data.frame(y = 2), data.table(x = TRUE, y = 1L)),
    data.table(x = NA, y = 2L)
  )
  expect_identical(
    vec_cast(data.table(y = 2), data.frame(x = TRUE, y = 1L)),
    data.frame(x = NA, y = 2L)
  )
})

test_that("data.table and tibble do not have a common type", {
  expect_equal(
    vec_ptype_common(data.table(x = TRUE), tibble(y = 2)),
    tibble(x = lgl(), y = dbl())
  )
  expect_equal(
    vec_ptype_common(tibble(y = 2), data.table(x = TRUE)),
    tibble(y = dbl(), x = lgl())
  )

  expect_identical(
    vec_cast(data.table(y = 2), tibble(x = TRUE, y = 1L)),
    tibble(x = lgl(NA), y = 2L)
  )
  expect_identical(
    vec_cast(tibble(y = 2), data.table(x = TRUE, y = 1L)),
    data_frame(x = lgl(NA), y = 2L)
  )
})

test_that("data table has formatting methods", {
  expect_snapshot({
    dt <- data.table(x = 1, y = 2, z = 3)
    vec_ptype_abbr(dt)
    vec_ptype_full(dt)
  })
})

test_that("sclr is a named list", {
  x <- new_sclr(x = 1, y = 2)
  expect_type(x, "list")
  expect_s3_class(x, "vctrs_sclr")
  expect_named(x, c("x", "y"))
})

test_that("scalar must have unique names", {
  expect_error(new_sclr(x = 1, x = 2), "not TRUE")
})

test_that("can get and set existing fields", {
  x <- new_sclr(x = 1, y = 2)

  x$x <- 3
  expect_equal(x$x, 3)
  x[["y"]] <- 4
  expect_equal(x[["y"]], 4)

  expect_error(x$z, "Invalid index")
  expect_error(x$z <- 1, "Invalid index")
})

test_that("as.list strips attributes apart from names", {
  x <- new_sclr(x = 1, y = 2)
  y <- as.list(x)

  expect_type(y, "list")
  expect_equal(attributes(y), list(names = names(x)))
})

test_that("as.data.frame works", {
  # #167: Not sure if this is the correct behavior.
  x <- new_sclr(x = 1, y = 2)
  expect_equal(
    as.data.frame(x, nm = "a"),
    new_data_frame(list(a = list(x)))
  )
})

test_that("putting in a data frame makes a list-col", {
  x <- new_sclr(x = 1, y = 2)
  df <- data.frame(x)

  expect_s3_class(df, "data.frame")
  expect_equal(df$x, list(x))
})

test_that("vector operations are unsupported", {
  x <- new_sclr(x = 1, y = 2)

  expect_error(x["a"], class = "vctrs_error_unsupported")
  expect_error(x["a"] <- 1, class = "vctrs_error_unsupported")

  expect_error(names(x) <- "x", class = "vctrs_error_unsupported")
  expect_error(dim(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(dimnames(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(levels(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(is.na(x) <- 1, class = "vctrs_error_unsupported")

  expect_error(c(x), class = "vctrs_error_unsupported")
  expect_error(abs(x), class = "vctrs_error_unsupported")
  expect_error(x + 1, class = "vctrs_error_unsupported")
  expect_error(Arg(x), class = "vctrs_error_unsupported")
  expect_error(sum(x), class = "vctrs_error_unsupported")
  expect_error(order(x), class = "vctrs_error_unsupported")
  expect_error(levels(x), class = "vctrs_error_unsupported")
  expect_error(t(x), class = "vctrs_error_unsupported")
  expect_error(unique(x), class = "vctrs_error_unsupported")
  expect_error(duplicated(x), class = "vctrs_error_unsupported")
  expect_error(anyDuplicated(x), class = "vctrs_error_unsupported")
  expect_error(as.logical(x), class = "vctrs_error_unsupported")
  expect_error(as.integer(x), class = "vctrs_error_unsupported")
  expect_error(as.double(x), class = "vctrs_error_unsupported")
  expect_error(as.character(x), class = "vctrs_error_unsupported")
  expect_error(as.Date(x), class = "vctrs_error_unsupported")
  expect_error(as.POSIXct(x), class = "vctrs_error_unsupported")
})

test_that("summary is unimplemented", {
  x <- new_sclr(x = 1, y = 2)
  expect_error(summary(x), class = "vctrs_error_unimplemented")
})

test_that("casting of integer64 works", {
  x <- bit64::as.integer64(1:10)
  expect_equal(vec_cast(x, bit64::integer64()), x)

  expect_equal(vec_cast(x, integer()), 1:10)
  expect_equal(vec_cast(1:10, bit64::integer64()), x)

  expect_equal(vec_cast(x, double()), as.double(x))
  expect_equal(vec_cast(as.numeric(1:10), bit64::integer64()), x)

  expect_equal(vec_cast(x, logical()), rep(TRUE, 10L))
  expect_equal(vec_cast(c(TRUE, FALSE), bit64::integer64()), bit64::as.integer64(c(1, 0)))

  expect_equal(vec_cast(NA, bit64::integer64()), bit64::as.integer64(NA))
  expect_equal(vec_cast(unspecified(2), bit64::integer64()), bit64::as.integer64(c(NA, NA)))

  expect_error(vec_cast(x, factor()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(factor(), x), class = "vctrs_error_incompatible_type")

  # These used to be allowed
  expect_error(vec_cast(x, character()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(as.character(1:10), bit64::integer64()), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2 for integer64 works", {
  x <- bit64::as.integer64(1:10)
  expect_equal(vec_ptype2(x, x), bit64::integer64())

  expect_equal(vec_ptype2(x, 1L), bit64::integer64())
  expect_equal(vec_ptype2(1L, x), bit64::integer64())

  expect_equal(vec_ptype2(x, TRUE), bit64::integer64())
  expect_equal(vec_ptype2(TRUE, x), bit64::integer64())

  expect_equal(vec_ptype2(x, NA), bit64::integer64())
  expect_equal(vec_ptype2(NA, x), bit64::integer64())

  expect_equal(vec_ptype2(unspecified(), x), bit64::integer64())
  expect_equal(vec_ptype2(x, unspecified()), bit64::integer64())

  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(x, ""), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2("", x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(data.frame(), x), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(x, data.frame()), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype_abbr.integer64", {
  expect_equal(vec_ptype_abbr(bit64::as.integer64(1:10)), "int64")
  expect_equal(vec_ptype_full(bit64::as.integer64(1:10)), "integer64")
})

test_that("can sort integer64", {
  x <- bit64::as.integer64(c(-1, -3, -2, 1))
  expect_identical(vec_order(x), int(2, 3, 1, 4))
  expect_identical(x[vec_order(x)], bit64::as.integer64(c(-3, -2, -1, 1)))
})

test_that("can slice integer64 objects of all dimensions", {
  x <- bit64::as.integer64(1:8)
  expect <- bit64::as.integer64(c(1, 3))
  expect_identical(vec_slice(x, c(1, 3)), expect)

  dim(x) <- c(4, 2)
  expect <- bit64::as.integer64(c(1, 3, 5, 7))
  dim(expect) <- c(2, 2)
  expect_identical(vec_slice(x, c(1, 3)), expect)

  dim(x) <- c(2, 2, 2)
  expect <- bit64::as.integer64(c(2, 4, 6, 8))
  dim(expect) <- c(1, 2, 2)
  expect_identical(vec_slice(x, 2), expect)
})

test_that("can slice integer64 objects with `NA_integer_`", {
  idx <- c(NA_integer_, 1)

  x <- bit64::as.integer64(1:8)
  expect <- bit64::as.integer64(c(NA, 1))
  expect_identical(vec_slice(x, idx), expect)

  dim(x) <- c(4, 2)
  expect <- bit64::as.integer64(c(NA, 1, NA, 5))
  dim(expect) <- c(2, 2)
  expect_identical(vec_slice(x, idx), expect)

  dim(x) <- c(2, 2, 2)
  expect <- bit64::as.integer64(c(NA, 1, NA, 3, NA, 5, NA, 7))
  dim(expect) <- c(2, 2, 2)
  expect_identical(vec_slice(x, idx), expect)
})

test_that("can init integer64 objects", {
  idx <- c(NA_integer_, NA_integer_)

  x <- bit64::as.integer64(1:8)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))

  dim(x) <- c(4, 2)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))

  dim(x) <- c(2, 2, 2)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))
})

test_that("can chop integer64 objects with `NA_integer_` indices", {
  idx <- list(NA_integer_, 1)

  x <- bit64::as.integer64(1:8)
  expect <- list(
    bit64::as.integer64(NA),
    bit64::as.integer64(1)
  )

  expect_identical(vec_chop(x, indices = idx), expect)

  dim(x) <- c(4, 2)
  expect <- list(
    bit64::as.integer64(c(NA, NA)),
    bit64::as.integer64(c(1, 5))
  )
  dim(expect[[1]]) <- c(1, 2)
  dim(expect[[2]]) <- c(1, 2)

  expect_identical(vec_chop(x, indices = idx), expect)

  dim(x) <- c(2, 2, 2)
  expect <- list(
    bit64::as.integer64(c(NA, NA, NA, NA)),
    bit64::as.integer64(c(1, 3, 5, 7))
  )
  dim(expect[[1]]) <- c(1, 2, 2)
  dim(expect[[2]]) <- c(1, 2, 2)

  expect_identical(vec_chop(x, indices = idx), expect)
})

test_that("equality proxy converts atomic input to data frames of doubles", {
  x <- bit64::as.integer64(1)
  expect_identical(
    vec_proxy_equal(x),
    data_frame(left = 2147483648, right = 1)
  )
})

test_that("equality proxy works with 1-D arrays", {
  x <- bit64::as.integer64(1:6)
  y <- x
  dim(y) <- 6

  expect_identical(
    vec_proxy_equal(x),
    vec_proxy_equal(y)
  )
})

test_that("equality proxy on >=2-D input converts to data frame and proxies each column", {
  x <- bit64::as.integer64(1:8)
  dim(x) <- c(2, 2, 2)

  proxy1 <- integer64_proxy(x[1:2, 1, 1])
  proxy2 <- integer64_proxy(x[1:2, 2, 1])
  proxy3 <- integer64_proxy(x[1:2, 1, 2])
  proxy4 <- integer64_proxy(x[1:2, 2, 2])

  expect_identical(
    vec_proxy_equal(x),
    vec_cbind(proxy1, proxy2, proxy3, proxy4, .name_repair = "minimal")
  )
})

test_that("can detect missing values with integer64 (#1304)", {
  x <- bit64::as.integer64(c(NA, NA, 2, NA, 2, 2))

  expect_identical(vec_detect_missing(x), c(TRUE, TRUE, FALSE, TRUE, FALSE, FALSE))

  dim(x) <- c(3, 2)
  expect_identical(vec_detect_missing(x), c(TRUE, FALSE, FALSE))
})

test_that("can fill missing values with integer64", {
  x <- bit64::as.integer64(c(NA, NA, 2, NA, 2, 2))

  expect <- bit64::as.integer64(c(NA, NA, 2, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "down"), expect)

  dim(x) <- c(3, 2)
  expect <- bit64::as.integer64(c(NA, NA, 2, 2, 2, 2))
  dim(expect) <- c(3, 2)
  expect_identical(vec_fill_missing(x, "up"), expect)
})

test_that("can compare values with integer64", {
  x <- bit64::as.integer64(c(1, NA, 2))
  y <- bit64::as.integer64(c(0, 2, 3))

  expect_identical(vec_compare(x, y), c(1L, NA, -1L))

  x <- bit64::as.integer64(1:8)
  y <- bit64::as.integer64(c(1, 2, 1, 5, 1, 5, 1, 5))
  dim(x) <- c(2, 2, 2)
  dim(y) <- c(2, 2, 2)

  expect_identical(vec_compare(x, y), c(1L, -1L))
})

test_that("integer64 <-> data frame works as expected", {
  x <- bit64::as.integer64(c(-2, -1, 0, 1))
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, c(2147483647, 2147483647, 2147483648, 2147483648))
  expect_identical(proxy$right, c(4294967294, 4294967295, 0, 1))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64("9223372036854775807") + -1:0
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, c(4294967295, 4294967295))
  expect_identical(proxy$right, c(4294967294, 4294967295))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64("-9223372036854775807") + 0:1
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, c(0, 0))
  expect_identical(proxy$right, c(1, 2))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64(NA)
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, NA_real_)
  expect_identical(proxy$right, NA_real_)
  expect_identical(integer64_restore(proxy), x)
})

test_that("`integer64_proxy()` doesn't allow arrays", {
  x <- bit64::as.integer64(1:6)
  dim(x) <- c(3, 2)
  expect_error(integer64_proxy(x), "should not have a `dim` attribute")
})

expect_dim <- function(x, shape) {
  dim <- dim2(x)
  expect_equal(dim, !!shape)
}

expect_lossy <- function(expr, result, x = NULL, to = NULL) {
  expr <- enquo(expr)
  expect_error(eval_tidy(expr), class = "vctrs_error_cast_lossy")

  out <- allow_lossy_cast(eval_tidy(expr), x_ptype = x, to_ptype = to)
  expect_identical(!!out, !!result)
}

expect_args <- function(x, y, x_arg, y_arg) {
  err <- catch_cnd(
    vec_ptype2(x, y, x_arg = x_arg, y_arg = y_arg),
    classes = "vctrs_error_incompatible_type"
  )
  expect_true(!is_null(err))

  expect_true(grepl(paste0("combine `", x_arg, "`"), err$message, fixed = TRUE))
  expect_true(grepl(paste0("and `", y_arg, "`"), err$message, fixed = TRUE))

  expect_identical(list(err$x_arg, err$y_arg), list(x_arg, y_arg))
}

# Work around deparsing of !! on old versions of R
as_label2 <- function(expr) {
  expr <- duplicate(expr, shallow = FALSE)

  label <- as_label(fix_bang(expr))
  label <- gsub("+++", "!!!", label, fixed = TRUE)
  label <- gsub("++", "!!", label, fixed = TRUE)
  label
}
fix_bang <- function(expr) {
  curr <- expr
  while (!is_null(curr)) {
    car <- node_car(curr)

    if (is_triple_bang(car)) {
      replace_triple_bang(car)
    } else if (is_double_bang(car)) {
      replace_double_bang(car)
    } else if (is_call(car)) {
      node_poke_car(curr, fix_bang(car))
    }

    curr <- node_cdr(curr)
  }

  expr
}
is_double_bang <- function(expr) {
  is_call(expr, "!") && is_call(node_cadr(expr), "!")
}
is_triple_bang <- function(expr) {
  is_double_bang(expr) && is_call(node_cadr(node_cadr(expr)), "!")
}
replace_double_bang <- function(expr) {
  node_poke_car(expr, sym("+"))
  node_poke_car(node_cadr(expr), sym("+"))
}
replace_triple_bang <- function(expr) {
  replace_double_bang(expr)
  node_poke_car(node_cadr(node_cadr(expr)), sym("+"))
}

expect_syntactic <- function(name, exp_syn_name) {
  expect_identical(
    syn_name <- make_syntactic(name),
    exp_syn_name
  )
  expect_identical(syn_name, make.names(syn_name))
}

expect_error_cnd <- function(
  object,
  class,
  message = NULL,
  ...,
  .fixed = TRUE
) {
  cnd <- expect_error(
    object,
    regexp = message,
    class = class[[1]],
    fixed = .fixed
  )
  expect_true(inherits_all(cnd, class))

  exp_fields <- list2(...)
  expect_true(is_empty(setdiff(!!names(exp_fields), names(cnd))))
  expect_equal(cnd[names(exp_fields)], exp_fields)
}

scrub_internal_error_line_number <- function(x) {
  # Because it varies by OS
  sub(
    pattern = "at line [[:digit:]]+",
    replacement = "at line <scrubbed>",
    x = x
  )
}

test_that("vec_recycle_common() reports error context", {
  my_function <- function(...) vec_recycle_common(...)
  expect_snapshot({
    (expect_error(my_function(this_arg = 1:2, that_arg = int())))
    (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2)))
    (expect_error(my_function(
      this_arg = 1:2,
      that_arg = int(),
      .arg = "my_arg"
    )))
    (expect_error(my_function(
      this_arg = 1:2,
      that_arg = int(),
      .size = 2,
      .arg = "my_arg"
    )))
  })
})


# vec_recycle() -------------------------------------------------------------

test_that("vec_recycle recycles size 1 to any other size", {
  x <- 1
  x0 <- numeric()
  x2 <- c(x, x)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)
})

test_that("vec_recycle(): incompatible lengths get error messages", {
  x2 <- c(1, 2)

  expect_snapshot({
    (expect_error(
      vec_recycle(x2, 1),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
  expect_error(
    vec_recycle(x2, 0),
    class = "vctrs_error_recycle_incompatible_size"
  )
  expect_error(
    vec_recycle(x2, 3),
    class = "vctrs_error_recycle_incompatible_size"
  )
})

test_that("can recycle arrays", {
  x <- matrix(1:2, 1)
  x2 <- matrix(1:2, 2, 2, byrow = TRUE)
  x0 <- matrix(integer(), 0, 2)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)

  # List arrays
  data <- c(list(1), list(2))
  x <- matrix(data, 1)
  x2 <- matrix(data, 2, 2, byrow = TRUE)
  x0 <- matrix(list(), 0, 2)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)
})

test_that("vec_recycle() evaluates x_arg lazily", {
  expect_silent(vec_recycle(1L, 1L, x_arg = print("oof")))
})

test_that("recycling to size 1 has informative error", {
  expect_snapshot({
    (expect_error(
      vec_recycle(1:2, 1),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

test_that("incompatible recycling size has informative error", {
  expect_snapshot(error = TRUE, vec_recycle(1:2, 4))
  expect_snapshot(error = TRUE, vec_recycle(1:2, 4, x_arg = "foo"))
})

# Empty -------------------------------------------------------------------

test_that("empty input returns empty list", {
  expect_equal(vec_recycle_common(), list())
})

# Vectors -----------------------------------------------------------------

test_that("NULL is idempotent", {
  expect_equal(vec_recycle_common(NULL, NULL), list(NULL, NULL))
  expect_equal(vec_recycle_common(1:5, NULL), list(1:5, NULL))
  expect_equal(vec_recycle_common(NULL, 1:5), list(NULL, 1:5))
})

test_that("equal lengths returned as is", {
  x <- 1:3

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x[1], x[1]), list(x[1], x[1]))
  expect_equal(vec_recycle_common(x[0], x[0]), list(x[0], x[0]))
})

test_that("vec_recycle_common recycles size 1 to any other size", {
  x1 <- 1
  x3 <- rep(1, 3)
  x0 <- numeric()

  expect_equal(vec_recycle_common(x1, x3), list(x3, x3))
  expect_equal(vec_recycle_common(x3, x1), list(x3, x3))
  expect_equal(vec_recycle_common(x1, x0), list(x0, x0))
})

test_that("vec_recycle_common(): incompatible lengths get error messages", {
  expect_snapshot({
    (expect_error(
      vec_recycle_common(1:2, 1:3),
      class = "vctrs_error_incompatible_size"
    ))
  })
  expect_error(
    vec_recycle_common(1:3, 1:2),
    class = "vctrs_error_incompatible_size"
  )
  expect_error(
    vec_recycle_common(numeric(), 1:2),
    class = "vctrs_error_incompatible_size"
  )
})

test_that("vec_recycle_common errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_recycle_common(1, lm(1 ~ 1))
  })

  # Index is correct with `NULL`s
  expect_snapshot(error = TRUE, {
    vec_recycle_common(1, NULL, lm(1 ~ 1))
  })
})

test_that("vec_recycle_common doesn't mutate the input in place", {
  x <- list(a = 1, b = 2:3)

  expect_identical(
    vec_recycle_common(!!!x),
    list(a = c(1, 1), b = 2:3)
  )
  expect_identical(
    x,
    list(a = 1, b = 2:3)
  )
})

test_that("vec_recycle_common retains names", {
  expect_named(vec_recycle_common(a = 1, b = 2), c("a", "b"))
  expect_named(vec_recycle_common(a = 1, b = 2:3), c("a", "b"))
})

# Matrices ----------------------------------------------------------------

test_that("can vec_recycle_common matrices", {
  x <- matrix(nrow = 4, ncol = 4)
  x1 <- x[1, , drop = FALSE]

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x1, x), list(x, x))
})

test_that("recycling matrices respects incompatible sizes", {
  x <- matrix(nrow = 4, ncol = 4)
  x2 <- x[1:2, , drop = FALSE]
  x0 <- x[0, , drop = FALSE]

  expect_snapshot({
    (expect_error(
      vec_recycle_common(x2, x),
      class = "vctrs_error_incompatible_size"
    ))
  })
  expect_error(
    vec_recycle_common(x0, x),
    class = "vctrs_error_incompatible_size"
  )
})

# Data frames ------------------------------------------------------------

test_that("can vec_recycle_common data frames", {
  x <- data.frame(a = rep(1, 3), b = rep(2, 3))
  x1 <- vec_slice(x, 1L)

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x1, x), list(x, x))
})

test_that("recycling data frames respects incompatible sizes", {
  x <- data.frame(a = rep(1, 3), b = rep(2, 3))
  x2 <- vec_slice(x, 1:2)
  x0 <- vec_slice(x, integer())

  expect_snapshot({
    (expect_error(
      vec_recycle_common(x2, x),
      class = "vctrs_error_incompatible_size"
    ))
  })
  expect_error(
    vec_recycle_common(x0, x),
    class = "vctrs_error_incompatible_size"
  )
})

test_that("can vec_recycle_common matrix and data frame", {
  mt <- matrix(nrow = 2, ncol = 2)
  df <- data.frame(x = c(1, 1), y = c(2, 2))

  expect_equal(
    vec_recycle_common(vec_slice(mt, 1L), df),
    list(mt, df)
  )

  expect_equal(
    vec_recycle_common(mt, vec_slice(df, 1L)),
    list(mt, df)
  )
})

test_that("recycling data frames with matrices respects incompatible sizes", {
  mt <- matrix(nrow = 2, ncol = 2)
  df <- data.frame(x = c(1, 1), y = c(2, 2))

  expect_error(
    vec_recycle_common(vec_slice(mt, integer()), df),
    class = "vctrs_error_incompatible_size"
  )

  expect_error(
    vec_recycle_common(mt, vec_slice(df, 0L)),
    class = "vctrs_error_incompatible_size"
  )
})

test_that("casting of integer64 works", {
  x <- bit64::as.integer64(1:10)
  expect_equal(vec_cast(x, bit64::integer64()), x)

  expect_equal(vec_cast(x, integer()), 1:10)
  expect_equal(vec_cast(1:10, bit64::integer64()), x)

  expect_equal(vec_cast(x, double()), as.double(x))
  expect_equal(vec_cast(as.numeric(1:10), bit64::integer64()), x)

  expect_equal(vec_cast(x, logical()), rep(TRUE, 10L))
  expect_equal(
    vec_cast(c(TRUE, FALSE), bit64::integer64()),
    bit64::as.integer64(c(1, 0))
  )

  expect_equal(vec_cast(NA, bit64::integer64()), bit64::as.integer64(NA))
  expect_equal(
    vec_cast(unspecified(2), bit64::integer64()),
    bit64::as.integer64(c(NA, NA))
  )

  expect_error(vec_cast(x, factor()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(factor(), x), class = "vctrs_error_incompatible_type")

  # These used to be allowed
  expect_error(
    vec_cast(x, character()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(as.character(1:10), bit64::integer64()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype2 for integer64 works", {
  x <- bit64::as.integer64(1:10)
  expect_equal(vec_ptype2(x, x), bit64::integer64())

  expect_equal(vec_ptype2(x, 1L), bit64::integer64())
  expect_equal(vec_ptype2(1L, x), bit64::integer64())

  expect_equal(vec_ptype2(x, TRUE), bit64::integer64())
  expect_equal(vec_ptype2(TRUE, x), bit64::integer64())

  expect_equal(vec_ptype2(x, NA), bit64::integer64())
  expect_equal(vec_ptype2(NA, x), bit64::integer64())

  expect_equal(vec_ptype2(unspecified(), x), bit64::integer64())
  expect_equal(vec_ptype2(x, unspecified()), bit64::integer64())

  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(x, ""), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2("", x), class = "vctrs_error_incompatible_type")

  expect_error(
    vec_ptype2(data.frame(), x),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(x, data.frame()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype_abbr.integer64", {
  expect_equal(vec_ptype_abbr(bit64::as.integer64(1:10)), "int64")
  expect_equal(vec_ptype_full(bit64::as.integer64(1:10)), "integer64")
})

test_that("can sort integer64", {
  x <- bit64::as.integer64(c(-1, -3, -2, 1))
  expect_identical(vec_order(x), int(2, 3, 1, 4))
  expect_identical(x[vec_order(x)], bit64::as.integer64(c(-3, -2, -1, 1)))
})

test_that("can slice integer64 objects of all dimensions", {
  x <- bit64::as.integer64(1:8)
  expect <- bit64::as.integer64(c(1, 3))
  expect_identical(vec_slice(x, c(1, 3)), expect)

  dim(x) <- c(4, 2)
  expect <- bit64::as.integer64(c(1, 3, 5, 7))
  dim(expect) <- c(2, 2)
  expect_identical(vec_slice(x, c(1, 3)), expect)

  dim(x) <- c(2, 2, 2)
  expect <- bit64::as.integer64(c(2, 4, 6, 8))
  dim(expect) <- c(1, 2, 2)
  expect_identical(vec_slice(x, 2), expect)
})

test_that("can slice integer64 objects with `NA_integer_`", {
  idx <- c(NA_integer_, 1)

  x <- bit64::as.integer64(1:8)
  expect <- bit64::as.integer64(c(NA, 1))
  expect_identical(vec_slice(x, idx), expect)

  dim(x) <- c(4, 2)
  expect <- bit64::as.integer64(c(NA, 1, NA, 5))
  dim(expect) <- c(2, 2)
  expect_identical(vec_slice(x, idx), expect)

  dim(x) <- c(2, 2, 2)
  expect <- bit64::as.integer64(c(NA, 1, NA, 3, NA, 5, NA, 7))
  dim(expect) <- c(2, 2, 2)
  expect_identical(vec_slice(x, idx), expect)
})

test_that("can init integer64 objects", {
  idx <- c(NA_integer_, NA_integer_)

  x <- bit64::as.integer64(1:8)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))

  dim(x) <- c(4, 2)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))

  dim(x) <- c(2, 2, 2)
  expect_identical(vec_init(x, 2), vec_slice(x, idx))
})

test_that("can chop integer64 objects with `NA_integer_` indices", {
  idx <- list(NA_integer_, 1)

  x <- bit64::as.integer64(1:8)
  expect <- list(
    bit64::as.integer64(NA),
    bit64::as.integer64(1)
  )

  expect_identical(vec_chop(x, indices = idx), expect)

  dim(x) <- c(4, 2)
  expect <- list(
    bit64::as.integer64(c(NA, NA)),
    bit64::as.integer64(c(1, 5))
  )
  dim(expect[[1]]) <- c(1, 2)
  dim(expect[[2]]) <- c(1, 2)

  expect_identical(vec_chop(x, indices = idx), expect)

  dim(x) <- c(2, 2, 2)
  expect <- list(
    bit64::as.integer64(c(NA, NA, NA, NA)),
    bit64::as.integer64(c(1, 3, 5, 7))
  )
  dim(expect[[1]]) <- c(1, 2, 2)
  dim(expect[[2]]) <- c(1, 2, 2)

  expect_identical(vec_chop(x, indices = idx), expect)
})

test_that("equality proxy converts atomic input to data frames of doubles", {
  x <- bit64::as.integer64(1)
  expect_identical(
    vec_proxy_equal(x),
    data_frame(left = 2147483648, right = 1)
  )
})

test_that("equality proxy works with 1-D arrays", {
  x <- bit64::as.integer64(1:6)
  y <- x
  dim(y) <- 6

  expect_identical(
    vec_proxy_equal(x),
    vec_proxy_equal(y)
  )
})

test_that("equality proxy on >=2-D input converts to data frame and proxies each column", {
  x <- bit64::as.integer64(1:8)
  dim(x) <- c(2, 2, 2)

  proxy1 <- integer64_proxy(x[1:2, 1, 1])
  proxy2 <- integer64_proxy(x[1:2, 2, 1])
  proxy3 <- integer64_proxy(x[1:2, 1, 2])
  proxy4 <- integer64_proxy(x[1:2, 2, 2])

  expect_identical(
    vec_proxy_equal(x),
    vec_cbind(proxy1, proxy2, proxy3, proxy4, .name_repair = "minimal")
  )
})

test_that("can detect missing values with integer64 (#1304)", {
  x <- bit64::as.integer64(c(NA, NA, 2, NA, 2, 2))

  expect_identical(
    vec_detect_missing(x),
    c(TRUE, TRUE, FALSE, TRUE, FALSE, FALSE)
  )

  dim(x) <- c(3, 2)
  expect_identical(vec_detect_missing(x), c(TRUE, FALSE, FALSE))
})

test_that("can fill missing values with integer64", {
  x <- bit64::as.integer64(c(NA, NA, 2, NA, 2, 2))

  expect <- bit64::as.integer64(c(NA, NA, 2, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "down"), expect)

  dim(x) <- c(3, 2)
  expect <- bit64::as.integer64(c(NA, NA, 2, 2, 2, 2))
  dim(expect) <- c(3, 2)
  expect_identical(vec_fill_missing(x, "up"), expect)
})

test_that("can compare values with integer64", {
  x <- bit64::as.integer64(c(1, NA, 2))
  y <- bit64::as.integer64(c(0, 2, 3))

  expect_identical(vec_compare(x, y), c(1L, NA, -1L))

  x <- bit64::as.integer64(1:8)
  y <- bit64::as.integer64(c(1, 2, 1, 5, 1, 5, 1, 5))
  dim(x) <- c(2, 2, 2)
  dim(y) <- c(2, 2, 2)

  expect_identical(vec_compare(x, y), c(1L, -1L))
})

test_that("integer64 <-> data frame works as expected", {
  x <- bit64::as.integer64(c(-2, -1, 0, 1))
  proxy <- integer64_proxy(x)

  expect_identical(
    proxy$left,
    c(2147483647, 2147483647, 2147483648, 2147483648)
  )
  expect_identical(proxy$right, c(4294967294, 4294967295, 0, 1))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64("9223372036854775807") + -1:0
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, c(4294967295, 4294967295))
  expect_identical(proxy$right, c(4294967294, 4294967295))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64("-9223372036854775807") + 0:1
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, c(0, 0))
  expect_identical(proxy$right, c(1, 2))
  expect_identical(integer64_restore(proxy), x)

  x <- bit64::as.integer64(NA)
  proxy <- integer64_proxy(x)

  expect_identical(proxy$left, NA_real_)
  expect_identical(proxy$right, NA_real_)
  expect_identical(integer64_restore(proxy), x)
})

test_that("`integer64_proxy()` doesn't allow arrays", {
  x <- bit64::as.integer64(1:6)
  dim(x) <- c(3, 2)
  expect_error(integer64_proxy(x), "should not have a `dim` attribute")
})

test_that("can compact missing elements", {
  x <- list(NULL, 1, NULL)
  expect_identical(list_drop_empty(x), list(1))
})

test_that("can compact empty elements", {
  x <- list(1, NULL, integer(), NULL)
  expect_identical(list_drop_empty(x), list(1))
})

test_that("emptyness works with data frames", {
  x <- data_frame()
  y <- data_frame(.size = 2L)

  lst <- list(x, y)

  expect_identical(list_drop_empty(lst), list(y))
})

test_that("emptyness works with rcrd types", {
  x <- new_rcrd(list(foo = integer(), bar = numeric()))
  y <- new_rcrd(list(foo = 1L, bar = 1))

  lst <- list(x, y)

  expect_identical(list_drop_empty(lst), list(y))
})

test_that("works with empty lists", {
  expect_identical(list_drop_empty(list()), list())
})

test_that("retains list type", {
  x <- list_of(NULL, integer())
  expect_identical(list_drop_empty(x), list_of(.ptype = integer()))
})

test_that("validates `x`", {
  expect_error(list_drop_empty(1), "must be a list")
  expect_error(list_drop_empty(data_frame()), "must be a list")

  # List arrays are not allowed, because we would not iterate over them
  # correctly
  expect_snapshot(error = TRUE, {
    x <- array(list(1), dim = c(1, 1))
    list_drop_empty(x)
  })
})

test_that("logical/integer/numeric works", {
  expect_equal(vec_arith("+", TRUE, TRUE), 2L)
  expect_equal(vec_arith("+", TRUE, 1L), 2L)
  expect_equal(vec_arith("+", TRUE, 1), 2)
  expect_equal(vec_arith("+", 1L, TRUE), 2L)
  expect_equal(vec_arith("+", 1L, 1L), 2L)
  expect_equal(vec_arith("+", 1L, 1), 2)
  expect_equal(vec_arith("+", 1, TRUE), 2L)
  expect_equal(vec_arith("+", 1, 1L), 2L)
  expect_equal(vec_arith("+", 1, 1), 2)
})

test_that("default is error", {
  f <- new_vctr(1:10, class = "foo")

  expect_error(vec_arith("+", f, 1), class = "vctrs_error_incompatible_op")

  expect_error(vec_arith("+", TRUE, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", 1L, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", 1, f), class = "vctrs_error_incompatible_op")
})

# ------------------------------------------------------------------------------
# vec_detect_missing()

test_that("can detect different types of NA", {
  expect_true(vec_detect_missing(NA))
  expect_true(vec_detect_missing(NA_integer_))
  expect_true(vec_detect_missing(NA_real_))
  expect_true(vec_detect_missing(NA_complex_))
  expect_true(vec_detect_missing(complex(real = NA, imaginary = 1)))
  expect_true(vec_detect_missing(NaN))
  expect_true(vec_detect_missing(NA_character_))
  expect_true(vec_detect_missing(list(NULL)))
})

test_that("can detect different types of NA in data frames", {
  # using multiple columns to prevent proxy unwrapping
  expect_true(vec_detect_missing(data.frame(x = NA, y = NA)))
  expect_true(vec_detect_missing(data.frame(x = NA_integer_, y = NA_integer_)))
  expect_true(vec_detect_missing(data.frame(x = NA_real_, y = NaN)))
  expect_true(vec_detect_missing(data.frame(x = NA_complex_, y = NA_complex_)))
  expect_true(vec_detect_missing(data.frame(
    x = complex(real = NA, imaginary = 1),
    y = complex(real = 1, imaginary = NA)
  )))
  expect_true(vec_detect_missing(data.frame(
    x = NA_character_,
    y = NA_character_
  )))
  expect_true(vec_detect_missing(new_data_frame(list(
    x = list(NULL),
    y = list(NULL)
  ))))
})

test_that("raw vectors can never be NA", {
  expect_false(vec_detect_missing(raw(1)))
  expect_false(vec_detect_missing(data.frame(x = raw(1), y = raw(1))))
})

test_that("vectorised over rows of a data frame", {
  df <- data.frame(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA))
  expect_equal(vec_detect_missing(df), c(FALSE, FALSE, FALSE, TRUE))
})

test_that("works recursively with data frame columns", {
  df <- data.frame(x = c(1, 1, NA, NA))
  df$df <- data.frame(y = c(NA, 1, 1, NA), z = c(1, NA, 1, NA))
  expect_equal(vec_detect_missing(df), c(FALSE, FALSE, FALSE, TRUE))
})

test_that("0 row, N col data frame always returns `logical()` (#1585)", {
  expect_identical(vec_detect_missing(data_frame()), logical())
  expect_identical(
    vec_detect_missing(data_frame(x = integer(), y = double())),
    logical()
  )
})

test_that(">0 row, 0 col data frame always returns `TRUE` for each row (#1585)", {
  # `vec_detect_missing()` returns `TRUE` for each row because it (in theory) does
  # `all()` on each row, and since there are 0 columns we get
  # `all(logical()) == TRUE` for each row.
  expect_identical(
    vec_detect_missing(data_frame(.size = 2L)),
    c(TRUE, TRUE)
  )
})

test_that("works with `NULL` input (#1494)", {
  expect_identical(vec_detect_missing(NULL), logical())
})

# ------------------------------------------------------------------------------
# vec_any_missing()

test_that("can check for any missing with all base vector types", {
  expect_false(vec_any_missing(TRUE))
  expect_false(vec_any_missing(1L))
  expect_false(vec_any_missing(1))
  expect_false(vec_any_missing(complex(real = 1, imaginary = 1)))
  expect_false(vec_any_missing("1"))
  expect_false(vec_any_missing(list(1)))

  expect_true(vec_any_missing(c(TRUE, NA)))
  expect_true(vec_any_missing(c(1L, NA_integer_)))
  expect_true(vec_any_missing(c(1, NA_real_)))
  expect_true(vec_any_missing(complex(real = c(1, NA), imaginary = c(1, NA))))
  expect_true(vec_any_missing(c("1", NA_character_)))
  expect_true(vec_any_missing(list(1, NULL)))
})

test_that("raw vectors can never be missing", {
  expect_false(vec_any_missing(raw(1)))
  expect_false(vec_any_missing(data.frame(x = raw(1), y = raw(1))))
})

test_that("works with empty vectors", {
  # Like `any(logical())`
  expect_false(vec_any_missing(integer()))
})

test_that("correctly detects complex missingness", {
  expect_false(vec_any_missing(complex(real = 1, imaginary = 1)))
  expect_true(vec_any_missing(complex(real = 1, imaginary = NA)))
  expect_true(vec_any_missing(complex(real = NA, imaginary = 1)))
  expect_true(vec_any_missing(complex(real = NA, imaginary = NA)))
})

test_that("treats NaN as missing", {
  expect_true(vec_any_missing(NaN))
})

test_that("works with `NULL` input", {
  expect_false(vec_any_missing(NULL))
})

test_that("entire row of a data frame must be missing", {
  df <- data.frame(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA))
  expect_true(vec_any_missing(df))
  expect_false(vec_any_missing(df[-4, ]))
})

test_that("works recursively with data frame columns", {
  df <- data.frame(x = c(1, 1, NA, NA))
  df$df <- data.frame(y = c(NA, 1, 1, NA), z = c(1, NA, 1, NA))
  expect_true(vec_any_missing(df))
  expect_false(vec_any_missing(df[-4, ]))
})

test_that("0 row, N col data frame always returns `FALSE` (#1585)", {
  df <- data_frame()

  expect_false(vec_any_missing(df))
  expect_false(vec_any_missing(data_frame(x = integer(), y = double())))

  # This is consistent with `vec_detect_missing()` returning `logical()` for 0 row
  # data frames. Then `any(logical()) == FALSE` to get `vec_any_missing()`.
  expect_identical(
    vec_any_missing(df),
    any(vec_detect_missing(df))
  )
})

test_that(">0 row, 0 col data frame always returns `TRUE` (#1585)", {
  df <- data_frame(.size = 2L)

  expect_true(vec_any_missing(df))

  # This is consistent with `vec_detect_missing()` returning `TRUE` for each row
  # because it (in theory) does `all()` on each row, and since there are 0
  # columns we get `all(logical()) == TRUE` for each row.
  # Then `any(c(TRUE, TRUE)) == TRUE` to get `vec_any_missing()`.
  expect_identical(
    vec_any_missing(df),
    any(vec_detect_missing(df))
  )
})

test_that("sclr is a named list", {
  x <- new_sclr(x = 1, y = 2)
  expect_type(x, "list")
  expect_s3_class(x, "vctrs_sclr")
  expect_named(x, c("x", "y"))
})

test_that("scalar must have unique names", {
  expect_error(new_sclr(x = 1, x = 2), "not TRUE")
})

test_that("can get and set existing fields", {
  x <- new_sclr(x = 1, y = 2)

  x$x <- 3
  expect_equal(x$x, 3)
  x[["y"]] <- 4
  expect_equal(x[["y"]], 4)

  expect_error(x$z, "Invalid index")
  expect_error(x$z <- 1, "Invalid index")
})

test_that("as.list strips attributes apart from names", {
  x <- new_sclr(x = 1, y = 2)
  y <- as.list(x)

  expect_type(y, "list")
  expect_equal(attributes(y), list(names = names(x)))
})

test_that("as.data.frame works", {
  # #167: Not sure if this is the correct behavior.
  x <- new_sclr(x = 1, y = 2)
  expect_equal(
    as.data.frame(x, nm = "a"),
    new_data_frame(list(a = list(x)))
  )
})

test_that("putting in a data frame makes a list-col", {
  x <- new_sclr(x = 1, y = 2)
  df <- data.frame(x)

  expect_s3_class(df, "data.frame")
  expect_equal(df$x, list(x))
})

test_that("vector operations are unsupported", {
  x <- new_sclr(x = 1, y = 2)

  expect_error(x["a"], class = "vctrs_error_unsupported")
  expect_error(x["a"] <- 1, class = "vctrs_error_unsupported")

  expect_error(names(x) <- "x", class = "vctrs_error_unsupported")
  expect_error(dim(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(dimnames(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(levels(x) <- 1, class = "vctrs_error_unsupported")
  expect_error(is.na(x) <- 1, class = "vctrs_error_unsupported")

  expect_error(c(x), class = "vctrs_error_unsupported")
  expect_error(abs(x), class = "vctrs_error_unsupported")
  expect_error(x + 1, class = "vctrs_error_unsupported")
  expect_error(Arg(x), class = "vctrs_error_unsupported")
  expect_error(sum(x), class = "vctrs_error_unsupported")
  expect_error(order(x), class = "vctrs_error_unsupported")
  expect_error(levels(x), class = "vctrs_error_unsupported")
  expect_error(t(x), class = "vctrs_error_unsupported")
  expect_error(unique(x), class = "vctrs_error_unsupported")
  expect_error(duplicated(x), class = "vctrs_error_unsupported")
  expect_error(anyDuplicated(x), class = "vctrs_error_unsupported")
  expect_error(as.logical(x), class = "vctrs_error_unsupported")
  expect_error(as.integer(x), class = "vctrs_error_unsupported")
  expect_error(as.double(x), class = "vctrs_error_unsupported")
  expect_error(as.character(x), class = "vctrs_error_unsupported")
  expect_error(as.Date(x), class = "vctrs_error_unsupported")
  expect_error(as.POSIXct(x), class = "vctrs_error_unsupported")
})

test_that("summary is unimplemented", {
  x <- new_sclr(x = 1, y = 2)
  expect_error(summary(x), class = "vctrs_error_unimplemented")
})

test_that("can rank with different types of `ties`", {
  x <- c(2L, 5L, 1L, 1L, 2L)

  expect_identical(vec_rank(x, ties = "min"), rank(x, ties.method = "min"))
  expect_identical(vec_rank(x, ties = "max"), rank(x, ties.method = "max"))
  expect_identical(
    vec_rank(x, ties = "sequential"),
    rank(x, ties.method = "first")
  )
  expect_identical(vec_rank(x, ties = "dense"), c(2L, 3L, 1L, 1L, 2L))
})

test_that("can rank in descending order", {
  x <- c(2L, 5L, 1L, 1L, 2L)

  expect_identical(
    vec_rank(x, ties = "min", direction = "desc"),
    rank(-x, ties.method = "min")
  )
  expect_identical(
    vec_rank(x, ties = "max", direction = "desc"),
    rank(-x, ties.method = "max")
  )
  expect_identical(
    vec_rank(x, ties = "sequential", direction = "desc"),
    rank(-x, ties.method = "first")
  )
  expect_identical(
    vec_rank(x, ties = "dense", direction = "desc"),
    c(2L, 1L, 3L, 3L, 2L)
  )
})

test_that("can rank incomplete values with `NA`", {
  x <- c(2, NA, 4, NaN, 4, 2, NA)

  expect_identical(
    vec_rank(x, ties = "min", incomplete = "na"),
    rank(x, ties.method = "min", na.last = "keep")
  )
  expect_identical(
    vec_rank(x, ties = "max", incomplete = "na"),
    rank(x, ties.method = "max", na.last = "keep")
  )
  expect_identical(
    vec_rank(x, ties = "sequential", incomplete = "na"),
    rank(x, ties.method = "first", na.last = "keep")
  )
  expect_identical(
    vec_rank(x, ties = "dense", incomplete = "na"),
    c(1L, NA, 2L, NA, 2L, 1L, NA)
  )

  # NaN are treated as missing, regardless of whether or not they are distinct from NA_real_
  expect_identical(
    vec_rank(x, ties = "min", incomplete = "na", nan_distinct = TRUE),
    vec_rank(x, ties = "min", incomplete = "na", nan_distinct = FALSE)
  )
})

test_that("works correctly when `incomplete = 'na'` with no missing values", {
  x <- c(1, 2, 1, 5, 2)
  expect_identical(vec_rank(x, incomplete = "na"), rank(x, ties.method = "min"))
})

test_that("when ranking incomplete values, all NA (or NaN) values get the same rank", {
  # this is in contrast to rank(), which treats all NA (NaN) as different
  x <- c(1, NA, 3, NaN, NA, 1, NaN)

  expect_identical(
    vec_rank(x, na_value = "largest"),
    c(1L, 4L, 3L, 4L, 4L, 1L, 4L)
  )
  expect_identical(
    vec_rank(x, na_value = "smallest"),
    c(5L, 1L, 7L, 1L, 1L, 5L, 1L)
  )

  # If distinct, NaN are always ranked between real numbers and NA_real_
  expect_identical(
    vec_rank(x, na_value = "largest", nan_distinct = TRUE),
    c(1L, 6L, 3L, 4L, 6L, 1L, 4L)
  )
  expect_identical(
    vec_rank(x, na_value = "smallest", nan_distinct = TRUE),
    c(5L, 1L, 7L, 3L, 1L, 5L, 3L)
  )
})

test_that("ranks character vectors in the C locale", {
  x <- c("B", "b", "a")
  expect_identical(vec_rank(x), c(1L, 3L, 2L))
})

test_that("works with data frames", {
  df <- data_frame(
    x = c(1, 2, 1, 2, 2),
    y = c(2, 2, 1, 2, 5)
  )

  expect_identical(vec_rank(df, ties = "min"), c(2L, 3L, 1L, 3L, 5L))
  expect_identical(vec_rank(df, ties = "sequential"), c(2L, 3L, 1L, 4L, 5L))
})

test_that("works with data frames with 0 columns and >0 rows (#1863)", {
  # All rows are treated as being from the same group
  df <- data_frame(.size = 5)

  expect_identical(vec_rank(df, ties = "min"), c(1L, 1L, 1L, 1L, 1L))
  expect_identical(vec_rank(df, ties = "sequential"), c(1L, 2L, 3L, 4L, 5L))
  expect_identical(
    vec_rank(df, ties = "sequential", direction = "desc"),
    c(1L, 2L, 3L, 4L, 5L)
  )
})

test_that("works with data frames with 0 columns and 0 rows (#1863)", {
  df <- data_frame(.size = 0)

  expect_identical(vec_rank(df, ties = "min"), integer())
  expect_identical(vec_rank(df, ties = "sequential"), integer())
})

test_that("can control the direction per column", {
  df <- data_frame(
    x = c(1, 2, 1, 2, 2),
    y = c(2, 2, 1, 2, 5)
  )

  df2 <- df
  df2$y <- -df2$y

  expect_identical(
    vec_rank(df, direction = c("asc", "desc")),
    vec_rank(df2, direction = "asc")
  )
})

test_that("incompleteness is respected in data frames and rcrds", {
  df <- data_frame(
    x = c(1, NA, NA, 1),
    y = c(NA, NA, 1, 1)
  )

  expect_identical(vec_rank(df, incomplete = "na"), c(NA, NA, NA, 1L))
  expect_identical(
    vec_rank(df, incomplete = "na", direction = "desc"),
    c(NA, NA, NA, 1L)
  )

  x <- new_rcrd(list(
    x = c(1, 1, NA, NA, 1),
    y = c(1, NA, 1, NA, 1)
  ))

  expect_identical(vec_rank(x, incomplete = "na"), c(1L, NA, NA, NA, 1L))
})

test_that("can control `na_value` per column", {
  df <- data_frame(
    x = c(1, 1, NA, NA, NA),
    y = c(3, NA, NA, 2, 1)
  )

  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest")),
    c(2L, 1L, 3L, 5L, 4L)
  )
  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest"), direction = "desc"),
    c(4L, 5L, 3L, 1L, 2L)
  )

  # But `incomplete = "na"` overrules it
  expect_identical(
    vec_rank(df, na_value = c("largest", "smallest"), incomplete = "na"),
    c(1L, NA, NA, NA, NA)
  )
  expect_identical(
    vec_rank(
      df,
      na_value = c("largest", "smallest"),
      incomplete = "na",
      direction = "desc"
    ),
    c(1L, NA, NA, NA, NA)
  )
})

test_that("`x` must be a vector", {
  expect_error(vec_rank(identity), class = "vctrs_error_scalar_type")
})

test_that("`x` must not be `NULL` (#1823, #1967)", {
  expect_snapshot(error = TRUE, {
    vec_rank(NULL)
  })
  expect_snapshot(error = TRUE, {
    vec_rank(NULL, incomplete = "na")
  })
  expect_snapshot(error = TRUE, {
    vec_rank(NULL, ties = "sequential", incomplete = "na")
  })
})

test_that("`ties` is validated", {
  expect_snapshot(error = TRUE, vec_rank(1, ties = "foo"))
  expect_snapshot(error = TRUE, vec_rank(1, ties = 1))
})

test_that("`incomplete` is validated", {
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = NA))
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = c(TRUE, FALSE)))
  expect_snapshot(error = TRUE, vec_rank(1, incomplete = "foo"))
})

# Never run on CRAN, even if they have sf, because we don't regularly
# check these on CI and we don't want a change in sf to force a CRAN
# failure for vctrs.
skip_on_cran()

# Avoids adding `sf` to Suggests.
# These tests are only run on the devs' machines.
testthat_import_from(
  "sf",
  c(
    "st_sf",
    "st_sfc",
    "st_point",
    "st_bbox",
    "st_precision",
    "st_crs",
    "st_linestring",
    "st_as_sf",
    "st_multipoint"
  )
)

# Need recent version to work around restore bug for sfc lists and changes
# to `c.sfc()`
skip_if_not_installed("sf", "1.0-11")

test_that("sf has a ptype2 method", {
  sfc1 = st_sfc(st_point(1:2), st_point(3:4))
  sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

  sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
  sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

  out = vctrs::vec_ptype2(sf1, sf2)
  exp = st_sf(
    x = double(),
    y = character(),
    geo1 = sfc1[0],
    geo2 = sfc2[0],
    stringsAsFactors = FALSE
  )
  expect_identical(out, exp)

  out = vctrs::vec_ptype2(sf1, new_data_frame(sf2))
  expect_identical(out, exp)

  out = vctrs::vec_ptype2(new_data_frame(sf1), sf2)
  exp_rhs = st_sf(
    x = double(),
    y = character(),
    geo1 = sfc1[0],
    geo2 = sfc2[0],
    stringsAsFactors = FALSE,
    sf_column_name = "geo2"
  )
  expect_identical(out, exp_rhs)
})

test_that("sf has a cast method", {
  sfc1 = st_sfc(st_point(1:2), st_point(3:4))
  sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

  sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
  sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

  expect_error(
    vctrs::vec_cast(sf1, sf2),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    vctrs::vec_cast(sf2, sf1),
    class = "vctrs_error_cast_lossy"
  )

  common = vec_ptype2(sf1, sf2)

  out = vctrs::vec_cast(sf1, common)
  exp = st_sf(
    x = c(1, 0),
    y = character(2)[NA],
    geo1 = sfc1,
    geo2 = sfc2[c(NA, NA) + 0L],
    stringsAsFactors = FALSE
  )
  expect_identical(out, exp)

  out = vctrs::vec_cast(new_data_frame(sf1), common)
  expect_identical(out, exp)

  out = vctrs::vec_cast(sf1, new_data_frame(common))
  expect_identical(out, new_data_frame(exp))

  out = vctrs::vec_cast(sf2, common)
  exp = st_sf(
    x = 0,
    y = "",
    geo1 = sfc1[NA + 0L],
    geo2 = sfc2,
    stringsAsFactors = FALSE
  )
  expect_identical(out, exp)
})

# https://github.com/r-lib/vctrs/issues/1136
test_that("can combine sf data frames", {
  testthat_import_from("dplyr", "bind_rows")

  sfc1 = st_sfc(st_point(1:2), st_point(3:4))
  sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

  sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
  sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

  exp = data_frame(
    x = c(1, 0, 0),
    geo1 = sfc1[c(1:2, NA)],
    y = c(NA, NA, ""),
    geo2 = sfc2[c(NA, NA, 1)]
  )
  expect_identical(vctrs::vec_rbind(sf1, sf2), exp)
  expect_identical(bind_rows(sf1, sf2), st_as_sf(exp))

  exp = data_frame(
    y = c("", NA, NA, ""),
    x = c(0, 1, 0, 0),
    geo2 = sfc2[c(1, NA, NA, 1)],
    geo1 = sfc1[c(NA, 1:2, NA)]
  )
  expect_identical(vctrs::vec_rbind(sf2, sf1, sf2), exp)
  expect_identical(bind_rows(sf2, sf1, sf2), st_as_sf(exp))
})

test_that("can combine sf and tibble", {
  sfc1 = st_sfc(st_point(1:2), st_point(3:4))
  sfc2 = st_sfc(st_linestring(matrix(1:4, 2)))

  sf1 = st_sf(x = c(TRUE, FALSE), geo1 = sfc1)
  sf2 = st_sf(y = "", geo2 = sfc2, x = 0, stringsAsFactors = FALSE)

  out = vctrs::vec_rbind(sf2, data.frame(x = 1))
  exp = data_frame(
    y = c("", NA),
    x = c(0, 1),
    geo2 = sfc2[c(1L, NA)]
  )
  expect_identical(out, exp)

  out = vctrs::vec_rbind(sf2, tibble::tibble(x = 1))
  expect_identical(out, exp)

  out = vctrs::vec_rbind(tibble::tibble(x = 1), sf2)
  exp = data_frame(
    x = c(1, 0),
    y = c(NA, ""),
    geo2 = sfc2[c(NA, 1L)]
  )
  expect_identical(out, exp)
})

# https://github.com/r-spatial/sf/issues/1390
test_that("can combine sfc lists", {
  ls <- st_linestring(matrix(1:3, ncol = 3))

  sfc <- st_sfc(ls)
  expect_identical(vec_c(sfc, sfc), c(sfc, sfc))

  sf <- st_as_sf(data.frame(id = 1, geometry = sfc))

  # Currently returns a bare data frame because of the workaround for
  # the `c()` fallback sentinels
  expect_identical(vec_rbind(sf, sf), new_data_frame(rbind(sf, sf)))
  expect_identical(vec_rbind(sf, sf, sf), new_data_frame(rbind(sf, sf, sf)))
})

test_that("can combine sfc lists with unspecified chunks", {
  point <- st_point(1:2)
  out <- vec_c(c(NA, NA), st_sfc(point), NA)
  expect_identical(out, st_sfc(NA, NA, point, NA))

  multipoint <- st_multipoint(matrix(1:4, 2))
  x <- st_sfc(point)
  y <- st_sfc(multipoint, multipoint)
  out <- vec_rbind(
    data_frame(x = x),
    data_frame(y = y)
  )
  expect_identical(
    out,
    data_frame(
      x = st_sfc(point, NA, NA),
      y = st_sfc(NA, multipoint, multipoint)
    )
  )
})

test_that("`n_empty` attribute of `sfc` vectors is restored", {
  pt1 = st_sfc(st_point(c(NA_real_, NA_real_)))
  pt2 = st_sfc(st_point(0:1))

  x = c(pt1, pt2)
  expect_identical(attr(vctrs::vec_slice(x, 1), "n_empty"), 1L)
  expect_identical(attr(vctrs::vec_slice(x, 2), "n_empty"), 0L)

  combined = vctrs::vec_c(pt1, pt2, pt1)
  expect_length(combined, 3)
  expect_identical(attr(combined, "n_empty"), 2L)
})

test_that("bbox attributes of `sfc` vectors are restored", {
  pt1 = st_sfc(st_point(c(1L, 2L)))
  pt2 = st_sfc(st_point(c(10L, 20L)))

  x = c(pt1, pt2)
  expect_identical(st_bbox(vctrs::vec_slice(x, 1)), st_bbox(pt1))
  expect_identical(st_bbox(vctrs::vec_slice(x, 2)), st_bbox(pt2))

  combined = vctrs::vec_c(pt1, pt2)
  expect_identical(st_bbox(x), st_bbox(combined))
})

test_that("`precision` and `crs` attributes of `sfc` vectors are restored", {
  x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
  out = vctrs::vec_slice(x, 1)
  expect_identical(st_precision(x), st_precision(out))
  expect_identical(st_crs(x), st_crs(out))
})

test_that("`precision` attributes of `sfc` vectors are combined", {
  x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
  y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 3857)

  out = vctrs::vec_c(x, y)
  expect_identical(st_precision(x), st_precision(out))

  # These used to be errors before we fell back to c()
  y = st_sfc(st_point(c(0, 0)), precision = 1e-2, crs = 3857)
  expect_identical(vctrs::vec_c(x, y), c(x, y))
  # expect_error(vctrs::vec_c(x, y), "precisions not equal")
})

test_that("`crs` attributes of `sfc` vectors must be the same", {
  x = st_sfc(st_point(c(pi, pi)), precision = 1e-4, crs = 3857)
  y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 3857)

  out = vctrs::vec_c(x, y)
  expect_identical(st_crs(x), st_crs(out))

  # Error on different `crs` comes from sf as of 1.0-10
  y = st_sfc(st_point(c(0, 0)), precision = 1e-4, crs = 4326)
  expect_snapshot(error = TRUE, {
    vctrs::vec_c(x, y)
  })
})

test_that("`vec_locate_matches()` works with `sfc` vectors", {
  x <- c(
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 1))),
    st_sfc(st_point(c(2, 1))),
    st_sfc(c(st_point(c(0, 1)), st_point(c(0, 1))))
  )

  y <- c(
    st_sfc(c(st_point(c(0, 1)), st_point(c(0, 1)))),
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 3))),
    st_sfc(st_point(c(0, 0))),
    st_sfc(st_point(c(0, 1)))
  )

  out <- vec_locate_matches(x, y)
  expect_identical(out$needles, c(1L, 1L, 2L, 3L, 4L))
  expect_identical(out$haystack, c(2L, 4L, 5L, NA, 1L))
})

test_that("`vec_rbind()` doesn't leak common type fallbacks (#1331)", {
  sf = st_sf(id = 1:2, geo = st_sfc(st_point(c(1, 1)), st_point(c(2, 2))))

  expect_equal(
    vec_rbind(sf, sf),
    data_frame(id = rep(1:2, 2), geo = rep(sf$geo, 2))
  )

  expect_equal(
    vec_rbind(sf, sf, .names_to = "id"),
    data_frame(id = rep(1:2, each = 2), geo = rep(sf$geo, 2))
  )
})

test_that("get expected re-encoded-ness of various encodings", {
  x <- unlist(encodings(bytes = TRUE), use.names = FALSE)

  # For the `°C` strings:
  # - `TRUE`  for marked as `CE_UTF8`
  # - `FALSE` for marked as `CE_NATIVE`, even if `utf8locale = true`
  # - `FALSE` for marked as `CE_LATIN1`
  # - `FALSE` for marked as `CE_BYTES`
  expect_identical(
    chr_is_ascii_or_utf8(x),
    c(TRUE, FALSE, FALSE, FALSE)
  )
})

test_that("can UTF-8 encode a character vector of various encodings (#553)", {
  x <- unlist(encodings(), use.names = FALSE)

  results <- obj_encode_utf8(x)

  expect_equal_encoding(results, encodings()$utf8)
})

test_that("UTF-8 encodes all encodings", {
  encs <- encodings()

  for (enc in encs) {
    expect_equal_encoding(obj_encode_utf8(enc), encodings()$utf8)
  }
})

test_that("can UTF-8 encode a list containing character vectors with different encodings", {
  results <- obj_encode_utf8(encodings())
  results <- unlist(results)

  expect_equal_encoding(results, encodings()$utf8)
})

test_that("UTF-8 encoding fails purposefully with any bytes", {
  expect_error(
    obj_encode_utf8(encoding_bytes()),
    "translating strings with \"bytes\" encoding"
  )
})

test_that("UTF-8 encoding fails purposefully when mixing with bytes with other encodings", {
  for (enc in encodings()) {
    x <- c(encoding_bytes(), enc)
    expect_error(
      obj_encode_utf8(x),
      "translating strings with \"bytes\" encoding"
    )
  }
})

test_that("attributes are kept through UTF-8 encoding (#599)", {
  encs <- encodings()

  x <- c(encs$utf8, encs$latin1)
  x <- structure(x, names = c("a", "b"), extra = 1)

  expect_equal(attributes(obj_encode_utf8(x)), attributes(x))
})

test_that("UTF-8 encoding is robust against scalar types contained in lists (#633)", {
  x <- list(a = z ~ y, b = z ~ z)
  expect_equal(obj_encode_utf8(x), x)
})

test_that("UTF-8 encoding can still occur even if a scalar type is in a list", {
  encs <- encodings()
  x <- list(a = z ~ y, b = encs$latin1)

  result <- obj_encode_utf8(x)

  expect_equal_encoding(result$b, encs$utf8)
})

test_that("UTF-8 encoding occurs inside scalars contained in a list", {
  encs <- encodings()

  scalar <- structure(list(x = encs$latin1), class = "scalar_list")
  lst <- list(scalar)

  result <- obj_encode_utf8(lst)

  expect_equal_encoding(result[[1]]$x, encs$utf8)
})

test_that("UTF-8 encoding treats data frames elements of lists as lists (#1233)", {
  encs <- encodings()

  field <- c(encs$utf8, encs$latin1)

  a <- new_rcrd(list(field = field))
  df <- data.frame(a = a, b = 1:2)
  x <- list(df)

  # Recursive proxy won't proxy list elements,
  # so the rcrd column in the data frame won't get proxied
  proxy <- vec_proxy_equal(x)

  result <- obj_encode_utf8(proxy)

  expect_identical(result, x)

  result_field <- field(result[[1]]$a, "field")
  expect_field <- c(encs$utf8, encs$utf8)

  expect_equal_encoding(result_field, expect_field)
})

test_that("attributes are UTF-8 encoded", {
  utf8 <- encodings()$utf8
  latin1 <- encodings()$latin1

  a <- structure(1, enc = utf8)
  b <- structure(1, enc = latin1)
  c <- structure(1, enc1 = utf8, enc2 = list(latin1), enc3 = latin1)
  x <- list(a, b, c)

  result <- obj_encode_utf8(x)

  a_enc <- attr(result[[1]], "enc")
  b_enc <- attr(result[[2]], "enc")
  c_enc1 <- attr(result[[3]], "enc1")
  c_enc2 <- attr(result[[3]], "enc2")[[1]]
  c_enc3 <- attr(result[[3]], "enc3")

  expect_utf8_encoded(a_enc)
  expect_utf8_encoded(b_enc)
  expect_utf8_encoded(c_enc1)
  expect_utf8_encoded(c_enc2)
  expect_utf8_encoded(c_enc3)

  expect <- list(
    structure(1, enc = utf8),
    structure(1, enc1 = utf8, enc2 = list(utf8), enc3 = utf8)
  )

  expect_identical(vec_unique(x), expect)

  # Ensure nothing has changed in the original objects
  a_enc <- attr(a, "enc")
  b_enc <- attr(b, "enc")
  c_enc1 <- attr(c, "enc1")
  c_enc2 <- attr(c, "enc2")[[1]]
  c_enc3 <- attr(c, "enc3")

  expect_utf8_encoded(a_enc)
  expect_latin1_encoded(b_enc)
  expect_utf8_encoded(c_enc1)
  expect_latin1_encoded(c_enc2)
  expect_latin1_encoded(c_enc3)
})

test_that("attributes are UTF-8 encoded recursively", {
  utf8 <- encodings()$utf8
  latin1 <- encodings()$latin1

  nested <- structure(1, latin1 = latin1)
  x <- structure(2, nested = nested, foo = 1, latin1 = latin1)

  result <- obj_encode_utf8(x)
  attrib <- attributes(result)
  attrib_nested <- attributes(attrib$nested)

  expect_equal_encoding(attrib$latin1, utf8)
  expect_equal_encoding(attrib_nested$latin1, utf8)
})

test_that("NAs aren't converted to 'NA' (#1291)", {
  utf8 <- c(NA, encodings()$utf8)
  latin1 <- c(NA, encodings()$latin1)

  result1 <- obj_encode_utf8(utf8)
  result2 <- obj_encode_utf8(latin1)

  expect_equal_encoding(result1, utf8)
  expect_equal_encoding(result2, utf8)

  expect_identical(result1[[1]], NA_character_)
  expect_identical(result2[[1]], NA_character_)
})

# Print -------------------------------------------------------------------

test_that("ptype print methods are descriptive", {
  tab1 <- new_table()
  tab2 <- new_table(dim = c(0L, 1L, 2L, 1L))

  expect_equal(vec_ptype_abbr(tab1), "table[1d]")
  expect_equal(vec_ptype_abbr(tab2), "table[,1,2,1]")

  expect_equal(vec_ptype_full(tab1), "table[1d]")
  expect_equal(vec_ptype_full(tab2), "table[,1,2,1]")
})

# Coercion ----------------------------------------------------------------

test_that("can find a common type among tables with identical dimensions", {
  tab1 <- new_table()
  tab2 <- new_table(1:2, dim = c(1L, 2L, 1L))

  expect_identical(vec_ptype2(tab1, tab1), new_table())
  expect_identical(vec_ptype2(tab2, tab2), new_table(dim = c(0L, 2L, 1L)))
})

test_that("size is not considered in the ptype", {
  x <- new_table(1:2, dim = 2L)
  y <- new_table(1:3, dim = 3L)

  expect_identical(vec_ptype2(x, y), new_table())
})

test_that("vec_ptype2() can broadcast table shapes", {
  x <- new_table(dim = c(0L, 1L))
  y <- new_table(dim = c(0L, 2L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L)))

  x <- new_table(dim = c(0L, 1L, 3L))
  y <- new_table(dim = c(0L, 2L, 1L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L, 3L)))
})

test_that("vec_ptype2() never propagates dimnames", {
  x <- new_table(dim = c(0L, 1L), dimnames = list(character(), "x1"))
  y <- new_table(dim = c(0L, 2L), dimnames = list(character(), c("y1", "y2")))

  expect_null(dimnames(vec_ptype2(x, x)))
  expect_null(dimnames(vec_ptype2(x, y)))
})

test_that("implicit axes are broadcast", {
  x <- new_table(dim = c(0L, 2L))
  y <- new_table(dim = c(0L, 1L, 3L))

  expect_identical(vec_ptype2(x, y), new_table(dim = c(0L, 2L, 3L)))
})

test_that("errors on non-broadcastable dimensions", {
  x <- new_table(dim = c(0L, 2L))
  y <- new_table(dim = c(0L, 3L))

  expect_error(vec_ptype2(x, y), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2() errors on non-tables", {
  expect_error(
    vec_ptype2(new_table(), 1),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(new_table(), 1L),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(new_table(), "1"),
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    vec_ptype2(1, new_table()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(1L, new_table()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2("1", new_table()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("common types have symmetry when mixed with unspecified input", {
  x <- new_table()

  expect_identical(vec_ptype2(x, NA), new_table())
  expect_identical(vec_ptype2(NA, x), new_table())

  x <- new_table(dim = c(0L, 2L))

  expect_identical(vec_ptype2(x, NA), new_table(dim = c(0L, 2L)))
  expect_identical(vec_ptype2(NA, x), new_table(dim = c(0L, 2L)))
})

test_that("`table` delegates coercion", {
  expect_identical(
    vec_ptype2(new_table(1), new_table(FALSE)),
    new_table(double())
  )
  expect_error(
    vec_ptype2(new_table(1), new_table("")),
    class = "vctrs_error_incompatible_type"
  )
})


# Casting -----------------------------------------------------------------

test_that("can cast to an identically shaped table", {
  x <- new_table(1:5, dim = 5L)
  y <- new_table(1:8, dim = c(2L, 2L, 2L))

  expect_identical(vec_cast(x, x), x)
  expect_identical(vec_cast(y, y), y)
})

test_that("vec_cast() can broadcast table shapes", {
  # We test only the dim here and not the class because on R 3.2
  # the `[.table` method did not exist and `shape_broadcast()`
  # gives back a matrix, not a table.

  x <- new_table(dim = c(0L, 1L))
  y <- new_table(dim = c(0L, 2L))

  expect_identical(dim(vec_cast(x, y)), c(0L, 2L))

  x <- new_table(dim = c(0L, 1L, 1L))
  y <- new_table(dim = c(0L, 2L, 3L))

  expect_identical(dim(vec_cast(x, y)), c(0L, 2L, 3L))
})

test_that("cannot decrease axis length", {
  x <- new_table(dim = c(0L, 3L))
  y <- new_table(dim = c(0L, 1L))

  expect_error(
    vec_cast(x, y),
    "Non-recyclable",
    class = "vctrs_error_incompatible_type"
  )
})

test_that("cannot decrease dimensionality", {
  x <- new_table(dim = c(0L, 1L, 1L))
  y <- new_table(dim = c(0L, 1L))

  expect_snapshot({
    (expect_error(vec_cast(x, y), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_cast() errors on non-tables", {
  expect_error(
    vec_cast(new_table(), 1),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(new_table(), 1L),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(new_table(), "1"),
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    vec_cast(1, new_table()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(1L, new_table()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast("1", new_table()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can cast from, but not to, unspecified", {
  x <- new_table()

  expect_error(vec_cast(x, NA), class = "vctrs_error_incompatible_type")
  expect_identical(vec_cast(NA, x), new_table(NA_integer_, dim = 1L))

  x <- new_table(dim = c(0L, 2L))

  expect_error(vec_cast(x, NA), class = "vctrs_error_incompatible_type")
  expect_identical(
    vec_cast(NA, x),
    new_table(c(NA_integer_, NA_integer_), dim = c(1L, 2L))
  )
})

test_that("`table` delegates casting", {
  expect_identical(
    vec_cast(new_table(1), new_table(FALSE)),
    new_table(TRUE)
  )
  expect_error(
    vec_cast(new_table(1), new_table("")),
    class = "vctrs_error_incompatible_type"
  )
})

# Misc --------------------------------------------------------------------

test_that("`new_table()` validates input", {
  expect_error(new_table(1L, 1), "`dim` must be an integer vector")
  expect_error(new_table(1:2, 1L), "must match the length of `x`")
})

test_that("ptype is correct", {
  tab1 <- new_table(1L, dim = 1L)
  tab2 <- new_table(1:2, dim = c(1L, 2L, 1L))

  expect_identical(vec_ptype(tab1), new_table())
  expect_identical(vec_ptype(tab2), new_table(dim = c(0L, 2L, 1L)))
})

test_that("can use a table in `vec_c()`", {
  expect_identical(vec_c(new_table()), new_table())
  expect_identical(vec_c(new_table(), new_table()), new_table())

  x <- new_table(1:5, 5L)
  y <- new_table(1:4, dim = c(2L, 2L))

  expect_identical(vec_c(x, x), new_table(c(1:5, 1:5), dim = 10L))
  expect_identical(
    vec_c(y, y),
    new_table(c(1:2, 1:2, 3:4, 3:4), dim = c(4L, 2L))
  )
  expect_identical(
    vec_c(x, y),
    new_table(c(1:5, 1:2, 1:5, 3:4), dim = c(7L, 2L))
  )
})

test_that("names of the first dimension are kept in `vec_c()`", {
  x <- new_table(1:4, c(2L, 2L))
  dimnames(x) <- list(c("r1", "r2"), c("c1", "c2"))

  xx <- vec_c(x, x)

  expect_identical(dimnames(xx), list(c("r1", "r2", "r1", "r2"), NULL))
})

test_that("can use a table in `list_unchop()`", {
  x <- new_table(1:4, dim = c(2L, 2L))

  expect_identical(list_unchop(list(x)), x)
  expect_identical(
    list_unchop(list(x, x), indices = list(1:2, 4:3)),
    vec_slice(x, c(1:2, 2:1))
  )
})

test_that("can concatenate tables", {
  x <- table(1:2)

  out <- vec_c(x, x)
  exp <- new_table(rep(1L, 4), dimnames = list(c("1", "2", "1", "2")))
  expect_identical(out, exp)

  out <- vec_rbind(x, x)
  exp <- data_frame(`1` = new_table(c(1L, 1L)), `2` = new_table(c(1L, 1L)))
  expect_identical(out, exp)

  y <- table(list(1:2, 3:4))

  # FIXME
  out <- vec_c(y, y)
  exp <- new_table(
    matrix(int(1, 0, 1, 0, 0, 1, 0, 1), nrow = 4),
    dim = c(4L, 2L),
    dimnames = list(c("1", "2", "1", "2"), NULL)
  )
  expect_identical(out, exp)

  out <- vec_rbind(y, y)
  exp <- new_data_frame(
    list(
      `3` = int(1, 0, 1, 0),
      `4` = int(0, 1, 0, 1)
    ),
    row.names = c("1...1", "2...2", "1...3", "2...4")
  )
  expect_identical(out, exp)

  skip("FIXME: dimnames of matrices are not properly concatenated")
})

test_that("can concatenate tables of type double (#1190)", {
  x <- table(c(1, 2)) / 2

  out <- vec_c(x, x)
  exp <- new_table(
    c(0.5, 0.5, 0.5, 0.5),
    dimnames = list(c("1", "2", "1", "2"))
  )
  expect_identical(out, exp)

  out <- vec_rbind(x, x)
  exp <- data_frame(`1` = new_table(c(0.5, 0.5)), `2` = new_table(c(0.5, 0.5)))
  expect_identical(out, exp)
})

# ------------------------------------------------------------------------------
# Printing

test_that("I() wraps contents", {
  f <- factor()

  expect_equal(vec_ptype_abbr(I(f)), "I<fct>")
  expect_equal(vec_ptype_full(I(f)), "I<factor<>>")
})

test_that("AsIs class stripped from I()", {
  df <- data.frame(x = 1, y = 1:2)
  class(df) <- c("myclass", "data.frame")

  expect_equal(
    vec_ptype_full(I(df)),
    "I<myclass<\n  x: double\n  y: integer\n>>"
  )
  expect_equal(vec_ptype_full(I(df[1])), "I<myclass<x:double>>")
  expect_equal(vec_ptype_full(I(df[0])), "I<myclass<>>")
})

# ------------------------------------------------------------------------------
# Proxy / restore

test_that("can slice AsIs class", {
  df <- data.frame(x = I(1:3), y = I(list(4, 5, 6)))
  expect_identical(vec_slice(df, 2:3), unrownames(df[2:3, ]))
})

test_that("equality proxy is forwarded correctly for atomic types (#1557)", {
  # We don't define any equality proxies for base atomic types, but we can fake it
  local_methods(vec_proxy_equal.integer = function(x, ...) "dispatched")
  asis <- I(1L)
  expect_identical(vec_proxy_equal(asis), "dispatched")
})

test_that("comparison proxy is forwarded correctly for atomic types (#1557)", {
  # vec_proxy_compare.raw() exists
  x <- raw()
  asis <- I(x)

  expect_identical(vec_proxy_compare(asis), vec_proxy_compare(x))
  expect_identical(vec_proxy_compare(asis), integer())
})

test_that("order proxy is forwarded correctly for atomic types (#1557)", {
  # vec_proxy_order.list() exists
  x <- list(2, 1, 2)
  asis <- I(x)

  expect_identical(vec_proxy_order(asis), vec_proxy_order(x))
  expect_identical(vec_proxy_order(asis), c(1L, 2L, 1L))
})

# ------------------------------------------------------------------------------
# Coercion

test_that("can take the common type of identical AsIs objects", {
  expect_identical(vec_ptype2(I(1), I(1)), I(numeric()))
})

test_that("AsIs objects throw ptype2 errors with their underlying types", {
  expect_snapshot({
    (expect_error(
      vec_ptype2(I(1), I("x")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("AsIs always wraps the common type", {
  expect_identical(vec_ptype2(I(1L), 1), I(numeric()))
  expect_identical(vec_ptype2(1, I(1L)), I(numeric()))
})

# ------------------------------------------------------------------------------
# Casting

test_that("can cast one AsIs to another AsIs", {
  expect_identical(vec_cast(I(1), I(1)), I(1))
  expect_identical(vec_cast(I(1), I(1L)), I(1L))
})

test_that("AsIs objects throw cast errors with their underlying types", {
  expect_snapshot({
    (expect_error(
      vec_cast(I(1), I(factor("x"))),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("casting from an AsIs drops the AsIs class", {
  expect_identical(vec_cast(I(1), 1), 1)
})

test_that("casting to an AsIs adds the AsIs class", {
  expect_identical(vec_cast(1, I(1)), I(1))
})

# ------------------------------------------------------------------------------
# Misc

test_that("can `vec_c()` with only AsIs objects", {
  expect_identical(vec_c(I(1), I(2)), I(c(1, 2)))
  expect_identical(vec_c(I(1), I(2L)), I(c(1, 2)))
})

test_that("can `vec_c()` with AsIs objects mixed with other types", {
  expect_identical(vec_c(I(1L), 1), I(c(1, 1)))
})

test_that("`condition` must be a condition vector", {
  # No scalars
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = lm(1 ~ 1),
    true = 1,
    false = 2
  )

  # No casting
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = 1,
    true = 1,
    false = 2
  )

  # No objects
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = structure(TRUE, class = "foo"),
    true = 1,
    false = 2
  )

  # No dim
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = array(TRUE),
    true = 1,
    false = 2
  )
})

test_that("`true`, `false`, and `missing` must be vectors", {
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = lm(1 ~ 1),
      false = 2,
      missing = 0
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = lm(1 ~ 1),
      missing = 0
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = 2,
      missing = lm(1 ~ 1)
    )
  })
})

test_that("`true`, `false`, and `missing` must recycle to size of `condition`", {
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = TRUE,
    true = c(1, 2),
    false = 2,
    missing = 0
  )
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = TRUE,
    true = 1,
    false = c(1, 2),
    missing = 0
  )
  expect_snapshot_vec_if_else(
    error = TRUE,
    condition = TRUE,
    true = 1,
    false = 2,
    missing = c(1, 2)
  )
})

test_that("all combinations of `true`, `false`, and `missing` recycling (including names) are tested", {
  # 8 combinations
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = set_names(7L),
    missing = set_names(13L),
    expect = set_names(int(1, 7, 13, 13, 1, 7))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1:6),
    false = set_names(7L),
    missing = set_names(13L),
    expect = set_names(int(1, 7, 13, 13, 5, 7))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = set_names(7:12),
    missing = set_names(13L),
    expect = set_names(int(1, 8, 13, 13, 1, 12))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = set_names(7L),
    missing = set_names(13:18),
    expect = set_names(int(1, 7, 15, 16, 1, 7))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1:6),
    false = set_names(7:12),
    missing = set_names(13L),
    expect = set_names(int(1, 8, 13, 13, 5, 12))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1:6),
    false = set_names(7L),
    missing = set_names(13:18),
    expect = set_names(int(1, 7, 15, 16, 5, 7))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = set_names(7:12),
    missing = set_names(13:18),
    expect = set_names(int(1, 8, 15, 16, 1, 12))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1:6),
    false = set_names(7:12),
    missing = set_names(13:18),
    expect = set_names(int(1, 8, 15, 16, 5, 12))
  )
})

test_that("not all of `true`, `false`, and `missing` must be named", {
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = 1L,
    false = set_names(7L),
    missing = set_names(13L),
    expect = set_names(int(1, 7, 13, 13, 1, 7), c("", "7", "13", "13", "", "7"))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = 7L,
    missing = set_names(13L),
    expect = set_names(int(1, 7, 13, 13, 1, 7), c("1", "", "13", "13", "1", ""))
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
    true = set_names(1L),
    false = set_names(7L),
    missing = 13L,
    expect = set_names(int(1, 7, 13, 13, 1, 7), c("1", "7", "", "", "1", "7"))
  )
})

test_that("names are retained even with casting", {
  expect_named(
    vec_if_else(
      condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
      true = set_names(1L),
      false = set_names(7L),
      missing = set_names(13L),
      ptype = double()
    ),
    as.character(c(1, 7, 13, 13, 1, 7))
  )

  # Generic
  expect_named(
    vec_if_else(
      condition = c(TRUE, FALSE, NA, NA, TRUE, FALSE),
      true = set_names(new_date(1)),
      false = set_names(new_date(7)),
      missing = set_names(new_date(13)),
      ptype = new_datetime(tzone = "UTC")
    ),
    as.character(new_date(c(1, 7, 13, 13, 1, 7)))
  )
})

test_that("`ptype` overrides common type", {
  expect_identical(
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = 2,
      missing = 0,
      ptype = integer()
    ),
    1L
  )
  # Generic
  expect_identical(
    vec_if_else(
      condition = TRUE,
      true = new_date(0),
      false = new_date(1),
      missing = new_date(2),
      ptype = new_datetime(tzone = "UTC")
    ),
    new_datetime(0, tzone = "UTC")
  )

  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1.5,
      false = 2,
      missing = 0,
      ptype = integer()
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = 2.5,
      missing = 0,
      ptype = integer()
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = 2,
      missing = 0.5,
      ptype = integer()
    )
  })

  # Generic
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = 1,
      false = new_date(2),
      missing = new_date(0),
      ptype = new_date()
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = new_date(1),
      false = 2,
      missing = new_date(0),
      ptype = new_date()
    )
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      condition = TRUE,
      true = new_date(1),
      false = new_date(2),
      missing = 0,
      ptype = new_date()
    )
  })
})

test_that("gives expected output with `true` and `false`", {
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, TRUE),
    true = c(1, 2, 3),
    false = c(4, 5, 6),
    expect = c(1, 5, 3)
  )
})

test_that("gives expected output with `missing`", {
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA),
    true = c(1, 2, 3),
    false = c(4, 5, 6),
    expect = c(1, 5, NA)
  )
  expect_identical_vec_if_else(
    condition = c(TRUE, FALSE, NA),
    true = c(1, 2, 3),
    false = c(4, 5, 6),
    missing = c(7, 8, 9),
    expect = c(1, 5, 9)
  )
})

test_that("works with arrays of all types", {
  mat <- as.matrix

  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(lgl(TRUE, FALSE, NA)),
      mat(lgl(FALSE, TRUE, FALSE))
    ),
    mat(lgl(TRUE, TRUE, NA))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(int(1, 2, 3)),
      mat(int(4, 5, 6))
    ),
    mat(int(1, 5, 3))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(dbl(1, 2, 3)),
      mat(dbl(4, 5, 6))
    ),
    mat(dbl(1, 5, 3))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(cpl(1, 2, 3)),
      mat(cpl(4, 5, 6))
    ),
    mat(cpl(1, 5, 3))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(as.raw(c(1, 2, 3))),
      mat(as.raw(c(4, 5, 6)))
    ),
    mat(as.raw(c(1, 5, 3)))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(chr("1", "2", "3")),
      mat(chr("4", "5", "6"))
    ),
    mat(chr("1", "5", "3"))
  )
  expect_identical(
    vec_if_else(
      c(TRUE, FALSE, TRUE),
      mat(list(1, 2, 3)),
      mat(list(4, 5, 6))
    ),
    mat(list(1, 5, 3))
  )
})

test_that("missing value fall through is right for all atomic types", {
  expect_identical(vec_if_else(NA, TRUE, TRUE), NA)
  expect_identical(vec_if_else(NA, 1L, 1L), NA_integer_)
  expect_identical(vec_if_else(NA, 1, 1), NA_real_)
  expect_identical(
    vec_if_else(NA, cpl(1), cpl(1)),
    complex(real = NA, imaginary = NA)
  )
  expect_identical(vec_if_else(NA, as.raw(1), as.raw(1)), as.raw(0))
  expect_identical(vec_if_else(NA, "1", "1"), NA_character_)
  expect_identical(vec_if_else(NA, list(1), list(1)), list(NULL))
})

test_that("extraneous attributes on `true`, `false`, and `missing` are dropped", {
  # We do not consider these as part of the `ptype`!
  expect_identical(
    vec_if_else(
      condition = c(TRUE, FALSE, TRUE, NA),
      true = structure(1:4, foo = "bar"),
      false = structure(5:8, foo = "bar"),
      missing = structure(9:12, foo = "bar")
    ),
    int(1, 6, 3, 12)
  )
})

test_that("`ptype` is finalized before being used", {
  # Without `vec_ptype_final()`, this could result in `unspecified()` as the `ptype`
  expect_identical(vec_if_else(TRUE, TRUE, FALSE, ptype = NA), TRUE)

  # We try not to leak <unspecified> when we can help it
  expect_identical(vec_if_else(TRUE, TRUE, FALSE, ptype = unspecified(1)), TRUE)
})

test_that("`vec_ptype2()`'s `left` is always set", {
  # This used to result in `left` being unset, so we'd hit an unreachable state
  expect_identical(
    vec_if_else(
      condition = FALSE,
      true = TRUE,
      false = NA,
      missing = FALSE
    ),
    NA
  )
})

# ------------------------------------------------------------------------------
# `dplyr::if_else()` tests

test_that("scalar true and false are vectorised", {
  x <- c(TRUE, TRUE, FALSE, FALSE)
  expect_equal(vec_if_else(x, 1, 2), c(1, 1, 2, 2))
})

test_that("vector true and false are ok", {
  x <- c(-1, 0, 1)

  expect_equal(vec_if_else(x < 0, x, 0), c(-1, 0, 0))
  expect_equal(vec_if_else(x > 0, x, 0), c(0, 0, 1))
})

test_that("missing values are missing", {
  expect_equal(vec_if_else(c(TRUE, NA, FALSE), -1, 1), c(-1, NA, 1))
})

test_that("works with lists", {
  x <- list(1, 2, 3)

  expect_equal(
    vec_if_else(c(TRUE, TRUE, FALSE), x, list(NULL)),
    list(1, 2, NULL)
  )
})

test_that("works with data frames", {
  true <- tibble(x = 1, y = 2)
  false <- tibble(x = 3, y = 4)

  expect_identical(
    vec_if_else(c(TRUE, FALSE, NA, TRUE), true, false),
    vec_c(true, false, NA, true)
  )
})

test_that("works with vctrs rcrd types", {
  true <- new_rcrd(list(x = 1, y = 2))
  false <- new_rcrd(list(x = 3, y = 4))

  expect_identical(
    vec_if_else(c(TRUE, FALSE, NA, TRUE), true, false),
    vec_c(true, false, NA, true)
  )
})

test_that("takes the common type of `true` and `false` (tidyverse/dplyr#6243)", {
  expect_identical(vec_if_else(TRUE, 1L, 1.5), 1)

  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(
      TRUE,
      1,
      "x",
      true_arg = "t",
      false_arg = "f",
      error_call = current_env()
    )
  })
})

test_that("includes `missing` in the common type computation if used", {
  expect_identical(vec_if_else(TRUE, 1L, 2L, missing = 3), 1)

  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1, 2, missing = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1L, 2, missing = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1, 2L, missing = "x")
  })
})

test_that("can recycle to size 0 `condition`", {
  expect_identical(vec_if_else(logical(), 1, 2, missing = 3), double())
})

test_that("accepts logical conditions with attributes (tidyverse/dplyr#6678)", {
  x <- structure(TRUE, label = "foo")
  expect_identical(vec_if_else(x, 1, 2), 1)
})

test_that("`condition` must be logical (and isn't cast to logical!)", {
  expect_snapshot(error = TRUE, {
    vec_if_else(1:10, 1, 2)
  })
})

test_that("`true`, `false`, and `missing` must recycle to the size of `condition`", {
  x <- 1:3
  bad <- 1:2

  expect_snapshot(error = TRUE, {
    vec_if_else(x < 2, bad, x)
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(x < 2, x, bad)
  })
  expect_snapshot(error = TRUE, {
    vec_if_else(x < 2, x, x, missing = bad)
  })
})

test_that("must have empty dots", {
  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1, 2, missing = 3, 4)
  })
})

test_that("`ptype` overrides the common type", {
  expect_identical(vec_if_else(TRUE, 2, 1L, ptype = integer()), 2L)

  expect_snapshot(error = TRUE, {
    vec_if_else(TRUE, 1L, 2.5, ptype = integer())
  })
})

# common shape ------------------------------------------------------------

test_that("vec_shape2() applies recycling rules", {
  expect_equal(vec_shape2(shaped_int(1, 5, 5), shaped_int(1)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1), shaped_int(1, 5, 5)), c(0L, 5L, 5L))
  expect_equal(vec_shape2(shaped_int(1, 1), shaped_int(1, 5, 5)), c(0L, 5L, 5L))
  expect_equal(
    vec_shape2(shaped_int(1, 1, 1), shaped_int(1, 5, 5)),
    c(0L, 5L, 5L)
  )

  expect_equal(
    vec_shape2(shaped_int(1, 1, 5), shaped_int(1, 5, 1)),
    c(0L, 5L, 5L)
  )
  expect_equal(
    vec_shape2(shaped_int(1, 5, 1), shaped_int(1, 1, 5)),
    c(0L, 5L, 5L)
  )
  expect_equal(
    vec_shape2(shaped_int(1, 1, 1), shaped_int(1, 5, 5)),
    c(0L, 5L, 5L)
  )

  expect_equal(
    vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 1, 1)),
    c(0L, 0L, 5L)
  )
})

test_that("incompatible shapes throw errors", {
  expect_snapshot({
    (expect_error(
      vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1)),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5)),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("can override error args", {
  expect_snapshot({
    (expect_error(
      vec_shape2(
        shaped_int(1, 0, 5),
        shaped_int(1, 5, 1),
        x_arg = "foo",
        y_arg = "bar"
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_shape2() evaluates arg lazily", {
  expect_silent(vec_shape2(
    shaped_int(1, 5, 5),
    shaped_int(1),
    x_arg = print("oof")
  ))
  expect_silent(vec_shape2(
    shaped_int(1, 5, 5),
    shaped_int(1),
    y_arg = print("oof")
  ))
})

# broadcasting -------------------------------------------------------------

test_that("can broadcast to higher dimension, but not lower", {
  expect_identical(shape_broadcast_(1, NULL), 1)
  expect_null(shape_broadcast_(NULL, 1))

  expect_equal(
    shape_broadcast_(1, shaped_int(0, 4)),
    array(1, c(1, 4))
  )
  expect_error(
    shape_broadcast_(shaped_int(1, 1, 1), shaped_int(4, 4)),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    shape_broadcast_(shaped_int(3, 2), shaped_int(3, 3)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("shape_broadcast_() applies recycling rules", {
  expect_equal(
    shape_broadcast_(array(1:4, c(1, 1, 4)), shaped_int(0, 4, 4))[1, , ],
    matrix(1:4, 4, 4, byrow = TRUE)
  )

  expect_equal(
    shape_broadcast_(array(1:4, c(1, 4, 1)), shaped_int(0, 4, 4))[1, , ],
    matrix(1:4, 4, 4)
  )

  expect_equal(
    shape_broadcast_(array(1L, c(1, 1)), shaped_int(1, 0)),
    matrix(integer(), nrow = 1)
  )

  expect_error(
    shape_broadcast_(array(1L, c(1, 2)), shaped_int(1, 0)),
    "Non-recyclable dimensions",
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    shape_broadcast_(array(1L, c(1, 0)), shaped_int(1, 1)),
    "Non-recyclable dimensions",
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can combine shaped native classes (#1290, #1329)", {
  x <- new_datetime(c(1, 1e6))
  dim(x) <- c(1, 2)
  out <- vec_c(x, x)

  expect_s3_class(out, c("POSIXct", "POSIXt"))
  expect_dim(out, c(2, 2))

  y <- new_datetime(1:3 + 0.0)
  dim(y) <- c(1, 3)

  expect_snapshot(error = TRUE, vec_c(x, y))

  d <- structure(Sys.Date(), dim = 1)
  expect_equal(
    vec_rbind(data.frame(d), data.frame(d)),
    data.frame(d = structure(rep(Sys.Date(), 2), dim = 2))
  )
})

test_that("factor casts support shape", {
  x <- factor(c("x", "y", "z"))
  dim(x) <- c(3, 1)
  dimnames(x) <- list(c("r1", "r2", "r3"), "c1")

  y <- factor(c("w", "x", "y", "z"))
  dim(y) <- c(2, 2)

  exp <- factor(
    c("x", "y", "z", "x", "y", "z"),
    levels = c("w", "x", "y", "z")
  )
  dim(exp) <- c(3, 2)
  dimnames(exp) <- list(c("r1", "r2", "r3"), c("c1", "c1"))

  expect_equal(vec_cast(x, y), exp)

  x <- factor(c("x", "y", "z"))
  dim(x) <- c(3, 1)
  y <- factor(c("x", "y", "z"))
  expect_snapshot(error = TRUE, vec_cast(x, y))
})

# Never run on CRAN, even if they have data.table, because we don't regularly
# check these on CI and we don't want a change in data.table to force a CRAN
# failure for vctrs.
skip_on_cran()

# Avoids adding `data.table` to Suggests.
# These tests are only run on the devs' machines.
testthat_import_from("data.table", "as.IDate")

# `as.IDate()` drops names https://github.com/Rdatatable/data.table/issues/7252
as_IDate_with_names <- function(x) {
  out <- as.IDate(x)
  names(out) <- names(x)
  out
}

# ------------------------------------------------------------------------------
# ptype

test_that("ptype abbr", {
  x <- as.IDate("2019-01-01")
  expect_identical(vec_ptype_abbr(x), "IDate")
})

test_that("ptype full", {
  x <- as.IDate("2019-01-01")
  expect_identical(vec_ptype_full(x), "IDate")
})

# ------------------------------------------------------------------------------
# ptype2

test_that("can get common type of IDate and IDate", {
  x <- as_IDate_with_names(c(a = "2019-01-01"))

  # It shouldn't have names, but thats a vctrs problem
  expect <- as_IDate_with_names(set_names(integer(), character()))

  expect_identical(vec_ptype2(x, x), expect)
})

test_that("can't get common type of Date and IDate", {
  x <- as.Date("2019-01-01")
  y <- as.IDate("2019-01-01")

  expect_snapshot(error = TRUE, {
    vec_ptype2(x, y)
  })
  expect_snapshot(error = TRUE, {
    vec_ptype2(y, x)
  })
})

# ------------------------------------------------------------------------------
# cast

test_that("can cast IDate to IDate", {
  x <- as_IDate_with_names(c(a = "2019-01-01"))
  expect_identical(vec_cast(x, x), x)
})

test_that("can't cast Date to IDate", {
  x <- as.Date("2019-01-01")
  y <- as.IDate("2019-01-01")

  expect_snapshot(error = TRUE, {
    vec_cast(x, y)
  })
})

test_that("can't cast IDate to Date", {
  x <- as.IDate("2019-01-01")
  y <- as.Date("2019-01-01")

  expect_snapshot(error = TRUE, {
    vec_cast(x, y)
  })
})

# ------------------------------------------------------------------------------
# proxy / restore

test_that("vec_proxy", {
  # Retains integer type https://github.com/r-lib/vctrs/issues/1961
  x <- as.IDate("2019-01-01")
  expect_identical(vec_proxy(x), x)
})

test_that("vec_restore", {
  x <- as.IDate("2019-01-01")
  proxy <- vec_proxy(x)
  expect_identical(vec_restore(proxy, x), x)
})

test_that("proxy / restore retains names", {
  x <- as_IDate_with_names(c(a = "2019-01-01"))

  proxy <- vec_proxy(x)
  expect_named(proxy, "a")

  restored <- vec_restore(proxy, x)
  expect_named(restored, "a")
})

test_that("vec_proxy_equal, vec_proxy_compare, vec_proxy_order", {
  # Doesn't change type, stays integer storage as well
  x <- as.IDate("2019-01-01")

  expect_identical(vec_proxy_equal(x), x)
  expect_identical(vec_proxy_compare(x), x)
  expect_identical(vec_proxy_order(x), x)
})

# ------------------------------------------------------------------------------
# manipulation

test_that("ptype retains integer type", {
  x <- as.IDate(c("2019-01-01", "2019-01-02"))
  expect_identical(typeof(vec_ptype(x)), "integer")
  expect_identical(vec_ptype(x), as.IDate(integer()))
})

test_that("slicing retains integer type", {
  x <- as.IDate(c("2019-01-01", "2019-01-02"))
  expect_identical(typeof(vec_slice(x, 1)), "integer")
})

test_that("slicing retains names", {
  x <- as_IDate_with_names(c(a = "2019-01-01"))
  expect_identical(vec_slice(x, 1), x)
})

test_that("can split empty vector", {
  out <- vec_split(integer(), character())

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, character())
  expect_equal(out$val, list())
})

test_that("split data frame with data frame", {
  df <- data.frame(x = c(1, 1, 2), y = c(1, 1, 1))
  out <- vec_split(df, df)

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, data.frame(x = c(1, 2), y = c(1, 1)))
  expect_equal(
    out$val,
    list(
      data.frame(x = c(1, 1), y = c(1, 1)),
      data.frame(x = 2, y = 1)
    )
  )
})

test_that("x and by must be same size", {
  expect_error(
    vec_split(1:3, 1:2),
    "same size"
  )
})

test_that("split takes the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)
  expect_identical(nrow(vec_split(1:3, x)), 2L)
})

test_that("split works with different encodings", {
  encs <- encodings()
  expect_identical(nrow(vec_split(1:3, encs)), 1L)
})

test_that("`key` and `value` retain names", {
  x <- c(a = 1, b = 2, c = 1, a = 1)
  split <- vec_split(x, x)
  expect_identical(split$key, c(a = 1, b = 2))
  expect_identical(split$val[[1]], c(a = 1, c = 1, a = 1))
  expect_identical(split$val[[2]], c(b = 2))
})

test_that("ptype methods are descriptive", {
  f <- factor()
  o <- ordered(character())

  expect_equal(vec_ptype_abbr(f), "fct")
  expect_equal(vec_ptype_abbr(o), "ord")

  expect_equal(vec_ptype_full(f), "factor<>")
  expect_equal(vec_ptype_full(o), "ordered<>")
})

test_that("slicing factors uses a proxy to not go through `[.factor`", {
  x <- factor("x")
  y <- ordered("y")

  local_methods(
    `[.factor` = function(x, ...) abort("should not be called")
  )

  expect_identical(vec_slice(x, 1), x)
  expect_identical(vec_slice(y, 1), y)
})

test_that("`vec_c()` throws the right error with subclassed factors (#1015)", {
  skip("Factors now have a `c()` method")

  a <- subclass(factor("a"))
  b <- subclass(factor("b"))

  # We used to return a subclass
  expect_identical(vec_c(a, a), subclass(factor(c("a", "a"))))

  # We used to fail if attributes were incoPatible
  expect_error(vec_c(a, b), class = "vctrs_error_incompatible_type")
})

# Coercion ----------------------------------------------------------------

test_that("factor/character coercions are symmetric and unchanging", {
  types <- list(
    ordered(character()),
    factor(),
    character()
  )
  mat <- maxtype_mat(types)
  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("factors level are unioned", {
  # This is technically incorrect, but because of R's existing behaviour
  # anything else will cause substantial friction.
  fa <- factor(levels = "a")
  fb <- factor(levels = "b")

  expect_equal(vec_ptype_common(fa, fb), factor(levels = c("a", "b")))
  expect_equal(vec_ptype_common(fb, fa), factor(levels = c("b", "a")))
})

test_that("coercion errors with factors", {
  f <- factor(levels = "a")

  expect_error(
    vec_ptype_common(f, logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype_common(logical(), f),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype2(<factor>, NA) is symmetric (#687)", {
  fct <- new_factor()
  expect_identical(
    vec_ptype2(fct, NA),
    vec_ptype2(NA, fct)
  )

  fct <- new_ordered()
  expect_identical(
    vec_ptype2(fct, NA),
    vec_ptype2(NA, fct)
  )
})

test_that("vec_ptype2(<integer64>, NA) is symmetric (#687)", {
  i64 <- bit64::integer64()
  expect_identical(
    vec_ptype2(i64, NA),
    vec_ptype2(NA, i64)
  )
})

test_that("vec_ptype2() errors with malformed factors", {
  x <- structure(1, class = "factor")
  y <- factor("x")

  expect_error(vec_ptype2(x, y, x_arg = "z"), "`z` is a corrupt factor")
  expect_error(vec_ptype2(y, x, y_arg = "z"), "`z` is a corrupt factor")
})

test_that("vec_ptype2() errors with malformed ordered factors", {
  x <- structure(1, class = c("ordered", "factor"))
  y <- as.ordered(factor("x"))

  expect_error(vec_ptype2(x, y, x_arg = "z"), "`z` is a corrupt ordered factor")
  expect_error(vec_ptype2(y, x, y_arg = "z"), "`z` is a corrupt ordered factor")
})

test_that("ordered factors with different levels are not compatible", {
  expect_error(
    vec_ptype2(ordered("a"), ordered("b")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(ordered("a"), ordered(c("a", "b"))),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), ordered("b")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), ordered(c("a", "b"))),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("factors and ordered factors are not compatible", {
  expect_error(
    vec_ptype2(factor("a"), ordered("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(ordered("a"), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(factor("a"), ordered("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
})


# Casting -----------------------------------------------------------------

test_that("safe casts work as expected", {
  fa <- factor("a")
  fab <- factor(c("a", "b"))

  expect_equal(vec_cast(NULL, fa), NULL)

  expect_equal(vec_cast(fa, fa), fa)
  expect_equal(vec_cast(fa, fab), fab[1])
  expect_equal(vec_cast("a", fab), fab[1])

  expect_equal(vec_cast("a", factor()), fa)
  expect_equal(vec_cast(fa, factor()), fa)

  # This used to be allowed
  expect_error(
    vec_cast(list("a", "b"), fab),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can cast to character", {
  expect_equal(vec_cast(factor("X"), character()), "X")
  expect_equal(vec_cast(ordered("X"), character()), "X")
})

test_that("can cast NA and unspecified to factor", {
  expect_identical(vec_cast(NA, new_factor()), factor(NA))
  expect_identical(vec_cast(NA, new_ordered()), ordered(NA))
  expect_identical(vec_cast(unspecified(2), new_factor()), factor(c(NA, NA)))
  expect_identical(vec_cast(unspecified(2), new_ordered()), ordered(c(NA, NA)))
})

test_that("lossy factor casts fail", {
  fa <- factor("a")
  fb <- factor("b")

  expect_lossy(vec_cast(fa, fb), factor(NA, levels = "b"), x = fa, to = fb)
  expect_lossy(vec_cast("a", fb), factor(NA, levels = "b"), x = chr(), to = fb)
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(double(), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(factor("a"), logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(ordered("a"), logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(logical(), factor("a")),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(logical(), ordered("a")),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("orderedness of factor is preserved", {
  ord <- ordered(c("a", "b"), levels = c("b", "a"))
  expect_equal(vec_cast("a", ord), ordered("a", levels = c("b", "a")))
})

test_that("NA are not considered lossy in factor cast (#109)", {
  f <- factor(c("itsy", "bitsy", NA, "spider", "spider"))
  expect_warning(vec_cast(f, f[1]), NA)
})

test_that("Casting to a factor with explicit NA levels retains them", {
  f <- factor(c("x", NA), exclude = NULL)
  expect_identical(vec_cast(f, f), f)
  expect_identical(vec_cast(f, factor()), f)
})

test_that("characters can be cast to ordered", {
  expect_identical(vec_cast("a", ordered("a")), ordered("a"))
  expect_error(
    vec_cast(c("a", "b"), ordered("a")),
    class = "vctrs_error_cast_lossy"
  )
})


# Proxy / restore ---------------------------------------------------------

test_that("subclassed factors / ordered factors can be restored (#1015)", {
  x <- subclass(factor("a"))
  proxy <- vec_proxy(x)
  expect_identical(vec_restore(proxy, x), x)

  y <- subclass(ordered("a"))
  proxy <- vec_proxy(y)
  expect_identical(vec_restore(proxy, y), y)
})

# Arithmetic and factor ---------------------------------------------------

test_that("factors don't support math or arthimetic", {
  f <- factor("x")
  expect_error(vec_math("sum", f), class = "vctrs_error_unsupported")
  expect_error(vec_arith("+", f, f), class = "vctrs_error_unsupported")
})

test_that("vec_chop() throws error with non-vector inputs", {
  expect_error(vec_chop(NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_chop(environment()), class = "vctrs_error_scalar_type")
})

test_that("atomics are split into a list", {
  x <- 1:5
  expect_equal(vec_chop(x), as.list(x))

  x <- letters[1:5]
  expect_equal(vec_chop(x), as.list(x))
})

test_that("atomic names are kept", {
  x <- set_names(1:5)
  result <- lapply(vec_chop(x), names)
  expect_equal(result, as.list(names(x)))
})

test_that("base R classed objects are split into a list", {
  fctr <- factor(c("a", "b"))
  expect <- lapply(vec_seq_along(fctr), vec_slice, x = fctr)
  expect_equal(vec_chop(fctr), expect)

  date <- new_date(c(0, 1))
  expect <- lapply(vec_seq_along(date), vec_slice, x = date)
  expect_equal(vec_chop(date), expect)
})

test_that("base R classed object names are kept", {
  fctr <- set_names(factor(c("a", "b")))
  result <- lapply(vec_chop(fctr), names)
  expect_equal(result, as.list(names(fctr)))
})

test_that("list elements are split", {
  x <- list(1, 2)
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("data frames are split rowwise", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("vec_chop() keeps data frame row names", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  rownames(x) <- c("r1", "r2")
  result <- lapply(vec_chop(x), rownames)
  expect_equal(result, list("r1", "r2"))
})

test_that("vec_chop() keeps data frame row names for data frames with 0 columns (#1722)", {
  x <- data_frame(.size = 3)
  rownames(x) <- c("r1", "r2", "r3")

  out <- lapply(vec_chop(x), rownames)
  expect_identical(out, list("r1", "r2", "r3"))

  out <- vec_chop(x, indices = list(c(2, NA), 3))
  out <- lapply(out, rownames)
  expect_identical(out, list(c("r2", "...2"), "r3"))

  out <- vec_chop(x, sizes = c(1, 2, 0))
  out <- lapply(out, rownames)
  expect_identical(out, list("r1", c("r2", "r3"), character()))
})

test_that("data frames with 0 columns retain the right number of rows (#1722)", {
  x <- data_frame(.size = 4)

  one <- data_frame(.size = 1L)
  expect_identical(
    vec_chop(x),
    list(one, one, one, one)
  )

  expect_identical(
    vec_chop(x, indices = list(c(1, 3, 2), c(3, NA))),
    list(
      data_frame(.size = 3),
      data_frame(.size = 2)
    )
  )

  expect_identical(
    vec_chop(x, sizes = c(3, 1, 0)),
    list(
      data_frame(.size = 3),
      data_frame(.size = 1),
      data_frame(.size = 0)
    )
  )
})

test_that("matrices / arrays are split rowwise", {
  x <- array(1:12, c(2, 2, 2))
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("matrix / array row names are kept", {
  x <- array(1:12, c(2, 2, 2), dimnames = list(c("r1", "r2"), c("c1", "c2")))
  result <- lapply(vec_chop(x), rownames)
  expect_equal(result, list("r1", "r2"))
})

test_that("matrices / arrays without row names have other dimension names kept", {
  x <- array(1:12, c(2, 2, 2), dimnames = list(NULL, c("c1", "c2")))
  result <- lapply(vec_chop(x), colnames)
  expect_equal(result, list(c("c1", "c2"), c("c1", "c2")))
})

test_that("vec_chop() doesn't restore when attributes have already been restored", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) structure("dispatched", foo = "bar"),
    vec_restore.vctrs_foobar = function(...) {
      structure("dispatched-and-restored", foo = "bar")
    }
  )

  result <- vec_chop(foobar(NA))[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))

  result <- vec_chop(foobar(NA), indices = list(1))[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))

  result <- vec_chop(foobar(NA), sizes = 1)[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))
})

test_that("vec_chop() does not restore when attributes have not been restored by `[`", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) "dispatched",
    vec_restore.vctrs_foobar = function(...) "dispatched-and-restored"
  )

  result <- vec_chop(foobar(NA))[[1]]
  expect_equal(result, "dispatched")

  result <- vec_chop(foobar(NA), indices = list(1))[[1]]
  expect_equal(result, "dispatched")

  result <- vec_chop(foobar(NA), sizes = 1)[[1]]
  expect_equal(result, "dispatched")
})

test_that("vec_chop() falls back to `[` for shaped objects with no proxy", {
  x <- foobar(1)
  dim(x) <- c(1, 1)

  result <- vec_chop(x)[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, indices = list(1))[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, sizes = 1)[[1]]
  expect_equal(result, x)
})

test_that("`indices` are validated", {
  expect_snapshot(error = TRUE, {
    vec_chop(1, indices = 1)
  })
  expect_snapshot({
    (expect_error(
      vec_chop(1, indices = list(1.5)),
      class = "vctrs_error_subscript_type"
    ))
  })
  expect_snapshot({
    (expect_error(
      vec_chop(1, indices = list(2)),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("`sizes` are validated", {
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = "a")
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = 2)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = -1)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = NA_integer_)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = c(1, 1))
  })
})

test_that("can't use both `indices` and `sizes`", {
  expect_snapshot(error = TRUE, {
    vec_chop(1, indices = list(1), sizes = 1)
  })
})

test_that("`sizes` allows `0`", {
  expect_identical(
    vec_chop(c("a", "b"), sizes = c(1, 0, 0, 1, 0)),
    list("a", character(), character(), "b", character())
  )
})

test_that("size 0 `indices` list is allowed", {
  expect_equal(vec_chop(1, indices = list()), list())
})

test_that("individual index values of size 0 are allowed", {
  expect_equal(vec_chop(1, indices = list(integer())), list(numeric()))

  df <- data.frame(a = 1, b = "1")
  expect_equal(vec_chop(df, indices = list(integer())), list(vec_ptype(df)))
})

test_that("individual index values of `NULL` are allowed", {
  expect_equal(vec_chop(1, indices = list(NULL)), list(numeric()))

  df <- data.frame(a = 1, b = "1")
  expect_equal(vec_chop(df, indices = list(NULL)), list(vec_ptype(df)))
})

test_that("data frame row names are kept when `indices` or `sizes` are used", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  rownames(x) <- c("r1", "r2")

  result <- lapply(vec_chop(x, indices = list(1, 1:2)), rownames)
  expect_equal(result, list("r1", c("r1", "r2")))

  result <- lapply(vec_chop(x, sizes = c(1, 0, 1)), rownames)
  expect_equal(result, list("r1", character(), "r2"))
})

test_that("vec_chop(<atomic>, indices/sizes =) can be equivalent to the default", {
  x <- 1:5

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("vec_chop(<data.frame>, indices/sizes =) can be equivalent to the default", {
  x <- data.frame(x = 1:5)

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("vec_chop(<array>, indices/sizes =) can be equivalent to the default", {
  x <- array(1:8, c(2, 2, 2))

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("`indices` cannot use names", {
  x <- set_names(1:3, c("a", "b", "c"))
  expect_error(
    vec_chop(x, indices = list("a", c("b", "c"))),
    class = "vctrs_error_subscript_type"
  )

  x <- array(1:4, c(2, 2), dimnames = list(c("r1", "r2")))
  expect_error(
    vec_chop(x, indices = list("r1")),
    class = "vctrs_error_subscript_type"
  )

  x <- data.frame(x = 1, row.names = "r1")
  expect_error(
    vec_chop(x, indices = list("r1")),
    class = "vctrs_error_subscript_type"
  )
})

test_that("fallback method with `indices` and `sizes` works", {
  fctr <- factor(c("a", "b"))

  indices <- list(1, c(1, 2))
  sizes <- c(1, 0, 1)

  expect_equal(
    vec_chop(fctr, indices = indices),
    map(indices, vec_slice, x = fctr)
  )
  expect_equal(
    vec_chop(fctr, sizes = sizes),
    list(vec_slice(fctr, 1), vec_slice(fctr, 0), vec_slice(fctr, 2))
  )
})

test_that("vec_chop() falls back to `[` for shaped objects with no proxy when `indices` or `sizes` are provided", {
  x <- foobar(1)
  dim(x) <- c(1, 1)

  result <- vec_chop(x, indices = list(1))[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, sizes = 1)[[1]]
  expect_equal(result, x)
})

test_that("vec_chop() with data frame proxies always uses the proxy's length info", {
  local_methods(
    vec_proxy.vctrs_proxy = function(x, ...) {
      x <- proxy_deref(x)
      new_data_frame(list(x = x$x, y = x$y))
    },
    vec_restore.vctrs_proxy = function(x, to, ...) {
      new_proxy(list(x = x$x, y = x$y))
    }
  )

  x <- new_proxy(list(x = 1:2, y = 3:4))
  result <- vec_chop(x)

  result1 <- result[[1]]
  result2 <- result[[2]]

  expect1 <- new_proxy(list(x = 1L, y = 3L))
  expect2 <- new_proxy(list(x = 2L, y = 4L))

  expect_identical(proxy_deref(result1), proxy_deref(expect1))
  expect_identical(proxy_deref(result2), proxy_deref(expect2))
})

test_that("ALTREP objects always generate materialized chops (#1450)", {
  x <- chr_rle(foo = 10L, bar = 5L)

  # `x` starts in compact form
  expect_false(chr_rle_is_materialized(x))

  result <- vec_chop(x)

  # Chopping materializes `x`
  expect_true(chr_rle_is_materialized(x))

  # And chopped elements are not ALTREP vectors
  expect_false(any(map_lgl(result, is_altrep)))

  expect <- vec_chop(c(rep("foo", 10), rep("bar", 5)))
  expect_identical(result, expect)
})

test_that("`vec_chop(x, indices)` backwards compatible behavior works", {
  # No issues here
  expect_identical(
    vec_chop(1:2, list(1, 2)),
    vec_chop(1:2, indices = list(1, 2))
  )

  # Errors still talk about `indices`
  expect_snapshot(error = TRUE, {
    vec_chop(1:2, 1)
  })
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), sizes = 1)
  })

  # These cases aren't allowed because they weren't possible previously either
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), 2)
  })
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), indices = list(1))
  })
})

# vec_chop + compact_seq --------------------------------------------------

# `start` is 0-based

test_that("can chop base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  expect_identical(vec_chop_seq(lgl(1, 0, 1), start, size), list(lgl(0, 1)))
  expect_identical(vec_chop_seq(int(1, 2, 3), start, size), list(int(2, 3)))
  expect_identical(vec_chop_seq(dbl(1, 2, 3), start, size), list(dbl(2, 3)))
  expect_identical(vec_chop_seq(cpl(1, 2, 3), start, size), list(cpl(2, 3)))
  expect_identical(
    vec_chop_seq(chr("1", "2", "3"), start, size),
    list(chr("2", "3"))
  )
  expect_identical(vec_chop_seq(raw2(1, 2, 3), start, size), list(raw2(2, 3)))
  expect_identical(vec_chop_seq(list(1, 2, 3), start, size), list(list(2, 3)))
})

test_that("can chop with a decreasing compact seq", {
  expect_equal(vec_chop_seq(int(1, 2, 3), 1L, 2L, FALSE), list(int(2, 1)))
})

test_that("can chop with multiple compact seqs", {
  start <- c(1L, 0L)
  size <- c(1L, 3L)

  expect_equal(
    vec_chop_seq(int(1, 2, 3), start, size),
    list(int(2), int(1, 2, 3))
  )
})

test_that("can chop S3 objects using the fallback method with compact seqs", {
  x <- factor(c("a", "b", "c", "d"))
  expect_equal(vec_chop_seq(x, 0L, 0L), list(vec_slice(x, integer())))
  expect_equal(vec_chop_seq(x, 0L, 1L), list(vec_slice(x, 1L)))
  expect_equal(vec_chop_seq(x, 2L, 2L), list(vec_slice(x, 3:4)))
})

test_that("data frames with 0 columns retain the right number of rows with compact seqs (#1722)", {
  x <- data_frame(.size = 4)

  out <- vec_chop_seq(x, starts = c(0L, 0L, 2L), sizes = c(0L, 2L, 1L))
  out <- map_int(out, vec_size)

  expect_identical(out, c(0L, 2L, 1L))
})

test_that("`vec_chop()` can't take `compact_seq()` indices directly", {
  expect_snapshot(error = TRUE, transform = scrub_internal_error_line_number, {
    vec_chop(1:2, indices = list(compact_seq(1, 2)))
  })
})

# Vectorised --------------------------------------------------------------

test_that("vec_hash() produces same hash for same values", {
  x <- vec_hash(1:3)
  y <- do.call(c, map(1:3, vec_hash))
  expect_identical(x, y)
})

test_that("F, T, and NA hash to different values", {
  x <- map(c(TRUE, FALSE, NA), vec_hash)
  expect_length(unique(x), 3)
})

test_that("vec_hash of double produces different values", {
  x <- vec_hash(c(1, 1, 2))
  expect_true(identical(x[1:4], x[5:8]))
  expect_false(identical(x[5:8], x[9:12]))
})

test_that("NA and NaN hash to different values", {
  x <- vec_hash(c(NA, NaN))
  expect_false(identical(x[1:4], x[5:8]))
})

test_that("same string hashes to same value", {
  x <- vec_hash(c("1", "1", "2"))
  expect_true(identical(x[1:4], x[5:8]))
  expect_false(identical(x[5:8], x[9:12]))
})

test_that("list hashes to values of individual values", {
  cpl <- complex(real = c(1, 2), imaginary = c(3, 4))

  x <- vec_hash(list(1:3, letters, cpl))
  expect_identical(x[1:4], obj_hash(1:3))
  expect_identical(x[5:8], obj_hash(letters))
  expect_identical(x[9:12], obj_hash(cpl))

  x <- map(list(list(1:3), list(letters), list(cpl)), vec_hash)
  expect_identical(x[[1]], obj_hash(1:3))
  expect_identical(x[[2]], obj_hash(letters))
  expect_identical(x[[3]], obj_hash(cpl))
})

test_that("hash of data frame works down rows", {
  df <- data.frame(x = 1:3, y = 1:3)
  x <- vec_hash(df)
  expect_length(x, 4 * vec_size(df))
  expect_identical(x[1:4], vec_hash(df[1, ]))
})

test_that("hashes are consistent from run to run", {
  # no string, since we're currently hashing the address in string pool
  df <- list(
    lgl = c(TRUE, FALSE, NA),
    int = 1:100,
    dbl1 = as.double(1:100),
    dbl2 = seq(0, 1, length = 100)
  )
  hash <- lapply(df, vec_hash)

  # Big-endian results are byte-swapped, but otherwise equivalent.
  # Swap results so that there's no need to save results twice.
  if (.Platform$endian == "big") {
    hash <- lapply(
      hash,
      function(x) {
        writeBin(readBin(x, "int", 100, endian = "big"), x, endian = "little")
      }
    )
  }

  local_options(max.print = 99999)
  expect_snapshot(hash)
})

test_that("can hash list of non-vectors", {
  x <- list(quote(x), mean)

  expect_equal(
    vec_hash(x),
    c(obj_hash(x[[1]]), obj_hash(x[[2]]))
  )
})

test_that("can hash matrices", {
  x <- matrix(c(1, 1, 1, 2, 2, 1), c(3, 2))

  expect_identical(
    vec_hash(x),
    vec_hash(x)
  )

  x <- matrix(c(1, 2, 3, 4), c(2, 2))

  expect_identical(
    vec_hash(x),
    vec_hash(x)
  )

  expect_false(identical(
    vec_hash(x),
    vec_hash(c(1, 2))
  ))

  y <- matrix(c(1, 2, 3, 5), c(2, 2))

  expect_false(identical(
    vec_hash(x),
    vec_hash(y)
  ))
})

test_that("can hash NA", {
  expect_identical(
    vec_hash(NA),
    vec_hash(NA),
  )
})

test_that("can hash 1D arrays", {
  # 1D arrays are dispatched to `as.data.frame.vector()` which
  # currently does not strip dimensions. This caused an infinite
  # recursion.
  expect_length(vec_hash(array(1:2)), 8)
  expect_identical(vec_hash(array(1:2)), vec_hash(1:2))
})

test_that("can hash raw vectors", {
  expect_identical(
    vec_hash(0:255),
    vec_hash(as.raw(0:255))
  )
  expect_identical(
    obj_hash(0:255),
    obj_hash(as.raw(0:255))
  )
})

test_that("can hash complex vectors", {
  expect_identical(
    vec_hash(c(1, 2) + 0i),
    c(obj_hash(c(1, 0)), obj_hash(c(2, 0)))
  )
  expect_identical(
    vec_hash(list(c(1, 2) + 0i)),
    obj_hash(c(1, 2) + 0i)
  )
})

test_that("hash treats positive and negative 0 as equivalent (#637)", {
  expect_equal(vec_hash(-0), vec_hash(0))
})

test_that("can hash lists of expressions", {
  expect_equal(
    vec_hash(list(expression(x), expression(y))),
    c(obj_hash(expression(x)), obj_hash(expression(y)))
  )
})

test_that("vec_hash() uses recursive equality proxy", {
  x <- new_data_frame(list(x = foobar(1:3)))
  default <- vec_hash(x)

  local_methods(vec_proxy_equal.vctrs_foobar = function(...) c(0, 0, 0))
  overridden <- vec_hash(x)

  expect_false(identical(default, overridden))
})


# Object ------------------------------------------------------------------

test_that("equal objects hash to same value", {
  # just test function since they'll recurse through every other object type
  f1 <- function(x, y = NULL) x + y
  f2 <- function(x, y = NULL) x + y
  expect_false(identical(obj_hash(f1), obj_hash(f2)))
  expect_false(identical(
    vec_hash(data_frame(x = list(f1))),
    vec_hash(data_frame(x = list(f2)))
  ))

  attr(f1, "srcref") <- NULL
  attr(f2, "srcref") <- NULL
  expect_equal(obj_hash(f1), obj_hash(f2))
  expect_equal(
    vec_hash(data_frame(x = list(f1))),
    vec_hash(data_frame(x = list(f2)))
  )
})

test_that("expression vectors hash to the same value as lists of calls/names", {
  expect_equal(
    obj_hash(expression(x, y)),
    obj_hash(list(as.name("x"), as.name("y")))
  )

  expect_equal(
    obj_hash(expression(mean(), sd())),
    obj_hash(list(call("mean"), call("sd")))
  )
})

test_that("language argument names are considered", {
  expect_false(identical(
    obj_hash(call("fn", foo = 1)),
    obj_hash(call("fn", bar = 1))
  ))
})

test_that("pairlist tags are considered", {
  expect_false(identical(
    obj_hash(pairlist(foo = 1)),
    obj_hash(pairlist(bar = 1))
  ))
})

test_that("attribute pairlist tags are considered", {
  expect_false(identical(
    obj_hash(structure(1, foo = 1)),
    obj_hash(structure(1, bar = 1))
  ))
})

encodings <- function(bytes = FALSE) {
  string <- "\u00B0C"

  utf8 <- iconv(string, from = Encoding(string), to = "UTF-8")
  unknown <- iconv(string, from = Encoding(string), to = "", mark = FALSE)
  latin1 <- iconv(string, from = Encoding(string), to = "latin1")

  out <- list(utf8 = utf8, unknown = unknown, latin1 = latin1)

  if (bytes) {
    out <- list2(!!!out, bytes = encoding_bytes())
  }

  out
}

encoding_bytes <- function() {
  string <- "\u00B0C"

  unknown <- iconv(string, from = Encoding(string), to = "", mark = FALSE)

  bytes <- unknown
  Encoding(bytes) <- "bytes"

  bytes
}

expect_equal_encoding <- function(object, expected) {
  args <- vec_recycle_common(object, expected)
  expect_identical(Encoding(args[[1L]]), Encoding(args[[2L]]))
}
expect_utf8_encoded <- function(object) {
  expect_identical(Encoding(object), rep("UTF-8", length(object)))
}
expect_latin1_encoded <- function(object) {
  expect_identical(Encoding(object), rep("latin1", length(object)))
}

# Example usage of ptype2 and cast restart. This handler treats any
# input that inherits from <ordered> as a <factor>. In other words, it
# allows incompatible <ordered> inputs to benefit from all <factor>
# coercion methods.
with_ordered_restart <- function(expr) {
  withCallingHandlers(
    expr,
    vctrs_error_incompatible_type = function(cnd) {
      x <- cnd[["x"]]
      y <- cnd[["y"]]

      restart <- FALSE

      if (is.ordered(x)) {
        restart <- TRUE
        x <- factor(as.character(x), levels = levels(x))
      }
      if (is.ordered(y)) {
        restart <- TRUE
        y <- factor(as.character(y), levels = levels(y))
      }

      # Don't recurse and let ptype2 error keep its course
      if (!restart) {
        return(zap())
      }

      x_arg <- cnd[["x_arg"]]
      y_arg <- cnd[["y_arg"]]
      call <- cnd[["call"]]

      # Recurse with factor methods and restart with the result
      if (inherits(cnd, "vctrs_error_ptype2")) {
        out <- vec_ptype2(x, y, x_arg = x_arg, y_arg = y_arg, call = call)
        restart <- "vctrs_restart_ptype2"
      } else if (inherits(cnd, "vctrs_error_cast")) {
        out <- vec_cast(x, y, x_arg = x_arg, to_arg = y_arg, call = call)
        restart <- "vctrs_restart_cast"
      } else {
        return(zap())
      }

      # Old-R compat for `tryInvokeRestart()`
      try_restart <- function(restart, ...) {
        if (!is_null(findRestart(restart))) {
          invokeRestart(restart, ...)
        }
      }
      try_restart(restart, out)
    }
  )
}

with_subscript_data <- function(expr,
                                subscript_arg,
                                subscript_elt = NULL,
                                subscript_action = NULL) {
  local_options(rlang_force_unhandled_error = TRUE)
  tryCatch(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- subscript_arg
      cnd$subscript_elt <- subscript_elt
      cnd$subscript_action <- subscript_action
      cnd_signal(cnd)
    }
  )
}

with_tibble_cols <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "rename"
  )
}
with_tibble_rows <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "row",
    subscript_action = "remove"
  )
}
with_dm_tables <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "table",
    subscript_action = "extract"
  )
}
with_tidyselect_select <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "select"
  )
}
with_tidyselect_relocate <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "relocate"
  )
}

my_vec_rep <- function(my_x, my_times) {
  vec_rep(
    my_x,
    my_times,
    error_call = current_env(),
    x_arg = "my_x",
    times_arg = "my_times"
  )
}

my_vec_rep_each <- function(my_x, my_times) {
  vec_rep_each(
    my_x,
    my_times,
    error_call = current_env(),
    x_arg = "my_x",
    times_arg = "my_times"
  )
}

my_vec_as_names <- function(my_names,
                            ...,
                            my_repair = "minimal",
                            my_quiet = FALSE) {
  vec_as_names(
    my_names,
    repair = my_repair,
    repair_arg = "my_repair",
    quiet = my_quiet
  )
}

# Don't call tibble::tibble() to avoid catch-22, because tibble now uses vctrs
bare_tibble <- structure(data.frame(), class = c("tbl_df", "tbl", "data.frame"))

base_empty_types <- list(
  null = NULL,
  logical = lgl(),
  integer = int(),
  double = dbl(),
  complex = cpl(),
  character = chr(),
  raw = bytes(),
  list = list(),
  dataframe = data.frame()
)

base_s3_empty_types <- list(
  bare_factor = new_factor(),
  bare_ordered = new_ordered(),
  bare_date = new_date(),
  bare_posixct = new_datetime(tzone = "UTC"),
  bare_posixlt = as.POSIXlt(new_datetime(tzone = "UTC")),
  bare_tibble = bare_tibble
)

proxied_empty_types <- list(
  double = new_hidden(),
  dataframe = bare_tibble,
  dataframe = structure(data.frame(), class = c("vctrs_foobar", "data.frame"))
)

empty_types <- c(
  base_empty_types,
  proxied_empty_types,
  # Non proxied type
  scalar = foobar(list()),
  scalar = function() NULL
)

atomics <- list(TRUE, 1L, 1.0, 1i, "foo", bytes(1))
vectors <- c(atomics, list(list()))
records <- list(
  df = data.frame(x = 1),
  rcrd = new_rcrd(list(x = 1)),
  posixlt = as.POSIXlt("2020-01-01")
)

tuple <- function(x = integer(), y = integer()) {
  fields <- vec_recycle_common(
    x = vec_cast(x, integer()),
    y = vec_cast(y, integer())
  )
  new_rcrd(fields, class = "vctrs_tuple")
}

tuple_methods <- list(
  format.vctrs_tuple = function(x, ...) {
    paste0("(", field(x, "x"), ",", field(x, "y"), ")")
  },
  vec_ptype2.vctrs_tuple.vctrs_tuple = function(x, y, ...) x,
  vec_cast.vctrs_tuple.vctrs_tuple = function(x, to, ...) x
)

local_tuple_methods <- function(frame = caller_env()) {
  local_methods(.frame = frame, !!!tuple_methods)
}
set_tuple_methods <- function(env = global_env()) {
  env_bind(env, !!!tuple_methods)
}


local_comparable_tuple <- function(frame = caller_env()) {
  local_tuple_methods(frame = frame)

  # Compare only on first field
  local_methods(
    .frame = frame,
    vec_proxy_equal.vctrs_tuple = function(x, ...) field(x, "x")
  )
}

c_na <- function(...) {
  x <- c(...)
  names(x)[names(x) == ""] <- NA_character_
  x
}

# Keep in sync with macros in `order.c`
GROUP_DATA_SIZE_DEFAULT <- 100000L
ORDER_INSERTION_BOUNDARY <- 128L
INT_ORDER_COUNTING_RANGE_BOUNDARY <- 100000L

# Force radix method for character comparisons
base_order <- function(x, na.last = TRUE, decreasing = FALSE) {
  if (is.data.frame(x)) {
    x <- unname(x)
  } else {
    x <- list(x)
  }

  args <- list(na.last = na.last, decreasing = decreasing)

  # `method` didn't exist on R < 3.3.
  # It would sometimes use radix sorting automatically.
  if (getRversion() < "3.3.0") {
    method <- list()
  } else {
    method <- list(method = "radix")
  }

  args <- c(x, args, method)


  rlang::exec("order", !!!args)
}

.rando <- setClass(
  "vctrs_rando",
  contains = "numeric",
  slots = list(.Data = "numeric")
)
rando <- function(n = 0) {
  .rando(as.numeric(seq_len(n)))
}

as_rando <- function(x) {
  rando(length(x))
}

setMethod("[", "vctrs_rando", function(x, i, j, ..., drop = TRUE) {
  new_n <- length(vec_as_location(i, length(x@.Data), names(x@.Data)))
  rando(new_n)
})

.Counts <- methods::setClass(
  "vctrs_Counts",
  contains = "integer",
  slots = c(name = "character")
)

local_c_counts <- function(frame = caller_env()) {
  c_counts <- function(x, ...) {
    xs <- list(x, ...)

    xs_data <- lapply(xs, function(x) x@.Data)
    new_data <- do.call(c, xs_data)

    .Counts(new_data, name = "Dispatched")
  }

  local_s4_method(
    frame = frame,
    "c",
    methods::signature(x = "vctrs_Counts"),
    c_counts
  )
}


local_s4_method <- function(generic, signature, method, frame = caller_env()) {
  methods::setMethod(generic, signature, method)
  exit_expr <- call2(methods::removeMethod, generic, signature, where = topenv(frame))
  local_exit(exit_expr, frame = frame)
}
with_s4_method <- function(generic, signature, method, expr) {
  local_s4_method(generic, signature, method)
  expr
}

local_exit <- function(expr, frame = caller_env()) {
  # We are at top-level when only one frame refers to the global environment
  if (is_reference(frame, global_env())) {
    is_global_frame <- sys.parents() == 0
    if (sum(is_global_frame) == 1) {
      abort("Can't add an exit event at top-level")
    }
  }

  # Inline everything so the call will succeed in any environment
  expr <- call2(on.exit, expr, add = TRUE)
  eval_bare(expr, frame)

  invisible(expr)
}

skip_if_cant_set_names_on_s4 <- function() {
  skip_if(getRversion() < "3.5.0", message = "Can't set names on S4 objects")
}

# Rational record class from the S3 vector vignette

new_rational <- function(n = integer(), d = integer()) {
  if (!is_integer(n)) {
    abort("`n` must be an integer.")
  }
  if (!is_integer(d)) {
    abort("`d` must be an integer.")
  }
  new_rcrd(list(n = n, d = d), class = "vctrs_rational")
}

rational <- function(n, d) {
  args <- vec_cast_common(n, d, .to = integer())
  args <- vec_recycle_common(!!! args)
  new_rational(args[[1L]], args[[2L]])
}

format.vctrs_rational <- function(x, ...) {
  n <- field(x, "n")
  d <- field(x, "d")
  out <- paste0(n, "/", d)
  out[is.na(n) | is.na(d)] <- NA
  out
}

vec_proxy_equal.vctrs_rational <- function(x) {
  n <- field(x, "n")
  d <- field(x, "d")
  gcd <- gcd(n, d)
  data.frame(n = n / gcd, d = d / gcd)
}
gcd <- function(x, y) {
  r <- x %% y
  ifelse(r, gcd(y, r), y)
}

vec_proxy_compare.vctrs_rational <- function(x, ...) {
  field(x, "n") / field(x, "d")
}

rational_methods <- list(
  vec_ptype_abbr.vctrs_rational = function(x, ...) "rtnl",
  vec_ptype_full.vctrs_rational = function(x, ...) "rational",

  vec_ptype2.vctrs_rational = function(x, y, ...) UseMethod("vec_ptype2.vctrs_rational"),
  vec_ptype2.vctrs_rational.vctrs_rational = function(x, y, ...) new_rational(),
  vec_ptype2.vctrs_rational.integer = function(x, y, ...) new_rational(),
  vec_ptype2.integer.vctrs_rational = function(x, y, ...) new_rational(),

  vec_cast.vctrs_rational = function(x, to, ...) UseMethod("vec_cast.vctrs_rational"),
  vec_cast.vctrs_rational.vctrs_rational = function(x, to, ...) x,
  vec_cast.double.vctrs_rational = function(x, to, ...) field(x, "n") / field(x, "d"),
  vec_cast.vctrs_rational.integer = function(x, to, ...) rational(x, 1),

  vec_proxy_equal.vctrs_rational = vec_proxy_equal.vctrs_rational,
  vec_proxy_compare.vctrs_rational = vec_proxy_compare.vctrs_rational
)

local_rational_class <- function(frame = caller_env()) {
  local_methods(.frame = frame, !!!rational_methods)
}

# These tests check the rational type from the S3 vignette

test_that("equality proxy is taken (#375)", {
  local_rational_class()
  x <- rational(c(1, 2, 1, 2, 6), c(1, 1, 2, 2, 2))

  expect_identical(x == rational(3, 1), c(FALSE, FALSE, FALSE, FALSE, TRUE))

  expect_identical(unique(x), rational(c(1, 2, 1, 6), c(1, 1, 2, 2)))
})

test_that("order proxy is taken", {
  local_rational_class()
  x <- rational(c(1, 2, 1, 2, 6), c(1, 1, 2, 2, 2))
  expect_identical(sort(x), rational(c(1, 1, 2, 2, 6), c(2, 1, 2, 1, 2)))
})

test_that("can find common type and cast to rational", {
  local_rational_class()
  x <- rational(1:2, 2:1)
  expect_identical(vec_cast_common(x, x), list(x, x))
})

# These tests check the rational type from the S3 vignette

test_that("equality proxy is taken (#375)", {
  local_rational_class()
  x <- rational(c(1, 2, 1, 2, 6), c(1, 1, 2, 2, 2))

  expect_identical(x == rational(3, 1), c(FALSE, FALSE, FALSE, FALSE, TRUE))

  expect_identical(unique(x), rational(c(1, 2, 1, 6), c(1, 1, 2, 2)))
})

test_that("order proxy is taken", {
  local_rational_class()
  x <- rational(c(1, 2, 1, 2, 6), c(1, 1, 2, 2, 2))
  expect_identical(sort(x), rational(c(1, 1, 2, 2, 6), c(2, 1, 2, 1, 2)))
})

test_that("can find common type and cast to rational", {
  local_rational_class()
  x <- rational(1:2, 2:1)
  expect_identical(vec_cast_common(x, x), list(x, x))
})

skip_if_not_testing_performance <- function(x) {
  opt <- Sys.getenv("VCTRS_TEST_PERFORMANCE", unset = "false")
  testing <- identical(opt, "true")

  if (testing) {
    return()
  }

  skip("Not testing performance")
}

expect_time_lt <- function(expr, expect) {
  time <- time_of({{ expr }})
  expect_lt(time, expect)
}

time_of <- function(expr) {
  expr <- enquo(expr)
  time <- system.time(eval_tidy(expr))
  unclass(time)[["elapsed"]]
}

# From r-lib/bench
with_memory_prof <- function(expr) {
  f <- tempfile()
  on.exit(unlink(f))

  tryCatch(
    utils::Rprofmem(f, threshold = 1),
    error = function(...) skip("Can't profile memory on this system.")
  )
  on.exit(utils::Rprofmem(NULL), add = TRUE)

  res <- force(expr)
  utils::Rprofmem(NULL)

  bytes <- parse_allocations(f)$bytes
  bytes <- sum(bytes, na.rm = TRUE)
  new_vctrs_bytes(bytes)
}
parse_allocations <- function(filename) {
  if (!is_installed("profmem")) {
    testthat::skip("profmem must be installed.")
  }
  readRprofmem <- env_get(ns_env("profmem"), "readRprofmem")

  tryCatch(
    readRprofmem(filename),
    error = function(cnd) {
      testthat::skip(sprintf(
        "Memory profiling failed: %s",
        conditionMessage(cnd)
      ))
    }
  )
}

new_ctor <- function(base_class) {
  function(x = list(), ..., class = NULL) {
    if (inherits(x, "tbl_df")) {
      tibble::new_tibble(x, class = c(class, base_class), nrow = nrow(x))
    } else if (is.data.frame(x)) {
      structure(x, class = c(class, base_class, "data.frame"), ...)
    } else {
      structure(x, class = c(class, base_class), ...)
    }
  }
}

foobar <- new_ctor("vctrs_foobar")
foobaz <- new_ctor("vctrs_foobaz")
quux <- new_ctor("vctrs_quux")

expect_foobar <- function(x) expect_s3_class({{ x }}, "vctrs_foobar")
expect_foobaz <- function(x) expect_s3_class({{ x }}, "vctrs_foobaz")
expect_quux <- function(x) expect_s3_class({{ x }}, "vctrs_quux")

with_c_foobar <- function(expr) {
  with_methods(
    expr,
    c.vctrs_foobar = function(...) foobar(NextMethod())
  )
}

unrownames <- function(x) {
  row.names(x) <- NULL
  x
}

local_methods <- function(..., .frame = caller_env()) {
  local_bindings(..., .env = global_env(), .frame = .frame)
}
with_methods <- function(.expr, ...) {
  local_methods(...)
  .expr
}

local_proxy <- function(frame = caller_env()) {
  local_methods(.frame = frame,
    vec_proxy.vctrs_proxy = function(x, ...) proxy_deref(x),
    vec_restore.vctrs_proxy = function(x, to, ...) new_proxy(x),

    vec_ptype2.vctrs_proxy = function(x, y, ...) UseMethod("vec_ptype2.vctrs_proxy"),
    vec_ptype2.vctrs_proxy.vctrs_proxy = function(x, y, ...) new_proxy(vec_ptype(proxy_deref(x))),

    vec_cast.vctrs_proxy = function(x, to, ...) UseMethod("vec_cast.vctrs_proxy"),
    vec_cast.vctrs_proxy.vctrs_proxy = function(x, to, ...) x
  )
}

new_proxy <- function(x) {
  structure(list(env(x = x)), class = "vctrs_proxy")
}
proxy_deref <- function(x, ...) {
  x[[1]]$x
}
local_env_proxy <- function(frame = caller_env()) {
  local_methods(.frame = frame,
    vec_proxy.vctrs_proxy = proxy_deref,
    vec_restore.vctrs_proxy = function(x, ...) new_proxy(x),
    vec_cast.vctrs_proxy = function(x, to, ...) UseMethod("vec_cast.vctrs_proxy"),
    vec_cast.vctrs_proxy.vctrs_proxy = function(x, to, ...) x,
    vec_ptype2.vctrs_proxy = function(x, y, ...) UseMethod("vec_ptype2.vctrs_proxy"),
    vec_ptype2.vctrs_proxy.vctrs_proxy = function(x, y, ...) new_proxy(proxy_deref(x)[0])
  )
}

local_no_stringsAsFactors <- function(frame = caller_env()) {
  local_options(.frame = frame, stringsAsFactors = FALSE)
}

tibble <- function(...) {
  tibble::tibble(...)
}

local_foobar_proxy <- function(frame = caller_env()) {
  local_methods(.frame = frame, vec_proxy.vctrs_foobar = function(x, ...) x)
}

subclass <- function(x) {
  class(x) <- c("vctrs_foo", "vctrs_foobar", class(x))
  x
}


# Subclass promoted to logical
new_lgl_subtype <- function(x) {
  stopifnot(is_logical(x))
  structure(x, class = "vctrs_lgl_subtype")
}
local_lgl_subtype <- function(frame = caller_env()) {
  local_methods(.frame = frame,
    vec_ptype2.vctrs_lgl_subtype = function(x, y, ...) UseMethod("vec_ptype2.vctrs_lgl_subtype"),
    vec_ptype2.vctrs_lgl_subtype.vctrs_lgl_subtype = function(x, y, ...) x,
    vec_ptype2.vctrs_lgl_subtype.logical = function(x, y, ...) y,
    vec_ptype2.logical.vctrs_lgl_subtype = function(x, y, ...) x,

    vec_cast.vctrs_lgl_subtype = function(x, to, ...) UseMethod("vec_cast.vctrs_lgl_subtype"),
    vec_cast.vctrs_lgl_subtype.vctrs_lgl_subtype = function(x, to, ...) x,
    vec_cast.vctrs_lgl_subtype.logical = function(x, to, ...) new_lgl_subtype(x),
    vec_cast.logical.vctrs_lgl_subtype = function(x, to, ...) unstructure(x)
  )
}
with_lgl_subtype <- function(expr) {
  local_lgl_subtype()
  expr
}

# Logical promoted to subclass
new_lgl_supertype <- function(x) {
  stopifnot(is_logical(x))
  structure(x, class = "vctrs_lgl_supertype")
}
local_lgl_supertype <- function(frame = caller_env()) {
  local_methods(.frame = frame,
    vec_ptype2.vctrs_lgl_supertype = function(x, y, ...) UseMethod("vec_ptype2.vctrs_lgl_supertype"),
    vec_ptype2.vctrs_lgl_supertype.vctrs_lgl_supertype = function(x, y, ...) x,
    vec_ptype2.vctrs_lgl_supertype.logical = function(x, y, ...) x,
    vec_ptype2.logical.vctrs_lgl_supertype = function(x, y, ...) y,

    vec_cast.vctrs_lgl_supertype = function(x, to, ...) UseMethod("vec_cast.vctrs_lgl_supertype"),
    vec_cast.vctrs_lgl_supertype.vctrs_lgl_supertype = function(x, to, ...) x,
    vec_cast.vctrs_lgl_supertype.logical = function(x, to, ...) new_lgl_subtype(x),
    vec_cast.logical.vctrs_lgl_supertype = function(x, to, ...) unstructure(x)
  )
}
with_lgl_supertype <- function(expr) {
  local_lgl_supertype()
  expr
}

foobar_df_ptype2 <- function(x, y, ...) {
  foobar(df_ptype2(x, y, ...))
}
foobar_df_cast <- function(x, y, ...) {
  foobar(df_cast(x, y, ...))
}
local_foobar_df_methods <- function(expr, frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_ptype2.vctrs_foobar.vctrs_foobar = foobar_df_ptype2,
    vec_ptype2.data.frame.vctrs_foobar = foobar_df_ptype2,
    vec_ptype2.vctrs_foobar.data.frame = foobar_df_ptype2,
    vec_cast.vctrs_foobar.vctrs_foobar = foobar_df_cast,
    vec_cast.data.frame.vctrs_foobar = foobar_df_cast,
    vec_cast.vctrs_foobar.data.frame = foobar_df_cast
  )
}
with_foobar_df_methods <- function(expr) {
  local_foobar_df_methods()
  expr
}

set_tibble <- function(x) {
  base <- class(x)[-length(class(x))]
  class(x) <- c(base, "tbl_df", "tbl", "data.frame")
  x
}

# printing ----------------------------------------------------------------

test_that("data frames print nicely", {
  expect_equal(vec_ptype_abbr(mtcars), "df[,11]")
  expect_snapshot(vec_ptype_show(mtcars))
  expect_snapshot(vec_ptype_show(iris))
})

test_that("embedded data frames print nicely", {
  df <- data.frame(x = 1:3)
  df$a <- data.frame(a = 1:3, b = letters[1:3])
  df$b <- list_of(1, 2, 3)
  df$c <- as_list_of(split(data.frame(x = 1:3, y = letters[1:3]), 1:3))

  expect_snapshot(vec_ptype_show(df))
})

# coercing ----------------------------------------------------------------

test_that("data frame only combines with other data frames or NULL", {
  dt <- data.frame(x = 1)
  expect_equal(vec_ptype_common(dt, NULL), vec_ptype(dt))
  expect_error(vec_ptype_common(dt, 1:10), class = "vctrs_error_incompatible_type")
})

test_that("data frame takes max of individual variables", {
  dt1 <- data.frame(x = FALSE, y = 1L)
  dt2 <- data.frame(x = 1.5, y = 1.5)

  expect_equal(vec_ptype_common(dt1, dt2), vec_ptype_common(dt2))
})

test_that("data frame combines variables", {
  dt1 <- data.frame(x = 1)
  dt2 <- data.frame(y = 1)

  dt3 <- max(dt1, dt2)
  expect_equal(
    vec_ptype_common(dt1, dt2),
    vec_ptype_common(data.frame(x = double(), y = double()))
  )
})

test_that("empty data frame still has names", {
  df <- data.frame()
  out <- vec_ptype_common(df, df)

  expect_equal(names(out), character())
})

test_that("combining data frames with foreign classes uses fallback", {
  foo <- foobar(data.frame())
  df <- data.frame()

  expect_identical(vec_ptype_common(foo, foo, foo), foo)
  expect_identical(vec_ptype_common(foo, foo, df, foo), df)

  expect_identical(vec_ptype2(foo, df), data.frame())
  expect_identical(vec_ptype2(df, foo), data.frame())
  expect_identical(vec_ptype_common(foo, df), df)
  expect_identical(vec_ptype_common(df, foo), df)

  cnds <- list()
  withCallingHandlers(
    warning = function(cnd) {
      cnds <<- append(cnds, list(cnd))
      invokeRestart("muffleWarning")
    },
    expect_identical(
      vec_ptype_common(foo, df, foo, foo),
      df
    )
  )

  # There are no longer any warnings
  expect_length(cnds, 0)

  expect_equal(
    vec_cbind(foobar(data.frame(x = 1)), data.frame(y = 2)),
    data.frame(x = 1, y = 2)
  )
  expect_equal(
    vec_rbind(foo, data.frame(), foo),
    df
  )
})


# casting -----------------------------------------------------------------

test_that("safe casts work as expected", {
  df <- data.frame(x = 1, y = 0)

  expect_equal(vec_cast(NULL, df), NULL)
  expect_equal(vec_cast(df, df), df)

  expect_equal(vec_cast(data.frame(x = TRUE, y = FALSE), df), df)
})

test_that("warn about lossy coercions", {
  df1 <- data.frame(x = factor("foo"), y = 1)
  df2 <- data.frame(x = factor("bar"))

  expect_lossy(vec_cast(df1, df1[1]), df1[1], x = df1, to = df1[1])

  expect_lossy(
    vec_cast(df1[1], df2),
    data.frame(x = factor(NA, levels = "bar")),
    x = factor("foo"),
    to = factor("bar")
  )

  out <-
    allow_lossy_cast(
      allow_lossy_cast(
        vec_cast(df1, df2),
        factor("foo"), factor("bar")
      ),
      df1, df2
    )

  expect_identical(out, data.frame(x = factor(NA, levels = "bar")))
})

test_that("invalid cast generates error", {
  expect_error(vec_cast(1L, data.frame()), class = "vctrs_error_incompatible_type")
})

test_that("column order matches type", {
  df1 <- data.frame(x = 1, y = "a")
  df2 <- data.frame(x = TRUE, z = 3)

  df3 <- vec_cast(df2, vec_ptype_common(df1, df2))
  expect_named(df3, c("x", "y", "z"))
})

test_that("restore generates correct row/col names", {
  df1 <- data.frame(x = NA, y = 1:4, z = 1:4)
  df1$x <- data.frame(a = 1:4, b = 1:4)

  df2 <- vec_restore(lapply(df1[1:3], vec_slice, 1:2), df1)

  expect_named(df2, c("x", "y", "z"))
  expect_equal(.row_names_info(df2), -2)
})

test_that("restore keeps automatic row/col names", {
  df1 <- data.frame(x = NA, y = 1:4, z = 1:4)
  df1$x <- data.frame(a = 1:4, b = 1:4)

  df2 <- vec_restore(df1, df1)

  expect_named(df2, c("x", "y", "z"))
  expect_equal(.row_names_info(df2), -4)
})

test_that("cast to empty data frame preserves number of rows", {
  out <- vec_cast(new_data_frame(n = 10L), new_data_frame())
  expect_equal(nrow(out), 10L)
})

test_that("can cast unspecified to data frame", {
  df <- data.frame(x = 1, y = 2L)
  expect_identical(vec_cast(unspecified(3), df), vec_init(df, 3))
})

test_that("cannot cast list to data frame", {
  df <- data.frame(x = 1, y = 2L)
  expect_error(vec_cast(list(df, df), df), class = "vctrs_error_incompatible_type")
})

test_that("can restore lists with empty names", {
  expect_identical(vec_restore(list(), data.frame()), data.frame())
})

test_that("can restore subclasses of data frames", {
  expect_identical(vec_restore(list(), subclass(data.frame())), subclass(data.frame()))
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ..., i) "dispatched"
  )
  expect_identical(vec_restore(list(), subclass(data.frame())), "dispatched")
})

test_that("df_cast() checks for names", {
  x <- new_data_frame(list(1))
  y <- new_data_frame(list(2))
  expect_error(vec_cast_common(x, y), "must have names")
})

test_that("casting to and from data frame preserves row names", {
  out <- vec_cast(mtcars, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))

  out <- vec_cast(out, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))
})

test_that("df_cast() evaluates arg lazily", {
  expect_silent(df_cast(data_frame(), data_frame(), x_arg = print("oof")))
  expect_silent(df_cast(data_frame(), data_frame(), to_arg = print("oof")))
})

# df_ptype2 ---------------------------------------------------------------

test_that("df_ptype2() evaluates arg lazily", {
  expect_silent(df_ptype2(data_frame(), data_frame(), x_arg = print("oof")))
  expect_silent(df_ptype2(data_frame(), data_frame(), y_arg = print("oof")))
})


# new_data_frame ----------------------------------------------------------

test_that("can construct an empty data frame", {
  expect_identical(new_data_frame(), data.frame())
})

test_that("can validly set the number of rows when there are no columns", {
  expect <- structure(
    list(),
    class = "data.frame",
    row.names = .set_row_names(2L),
    names = character()
  )

  expect_identical(new_data_frame(n = 2L), expect)
})

test_that("can add additional classes", {
  expect_s3_class(new_data_frame(class = "foobar"), "foobar")
  expect_s3_class(new_data_frame(class = c("foo", "bar")), c("foo", "bar"))
})

test_that("can add additional attributes", {
  expect <- data.frame()
  attr(expect, "foo") <- "bar"
  attr(expect, "a") <- "b"

  expect_identical(new_data_frame(foo = "bar", a = "b"), expect)
})

test_that("size is pulled from first column if not supplied", {
  x <- new_data_frame(list(x = 1:5, y = 1:6))
  expect_identical(.row_names_info(x, type = 1), -5L)
})

test_that("can construct a data frame without column names", {
  expect_named(new_data_frame(list(1, 2)), NULL)
})

test_that("the names on an empty data frame are an empty character vector", {
  expect_identical(names(new_data_frame()), character())
})

test_that("class attribute", {
  expect_identical(
    class(new_data_frame(list(a = 1))),
    "data.frame"
  )
  expect_identical(
    class(new_data_frame(list(a = 1), class = "tbl_df")),
    c("tbl_df", "data.frame")
  )
  expect_identical(
    class(new_data_frame(list(a = 1), class = c("tbl_df", "tbl", "data.frame"))),
    c("tbl_df", "tbl", "data.frame", "data.frame")
  )
  expect_identical(
    class(new_data_frame(list(a = 1), class = "foo_frame")),
    c("foo_frame", "data.frame")
  )
  expect_identical(
    class(exec(new_data_frame, list(a = 1), !!!attributes(new_data_frame(list(), class = "tbl_df")))),
    c("tbl_df", "data.frame", "data.frame")
  )
  expect_identical(
    class(exec(new_data_frame, list(a = 1), !!!attributes(new_data_frame(list(b = 1), class = "tbl_df")))),
    c("tbl_df", "data.frame", "data.frame")
  )
})

test_that("attributes with special names are merged", {
  expect_identical(
    names(new_data_frame(list(a = 1))),
    "a"
  )

  expect_identical(
    names(new_data_frame(list(a = 1), names = "name")),
    "name"
  )

  expect_identical(
    names(new_data_frame(list(1), names = "name")),
    "name"
  )

  expect_identical(
    attr(new_data_frame(list()), "row.names"),
    integer()
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L)),
    -3L
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L, row.names = 1:3)),
    3L
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L, row.names = c(NA, -3L))),
    -3L
  )

  expect_identical(
    attr(new_data_frame(list(), n = 1L, row.names = "rowname"), "row.names"),
    "rowname"
  )
})

test_that("n and row.names (#894)", {
  # Can omit n if row.names attribute is given
  expect_identical(
    row.names(new_data_frame(list(), row.names = "rowname")),
    "rowname"
  )
  expect_identical(
    attr(new_data_frame(list(), row.names = 2L), "row.names"),
    2L
  )
  expect_identical(
    row.names(new_data_frame(list(), row.names = chr())),
    chr()
  )
})

test_that("`row.names` completely overrides `n` and the implied size of `x`, even if incompatible (tidyverse/dplyr#6596)", {
  row_names <- c(NA, -3L)

  df <- new_data_frame(list(), n = 2L, row.names = row_names)
  expect_identical(.row_names_info(df, type = 0L), row_names)

  df <- new_data_frame(list(x = 1:2), row.names = row_names)
  expect_identical(.row_names_info(df, type = 0L), row_names)
})

test_that("ALTREP `row.names` are not materialized by `new_data_frame()` (tidyverse/dplyr#6596)", {
  skip_if(getRversion() <= "3.5.0")

  # We are careful in `new_data_frame()` to not call the `Dataptr()` or
  # `Length()` ALTREP methods, both of which would materialize our lazy
  # character here
  row_names <- new_lazy_character(~ c("a", "b"))

  x <- new_data_frame(list(), row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))

  x <- new_data_frame(list(x = 1:2), row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))

  x <- new_data_frame(list(), n = 2L, row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))
})

test_that("`x` must be a list", {
  expect_snapshot((expect_error(
    new_data_frame(1),
    "`x` must be a list"
  )))
})

test_that("if supplied, `n` must be an integer of size 1", {
  expect_snapshot({
    (expect_error(new_data_frame(n = c(1L, 2L)), "must be an integer of size 1"))
    (expect_error(new_data_frame(n = "x"), "must be an integer of size 1"))
  })
})

test_that("if supplied, `n` can't be negative or missing (#1477)", {
  expect_snapshot({
    (expect_error(new_data_frame(n = -1L)))
    (expect_error(new_data_frame(n = NA_integer_)))
  })
})

test_that("`class` must be a character vector", {
  expect_snapshot((expect_error(
    new_data_frame(class = 1),
    "must be NULL or a character vector"
  )))
})

test_that("flatten info is computed", {
  df_flatten_info <- function(x) {
    .Call(ffi_df_flatten_info, x)
  }
  expect_identical(df_flatten_info(mtcars), list(FALSE, ncol(mtcars)))

  df <- tibble(x = 1, y = tibble(x = 2, y = tibble(x = 3), z = 4), z = 5)
  expect_identical(df_flatten_info(df), list(TRUE, 5L))
})

test_that("can flatten data frames", {
  df_flatten <- function(x) {
    .Call(ffi_df_flatten, x)
  }
  expect_identical(df_flatten(mtcars), mtcars)

  df <- tibble(x = 1, y = tibble(x = 2, y = tibble(x = 3), z = 4), z = 5)
  expect_identical(df_flatten(df), new_data_frame(list(x = 1, x = 2, x = 3, z = 4, z = 5)))
})

test_that("can flatten data frames with rcrd columns (#1318)", {
  col <- new_rcrd(list(a = 1))
  df <- data_frame(col = col, y = 1)
  expect_identical(vec_proxy_equal(df), data_frame(col = 1, y = 1))

  col <- new_rcrd(list(a = 1, b = 2))
  df <- data_frame(col = col, y = 1)
  expect_identical(vec_proxy_equal(df), data_frame(a = 1, b = 2, y = 1))
})

test_that("new_data_frame() zaps existing attributes", {
  struct <- structure(list(), foo = 1)
  expect_identical(
    attributes(new_data_frame(struct)),
    attributes(new_data_frame(list())),
  )
  expect_identical(
    attributes(new_data_frame(struct, bar = 2)),
    attributes(new_data_frame(list(), bar = 2)),
  )
})

# data_frame --------------------------------------------------------------

test_that("data_frame() and df_list() report error context", {
  expect_snapshot({
    (expect_error(data_frame(a = 1, a = 1)))
    (expect_error(data_frame(a = 1, a = 1, .error_call = call("foo"))))
    (expect_error(data_frame(a = 1:2, b = int())))
    (expect_error(data_frame(a = 1:2, b = int(), .error_call = call("foo"))))

    (expect_error(df_list(a = 1, a = 1)))
    (expect_error(df_list(a = 1, a = 1, .error_call = call("foo"))))
    (expect_error(df_list(a = 1:2, b = int())))
    (expect_error(df_list(a = 1:2, b = int(), .error_call = call("foo"))))
  })
})

test_that("can construct data frames with empty input", {
  expect_identical(data_frame(), new_data_frame())
  expect_named(data_frame(), character())
})

test_that("input is tidy recycled", {
  expect_identical(
    data_frame(x = 1, y = 1:3),
    data_frame(x = c(1, 1, 1), y = 1:3)
  )

  expect_identical(
    data_frame(x = 1, y = integer()),
    data_frame(x = double(), y = integer())
  )

  expect_snapshot({
    expect_error(data_frame(1:2, 1:3), class = "vctrs_error_incompatible_size")
  })
})

test_that("dots are dynamic", {
  list_2_data_frame <- function(x) data_frame(!!!x)

  expect_identical(
    list_2_data_frame(list(x = 1, y = 2)),
    data_frame(x = 1, y = 2)
  )
})

test_that("unnamed input is auto named with empty strings", {
  expect_named(data_frame(1, 2, .name_repair = "minimal"), c("", ""))
})

test_that("unnamed data frames are auto unpacked", {
  expect_identical(
    data_frame(w = 1, data_frame(x = 2, y = 3), z = 4),
    data_frame(w = 1, x = 2, y = 3, z = 4)
  )
})

test_that("named data frames are not unpacked", {
  df_col <- data_frame(x = 2, y = 3)
  df <- data_frame(w = 1, col = data_frame(x = 2, y = 3), z = 4)

  expect_identical(df$col, df_col)
})

test_that("unpacked data frames without names are caught", {
  df_col <- new_data_frame(list(1))
  expect_error(data_frame(df_col), "corrupt data frame")
})

test_that("unpacking in `df_list()` can be disabled with `.unpack = FALSE`", {
  out <- df_list(
    w = 1,
    data_frame(x = 2, y = 3),
    z = 4,
    .unpack = FALSE,
    .name_repair = "minimal"
  )

  expect <- list(
    w = 1,
    data_frame(x = 2, y = 3),
    z = 4
  )

  expect_identical(out, expect)
})

test_that("`.unpack` is validated", {
  expect_snapshot(error = TRUE, {
    df_list(.unpack = 1)
  })
  expect_snapshot(error = TRUE, {
    df_list(.unpack = c(TRUE, FALSE))
  })
})

test_that("`NULL` inputs are dropped", {
  expect_identical(data_frame(NULL, x = 1, NULL), data_frame(x = 1))
})

test_that("`NULL` inputs are dropped before name repair", {
  expect_identical(
    data_frame(x = NULL, x = 1, .name_repair = "check_unique"),
    data_frame(x = 1)
  )
})

test_that("`.size` can force a desired size", {
  df <- data_frame(x = 1, .size = 5)
  expect_identical(df$x, rep(1, 5))

  expect_size(data_frame(.size = 5), 5L)
})

test_that("`.name_repair` repairs names", {
  expect_message(res <- data_frame(x = 1, x = 1, .name_repair = "unique"))
  expect_named(res, c("x...1", "x...2"))
})

test_that("`.name_repair` happens after auto-naming with empty strings", {
  expect_message(res <- data_frame(1, 2, .name_repair = "unique"))
  expect_named(res, c("...1", "...2"))
})

test_that("`.name_repair` happens after splicing", {
  expect_message(res <- data_frame(x = 1, data_frame(x = 2), .name_repair = "unique"))
  expect_named(res, c("x...1", "x...2"))
})

test_that("`.name_repair` can be quiet", {
  local_name_repair_verbose()

  expect_snapshot({
    dfl_unique <- df_list(1, 2, .name_repair = "unique_quiet")
    dfl_universal <- df_list("if" = 1, "in" = 2, .name_repair = "universal_quiet")
    df_unique <- data_frame(1, 2, .name_repair = "unique_quiet")
    df_universal <- data_frame("if" = 1, "in" = 2, .name_repair = "universal_quiet")
  })

  expect_named(dfl_unique, c("...1", "...2"))
  expect_named(dfl_universal, c(".if", ".in"))
  expect_named(df_unique, c("...1", "...2"))
  expect_named(df_universal, c(".if", ".in"))
})

# fallback ----------------------------------------------------------------

test_that("data frame fallback handles column types (#999)", {
  df1 <- foobar(data.frame(x = 1))
  df2 <- foobar(data.frame(x = 1, y = 2))
  df3 <- foobar(data.frame(x = "", y = 2))

  common <- foobar(data.frame(x = dbl(), y = dbl()))
  expect_identical(vec_ptype2(df1, df2), common)
  expect_identical(vec_ptype2(df2, df1), common)

  exp <- foobar(data_frame(x = 1, y = na_dbl))
  expect_identical(vec_cast(df1, df2), exp)
  expect_identical(vec_cast(set_tibble(df1), set_tibble(df2)), set_tibble(exp))

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(
      vec_ptype2(df1, df3),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_ptype2(df3, df1),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_cast(df2, df1),
      class = "vctrs_error_incompatible_type"
    ))
  })

  expect_identical(
    vec_rbind(df1, df2),
    foobar(data.frame(x = c(1, 1), y = c(NA, 2)))
  )

  # Attributes are not restored
  df1_attrib <- foobar(df1, foo = "foo")
  df2_attrib <- foobar(df2, bar = "bar")
  exp <- data.frame(x = c(1, 1), y = c(NA, 2))
  expect_equal(
    vec_rbind(df1_attrib, df2_attrib),
    exp
  )

  out <- with_methods(
    `[.vctrs_foobar` = function(x, i, ...) {
      new_data_frame(
        NextMethod(),
        dispatched = TRUE,
        class = "vctrs_foobar"
      )
    },
    vec_rbind(df1_attrib, df2_attrib)
  )
  expect_identical(out, exp)
})

test_that("falls back to tibble for tibble subclasses (#1025)", {
  foo <- foobar(tibble::as_tibble(mtcars))
  expect_s3_class(vec_rbind(foo, mtcars), "tbl_df")
  expect_s3_class(vec_rbind(foo, mtcars, mtcars), "tbl_df")
  expect_s3_class(vec_rbind(foo, mtcars, foobar(mtcars)), "tbl_df")
})

test_that("fallback is recursive", {
  df <- mtcars[1:3, 1, drop = FALSE]

  foo <- new_data_frame(list(x = foobar(df, foo = TRUE)))
  bar <- new_data_frame(list(x = foobar(df, bar = TRUE)))
  baz <- new_data_frame(list(y = 1:3, x = foobar(df, bar = TRUE)))

  exp <- new_data_frame(list(x = vec_rbind(df, df)))
  expect_equal(vec_rbind(foo, bar), exp)

  exp <- new_data_frame(list(x = vec_rbind(df, df), y = c(NA, NA, NA, 1:3)))
  expect_equal(vec_rbind(foo, baz), exp)
})

local_name_repair_quiet()

# rows --------------------------------------------------------------------

test_that("empty inputs return an empty data frame", {
  expect_equal(vec_rbind(), data_frame())
  expect_equal(vec_rbind(NULL, NULL), data_frame())
})

test_that("vec_rbind(): NULL is idempotent", {
  df <- data_frame(x = 1)
  expect_equal(vec_rbind(df, NULL), df)
})

test_that("vec_rbind() output is tibble if any input is tibble", {
  df <- data_frame(x = 1)
  dt <- tibble::tibble(x = 1)

  expect_s3_class(vec_rbind(dt), "tbl_df")
  expect_s3_class(vec_rbind(dt, df), "tbl_df")
  expect_s3_class(vec_rbind(df, dt), "tbl_df")
})

test_that("type of column is common type of individual columns", {
  x_int <- data_frame(x = 1L)
  x_dbl <- data_frame(x = 2.5)

  expect_equal(vec_rbind(x_int, x_int), data_frame(x = c(1L, 1L)))
  expect_equal(vec_rbind(x_int, x_dbl), data_frame(x = c(1, 2.5)))
})

test_that("incompatible columns throws common type error", {
  x_int <- data_frame(x = 1L)
  x_chr <- data_frame(x = "a")

  expect_snapshot({
    (expect_error(
      vec_rbind(x_int, x_chr),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_rbind(x_int, x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_rbind(x_int, x_chr, .ptype = x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("result contains union of columns", {
  expect_named(
    vec_rbind(
      data_frame(x = 1),
      data_frame(y = 1)
    ),
    c("x" , "y")
  )

  expect_named(
    vec_rbind(
      data_frame(y = 1, x = 1),
      data_frame(y = 1, z = 2)
    ),
    c("y", "x", "z")
  )
})

test_that("all inputs coerced to data frames", {
  expect_equal(
    vec_rbind(data_frame(x = 1L), c(x = 1.5)),
    data_frame(x = c(1, 1.5))
  )
})

test_that("names are supplied if needed", {
  local_name_repair_verbose()
  expect_snapshot(out <- vec_rbind(data_frame(...1 = 1), 1))
  expect_equal(out, data_frame(...1 = c(1, 1)))
})

test_that("matrix becomes data frame and has names properly repaired", {
  x <- matrix(1:4, nrow = 2)
  expect_equal(vec_rbind(x), data.frame(...1 = 1:2, ...2 = 3:4))
})

test_that("can bind data.frame columns", {
  df <- data.frame(x = NA, y = 1:2)
  df$x <- data.frame(a = 1:2)

  expected <- data.frame(x = NA, y = c(1:2, 1:2))
  expected$x <- data.frame(a = c(1:2, 1:2))

  expect_equal(vec_rbind(df, df), expected)
})

test_that("can rbind unspecified vectors", {
  expect_identical(vec_rbind(NA), data_frame(...1 = NA))
  expect_identical(vec_rbind(NA, NA), data_frame(...1 = lgl(NA, NA)))

  df <- data.frame(x = 1)
  expect_identical(vec_rbind(NA, df), data.frame(x = c(NA, 1)))
  expect_identical(vec_rbind(df, NA), data.frame(x = c(1, NA)))
  expect_identical(vec_rbind(NA, df, NA), data.frame(x = c(NA, 1, NA)))
  expect_identical(vec_rbind(c(x = NA), data.frame(x = 1)), data.frame(x = c(NA, 1)))
  expect_identical(vec_rbind(c(y = NA), df), data.frame(y = c(NA, NA), x = c(NA, 1)))

  out <- suppressMessages(vec_rbind(c(x = NA, x = NA), df))
  exp <- data.frame(x...1 = c(NA, NA), x...2 = c(NA, NA), x = c(NA, 1))
  expect_identical(out, exp)
})

test_that("as_df_row() tidies the names of unspecified vectors", {
  expect_identical(as_df_row(c(NA, NA)), c(NA, NA))
  expect_identical(as_df_row(unspecified(2)), unspecified(2))
  expect_identical(as_df_row(c(a = NA, a = NA), quiet = TRUE), data.frame(a...1 = NA, a...2 = NA))
  expect_identical(as_df_row(c(a = TRUE, a = TRUE), quiet = TRUE), data.frame(a...1 = TRUE, a...2 = TRUE))
})

test_that("can rbind spliced lists", {
  data <- list(c(a = 1, b = 2), c(a = TRUE, b = FALSE))
  expect_identical(vec_rbind(!!!data), data_frame(a = c(1, 1), b = c(2, 0)))
})

test_that("can rbind list columns", {
  out <- vec_rbind(data_frame(x = list(1, 2)), data_frame(x = list(3)))
  expect_identical(out, data_frame(x = list(1, 2, 3)))
})

test_that("can rbind lists", {
  out <- vec_rbind(list(x = 1:2))
  expect_identical(out, data_frame(x = list(c(1L, 2L))))

  out <- vec_rbind(list(x = 1:2, y = 3L))
  expect_identical(out, data_frame(x = list(c(1L, 2L)), y = list(3L)))

  out <- vec_rbind(list(x = 1, y = 2), list(y = "string"))
  expect_identical(out, data_frame(x = list(1, NULL), y = list(2, "string")))
})

test_that("can rbind factors", {
  fctr <- factor(c("a", "b"))
  expect_equal(vec_rbind(fctr), data_frame(...1 = fctr[1], ...2 = fctr[2]))

  fctr_named <- set_names(fctr)
  expect_equal(vec_rbind(fctr_named), data_frame(a = fctr[1], b = fctr[2]))
})

test_that("can rbind dates", {
  date <- new_date(c(0, 1))
  expect_equal(vec_rbind(date), data_frame(...1 = date[1], ...2 = date[2]))

  date_named <- set_names(date, c("a", "b"))
  expect_equal(vec_rbind(date_named), data_frame(a = date[1], b = date[2]))
})

test_that("can rbind POSIXlt objects into POSIXct objects", {
  datetime <- as.POSIXlt(new_datetime(0))
  expect_s3_class(vec_rbind(datetime, datetime)[[1]], "POSIXct")

  datetime_named <- set_names(datetime, "col")
  expect_named(vec_rbind(datetime_named, datetime_named), "col")
})

test_that("can rbind table objects (#913)", {
  x <- new_table(1:4, c(2L, 2L))
  y <- x

  colnames <- c("c1", "c2")
  rownames <- c("r1", "r2", "r3", "r4")

  dimnames(x) <- list(rownames[1:2], colnames)
  dimnames(y) <- list(rownames[3:4], colnames)

  expect <- data.frame(c1 = c(1:2, 1:2), c2 = c(3:4, 3:4), row.names = rownames)

  expect_identical(vec_rbind(x, y), expect)
})

test_that("can rbind missing vectors", {
  expect_identical(vec_rbind(c(x = na_int)), data_frame(x = na_int))
  expect_identical(vec_rbind(c(x = na_int), c(x = na_int)), data_frame(x = int(na_int, na_int)))
})

test_that("vec_rbind() respects size invariants (#286)", {
  expect_identical(vec_rbind(), new_data_frame(n = 0L))

  expect_identical(vec_rbind(int(), int()), new_data_frame(n = 2L))
  expect_identical(vec_rbind(c(x = int()), c(x = TRUE)), new_data_frame(list(x = lgl(NA, TRUE))))

  expect_identical(vec_rbind(int(), new_data_frame(n = 2L), int()), new_data_frame(n = 4L))
})

test_that("can repair names in `vec_rbind()` (#229)", {
  expect_snapshot({
    (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
    (expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`"))
    (expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
  })

  expect_named(vec_rbind(list(a = 1, a = 2), .name_repair = "unique"), c("a...1", "a...2"))

  expect_named(vec_rbind(list(`_` = 1)), "_")
  expect_named(vec_rbind(list(`_` = 1), .name_repair = "universal"), c("._"))

  expect_named(vec_rbind(list(a = 1, a = 2), .name_repair = ~ toupper(.)), c("A", "A"))
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_rbind(c(x = 1, x = 2), c(x = 3, x = 4), .name_repair = "unique_quiet")
    res_universal <- vec_rbind(c("if" = 1, "in" = 2), c("if" = 3, "for" = 4), .name_repair = "universal_quiet")
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in", ".for"))
})

test_that("can construct an id column", {
  df <- data.frame(x = 1)

  expect_named(vec_rbind(df, df, .names_to = "id"), c("id", "x"))
  expect_equal(vec_rbind(df, df, .names_to = "id")$id, c(1L, 2L))

  expect_equal(vec_rbind(a = df, b = df, .names_to = "id")$id, c("a", "b"))

  expect_equal(vec_rbind(a = df, df, .names_to = "id")$id, c("a", ""))
})

test_that("vec_rbind() fails with arrays of dimensionality > 3", {
  expect_snapshot({
    (expect_error(vec_rbind(array(NA, c(1, 1, 1)))))
    (expect_error(vec_rbind(array(NA, c(1, 1, 1)), .error_call = call("foo"))))
  })
})

test_that("row names are preserved by vec_rbind()", {
  df1 <- mtcars[1:3, ]
  df2 <- mtcars[4:5, ]
  expect_identical(vec_rbind(df1, df2), mtcars[1:5, ])

  row.names(df2) <- NULL
  out <- mtcars[1:5, ]
  row.names(out) <- c(row.names(df1), "...4", "...5")
  expect_identical(vec_rbind(df1, df2), out)
})

test_that("can assign row names in vec_rbind()", {
  df1 <- mtcars[1:3, ]
  df2 <- mtcars[4:5, ]

  expect_snapshot({
    (expect_error(
      vec_rbind(
        foo = df1,
        df2,
        .names_to = NULL
      ),
      "specification"
    ))
  })

  # Combination
  out <- vec_rbind(
    foo = df1,
    df2,
    .names_to = NULL,
    .name_spec = "{outer}_{inner}"
  )
  exp <- mtcars[1:5, ]
  row.names(exp) <- c(paste0("foo_", row.names(df1)), row.names(df2))
  expect_identical(out, exp)

  out <- vec_rbind(foo = df1, df2, .names_to = "id")
  exp <- mtcars[1:5, ]
  exp <- vec_cbind(id = c(rep("foo", 3), rep("", 2)), exp)
  expect_identical(out, exp)

  # Sequence
  out <- vec_rbind(
    foo = unrownames(df1),
    df2,
    bar = unrownames(mtcars[6, ]),
    .names_to = NULL,
    .name_spec = "{outer}_{inner}"
  )
  exp <- mtcars[1:6, ]
  row.names(exp) <- c(paste0("foo_", 1:3), row.names(df2), "bar")
  expect_identical(out, exp)

  out <- vec_rbind(
    foo = unrownames(df1),
    df2,
    bar = unrownames(mtcars[6, ]),
    .names_to = "id"
  )
  exp <- mtcars[1:6, ]
  exp <- vec_cbind(id = c(rep("foo", 3), rep("", 2), "bar"), exp)
  row.names(exp) <- c(paste0("...", 1:3), row.names(df2), "...6")
  expect_identical(out, exp)
})

test_that("vec_rbind() takes the proxy and restores", {
  df <- foobar(data.frame(x = 1))

  # This data frame subclass has an identity proxy and the restore
  # method falls back to a bare data frame if `$x` has any missing values.
  # In `vec_rbind()`, the `vec_init()` call will create a bare data frame,
  # but at the end it is `vec_restore()`d to the right class.
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
      x
    },
    vec_proxy.vctrs_foobar = function(x, ...) {
      x
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      if (any(is.na(x$x))) {
        new_data_frame(x)
      } else {
        vec_restore_default(x, to)
      }
    }
  )

  expect_identical(
    vec_rbind(df, df),
    foobar(data.frame(x = c(1, 1)))
  )
})

test_that("vec_rbind() proxies before initializing", {
  df <- foobar(data.frame(x = 1))

  # This data frame subclass doesn't allow `NA`s in columns.
  # If initialization happened before proxying, it would try to
  # create `NA` rows with `vec_init()`.
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
      x
    },
    vec_proxy.vctrs_foobar = function(x, ...) {
      new_data_frame(x)
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      if (any(is.na(x$x))) {
        abort("`x` can't have NA values.")
      }
      vec_restore_default(x, to)
    }
  )

  expect_identical(
    vec_rbind(df, df),
    foobar(data.frame(x = c(1, 1)))
  )
})

test_that("vec_rbind() requires a data frame proxy for data frame ptypes", {
  df <- foobar(data.frame(x = 1))

  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
    vec_proxy.vctrs_foobar = function(x, ...) 1
  )

  expect_error(
    vec_rbind(df, df),
    "Can't fill a data frame that doesn't have a data frame proxy"
  )
})

test_that("monitoring: name repair while rbinding doesn't modify in place", {
  df <- new_data_frame(list(x = 1, x = 1))
  expect <- new_data_frame(list(x = 1, x = 1))

  # Name repair occurs
  expect_named(vec_rbind(df), c("x...1", "x...2"))

  # No changes to `df`
  expect_identical(df, expect)
})

test_that("performance: Row binding with S3 columns doesn't duplicate on every assignment (#1151)", {
  skip_if_not_testing_performance()

  x <- as.Date("2000-01-01")
  x <- rep(x, 100)
  df <- data.frame(x = x)
  lst <- rep_len(list(df), 10000)

  expect_time_lt(vec_rbind(!!!lst), 5)
})

test_that("performance: Row binding with df-cols doesn't duplicate on every assignment (#1122)", {
  skip_if_not_testing_performance()

  df_col <- new_data_frame(list(x = 1:1000))
  df <- new_data_frame(list(y = df_col))

  lst <- rep_len(list(df), 10000)

  expect_time_lt(vec_rbind(!!!lst), 5)
})

# cols --------------------------------------------------------------------

test_that("vec_cbind() reports error context", {
  expect_snapshot({
    (expect_error(vec_cbind(foobar(list()))))
    (expect_error(vec_cbind(foobar(list()), .error_call = call("foo"))))

    (expect_error(vec_cbind(a = 1:2, b = int())))
    (expect_error(vec_cbind(a = 1:2, b = int(), .error_call = call("foo"))))
  })
})

test_that("empty inputs give data frame", {
  expect_equal(vec_cbind(), data_frame())
  expect_equal(vec_cbind(NULL), data_frame())
  expect_equal(vec_cbind(data.frame(a = 1), NULL), data_frame(a = 1))
})

test_that("number of rows is preserved with zero column data frames (#1281)", {
  df <- new_data_frame(n = 2L)
  expect_size(vec_cbind(df, df), 2L)
})

test_that("vec_cbind(): NULL is idempotent", {
  df <- data_frame(x = 1)
  expect_equal(vec_cbind(df, NULL), df)
})

test_that("outer names are respected", {
  expect_named(vec_cbind(x = 1, y = 4), c("x", "y"))
  expect_named(vec_cbind(a = data.frame(x = 1)), "a")
})

test_that("inner names are respected", {
  expect_named(vec_cbind(data_frame(x = 1), data_frame(y = 1)), c("x", "y"))
})

test_that("nameless vectors get tidy defaults", {
  expect_named(vec_cbind(1:2, 1), c("...1", "...2"))
})

test_that("matrix becomes data frame", {
  x <- matrix(1:4, nrow = 2)
  expect_equal(vec_cbind(x), data.frame(...1 = 1:2, ...2 = 3:4))

  # Packed if named
  expect_equal(vec_cbind(x = x), data_frame(x = x))
})

test_that("duplicate names are de-deduplicated", {
  local_name_repair_verbose()

  expect_snapshot({
    (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
    (expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1", "x...2")))
  })
})

test_that("rows recycled to longest", {
  df <- data.frame(x = 1:3)

  expect_dim(vec_cbind(df), c(3, 1))
  expect_dim(vec_cbind(df, NULL), c(3, 1))
  expect_dim(vec_cbind(df, y = 1), c(3, 2))
  expect_dim(vec_cbind(data.frame(x = 1), y = 1:3), c(3, 2))

  expect_dim(
    vec_cbind(
      data.frame(a = 1, b = 2),
      y = 1:3
    ),
    c(3, 3)
  )
})

test_that("vec_cbind() output is tibble if any input is tibble", {
  df <- data.frame(x = 1)
  dt <- tibble::tibble(y = 2)

  expect_s3_class(vec_cbind(dt), "tbl_df")
  expect_s3_class(vec_cbind(df, dt), "tbl_df")
  expect_s3_class(vec_cbind(dt, df), "tbl_df")
})

test_that("can override default .nrow", {
  expect_dim(vec_cbind(x = 1, .size = 3), c(3, 1))
})

test_that("can repair names in `vec_cbind()` (#227)", {
  expect_snapshot({
    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"), "can't be `\"none\"`"))
    (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
  })

  expect_named(vec_cbind(a = 1, a = 2, .name_repair = "unique"), c("a...1", "a...2"))

  expect_named(vec_cbind(`_` = 1, .name_repair = "universal"), "._")

  expect_named(vec_cbind(a = 1, a = 2, .name_repair = "minimal"), c("a", "a"))
  expect_named(vec_cbind(a = 1, a = 2, .name_repair = toupper), c("A", "A"))
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_cbind(x = 1, x = 2, .name_repair = "unique_quiet")
    res_universal <- vec_cbind("if" = 1, "in" = 2, .name_repair = "universal_quiet")
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in"))
})

test_that("can supply `.names_to` to `vec_rbind()` (#229)", {
  expect_snapshot({
    (expect_error(vec_rbind(.names_to = letters)))
    (expect_error(vec_rbind(.names_to = 10)))
    (expect_error(vec_rbind(.names_to = letters, .error_call = call("foo"))))
  })

  x <- data_frame(foo = 1:2, bar = 3:4)
  y <- data_frame(foo = 5L, bar = 6L)

  expect_identical(
    vec_rbind(a = x, b = y, .names_to = "quux"),
    data_frame(quux = c("a", "a", "b"), foo = c(1L, 2L, 5L), bar = c(3L, 4L, 6L))
  )
  expect_identical(
    vec_rbind(a = x, b = y, .names_to = "foo"),
    data_frame(foo = c("a", "a", "b"), bar = c(3L, 4L, 6L))
  )

  # No names
  expect_identical(
    vec_rbind(x, y, .names_to = "quux"),
    data_frame(quux = c(1L, 1L, 2L), foo = c(1L, 2L, 5L), bar = c(3L, 4L, 6L))
  )
  expect_identical(
    vec_rbind(x, y, .names_to = "foo"),
    data_frame(foo = c(1L, 1L, 2L), bar = c(3L, 4L, 6L))
  )

  # Partial names
  expect_identical(vec_rbind(x, b = y, .names_to = "quux")$quux, c("", "", "b"))
})

test_that("can supply existing `.names_to`", {
  x <- data.frame(a = 1, id = TRUE)
  expect_identical(
    vec_rbind(foo = x, bar = c(a = 2), .names_to = "id"),
    data_frame(a = c(1, 2), id = c("foo", "bar"))
  )

  y <- data.frame(id = TRUE, a = 1)
  expect_identical(
    vec_rbind(foo = y, bar = c(a = 2), .names_to = "id"),
    data_frame(id = c("foo", "bar"), a = c(1, 2))
  )
})

test_that("vec_cbind() returns visibly (#452)", {
  # Shouldn't be needed once `check_unique` is implemented in C
  expect_visible(vctrs::vec_cbind(x = 1, .name_repair = "check_unique"))
})

test_that("vec_cbind() packs named data frames (#446)", {
  expect_identical(vec_cbind(data_frame(y = 1:3)), data_frame(y = 1:3))
  expect_identical(vec_cbind(x = data_frame(y = 1:3)), data_frame(x = data_frame(y = 1:3)))
})

test_that("vec_cbind() packs 1d arrays", {
  a <- array(1:2)
  expect_identical(vec_cbind(a), data_frame(...1 = 1:2))
  expect_identical(vec_cbind(x = a), data_frame(x = a))
})

test_that("vec_cbind() packs named matrices", {
  m <- matrix(1:4, 2)
  expect_identical(vec_cbind(m), data_frame(...1 = 1:2, ...2 = 3:4))
  expect_identical(vec_cbind(x = m), data_frame(x = m))
})

test_that("vec_cbind() never packs named vectors", {
  expect_identical(vec_cbind(1:2), data_frame(...1 = 1:2))
  expect_identical(vec_cbind(x = 1:2), data_frame(x = 1:2))
})

test_that("names are repaired late if unpacked", {
  df <- data_frame(b = 2, b = 3, .name_repair = "minimal")
  out1 <- vec_cbind(a = 1, df)
  out2 <- vec_cbind(a = 1, as.matrix(df))
  out3 <- vec_cbind(a = 1, matrix(1:2, nrow = 1))
  expect_named(out1, c("a", "b...2", "b...3"))
  expect_named(out2, c("a", "b...2", "b...3"))
  expect_named(out3, c("a", "...2", "...3"))
})

test_that("names are not repaired if packed", {
  df <- data_frame(b = 2, b = 3, .name_repair = "minimal")
  out1 <- vec_cbind(a = 1, packed = df)
  out2 <- vec_cbind(a = 1, packed = as.matrix(df))
  out3 <- vec_cbind(a = 1, packed = matrix(1:2, nrow = 1))

  expect_named(out1, c("a", "packed"))
  expect_named(out2, c("a", "packed"))
  expect_named(out3, c("a", "packed"))

  expect_named(out1$packed, c("b", "b"))
  expect_identical(colnames(out2$packed), c("b", "b"))
  expect_identical(colnames(out3$packed), NULL)
})

test_that("vec_cbind() fails with arrays of dimensionality > 3", {
  a <- array(NA, c(1, 1, 1))

  expect_snapshot({
    (expect_error(vec_cbind(a)))
    (expect_error(vec_cbind(a, .error_call = call("foo"))))
    (expect_error(vec_cbind(x = a)))
  })
})

test_that("monitoring: name repair while cbinding doesn't modify in place", {
  df <- new_data_frame(list(x = 1, x = 1))
  expect <- new_data_frame(list(x = 1, x = 1))

  # Name repair occurs
  expect_named(vec_cbind(df), c("x...1", "x...2"))

  # No changes to `df`
  expect_identical(df, expect)
})

test_that("vec_rbind() consistently handles unnamed outputs", {
  # Name repair of columns is a little weird but unclear we can do better
  expect_identical(
    vec_rbind(1, 2, .names_to = NULL),
    data.frame(...1 = c(1, 2))
  )
  expect_identical(
    vec_rbind(1, 2, ...10 = 3, .names_to = NULL),
    data.frame(...1 = c(1, 2, 3), row.names = c("...1", "...2", "...3"))
  )

  expect_identical(
    vec_rbind(a = 1, b = 2, .names_to = NULL),
    data.frame(...1 = c(1, 2), row.names = c("a", "b"))
  )
  expect_identical(
    vec_rbind(c(a = 1), c(b = 2), .names_to = NULL),
    data.frame(a = c(1, NA), b = c(NA, 2))
  )
})

test_that("vec_rbind() ignores named inputs by default (#966)", {
  expect_identical(
    vec_rbind(foo = c(a = 1)),
    data.frame(a = 1)
  )
  expect_identical(
    vec_rbind(foo = c(a = 1), .names_to = NULL),
    data.frame(a = 1, row.names = "foo")
  )
})

test_that("vec_cbind() consistently handles unnamed outputs", {
  expect_identical(
    vec_cbind(1, 2),
    data.frame(...1 = 1, ...2 = 2)
  )
  expect_identical(
    vec_cbind(1, 2, ...10 = 3),
    data.frame(...1 = 1, ...2 = 2, ...3 = 3)
  )
  expect_identical(
    vec_cbind(a = 1, b = 2),
    data.frame(a = 1, b = 2)
  )
  expect_identical(
    vec_cbind(c(a = 1), c(b = 2)),
    new_data_frame(list(...1 = c(a = 1), ...2 = c(b = 2)))
  )
})

test_that("vec_rbind() name repair messages are useful", {
  local_name_repair_verbose()

  expect_snapshot({
    vec_rbind(1, 2)
    vec_rbind(1, 2, .names_to = NULL)

    vec_rbind(1, 2, ...10 = 3)
    vec_rbind(1, 2, ...10 = 3, .names_to = NULL)

    vec_rbind(a = 1, b = 2)
    vec_rbind(a = 1, b = 2, .names_to = NULL)

    vec_rbind(c(a = 1), c(b = 2))
    vec_rbind(c(a = 1), c(b = 2), .names_to = NULL)
  })
})

test_that("vec_rbind() is silent when assigning duplicate row names of df-cols", {
  df <- new_data_frame(list(x = mtcars[1:3, 1, drop = FALSE]))

  expect_snapshot(vec_rbind(df, df))
  expect_snapshot(vec_rbind(mtcars[1:4, ], mtcars[1:3, ]))
})

test_that("vec_cbind() name repair messages are useful", {
  local_name_repair_verbose()

  expect_snapshot({
    vec_cbind(1, 2)
    vec_cbind(1, 2, ...10 = 3)
    vec_cbind(a = 1, b = 2)
    vec_cbind(c(a = 1), c(b = 2))
  })
})

test_that("cbind() deals with row names", {
  expect_identical(
    vec_cbind(mtcars[1:3], foo = 1),
    cbind(mtcars[1:3], foo = 1)
  )
  expect_identical(
    vec_cbind(mtcars[1:3], mtcars[4]),
    cbind(mtcars[1:3], mtcars[4])
  )

  out <- vec_cbind(
    mtcars[1, 1, drop = FALSE],
    unrownames(mtcars[1:3, 2, drop = FALSE])
  )
  exp <- mtcars[1:3, c(1, 2)]
  exp[[1]] <- exp[[1, 1]]
  row.names(exp) <- paste0(c("Mazda RX4..."), 1:3)
  expect_identical(out, exp)
})

test_that("prefer row names of first named input (#1058)", {
  df0 <- unrownames(mtcars[1:5, 1:3])
  df1 <- mtcars[1:5, 4:6]
  df2 <- mtcars[5:1, 7:9]

  expect_identical(
    row.names(vec_cbind(df0, df1, df2)),
    row.names(df1)
  )
  expect_identical(
    row.names(vec_cbind(df0, df2, df1)),
    row.names(df2)
  )
})

test_that("can rbind data frames with matrix columns (#625)", {
  df <- tibble(x = 1:2, y = matrix(1:4, nrow = 2))
  expect_identical(vec_rbind(df, df), vec_slice(df, c(1, 2, 1, 2)))
})

test_that("rbind repairs names of data frames (#704)", {
  df <- data_frame(x = 1, x = 2, .name_repair = "minimal")
  df_repaired <- data_frame(x...1 = 1, x...2 = 2)
  expect_identical(vec_rbind(df), df_repaired)
  expect_identical(vec_rbind(df, df), vec_rbind(df_repaired, df_repaired))

  expect_snapshot({
    (expect_error(
      vec_rbind(df, df, .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
    (expect_error(
      vec_rbind(df, df, .name_repair = "check_unique", .error_call = call("foo")),
      class = "vctrs_error_names_must_be_unique"
    ))
  })
})

test_that("vec_rbind() works with simple homogeneous foreign S3 classes", {
  expect_identical(
    vec_rbind(set_names(foobar(1), "x"), set_names(foobar(2), "x")),
    data_frame(x = foobar(c(1, 2)))
  )
})

test_that("vec_rbind() works with simple homogeneous foreign S4 classes", {
  skip_if_cant_set_names_on_s4()

  joe1 <- .Counts(1L, name = "Joe")
  joe2 <- .Counts(2L, name = "Joe")

  expect_identical(
    vec_rbind(set_names(joe1, "x"), set_names(joe2, "x")),
    data_frame(x = .Counts(1:2, name = "Joe"))
  )
})

test_that("vec_rbind() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")

    (expect_error(
      vec_rbind(set_names(x, "x"), set_names(y, "x")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_rbind() fails with complex foreign S4 classes", {
  skip_if_cant_set_names_on_s4()

  expect_snapshot({
    joe <- .Counts(1L, name = "Joe")
    jane <- .Counts(2L, name = "Jane")
    (expect_error(vec_rbind(set_names(joe, "x"), set_names(jane, "y")), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_rbind() falls back to c() if S3 method is available", {
  x <- foobar(1, foo = 1)
  y <- foobar(2, bar = 2)

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  expect_error(vec_rbind(x_df, y_df), class = "vctrs_error_incompatible_type")
  expect_error(vec_c(x_df, y_df), class = "vctrs_error_incompatible_type")
  expect_error(list_unchop(list(x_df, y_df), indices = list(1, 2)), class = "vctrs_error_incompatible_type")

  with_c_method <- function(expr) {
    with_methods(
      c.vctrs_foobar = function(...) quux(NextMethod()),
      expr
    )
  }

  out <- with_c_method(vec_rbind(x_df, y_df))
  exp <- data_frame(x = quux(c(1, 2)))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(x_df, y_df)), exp)
  expect_identical(
    with_c_method(list_unchop(list(x_df, y_df), indices = list(1, 2))),
    exp
  )

  # Fallback is used with data frame subclasses, with or without
  # ptype2 method
  foo_df <- foobaz(x_df)
  bar_df <- foobaz(y_df)

  out <- with_c_method(vec_rbind(foo_df, bar_df))
  exp <- foobaz(data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(foo_df, bar_df)), exp)
  expect_identical(
    with_c_method(list_unchop(list(foo_df, bar_df), indices = list(1, 2))),
    exp
  )

  with_hybrid_methods <- function(expr, cast = TRUE) {
    methods <- list(
      c.vctrs_foobar = function(...) quux(NextMethod()),
      vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_ptype2(...)),
      vec_cast.vctrs_foobaz.vctrs_foobaz = if (cast) function(...) foobaz(df_cast(...))
    )
    with_methods(expr, !!!compact(methods))
  }

  expect_equal(
    with_hybrid_methods(
      cast = FALSE,
      vec_rbind(foo_df, bar_df)
    ),
    foobaz(data_frame(x = quux(c(1, 2))))
  )

  # Falls back to data frame because there is no ptype2/cast methods
  out <- with_hybrid_methods(vec_rbind(foo_df, bar_df))
  exp <- foobaz(data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_hybrid_methods(vec_c(foo_df, bar_df)), exp)
  expect_identical(
    with_hybrid_methods(list_unchop(list(foo_df, bar_df), indices = list(1, 2))),
    exp
  )

  wrapper_x_df <- data_frame(x = x_df)
  wrapper_y_df <- data_frame(x = y_df)

  out <- with_c_method(vec_rbind(wrapper_x_df, wrapper_y_df))
  exp <- data_frame(x = data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(wrapper_x_df, wrapper_y_df)), exp)
  expect_identical(
    with_c_method(list_unchop(list(wrapper_x_df, wrapper_y_df), indices = list(1, 2))),
    exp
  )
})

test_that("c() fallback works with unspecified columns", {
  local_methods(
    c.vctrs_foobar = function(...) foobar(NextMethod()),
    `[.vctrs_foobar` = function(x, i, ...) foobar(NextMethod(), dispatched = TRUE)
  )

  out <- vec_rbind(
    data_frame(x = foobar(1)),
    data_frame(y = foobar(2))
  )
  expect_identical(out, data_frame(
    x = foobar(c(1, NA), dispatched = TRUE),
    y = foobar(c(NA, 2), dispatched = TRUE)
  ))
})

test_that("c() fallback works with vctrs-powered data frame subclass", {
  local_methods(
    c.vctrs_quux = function(...) quux(vec_paste0(NextMethod(), "-c")),
    `[.vctrs_quux` = function(x, i, ...) quux(vec_paste0(NextMethod(), "-["))
  )
  local_foobar_df_methods()

  ### Joint case
  df1 <- foobar(data_frame(x = quux(1:3)))
  df2 <- data_frame(x = quux(4:5))

  out <- vctrs::vec_rbind(df1, df2)
  exp <- foobar(data_frame(x = quux(paste0(1:5, "-c"))))
  expect_identical(out, exp)

  out <- vctrs::vec_rbind(df2, df1)
  exp <- foobar(data_frame(x = quux(paste0(c(4:5, 1:3), "-c"))))
  expect_identical(out, exp)

  ### Disjoint case
  df1 <- foobar(data_frame(x = quux(1:3)))
  df2 <- data.frame(y = 4:5)

  out <- vctrs::vec_rbind(df1, df2)
  exp <- foobar(data_frame(
    x = quux(c(paste0(1:3, "-c-["), paste0(c(NA, NA), "-["))),
    y = c(rep(NA, 3), 4:5)
  ))
  expect_identical(out, exp)

  out <- vctrs::vec_rbind(df2, df1)
  exp <- foobar(data_frame(
    y = c(4:5, rep(NA, 3)),
    x = quux(c(paste0(c(NA, NA), "-["), paste0(1:3, "-c-[")))
  ))
  expect_identical(out, exp)
})

test_that("vec_rbind() falls back to c() if S3 method is available for S4 class", {
  joe <- data_frame(x = .Counts(c(1L, 2L), name = "Joe"))
  jane <- data_frame(x = .Counts(3L, name = "Jane"))

  expect_error(vec_rbind(joe, jane), class = "vctrs_error_incompatible_type")

  out <- with_methods(
    c.vctrs_Counts = function(...) .Counts(NextMethod(), name = "dispatched"),
    vec_rbind(joe, jane)
  )
  expect_identical(out$x, .Counts(1:3, name = "dispatched"))
})

test_that("rbind supports names and inner names (#689)", {
  out <- vec_rbind(
    data_frame(x = list(a = 1, b = 2)),
    data_frame(x = list(3)),
    data_frame(x = list(d = 4))
  )
  expect_identical(out$x, list(a = 1, b = 2, 3, d = 4))

  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  oo_x <- set_names(as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")), letters[1:3])
  oo_y <- c(FOO = as.POSIXlt(c("2020-01-04")))
  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "d")
  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("d"))
  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x, oo = oo_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y, oo = oo_y),
    row.names = c("quux")
  )

  nested_out <- vec_rbind(nested_x, nested_y)
  expect_identical(row.names(nested_out), c("foo", "bar", "baz", "quux"))
  expect_identical(row.names(nested_out$df), c("a", "b", "c", "d"))
  expect_identical(row.names(nested_out$mat), c("a", "b", "c", "d"))
  expect_identical(names(nested_out$vec), c("a", "b", "c", "FOO"))
  expect_identical(names(nested_out$oo), c("a", "b", "c", "FOO"))
})

test_that("vec_rbind() doesn't fall back to c() with proxied classes (#1119)", {
  foobar_rcrd <- function(x, y) new_rcrd(list(x = x, y = y), class = "vctrs_foobar")

  x <- foobar_rcrd(x = 1:2, y = 3:4)

  out <- vec_rbind(x, x)
  exp <- data_frame(
    ...1 = foobar_rcrd(x = c(1L, 1L), y = c(3L, 3L)),
    ...2 = foobar_rcrd(x = c(2L, 2L), y = c(4L, 4L))
  )
  expect_identical(out, exp)

  out <- vec_rbind(data_frame(x = x), data_frame(x = x))
  exp <- data_frame(
    x = foobar_rcrd(x = c(1L, 2L, 1L, 2L), y = c(3L, 4L, 3L, 4L))
  )
  expect_identical(out, exp)
})

test_that("vec_rbind() fallback works with tibbles", {
  x <- foobar("foo")
  df <- data_frame(x = x)
  tib <- tibble(x = x)

  local_methods(c.vctrs_foobar = function(...) quux(NextMethod()))

  exp <- tibble(x = quux(c("foo", "foo")))

  expect_identical(vec_rbind(tib, tib), exp)
  expect_identical(vec_rbind(df, tib), exp)
  expect_identical(vec_rbind(tib, df), exp)
})

test_that("vec_rbind() zaps names when name-spec is zap() and names-to is NULL", {
  expect_identical(
    vec_rbind(foo = c(x = 1), .names_to = NULL, .name_spec = zap()),
    data.frame(x = 1)
  )
})

test_that("can't zap names when `.names_to` is supplied", {
  expect_identical(
    vec_rbind(foo = c(x = 1), .names_to = zap(), .name_spec = zap()),
    data.frame(x = 1)
  )

  expect_snapshot({
    (expect_error(
      vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap())
    ))
    (expect_error(
      vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap(), .error_call = call("foo"))
    ))
  })
})

test_that("can zap outer names from a name-spec (#1215)", {
  zap_outer_spec <- function(outer, inner) if (is_character(inner)) inner

  df <- data.frame(x = 1:2)
  df_named <- data.frame(x = 3L, row.names = "foo")

  expect_null(
    vec_names(vec_rbind(a = df, .names_to = NULL, .name_spec = zap_outer_spec))
  )
  expect_identical(
    vec_names(vec_rbind(a = df, df_named, .name_spec = zap_outer_spec)),
    c("...1", "...2", "foo")
  )
})

test_that("column names are treated consistently in vec_rbind()", {
  exp <- data.frame(a = c(1L, 1L), b = c(2L, 2L))

  x <- c(a = 1L, b = 2L)
  expect_identical(vec_rbind(x, x), exp)

  x <- array(1:2, dimnames = list(c("a", "b")))
  expect_identical(vec_rbind(x, x), exp)

  x <- matrix(1:2, nrow = 1, dimnames = list(NULL, c("a", "b")))
  expect_identical(vec_rbind(x, x), exp)

  x <- array(1:6, c(1, 2, 1), dimnames = list(NULL, c("a", "b"), NULL))
  expect_error(vec_rbind(x, x), "Can't bind arrays")
})

test_that("can repair names of row-binded vectors (#1567)", {
  local_name_repair_verbose()
  expect_silent(
    expect_named(
      vec_rbind(
        x = 1:3,
        y = 4:6,
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )
  )
})

test_that("can repair names of row-binded matrices", {
  local_name_repair_verbose()
  expect_silent({
    expect_named(
      vec_rbind(
        x = matrix(1:3, 1),
        y = matrix(4:6, 1),
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )

    expect_named(
      vec_rbind(
        x = matrix(1:3, 1),
        y = 4:6,
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )
  })
})

test_that("vec_rbind() only restores one time", {
  restored <- list()

  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) {
      # Ignore proxying and restoration of ptypes
      if (length(x)) {
        restored <<- c(restored, list(x))
      }
      foobar(x)
    }
  )

  df <- data_frame(x = foobar(1:3))
  vec_rbind(df, df)

  expect_equal(restored, list(
    rep(na_int, 6),       # From `vec_init()`
    foobar(c(1:3, 1:3))   # Final restoration
  ))
})

test_that("vec_rbind() applies `base::c()` fallback to df-cols (#1462, #1640)", {
  x <- structure(1, class = "myclass")
  df <- tibble(a = tibble(x = x))
  df <- vec_rbind(df, df)

  expect_equal(df$a$x, structure(c(1, 1), class = "myclass"))
})


# Golden tests -------------------------------------------------------

test_that("row-binding performs expected allocations", {
  vec_rbind_list <- function(x) {
    vec_rbind(!!!x)
  }

  expect_snapshot({
    ints <- rep(list(1L), 1e2)
    named_ints <- rep(list(set_names(1:3, letters[1:3])), 1e2)

    "Integers as rows"
    suppressMessages(with_memory_prof(vec_rbind_list(ints)))
    suppressMessages(with_memory_prof(vec_rbind_list(named_ints)))

    "Data frame with named columns"
    df <- data_frame(
      x = set_names(as.list(1:2), c("a", "b")),
      y = set_names(1:2, c("A", "B")),
      z = data_frame(Z = set_names(1:2, c("Za", "Zb")))
    )
    dfs <- rep(list(df), 1e2)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (non-repaired, non-recursive case)"
    df <- data_frame(x = 1:2)
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (repaired, non-recursive case)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (non-repaired, recursive case) (#1217)"
    df <- data_frame(
      x = 1:2,
      y = data_frame(x = 1:2)
    )
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (repaired, recursive case) (#1217)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))
  })
})

test_that("`.names_to` is assigned after restoration (#1648)", {
  df <- data_frame(x = factor("foo"))
  expect_equal(
    vec_rbind(name = df, .names_to = "x"),
    data_frame(x = "name")
  )

  # This used to fail with:
  #> Error in `vctrs::vec_rbind()`:
  #> ! adding class "factor" to an invalid object
})

test_that("vec_as_subscript() coerces unspecified vectors", {
  expect_identical(
    vec_as_subscript(NA),
    NA
  )
  expect_identical(
    vec_as_subscript(NA, logical = "error"),
    na_int
  )
  expect_identical(
    vec_as_subscript(NA, logical = "error", numeric = "error"),
    na_chr
  )
})

test_that("vec_as_subscript() coerces subtypes and supertypes", {
  expect_identical(vec_as_subscript(factor("foo")), "foo")

  with_lgl_subtype({
    expect_identical(vec_as_subscript(new_lgl_subtype(TRUE)), TRUE)
  })
  with_lgl_supertype({
    expect_identical(vec_as_subscript(new_lgl_supertype(TRUE)), TRUE)
  })
})

test_that("vec_as_subscript() handles NULL", {
  expect_identical(vec_as_subscript(NULL), int())
  expect_error(
    vec_as_subscript(NULL, numeric = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("vec_as_subscript() handles symbols", {
  expect_identical(vec_as_subscript(quote(foo)), "foo")
  expect_identical(vec_as_subscript(quote(`<U+5E78>`)), "\u5e78")
  expect_error(
    vec_as_subscript(quote(foo), character = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("can customise subscript errors", {
  expect_snapshot({
    (expect_error(
      with_tibble_cols(vec_as_subscript(env())),
      class = "vctrs_error_subscript_type"
    ))
  })

  expect_snapshot({
    (expect_error(
      with_dm_tables(vec_as_subscript(env())),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_subscript() checks dimensionality", {
  expect_snapshot({
    (expect_error(vec_as_subscript(matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_subscript(array(TRUE, dim = c(1, 1, 1))), class = "vctrs_error_subscript_type"))
    (expect_error(with_tibble_rows(vec_as_subscript(matrix(TRUE, nrow = 1))), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_subscript() works with vectors of dimensionality 1", {
  arr <- array(TRUE, dim = 1)
  expect_identical(vec_as_subscript(arr), arr)
})

test_that("vec_as_subscript() forbids subscript types", {
  expect_snapshot(error = TRUE, vec_as_subscript(1L, logical = "error", numeric = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript("foo", logical = "error", character = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript(TRUE, logical = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript("foo", character = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript(NULL, numeric = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript(quote(foo), character = "error"))
})

test_that("vec_as_subscript2() forbids subscript types", {
  expect_snapshot(error = TRUE, vec_as_subscript2(1L, numeric = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript2("foo", character = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript2(TRUE))
})

test_that("vec_as_subscript2() retains the call when throwing vec_as_subscript() errors (#1605)", {
  expect_snapshot(error = TRUE, vec_as_subscript2(1L, numeric = "error", call = call("foo")))
  expect_snapshot(error = TRUE, vec_as_subscript2(1.5, call = call("foo")))
})

test_that("vec_as_subscript2() retains the call when erroring on logical input (#1605)", {
  expect_snapshot(error = TRUE, vec_as_subscript2(TRUE, call = call("foo")))
})

test_that("vec_as_subscript() evaluates arg lazily", {
  expect_silent(vec_as_subscript(1L, arg = print("oof")))
  expect_silent(vec_as_subscript_result(1L, arg = print("oof"), NULL, logical = "error", numeric = "cast", character = "error"))
})

test_that("vec_as_subscript2() evaluates arg lazily", {
  expect_silent(vec_as_subscript2(1L, arg = print("oof")))
  expect_silent(vec_as_subscript2_result(1L, arg = print("oof"), NULL, numeric = "cast", character = "error"))
})

test_that("`logical = 'cast'` is deprecated", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(TRUE, logical = "cast")
  )

  # `logical = "error"` still works
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(TRUE, logical = "error")
  )
})

test_that("lossy cast errors for scalar subscripts work (#1606)", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(1.5)
  )
})

test_that("slice-assign throws error with non-vector inputs", {
  x <- environment()
  expect_error(vec_slice(x, 1L) <- 1L, class = "vctrs_error_scalar_type")
})

test_that("slice-assign throws error with non-vector `value`", {
  x <- 1L
  expect_error(vec_slice(x, 1L) <- NULL, class = "vctrs_error_scalar_type")
  expect_error(vec_slice(x, 1L) <- environment(), class = "vctrs_error_scalar_type")
})

test_that("can slice-assign NULL", {
  x <- NULL
  vec_slice(x, 1L) <- 1
  expect_identical(x, NULL)
})

test_that("can slice-assign base vectors", {
  x <- rep(FALSE, 3)
  vec_slice(x, 2) <- TRUE
  expect_identical(x, lgl(FALSE, TRUE, FALSE))

  x <- rep(0L, 3)
  vec_slice(x, 2) <- 1L
  expect_identical(x, int(0L, 1L, 0L))

  x <- rep(0., 3)
  vec_slice(x, 2) <- 1
  expect_identical(x, dbl(0, 1, 0))

  x <- rep(0i, 3)
  vec_slice(x, 2) <- 1i
  expect_identical(x, cpl(0i, 1i, 0i))

  x <- rep("", 3)
  vec_slice(x, 2) <- "foo"
  expect_identical(x, chr("", "foo", ""))

  x <- as.raw(rep(0, 3))
  vec_slice(x, 2) <- as.raw(1)
  expect_identical(x, as.raw(c(0, 1, 0)))
})

test_that("can assign base vectors", {
  x <- rep(FALSE, 3)
  expect_identical(vec_assign(x, 2, TRUE), lgl(FALSE, TRUE, FALSE))
  expect_identical(x, rep(FALSE, 3))

  x <- rep(0L, 3)
  expect_identical(vec_assign(x, 2, 1L), int(0L, 1L, 0L))
  expect_identical(x, rep(0L, 3))

  x <- rep(0., 3)
  expect_identical(vec_assign(x, 2, 1), dbl(0, 1, 0))
  expect_identical(x, rep(0., 3))

  x <- rep(0i, 3)
  expect_identical(vec_assign(x, 2, 1i), cpl(0i, 1i, 0i))
  expect_identical(x, rep(0i, 3))

  x <- rep("", 3)
  expect_identical(vec_assign(x, 2, "foo"), chr("", "foo", ""))
  expect_identical(x, rep("", 3))

  x <- as.raw(rep(0, 3))
  expect_identical(vec_assign(x, 2, as.raw(1)), as.raw(c(0, 1, 0)))
  expect_identical(x, as.raw(rep(0, 3)))
})

test_that("can assign shaped base vectors", {
  mat <- as.matrix

  x <- mat(rep(FALSE, 3))
  expect_identical(vec_assign(x, 2, TRUE), mat(lgl(FALSE, TRUE, FALSE)))
  expect_identical(x, mat(rep(FALSE, 3)))

  x <- mat(rep(0L, 3))
  expect_identical(vec_assign(x, 2, 1L), mat(int(0L, 1L, 0L)))
  expect_identical(x, mat(rep(0L, 3)))

  x <- mat(rep(0, 3))
  expect_identical(vec_assign(x, 2, 1), mat(dbl(0, 1, 0)))
  expect_identical(x, mat(rep(0, 3)))

  x <- mat(rep(0i, 3))
  expect_identical(vec_assign(x, 2, 1i), mat(cpl(0i, 1i, 0i)))
  expect_identical(x, mat(rep(0i, 3)))

  x <- mat(rep("", 3))
  expect_identical(vec_assign(x, 2, "foo"), mat(chr("", "foo", "")))
  expect_identical(x, mat(rep("", 3)))

  x <- mat(as.raw(rep(0, 3)))
  expect_identical(vec_assign(x, 2, as.raw(1)), mat(as.raw(c(0, 1, 0))))
  expect_identical(x, mat(as.raw(rep(0, 3))))
})

test_that("can slice-assign lists", {
  x <- rep(list(NULL), 3)
  vec_slice(x, 2) <- list(NA)
  expect_identical(x, list(NULL, NA, NULL))
})

test_that("can slice-assign shaped lists", {
  mat <- as.matrix
  x <- mat(rep(list(NULL), 3))
  vec_slice(x, 2) <- list(NA)
  expect_identical(x, mat(list(NULL, NA, NULL)))
})

test_that("can assign lists", {
  x <- rep(list(NULL), 3)
  expect_identical(vec_assign(x, 2, list(NA)), list(NULL, NA, NULL))
  expect_identical(x, rep(list(NULL), 3))
})

test_that("can assign shaped lists", {
  mat <- as.matrix
  x <- mat(rep(list(NULL), 3))
  expect_identical(vec_assign(x, 2, list(NA)), mat(list(NULL, NA, NULL)))
  expect_identical(x, mat(rep(list(NULL), 3)))
})

test_that("can assign object of any dimensionality", {
  x1 <- ones(2)
  x2 <- ones(2, 3)
  x3 <- ones(2, 3, 4)
  x4 <- ones(2, 3, 4, 5)

  expect_identical(vec_assign(x1, 1L, 2L), array(rep(c(2, 1), 1),  dim = 2))
  expect_identical(vec_assign(x2, 1L, 2L), array(rep(c(2, 1), 3),  dim = c(2, 3)))
  expect_identical(vec_assign(x3, 1L, 2L), array(rep(c(2, 1), 12), dim = c(2, 3, 4)))
  expect_identical(vec_assign(x4, 1L, 2L), array(rep(c(2, 1), 60), dim = c(2, 3, 4, 5)))
})

test_that("atomics can't be assigned in lists", {
  x <- list(NULL)
  expect_error(vec_slice(x, 1) <- 1, class = "vctrs_error_incompatible_type")
  expect_error(vec_assign(x, 1, 2), class = "vctrs_error_incompatible_type")

  expect_error(vec_slice(x, 1) <- "foo", class = "vctrs_error_incompatible_type")
  expect_error(vec_assign(x, 1, "foo"), class = "vctrs_error_incompatible_type")
})

test_that("Unspecified `NA` vector can be assigned into lists", {
  x <- list(1, 2)
  vec_slice(x, 1) <- NA
  expect_identical(x, list(NULL, 2))
})

test_that("monitoring test - unspecified() can be assigned in lists", {
  x <- list(1, 2)
  expect_error(vec_slice(x, 1) <- unspecified(1), NA)
  expect_equal(x, list(NULL, 2))
})

test_that("can assign and slice-assign data frames", {
  df <- data.frame(x = 1:2)
  df$y <- data.frame(a = 2:1)

  orig <- duplicate(df, shallow = FALSE)

  other <- data.frame(x = 3)
  other$y <- data.frame(a = 3)

  exp <- data.frame(x = int(3, 2))
  exp$y <- data.frame(a = int(3, 1))

  expect_identical(vec_assign(df, 1, other), exp)
  expect_identical(df, orig)

  vec_slice(df, 1) <- other
  expect_identical(df, exp)
})

test_that("can slice-assign using logical index", {
  x <- c(2, 1)
  vec_slice(x, TRUE) <- 3
  expect_equal(x, c(3, 3))

  vec_slice(x, c(TRUE, FALSE)) <- 4
  expect_equal(x, c(4, 3))

  expect_error(
    vec_assign(x, c(TRUE, FALSE, TRUE), 5),
    class = "vctrs_error_subscript_size"
  )
  expect_error(
    vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]),
    class = "vctrs_error_subscript_size"
  )
})

test_that("slice-assign ignores NA in logical subsetting", {
  x <- c(NA, 1, 2)
  expect_equal(`vec_slice<-`(x, x > 0, 1), c(NA, 1, 1))
  expect_equal(`vec_slice<-`(x, x > 0, c(NA, 2:1)), c(NA, 2, 1))
})

test_that("slice-assign with arrays ignores NA in logical subsetting", {
  mat <- as.matrix
  x <- c(NA, 1, 2)
  expect_equal(`vec_slice<-`(mat(x), x > 0, 1), mat(c(NA, 1, 1)))
  expect_equal(`vec_slice<-`(mat(x), x > 0, c(NA, 2:1)), mat(c(NA, 2, 1)))
})

test_that("slice-assign ignores NA in integer subsetting", {
  x <- 0:2
  expect_equal(`vec_slice<-`(x, c(NA, 2:3), 1), c(0, 1, 1))
  expect_equal(`vec_slice<-`(x, c(NA, 2:3), c(NA, 2:1)), c(0, 2, 1))
})

test_that("slice-assign with arrays ignores NA in integer subsetting", {
  mat <- as.matrix
  x <- mat(0:2)
  expect_equal(`vec_slice<-`(x, c(NA, 2:3), 1), mat(c(0, 1, 1)))
  expect_equal(`vec_slice<-`(x, c(NA, 2:3), c(NA, 2:1)), mat(c(0, 2, 1)))
})

test_that("can't modify subset with missing argument", {
  x <- 1:3
  expect_error(vec_slice(x, ) <- 2L)
})

test_that("can modify subset with recycled NA argument", {
  x <- 1:3
  vec_slice(x, NA) <- 2L
  expect_identical(x, 1:3)
})

test_that("can modify subset with recycled TRUE argument", {
  x <- 1:3
  vec_slice(x, TRUE) <- 2L
  expect_identical(x, rep(2L, 3))
})

test_that("can modify subset with recycled FALSE argument", {
  x <- 1:3
  vec_slice(x, FALSE) <- 2L
  expect_identical(x, 1:3)
})

test_that("can modify subset with NULL argument", {
  x <- 1:3
  vec_slice(x, NULL) <- 2L

  expect_identical(x, 1:3)
})

test_that("can slice-assign with missing indices", {
  x <- 1:3
  y <- 4:6
  test <- c(NA, TRUE, FALSE)
  vec_slice(x, test) <- vec_slice(y, test)
  expect_identical(x, int(1, 5, 3))
})

test_that("slice-assign checks vectorness", {
  x <- foobar(list(1))
  expect_error(vec_slice(x, 1) <- 10, class = "vctrs_error_scalar_type")
})

test_that("a coercible RHS is cast to LHS before assignment (#140)", {
  x <- 1:2
  expect_error(vec_slice(x, 1) <- "1", class = "vctrs_error_incompatible_type")

  x <- c("foo", "bar")
  expect_error(vec_slice(x, 1) <- 1, class = "vctrs_error_incompatible_type")

  x <- 1:2
  expect_error(vec_slice(x, 1) <- 3.5, class = "vctrs_error_cast_lossy")

  allow_lossy_cast(vec_slice(x, 1) <- 3.5)
  expect_identical(x, int(3, 2))

  x <- matrix(1:4, 2)
  vec_slice(x, 1) <- matrix(c(FALSE, FALSE), 1)
  expect_identical(x, matrix(int(0, 2, 0, 4), 2))
  expect_error(vec_assign(x, 1, matrix(c("", ""), 1)), class = "vctrs_error_incompatible_type")
})

test_that("slice-assign takes the proxy", {
  local_proxy()

  x <- new_proxy(1:3)
  y <- new_proxy(20:21)

  vec_slice(x, 2:3) <- y

  expect_identical(proxy_deref(x), int(1, 20, 21))
})

test_that("can use names to vec_slice<-() a named object", {
  x0 <- c(a = 1, b = 2)
  x1 <- c(a = 1, a = 2)

  vec_slice(x0, "b") <- 3
  expect_identical(x0, c(a = 1, b = 3))

  vec_slice(x1, "a") <- 3
  expect_identical(x1, c(a = 3, a = 2))
})

test_that("can't use names to vec_slice<-() an unnamed object", {
  x0 <- 1:3

  expect_error(
    vec_slice(x0, letters[1]) <- 4L,
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
  expect_error(
    vec_slice(x0, letters[25:27]) <- 5L,
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
})

test_that("slice-assign falls back to `[<-` when proxy is not implemented", {
  obj <- foobar(c("foo", "bar", "baz"))
  expect_error(vec_slice(obj, 1:2) <- TRUE, class = "vctrs_error_incompatible_type")

  vec_slice(obj, 1:2) <- foobar("quux")

  vec_ptype2(foobar(""), foobar(""))
  vec_cast(foobar(""), foobar(""))
  #> Error: Can't cast <vctrs_foobar> to <vctrs_foobar>

  local_methods(
    `[<-.vctrs_foobar` = function(x, i, value) {
      x <- unclass(x)
      x[i] <- "dispatched"
      x
    },
    vec_ptype2.logical.vctrs_foobar = function(...) foobar(""),
    vec_ptype2.vctrs_foobar = function(...) foobar(""),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.logical = function(x, to, ...) foobar(rep("", length(x)))
  )

  obj <- foobar(c("foo", "bar", "baz"))
  vec_slice(obj, 1:2) <- TRUE
  expect_identical(obj, foobar(c("dispatched", "dispatched", "baz")))
})

test_that("vec_assign() can always assign unspecified values into foreign vector types", {
  obj <- foobar(c("foo", "bar", "baz"))
  expect <- foobar(c(NA, "bar", "baz"))

  expect_identical(vec_assign(obj, 1, NA), expect)
  expect_identical(vec_assign(obj, 1, unspecified(1)), expect)
})

test_that("slice-assign casts to `to` before falling back to `[<-` (#443)", {
  called <- FALSE

  local_methods(
    vec_proxy.vctrs_proxy = proxy_deref,
    vec_ptype2.vctrs_proxy = function(...) NULL,
    vec_ptype2.vctrs_proxy.vctrs_foobar = function(...) new_proxy(NA),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.vctrs_proxy = function(x, ...) foobar(proxy_deref(x)),
    `[<-.vctrs_foobar` = function(x, i, value) {
      called <<- TRUE
      expect_identical(value, foobar(10))
    }
  )

  x <- foobar(1)
  y <- new_proxy(10)
  vec_slice(x, 1) <- y
  expect_true(called)
})

test_that("index and value are sliced before falling back", {
  # Work around a bug in base R `[<-`
  lhs <- foobar(c(NA, 1:4))
  rhs <- foobar(int(0L, 10L))
  exp <- foobar(int(10L, 1:4))
  expect_identical(vec_assign(lhs, c(NA, 1), rhs), exp)
})

test_that("can assign to data frame", {
  x <- data_frame(x = 1:3)
  y <- data_frame(x = 20)
  expect_identical(vec_assign(x, 2, y), data_frame(x = int(1, 20, 3)))
})

test_that("can assign to a data frame with matrix columns (#625)", {
  df <- tibble(x = 1:2, y = matrix(1:4, nrow = 2))
  expect_identical(vec_assign(df, 2L, df[1,]), vec_slice(df, c(1, 1)))
})

test_that("assigning to a factor doesn't produce corrupt levels (#853)", {
  x <- factor(c("a", NA), levels = c("a", "b"))
  value <- factor("b", levels = "b")

  res <- vec_assign(x, 2, value)
  expect_identical(res, factor(c("a", "b")))

  res <- vec_assign(x, 1:2, value)
  expect_identical(res, factor(c("b", "b"), levels = c("a", "b")))
})

test_that("can slice-assign unspecified vectors with default type2 method", {
  local_rational_class()
  x <- rational(1:2, 2:3)
  x[[1]] <- NA
  expect_identical(x, rational(c(NA, 2L), c(NA, 3L)))
})

test_that("`vec_assign()` evaluates arg lazily", {
  expect_silent(vec_assign(1L, 1L, 1L, x_arg = print("oof")))
  expect_silent(vec_assign(1L, 1L, 1L, value_arg = print("oof")))
})

test_that("`vec_assign()` requires recyclable value", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, 1:3, 1:2),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

test_that("logical subscripts must match size of indexed vector", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:2, c(TRUE, FALSE, TRUE), 5),
      class = "vctrs_error_subscript_size"
    ))
  })

  expect_snapshot(
    (expect_error(
      vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]),
      class = "vctrs_error_subscript_size"
    ))
  )
})

test_that("must assign existing elements", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, 5, 10),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_assign(1:3, "foo", 10),
      "unnamed vector"
    ))
    (expect_error(
      vec_slice(letters, -100) <- "foo",
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_assign(set_names(letters), "foo", "bar"),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("must assign with proper negative locations", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, c(-1, 1), 1:2),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_assign(1:3, c(-1, NA), 1:2),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("`vec_assign()` error args can be overridden", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:2, 1L, "x", x_arg = "foo", value_arg = "bar"),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_assign(1:2, 1L, 1:2, value_arg = "bar"),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

test_that("names are not assigned by default", {
  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  vec_out <- c(a = 1L, b = 4L, c = 3L)
  expect_identical(
    vec_assign(vec_x, 2, vec_y),
    vec_out
  )

  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "FOO")
  df_out <- new_data_frame(list(x = c(1L, 4L, 3L)), row.names = letters[1:3])
  expect_identical(
    vec_assign(df_x, 2, df_y),
    df_out
  )

  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("FOO"))
  mat_out <- matrix(c(1L, 4L, 3L), dimnames = list(letters[1:3]))
  expect_identical(
    vec_assign(mat_x, 2, mat_y),
    mat_out
  )

  nested_x <- new_data_frame(list(df = df_x, mat = mat_x, vec = vec_x), row.names = c("foo", "bar", "baz"))
  nested_y <- new_data_frame(list(df = df_y, mat = mat_y, vec = vec_y), row.names = c("quux"))
  nested_out <- new_data_frame(list(df = df_out, mat = mat_out, vec = vec_out), row.names = c("foo", "bar", "baz"))
  expect_identical(
    vec_assign(nested_x, 2, nested_y),
    nested_out
  )
})

test_that("can optionally assign names", {
  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  vec_out <- c(a = 1L, FOO = 4L, c = 3L)
  expect_identical(
    vec_assign_params(vec_x, 2, vec_y, assign_names = TRUE),
    vec_out
  )

  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "FOO")
  df_out <- new_data_frame(list(x = c(1L, 4L, 3L)), row.names = c("a", "FOO", "c"))
  expect_identical(
    vec_assign_params(df_x, 2, df_y, assign_names = TRUE),
    df_out
  )

  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("FOO"))
  mat_out <- matrix(c(1L, 4L, 3L), dimnames = list(c("a", "FOO", "c")))
  expect_identical(
    vec_assign_params(mat_x, 2, mat_y, assign_names = TRUE),
    mat_out
  )

  nested_x <- new_data_frame(list(df = df_x, mat = mat_x, vec = vec_x), row.names = c("foo", "bar", "baz"))
  nested_y <- new_data_frame(list(df = df_y, mat = mat_y, vec = vec_y), row.names = c("quux"))
  nested_out <- new_data_frame(list(df = df_out, mat = mat_out, vec = vec_out), row.names = c("foo", "quux", "baz"))

  expect_identical(
    vec_assign_params(nested_x, 2, nested_y, assign_names = TRUE),
    nested_out
  )
})

test_that("can optionally assign names (OO case)", {
  # In case upstream attributes handling changes
  skip_on_cran()

  # `set_names()` must be on the inside, otherwise the POSIXlt object
  # gets a `balanced` attribute of `NA`
  oo_x <- as_posixlt(set_names(c("2020-01-01", "2020-01-02", "2020-01-03"), letters[1:3]))
  oo_y <- as_posixlt(c(FOO = "2020-01-04"))
  oo_out <- as_posixlt(c(a = "2020-01-01", FOO = "2020-01-04", c = "2020-01-03"))
  expect_identical(
    vec_assign_params(oo_x, 2, oo_y, assign_names = TRUE),
    oo_out
  )

  nested_x <- new_data_frame(list(oo = oo_x), row.names = c("foo", "bar", "baz"))
  nested_y <- new_data_frame(list(oo = oo_y), row.names = c("quux"))
  nested_out <- new_data_frame(list(oo = oo_out), row.names = c("foo", "quux", "baz"))

  expect_identical(
    vec_assign_params(nested_x, 2, nested_y, assign_names = TRUE),
    nested_out
  )
})

test_that("assignment requires that the value proxy is the same type as the output proxy", {
  x <- foobar(1)
  y <- foobar("a")

  local_foobar_proxy()
  local_methods(
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x
  )

  expect_error(
    vec_assign(x, 1, y),
    "`double` incompatible with `value` proxy of type `character`"
  )
})

test_that("assignment allows a df `value`'s column to be a different type than its proxy (#1082)", {
  x <- new_data_frame(list(x = foobar(1)))
  y <- new_data_frame(list(x = foobar(2)))

  local_methods(
    # proxying foobar wraps it in a 1 col df
    vec_proxy.vctrs_foobar = function(x, ...) {
      attributes(x) <- NULL
      new_data_frame(list(vec = x))
    },
    # restoring extracts the column
    vec_restore.vctrs_foobar = function(x, to, ...) {
      foobar(x$vec)
    },
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x
  )

  expect1 <- new_data_frame(list(x = foobar(c(1, 1))))
  expect2 <- new_data_frame(list(x = foobar(2)))

  expect_identical(vec_rbind(x, x), expect1)
  expect_identical(vec_assign(x, 1, y), expect2)
})

test_that("monitoring: assignment to a data frame with unshared columns doesn't overwrite (#986)", {
  x <- new_df_unshared_col()
  value <- new_data_frame(list(x = 2))
  expect <- new_data_frame(list(x = 1L))

  # - On R < 4.0.0, the NAMED value of the column is 0.
  # - On R >= 4.0.0, the refcnt of the column is 1 from the call to
  #   `SET_VECTOR_ELT()` in `new_df_unshared_col()`.
  expect_false(maybe_shared_col(x, 1L))

  new <- vec_assign(x, 1, value)

  # On R < 4.0.0, `vec_assign()` shallow duplicates `x`, which recursively
  # bumps the NAMED-ness of each column of `x` to the max value of 7 by
  # calling `ENSURE_NAMEDMAX()` on it. So the columns of `x` are all considered
  # shared from that.

  # On R >= 4.0.0, references are tracked more precisely.
  # - `new_df_unshared_col()` calls `SET_VECTOR_ELT()` when setting the
  #   column into `x`, bumping the column's namedness to 1.
  # - Then, at the start of `df_assign()`, `x` is shallow duplicated and
  #   assigned to `out`. This calls `ENSURE_NAMEDMAX()` on each column,
  #   however this does nothing on R 4.0.0. The refcnt of each column is instead
  #   incremented by 1 by calls to `SET_VECTOR_ELT()` in `duplicate1()`.
  #   So now it is at 2.
  # - But then in `df_assign()` we use `SET_VECTOR_ELT()` on `out`, overwriting
  #   each column. This actually decrements the refcnt on the value that was
  #   in `out` before the column was overwritten. The column of `out` that it
  #   decrements the refcnt for is the same SEXP as that column in `x`, so now
  #   it is back to 1, and it is not considered shared.

  if (getRversion() >= "4.0.0") {
    expect_false(maybe_shared_col(x, 1L))
  } else {
    expect_true(maybe_shared_col(x, 1L))
  }

  # Expect no changes to `x`!
  expect_identical(x, expect)
})

test_that("monitoring: assignment to atomic vectors doesn't modify by reference", {
  x <- c(1, 2, 3)
  expect <- c(1, 2, 3)

  vec_assign(x, 2, 3)

  expect_identical(x, expect)
})

# vec_assign + compact_seq -------------------------------------------------

# `start` is 0-based

test_that("can assign shaped base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing), mat(lgl(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing), mat(int(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing), mat(dbl(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(cpl2(1, 2, 3)), NA, start, size, increasing), mat(cpl2(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing), mat(chr("1", NA, NA)))
  expect_identical(vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing), mat(raw2(1, 1, 1)))
  expect_identical(vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing), mat(list(1, NULL, NULL)))
})

test_that("can assign shaped base vectors with decreasing compact seqs", {
  start <- 2L
  size <- 2L
  increasing <- FALSE
  mat <- as.matrix
  expect_identical(vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing), mat(lgl(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing), mat(int(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing), mat(dbl(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(cpl2(1, 2, 3)), NA, start, size, increasing), mat(cpl2(1, NA, NA)))
  expect_identical(vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing), mat(chr("1", NA, NA)))
  expect_identical(vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing), mat(raw2(1, 1, 1)))
  expect_identical(vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing), mat(list(1, NULL, NULL)))
})

test_that("can assign shaped base vectors with size 0 compact seqs", {
  start <- 1L
  size <- 0L
  increasing <- TRUE
  mat <- as.matrix

  expect_identical(vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing), mat(mat(lgl(1, 0, 1))))
  expect_identical(vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing), mat(int(1, 2, 3)))
  expect_identical(vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing), mat(dbl(1, 2, 3)))
  expect_identical(vec_assign_seq(mat(cpl(1, 2, 3)), NA, start, size, increasing), mat(cpl(1, 2, 3)))
  expect_identical(vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing), mat(chr("1", "2", "3")))
  expect_identical(vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing), mat(raw2(1, 2, 3)))
  expect_identical(vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing), mat(list(1, 2, 3)))
})

test_that("can assign object of any dimensionality with compact seqs", {
  x1 <- ones(3)
  x2 <- ones(3, 4)
  x3 <- ones(3, 4, 5)
  x4 <- ones(3, 4, 5, 6)

  start <- 0L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix

  expect_identical(vec_assign_seq(x1, 2, start, size, increasing), array(rep(c(2, 2, 1), 1), dim = 3))
  expect_identical(vec_assign_seq(x2, 2, start, size, increasing), array(rep(c(2, 2, 1), 4), dim = c(3, 4)))
  expect_identical(vec_assign_seq(x3, 2, start, size, increasing), array(rep(c(2, 2, 1), 20), dim = c(3, 4, 5)))
  expect_identical(vec_assign_seq(x4, 2, start, size, increasing), array(rep(c(2, 2, 1), 120), dim = c(3, 4, 5, 6)))
})

local_name_repair_quiet()

test_that("zero length input returns NULL", {
  expect_equal(vec_c(), NULL)
  expect_equal(vec_c(NULL), NULL)
  expect_equal(vec_c(NULL,), NULL)
  expect_equal(vec_c(NULL, NULL), NULL)
})

test_that("NULL is idempotent", {
  expect_equal(vec_c(NULL, 1L), 1L)
  expect_equal(vec_c(1L, NULL), 1L)
})

test_that("NA is idempotent", {
  expect_equal(vec_c(NA, 1L), c(NA, 1L))
  expect_equal(vec_c(NA, "x"), c(NA, "x"))
  expect_equal(vec_c(NA, factor("x")), factor(c(NA, "x")))
  expect_equal(vec_c(NA, new_date(0)), new_date(c(NA, 0)))
  expect_equal(vec_c(NA, new_datetime(0)), new_datetime(c(NA, 0)))
  expect_equal(vec_c(NA, new_duration(0)), new_duration(c(NA, 0)))
})

test_that("NA is logical if no other types intervene", {
  expect_equal(vec_c(logical()), logical())
  expect_equal(vec_c(NA), NA)
  expect_equal(vec_c(NA, NA), c(NA, NA))
})


test_that("different types are coerced to common", {
  expect_equal(vec_c(TRUE, 1L, 1), c(1, 1, 1))
  expect_equal(vec_c(TRUE, 2:4), 1:4)
})

test_that("specified .ptypes do not allow more casts", {
  expect_error(
    vec_c(TRUE, .ptype = character()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("common type failure uses error call and error arg (#1641, #1692)", {
  expect_snapshot(error = TRUE, {
    vec_c("x", 1, .error_call = call("foo"), .error_arg = "arg")
  })
  expect_snapshot(error = TRUE, {
    vec_c("x", .ptype = integer(), .error_call = call("foo"), .error_arg = "arg")
  })
})

test_that("common type failure uses positional errors", {
  expect_snapshot({
    # Looking for `..1` and `a`
    (expect_error(vec_c(1, a = "x", 2)))

    # Directed cast should also produce positional errors (#1690)
    (expect_error(vec_c(1, a = "x", 2, .ptype = double(), .error_arg = "arg")))

    # Lossy cast
    (expect_error(vec_c(1, a = 2.5, .ptype = integer())))
  })
})

test_that("combines outer an inner names", {
  expect_equal(vec_c(x = 1), c(x = 1))
  expect_equal(vec_c(c(x = 1)), c(x = 1))

  expect_equal(vec_c(c(x = 1:2)), c(x1 = 1, x2 = 2))
  expect_error(vec_c(y = c(x = 1)), "Please supply")
})

test_that("can bind data.frame columns", {
  df <- data.frame(x = NA, y = 1:2)
  df$x <- data.frame(a = 1:2)

  expected <- data.frame(x = NA, y = c(1:2, 1:2))
  expected$x <- data.frame(a = c(1:2, 1:2))

  expect_equal(vec_c(df, df), expected)
})

test_that("vec_c() handles matrices", {
  m <- matrix(1:4, nrow = 2)
  dimnames(m) <- list(c("foo", "bar"), c("baz", "quux"))

  # FIXME: `vec_ptype_common(m, m)` doesn't return dimension names
  exp <- matrix(c(1:2, 1:2, 3:4, 3:4), nrow = 4)
  rownames(exp) <- c("foo", "bar", "foo", "bar")

  expect_identical(vec_c(m, m), exp)

  expect_error(vec_c(outer = m), "Please supply")
})

test_that("vec_c() includes index in argument tag", {
  df1 <- tibble(x = tibble(y = tibble(z = 1)))
  df2 <- tibble(x = tibble(y = tibble(z = "a")))

  expect_snapshot(error = TRUE, vec_c(df1, df2))
  expect_snapshot(error = TRUE, vec_c(df1, df1, df2))
  expect_snapshot(error = TRUE, vec_c(foo = df1, bar = df2))
})

test_that("vec_c() handles record classes", {
  local_rational_class()

  out <- vec_c(rational(1, 2), 1L, NA)

  expect_true(vec_is(out, rational(1, 2)))
  expect_size(out, 3)
  expect_identical(vec_proxy(out), data.frame(n = c(1L, 1L, NA), d = c(2L, 1L, NA)))
})

test_that("can mix named and unnamed vectors (#271)", {
  expect_identical(vec_c(c(a = 1), 2), c(a = 1, 2))
  expect_identical(vec_c(0, c(a = 1), 2, b = 3), c(0, a = 1, 2, b =3))
})

test_that("preserves names when inputs are cast to a common type (#1690)", {
  expect_named(vec_c(c(a = 1), .ptype = integer()), "a")
  expect_named(vec_c(foo = c(a = 1), .ptype = integer(), .name_spec = "{outer}_{inner}"), "foo_a")
})

test_that("vec_c() repairs names", {
  local_name_repair_quiet()

  # Default minimal repair
  expect_named(vec_c(a = 1, a = 2, `_` = 3), c("a", "a", "_"))
  out <- vec_c(!!!set_names(1, NA))
  expect_named(out, "")

  expect_named(vec_c(a = 1, a = 2, `_` = 3, .name_repair = "unique"), c("a...1", "a...2", "_"))
  expect_error(vec_c(a = 1, a = 2, `_` = 3, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")

  expect_named(vec_c(a = 1, a = 2, `_` = 3, .name_repair = "universal"), c("a...1", "a...2", "._"))

  expect_named(vec_c(a = 1, a = 2, .name_repair = ~ toupper(.)), c("A", "A"))
})

test_that("vec_c() can repair names quietly", {
  local_name_repair_verbose()

   expect_snapshot({
     res_unique <- vec_c(x = TRUE, x = 0, .name_repair = "unique_quiet")
     res_universal <- vec_c("if" = TRUE, "in" = 0, .name_repair = "universal_quiet")
   })
   expect_named(res_unique, c("x...1", "x...2"))
   expect_named(res_universal, c(".if", ".in"))
})

test_that("vec_c() doesn't use outer names for data frames (#524)", {
  x <- data.frame(inner = 1)
  expect_equal(vec_c(outer = x), x)

  a <- data.frame(x = 1L)
  b <- data.frame(x = 2L)
  expect_equal(vec_c(foo = a, bar = b), data.frame(x = 1:2))
})

test_that("vec_c() preserves row names and inner names", {
  x <- data.frame(a = 1, row.names = "r1")
  y <- data.frame(a = 2, row.names = "r2")
  expect_equal(rownames(vec_c(x, y)), c("r1", "r2"))
  expect_equal(rownames(vec_c(x, x)), c("r1...1", "r1...2"))

  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  oo_x <- set_names(as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")), letters[1:3])
  oo_y <- as.POSIXlt(c(FOO = "2020-01-04"))
  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "d")
  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("d"))
  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x, oo = oo_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y, oo = oo_y),
    row.names = c("quux")
  )

  nested_out <- vec_c(nested_x, nested_y)
  expect_identical(row.names(nested_out), c("foo", "bar", "baz", "quux"))
  expect_identical(row.names(nested_out$df), c("a", "b", "c", "d"))
  expect_identical(row.names(nested_out$mat), c("a", "b", "c", "d"))
  expect_identical(names(nested_out$vec), c("a", "b", "c", "FOO"))
  expect_identical(names(nested_out$oo), c("a", "b", "c", "FOO"))
})

test_that("vec_c() outer names work with proxied objects", {
  x <- as.POSIXlt(new_datetime(0))
  exp <- set_names(x, "outer")
  expect_equal(vec_c(outer = x), exp)

  named_x <- set_names(x, "inner")
  exp <- set_names(named_x, "outer_inner")
  expect_error(vec_c(outer = named_x), "Please supply")
  expect_equal(vec_c(outer = named_x, .name_spec = "{outer}_{inner}"), exp)

  xs <- as.POSIXlt(new_datetime(c(0, 1)))
  exp <- set_names(xs, c("outer_1", "outer_2"))
  expect_error(vec_c(outer = xs), "Please supply")
  expect_equal(vec_c(outer = xs, .name_spec = "{outer}_{inner}"), exp)
})

test_that("vec_c() works with simple homogeneous foreign S3 classes", {
  expect_identical(vec_c(foobar(1), foobar(2)), vec_c(foobar(c(1, 2))))
  expect_identical(vec_c(NULL, foobar(1), foobar(2)), vec_c(foobar(c(1, 2))))
})

test_that("vec_c() works with simple homogeneous foreign S4 classes", {
  joe1 <- .Counts(c(1L, 2L), name = "Joe")
  joe2 <- .Counts(3L, name = "Joe")
  expect_identical(vec_c(joe1, joe2), .Counts(1:3, name = "Joe"))
})

test_that("vec_c() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    (expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_c(x, y, .error_call = call("foo"), .error_arg = "arg"), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_c() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(c(1L, 2L), name = "Joe")
    jane <- .Counts(3L, name = "Jane")
    (expect_error(vec_c(joe, jane), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_c(joe, jane, .error_call = call("foo"), .error_arg = "arg"), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_c() falls back to c() if S3 method is available", {
  # Check off-by-one error
  expect_error(
    vec_c(foobar(1), "", foobar(2)),
    class = "vctrs_error_incompatible_type"
  )

  # Fallback when the class implements `c()`
  method <- function(...) rep_along(list(...), "dispatched")
  local_methods(
    c.vctrs_foobar = method
  )
  expect_identical(
    vec_c(foobar(1), foobar(2, class = "foo")),
    c("dispatched", "dispatched")
  )
  expect_identical(
    vec_c(NULL, foobar(1), NULL, foobar(2, class = "foo")),
    c("dispatched", "dispatched")
  )

  # Registered fallback
  s3_register("base::c", "vctrs_c_fallback", method)
  expect_identical(
    vec_c(
      structure(1, class = "vctrs_c_fallback"),
      structure(2, class = "vctrs_c_fallback")
    ),
    c("dispatched", "dispatched")
  )

  # Don't fallback for S3 lists which are treated as scalars by default
  expect_error(
    vec_c(foobar(list(1)), foobar(list(2))),
    class = "vctrs_error_scalar_type"
  )
})

test_that("c() fallback is consistent", {
  dispatched <- function(x) structure(x, class = "dispatched")
  c_method <- function(...) dispatched(NextMethod())

  out <- with_methods(
    vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_ptype2(...)),
    vec_cast.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_cast(...)),
    c.vctrs_foobar = c_method,
    list(
      direct = vec_c(foobar(1L), foobar(2L)),
      df = vec_c(data_frame(x = foobar(1L)), data_frame(x = foobar(2L))),
      tib = vec_c(tibble(x = foobar(1L)), tibble(x = foobar(2L))),
      foreign_df = vec_c(foobaz(data_frame(x = foobar(1L))), foobaz(data_frame(x = foobar(2L))))
    )
  )
  expect_equal(out$direct, dispatched(1:2))
  expect_equal(out$df$x, dispatched(1:2))
  expect_equal(out$tib$x, dispatched(1:2))
  expect_equal(out$foreign_df$x, dispatched(1:2))

  # Hard case: generic record vectors
  my_rec_record <- function(x) {
    new_rcrd(list(x = x), class = "my_rec_record")
  }

  out <- with_methods(
    c.vctrs_foobar = c_method,
    vec_ptype2.my_rec_record.my_rec_record = function(x, y, ...) {
      my_rec_record(vec_ptype2(field(x, "x"), field(y, "x"), ...))
    },
    vec_cast.my_rec_record.my_rec_record = function(x, to, ...) {
      x
    },
    vec_c(
      data_frame(x = my_rec_record(foobar(1L))),
      data_frame(x = my_rec_record(foobar(2L)))
    )
  )
  expect_equal(field(out$x, "x"), dispatched(1:2))
})

test_that("vec_c() falls back to c() if S4 method is available", {
  joe1 <- .Counts(c(1L, 2L), name = "Joe")
  joe2 <- .Counts(3L, name = "Joe")

  c_counts <- function(x, ...) {
    xs <- list(x, ...)

    xs_data <- lapply(xs, function(x) x@.Data)
    new_data <- do.call(c, xs_data)

    .Counts(new_data, name = x@name)
  }

  local_s4_method("c", methods::signature(x = "vctrs_Counts"), c_counts)

  expect_identical(
    vec_c(joe1, joe2),
    .Counts(c(1L, 2L, 3L), name = "Joe")
  )

  expect_identical(
    vec_c(NULL, joe1, joe2),
    .Counts(c(1L, 2L, 3L), name = "Joe")
  )
})

test_that("vec_c() fallback doesn't support `name_spec` or `ptype`", {
  expect_snapshot({
    (expect_error(
      with_c_foobar(vec_c(foobar(1), foobar(2), .name_spec = "{outer}_{inner}")),
      "name specification"
    ))

    # Used to be an error about `ptype`
    (expect_error(
      with_c_foobar(vec_c(foobar(1), foobar(2), .ptype = "")),
      class = "vctrs_error_incompatible_type"
    ))

    # Uses error call (#1641)
    (expect_error(
      with_c_foobar(vec_c(
        foobar(1), foobar(2),
        .error_call = call("foo"),
        .name_spec = "{outer}_{inner}"
      ))
    ))
  })
})

test_that("vec_c() doesn't fall back when ptype2 is implemented", {
  new_quux <- function(x) structure(x, class = "vctrs_quux")

  with_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) new_quux(int()),
    vec_cast.vctrs_quux.vctrs_foobar = function(x, to, ...) new_quux(x),
    vec_restore.vctrs_quux = function(x, ...) new_quux(x),
    c.vctrs_foobar = function(...) foobar(NextMethod()),
    {
      expect_s3_class(c(foobar(1:3), foobar(4L)), "vctrs_foobar")
      expect_s3_class(vec_c(foobar(1:3), foobar(4L)), "vctrs_quux")
    }
  )
})

test_that("vec_c() falls back even when ptype is supplied", {
  expect_foobar(vec_c(foobar(1), foobar(2), .ptype = foobar(dbl())))

  with_methods(
    c.vctrs_foobar = function(...) quux(NextMethod()),
    {
      expect_quux(vec_c(foobar(1), foobar(2), .ptype = foobar(dbl())))
      expect_quux(vec_c(foobar(1, foo = TRUE), foobar(2, bar = TRUE), .ptype = foobar(dbl())))
    }
  )
})

test_that("vec_implements_ptype2() is FALSE for scalars", {
  expect_false(vec_implements_ptype2(quote(foo)))
})

test_that("vec_implements_ptype2() and vec_c() fallback are compatible with old registration", {
  foo <- structure(NA, class = "vctrs_implements_ptype2_false")
  expect_false(vec_implements_ptype2(foo))

  vec_ptype2.vctrs_implements_ptype2_true <- function(...) NULL
  s3_register(
    "vctrs::vec_ptype2",
    "vctrs_implements_ptype2_true",
    vec_ptype2.vctrs_implements_ptype2_true
  )

  bar <- structure(NA, class = "vctrs_implements_ptype2_true")
  expect_true(vec_implements_ptype2(bar))

  local_methods(
    `c.vctrs_implements_ptype2_true` = function(...) stop("never called")
  )

  expect_identical(vec_c(bar), bar)
})

test_that("can ignore names in `vec_c()` by providing a `zap()` name-spec (#232)", {
  expect_error(vec_c(a = c(b = 1:2)))
  expect_identical(vec_c(a = c(b = 1:2), b = 3L, .name_spec = zap()), 1:3)

  expect_snapshot({
    (expect_error(
      vec_c(a = c(b = letters), b = 1, .name_spec = zap()),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("can concatenate subclasses of `vctrs_vctr` which don't have ptype2 methods", {
  x <- new_vctr(1, class = "vctrs_foo")
  expect_identical(vec_c(x, x), new_vctr(c(1, 1), class = "vctrs_foo"))
})

test_that("base c() fallback handles unspecified chunks", {
  local_methods(
    c.vctrs_foobar = function(...) {
      x <- NextMethod()

      # Should not be passed any unspecified chunks
      if (anyNA(x)) {
        abort("tilt")
      }

      foobar(x)
    },
    `[.vctrs_foobar` = function(x, i, ...) {
      # Return a quux to detect dispatch
      quux(NextMethod())
    }
  )

  out <- vec_c(foobar(1:2), rep(NA, 2))
  expect_identical(out, quux(c(1:2, NA, NA)))

  out <- vec_c(rep(NA, 2), foobar(1:2), NA)
  expect_identical(out, quux(c(NA, NA, 1:2, NA)))
})

test_that("can zap outer names from a name-spec (#1215)", {
  zap_outer_spec <- function(outer, inner) if (is_character(inner)) inner

  expect_null(
    names(vec_c(a = 1:2, .name_spec = zap_outer_spec))
  )
  expect_identical(
    names(vec_c(a = 1:2, c(foo = 3L), .name_spec = zap_outer_spec)),
    c("", "", "foo")
  )

  expect_null(
    names(list_unchop(list(a = 1:2), indices = list(1:2), name_spec = zap_outer_spec))
  )
  expect_identical(
    names(list_unchop(list(a = 1:2, c(foo = 3L)), indices = list(1:2, 3), name_spec = zap_outer_spec)),
    c("", "", "foo")
  )
})

test_that("named empty vectors force named output (#1263)", {
  x <- set_names(int(), chr())

  expect_named(vec_c(x), chr())
  expect_named(vec_c(x, x), chr())
  expect_named(vec_c(x, 1L), "")
  expect_named(vec_c(x, 1), "")

  expect_named(list_unchop(list(x), indices = list(int())), chr())
  expect_named(list_unchop(list(x, x), indices = list(int(), int())), chr())
  expect_named(list_unchop(list(x, 1L), indices = list(int(), 1)), "")

  expect_named(list_unchop(list(x, 1), indices = list(int(), 1)), "")
})

# Golden tests -------------------------------------------------------

test_that("concatenation performs expected allocations", {
  vec_c_list <- function(x, ptype = NULL) {
    vec_c(!!!x, .ptype = ptype)
  }

  expect_snapshot({
    ints <- rep(list(1L), 1e2)
    dbls <- rep(list(1), 1e2)

    # Extra allocations from `list2()`, see r-lib/rlang#937
    "# `vec_c()` "
    "Integers"
    with_memory_prof(vec_c_list(ints))

    "Doubles"
    with_memory_prof(vec_c_list(dbls))

    "Integers to integer"
    with_memory_prof(vec_c_list(ints, ptype = int()))

    "Doubles to integer"
    with_memory_prof(vec_c_list(dbls, ptype = int()))


    "# `list_unchop()` "
    "Integers"
    with_memory_prof(list_unchop(ints))

    "Doubles"
    with_memory_prof(list_unchop(dbls))

    "Integers to integer"
    with_memory_prof(list_unchop(ints, ptype = int()))

    "Doubles to integer"
    with_memory_prof(list_unchop(dbls, ptype = int()))


    "# Concatenation with names"

    "Named integers"
    ints <- rep(list(set_names(1:3, letters[1:3])), 1e2)
    with_memory_prof(list_unchop(ints))

    "Named matrices"
    mat <- matrix(1:4, 2, dimnames = list(c("foo", "bar")))
    mats <- rep(list(mat), 1e2)
    with_memory_prof(list_unchop(mats))

    "Data frame with named columns"
    df <- data_frame(
      x = set_names(as.list(1:2), c("a", "b")),
      y = set_names(1:2, c("A", "B")),
      z = data_frame(Z = set_names(1:2, c("Za", "Zb")))
    )
    dfs <- rep(list(df), 1e2)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (non-repaired, non-recursive case)"
    df <- data_frame(x = 1:2)
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (repaired, non-recursive case)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (non-repaired, recursive case) (#1217)"
    df <- data_frame(
      x = 1:2,
      y = data_frame(x = 1:2)
    )
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (repaired, recursive case) (#1217)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "list-ofs (#1496)"
    make_list_of <- function(n) {
      df <- tibble::tibble(
        x = new_list_of(vec_chop(1:n), ptype = integer())
      )
      vec_chop(df)
    }
    with_memory_prof(list_unchop(make_list_of(1e3)))
    with_memory_prof(list_unchop(make_list_of(2e3)))
    with_memory_prof(list_unchop(make_list_of(4e3)))
  })
})

test_that("can dispatch many times", {
  # This caused a crash when counters were not correctly protected
  foo <- structure(
    list(x.sorted = numeric(0), tp = numeric(0), fp = numeric(0)),
    row.names = integer(0),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame")
  )
  x <- lapply(1:200, function(...) foo)
  expect_error(NA, object = vctrs::list_unchop(x))
})

test_that("dots splicing clones as appropriate", {
  x <- list(a = 1)
  vctrs::vec_cbind(!!!x)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_rbind(!!!x)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_c(!!!x)
  expect_equal(x, list(a = 1))


  x <- list(a = 1)
  vctrs::vec_cbind(!!!x, 2)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_rbind(!!!x, 2)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_c(!!!x, 2)
  expect_equal(x, list(a = 1))
})

test_that("can combine records wrapped in data frames", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) {
      data_frame(x = unclass(x), y = seq_along(x))
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      foobar(x$x)
    }
  )

  x <- foobar(1:2)
  y <- foobar(3:4)

  expect_equal(
    vec_c(x, y),
    foobar(1:4)
  )

  expect_equal(
    list_unchop(list(x, y), indices = list(1:2, 3:4)),
    foobar(1:4)
  )

  expect_equal(
    vec_rbind(data_frame(x = x), data_frame(x = y)),
    data_frame(x = foobar(1:4))
  )
})

test_that("fallback works with subclasses of `vctrs_vctr`", {
  # Used to fail because of interaction between common class fallback
  # for `base::c()` and the `c()` method for `vctrs_vctr` that called
  # back into `vec_c()`.

  # Reprex for failure in the ricu package
  x <- new_rcrd(list(a = 1), class = "vctrs_foobar")
  expect_equal(
    vec_c(x, x, .name_spec = "{inner}"),
    new_rcrd(list(a = c(1, 1)), class = "vctrs_foobar")
  )

  # Reprex for failure in the groupr package
  x <- new_rcrd(list(a = 1), class = "vctrs_foobar")
  df <- data_frame(x = x)
  expect_equal(
    vec_rbind(df, data.frame()),
    df
  )
  expect_equal(
    vec_cast_common(df, data.frame()),
    list(df, data_frame(x = x[0]))
  )
})

# `grouped_df` -------------------------------------------------------

bare_mtcars <- unrownames(mtcars)

test_that("grouped-df is proxied and restored", {
  gdf <- dplyr::group_by(bare_mtcars, cyl)

  expect_identical(vec_proxy(gdf), gdf)
  expect_identical(vec_restore(bare_mtcars, gdf), gdf)

  expect_identical(vec_ptype(gdf), gdf[0, ])

  gdf <- dplyr::group_by(bare_mtcars, cyl, am, vs)
  expect_identical(gdf[0, ], vec_ptype(gdf))

  out <- vec_ptype(dplyr::group_by(bare_mtcars, cyl, .drop = FALSE))
  expect_drop(out, FALSE)
})

test_that("can take the common type of grouped tibbles and tibbles", {
  gdf <- dplyr::group_by(bare_mtcars, cyl)
  expect_identical(vec_ptype2(gdf, data.frame()), vec_ptype(gdf))
  expect_identical(vec_ptype2(data.frame(), gdf), vec_ptype(gdf))
  expect_identical(vec_ptype2(gdf, tibble()), vec_ptype(gdf))
  expect_identical(vec_ptype2(tibble(), gdf), vec_ptype(gdf))

  gdf_nodrop <- dplyr::group_by(bare_mtcars, cyl, .drop = FALSE)
  expect_drop(vec_ptype2(gdf, gdf_nodrop), FALSE)
  expect_drop(vec_ptype2(gdf_nodrop, gdf), FALSE)
  expect_drop(vec_ptype2(gdf_nodrop, bare_mtcars), FALSE)
  expect_drop(vec_ptype2(bare_mtcars, gdf_nodrop), FALSE)
})

test_that("the common type of grouped tibbles includes the union of grouping variables", {
  gdf1 <- dplyr::group_by(bare_mtcars, cyl)
  gdf2 <- dplyr::group_by(bare_mtcars, am, vs)
  expect_identical(
    vec_ptype2(gdf1, gdf2),
    vec_ptype(dplyr::group_by(bare_mtcars, cyl, am, vs))
  )
})

test_that("can cast to and from `grouped_df`", {
  gdf <- dplyr::group_by(unrownames(bare_mtcars), cyl)
  input <- bare_mtcars[10]
  cast_gdf <- dplyr::group_by(vec_cast(bare_mtcars[10], bare_mtcars), cyl)

  expect_error(
    vec_cast(input, dplyr::group_by(bare_mtcars["cyl"], cyl)),
    class = "vctrs_error_cast_lossy"
  )

  expect_identical(
    vec_cast(input, gdf),
    cast_gdf
  )
  expect_identical(
    vec_cast(gdf, bare_mtcars),
    unrownames(bare_mtcars)
  )

  expect_identical(
    vec_cast(tibble::as_tibble(input), gdf),
    unrownames(cast_gdf)
  )
  tib <- tibble::as_tibble(bare_mtcars)
  expect_identical(
    unrownames(vec_cast(gdf, tib)),
    tib
  )
})

test_that("casting to `grouped_df` doesn't require grouping variables", {
  expect_identical(
    vec_cast(bare_mtcars[10], dplyr::group_by(bare_mtcars, cyl)),
    dplyr::group_by(vec_cast(bare_mtcars[10], bare_mtcars), cyl)
  )
})

test_that("casting to `grouped_df` handles `drop`", {
  gdf_nodrop <- dplyr::group_by(bare_mtcars, cyl, .drop = FALSE)
  expect_identical(vec_cast(bare_mtcars, gdf_nodrop), gdf_nodrop)
})

test_that("can cbind grouped data frames", {
  gdf <- dplyr::group_by(bare_mtcars[-10], cyl)
  df <- unrownames(bare_mtcars)[10]

  expect_identical(
    unrownames(vec_cbind(gdf, df)),
    tibble::as_tibble(bare_mtcars)[c(1:9, 11, 10)]
  )

  gdf1 <- dplyr::group_by(bare_mtcars[2], cyl)
  gdf2 <- dplyr::group_by(bare_mtcars[8:9], vs, am)
  expect_identical(
    unrownames(vec_cbind(gdf1, gdf2)),
    tibble::as_tibble(bare_mtcars)[c(2, 8, 9)]
  )
})


# `rowwise` ----------------------------------------------------------

test_that("rowwise can be proxied and restored", {
  rww <- dplyr::rowwise(unrownames(bare_mtcars))

  expect_identical(vec_proxy(rww), rww)
  expect_identical(vec_restore(unrownames(bare_mtcars), rww), rww)

  expect_identical(vec_ptype(rww), rww[0, ])
})

test_that("can take the common type of rowwise tibbles and tibbles", {
  rww <- dplyr::rowwise(bare_mtcars)
  expect_identical(vec_ptype2(rww, data.frame()), vec_ptype(rww))
  expect_identical(vec_ptype2(data.frame(), rww), vec_ptype(rww))
  expect_identical(vec_ptype2(rww, tibble()), vec_ptype(rww))
  expect_identical(vec_ptype2(tibble(), rww), vec_ptype(rww))
})

test_that("can cast to and from `rowwise_df`", {
  rww <- unrownames(dplyr::rowwise(bare_mtcars))
  input <- bare_mtcars[10]
  cast_rww <- dplyr::rowwise(vec_cast(bare_mtcars[10], bare_mtcars))

  expect_error(
    vec_cast(input, dplyr::rowwise(bare_mtcars["cyl"])),
    class = "vctrs_error_cast_lossy"
  )

  expect_identical(
    vec_cast(input, rww),
    cast_rww
  )
  expect_identical(
    vec_cast(rww, bare_mtcars),
    unrownames(bare_mtcars)
  )

  expect_identical(
    vec_cast(tibble::as_tibble(input), rww),
    unrownames(cast_rww)
  )
  tib <- tibble::as_tibble(bare_mtcars)
  expect_identical(
    unrownames(vec_cast(rww, tib)),
    tib
  )
})

test_that("can cbind rowwise data frames", {
  df <- unrownames(bare_mtcars)
  rww <- dplyr::rowwise(df[-2])
  gdf <- dplyr::group_by(df[2], cyl)

  exp <- dplyr::rowwise(df[c(1, 3:11, 2)])
  expect_identical(vec_cbind(rww, df[2]), exp)

  # Suboptimal
  expect_identical(vec_cbind(rww, gdf), exp)
})

test_that("common type between rowwise and grouped data frames is a bare df", {
  out <- vec_ptype_common(dplyr::rowwise(bare_mtcars), dplyr::group_by(bare_mtcars, cyl))
  expect_identical(out, tibble::as_tibble(bare_mtcars[0, ]))
})

test_that("vec_data() preserves names (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_names(x), vec_names(vec_data(x)))

  x <- diag(2)
  rownames(x) <- letters[1:2]
  colnames(x) <- LETTERS[1:2]
  expect_identical(vec_names(x), vec_names(vec_data(x)))
})

test_that("vec_data() preserves size (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_size(x), vec_size(vec_data(x)))

  x <- diag(2)
  expect_identical(vec_size(x), vec_size(vec_data(x)))
})

test_that("vec_data() preserves dim and dimnames (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_dim(x), vec_dim(vec_data(x)))

  x <- diag(2)
  expect_identical(vec_dim(x), vec_dim(vec_data(x)))

  x <- diag(2)
  rownames(x) <- letters[1:2]
  colnames(x) <- LETTERS[1:2]
  expect_identical(dimnames(x), dimnames(vec_data(x)))
})

test_that("strips vector attributes apart from names, dim and dimnames", {
  x <- new_vctr(1:10, a = 1, b = 2)
  expect_null(attributes(vec_data(x)))

  x <- new_vctr(c(x = 1, y = 2), a = 1, b = 2)
  expect_equal(names(attributes(vec_data(x))), "names")

  x <- new_vctr(1, a = 1, dim = c(1L, 1L))
  expect_equal(names(attributes(vec_data(x))), "dim")

  x <- new_vctr(1, a = 1, dim = c(1L, 1L), dimnames = list("foo", "bar"))
  expect_equal(names(attributes(vec_data(x))), c("dim", "dimnames"))
})

test_that("vec_proxy() is a no-op with data vectors", {
  for (x in vectors) {
    expect_identical(vec_proxy(!!x), !!x)
  }

  x <- structure(1:3, foo = "bar")
  expect_identical(vec_proxy(!!x), !!x)
})

test_that("vec_proxy() transforms records to data frames", {
  for (x in records) {
    expect_identical(vec_proxy(x), new_data_frame(unclass(x)))
  }
})

test_that("vec_proxy() is a no-op with non vectors", {
  x <- foobar(list())
  expect_identical(vec_proxy(x), x)
})

test_that("can take the proxy of non-vector objects", {
  local_env_proxy()
  expect_identical(vec_proxy(new_proxy(1:3)), 1:3)
})

test_that("vec_data() asserts vectorness", {
  expect_error(vec_data(new_sclr()), class = "vctrs_error_scalar_type")
  expect_error(vec_data(~foo), class = "vctrs_error_scalar_type")
})

test_that("vec_data() is proxied", {
  local_env_proxy()
  x <- new_proxy(mtcars)
  expect_identical(vec_data(x), vec_data(mtcars))
})

test_that("vec_proxy_equal() is recursive over data frames (#641)", {
  x <- new_data_frame(list(x = foobar(1:3), y = 41:43))
  default <- vec_proxy_equal(x)
  expect_s3_class(default$x, "vctrs_foobar")

  local_methods(vec_proxy_equal.vctrs_foobar = function(...) c(0, 0, 0))
  overridden <- vec_proxy_equal(x)
  expect_identical(overridden$x, c(0, 0, 0))
})

test_that("vec_proxy_equal() returns a POSIXct for POSIXlt objects (#901)", {
  x <- as.POSIXlt(new_date(0), tz = "UTC")
  expect_s3_class(vec_proxy_equal(x), "POSIXct")
})

test_that("vec_proxy_equal() defaults to vec_proxy() and vec_proxy_compare() defaults to vec_proxy_equal() (#1140)", {
  foobar_proxy <- function(x, ...) data_frame(x = unclass(x), y = seq_along(x))

  local_methods(vec_proxy.vctrs_foobar = foobar_proxy)

  x <- foobar(3:1)
  expect_identical(vec_proxy(x), foobar_proxy(x))
  expect_identical(vec_proxy_equal(x), foobar_proxy(x))
  expect_identical(vec_proxy_compare(x), foobar_proxy(x))

  local_methods(vec_proxy_equal.vctrs_foobar = function(x, ...) foobar_proxy(letters[x]))

  expect_identical(vec_proxy_equal(x), data_frame(x = letters[3:1], y = 1:3))
  expect_identical(vec_proxy_compare(x), data_frame(x = letters[3:1], y = 1:3))
})

test_that("equal/compare/order proxy methods that return data frames are automatically flattened", {
  x <- new_vctr(1:2, class = "custom")

  equal <- data_frame(a = 1:2, b = 3:4)
  order <- data_frame(a = 3:4, b = 4:5)

  local_methods(
    vec_proxy_equal.custom = function(x, ...) data_frame(col = equal),
    vec_proxy_order.custom = function(x, ...) data_frame(col = order)
  )

  expect_identical(vec_proxy_equal(x), equal)
  expect_identical(vec_proxy_compare(x), equal)
  expect_identical(vec_proxy_order(x), order)
})

test_that("equal/compare/order proxy methods that return 1 column data frames are automatically unwrapped", {
  x <- new_vctr(1:2, class = "custom")

  equal <- 1:2
  order <- 3:4

  local_methods(
    vec_proxy_equal.custom = function(x, ...) data_frame(a = equal),
    vec_proxy_order.custom = function(x, ...) data_frame(col = data_frame(a = order))
  )

  expect_identical(vec_proxy_equal(x), equal)
  expect_identical(vec_proxy_compare(x), equal)
  expect_identical(vec_proxy_order(x), order)
})

test_that("vec_data() preserves data frames", {
  expect_identical(
    vec_data(tibble(x = 1)),
    data_frame(x = 1)
  )

  # Rownames are preserved
  expect_identical(
    vec_data(mtcars),
    mtcars
  )
})

test_that("vec_chop() throws error with non-vector inputs", {
  expect_error(vec_chop(NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_chop(environment()), class = "vctrs_error_scalar_type")
})

test_that("atomics are split into a list", {
  x <- 1:5
  expect_equal(vec_chop(x), as.list(x))

  x <- letters[1:5]
  expect_equal(vec_chop(x), as.list(x))
})

test_that("atomic names are kept", {
  x <- set_names(1:5)
  result <- lapply(vec_chop(x), names)
  expect_equal(result, as.list(names(x)))
})

test_that("base R classed objects are split into a list", {
  fctr <- factor(c("a", "b"))
  expect <- lapply(vec_seq_along(fctr), vec_slice, x = fctr)
  expect_equal(vec_chop(fctr), expect)

  date <- new_date(c(0, 1))
  expect <- lapply(vec_seq_along(date), vec_slice, x = date)
  expect_equal(vec_chop(date), expect)
})

test_that("base R classed object names are kept", {
  fctr <- set_names(factor(c("a", "b")))
  result <- lapply(vec_chop(fctr), names)
  expect_equal(result, as.list(names(fctr)))
})

test_that("list elements are split", {
  x <- list(1, 2)
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("data frames are split rowwise", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("vec_chop() keeps data frame row names", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  rownames(x) <- c("r1", "r2")
  result <- lapply(vec_chop(x), rownames)
  expect_equal(result, list("r1", "r2"))
})

test_that("vec_chop() keeps data frame row names for data frames with 0 columns (#1722)", {
  x <- data_frame(.size = 3)
  rownames(x) <- c("r1", "r2", "r3")

  out <- lapply(vec_chop(x), rownames)
  expect_identical(out, list("r1", "r2", "r3"))

  out <- vec_chop(x, indices = list(c(2, NA), 3))
  out <- lapply(out, rownames)
  expect_identical(out, list(c("r2", "...2"), "r3"))

  out <- vec_chop(x, sizes = c(1, 2, 0))
  out <- lapply(out, rownames)
  expect_identical(out, list("r1", c("r2", "r3"), character()))
})

test_that("data frames with 0 columns retain the right number of rows (#1722)", {
  x <- data_frame(.size = 4)

  one <- data_frame(.size = 1L)
  expect_identical(
    vec_chop(x),
    list(one, one, one, one)
  )

  expect_identical(
    vec_chop(x, indices = list(c(1, 3, 2), c(3, NA))),
    list(
      data_frame(.size = 3),
      data_frame(.size = 2)
    )
  )

  expect_identical(
    vec_chop(x, sizes = c(3, 1, 0)),
    list(
      data_frame(.size = 3),
      data_frame(.size = 1),
      data_frame(.size = 0)
    )
  )
})

test_that("matrices / arrays are split rowwise", {
  x <- array(1:12, c(2, 2, 2))
  result <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect_equal(vec_chop(x), result)
})

test_that("matrix / array row names are kept", {
  x <- array(1:12, c(2, 2, 2), dimnames = list(c("r1", "r2"), c("c1", "c2")))
  result <- lapply(vec_chop(x), rownames)
  expect_equal(result, list("r1", "r2"))
})

test_that("matrices / arrays without row names have other dimension names kept", {
  x <- array(1:12, c(2, 2, 2), dimnames = list(NULL, c("c1", "c2")))
  result <- lapply(vec_chop(x), colnames)
  expect_equal(result, list(c("c1", "c2"), c("c1", "c2")))
})

test_that("vec_chop() doesn't restore when attributes have already been restored", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) structure("dispatched", foo = "bar"),
    vec_restore.vctrs_foobar = function(...) structure("dispatched-and-restored", foo = "bar")
  )

  result <- vec_chop(foobar(NA))[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))

  result <- vec_chop(foobar(NA), indices = list(1))[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))

  result <- vec_chop(foobar(NA), sizes = 1)[[1]]
  expect_equal(result, structure("dispatched", foo = "bar"))
})

test_that("vec_chop() does not restore when attributes have not been restored by `[`", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) "dispatched",
    vec_restore.vctrs_foobar = function(...) "dispatched-and-restored"
  )

  result <- vec_chop(foobar(NA))[[1]]
  expect_equal(result, "dispatched")

  result <- vec_chop(foobar(NA), indices = list(1))[[1]]
  expect_equal(result, "dispatched")

  result <- vec_chop(foobar(NA), sizes = 1)[[1]]
  expect_equal(result, "dispatched")
})

test_that("vec_chop() falls back to `[` for shaped objects with no proxy", {
  x <- foobar(1)
  dim(x) <- c(1, 1)

  result <- vec_chop(x)[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, indices = list(1))[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, sizes = 1)[[1]]
  expect_equal(result, x)
})

test_that("`indices` are validated", {
  expect_snapshot(error = TRUE, {
    vec_chop(1, indices = 1)
  })
  expect_snapshot({
    (expect_error(vec_chop(1, indices = list(1.5)), class = "vctrs_error_subscript_type"))
  })
  expect_snapshot({
    (expect_error(vec_chop(1, indices = list(2)), class = "vctrs_error_subscript_oob"))
  })
})

test_that("`sizes` are validated", {
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = "a")
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = 2)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = -1)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = NA_integer_)
  })
  expect_snapshot(error = TRUE, {
    vec_chop("a", sizes = c(1, 1))
  })
})

test_that("can't use both `indices` and `sizes`", {
  expect_snapshot(error = TRUE, {
    vec_chop(1, indices = list(1), sizes = 1)
  })
})

test_that("`sizes` allows `0`", {
  expect_identical(
    vec_chop(c("a", "b"), sizes = c(1, 0, 0, 1, 0)),
    list("a", character(), character(), "b", character())
  )
})

test_that("size 0 `indices` list is allowed", {
  expect_equal(vec_chop(1, indices = list()), list())
})

test_that("individual index values of size 0 are allowed", {
  expect_equal(vec_chop(1, indices = list(integer())), list(numeric()))

  df <- data.frame(a = 1, b = "1")
  expect_equal(vec_chop(df, indices = list(integer())), list(vec_ptype(df)))
})

test_that("individual index values of `NULL` are allowed", {
  expect_equal(vec_chop(1, indices = list(NULL)), list(numeric()))

  df <- data.frame(a = 1, b = "1")
  expect_equal(vec_chop(df, indices = list(NULL)), list(vec_ptype(df)))
})

test_that("data frame row names are kept when `indices` or `sizes` are used", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  rownames(x) <- c("r1", "r2")

  result <- lapply(vec_chop(x, indices = list(1, 1:2)), rownames)
  expect_equal(result, list("r1", c("r1", "r2")))

  result <- lapply(vec_chop(x, sizes = c(1, 0, 1)), rownames)
  expect_equal(result, list("r1", character(), "r2"))
})

test_that("vec_chop(<atomic>, indices/sizes =) can be equivalent to the default", {
  x <- 1:5

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("vec_chop(<data.frame>, indices/sizes =) can be equivalent to the default", {
  x <- data.frame(x = 1:5)

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("vec_chop(<array>, indices/sizes =) can be equivalent to the default", {
  x <- array(1:8, c(2, 2, 2))

  indices <- as.list(vec_seq_along(x))
  expect_equal(vec_chop(x, indices = indices), vec_chop(x))

  sizes <- vec_rep(1L, times = vec_size(x))
  expect_equal(vec_chop(x, sizes = sizes), vec_chop(x))
})

test_that("`indices` cannot use names", {
  x <- set_names(1:3, c("a", "b", "c"))
  expect_error(vec_chop(x, indices = list("a", c("b", "c"))), class = "vctrs_error_subscript_type")

  x <- array(1:4, c(2, 2), dimnames = list(c("r1", "r2")))
  expect_error(vec_chop(x, indices = list("r1")), class = "vctrs_error_subscript_type")

  x <- data.frame(x = 1, row.names = "r1")
  expect_error(vec_chop(x, indices = list("r1")), class = "vctrs_error_subscript_type")
})

test_that("fallback method with `indices` and `sizes` works", {
  fctr <- factor(c("a", "b"))

  indices <- list(1, c(1, 2))
  sizes <- c(1, 0, 1)

  expect_equal(
    vec_chop(fctr, indices = indices),
    map(indices, vec_slice, x = fctr)
  )
  expect_equal(
    vec_chop(fctr, sizes = sizes),
    list(vec_slice(fctr, 1), vec_slice(fctr, 0), vec_slice(fctr, 2))
  )
})

test_that("vec_chop() falls back to `[` for shaped objects with no proxy when `indices` or `sizes` are provided", {
  x <- foobar(1)
  dim(x) <- c(1, 1)

  result <- vec_chop(x, indices = list(1))[[1]]
  expect_equal(result, x)

  result <- vec_chop(x, sizes = 1)[[1]]
  expect_equal(result, x)
})

test_that("vec_chop() with data frame proxies always uses the proxy's length info", {
  local_methods(
    vec_proxy.vctrs_proxy = function(x, ...) {
      x <- proxy_deref(x)
      new_data_frame(list(x = x$x, y = x$y))
    },
    vec_restore.vctrs_proxy = function(x, to, ...) {
      new_proxy(list(x = x$x, y = x$y))
    }
  )

  x <- new_proxy(list(x = 1:2, y = 3:4))
  result <- vec_chop(x)

  result1 <- result[[1]]
  result2 <- result[[2]]

  expect1 <- new_proxy(list(x = 1L, y = 3L))
  expect2 <- new_proxy(list(x = 2L, y = 4L))

  expect_identical(proxy_deref(result1), proxy_deref(expect1))
  expect_identical(proxy_deref(result2), proxy_deref(expect2))
})

test_that("ALTREP objects always generate materialized chops (#1450)", {
  skip_if(getRversion() <= "3.5.0")

  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))

  # `x` starts in compact form
  expect_false(.Call(vctrs_altrep_rle_is_materialized, x))

  result <- vec_chop(x)

  # Chopping materializes `x`
  expect_true(.Call(vctrs_altrep_rle_is_materialized, x))

  # And chopped elements are not ALTREP vectors
  expect_false(any(map_lgl(result, is_altrep)))

  expect <- vec_chop(c(rep("foo", 10), rep("bar", 5)))
  expect_identical(result, expect)
})

test_that("`vec_chop(x, indices)` backwards compatible behavior works", {
  # No issues here
  expect_identical(
    vec_chop(1:2, list(1, 2)),
    vec_chop(1:2, indices = list(1, 2))
  )

  # Errors still talk about `indices`
  expect_snapshot(error = TRUE, {
    vec_chop(1:2, 1)
  })
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), sizes = 1)
  })

  # These cases aren't allowed because they weren't possible previously either
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), 2)
  })
  expect_snapshot(error = TRUE, {
    vec_chop(1, list(1), indices = list(1))
  })
})

# vec_chop + compact_seq --------------------------------------------------

# `start` is 0-based

test_that("can chop base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  expect_identical(vec_chop_seq(lgl(1, 0, 1), start, size), list(lgl(0, 1)))
  expect_identical(vec_chop_seq(int(1, 2, 3), start, size), list(int(2, 3)))
  expect_identical(vec_chop_seq(dbl(1, 2, 3), start, size), list(dbl(2, 3)))
  expect_identical(vec_chop_seq(cpl(1, 2, 3), start, size), list(cpl(2, 3)))
  expect_identical(vec_chop_seq(chr("1", "2", "3"), start, size), list(chr("2", "3")))
  expect_identical(vec_chop_seq(raw2(1, 2, 3), start, size), list(raw2(2, 3)))
  expect_identical(vec_chop_seq(list(1, 2, 3), start, size), list(list(2, 3)))
})

test_that("can chop with a decreasing compact seq", {
  expect_equal(vec_chop_seq(int(1, 2, 3), 1L, 2L, FALSE), list(int(2, 1)))
})

test_that("can chop with multiple compact seqs", {
  start <- c(1L, 0L)
  size <- c(1L, 3L)

  expect_equal(
    vec_chop_seq(int(1, 2, 3), start, size),
    list(int(2), int(1, 2, 3))
  )
})

test_that("can chop S3 objects using the fallback method with compact seqs", {
  x <- factor(c("a", "b", "c", "d"))
  expect_equal(vec_chop_seq(x, 0L, 0L), list(vec_slice(x, integer())))
  expect_equal(vec_chop_seq(x, 0L, 1L), list(vec_slice(x, 1L)))
  expect_equal(vec_chop_seq(x, 2L, 2L), list(vec_slice(x, 3:4)))
})

test_that("data frames with 0 columns retain the right number of rows with compact seqs (#1722)", {
  x <- data_frame(.size = 4)

  out <- vec_chop_seq(x, starts = c(0L, 0L, 2L), sizes = c(0L, 2L, 1L))
  out <- map_int(out, vec_size)

  expect_identical(out, c(0L, 2L, 1L))
})

# list_unchop --------------------------------------------------------------

test_that("`x` must be a list", {
  expect_snapshot(error = TRUE, {
    list_unchop(1, indices = list(1))
  })
  expect_snapshot(error = TRUE, {
    list_unchop(1, indices = list(1), error_call = call("foo"), error_arg = "arg")
  })
  expect_snapshot(error = TRUE, {
    list_unchop(data.frame(x=1), indices = list(1))
  })
})

test_that("`indices` must be a list", {
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = 1)
  })
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = 1, error_call = call("foo"))
  })
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = data.frame(x=1))
  })
})

test_that("`indices` must be a list of integers", {
  expect_error(list_unchop(list(1), indices = list("x")), class = "vctrs_error_subscript_type")
  expect_error(list_unchop(list(1), indices = list(TRUE)), class = "vctrs_error_subscript_type")
  expect_error(list_unchop(list(1), indices = list(quote(name))), class = "vctrs_error_subscript_type")
})

test_that("`x` and `indices` must be lists of the same size", {
  expect_error(list_unchop(list(1, 2), indices = list(1)), "`x` and `indices` must be lists of the same size")
})

test_that("can unchop with an AsIs list (#1463)", {
  x <- I(list(1, 2))
  expect_identical(list_unchop(x), c(1, 2))
})

test_that("can unchop empty vectors", {
  expect_null(list_unchop(list()))
  expect_null(list_unchop(list(), indices = list()))
  expect_identical(list_unchop(list(), indices = list(), ptype = numeric()), numeric())
})

test_that("can unchop a list of NULL", {
  expect_null(list_unchop(list(NULL), indices = list(integer())))
  expect_identical(list_unchop(list(NULL), indices = list(integer()), ptype = numeric()), numeric())
  expect_identical(list_unchop(list(NULL, NULL), indices = list(integer(), integer()), ptype = numeric()), numeric())
})

test_that("NULLs are ignored when unchopped with other vectors", {
  expect_identical(list_unchop(list("a", NULL, "b")), c("a", "b"))
  expect_identical(list_unchop(list("a", NULL, "b"), indices = list(2, integer(), 1)), c("b", "a"))
})

test_that("can use a `NULL` element with a corresponding index", {
  # FIXME: Probably not quite right, but not entirely clear what it should be:
  # - Maybe `unspecified(2)`?
  # - Or should `NULL`s even be allowed in `list_unchop()`?
  expect_null(list_unchop(list(NULL), indices = list(1:2)))

  expect_identical(
    list_unchop(list(NULL), indices = list(1:2), ptype = integer()),
    c(NA_integer_, NA_integer_)
  )

  x <- list("a", NULL, c("b", "c"))
  indices <- list(3L, c(1L, 4L), c(2L, 5L))
  expect_identical(list_unchop(x, indices = indices), c(NA, "b", "a", NA, "c"))
})

test_that("can unchop atomic vectors", {
  expect_identical(list_unchop(list(1, 2), indices = list(2, 1)), c(2, 1))
  expect_identical(list_unchop(list("a", "b"), indices = list(2, 1)), c("b", "a"))
})

test_that("can unchop lists", {
  x <- list(list("a", "b"), list("c"))
  indices <- list(c(2, 3), 1)

  expect_identical(list_unchop(x, indices = indices), list("c", "a", "b"))
})

test_that("can unchop data frames", {
  df1 <- data_frame(x = 1:2)
  df2 <- data_frame(x = 3:4)

  x <- list(df1, df2)
  indices <- list(c(3, 1), c(2, 4))

  expect <- vec_slice(vec_c(df1, df2), vec_order(vec_c(!!! indices)))

  expect_identical(list_unchop(x, indices = indices), expect)
})

test_that("can unchop factors", {
  fctr1 <- factor("z")
  fctr2 <- factor(c("x", "y"))

  x <- list(fctr1, fctr2)
  indices <- list(2, c(3, 1))

  # levels are in the order they are seen!
  expect <- factor(c("y", "z", "x"), levels = c("z", "x", "y"))

  expect_identical(list_unchop(x, indices = indices), expect)
})

test_that("can fallback when unchopping matrices", {
  mat1 <- matrix(1:4, nrow = 2, ncol = 2)
  mat2 <- matrix(5:10, nrow = 3, ncol = 2)

  x <- list(mat1, mat2)
  indices <- list(c(4, 1), c(2, 3, 5))

  expect <- vec_slice(vec_c(mat1, mat2), vec_order(vec_c(!!! indices)))

  expect_identical(list_unchop(x, indices = indices), expect)
  expect_identical(list_unchop(x), vec_c(mat1, mat2))
})

test_that("can fallback when unchopping arrays of >2D", {
  arr1 <- array(1:8, c(2, 2, 2))
  arr2 <- matrix(9:10, c(1, 2))

  x <- list(arr1, arr2)
  indices <- list(c(3, 1), 2)

  expect <- vec_slice(vec_c(arr1, arr2), vec_order(vec_c(!!! indices)))

  expect_identical(list_unchop(x, indices = indices), expect)
  expect_identical(list_unchop(x), vec_c(arr1, arr2))
})

test_that("can unchop with all size 0 elements and get the right ptype", {
  x <- list(integer(), integer())
  indices <- list(integer(), integer())
  expect_identical(list_unchop(x, indices = indices), integer())
})

test_that("can unchop with some size 0 elements", {
  x <- list(integer(), 1:2, integer())
  indices <- list(integer(), 2:1, integer())
  expect_identical(list_unchop(x, indices = indices), 2:1)
})

test_that("NULL is a valid index", {
  expect_equal(list_unchop(list(1, 2), indices = list(NULL, 1)), 2)
  expect_error(list_unchop(list(1, 2), indices = list(NULL, 2)), class = "vctrs_error_subscript_oob")
})

test_that("unchopping recycles elements of x to the size of the index", {
  x <- list(1, 2)
  indices <- list(c(3, 4, 5), c(2, 1))
  expect_identical(list_unchop(x, indices = indices), c(2, 2, 1, 1, 1))

  x <- list(1:2)
  indices <- list(1:3)
  expect_snapshot({
    (expect_error(list_unchop(x, indices = indices)))
    (expect_error(list_unchop(x, indices = indices, error_call = call("foo"), error_arg = "arg")))
  })
})

test_that("unchopping takes the common type", {
  x <- list(1, "a")
  indices <- list(1, 2)

  expect_snapshot({
    (expect_error(list_unchop(x, indices = indices), class = "vctrs_error_incompatible_type"))
    (expect_error(list_unchop(x, indices = indices, error_call = call("foo"), error_arg = "arg"), class = "vctrs_error_incompatible_type"))
  })

  x <- list(1, 2L)

  expect_type(list_unchop(x, indices = indices), "double")
})

test_that("common type failure uses positional errors", {
  expect_snapshot({
    x <- list(1, a = "x", 2)

    # Looking for `x[[1]]` and `x$a`
    (expect_error(list_unchop(x)))
    (expect_error(list_unchop(x, indices = list(2, 1, 3))))

    # Directed cast should also produce directional errors (#1690)
    (expect_error(list_unchop(x, ptype = double())))
    (expect_error(list_unchop(x, indices = list(2, 1, 3), ptype = double())))

    # Lossy cast
    y <- list(1, a = 2.5)
    (expect_error(list_unchop(y, ptype = integer())))
    (expect_error(list_unchop(y, indices = list(2, 1), ptype = integer())))
  })
})

test_that("can specify a ptype to override common type", {
  indices <- list(1, 2)

  x <- list(1, 2L)
  expect_identical(list_unchop(x, indices = indices, ptype = integer()), c(1L, 2L))

  x <- list(1.5, 2)
  expect_snapshot({
    (expect_error(list_unchop(x, indices = indices, ptype = integer())))
    (expect_error(list_unchop(x, indices = indices, ptype = integer(), error_call = call("foo"), error_arg = "arg")))
  })
})

test_that("leaving `indices = NULL` unchops sequentially", {
  x <- list(1:2, 3:5, 6L)
  expect_identical(list_unchop(x), 1:6)
})

test_that("outer names are kept", {
  x <- list(x = 1, y = 2)
  expect_named(list_unchop(x), c("x", "y"))
  expect_named(list_unchop(x, indices = list(2, 1)), c("y", "x"))
})

test_that("outer names are recycled in the right order", {
  x <- list(x = 1, y = 2)
  expect_error(list_unchop(x, indices = list(c(1, 2), 3)), "Can't merge")
  expect_named(list_unchop(x, indices = list(c(1, 3), 2), name_spec = "{outer}_{inner}"), c("x_1", "y", "x_2"))
  expect_named(list_unchop(x, indices = list(c(3, 1), 2), name_spec = "{outer}_{inner}"), c("x_2", "y", "x_1"))
})

test_that("outer names can be merged with inner names", {
  x <- list(x = c(a = 1), y = c(b = 2))
  expect_error(list_unchop(x), "Can't merge")
  expect_named(list_unchop(x, name_spec = "{outer}_{inner}"), c("x_a", "y_b"))
  expect_named(list_unchop(x, indices = list(2, 1), name_spec = "{outer}_{inner}"), c("y_b", "x_a"))
})

test_that("preserves names when inputs are cast to a common type (#1689)", {
  expect_named(list_unchop(list(c(a = 1)), ptype = integer()), "a")
  expect_named(list_unchop(list(c(a = 1)), ptype = integer(), indices = list(1)), "a")

  # With name spec
  name_spec <- "{outer}_{inner}"
  expect_named(list_unchop(list(foo = c(a = 1)), ptype = integer(), name_spec = name_spec), "foo_a")
  expect_named(list_unchop(list(foo = c(a = 1)), ptype = integer(), name_spec = name_spec, indices = list(1)), "foo_a")

  # When `x` elements are recycled, names are also recycled
  x <- list(c(a = 1), c(b = 2))
  indices <- list(1:2, 3:4)
  expect_named(list_unchop(x, indices = indices, ptype = integer()), c("a", "a", "b", "b"))
})

test_that("not all inputs have to be named", {
  x <- list(c(a = 1), 2, c(c = 3))
  indices <- list(2, 1, 3)

  expect_named(list_unchop(x, indices = indices), c("", "a", "c"))
})

test_that("list_unchop() keeps data frame row names", {
  df1 <- data.frame(x = 1:2, row.names = c("r1", "r2"))
  df2 <- data.frame(x = 3:4, row.names = c("r3", "r4"))

  x <- list(df1, df2)
  indices <- list(c(3, 1), c(2, 4))

  result <- list_unchop(x, indices = indices)
  expect <- c("r2", "r3", "r1", "r4")

  expect_identical(vec_names(result), expect)
})

test_that("individual data frame columns retain vector names", {
  df1 <- data_frame(x = c(a = 1, b = 2))
  df2 <- data_frame(x = c(c = 3))

  x <- list(df1, df2)
  indices <- list(c(1, 2), 3)

  result <- list_unchop(x, indices = indices)

  expect_named(result$x, c("a", "b", "c"))

  # Names should be identical to equivalent `vec_c()` call
  expect_identical(list_unchop(x, indices = indices), vec_c(!!!x))
})

test_that("df-col row names are repaired silently", {
  df1 <- data_frame(x = new_data_frame(list(a = 1), row.names = "inner"))
  df2 <- data_frame(x = new_data_frame(list(a = 2), row.names = "inner"))

  x <- list(df1, df2)
  indices <- list(1, 2)

  expect_silent({
    result <- list_unchop(x, indices = indices)
  })

  expect_identical(vec_names(result$x), c("inner...1", "inner...2"))
})

test_that("monitoring - can technically assign to the same location twice", {
  x <- list(1:2, 3L)
  indices <- list(1:2, 1L)

  expect_identical(list_unchop(x, indices = indices), c(3L, 2L, NA))
})

test_that("index values are validated", {
  x <- list(1, 2)
  indices1 <- list(4, 1)
  indices2 <- list(c(1, 4), 2)
  indices3 <- list(c(1, 3, 4), 2)

  expect_error(list_unchop(x, indices = indices1), class = "vctrs_error_subscript_oob")
  expect_error(list_unchop(x, indices = indices2), class = "vctrs_error_subscript_oob")

  expect_identical(list_unchop(x, indices = indices3), c(1, 2, 1, 1))
})

test_that("name repair is respected and happens after ordering according to `indices`", {
  local_name_repair_quiet()

  x <- list(c(a = 1), c(a = 2))
  indices <- list(2, 1)

  expect_named(list_unchop(x, indices = indices), c("a", "a"))
  expect_named(list_unchop(x, indices = indices, name_repair = "unique"), c("a...1", "a...2"))
})

test_that("list_unchop() can repair names quietly", {
  local_name_repair_verbose()

  x <- c(x = "a", x = "b", x = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_unchop(vec_chop(x, indices = indices), indices = indices, name_repair = "unique_quiet")
  })
  expect_named(res, c("x...1", "x...2", "x...3"))

  x <- c("if" = "a", "in" = "b", "for" = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_unchop(vec_chop(x, indices = indices), indices = indices, name_repair = "universal_quiet")
  })
  expect_named(res, c(".if", ".in", ".for"))
})

test_that("list_unchop() errors on unsupported location values", {
  expect_snapshot({
    (expect_error(
      list_unchop(list(1, 2), indices = list(c(1, 2), 0)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      list_unchop(list(1), indices = list(-1)),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("missing values propagate", {
  expect_identical(
    list_unchop(list(1, 2), indices = list(c(NA_integer_, NA_integer_), c(NA_integer_, 3))),
    c(NA, NA, 2, NA)
  )
})

test_that("list_unchop() works with simple homogeneous foreign S3 classes", {
  expect_identical(list_unchop(list(foobar(1), foobar(2))), vec_c(foobar(c(1, 2))))
})

test_that("list_unchop() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    (expect_error(list_unchop(list(x, y)), class = "vctrs_error_incompatible_type"))
    (expect_error(list_unchop(list(x, y), error_call = call("foo"), error_arg = "arg"), class = "vctrs_error_incompatible_type"))
  })
})

test_that("list_unchop() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(c(1L, 2L), name = "Joe")
    jane <- .Counts(3L, name = "Jane")
    (expect_error(list_unchop(list(joe, jane)), class = "vctrs_error_incompatible_type"))
    (expect_error(list_unchop(list(joe, jane), error_call = call("foo"), error_arg = "arg"), class = "vctrs_error_incompatible_type"))
  })
})

test_that("list_unchop() falls back to c() if S3 method is available", {
  # Check off-by-one error
  expect_error(
    list_unchop(list(foobar(1), "", foobar(2)), indices = list(1, 2, 3)),
    class = "vctrs_error_incompatible_type"
  )

  # Fallback when the class implements `c()`
  method_foobar <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    foobar(res)
  }

  local_methods(
    c.vctrs_foobar = method_foobar
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2))),
    foobar(c(1, 2))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(1, 2)),
    foobar(c(1, 2))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(2, 1)),
    foobar(c(2, 1))
  )
  expect_identical(
    list_unchop(list(NULL, foobar(1), NULL, foobar(2))),
    foobar(c(1, 2))
  )

  # OOB error is respected
  expect_error(
    list_unchop(list(foobar(1), foobar(2)), indices = list(1, 3)),
    class = "vctrs_error_subscript_oob"
  )

  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_unchop(list(foobar(c(1, 2)), foobar(3)), indices = list(c(1, 3), 1)),
    foobar(c(3, NA, 2))
  )
  expect_identical(
    list_unchop(list(foobar(c(1, 2)), foobar(3)), indices = list(c(2, NA), NA)),
    foobar(c(NA, 1, NA))
  )

  # Names are kept
  expect_identical(
    list_unchop(list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))), indices = list(c(2, 1), 3)),
    foobar(c(y = 2, x = 1, x = 1))
  )

  # Recycles to the size of index
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 3), 2)),
    foobar(c(1, 2, 1))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), integer())),
    foobar(c(1, 1))
  )
  expect_snapshot({
    (expect_error(
      list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 3), integer())),
      class = "vctrs_error_subscript_oob"
    ))
  })
  expect_snapshot({
    x <- list(foobar(1:2))
    indices <- list(1:3)
    (expect_error(list_unchop(x, indices = indices)))
    (expect_error(list_unchop(x, indices = indices, error_arg = "arg", error_call = call("foo"))))
  })

  method_vctrs_c_fallback <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    structure(res, class = "vctrs_c_fallback")
  }

  # Registered fallback
  s3_register("base::c", "vctrs_c_fallback", method_vctrs_c_fallback)
  expect_identical(
    list_unchop(
      list(
        structure(1, class = "vctrs_c_fallback"),
        structure(2, class = "vctrs_c_fallback")
      ),
      indices = list(2, 1)
    ),
    structure(c(2, 1), class = "vctrs_c_fallback")
  )

  # Don't fallback for S3 lists which are treated as scalars by default
  expect_error(
    list_unchop(list(foobar(list(1)), foobar(list(2)))),
    class = "vctrs_error_scalar_type"
  )
})

test_that("list_unchop() falls back for S4 classes with a registered c() method", {
  joe <- .Counts(c(1L, 2L), name = "Joe")
  jane <- .Counts(3L, name = "Jane")

  expect_snapshot({
    (expect_error(
      list_unchop(list(joe, 1, jane), indices = list(c(1, 2), 3, 4)),
      class = "vctrs_error_incompatible_type"
    ))
  })

  local_c_counts()

  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(1, 3), 2)),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )

  expect_identical(
    list_unchop(list(NULL, joe, jane), indices = list(integer(), c(1, 3), 2)),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )

  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(1, 3), 1)),
    .Counts(c(3L, NA, 2L), name = "Dispatched")
  )
  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(2, NA), NA)),
    .Counts(c(NA, 1L, NA), name = "Dispatched")
  )
})

test_that("list_unchop() fallback doesn't support `name_spec` or `ptype`", {
  expect_snapshot({
    foo <- structure(foobar(1), foo = "foo")
    bar <- structure(foobar(2), bar = "bar")
    (expect_error(
      with_c_foobar(list_unchop(list(foo, bar), name_spec = "{outer}_{inner}")),
      "name specification"
    ))
    # With error call
    (expect_error(
      with_c_foobar(list_unchop(list(foo, bar), name_spec = "{outer}_{inner}", error_call = call("foo"))),
      "name specification"
    ))
    # Used to be an error about `ptype`
    x <- list(foobar(1))
    (expect_error(
      with_c_foobar(list_unchop(x, ptype = "")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_unchop() supports numeric S3 indices", {
  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) UseMethod("vec_ptype2.vctrs_foobar"),
    vec_ptype2.vctrs_foobar.integer = function(x, y, ...) foobar(integer()),
    vec_cast.integer.vctrs_foobar = function(x, to, ...) vec_data(x)
  )

  expect_identical(list_unchop(list(1), indices = list(foobar(1L))), 1)
})

test_that("list_unchop() does not support non-numeric S3 indices", {
  expect_snapshot({
    (expect_error(
      list_unchop(list(1), indices = list(factor("x"))),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      list_unchop(list(1), indices = list(foobar(1L))),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can ignore names in `list_unchop()` by providing a `zap()` name-spec (#232)", {
  expect_snapshot({
    (expect_error(list_unchop(list(a = c(b = 1:2)))))
    (expect_error(list_unchop(list(a = c(b = 1:2)), error_call = call("foo"))))
  })

  expect_identical(
    list_unchop(list(a = c(b = 1:2), b = 3L), name_spec = zap()),
    1:3
  )
  expect_identical(
    list_unchop(
      list(a = c(foo = 1:2), b = c(bar = 3L)),
      indices = list(2:1, 3),
      name_spec = zap()
    ),
    c(2L, 1L, 3L)
  )

  expect_snapshot({
    x <- list(a = c(b = letters), b = 3L)
    (expect_error(
      list_unchop(x, name_spec = zap()),
      class = "vctrs_error_incompatible_type"
    ))

    x <- list(a = c(foo = 1:2), b = c(bar = ""))
    (expect_error(
      list_unchop(
        x,
        indices = list(2:1, 3),
        name_spec = zap()
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_unchop() falls back to c() methods (#1120)", {
  expect_error(
    list_unchop(list(foobar(1), foobar(2, class = "foo"))),
    class = "vctrs_error_incompatible_type"
  )

  local_methods(
    c.vctrs_foobar = function(...) {
      out <- NextMethod()
      paste0(rep_along(out, "dispatched"), seq_along(out))
    }
  )

  # Homogeneous subclasses
  xs <- list(foobar(1), foobar(2, class = "foo"))

  expect_identical(
    list_unchop(xs),
    c("dispatched1", "dispatched2")
  )
  expect_identical(
    list_unchop(xs, indices = list(2, 1)),
    c("dispatched2", "dispatched1")
  )

  # Different subclasses
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )

  expect_identical(
    list_unchop(xs),
    c("dispatched1", "dispatched2", "dispatched3")
  )
  expect_identical(
    list_unchop(xs, indices = list(c(2, 1), 3)),
    c("dispatched2", "dispatched1", "dispatched3")
  )
})

test_that("list_unchop() fails if foreign classes are not homogeneous and there is no c() method", {
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )
  expect_error(
    list_unchop(xs),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    list_unchop(xs, indices = list(c(2, 1), 3)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("tibble beats data frame", {
  df <- new_data_frame()
  dt <- tibble::tibble()

  expect_s3_class(vec_ptype_common(dt, df), "tbl_df")
  expect_s3_class(vec_ptype_common(df, dt), "tbl_df")
})

test_that("can cast tibble to df and vice versa", {
  df <- new_data_frame()
  tib <- tibble::tibble()

  expect_identical(vec_cast(df, tib), tib)
  expect_identical(vec_cast(tib, df), df)
  expect_identical(vec_cast(tib, tib), tib)
})

test_that("can't cast vector to tibble", {
  dt <- tibble::tibble()
  v <- logical()

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(vec_ptype2(v, dt), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_ptype2(dt, v), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_cast(v, dt), class = "vctrs_error_incompatible_type"))
  })
})

test_that("casting to and from tibble preserves row names", {
  out <- vec_cast(mtcars, tibble::as_tibble(mtcars))
  expect_identical(row.names(out), row.names(mtcars))

  out <- vec_cast(out, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))
})

test_that("no common type between list and tibble", {
  dt <- tibble::tibble()
  l <- list()

  expect_error(vec_ptype2(l, dt), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(dt, l), class = "vctrs_error_incompatible_type")
})

test_that("vec_restore restores tibbles", {
  df1 <- tibble::tibble(x = 1:4)
  df2 <- vec_restore(vec_data(df1), df1)

  expect_s3_class(df2, "tbl_df")
})

test_that("the type of a tibble with an unspecified column retains unspecifiedness", {
  df1 <- tibble::tibble(x = 1, y = NA)
  df2 <- tibble::tibble(x = 1, y = unspecified(1))
  expect <- tibble::tibble(x = numeric(), y = unspecified())

  expect_identical(vec_ptype(df1), expect)
  expect_identical(vec_ptype(df2), expect)
})

test_that("vec_ptype_finalise() works recursively over tibbles", {
  df <- tibble(x = numeric(), y = unspecified())
  expect <- tibble(x = numeric(), y = logical())

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() can handle tibble df columns", {
  df <- tibble(x = numeric(), y = tibble(z = unspecified()))
  expect <- tibble(x = numeric(), y = tibble(z = logical()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("can use ptype2 and cast with tibble that has incorrect class vector", {
  tib1 <- structure(data.frame(x = 1), class = c("tbl_df", "data.frame"))
  tib2 <- structure(data.frame(y = 2), class = c("tbl_df", "data.frame"))
  exp <- structure(data.frame(x = dbl(), y = dbl()), class = c("tbl_df", "data.frame"))

  requireNamespace("tibble")

  expect_identical(
    vec_ptype_common(tib1, tib2),
    exp
  )
  expect_identical(
    vec_ptype_common(tib1, data.frame(y = 2)),
    tibble::new_tibble(exp, nrow = nrow(exp))
  )
  expect_identical(
    vec_ptype_common(data.frame(x = 1), tib2),
    tibble::new_tibble(exp, nrow = nrow(exp))
  )
  expect_identical(
    vec_cast(tib1, tib1),
    tib1
  )

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(
      vec_cast(tib1, tib2),
      class = "vctrs_error_cast"
    ))
    (expect_error(
      vec_cast(tib1, data.frame(y = 2)),
      class = "vctrs_error_cast"
    ))
    (expect_error(
      vec_cast(data.frame(x = 1), tib2),
      class = "vctrs_error_cast"
    ))
  })
})

test_that("vec_as_subscript() coerces unspecified vectors", {
  expect_identical(
    vec_as_subscript(NA),
    NA
  )
  expect_identical(
    vec_as_subscript(NA, logical = "error"),
    na_int
  )
  expect_identical(
    vec_as_subscript(NA, logical = "error", numeric = "error"),
    na_chr
  )
})

test_that("vec_as_subscript() coerces subtypes and supertypes", {
  expect_identical(vec_as_subscript(factor("foo")), "foo")

  with_lgl_subtype({
    expect_identical(vec_as_subscript(new_lgl_subtype(TRUE)), TRUE)
  })
  with_lgl_supertype({
    expect_identical(vec_as_subscript(new_lgl_supertype(TRUE)), TRUE)
  })
})

test_that("vec_as_subscript() handles NULL", {
  expect_identical(vec_as_subscript(NULL), int())
  expect_error(
    vec_as_subscript(NULL, numeric = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("vec_as_subscript() handles symbols", {
  expect_identical(vec_as_subscript(quote(foo)), "foo")
  expect_identical(vec_as_subscript(quote(`<U+5E78>`)), "\u5e78")
  expect_error(
    vec_as_subscript(quote(foo), character = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("can customise subscript errors", {
  expect_snapshot({
    (expect_error(
      with_tibble_cols(vec_as_subscript(env())),
      class = "vctrs_error_subscript_type"
    ))
  })

  expect_snapshot({
    (expect_error(
      with_dm_tables(vec_as_subscript(env())),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_subscript() checks dimensionality", {
  expect_snapshot({
    (expect_error(
      vec_as_subscript(matrix(TRUE, nrow = 1)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_subscript(array(TRUE, dim = c(1, 1, 1))),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_rows(vec_as_subscript(matrix(TRUE, nrow = 1))),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_subscript() works with vectors of dimensionality 1", {
  arr <- array(TRUE, dim = 1)
  expect_identical(vec_as_subscript(arr), arr)
})

test_that("vec_as_subscript() forbids subscript types", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript(1L, logical = "error", numeric = "error")
  )
  expect_snapshot(
    error = TRUE,
    vec_as_subscript("foo", logical = "error", character = "error")
  )
  expect_snapshot(error = TRUE, vec_as_subscript(TRUE, logical = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript("foo", character = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript(NULL, numeric = "error"))
  expect_snapshot(
    error = TRUE,
    vec_as_subscript(quote(foo), character = "error")
  )
})

test_that("vec_as_subscript2() forbids subscript types", {
  expect_snapshot(error = TRUE, vec_as_subscript2(1L, numeric = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript2("foo", character = "error"))
  expect_snapshot(error = TRUE, vec_as_subscript2(TRUE))
})

test_that("vec_as_subscript2() retains the call when throwing vec_as_subscript() errors (#1605)", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(1L, numeric = "error", call = call("foo"))
  )
  expect_snapshot(error = TRUE, vec_as_subscript2(1.5, call = call("foo")))
})

test_that("vec_as_subscript2() retains the call when erroring on logical input (#1605)", {
  expect_snapshot(error = TRUE, vec_as_subscript2(TRUE, call = call("foo")))
})

test_that("vec_as_subscript() evaluates arg lazily", {
  expect_silent(vec_as_subscript(1L, arg = print("oof")))
  expect_silent(vec_as_subscript_result(
    1L,
    arg = print("oof"),
    NULL,
    logical = "error",
    numeric = "cast",
    character = "error"
  ))
})

test_that("vec_as_subscript2() evaluates arg lazily", {
  expect_silent(vec_as_subscript2(1L, arg = print("oof")))
  expect_silent(vec_as_subscript2_result(
    1L,
    arg = print("oof"),
    NULL,
    numeric = "cast",
    character = "error"
  ))
})

test_that("`logical = 'cast'` is deprecated", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(TRUE, logical = "cast")
  )

  # `logical = "error"` still works
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(TRUE, logical = "error")
  )
})

test_that("lossy cast errors for scalar subscripts work (#1606)", {
  expect_snapshot(
    error = TRUE,
    vec_as_subscript2(1.5)
  )
})

# printing ----------------------------------------------------------------

test_that("data frames print nicely", {
  expect_equal(vec_ptype_abbr(mtcars), "df[,11]")
  expect_snapshot(vec_ptype_show(mtcars))
  expect_snapshot(vec_ptype_show(iris))
})

test_that("embedded data frames print nicely", {
  df <- data.frame(x = 1:3)
  df$a <- data.frame(a = 1:3, b = letters[1:3])
  df$b <- list_of(1, 2, 3)
  df$c <- as_list_of(split(data.frame(x = 1:3, y = letters[1:3]), 1:3))

  expect_snapshot(vec_ptype_show(df))
})

# coercing ----------------------------------------------------------------

test_that("data frame only combines with other data frames or NULL", {
  dt <- data.frame(x = 1)
  expect_equal(vec_ptype_common(dt, NULL), vec_ptype(dt))
  expect_error(
    vec_ptype_common(dt, 1:10),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("data frame takes max of individual variables", {
  dt1 <- data.frame(x = FALSE, y = 1L)
  dt2 <- data.frame(x = 1.5, y = 1.5)

  expect_equal(vec_ptype_common(dt1, dt2), vec_ptype_common(dt2))
})

test_that("data frame combines variables", {
  dt1 <- data.frame(x = 1)
  dt2 <- data.frame(y = 1)

  dt3 <- max(dt1, dt2)
  expect_equal(
    vec_ptype_common(dt1, dt2),
    vec_ptype_common(data.frame(x = double(), y = double()))
  )
})

test_that("empty data frame still has names", {
  df <- data.frame()
  out <- vec_ptype_common(df, df)

  expect_equal(names(out), character())
})

test_that("combining data frames with foreign classes uses fallback", {
  foo <- foobar(data.frame())
  df <- data.frame()

  expect_identical(vec_ptype_common(foo, foo, foo), foo)
  expect_identical(vec_ptype_common(foo, foo, df, foo), df)

  expect_identical(vec_ptype2(foo, df), data.frame())
  expect_identical(vec_ptype2(df, foo), data.frame())
  expect_identical(vec_ptype_common(foo, df), df)
  expect_identical(vec_ptype_common(df, foo), df)

  cnds <- list()
  withCallingHandlers(
    warning = function(cnd) {
      cnds <<- append(cnds, list(cnd))
      invokeRestart("muffleWarning")
    },
    expect_identical(
      vec_ptype_common(foo, df, foo, foo),
      df
    )
  )

  # There are no longer any warnings
  expect_length(cnds, 0)

  expect_equal(
    vec_cbind(foobar(data.frame(x = 1)), data.frame(y = 2)),
    data.frame(x = 1, y = 2)
  )
  expect_equal(
    vec_rbind(foo, data.frame(), foo),
    df
  )
})


# casting -----------------------------------------------------------------

test_that("safe casts work as expected", {
  df <- data.frame(x = 1, y = 0)

  expect_equal(vec_cast(NULL, df), NULL)
  expect_equal(vec_cast(df, df), df)

  expect_equal(vec_cast(data.frame(x = TRUE, y = FALSE), df), df)
})

test_that("warn about lossy coercions", {
  df1 <- data.frame(x = factor("foo"), y = 1)
  df2 <- data.frame(x = factor("bar"))

  expect_lossy(vec_cast(df1, df1[1]), df1[1], x = df1, to = df1[1])

  expect_lossy(
    vec_cast(df1[1], df2),
    data.frame(x = factor(NA, levels = "bar")),
    x = factor("foo"),
    to = factor("bar")
  )

  out <-
    allow_lossy_cast(
      allow_lossy_cast(
        vec_cast(df1, df2),
        factor("foo"),
        factor("bar")
      ),
      df1,
      df2
    )

  expect_identical(out, data.frame(x = factor(NA, levels = "bar")))
})

test_that("invalid cast generates error", {
  expect_error(
    vec_cast(1L, data.frame()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("column order matches type", {
  df1 <- data.frame(x = 1, y = "a")
  df2 <- data.frame(x = TRUE, z = 3)

  df3 <- vec_cast(df2, vec_ptype_common(df1, df2))
  expect_named(df3, c("x", "y", "z"))
})

test_that("restore generates correct row/col names", {
  df1 <- data.frame(x = NA, y = 1:4, z = 1:4)
  df1$x <- data.frame(a = 1:4, b = 1:4)

  df2 <- vec_restore(lapply(df1[1:3], vec_slice, 1:2), df1)

  expect_named(df2, c("x", "y", "z"))
  expect_equal(.row_names_info(df2), -2)
})

test_that("restore keeps automatic row/col names", {
  df1 <- data.frame(x = NA, y = 1:4, z = 1:4)
  df1$x <- data.frame(a = 1:4, b = 1:4)

  df2 <- vec_restore(df1, df1)

  expect_named(df2, c("x", "y", "z"))
  expect_equal(.row_names_info(df2), -4)
})

test_that("cast to empty data frame preserves number of rows", {
  out <- vec_cast(new_data_frame(n = 10L), new_data_frame())
  expect_equal(nrow(out), 10L)
})

test_that("can cast unspecified to data frame", {
  df <- data.frame(x = 1, y = 2L)
  expect_identical(vec_cast(unspecified(3), df), vec_init(df, 3))
})

test_that("cannot cast list to data frame", {
  df <- data.frame(x = 1, y = 2L)
  expect_error(
    vec_cast(list(df, df), df),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can restore lists with empty names", {
  expect_identical(vec_restore(list(), data.frame()), data.frame())
})

test_that("can restore subclasses of data frames", {
  expect_identical(
    vec_restore(list(), subclass(data.frame())),
    subclass(data.frame())
  )
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ..., i) "dispatched"
  )
  expect_identical(vec_restore(list(), subclass(data.frame())), "dispatched")
})

test_that("df_cast() checks for names", {
  x <- new_data_frame(list(1))
  y <- new_data_frame(list(2))
  expect_error(vec_cast_common(x, y), "must have names")
})

test_that("casting to and from data frame preserves row names", {
  out <- vec_cast(mtcars, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))

  out <- vec_cast(out, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))
})

test_that("df_cast() evaluates arg lazily", {
  expect_silent(df_cast(data_frame(), data_frame(), x_arg = print("oof")))
  expect_silent(df_cast(data_frame(), data_frame(), to_arg = print("oof")))
})

# df_ptype2 ---------------------------------------------------------------

test_that("df_ptype2() evaluates arg lazily", {
  expect_silent(df_ptype2(data_frame(), data_frame(), x_arg = print("oof")))
  expect_silent(df_ptype2(data_frame(), data_frame(), y_arg = print("oof")))
})


# new_data_frame ----------------------------------------------------------

test_that("can construct an empty data frame", {
  expect_identical(new_data_frame(), data.frame())
})

test_that("can validly set the number of rows when there are no columns", {
  expect <- structure(
    list(),
    class = "data.frame",
    row.names = .set_row_names(2L),
    names = character()
  )

  expect_identical(new_data_frame(n = 2L), expect)
})

test_that("can add additional classes", {
  expect_s3_class(new_data_frame(class = "foobar"), "foobar")
  expect_s3_class(new_data_frame(class = c("foo", "bar")), c("foo", "bar"))
})

test_that("can add additional attributes", {
  expect <- data.frame()
  attr(expect, "foo") <- "bar"
  attr(expect, "a") <- "b"

  expect_identical(new_data_frame(foo = "bar", a = "b"), expect)
})

test_that("size is pulled from first column if not supplied", {
  x <- new_data_frame(list(x = 1:5, y = 1:6))
  expect_identical(.row_names_info(x, type = 1), -5L)
})

test_that("can construct a data frame without column names", {
  expect_named(new_data_frame(list(1, 2)), NULL)
})

test_that("the names on an empty data frame are an empty character vector", {
  expect_identical(names(new_data_frame()), character())
})

test_that("class attribute", {
  expect_identical(
    class(new_data_frame(list(a = 1))),
    "data.frame"
  )
  expect_identical(
    class(new_data_frame(list(a = 1), class = "tbl_df")),
    c("tbl_df", "data.frame")
  )
  expect_identical(
    class(new_data_frame(
      list(a = 1),
      class = c("tbl_df", "tbl", "data.frame")
    )),
    c("tbl_df", "tbl", "data.frame", "data.frame")
  )
  expect_identical(
    class(new_data_frame(list(a = 1), class = "foo_frame")),
    c("foo_frame", "data.frame")
  )
  expect_identical(
    class(exec(
      new_data_frame,
      list(a = 1),
      !!!attributes(new_data_frame(list(), class = "tbl_df"))
    )),
    c("tbl_df", "data.frame", "data.frame")
  )
  expect_identical(
    class(exec(
      new_data_frame,
      list(a = 1),
      !!!attributes(new_data_frame(list(b = 1), class = "tbl_df"))
    )),
    c("tbl_df", "data.frame", "data.frame")
  )
})

test_that("attributes with special names are merged", {
  expect_identical(
    names(new_data_frame(list(a = 1))),
    "a"
  )

  expect_identical(
    names(new_data_frame(list(a = 1), names = "name")),
    "name"
  )

  expect_identical(
    names(new_data_frame(list(1), names = "name")),
    "name"
  )

  expect_identical(
    attr(new_data_frame(list()), "row.names"),
    integer()
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L)),
    -3L
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L, row.names = 1:3)),
    3L
  )

  expect_identical(
    .row_names_info(new_data_frame(list(), n = 3L, row.names = c(NA, -3L))),
    -3L
  )

  expect_identical(
    attr(new_data_frame(list(), n = 1L, row.names = "rowname"), "row.names"),
    "rowname"
  )
})

test_that("n and row.names (#894)", {
  # Can omit n if row.names attribute is given
  expect_identical(
    row.names(new_data_frame(list(), row.names = "rowname")),
    "rowname"
  )
  expect_identical(
    attr(new_data_frame(list(), row.names = 2L), "row.names"),
    2L
  )
  expect_identical(
    row.names(new_data_frame(list(), row.names = chr())),
    chr()
  )
})

test_that("`row.names` completely overrides `n` and the implied size of `x`, even if incompatible (tidyverse/dplyr#6596)", {
  row_names <- c(NA, -3L)

  df <- new_data_frame(list(), n = 2L, row.names = row_names)
  expect_identical(.row_names_info(df, type = 0L), row_names)

  df <- new_data_frame(list(x = 1:2), row.names = row_names)
  expect_identical(.row_names_info(df, type = 0L), row_names)
})

test_that("ALTREP `row.names` are not materialized by `new_data_frame()` (tidyverse/dplyr#6596)", {
  # We are careful in `new_data_frame()` to not call the `Dataptr()` or
  # `Length()` ALTREP methods, both of which would materialize our lazy
  # character here
  row_names <- new_lazy_character(~ c("a", "b"))

  x <- new_data_frame(list(), row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))

  x <- new_data_frame(list(x = 1:2), row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))

  x <- new_data_frame(list(), n = 2L, row.names = row_names)
  expect_false(lazy_character_is_materialized(.row_names_info(x, type = 0L)))
})

test_that("`x` must be a list", {
  expect_snapshot(
    (expect_error(
      new_data_frame(1),
      "`x` must be a list"
    ))
  )
})

test_that("if supplied, `n` must be an integer of size 1", {
  expect_snapshot({
    (expect_error(
      new_data_frame(n = c(1L, 2L)),
      "must be an integer of size 1"
    ))
    (expect_error(new_data_frame(n = "x"), "must be an integer of size 1"))
  })
})

test_that("if supplied, `n` can't be negative or missing (#1477)", {
  expect_snapshot({
    (expect_error(new_data_frame(n = -1L)))
    (expect_error(new_data_frame(n = NA_integer_)))
  })
})

test_that("`class` must be a character vector", {
  expect_snapshot(
    (expect_error(
      new_data_frame(class = 1),
      "must be NULL or a character vector"
    ))
  )
})

test_that("flatten info is computed", {
  df_flatten_info <- function(x) {
    .Call(ffi_df_flatten_info, x)
  }
  expect_identical(df_flatten_info(mtcars), list(FALSE, ncol(mtcars)))

  df <- tibble(x = 1, y = tibble(x = 2, y = tibble(x = 3), z = 4), z = 5)
  expect_identical(df_flatten_info(df), list(TRUE, 5L))
})

test_that("can flatten data frames", {
  df_flatten <- function(x) {
    .Call(ffi_df_flatten, x)
  }
  expect_identical(df_flatten(mtcars), mtcars)

  df <- tibble(x = 1, y = tibble(x = 2, y = tibble(x = 3), z = 4), z = 5)
  expect_identical(
    df_flatten(df),
    new_data_frame(list(x = 1, x = 2, x = 3, z = 4, z = 5))
  )
})

test_that("can flatten data frames with rcrd columns (#1318)", {
  col <- new_rcrd(list(a = 1))
  df <- data_frame(col = col, y = 1)
  expect_identical(vec_proxy_equal(df), data_frame(col = 1, y = 1))

  col <- new_rcrd(list(a = 1, b = 2))
  df <- data_frame(col = col, y = 1)
  expect_identical(vec_proxy_equal(df), data_frame(a = 1, b = 2, y = 1))
})

test_that("new_data_frame() zaps existing attributes", {
  struct <- structure(list(), foo = 1)
  expect_identical(
    attributes(new_data_frame(struct)),
    attributes(new_data_frame(list())),
  )
  expect_identical(
    attributes(new_data_frame(struct, bar = 2)),
    attributes(new_data_frame(list(), bar = 2)),
  )
})

test_that("with zero columns, implied size is 0", {
  # Empty data frame input
  df <- new_data_frame(n = 10L)

  # Input to `new_data_frame()` is treated as a list.
  # Existing `row.names` are ignored, even if they might provide
  # information in the case of 0 columns.
  expect_identical(nrow(new_data_frame(df)), 0L)
})

# data_frame --------------------------------------------------------------

test_that("data_frame() and df_list() report error context", {
  expect_snapshot({
    (expect_error(data_frame(a = 1, a = 1)))
    (expect_error(data_frame(a = 1, a = 1, .error_call = call("foo"))))
    (expect_error(data_frame(a = 1:2, b = int())))
    (expect_error(data_frame(a = 1:2, b = int(), .error_call = call("foo"))))

    (expect_error(df_list(a = 1, a = 1)))
    (expect_error(df_list(a = 1, a = 1, .error_call = call("foo"))))
    (expect_error(df_list(a = 1:2, b = int())))
    (expect_error(df_list(a = 1:2, b = int(), .error_call = call("foo"))))
  })
})

test_that("can construct data frames with empty input", {
  expect_identical(data_frame(), new_data_frame())
  expect_named(data_frame(), character())
})

test_that("input is tidy recycled", {
  expect_identical(
    data_frame(x = 1, y = 1:3),
    data_frame(x = c(1, 1, 1), y = 1:3)
  )

  expect_identical(
    data_frame(x = 1, y = integer()),
    data_frame(x = double(), y = integer())
  )

  expect_snapshot({
    expect_error(data_frame(1:2, 1:3), class = "vctrs_error_incompatible_size")
  })
})

test_that("dots are dynamic", {
  list_2_data_frame <- function(x) data_frame(!!!x)

  expect_identical(
    list_2_data_frame(list(x = 1, y = 2)),
    data_frame(x = 1, y = 2)
  )
})

test_that("unnamed input is auto named with empty strings", {
  expect_named(data_frame(1, 2, .name_repair = "minimal"), c("", ""))
})

test_that("unnamed data frames are auto unpacked", {
  expect_identical(
    data_frame(w = 1, data_frame(x = 2, y = 3), z = 4),
    data_frame(w = 1, x = 2, y = 3, z = 4)
  )
})

test_that("named data frames are not unpacked", {
  df_col <- data_frame(x = 2, y = 3)
  df <- data_frame(w = 1, col = data_frame(x = 2, y = 3), z = 4)

  expect_identical(df$col, df_col)
})

test_that("unpacked data frames without names are caught", {
  df_col <- new_data_frame(list(1))
  expect_error(data_frame(df_col), "corrupt data frame")
})

test_that("unpacking in `df_list()` can be disabled with `.unpack = FALSE`", {
  out <- df_list(
    w = 1,
    data_frame(x = 2, y = 3),
    z = 4,
    .unpack = FALSE,
    .name_repair = "minimal"
  )

  expect <- list(
    w = 1,
    data_frame(x = 2, y = 3),
    z = 4
  )

  expect_identical(out, expect)
})

test_that("`.unpack` is validated", {
  expect_snapshot(error = TRUE, {
    df_list(.unpack = 1)
  })
  expect_snapshot(error = TRUE, {
    df_list(.unpack = c(TRUE, FALSE))
  })
})

test_that("`NULL` inputs are dropped", {
  expect_identical(data_frame(NULL, x = 1, NULL), data_frame(x = 1))
})

test_that("`NULL` inputs are dropped before name repair", {
  expect_identical(
    data_frame(x = NULL, x = 1, .name_repair = "check_unique"),
    data_frame(x = 1)
  )
})

test_that("`.size` can force a desired size", {
  df <- data_frame(x = 1, .size = 5)
  expect_identical(df$x, rep(1, 5))

  expect_size(data_frame(.size = 5), 5L)
})

test_that("`.name_repair` repairs names", {
  expect_message(res <- data_frame(x = 1, x = 1, .name_repair = "unique"))
  expect_named(res, c("x...1", "x...2"))
})

test_that("`.name_repair` happens after auto-naming with empty strings", {
  expect_message(res <- data_frame(1, 2, .name_repair = "unique"))
  expect_named(res, c("...1", "...2"))
})

test_that("`.name_repair` happens after splicing", {
  expect_message(
    res <- data_frame(x = 1, data_frame(x = 2), .name_repair = "unique")
  )
  expect_named(res, c("x...1", "x...2"))
})

test_that("`.name_repair` can be quiet", {
  local_name_repair_verbose()

  expect_snapshot({
    dfl_unique <- df_list(1, 2, .name_repair = "unique_quiet")
    dfl_universal <- df_list(
      "if" = 1,
      "in" = 2,
      .name_repair = "universal_quiet"
    )
    df_unique <- data_frame(1, 2, .name_repair = "unique_quiet")
    df_universal <- data_frame(
      "if" = 1,
      "in" = 2,
      .name_repair = "universal_quiet"
    )
  })

  expect_named(dfl_unique, c("...1", "...2"))
  expect_named(dfl_universal, c(".if", ".in"))
  expect_named(df_unique, c("...1", "...2"))
  expect_named(df_universal, c(".if", ".in"))
})

# fallback ----------------------------------------------------------------

test_that("data frame fallback handles column types (#999)", {
  df1 <- foobar(data.frame(x = 1))
  df2 <- foobar(data.frame(x = 1, y = 2))
  df3 <- foobar(data.frame(x = "", y = 2))

  common <- foobar(data.frame(x = dbl(), y = dbl()))
  expect_identical(vec_ptype2(df1, df2), common)
  expect_identical(vec_ptype2(df2, df1), common)

  exp <- foobar(data_frame(x = 1, y = na_dbl))
  expect_identical(vec_cast(df1, df2), exp)
  expect_identical(vec_cast(set_tibble(df1), set_tibble(df2)), set_tibble(exp))

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(
      vec_ptype2(df1, df3),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_ptype2(df3, df1),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_cast(df2, df1),
      class = "vctrs_error_incompatible_type"
    ))
  })

  expect_identical(
    vec_rbind(df1, df2),
    foobar(data.frame(x = c(1, 1), y = c(NA, 2)))
  )

  # Attributes are not restored
  df1_attrib <- foobar(df1, foo = "foo")
  df2_attrib <- foobar(df2, bar = "bar")
  exp <- data.frame(x = c(1, 1), y = c(NA, 2))
  expect_equal(
    vec_rbind(df1_attrib, df2_attrib),
    exp
  )

  out <- with_methods(
    `[.vctrs_foobar` = function(x, i, ...) {
      new_data_frame(
        NextMethod(),
        dispatched = TRUE,
        class = "vctrs_foobar"
      )
    },
    vec_rbind(df1_attrib, df2_attrib)
  )
  expect_identical(out, exp)
})

test_that("falls back to tibble for tibble subclasses (#1025)", {
  foo <- foobar(tibble::as_tibble(mtcars))
  expect_s3_class(vec_rbind(foo, mtcars), "tbl_df")
  expect_s3_class(vec_rbind(foo, mtcars, mtcars), "tbl_df")
  expect_s3_class(vec_rbind(foo, mtcars, foobar(mtcars)), "tbl_df")
})

test_that("fallback is recursive", {
  df <- mtcars[1:3, 1, drop = FALSE]

  foo <- new_data_frame(list(x = foobar(df, foo = TRUE)))
  bar <- new_data_frame(list(x = foobar(df, bar = TRUE)))
  baz <- new_data_frame(list(y = 1:3, x = foobar(df, bar = TRUE)))

  exp <- new_data_frame(list(x = vec_rbind(df, df)))
  expect_equal(vec_rbind(foo, bar), exp)

  exp <- new_data_frame(list(x = vec_rbind(df, df), y = c(NA, NA, NA, 1:3)))
  expect_equal(vec_rbind(foo, baz), exp)
})

local_name_repair_quiet()

test_that("zero length input returns NULL", {
  expect_equal(vec_c(), NULL)
  expect_equal(vec_c(NULL), NULL)
  expect_equal(vec_c(NULL, ), NULL)
  expect_equal(vec_c(NULL, NULL), NULL)
})

test_that("NULL is idempotent", {
  expect_equal(vec_c(NULL, 1L), 1L)
  expect_equal(vec_c(1L, NULL), 1L)
})

test_that("NA is idempotent", {
  expect_equal(vec_c(NA, 1L), c(NA, 1L))
  expect_equal(vec_c(NA, "x"), c(NA, "x"))
  expect_equal(vec_c(NA, factor("x")), factor(c(NA, "x")))
  expect_equal(vec_c(NA, new_date(0)), new_date(c(NA, 0)))
  expect_equal(vec_c(NA, new_datetime(0)), new_datetime(c(NA, 0)))
  expect_equal(vec_c(NA, new_duration(0)), new_duration(c(NA, 0)))
})

test_that("NA is logical if no other types intervene", {
  expect_equal(vec_c(logical()), logical())
  expect_equal(vec_c(NA), NA)
  expect_equal(vec_c(NA, NA), c(NA, NA))
})


test_that("different types are coerced to common", {
  expect_equal(vec_c(TRUE, 1L, 1), c(1, 1, 1))
  expect_equal(vec_c(TRUE, 2:4), 1:4)
})

test_that("specified .ptypes do not allow more casts", {
  expect_error(
    vec_c(TRUE, .ptype = character()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("common type failure uses error call and error arg (#1641, #1692)", {
  expect_snapshot(error = TRUE, {
    vec_c("x", 1, .error_call = call("foo"), .error_arg = "arg")
  })
  expect_snapshot(error = TRUE, {
    vec_c(
      "x",
      .ptype = integer(),
      .error_call = call("foo"),
      .error_arg = "arg"
    )
  })
})

test_that("common type failure uses positional errors", {
  expect_snapshot({
    # Looking for `..1` and `a`
    (expect_error(vec_c(1, a = "x", 2)))

    # Directed cast should also produce positional errors (#1690)
    (expect_error(vec_c(1, a = "x", 2, .ptype = double(), .error_arg = "arg")))

    # Lossy cast
    (expect_error(vec_c(1, a = 2.5, .ptype = integer())))
  })
})

test_that("combines outer an inner names", {
  expect_equal(vec_c(x = 1), c(x = 1))
  expect_equal(vec_c(c(x = 1)), c(x = 1))

  expect_equal(vec_c(c(x = 1:2)), c(x1 = 1, x2 = 2))
  expect_error(vec_c(y = c(x = 1)), "Please supply")
})

test_that("can bind data.frame columns", {
  df <- data.frame(x = NA, y = 1:2)
  df$x <- data.frame(a = 1:2)

  expected <- data.frame(x = NA, y = c(1:2, 1:2))
  expected$x <- data.frame(a = c(1:2, 1:2))

  expect_equal(vec_c(df, df), expected)
})

test_that("vec_c() handles matrices", {
  m <- matrix(1:4, nrow = 2)
  dimnames(m) <- list(c("foo", "bar"), c("baz", "quux"))

  # FIXME: `vec_ptype_common(m, m)` doesn't return dimension names
  exp <- matrix(c(1:2, 1:2, 3:4, 3:4), nrow = 4)
  rownames(exp) <- c("foo", "bar", "foo", "bar")

  expect_identical(vec_c(m, m), exp)

  expect_error(vec_c(outer = m), "Please supply")
})

test_that("vec_c() includes index in argument tag", {
  df1 <- tibble(x = tibble(y = tibble(z = 1)))
  df2 <- tibble(x = tibble(y = tibble(z = "a")))

  expect_snapshot(error = TRUE, vec_c(df1, df2))
  expect_snapshot(error = TRUE, vec_c(df1, df1, df2))
  expect_snapshot(error = TRUE, vec_c(foo = df1, bar = df2))
})

test_that("vec_c() handles record classes", {
  local_rational_class()

  out <- vec_c(rational(1, 2), 1L, NA)

  expect_true(vec_is(out, rational(1, 2)))
  expect_size(out, 3)
  expect_identical(
    vec_proxy(out),
    data.frame(n = c(1L, 1L, NA), d = c(2L, 1L, NA))
  )
})

test_that("can mix named and unnamed vectors (#271)", {
  expect_identical(vec_c(c(a = 1), 2), c(a = 1, 2))
  expect_identical(vec_c(0, c(a = 1), 2, b = 3), c(0, a = 1, 2, b = 3))
})

test_that("preserves names when inputs are cast to a common type (#1690)", {
  expect_named(vec_c(c(a = 1), .ptype = integer()), "a")
  expect_named(
    vec_c(foo = c(a = 1), .ptype = integer(), .name_spec = "{outer}_{inner}"),
    "foo_a"
  )
  expect_named(
    vec_c(foo = c(a = 1), .ptype = integer(), .name_spec = "inner"),
    "a"
  )
})

test_that("vec_c() repairs names", {
  local_name_repair_quiet()

  # Default minimal repair
  expect_named(vec_c(a = 1, a = 2, `_` = 3), c("a", "a", "_"))
  out <- vec_c(!!!set_names(1, NA))
  expect_named(out, "")

  expect_named(
    vec_c(a = 1, a = 2, `_` = 3, .name_repair = "unique"),
    c("a...1", "a...2", "_")
  )
  expect_error(
    vec_c(a = 1, a = 2, `_` = 3, .name_repair = "check_unique"),
    class = "vctrs_error_names_must_be_unique"
  )

  expect_named(
    vec_c(a = 1, a = 2, `_` = 3, .name_repair = "universal"),
    c("a...1", "a...2", "._")
  )

  expect_named(vec_c(a = 1, a = 2, .name_repair = ~ toupper(.)), c("A", "A"))
})

test_that("vec_c() can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_c(x = TRUE, x = 0, .name_repair = "unique_quiet")
    res_universal <- vec_c(
      "if" = TRUE,
      "in" = 0,
      .name_repair = "universal_quiet"
    )
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in"))
})

test_that("vec_c() doesn't use outer names for data frames (#524)", {
  x <- data.frame(inner = 1)
  expect_equal(vec_c(outer = x), x)

  a <- data.frame(x = 1L)
  b <- data.frame(x = 2L)
  expect_equal(vec_c(foo = a, bar = b), data.frame(x = 1:2))
})

test_that("vec_c() preserves row names and inner names", {
  x <- data.frame(a = 1, row.names = "r1")
  y <- data.frame(a = 2, row.names = "r2")
  expect_equal(rownames(vec_c(x, y)), c("r1", "r2"))
  expect_equal(rownames(vec_c(x, x)), c("r1...1", "r1...2"))

  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  oo_x <- set_names(
    as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")),
    letters[1:3]
  )
  oo_y <- as.POSIXlt(c(FOO = "2020-01-04"))
  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "d")
  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("d"))
  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x, oo = oo_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y, oo = oo_y),
    row.names = c("quux")
  )

  nested_out <- vec_c(nested_x, nested_y)
  expect_identical(row.names(nested_out), c("foo", "bar", "baz", "quux"))
  expect_identical(row.names(nested_out$df), c("a", "b", "c", "d"))
  expect_identical(row.names(nested_out$mat), c("a", "b", "c", "d"))
  expect_identical(names(nested_out$vec), c("a", "b", "c", "FOO"))
  expect_identical(names(nested_out$oo), c("a", "b", "c", "FOO"))
})

test_that("vec_c() outer names work with proxied objects", {
  x <- as.POSIXlt(new_datetime(0))
  exp <- set_names(x, "outer")
  expect_equal(vec_c(outer = x), exp)

  named_x <- set_names(x, "inner")
  exp <- set_names(named_x, "outer_inner")
  expect_error(vec_c(outer = named_x), "Please supply")
  expect_equal(vec_c(outer = named_x, .name_spec = "{outer}_{inner}"), exp)

  xs <- as.POSIXlt(new_datetime(c(0, 1)))
  exp <- set_names(xs, c("outer_1", "outer_2"))
  expect_error(vec_c(outer = xs), "Please supply")
  expect_equal(vec_c(outer = xs, .name_spec = "{outer}_{inner}"), exp)
})

test_that("vec_c() works with simple homogeneous foreign S3 classes", {
  expect_identical(vec_c(foobar(1), foobar(2)), vec_c(foobar(c(1, 2))))
  expect_identical(vec_c(NULL, foobar(1), foobar(2)), vec_c(foobar(c(1, 2))))
})

test_that("vec_c() works with simple homogeneous foreign S4 classes", {
  joe1 <- .Counts(c(1L, 2L), name = "Joe")
  joe2 <- .Counts(3L, name = "Joe")
  expect_identical(vec_c(joe1, joe2), .Counts(1:3, name = "Joe"))
})

test_that("vec_c() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    (expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type"))
    (expect_error(
      vec_c(x, y, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_c() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(c(1L, 2L), name = "Joe")
    jane <- .Counts(3L, name = "Jane")
    (expect_error(vec_c(joe, jane), class = "vctrs_error_incompatible_type"))
    (expect_error(
      vec_c(joe, jane, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_c() falls back to c() if S3 method is available", {
  # Check off-by-one error
  expect_error(
    vec_c(foobar(1), "", foobar(2)),
    class = "vctrs_error_incompatible_type"
  )

  # Fallback when the class implements `c()`
  method <- function(...) rep_along(list(...), "dispatched")
  local_methods(
    c.vctrs_foobar = method
  )
  expect_identical(
    vec_c(foobar(1), foobar(2, class = "foo")),
    c("dispatched", "dispatched")
  )
  expect_identical(
    vec_c(NULL, foobar(1), NULL, foobar(2, class = "foo")),
    c("dispatched", "dispatched")
  )

  # Registered fallback
  s3_register("base::c", "vctrs_c_fallback", method)
  expect_identical(
    vec_c(
      structure(1, class = "vctrs_c_fallback"),
      structure(2, class = "vctrs_c_fallback")
    ),
    c("dispatched", "dispatched")
  )

  # Don't fallback for S3 lists which are treated as scalars by default
  expect_error(
    vec_c(foobar(list(1)), foobar(list(2))),
    class = "vctrs_error_scalar_type"
  )
})

test_that("c() fallback is consistent", {
  dispatched <- function(x) structure(x, class = "dispatched")
  c_method <- function(...) dispatched(NextMethod())

  out <- with_methods(
    vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_ptype2(...)),
    vec_cast.vctrs_foobaz.vctrs_foobaz = function(...) foobaz(df_cast(...)),
    c.vctrs_foobar = c_method,
    list(
      direct = vec_c(foobar(1L), foobar(2L)),
      df = vec_c(data_frame(x = foobar(1L)), data_frame(x = foobar(2L))),
      tib = vec_c(tibble(x = foobar(1L)), tibble(x = foobar(2L))),
      foreign_df = vec_c(
        foobaz(data_frame(x = foobar(1L))),
        foobaz(data_frame(x = foobar(2L)))
      )
    )
  )
  expect_equal(out$direct, dispatched(1:2))
  expect_equal(out$df$x, dispatched(1:2))
  expect_equal(out$tib$x, dispatched(1:2))
  expect_equal(out$foreign_df$x, dispatched(1:2))

  # Hard case: generic record vectors
  my_rec_record <- function(x) {
    new_rcrd(list(x = x), class = "my_rec_record")
  }

  out <- with_methods(
    c.vctrs_foobar = c_method,
    vec_ptype2.my_rec_record.my_rec_record = function(x, y, ...) {
      my_rec_record(vec_ptype2(field(x, "x"), field(y, "x"), ...))
    },
    vec_cast.my_rec_record.my_rec_record = function(x, to, ...) {
      x
    },
    vec_c(
      data_frame(x = my_rec_record(foobar(1L))),
      data_frame(x = my_rec_record(foobar(2L)))
    )
  )
  expect_equal(field(out$x, "x"), dispatched(1:2))
})

test_that("vec_c() falls back to c() if S4 method is available", {
  joe1 <- .Counts(c(1L, 2L), name = "Joe")
  joe2 <- .Counts(3L, name = "Joe")

  c_counts <- function(x, ...) {
    xs <- list(x, ...)

    xs_data <- lapply(xs, function(x) x@.Data)
    new_data <- do.call(c, xs_data)

    .Counts(new_data, name = x@name)
  }

  local_s4_method("c", methods::signature(x = "vctrs_Counts"), c_counts)

  expect_identical(
    vec_c(joe1, joe2),
    .Counts(c(1L, 2L, 3L), name = "Joe")
  )

  expect_identical(
    vec_c(NULL, joe1, joe2),
    .Counts(c(1L, 2L, 3L), name = "Joe")
  )
})

test_that("vec_c() fallback doesn't support (most) `name_spec` or `ptype`", {
  expect_snapshot({
    (expect_error(
      with_c_foobar(vec_c(
        foobar(1),
        foobar(2),
        .name_spec = "{outer}_{inner}"
      )),
      "name specification"
    ))

    # Used to be an error about `ptype`
    (expect_error(
      with_c_foobar(vec_c(foobar(1), foobar(2), .ptype = "")),
      class = "vctrs_error_incompatible_type"
    ))

    # Uses error call (#1641)
    (expect_error(
      with_c_foobar(vec_c(
        foobar(1),
        foobar(2),
        .error_call = call("foo"),
        .name_spec = "{outer}_{inner}"
      ))
    ))
  })
})

test_that("vec_c() fallback does support `.name_spec = 'inner'`", {
  # Because of how useful it is, and how easy it is to implement!
  # It's extremely useful to be able to hardcode `.name_spec = "inner"`
  # and not have to worry about hitting a fallback type that wouldn't support
  # `name_spec`.
  expect_identical(
    with_c_foobar(vec_c(foobar(1), foobar(2), .name_spec = "inner")),
    foobar_c(c(1, 2))
  )
  expect_identical(
    with_c_foobar(vec_c(x = foobar(1), y = foobar(2), .name_spec = "inner")),
    foobar_c(c(1, 2))
  )
  expect_identical(
    with_c_foobar(vec_c(
      x = foobar(c(a = 1)),
      y = foobar(c(b = 2)),
      z = foobar(3),
      .name_spec = "inner"
    )),
    foobar_c(c(a = 1, b = 2, 3))
  )
})

test_that("vec_c() doesn't fall back when ptype2 is implemented", {
  new_quux <- function(x) structure(x, class = "vctrs_quux")

  with_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) new_quux(int()),
    vec_cast.vctrs_quux.vctrs_foobar = function(x, to, ...) new_quux(x),
    vec_restore.vctrs_quux = function(x, ...) new_quux(x),
    c.vctrs_foobar = function(...) foobar(NextMethod()),
    {
      expect_s3_class(c(foobar(1:3), foobar(4L)), "vctrs_foobar")
      expect_s3_class(vec_c(foobar(1:3), foobar(4L)), "vctrs_quux")
    }
  )
})

test_that("vec_c() falls back even when ptype is supplied", {
  expect_foobar(vec_c(foobar(1), foobar(2), .ptype = foobar(dbl())))

  with_methods(
    c.vctrs_foobar = function(...) quux(NextMethod()),
    {
      expect_quux(vec_c(foobar(1), foobar(2), .ptype = foobar(dbl())))
      expect_quux(vec_c(
        foobar(1, foo = TRUE),
        foobar(2, bar = TRUE),
        .ptype = foobar(dbl())
      ))
    }
  )
})

test_that("vec_implements_ptype2() is FALSE for scalars", {
  expect_false(vec_implements_ptype2(quote(foo)))
})

test_that("vec_implements_ptype2() and vec_c() fallback are compatible with old registration", {
  foo <- structure(NA, class = "vctrs_implements_ptype2_false")
  expect_false(vec_implements_ptype2(foo))

  vec_ptype2.vctrs_implements_ptype2_true <- function(...) NULL
  s3_register(
    "vctrs::vec_ptype2",
    "vctrs_implements_ptype2_true",
    vec_ptype2.vctrs_implements_ptype2_true
  )

  bar <- structure(NA, class = "vctrs_implements_ptype2_true")
  expect_true(vec_implements_ptype2(bar))

  local_methods(
    `c.vctrs_implements_ptype2_true` = function(...) stop("never called")
  )

  expect_identical(vec_c(bar), bar)
})

test_that("can ignore names in `vec_c()` by providing a `zap()` name-spec (#232)", {
  expect_error(vec_c(a = c(b = 1:2)))
  expect_identical(vec_c(a = c(b = 1:2), b = 3L, .name_spec = zap()), 1:3)

  expect_snapshot({
    (expect_error(
      vec_c(a = c(b = letters), b = 1, .name_spec = zap()),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("can ignore outer names in `vec_c()` by providing an 'inner' name-spec (#1988)", {
  expect_identical(
    vec_c(x = c(a = 1, 2), y = c(3, b = 4), .name_spec = "inner"),
    c(a = 1, 2, 3, b = 4)
  )

  # Importantly, outer names are still used in error messages!
  expect_snapshot(error = TRUE, {
    vec_c(x = c(a = 1), y = c(b = "2"), .name_spec = "inner")
  })
})

test_that("can concatenate subclasses of `vctrs_vctr` which don't have ptype2 methods", {
  x <- new_vctr(1, class = "vctrs_foo")
  expect_identical(vec_c(x, x), new_vctr(c(1, 1), class = "vctrs_foo"))
})

test_that("base c() fallback handles unspecified chunks", {
  local_methods(
    c.vctrs_foobar = function(...) {
      x <- NextMethod()

      # Should not be passed any unspecified chunks
      if (anyNA(x)) {
        abort("tilt")
      }

      foobar(x)
    },
    `[.vctrs_foobar` = function(x, i, ...) {
      # Return a quux to detect dispatch
      quux(NextMethod())
    }
  )

  out <- vec_c(foobar(1:2), rep(NA, 2))
  expect_identical(out, quux(c(1:2, NA, NA)))

  out <- vec_c(rep(NA, 2), foobar(1:2), NA)
  expect_identical(out, quux(c(NA, NA, 1:2, NA)))
})

test_that("can zap outer names from a name-spec (#1215)", {
  zap_outer_spec <- function(outer, inner) if (is_character(inner)) inner

  expect_null(
    names(vec_c(a = 1:2, .name_spec = zap_outer_spec))
  )
  expect_identical(
    names(vec_c(a = 1:2, c(foo = 3L), .name_spec = zap_outer_spec)),
    c("", "", "foo")
  )

  expect_null(
    names(list_unchop(
      list(a = 1:2),
      indices = list(1:2),
      name_spec = zap_outer_spec
    ))
  )
  expect_identical(
    names(list_unchop(
      list(a = 1:2, c(foo = 3L)),
      indices = list(1:2, 3),
      name_spec = zap_outer_spec
    )),
    c("", "", "foo")
  )

  # These days it is more efficient to use a name-spec of "inner" (#1988)
  expect_identical(
    vec_c(a = 1:2, .name_spec = zap_outer_spec),
    vec_c(a = 1:2, .name_spec = "inner")
  )
  expect_identical(
    vec_c(a = 1:2, c(foo = 3L), .name_spec = zap_outer_spec),
    vec_c(a = 1:2, c(foo = 3L), .name_spec = "inner")
  )
  expect_identical(
    list_unchop(list(a = 1:2), indices = list(1:2), name_spec = zap_outer_spec),
    list_unchop(list(a = 1:2), indices = list(1:2), name_spec = "inner")
  )
  expect_identical(
    list_unchop(
      list(a = 1:2, c(foo = 3L)),
      indices = list(1:2, 3),
      name_spec = zap_outer_spec
    ),
    list_unchop(
      list(a = 1:2, c(foo = 3L)),
      indices = list(1:2, 3),
      name_spec = "inner"
    )
  )
})

test_that("named empty vectors force named output (#1263)", {
  x <- set_names(int(), chr())

  expect_named(vec_c(x), chr())
  expect_named(vec_c(x, x), chr())
  expect_named(vec_c(x, 1L), "")
  expect_named(vec_c(x, 1), "")

  expect_named(list_unchop(list(x), indices = list(int())), chr())
  expect_named(list_unchop(list(x, x), indices = list(int(), int())), chr())
  expect_named(list_unchop(list(x, 1L), indices = list(int(), 1)), "")

  expect_named(list_unchop(list(x, 1), indices = list(int(), 1)), "")
})

test_that("calls cast method even with empty objects", {
  # https://github.com/paleolimbot/wk/issues/230

  # There is a common type, but the cast method is intended
  # to fail here for this test
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.default = function(x, to, ...) {
      vec_default_cast(x, to)
    }
  )

  expect_snapshot(error = TRUE, {
    vec_c(
      foobar(integer()),
      foobar(integer(), foo = "bar")
    )
  })
})

test_that("can handle spliced lists (#1578)", {
  expect_identical(
    vec_c(!!!list(foo = 1), !!!list(bar = 2)),
    c(foo = 1, bar = 2)
  )
})

# Golden tests -------------------------------------------------------

test_that("concatenation performs expected allocations", {
  vec_c_list <- function(x, ptype = NULL) {
    vec_c(!!!x, .ptype = ptype)
  }

  expect_snapshot({
    ints <- rep(list(1L), 1e2)
    dbls <- rep(list(1), 1e2)

    # Extra allocations from `list2()`, see r-lib/rlang#937
    "# `vec_c()` "
    "Integers"
    with_memory_prof(vec_c_list(ints))

    "Doubles"
    with_memory_prof(vec_c_list(dbls))

    "Integers to integer"
    with_memory_prof(vec_c_list(ints, ptype = int()))

    "Doubles to integer"
    with_memory_prof(vec_c_list(dbls, ptype = int()))

    "# `list_unchop()` "
    "Integers"
    with_memory_prof(list_unchop(ints))

    "Doubles"
    with_memory_prof(list_unchop(dbls))

    "Integers to integer"
    with_memory_prof(list_unchop(ints, ptype = int()))

    "Doubles to integer"
    with_memory_prof(list_unchop(dbls, ptype = int()))

    "# Concatenation with names"

    "Named integers"
    ints <- rep(list(set_names(1:3, letters[1:3])), 1e2)
    with_memory_prof(list_unchop(ints))

    "Named matrices"
    mat <- matrix(1:4, 2, dimnames = list(c("foo", "bar")))
    mats <- rep(list(mat), 1e2)
    with_memory_prof(list_unchop(mats))

    "Data frame with named columns"
    df <- data_frame(
      x = set_names(as.list(1:2), c("a", "b")),
      y = set_names(1:2, c("A", "B")),
      z = data_frame(Z = set_names(1:2, c("Za", "Zb")))
    )
    dfs <- rep(list(df), 1e2)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (non-repaired, non-recursive case)"
    df <- data_frame(x = 1:2)
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (repaired, non-recursive case)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (non-repaired, recursive case) (#1217)"
    df <- data_frame(
      x = 1:2,
      y = data_frame(x = 1:2)
    )
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "Data frame with rownames (repaired, recursive case) (#1217)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(list_unchop(dfs))

    "list-ofs (#1496)"
    make_list_of <- function(n) {
      df <- tibble::tibble(
        x = new_list_of(vec_chop(1:n), ptype = integer())
      )
      vec_chop(df)
    }
    with_memory_prof(list_unchop(make_list_of(1e3)))
    with_memory_prof(list_unchop(make_list_of(2e3)))
    with_memory_prof(list_unchop(make_list_of(4e3)))
  })
})

test_that("can dispatch many times", {
  # This caused a crash when counters were not correctly protected
  foo <- structure(
    list(x.sorted = numeric(0), tp = numeric(0), fp = numeric(0)),
    row.names = integer(0),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame")
  )
  x <- lapply(1:200, function(...) foo)
  expect_error(NA, object = vctrs::list_unchop(x))
})

test_that("dots splicing clones as appropriate", {
  x <- list(a = 1)
  vctrs::vec_cbind(!!!x)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_rbind(!!!x)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_c(!!!x)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_cbind(!!!x, 2)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_rbind(!!!x, 2)
  expect_equal(x, list(a = 1))

  x <- list(a = 1)
  vctrs::vec_c(!!!x, 2)
  expect_equal(x, list(a = 1))
})

test_that("can combine records wrapped in data frames", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) {
      data_frame(x = unclass(x), y = seq_along(x))
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      foobar(x$x)
    }
  )

  x <- foobar(1:2)
  y <- foobar(3:4)

  expect_equal(
    vec_c(x, y),
    foobar(1:4)
  )

  expect_equal(
    list_unchop(list(x, y), indices = list(1:2, 3:4)),
    foobar(1:4)
  )

  expect_equal(
    vec_rbind(data_frame(x = x), data_frame(x = y)),
    data_frame(x = foobar(1:4))
  )
})

test_that("fallback works with subclasses of `vctrs_vctr`", {
  # Used to fail because of interaction between common class fallback
  # for `base::c()` and the `c()` method for `vctrs_vctr` that called
  # back into `vec_c()`.

  # Reprex for failure in the ricu package
  x <- new_rcrd(list(a = 1), class = "vctrs_foobar")
  expect_equal(
    vec_c(x, x, .name_spec = "{inner}"),
    new_rcrd(list(a = c(1, 1)), class = "vctrs_foobar")
  )

  # Reprex for failure in the groupr package
  x <- new_rcrd(list(a = 1), class = "vctrs_foobar")
  df <- data_frame(x = x)
  expect_equal(
    vec_rbind(df, data.frame()),
    df
  )
  expect_equal(
    vec_cast_common(df, data.frame()),
    list(df, data_frame(x = x[0]))
  )
})

local_name_repair_quiet()

# rows --------------------------------------------------------------------

test_that("empty inputs return an empty data frame", {
  expect_equal(vec_rbind(), data_frame())
  expect_equal(vec_rbind(NULL, NULL), data_frame())
})

test_that("vec_rbind(): NULL is idempotent", {
  df <- data_frame(x = 1)
  expect_equal(vec_rbind(df, NULL), df)
})

test_that("vec_rbind() output is tibble if any input is tibble", {
  df <- data_frame(x = 1)
  dt <- tibble::tibble(x = 1)

  expect_s3_class(vec_rbind(dt), "tbl_df")
  expect_s3_class(vec_rbind(dt, df), "tbl_df")
  expect_s3_class(vec_rbind(df, dt), "tbl_df")
})

test_that("type of column is common type of individual columns", {
  x_int <- data_frame(x = 1L)
  x_dbl <- data_frame(x = 2.5)

  expect_equal(vec_rbind(x_int, x_int), data_frame(x = c(1L, 1L)))
  expect_equal(vec_rbind(x_int, x_dbl), data_frame(x = c(1, 2.5)))
})

test_that("incompatible columns throws common type error", {
  x_int <- data_frame(x = 1L)
  x_chr <- data_frame(x = "a")

  expect_snapshot({
    (expect_error(
      vec_rbind(x_int, x_chr),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_rbind(x_int, x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_rbind(x_int, x_chr, .ptype = x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("result contains union of columns", {
  expect_named(
    vec_rbind(
      data_frame(x = 1),
      data_frame(y = 1)
    ),
    c("x", "y")
  )

  expect_named(
    vec_rbind(
      data_frame(y = 1, x = 1),
      data_frame(y = 1, z = 2)
    ),
    c("y", "x", "z")
  )
})

test_that("all inputs coerced to data frames", {
  expect_equal(
    vec_rbind(data_frame(x = 1L), c(x = 1.5)),
    data_frame(x = c(1, 1.5))
  )
})

test_that("names are supplied if needed", {
  local_name_repair_verbose()
  expect_snapshot(out <- vec_rbind(data_frame(...1 = 1), 1))
  expect_equal(out, data_frame(...1 = c(1, 1)))
})

test_that("matrix becomes data frame and has names properly repaired", {
  x <- matrix(1:4, nrow = 2)
  expect_equal(vec_rbind(x), data.frame(...1 = 1:2, ...2 = 3:4))
})

test_that("can bind data.frame columns", {
  df <- data.frame(x = NA, y = 1:2)
  df$x <- data.frame(a = 1:2)

  expected <- data.frame(x = NA, y = c(1:2, 1:2))
  expected$x <- data.frame(a = c(1:2, 1:2))

  expect_equal(vec_rbind(df, df), expected)
})

test_that("can rbind unspecified vectors", {
  expect_identical(vec_rbind(NA), data_frame(...1 = NA))
  expect_identical(vec_rbind(NA, NA), data_frame(...1 = lgl(NA, NA)))

  df <- data.frame(x = 1)
  expect_identical(vec_rbind(NA, df), data.frame(x = c(NA, 1)))
  expect_identical(vec_rbind(df, NA), data.frame(x = c(1, NA)))
  expect_identical(vec_rbind(NA, df, NA), data.frame(x = c(NA, 1, NA)))
  expect_identical(
    vec_rbind(c(x = NA), data.frame(x = 1)),
    data.frame(x = c(NA, 1))
  )
  expect_identical(
    vec_rbind(c(y = NA), df),
    data.frame(y = c(NA, NA), x = c(NA, 1))
  )

  out <- suppressMessages(vec_rbind(c(x = NA, x = NA), df))
  exp <- data.frame(x...1 = c(NA, NA), x...2 = c(NA, NA), x = c(NA, 1))
  expect_identical(out, exp)
})

test_that("as_df_row() tidies the names of unspecified vectors", {
  expect_identical(as_df_row(c(NA, NA)), c(NA, NA))
  expect_identical(as_df_row(unspecified(2)), unspecified(2))
  expect_identical(
    as_df_row(c(a = NA, a = NA), quiet = TRUE),
    data.frame(a...1 = NA, a...2 = NA)
  )
  expect_identical(
    as_df_row(c(a = TRUE, a = TRUE), quiet = TRUE),
    data.frame(a...1 = TRUE, a...2 = TRUE)
  )
})

test_that("can rbind spliced lists", {
  data <- list(c(a = 1, b = 2), c(a = TRUE, b = FALSE))
  expect_identical(vec_rbind(!!!data), data_frame(a = c(1, 1), b = c(2, 0)))
})

test_that("can rbind list columns", {
  out <- vec_rbind(data_frame(x = list(1, 2)), data_frame(x = list(3)))
  expect_identical(out, data_frame(x = list(1, 2, 3)))
})

test_that("can rbind lists", {
  out <- vec_rbind(list(x = 1:2))
  expect_identical(out, data_frame(x = list(c(1L, 2L))))

  out <- vec_rbind(list(x = 1:2, y = 3L))
  expect_identical(out, data_frame(x = list(c(1L, 2L)), y = list(3L)))

  out <- vec_rbind(list(x = 1, y = 2), list(y = "string"))
  expect_identical(out, data_frame(x = list(1, NULL), y = list(2, "string")))
})

test_that("can rbind factors", {
  fctr <- factor(c("a", "b"))
  expect_equal(vec_rbind(fctr), data_frame(...1 = fctr[1], ...2 = fctr[2]))

  fctr_named <- set_names(fctr)
  expect_equal(vec_rbind(fctr_named), data_frame(a = fctr[1], b = fctr[2]))
})

test_that("can rbind dates", {
  date <- new_date(c(0, 1))
  expect_equal(vec_rbind(date), data_frame(...1 = date[1], ...2 = date[2]))

  date_named <- set_names(date, c("a", "b"))
  expect_equal(vec_rbind(date_named), data_frame(a = date[1], b = date[2]))
})

test_that("can rbind POSIXlt objects into POSIXct objects", {
  datetime <- as.POSIXlt(new_datetime(0))
  expect_s3_class(vec_rbind(datetime, datetime)[[1]], "POSIXct")

  datetime_named <- set_names(datetime, "col")
  expect_named(vec_rbind(datetime_named, datetime_named), "col")
})

test_that("can rbind table objects (#913)", {
  x <- new_table(1:4, c(2L, 2L))
  y <- x

  colnames <- c("c1", "c2")
  rownames <- c("r1", "r2", "r3", "r4")

  dimnames(x) <- list(rownames[1:2], colnames)
  dimnames(y) <- list(rownames[3:4], colnames)

  expect <- data.frame(c1 = c(1:2, 1:2), c2 = c(3:4, 3:4), row.names = rownames)

  expect_identical(vec_rbind(x, y), expect)
})

test_that("can rbind missing vectors", {
  expect_identical(vec_rbind(c(x = na_int)), data_frame(x = na_int))
  expect_identical(
    vec_rbind(c(x = na_int), c(x = na_int)),
    data_frame(x = int(na_int, na_int))
  )
})

test_that("vec_rbind() respects size invariants (#286)", {
  expect_identical(vec_rbind(), new_data_frame(n = 0L))

  expect_identical(vec_rbind(int(), int()), new_data_frame(n = 2L))
  expect_identical(
    vec_rbind(c(x = int()), c(x = TRUE)),
    new_data_frame(list(x = lgl(NA, TRUE)))
  )

  expect_identical(
    vec_rbind(int(), new_data_frame(n = 2L), int()),
    new_data_frame(n = 4L)
  )
})

test_that("can repair names in `vec_rbind()` (#229)", {
  expect_snapshot({
    (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
    (expect_error(
      vec_rbind(.name_repair = "minimal"),
      "can't be `\"minimal\"`"
    ))
    (expect_error(
      vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
  })

  expect_named(
    vec_rbind(list(a = 1, a = 2), .name_repair = "unique"),
    c("a...1", "a...2")
  )

  expect_named(vec_rbind(list(`_` = 1)), "_")
  expect_named(vec_rbind(list(`_` = 1), .name_repair = "universal"), c("._"))

  expect_named(
    vec_rbind(list(a = 1, a = 2), .name_repair = ~ toupper(.)),
    c("A", "A")
  )
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_rbind(
      c(x = 1, x = 2),
      c(x = 3, x = 4),
      .name_repair = "unique_quiet"
    )
    res_universal <- vec_rbind(
      c("if" = 1, "in" = 2),
      c("if" = 3, "for" = 4),
      .name_repair = "universal_quiet"
    )
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in", ".for"))
})

test_that("can construct an id column", {
  df <- data.frame(x = 1)

  expect_named(vec_rbind(df, df, .names_to = "id"), c("id", "x"))
  expect_equal(vec_rbind(df, df, .names_to = "id")$id, c(1L, 2L))

  expect_equal(vec_rbind(a = df, b = df, .names_to = "id")$id, c("a", "b"))

  expect_equal(vec_rbind(a = df, df, .names_to = "id")$id, c("a", ""))
})

test_that("vec_rbind() fails with arrays of dimensionality > 3", {
  expect_snapshot({
    (expect_error(vec_rbind(array(NA, c(1, 1, 1)))))
    (expect_error(vec_rbind(array(NA, c(1, 1, 1)), .error_call = call("foo"))))
  })
})

test_that("row names are preserved by vec_rbind()", {
  df1 <- mtcars[1:3, ]
  df2 <- mtcars[4:5, ]
  expect_identical(vec_rbind(df1, df2), mtcars[1:5, ])

  row.names(df2) <- NULL
  out <- mtcars[1:5, ]
  row.names(out) <- c(row.names(df1), "...4", "...5")
  expect_identical(vec_rbind(df1, df2), out)
})

test_that("can assign row names in vec_rbind()", {
  df1 <- mtcars[1:3, ]
  df2 <- mtcars[4:5, ]

  expect_snapshot({
    (expect_error(
      vec_rbind(
        foo = df1,
        df2,
        .names_to = NULL
      ),
      "specification"
    ))
  })

  # Combination
  out <- vec_rbind(
    foo = df1,
    df2,
    .names_to = NULL,
    .name_spec = "{outer}_{inner}"
  )
  exp <- mtcars[1:5, ]
  row.names(exp) <- c(paste0("foo_", row.names(df1)), row.names(df2))
  expect_identical(out, exp)

  out <- vec_rbind(foo = df1, df2, .names_to = "id")
  exp <- mtcars[1:5, ]
  exp <- vec_cbind(id = c(rep("foo", 3), rep("", 2)), exp)
  expect_identical(out, exp)

  # Sequence
  out <- vec_rbind(
    foo = unrownames(df1),
    df2,
    bar = unrownames(mtcars[6, ]),
    .names_to = NULL,
    .name_spec = "{outer}_{inner}"
  )
  exp <- mtcars[1:6, ]
  row.names(exp) <- c(paste0("foo_", 1:3), row.names(df2), "bar")
  expect_identical(out, exp)

  out <- vec_rbind(
    foo = unrownames(df1),
    df2,
    bar = unrownames(mtcars[6, ]),
    .names_to = "id"
  )
  exp <- mtcars[1:6, ]
  exp <- vec_cbind(id = c(rep("foo", 3), rep("", 2), "bar"), exp)
  row.names(exp) <- c(paste0("...", 1:3), row.names(df2), "...6")
  expect_identical(out, exp)
})

test_that("vec_rbind() takes the proxy and restores", {
  df <- foobar(data.frame(x = 1))

  # This data frame subclass has an identity proxy and the restore
  # method falls back to a bare data frame if `$x` has any missing values.
  # In `vec_rbind()`, the `vec_init()` call will create a bare data frame,
  # but at the end it is `vec_restore()`d to the right class.
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
      x
    },
    vec_proxy.vctrs_foobar = function(x, ...) {
      x
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      if (any(is.na(x$x))) {
        new_data_frame(x)
      } else {
        vec_restore_default(x, to)
      }
    }
  )

  expect_identical(
    vec_rbind(df, df),
    foobar(data.frame(x = c(1, 1)))
  )
})

test_that("vec_rbind() proxies before initializing", {
  df <- foobar(data.frame(x = 1))

  # This data frame subclass doesn't allow `NA`s in columns.
  # If initialization happened before proxying, it would try to
  # create `NA` rows with `vec_init()`.
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
      x
    },
    vec_proxy.vctrs_foobar = function(x, ...) {
      new_data_frame(x)
    },
    vec_restore.vctrs_foobar = function(x, to, ...) {
      if (any(is.na(x$x))) {
        abort("`x` can't have NA values.")
      }
      vec_restore_default(x, to)
    }
  )

  expect_identical(
    vec_rbind(df, df),
    foobar(data.frame(x = c(1, 1)))
  )
})

test_that("vec_rbind() requires a data frame proxy for data frame ptypes", {
  df <- foobar(data.frame(x = 1))

  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
    vec_proxy.vctrs_foobar = function(x, ...) 1
  )

  expect_error(
    vec_rbind(df, df),
    "Can't fill a data frame that doesn't have a data frame proxy"
  )
})

test_that("names of `...` are used for type and cast errors even when zapped", {
  xs <- list(a = data_frame(x = 1), b = data_frame(x = "2"))

  expect_snapshot(error = TRUE, {
    vec_rbind(!!!xs)
  })
  expect_snapshot(error = TRUE, {
    vec_rbind(!!!xs, .ptype = data_frame(x = double()))
  })
})

test_that("monitoring: name repair while rbinding doesn't modify in place", {
  df <- new_data_frame(list(x = 1, x = 1))
  expect <- new_data_frame(list(x = 1, x = 1))

  # Name repair occurs
  expect_named(vec_rbind(df), c("x...1", "x...2"))

  # No changes to `df`
  expect_identical(df, expect)
})

test_that("performance: Row binding with S3 columns doesn't duplicate on every assignment (#1151)", {
  skip_if_not_testing_performance()

  x <- as.Date("2000-01-01")
  x <- rep(x, 100)
  df <- data.frame(x = x)
  lst <- rep_len(list(df), 10000)

  expect_time_lt(vec_rbind(!!!lst), 5)
})

test_that("performance: Row binding with df-cols doesn't duplicate on every assignment (#1122)", {
  skip_if_not_testing_performance()

  df_col <- new_data_frame(list(x = 1:1000))
  df <- new_data_frame(list(y = df_col))

  lst <- rep_len(list(df), 10000)

  expect_time_lt(vec_rbind(!!!lst), 5)
})

# cols --------------------------------------------------------------------

test_that("vec_cbind() reports error context", {
  expect_snapshot({
    (expect_error(vec_cbind(foobar(list()))))
    (expect_error(vec_cbind(foobar(list()), .error_call = call("foo"))))

    (expect_error(vec_cbind(a = 1:2, b = int())))
    (expect_error(vec_cbind(a = 1:2, b = int(), .error_call = call("foo"))))
  })
})

test_that("empty inputs give data frame", {
  expect_equal(vec_cbind(), data_frame())
  expect_equal(vec_cbind(NULL), data_frame())
  expect_equal(vec_cbind(data.frame(a = 1), NULL), data_frame(a = 1))
})

test_that("number of rows is preserved with zero column data frames (#1281)", {
  df <- new_data_frame(n = 2L)
  expect_size(vec_cbind(df, df), 2L)
})

test_that("vec_cbind(): NULL is idempotent", {
  df <- data_frame(x = 1)
  expect_equal(vec_cbind(df, NULL), df)
})

test_that("outer names are respected", {
  expect_named(vec_cbind(x = 1, y = 4), c("x", "y"))
  expect_named(vec_cbind(a = data.frame(x = 1)), "a")
})

test_that("inner names are respected", {
  expect_named(vec_cbind(data_frame(x = 1), data_frame(y = 1)), c("x", "y"))
})

test_that("nameless vectors get tidy defaults", {
  expect_named(vec_cbind(1:2, 1), c("...1", "...2"))
})

test_that("matrix becomes data frame", {
  x <- matrix(1:4, nrow = 2)
  expect_equal(vec_cbind(x), data.frame(...1 = 1:2, ...2 = 3:4))

  # Packed if named
  expect_equal(vec_cbind(x = x), data_frame(x = x))
})

test_that("duplicate names are de-deduplicated", {
  local_name_repair_verbose()

  expect_snapshot({
    (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
    (expect_named(
      vec_cbind(data.frame(x = 1), data.frame(x = 1)),
      c("x...1", "x...2")
    ))
  })
})

test_that("rows recycled to longest", {
  df <- data.frame(x = 1:3)

  expect_dim(vec_cbind(df), c(3, 1))
  expect_dim(vec_cbind(df, NULL), c(3, 1))
  expect_dim(vec_cbind(df, y = 1), c(3, 2))
  expect_dim(vec_cbind(data.frame(x = 1), y = 1:3), c(3, 2))

  expect_dim(
    vec_cbind(
      data.frame(a = 1, b = 2),
      y = 1:3
    ),
    c(3, 3)
  )
})

test_that("vec_cbind() output is tibble if any input is tibble", {
  df <- data.frame(x = 1)
  dt <- tibble::tibble(y = 2)

  expect_s3_class(vec_cbind(dt), "tbl_df")
  expect_s3_class(vec_cbind(df, dt), "tbl_df")
  expect_s3_class(vec_cbind(dt, df), "tbl_df")
})

test_that("can override default .nrow", {
  expect_dim(vec_cbind(x = 1, .size = 3), c(3, 1))
})

test_that("can repair names in `vec_cbind()` (#227)", {
  expect_snapshot({
    (expect_error(
      vec_cbind(a = 1, a = 2, .name_repair = "none"),
      "can't be `\"none\"`"
    ))
    (expect_error(
      vec_cbind(a = 1, a = 2, .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
  })

  expect_named(
    vec_cbind(a = 1, a = 2, .name_repair = "unique"),
    c("a...1", "a...2")
  )

  expect_named(vec_cbind(`_` = 1, .name_repair = "universal"), "._")

  expect_named(vec_cbind(a = 1, a = 2, .name_repair = "minimal"), c("a", "a"))
  expect_named(vec_cbind(a = 1, a = 2, .name_repair = toupper), c("A", "A"))
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_cbind(x = 1, x = 2, .name_repair = "unique_quiet")
    res_universal <- vec_cbind(
      "if" = 1,
      "in" = 2,
      .name_repair = "universal_quiet"
    )
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in"))
})

test_that("can supply `.names_to` to `vec_rbind()` (#229)", {
  expect_snapshot({
    (expect_error(vec_rbind(data_frame(), .names_to = letters)))
    (expect_error(vec_rbind(data_frame(), .names_to = 10)))
    (expect_error(vec_rbind(
      data_frame(),
      .names_to = letters,
      .error_call = call("foo")
    )))
  })

  x <- data_frame(foo = 1:2, bar = 3:4)
  y <- data_frame(foo = 5L, bar = 6L)

  expect_identical(
    vec_rbind(a = x, b = y, .names_to = "quux"),
    data_frame(
      quux = c("a", "a", "b"),
      foo = c(1L, 2L, 5L),
      bar = c(3L, 4L, 6L)
    )
  )
  expect_identical(
    vec_rbind(a = x, b = y, .names_to = "foo"),
    data_frame(foo = c("a", "a", "b"), bar = c(3L, 4L, 6L))
  )

  # No names
  expect_identical(
    vec_rbind(x, y, .names_to = "quux"),
    data_frame(quux = c(1L, 1L, 2L), foo = c(1L, 2L, 5L), bar = c(3L, 4L, 6L))
  )
  expect_identical(
    vec_rbind(x, y, .names_to = "foo"),
    data_frame(foo = c(1L, 1L, 2L), bar = c(3L, 4L, 6L))
  )

  # Partial names
  expect_identical(vec_rbind(x, b = y, .names_to = "quux")$quux, c("", "", "b"))
})

test_that("can supply existing `.names_to`", {
  x <- data.frame(a = 1, id = TRUE)
  expect_identical(
    vec_rbind(foo = x, bar = c(a = 2), .names_to = "id"),
    data_frame(a = c(1, 2), id = c("foo", "bar"))
  )

  y <- data.frame(id = TRUE, a = 1)
  expect_identical(
    vec_rbind(foo = y, bar = c(a = 2), .names_to = "id"),
    data_frame(id = c("foo", "bar"), a = c(1, 2))
  )
})

test_that("vec_cbind() returns visibly (#452)", {
  # Shouldn't be needed once `check_unique` is implemented in C
  expect_visible(vctrs::vec_cbind(x = 1, .name_repair = "check_unique"))
})

test_that("vec_cbind() packs named data frames (#446)", {
  expect_identical(vec_cbind(data_frame(y = 1:3)), data_frame(y = 1:3))
  expect_identical(
    vec_cbind(x = data_frame(y = 1:3)),
    data_frame(x = data_frame(y = 1:3))
  )
})

test_that("vec_cbind() packs 1d arrays", {
  a <- array(1:2)
  expect_identical(vec_cbind(a), data_frame(...1 = 1:2))
  expect_identical(vec_cbind(x = a), data_frame(x = a))
})

test_that("vec_cbind() packs named matrices", {
  m <- matrix(1:4, 2)
  expect_identical(vec_cbind(m), data_frame(...1 = 1:2, ...2 = 3:4))
  expect_identical(vec_cbind(x = m), data_frame(x = m))
})

test_that("vec_cbind() never packs named vectors", {
  expect_identical(vec_cbind(1:2), data_frame(...1 = 1:2))
  expect_identical(vec_cbind(x = 1:2), data_frame(x = 1:2))
})

test_that("names are repaired late if unpacked", {
  df <- data_frame(b = 2, b = 3, .name_repair = "minimal")
  out1 <- vec_cbind(a = 1, df)
  out2 <- vec_cbind(a = 1, as.matrix(df))
  out3 <- vec_cbind(a = 1, matrix(1:2, nrow = 1))
  expect_named(out1, c("a", "b...2", "b...3"))
  expect_named(out2, c("a", "b...2", "b...3"))
  expect_named(out3, c("a", "...2", "...3"))
})

test_that("names are not repaired if packed", {
  df <- data_frame(b = 2, b = 3, .name_repair = "minimal")
  out1 <- vec_cbind(a = 1, packed = df)
  out2 <- vec_cbind(a = 1, packed = as.matrix(df))
  out3 <- vec_cbind(a = 1, packed = matrix(1:2, nrow = 1))

  expect_named(out1, c("a", "packed"))
  expect_named(out2, c("a", "packed"))
  expect_named(out3, c("a", "packed"))

  expect_named(out1$packed, c("b", "b"))
  expect_identical(colnames(out2$packed), c("b", "b"))
  expect_identical(colnames(out3$packed), NULL)
})

test_that("vec_cbind() fails with arrays of dimensionality > 3", {
  a <- array(NA, c(1, 1, 1))

  expect_snapshot({
    (expect_error(vec_cbind(a)))
    (expect_error(vec_cbind(a, .error_call = call("foo"))))
    (expect_error(vec_cbind(x = a)))
  })
})

test_that("monitoring: name repair while cbinding doesn't modify in place", {
  df <- new_data_frame(list(x = 1, x = 1))
  expect <- new_data_frame(list(x = 1, x = 1))

  # Name repair occurs
  expect_named(vec_cbind(df), c("x...1", "x...2"))

  # No changes to `df`
  expect_identical(df, expect)
})

test_that("vec_rbind() consistently handles unnamed outputs", {
  # Name repair of columns is a little weird but unclear we can do better
  expect_identical(
    vec_rbind(1, 2, .names_to = NULL),
    data.frame(...1 = c(1, 2))
  )
  expect_identical(
    vec_rbind(1, 2, ...10 = 3, .names_to = NULL),
    data.frame(...1 = c(1, 2, 3), row.names = c("...1", "...2", "...3"))
  )

  expect_identical(
    vec_rbind(a = 1, b = 2, .names_to = NULL),
    data.frame(...1 = c(1, 2), row.names = c("a", "b"))
  )
  expect_identical(
    vec_rbind(c(a = 1), c(b = 2), .names_to = NULL),
    data.frame(a = c(1, NA), b = c(NA, 2))
  )
})

test_that("vec_rbind() ignores named inputs by default (#966)", {
  expect_identical(
    vec_rbind(foo = c(a = 1)),
    data.frame(a = 1)
  )
  expect_identical(
    vec_rbind(foo = c(a = 1), .names_to = NULL),
    data.frame(a = 1, row.names = "foo")
  )
})

test_that("vec_cbind() consistently handles unnamed outputs", {
  expect_identical(
    vec_cbind(1, 2),
    data.frame(...1 = 1, ...2 = 2)
  )
  expect_identical(
    vec_cbind(1, 2, ...10 = 3),
    data.frame(...1 = 1, ...2 = 2, ...3 = 3)
  )
  expect_identical(
    vec_cbind(a = 1, b = 2),
    data.frame(a = 1, b = 2)
  )
  expect_identical(
    vec_cbind(c(a = 1), c(b = 2)),
    new_data_frame(list(...1 = c(a = 1), ...2 = c(b = 2)))
  )
})

test_that("vec_rbind() name repair messages are useful", {
  local_name_repair_verbose()

  expect_snapshot({
    vec_rbind(1, 2)
    vec_rbind(1, 2, .names_to = NULL)

    vec_rbind(1, 2, ...10 = 3)
    vec_rbind(1, 2, ...10 = 3, .names_to = NULL)

    vec_rbind(a = 1, b = 2)
    vec_rbind(a = 1, b = 2, .names_to = NULL)

    vec_rbind(c(a = 1), c(b = 2))
    vec_rbind(c(a = 1), c(b = 2), .names_to = NULL)
  })
})

test_that("vec_rbind() is silent when assigning duplicate row names of df-cols", {
  df <- new_data_frame(list(x = mtcars[1:3, 1, drop = FALSE]))

  expect_snapshot(vec_rbind(df, df))
  expect_snapshot(vec_rbind(mtcars[1:4, ], mtcars[1:3, ]))
})

test_that("vec_cbind() name repair messages are useful", {
  local_name_repair_verbose()

  expect_snapshot({
    vec_cbind(1, 2)
    vec_cbind(1, 2, ...10 = 3)
    vec_cbind(a = 1, b = 2)
    vec_cbind(c(a = 1), c(b = 2))
  })
})

test_that("cbind() deals with row names", {
  expect_identical(
    vec_cbind(mtcars[1:3], foo = 1),
    cbind(mtcars[1:3], foo = 1)
  )
  expect_identical(
    vec_cbind(mtcars[1:3], mtcars[4]),
    cbind(mtcars[1:3], mtcars[4])
  )

  out <- vec_cbind(
    mtcars[1, 1, drop = FALSE],
    unrownames(mtcars[1:3, 2, drop = FALSE])
  )
  exp <- mtcars[1:3, c(1, 2)]
  exp[[1]] <- exp[[1, 1]]
  row.names(exp) <- paste0(c("Mazda RX4..."), 1:3)
  expect_identical(out, exp)
})

test_that("prefer row names of first named input (#1058)", {
  df0 <- unrownames(mtcars[1:5, 1:3])
  df1 <- mtcars[1:5, 4:6]
  df2 <- mtcars[5:1, 7:9]

  expect_identical(
    row.names(vec_cbind(df0, df1, df2)),
    row.names(df1)
  )
  expect_identical(
    row.names(vec_cbind(df0, df2, df1)),
    row.names(df2)
  )
})

test_that("can rbind data frames with matrix columns (#625)", {
  df <- tibble(x = 1:2, y = matrix(1:4, nrow = 2))
  expect_identical(vec_rbind(df, df), vec_slice(df, c(1, 2, 1, 2)))
})

test_that("rbind repairs names of data frames (#704)", {
  df <- data_frame(x = 1, x = 2, .name_repair = "minimal")
  df_repaired <- data_frame(x...1 = 1, x...2 = 2)
  expect_identical(vec_rbind(df), df_repaired)
  expect_identical(vec_rbind(df, df), vec_rbind(df_repaired, df_repaired))

  expect_snapshot({
    (expect_error(
      vec_rbind(df, df, .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
    (expect_error(
      vec_rbind(
        df,
        df,
        .name_repair = "check_unique",
        .error_call = call("foo")
      ),
      class = "vctrs_error_names_must_be_unique"
    ))
  })
})

test_that("vec_rbind() works with simple homogeneous foreign S3 classes", {
  expect_identical(
    vec_rbind(set_names(foobar(1), "x"), set_names(foobar(2), "x")),
    data_frame(x = foobar(c(1, 2)))
  )
})

test_that("vec_rbind() works with simple homogeneous foreign S4 classes", {
  joe1 <- .Counts(1L, name = "Joe")
  joe2 <- .Counts(2L, name = "Joe")

  expect_identical(
    vec_rbind(set_names(joe1, "x"), set_names(joe2, "x")),
    data_frame(x = .Counts(1:2, name = "Joe"))
  )
})

test_that("vec_rbind() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")

    (expect_error(
      vec_rbind(set_names(x, "x"), set_names(y, "x")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_rbind() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(1L, name = "Joe")
    jane <- .Counts(2L, name = "Jane")
    (expect_error(
      vec_rbind(set_names(joe, "x"), set_names(jane, "y")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_rbind() falls back to c() if S3 method is available", {
  x <- foobar(1, foo = 1)
  y <- foobar(2, bar = 2)

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  expect_error(vec_rbind(x_df, y_df), class = "vctrs_error_incompatible_type")
  expect_error(vec_c(x_df, y_df), class = "vctrs_error_incompatible_type")
  expect_error(
    list_unchop(list(x_df, y_df), indices = list(1, 2)),
    class = "vctrs_error_incompatible_type"
  )

  with_c_method <- function(expr) {
    with_methods(
      c.vctrs_foobar = function(...) quux(NextMethod()),
      expr
    )
  }

  out <- with_c_method(vec_rbind(x_df, y_df))
  exp <- data_frame(x = quux(c(1, 2)))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(x_df, y_df)), exp)
  expect_identical(
    with_c_method(list_unchop(list(x_df, y_df), indices = list(1, 2))),
    exp
  )

  # Fallback is used with data frame subclasses, with or without
  # ptype2 method
  foo_df <- foobaz(x_df)
  bar_df <- foobaz(y_df)

  out <- with_c_method(vec_rbind(foo_df, bar_df))
  exp <- foobaz(data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(foo_df, bar_df)), exp)
  expect_identical(
    with_c_method(list_unchop(list(foo_df, bar_df), indices = list(1, 2))),
    exp
  )

  with_hybrid_methods <- function(expr, cast = TRUE) {
    methods <- list(
      c.vctrs_foobar = function(...) quux(NextMethod()),
      vec_ptype2.vctrs_foobaz.vctrs_foobaz = function(...) {
        foobaz(df_ptype2(...))
      },
      vec_cast.vctrs_foobaz.vctrs_foobaz = if (cast) {
        function(...) foobaz(df_cast(...))
      }
    )
    with_methods(expr, !!!compact(methods))
  }

  expect_equal(
    with_hybrid_methods(
      cast = FALSE,
      vec_rbind(foo_df, bar_df)
    ),
    foobaz(data_frame(x = quux(c(1, 2))))
  )

  # Falls back to data frame because there is no ptype2/cast methods
  out <- with_hybrid_methods(vec_rbind(foo_df, bar_df))
  exp <- foobaz(data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_hybrid_methods(vec_c(foo_df, bar_df)), exp)
  expect_identical(
    with_hybrid_methods(list_unchop(
      list(foo_df, bar_df),
      indices = list(1, 2)
    )),
    exp
  )

  wrapper_x_df <- data_frame(x = x_df)
  wrapper_y_df <- data_frame(x = y_df)

  out <- with_c_method(vec_rbind(wrapper_x_df, wrapper_y_df))
  exp <- data_frame(x = data_frame(x = quux(c(1, 2))))
  expect_identical(out, exp)

  expect_identical(with_c_method(vec_c(wrapper_x_df, wrapper_y_df)), exp)
  expect_identical(
    with_c_method(list_unchop(
      list(wrapper_x_df, wrapper_y_df),
      indices = list(1, 2)
    )),
    exp
  )
})

test_that("c() fallback works with unspecified columns", {
  local_methods(
    c.vctrs_foobar = function(...) foobar(NextMethod()),
    `[.vctrs_foobar` = function(x, i, ...) {
      foobar(NextMethod(), dispatched = TRUE)
    }
  )

  out <- vec_rbind(
    data_frame(x = foobar(1)),
    data_frame(y = foobar(2))
  )
  expect_identical(
    out,
    data_frame(
      x = foobar(c(1, NA), dispatched = TRUE),
      y = foobar(c(NA, 2), dispatched = TRUE)
    )
  )
})

test_that("c() fallback works with vctrs-powered data frame subclass", {
  local_methods(
    c.vctrs_quux = function(...) quux(vec_paste0(NextMethod(), "-c")),
    `[.vctrs_quux` = function(x, i, ...) quux(vec_paste0(NextMethod(), "-["))
  )
  local_foobar_df_methods()

  ### Joint case
  df1 <- foobar(data_frame(x = quux(1:3)))
  df2 <- data_frame(x = quux(4:5))

  out <- vctrs::vec_rbind(df1, df2)
  exp <- foobar(data_frame(x = quux(paste0(1:5, "-c"))))
  expect_identical(out, exp)

  out <- vctrs::vec_rbind(df2, df1)
  exp <- foobar(data_frame(x = quux(paste0(c(4:5, 1:3), "-c"))))
  expect_identical(out, exp)

  ### Disjoint case
  df1 <- foobar(data_frame(x = quux(1:3)))
  df2 <- data.frame(y = 4:5)

  out <- vctrs::vec_rbind(df1, df2)
  exp <- foobar(data_frame(
    x = quux(c(paste0(1:3, "-c-["), paste0(c(NA, NA), "-["))),
    y = c(rep(NA, 3), 4:5)
  ))
  expect_identical(out, exp)

  out <- vctrs::vec_rbind(df2, df1)
  exp <- foobar(data_frame(
    y = c(4:5, rep(NA, 3)),
    x = quux(c(paste0(c(NA, NA), "-["), paste0(1:3, "-c-[")))
  ))
  expect_identical(out, exp)
})

test_that("vec_rbind() falls back to c() if S3 method is available for S4 class", {
  joe <- data_frame(x = .Counts(c(1L, 2L), name = "Joe"))
  jane <- data_frame(x = .Counts(3L, name = "Jane"))

  expect_error(vec_rbind(joe, jane), class = "vctrs_error_incompatible_type")

  out <- with_methods(
    c.vctrs_Counts = function(...) .Counts(NextMethod(), name = "dispatched"),
    vec_rbind(joe, jane)
  )
  expect_identical(out$x, .Counts(1:3, name = "dispatched"))
})

test_that("rbind supports names and inner names (#689)", {
  out <- vec_rbind(
    data_frame(x = list(a = 1, b = 2)),
    data_frame(x = list(3)),
    data_frame(x = list(d = 4))
  )
  expect_identical(out$x, list(a = 1, b = 2, 3, d = 4))

  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  oo_x <- set_names(
    as.POSIXlt(c("2020-01-01", "2020-01-02", "2020-01-03")),
    letters[1:3]
  )
  oo_y <- c(FOO = as.POSIXlt(c("2020-01-04")))
  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "d")
  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("d"))
  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x, oo = oo_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y, oo = oo_y),
    row.names = c("quux")
  )

  nested_out <- vec_rbind(nested_x, nested_y)
  expect_identical(row.names(nested_out), c("foo", "bar", "baz", "quux"))
  expect_identical(row.names(nested_out$df), c("a", "b", "c", "d"))
  expect_identical(row.names(nested_out$mat), c("a", "b", "c", "d"))
  expect_identical(names(nested_out$vec), c("a", "b", "c", "FOO"))
  expect_identical(names(nested_out$oo), c("a", "b", "c", "FOO"))
})

test_that("vec_rbind() doesn't fall back to c() with proxied classes (#1119)", {
  foobar_rcrd <- function(x, y) {
    new_rcrd(list(x = x, y = y), class = "vctrs_foobar")
  }

  x <- foobar_rcrd(x = 1:2, y = 3:4)

  out <- vec_rbind(x, x)
  exp <- data_frame(
    ...1 = foobar_rcrd(x = c(1L, 1L), y = c(3L, 3L)),
    ...2 = foobar_rcrd(x = c(2L, 2L), y = c(4L, 4L))
  )
  expect_identical(out, exp)

  out <- vec_rbind(data_frame(x = x), data_frame(x = x))
  exp <- data_frame(
    x = foobar_rcrd(x = c(1L, 2L, 1L, 2L), y = c(3L, 4L, 3L, 4L))
  )
  expect_identical(out, exp)
})

test_that("vec_rbind() fallback works with tibbles", {
  x <- foobar("foo")
  df <- data_frame(x = x)
  tib <- tibble(x = x)

  local_methods(c.vctrs_foobar = function(...) quux(NextMethod()))

  exp <- tibble(x = quux(c("foo", "foo")))

  expect_identical(vec_rbind(tib, tib), exp)
  expect_identical(vec_rbind(df, tib), exp)
  expect_identical(vec_rbind(tib, df), exp)
})

test_that("vec_rbind() zaps names when name-spec is zap() and names-to is NULL", {
  expect_identical(
    vec_rbind(foo = c(x = 1), .names_to = NULL, .name_spec = zap()),
    data.frame(x = 1)
  )
})

test_that("can zap names even when `.names_to` is supplied", {
  expect_identical(
    vec_rbind(foo = c(x = 1), .names_to = zap(), .name_spec = zap()),
    data.frame(x = 1)
  )
  expect_identical(
    vec_rbind(
      foo = data.frame(x = 1, row.names = "row"),
      .names_to = zap(),
      .name_spec = zap()
    ),
    data.frame(x = 1)
  )

  # We previously didn't allow `.name_spec = zap()` when `.names_to = "id"`,
  # but this does have a use case - zapping inner row names while also moving
  # outer names into a new column
  expect_identical(
    vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap()),
    data.frame(id = "foo", x = 1)
  )
  expect_identical(
    vec_rbind(
      foo = data.frame(x = 1, row.names = "row"),
      .names_to = "id",
      .name_spec = zap()
    ),
    data.frame(id = "foo", x = 1)
  )
})

test_that("can request 'inner' names when `.names_to` is supplied", {
  # Note how it can be useful to lock `.name_spec` to `"inner"` in your API, but
  # still expose `.names_to` to your users and allow all of its options.
  # `purrr::list_rbind()` does this.
  expect_identical(
    vec_rbind(
      foo = data.frame(x = 1, row.names = "row"),
      .names_to = zap(),
      .name_spec = "inner"
    ),
    data.frame(x = 1, row.names = "row")
  )
  expect_identical(
    vec_rbind(
      foo = data.frame(x = 1, row.names = "row"),
      .names_to = "id",
      .name_spec = "inner"
    ),
    data.frame(id = "foo", x = 1, row.names = "row")
  )
  expect_identical(
    vec_rbind(
      foo = data.frame(x = 1, row.names = "row"),
      .names_to = NULL,
      .name_spec = "inner"
    ),
    data.frame(x = 1, row.names = "row")
  )
})

test_that("can zap outer names from a name-spec (#1215)", {
  zap_outer_spec <- function(outer, inner) if (is_character(inner)) inner

  df <- data.frame(x = 1:2)
  df_named <- data.frame(x = 3L, row.names = "foo")

  expect_null(
    vec_names(vec_rbind(a = df, .names_to = NULL, .name_spec = zap_outer_spec))
  )
  expect_identical(
    vec_names(vec_rbind(a = df, df_named, .name_spec = zap_outer_spec)),
    c("...1", "...2", "foo")
  )

  # These days it is more efficient to use a name-spec of "inner" (#1988)
  expect_identical(
    vec_rbind(a = df, .names_to = NULL, .name_spec = zap_outer_spec),
    vec_rbind(a = df, .names_to = NULL, .name_spec = "inner")
  )
  expect_identical(
    vec_rbind(a = df, df_named, .name_spec = zap_outer_spec),
    vec_rbind(a = df, df_named, .name_spec = "inner")
  )
})

test_that("column names are treated consistently in vec_rbind()", {
  exp <- data.frame(a = c(1L, 1L), b = c(2L, 2L))

  x <- c(a = 1L, b = 2L)
  expect_identical(vec_rbind(x, x), exp)

  x <- array(1:2, dimnames = list(c("a", "b")))
  expect_identical(vec_rbind(x, x), exp)

  x <- matrix(1:2, nrow = 1, dimnames = list(NULL, c("a", "b")))
  expect_identical(vec_rbind(x, x), exp)

  x <- array(1:6, c(1, 2, 1), dimnames = list(NULL, c("a", "b"), NULL))
  expect_error(vec_rbind(x, x), "Can't bind arrays")
})

test_that("can repair names of row-binded vectors (#1567)", {
  local_name_repair_verbose()
  expect_silent(
    expect_named(
      vec_rbind(
        x = 1:3,
        y = 4:6,
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )
  )
})

test_that("can repair names of row-binded matrices", {
  local_name_repair_verbose()
  expect_silent({
    expect_named(
      vec_rbind(
        x = matrix(1:3, 1),
        y = matrix(4:6, 1),
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )

    expect_named(
      vec_rbind(
        x = matrix(1:3, 1),
        y = 4:6,
        .name_repair = function(x) c("a", "a", "a")
      ),
      c("a", "a", "a")
    )
  })
})

test_that("vec_rbind() only restores one time", {
  restored <- list()

  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x,
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x,
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) {
      # Ignore proxying and restoration of ptypes
      if (length(x)) {
        restored <<- c(restored, list(x))
      }
      foobar(x)
    }
  )

  df <- data_frame(x = foobar(1:3))
  vec_rbind(df, df)

  expect_equal(
    restored,
    list(
      rep(na_int, 6), # From `vec_init()`
      foobar(c(1:3, 1:3)) # Final restoration
    )
  )
})

test_that("vec_rbind() applies `base::c()` fallback to df-cols (#1462, #1640)", {
  x <- structure(1, class = "myclass")
  df <- tibble(a = tibble(x = x))
  df <- vec_rbind(df, df)

  expect_equal(df$a$x, structure(c(1, 1), class = "myclass"))
})


# Golden tests -------------------------------------------------------

test_that("row-binding performs expected allocations", {
  vec_rbind_list <- function(x) {
    vec_rbind(!!!x)
  }

  expect_snapshot({
    ints <- rep(list(1L), 1e2)
    named_ints <- rep(list(set_names(1:3, letters[1:3])), 1e2)

    "Integers as rows"
    suppressMessages(with_memory_prof(vec_rbind_list(ints)))
    suppressMessages(with_memory_prof(vec_rbind_list(named_ints)))

    "Data frame with named columns"
    df <- data_frame(
      x = set_names(as.list(1:2), c("a", "b")),
      y = set_names(1:2, c("A", "B")),
      z = data_frame(Z = set_names(1:2, c("Za", "Zb")))
    )
    dfs <- rep(list(df), 1e2)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (non-repaired, non-recursive case)"
    df <- data_frame(x = 1:2)
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (repaired, non-recursive case)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (non-repaired, recursive case) (#1217)"
    df <- data_frame(
      x = 1:2,
      y = data_frame(x = 1:2)
    )
    dfs <- rep(list(df), 1e2)
    dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))

    "Data frame with rownames (repaired, recursive case) (#1217)"
    dfs <- map(dfs, set_rownames_recursively)
    with_memory_prof(vec_rbind_list(dfs))
  })
})

test_that("`.names_to` is assigned after restoration (#1648)", {
  df <- data_frame(x = factor("foo"))
  expect_equal(
    vec_rbind(name = df, .names_to = "x"),
    data_frame(x = "name")
  )

  # This used to fail with:
  #> Error in `vctrs::vec_rbind()`:
  #> ! adding class "factor" to an invalid object
})

test_that("tibble beats data frame", {
  df <- new_data_frame()
  dt <- tibble::tibble()

  expect_s3_class(vec_ptype_common(dt, df), "tbl_df")
  expect_s3_class(vec_ptype_common(df, dt), "tbl_df")
})

test_that("can cast tibble to df and vice versa", {
  df <- new_data_frame()
  tib <- tibble::tibble()

  expect_identical(vec_cast(df, tib), tib)
  expect_identical(vec_cast(tib, df), df)
  expect_identical(vec_cast(tib, tib), tib)
})

test_that("can't cast vector to tibble", {
  dt <- tibble::tibble()
  v <- logical()

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(vec_ptype2(v, dt), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_ptype2(dt, v), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_cast(v, dt), class = "vctrs_error_incompatible_type"))
  })
})

test_that("casting to and from tibble preserves row names", {
  out <- vec_cast(mtcars, tibble::as_tibble(mtcars))
  expect_identical(row.names(out), row.names(mtcars))

  out <- vec_cast(out, unrownames(mtcars))
  expect_identical(row.names(out), row.names(mtcars))
})

test_that("no common type between list and tibble", {
  dt <- tibble::tibble()
  l <- list()

  expect_error(vec_ptype2(l, dt), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(dt, l), class = "vctrs_error_incompatible_type")
})

test_that("vec_restore restores tibbles", {
  df1 <- tibble::tibble(x = 1:4)
  df2 <- vec_restore(vec_data(df1), df1)

  expect_s3_class(df2, "tbl_df")
})

test_that("the type of a tibble with an unspecified column retains unspecifiedness", {
  df1 <- tibble::tibble(x = 1, y = NA)
  df2 <- tibble::tibble(x = 1, y = unspecified(1))
  expect <- tibble::tibble(x = numeric(), y = unspecified())

  expect_identical(vec_ptype(df1), expect)
  expect_identical(vec_ptype(df2), expect)
})

test_that("vec_ptype_finalise() works recursively over tibbles", {
  df <- tibble(x = numeric(), y = unspecified())
  expect <- tibble(x = numeric(), y = logical())

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() can handle tibble df columns", {
  df <- tibble(x = numeric(), y = tibble(z = unspecified()))
  expect <- tibble(x = numeric(), y = tibble(z = logical()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("can use ptype2 and cast with tibble that has incorrect class vector", {
  tib1 <- structure(data.frame(x = 1), class = c("tbl_df", "data.frame"))
  tib2 <- structure(data.frame(y = 2), class = c("tbl_df", "data.frame"))
  exp <- structure(
    data.frame(x = dbl(), y = dbl()),
    class = c("tbl_df", "data.frame")
  )

  requireNamespace("tibble")

  expect_identical(
    vec_ptype_common(tib1, tib2),
    exp
  )
  expect_identical(
    vec_ptype_common(tib1, data.frame(y = 2)),
    tibble::new_tibble(exp, nrow = nrow(exp))
  )
  expect_identical(
    vec_ptype_common(data.frame(x = 1), tib2),
    tibble::new_tibble(exp, nrow = nrow(exp))
  )
  expect_identical(
    vec_cast(tib1, tib1),
    tib1
  )

  expect_snapshot({
    local_error_call(call("my_function"))
    (expect_error(
      vec_cast(tib1, tib2),
      class = "vctrs_error_cast"
    ))
    (expect_error(
      vec_cast(tib1, data.frame(y = 2)),
      class = "vctrs_error_cast"
    ))
    (expect_error(
      vec_cast(data.frame(x = 1), tib2),
      class = "vctrs_error_cast"
    ))
  })
})

test_that("vec_data() preserves names (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_names(x), vec_names(vec_data(x)))

  x <- diag(2)
  rownames(x) <- letters[1:2]
  colnames(x) <- LETTERS[1:2]
  expect_identical(vec_names(x), vec_names(vec_data(x)))
})

test_that("vec_data() preserves size (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_size(x), vec_size(vec_data(x)))

  x <- diag(2)
  expect_identical(vec_size(x), vec_size(vec_data(x)))
})

test_that("vec_data() preserves dim and dimnames (#245)", {
  x <- set_names(letters, LETTERS)
  expect_identical(vec_dim(x), vec_dim(vec_data(x)))

  x <- diag(2)
  expect_identical(vec_dim(x), vec_dim(vec_data(x)))

  x <- diag(2)
  rownames(x) <- letters[1:2]
  colnames(x) <- LETTERS[1:2]
  expect_identical(dimnames(x), dimnames(vec_data(x)))
})

test_that("strips vector attributes apart from names, dim and dimnames", {
  x <- new_vctr(1:10, a = 1, b = 2)
  expect_null(attributes(vec_data(x)))

  x <- new_vctr(c(x = 1, y = 2), a = 1, b = 2)
  expect_equal(names(attributes(vec_data(x))), "names")

  x <- new_vctr(1, a = 1, dim = c(1L, 1L))
  expect_equal(names(attributes(vec_data(x))), "dim")

  x <- new_vctr(1, a = 1, dim = c(1L, 1L), dimnames = list("foo", "bar"))
  expect_equal(names(attributes(vec_data(x))), c("dim", "dimnames"))
})

test_that("vec_proxy() is a no-op with data vectors", {
  for (x in vectors) {
    expect_identical(vec_proxy(!!x), !!x)
  }

  x <- structure(1:3, foo = "bar")
  expect_identical(vec_proxy(!!x), !!x)
})

test_that("vec_proxy() transforms records to data frames", {
  for (x in records) {
    expect_identical(vec_proxy(x), new_data_frame(unclass(x)))
  }
})

test_that("vec_proxy() is a no-op with non vectors", {
  x <- foobar(list())
  expect_identical(vec_proxy(x), x)
})

test_that("can take the proxy of non-vector objects", {
  local_env_proxy()
  expect_identical(vec_proxy(new_proxy(1:3)), 1:3)
})

test_that("vec_data() asserts vectorness", {
  expect_error(vec_data(new_sclr()), class = "vctrs_error_scalar_type")
  expect_error(vec_data(~foo), class = "vctrs_error_scalar_type")
})

test_that("vec_data() is proxied", {
  local_env_proxy()
  x <- new_proxy(mtcars)
  expect_identical(vec_data(x), vec_data(mtcars))
})

test_that("vec_proxy_equal() is recursive over data frames (#641)", {
  x <- new_data_frame(list(x = foobar(1:3), y = 41:43))
  default <- vec_proxy_equal(x)
  expect_s3_class(default$x, "vctrs_foobar")

  local_methods(vec_proxy_equal.vctrs_foobar = function(...) c(0, 0, 0))
  overridden <- vec_proxy_equal(x)
  expect_identical(overridden$x, c(0, 0, 0))
})

test_that("vec_proxy_equal() returns a POSIXct for POSIXlt objects (#901)", {
  x <- as.POSIXlt(new_date(0), tz = "UTC")
  expect_s3_class(vec_proxy_equal(x), "POSIXct")
})

test_that("vec_proxy_equal() defaults to vec_proxy() and vec_proxy_compare() defaults to vec_proxy_equal() (#1140)", {
  foobar_proxy <- function(x, ...) data_frame(x = unclass(x), y = seq_along(x))

  local_methods(vec_proxy.vctrs_foobar = foobar_proxy)

  x <- foobar(3:1)
  expect_identical(vec_proxy(x), foobar_proxy(x))
  expect_identical(vec_proxy_equal(x), foobar_proxy(x))
  expect_identical(vec_proxy_compare(x), foobar_proxy(x))

  local_methods(vec_proxy_equal.vctrs_foobar = function(x, ...) {
    foobar_proxy(letters[x])
  })

  expect_identical(vec_proxy_equal(x), data_frame(x = letters[3:1], y = 1:3))
  expect_identical(vec_proxy_compare(x), data_frame(x = letters[3:1], y = 1:3))
})

test_that("equal/compare/order proxy methods that return data frames are automatically flattened", {
  x <- new_vctr(1:2, class = "custom")

  equal <- data_frame(a = 1:2, b = 3:4)
  order <- data_frame(a = 3:4, b = 4:5)

  local_methods(
    vec_proxy_equal.custom = function(x, ...) data_frame(col = equal),
    vec_proxy_order.custom = function(x, ...) data_frame(col = order)
  )

  expect_identical(vec_proxy_equal(x), equal)
  expect_identical(vec_proxy_compare(x), equal)
  expect_identical(vec_proxy_order(x), order)
})

test_that("equal/compare/order proxy methods that return 1 column data frames are automatically unwrapped", {
  x <- new_vctr(1:2, class = "custom")

  equal <- 1:2
  order <- 3:4

  local_methods(
    vec_proxy_equal.custom = function(x, ...) data_frame(a = equal),
    vec_proxy_order.custom = function(x, ...) {
      data_frame(col = data_frame(a = order))
    }
  )

  expect_identical(vec_proxy_equal(x), equal)
  expect_identical(vec_proxy_compare(x), equal)
  expect_identical(vec_proxy_order(x), order)
})

test_that("vec_data() preserves data frames", {
  expect_identical(
    vec_data(tibble(x = 1)),
    data_frame(x = 1)
  )

  # Rownames are preserved
  expect_identical(
    vec_data(mtcars),
    mtcars
  )
})

# `grouped_df` -------------------------------------------------------

bare_mtcars <- unrownames(mtcars)

test_that("grouped-df is proxied and restored", {
  gdf <- dplyr::group_by(bare_mtcars, cyl)

  expect_identical(vec_proxy(gdf), gdf)
  expect_identical(vec_restore(bare_mtcars, gdf), gdf)

  expect_identical(vec_ptype(gdf), gdf[0, ])

  gdf <- dplyr::group_by(bare_mtcars, cyl, am, vs)
  expect_identical(gdf[0, ], vec_ptype(gdf))

  out <- vec_ptype(dplyr::group_by(bare_mtcars, cyl, .drop = FALSE))
  expect_drop(out, FALSE)
})

test_that("can take the common type of grouped tibbles and tibbles", {
  gdf <- dplyr::group_by(bare_mtcars, cyl)
  expect_identical(vec_ptype2(gdf, data.frame()), vec_ptype(gdf))
  expect_identical(vec_ptype2(data.frame(), gdf), vec_ptype(gdf))
  expect_identical(vec_ptype2(gdf, tibble()), vec_ptype(gdf))
  expect_identical(vec_ptype2(tibble(), gdf), vec_ptype(gdf))

  gdf_nodrop <- dplyr::group_by(bare_mtcars, cyl, .drop = FALSE)
  expect_drop(vec_ptype2(gdf, gdf_nodrop), FALSE)
  expect_drop(vec_ptype2(gdf_nodrop, gdf), FALSE)
  expect_drop(vec_ptype2(gdf_nodrop, bare_mtcars), FALSE)
  expect_drop(vec_ptype2(bare_mtcars, gdf_nodrop), FALSE)
})

test_that("the common type of grouped tibbles includes the union of grouping variables", {
  gdf1 <- dplyr::group_by(bare_mtcars, cyl)
  gdf2 <- dplyr::group_by(bare_mtcars, am, vs)
  expect_identical(
    vec_ptype2(gdf1, gdf2),
    vec_ptype(dplyr::group_by(bare_mtcars, cyl, am, vs))
  )
})

test_that("can cast to and from `grouped_df`", {
  gdf <- dplyr::group_by(unrownames(bare_mtcars), cyl)
  input <- bare_mtcars[10]
  cast_gdf <- dplyr::group_by(vec_cast(bare_mtcars[10], bare_mtcars), cyl)

  expect_error(
    vec_cast(input, dplyr::group_by(bare_mtcars["cyl"], cyl)),
    class = "vctrs_error_cast_lossy"
  )

  expect_identical(
    vec_cast(input, gdf),
    cast_gdf
  )
  expect_identical(
    vec_cast(gdf, bare_mtcars),
    unrownames(bare_mtcars)
  )

  expect_identical(
    vec_cast(tibble::as_tibble(input), gdf),
    unrownames(cast_gdf)
  )
  tib <- tibble::as_tibble(bare_mtcars)
  expect_identical(
    unrownames(vec_cast(gdf, tib)),
    tib
  )
})

test_that("casting to `grouped_df` doesn't require grouping variables", {
  expect_identical(
    vec_cast(bare_mtcars[10], dplyr::group_by(bare_mtcars, cyl)),
    dplyr::group_by(vec_cast(bare_mtcars[10], bare_mtcars), cyl)
  )
})

test_that("casting to `grouped_df` handles `drop`", {
  gdf_nodrop <- dplyr::group_by(bare_mtcars, cyl, .drop = FALSE)
  expect_identical(vec_cast(bare_mtcars, gdf_nodrop), gdf_nodrop)
})

test_that("can cbind grouped data frames", {
  gdf <- dplyr::group_by(bare_mtcars[-10], cyl)
  df <- unrownames(bare_mtcars)[10]

  expect_identical(
    unrownames(vec_cbind(gdf, df)),
    tibble::as_tibble(bare_mtcars)[c(1:9, 11, 10)]
  )

  gdf1 <- dplyr::group_by(bare_mtcars[2], cyl)
  gdf2 <- dplyr::group_by(bare_mtcars[8:9], vs, am)
  expect_identical(
    unrownames(vec_cbind(gdf1, gdf2)),
    tibble::as_tibble(bare_mtcars)[c(2, 8, 9)]
  )
})


# `rowwise` ----------------------------------------------------------

test_that("rowwise can be proxied and restored", {
  rww <- dplyr::rowwise(unrownames(bare_mtcars))

  expect_identical(vec_proxy(rww), rww)
  expect_identical(vec_restore(unrownames(bare_mtcars), rww), rww)

  expect_identical(vec_ptype(rww), rww[0, ])
})

test_that("can take the common type of rowwise tibbles and tibbles", {
  rww <- dplyr::rowwise(bare_mtcars)
  expect_identical(vec_ptype2(rww, data.frame()), vec_ptype(rww))
  expect_identical(vec_ptype2(data.frame(), rww), vec_ptype(rww))
  expect_identical(vec_ptype2(rww, tibble()), vec_ptype(rww))
  expect_identical(vec_ptype2(tibble(), rww), vec_ptype(rww))
})

test_that("can cast to and from `rowwise_df`", {
  rww <- unrownames(dplyr::rowwise(bare_mtcars))
  input <- bare_mtcars[10]
  cast_rww <- dplyr::rowwise(vec_cast(bare_mtcars[10], bare_mtcars))

  expect_error(
    vec_cast(input, dplyr::rowwise(bare_mtcars["cyl"])),
    class = "vctrs_error_cast_lossy"
  )

  expect_identical(
    vec_cast(input, rww),
    cast_rww
  )
  expect_identical(
    vec_cast(rww, bare_mtcars),
    unrownames(bare_mtcars)
  )

  expect_identical(
    vec_cast(tibble::as_tibble(input), rww),
    unrownames(cast_rww)
  )
  tib <- tibble::as_tibble(bare_mtcars)
  expect_identical(
    unrownames(vec_cast(rww, tib)),
    tib
  )
})

test_that("can cbind rowwise data frames", {
  df <- unrownames(bare_mtcars)
  rww <- dplyr::rowwise(df[-2])
  gdf <- dplyr::group_by(df[2], cyl)

  exp <- dplyr::rowwise(df[c(1, 3:11, 2)])
  expect_identical(vec_cbind(rww, df[2]), exp)

  # Suboptimal
  expect_identical(vec_cbind(rww, gdf), exp)
})

test_that("common type between rowwise and grouped data frames is a bare df", {
  out <- vec_ptype_common(
    dplyr::rowwise(bare_mtcars),
    dplyr::group_by(bare_mtcars, cyl)
  )
  expect_identical(out, tibble::as_tibble(bare_mtcars[0, ]))
})

local_name_repair_quiet <- function(frame = caller_env()) {
  local_options(rlib_name_repair_verbosity = "quiet", .frame = frame)
}
local_name_repair_verbose <- function(frame = caller_env()) {
  local_options(rlib_name_repair_verbosity = "verbose", .frame = frame)
}

#' Proxy and restore
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' `vec_proxy()` returns the data structure containing the values of a
#' vector. This data structure is usually the vector itself. In this
#' case the proxy is the [identity function][base::identity], which is
#' the default `vec_proxy()` method.
#'
#' Only experts should implement special `vec_proxy()` methods, for
#' these cases:
#'
#' - A vector has vectorised attributes, i.e. metadata for
#'   each element of the vector. These _record types_ are implemented
#'   in vctrs by returning a data frame in the proxy method. If you're
#'   starting your class from scratch, consider deriving from the
#'   [`rcrd`][new_rcrd] class. It implements the appropriate data
#'   frame proxy and is generally the preferred way to create a record
#'   class.
#'
#' - When you're implementing a vector on top of a non-vector type,
#'   like an environment or an S4 object. This is currently only
#'   partially supported.
#'
#' - S3 lists are considered scalars by default. This is the safe
#'   choice for list objects such as returned by `stats::lm()`. To
#'   declare that your S3 list class is a vector, you normally add
#'   `"list"` to the right of your class vector. Explicit inheritance
#'   from list is generally the preferred way to declare an S3 list in
#'   R, for instance it makes it possible to dispatch on
#'   `generic.list` S3 methods.
#'
#'   If you can't modify your class vector, you can implement an
#'   identity proxy (i.e. a proxy method that just returns its input)
#'   to let vctrs know this is a vector list and not a scalar.
#'
#' `vec_restore()` is the inverse operation of `vec_proxy()`. It
#' should only be called on vector proxies.
#'
#' - It undoes the transformations of `vec_proxy()`.
#'
#' - It restores attributes and classes. These may be lost when the
#'   memory values are manipulated. For example slicing a subset of a
#'   vector's proxy causes a new proxy to be allocated.
#'
#' By default vctrs restores all attributes and classes
#' automatically. You only need to implement a `vec_restore()` method
#' if your class has attributes that depend on the data.
#'
#' @param x A vector.
#' @inheritParams rlang::args_dots_empty
#'
#' @section Proxying:
#'
#' You should only implement `vec_proxy()` when your type is designed
#' around a non-vector class. I.e. anything that is not either:
#'
#' * An atomic vector
#' * A bare list
#' * A data frame
#'
#' In this case, implement `vec_proxy()` to return such a vector
#' class. The vctrs operations such as [vec_slice()] are applied on
#' the proxy and `vec_restore()` is called to restore the original
#' representation of your type.
#'
#' The most common case where you need to implement `vec_proxy()` is
#' for S3 lists. In vctrs, S3 lists are treated as scalars by
#' default. This way we don't treat objects like model fits as
#' vectors. To prevent vctrs from treating your S3 list as a scalar,
#' unclass it in the `vec_proxy()` method. For instance, here is the
#' definition for `list_of`:
#'
#' ```
#' vec_proxy.vctrs_list_of <- function(x) {
#'   unclass(x)
#' }
#' ```
#'
#' Another case where you need to implement a proxy is [record
#' types][new_rcrd]. Record types should return a data frame, as in
#' the `POSIXlt` method:
#'
#' ```
#' vec_proxy.POSIXlt <- function(x) {
#'   new_data_frame(unclass(x))
#' }
#' ```
#'
#' Note that you don't need to implement `vec_proxy()` when your class
#' inherits from `vctrs_vctr` or `vctrs_rcrd`.
#'
#'
#' @section Restoring:
#'
#' A restore is a specialised type of cast, primarily used in
#' conjunction with `NextMethod()` or a C-level function that works on
#' the underlying data structure. A `vec_restore()` method can make
#' the following assumptions about `x`:
#'
#' * It has the correct type.
#' * It has the correct names.
#' * It has the correct `dim` and `dimnames` attributes.
#' * It is unclassed. This way you can call vctrs generics with `x`
#'   without triggering an infinite loop of restoration.
#'
#' The length may be different (for example after [vec_slice()] has
#' been called), and all other attributes may have been lost. The
#' method should restore all attributes so that after restoration,
#' `vec_restore(vec_data(x), x)` yields `x`.
#'
#' To understand the difference between `vec_cast()` and `vec_restore()`
#' think about factors: it doesn't make sense to cast an integer to a factor,
#' but if `NextMethod()` or another low-level function has stripped attributes,
#' you still need to be able to restore them.
#'
#' The default method copies across all attributes so you only need to
#' provide your own method if your attributes require special care
#' (i.e. they are dependent on the data in some way). When implementing
#' your own method, bear in mind that many R users add attributes to track
#' additional metadata that is important to them, so you should preserve any
#' attributes that don't require special handling for your class.
#'
#' @section Dependencies:
#' - `x` must be a vector in the vctrs sense (see [vec_is()])
#' - By default the underlying data is returned as is (identity proxy)
#'
#' All vector classes have a proxy, even those who don't implement any
#' vctrs methods. The exception is S3 lists that don't inherit from
#' `"list"` explicitly. These might have to implement an identity
#' proxy for compatibility with vctrs (see discussion above).
#'
#' @keywords internal
#' @export
vec_proxy <- function(x, ...) {
  check_dots_empty0(...)
  return(.Call(ffi_vec_proxy, x))
  UseMethod("vec_proxy")
}
#' @export
vec_proxy.default <- function(x, ...) {
  x
}

#' @rdname vec_proxy
#' @param to The original vector to restore to.
#' @export
vec_restore <- function(x, to, ...) {
  check_dots_empty0(...)
  return(.Call(ffi_vec_restore, x, to))
  UseMethod("vec_restore", to)
}
vec_restore_dispatch <- function(x, to, ...) {
  UseMethod("vec_restore", to)
}
#' @export
vec_restore.default <- function(x, to, ...) {
  .Call(ffi_vec_restore_default, x, to)
}
vec_restore_default <- function(x, to, ...) {
  .Call(ffi_vec_restore_default, x, to)
}

vec_proxy_recurse <- function(x, ...) {
  .Call(ffi_vec_proxy_recurse, x)
}
vec_restore_recurse <- function(x, to, ...) {
  .Call(ffi_vec_restore_recurse, x, to)
}

#' Extract underlying data
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Extract the data underlying an S3 vector object, i.e. the underlying
#' (named) atomic vector, data frame, or list.
#'
#' @param x A vector or object implementing `vec_proxy()`.
#' @return The data underlying `x`, free from any attributes except the names.
#'
#' @section Difference with `vec_proxy()`:
#'
#' * `vec_data()` returns unstructured data. The only attributes
#'   preserved are names, dims, and dimnames.
#'
#'   Currently, due to the underlying memory architecture of R, this
#'   creates a full copy of the data for atomic vectors.
#'
#' * `vec_proxy()` may return structured data. This generic is the
#'   main customisation point for accessing memory values in vctrs,
#'   along with [vec_restore()].
#'
#'   Methods must return a vector type. Records and data frames will
#'   be processed rowwise.
#'
#' @keywords internal
#' @export
vec_data <- function(x) {
  obj_check_vector(x)
  x <- vec_proxy(x)

  if (is.data.frame(x)) {
    return(new_data_frame(x, row.names = .row_names_info(x, 0L)))
  }

  if (has_dim(x)) {
    x <- vec_set_attributes(x, list(dim = dim(x), dimnames = dimnames(x)))
  } else {
    x <- vec_set_attributes(x, list(names = names(x)))
  }

  # Unset S4 bit in vector-like S4 objects
  as_not_s4(x)
}
as_not_s4 <- function(x) {
  .Call(ffi_as_not_s4, x)
}

test_that("default vec_restore() restores attributes except names", {
  to <- structure(NA, foo = "foo", bar = "bar")
  expect_identical(vec_restore.default(NA, to), structure(NA, foo = "foo", bar = "bar"))

  to <- structure(NA, names = "a", foo = "foo", bar = "bar")
  expect_identical(vec_restore.default(NA, to), structure(NA, foo = "foo", bar = "bar"))

  to <- structure(NA, foo = "foo", names = "a", bar = "bar")
  expect_identical(vec_restore.default(NA, to), structure(NA, foo = "foo", bar = "bar"))

  to <- structure(NA, foo = "foo", bar = "bar", names = "a")
  expect_identical(vec_restore.default(NA, to), structure(NA, foo = "foo", bar = "bar"))
})

test_that("default vec_restore() restores objectness", {
  to <- structure(NA, class = "foo")
  x <- vec_restore.default(NA, to)
  expect_true(is.object(x))
  expect_s3_class(x, "foo")
})

test_that("data frame vec_restore() checks type", {
  expect_error(vec_restore(NA, mtcars), "Attempt to restore data frame from a logical")
})

test_that("data frame restore forces character column names", {
  df <- new_data_frame(list(1))
  expect_named(vec_restore(df, df), "")
})

test_that("can use vctrs primitives from vec_restore() without inflooping", {
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) {
      vec_ptype(x)
      vec_init(x)
      obj_check_vector(x)
      vec_slice(x, 0)
      "woot"
    }
  )

  foobar <- new_vctr(1:3, class = "vctrs_foobar")
  expect_identical(vec_slice(foobar, 2), "woot")
})

test_that("dimensions are preserved by default restore method", {
  x <- foobar(1:4)
  dim(x) <- c(2, 2)
  dimnames(x) <- list(a = c("foo", "bar"), b = c("quux", "hunoz"))

  exp <- foobar(c(1L, 3L))
  dim(exp) <- c(1, 2)
  dimnames(exp) <- list(a = "foo", b = c("quux", "hunoz"))

  expect_identical(vec_slice(x, 1), exp)
})

test_that("names attribute isn't set when restoring 1D arrays using 2D+ objects", {
  x <- foobar(1:2)
  dim(x) <- c(2)
  nms <- c("foo", "bar")
  dimnames(x) <- list(nms)

  res <- vec_restore(x, matrix(1))

  expect_null(attributes(res)$names)
  expect_equal(attr(res, "names"), nms)
  expect_equal(names(res), nms)
})

test_that("arguments are not inlined in the dispatch call (#300)", {
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) sys.call(),
    vec_proxy.vctrs_foobar = unclass
  )
  call <- vec_restore(foobar(list(1)), foobar(list(1)))
  expect_equal(call, quote(vec_restore.vctrs_foobar(x = x, to = to)))
})

test_that("restoring to non-bare data frames calls `vec_bare_df_restore()` before dispatching", {
  x <- list(x = numeric())
  to <- new_data_frame(x, class = "tbl_foobar")

  local_methods(
    vec_restore.tbl_foobar = function(x, to, ..., n) {
      if (is.data.frame(x)) {
        abort(class = "error_df_restore_was_called")
      }
    }
  )

  expect_error(vec_restore(x, to), class = "error_df_restore_was_called")
})

test_that("row names are not restored if target is not a data frame", {
  proxy <- data.frame(x = 1)
  out <- vec_restore(proxy, to = foobar(""))
  exp <- list(names = "x", class = "vctrs_foobar")
  expect_identical(attributes(out), exp)
})

test_that("attributes are properly restored when they contain special attributes", {
  exp <- list(foo = TRUE, bar = TRUE)

  x <- structure(list(), foo = TRUE, names = chr(), bar = TRUE)
  out <- vec_restore_default(list(), x)
  expect_identical(attributes(out), exp)

  # Was broken by #943
  x <- structure(list(), foo = TRUE, names = chr(), row.names = int(), bar = TRUE)
  out <- vec_restore_default(list(), x)
  expect_identical(attributes(out), exp)
})

test_that("names<- is not called with partial data (#1108)", {
  x <- set_names(foobar(1:2), c("a", "b"))

  values <- list()
  local_methods(
    `names<-.vctrs_foobar` = function(x, value) {
      if (!is_null(value)) {
        values <<- c(values, list(value))
      }
      NextMethod()
    }
  )

  vec_c(x, x)
  expect_equal(values, list(c("a", "b", "a", "b")))
})

test_that("recursive proxy and restore work with recursive records", {
  new_recursive_rcrd <- function(x) {
    new_rcrd(
      list(field = x),
      class = "my_recursive_rcrd"
    )
  }

  internal <- new_rcrd(list(internal_field = 1:2))
  x <- new_recursive_rcrd(data_frame(col = internal))

  proxy <- vec_proxy_recurse(x)
  exp <- data_frame(field = data_frame(col = data_frame(internal_field = 1:2)))
  expect_equal(proxy, exp)
  expect_equal(vec_restore_recurse(proxy, x), x)

  # Non-recursive case doesn't proxy `internal`
  proxy <- vec_proxy(x)
  exp <- data_frame(field = data_frame(col = internal))
  expect_equal(proxy, exp)
  expect_equal(vec_restore(proxy, x), x)

  x_exp <- new_recursive_rcrd(data_frame(col = vec_rep(internal, 2)))
  expect_equal(
    list_unchop(list(x, x)),
    x_exp
  )

  df <- data_frame(x = x)
  df_exp <- data_frame(x = x_exp)
  expect_equal(vec_rbind(df, df), df_exp)
  expect_equal(vec_c(df, df), df_exp)
})

test_that("generics are extensible", {
  expect_error(vec_cast(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_restore(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy_equal(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy_compare(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype2(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_abbr(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_full(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_arith(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_finalise(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_assign(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
})

test_that("basics", {
  x <- rando(10)

  expect_true(vec_is(x))
  expect_equal(vec_size(x), 10)
  expect_identical(vec_ptype_common(x, x), vec_ptype(x))
})

test_that("casting of rando works", {
  x <- as_rando(1:10)
  expect_equal(vec_cast(x, rando()), x)

  expect_equal(vec_cast(NA, rando()), as_rando(NA))
  expect_equal(vec_cast(unspecified(2), rando()), as_rando(c(NA, NA)))

  expect_error(vec_cast(x, factor()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(factor(), x), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2 for rando works", {
  x <- as_rando(1:10)
  expect_equal(vec_ptype(vec_ptype2(x, x)), rando())

  expect_equal(vec_ptype2(x, NA), rando())
  expect_equal(vec_ptype2(NA, x), rando())

  expect_equal(vec_ptype2(unspecified(), x), rando())
  expect_equal(vec_ptype2(x, unspecified()), rando())

  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(x, ""), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2("", x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(data.frame(), x), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(x, data.frame()), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype_abbr.rando", {
  expect_equal(vec_ptype_abbr(as_rando(1:10)), "vctrs_rn")
  expect_equal(vec_ptype_full(as_rando(1:10)), "vctrs_rando")
})

test_that("proxy and data", {
  x <- rando(10)

  expect_identical(vec_ptype(vec_proxy(x)), x[0])
  expect_identical(vec_data(x), x@.Data)

  expect_false(isS4(vec_data(x)))

  expect_s4_class(vec_restore(vec_data(x), x), "vctrs_rando")
  expect_true(isS4(vec_restore(vec_data(x), x)))
})

test_that("unset_s4() copies and works", {
  # Initial condition
  x <- rando()
  expect_true(isS4(x))

  # Unsetting has no side effect on x
  unset_s4(x)
  expect_true(isS4(x))

  # Unsetting actually works
  y <- unset_s4(x)
  expect_false(isS4(y))
})

test_that("basics", {
  x <- rando(10)

  expect_true(vec_is(x))
  expect_equal(vec_size(x), 10)
  expect_identical(vec_ptype_common(x, x), vec_ptype(x))
})

test_that("casting of rando works", {
  x <- as_rando(1:10)
  expect_equal(vec_cast(x, rando()), x)

  expect_equal(vec_cast(NA, rando()), as_rando(NA))
  expect_equal(vec_cast(unspecified(2), rando()), as_rando(c(NA, NA)))

  expect_error(vec_cast(x, factor()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(factor(), x), class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2 for rando works", {
  x <- as_rando(1:10)
  expect_equal(vec_ptype(vec_ptype2(x, x)), rando())

  expect_equal(vec_ptype2(x, NA), rando())
  expect_equal(vec_ptype2(NA, x), rando())

  expect_equal(vec_ptype2(unspecified(), x), rando())
  expect_equal(vec_ptype2(x, unspecified()), rando())

  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")

  expect_error(vec_ptype2(x, ""), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2("", x), class = "vctrs_error_incompatible_type")

  expect_error(
    vec_ptype2(data.frame(), x),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(x, data.frame()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype_abbr.rando", {
  expect_equal(vec_ptype_abbr(as_rando(1:10)), "vctrs_rn")
  expect_equal(vec_ptype_full(as_rando(1:10)), "vctrs_rando")
})

test_that("proxy and data", {
  x <- rando(10)

  expect_identical(vec_ptype(vec_proxy(x)), x[0])
  expect_identical(vec_data(x), x@.Data)

  expect_false(isS4(vec_data(x)))

  expect_s4_class(vec_restore(vec_data(x), x), "vctrs_rando")
  expect_true(isS4(vec_restore(vec_data(x), x)))
})

test_that("as_not_s4() copies and works", {
  # Initial condition
  x <- rando()
  expect_true(isS4(x))

  # Unsetting has no side effect on x
  as_not_s4(x)
  expect_true(isS4(x))

  # Unsetting actually works
  y <- as_not_s4(x)
  expect_false(isS4(y))
})

test_that("vec_restore() works with `NULL`", {
  expect_null(vec_restore(NULL, 1))
})

test_that("default vec_restore() restores attributes except names", {
  to <- structure(NA, foo = "foo", bar = "bar")
  expect_identical(
    vec_restore.default(NA, to),
    structure(NA, foo = "foo", bar = "bar")
  )

  to <- structure(NA, names = "a", foo = "foo", bar = "bar")
  expect_identical(
    vec_restore.default(NA, to),
    structure(NA, foo = "foo", bar = "bar")
  )

  to <- structure(NA, foo = "foo", names = "a", bar = "bar")
  expect_identical(
    vec_restore.default(NA, to),
    structure(NA, foo = "foo", bar = "bar")
  )

  to <- structure(NA, foo = "foo", bar = "bar", names = "a")
  expect_identical(
    vec_restore.default(NA, to),
    structure(NA, foo = "foo", bar = "bar")
  )
})

test_that("default vec_restore() restores / clears objectness", {
  # Non-object -> object
  x <- NA
  to <- structure(NA, class = "foo")
  out <- vec_restore.default(x, to)
  expect_true(is.object(out))
  expect_s3_class(out, "foo")

  # Object -> non-object
  x <- structure(NA, class = "foo")
  to <- NA
  out <- vec_restore.default(x, to)
  expect_false(is.object(out))
  expect_null(attributes(out))
})

test_that("data frame vec_restore() checks type", {
  expect_error(
    vec_restore(NA, mtcars),
    "Attempt to restore data frame from a logical"
  )
})

test_that("data frame restore forces character column names", {
  df <- new_data_frame(list(1))
  expect_named(vec_restore(df, df), "")
})

test_that("can use vctrs primitives from vec_restore() without inflooping", {
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) {
      vec_ptype(x)
      vec_init(x)
      obj_check_vector(x)
      vec_slice(x, 0)
      "woot"
    }
  )

  foobar <- new_vctr(1:3, class = "vctrs_foobar")
  expect_identical(vec_slice(foobar, 2), "woot")
})

test_that("dimensions are preserved by default restore method", {
  x <- foobar(1:4)
  dim(x) <- c(2, 2)
  dimnames(x) <- list(a = c("foo", "bar"), b = c("quux", "hunoz"))

  exp <- foobar(c(1L, 3L))
  dim(exp) <- c(1, 2)
  dimnames(exp) <- list(a = "foo", b = c("quux", "hunoz"))

  expect_identical(vec_slice(x, 1), exp)
})

test_that("names attribute isn't set when restoring 1D arrays using 2D+ objects", {
  x <- foobar(1:2)
  dim(x) <- c(2)
  nms <- c("foo", "bar")
  dimnames(x) <- list(nms)

  res <- vec_restore(x, matrix(1))

  expect_null(attributes(res)$names)
  expect_equal(attr(res, "names"), nms)
  expect_equal(names(res), nms)
})

test_that("arguments are not inlined in the dispatch call (#300)", {
  local_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) sys.call(),
    vec_proxy.vctrs_foobar = unclass
  )
  call <- vec_restore(foobar(list(1)), foobar(list(1)))
  expect_equal(call, quote(vec_restore.vctrs_foobar(x = x, to = to)))
})

test_that("restoring to non-bare data frames calls `vec_bare_df_restore()` before dispatching", {
  x <- list(x = numeric())
  to <- new_data_frame(x, class = "tbl_foobar")

  local_methods(
    vec_restore.tbl_foobar = function(x, to, ..., n) {
      if (is.data.frame(x)) {
        abort(class = "error_df_restore_was_called")
      }
    }
  )

  expect_error(vec_restore(x, to), class = "error_df_restore_was_called")
})

test_that("names of `x` are retained", {
  x <- structure(1, names = "x")

  to <- 2
  expect_identical(vec_restore(x, to), x)

  to <- structure(2, names = "y")
  expect_identical(vec_restore(x, to), x)

  to <- structure(2, foo = "bar")
  expect_identical(vec_restore(x, to), structure(1, names = "x", foo = "bar"))
})

test_that("column names and row names of `x` are retained", {
  x <- data.frame(x = 1, row.names = "foo")

  to <- data_frame(x = 2)
  expect_identical(vec_restore(x, to), x)

  to <- data_frame(x = 2, y = 3)
  expect_identical(vec_restore(x, to), x)

  to <- data.frame(x = 1, row.names = "a")
  expect_identical(vec_restore(x, to), x)

  to <- data_frame(x = 2)
  attr(to, "foo") <- "bar"
  expect <- data.frame(x = 1, row.names = "foo")
  attr(expect, "foo") <- "bar"
  expect_identical(vec_restore(x, to), expect)
})

test_that("dim of `x` is retained", {
  x <- structure(1L, dim = c(1L, 1L))

  to <- 2L
  expect_identical(vec_restore(x, to), x)

  to <- structure(1:2, dim = c(2L, 1L))
  expect_identical(vec_restore(x, to), x)

  to <- structure(2L, foo = "bar")
  expect_identical(
    vec_restore(x, to),
    structure(1L, dim = c(1L, 1L), foo = "bar")
  )
})

test_that("dimnames of `x` are retained", {
  x <- structure(1L, dim = c(1L, 1L), dimnames = list("x", "y"))

  to <- 2L
  expect_identical(vec_restore(x, to), x)

  to <- structure(1:2, dim = c(2L, 1L))
  expect_identical(vec_restore(x, to), x)

  to <- structure(2L, foo = "bar")
  expect_identical(
    vec_restore(x, to),
    structure(1L, dim = c(1L, 1L), dimnames = list("x", "y"), foo = "bar")
  )
})

test_that("row names are not restored if target is not a data frame", {
  proxy <- data.frame(x = 1)
  out <- vec_restore(proxy, to = foobar(""))
  exp <- list(names = "x", class = "vctrs_foobar")
  expect_mapequal(attributes(out), exp)
})

test_that("names / dim / dimnames / row.names of `to` are cleared", {
  exp <- list(foo = TRUE, bar = TRUE)

  # `names`
  to <- structure(list(), foo = TRUE, names = chr(), bar = TRUE)
  out <- vec_restore_default(list(), to)
  expect_mapequal(attributes(out), exp)

  # `names` / `row.names`
  # Was broken by #943
  to <- structure(
    list(),
    foo = TRUE,
    names = chr(),
    row.names = int(),
    bar = TRUE
  )
  out <- vec_restore_default(list(), to)
  expect_mapequal(attributes(out), exp)

  # `dim`
  to <- structure(list(), foo = TRUE, dim = c(1L, 0L), bar = TRUE)
  out <- vec_restore_default(list(), to)
  expect_mapequal(attributes(out), exp)

  # `dim` / `dimnames`
  to <- structure(
    list(),
    foo = TRUE,
    dim = c(1L, 0L),
    dimnames = list("a", character()),
    bar = TRUE
  )
  out <- vec_restore_default(list(), to)
  expect_mapequal(attributes(out), exp)
})

test_that("vec_restore_default() is a no-op with no attributes", {
  x <- 1
  expect_identical(vec_restore(x, x), x)
})

test_that("vec_restore() clears unknown attributes from `x`", {
  # When `to` has no attributes
  x <- structure(1, foo = "bar")
  to <- 2
  expect_identical(vec_restore(x, to), 1)

  # When `to` has attributes
  x <- structure(1, foo = "bar")
  to <- structure(2, a = "b")
  expect_identical(vec_restore(x, to), structure(1, a = "b"))
})

test_that("vec_restore() clears incongruent names / dim / dimnames / rownames attributes", {
  # This is an array and should not have `names`
  x <- structure(1, names = "x", dim = c(1L, 1L))
  to <- 2
  expect_identical(vec_restore(x, to), structure(1, dim = c(1L, 1L)))

  # This is an array and should not have `row.names`
  x <- structure(1, row.names = "x", dim = c(1L, 1L))
  to <- 2
  expect_identical(vec_restore(x, to), structure(1, dim = c(1L, 1L)))

  # R won't even let you make `x`! So no need to test this case.
  # # This is a vector and should not have `dimnames`
  # x <- structure(1, names = "x", dimnames = list("a"))
  # to <- 2
  # expect_identical(vec_restore(x, to), structure(1, names = "x"))
})

test_that("names<- is not called with partial data (#1108)", {
  x <- set_names(foobar(1:2), c("a", "b"))

  values <- list()
  local_methods(
    `names<-.vctrs_foobar` = function(x, value) {
      if (!is_null(value)) {
        values <<- c(values, list(value))
      }
      NextMethod()
    }
  )

  vec_c(x, x)
  expect_equal(values, list(c("a", "b", "a", "b")))
})

test_that("recursive proxy and restore work with recursive records", {
  new_recursive_rcrd <- function(x) {
    new_rcrd(
      list(field = x),
      class = "my_recursive_rcrd"
    )
  }

  internal <- new_rcrd(list(internal_field = 1:2))
  x <- new_recursive_rcrd(data_frame(col = internal))

  proxy <- vec_proxy_recurse(x)
  exp <- data_frame(field = data_frame(col = data_frame(internal_field = 1:2)))
  expect_equal(proxy, exp)
  expect_equal(vec_restore_recurse(proxy, x), x)

  # Non-recursive case doesn't proxy `internal`
  proxy <- vec_proxy(x)
  exp <- data_frame(field = data_frame(col = internal))
  expect_equal(proxy, exp)
  expect_equal(vec_restore(proxy, x), x)

  x_exp <- new_recursive_rcrd(data_frame(col = vec_rep(internal, 2)))
  expect_equal(
    list_unchop(list(x, x)),
    x_exp
  )

  df <- data_frame(x = x)
  df_exp <- data_frame(x = x_exp)
  expect_equal(vec_rbind(df, df), df_exp)
  expect_equal(vec_c(df, df), df_exp)
})

test_that("generics are extensible", {
  expect_error(vec_cast(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_restore(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy_equal(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_proxy_compare(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype2(NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_abbr(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_full(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_arith(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_ptype_finalise(NA, NA), class = "rlib_error_dots_nonempty")
  expect_error(vec_assign(NA, NA, NA, NA), class = "rlib_error_dots_nonempty")
})

# These return raw vectors of hashes. Vector elements are coded with
# 32 bit hashes. Thus, the size of the raw vector of hashes is 4 times
# the size of the input.

vec_hash <- function(x) {
  .Call(ffi_vec_hash, x)
}

obj_hash <- function(x) {
  .Call(ffi_obj_hash, x)
}

#' Group overlapping intervals
#'
#' @description
#' These functions are used to group together any overlaps that are present
#' within a set of vector intervals. When multiple overlapping intervals are
#' grouped together they result in a wider interval containing the smallest
#' `start` and the largest `end` of the overlaps.
#'
#' - `vec_interval_groups()` merges all overlapping intervals found within
#' `start` and `end`. The resulting intervals are known as the interval
#' "groups".
#'
#' - `vec_interval_locate_groups()` returns a two column data frame with a `key`
#' column containing the result of `vec_interval_groups()` and a `loc`
#' list-column containing integer vectors that map each interval in `start` and
#' `end` to the group that it falls in.
#'
#' These functions require that `start < end`. Additionally, intervals are
#' treated as if they are right-open, i.e. `[start, end)`.
#'
#' @section Assumptions:
#' For performance and simplicity, these functions make a few assumptions about
#' `start` and `end` that are not checked internally:
#'
#' - `start < end` must be true, with an exception for missing intervals.
#'
#' - If the i-th observation of `start` is missing, then the i-th observation
#' of `end` must also be missing.
#'
#' - Each observation of `start` and `end` must be either
#' [complete][vec_detect_complete] or [missing][vec_detect_missing]. Partially
#' complete values such as `start = data_frame(x = 1, y = NA)` are not allowed.
#'
#' If any of these assumptions are invalid, then the result is undefined.
#'
#' Developer note: These assumptions stem from the idea that if these functions
#' were in ivs itself, then we could safely make these assumptions in the C
#' code, because the `iv()` helper would assert them for us ahead of time.
#' Trying to re-assert these checks in the C code here is wasteful and makes the
#' code more complex.
#'
#' @inheritParams rlang::args_dots_empty
#'
#' @param start,end
#'   A pair of vectors representing the starts and ends of the intervals.
#'
#'   It is required that `start < end`.
#'
#'   `start` and `end` will be cast to their common type, and must have the same
#'   size.
#'
#' @param abutting
#'   A single logical controlling whether or not abutting intervals should be
#'   grouped together. If `TRUE`, `[a, b)` and `[b, c)` will be grouped.
#'
#' @param missing
#'   Handling of missing intervals.
#'
#'   - `"group"`: Group all missing intervals together.
#'
#'   - `"drop"`: Drop all missing intervals from the result.
#'
#' @return
#' - `vec_interval_groups()` returns a data frame with two columns, `start` and
#' `end`, which contain vectors matching the types of `start` and `end`.
#'
#' - `vec_interval_locate_groups()` returns a data frame with two columns, `key`
#' and `loc`. `key` contains the result of `vec_interval_groups()` and `loc` is
#' a list of integer vectors.
#'
#' @name interval-groups
#'
#' @examples
#' bounds <- data_frame(
#'   start = c(1, 2, NA, 5, NA, 9, 12),
#'   end = c(5, 3, NA, 6, NA, 12, 14)
#' )
#' bounds
#'
#' # Group overlapping intervals together
#' vec_interval_groups(bounds$start, bounds$end)
#'
#' # You can choose not to group abutting intervals if you want to retain
#' # those boundaries
#' vec_interval_groups(bounds$start, bounds$end, abutting = FALSE)
#'
#' # You can also choose to drop all missing intervals if you don't consider
#' # them part of the result
#' vec_interval_groups(bounds$start, bounds$end, missing = "drop")
#'
#' # You can also locate the groups, which allows you to map each original
#' # interval to its corresponding group
#' vec_interval_locate_groups(bounds$start, bounds$end)
#'
#' @noRd
vec_interval_groups <- function(
  start,
  end,
  ...,
  abutting = TRUE,
  missing = "group"
) {
  check_dots_empty0(...)
  .Call(ffi_interval_groups, start, end, abutting, missing)
}

#' @noRd
#' @rdname interval-groups
vec_interval_locate_groups <- function(
  start,
  end,
  ...,
  abutting = TRUE,
  missing = "group"
) {
  check_dots_empty0(...)
  .Call(ffi_interval_locate_groups, start, end, abutting, missing)
}

# ------------------------------------------------------------------------------

#' Interval complement
#'
#' @description
#' `vec_interval_complement()` takes the complement of the intervals defined by
#' `start` and `end`. The complement can also be thought of as the "gaps"
#' between the intervals. By default, the minimum of `start` and the maximum of
#' `end` define the bounds to take the complement over, but this can be adjusted
#' with `lower` and `upper`. Missing intervals are always dropped from the
#' complement.
#'
#' These functions require that `start < end`. Additionally, intervals are
#' treated as if they are right-open, i.e. `[start, end)`.
#'
#' @inheritSection interval-groups Assumptions
#'
#' @inheritParams rlang::args_dots_empty
#'
#' @param start,end
#'   A pair of vectors representing the starts and ends of the intervals.
#'
#'   It is required that `start < end`.
#'
#'   `start` and `end` will be cast to their common type, and must have the same
#'   size.
#'
#' @param lower,upper
#'   Bounds for the universe over which to compute the complement. These should
#'   be singular values with the same type as `start` and `end`.
#'
#' @return
#' A two column data frame with a `start` column containing a vector of the
#' same type as `start` and an `end` column containing a vector of the same
#' type as `end`.
#'
#' @examples
#' x <- data_frame(
#'   start = c(10, 0, NA, 3, -5, NA),
#'   end = c(12, 5, NA, 6, -2, NA)
#' )
#' x
#'
#' # The complement contains any values from `[-5, 12)` that aren't represented
#' # in these intervals. Missing intervals are dropped.
#' vec_interval_complement(x$start, x$end)
#'
#' # Expand out the "universe" of possible values
#' vec_interval_complement(x$start, x$end, lower = -Inf)
#' vec_interval_complement(x$start, x$end, lower = -Inf, upper = Inf)
#'
#' @noRd
vec_interval_complement <- function(
  start,
  end,
  ...,
  lower = NULL,
  upper = NULL
) {
  check_dots_empty0(...)
  .Call(ffi_interval_complement, start, end, lower, upper)
}

# ------------------------------------------------------------------------------

#' Interval containers
#'
#' @description
#' `vec_interval_locate_containers()` locates interval _containers_. Containers
#' are defined as the widest intervals that aren't contained by any other
#' interval. The returned locations will arrange the containers in ascending
#' order.
#'
#' For example, with the following vector of intervals: `[1, 5), [2, 6), [3, 4),
#' [5, 9), [5, 8)`, the containers are: `[1, 5), [2, 6), [5, 9)`. The intervals
#' `[3, 4)` and `[5, 8)` aren't containers because they are completely contained
#' within at least one other interval. Note that containers can partially
#' overlap, i.e. `[1, 5)` and `[2, 6)`, and multiple containers can contain the
#' same intervals, i.e. both `[1, 5)` and `[2, 6)` contain `[3, 4)`.
#'
#' Missing intervals are placed into their own container at the end, separate
#' from all other intervals.
#'
#' These functions require that `start < end`. Additionally, intervals are
#' treated as if they are right-open, i.e. `[start, end)`.
#'
#' @inheritSection interval-groups Assumptions
#'
#' @param start,end
#'   A pair of vectors representing the starts and ends of the intervals.
#'
#'   It is required that `start < end`.
#'
#'   `start` and `end` will be cast to their common type, and must have the same
#'   size.
#'
#' @return
#' An integer vector that represents the locations of the containers in `start`
#' and `end`.
#'
#' @examples
#' x <- data_frame(
#'   start = c(10, 0, NA, 3, 2, 2, NA, 11),
#'   end = c(12, 5, NA, 5, 6, 6, NA, 12)
#' )
#' x
#'
#' loc <- vec_interval_locate_containers(x$start, x$end)
#' loc
#'
#' vec_slice(x, loc)
#'
#' @noRd
vec_interval_locate_containers <- function(start, end) {
  .Call(ffi_interval_locate_containers, start, end)
}

# ------------------------------------------------------------------------------

# Experimental shims of interval functions used by other packages (mainly, ivs).
#
# This gives us the freedom to experiment with the signature of these functions
# while being backwards compatible with ivs in the meantime.
#
# We can remove these after:
# - The interval functions are exported
# - ivs updates to use them directly
# - A short deprecation period goes by that allows users time to update their
#   version of ivs

exp_vec_interval_groups <- function(
  start,
  end,
  ...,
  abutting = TRUE,
  missing = "group"
) {
  vec_interval_groups(
    start = start,
    end = end,
    ...,
    abutting = abutting,
    missing = missing
  )
}

exp_vec_interval_locate_groups <- function(
  start,
  end,
  ...,
  abutting = TRUE,
  missing = "group"
) {
  vec_interval_locate_groups(
    start = start,
    end = end,
    ...,
    abutting = abutting,
    missing = missing
  )
}

exp_vec_interval_complement <- function(
  start,
  end,
  ...,
  lower = NULL,
  upper = NULL
) {
  vec_interval_complement(
    start = start,
    end = end,
    ...,
    lower = lower,
    upper = upper
  )
}

exp_vec_interval_locate_containers <- function(start, end) {
  vec_interval_locate_containers(
    start = start,
    end = end
  )
}

# ------------------------------------------------------------------------------
# vec_interval_groups()

test_that("can compute groups", {
  x <- data_frame(
    start = c(1L, 9L,  2L, 2L, 10L),
    end = c(5L, 11L, 6L, 8L, 12L)
  )

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(1L, 9L), end = c(8L, 12L))
  )
})

test_that("can group with size one input", {
  x <- data_frame(start = 1L, end = 2L)

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )
})

test_that("can group with size zero input", {
  x <- data_frame(start = integer(), end = integer())

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x[1,]
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(2, NA), end = c(5, NA))
  )
})

test_that("missing intervals can be dropped", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = logical(), end = logical())
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = logical(), end = logical())
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = 2, end = 5)
  )
})

test_that("max endpoint is retained even if it isn't the last in the group", {
  # 10 is max end of first group, but 5 is last value in that group
  x <- data_frame(start = c(1L, 2L, 12L), end = c(10L, 5L, 15L))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(1L, 12L), end = c(10L, 15L))
  )
})

# ------------------------------------------------------------------------------
# vec_interval_locate_groups()

test_that("can locate groups", {
  x <- data_frame(
    start = c(1L, 9L,  2L, 2L, 10L),
    end = c(5L, 11L, 6L, 8L, 12L)
  )

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = c(1L, 9L), end = c(8L, 12L))
  )

  expect_identical(
    out$loc,
    list(c(1L, 3L, 4L), c(2L, 5L))
  )
})

test_that("can locate groups with size one input", {
  expect_identical(
    vec_interval_locate_groups(1L, 2L),
    data_frame(
      key = data_frame(start = 1L, end = 2L),
      loc = list(1L)
    )
  )
})

test_that("can locate groups with size zero input", {
  expect_identical(
    vec_interval_locate_groups(integer(), integer()),
    data_frame(
      key = data_frame(start = integer(), end = integer()),
      loc = list()
    )
  )
})

test_that("locations are ordered by both `start` and `end`", {
  x <- data_frame(start = c(4L, 4L, 1L), end = c(6L, 5L, 2L))

  out <- vec_interval_locate_groups(x$start, x$end)

  # Ties of `start = 4` are broken by `end` values and reordered
  expect_identical(
    out$loc,
    list(3L, c(2L, 1L))
  )

  # So this orders `x`
  expect_identical(
    vec_slice(x, unlist(out$loc)),
    vec_sort(x)
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = NA, end = NA)
  )
  expect_identical(
    out$loc,
    list(1L)
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = c(2, NA), end = c(5, NA))
  )
  expect_identical(
    out$loc,
    list(c(3L, 1L), c(2L, 4L)),
  )
})

test_that("missing intervals can be dropped", {
  x <- data_frame(start = NA, end = NA)

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = logical(), end = logical())
  )
  expect_identical(
    out$loc,
    list()
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = 2, end = 5)
  )
  expect_identical(
    out$loc,
    list(c(3L, 1L)),
  )
})

test_that("treats NA and NaN as equivalent with doubles", {
  x <- data_frame(start = c(NA, NaN, NA, NaN), end = c(NA, NA, NaN, NaN))

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = NA_real_, end = NaN)
  )
  expect_identical(
    out$loc,
    list(1:4),
  )

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = double(), end = double())
  )
  expect_identical(
    out$loc,
    list(),
  )
})

test_that("recognizes missing rows in data frames", {
  start <- data_frame(year = c(2019, NA, NA, 2019, 2019), month = c(12, NA, NA, 12, 12))
  end <- data_frame(year = c(2020, NA, NA, 2020, 2020), month = c(2, NA, NA, 11, 12))
  x <- data_frame(start = start, end = end)

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_start <- data_frame(year = c(2019, NA), month = c(12, NA))
  expect_end <- data_frame(year = c(2020, NA), month = c(12, NA))
  expect <- data_frame(start = expect_start, end = expect_end)

  expect_identical(out$key, expect)
  expect_identical(out$loc, list(c(1L, 4L, 5L), c(2L, 3L)))
})

test_that("works on various types", {
  x <- data_frame(start = c(1.5, 2, 3.1, NA), end = c(1.7, 3.2, 4.5, NA))

  out <- vec_interval_locate_groups(x$start, x$end)
  expect_identical(out$key, data_frame(start = c(1.5, 2, NA), end = c(1.7, 4.5, NA)))
  expect_identical(out$loc, list(1L, 2:3, 4L))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")
  expect_identical(out$key, data_frame(start = c(1.5, 2), end = c(1.7, 4.5)))
  expect_identical(out$loc, list(1L, 2:3))

  x <- data_frame(start = c("a", "c", "f", NA), end = c("b", "g", "h", NA))

  out <- vec_interval_locate_groups(x$start, x$end)
  expect_identical(out$key, data_frame(start = c("a", "c", NA), end = c("b", "h", NA)))
  expect_identical(out$loc, list(1L, 2:3, 4L))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")
  expect_identical(out$key, data_frame(start = c("a", "c"), end = c("b", "h")))
  expect_identical(out$loc, list(1L, 2:3))
})

test_that("can keep abutting intervals separate", {
  # after
  x <- data_frame(start = c(1L, 2L, 0L), end = c(2L, 3L, 2L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(out$key, data_frame(start = c(0L, 2L), end = c(2L, 3L)))
  expect_identical(out$loc, list(c(3L, 1L), 2L))

  # before
  x <- data_frame(start = c(1L, 0L), end = c(2L, 1L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(out$key, data_frame(start = c(0L, 1L), end = c(1L, 2L)))
  expect_identical(out$loc, list(2L, 1L))

  # both
  x <- data_frame(start = c(1L, 0L, 2L), end = c(2L, 1L, 3L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(out$key, data_frame(start = c(0L, 1L, 2L), end = c(1L, 2L, 3L)))
  expect_identical(out$loc, list(2L, 1L, 3L))
})

test_that("`missing` is validated", {
  expect_snapshot((expect_error(vec_interval_locate_groups(1, 2, missing = "s"))))
  expect_snapshot((expect_error(vec_interval_locate_groups(1, 2, missing = c("group", "drop")))))
})

test_that("common type is taken", {
  expect_snapshot((expect_error(vec_interval_locate_groups(1, "x"))))
})

# ------------------------------------------------------------------------------
# vec_interval_complement()

test_that("computes the complement", {
  x <- data_frame(
    start = c(6L, 1L, 2L, 12L),
    end = c(9L, 3L, 4L, 14L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = c(4L, 9L), end = c(6L, 12L))
  )
})

test_that("treats intervals as half-open like [a, b)", {
  x <- data_frame(
    start = c(1L, 5L),
    end = c(4L, 6L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = 4L, end = 5L)
  )
})

test_that("`[a, b)` and `[b, c)` result in no complement values", {
  x <- data_frame(
    start = c(1L, 5L),
    end = c(5L, 6L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with `lower == upper`", {
  x <- data_frame(
    start = c(1L, 2L, 12L, NA),
    end = c(10L, 5L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10L, upper = 10L),
    data_frame(start = integer(), end = integer())
  )
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -1L, upper = -1L),
    data_frame(start = integer(), end = integer())
  )
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 20L, upper = 20L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with `lower` before any values", {
  x <- data_frame(
    start = c(1L, 2L, 12L, NA),
    end = c(10L, 5L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -1L),
    data_frame(start = c(-1L, 10L), end = c(1L, 12L))
  )
})

test_that("works if both `lower` and `upper` are before any values", {
  x <- data_frame(
    start = c(2L, 1L, 12L, NA),
    end = c(5L, 10L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -5L, upper = -2L),
    data_frame(start = -5L, end = -2L)
  )
})

test_that("works with `upper` after any values", {
  x <- data_frame(
    start = c(2L, 1L, 13L, 12L, NA),
    end = c(5L, 10L, 17L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, upper = 20L),
    data_frame(start = c(10L, 17L), end = c(12L, 20L))
  )
})

test_that("works if both `lower` and `upper` are after any values", {
  x <- data_frame(
    start = c(2L, 1L, 12L, NA),
    end = c(5L, 10L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 17L, upper = 19L),
    data_frame(start = 17L, end = 19L)
  )
})

test_that("works with only NA and `lower`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(vec_interval_complement(x$start, x$end, lower = 5L), data_frame(start = integer(), end = integer()))
})

test_that("works with only NA and `upper`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(vec_interval_complement(x$start, x$end, upper = 5L), data_frame(start = integer(), end = integer()))
})

test_that("works with only NA and both `lower` and `upper`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(vec_interval_complement(x$start, x$end, lower = 2L, upper = 5L), data_frame(start = 2L, end = 5L))
})

test_that("works with `lower` that is on the max set value", {
  x <- data_frame(
    start = c(1L, 12L),
    end = c(9L, 13L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 9L),
    data_frame(start = 9L, end = 12L)
  )
})

test_that("works with `upper` that is on the max set value", {
  x <- data_frame(
    start = c(-5L, 1L, 2L, 12L),
    end = c(0L, 10L, 5L, 15L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, upper = 10L),
    data_frame(start = 0L, end = 1L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10L, upper = 10L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("size zero case generally returns nothing", {
  expect_identical(
    vec_interval_complement(integer(), integer()),
    data_frame(start = integer(), end = integer())
  )

  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L),
    data_frame(start = integer(), end = integer())
  )

  expect_identical(
    vec_interval_complement(integer(), integer(), upper = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("size zero case with both `lower` and `upper` returns an interval", {
  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L, upper = 10L),
    data_frame(start = 5L, end = 10L)
  )
})

test_that("size zero case with `lower == upper` doesn't return anything", {
  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L, upper = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works when `lower` is contained in an interval", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 3),
    data_frame(start = 5, end = 10)
  )
})

test_that("works when `lower` is in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 7),
    data_frame(start = 7, end = 10)
  )
})

test_that("works when `upper` is in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), upper = 7),
    data_frame(start = c(-3, 5), end = c(1, 7))
  )
})

test_that("works when `lower` and `upper` are in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 6, upper = 7),
    data_frame(start = 6, end = 7)
  )
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 7, upper = 7),
    data_frame(start = double(), end = double())
  )
})

test_that("works when `lower` and `upper` have an interval between them", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 0, upper = 7),
    data_frame(start = c(0, 5), end = c(1, 7))
  )
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = -6, upper = 7),
    data_frame(start = c(-6, -3, 5), end = c(-5, 1, 7))
  )
})

test_that("allow `lower > upper` which returns an empty interval", {
  x <- data_frame(start = c(1, 2), end = c(5, 12))
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10, upper = 9),
    data_frame(start = double(), end = double())
  )
})

test_that("complement works when `lower` and `upper` are in the same interval", {
  x <- data_frame(start = 1, end = 5)

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 2, upper = 4),
    data_frame(start = double(), end = double())
  )
})

test_that("`lower` and `upper` can't contain missing values", {
  expect_snapshot({
    (expect_error(vec_interval_complement(1, 2, lower = NA)))
    (expect_error(vec_interval_complement(1, 2, upper = NA)))

    start <- data_frame(x = 1, y = 2)
    end <- data_frame(x = 1, y = 3)
    (expect_error(vec_interval_complement(start, end, lower = data_frame(x = 1, y = NA))))
    (expect_error(vec_interval_complement(start, end, upper = data_frame(x = 1, y = NA))))
  })
})

# ------------------------------------------------------------------------------
# vec_interval_locate_containers()

test_that("can locate containers", {
  x <- data_frame(
    start = c(1L, 9L,  2L, 2L, 10L),
    end = c(5L, 12L, 6L, 8L, 12L)
  )

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(1L, 4L, 2L)
  )
})

test_that("can locate containers with size one input", {
  x <- data_frame(start = 1L, end = 2L)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )
})

test_that("can locate containers with size zero input", {
  x <- data_frame(start = integer(), end = integer())

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    integer()
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  # Ties use first missing value seen
  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 5, NA))

  # Missing intervals at the end
  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(3L, 2L)
  )
})

test_that("locations order the intervals", {
  x <- data_frame(start = c(4L, 4L, 1L, NA, 4L), end = c(5L, 6L, 2L, NA, 6L))

  out <- vec_interval_locate_containers(x$start, x$end)

  expect_identical(
    out,
    c(3L, 2L, 4L)
  )

  # This orders `x`
  expect_identical(
    vec_slice(x, out),
    vec_sort(vec_slice(x, out))
  )
})

test_that("treats NA and NaN as equivalent with doubles", {
  x <- data_frame(start = c(NA, NaN, NA, NaN), end = c(NA, NA, NaN, NaN))

  expect_identical(vec_interval_locate_containers(x$start, x$end), 1L)
})

test_that("recognizes missing rows in data frames", {
  start <- data_frame(year = c(2019, NA, NA, 2019, 2019), month = c(12, NA, NA, 12, 12))
  end <- data_frame(year = c(2020, NA, NA, 2020, 2020), month = c(2, NA, NA, 11, 12))
  x <- data_frame(start = start, end = end)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(5L, 2L)
  )
})

test_that("duplicate containers return the first", {
  x <- data_frame(start = c(1, 1, 2, 1, 2), end = c(2, 2, 3, 2, 3))
  expect_identical(vec_interval_locate_containers(x$start, x$end), c(1L, 3L))
})

test_that("works on various types", {
  x <- data_frame(start = c(1.5, 3, NA, 1.6, NA), end = c(1.7, 3.1, NA, 3.2, NA))

  out <- vec_interval_locate_containers(x$start, x$end)
  expect_identical(out, c(1L, 4L, 3L))

  x <- data_frame(start = c("a", "a", NA, "f", NA), end = c("b", "g", NA, "h", NA))

  out <- vec_interval_locate_containers(x$start, x$end)
  expect_identical(out, c(2L, 4L, 3L))
})

test_that("common type is taken", {
  expect_snapshot((expect_error(vec_interval_locate_containers(1, "x"))))
})

# ------------------------------------------------------------------------------
# vec_interval_groups()

test_that("can compute groups", {
  x <- data_frame(
    start = c(1L, 9L, 2L, 2L, 10L),
    end = c(5L, 11L, 6L, 8L, 12L)
  )

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(1L, 9L), end = c(8L, 12L))
  )
})

test_that("can group with size one input", {
  x <- data_frame(start = 1L, end = 2L)

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )
})

test_that("can group with size zero input", {
  x <- data_frame(start = integer(), end = integer())

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    x[1, ]
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(2, NA), end = c(5, NA))
  )
})

test_that("missing intervals can be dropped", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = logical(), end = logical())
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = logical(), end = logical())
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  expect_identical(
    vec_interval_groups(x$start, x$end, missing = "drop"),
    data_frame(start = 2, end = 5)
  )
})

test_that("max endpoint is retained even if it isn't the last in the group", {
  # 10 is max end of first group, but 5 is last value in that group
  x <- data_frame(start = c(1L, 2L, 12L), end = c(10L, 5L, 15L))

  expect_identical(
    vec_interval_groups(x$start, x$end),
    data_frame(start = c(1L, 12L), end = c(10L, 15L))
  )
})

# ------------------------------------------------------------------------------
# vec_interval_locate_groups()

test_that("can locate groups", {
  x <- data_frame(
    start = c(1L, 9L, 2L, 2L, 10L),
    end = c(5L, 11L, 6L, 8L, 12L)
  )

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = c(1L, 9L), end = c(8L, 12L))
  )

  expect_identical(
    out$loc,
    list(c(1L, 3L, 4L), c(2L, 5L))
  )
})

test_that("can locate groups with size one input", {
  expect_identical(
    vec_interval_locate_groups(1L, 2L),
    data_frame(
      key = data_frame(start = 1L, end = 2L),
      loc = list(1L)
    )
  )
})

test_that("can locate groups with size zero input", {
  expect_identical(
    vec_interval_locate_groups(integer(), integer()),
    data_frame(
      key = data_frame(start = integer(), end = integer()),
      loc = list()
    )
  )
})

test_that("locations are ordered by both `start` and `end`", {
  x <- data_frame(start = c(4L, 4L, 1L), end = c(6L, 5L, 2L))

  out <- vec_interval_locate_groups(x$start, x$end)

  # Ties of `start = 4` are broken by `end` values and reordered
  expect_identical(
    out$loc,
    list(3L, c(2L, 1L))
  )

  # So this orders `x`
  expect_identical(
    vec_slice(x, unlist(out$loc)),
    vec_sort(x)
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = NA, end = NA)
  )
  expect_identical(
    out$loc,
    list(1L)
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = c(2, NA), end = c(5, NA))
  )
  expect_identical(
    out$loc,
    list(c(3L, 1L), c(2L, 4L)),
  )
})

test_that("missing intervals can be dropped", {
  x <- data_frame(start = NA, end = NA)

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = logical(), end = logical())
  )
  expect_identical(
    out$loc,
    list()
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 3, NA))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = 2, end = 5)
  )
  expect_identical(
    out$loc,
    list(c(3L, 1L)),
  )
})

test_that("treats NA and NaN as equivalent with doubles", {
  x <- data_frame(start = c(NA, NaN, NA, NaN), end = c(NA, NA, NaN, NaN))

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_identical(
    out$key,
    data_frame(start = NA_real_, end = NaN)
  )
  expect_identical(
    out$loc,
    list(1:4),
  )

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")

  expect_identical(
    out$key,
    data_frame(start = double(), end = double())
  )
  expect_identical(
    out$loc,
    list(),
  )
})

test_that("recognizes missing rows in data frames", {
  start <- data_frame(
    year = c(2019, NA, NA, 2019, 2019),
    month = c(12, NA, NA, 12, 12)
  )
  end <- data_frame(
    year = c(2020, NA, NA, 2020, 2020),
    month = c(2, NA, NA, 11, 12)
  )
  x <- data_frame(start = start, end = end)

  out <- vec_interval_locate_groups(x$start, x$end)

  expect_start <- data_frame(year = c(2019, NA), month = c(12, NA))
  expect_end <- data_frame(year = c(2020, NA), month = c(12, NA))
  expect <- data_frame(start = expect_start, end = expect_end)

  expect_identical(out$key, expect)
  expect_identical(out$loc, list(c(1L, 4L, 5L), c(2L, 3L)))
})

test_that("works on various types", {
  x <- data_frame(start = c(1.5, 2, 3.1, NA), end = c(1.7, 3.2, 4.5, NA))

  out <- vec_interval_locate_groups(x$start, x$end)
  expect_identical(
    out$key,
    data_frame(start = c(1.5, 2, NA), end = c(1.7, 4.5, NA))
  )
  expect_identical(out$loc, list(1L, 2:3, 4L))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")
  expect_identical(out$key, data_frame(start = c(1.5, 2), end = c(1.7, 4.5)))
  expect_identical(out$loc, list(1L, 2:3))

  x <- data_frame(start = c("a", "c", "f", NA), end = c("b", "g", "h", NA))

  out <- vec_interval_locate_groups(x$start, x$end)
  expect_identical(
    out$key,
    data_frame(start = c("a", "c", NA), end = c("b", "h", NA))
  )
  expect_identical(out$loc, list(1L, 2:3, 4L))

  out <- vec_interval_locate_groups(x$start, x$end, missing = "drop")
  expect_identical(out$key, data_frame(start = c("a", "c"), end = c("b", "h")))
  expect_identical(out$loc, list(1L, 2:3))
})

test_that("can keep abutting intervals separate", {
  # after
  x <- data_frame(start = c(1L, 2L, 0L), end = c(2L, 3L, 2L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(out$key, data_frame(start = c(0L, 2L), end = c(2L, 3L)))
  expect_identical(out$loc, list(c(3L, 1L), 2L))

  # before
  x <- data_frame(start = c(1L, 0L), end = c(2L, 1L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(out$key, data_frame(start = c(0L, 1L), end = c(1L, 2L)))
  expect_identical(out$loc, list(2L, 1L))

  # both
  x <- data_frame(start = c(1L, 0L, 2L), end = c(2L, 1L, 3L))

  out <- vec_interval_locate_groups(x$start, x$end, abutting = FALSE)

  expect_identical(
    out$key,
    data_frame(start = c(0L, 1L, 2L), end = c(1L, 2L, 3L))
  )
  expect_identical(out$loc, list(2L, 1L, 3L))
})

test_that("`missing` is validated", {
  expect_snapshot(
    (expect_error(vec_interval_locate_groups(1, 2, missing = "s")))
  )
  expect_snapshot(
    (expect_error(vec_interval_locate_groups(
      1,
      2,
      missing = c("group", "drop")
    )))
  )
})

test_that("common type is taken", {
  expect_snapshot((expect_error(vec_interval_locate_groups(1, "x"))))
})

# ------------------------------------------------------------------------------
# vec_interval_complement()

test_that("computes the complement", {
  x <- data_frame(
    start = c(6L, 1L, 2L, 12L),
    end = c(9L, 3L, 4L, 14L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = c(4L, 9L), end = c(6L, 12L))
  )
})

test_that("treats intervals as half-open like [a, b)", {
  x <- data_frame(
    start = c(1L, 5L),
    end = c(4L, 6L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = 4L, end = 5L)
  )
})

test_that("`[a, b)` and `[b, c)` result in no complement values", {
  x <- data_frame(
    start = c(1L, 5L),
    end = c(5L, 6L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with `lower == upper`", {
  x <- data_frame(
    start = c(1L, 2L, 12L, NA),
    end = c(10L, 5L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10L, upper = 10L),
    data_frame(start = integer(), end = integer())
  )
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -1L, upper = -1L),
    data_frame(start = integer(), end = integer())
  )
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 20L, upper = 20L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with `lower` before any values", {
  x <- data_frame(
    start = c(1L, 2L, 12L, NA),
    end = c(10L, 5L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -1L),
    data_frame(start = c(-1L, 10L), end = c(1L, 12L))
  )
})

test_that("works if both `lower` and `upper` are before any values", {
  x <- data_frame(
    start = c(2L, 1L, 12L, NA),
    end = c(5L, 10L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = -5L, upper = -2L),
    data_frame(start = -5L, end = -2L)
  )
})

test_that("works with `upper` after any values", {
  x <- data_frame(
    start = c(2L, 1L, 13L, 12L, NA),
    end = c(5L, 10L, 17L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, upper = 20L),
    data_frame(start = c(10L, 17L), end = c(12L, 20L))
  )
})

test_that("works if both `lower` and `upper` are after any values", {
  x <- data_frame(
    start = c(2L, 1L, 12L, NA),
    end = c(5L, 10L, 15L, NA)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 17L, upper = 19L),
    data_frame(start = 17L, end = 19L)
  )
})

test_that("works with only NA and `lower`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with only NA and `upper`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(
    vec_interval_complement(x$start, x$end, upper = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works with only NA and both `lower` and `upper`", {
  x <- data_frame(start = NA_integer_, end = NA_integer_)
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 2L, upper = 5L),
    data_frame(start = 2L, end = 5L)
  )
})

test_that("works with `lower` that is on the max set value", {
  x <- data_frame(
    start = c(1L, 12L),
    end = c(9L, 13L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 9L),
    data_frame(start = 9L, end = 12L)
  )
})

test_that("works with `upper` that is on the max set value", {
  x <- data_frame(
    start = c(-5L, 1L, 2L, 12L),
    end = c(0L, 10L, 5L, 15L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, upper = 10L),
    data_frame(start = 0L, end = 1L)
  )

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10L, upper = 10L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("size zero case generally returns nothing", {
  expect_identical(
    vec_interval_complement(integer(), integer()),
    data_frame(start = integer(), end = integer())
  )

  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L),
    data_frame(start = integer(), end = integer())
  )

  expect_identical(
    vec_interval_complement(integer(), integer(), upper = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("size zero case with both `lower` and `upper` returns an interval", {
  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L, upper = 10L),
    data_frame(start = 5L, end = 10L)
  )
})

test_that("size zero case with `lower == upper` doesn't return anything", {
  expect_identical(
    vec_interval_complement(integer(), integer(), lower = 5L, upper = 5L),
    data_frame(start = integer(), end = integer())
  )
})

test_that("works when `lower` is contained in an interval", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 3),
    data_frame(start = 5, end = 10)
  )
})

test_that("works when `lower` is in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 7),
    data_frame(start = 7, end = 10)
  )
})

test_that("works when `upper` is in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), upper = 7),
    data_frame(start = c(-3, 5), end = c(1, 7))
  )
})

test_that("works when `lower` and `upper` are in a gap between intervals", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 6, upper = 7),
    data_frame(start = 6, end = 7)
  )
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 7, upper = 7),
    data_frame(start = double(), end = double())
  )
})

test_that("works when `lower` and `upper` have an interval between them", {
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = 0, upper = 7),
    data_frame(start = c(0, 5), end = c(1, 7))
  )
  expect_identical(
    vec_interval_complement(c(-5, 1, 10), c(-3, 5, 15), lower = -6, upper = 7),
    data_frame(start = c(-6, -3, 5), end = c(-5, 1, 7))
  )
})

test_that("allow `lower > upper` which returns an empty interval", {
  x <- data_frame(start = c(1, 2), end = c(5, 12))
  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 10, upper = 9),
    data_frame(start = double(), end = double())
  )
})

test_that("complement works when `lower` and `upper` are in the same interval", {
  x <- data_frame(start = 1, end = 5)

  expect_identical(
    vec_interval_complement(x$start, x$end, lower = 2, upper = 4),
    data_frame(start = double(), end = double())
  )
})

test_that("`lower` and `upper` can't contain missing values", {
  expect_snapshot({
    (expect_error(vec_interval_complement(1, 2, lower = NA)))
    (expect_error(vec_interval_complement(1, 2, upper = NA)))

    start <- data_frame(x = 1, y = 2)
    end <- data_frame(x = 1, y = 3)
    (expect_error(vec_interval_complement(
      start,
      end,
      lower = data_frame(x = 1, y = NA)
    )))
    (expect_error(vec_interval_complement(
      start,
      end,
      upper = data_frame(x = 1, y = NA)
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_interval_locate_containers()

test_that("can locate containers", {
  x <- data_frame(
    start = c(1L, 9L, 2L, 2L, 10L),
    end = c(5L, 12L, 6L, 8L, 12L)
  )

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(1L, 4L, 2L)
  )
})

test_that("can locate containers with size one input", {
  x <- data_frame(start = 1L, end = 2L)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )
})

test_that("can locate containers with size zero input", {
  x <- data_frame(start = integer(), end = integer())

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    integer()
  )
})

test_that("missing intervals are retained", {
  x <- data_frame(start = NA, end = NA)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )

  x <- data_frame(start = c(NA, NA), end = c(NA, NA))

  # Ties use first missing value seen
  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    1L
  )

  x <- data_frame(start = c(3, NA, 2, NA), end = c(5, NA, 5, NA))

  # Missing intervals at the end
  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(3L, 2L)
  )
})

test_that("locations order the intervals", {
  x <- data_frame(start = c(4L, 4L, 1L, NA, 4L), end = c(5L, 6L, 2L, NA, 6L))

  out <- vec_interval_locate_containers(x$start, x$end)

  expect_identical(
    out,
    c(3L, 2L, 4L)
  )

  # This orders `x`
  expect_identical(
    vec_slice(x, out),
    vec_sort(vec_slice(x, out))
  )
})

test_that("treats NA and NaN as equivalent with doubles", {
  x <- data_frame(start = c(NA, NaN, NA, NaN), end = c(NA, NA, NaN, NaN))

  expect_identical(vec_interval_locate_containers(x$start, x$end), 1L)
})

test_that("recognizes missing rows in data frames", {
  start <- data_frame(
    year = c(2019, NA, NA, 2019, 2019),
    month = c(12, NA, NA, 12, 12)
  )
  end <- data_frame(
    year = c(2020, NA, NA, 2020, 2020),
    month = c(2, NA, NA, 11, 12)
  )
  x <- data_frame(start = start, end = end)

  expect_identical(
    vec_interval_locate_containers(x$start, x$end),
    c(5L, 2L)
  )
})

test_that("duplicate containers return the first", {
  x <- data_frame(start = c(1, 1, 2, 1, 2), end = c(2, 2, 3, 2, 3))
  expect_identical(vec_interval_locate_containers(x$start, x$end), c(1L, 3L))
})

test_that("works on various types", {
  x <- data_frame(
    start = c(1.5, 3, NA, 1.6, NA),
    end = c(1.7, 3.1, NA, 3.2, NA)
  )

  out <- vec_interval_locate_containers(x$start, x$end)
  expect_identical(out, c(1L, 4L, 3L))

  x <- data_frame(
    start = c("a", "a", NA, "f", NA),
    end = c("b", "g", NA, "h", NA)
  )

  out <- vec_interval_locate_containers(x$start, x$end)
  expect_identical(out, c(2L, 4L, 3L))
})

test_that("common type is taken", {
  expect_snapshot((expect_error(vec_interval_locate_containers(1, "x"))))
})

#' Runs
#'
#' @description
#' - `vec_identify_runs()` returns a vector of identifiers for the elements of
#'   `x` that indicate which run of repeated values they fall in. The number of
#'   runs is also returned as an attribute, `n`.
#'
#' - `vec_run_sizes()` returns an integer vector corresponding to the size of
#'   each run. This is identical to the `times` column from `vec_unrep()`, but
#'   is faster if you don't need the run keys.
#'
#' - [vec_unrep()] is a generalized [base::rle()]. It is documented alongside
#'   the "repeat" functions of [vec_rep()] and [vec_rep_each()]; look there for
#'   more information.
#'
#' @details
#' Unlike [base::rle()], adjacent missing values are considered identical when
#' constructing runs. For example, `vec_identify_runs(c(NA, NA))` will return
#' `c(1, 1)`, not `c(1, 2)`.
#'
#' @param x A vector.
#'
#' @return
#' - For `vec_identify_runs()`, an integer vector with the same size as `x`. A
#'   scalar integer attribute, `n`, is attached.
#'
#' - For `vec_run_sizes()`, an integer vector with size equal to the number of
#'   runs in `x`.
#'
#' @seealso
#' [vec_unrep()] for a generalized [base::rle()].
#'
#' @name runs
#' @examples
#' x <- c("a", "z", "z", "c", "a", "a")
#'
#' vec_identify_runs(x)
#' vec_run_sizes(x)
#' vec_unrep(x)
#'
#' y <- c(1, 1, 1, 2, 2, 3)
#'
#' # With multiple columns, the runs are constructed rowwise
#' df <- data_frame(
#'   x = x,
#'   y = y
#' )
#'
#' vec_identify_runs(df)
#' vec_run_sizes(df)
#' vec_unrep(df)
NULL

#' @rdname runs
#' @export
vec_identify_runs <- function(x) {
  .Call(ffi_vec_identify_runs, x, environment())
}

#' @rdname runs
#' @export
vec_run_sizes <- function(x) {
  .Call(ffi_vec_run_sizes, x, environment())
}

vec_locate_run_bounds <- function(x, which = c("start", "end")) {
  .Call(ffi_vec_locate_run_bounds, x, which, environment())
}

vec_detect_run_bounds <- function(x, which = c("start", "end")) {
  .Call(ffi_vec_detect_run_bounds, x, which, environment())
}

# vec_identify_runs ------------------------------------------------------------

test_that("vec_identify_runs() works with size zero input", {
  expect <- structure(integer(), n = 0L)

  expect_identical(vec_identify_runs(integer()), expect)
  expect_identical(vec_identify_runs(data.frame()), expect)
})

test_that("works with atomic input of various types", {
  expect <- structure(c(1L, 1L, 2L, 2L, 3L), n = 3L)

  expect_identical(vec_identify_runs(c(TRUE, TRUE, FALSE, FALSE, TRUE)), expect)
  expect_identical(vec_identify_runs(c(1L, 1L, 2L, 2L, 3L)), expect)
  expect_identical(vec_identify_runs(c(1, 1, 2, 2, 3)), expect)
  expect_identical(vec_identify_runs(complex(real = c(1, 1, 2, 2, 2), imaginary = c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_identify_runs(c("a", "a", "b", "b", "c")), expect)
  expect_identical(vec_identify_runs(as.raw(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_identify_runs(list(1, 1, 2, 2, 3)), expect)
})

test_that("NA values are identical", {
  expect <- structure(c(1L, 1L), n = 1L)

  expect_identical(vec_identify_runs(c(NA, NA)), expect)
  expect_identical(vec_identify_runs(c(NA_integer_, NA_integer_)), expect)
  expect_identical(vec_identify_runs(c(NA_real_, NA_real_)), expect)
  expect_identical(vec_identify_runs(c(NA_complex_, NA_complex_)), expect)
  expect_identical(vec_identify_runs(c(NA_character_, NA_character_)), expect)
  # No NA type for raw
  expect_identical(vec_identify_runs(list(NULL, NULL)), expect)
})

test_that("NA and NaN are different", {
  expect <- structure(c(1L, 2L), n = 2L)
  expect_identical(vec_identify_runs(c(NA_real_, NaN)), expect)
})

test_that("normalizes character encodings", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1)
  expect_identical(vec_identify_runs(x), structure(rep(1L, 3), n = 1L))
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_identify_runs(foobar())
  })
})

test_that("works with data frames rowwise", {
  df <- data_frame(x = c(1, 1, 1, 2), y = c(1, 1, 2, 3))
  expect <- structure(c(1L, 1L, 2L, 3L), n = 3L)
  expect_identical(vec_identify_runs(df), expect)

  df <- data_frame(x = c(1, 1, 1), y = c(2, 2, 2), z = c("b", "a", "a"))
  expect <- structure(c(1L, 2L, 2L), n = 2L)
  expect_identical(vec_identify_runs(df), expect)
})

test_that("works with data frames with rows but no columns", {
  expect <- structure(rep(1L, 5), n = 1L)
  expect_identical(vec_identify_runs(new_data_frame(n = 5L)), expect)
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, 1, 2, 2), b = c(1, 2, 3, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect <- structure(c(1L, 2L, 3L, 3L), n = 3L)
  expect_identical(vec_identify_runs(df), expect)
})

test_that("works with columns of various types", {
  # Use two columns to keep the data frame from being squashed to a vector
  add_col <- function(col) {
    x <- rep(1L, 5)
    data_frame(x = x, y = col)
  }

  expect <- structure(c(1L, 1L, 2L, 2L, 3L), n = 3L)

  expect_identical(vec_identify_runs(add_col(c(TRUE, TRUE, FALSE, FALSE, TRUE))), expect)
  expect_identical(vec_identify_runs(add_col(c(1L, 1L, 2L, 2L, 3L))), expect)
  expect_identical(vec_identify_runs(add_col(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_identify_runs(add_col(complex(real = c(1, 1, 2, 2, 2), imaginary = c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_identify_runs(add_col(c("a", "a", "b", "b", "c"))), expect)
  expect_identical(vec_identify_runs(add_col(as.raw(c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_identify_runs(add_col(list(1, 1, 2, 2, 3))), expect)
})

# vec_run_sizes ----------------------------------------------------------------

test_that("vec_run_sizes() works with size zero input", {
  expect_identical(vec_run_sizes(integer()), integer())
  expect_identical(vec_run_sizes(data.frame()), integer())
})

test_that("works with atomic input of various types", {
  expect <- c(2L, 2L, 1L)

  expect_identical(vec_run_sizes(c(TRUE, TRUE, FALSE, FALSE, TRUE)), expect)
  expect_identical(vec_run_sizes(c(1L, 1L, 2L, 2L, 3L)), expect)
  expect_identical(vec_run_sizes(c(1, 1, 2, 2, 3)), expect)
  expect_identical(vec_run_sizes(complex(real = c(1, 1, 2, 2, 2), imaginary = c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_run_sizes(c("a", "a", "b", "b", "c")), expect)
  expect_identical(vec_run_sizes(as.raw(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_run_sizes(list(1, 1, 2, 2, 3)), expect)
})

test_that("NA values are identical", {
  expect <- 2L

  expect_identical(vec_run_sizes(c(NA, NA)), expect)
  expect_identical(vec_run_sizes(c(NA_integer_, NA_integer_)), expect)
  expect_identical(vec_run_sizes(c(NA_real_, NA_real_)), expect)
  expect_identical(vec_run_sizes(c(NA_complex_, NA_complex_)), expect)
  expect_identical(vec_run_sizes(c(NA_character_, NA_character_)), expect)
  # No NA type for raw
  expect_identical(vec_run_sizes(list(NULL, NULL)), expect)
})

test_that("NA and NaN are different", {
  expect_identical(vec_run_sizes(c(NA_real_, NaN)), c(1L, 1L))
})

test_that("normalizes character encodings", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1)
  expect_identical(vec_run_sizes(x), 3L)
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_run_sizes(foobar())
  })
})

test_that("works with data frames rowwise", {
  df <- data_frame(x = c(1, 1, 1, 2), y = c(1, 1, 2, 3))
  expect_identical(vec_run_sizes(df), c(2L, 1L, 1L))

  df <- data_frame(x = c(1, 1, 1), y = c(2, 2, 2), z = c("b", "a", "a"))
  expect_identical(vec_run_sizes(df), c(1L, 2L))
})

test_that("works with data frames with rows but no columns", {
  expect_identical(vec_run_sizes(new_data_frame(n = 5L)), 5L)
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, 1, 2, 2), b = c(1, 2, 3, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect_identical(vec_run_sizes(df), c(1L, 1L, 2L))
})

test_that("works with columns of various types", {
  # Use two columns to keep the data frame from being squashed to a vector
  add_col <- function(col) {
    x <- rep(1L, 5)
    data_frame(x = x, y = col)
  }

  expect <- c(2L, 2L, 1L)

  expect_identical(vec_run_sizes(add_col(c(TRUE, TRUE, FALSE, FALSE, TRUE))), expect)
  expect_identical(vec_run_sizes(add_col(c(1L, 1L, 2L, 2L, 3L))), expect)
  expect_identical(vec_run_sizes(add_col(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_run_sizes(add_col(complex(real = c(1, 1, 2, 2, 2), imaginary = c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_run_sizes(add_col(c("a", "a", "b", "b", "c"))), expect)
  expect_identical(vec_run_sizes(add_col(as.raw(c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_run_sizes(add_col(list(1, 1, 2, 2, 3))), expect)
})

# vec_locate_run_bounds --------------------------------------------------------

test_that("can locate run starts", {
  expect_identical(
    vec_locate_run_bounds(c(1, 3, 3, 1, 5, 5, 6)),
    c(1L, 2L, 4L, 5L, 7L)
  )
})

test_that("can locate run ends", {
  expect_identical(
    vec_locate_run_bounds(c(1, 3, 3, 1, 5, 5, 6), which = "end"),
    c(1L, 3L, 4L, 6L, 7L)
  )
})

test_that("vec_locate_run_bounds() works with size zero input", {
  expect_identical(vec_locate_run_bounds(integer(), which = "start"), integer())
  expect_identical(vec_locate_run_bounds(integer(), which = "end"), integer())
})

test_that("vec_locate_run_bounds() validates `which`", {
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = c("foo", "bar"))
  })
})

# vec_detect_run_bounds --------------------------------------------------------

test_that("can detect run starts", {
  expect_identical(
    vec_detect_run_bounds(c(1, 3, 3, 1, 5, 5, 6)),
    c(TRUE, TRUE, FALSE, TRUE, TRUE, FALSE, TRUE)
  )
})

test_that("can detect run ends", {
  expect_identical(
    vec_detect_run_bounds(c(1, 3, 3, 1, 5, 5, 6), which = "end"),
    c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE, TRUE)
  )
})

test_that("vec_detect_run_bounds() works with size zero input", {
  expect_identical(vec_detect_run_bounds(integer(), which = "start"), logical())
  expect_identical(vec_detect_run_bounds(integer(), which = "end"), logical())
})

test_that("vec_detect_run_bounds() validates `which`", {
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = c("foo", "bar"))
  })
})

# vec_identify_runs ------------------------------------------------------------

test_that("vec_identify_runs() works with size zero input", {
  expect <- structure(integer(), n = 0L)

  expect_identical(vec_identify_runs(integer()), expect)
  expect_identical(vec_identify_runs(data.frame()), expect)
})

test_that("works with atomic input of various types", {
  expect <- structure(c(1L, 1L, 2L, 2L, 3L), n = 3L)

  expect_identical(vec_identify_runs(c(TRUE, TRUE, FALSE, FALSE, TRUE)), expect)
  expect_identical(vec_identify_runs(c(1L, 1L, 2L, 2L, 3L)), expect)
  expect_identical(vec_identify_runs(c(1, 1, 2, 2, 3)), expect)
  expect_identical(
    vec_identify_runs(complex(
      real = c(1, 1, 2, 2, 2),
      imaginary = c(1, 1, 2, 2, 3)
    )),
    expect
  )
  expect_identical(vec_identify_runs(c("a", "a", "b", "b", "c")), expect)
  expect_identical(vec_identify_runs(as.raw(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_identify_runs(list(1, 1, 2, 2, 3)), expect)
})

test_that("NA values are identical", {
  expect <- structure(c(1L, 1L), n = 1L)

  expect_identical(vec_identify_runs(c(NA, NA)), expect)
  expect_identical(vec_identify_runs(c(NA_integer_, NA_integer_)), expect)
  expect_identical(vec_identify_runs(c(NA_real_, NA_real_)), expect)
  expect_identical(vec_identify_runs(c(NA_complex_, NA_complex_)), expect)
  expect_identical(vec_identify_runs(c(NA_character_, NA_character_)), expect)
  # No NA type for raw
  expect_identical(vec_identify_runs(list(NULL, NULL)), expect)
})

test_that("NA and NaN are different", {
  expect <- structure(c(1L, 2L), n = 2L)
  expect_identical(vec_identify_runs(c(NA_real_, NaN)), expect)
})

test_that("normalizes character encodings", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1)
  expect_identical(vec_identify_runs(x), structure(rep(1L, 3), n = 1L))
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_identify_runs(foobar())
  })
})

test_that("works with data frames rowwise", {
  df <- data_frame(x = c(1, 1, 1, 2), y = c(1, 1, 2, 3))
  expect <- structure(c(1L, 1L, 2L, 3L), n = 3L)
  expect_identical(vec_identify_runs(df), expect)

  df <- data_frame(x = c(1, 1, 1), y = c(2, 2, 2), z = c("b", "a", "a"))
  expect <- structure(c(1L, 2L, 2L), n = 2L)
  expect_identical(vec_identify_runs(df), expect)
})

test_that("works with data frames with rows but no columns", {
  expect <- structure(rep(1L, 5), n = 1L)
  expect_identical(vec_identify_runs(new_data_frame(n = 5L)), expect)
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, 1, 2, 2), b = c(1, 2, 3, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect <- structure(c(1L, 2L, 3L, 3L), n = 3L)
  expect_identical(vec_identify_runs(df), expect)
})

test_that("works with columns of various types", {
  # Use two columns to keep the data frame from being squashed to a vector
  add_col <- function(col) {
    x <- rep(1L, 5)
    data_frame(x = x, y = col)
  }

  expect <- structure(c(1L, 1L, 2L, 2L, 3L), n = 3L)

  expect_identical(
    vec_identify_runs(add_col(c(TRUE, TRUE, FALSE, FALSE, TRUE))),
    expect
  )
  expect_identical(vec_identify_runs(add_col(c(1L, 1L, 2L, 2L, 3L))), expect)
  expect_identical(vec_identify_runs(add_col(c(1, 1, 2, 2, 3))), expect)
  expect_identical(
    vec_identify_runs(add_col(complex(
      real = c(1, 1, 2, 2, 2),
      imaginary = c(1, 1, 2, 2, 3)
    ))),
    expect
  )
  expect_identical(
    vec_identify_runs(add_col(c("a", "a", "b", "b", "c"))),
    expect
  )
  expect_identical(vec_identify_runs(add_col(as.raw(c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_identify_runs(add_col(list(1, 1, 2, 2, 3))), expect)
})

# vec_run_sizes ----------------------------------------------------------------

test_that("vec_run_sizes() works with size zero input", {
  expect_identical(vec_run_sizes(integer()), integer())
  expect_identical(vec_run_sizes(data.frame()), integer())
})

test_that("works with atomic input of various types", {
  expect <- c(2L, 2L, 1L)

  expect_identical(vec_run_sizes(c(TRUE, TRUE, FALSE, FALSE, TRUE)), expect)
  expect_identical(vec_run_sizes(c(1L, 1L, 2L, 2L, 3L)), expect)
  expect_identical(vec_run_sizes(c(1, 1, 2, 2, 3)), expect)
  expect_identical(
    vec_run_sizes(complex(
      real = c(1, 1, 2, 2, 2),
      imaginary = c(1, 1, 2, 2, 3)
    )),
    expect
  )
  expect_identical(vec_run_sizes(c("a", "a", "b", "b", "c")), expect)
  expect_identical(vec_run_sizes(as.raw(c(1, 1, 2, 2, 3))), expect)
  expect_identical(vec_run_sizes(list(1, 1, 2, 2, 3)), expect)
})

test_that("NA values are identical", {
  expect <- 2L

  expect_identical(vec_run_sizes(c(NA, NA)), expect)
  expect_identical(vec_run_sizes(c(NA_integer_, NA_integer_)), expect)
  expect_identical(vec_run_sizes(c(NA_real_, NA_real_)), expect)
  expect_identical(vec_run_sizes(c(NA_complex_, NA_complex_)), expect)
  expect_identical(vec_run_sizes(c(NA_character_, NA_character_)), expect)
  # No NA type for raw
  expect_identical(vec_run_sizes(list(NULL, NULL)), expect)
})

test_that("NA and NaN are different", {
  expect_identical(vec_run_sizes(c(NA_real_, NaN)), c(1L, 1L))
})

test_that("normalizes character encodings", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1)
  expect_identical(vec_run_sizes(x), 3L)
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_run_sizes(foobar())
  })
})

test_that("works with data frames rowwise", {
  df <- data_frame(x = c(1, 1, 1, 2), y = c(1, 1, 2, 3))
  expect_identical(vec_run_sizes(df), c(2L, 1L, 1L))

  df <- data_frame(x = c(1, 1, 1), y = c(2, 2, 2), z = c("b", "a", "a"))
  expect_identical(vec_run_sizes(df), c(1L, 2L))
})

test_that("works with data frames with rows but no columns", {
  expect_identical(vec_run_sizes(new_data_frame(n = 5L)), 5L)
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, 1, 2, 2), b = c(1, 2, 3, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect_identical(vec_run_sizes(df), c(1L, 1L, 2L))
})

test_that("works with columns of various types", {
  # Use two columns to keep the data frame from being squashed to a vector
  add_col <- function(col) {
    x <- rep(1L, 5)
    data_frame(x = x, y = col)
  }

  expect <- c(2L, 2L, 1L)

  expect_identical(
    vec_run_sizes(add_col(c(TRUE, TRUE, FALSE, FALSE, TRUE))),
    expect
  )
  expect_identical(vec_run_sizes(add_col(c(1L, 1L, 2L, 2L, 3L))), expect)
  expect_identical(vec_run_sizes(add_col(c(1, 1, 2, 2, 3))), expect)
  expect_identical(
    vec_run_sizes(add_col(complex(
      real = c(1, 1, 2, 2, 2),
      imaginary = c(1, 1, 2, 2, 3)
    ))),
    expect
  )
  expect_identical(vec_run_sizes(add_col(c("a", "a", "b", "b", "c"))), expect)
  expect_identical(vec_run_sizes(add_col(as.raw(c(1, 1, 2, 2, 3)))), expect)
  expect_identical(vec_run_sizes(add_col(list(1, 1, 2, 2, 3))), expect)
})

# vec_locate_run_bounds --------------------------------------------------------

test_that("can locate run starts", {
  expect_identical(
    vec_locate_run_bounds(c(1, 3, 3, 1, 5, 5, 6)),
    c(1L, 2L, 4L, 5L, 7L)
  )
})

test_that("can locate run ends", {
  expect_identical(
    vec_locate_run_bounds(c(1, 3, 3, 1, 5, 5, 6), which = "end"),
    c(1L, 3L, 4L, 6L, 7L)
  )
})

test_that("vec_locate_run_bounds() works with size zero input", {
  expect_identical(vec_locate_run_bounds(integer(), which = "start"), integer())
  expect_identical(vec_locate_run_bounds(integer(), which = "end"), integer())
})

test_that("vec_locate_run_bounds() validates `which`", {
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_locate_run_bounds(1, which = c("foo", "bar"))
  })
})

# vec_detect_run_bounds --------------------------------------------------------

test_that("can detect run starts", {
  expect_identical(
    vec_detect_run_bounds(c(1, 3, 3, 1, 5, 5, 6)),
    c(TRUE, TRUE, FALSE, TRUE, TRUE, FALSE, TRUE)
  )
})

test_that("can detect run ends", {
  expect_identical(
    vec_detect_run_bounds(c(1, 3, 3, 1, 5, 5, 6), which = "end"),
    c(TRUE, FALSE, TRUE, TRUE, FALSE, TRUE, TRUE)
  )
})

test_that("vec_detect_run_bounds() works with size zero input", {
  expect_identical(vec_detect_run_bounds(integer(), which = "start"), logical())
  expect_identical(vec_detect_run_bounds(integer(), which = "end"), logical())
})

test_that("vec_detect_run_bounds() validates `which`", {
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_detect_run_bounds(1, which = c("foo", "bar"))
  })
})

#' Combine a list of vectors
#'
#' @description
#' `list_combine()` is a more powerful version of [vec_c()]. While `vec_c()` is
#' used for sequential combination, `list_combine()` takes a list of `indices`
#' that specify where to place each element in the output.
#'
#' If you have a list of vectors and just need to combine them sequentially,
#' you'll still want to use `vec_c(!!!x)`.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#' @inheritParams name_spec
#' @inheritParams vec_as_names
#'
#' @param x A list of vectors.
#'
#'   If `slice_x = FALSE`, each element must be size 1 or the same size as its
#'   corresponding index in `indices` after that index has been converted to
#'   a positive integer location vector with [vec_as_location()].
#'
#'   If `slice_x = TRUE`, each element must be size 1 or size `size`.
#'
#' @param indices A list of indices.
#'
#'   Indices can be provided in one of two forms:
#'
#'   - Positive integer vectors of locations less than or equal to `size`. Each
#'     vector can be any size.
#'
#'   - Logical vectors of size `size` where `TRUE` denotes the location in the
#'     output to assign to, and the location from the `x` element to pull from.
#'     Both `NA` and `FALSE` are considered unmatched.
#'
#'   The size of `indices` must match the size of `x`.
#'
#' @param size The output size.
#'
#' @param default If `NULL`, a missing value is used for locations unmatched by
#'   `indices`, otherwise the provided `default` is used.
#'
#'   If provided, `default` must be size 1 or size `size`.
#'
#'   Can only be set when `unmatched = "default"`.
#'
#' @param unmatched Handling of locations in the output unmatched by `indices`.
#'   One of:
#'
#'   - `"default"` to use `default` in unmatched locations.
#'
#'   - `"error"` to error when there are unmatched locations.
#'
#' @param multiple Handling of locations in the output matched by multiple
#'   `indices`.
#'
#'   - `"last"` uses the value from the last matched index.
#'
#'   - `"first"` uses the value from the first matched index.
#'
#'   Note that `multiple` only applies across `indices`. Within a single index
#'   if there are overlapping locations, then the last will always win. This can
#'   only occur with integer `indices`, as you can't overlap within an index
#'   when using logical `indices`.
#'
#' @param slice_x A boolean.
#'
#'   If `TRUE`, each element of `x` is sliced by its corresponding index from
#'   `indices` before being assigned into the output, which is effectively the
#'   same as `map2(list(x, indices), function(x, index) vec_slice(x, index))`,
#'   but is optimized to avoid materializing the slices.
#'
#'   See the `slice_value` argument of [vec_assign()] for more examples.
#'
#' @param ptype If `NULL`, the output type is determined by computing the common
#'   type across all elements of `x` and `default`. Alternatively, you can
#'   supply `ptype` to give the output a known type.
#'
#' @param name_repair How to repair names, see `repair` options in
#'   [vec_as_names()].
#'
#' @param x_arg,indices_arg,default_arg An argument name as a string. This
#'   argument will be mentioned in error messages as the input that is at the
#'   origin of a problem.
#'
#' @returns
#' A vector of type `vec_ptype_common(!!!x)`, or `ptype`, if specified.
#'
#' The size of the output is determined by `size`.
#'
#' @export
#' @examples
#' # Combine a list of vectors using
#' # a list of `indices`
#' x <- list(
#'   1:3,
#'   4:6,
#'   7:8
#' )
#' indices <- list(
#'   c(1, 3, 7),
#'   c(8, 6, 5),
#'   c(2, 4)
#' )
#' list_combine(x, indices = indices, size = 8)
#'
#' # Overlapping `indices` are allowed.
#' # The last match "wins" by default.
#' x <- list(
#'   1:3,
#'   4:6
#' )
#' indices <- list(
#'   c(1, 2, 3),
#'   c(1, 2, 6)
#' )
#' list_combine(x, indices = indices, size = 6)
#'
#' # Use `multiple` to force the first match to win.
#' # This is similar to how `dplyr::case_when()` works.
#' list_combine(x, indices = indices, size = 6, multiple = "first")
#'
#' # Works with data frames as well.
#' # Now how index 2 is not assigned to.
#' x <- list(
#'   data.frame(x = 1:2, y = c("a", "b")),
#'   data.frame(x = 3:4, y = c("c", "d"))
#' )
#' indices <- list(
#'   c(4, 1),
#'   c(3, NA)
#' )
#' list_combine(x, indices = indices, size = 4)
#'
#' # You can use `size` to combine into a larger object than you have values for
#' list_combine(list(1:2, 4:5), indices = list(1:2, 4:5), size = 8)
#'
#' # Additionally specifying `default` allows you to control the value used in
#' # unfilled locations
#' list_combine(
#'   list(1:2, 4:5),
#'   indices = list(1:2, 4:5),
#'   size = 8,
#'   default = 0L
#' )
#'
#' # Alternatively, if you'd like to assert that you've covered all output
#' # locations through `indices`, set `unmatched = "error"`.
#' # Here, we've set the size to 5 but missed location 3:
#' try(list_combine(
#'   list(1:2, 4:5),
#'   indices = list(1:2, 4:5),
#'   size = 5,
#'   unmatched = "error"
#' ))
list_combine <- function(
  x,
  ...,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c(
    "minimal",
    "unique",
    "check_unique",
    "universal",
    "unique_quiet",
    "universal_quiet"
  ),
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_list_combine,
    x,
    indices,
    size,
    default,
    unmatched,
    multiple,
    slice_x,
    ptype,
    name_spec,
    name_repair,
    environment()
  )
}
list_combine <- fn_inline_formals(list_combine, "name_repair")

# ------------------------------------------------------------------------------

stop_combine <- function(
  message = NULL,
  class = NULL,
  ...,
  call = caller_env()
) {
  stop_vctrs(
    message = message,
    class = c(class, "vctrs_error_combine"),
    ...,
    call = call
  )
}

# ------------------------------------------------------------------------------

stop_combine_unmatched <- function(loc, call) {
  stop_combine(
    class = "vctrs_error_combine_unmatched",
    loc = loc,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_combine_unmatched <- function(cnd, ...) {
  "Each location must be matched."
}

#' @export
cnd_body.vctrs_error_combine_unmatched <- function(cnd, ...) {
  # cli's pluralization length feature only kicks in on character vectors
  loc <- as.character(cnd$loc)
  bullet <- cli::format_inline("Location{?s} {loc} {?is/are} unmatched.")
  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

# ------------------------------------------------------------------------------

# Called from C
base_c_invoke <- function(xs) {
  local_options("vctrs:::base_c_in_progress" = TRUE)

  # Remove all `NULL` arguments which prevent dispatch if in first
  # position and might not be handled correctly by methods
  xs <- compact(xs)

  unspecified <- map_lgl(xs, fallback_is_unspecified)
  if (!any(unspecified)) {
    return(base_c(xs))
  }

  # First concatenate without the unspecified chunks. This way the
  # `c()` method doesn't need to handle unspecified inputs correctly,
  # and we're guaranteed to dispatch on the correct class even if the
  # first input is unspecified.
  out <- base_c(xs[!unspecified])

  # Create index vector with `NA` locations for unspecified chunks
  locs <- c_locs(xs)
  locs[unspecified] <- map(locs[unspecified], rep_along, na_int)
  locs[!unspecified] <- c_locs(xs[!unspecified])
  locs <- vec_c(!!!locs, .ptype = int())

  # Expand the concatenated vector with unspecified chunks
  out[locs]
}

base_c <- function(xs) {
  # Dispatch in the base namespace which inherits from the global env
  exec("c", !!!xs, .env = ns_env("base"))
}

# FIXME: Should be unnecessary in the future. We currently attach an
# attribute to unspecified columns initialised in `df_cast()`. We
# can't use an unspecified vector because we (unnecessarily but for
# convenience) go through `vec_assign()` before falling back in
# `vec_rbind()`.
fallback_is_unspecified <- function(x) {
  is_unspecified(x) || is_true(attr(x, "vctrs:::unspecified"))
}

c_locs <- function(xs) {
  locs <- reduce(lengths(xs), .init = list(0), function(output, input) {
    n <- last(last(output))
    c(output, list(seq(n + 1, n + input)))
  })

  locs[-1]
}

# Constructor and basic methods  ---------------------------------------------

#' rcrd (record) S3 class
#'
#' The rcrd class extends [vctr]. A rcrd is composed of 1 or more [field]s,
#' which must be vectors of the same length. Is designed specifically for
#' classes that can naturally be decomposed into multiple vectors of the same
#' length, like [POSIXlt], but where the organisation should be considered
#' an implementation detail invisible to the user (unlike a [data.frame]).
#'
#' @details
#' Record-style objects created with [new_rcrd()] do not do much on their own.
#' For instance they do not have a default [format()] method, which means
#' printing the object causes an error. See [Record-style
#' objects](https://vctrs.r-lib.org/articles/s3-vector.html#record-style-objects)
#' for details on implementing methods for record vectors.
#'
#' @param fields A list or a data frame. Lists must be rectangular
#'   (same sizes), and contain uniquely named vectors (at least
#'   one). `fields` is validated with [df_list()] to ensure uniquely
#'   named vectors.
#' @param ... Additional attributes
#' @param class Name of subclass.
#' @export
#' @aliases ses rcrd
#' @keywords internal
new_rcrd <- function(fields, ..., class = character()) {
  if (obj_is_list(fields) && length(vec_unique(list_sizes(fields))) > 1L) {
    abort("All fields must be the same size.")
  }

  fields <- df_list(!!!fields)
  if (!length(fields)) {
    abort("`fields` must be a list of length 1 or greater.")
  }
  structure(fields, ..., class = c(class, "vctrs_rcrd", "vctrs_vctr"))
}

#' @export
vec_proxy.vctrs_rcrd <- function(x, ...) {
  new_data_frame(x)
}
#' @export
vec_restore.vctrs_rcrd <- function(x, to, ...) {
  x <- NextMethod()
  attr(x, "row.names") <- NULL
  x
}

#' @export
length.vctrs_rcrd <- function(x) {
  vec_size(x)
}

#' @export
names.vctrs_rcrd <- function(x) {
  NULL
}

#' @export
`names<-.vctrs_rcrd` <- function(x, value) {
  if (is_null(value)) {
    x
  } else {
    abort("Can't assign names to a <vctrs_rcrd>.")
  }
}

#' @export
format.vctrs_rcrd <- function(x, ...) {
  if (inherits(x, "vctrs_foobar")) {
    # For unit tests
    exec("paste", !!!vec_data(x), sep = ":")
  } else {
    stop_unimplemented(x, "format")
  }
}

#' @export
obj_str_data.vctrs_rcrd <- function(x, ...) {
  obj_str_leaf(x, ...)
}

#' @method vec_cast vctrs_rcrd
#' @export
vec_cast.vctrs_rcrd <- function(x, to, ...) UseMethod("vec_cast.vctrs_rcrd")

#' @export
vec_cast.vctrs_rcrd.vctrs_rcrd <- function(x, to, ...) {
  out <- vec_cast(vec_data(x), vec_data(to), ...)
  new_rcrd(out)
}


# Subsetting --------------------------------------------------------------

#' @export
`[.vctrs_rcrd` <- function(x, i, ...) {
  if (!missing(...)) {
    abort("Can't index record vectors on dimensions greater than 1.")
  }
  vec_slice(x, maybe_missing(i))
}

#' @export
`[[.vctrs_rcrd` <- function(x, i, ...) {
  out <- vec_slice(vec_data(x), i)
  vec_restore(out, x)
}

#' @export
`$.vctrs_rcrd` <- function(x, i, ...) {
  stop_unsupported(x, "subsetting with $")
}

#' @export
rep.vctrs_rcrd <- function(x, ...) {
  out <- lapply(vec_data(x), base_vec_rep, ...)
  vec_restore(out, x)
}

#' @export
`length<-.vctrs_rcrd` <- function(x, value) {
  out <- vec_size_assign(vec_data(x), value)
  vec_restore(out, x)
}

# Replacement -------------------------------------------------------------

#' @export
`[[<-.vctrs_rcrd` <- function(x, i, value) {
  force(i)
  x[i] <- value
  x
}

#' @export
`$<-.vctrs_rcrd` <- function(x, i, value) {
  stop_unsupported(x, "subset assignment with $")
}

#' @export
`[<-.vctrs_rcrd` <- function(x, i, value) {
  i <- maybe_missing(i, TRUE)
  value <- vec_cast(value, x)
  out <- vec_assign(vec_data(x), i, vec_data(value))
  vec_restore(out, x)
}

# Equality and ordering ---------------------------------------------------

#' @export
vec_math.vctrs_rcrd <- function(.fn, .x, ...) {
  stop_unsupported(.x, "vec_math")
}

#' Mathematical operations
#'
#' This generic provides a common dispatch mechanism for all regular unary
#' mathematical functions. It is used as a common wrapper around many of the
#' Summary group generics, the Math group generics, and a handful of other
#' mathematical functions like `mean()` (but not `var()` or `sd()`).
#'
#' `vec_math_base()` is provided as a convenience for writing methods. It
#' calls the base `.fn` on the underlying [vec_data()].
#'
#' @section Included functions:
#'
#' * From the [Summary] group generic:
#'   `prod()`, `sum()`, `any()`, `all()`.
#'
#' * From the [Math] group generic:
#'   `abs()`, `sign()`, `sqrt()`, `ceiling()`, `floor()`, `trunc()`, `cummax()`,
#'   `cummin()`, `cumprod()`, `cumsum()`, `log()`, `log10()`, `log2()`,
#'   `log1p()`, `acos()`, `acosh()`, `asin()`, `asinh()`, `atan()`, `atanh()`,
#'   `exp()`, `expm1()`, `cos()`, `cosh()`, `cospi()`, `sin()`, `sinh()`,
#'   `sinpi()`, `tan()`, `tanh()`, `tanpi()`, `gamma()`, `lgamma()`,
#'   `digamma()`, `trigamma()`.
#'
#' * Additional generics: `mean()`, `is.nan()`, `is.finite()`, `is.infinite()`.
#'
#' Note that `median()` is currently not implemented, and `sd()` and
#' `var()` are currently not generic and so do not support custom
#' classes.
#'
#' @seealso [vec_arith()] for the equivalent for the arithmetic infix operators.
#' @param .fn A mathematical function from the base package, as a string.
#' @param .x A vector.
#' @param ... Additional arguments passed to `.fn`.
#' @keywords internal
#' @export
#' @examples
#' x <- new_vctr(c(1, 2.5, 10))
#' x
#'
#' abs(x)
#' sum(x)
#' cumsum(x)
vec_math <- function(.fn, .x, ...) {
  UseMethod("vec_math", .x)
}

#' @export
vec_math.default <- function(.fn, .x, ...) {
  if (!is_double(.x) && !is_logical_dispatch(.fn, .x)) {
    stop_unimplemented(.x, "vec_math")
  }

  out <- vec_math_base(.fn, .x, ...)

  # Don't restore output of logical predicates like `any()`,
  # `is.finite()`, or `is.nan()`
  if (is_double(out)) {
    out <- vec_restore(out, .x)
  }

  out
}
is_logical_dispatch <- function(fn, x) {
  is_logical(x) && fn %in% c("any", "all")
}

#' @export
#' @rdname vec_math
vec_math_base <- function(.fn, .x, ...) {
  .fn <- getExportedValue("base", .fn)
  .fn(vec_data(.x), ...)
}

# Helpers for testing `vec_if_else()`
#
# These ensure that we have consistent output and errors across the
# atomic and generic paths. We use them when possible.
expect_identical_vec_if_else <- function(
  ...,
  condition,
  true,
  false,
  missing = NULL,
  ptype = NULL,
  condition_arg = "condition",
  true_arg = "true",
  false_arg = "false",
  missing_arg = "missing",
  expect
) {
  expect_something_vec_if_else(
    expect_fn = expect_identical,
    condition = condition,
    true = true,
    false = false,
    missing = missing,
    ptype = ptype,
    condition_arg = condition_arg,
    true_arg = true_arg,
    false_arg = false_arg,
    missing_arg = missing_arg,
    expect = expect
  )
}

expect_something_vec_if_else <- function(
  expect_fn,
  ...,
  condition,
  true,
  false,
  missing = NULL,
  ptype = NULL,
  condition_arg = "condition",
  true_arg = "true",
  false_arg = "false",
  missing_arg = "missing",
  expect
) {
  check_dots_empty0(...)

  expect_fn(
    vec_if_else(
      condition = condition,
      true = true,
      false = false,
      missing = missing,
      ptype = ptype,
      condition_arg = condition_arg,
      true_arg = true_arg,
      false_arg = false_arg,
      missing_arg = missing_arg
    ),
    expected = expect
  )

  true_vctr <- new_vctr(true)
  false_vctr <- new_vctr(false)

  if (is.null(missing)) {
    missing_vctr <- NULL
  } else {
    missing_vctr <- new_vctr(missing)
  }

  expect_vctr <- new_vctr(expect)

  expect_fn(
    vec_if_else(
      condition = condition,
      true = true_vctr,
      false = false_vctr,
      missing = missing_vctr,
      ptype = ptype,
      condition_arg = condition_arg,
      true_arg = true_arg,
      false_arg = false_arg,
      missing_arg = missing_arg
    ),
    expected = expect_vctr
  )
}

expect_snapshot_vec_if_else <- function(
  ...,
  condition,
  true,
  false,
  missing = NULL,
  ptype = NULL,
  condition_arg = "condition",
  true_arg = "true",
  false_arg = "false",
  missing_arg = "missing",
  error = FALSE
) {
  check_dots_empty0(...)

  expect_snapshot(error = error, {
    vec_if_else(
      condition = condition,
      true = true,
      false = false,
      missing = missing,
      ptype = ptype,
      condition_arg = condition_arg,
      true_arg = true_arg,
      false_arg = false_arg,
      missing_arg = missing_arg
    )
  })

  true_vctr <- new_vctr(true)
  false_vctr <- new_vctr(false)

  if (is.null(missing)) {
    missing_vctr <- NULL
  } else {
    missing_vctr <- new_vctr(missing)
  }

  expect_snapshot(error = error, {
    vec_if_else(
      condition = condition,
      true = true_vctr,
      false = false_vctr,
      missing = missing_vctr,
      ptype = ptype,
      condition_arg = condition_arg,
      true_arg = true_arg,
      false_arg = false_arg,
      missing_arg = missing_arg
    )
  })
}

#' Unspecified vectors and prototype finalisation
#'
#' @description
#' `unspecified()` is the underlying type used to represent logical vectors that
#' only contain `NA`. These require special handling because we want to allow
#' logical `NA` to specify missingness that can be cast to any other type.
#'
#' In vctrs, the `<unspecified>` type is considered _unfinalised_ and is not
#' suitable for use in most vctrs functions that take a `ptype` argument, like
#' [vec_c()]. The purpose of `vec_ptype_finalise()` is to finalise any
#' `<unspecified>` types into `<logical>` after common type determination
#' has been completed.
#'
#' [vec_ptype()] and [vec_ptype2()] return _unfinalised_ types, and will convert
#' a logical vector of `NA` into an empty `<unspecified>` type that can combine
#' with any other type. It is unlikely that you will call these yourself, but,
#' if you do, you'll need to manually finalise with `vec_ptype_finalise()` to
#' take care of any `<unspecified>` types.
#'
#' [vec_ptype_common()] uses both [vec_ptype()] and [vec_ptype2()] to compute
#' the common type, but typically returns a _finalised_ type for immediate usage
#' in other vctrs functions. You can optionally skip finalisation by setting
#' `.finalise = FALSE`, in which case `vec_ptype_common()` can return
#' `<unspecified>` and you'll need to manually call `vec_ptype_finalise()`
#' yourself.
#'
#' `vec_ptype_finalise()` is an S3 generic, but it is extremely rare to need to
#' write an S3 method for this. Data frames (and data frame subclasses) are
#' already recursively finalised by the default method. The only time you may
#' need to write an S3 method for `vec_ptype_finalise()` is if your class
#' _wraps_ an arbitrary vector that has the potential to be a logical vector
#' containing only `NA`s. See `ivs::iv()` for an example of this, which wraps
#' arbitrary `start` and `end` vectors of the same type into a single interval
#' vector class.
#'
#' @keywords internal
#' @name vctrs-unspecified
#'
#' @examples
#' # Returns `unspecified()`
#' vec_ptype(NA)
#' vec_ptype(c(NA, NA))
#'
#' # We've chosen to make this return `logical()`, but this is admittedly
#' # ambiguous, as it could be seen as "an empty vector of `NA`s" that could
#' # also be treated as unspecified.
#' vec_ptype(logical())
#'
#' # These return `unspecified()`
#' vec_ptype2(NA, NA)
#' vec_ptype2(NA, NULL)
#' vec_ptype2(NULL, NA)
#'
#' # An unspecified vector can combine with any other type
#' vec_ptype2(NA, "x")
#' vec_ptype2("x", NA)
#'
#' # Same as using `unspecified()` directly
#' vec_ptype2(unspecified(1), "x")
#' vec_ptype2("x", unspecified(1))
#'
#' # Finalising a ptype turns unspecified back to logical
#' vec_ptype(NA)
#' vec_ptype_finalise(vec_ptype(NA))
#'
#' # This works recursively over data frames
#' df <- data_frame(x = NA, y = data_frame(z = NA))
#' vec_ptype_show(vec_ptype(df))
#' vec_ptype_show(vec_ptype_finalise(vec_ptype(df)))
#'
#' # `vec_ptype_common()` finalises automatically rather than returning an
#' # unspecified type
#' vec_ptype_common(NA)
#' vec_ptype_common(NA, NA)
#' vec_ptype_show(vec_ptype_common(df))
#'
#' # `vec_ptype_common()` lets you opt out of finalisation using `.finalise`
#' vec_ptype_common(NA, .finalise = FALSE)
#' vec_ptype_show(vec_ptype_common(df, .finalise = FALSE))
NULL

#' @param n Length of vector
#'
#' @rdname vctrs-unspecified
#' @export
unspecified <- function(n = 0) {
  .Call(vctrs_unspecified, n)
}

#' @export
`[.vctrs_unspecified` <- function(x, i, ...) {
  unspecified(length(NextMethod()))
}

#' @export
print.vctrs_unspecified <- function(x, ...) {
  cat("<unspecified> [", length(x), "]\n", sep = "")
}

#' @export
vec_ptype_abbr.vctrs_unspecified <- function(x, ...) {
  "???"
}

is_unspecified <- function(x) {
  .Call(vctrs_is_unspecified, x)
}

ununspecify <- function(x) {
  if (is_unspecified(x)) {
    new_logical(length(x))
  } else {
    x
  }
}

#' @inheritParams rlang::args_dots_empty
#'
#' @param x A `ptype` to finalize, typically a result of [vec_ptype()],
#'   [vec_ptype2()], or [`vec_ptype_common(.finalise =
#'   FALSE)`][vec_ptype_common].
#'
#' @rdname vctrs-unspecified
#' @export
vec_ptype_finalise <- function(x, ...) {
  check_dots_empty0(...)
  return(.Call(vctrs_ptype_finalise, x))
  UseMethod("vec_ptype_finalise")
}

vec_ptype_finalise_dispatch <- function(x, ...) {
  UseMethod("vec_ptype_finalise")
}

#' @export
vec_ptype_finalise.default <- function(x, ...) {
  x
}

test_that("unknown type is idempotent", {
  types <- list(
    unspecified(),
    logical(),
    integer(),
    double(),
    character(),
    list(),
    new_list_of(ptype = integer()),
    new_factor(),
    new_ordered(),
    new_date(),
    new_datetime(),
    new_duration(),
    vec_ptype(matrix(1:4, 2)),
    vec_ptype(array(1:5, c(1, 5)))
  )

  lhs <- map(types, vec_ptype2, x = unspecified())
  expect_identical(types, lhs)

  lhs <- map(types, vec_ptype2, x = NA)
  expect_identical(types, lhs)

  rhs <- map(types, vec_ptype2, y = unspecified())
  expect_identical(types, rhs)

  rhs <- map(types, vec_ptype2, y = NA)
  expect_identical(types, rhs)
})

test_that("common type of unspecified and NULL is unspecified", {
  expect_identical(vec_ptype2(unspecified(), NULL), unspecified())
  expect_identical(vec_ptype2(NULL, unspecified()), unspecified())

  expect_identical(vec_ptype2(NA, NULL), unspecified())
  expect_identical(vec_ptype2(NULL, NA), unspecified())
})

test_that("cannot take the common type of unspecified and a scalar list", {
  expect_error(vec_ptype2(unspecified(), foobar()), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(foobar(), unspecified()), class = "vctrs_error_scalar_type")
})

test_that("subsetting works", {
  expect_identical(unspecified(4)[2:3], unspecified(2))
})

test_that("has useful print method", {
  expect_snapshot(unspecified())
})

test_that("can finalise data frame containing unspecified columns", {
  df <- data.frame(y = NA, x = c(1, 2, NA))

  ptype <- vec_ptype(df)
  expect_identical(ptype$y, unspecified())

  finalised <- vec_ptype_finalise(ptype)
  expect_identical(finalised$y, lgl())

  common <- vec_ptype_common(df, df)
  expect_identical(common$y, lgl())
})

test_that("can cast to common type data frame containing unspecified columns", {
  df <- data.frame(y = NA, x = c(1, 2, NA))
  expect_identical(vec_cast_common(df, df), list(df, df))
})

test_that("unspecified vectors are always unspecified (#222)", {
  expect_true(is_unspecified(unspecified()))
  expect_true(is_unspecified(unspecified(1)))
})

test_that("S3 vectors and shaped vectors are never unspecified", {
  expect_false(is_unspecified(foobar(NA)))
  expect_false(is_unspecified(foobar(lgl(NA, NA))))
  expect_false(is_unspecified(matrix(NA, 2)))
})

test_that("can finalise lengthy unspecified vectors", {
  expect_identical(vec_ptype_finalise(unspecified(3)), rep(NA, 3))
  expect_identical(ununspecify(unspecified(3)), rep(NA, 3))
})

test_that("unspecified() validates input", {
  expect_identical(unspecified(1), unspecified(1L))
  expect_error(unspecified(1:3), "must be a single number")
})

test_that("tibble::type_sum() knows about unspecified", {
  expect_identical(tibble::type_sum(unspecified(3)), "???")
})

test_that("casting to a scalar type errors", {
  expect_error(vec_cast(NA, quote(x)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(unspecified(1), quote(x)), class = "vctrs_error_scalar_type")
})

test_that("monitoring test - can cast to unspecified from unspecified", {
  expect_identical(vec_cast(NA, unspecified()), unspecified(1))
  expect_identical(vec_cast(unspecified(1), unspecified()), unspecified(1))
})

test_that("monitoring test - casting unspecified input to NA unspecified results in NA vector", {
  expect_identical(vec_cast(unspecified(1), NA), NA)
  expect_identical(vec_cast(NA, NA), NA)
})

test_that("unknown type is idempotent", {
  types <- list(
    unspecified(),
    logical(),
    integer(),
    double(),
    character(),
    list(),
    new_list_of(ptype = integer()),
    new_factor(),
    new_ordered(),
    new_date(),
    new_datetime(),
    new_duration(),
    vec_ptype(matrix(1:4, 2)),
    vec_ptype(array(1:5, c(1, 5)))
  )

  lhs <- map(types, vec_ptype2, x = unspecified())
  expect_identical(types, lhs)

  lhs <- map(types, vec_ptype2, x = NA)
  expect_identical(types, lhs)

  rhs <- map(types, vec_ptype2, y = unspecified())
  expect_identical(types, rhs)

  rhs <- map(types, vec_ptype2, y = NA)
  expect_identical(types, rhs)
})

test_that("common type of unspecified and NULL is unspecified", {
  expect_identical(vec_ptype2(unspecified(), NULL), unspecified())
  expect_identical(vec_ptype2(NULL, unspecified()), unspecified())

  expect_identical(vec_ptype2(NA, NULL), unspecified())
  expect_identical(vec_ptype2(NULL, NA), unspecified())
})

test_that("cannot take the common type of unspecified and a scalar list", {
  expect_error(
    vec_ptype2(unspecified(), foobar()),
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype2(foobar(), unspecified()),
    class = "vctrs_error_scalar_type"
  )
})

test_that("subsetting works", {
  expect_identical(unspecified(4)[2:3], unspecified(2))
})

test_that("has useful print method", {
  expect_snapshot(unspecified())
})

test_that("can finalise data frame containing unspecified columns", {
  df <- data.frame(y = NA, x = c(1, 2, NA))

  ptype <- vec_ptype(df)
  expect_identical(ptype$y, unspecified())

  finalised <- vec_ptype_finalise(ptype)
  expect_identical(finalised$y, lgl())

  common <- vec_ptype_common(df, df)
  expect_identical(common$y, lgl())
})

test_that("can cast to common type data frame containing unspecified columns", {
  df <- data.frame(y = NA, x = c(1, 2, NA))
  expect_identical(vec_cast_common(df, df), list(df, df))
})

test_that("unspecified vectors are always unspecified (#222)", {
  expect_true(is_unspecified(unspecified()))
  expect_true(is_unspecified(unspecified(1)))
})

test_that("S3 vectors and shaped vectors are never unspecified", {
  expect_false(is_unspecified(foobar(NA)))
  expect_false(is_unspecified(foobar(lgl(NA, NA))))
  expect_false(is_unspecified(matrix(NA, 2)))
})

test_that("empty logical vector is not unspecified", {
  expect_false(is_unspecified(logical()))
})

test_that("logical vectors of only `NA` are considered unspecified", {
  expect_true(is_unspecified(NA))
  expect_true(is_unspecified(c(NA, NA)))
  expect_false(is_unspecified(c(NA, FALSE, NA)))
})

test_that("extraneous attributes don't affect unspecifiedness", {
  # `names`
  expect_true(is_unspecified(set_names(c(NA, NA), c("a", "b"))))
  expect_true(is_unspecified(set_names(unspecified(2), c("a", "b"))))

  # Extraneous attributes
  expect_true(is_unspecified(structure(c(NA, NA), foo = "bar")))
  expect_true(is_unspecified(structure(unspecified(2), foo = "bar")))
})

test_that("can finalise lengthy unspecified vectors", {
  expect_identical(vec_ptype_finalise(unspecified(3)), rep(NA, 3))
  expect_identical(ununspecify(unspecified(3)), rep(NA, 3))
})

test_that("unspecified() validates input", {
  expect_identical(unspecified(1), unspecified(1L))
  expect_error(unspecified(1:3), "must be a single number")
})

test_that("tibble::type_sum() knows about unspecified", {
  expect_identical(tibble::type_sum(unspecified(3)), "???")
})

test_that("casting to a scalar type errors", {
  expect_error(vec_cast(NA, quote(x)), class = "vctrs_error_scalar_type")
  expect_error(
    vec_cast(unspecified(1), quote(x)),
    class = "vctrs_error_scalar_type"
  )
})

test_that("monitoring test - can cast to unspecified from unspecified", {
  expect_identical(vec_cast(NA, unspecified()), unspecified(1))
  expect_identical(vec_cast(unspecified(1), unspecified()), unspecified(1))
})

test_that("monitoring test - casting unspecified input to NA unspecified results in NA vector", {
  expect_identical(vec_cast(unspecified(1), NA), NA)
  expect_identical(vec_cast(NA, NA), NA)
})

# nocov start

.onLoad <- function(libname, pkgname) {
  check_linked_version(pkgname)
  ns <- ns_env("vctrs")

  run_on_load()

  s3_register("generics::as.factor", "vctrs_vctr")
  s3_register("generics::as.ordered", "vctrs_vctr")
  s3_register("generics::as.difftime", "vctrs_vctr")

  # Remove once tibble has implemented the methods
  on_package_load("tibble", {
    if (!env_has(ns_env("tibble"), "vec_ptype2.tbl_df.tbl_df")) {
      s3_register(
        "vctrs::vec_ptype2",
        "tbl_df.tbl_df",
        vec_ptype2_tbl_df_tbl_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "tbl_df.data.frame",
        vec_ptype2_tbl_df_data.frame
      )
      s3_register(
        "vctrs::vec_ptype2",
        "data.frame.tbl_df",
        vec_ptype2_data.frame_tbl_df
      )
    }

    if (!env_has(ns_env("tibble"), "vec_cast.tbl_df.tbl_df")) {
      s3_register("vctrs::vec_cast", "tbl_df.tbl_df", vec_cast_tbl_df_tbl_df)
      s3_register(
        "vctrs::vec_cast",
        "tbl_df.data.frame",
        vec_cast_tbl_df_data.frame
      )
      s3_register(
        "vctrs::vec_cast",
        "data.frame.tbl_df",
        vec_cast_data.frame_tbl_df
      )
    }
  })

  on_package_load("dplyr", {
    if (!env_has(ns_env("dplyr"), "vec_restore.grouped_df")) {
      s3_register("vctrs::vec_restore", "grouped_df", vec_restore_grouped_df)
    }

    if (!env_has(ns_env("dplyr"), "vec_ptype2.grouped_df.grouped_df")) {
      s3_register(
        "vctrs::vec_ptype2",
        "grouped_df.grouped_df",
        vec_ptype2_grouped_df_grouped_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "grouped_df.data.frame",
        vec_ptype2_grouped_df_data.frame
      )
      s3_register(
        "vctrs::vec_ptype2",
        "grouped_df.tbl_df",
        vec_ptype2_grouped_df_tbl_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "data.frame.grouped_df",
        vec_ptype2_data.frame_grouped_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "tbl_df.grouped_df",
        vec_ptype2_tbl_df_grouped_df
      )
    }

    if (!env_has(ns_env("dplyr"), "vec_cast.grouped_df.grouped_df")) {
      s3_register(
        "vctrs::vec_cast",
        "grouped_df.grouped_df",
        vec_cast_grouped_df_grouped_df
      )
      s3_register(
        "vctrs::vec_cast",
        "grouped_df.data.frame",
        vec_cast_grouped_df_data.frame
      )
      s3_register(
        "vctrs::vec_cast",
        "grouped_df.tbl_df",
        vec_cast_grouped_df_tbl_df
      )
      s3_register(
        "vctrs::vec_cast",
        "data.frame.grouped_df",
        vec_cast_data.frame_grouped_df
      )
      s3_register(
        "vctrs::vec_cast",
        "tbl_df.grouped_df",
        vec_cast_tbl_df_grouped_df
      )
    }

    if (!env_has(ns_env("dplyr"), "vec_restore.rowwise_df")) {
      s3_register("vctrs::vec_restore", "rowwise_df", vec_restore_rowwise_df)
    }

    if (!env_has(ns_env("dplyr"), "vec_ptype2.rowwise_df.rowwise_df")) {
      s3_register(
        "vctrs::vec_ptype2",
        "rowwise_df.rowwise_df",
        vec_ptype2_rowwise_df_rowwise_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "rowwise_df.data.frame",
        vec_ptype2_rowwise_df_data.frame
      )
      s3_register(
        "vctrs::vec_ptype2",
        "rowwise_df.tbl_df",
        vec_ptype2_rowwise_df_tbl_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "data.frame.rowwise_df",
        vec_ptype2_data.frame_rowwise_df
      )
      s3_register(
        "vctrs::vec_ptype2",
        "tbl_df.rowwise_df",
        vec_ptype2_tbl_df_rowwise_df
      )
    }

    if (!env_has(ns_env("dplyr"), "vec_cast.rowwise_df.rowwise_df")) {
      s3_register(
        "vctrs::vec_cast",
        "rowwise_df.rowwise_df",
        vec_cast_rowwise_df_rowwise_df
      )
      s3_register(
        "vctrs::vec_cast",
        "rowwise_df.data.frame",
        vec_cast_rowwise_df_data.frame
      )
      s3_register(
        "vctrs::vec_cast",
        "rowwise_df.tbl_df",
        vec_cast_rowwise_df_tbl_df
      )
      s3_register(
        "vctrs::vec_cast",
        "data.frame.rowwise_df",
        vec_cast_data.frame_rowwise_df
      )
      s3_register(
        "vctrs::vec_cast",
        "tbl_df.rowwise_df",
        vec_cast_tbl_df_rowwise_df
      )
    }
  })

  on_package_load("sf", {
    import_from("sf", sf_deps, env = sf_env)

    if (!env_has(ns_env("sf"), "vec_restore.sf")) {
      s3_register("vctrs::vec_proxy", "sf", vec_proxy_sf)
      s3_register("vctrs::vec_restore", "sf", vec_restore_sf)
    }
    if (!env_has(ns_env("sf"), "vec_ptype2.sf.sf")) {
      s3_register("vctrs::vec_ptype2", "sf.sf", vec_ptype2_sf_sf)
      s3_register(
        "vctrs::vec_ptype2",
        "sf.data.frame",
        vec_ptype2_sf_data.frame
      )
      s3_register(
        "vctrs::vec_ptype2",
        "data.frame.sf",
        vec_ptype2_data.frame_sf
      )
      s3_register("vctrs::vec_ptype2", "sf.tbl_df", vec_ptype2_sf_tbl_df)
      s3_register("vctrs::vec_ptype2", "tbl_df.sf", vec_ptype2_tbl_df_sf)
      s3_register("vctrs::vec_cast", "sf.sf", vec_cast_sf_sf)
      s3_register("vctrs::vec_cast", "sf.data.frame", vec_cast_sf_data.frame)
      s3_register("vctrs::vec_cast", "data.frame.sf", vec_cast_data.frame_sf)
    }
    if (!env_has(ns_env("sf"), "vec_proxy_order.sfc")) {
      s3_register("vctrs::vec_proxy_order", "sfc", vec_proxy_order_sfc)
    }
  })

  on_package_load("data.table", {
    if (!env_has(ns_env("data.table"), "vec_proxy.IDate")) {
      s3_register("vctrs::vec_proxy", "IDate", vec_proxy_IDate)
      s3_register("vctrs::vec_restore", "IDate", vec_restore_IDate)
    }
  })

  .Call(vctrs_init_library, ns_env())
}

# nocov end

# print -------------------------------------------------------------------

#' `print()` and `str()` generics.
#'
#' These are constructed to be more easily extensible since you can override
#' the `_header()`, `_data()` or `_footer()` components individually. The
#' default methods are built on top of `format()`.
#'
#' @param x A vector
#' @param ... Additional arguments passed on to methods. See [print()] and
#'   [str()] for commonly used options
#' @keywords internal
#' @export
obj_print <- function(x, ...) {
  obj_print_header(x, ...)
  obj_print_data(x, ...)
  obj_print_footer(x, ...)
  invisible(x)
}

#' @export
#' @rdname obj_print
obj_print_header <- function(x, ...) {
  UseMethod("obj_print_header")
}

#' @export
obj_print_header.default <- function(x, ...) {
  cat_line("<", vec_ptype_full(x), "[", vec_size(x), "]>")
  invisible(x)
}

#' @export
#' @rdname obj_print
obj_print_data <- function(x, ...) {
  UseMethod("obj_print_data")
}

#' @export
obj_print_data.default <- function(x, ...) {
  if (length(x) == 0) {
    return(invisible(x))
  }

  out <- stats::setNames(format(x), names(x))
  print(out, quote = FALSE)

  invisible(x)
}

#' @export
#' @rdname obj_print
obj_print_footer <- function(x, ...) {
  UseMethod("obj_print_footer")
}

#' @export
obj_print_footer.default <- function(x, ...) {
  invisible(x)
}


# str ---------------------------------------------------------------------

#' @export
#' @rdname obj_print
obj_str <- function(x, ...) {
  obj_str_header(x, ...)
  obj_str_data(x, ...)
  obj_str_footer(x, ...)
}

#' @export
#' @rdname obj_print
obj_str_header <- function(x, ...) {
  UseMethod("obj_str_header")
}

#' @export
obj_str_header.default <- function(x, ...) {
  invisible(x)
}

#' @export
#' @rdname obj_print
obj_str_data <- function(x, ...) {
  UseMethod("obj_str_data")
}

#' @export
obj_str_data.default <- function(x, ...) {
  if (is.list(x)) {
    obj_str_recursive(x, ...)
  } else {
    obj_str_leaf(x, ...)
  }
}

obj_str_recursive <- function(x, ..., indent.str = "", nest.lev = 0) {
  if (nest.lev != 0L) {
    cat(" ")
  }
  cat_line(glue::glue("{vec_ptype_abbr(x)} [1:{vec_size(x)}] "))

  utils::str(
    vec_data(x),
    no.list = TRUE,
    ...,
    nest.lev = nest.lev + 1L,
    indent.str = indent.str
  )
}

obj_str_leaf <- function(x, ..., indent.str = "", width = getOption("width")) {
  width <- width - nchar(indent.str) - 2

  # Avoid spending too much time formatting elements that won't see
  length <- ceiling(width / 2)
  if (length(x) > length) {
    out <- x[seq2(1, length)]
  } else {
    out <- x
  }

  title <- glue::glue(" {vec_ptype_abbr(x)} [1:{length(x)}] ")
  cat_line(inline_list(title, format(out), width = width))

  invisible(x)
}

#' @export
#' @rdname obj_print
obj_str_footer <- function(x, ...) {
  UseMethod("obj_str_footer")
}

#' @export
obj_str_footer.default <- function(x, ..., indent.str = "", nest.lev = 0) {
  attr <- attributes(x)
  attr[["class"]] <- NULL
  attr[["names"]] <- NULL

  if (length(attr) == 0) {
    return(invisible(x))
  }

  if (!is.list(x)) {
    indent.str <- paste0(" ", indent.str)
  }

  utils::str(
    attr,
    no.list = TRUE,
    ...,
    comp.str = "@ ",
    nest.lev = nest.lev + 1L,
    indent.str = indent.str
  )

  invisible(x)
}

#' Assert an argument has known prototype and/or size
#'
#' @description
#' `r lifecycle::badge("questioning")`
#'
#' * `vec_is()` is a predicate that checks if its input is a vector that
#'   conforms to a prototype and/or a size.
#'
#' * `vec_assert()` throws an error when the input is not a vector or
#'   doesn't conform.
#'
#' @inheritSection vector-checks Vectors and scalars
#'
#' @section Error types:
#'
#' `vec_is()` never throws.
#' `vec_assert()` throws the following errors:
#'
#' * If the input is not a vector, an error of class
#'   `"vctrs_error_scalar_type"` is raised.
#'
#' * If the prototype doesn't match, an error of class
#'   `"vctrs_error_assert_ptype"` is raised.
#'
#' * If the size doesn't match, an error of class
#' `"vctrs_error_assert_size"` is raised.
#'
#' Both errors inherit from `"vctrs_error_assert"`.
#'
#' @section Lifecycle:
#'
#' Both `vec_is()` and `vec_assert()` are questioning because their `ptype`
#' arguments have semantics that are challenging to define clearly and are
#' rarely useful.
#'
#' - Use [obj_is_vector()] or [obj_check_vector()] for vector checks
#'
#' - Use [vec_check_size()] for size checks
#'
#' - Use [vec_cast()], [inherits()], or simple type predicates like
#'   [rlang::is_logical()] for specific type checks
#'
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector argument to check.
#' @param ptype Prototype to compare against. If the prototype has a
#'   class, its [vec_ptype()] is compared to that of `x` with
#'   `identical()`. Otherwise, its [typeof()] is compared to that of
#'   `x` with `==`.
#' @param size A single integer size against which to compare.
#' @param arg Name of argument being checked. This is used in error
#'   messages. The label of the expression passed as `x` is taken as
#'   default.
#'
#' @return `vec_is()` returns `TRUE` or `FALSE`. `vec_assert()` either
#'   throws a typed error (see section on error types) or returns `x`,
#'   invisibly.
#' @keywords internal
#' @export
vec_assert <- function(
  x,
  ptype = NULL,
  size = NULL,
  arg = caller_arg(x),
  call = caller_env()
) {
  if (!obj_is_vector(x)) {
    stop_scalar_type(x, arg, call = call)
  }

  if (!is_null(ptype)) {
    ptype <- vec_ptype(ptype)
    x_type <- vec_ptype_finalise(vec_ptype(x))
    if (!is_same_type(x_type, ptype)) {
      msg <- vec_assert_type_explain(x_type, ptype, arg)
      abort(
        msg,
        class = c("vctrs_error_assert_ptype", "vctrs_error_assert"),
        required = ptype,
        actual = x_type,
        call = call
      )
    }
  }

  if (!is_null(size)) {
    size <- vec_cast(size, integer(), x_arg = "size")

    n_size <- length(size)
    if (n_size != 1L) {
      abort(glue::glue("`size` must be length 1, not length {n_size}."))
    }

    x_size <- vec_size(x)
    if (!identical(x_size, size)) {
      stop_assert_size(
        x_size,
        size,
        arg,
        call = call
      )
    }
  }

  invisible(x)
}

# Also thrown from C
stop_assert_size <- function(actual, required, arg, call = caller_env()) {
  arg <- arg_backtick(arg)
  message <- glue::glue("{arg} must have size {required}, not size {actual}.")

  stop_assert(
    message,
    class = "vctrs_error_assert_size",
    actual = actual,
    required = required,
    call = call
  )
}

stop_assert <- function(
  message = NULL,
  class = NULL,
  ...,
  call = caller_env()
) {
  stop_vctrs(
    message,
    class = c(class, "vctrs_error_assert"),
    ...,
    call = call
  )
}

#' @rdname vec_assert
#' @export
vec_is <- function(x, ptype = NULL, size = NULL) {
  if (!obj_is_vector(x)) {
    return(FALSE)
  }

  if (!is_null(ptype)) {
    ptype <- vec_ptype(ptype)
    x_type <- vec_ptype_finalise(vec_ptype(x))
    if (!is_same_type(x_type, ptype)) {
      return(FALSE)
    }
  }

  if (!is_null(size)) {
    size <- vec_recycle(vec_cast(size, integer()), 1L)
    x_size <- vec_size(x)
    if (!identical(x_size, size)) {
      return(FALSE)
    }
  }

  TRUE
}

#' Vector checks
#'
#' @description
#'
#' - `obj_is_vector()` tests if `x` is considered a vector in the vctrs sense.
#'   See _Vectors and scalars_ below for the exact details.
#'
#' - `obj_check_vector()` uses `obj_is_vector()` and throws a standardized and
#'   informative error if it returns `FALSE`.
#'
#' - `vec_check_size()` tests if `x` has size `size`, and throws an informative
#'   error if it doesn't.
#'
#' - `vec_check_recyclable()` tests if `x` can recycle to size `size`, and
#'   throws an informative error if it can't.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x For `obj_*()` functions, an object. For `vec_*()` functions, a
#'   vector.
#'
#' @param size The size to check for compatibility with.
#'
#' @returns
#' - `obj_is_vector()` returns a single `TRUE` or `FALSE`.
#'
#' - `obj_check_vector()` returns `NULL` invisibly, or errors.
#'
#' - `vec_check_size()` returns `NULL` invisibly, or errors.
#'
#' - `vec_check_recyclable()` returns `NULL` invisibly, or errors.
#'
#' @section Vectors and scalars:
#'
#' Informally, a vector is a collection that makes sense to use as column in a
#' data frame. The following rules define whether or not `x` is considered a
#' vector.
#'
#' If no [vec_proxy()] method has been registered, `x` is a vector if:
#'
#' - The [base type][typeof] of the object is atomic: `"logical"`, `"integer"`,
#'   `"double"`, `"complex"`, `"character"`, or `"raw"`.
#'
#' - `x` is a list, as defined by [obj_is_list()].
#'
#' - `x` is a [data.frame].
#'
#' If a `vec_proxy()` method has been registered, `x` is a vector if:
#'
#' - The proxy satisfies one of the above conditions.
#'
#' - The base type of the proxy is `"list"`, regardless of its class. S3 lists
#'   are thus treated as scalars unless they implement a `vec_proxy()` method.
#'
#' Otherwise an object is treated as scalar and cannot be used as a vector.
#' In particular:
#'
#' - `NULL` is not a vector.
#'
#' - S3 lists like `lm` objects are treated as scalars by default.
#'
#' - Objects of type [expression] are not treated as vectors.
#'
#' @section Technical limitations:
#'
#' - Support for S4 vectors is currently limited to objects that inherit from an
#'   atomic type.
#'
#' - Subclasses of [data.frame] that *append* their class to the back of the
#'   `"class"` attribute are not treated as vectors. If you inherit from an S3
#'   class, always prepend your class to the front of the `"class"` attribute
#'   for correct dispatch. This matches our general principle of allowing
#'   subclasses but not mixins.
#'
#' @name vector-checks
#' @examples
#' obj_is_vector(1)
#'
#' # Data frames are vectors
#' obj_is_vector(data_frame())
#'
#' # Bare lists are vectors
#' obj_is_vector(list())
#'
#' # S3 lists are vectors if they explicitly inherit from `"list"`
#' x <- structure(list(), class = c("my_list", "list"))
#' obj_is_list(x)
#' obj_is_vector(x)
#'
#' # But if they don't explicitly inherit from `"list"`, they aren't
#' # automatically considered to be vectors. Instead, vctrs considers this
#' # to be a scalar object, like a linear model returned from `lm()`.
#' y <- structure(list(), class = "my_list")
#' obj_is_list(y)
#' obj_is_vector(y)
#'
#' # `obj_check_vector()` throws an informative error if the input
#' # isn't a vector
#' try(obj_check_vector(y))
#'
#' # `vec_check_size()` throws an informative error if the size of the
#' # input doesn't match `size`
#' vec_check_size(1:5, size = 5)
#' try(vec_check_size(1:5, size = 4))
#'
#' # `vec_check_recyclable()` throws an informative error if the input can't
#' # recycle to size `size`
#' vec_check_recyclable(1:5, size = 5)
#' vec_check_recyclable(1, size = 5)
#' try(vec_check_recyclable(1:2, size = 5))
NULL

#' @export
#' @rdname vector-checks
obj_is_vector <- function(x) {
  .Call(ffi_obj_is_vector, x)
}

#' @export
#' @rdname vector-checks
obj_check_vector <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
  check_dots_empty0(...)
  invisible(.Call(ffi_obj_check_vector, x, environment()))
}

#' @export
#' @rdname vector-checks
vec_check_size <- function(
  x,
  size,
  ...,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty0(...)
  invisible(.Call(ffi_vec_check_size, x, size, environment()))
}

#' @export
#' @rdname vector-checks
vec_check_recyclable <- function(
  x,
  size,
  ...,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty0(...)
  invisible(.Call(ffi_vec_check_recyclable, x, size, environment()))
}

#' List checks
#'
#' @description
#' - `obj_is_list()` tests if `x` is considered a list in the vctrs sense. It
#'   returns `TRUE` if all of the following hold:
#'   - `x` must have list storage, i.e. `typeof(x)` returns `"list"`
#'   - `x` must not have a `dim` attribute
#'   - `x` must not have a `class` attribute, or must explicitly inherit from
#'     `"list"` as the last class
#'
#' - `list_all_vectors()` takes a list and returns `TRUE` if all elements of
#'   that list are vectors.
#'
#' - `list_all_size()` takes a list and returns `TRUE` if all elements of that
#'   list have the same `size`.
#'
#' - `list_all_recyclable()` takes a list and returns `TRUE` if all elements of
#'   that list can recycle to `size`.
#'
#' - `obj_check_list()`, `list_check_all_vectors()`, `list_check_all_size()`,
#'   and `list_check_all_recyclable()` use the above functions, but throw a
#'   standardized and informative error if they return `FALSE`.
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams rlang::args_dots_empty
#'
#' @param x For `vec_*()` functions, an object. For `list_*()` functions, a
#'   list.
#'
#' @param size The size to check each element for compatibility with.
#'
#' @param allow_null Whether `NULL` elements should be skipped over
#'   automatically or not.
#'
#' @details
#' Notably, data frames and S3 record style classes like POSIXlt are not
#' considered lists.
#'
#' @seealso [list_sizes()]
#' @export
#' @examples
#' obj_is_list(list())
#' obj_is_list(list_of(1))
#' obj_is_list(data.frame())
#'
#' list_all_vectors(list(1, mtcars))
#' list_all_vectors(list(1, environment()))
#'
#' list_all_size(list(1:2, 2:3), 2)
#' list_all_size(list(1:2, 2:4), 2)
#'
#' list_all_recyclable(list(1, 2:3), 2)
#' list_all_recyclable(list(1, 2:4), 2)
#'
#' # `list_`-prefixed functions assume a list:
#' try(list_all_vectors(environment()))
#'
#' # `NULL` elements are not considered vectors and generally have a size of 0
#' try(list_check_all_vectors(list(1, NULL, 2)))
#' try(list_check_all_size(list(1, NULL, 2), size = 1))
#'
#' # However, it is often useful to perform upfront vector/size checks on a
#' # list, excluding `NULL`s, and then filter them out later on
#' list_check_all_vectors(list(1, NULL, 2), allow_null = TRUE)
#' list_check_all_size(list(1, NULL, 2), size = 1, allow_null = TRUE)
#'
#' # Performing the checks before removing `NULL`s from the list ensures that
#' # any errors report the correct index. Note how the index is incorrect from a
#' # user's point of view if we filter out `NULL` too soon.
#' xs <- list(1, NULL, 2:3)
#' try(list_check_all_size(xs, size = 1, allow_null = TRUE))
#' xs <- vec_slice(xs, !vec_detect_missing(xs))
#' try(list_check_all_size(xs, size = 1))
obj_is_list <- function(x) {
  .Call(ffi_obj_is_list, x)
}
#' @rdname obj_is_list
#' @export
obj_check_list <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
  check_dots_empty0(...)
  invisible(.Call(ffi_check_list, x, environment()))
}

#' @rdname obj_is_list
#' @export
list_all_vectors <- function(x, ..., allow_null = FALSE) {
  check_dots_empty0(...)
  .Call(ffi_list_all_vectors, x, allow_null, environment())
}

#' @rdname obj_is_list
#' @export
list_check_all_vectors <- function(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty0(...)
  invisible(.Call(ffi_list_check_all_vectors, x, allow_null, environment()))
}

#' @rdname obj_is_list
#' @export
list_all_size <- function(x, size, ..., allow_null = FALSE) {
  check_dots_empty0(...)
  .Call(ffi_list_all_size, x, size, allow_null, environment())
}

#' @rdname obj_is_list
#' @export
list_check_all_size <- function(
  x,
  size,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty0(...)
  invisible(.Call(
    ffi_list_check_all_size,
    x,
    size,
    allow_null,
    environment()
  ))
}

#' @rdname obj_is_list
#' @export
list_all_recyclable <- function(x, size, ..., allow_null = FALSE) {
  check_dots_empty0(...)
  .Call(ffi_list_all_recyclable, x, size, allow_null, environment())
}

#' @rdname obj_is_list
#' @export
list_check_all_recyclable <- function(
  x,
  size,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
) {
  check_dots_empty0(...)
  invisible(.Call(
    ffi_list_check_all_recyclable,
    x,
    size,
    allow_null,
    environment()
  ))
}

# Called from C
stop_non_list_type <- function(x, arg, call) {
  arg <- arg_backtick(arg)

  cli::cli_abort(
    "{arg} must be a list, not {obj_type_friendly(x)}.",
    call = call
  )
}

is_same_type <- function(x, ptype) {
  x <- vec_slice(x, integer())
  ptype <- vec_slice(ptype, integer())

  # FIXME: Remove row names for matrices and arrays, and handle empty
  # but existing dimnames
  x <- vec_set_names(x, NULL)
  ptype <- vec_set_names(ptype, NULL)

  identical(x, ptype)
}

vec_assert_type_explain <- function(x, type, arg) {
  arg <- str_backtick(arg)
  x <- paste0("<", vec_ptype_full(x), ">")
  type <- paste0("<", vec_ptype_full(type), ">")

  intro <- paste0(arg, " must be a vector with type")
  intro <- layout_type(intro, type)

  outro <- paste0("Instead, it has type")
  outro <- layout_type(outro, x)

  paste_line(
    !!!intro,
    if (str_is_multiline(intro)) "",
    !!!outro
  )
}

layout_type <- function(start, type) {
  if (str_is_multiline(type)) {
    paste_line(
      paste0(start, ":"),
      "",
      paste0("  ", indent(type, 2))
    )
  } else {
    paste0(start, " ", type, ".")
  }
}

test_that("basic assert is idempotent", {
  x <- new_vctr(1:4)

  expect_true(vec_is(x))
  expect_identical(vec_assert(x), x)

  expect_identical(vec_assert(x), new_vctr(1:4))
  expect_false(withVisible(vec_assert(x))$visible)

  expect_true(vec_is(1:4))
  expect_identical(vec_assert(1:4), 1:4)
})

test_that("asserting ptype", {
  x <- new_vctr(1:4)
  good <- new_vctr(integer())

  expect_true(vec_is(x, good))
  expect_error(vec_assert(x, good), NA)

  # Is this the correct error message?
  bad <- new_vctr(double())
  expect_false(vec_is(x, bad))
  expect_error(vec_assert(x, bad), class = "vctrs_error_assert_ptype")
})

test_that("asserting size", {
  x <- new_vctr(1:4)

  expect_true(vec_is(x, size = 4))
  expect_error(vec_assert(x, size = 4), NA)

  expect_false(vec_is(x, size = 5))
  expect_error(vec_assert(x, size = 5), class = "vctrs_error_assert_size")
})

test_that("vec_assert() labels input", {
  expect_error(
    vec_assert(new_vctr(1:4), size = 5),
    regexp = "`new_vctr\\(1:4\\)` must have",
    class = "vctrs_error_assert_size"
  )
  expect_error(
    vec_assert(new_vctr(1:4), size = 5, arg = "foobar"),
    regexp = "`foobar` must have",
    class = "vctrs_error_assert_size"
  )
})

test_that("bare atomic vectors are vectors but not recursive", {
  expect_true(obj_is_vector(TRUE))
  expect_true(obj_is_vector(1L))
  expect_true(obj_is_vector(1))
  expect_true(obj_is_vector(1i))
  expect_true(obj_is_vector("foo"))
  expect_true(obj_is_vector(as.raw(1)))
})

test_that("S3 atomic vectors are vectors", {
  expect_true(obj_is_vector(foobar(TRUE)))
  expect_true(obj_is_vector(foobar(1L)))
  expect_true(obj_is_vector(foobar(1)))
  expect_true(obj_is_vector(foobar(1i)))
  expect_true(obj_is_vector(foobar("foo")))
  expect_true(obj_is_vector(foobar(as.raw(1))))
})

test_that("bare lists are vectors", {
  expect_true(obj_is_vector(list()))
})

test_that("S3 lists are not vectors by default", {
  expect_false(obj_is_vector(foobar()))
  expect_false(obj_is_list(foobar()))

  local_foobar_proxy()

  # TODO: These seem inconsistent.
  # Should we require that S3 list proxies satisfy `obj_is_list()`?
  # (i.e. unclass themselves or explicitly inherit from `"list"`?)
  expect_true(obj_is_vector(foobar()))
  expect_false(obj_is_list(foobar()))
})

test_that("data frames and records are vectors", {
  expect_true(obj_is_vector(mtcars))
  expect_true(obj_is_vector(new_rcrd(list(x = 1, y = 2))))
})

test_that("non-vector base types are scalars", {
  expect_identical(vec_typeof(quote(foo)), "scalar")
  expect_identical(vec_typeof(pairlist("")), "scalar")
  expect_identical(vec_typeof(function() NULL), "scalar")
  expect_identical(vec_typeof(env()), "scalar")
  expect_identical(vec_typeof(quote(foo)), "scalar")
  expect_identical(vec_typeof(~foo), "scalar")
  expect_identical(vec_typeof(base::`{`), "scalar")
  expect_identical(vec_typeof(base::c), "scalar")
  expect_identical(vec_typeof(expression()), "scalar")

  expect_false(obj_is_vector(quote(foo)))
  expect_false(obj_is_vector(pairlist("")))
  expect_false(obj_is_vector(function() NULL))
  expect_false(obj_is_vector(env()))
  expect_false(obj_is_vector(~foo))
  expect_false(obj_is_vector(base::`{`))
  expect_false(obj_is_vector(base::c))
  expect_false(obj_is_vector(expression()))

  expect_false(vec_is(quote(foo)))
  expect_false(vec_is(pairlist("")))
  expect_false(vec_is(function() NULL))
  expect_false(vec_is(env()))
  expect_false(vec_is(~foo))
  expect_false(vec_is(base::`{`))
  expect_false(vec_is(base::c))
  expect_false(vec_is(expression()))

  expect_error(vec_assert(quote(foo)), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(pairlist("")), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(function() NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(env()), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(~foo), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(base::`{`), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(base::c), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(expression()), class = "vctrs_error_scalar_type")
})

test_that("non-vector types can be proxied", {
  x <- new_proxy(1:3)

  expect_identical(vec_typeof(x), "scalar")
  expect_false(obj_is_vector(x))
  expect_false(vec_is(x))
  expect_error(vec_assert(x), class = "vctrs_error_scalar_type")

  local_env_proxy()

  expect_identical(vec_typeof(x), "integer")
  expect_true(obj_is_vector(x))
  expect_true(vec_is(x))
  expect_error(regexp = NA, vec_assert(x))
})

test_that("obj_check_vector() is silent on vectors", {
  expect_null(obj_check_vector(1))
  expect_null(obj_check_vector(data_frame()))
})

test_that("obj_check_vector() errors on scalars", {
  expect_snapshot(error = TRUE, {
    obj_check_vector(quote(foo))
  })
  expect_snapshot(error = TRUE, {
    obj_check_vector(foobar())
  })
})

test_that("obj_check_vector() error respects `arg` and `call`", {
  my_check_vector <- function(foo) {
    obj_check_vector(foo)
  }

  expect_snapshot(error = TRUE, {
    my_check_vector(foobar())
  })
})

test_that("vec_assert() uses friendly type in error messages", {
   # Friendly type will be generated in rlang in the future. Upstream
   # changes should not cause CRAN failures.
  skip_on_cran()
  expect_error(vec_assert(function() NULL), class = "vctrs_error_scalar_type")
})

test_that("vec_typeof() handles all types", {
  for (i in seq_along(empty_types)) {
    expect_identical(vec_typeof(!!empty_types[[i]]), !!names(empty_types)[[i]])
  }
})

test_that("bare prototypes don't act as partial types", {
  expect_false(vec_is(foobar(1), dbl()))
  expect_error(vec_assert(foobar(1), dbl()), class = "vctrs_error_assert_ptype")
})

test_that("data frames are always classified as such even when dispatch is off", {
  expect_identical(vec_typeof_bare(mtcars), "dataframe")
})

test_that("assertion is not applied on proxy", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) unclass(x),
    vec_restore.vctrs_foobar = function(x, ...) foobar(x),
    `[.vctrs_foobar` = function(x, i) vec_slice(x, i)
  )
  x <- foobar(list())

  expect_true(vec_is(x, x))
  expect_false(vec_is(x, list()))

  expect_error(vec_assert(x, list()), class = "vctrs_error_assert_ptype")
  expect_error(vec_assert(x, x), regexp = NA)
})

test_that("attributes of unclassed vectors are asserted", {
  x <- structure(FALSE, foo = "bar")
  y <- structure(TRUE, foo = "bar")
  expect_false(vec_is(x, FALSE))
  expect_false(vec_is(FALSE, x))
  expect_true(vec_is(y, x))
  expect_true(vec_is(x, y))
})

test_that("unspecified is finalised before assertion", {
  expect_true(vec_is(NA, TRUE))
  expect_true(vec_is(data.frame(x = NA), data.frame(x = lgl())))

  expect_error(regexp = NA, vec_assert(NA, TRUE))
  expect_error(regexp = NA, vec_assert(data.frame(x = NA), data.frame(x = lgl())))
})

test_that("assertion failures are explained", {
  local_no_stringsAsFactors()
  local_options(rlang_backtrace_on_error = "none")

  expect_snapshot(error = TRUE, vec_assert(lgl(), chr()))

  expect_snapshot(error = TRUE, vec_assert(lgl(), factor()))
  expect_snapshot(error = TRUE, vec_assert(lgl(), factor(levels = "foo")))
  expect_snapshot(error = TRUE, vec_assert(factor(levels = "bar"), factor(levels = "foo")))

  expect_snapshot(error = TRUE, vec_assert(factor(), chr()))

  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame()))
  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame(x = 1)))
  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame(x = 1, y = 2)))

  expect_snapshot(error = TRUE, vec_assert(data.frame(), chr()))

  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1), chr()))
  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1), data.frame(x = "foo")))
  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1), data.frame(x = "foo", y = 2)))

  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1, y = 2), chr()))

  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo")))
  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo", y = 2)))
})

test_that("vec_assert() validates `size` (#1470)", {
  expect_snapshot({
    (expect_error(vec_assert(1, size = c(2, 3))))
    (expect_error(vec_assert(1, size = 1.5)))
    (expect_error(vec_assert(1, size = "x")))
  })
})

test_that("NULL is not a vector", {
  expect_false(obj_is_vector(NULL))
  expect_false(vec_is(NULL))
})

test_that("names and row names do not influence type identity (#707)", {
  expect_true(vec_is(c(a = TRUE), logical()))
  expect_true(vec_is(TRUE, c(a = TRUE)))
  expect_true(vec_is(structure(mtcars, row.names = 1:32), mtcars))
  expect_true(vec_is(mtcars, structure(mtcars, row.names = 1:32)))
})

# vec_check_size --------------------------------------------------------

test_that("vec_check_size() is silent if the size is right", {
  expect_null(vec_check_size(1:5, size = 5L))
  expect_null(vec_check_size(data_frame(.size = 10L), size = 10L))
})

test_that("vec_check_size() errors on the wrong size", {
  expect_snapshot(error = TRUE, {
    vec_check_size(1:5, size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1:5, size = 10L)
  })
})

test_that("vec_check_size() errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_check_size(quote(foo), size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(foobar(), size = 1L)
  })
})

test_that("vec_check_size() error respects `arg` and `call`", {
  my_check_size <- function(foo, size) {
    vec_check_size(foo, size)
  }

  expect_snapshot(error = TRUE, {
    my_check_size(1L, size = 5L)
  })
  expect_snapshot(error = TRUE, {
    my_check_size(foobar(), size = 5L)
  })
})

test_that("vec_check_size() validates `size`", {
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = c(1L, 2L))
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = 1.5)
  })
})

# obj_is_list -----------------------------------------------------------

test_that("bare lists are lists", {
  expect_true(obj_is_list(list()))
})

test_that("AsIs lists are lists (#1463)", {
  expect_true(obj_is_list(I(list())))
  expect_true(obj_is_list(I(list_of(1))))
  expect_false(obj_is_list(I(double())))
})

test_that("list_of are lists", {
  expect_true(obj_is_list(new_list_of()))
})

test_that("Vectors with a non-VECSXP type are not lists", {
  expect_false(obj_is_list(1))
  expect_false(obj_is_list("a"))
  expect_false(obj_is_list(quote(name)))
})

test_that("explicitly classed lists are lists", {
  x <- structure(list(), class = "list")

  expect_true(obj_is_list(x))
  expect_true(obj_is_list(subclass(x)))
})

test_that("explicit inheritance must be in the base class", {
  x <- structure(1:2, class = c("list", "foobar"))
  expect_false(obj_is_list(x))
})

test_that("POSIXlt are not considered a list", {
  expect_false(obj_is_list(as.POSIXlt(new_datetime())))
})

test_that("rcrd types are not lists", {
  expect_false(obj_is_list(new_rcrd(list(x = 1))))
})

test_that("scalars are not lists", {
  expect_false(obj_is_list(foobar()))
})

test_that("S3 types can't lie about their internal representation", {
  x <- structure(1:2, class = c("foobar", "list"))
  expect_false(obj_is_list(x))
})

test_that("data frames of all types are not lists", {
  expect_false(obj_is_list(data.frame()))
  expect_false(obj_is_list(subclass(data.frame())))
  expect_false(obj_is_list(tibble::tibble()))
})

test_that("S3 list with non-list proxy is still a list (#1208)", {
  x <- structure(list(), class = c("foobar", "list"))
  local_methods(vec_proxy.foobar = function(x) 1)
  # This used to be an error (#1003)
  # expect_error(obj_is_list(x), "`x` inherits")
  expect_true(obj_is_list(x))
})

test_that("list-rcrds with data frame proxies are considered lists (#1208)", {
  x <- structure(
    list(1:2, "x"),
    special = c("a", "b"),
    class = c("list_rcrd", "list")
  )

  local_methods(
    vec_proxy.list_rcrd = function(x) {
      special <- attr(x, "special")
      data <- unstructure(x)
      new_data_frame(list(data = data, special = special))
    }
  )

  expect_true(obj_is_list(x))
})

test_that("list_all_vectors() works", {
  expect_true(list_all_vectors(list(1)))
  expect_true(list_all_vectors(list_of(1)))
  expect_false(list_all_vectors(list(1, env())))
  expect_snapshot((expect_error(list_all_vectors(env()))))
})

test_that("obj_check_list() works", {
  expect_null(obj_check_list(list(1)))
  expect_null(obj_check_list(list_of(1)))
  expect_snapshot({
    my_function <- function(my_arg) obj_check_list(my_arg)
    (expect_error(my_function(env())))
  })
})

test_that("obj_check_list() uses a special error when `arg` is the empty string (#1604)", {
  expect_snapshot(error = TRUE, {
    obj_check_list(1, arg = "")
  })
})

test_that("obj_check_list() and list_check_all_vectors() work", {
  expect_null(list_check_all_vectors(list()))
  expect_null(list_check_all_vectors(list(1, mtcars)))
  expect_snapshot({
    my_function <- function(my_arg) list_check_all_vectors(my_arg)
    (expect_error(my_function(env())))
    (expect_error(my_function(list(1, env()))))
    (expect_error(my_function(list(1, name = env()))))
    (expect_error(my_function(list(1, foo = env()))))
  })
})

test_that("list_all_size() works", {
  expect_true(list_all_size(list(), 2))
  expect_true(list_all_size(list(integer()), 0))
  expect_true(list_all_size(list(NULL), 0))
  expect_true(list_all_size(list(1:2, 2:3), 2))

  expect_false(list_all_size(list(1:2, 1:3), 2))
  expect_false(list_all_size(list(NULL, 1:2), 2))

  expect_true(list_all_size(list_of(1:3, 2:4), 3))
  expect_false(list_all_size(list_of(1:3, 2:4), 4))
})

test_that("list_check_all_size() works", {
  expect_null(list_check_all_size(list(), 2))
  expect_null(list_check_all_size(list(integer()), 0))
  expect_null(list_check_all_size(list(NULL), 0))
  expect_null(list_check_all_size(list(1:2, 2:3), 2))

  expect_snapshot({
    my_function <- function(my_arg, size) list_check_all_size(my_arg, size)

    # Validates sizes
    (expect_error(list_check_all_size(list(1:2, 1:3), 2)))
    (expect_error(my_function(list(1:2, 1:3), 2)))

    # `NULL` is not ignored
    (expect_error(my_function(list(NULL, 1:2), 2)))
  })
})

test_that("list_all_size() and list_check_all_size() error on scalars", {
  x <- list(env())

  expect_snapshot({
    # Error considered internal to `list_all_size()`
    (expect_error(list_all_size(x, 2)))

    my_function <- function(my_arg, size) list_check_all_size(my_arg, size)
    (expect_error(my_function(x, 2)))
  })
})

test_that("list_all_size() and list_check_all_size() throw error using internal call on non-list input", {
  expect_snapshot({
    (expect_error(list_all_size(1, 2)))

    # `arg` and `call` are ignored
    (expect_error(list_check_all_size(1, 2, arg = "arg", call = call("foo"))))
  })
})

test_that("list_all_size() and list_check_all_size() validate `size`", {
  expect_snapshot({
    (expect_error(list_all_size(list(), size = "x")))
    (expect_error(list_check_all_size(list(), size = "x")))
  })
})

test_that("informative messages when 1d array doesn't match vector", {
  x <- array(1:3)
  expect_snapshot((expect_error(vec_assert(x, int()))))
})

test_that("basic assert is idempotent", {
  x <- new_vctr(1:4)

  expect_true(vec_is(x))
  expect_identical(vec_assert(x), x)

  expect_identical(vec_assert(x), new_vctr(1:4))
  expect_false(withVisible(vec_assert(x))$visible)

  expect_true(vec_is(1:4))
  expect_identical(vec_assert(1:4), 1:4)
})

test_that("asserting ptype", {
  x <- new_vctr(1:4)
  good <- new_vctr(integer())

  expect_true(vec_is(x, good))
  expect_error(vec_assert(x, good), NA)

  # Is this the correct error message?
  bad <- new_vctr(double())
  expect_false(vec_is(x, bad))
  expect_error(vec_assert(x, bad), class = "vctrs_error_assert_ptype")
})

test_that("asserting size", {
  x <- new_vctr(1:4)

  expect_true(vec_is(x, size = 4))
  expect_error(vec_assert(x, size = 4), NA)

  expect_false(vec_is(x, size = 5))
  expect_error(vec_assert(x, size = 5), class = "vctrs_error_assert_size")
})

test_that("vec_assert() labels input", {
  expect_error(
    vec_assert(new_vctr(1:4), size = 5),
    regexp = "`new_vctr\\(1:4\\)` must have",
    class = "vctrs_error_assert_size"
  )
  expect_error(
    vec_assert(new_vctr(1:4), size = 5, arg = "foobar"),
    regexp = "`foobar` must have",
    class = "vctrs_error_assert_size"
  )
})

test_that("bare atomic vectors are vectors but not recursive", {
  expect_true(obj_is_vector(TRUE))
  expect_true(obj_is_vector(1L))
  expect_true(obj_is_vector(1))
  expect_true(obj_is_vector(1i))
  expect_true(obj_is_vector("foo"))
  expect_true(obj_is_vector(as.raw(1)))
})

test_that("S3 atomic vectors are vectors", {
  expect_true(obj_is_vector(foobar(TRUE)))
  expect_true(obj_is_vector(foobar(1L)))
  expect_true(obj_is_vector(foobar(1)))
  expect_true(obj_is_vector(foobar(1i)))
  expect_true(obj_is_vector(foobar("foo")))
  expect_true(obj_is_vector(foobar(as.raw(1))))
})

test_that("bare lists are vectors", {
  expect_true(obj_is_vector(list()))
})

test_that("S3 lists are not vectors by default", {
  expect_false(obj_is_vector(foobar()))
  expect_false(obj_is_list(foobar()))

  local_foobar_proxy()

  # TODO: These seem inconsistent.
  # Should we require that S3 list proxies satisfy `obj_is_list()`?
  # (i.e. unclass themselves or explicitly inherit from `"list"`?)
  expect_true(obj_is_vector(foobar()))
  expect_false(obj_is_list(foobar()))
})

test_that("data frames and records are vectors", {
  expect_true(obj_is_vector(mtcars))
  expect_true(obj_is_vector(new_rcrd(list(x = 1, y = 2))))
})

test_that("non-vector base types are scalars", {
  expect_identical(vec_typeof(quote(foo)), "scalar")
  expect_identical(vec_typeof(pairlist("")), "scalar")
  expect_identical(vec_typeof(function() NULL), "scalar")
  expect_identical(vec_typeof(env()), "scalar")
  expect_identical(vec_typeof(quote(foo)), "scalar")
  expect_identical(vec_typeof(~foo), "scalar")
  expect_identical(vec_typeof(base::`{`), "scalar")
  expect_identical(vec_typeof(base::c), "scalar")
  expect_identical(vec_typeof(expression()), "scalar")

  expect_false(obj_is_vector(quote(foo)))
  expect_false(obj_is_vector(pairlist("")))
  expect_false(obj_is_vector(function() NULL))
  expect_false(obj_is_vector(env()))
  expect_false(obj_is_vector(~foo))
  expect_false(obj_is_vector(base::`{`))
  expect_false(obj_is_vector(base::c))
  expect_false(obj_is_vector(expression()))

  expect_false(vec_is(quote(foo)))
  expect_false(vec_is(pairlist("")))
  expect_false(vec_is(function() NULL))
  expect_false(vec_is(env()))
  expect_false(vec_is(~foo))
  expect_false(vec_is(base::`{`))
  expect_false(vec_is(base::c))
  expect_false(vec_is(expression()))

  expect_error(vec_assert(quote(foo)), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(pairlist("")), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(function() NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(env()), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(~foo), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(base::`{`), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(base::c), class = "vctrs_error_scalar_type")
  expect_error(vec_assert(expression()), class = "vctrs_error_scalar_type")
})

test_that("non-vector types can be proxied", {
  x <- new_proxy(1:3)

  expect_identical(vec_typeof(x), "scalar")
  expect_false(obj_is_vector(x))
  expect_false(vec_is(x))
  expect_error(vec_assert(x), class = "vctrs_error_scalar_type")

  local_env_proxy()

  expect_identical(vec_typeof(x), "integer")
  expect_true(obj_is_vector(x))
  expect_true(vec_is(x))
  expect_error(regexp = NA, vec_assert(x))
})

test_that("obj_check_vector() is silent on vectors", {
  expect_null(obj_check_vector(1))
  expect_null(obj_check_vector(data_frame()))
})

test_that("obj_check_vector() errors on scalars", {
  expect_snapshot(error = TRUE, {
    obj_check_vector(quote(foo))
  })
  expect_snapshot(error = TRUE, {
    obj_check_vector(foobar())
  })
})

test_that("obj_check_vector() error respects `arg` and `call`", {
  my_check_vector <- function(foo) {
    obj_check_vector(foo)
  }

  expect_snapshot(error = TRUE, {
    my_check_vector(foobar())
  })
})

test_that("obj_check_vector() error contains FAQ links and correct bullets", {
  # Expect to see:
  # - Link to general FAQ about scalar types
  x <- expression()
  expect_snapshot(error = TRUE, obj_check_vector(x))

  # Expect to see:
  # - Bullet about incompatible S3 list
  # - Full class list
  # - Link to specific FAQ about creating vectors
  x <- structure(list(), class = "my_list")
  expect_snapshot(error = TRUE, obj_check_vector(x))

  # Expect to see:
  # - Bullet about incompatible data frame
  # - Full class list
  # - Link to specific FAQ about creating vectors
  x <- data_frame(a = 1)
  class(x) <- c("data.frame", "my_df")
  expect_snapshot(error = TRUE, obj_check_vector(x))
})

test_that("vec_assert() uses friendly type in error messages", {
  # Friendly type will be generated in rlang in the future. Upstream
  # changes should not cause CRAN failures.
  skip_on_cran()
  expect_error(vec_assert(function() NULL), class = "vctrs_error_scalar_type")
})

test_that("vec_typeof() handles all types", {
  for (i in seq_along(empty_types)) {
    expect_identical(vec_typeof(!!empty_types[[i]]), !!names(empty_types)[[i]])
  }
})

test_that("bare prototypes don't act as partial types", {
  expect_false(vec_is(foobar(1), dbl()))
  expect_error(vec_assert(foobar(1), dbl()), class = "vctrs_error_assert_ptype")
})

test_that("data frames are always classified as such even when dispatch is off", {
  expect_identical(vec_typeof_bare(mtcars), "dataframe")
})

test_that("assertion is not applied on proxy", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) unclass(x),
    vec_restore.vctrs_foobar = function(x, ...) foobar(x),
    `[.vctrs_foobar` = function(x, i) vec_slice(x, i)
  )
  x <- foobar(list())

  expect_true(vec_is(x, x))
  expect_false(vec_is(x, list()))

  expect_error(vec_assert(x, list()), class = "vctrs_error_assert_ptype")
  expect_error(vec_assert(x, x), regexp = NA)
})

test_that("attributes of unclassed vectors are asserted", {
  x <- structure(FALSE, foo = "bar")
  y <- structure(TRUE, foo = "bar")
  expect_false(vec_is(x, FALSE))
  expect_false(vec_is(FALSE, x))
  expect_true(vec_is(y, x))
  expect_true(vec_is(x, y))
})

test_that("unspecified is finalised before assertion", {
  expect_true(vec_is(NA, TRUE))
  expect_true(vec_is(data.frame(x = NA), data.frame(x = lgl())))

  expect_error(regexp = NA, vec_assert(NA, TRUE))
  expect_error(
    regexp = NA,
    vec_assert(data.frame(x = NA), data.frame(x = lgl()))
  )
})

test_that("assertion failures are explained", {
  local_no_stringsAsFactors()
  local_options(rlang_backtrace_on_error = "none")

  expect_snapshot(error = TRUE, vec_assert(lgl(), chr()))

  expect_snapshot(error = TRUE, vec_assert(lgl(), factor()))
  expect_snapshot(error = TRUE, vec_assert(lgl(), factor(levels = "foo")))
  expect_snapshot(
    error = TRUE,
    vec_assert(factor(levels = "bar"), factor(levels = "foo"))
  )

  expect_snapshot(error = TRUE, vec_assert(factor(), chr()))

  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame()))
  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame(x = 1)))
  expect_snapshot(error = TRUE, vec_assert(lgl(), data.frame(x = 1, y = 2)))

  expect_snapshot(error = TRUE, vec_assert(data.frame(), chr()))

  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1), chr()))
  expect_snapshot(
    error = TRUE,
    vec_assert(data.frame(x = 1), data.frame(x = "foo"))
  )
  expect_snapshot(
    error = TRUE,
    vec_assert(data.frame(x = 1), data.frame(x = "foo", y = 2))
  )

  expect_snapshot(error = TRUE, vec_assert(data.frame(x = 1, y = 2), chr()))

  expect_snapshot(
    error = TRUE,
    vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo"))
  )
  expect_snapshot(
    error = TRUE,
    vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo", y = 2))
  )
})

test_that("vec_assert() validates `size` (#1470)", {
  expect_snapshot({
    (expect_error(vec_assert(1, size = c(2, 3))))
    (expect_error(vec_assert(1, size = 1.5)))
    (expect_error(vec_assert(1, size = "x")))
  })
})

test_that("NULL is not a vector", {
  expect_false(obj_is_vector(NULL))
  expect_false(vec_is(NULL))
})

test_that("names and row names do not influence type identity (#707)", {
  expect_true(vec_is(c(a = TRUE), logical()))
  expect_true(vec_is(TRUE, c(a = TRUE)))
  expect_true(vec_is(structure(mtcars, row.names = 1:32), mtcars))
  expect_true(vec_is(mtcars, structure(mtcars, row.names = 1:32)))
})

# vec_check_size --------------------------------------------------------

test_that("vec_check_size() is silent if the size is right", {
  expect_null(vec_check_size(1:5, size = 5L))
  expect_null(vec_check_size(data_frame(.size = 10L), size = 10L))
})

test_that("vec_check_size() errors on the wrong size", {
  expect_snapshot(error = TRUE, {
    vec_check_size(1:5, size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1:5, size = 10L)
  })
})

test_that("vec_check_size() errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_check_size(quote(foo), size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(foobar(), size = 1L)
  })
})

test_that("vec_check_size() error respects `arg` and `call`", {
  my_check_size <- function(foo, size) {
    vec_check_size(foo, size)
  }

  expect_snapshot(error = TRUE, {
    my_check_size(1L, size = 5L)
  })
  expect_snapshot(error = TRUE, {
    my_check_size(foobar(), size = 5L)
  })
})

test_that("vec_check_size() validates `size`", {
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = c(1L, 2L))
  })
  expect_snapshot(error = TRUE, {
    vec_check_size(1, size = 1.5)
  })
})

# vec_check_recyclable --------------------------------------------------------

test_that("vec_check_recyclable() is silent if the size is right", {
  expect_null(vec_check_recyclable(1:5, size = 5L))
  expect_null(vec_check_recyclable(1, size = 5L))
  expect_null(vec_check_recyclable(data_frame(.size = 10L), size = 10L))
  expect_null(vec_check_recyclable(data_frame(.size = 1L), size = 10L))
})

test_that("vec_check_recyclable() errors on the wrong size", {
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(1:5, size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(1:5, size = 10L)
  })
})

test_that("vec_check_recyclable() errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(quote(foo), size = 1L)
  })
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(foobar(), size = 1L)
  })
})

test_that("vec_check_recyclable() error respects `arg` and `call`", {
  my_check_recyclable <- function(foo, size) {
    vec_check_recyclable(foo, size)
  }

  expect_snapshot(error = TRUE, {
    my_check_recyclable(1:2, size = 5L)
  })
  expect_snapshot(error = TRUE, {
    my_check_recyclable(foobar(), size = 5L)
  })
})

test_that("vec_check_recyclable() validates `size`", {
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(1, size = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(1, size = c(1L, 2L))
  })
  expect_snapshot(error = TRUE, {
    vec_check_recyclable(1, size = 1.5)
  })
})

# obj_is_list and friends -----------------------------------------------

test_that("bare lists are lists", {
  expect_true(obj_is_list(list()))
})

test_that("AsIs lists are lists (#1463)", {
  expect_true(obj_is_list(I(list())))
  expect_true(obj_is_list(I(list_of(1))))
  expect_false(obj_is_list(I(double())))
})

test_that("list_of are lists", {
  expect_true(obj_is_list(new_list_of()))
})

test_that("Vectors with a non-VECSXP type are not lists", {
  expect_false(obj_is_list(1))
  expect_false(obj_is_list("a"))
  expect_false(obj_is_list(quote(name)))
})

test_that("List arrays are not lists", {
  # Bare list arrays
  x <- array(list(1))
  expect_false(obj_is_list(x))

  # Classed list arrays
  x <- structure(list(1), class = "list", dim = 1)
  expect_false(obj_is_list(x))
})

test_that("explicitly classed lists are lists", {
  x <- structure(list(), class = "list")

  expect_true(obj_is_list(x))
  expect_true(obj_is_list(subclass(x)))
})

test_that("explicit inheritance must be in the base class", {
  x <- structure(1:2, class = c("list", "foobar"))
  expect_false(obj_is_list(x))
})

test_that("POSIXlt are not considered a list", {
  expect_false(obj_is_list(as.POSIXlt(new_datetime())))
})

test_that("rcrd types are not lists", {
  expect_false(obj_is_list(new_rcrd(list(x = 1))))
})

test_that("scalars are not lists", {
  expect_false(obj_is_list(foobar()))
})

test_that("S3 types can't lie about their internal representation", {
  x <- structure(1:2, class = c("foobar", "list"))
  expect_false(obj_is_list(x))
})

test_that("data frames of all types are not lists", {
  expect_false(obj_is_list(data.frame()))
  expect_false(obj_is_list(subclass(data.frame())))
  expect_false(obj_is_list(tibble::tibble()))
})

test_that("S3 list with non-list proxy is still a list (#1208)", {
  x <- structure(list(), class = c("foobar", "list"))
  local_methods(vec_proxy.foobar = function(x) 1)
  # This used to be an error (#1003)
  # expect_error(obj_is_list(x), "`x` inherits")
  expect_true(obj_is_list(x))
})

test_that("list-rcrds with data frame proxies are considered lists (#1208)", {
  x <- structure(
    list(1:2, "x"),
    special = c("a", "b"),
    class = c("list_rcrd", "list")
  )

  local_methods(
    vec_proxy.list_rcrd = function(x) {
      special <- attr(x, "special")
      data <- unstructure(x)
      new_data_frame(list(data = data, special = special))
    }
  )

  expect_true(obj_is_list(x))
})

test_that("list_all_vectors() works", {
  expect_true(list_all_vectors(list(1)))
  expect_true(list_all_vectors(list_of(1)))
  expect_false(list_all_vectors(list(1, env())))
  expect_snapshot((expect_error(list_all_vectors(env()))))
})

test_that("obj_check_list() works", {
  expect_null(obj_check_list(list(1)))
  expect_null(obj_check_list(list_of(1)))
  expect_snapshot({
    my_function <- function(my_arg) obj_check_list(my_arg)
    (expect_error(my_function(env())))
  })
})

test_that("obj_check_list() uses a special error when `arg` is the empty string (#1604)", {
  expect_snapshot(error = TRUE, {
    obj_check_list(1, arg = "")
  })
})

test_that("obj_check_list() and list_check_all_vectors() work", {
  expect_null(list_check_all_vectors(list()))
  expect_null(list_check_all_vectors(list(1, mtcars)))
  expect_snapshot({
    my_function <- function(my_arg) list_check_all_vectors(my_arg)
    (expect_error(my_function(env())))
    (expect_error(my_function(list(1, env()))))
    (expect_error(my_function(list(1, name = env()))))
    (expect_error(my_function(list(1, foo = env()))))
  })
})

test_that("list_all_size() works", {
  expect_true(list_all_size(list(), 2))
  expect_true(list_all_size(list(integer()), 0))
  expect_true(list_all_size(list(NULL), 0))
  expect_true(list_all_size(list(1:2, 2:3), 2))

  expect_false(list_all_size(list(1:2, 1:3), 2))
  expect_false(list_all_size(list(NULL, 1:2), 2))

  expect_true(list_all_size(list_of(1:3, 2:4), 3))
  expect_false(list_all_size(list_of(1:3, 2:4), 4))
})

test_that("list_check_all_size() works", {
  expect_null(list_check_all_size(list(), 2))
  expect_null(list_check_all_size(list(integer()), 0))
  expect_null(list_check_all_size(list(NULL), 0))
  expect_null(list_check_all_size(list(1:2, 2:3), 2))

  expect_snapshot({
    my_function <- function(my_arg, size) list_check_all_size(my_arg, size)

    # Validates sizes
    (expect_error(list_check_all_size(list(1:2, 1:3), 2)))
    (expect_error(my_function(list(1:2, 1:3), 2)))

    # `NULL` is not ignored
    (expect_error(my_function(list(NULL, 1:2), 2)))
  })
})

test_that("list_all_recyclable() works", {
  expect_true(list_all_recyclable(list(), 2))
  expect_true(list_all_recyclable(list(integer()), 0))
  expect_true(list_all_recyclable(list(NULL), 0))
  expect_true(list_all_recyclable(list(1:2, 2:3), 2))
  expect_true(list_all_recyclable(list(1, 2:3), 2))

  expect_false(list_all_recyclable(list(1:2, 1:3), 2))
  expect_false(list_all_recyclable(list(NULL, 1:2), 2))
  expect_false(list_all_recyclable(list(1, 1:2), 1))

  expect_true(list_all_recyclable(list_of(1:3, 2:4), 3))
  expect_false(list_all_recyclable(list_of(1:3, 2:4), 4))
})

test_that("list_check_all_recyclable() works", {
  expect_null(list_check_all_recyclable(list(), 2))
  expect_null(list_check_all_recyclable(list(integer()), 0))
  expect_null(list_check_all_recyclable(list(NULL), 0))
  expect_null(list_check_all_recyclable(list(1:2, 2:3), 2))
  expect_null(list_check_all_recyclable(list(1, 2:3), 2))

  expect_snapshot({
    my_function <- function(my_arg, size) {
      list_check_all_recyclable(my_arg, size)
    }

    # Validates sizes
    (expect_error(list_check_all_recyclable(list(1:2, 1:3), 2)))
    (expect_error(my_function(list(1:2, 1:3), 2)))

    # `NULL` is not ignored
    (expect_error(my_function(list(NULL, 1:2), 2)))
  })
})

test_that("list_all_size() and list_check_all_size() error on scalars", {
  x <- list(env())

  expect_snapshot({
    # Error considered internal to `list_all_size()`
    (expect_error(list_all_size(x, 2)))

    my_function <- function(my_arg, size) list_check_all_size(my_arg, size)
    (expect_error(my_function(x, 2)))
  })
})

test_that("list_all_recyclable() and list_check_all_recyclable() error on scalars", {
  x <- list(env())

  expect_snapshot({
    # Error considered internal to `list_all_recyclable()`
    (expect_error(list_all_recyclable(x, 2)))

    my_function <- function(my_arg, size) {
      list_check_all_recyclable(my_arg, size)
    }
    (expect_error(my_function(x, 2)))
  })
})

test_that("list_all_size() and list_check_all_size() throw error using internal call on non-list input", {
  expect_snapshot({
    (expect_error(list_all_size(1, 2)))

    # `arg` and `call` are ignored
    (expect_error(list_check_all_size(1, 2, arg = "arg", call = call("foo"))))
  })
})

test_that("list_all_size() and list_check_all_size() validate `size`", {
  expect_snapshot({
    (expect_error(list_all_size(list(), size = "x")))
    (expect_error(list_check_all_size(list(), size = "x")))
  })
})

test_that("list_all_recyclable() and list_check_all_recyclable() validate `size`", {
  expect_snapshot({
    (expect_error(list_all_recyclable(list(), size = "x")))
    (expect_error(list_check_all_recyclable(list(), size = "x")))
  })
})

test_that("list_all_size() works with `allow_null`", {
  x <- list(1, NULL, 2)
  expect_false(list_all_size(x, size = 1))
  expect_true(list_all_size(x, size = 1, allow_null = TRUE))

  x <- list(1, NULL, 2:3)
  expect_false(list_all_size(x, size = 1))
  expect_false(list_all_size(x, size = 1, allow_null = TRUE))

  # `NULL` size is 0 by default
  x <- list(integer(), NULL, integer())
  expect_true(list_all_size(x, size = 0))
  expect_true(list_all_size(x, size = 0, allow_null = TRUE))
})

test_that("list_check_all_size() works with `allow_null`", {
  x <- list(1, NULL, 2)
  expect_snapshot(error = TRUE, {
    list_check_all_size(x, size = 1)
  })
  expect_null(list_check_all_size(x, size = 1, allow_null = TRUE))

  # Index of 3rd element is reported correctly
  x <- list(1, NULL, 2:3)
  expect_snapshot(error = TRUE, {
    list_check_all_size(x, size = 1)
  })
  expect_snapshot(error = TRUE, {
    list_check_all_size(x, size = 1, allow_null = TRUE)
  })

  # `NULL` size is 0 by default
  x <- list(integer(), NULL, integer())
  expect_null(list_check_all_size(x, size = 0))
  expect_null(list_check_all_size(x, size = 0, allow_null = TRUE))
})

test_that("list_all_vectors() works with `allow_null`", {
  x <- list(1, NULL, 2)
  expect_false(list_all_vectors(x))
  expect_true(list_all_vectors(x, allow_null = TRUE))

  x <- list(1, NULL, environment())
  expect_false(list_all_vectors(x))
  expect_false(list_all_vectors(x, allow_null = TRUE))
})

test_that("list_check_all_vectors() works with `allow_null`", {
  x <- list(1, NULL, 2)
  expect_snapshot(error = TRUE, {
    list_check_all_vectors(x)
  })
  expect_null(list_check_all_vectors(x, allow_null = TRUE))

  # Index of 3rd element is reported correctly
  x <- list(1, NULL, environment())
  expect_snapshot(error = TRUE, {
    list_check_all_vectors(x)
  })
  expect_snapshot(error = TRUE, {
    list_check_all_vectors(x, allow_null = TRUE)
  })
})

test_that("list_all_recyclable() works with `allow_null`", {
  x <- list(1, NULL, 2:3)
  expect_false(list_all_recyclable(x, size = 2))
  expect_true(list_all_recyclable(x, size = 2, allow_null = TRUE))

  x <- list(1, NULL, 2:4)
  expect_false(list_all_recyclable(x, size = 2))
  expect_false(list_all_recyclable(x, size = 2, allow_null = TRUE))

  # `NULL` size is 0 by default
  x <- list(integer(), NULL, integer())
  expect_true(list_all_recyclable(x, size = 0))
  expect_true(list_all_recyclable(x, size = 0, allow_null = TRUE))
})

test_that("list_check_all_recyclable() works with `allow_null`", {
  x <- list(1, NULL, 2:3)
  expect_snapshot(error = TRUE, {
    list_check_all_recyclable(x, size = 2)
  })
  expect_null(list_check_all_recyclable(x, size = 2, allow_null = TRUE))

  # Index of 3rd element is reported correctly
  x <- list(1, NULL, 2:4)
  expect_snapshot(error = TRUE, {
    list_check_all_recyclable(x, size = 2)
  })
  expect_snapshot(error = TRUE, {
    list_check_all_recyclable(x, size = 2, allow_null = TRUE)
  })

  # `NULL` size is 0 by default
  x <- list(integer(), NULL, integer())
  expect_null(list_check_all_recyclable(x, size = 0))
  expect_null(list_check_all_recyclable(x, size = 0, allow_null = TRUE))
})

test_that("informative messages when 1d array doesn't match vector", {
  x <- array(1:3)
  expect_snapshot((expect_error(vec_assert(x, int()))))
})

#' Table S3 class
#'
#' These functions help the base table class fit into the vctrs type system
#' by providing coercion and casting functions.
#'
#' @keywords internal
#' @name table
NULL

#' @export
vec_restore.table <- function(x, to, ...) {
  new_table(x, dim = dim(x), dimnames = dimnames(x))
}

# Print -------------------------------------------------------------------

#' @export
vec_ptype_full.table <- function(x, ...) {
  paste0("table", vec_ptype_shape(x))
}

#' @export
vec_ptype_abbr.table <- function(x, ...) {
  "table"
}

# Coercion ----------------------------------------------------------------

#' @export
vec_ptype2.table.table <- function(x, y, ..., x_arg = "", y_arg = "") {
  ptype <- vec_ptype2(unstructure(x), unstructure(y))
  vec_shaped_ptype(new_table(ptype), x, y, x_arg = x_arg, y_arg = y_arg)
}

#' @export
vec_cast.table.table <- function(x, to, ...) {
  out <- vec_cast(unstructure(x), unstructure(to))
  out <- new_table(out, dim = dim(x), dimnames = dimnames(x))
  shape_broadcast(out, to, ...)
}

# ------------------------------------------------------------------------------

new_table <- function(x = integer(), dim = NULL, dimnames = NULL) {
  if (is_null(dim)) {
    dim <- length(x)
  } else if (!is.integer(dim)) {
    abort("`dim` must be an integer vector.")
  }

  n_elements <- prod(dim)
  n_x <- length(x)

  if (n_elements != n_x) {
    abort(glue::glue(
      "Length implied by `dim`, {n_elements}, must match the length of `x`, {n_x}."
    ))
  }

  structure(x, dim = dim, dimnames = dimnames, class = "table")
}

is_bare_table <- function(x) {
  identical(class(x), "table")
}

#' Parallel `any()` and `all()`
#'
#' @description
#' These functions are variants of [any()] and [all()] that work in parallel on
#' multiple inputs at once. They work similarly to how [pmin()] and [pmax()] are
#' parallel variants of [min()] and [max()].
#'
#' @inheritParams rlang::args_error_context
#'
#' @param ... Logical vectors of equal size.
#'
#' @param .missing Value to use when a missing value is encountered. One of:
#'
#'   - `NA` to propagate missing values. With this, missings are treated the
#'     same way as `|` or `&`.
#'
#'   - `FALSE` to treat missing values as `FALSE`.
#'
#'   - `TRUE` to treat missing values as `TRUE`.
#'
#' @param .size An optional output size. Only useful to specify if it is possible
#'   for no inputs to be provided.
#'
#' @param .arg Argument name used in error messages.
#'
#' @returns A logical vector the same size as the vectors in `...`.
#'
#' @details
#' `vec_pany()` and `vec_pall()` are consistent with [any()] and [all()] when
#' there are no inputs to process in parallel:
#'
#' - `any()` returns `FALSE` with no inputs. Similarly, `vec_pany(.size = 1)`
#'   returns `FALSE`.
#'
#' - `all()` returns `TRUE` with no inputs. Similarly, `vec_pall(.size = 1)`
#'   returns `TRUE`.
#'
#' @name parallel-operators
#'
#' @examples
#' a <- c(TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, NA, NA, NA)
#' b <- c(TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA)
#'
#' # Default behavior treats missings like `|` does
#' vec_pany(a, b)
#' a | b
#'
#' # Default behavior treats missings like `&` does
#' vec_pall(a, b)
#' a & b
#'
#' # Remove missings from the computation, like `na_rm = TRUE`
#' vec_pany(a, b, .missing = FALSE)
#' (a & !is.na(a)) | (b & !is.na(b))
#'
#' vec_pall(a, b, .missing = TRUE)
#' (a | is.na(a)) & (b | is.na(b))
#'
#' # `vec_pall()` can be used to implement a `dplyr::filter()` style API
#' df <- data_frame(id = seq_along(a), a = a, b = b)
#'
#' keep_rows <- function(x, ...) {
#'   vec_slice(x, vec_pall(..., .missing = FALSE))
#' }
#' drop_rows <- function(x, ...) {
#'   vec_slice(x, !vec_pall(..., .missing = FALSE))
#' }
#'
#' # "Keep / Drop the rows when both a and b are TRUE"
#' # These form complements of one another, even with `NA`s.
#' keep_rows(df, a, b)
#' drop_rows(df, a, b)
#'
#' # Same empty behavior as `any()` and `all()`
#' vec_pany(.size = 1)
#' any()
#'
#' vec_pall(.size = 1)
#' all()
NULL

#' @rdname parallel-operators
#' @export
vec_pany <- function(
  ...,
  .missing = NA,
  .size = NULL,
  .arg = "",
  .error_call = current_env()
) {
  .Call(ffi_vec_pany, list2(...), .missing, .size, environment())
}

#' @rdname parallel-operators
#' @export
vec_pall <- function(
  ...,
  .missing = NA,
  .size = NULL,
  .arg = "",
  .error_call = current_env()
) {
  .Call(ffi_vec_pall, list2(...), .missing, .size, environment())
}

test_that("9 possible variations of each combination are right", {
  N <- NA

  expect_identical(vec_pall(T, T, .missing = NA), T)
  expect_identical(vec_pall(T, F, .missing = NA), F)
  expect_identical(vec_pall(T, N, .missing = NA), N)
  expect_identical(vec_pall(F, T, .missing = NA), F)
  expect_identical(vec_pall(F, F, .missing = NA), F)
  expect_identical(vec_pall(F, N, .missing = NA), F)
  expect_identical(vec_pall(N, T, .missing = NA), N)
  expect_identical(vec_pall(N, F, .missing = NA), F)
  expect_identical(vec_pall(N, N, .missing = NA), N)

  expect_identical(vec_pall(T, T, .missing = TRUE), T)
  expect_identical(vec_pall(T, F, .missing = TRUE), F)
  expect_identical(vec_pall(T, N, .missing = TRUE), T)
  expect_identical(vec_pall(F, T, .missing = TRUE), F)
  expect_identical(vec_pall(F, F, .missing = TRUE), F)
  expect_identical(vec_pall(F, N, .missing = TRUE), F)
  expect_identical(vec_pall(N, T, .missing = TRUE), T)
  expect_identical(vec_pall(N, F, .missing = TRUE), F)
  expect_identical(vec_pall(N, N, .missing = TRUE), T)

  expect_identical(vec_pall(T, T, .missing = FALSE), T)
  expect_identical(vec_pall(T, F, .missing = FALSE), F)
  expect_identical(vec_pall(T, N, .missing = FALSE), F)
  expect_identical(vec_pall(F, T, .missing = FALSE), F)
  expect_identical(vec_pall(F, F, .missing = FALSE), F)
  expect_identical(vec_pall(F, N, .missing = FALSE), F)
  expect_identical(vec_pall(N, T, .missing = FALSE), F)
  expect_identical(vec_pall(N, F, .missing = FALSE), F)
  expect_identical(vec_pall(N, N, .missing = FALSE), F)

  expect_identical(vec_pany(T, T, .missing = NA), T)
  expect_identical(vec_pany(T, F, .missing = NA), T)
  expect_identical(vec_pany(T, N, .missing = NA), T)
  expect_identical(vec_pany(F, T, .missing = NA), T)
  expect_identical(vec_pany(F, F, .missing = NA), F)
  expect_identical(vec_pany(F, N, .missing = NA), N)
  expect_identical(vec_pany(N, T, .missing = NA), T)
  expect_identical(vec_pany(N, F, .missing = NA), N)
  expect_identical(vec_pany(N, N, .missing = NA), N)

  expect_identical(vec_pany(T, T, .missing = TRUE), T)
  expect_identical(vec_pany(T, F, .missing = TRUE), T)
  expect_identical(vec_pany(T, N, .missing = TRUE), T)
  expect_identical(vec_pany(F, T, .missing = TRUE), T)
  expect_identical(vec_pany(F, F, .missing = TRUE), F)
  expect_identical(vec_pany(F, N, .missing = TRUE), T)
  expect_identical(vec_pany(N, T, .missing = TRUE), T)
  expect_identical(vec_pany(N, F, .missing = TRUE), T)
  expect_identical(vec_pany(N, N, .missing = TRUE), T)

  expect_identical(vec_pany(T, T, .missing = FALSE), T)
  expect_identical(vec_pany(T, F, .missing = FALSE), T)
  expect_identical(vec_pany(T, N, .missing = FALSE), T)
  expect_identical(vec_pany(F, T, .missing = FALSE), T)
  expect_identical(vec_pany(F, F, .missing = FALSE), F)
  expect_identical(vec_pany(F, N, .missing = FALSE), F)
  expect_identical(vec_pany(N, T, .missing = FALSE), T)
  expect_identical(vec_pany(N, F, .missing = FALSE), F)
  expect_identical(vec_pany(N, N, .missing = FALSE), F)
})

test_that("works with empty inputs", {
  expect_identical(vec_pall(logical(), logical()), logical())
  expect_identical(vec_pany(logical(), logical()), logical())
})

test_that("works with no inputs", {
  expect_identical(vec_pall(), logical())
  expect_identical(vec_pany(), logical())
})

test_that("works with no inputs and specified `.size`", {
  expect_identical(vec_pall(.size = 3), c(TRUE, TRUE, TRUE))
  expect_identical(vec_pany(.size = 3), c(FALSE, FALSE, FALSE))
})

test_that("no casting is done", {
  expect_snapshot(error = TRUE, {
    vec_pall(1)
  })
  expect_snapshot(error = TRUE, {
    vec_pany(1)
  })

  # Arrays
  expect_snapshot(error = TRUE, {
    vec_pall(array(TRUE))
  })
  expect_snapshot(error = TRUE, {
    vec_pany(array(TRUE))
  })

  # Class
  expect_snapshot(error = TRUE, {
    vec_pall(structure(TRUE, class = "foo"))
  })
  expect_snapshot(error = TRUE, {
    vec_pany(structure(TRUE, class = "foo"))
  })
})

test_that("no recycling is done", {
  expect_snapshot(error = TRUE, {
    vec_pall(TRUE, c(TRUE, TRUE, TRUE))
  })
  expect_snapshot(error = TRUE, {
    vec_pany(TRUE, c(TRUE, TRUE, TRUE))
  })

  # With `.size`
  expect_snapshot(error = TRUE, {
    vec_pall(TRUE, c(TRUE, TRUE, TRUE), .size = 3L)
  })
  expect_snapshot(error = TRUE, {
    vec_pany(TRUE, c(TRUE, TRUE, TRUE), .size = 3L)
  })
})

test_that("validates `.missing`", {
  expect_snapshot(error = TRUE, vec_pall(.missing = c(TRUE, FALSE)))
  expect_snapshot(error = TRUE, vec_pany(.missing = c(TRUE, FALSE)))

  expect_snapshot(error = TRUE, vec_pall(.missing = 1))
  expect_snapshot(error = TRUE, vec_pany(.missing = 1))

  expect_snapshot(error = TRUE, vec_pall(.missing = NULL))
  expect_snapshot(error = TRUE, vec_pany(.missing = NULL))
})

test_that("validates `.size`", {
  expect_snapshot(error = TRUE, vec_pall(.size = c(1, 2)))
  expect_snapshot(error = TRUE, vec_pany(.size = c(1, 2)))

  expect_snapshot(error = TRUE, vec_pall(.size = 1.5))
  expect_snapshot(error = TRUE, vec_pany(.size = 1.5))

  expect_snapshot(error = TRUE, vec_pall(.size = NA_integer_))
  expect_snapshot(error = TRUE, vec_pany(.size = NA_integer_))
})

test_that("names are used in errors", {
  expect_snapshot(error = TRUE, {
    vec_pall(1.5, .arg = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_pall(a = 1.5, .arg = "x")
  })
  expect_snapshot(error = TRUE, {
    x <- c(TRUE, FALSE)
    y <- logical()
    vec_pany(x, y)
  })
  expect_snapshot(error = TRUE, {
    x <- c(TRUE, FALSE)
    y <- logical()
    vec_pany(a = x, b = y, .arg = "x", .error_call = quote(foo()))
  })
})

#' Fill in missing values with the previous or following value
#'
#' @description
#' `vec_fill_missing()` fills gaps of missing values with the previous or
#' following non-missing value.
#'
#' @param x A vector
#' @param direction Direction in which to fill missing values. Must be either
#'   `"down"`, `"up"`, `"downup"`, or `"updown"`.
#' @param max_fill A single positive integer specifying the maximum number of
#'   sequential missing values that will be filled. If `NULL`, there is
#'   no limit.
#'
#' @export
#' @examples
#' x <- c(NA, NA, 1, NA, NA, NA, 3, NA, NA)
#'
#' # Filling down replaces missing values with the previous non-missing value
#' vec_fill_missing(x, direction = "down")
#'
#' # To also fill leading missing values, use `"downup"`
#' vec_fill_missing(x, direction = "downup")
#'
#' # Limit the number of sequential missing values to fill with `max_fill`
#' vec_fill_missing(x, max_fill = 1)
#'
#' # Data frames are filled rowwise. Rows are only considered missing
#' # if all elements of that row are missing.
#' y <- c(1, NA, 2, NA, NA, 3, 4, NA, 5)
#' df <- data_frame(x = x, y = y)
#' df
#'
#' vec_fill_missing(df)
vec_fill_missing <- function(
  x,
  direction = c("down", "up", "downup", "updown"),
  max_fill = NULL
) {
  .Call(vctrs_fill_missing, x, direction, max_fill)
}

#' Convert to a base subscript type
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' Convert `i` to the base type expected by [vec_as_location()] or
#' [vec_as_location2()]. The values of the subscript type are
#' not checked in any way (length, missingness, negative elements).
#'
#' @inheritParams vec_as_location
#'
#' @param logical,numeric,character How to handle logical, numeric,
#'   and character subscripts.
#'
#'   If `"cast"` and the subscript is not one of the three base types
#'   (logical, integer or character), the subscript is
#'   [cast][vec_cast] to the relevant base type, e.g. factors are
#'   coerced to character. `NULL` is treated as an empty integer
#'   vector, and is thus coercible depending on the setting of
#'   `numeric`. Symbols are treated as character vectors and thus
#'   coercible depending on the setting of `character`.
#'
#'   If `"error"`, the subscript type is disallowed and triggers an
#'   informative error.
#' @keywords internal
#' @export
vec_as_subscript <- function(
  i,
  ...,
  logical = c("cast", "error"),
  numeric = c("cast", "error"),
  character = c("cast", "error"),
  arg = NULL,
  call = caller_env()
) {
  check_dots_empty0(...)

  .Call(
    ffi_as_subscript,
    i = i,
    logical = logical,
    numeric = numeric,
    character = character,
    frame = environment()
  )
}
vec_as_subscript_result <- function(i, arg, call, logical, numeric, character) {
  .Call(
    ffi_as_subscript_result,
    i = i,
    logical = logical,
    numeric = numeric,
    character = character,
    frame = environment()
  )
}


#' @rdname vec_as_subscript
#' @export
vec_as_subscript2 <- function(
  i,
  ...,
  numeric = c("cast", "error"),
  character = c("cast", "error"),
  arg = NULL,
  call = caller_env()
) {
  check_dots <- function(..., logical = "error", call = caller_env()) {
    if (!is_string(logical, "error")) {
      abort(
        "`vctrs::vec_as_subscript2(logical = 'cast')` is deprecated.",
        call = caller_env()
      )
    }
    check_dots_empty0(..., call = call)
  }
  check_dots(...)

  result_get(vec_as_subscript2_result(
    i,
    arg,
    call,
    numeric = numeric,
    character = character
  ))
}
vec_as_subscript2_result <- function(
  i,
  arg,
  call,
  numeric = "cast",
  character = "cast"
) {
  numeric <- arg_match0(numeric, c("cast", "error"))
  character <- arg_match0(character, c("cast", "error"))

  result <- vec_as_subscript_result(
    i,
    arg = arg,
    call = call,
    logical = "error",
    numeric = numeric,
    character = character
  )

  # This should normally be a `vctrs_error_subscript`. Indicate to
  # message methods that this error refers to a `[[` subscript.
  if (!is_null(result$err)) {
    result$err$subscript_scalar <- TRUE
  }

  result
}


subscript_type_opts <- c("logical", "numeric", "character")
subscript_type_opts_indefinite_singular <- c(
  "a logical flag",
  "a location",
  "a name"
)
subscript_type_opts_indefinite_plural <- c(
  "logical flags",
  "locations",
  "names"
)

as_opts_subscript_type <- function(x, arg = NULL) {
  if (inherits(x, "vctrs_opts_subscript_type")) {
    return(x)
  }
  new_opts(
    x,
    subscript_type_opts,
    subclass = "vctrs_opts_subscript_type",
    arg = arg
  )
}
as_opts_subscript2_type <- function(x, arg = NULL) {
  if ("logical" %in% x) {
    abort("Logical subscripts can't be converted to a single location.")
  }
  as_opts_subscript_type(x, arg = arg)
}


stop_subscript <- function(i, ..., class = NULL, call = caller_env()) {
  abort(
    class = c(class, "vctrs_error_subscript"),
    i = i,
    ...,
    call = call
  )
}
new_error_subscript <- function(class = NULL, i, ...) {
  error_cnd(
    c(class, "vctrs_error_subscript"),
    i = i,
    ...
  )
}
new_error_subscript_type <- function(
  i,
  logical = "cast",
  numeric = "cast",
  character = "cast",
  ...,
  call = NULL,
  class = NULL
) {
  new_error_subscript(
    class = c(class, "vctrs_error_subscript_type"),
    i = i,
    logical = logical,
    numeric = numeric,
    character = character,
    ...,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_subscript_type <- function(cnd, ...) {
  arg <- cnd[["subscript_arg"]]
  if (is_subscript_arg(arg)) {
    with <- glue::glue(" with {format_subscript_arg(arg)}")
  } else {
    with <- ""
  }

  action <- cnd_subscript_action(cnd, assign_to = FALSE)
  elt <- cnd_subscript_element(cnd)

  if (cnd_subscript_scalar(cnd)) {
    glue::glue("Can't {action} {elt[[1]]}{with}.")
  } else {
    glue::glue("Can't {action} {elt[[2]]}{with}.")
  }
}
#' @export
cnd_body.vctrs_error_subscript_type <- function(cnd, ...) {
  arg <- cnd_subscript_arg(cnd)
  type <- obj_type_friendly(cnd$i)
  expected_types <- cnd_subscript_expected_types(cnd)

  format_error_bullets(c(
    x = cli::format_inline("{arg} must be {.or {expected_types}}, not {type}.")
  ))
}
new_cnd_bullets_subscript_lossy_cast <- function(lossy_err) {
  function(cnd, ...) {
    format_error_bullets(c(x = cnd_header(lossy_err)))
  }
}

collapse_subscript_type <- function(cnd) {
  types <- cnd_subscript_expected_types(cnd)

  if (length(types) == 2) {
    last <- " or "
  } else {
    last <- ", or "
  }

  glue::glue_collapse(types, sep = ", ", last = last)
}
cnd_subscript_expected_types <- function(cnd) {
  types <- c("logical", "numeric", "character")
  allowed <- cnd[types] != "error"
  types[allowed]
}

new_error_subscript_size <- function(i, ..., class = NULL) {
  new_error_subscript(
    class = c(class, "vctrs_error_subscript_size"),
    i = i,
    ...
  )
}
#' @export
cnd_header.vctrs_error_subscript_size <- function(cnd, ...) {
  cnd_header.vctrs_error_subscript_type(cnd, ...)
}

new_error_subscript2_type <- function(i, numeric, character, ...) {
  new_error_subscript_type(
    i = i,
    logical = "error",
    numeric = numeric,
    character = character,
    subscript_scalar = TRUE,
    ...
  )
}

cnd_body_subscript_dim <- function(cnd, ...) {
  arg <- append_arg("Subscript", cnd$subscript_arg)

  dim <- length(dim(cnd$i))
  if (dim < 2) {
    abort(
      "Internal error: Unexpected dimensionality in `cnd_body_subcript_dim()`."
    )
  }
  if (dim == 2) {
    shape <- "a matrix"
  } else {
    shape <- "an array"
  }

  format_error_bullets(c(
    x = glue::glue("{arg} must be a simple vector, not {shape}.")
  ))
}

cnd_subscript_element <- function(cnd, capital = FALSE) {
  elt <- cnd$subscript_elt %||% "element"

  if (!is_string(elt, c("element", "row", "column", "table"))) {
    abort(paste0(
      "Internal error: `cnd$subscript_elt` must be one of ",
      "`element`, `row`, `column` or `table`."
    ))
  }

  if (capital) {
    switch(
      elt,
      element = c("Element", "Elements"),
      row = c("Row", "Rows"),
      column = c("Column", "Columns"),
      table = c("Table", "Tables")
    )
  } else {
    switch(
      elt,
      element = c("element", "elements"),
      row = c("row", "rows"),
      column = c("column", "columns"),
      table = c("table", "tables")
    )
  }
}

cnd_subscript_element_cli <- function(n, cnd, capital = FALSE) {
  elt <- cnd$subscript_elt %||% "element"

  if (!is_string(elt, c("element", "row", "column", "table"))) {
    abort(paste0(
      "Internal error: `cnd$subscript_elt` must be one of ",
      "`element`, `row`, `column` or `table`."
    ))
  }

  if (capital) {
    elt <- switch(
      elt,
      element = "Element{?s}",
      row = "Row{?s}",
      column = "Column{?s}",
      table = "Table{?s}"
    )
  } else {
    elt <- switch(
      elt,
      element = "element{?s}",
      row = "row{?s}",
      column = "column{?s}",
      table = "table{?s}"
    )
  }

  cli::pluralize("{n} ", elt)
}

subscript_actions <- c(
  "select",
  "subset",
  "extract",
  "assign",
  "rename",
  "relocate",
  "remove",
  "negate"
)
cnd_subscript_action <- function(cnd, assign_to = TRUE) {
  action <- cnd$subscript_action

  if (is_null(action)) {
    if (cnd_subscript_scalar(cnd)) {
      action <- "extract"
    } else {
      action <- "subset"
    }
  }

  if (!is_string(action, subscript_actions)) {
    cli::cli_abort(
      "`cnd$subscript_action` must be one of {.or {.arg {subscript_actions}}}.",
      .internal = TRUE
    )
  }

  if (assign_to && action == "assign") {
    "assign to"
  } else {
    action
  }
}

cnd_subscript_arg <- function(cnd, ...) {
  format_subscript_arg(cnd[["subscript_arg"]], ...)
}
format_subscript_arg <- function(arg, capitalise = TRUE) {
  if (is_subscript_arg(arg)) {
    if (!is_string(arg)) {
      arg <- as_label(arg)
    }
    cli::format_inline("{.arg {arg}}")
  } else {
    if (capitalise) {
      "Subscript"
    } else {
      "subscript"
    }
  }
}
is_subscript_arg <- function(x) {
  !is_null(x) && !is_string(x, "")
}

cnd_subscript_type <- function(cnd) {
  type <- cnd$subscript_type

  if (!is_string(type, c("logical", "numeric", "character"))) {
    abort(
      "Internal error: `cnd$subscript_type` must be `logical`, `numeric`, or `character`."
    )
  }

  type
}

cnd_subscript_scalar <- function(cnd) {
  out <- cnd$subscript_scalar %||% FALSE

  if (!is_bool(out)) {
    abort("Internal error: `cnd$subscript_scalar` must be a boolean.")
  }

  out
}

# TODO: Use this NEWS bullet when we move to the new `vec_order()` algorithm
#
# * `vec_order()` and `vec_sort()` now use a custom radix sort algorithm, rather
#    than relying on `order()`. The implementation is based on data.table’s
#    `forder()` and their earlier contribution to R’s `order()`. There are four
#    major changes, outlined below, the first two of which are breaking changes.
#    If you need to retain the old ordering behavior, use `vec_order_base()`.
#
#    * Character vectors now order in the C locale by default, which is _much_
#      faster than ordering in the system's locale. To order in a specific locale,
#      you can provide a character proxy function through `chr_proxy_collate`,
#      such as `stringi::stri_sort_key()`.
#
#    * Optional arguments, such as `direction` and `na_value`, must now be
#      specified by name. Specifying by position will result in an error.
#
#    * When ordering data frames, you can now control the behavior of `direction`
#      and `na_value` on a per column basis.
#
#    * There is a new `nan_distinct` argument for differentiating between `NaN`
#      and `NA` in double and complex vectors.

#' Order and sort vectors
#'
#' @description
#' `vec_order_radix()` computes the order of `x`. For data frames, the order is
#' computed along the rows by computing the order of the first column and
#' using subsequent columns to break ties.
#'
#' `vec_sort_radix()` sorts `x`. It is equivalent to `vec_slice(x, vec_order_radix(x))`.
#'
#' @inheritParams rlang::args_dots_empty
#'
#' @param x A vector
#' @param direction Direction to sort in.
#'   - A single `"asc"` or `"desc"` for ascending or descending order
#'     respectively.
#'   - For data frames, a length `1` or `ncol(x)` character vector containing
#'     only `"asc"` or `"desc"`, specifying the direction for each column.
#' @param na_value Ordering of missing values.
#'   - A single `"largest"` or `"smallest"` for ordering missing values as the
#'     largest or smallest values respectively.
#'   - For data frames, a length `1` or `ncol(x)` character vector containing
#'     only `"largest"` or `"smallest"`, specifying how missing values should
#'     be ordered within each column.
#' @param nan_distinct A single logical specifying whether or not `NaN` should
#'   be considered distinct from `NA` for double and complex vectors. If `TRUE`,
#'   `NaN` will always be ordered between `NA` and non-missing numbers.
#' @param chr_proxy_collate A function generating an alternate representation
#'   of character vectors to use for collation, often used for locale-aware
#'   ordering.
#'   - If `NULL`, no transformation is done.
#'   - Otherwise, this must be a function of one argument. If the input contains
#'     a character vector, it will be passed to this function after it has been
#'     translated to UTF-8. This function should return a character vector with
#'     the same length as the input. The result should sort as expected in the
#'     C-locale, regardless of encoding.
#'
#'   For data frames, `chr_proxy_collate` will be applied to all character
#'   columns.
#'
#'   Common transformation functions include: `tolower()` for case-insensitive
#'   ordering and `stringi::stri_sort_key()` for locale-aware ordering.
#'
#' @return
#' * `vec_order_radix()` an integer vector the same size as `x`.
#' * `vec_sort_radix()` a vector with the same size and type as `x`.
#'
#' @section Differences with `order()`:
#'
#' Unlike the `na.last` argument of `order()` which decides the positions of
#' missing values irrespective of the `decreasing` argument, the `na_value`
#' argument of `vec_order_radix()` interacts with `direction`. If missing values
#' are considered the largest value, they will appear last in ascending order,
#' and first in descending order.
#'
#' Character vectors are ordered in the C-locale. This is different from
#' `base::order()`, which respects `base::Sys.setlocale()`. Sorting in a
#' consistent locale can produce more reproducible results between different
#' sessions and platforms, however, the results of sorting in the C-locale
#' can be surprising. For example, capital letters sort before lower case
#' letters. Sorting `c("b", "C", "a")` with `vec_sort_radix()` will return
#' `c("C", "a", "b")`, but with `base::order()` will return `c("a", "b", "C")`
#' unless `base::order(method = "radix")` is explicitly set, which also uses
#' the C-locale. While sorting with the C-locale can be useful for
#' algorithmic efficiency, in many real world uses it can be the cause of
#' data analysis mistakes. To balance these trade-offs, you can supply a
#' `chr_proxy_collate` function to transform character vectors into an
#' alternative representation that orders in the C-locale in a less surprising
#' way. For example, providing [base::tolower()] as a transform will order the
#' original vector in a case-insensitive manner. Locale-aware ordering can be
#' achieved by providing `stringi::stri_sort_key()` as a transform, setting the
#' collation options as appropriate for your locale.
#'
#' Character vectors are always translated to UTF-8 before ordering, and before
#' any transform is applied by `chr_proxy_collate`.
#'
#' For complex vectors, if either the real or imaginary component is `NA` or
#' `NaN`, then the entire observation is considered missing.
#'
#' @section Dependencies of `vec_order_radix()`:
#' * [vec_proxy_order()]
#'
#' @section Dependencies of `vec_sort_radix()`:
#' * [vec_order_radix()]
#' * [vec_slice()]
#'
#' @name order-radix
#' @keywords internal
#'
#' @examples
#' if (FALSE) {
#'
#' x <- round(sample(runif(5), 9, replace = TRUE), 3)
#' x <- c(x, NA)
#'
#' vec_order_radix(x)
#' vec_sort_radix(x)
#' vec_sort_radix(x, direction = "desc")
#'
#' # Can also handle data frames
#' df <- data.frame(g = sample(2, 10, replace = TRUE), x = x)
#' vec_order_radix(df)
#' vec_sort_radix(df)
#' vec_sort_radix(df, direction = "desc")
#'
#' # For data frames, `direction` and `na_value` are allowed to be vectors
#' # with length equal to the number of columns in the data frame
#' vec_sort_radix(
#'   df,
#'   direction = c("desc", "asc"),
#'   na_value = c("largest", "smallest")
#' )
#'
#' # Character vectors are ordered in the C locale, which orders capital letters
#' # below lowercase ones
#' y <- c("B", "A", "a")
#' vec_sort_radix(y)
#'
#' # To order in a case-insensitive manner, provide a `chr_proxy_collate`
#' # function that transforms the strings to all lowercase
#' vec_sort_radix(y, chr_proxy_collate = tolower)
#'
#' }
NULL

#' @rdname order-radix
vec_order_radix <- function(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
) {
  check_dots_empty0(...)
  .Call(vctrs_order, x, direction, na_value, nan_distinct, chr_proxy_collate)
}

#' @rdname order-radix
vec_sort_radix <- function(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
) {
  check_dots_empty0(...)

  idx <- vec_order_radix(
    x = x,
    direction = direction,
    na_value = na_value,
    nan_distinct = nan_distinct,
    chr_proxy_collate = chr_proxy_collate
  )

  vec_slice(x, idx)
}

# ------------------------------------------------------------------------------

#' Locate sorted groups
#'
#' @description
#' `r lifecycle::badge("experimental")`
#'
#' `vec_locate_sorted_groups()` returns a data frame containing a `key` column
#' with sorted unique groups, and a `loc` column with the locations of each
#' group in `x`. It is similar to [vec_group_loc()], except the groups are
#' returned sorted rather than by first appearance.
#'
#' @details
#' `vec_locate_sorted_groups(x)` is equivalent to, but faster than:
#'
#' ```
#' info <- vec_group_loc(x)
#' vec_slice(info, vec_order(info$key))
#' ```
#'
#' @inheritParams order-radix
#'
#' @return
#' A two column data frame with size equal to `vec_size(vec_unique(x))`.
#' * A `key` column of type `vec_ptype(x)`.
#' * A `loc` column of type list, with elements of type integer.
#'
#' @section Dependencies of `vec_locate_sorted_groups()`:
#' * [vec_proxy_order()]
#'
#' @export
#' @keywords internal
#' @examples
#' df <- data.frame(
#'   g = sample(2, 10, replace = TRUE),
#'   x = c(NA, sample(5, 9, replace = TRUE))
#' )
#'
#' # `vec_locate_sorted_groups()` is similar to `vec_group_loc()`, except keys
#' # are returned ordered rather than by first appearance.
#' vec_locate_sorted_groups(df)
#'
#' vec_group_loc(df)
vec_locate_sorted_groups <- function(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
) {
  check_dots_empty0(...)

  .Call(
    vctrs_locate_sorted_groups,
    x,
    direction,
    na_value,
    nan_distinct,
    chr_proxy_collate
  )
}

# ------------------------------------------------------------------------------

vec_order_info <- function(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
) {
  check_dots_empty0(...)
  .Call(
    vctrs_order_info,
    x,
    direction,
    na_value,
    nan_distinct,
    chr_proxy_collate
  )
}

# ------------------------------------------------------------------------------

#' Order and sort vectors
#'
#' @inheritParams rlang::args_dots_empty
#'
#' @param x A vector
#' @param direction Direction to sort in. Defaults to `asc`ending.
#' @param na_value Should `NA`s be treated as the largest or smallest values?
#' @return
#' * `vec_order()` an integer vector the same size as `x`.
#' * `vec_sort()` a vector with the same size and type as `x`.
#'
#' @section Differences with `order()`:
#' Unlike the `na.last` argument of `order()` which decides the
#' positions of missing values irrespective of the `decreasing`
#' argument, the `na_value` argument of `vec_order()` interacts with
#' `direction`. If missing values are considered the largest value,
#' they will appear last in ascending order, and first in descending
#' order.
#'
#' @section Dependencies of `vec_order()`:
#' * [vec_proxy_order()]
#'
#' @section Dependencies of `vec_sort()`:
#' * [vec_proxy_order()]
#' * [vec_order()]
#' * [vec_slice()]
#' @export
#' @examples
#' x <- round(c(runif(9), NA), 3)
#' vec_order(x)
#' vec_sort(x)
#' vec_sort(x, direction = "desc")
#'
#' # Can also handle data frames
#' df <- data.frame(g = sample(2, 10, replace = TRUE), x = x)
#' vec_order(df)
#' vec_sort(df)
#' vec_sort(df, direction = "desc")
#'
#' # Missing values interpreted as largest values are last when
#' # in increasing order:
#' vec_order(c(1, NA), na_value = "largest", direction = "asc")
#' vec_order(c(1, NA), na_value = "largest", direction = "desc")
vec_order <- function(
  x,
  ...,
  direction = c("asc", "desc"),
  na_value = c("largest", "smallest")
) {
  check_dots_empty0(...)

  direction <- arg_match0(direction, c("asc", "desc"))
  na_value <- arg_match0(na_value, c("largest", "smallest"))

  decreasing <- !identical(direction, "asc")
  na.last <- identical(na_value, "largest")
  if (decreasing) {
    na.last <- !na.last
  }

  proxy <- vec_proxy_order(x)

  if (is.data.frame(proxy)) {
    if (length(proxy) == 0L) {
      # Work around type-instability in `base::order()`
      return(vec_seq_along(proxy))
    }
    args <- map(unstructure(proxy), function(.x) {
      if (is.data.frame(.x)) {
        .x <- order(vec_order(.x, direction = direction, na_value = na_value))
      }
      .x
    })
    exec("order", !!!args, decreasing = decreasing, na.last = na.last)
  } else if (
    is_character(proxy) ||
      is_logical(proxy) ||
      is_integer(proxy) ||
      is_double(proxy) ||
      is.complex(proxy)
  ) {
    if (is.object(proxy)) {
      proxy <- unstructure(proxy)
    }
    order(proxy, decreasing = decreasing, na.last = na.last)
  } else {
    abort("Invalid type returned by `vec_proxy_order()`.")
  }
}

#' @export
#' @rdname vec_order
vec_sort <- function(
  x,
  ...,
  direction = c("asc", "desc"),
  na_value = c("largest", "smallest")
) {
  check_dots_empty0(...)

  direction <- arg_match0(direction, c("asc", "desc"))
  na_value <- arg_match0(na_value, c("largest", "smallest"))

  idx <- vec_order(x, direction = direction, na_value = na_value)
  vec_slice(x, idx)
}

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(integer()), integer())
})

test_that("can order integers", {
  x <- c(2L, 3L, 1L, 5L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:5
  expect_identical(vec_order_radix(x), order(x))
})

test_that("orders correctly around the UINT8_MAX boundary", {
  x <- 251:255
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1L, 3L, 1L, 3L)
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c(1L, NA_integer_, 3L)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("integer, small: `NA` order can be first", {
  x <- c(1L, NA_integer_, 3L)
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("double: `direction` can be set to `desc`", {
  x <- c(1, 5, 3)
  expect_identical(vec_order_radix(x, direction = "desc"), c(2L, 3L, 1L))
  x <- c(1L, .Machine$integer.max, 3L)
  expect_identical(vec_order_radix(x, direction = "desc"), c(2L, 3L, 1L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3L, NA_integer_, 1L, 2L)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_integer_, NA_integer_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order when in expected order", {
  x <- c(1L, 1L, 2L, NA, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 1:5)

  x <- c(NA, NA, 3L, 3L, 2L)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 1:5)

  x <- c(NA, NA, 1L, 1L, 2L)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 1:5)

  x <- c(3L, 3L, 2L, NA, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 1:5)
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, 2L, 1L)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 3:1)

  x <- c(1L, 2L, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 3:1)

  x <- c(2L, 1L, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 3:1)

  x <- c(NA, 1L, 2L)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 3:1)
})

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - counting

# To trigger counting ordering, get above the insertion order boundary and then
# have a range less than the counting order range boundary.

test_that("can order integers with counting order", {
  x <- (ORDER_INSERTION_BOUNDARY + 1L):1L
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:(ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L)
  expect_identical(vec_order_radix(x)[1:2], c(1L, ORDER_INSERTION_BOUNDARY + 1L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3L, NA_integer_, 1L, 2L, 1:ORDER_INSERTION_BOUNDARY)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - radix

# To trigger radix ordering, get above the insertion order boundary and then
# have a range greater than the counting order range boundary.

test_that("can order integers with radix order", {
  x <- c(INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L, 1:ORDER_INSERTION_BOUNDARY)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L, INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x)[1:2], c(1L, ORDER_INSERTION_BOUNDARY + 1L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3L, NA_integer_, 1L, 2L, 1:ORDER_INSERTION_BOUNDARY, INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("can order all 1 value", {
  x <- rep(1L, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(vec_order_radix(x, direction = "desc"), base_order(x, decreasing = TRUE))
})

test_that("all `NA` values works - ensures that we can compute the 'range' of all NAs", {
  x <- rep(NA_integer_, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(vec_order_radix(x, direction = "desc"), base_order(x, decreasing = TRUE))
})

test_that("can order with many NAs first", {
  x <- c(rep(NA_integer_, ORDER_INSERTION_BOUNDARY + 1L), 2L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(vec_order_radix(x, na_value = "smallest"), base_order(x, na.last = FALSE))
})

test_that("subtraction in counting order range computation works correctly (#1399)", {
  x <- c(rep(1L, ORDER_INSERTION_BOUNDARY), -2147483647L)
  expect_identical(vec_order_radix(x), base_order(x))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<logical>)

# Really this just goes through the integer infrastructure. Just checking that
# it is working.

test_that("can order size zero input", {
  expect_identical(vec_order_radix(logical()), integer())
})

test_that("can order logicals", {
  x <- c(FALSE, TRUE, FALSE)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(TRUE, NA, FALSE)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA, NA)
  expect_identical(vec_order_radix(x), order(x))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<double>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(double()), integer())
})

test_that("can order doubles", {
  x <- c(2, 3, 1, 5)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:5 + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1, 3, 1, 3)
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c(1, NA_real_, 3)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("double: `NA` order can be first", {
  x <- c(1, NA_real_, 3)
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3, NA_real_, 1, 2)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_real_, NA_real_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("NA_real_ and NaN look identical for ordering", {
  x <- c(NA_real_, NaN)
  expect_identical(vec_order_radix(x, na_value = "largest"), c(1L, 2L))
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(1L, 2L))
})

test_that("double: -Inf / Inf order correctly", {
  x <- c(0, -Inf, Inf)
  expect_identical(vec_order_radix(x, direction = "asc"), c(2L, 1L, 3L))
  expect_identical(vec_order_radix(x, direction = "desc"), c(3L, 1L, 2L))
})

test_that("double: -0 and 0 order identically / stably", {
  x <- c(0, -0)
  expect_identical(vec_order_radix(x, direction = "desc"), c(1L, 2L))
  expect_identical(vec_order_radix(x, direction = "asc"), c(1L, 2L))
})

test_that("can order when in expected order", {
  x <- c(1, 1, 2, NA, NaN)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 1:5)

  x <- c(NA, NaN, 3, 3, 2)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 1:5)

  x <- c(NA, NaN, 1, 1, 2)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 1:5)

  x <- c(3, 3, 2, NA, NaN)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 1:5)
})

test_that("can order when in expected order - using distinct NaN values", {
  x <- c(1, 1, 2, NaN, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest", nan_distinct = TRUE), 1:5)

  x <- c(NA, NaN, 3, 3, 2)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest", nan_distinct = TRUE), 1:5)

  x <- c(NA, NaN, 1, 1, 2)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest", nan_distinct = TRUE), 1:5)

  x <- c(3, 3, 2, NaN, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest", nan_distinct = TRUE), 1:5)
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, 2, 1)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 3:1)

  x <- c(1, 2, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 3:1)

  x <- c(2, 1, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 3:1)

  x <- c(NA, 1, 2)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 3:1)
})

test_that("can order when in strictly opposite of expected order (no ties) - using distinct NaN values", {
  x <- c(NA, NaN, 2, 1)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest", nan_distinct = TRUE), 4:1)

  x <- c(1, 2, NaN, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest", nan_distinct = TRUE), 4:1)

  x <- c(2, 1, NaN, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest", nan_distinct = TRUE), 4:1)

  x <- c(NA, NaN, 1, 2)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest", nan_distinct = TRUE), 4:1)
})

test_that("NaN is always placed next to numbers when treated as distinct", {
  x <- c(1, 2, NA, NaN)

  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest", nan_distinct = TRUE), c(1L, 2L, 4L, 3L))
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest", nan_distinct = TRUE), c(3L, 4L, 1L, 2L))
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest", nan_distinct = TRUE), c(3L, 4L, 2L, 1L))
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest", nan_distinct = TRUE), c(2L, 1L, 4L, 3L))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<double>) - radix

# To trigger radix ordering, get above the insertion order boundary. There is
# no intermediate counting sort for doubles.

test_that("can order doubles with radix order", {
  x <- (ORDER_INSERTION_BOUNDARY + 1L):1L + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:(ORDER_INSERTION_BOUNDARY + 1L) + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L) + 0
  expect_identical(vec_order_radix(x)[1:2], c(1L, ORDER_INSERTION_BOUNDARY + 1L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3, NA_real_, 1, 2, 1:ORDER_INSERTION_BOUNDARY)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- rep(NA_real_, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("NA_real_ and NaN generally look identical for ordering", {
  x <- rep(c(NA_real_, NaN), ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x, na_value = "largest"), seq_along(x))
  expect_identical(vec_order_radix(x, na_value = "smallest"), seq_along(x))
})

test_that("NA_real_ and NaN can be considered distinct with `nan_distinct`", {
  x <- rep(c(NA_real_, NaN), ORDER_INSERTION_BOUNDARY + 1L)

  loc_nan <- seq(2L, length(x), by = 2L)
  loc_na <- seq(1L, length(x), by = 2L)

  expect_identical(vec_order_radix(x, na_value = "largest", nan_distinct = TRUE), c(loc_nan, loc_na))
  expect_identical(vec_order_radix(x, na_value = "smallest", nan_distinct = TRUE), c(loc_na, loc_nan))
})

test_that("-Inf / Inf order correctly", {
  x <- c(rep(0, ORDER_INSERTION_BOUNDARY), -Inf, Inf)
  expect_identical(vec_order_radix(x, direction = "asc"), order(x, decreasing = FALSE))
  expect_identical(vec_order_radix(x, direction = "desc"), order(x, decreasing = TRUE))
})

test_that("double, large: -0 and 0 order identically / stably", {
  x <- c(rep(0, ORDER_INSERTION_BOUNDARY), -0)
  expect_identical(vec_order_radix(x, direction = "desc"), order(x, decreasing = TRUE))
  expect_identical(vec_order_radix(x, direction = "asc"), order(x, decreasing = FALSE))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<complex>)

test_that("can order size zero input", {
  expect_identical(vec_order_radix(complex()), integer())
})

test_that("can order complex", {
  x <- complex(real = c(3, 1, 2))
  expect_identical(vec_order_radix(x), c(2L, 3L, 1L))
})

test_that("ordering on ties is done stably", {
  x <- complex(real = c(1, 3, 1, 3))
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("imaginary section is used to break ties", {
  x <- complex(
    real = c(1L, 2L, 1L),
    imaginary = c(3L, 2L, 1L)
  )
  expect_identical(vec_order_radix(x), c(3L, 1L, 2L))
})

test_that("can be used in a data frame", {
  x <- c(1L, 1L, 1L, 2L, 1L)

  y <- complex(
    real = c(1L, 2L, 1L, 3L, 1L),
    imaginary = c(3L, 2L, 1L, 4L, 1L)
  )

  z <- c(1, 2, 5, 4, 3)

  # as second column
  df1 <- data.frame(x = x, y = y)

  # as first column
  df2 <- data.frame(y = y, x = x)

  # as second column with a third after it to break ties
  df3 <- data.frame(x = x, y = y, z = z)

  # Base R can't do radix sorting with complex
  expect_identical(vec_order_radix(df1), c(3L, 5L, 1L, 2L, 4L))
  expect_identical(vec_order_radix(df2), c(3L, 5L, 1L, 2L, 4L))
  expect_identical(vec_order_radix(df3), c(5L, 3L, 1L, 2L, 4L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- complex(real = c(3, NA, 1.5, 2, NA), imaginary = c(1, 1, 1, 1, 2))

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )

  # In fixing #1403, we now align with base R
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("full gambit of tests involving missing values are working as expected (#1403)", {
  x <- complex(
    real      = c(NaN, NA, NA,  NA, NaN, NaN, 1,  1,   1, 2),
    imaginary = c(NA,  NA, NaN, 1,  NaN, 1,   NA, NaN, 1, NA)
  )

  df <- data_frame(a = rep(1L, length(x)), x = x)

  # {number}, {NaN}, {NaN + NA}, {NA + NaN}, {NA}
  expect <- c(9L,  5L, 6L, 8L,  1L,  3L,  2L, 4L, 7L, 10L)
  expect_identical(vec_order_radix(x, direction = "asc",  na_value = "largest",  nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest", nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(df, direction = "asc",  na_value = "largest",  nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(df, direction = "desc", na_value = "smallest", nan_distinct = TRUE), expect)

  # {NA}, {NA + NaN}, {NaN + NA}, {NaN}, {number}
  expect <- c(2L, 4L, 7L, 10L,  3L,  1L,  5L, 6L, 8L,  9L)
  expect_identical(vec_order_radix(x, direction = "asc",  na_value = "smallest", nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest",  nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(df, direction = "asc",  na_value = "smallest", nan_distinct = TRUE), expect)
  expect_identical(vec_order_radix(df, direction = "desc", na_value = "largest",  nan_distinct = TRUE), expect)

  # {number}, {NA or NaN}
  expect <- c(9L, 1:7, 8L, 10L)
  expect_identical(vec_order_radix(x, direction = "asc",  na_value = "largest",  nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest", nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(df, direction = "asc",  na_value = "largest",  nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(df, direction = "desc", na_value = "smallest", nan_distinct = FALSE), expect)

  # {NA or NaN}, {number}
  expect <- c(1:8, 10L, 9L)
  expect_identical(vec_order_radix(x, direction = "asc",  na_value = "smallest", nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest",  nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(df, direction = "asc",  na_value = "smallest", nan_distinct = FALSE), expect)
  expect_identical(vec_order_radix(df, direction = "desc", na_value = "largest",  nan_distinct = FALSE), expect)
})

# ------------------------------------------------------------------------------
# vec_order_radix(<character>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(character()), integer())
})

test_that("can order characters", {
  x <- c("xy", "x", "a", "bc")
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- c("a", "b", "c")
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c("ab", "ba", "ab", "ba")
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c("x", NA_character_, "y")
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("character, small: `NA` order can be first", {
  x <- c("x", NA_character_, "y")
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("character, small: `direction` can be set to `desc`", {
  x <- c("x", "abcde", "yz")
  expect_identical(vec_order_radix(x, direction = "desc"), c(3L, 1L, 2L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c("aaa", NA_character_, "a", "aa")

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_character_, NA_character_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order empty string vs ASCII value 1 'Start of Header'", {
  x <- c("\001", "")
  expect_identical(vec_order_radix(x), c(2L, 1L))
})

test_that("can be used in a data frame", {
  x <- c(1L, 4L, 1L, 3L, 1L)

  y <- c("zy", "zz", "abcd", "gfa", "zy")

  z <- c(1, 2, 5, 4, 3)

  # as second column
  df1 <- data.frame(x = x, y = y)

  # as first column
  df2 <- data.frame(y = y, x = x)

  # as second column with a third after it to break ties
  df3 <- data.frame(x = x, y = y, z = z)

  expect_identical(vec_order_radix(df1), base_order(df1))
  expect_identical(vec_order_radix(df2), base_order(df2))
  expect_identical(vec_order_radix(df3), base_order(df3))
})

test_that("can have multiple character columns in a data frame", {
  df <- data.frame(
    x = c("def", "aba", "aba", "aba", "q"),
    y = c("zy", "zz", "zz", "gfa", "zy"),
    z = c("foo", "qux", "bar", "baz", "boo")
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

test_that("can order with varying encodings by converting to UTF-8", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1, "AC")

  expect_identical(vec_order_radix(x), c(4L, 1L, 2L, 3L))
  expect_identical(vec_order_radix(x, direction = "desc"), c(1L, 2L, 3L, 4L))
})

test_that("can order when in expected order", {
  x <- c("a", "a", "b", NA, NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 1:5)

  x <- c(NA, NA, "c", "c", "b")
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 1:5)

  x <- c(NA, NA, "a", "a", "b")
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 1:5)

  x <- c("c", "c", "b", NA, NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 1:5)
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, "b", "a")
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "largest"), 3:1)

  x <- c("a", "b", NA)
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "largest"), 3:1)

  x <- c("b", "a", NA)
  expect_identical(vec_order_radix(x, direction = "asc", na_value = "smallest"), 3:1)

  x <- c(NA, "a", "b")
  expect_identical(vec_order_radix(x, direction = "desc", na_value = "smallest"), 3:1)
})

# ------------------------------------------------------------------------------
# vec_order_radix(<character>) - radix

# Have to get the number of unique strings above the ORDER_INSERTION_BOUNDARY
# to trigger radix ordering.

test_that("can order character vectors", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  expect_identical(vec_order_radix(x), base_order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L)), "x1")
  expect_identical(vec_order_radix(x)[1:2], c(1L, length(x)))
})

test_that("`NA` order defaults to last", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "y")
  expect_identical(vec_order_radix(x)[length(x)], length(x) - 1L)
})

test_that("character, large: `NA` order can be first", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "y")
  expect_identical(vec_order_radix(x, na_value = "smallest")[[1L]], length(x) - 1L)
})

test_that("character, large: `direction` can be set to `desc`", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  expect_identical(vec_order_radix(x, direction = "desc"), base_order(x, decreasing = TRUE))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "x", "aa", "x1")

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[base_order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[base_order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[base_order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[base_order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<list>)

test_that("list elements are ordered by first appearance", {
  expect_identical(vec_order_radix(list(1:2, "a", 1:2)), c(1L, 3L, 2L))
})

test_that("missing values in lists are respected (#1401)", {
  x <- list(1, NULL, 2, NULL)
  expect_identical(vec_order_radix(x, na_value = "largest"), c(1L, 3L, 2L, 4L))
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 4L, 1L, 3L))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - insertion

test_that("data frame with no columns and no rows returns integer()", {
  x <- data.frame()
  expect_identical(vec_order_radix(x), integer())
})

test_that("data frame with no columns and some rows returns sequential rows", {
  x <- new_data_frame(n = 5L)
  expect_identical(vec_order_radix(x), 1:5)
})

test_that("can order with multiple pre-sorted keys", {
  df <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_order_radix(df), 1:2)
})

test_that("first column has ordering presedence", {
  df <- data.frame(x = c(3L, 2L, 1L), y = c(1L, 2L, 3L))
  expect_identical(vec_order_radix(df), 3:1)
})

test_that("secondary columns break ties - integer", {
  df <- data.frame(
    x = c(1L, 2L, 1L),
    y = c(3L, 2L, 1L)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("secondary columns break ties - double", {
  df <- data.frame(
    x = c(1, 2, 1),
    y = c(3, 2, 1)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("secondary columns break ties - logical", {
  df <- data.frame(
    x = c(FALSE, TRUE, FALSE),
    y = c(TRUE, TRUE, FALSE)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("orders correctly when first column is already ordered but second isn't", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L),
    y = c(3L, 2L, 4L, 1L)
  )
  expect_identical(vec_order_radix(df), c(2L, 1L, 4L, 3L))
})

test_that("orders correctly when first column is already ordered but second isn't - character", {
  df <- data.frame(
    x = c("a", "a", "b", "b"),
    y = c("c", "b", "d", "a")
  )

  expect_identical(vec_order_radix(df), c(2L, 1L, 4L, 3L))
})

test_that("`direction` is recycled", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L),
    y = c(3L, 2L, 4L, 1L)
  )
  expect_identical(vec_order_radix(df, direction = "desc"), c(3L, 4L, 1L, 2L))
})

test_that("`na_value` is recycled", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA),
    y = c(3L, 2L, 4L, 1L, NA)
  )
  expect_identical(vec_order_radix(df, na_value = "smallest"), c(5L, 2L, 1L, 4L, 3L))
})

test_that("`direction` can be a vector", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, 1L),
    y = c(3L, 2L, 4L, 1L, 3L, NA)
  )
  expect_identical(vec_order_radix(df, direction = c("desc", "asc")), c(5L, 4L, 3L, 2L, 1L, 6L))
})

test_that("`na_value` can be a vector", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, NA),
    y = c(3L, 2L, 4L, 1L, NA, 2)
  )
  expect_identical(vec_order_radix(df, na_value = c("smallest", "largest")), c(6L, 5L, 2L, 1L, 4L, 3L))
})

test_that("`na_value` and `direction` can both be vectors", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, NA),
    y = c(3L, 2L, 4L, 1L, NA, 2)
  )

  expect_identical(
    vec_order_radix(df, direction = c("desc", "asc"), na_value = c("smallest", "largest")),
    c(4:1, 6:5)
  )
})

test_that("`direction` is recycled right with array columns (#1753)", {
  df <- data_frame(
    x = matrix(c(1, 1, 1, 3, 2, 2), ncol = 2),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(2L, 3L, 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_order_radix(df, direction = c("asc", "desc", "desc"))
  })

  df <- data_frame(
    x = array(c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 3, 3), dim = c(3, 2, 2)),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(2L, 3L, 1L)
  )
})

test_that("`na_value` is recycled right with array columns (#1753)", {
  df <- data_frame(
    x = matrix(c(1, 1, 1, 3, NA, 2), ncol = 2),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, na_value = c("largest", "smallest")),
    c(3L, 1L, 2L)
  )
  expect_identical(
    vec_order_radix(df, na_value = c("smallest", "largest")),
    c(2L, 3L, 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_order_radix(df, direction = c("smallest", "largest", "largest"))
  })

  df <- data_frame(
    x = array(c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, NA, 3), dim = c(3, 2, 2)),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, na_value = c("largest", "smallest")),
    c(3L, 1L, 2L)
  )
  expect_identical(
    vec_order_radix(df, na_value = c("smallest", "largest")),
    c(2L, 3L, 1L)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - counting

test_that("can order 2+ integer column chunks with counting sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1L,
    y = c(rep(2L, quarter_low), rep(1L, quarter_high), rep(3L, half))
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - radix

test_that("can order 2+ integer column chunks with radix sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1L,
    y = c(rep(2L, quarter_low), rep(1L, quarter_high), rep(3L, half), INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

test_that("can order 2+ double column chunks with radix sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1,
    y = c(rep(2, quarter_low), rep(1, quarter_high), rep(3, half), INT_ORDER_COUNTING_RANGE_BOUNDARY + 1)
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix() - chr_proxy_collate

test_that("`chr_proxy_collate` transforms string input", {
  x <- c("b", "a", "A")
  expect_identical(vec_order_radix(x, chr_proxy_collate = tolower), c(2L, 3L, 1L))
  expect_identical(vec_order_radix(x, chr_proxy_collate = ~tolower(.x)), c(2L, 3L, 1L))
})

test_that("`chr_proxy_collate` works with data frame columns and is applied to all string columns", {
  df <- data_frame(x = c(1, 1, 1), y = c("B", "a", "a"), z = c("a", "D", "c"))
  expect_identical(vec_order_radix(df, chr_proxy_collate = tolower), c(3L, 2L, 1L))
})

test_that("`chr_proxy_collate` is validated", {
  expect_error(vec_order_radix("x", chr_proxy_collate = 1), "Can't convert `chr_proxy_collate` to a function")
  expect_error(vec_order_radix("x", chr_proxy_collate = ~c("y", "z")), "1, not 2")
  expect_error(vec_order_radix("x", chr_proxy_collate = ~1), "character vector")
  expect_error(vec_order_radix("x", chr_proxy_collate = function() {"y"}))
})

test_that("`chr_proxy_collate` can return bytes-encoded strings (like `stringi::stri_sort_key()`)", {
  x <- c("A", "a", "b", "B")

  # Mimic stringi::stri_sort_key(x, locale = "en")
  sort_key <- function(x) {
    # dput(lapply(stringi::stri_sort_key(x, locale = "en"), charToRaw))
    out <- list(
      as.raw(c(0x2a, 0x01, 0x05, 0x01, 0xdc)),
      as.raw(c(0x2a, 0x01, 0x05, 0x01, 0x05)),
      as.raw(c(0x2c, 0x01, 0x05, 0x01, 0x05)),
      as.raw(c(0x2c, 0x01, 0x05, 0x01, 0xdc))
    )

    out <- vapply(out, FUN.VALUE = character(1), function(x) {
      # Uses native encoding
      x <- rawToChar(x)
      Encoding(x) <- "bytes"
      x
    })

    out
  }

  expect_identical(
    vec_order_radix(x, chr_proxy_collate = sort_key),
    c(2L, 1L, 3L, 4L)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix() - error checking

test_that("`na_value` is checked", {
  expect_error(vec_order_radix(1L, na_value = "x"), "\"largest\" or \"smallest\"")
  expect_error(vec_order_radix(1L, na_value = c(TRUE, TRUE)), "must be a character vector")
  expect_error(vec_order_radix(1L, na_value = NA_character_), "can't be missing")
})

test_that("`direction` is checked", {
  expect_error(vec_order_radix(1L, direction = "x"), "must contain only")
  expect_error(vec_order_radix(1L, direction = c("asc", "asc")), "single value")
  expect_error(vec_order_radix(1L, direction = NA_character_), "can't be missing")
  expect_error(vec_order_radix(data.frame(x = 1), direction = c("asc", "asc")), "length 1 or")
})

test_that("`x` is checked", {
  expect_error(vec_order_radix(foobar()), class = "vctrs_error_scalar_type")
})

# ------------------------------------------------------------------------------
# vec_order_radix() - groups

test_that("groups can be reallocated if we exceed the max group data size", {
  set.seed(123)

  # The first column has all unique groups so 1 more than the default group
  # data size is needed and will be reallocated on the fly
  df <- data.frame(
    x = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE),
    y = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE),
    z = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE)
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix() - comparison proxy

test_that("ordering works with rcrd types", {
  x <- tuple(c(1, 2, 1), c(3, 2, 1))
  expect_identical(vec_order_radix(x), c(3L, 1L, 2L))
})

test_that("data frame comparison proxies don't allow vector `direction` or `na_value`", {
  x <- tuple(c(1, 2, 1), c(3, 2, 1))
  expect_error(vec_order_radix(x, direction = c("desc", "asc")), "single value")
  expect_error(vec_order_radix(x, na_value = c("largest", "smallest")), "single value")
})

test_that("ordering works with df-cols", {
  df_col <- new_data_frame(list(y = c(2, 1, 2), z = c(3, 3, 3)))
  df <- new_data_frame(list(x = c(1, 1, 1), y = df_col))

  expect_identical(vec_order_radix(df), c(2L, 1L, 3L))

  # Can only supply a max of 2 `direction` or `na_value` values which get internally
  # expanded to 3 to match the flattened df proxy
  expect_identical(vec_order_radix(df, direction = c("asc", "desc")), c(1L, 3L, 2L))

  expect_error(vec_order_radix(df, direction = c("desc", "desc", "asc")), "or length equal to")
})

test_that("ordering works with df-cols with 0 cols", {
  df_col <- new_data_frame(list(), n = 3L)
  df <- new_data_frame(list(x = c(1, 3, 1), y = df_col, z = c(2, 1, 1)))

  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))

  # Can supply 3 `direction` values even though the 0-col df-col gets dropped
  expect_identical(vec_order_radix(df, direction = c("asc", "desc", "desc")), c(1L, 3L, 2L))

  expect_error(vec_order_radix(df, direction = c("desc", "asc")), "or length equal to")
})

test_that("ordering works with rcrd cols", {
  y <- tuple(c(1, 2, 1), c(3, 2, 1))
  df <- new_data_frame(list(z = c(1, 1, 1), y = y))

  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))

  # Can only supply a max of 2 `direction` values which get internally
  # expanded to 3 to match the flattened df proxy
  expect_identical(vec_order_radix(df, direction = c("asc", "desc")), c(2L, 1L, 3L))

  expect_error(vec_order_radix(df, direction = c("desc", "desc", "asc")), "or length equal to")
})

# ------------------------------------------------------------------------------
# `vec_order_radix()` - Pre-existing tests

test_that("can request NAs sorted first", {
  expect_equal(vec_order_radix(c(1, NA), direction = "asc", na_value = "largest"), 1:2)
  expect_equal(vec_order_radix(c(1, NA), direction = "desc", na_value = "largest"), 2:1)

  expect_equal(vec_order_radix(c(1, NA), direction = "asc", na_value = "smallest"), 2:1)
  expect_equal(vec_order_radix(c(1, NA), direction = "desc", na_value = "smallest"), 1:2)
})

test_that("can sort data frames", {
  df <- data.frame(x = c(1, 2, 1), y = c(1, 2, 2))

  out1 <- vec_sort(df)
  expect_equal(out1, data.frame(x = c(1, 1, 2), y = c(1, 2, 2)))

  out2 <- vec_sort(df, direction = "desc")
  expect_equal(out2, data.frame(x = c(2, 1, 1), y = c(2, 2, 1)))
})

test_that("can sort empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_sort(df1), df1)

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_sort(df2), df2)
})

test_that("can order tibbles that contain non-comparable objects", {
  expect_equal(vec_order_radix(data_frame(x = list(10, 2, 1))), 1:3)
})

test_that("can order matrices and arrays (#306)", {
  x <- matrix(c(1, 1, 1, 1, 2, 1), ncol = 2)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))

  x <- array(1:8, c(2, 2, 2))
  x[2] <- 1
  x[3] <- 5
  expect_identical(vec_order_radix(x), 2:1)
})

test_that("can order empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_order_radix(df1), integer())

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_order_radix(df2), integer())
})

test_that("can order data frames with data frame columns (#527)", {
  expect_equal(
    vec_order_radix(iris),
    vec_order_radix(data_frame(iris = iris))
  )
})

test_that("can order data frames (and subclasses) with matrix columns", {
  df <- new_data_frame(n = 2L)

  df$x <- new_data_frame(list(y = matrix(1:2, 2)))
  expect_identical(vec_order_radix(df), 1:2)

  df$x <- tibble::tibble(y = matrix(1:2, 2))
  expect_identical(vec_order_radix(df), 1:2)
})

# ------------------------------------------------------------------------------
# vec_locate_sorted_groups()

test_that("`vec_locate_sorted_groups()` is working", {
  x <- c(1, 3, 1, 5, 2, 5, 1)

  expect <- new_data_frame(
    list(
      key = c(1, 2, 3, 5),
      loc = list(c(1L, 3L, 7L), 5L, 2L, c(4L, 6L))
    )
  )

  expect_identical(vec_locate_sorted_groups(x), expect)
})

test_that("`chr_proxy_collate` can result in keys being seen as identical", {
  x <- c("b", "A", "a")
  y <- c("b", "a", "A")

  x_expect <- data_frame(key = c("A", "b"), loc = list(c(2L, 3L), 1L))
  y_expect <- data_frame(key = c("a", "b"), loc = list(c(2L, 3L), 1L))

  expect_identical(vec_locate_sorted_groups(x, chr_proxy_collate = tolower), x_expect)
  expect_identical(vec_locate_sorted_groups(y, chr_proxy_collate = tolower), y_expect)
})

# ------------------------------------------------------------------------------
# `vec_order_info(chr_ordered = FALSE)`

test_that("can order character vectors in appearance order", {
  x <- c("b", "a", "B", "B", "a")
  info <- vec_order_info(x, chr_ordered = FALSE)

  expect_identical(info[[1]], c(1L, 2L, 5L, 3L, 4L))
  expect_identical(info[[2]], c(1L, 2L, 2L))
  expect_identical(info[[3]], 2L)
})

test_that("using appearance order means `direction` has no effect", {
  x <- c("b", "a", "B", "B", "a")

  info1 <- vec_order_info(x, direction = "asc", chr_ordered = FALSE)
  info2 <- vec_order_info(x, direction = "desc", chr_ordered = FALSE)

  expect_identical(info1[[1]], info2[[1]])
  expect_identical(info1[[2]], info2[[2]])
  expect_identical(info1[[3]], info2[[3]])
})

test_that("appearance order works with NA - `na_value` has no effect", {
  x <- c(NA, "foo", NA, "bar")
  info <- vec_order_info(x, chr_ordered = FALSE)

  expect_identical(info[[1]], c(1L, 3L, 2L, 4L))
  expect_identical(info[[2]], c(2L, 1L, 1L))
  expect_identical(info[[3]], 2L)
})

test_that("appearance order can be mixed with regular ordering", {
  x <- c("b", "a", "B", "B", "a", "a")
  y <- c(1, 2, 3, 4, 1, 2)
  df <- data_frame(x = x, y = y)

  # `y` breaks ties
  info <- vec_order_info(df, chr_ordered = FALSE)

  expect_identical(info[[1]], c(1L, 5L, 2L, 6L, 3L, 4L))
  expect_identical(info[[2]], c(1L, 1L, 2L, 1L, 1L))
  expect_identical(info[[3]], 2L)
})

# ------------------------------------------------------------------------------
# `vec_order_info(nan_distinct = FALSE)`

test_that("Indistinct NA and NaN are reported in the same group", {
  x <- c(NA, NaN)
  info <- vec_order_info(x, nan_distinct = FALSE)

  expect_identical(info[[1]], c(1L, 2L))
  expect_identical(info[[2]], 2L)
  expect_identical(info[[3]], 2L)
})

# ------------------------------------------------------------------------------
# `vec_order_info(<data.frame>)`

test_that("Zero column data frames with >0 rows work (#1863)", {
  # All rows are treated as being from the same group
  x <- data_frame(.size = 5)
  info <- vec_order_info(x)

  expect_identical(info[[1]], 1:5) # Order
  expect_identical(info[[2]], 5L)  # Group sizes
  expect_identical(info[[3]], 5L)  # Max group size
})

test_that("Zero column data frames with exactly 0 rows work (#1863)", {
  # This is a particularly special case, since we don't actually push a group size
  x <- data_frame(.size = 0L)
  info <- vec_order_info(x)

  expect_identical(info[[1]], integer())
  expect_identical(info[[2]], integer())
  expect_identical(info[[3]], 0L)
})

# ------------------------------------------------------------------------------
# vec_sort

test_that("can sort data frames", {
  df <- data.frame(x = c(1, 2, 1), y = c(1, 2, 2))

  out1 <- vec_sort(df)
  expect_equal(out1, data.frame(x = c(1, 1, 2), y = c(1, 2, 2)))

  out2 <- vec_sort(df, direction = "desc")
  expect_equal(out2, data.frame(x = c(2, 1, 1), y = c(2, 2, 1)))
})

test_that("can sort empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_sort(df1), df1)

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_sort(df2), df2)
})

# ------------------------------------------------------------------------------
# vec_order

test_that("can request NAs sorted first", {
  expect_equal(vec_order(c(1, NA), direction = "asc", na_value = "largest"), 1:2)
  expect_equal(vec_order(c(1, NA), direction = "desc", na_value = "largest"), 2:1)

  expect_equal(vec_order(c(1, NA), direction = "asc", na_value = "smallest"), 2:1)
  expect_equal(vec_order(c(1, NA), direction = "desc", na_value = "smallest"), 1:2)
})

test_that("can order complex vectors", {
  x <- complex(real = c(1, 2, 2, 3, 3), imaginary = c(5, 4, 3, 2, NA))

  expect_equal(vec_order(x, direction = "asc", na_value = "largest"), c(1, 3, 2, 4, 5))
  expect_equal(vec_order(x, direction = "desc", na_value = "largest"), rev(c(1, 3, 2, 4, 5)))
  expect_equal(vec_order(x, direction = "asc", na_value = "smallest"), c(5, 1, 3, 2, 4))
  expect_equal(vec_order(x, direction = "desc", na_value = "smallest"), rev(c(5, 1, 3, 2, 4)))
})

test_that("can order tibbles that contain non-comparable objects", {
  expect_equal(vec_order(data_frame(x = list(10, 2, 1))), 1:3)
})

test_that("can order matrices and arrays (#306)", {
  x <- matrix(c(1, 1, 1, 1, 2, 1), ncol = 2)
  expect_identical(vec_order(x), c(1L, 3L, 2L))

  x <- array(1:8, c(2, 2, 2))
  x[2] <- 1
  x[3] <- 5
  expect_identical(vec_order(x), 2:1)
})

test_that("can order zero column data frames (#356, #1499)", {
  df <- data_frame()
  expect_identical(vec_order(df), integer())

  df <- data_frame(.size = 5L)
  expect_identical(vec_order(df), 1:5)

  df <- data_frame(.size = 5L)
  expect_identical(vec_order(df, direction = "desc"), 1:5)
})

test_that("can order zero row data frames (#356, #1499)", {
  df <- data.frame(x = numeric())
  expect_identical(vec_order(df), integer())

  df <- data.frame(x = numeric(), y = integer())
  expect_identical(vec_order(df), integer())

  df <- data.frame(x = numeric(), y = integer())
  expect_identical(vec_order(df, direction = "desc"), integer())
})

test_that("can order data frames with data frame columns (#527)", {
  expect_equal(
    vec_order(iris),
    vec_order(data_frame(iris = iris))
  )
})

test_that("can order data frames (and subclasses) with matrix columns", {
  df <- new_data_frame(n = 2L)

  df$x <- new_data_frame(list(y = matrix(1:2, 2)))
  expect_identical(vec_order(df), 1:2)

  df$x <- tibble::tibble(y = matrix(1:2, 2))
  expect_identical(vec_order(df), 1:2)
})

test_that("classed proxies do not affect performance (tidyverse/dplyr#5423)", {
  skip_on_cran()
  x <- glue::glue("{1:10000}")
  expect_time_lt(vec_order(x), 0.2)
})

test_that("can order data frames that don't allow removing the column names (#1298)", {
  skip_if_not_installed("withr")

  local_methods(
    `names<-.vctrs_foobar` = function(x, value) {
      if (is.null(value)) {
        abort("Cannot remove names.")
      }
      NextMethod()
    }
  )

  df <- foobar(data.frame(x = 1, y = 2))

  expect_silent(expect_identical(vec_order(df), 1L))
})

test_that("missing values in lists are respected (#1401)", {
  x <- list(1, NULL, 2, NULL)
  expect_identical(vec_order(x, na_value = "largest"), c(1L, 3L, 2L, 4L))
  expect_identical(vec_order(x, na_value = "smallest"), c(2L, 4L, 1L, 3L))
})

test_that("dots must be empty (#1647)", {
  expect_snapshot(error = TRUE, {
    vec_order(1, 2)
  })
  expect_snapshot(error = TRUE, {
    vec_sort(1, 2)
  })
})

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(integer()), integer())
})

test_that("can order integers", {
  x <- c(2L, 3L, 1L, 5L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:5
  expect_identical(vec_order_radix(x), order(x))
})

test_that("orders correctly around the UINT8_MAX boundary", {
  x <- 251:255
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1L, 3L, 1L, 3L)
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c(1L, NA_integer_, 3L)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("integer, small: `NA` order can be first", {
  x <- c(1L, NA_integer_, 3L)
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("double: `direction` can be set to `desc`", {
  x <- c(1, 5, 3)
  expect_identical(vec_order_radix(x, direction = "desc"), c(2L, 3L, 1L))
  x <- c(1L, .Machine$integer.max, 3L)
  expect_identical(vec_order_radix(x, direction = "desc"), c(2L, 3L, 1L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3L, NA_integer_, 1L, 2L)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_integer_, NA_integer_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order when in expected order", {
  x <- c(1L, 1L, 2L, NA, NA)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NA, 3L, 3L, 2L)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NA, 1L, 1L, 2L)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    1:5
  )

  x <- c(3L, 3L, 2L, NA, NA)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    1:5
  )
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, 2L, 1L)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    3:1
  )

  x <- c(1L, 2L, NA)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    3:1
  )

  x <- c(2L, 1L, NA)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    3:1
  )

  x <- c(NA, 1L, 2L)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    3:1
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - counting

# To trigger counting ordering, get above the insertion order boundary and then
# have a range less than the counting order range boundary.

test_that("can order integers with counting order", {
  x <- (ORDER_INSERTION_BOUNDARY + 1L):1L
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:(ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L)
  expect_identical(
    vec_order_radix(x)[1:2],
    c(1L, ORDER_INSERTION_BOUNDARY + 1L)
  )
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3L, NA_integer_, 1L, 2L, 1:ORDER_INSERTION_BOUNDARY)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<integer>) - radix

# To trigger radix ordering, get above the insertion order boundary and then
# have a range greater than the counting order range boundary.

test_that("can order integers with radix order", {
  x <- c(INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L, 1:ORDER_INSERTION_BOUNDARY)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L, INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L)
  expect_identical(
    vec_order_radix(x)[1:2],
    c(1L, ORDER_INSERTION_BOUNDARY + 1L)
  )
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(
    3L,
    NA_integer_,
    1L,
    2L,
    1:ORDER_INSERTION_BOUNDARY,
    INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L
  )

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("can order all 1 value", {
  x <- rep(1L, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    base_order(x, decreasing = TRUE)
  )
})

test_that("all `NA` values works - ensures that we can compute the 'range' of all NAs", {
  x <- rep(NA_integer_, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    base_order(x, decreasing = TRUE)
  )
})

test_that("can order with many NAs first", {
  x <- c(rep(NA_integer_, ORDER_INSERTION_BOUNDARY + 1L), 2L)
  expect_identical(vec_order_radix(x), base_order(x))
  expect_identical(
    vec_order_radix(x, na_value = "smallest"),
    base_order(x, na.last = FALSE)
  )
})

test_that("subtraction in counting order range computation works correctly (#1399)", {
  x <- c(rep(1L, ORDER_INSERTION_BOUNDARY), -2147483647L)
  expect_identical(vec_order_radix(x), base_order(x))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<logical>)

# Really this just goes through the integer infrastructure. Just checking that
# it is working.

test_that("can order size zero input", {
  expect_identical(vec_order_radix(logical()), integer())
})

test_that("can order logicals", {
  x <- c(FALSE, TRUE, FALSE)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(TRUE, NA, FALSE)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA, NA)
  expect_identical(vec_order_radix(x), order(x))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<double>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(double()), integer())
})

test_that("can order doubles", {
  x <- c(2, 3, 1, 5)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:5 + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1, 3, 1, 3)
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c(1, NA_real_, 3)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("double: `NA` order can be first", {
  x <- c(1, NA_real_, 3)
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3, NA_real_, 1, 2)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_real_, NA_real_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("NA_real_ and NaN look identical for ordering", {
  x <- c(NA_real_, NaN)
  expect_identical(vec_order_radix(x, na_value = "largest"), c(1L, 2L))
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(1L, 2L))
})

test_that("double: -Inf / Inf order correctly", {
  x <- c(0, -Inf, Inf)
  expect_identical(vec_order_radix(x, direction = "asc"), c(2L, 1L, 3L))
  expect_identical(vec_order_radix(x, direction = "desc"), c(3L, 1L, 2L))
})

test_that("double: -0 and 0 order identically / stably", {
  x <- c(0, -0)
  expect_identical(vec_order_radix(x, direction = "desc"), c(1L, 2L))
  expect_identical(vec_order_radix(x, direction = "asc"), c(1L, 2L))
})

test_that("can order when in expected order", {
  x <- c(1, 1, 2, NA, NaN)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NaN, 3, 3, 2)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NaN, 1, 1, 2)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    1:5
  )

  x <- c(3, 3, 2, NA, NaN)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    1:5
  )
})

test_that("can order when in expected order - using distinct NaN values", {
  x <- c(1, 1, 2, NaN, NA)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    1:5
  )

  x <- c(NA, NaN, 3, 3, 2)
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    1:5
  )

  x <- c(NA, NaN, 1, 1, 2)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    1:5
  )

  x <- c(3, 3, 2, NaN, NA)
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    1:5
  )
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, 2, 1)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    3:1
  )

  x <- c(1, 2, NA)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    3:1
  )

  x <- c(2, 1, NA)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    3:1
  )

  x <- c(NA, 1, 2)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    3:1
  )
})

test_that("can order when in strictly opposite of expected order (no ties) - using distinct NaN values", {
  x <- c(NA, NaN, 2, 1)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    4:1
  )

  x <- c(1, 2, NaN, NA)
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    4:1
  )

  x <- c(2, 1, NaN, NA)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    4:1
  )

  x <- c(NA, NaN, 1, 2)
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    4:1
  )
})

test_that("NaN is always placed next to numbers when treated as distinct", {
  x <- c(1, 2, NA, NaN)

  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    c(1L, 2L, 4L, 3L)
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    c(3L, 4L, 1L, 2L)
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    c(3L, 4L, 2L, 1L)
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    c(2L, 1L, 4L, 3L)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<double>) - radix

# To trigger radix ordering, get above the insertion order boundary. There is
# no intermediate counting sort for doubles.

test_that("can order doubles with radix order", {
  x <- (ORDER_INSERTION_BOUNDARY + 1L):1L + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- 1:(ORDER_INSERTION_BOUNDARY + 1L) + 0
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(1:ORDER_INSERTION_BOUNDARY, 1L) + 0
  expect_identical(
    vec_order_radix(x)[1:2],
    c(1L, ORDER_INSERTION_BOUNDARY + 1L)
  )
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c(3, NA_real_, 1, 2, 1:ORDER_INSERTION_BOUNDARY)

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- rep(NA_real_, ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("NA_real_ and NaN generally look identical for ordering", {
  x <- rep(c(NA_real_, NaN), ORDER_INSERTION_BOUNDARY + 1L)
  expect_identical(vec_order_radix(x, na_value = "largest"), seq_along(x))
  expect_identical(vec_order_radix(x, na_value = "smallest"), seq_along(x))
})

test_that("NA_real_ and NaN can be considered distinct with `nan_distinct`", {
  x <- rep(c(NA_real_, NaN), ORDER_INSERTION_BOUNDARY + 1L)

  loc_nan <- seq(2L, length(x), by = 2L)
  loc_na <- seq(1L, length(x), by = 2L)

  expect_identical(
    vec_order_radix(x, na_value = "largest", nan_distinct = TRUE),
    c(loc_nan, loc_na)
  )
  expect_identical(
    vec_order_radix(x, na_value = "smallest", nan_distinct = TRUE),
    c(loc_na, loc_nan)
  )
})

test_that("-Inf / Inf order correctly", {
  x <- c(rep(0, ORDER_INSERTION_BOUNDARY), -Inf, Inf)
  expect_identical(
    vec_order_radix(x, direction = "asc"),
    order(x, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    order(x, decreasing = TRUE)
  )
})

test_that("double, large: -0 and 0 order identically / stably", {
  x <- c(rep(0, ORDER_INSERTION_BOUNDARY), -0)
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    order(x, decreasing = TRUE)
  )
  expect_identical(
    vec_order_radix(x, direction = "asc"),
    order(x, decreasing = FALSE)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<complex>)

test_that("can order size zero input", {
  expect_identical(vec_order_radix(complex()), integer())
})

test_that("can order complex", {
  x <- complex(real = c(3, 1, 2))
  expect_identical(vec_order_radix(x), c(2L, 3L, 1L))
})

test_that("ordering on ties is done stably", {
  x <- complex(real = c(1, 3, 1, 3))
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("imaginary section is used to break ties", {
  x <- complex(
    real = c(1L, 2L, 1L),
    imaginary = c(3L, 2L, 1L)
  )
  expect_identical(vec_order_radix(x), c(3L, 1L, 2L))
})

test_that("can be used in a data frame", {
  x <- c(1L, 1L, 1L, 2L, 1L)

  y <- complex(
    real = c(1L, 2L, 1L, 3L, 1L),
    imaginary = c(3L, 2L, 1L, 4L, 1L)
  )

  z <- c(1, 2, 5, 4, 3)

  # as second column
  df1 <- data.frame(x = x, y = y)

  # as first column
  df2 <- data.frame(y = y, x = x)

  # as second column with a third after it to break ties
  df3 <- data.frame(x = x, y = y, z = z)

  # Base R can't do radix sorting with complex
  expect_identical(vec_order_radix(df1), c(3L, 5L, 1L, 2L, 4L))
  expect_identical(vec_order_radix(df2), c(3L, 5L, 1L, 2L, 4L))
  expect_identical(vec_order_radix(df3), c(5L, 3L, 1L, 2L, 4L))
})

test_that("can be used in a data frame when all group sizes are >1", {
  # https://github.com/tidyverse/dplyr/issues/7708

  # Group sizes of 3 (1L) and 2 (2L)
  x <- c(1L, 2L, 1L, 2L, 1L)

  y <- complex(
    # Group sizes of 3 (1L, 1L) and 2 (2L, 2L)
    real = c(1L, 2L, 1L, 2L, 1L),
    # This breaks ties within each group
    imaginary = c(3L, 2L, 2L, 4L, 1L)
  )

  df <- data.frame(x = x, y = y)

  expect_identical(vec_order_radix(df), c(5L, 3L, 1L, 2L, 4L))
})

test_that("can be used in a data frame as the third column", {
  # Testing that `complex_first_pass` is reset between columns

  x <- c(1L, 1L, 1L, 2L, 2L)

  y <- complex(
    real = x,
    # This breaks one tie
    imaginary = c(1L, 2L, 1L, 2L, 2L)
  )

  z <- complex(
    # This breaks one tie
    real = c(3L, 2L, 1L, 2L, 2L),
    # This breaks one tie
    imaginary = c(3L, 2L, 1L, 2L, 1L)
  )

  df <- data.frame(x = x, y = y, z = z)

  expect_identical(vec_order_radix(df), c(3L, 1L, 2L, 5L, 4L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- complex(real = c(3, NA, 1.5, 2, NA), imaginary = c(1, 1, 1, 1, 2))

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )

  # In fixing #1403, we now align with base R
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("full gambit of tests involving missing values are working as expected (#1403)", {
  x <- complex(
    real = c(NaN, NA, NA, NA, NaN, NaN, 1, 1, 1, 2),
    imaginary = c(NA, NA, NaN, 1, NaN, 1, NA, NaN, 1, NA)
  )

  df <- data_frame(a = rep(1L, length(x)), x = x)

  # {number}, {NaN}, {NaN + NA}, {NA + NaN}, {NA}
  expect <- c(9L, 5L, 6L, 8L, 1L, 3L, 2L, 4L, 7L, 10L)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "asc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    expect
  )

  # {NA}, {NA + NaN}, {NaN + NA}, {NaN}, {number}
  expect <- c(2L, 4L, 7L, 10L, 3L, 1L, 5L, 6L, 8L, 9L)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = TRUE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "desc",
      na_value = "largest",
      nan_distinct = TRUE
    ),
    expect
  )

  # {number}, {NA or NaN}
  expect <- c(9L, 1:7, 8L, 10L)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "largest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "asc",
      na_value = "largest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "desc",
      na_value = "smallest",
      nan_distinct = FALSE
    ),
    expect
  )

  # {NA or NaN}, {number}
  expect <- c(1:8, 10L, 9L)
  expect_identical(
    vec_order_radix(
      x,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      x,
      direction = "desc",
      na_value = "largest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "asc",
      na_value = "smallest",
      nan_distinct = FALSE
    ),
    expect
  )
  expect_identical(
    vec_order_radix(
      df,
      direction = "desc",
      na_value = "largest",
      nan_distinct = FALSE
    ),
    expect
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<character>) - insertion

test_that("can order size zero input", {
  expect_identical(vec_order_radix(character()), integer())
})

test_that("can order characters", {
  x <- c("xy", "x", "a", "bc")
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order sorted vector", {
  x <- c("a", "b", "c")
  expect_identical(vec_order_radix(x), order(x))
})

test_that("ordering on ties is done stably", {
  x <- c("ab", "ba", "ab", "ba")
  expect_identical(vec_order_radix(x)[1:2], c(1L, 3L))
  expect_identical(vec_order_radix(x)[3:4], c(2L, 4L))
})

test_that("`NA` order defaults to last", {
  x <- c("x", NA_character_, "y")
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))
})

test_that("character, small: `NA` order can be first", {
  x <- c("x", NA_character_, "y")
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 1L, 3L))
})

test_that("character, small: `direction` can be set to `desc`", {
  x <- c("x", "abcde", "yz")
  expect_identical(vec_order_radix(x, direction = "desc"), c(3L, 1L, 2L))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- c("aaa", NA_character_, "a", "aa")

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

test_that("all `NA` values works", {
  x <- c(NA_character_, NA_character_)
  expect_identical(vec_order_radix(x), order(x))
})

test_that("all `''` values works", {
  x <- c("", "")
  expect_identical(vec_order_radix(x), order(x))
})

test_that("can order empty string vs ASCII value 1 'Start of Header'", {
  x <- c("\001", "")
  expect_identical(vec_order_radix(x), c(2L, 1L))
})

test_that("can be used in a data frame", {
  x <- c(1L, 4L, 1L, 3L, 1L)

  y <- c("zy", "zz", "abcd", "gfa", "zy")

  z <- c(1, 2, 5, 4, 3)

  # as second column
  df1 <- data.frame(x = x, y = y)

  # as first column
  df2 <- data.frame(y = y, x = x)

  # as second column with a third after it to break ties
  df3 <- data.frame(x = x, y = y, z = z)

  expect_identical(vec_order_radix(df1), base_order(df1))
  expect_identical(vec_order_radix(df2), base_order(df2))
  expect_identical(vec_order_radix(df3), base_order(df3))
})

test_that("can have multiple character columns in a data frame", {
  df <- data.frame(
    x = c("def", "aba", "aba", "aba", "q"),
    y = c("zy", "zz", "zz", "gfa", "zy"),
    z = c("foo", "qux", "bar", "baz", "boo")
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

test_that("can order with varying encodings by converting to UTF-8", {
  encs <- encodings()
  x <- c(encs$utf8, encs$unknown, encs$latin1, "AC")

  expect_identical(vec_order_radix(x), c(4L, 1L, 2L, 3L))
  expect_identical(vec_order_radix(x, direction = "desc"), c(1L, 2L, 3L, 4L))
})

test_that("can order when in expected order", {
  x <- c("a", "a", "b", NA, NA)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NA, "c", "c", "b")
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    1:5
  )

  x <- c(NA, NA, "a", "a", "b")
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    1:5
  )

  x <- c("c", "c", "b", NA, NA)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    1:5
  )
})

test_that("can order when in strictly opposite of expected order (no ties)", {
  x <- c(NA, "b", "a")
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    3:1
  )

  x <- c("a", "b", NA)
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    3:1
  )

  x <- c("b", "a", NA)
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    3:1
  )

  x <- c(NA, "a", "b")
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    3:1
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<character>) - radix

# Have to get the number of unique strings above the ORDER_INSERTION_BOUNDARY
# to trigger radix ordering.

test_that("can order character vectors", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  expect_identical(vec_order_radix(x), base_order(x))
})

test_that("ordering on ties is done stably", {
  x <- c(paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L)), "x1")
  expect_identical(vec_order_radix(x)[1:2], c(1L, length(x)))
})

test_that("`NA` order defaults to last", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "y")
  expect_identical(vec_order_radix(x)[length(x)], length(x) - 1L)
})

test_that("character, large: `NA` order can be first", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "y")
  expect_identical(
    vec_order_radix(x, na_value = "smallest")[[1L]],
    length(x) - 1L
  )
})

test_that("character, large: `direction` can be set to `desc`", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    base_order(x, decreasing = TRUE)
  )
})

test_that("after `NA` removal, all `''` works", {
  # `NA` prevents it from looking sorted to start with
  x <- rep("", ORDER_INSERTION_BOUNDARY + 1L)
  x <- c(NA, x, NA)

  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "largest"),
    base_order(x, na.last = TRUE, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "largest"),
    base_order(x, na.last = FALSE, decreasing = TRUE)
  )
  expect_identical(
    vec_order_radix(x, direction = "asc", na_value = "smallest"),
    base_order(x, na.last = FALSE, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(x, direction = "desc", na_value = "smallest"),
    base_order(x, na.last = TRUE, decreasing = TRUE)
  )

  # `""` coming through as a "chunk"
  y <- vec_rep_each(c("a", "b", "c", "d", "e"), length(x))
  x <- vec_rep(x, 5)
  df <- data_frame(y = y, x = x)

  expect_identical(
    vec_order_radix(df, direction = "asc", na_value = "largest"),
    base_order(df, na.last = TRUE, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(df, direction = "desc", na_value = "largest"),
    base_order(df, na.last = FALSE, decreasing = TRUE)
  )
  expect_identical(
    vec_order_radix(df, direction = "asc", na_value = "smallest"),
    base_order(df, na.last = FALSE, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(df, direction = "desc", na_value = "smallest"),
    base_order(df, na.last = TRUE, decreasing = TRUE)
  )
})

test_that("mixing `''` with other strings works", {
  x <- rep("", ORDER_INSERTION_BOUNDARY + 1L)
  x <- c("xyz", x, "abc", "xyz", "a")

  expect_identical(
    vec_order_radix(x, direction = "asc"),
    base_order(x, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    base_order(x, decreasing = TRUE)
  )

  x <- rep(c("xyz", "abc", "xyz"), ORDER_INSERTION_BOUNDARY + 1L)
  x <- c("", x, "")

  expect_identical(
    vec_order_radix(x, direction = "asc"),
    base_order(x, decreasing = FALSE)
  )
  expect_identical(
    vec_order_radix(x, direction = "desc"),
    base_order(x, decreasing = TRUE)
  )
})

test_that("we aren't indexing past an individual string", {
  # This is why the `\0` check is practically important
  # - `max_string_size = 3`
  # - After `pass = 0` we learn nothing
  # - After `pass = 1` we have two groups:
  #   - byte b: `"abc\0"`, `"abd\0"`
  #   - byte \0: `"a\0"`
  # - We refuse to recurse further into the `\0` byte group, otherwise we'd
  #   index OOB
  x <- c(
    "abc",
    "abd",
    rep("a", ORDER_INSERTION_BOUNDARY + 1)
  )

  expect_identical(vec_order_radix(x), base_order(x))
})

test_that("all combinations of `direction` and `na_value` work", {
  x <- paste0("x", seq(1L, ORDER_INSERTION_BOUNDARY + 1L))
  x <- c(x, NA_character_, "x", "aa", "x1")

  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "asc")],
    x[base_order(x, na.last = TRUE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "asc")],
    x[base_order(x, na.last = FALSE, decreasing = FALSE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "largest", direction = "desc")],
    x[base_order(x, na.last = FALSE, decreasing = TRUE)]
  )
  expect_identical(
    x[vec_order_radix(x, na_value = "smallest", direction = "desc")],
    x[base_order(x, na.last = TRUE, decreasing = TRUE)]
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<list>)

test_that("list elements are ordered by first appearance", {
  expect_identical(vec_order_radix(list(1:2, "a", 1:2)), c(1L, 3L, 2L))
})

test_that("missing values in lists are respected (#1401)", {
  x <- list(1, NULL, 2, NULL)
  expect_identical(vec_order_radix(x, na_value = "largest"), c(1L, 3L, 2L, 4L))
  expect_identical(vec_order_radix(x, na_value = "smallest"), c(2L, 4L, 1L, 3L))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - insertion

test_that("data frame with no columns and no rows returns integer()", {
  x <- data.frame()
  expect_identical(vec_order_radix(x), integer())
})

test_that("data frame with no columns and some rows returns sequential rows", {
  x <- new_data_frame(n = 5L)
  expect_identical(vec_order_radix(x), 1:5)
})

test_that("can order with multiple pre-sorted keys", {
  df <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_order_radix(df), 1:2)
})

test_that("first column has ordering presedence", {
  df <- data.frame(x = c(3L, 2L, 1L), y = c(1L, 2L, 3L))
  expect_identical(vec_order_radix(df), 3:1)
})

test_that("secondary columns break ties - integer", {
  df <- data.frame(
    x = c(1L, 2L, 1L),
    y = c(3L, 2L, 1L)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("secondary columns break ties - double", {
  df <- data.frame(
    x = c(1, 2, 1),
    y = c(3, 2, 1)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("secondary columns break ties - logical", {
  df <- data.frame(
    x = c(FALSE, TRUE, FALSE),
    y = c(TRUE, TRUE, FALSE)
  )
  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))
})

test_that("orders correctly when first column is already ordered but second isn't", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L),
    y = c(3L, 2L, 4L, 1L)
  )
  expect_identical(vec_order_radix(df), c(2L, 1L, 4L, 3L))
})

test_that("orders correctly when first column is already ordered but second isn't - character", {
  df <- data.frame(
    x = c("a", "a", "b", "b"),
    y = c("c", "b", "d", "a")
  )

  expect_identical(vec_order_radix(df), c(2L, 1L, 4L, 3L))
})

test_that("`direction` is recycled", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L),
    y = c(3L, 2L, 4L, 1L)
  )
  expect_identical(vec_order_radix(df, direction = "desc"), c(3L, 4L, 1L, 2L))
})

test_that("`na_value` is recycled", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA),
    y = c(3L, 2L, 4L, 1L, NA)
  )
  expect_identical(
    vec_order_radix(df, na_value = "smallest"),
    c(5L, 2L, 1L, 4L, 3L)
  )
})

test_that("`direction` can be a vector", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, 1L),
    y = c(3L, 2L, 4L, 1L, 3L, NA)
  )
  expect_identical(
    vec_order_radix(df, direction = c("desc", "asc")),
    c(5L, 4L, 3L, 2L, 1L, 6L)
  )
})

test_that("`na_value` can be a vector", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, NA),
    y = c(3L, 2L, 4L, 1L, NA, 2)
  )
  expect_identical(
    vec_order_radix(df, na_value = c("smallest", "largest")),
    c(6L, 5L, 2L, 1L, 4L, 3L)
  )
})

test_that("`na_value` and `direction` can both be vectors", {
  df <- data.frame(
    x = c(1L, 1L, 2L, 2L, NA, NA),
    y = c(3L, 2L, 4L, 1L, NA, 2)
  )

  expect_identical(
    vec_order_radix(
      df,
      direction = c("desc", "asc"),
      na_value = c("smallest", "largest")
    ),
    c(4:1, 6:5)
  )
})

test_that("`direction` is recycled right with array columns (#1753)", {
  df <- data_frame(
    x = matrix(c(1, 1, 1, 3, 2, 2), ncol = 2),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(2L, 3L, 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_order_radix(df, direction = c("asc", "desc", "desc"))
  })

  df <- data_frame(
    x = array(c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, 3, 3), dim = c(3, 2, 2)),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(2L, 3L, 1L)
  )
})

test_that("`na_value` is recycled right with array columns (#1753)", {
  df <- data_frame(
    x = matrix(c(1, 1, 1, 3, NA, 2), ncol = 2),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, na_value = c("largest", "smallest")),
    c(3L, 1L, 2L)
  )
  expect_identical(
    vec_order_radix(df, na_value = c("smallest", "largest")),
    c(2L, 3L, 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_order_radix(df, direction = c("smallest", "largest", "largest"))
  })

  df <- data_frame(
    x = array(c(1, 1, 1, 2, 2, 2, 3, 3, 3, 4, NA, 3), dim = c(3, 2, 2)),
    y = 3:1
  )
  expect_identical(
    vec_order_radix(df, na_value = c("largest", "smallest")),
    c(3L, 1L, 2L)
  )
  expect_identical(
    vec_order_radix(df, na_value = c("smallest", "largest")),
    c(2L, 3L, 1L)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - counting

test_that("can order 2+ integer column chunks with counting sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1L,
    y = c(rep(2L, quarter_low), rep(1L, quarter_high), rep(3L, half))
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix(<data.frame>) - radix

test_that("can order 2+ integer column chunks with radix sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1L,
    y = c(
      rep(2L, quarter_low),
      rep(1L, quarter_high),
      rep(3L, half),
      INT_ORDER_COUNTING_RANGE_BOUNDARY + 1L
    )
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

test_that("can order 2+ double column chunks with radix sort", {
  half <- floor(ORDER_INSERTION_BOUNDARY / 2) + 1L
  quarter_low <- floor(half / 2)
  quarter_high <- ceiling(half / 2)

  df <- data.frame(
    x = 1,
    y = c(
      rep(2, quarter_low),
      rep(1, quarter_high),
      rep(3, half),
      INT_ORDER_COUNTING_RANGE_BOUNDARY + 1
    )
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix() - chr_proxy_collate

test_that("`chr_proxy_collate` transforms string input", {
  x <- c("b", "a", "A")
  expect_identical(
    vec_order_radix(x, chr_proxy_collate = tolower),
    c(2L, 3L, 1L)
  )
  expect_identical(
    vec_order_radix(x, chr_proxy_collate = ~ tolower(.x)),
    c(2L, 3L, 1L)
  )
})

test_that("`chr_proxy_collate` works with data frame columns and is applied to all string columns", {
  df <- data_frame(x = c(1, 1, 1), y = c("B", "a", "a"), z = c("a", "D", "c"))
  expect_identical(
    vec_order_radix(df, chr_proxy_collate = tolower),
    c(3L, 2L, 1L)
  )
})

test_that("`chr_proxy_collate` is validated", {
  expect_error(
    vec_order_radix("x", chr_proxy_collate = 1),
    "Can't convert `chr_proxy_collate` to a function"
  )
  expect_error(
    vec_order_radix("x", chr_proxy_collate = ~ c("y", "z")),
    "1, not 2"
  )
  expect_error(vec_order_radix("x", chr_proxy_collate = ~1), "character vector")
  expect_error(vec_order_radix("x", chr_proxy_collate = function() {
    "y"
  }))
})

test_that("`chr_proxy_collate` can return bytes-encoded strings (like `stringi::stri_sort_key()`)", {
  x <- c("A", "a", "b", "B")

  # Mimic stringi::stri_sort_key(x, locale = "en")
  sort_key <- function(x) {
    # dput(lapply(stringi::stri_sort_key(x, locale = "en"), charToRaw))
    out <- list(
      as.raw(c(0x2a, 0x01, 0x05, 0x01, 0xdc)),
      as.raw(c(0x2a, 0x01, 0x05, 0x01, 0x05)),
      as.raw(c(0x2c, 0x01, 0x05, 0x01, 0x05)),
      as.raw(c(0x2c, 0x01, 0x05, 0x01, 0xdc))
    )

    out <- vapply(out, FUN.VALUE = character(1), function(x) {
      # Uses native encoding
      x <- rawToChar(x)
      Encoding(x) <- "bytes"
      x
    })

    out
  }

  expect_identical(
    vec_order_radix(x, chr_proxy_collate = sort_key),
    c(2L, 1L, 3L, 4L)
  )
})

# ------------------------------------------------------------------------------
# vec_order_radix() - error checking

test_that("`na_value` is checked", {
  expect_error(
    vec_order_radix(1L, na_value = "x"),
    "\"largest\" or \"smallest\""
  )
  expect_error(
    vec_order_radix(1L, na_value = c(TRUE, TRUE)),
    "must be a character vector"
  )
  expect_error(
    vec_order_radix(1L, na_value = NA_character_),
    "can't be missing"
  )
})

test_that("`direction` is checked", {
  expect_error(vec_order_radix(1L, direction = "x"), "must contain only")
  expect_error(vec_order_radix(1L, direction = c("asc", "asc")), "single value")
  expect_error(
    vec_order_radix(1L, direction = NA_character_),
    "can't be missing"
  )
  expect_error(
    vec_order_radix(data.frame(x = 1), direction = c("asc", "asc")),
    "length 1 or"
  )
})

test_that("`x` is checked", {
  expect_error(vec_order_radix(foobar()), class = "vctrs_error_scalar_type")
})

# ------------------------------------------------------------------------------
# vec_order_radix() - groups

test_that("groups can be reallocated if we exceed the max group data size", {
  set.seed(123)

  # The first column has all unique groups so 1 more than the default group
  # data size is needed and will be reallocated on the fly
  df <- data.frame(
    x = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE),
    y = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE),
    z = sample(GROUP_DATA_SIZE_DEFAULT + 1L, replace = TRUE)
  )

  expect_identical(vec_order_radix(df), base_order(df))
})

# ------------------------------------------------------------------------------
# vec_order_radix() - comparison proxy

test_that("ordering works with rcrd types", {
  x <- tuple(c(1, 2, 1), c(3, 2, 1))
  expect_identical(vec_order_radix(x), c(3L, 1L, 2L))
})

test_that("data frame comparison proxies don't allow vector `direction` or `na_value`", {
  x <- tuple(c(1, 2, 1), c(3, 2, 1))
  expect_error(vec_order_radix(x, direction = c("desc", "asc")), "single value")
  expect_error(
    vec_order_radix(x, na_value = c("largest", "smallest")),
    "single value"
  )
})

test_that("ordering works with df-cols", {
  df_col <- new_data_frame(list(y = c(2, 1, 2), z = c(3, 3, 3)))
  df <- new_data_frame(list(x = c(1, 1, 1), y = df_col))

  expect_identical(vec_order_radix(df), c(2L, 1L, 3L))

  # Can only supply a max of 2 `direction` or `na_value` values which get internally
  # expanded to 3 to match the flattened df proxy
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(1L, 3L, 2L)
  )

  expect_error(
    vec_order_radix(df, direction = c("desc", "desc", "asc")),
    "or length equal to"
  )
})

test_that("ordering works with df-cols with 0 cols", {
  df_col <- new_data_frame(list(), n = 3L)
  df <- new_data_frame(list(x = c(1, 3, 1), y = df_col, z = c(2, 1, 1)))

  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))

  # Can supply 3 `direction` values even though the 0-col df-col gets dropped
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc", "desc")),
    c(1L, 3L, 2L)
  )

  expect_error(
    vec_order_radix(df, direction = c("desc", "asc")),
    "or length equal to"
  )
})

test_that("ordering works with rcrd cols", {
  y <- tuple(c(1, 2, 1), c(3, 2, 1))
  df <- new_data_frame(list(z = c(1, 1, 1), y = y))

  expect_identical(vec_order_radix(df), c(3L, 1L, 2L))

  # Can only supply a max of 2 `direction` values which get internally
  # expanded to 3 to match the flattened df proxy
  expect_identical(
    vec_order_radix(df, direction = c("asc", "desc")),
    c(2L, 1L, 3L)
  )

  expect_error(
    vec_order_radix(df, direction = c("desc", "desc", "asc")),
    "or length equal to"
  )
})

# ------------------------------------------------------------------------------
# `vec_order_radix()` - Pre-existing tests

test_that("can request NAs sorted first", {
  expect_equal(
    vec_order_radix(c(1, NA), direction = "asc", na_value = "largest"),
    1:2
  )
  expect_equal(
    vec_order_radix(c(1, NA), direction = "desc", na_value = "largest"),
    2:1
  )

  expect_equal(
    vec_order_radix(c(1, NA), direction = "asc", na_value = "smallest"),
    2:1
  )
  expect_equal(
    vec_order_radix(c(1, NA), direction = "desc", na_value = "smallest"),
    1:2
  )
})

test_that("can sort data frames", {
  df <- data.frame(x = c(1, 2, 1), y = c(1, 2, 2))

  out1 <- vec_sort(df)
  expect_equal(out1, data.frame(x = c(1, 1, 2), y = c(1, 2, 2)))

  out2 <- vec_sort(df, direction = "desc")
  expect_equal(out2, data.frame(x = c(2, 1, 1), y = c(2, 2, 1)))
})

test_that("can sort empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_sort(df1), df1)

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_sort(df2), df2)
})

test_that("can order tibbles that contain non-comparable objects", {
  expect_equal(vec_order_radix(data_frame(x = list(10, 2, 1))), 1:3)
})

test_that("can order matrices and arrays (#306)", {
  x <- matrix(c(1, 1, 1, 1, 2, 1), ncol = 2)
  expect_identical(vec_order_radix(x), c(1L, 3L, 2L))

  x <- array(1:8, c(2, 2, 2))
  x[2] <- 1
  x[3] <- 5
  expect_identical(vec_order_radix(x), 2:1)
})

test_that("can order empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_order_radix(df1), integer())

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_order_radix(df2), integer())
})

test_that("can order data frames with data frame columns (#527)", {
  expect_equal(
    vec_order_radix(iris),
    vec_order_radix(data_frame(iris = iris))
  )
})

test_that("can order data frames (and subclasses) with matrix columns", {
  df <- new_data_frame(n = 2L)

  df$x <- new_data_frame(list(y = matrix(1:2, 2)))
  expect_identical(vec_order_radix(df), 1:2)

  df$x <- tibble::tibble(y = matrix(1:2, 2))
  expect_identical(vec_order_radix(df), 1:2)
})

# ------------------------------------------------------------------------------
# vec_locate_sorted_groups()

test_that("`vec_locate_sorted_groups()` is working", {
  x <- c(1, 3, 1, 5, 2, 5, 1)

  expect <- new_data_frame(
    list(
      key = c(1, 2, 3, 5),
      loc = list(c(1L, 3L, 7L), 5L, 2L, c(4L, 6L))
    )
  )

  expect_identical(vec_locate_sorted_groups(x), expect)
})

test_that("`chr_proxy_collate` can result in keys being seen as identical", {
  x <- c("b", "A", "a")
  y <- c("b", "a", "A")

  x_expect <- data_frame(key = c("A", "b"), loc = list(c(2L, 3L), 1L))
  y_expect <- data_frame(key = c("a", "b"), loc = list(c(2L, 3L), 1L))

  expect_identical(
    vec_locate_sorted_groups(x, chr_proxy_collate = tolower),
    x_expect
  )
  expect_identical(
    vec_locate_sorted_groups(y, chr_proxy_collate = tolower),
    y_expect
  )
})

# ------------------------------------------------------------------------------
# `vec_order_info(nan_distinct = FALSE)`

test_that("Indistinct NA and NaN are reported in the same group", {
  x <- c(NA, NaN)
  info <- vec_order_info(x, nan_distinct = FALSE)

  expect_identical(info[[1]], c(1L, 2L))
  expect_identical(info[[2]], 2L)
  expect_identical(info[[3]], 2L)
})

# ------------------------------------------------------------------------------
# `vec_order_info(<data.frame>)`

test_that("Zero column data frames with >0 rows work (#1863)", {
  # All rows are treated as being from the same group
  x <- data_frame(.size = 5)
  info <- vec_order_info(x)

  expect_identical(info[[1]], 1:5) # Order
  expect_identical(info[[2]], 5L) # Group sizes
  expect_identical(info[[3]], 5L) # Max group size
})

test_that("Zero column data frames with exactly 0 rows work (#1863)", {
  # This is a particularly special case, since we don't actually push a group size
  x <- data_frame(.size = 0L)
  info <- vec_order_info(x)

  expect_identical(info[[1]], integer())
  expect_identical(info[[2]], integer())
  expect_identical(info[[3]], 0L)
})

# ------------------------------------------------------------------------------
# vec_sort

test_that("can sort data frames", {
  df <- data.frame(x = c(1, 2, 1), y = c(1, 2, 2))

  out1 <- vec_sort(df)
  expect_equal(out1, data.frame(x = c(1, 1, 2), y = c(1, 2, 2)))

  out2 <- vec_sort(df, direction = "desc")
  expect_equal(out2, data.frame(x = c(2, 1, 1), y = c(2, 2, 1)))
})

test_that("can sort empty data frames (#356)", {
  df1 <- data.frame()
  expect_equal(vec_sort(df1), df1)

  df2 <- data.frame(x = numeric(), y = integer())
  expect_equal(vec_sort(df2), df2)
})

# ------------------------------------------------------------------------------
# vec_order

test_that("can request NAs sorted first", {
  expect_equal(
    vec_order(c(1, NA), direction = "asc", na_value = "largest"),
    1:2
  )
  expect_equal(
    vec_order(c(1, NA), direction = "desc", na_value = "largest"),
    2:1
  )

  expect_equal(
    vec_order(c(1, NA), direction = "asc", na_value = "smallest"),
    2:1
  )
  expect_equal(
    vec_order(c(1, NA), direction = "desc", na_value = "smallest"),
    1:2
  )
})

test_that("can order complex vectors", {
  x <- complex(real = c(1, 2, 2, 3, 3), imaginary = c(5, 4, 3, 2, NA))

  expect_equal(
    vec_order(x, direction = "asc", na_value = "largest"),
    c(1, 3, 2, 4, 5)
  )
  expect_equal(
    vec_order(x, direction = "desc", na_value = "largest"),
    rev(c(1, 3, 2, 4, 5))
  )
  expect_equal(
    vec_order(x, direction = "asc", na_value = "smallest"),
    c(5, 1, 3, 2, 4)
  )
  expect_equal(
    vec_order(x, direction = "desc", na_value = "smallest"),
    rev(c(5, 1, 3, 2, 4))
  )
})

test_that("can order tibbles that contain non-comparable objects", {
  expect_equal(vec_order(data_frame(x = list(10, 2, 1))), 1:3)
})

test_that("can order matrices and arrays (#306)", {
  x <- matrix(c(1, 1, 1, 1, 2, 1), ncol = 2)
  expect_identical(vec_order(x), c(1L, 3L, 2L))

  x <- array(1:8, c(2, 2, 2))
  x[2] <- 1
  x[3] <- 5
  expect_identical(vec_order(x), 2:1)
})

test_that("can order zero column data frames (#356, #1499)", {
  df <- data_frame()
  expect_identical(vec_order(df), integer())

  df <- data_frame(.size = 5L)
  expect_identical(vec_order(df), 1:5)

  df <- data_frame(.size = 5L)
  expect_identical(vec_order(df, direction = "desc"), 1:5)
})

test_that("can order zero row data frames (#356, #1499)", {
  df <- data.frame(x = numeric())
  expect_identical(vec_order(df), integer())

  df <- data.frame(x = numeric(), y = integer())
  expect_identical(vec_order(df), integer())

  df <- data.frame(x = numeric(), y = integer())
  expect_identical(vec_order(df, direction = "desc"), integer())
})

test_that("can order data frames with data frame columns (#527)", {
  expect_equal(
    vec_order(iris),
    vec_order(data_frame(iris = iris))
  )
})

test_that("can order data frames (and subclasses) with matrix columns", {
  df <- new_data_frame(n = 2L)

  df$x <- new_data_frame(list(y = matrix(1:2, 2)))
  expect_identical(vec_order(df), 1:2)

  df$x <- tibble::tibble(y = matrix(1:2, 2))
  expect_identical(vec_order(df), 1:2)
})

test_that("classed proxies do not affect performance (tidyverse/dplyr#5423)", {
  skip_on_cran()
  x <- glue::glue("{1:10000}")
  expect_time_lt(vec_order(x), 0.2)
})

test_that("can order data frames that don't allow removing the column names (#1298)", {
  skip_if_not_installed("withr")

  local_methods(
    `names<-.vctrs_foobar` = function(x, value) {
      if (is.null(value)) {
        abort("Cannot remove names.")
      }
      NextMethod()
    }
  )

  df <- foobar(data.frame(x = 1, y = 2))

  expect_silent(expect_identical(vec_order(df), 1L))
})

test_that("missing values in lists are respected (#1401)", {
  x <- list(1, NULL, 2, NULL)
  expect_identical(vec_order(x, na_value = "largest"), c(1L, 3L, 2L, 4L))
  expect_identical(vec_order(x, na_value = "smallest"), c(2L, 4L, 1L, 3L))
})

test_that("dots must be empty (#1647)", {
  expect_snapshot(error = TRUE, {
    vec_order(1, 2)
  })
  expect_snapshot(error = TRUE, {
    vec_sort(1, 2)
  })
})

# Rational record class from the S3 vector vignette

new_rational <- function(n = integer(), d = integer()) {
  if (!is_integer(n)) {
    abort("`n` must be an integer.")
  }
  if (!is_integer(d)) {
    abort("`d` must be an integer.")
  }
  new_rcrd(list(n = n, d = d), class = "vctrs_rational")
}

rational <- function(n, d) {
  args <- vec_cast_common(n, d, .to = integer())
  args <- vec_recycle_common(!!!args)
  new_rational(args[[1L]], args[[2L]])
}

format.vctrs_rational <- function(x, ...) {
  n <- field(x, "n")
  d <- field(x, "d")
  out <- paste0(n, "/", d)
  out[is.na(n) | is.na(d)] <- NA
  out
}

vec_proxy_equal.vctrs_rational <- function(x) {
  n <- field(x, "n")
  d <- field(x, "d")
  gcd <- gcd(n, d)
  data.frame(n = n / gcd, d = d / gcd)
}
gcd <- function(x, y) {
  r <- x %% y
  ifelse(r, gcd(y, r), y)
}

vec_proxy_compare.vctrs_rational <- function(x, ...) {
  field(x, "n") / field(x, "d")
}

rational_methods <- list(
  vec_ptype_abbr.vctrs_rational = function(x, ...) {
    "rtnl"
  },
  vec_ptype_full.vctrs_rational = function(x, ...) {
    "rational"
  },

  vec_ptype2.vctrs_rational = function(x, y, ...) {
    UseMethod("vec_ptype2.vctrs_rational")
  },
  vec_ptype2.vctrs_rational.vctrs_rational = function(x, y, ...) {
    new_rational()
  },
  vec_ptype2.vctrs_rational.integer = function(x, y, ...) {
    new_rational()
  },
  vec_ptype2.integer.vctrs_rational = function(x, y, ...) {
    new_rational()
  },

  vec_cast.vctrs_rational = function(x, to, ...) {
    UseMethod("vec_cast.vctrs_rational")
  },
  vec_cast.vctrs_rational.vctrs_rational = function(x, to, ...) {
    x
  },
  vec_cast.double.vctrs_rational = function(x, to, ...) {
    field(x, "n") / field(x, "d")
  },
  vec_cast.vctrs_rational.integer = function(x, to, ...) {
    rational(x, 1)
  },

  vec_proxy_equal.vctrs_rational = vec_proxy_equal.vctrs_rational,
  vec_proxy_compare.vctrs_rational = vec_proxy_compare.vctrs_rational
)

local_rational_class <- function(frame = caller_env()) {
  local_methods(.frame = frame, !!!rational_methods)
}

#' Locate observations matching specified conditions
#'
#' @description
#' `vec_locate_matches()` is a more flexible version of [vec_match()] used to
#' identify locations where each value of `needles` matches one or multiple
#' values in `haystack`. Unlike `vec_match()`, `vec_locate_matches()` returns
#' all matches by default, and can match on binary conditions other than
#' equality, such as `>`, `>=`, `<`, and `<=`.
#'
#' @details
#' [vec_match()] is identical to (but often slightly faster than):
#'
#' ```
#' vec_locate_matches(
#'   needles,
#'   haystack,
#'   condition = "==",
#'   multiple = "first",
#'   nan_distinct = TRUE
#' )
#' ```
#'
#' `vec_locate_matches()` is extremely similar to a SQL join between `needles`
#' and `haystack`, with the default being most similar to a left join.
#'
#' Be very careful when specifying match `condition`s. If a condition is
#' misspecified, it is very easy to accidentally generate an exponentially
#' large number of matches.
#'
#' @section Dependencies of `vec_locate_matches()`:
#' - [vec_order_radix()]
#' - [vec_detect_complete()]
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#' @inheritParams order-radix
#'
#' @param needles,haystack Vectors used for matching.
#'
#'   - `needles` represents the vector to search for.
#'
#'   - `haystack` represents the vector to search in.
#'
#'   Prior to comparison, `needles` and `haystack` are coerced to the same type.
#'
#' @param condition Condition controlling how `needles` should be compared
#'   against `haystack` to identify a successful match.
#'
#'   - One of: `"=="`, `">"`, `">="`, `"<"`, or `"<="`.
#'
#'   - For data frames, a length `1` or `ncol(needles)` character vector
#'     containing only the above options, specifying how matching is determined
#'     for each column.
#'
#' @param filter Filter to be applied to the matched results.
#'
#'   - `"none"` doesn't apply any filter.
#'
#'   - `"min"` returns only the minimum haystack value matching the current
#'     needle.
#'
#'   - `"max"` returns only the maximum haystack value matching the current
#'     needle.
#'
#'   - For data frames, a length `1` or `ncol(needles)` character vector
#'     containing only the above options, specifying a filter to apply to
#'     each column.
#'
#'   Filters don't have any effect on `"=="` conditions, but are useful for
#'   computing "rolling" matches with other conditions.
#'
#'   A filter can return multiple haystack matches for a particular needle
#'   if the maximum or minimum haystack value is duplicated in `haystack`. These
#'   can be further controlled with `multiple`.
#'
#' @param incomplete Handling of missing and [incomplete][vec_detect_complete]
#'   values in `needles`.
#'
#'   - `"compare"` uses `condition` to determine whether or not a missing value
#'     in `needles` matches a missing value in `haystack`. If `condition` is
#'     `==`, `>=`, or `<=`, then missing values will match.
#'
#'   - `"match"` always allows missing values in `needles` to match missing
#'     values in `haystack`, regardless of the `condition`.
#'
#'   - `"drop"` drops incomplete values in `needles` from the result.
#'
#'   - `"error"` throws an error if any `needles` are incomplete.
#'
#'   - If a single integer is provided, this represents the value returned
#'     in the `haystack` column for values of `needles` that are incomplete. If
#'     `no_match = NA`, setting `incomplete = NA` forces incomplete values in
#'     `needles` to be treated like unmatched values.
#'
#'   `nan_distinct` determines whether a `NA` is allowed to match a `NaN`.
#'
#' @param no_match Handling of `needles` without a match.
#'
#'   - `"drop"` drops `needles` with zero matches from the result.
#'
#'   - `"error"` throws an error if any `needles` have zero matches.
#'
#'   - If a single integer is provided, this represents the value returned in
#'     the `haystack` column for values of `needles` that have zero matches. The
#'     default represents an unmatched needle with `NA`.
#'
#' @param remaining Handling of `haystack` values that `needles` never matched.
#'
#'   - `"drop"` drops remaining `haystack` values from the result.
#'     Typically, this is the desired behavior if you only care when `needles`
#'     has a match.
#'
#'   - `"error"` throws an error if there are any remaining `haystack`
#'     values.
#'
#'   - If a single integer is provided (often `NA`), this represents the value
#'     returned in the `needles` column for the remaining `haystack` values
#'     that `needles` never matched. Remaining `haystack` values are always
#'     returned at the end of the result.
#'
#' @param multiple Handling of `needles` with multiple matches. For each needle:
#'
#'   - `"all"` returns all matches detected in `haystack`.
#'
#'   - `"any"` returns any match detected in `haystack` with no guarantees on
#'     which match will be returned. It is often faster than `"first"` and
#'     `"last"` if you just need to detect if there is at least one match.
#'
#'   - `"first"` returns the first match detected in `haystack`.
#'
#'   - `"last"` returns the last match detected in `haystack`.
#'
#' @param relationship Handling of the expected relationship between
#'   `needles` and `haystack`. If the expectations chosen from the list below
#'   are invalidated, an error is thrown.
#'
#'   - `"none"` doesn't perform any relationship checks.
#'
#'   - `"one-to-one"` expects:
#'     - Each value in `needles` matches at most 1 value in `haystack`.
#'     - Each value in `haystack` matches at most 1 value in `needles`.
#'
#'   - `"one-to-many"` expects:
#'     - Each value in `needles` matches any number of values in `haystack`.
#'     - Each value in `haystack` matches at most 1 value in `needles`.
#'
#'   - `"many-to-one"` expects:
#'     - Each value in `needles` matches at most 1 value in `haystack`.
#'     - Each value in `haystack` matches any number of values in `needles`.
#'
#'   - `"many-to-many"` expects:
#'     - Each value in `needles` matches any number of values in `haystack`.
#'     - Each value in `haystack` matches any number of values in `needles`.
#'
#'     This performs no checks, and is identical to `"none"`, but is provided to
#'     allow you to be explicit about this relationship if you know it exists.
#'
#'   - `"warn-many-to-many"` doesn't assume there is any known relationship, but
#'     will warn if `needles` and `haystack` have a many-to-many relationship
#'     (which is typically unexpected), encouraging you to either take a closer
#'     look at your inputs or make this relationship explicit by specifying
#'     `"many-to-many"`.
#'
#'   `relationship` is applied after `filter` and `multiple` to allow potential
#'   multiple matches to be filtered out first.
#'
#'   `relationship` doesn't handle cases where there are zero matches. For that,
#'   see `no_match` and `remaining`.
#'
#' @param needles_arg,haystack_arg Argument tags for `needles` and `haystack`
#'   used in error messages.
#'
#' @return A two column data frame containing the locations of the matches.
#'
#'   - `needles` is an integer vector containing the location of
#'     the needle currently being matched.
#'
#'   - `haystack` is an integer vector containing the location of the
#'     corresponding match in the haystack for the current needle.
#'
#' @export
#' @examples
#' x <- c(1, 2, NA, 3, NaN)
#' y <- c(2, 1, 4, NA, 1, 2, NaN)
#'
#' # By default, for each value of `x`, all matching locations in `y` are
#' # returned
#' matches <- vec_locate_matches(x, y)
#' matches
#'
#' # The result can be used to slice the inputs to align them
#' data_frame(
#'   x = vec_slice(x, matches$needles),
#'   y = vec_slice(y, matches$haystack)
#' )
#'
#' # If multiple matches are present, control which is returned with `multiple`
#' vec_locate_matches(x, y, multiple = "first")
#' vec_locate_matches(x, y, multiple = "last")
#' vec_locate_matches(x, y, multiple = "any")
#'
#' # Use `relationship` to add constraints and error on multiple matches if
#' # they aren't expected
#' try(vec_locate_matches(x, y, relationship = "one-to-one"))
#'
#' # In this case, the `NA` in `y` matches two rows in `x`
#' try(vec_locate_matches(x, y, relationship = "one-to-many"))
#'
#' # By default, `NA` is treated as being identical to `NaN`.
#' # Using `nan_distinct = TRUE` treats `NA` and `NaN` as different values, so
#' # `NA` can only match `NA`, and `NaN` can only match `NaN`.
#' vec_locate_matches(x, y, nan_distinct = TRUE)
#'
#' # If you never want missing values to match, set `incomplete = NA` to return
#' # `NA` in the `haystack` column anytime there was an incomplete value
#' # in `needles`.
#' vec_locate_matches(x, y, incomplete = NA)
#'
#' # Using `incomplete = NA` allows us to enforce the one-to-many relationship
#' # that we couldn't before
#' vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA)
#'
#' # `no_match` allows you to specify the returned value for a needle with
#' # zero matches. Note that this is different from an incomplete value,
#' # so specifying `no_match` allows you to differentiate between incomplete
#' # values and unmatched values.
#' vec_locate_matches(x, y, incomplete = NA, no_match = 0L)
#'
#' # If you want to require that every `needle` has at least 1 match, set
#' # `no_match` to `"error"`:
#' try(vec_locate_matches(x, y, incomplete = NA, no_match = "error"))
#'
#' # By default, `vec_locate_matches()` detects equality between `needles` and
#' # `haystack`. Using `condition`, you can detect where an inequality holds
#' # true instead. For example, to find every location where `x[[i]] >= y`:
#' matches <- vec_locate_matches(x, y, condition = ">=")
#'
#' data_frame(
#'   x = vec_slice(x, matches$needles),
#'   y = vec_slice(y, matches$haystack)
#' )
#'
#' # You can limit which matches are returned with a `filter`. For example,
#' # with the above example you can filter the matches returned by `x[[i]] >= y`
#' # down to only the ones containing the maximum `y` value of those matches.
#' matches <- vec_locate_matches(x, y, condition = ">=", filter = "max")
#'
#' # Here, the matches for the `3` needle value have been filtered down to
#' # only include the maximum haystack value of those matches, `2`. This is
#' # often referred to as a rolling join.
#' data_frame(
#'   x = vec_slice(x, matches$needles),
#'   y = vec_slice(y, matches$haystack)
#' )
#'
#' # In the very rare case that you need to generate locations for a
#' # cross match, where every value of `x` is forced to match every
#' # value of `y` regardless of what the actual values are, you can
#' # replace `x` and `y` with integer vectors of the same size that contain
#' # a single value and match on those instead.
#' x_proxy <- vec_rep(1L, vec_size(x))
#' y_proxy <- vec_rep(1L, vec_size(y))
#' nrow(vec_locate_matches(x_proxy, y_proxy))
#' vec_size(x) * vec_size(y)
#'
#' # By default, missing values will match other missing values when using
#' # `==`, `>=`, or `<=` conditions, but not when using `>` or `<` conditions.
#' # This is similar to how `vec_compare(x, y, na_equal = TRUE)` works.
#' x <- c(1, NA)
#' y <- c(NA, 2)
#'
#' vec_locate_matches(x, y, condition = "<=")
#' vec_locate_matches(x, y, condition = "<")
#'
#' # You can force missing values to match regardless of the `condition`
#' # by using `incomplete = "match"`
#' vec_locate_matches(x, y, condition = "<", incomplete = "match")
#'
#' # You can also use data frames for `needles` and `haystack`. The
#' # `condition` will be recycled to the number of columns in `needles`, or
#' # you can specify varying conditions per column. In this example, we take
#' # a vector of date `values` and find all locations where each value is
#' # between lower and upper bounds specified by the `haystack`.
#' values <- as.Date("2019-01-01") + 0:9
#' needles <- data_frame(lower = values, upper = values)
#'
#' set.seed(123)
#' lower <- as.Date("2019-01-01") + sample(10, 10, replace = TRUE)
#' upper <- lower + sample(3, 10, replace = TRUE)
#' haystack <- data_frame(lower = lower, upper = upper)
#'
#' # (values >= lower) & (values <= upper)
#' matches <- vec_locate_matches(needles, haystack, condition = c(">=", "<="))
#'
#' data_frame(
#'   lower = vec_slice(lower, matches$haystack),
#'   value = vec_slice(values, matches$needle),
#'   upper = vec_slice(upper, matches$haystack)
#' )
vec_locate_matches <- function(
  needles,
  haystack,
  ...,
  condition = "==",
  filter = "none",
  incomplete = "compare",
  no_match = NA_integer_,
  remaining = "drop",
  multiple = "all",
  relationship = "none",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL,
  needles_arg = "needles",
  haystack_arg = "haystack",
  error_call = current_env()
) {
  check_dots_empty0(...)
  frame <- environment()

  .Call(
    ffi_locate_matches,
    needles,
    haystack,
    condition,
    filter,
    incomplete,
    no_match,
    remaining,
    multiple,
    relationship,
    nan_distinct,
    chr_proxy_collate,
    needles_arg,
    haystack_arg,
    frame
  )
}

# ------------------------------------------------------------------------------

#' Internal FAQ - Implementation of `vec_locate_matches()`
#'
#' ```{r, child = "man/faq/internal/matches-algorithm.Rmd"}
#' ```
#'
#' @name internal-faq-matches-algorithm
NULL

# ------------------------------------------------------------------------------

# Helper used for testing and in the internal FAQ.
# It needs to live in R/ to be usable by the FAQ Rmd.
compute_nesting_container_info <- function(x, condition) {
  .Call(ffi_compute_nesting_container_info, x, condition)
}

# ------------------------------------------------------------------------------

stop_matches <- function(
  message = NULL,
  class = NULL,
  ...,
  call = caller_env()
) {
  stop_vctrs(
    message = message,
    class = c(class, "vctrs_error_matches"),
    ...,
    call = call
  )
}

warn_matches <- function(message, class = NULL, ..., call = caller_env()) {
  warn_vctrs(
    message = message,
    class = c(class, "vctrs_warning_matches"),
    ...,
    call = call
  )
}

# ------------------------------------------------------------------------------

stop_matches_overflow <- function(size, call) {
  size <- format(size, scientific = FALSE)

  # Pre-generating the message in this case because we want to use
  # `.internal = TRUE` and that doesn't work with lazy messages
  message <- c(
    "Match procedure results in an allocation larger than 2^31-1 elements.",
    i = glue::glue("Attempted allocation size was {size}.")
  )

  stop_matches(
    message = message,
    class = "vctrs_error_matches_overflow",
    size = size,
    call = call,
    .internal = TRUE
  )
}

# ------------------------------------------------------------------------------

stop_matches_nothing <- function(i, needles_arg, haystack_arg, call) {
  stop_matches(
    class = "vctrs_error_matches_nothing",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_nothing <- function(cnd, ...) {
  glue::glue(
    "Each value of `{cnd$needles_arg}` must have a match in `{cnd$haystack_arg}`."
  )
}

#' @export
cnd_body.vctrs_error_matches_nothing <- function(cnd, ...) {
  bullet <- glue::glue(
    "Location {cnd$i} of `{cnd$needles_arg}` does not have a match."
  )
  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

# ------------------------------------------------------------------------------

stop_matches_remaining <- function(i, needles_arg, haystack_arg, call) {
  stop_matches(
    class = "vctrs_error_matches_remaining",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_remaining <- function(cnd, ...) {
  glue::glue(
    "Each value of `{cnd$haystack_arg}` must be matched by `{cnd$needles_arg}`."
  )
}

#' @export
cnd_body.vctrs_error_matches_remaining <- function(cnd, ...) {
  bullet <- glue::glue(
    "Location {cnd$i} of `{cnd$haystack_arg}` was not matched."
  )
  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

# ------------------------------------------------------------------------------

stop_matches_incomplete <- function(i, needles_arg, call) {
  stop_matches(
    class = "vctrs_error_matches_incomplete",
    i = i,
    needles_arg = needles_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_incomplete <- function(cnd, ...) {
  glue::glue("`{cnd$needles_arg}` can't contain missing values.")
}

#' @export
cnd_body.vctrs_error_matches_incomplete <- function(cnd, ...) {
  bullet <- glue::glue("Location {cnd$i} contains missing values.")
  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

# ------------------------------------------------------------------------------

stop_matches_multiple <- function(i, needles_arg, haystack_arg, call) {
  stop_matches(
    class = "vctrs_error_matches_multiple",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_multiple <- function(cnd, ...) {
  cnd_matches_multiple_header(cnd$needles_arg, cnd$haystack_arg)
}

#' @export
cnd_body.vctrs_error_matches_multiple <- function(cnd, ...) {
  cnd_matches_multiple_body(cnd$i, cnd$needles_arg)
}

# ------------------------------------------------------------------------------

warn_matches_multiple <- function(i, needles_arg, haystack_arg, call) {
  message <- paste(
    cnd_matches_multiple_header(needles_arg, haystack_arg),
    cnd_matches_multiple_body(i, needles_arg),
    sep = "\n"
  )

  warn_matches(
    message = message,
    class = "vctrs_warning_matches_multiple",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

# ------------------------------------------------------------------------------

stop_matches_relationship_one_to_one <- function(
  i,
  which,
  needles_arg,
  haystack_arg,
  call
) {
  stop_matches_relationship(
    class = "vctrs_error_matches_relationship_one_to_one",
    i = i,
    which = which,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_relationship_one_to_one <- function(cnd, ...) {
  if (cnd$which == "needles") {
    cnd_matches_multiple_header(cnd$needles_arg, cnd$haystack_arg)
  } else {
    cnd_matches_multiple_header(cnd$haystack_arg, cnd$needles_arg)
  }
}

#' @export
cnd_body.vctrs_error_matches_relationship_one_to_one <- function(cnd, ...) {
  if (cnd$which == "needles") {
    cnd_matches_multiple_body(cnd$i, cnd$needles_arg)
  } else {
    cnd_matches_multiple_body(cnd$i, cnd$haystack_arg)
  }
}


stop_matches_relationship_one_to_many <- function(
  i,
  needles_arg,
  haystack_arg,
  call
) {
  stop_matches_relationship(
    class = "vctrs_error_matches_relationship_one_to_many",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_relationship_one_to_many <- function(cnd, ...) {
  cnd_matches_multiple_header(cnd$haystack_arg, cnd$needles_arg)
}

#' @export
cnd_body.vctrs_error_matches_relationship_one_to_many <- function(cnd, ...) {
  cnd_matches_multiple_body(cnd$i, cnd$haystack_arg)
}


stop_matches_relationship_many_to_one <- function(
  i,
  needles_arg,
  haystack_arg,
  call
) {
  stop_matches_relationship(
    class = "vctrs_error_matches_relationship_many_to_one",
    i = i,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_matches_relationship_many_to_one <- function(cnd, ...) {
  cnd_matches_multiple_header(cnd$needles_arg, cnd$haystack_arg)
}

#' @export
cnd_body.vctrs_error_matches_relationship_many_to_one <- function(cnd, ...) {
  cnd_matches_multiple_body(cnd$i, cnd$needles_arg)
}


stop_matches_relationship <- function(class = NULL, ..., call = caller_env()) {
  stop_matches(
    class = c(class, "vctrs_error_matches_relationship"),
    ...,
    call = call
  )
}

cnd_matches_multiple_header <- function(x_arg, y_arg) {
  glue::glue(
    "Each value of `{x_arg}` can match at most 1 value from `{y_arg}`."
  )
}

cnd_matches_multiple_body <- function(i, name) {
  bullet <- glue::glue("Location {i} of `{name}` matches multiple values.")
  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

# ------------------------------------------------------------------------------

warn_matches_relationship_many_to_many <- function(
  i,
  j,
  needles_arg,
  haystack_arg,
  call
) {
  message <- paste(
    glue::glue(
      "Detected an unexpected many-to-many relationship between `{needles_arg}` and `{haystack_arg}`."
    ),
    cnd_matches_multiple_body(i, needles_arg),
    cnd_matches_multiple_body(j, haystack_arg),
    sep = "\n"
  )

  warn_matches_relationship(
    message = message,
    class = "vctrs_warning_matches_relationship_many_to_many",
    i = i,
    j = j,
    needles_arg = needles_arg,
    haystack_arg = haystack_arg,
    call = call
  )
}

warn_matches_relationship <- function(
  message,
  class = NULL,
  ...,
  call = caller_env()
) {
  warn_matches(
    message = message,
    class = c(class, "vctrs_warning_matches_relationship"),
    ...,
    call = call
  )
}

# ------------------------------------------------------------------------------
# vec_locate_matches() - logicals

test_that("isn't confused by unspecified logical vectors", {
  x <- vec_locate_matches(logical(), NA)
  expect_identical(x$needles, integer())
  expect_identical(x$haystack, integer())

  x <- vec_locate_matches(NA, logical())
  expect_identical(x$needles, 1L)
  expect_identical(x$haystack, NA_integer_)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - integers

test_that("can match in increasing order", {
  x <- vec_locate_matches(1:2, 1:3)
  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - doubles

test_that("can match doubles", {
  x <- vec_locate_matches(c(1, 2, 5), c(2, 2, 3, 1))

  expect_identical(x$needles, c(1L, 2L, 2L, 3L))
  expect_identical(x$haystack, c(4L, 1L, 2L, NA))
})

test_that("can match Inf and -Inf with all conditions", {
  x <- c(Inf, -Inf)
  y <- c(-Inf, 0, Inf)

  res <- vec_locate_matches(x, y, condition = "==")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(3L, 1L))

  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 2L, 3L))

  res <- vec_locate_matches(x, y, condition = "<=")
  expect_identical(res$needles, c(1L, 2L, 2L, 2L))
  expect_identical(res$haystack, c(3L, 1L, 2L, 3L))

  res <- vec_locate_matches(x, y, condition = ">")
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, y, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, 3L, 1L))
})

test_that("NA and NaN don't match numbers with equality conditions", {
  expect_identical(vec_locate_matches(1, NA_real_)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(1, NaN)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(NA_real_, 1)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(NaN, 1)$haystack, NA_integer_)
})

test_that("NA and NaN are the same by default", {
  res <- vec_locate_matches(NA_real_, NaN)
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(NaN, NA_real_)
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "==")
  expect_identical(res$needles, rep(c(1L, 2L, 3L), each = 3))
  expect_identical(res$haystack, rep(c(1L, 2L, 3L), times = 3))

  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "==", multiple = "first")
  expect_identical(res$needles, c(1L, 2L, 3L))
  expect_identical(res$haystack, c(1L, 1L, 1L))
})

test_that("NA and NaN are distinct if requested", {
  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "==", nan_distinct = TRUE)
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))

  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "==", multiple = "first", nan_distinct = TRUE)
  expect_identical(res$needles, c(1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 2L))
})

test_that("NA and NaN match each other in non-equi conditions by default", {
  res <- vec_locate_matches(c(NaN, NA, NaN, 1), c(NA, NaN, NA), condition = ">=", nan_distinct = FALSE)
  expect_identical(res$needles, c(rep(c(1L, 2L, 3L), each = 3), 4L))
  expect_identical(res$haystack, c(rep(c(1L, 2L, 3L), times = 3), NA))

  res <- vec_locate_matches(c(NaN, NA, NaN, 1), c(NA, NaN, NA), condition = "<=", nan_distinct = FALSE)
  expect_identical(res$needles, c(rep(c(1L, 2L, 3L), each = 3), 4L))
  expect_identical(res$haystack, c(rep(c(1L, 2L, 3L), times = 3), NA))
})

test_that("NA and NaN never match each other in non-equi conditions if treated as distinct", {
  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = ">=", nan_distinct = TRUE)
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))

  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "<=", nan_distinct = TRUE)
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))
})

test_that("NA and NaN are both considered incomplete no matter the value of `nan_distinct`", {
  res <- vec_locate_matches(c(NA, NaN), c(NA, NaN), incomplete = NA, nan_distinct = FALSE)
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))

  res <- vec_locate_matches(c(NA, NaN), c(NA, NaN), incomplete = NA, nan_distinct = TRUE)
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - complex

test_that("complex can be matched", {
  x <- complex(real = 1, imaginary = c(1, 2))
  y <- complex(real = 1, imaginary = c(1, 1, 3))
  z <- complex(real = 2, imaginary = 1)

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, z)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

test_that("complex order lexicographically", {
  x <- complex(real = 1, imaginary = c(1, 2, 5))
  y <- complex(real = 1, imaginary = c(1, 4, 3))

  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, 2L, 3L, NA))
})

test_that("complex incomplete values match correctly", {
  x <- complex(real = c(NA, NA, NaN, NaN), imaginary = c(NA, NaN, NA, NaN))
  y <- complex(real = c(NA, NA, NaN, NaN), imaginary = c(NA, NaN, NA, NaN))

  # Missings can match, and all missing values should be treated equally
  res <- vec_locate_matches(x, y, condition = "==", incomplete = "compare", nan_distinct = FALSE)
  expect_identical(res$needles, rep(1:4, each = 4))
  expect_identical(res$haystack, rep(1:4, times = 4))

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "match", nan_distinct = FALSE)
  expect_identical(res$needles, rep(1:4, each = 4))
  expect_identical(res$haystack, rep(1:4, times = 4))

  # Missings can match, but all combinations are different
  res <- vec_locate_matches(x, y, condition = "==", incomplete = "compare", nan_distinct = TRUE)
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, 1:4)

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "match", nan_distinct = TRUE)
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, 1:4)

  # Missings don't match
  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, rep(NA_integer_, 4))

  # Missings don't match, but are never considered no-matches
  expect_identical(
    vec_locate_matches(x, y, condition = "==", incomplete = NA, no_match = "error"),
    vec_locate_matches(x, y, condition = "==", incomplete = NA)
  )
})

test_that("complex missing values are always grouped together (#1403)", {
  # Unlike data frames and rcrd types, for complex vectors if either element
  # is missing then the whole observation is normalised to have both components
  # be missing. This means `1+NAi` matches `2+NAi`. It also matches `2+NaNi`
  # unless `nan_distinct = TRUE`.
  x <- complex(real = c(1, 1, 2, 2, 2), imaginary = c(NA, 1, NA, 2, NaN))
  y <- x[-1]

  res <- vec_locate_matches(x, y, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 2L, 4L, 1L, 3L, 2L, 4L))
})

test_that("behavior with complex missing values matches base R", {
  skip_if(getRversion() < "3.4.0", message = "`match()` is broken with complex missings")

  x <- complex(real = c(1, 1, 2, 2, 2), imaginary = c(NA, 1, NA, 2, NaN))

  expect_identical(
    vec_locate_matches(x, x, nan_distinct = TRUE, multiple = "first")$haystack,
    match(x, x)
  )
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - characters

test_that("character ordering is done in the C locale", {
  x <- c("a", "A")
  y <- c("a", "A", "b", "B")

  # a < b, but a > A and a > B
  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 2L, 2L, 2L))
  expect_identical(res$haystack, c(3L, 1L, 3L, 4L))
})

test_that("`chr_proxy_collate` can affect the matching process", {
  x <- c("a", "A")
  y <- c("a", "A")

  res <- vec_locate_matches(x, y, condition = "==")
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)

  res <- vec_locate_matches(x, y, condition = "==", chr_proxy_collate = tolower)
  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(1L, 2L, 1L, 2L))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - lists

test_that("lists can be matched", {
  x <- list(1, 2, 1, NULL)
  y <- list(1, 1, 3, NULL)

  res <- vec_locate_matches(x, y)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, 2L, NA, 1L, 2L, 4L))
})

test_that("list incompleteness is detected", {
  res <- vec_locate_matches(list(NULL), list(NULL), incomplete = NA)

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("list ordering is by first appearance in `needles` (so non-equi joins don't make much sense)", {
  x <- list(3, 2, 1, NULL)
  y <- list(1, 3, 1, 3)

  res <- vec_locate_matches(x, y, condition = ">")

  # x[1] appears first, so it isn't greater than anything
  # x[2] is greater than x[1] (when x[1] is in y)
  # and so on...
  # NULL still doesn't match anything
  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(NA, 2L, 4L, 2L, 4L, NA))

  # With data frame columns containing list-columns
  df1 <- data_frame(col = data_frame(x = x))
  df2 <- data_frame(col = data_frame(x = y))

  expect_identical(vec_locate_matches(x, y, condition = ">"), res)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - data frame

test_that("can match with 1 column data frames", {
  df1 <- data_frame(x = c(1L, 3L, 1L, 3L))
  df2 <- data_frame(x = c(1L, 3L, 1L))

  expect_identical(
    vec_locate_matches(df1, df2),
    vec_locate_matches(df1$x, df2$x)
  )
})

test_that("can match with >1 column data frames", {
  df1 <- data_frame(x = c(1L, 3L, 1L, 3L), y = c(1L, 4L, 1L, 2L))
  df2 <- data_frame(x = c(1L, 3L, 1L), y = c(1L, 2L, 1L))

  res <- vec_locate_matches(df1, df2, condition = c("==", "=="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, 3L, NA, 1L, 3L, 2L))
})

test_that("can match with df-cols of varying types", {
  y <- c(1L, 1L)

  expect_needles <- c(1L, 2L)
  expect_haystack <- c(NA, 1L)

  df1 <- data_frame(x = data_frame(x = c(2L, 1L), y = y))
  df2 <- data_frame(x = data_frame(x = c(1L, 3L), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c(2, 1), y = y))
  df2 <- data_frame(x = data_frame(x = c(1, 3), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c(TRUE, FALSE), y = y))
  df2 <- data_frame(x = data_frame(x = c(FALSE, NA), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c("x", "y"), y = y))
  df2 <- data_frame(x = data_frame(x = c("y", "z"), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = complex(real = c(1, 2), imaginary = c(2, 1)), y = y))
  df2 <- data_frame(x = data_frame(x = complex(real = c(2, 3), imaginary = c(1, 1)), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)
})

test_that("ensure that matching works if outer runs are present (i.e. `==` comes before non-equi condition)", {
  df1 <- data_frame(x = c(1, 2, 1, 1), y = c(2, 2, 3, 2))
  df2 <- data_frame(x = c(1, 1), y = c(2, 3))

  res <- vec_locate_matches(df1, df2, condition = c("==", "<="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(1L, 2L, NA, 2L, 1L, 2L))

  df1$z <- c(1L, 2L, 1L, 3L)
  df2$z <- c(5L, 2L)

  res <- vec_locate_matches(df1, df2, condition = c("==", "==", "<"))

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(1L, NA, 2L, 1L))

  res <- vec_locate_matches(df1, df2, condition = c("==", ">=", "<"))

  expect_identical(res$needles, c(1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, NA, 1L, 2L, 1L))
})

test_that("df-cols propagate an NA if any columns are incomplete", {
  df <- data_frame(x = 1, y = data_frame(x = c(1, 1, NA), y = c(1, NA, 2)))

  res <- vec_locate_matches(df, df, incomplete = "compare")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, 1:3)

  res <- vec_locate_matches(df, df, incomplete = "match")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, 1:3)

  # 2nd and 3rd rows aren't fully complete
  res <- vec_locate_matches(df, df, incomplete = NA)
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(1L, NA, NA))

  res <- vec_locate_matches(df, df, incomplete = "drop")
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)
})

test_that("df-cols aren't flattened, so `condition` is applied jointly on the df-col columns", {
  x <- data_frame(a = 1L, b = data_frame(x = 3L, y = 4L))
  y <- data_frame(a = 1L, b = data_frame(x = 2L, y = 5L))

  # In particular `x$b[1,] > y$b[1,]` because `3 > 4` and that breaks the tie
  # before any values of the `x$b$y` column are checked
  res <- vec_locate_matches(x, y, condition = c("==", ">"))
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)
})

test_that("must have at least 1 column to match", {
  expect_snapshot(error = TRUE, {
    vec_locate_matches(data_frame(), data_frame())
  })
  expect_snapshot(error = TRUE, {
    vec_locate_matches(data_frame(), data_frame(), error_call = call("foo"))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - rcrd

test_that("rcrd types can be matched", {
  x <- new_rcrd(list(x = c(1L, 3L), y = c(1L, 4L)))
  y <- new_rcrd(list(x = c(1L, 2L), y = c(1L, 5L)))

  res <- vec_locate_matches(x, y, condition = "<=")
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  # In particular: `(3, 4) > (2, 5)` since the first elt breaks the tie
  res <- vec_locate_matches(x, y, condition = ">")
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 1L, 2L))
})

test_that("rcrd type matching works with rcrd-cols", {
  x <- data_frame(a = c(1L, 1L), b = new_rcrd(list(x = c(1L, 3L), y = c(1L, 4L))))
  y <- data_frame(a = c(1L, 1L), b = new_rcrd(list(x = c(1L, 2L), y = c(1L, 5L))))

  res <- vec_locate_matches(x, y, condition = c("==", "<="))
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, y, condition = c("==", ">"))
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 1L, 2L))
})

test_that("rcrd type incompleteness is handled correctly", {
  x <- new_rcrd(list(x = c(1L, NA), y = c(NA_integer_, NA_integer_)))
  y <- new_rcrd(list(x = c(1L, 2L, NA), y = c(NA, 5L, NA)))

  # When `incomplete = "compare"`, the types of incompleteness still must
  # match exactly to have a match. i.e. (x=1L, y=NA) doesn't match (x=NA, y=1L).
  # This is the same as the rule for data frames.
  res <- vec_locate_matches(x, y, condition = "==", incomplete = "compare")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(1L, 3L))

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "match")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(1L, 3L))

  # If any field contains NA, the entire observation is incomplete.
  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - S3

test_that("S3 types with order proxies that depend on the data are combined before the proxy is taken", {
  # i.e. `bignum:::vec_proxy_order.bignum_biginteger()`

  x <- structure(c(5L, 1L), class = "foo")
  y <- structure(c(8L, 5L), class = "foo")

  local_methods(
    vec_proxy_order.foo = function(x, ...) {
      rank(unclass(x))
    }
  )

  # Can't take the order proxies separately because they are the same!
  expect_identical(vec_proxy_order(x), vec_proxy_order(y))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, NA))

  x_df <- data_frame(a = x, b = x)
  y_df <- data_frame(a = y, b = y)

  res <- vec_locate_matches(x_df, y_df)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, NA))
})

test_that("Works with base R S3 types we support natively", {
  x <- new_factor(c(1L, 2L), levels = c("x", "y"))
  y <- new_factor(c(3L, 1L, 1L), levels = c("x", "y", "z"))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- new_ordered(c(1L, 2L), levels = c("x", "y"))
  y <- new_ordered(c(2L, 1L, 1L), levels = c("x", "y"))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, 1L))

  x <- new_date(c(1, 2))
  y <- new_date(c(3, 1, 1))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- new_datetime(c(1, 2))
  y <- new_datetime(c(3, 1, 1))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- as.POSIXlt(new_datetime(c(1, 2)))
  y <- as.POSIXlt(new_datetime(c(3, 1, 1)))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

test_that("Works with classed data frame columns", {
  x_col <- new_data_frame(list(a = c(1L, 2L), b = c(2, 3)), class = "foo")
  y_col <- new_data_frame(list(a = c(1L, 1L, 1L), b = c(2, 4, 2)), class = "foo")

  x <- new_data_frame(list(c = c(1L, 1L), d = x_col))
  y <- new_data_frame(list(c = c(1L, 1L, 1L), d = y_col))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 3L, NA))
})

test_that("AsIs types are combined before order proxies are taken (#1557)", {
  x <- I(list(5, 1))
  y <- I(list(8, 5, 5))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - ptype2 / casting

test_that("common type of `needles` and `haystack` is taken", {
  x <- 1
  y <- "a"

  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y)
  })
  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y, needles_arg = "x", error_call = call("foo"))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - missing values

test_that("integer missing values match with `==`, `>=`, and `<=` when `incomplete = 'compare'", {
  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "==")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">=")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("integer missing values can match with any condition when `incomplete = 'match'`", {
  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "==", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">=", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("integer missing values report all matches even with a `filter`", {
  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=", filter = "min")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<", filter = "min", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">=", filter = "max")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">", filter = "max", incomplete = "match")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("integer missing value matches can be limited by `multiple`", {
  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=", multiple = "first")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=", multiple = "last")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 4L)

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=", multiple = "any")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 2L)
})

test_that("missing values match within columns", {
  df1 <- data_frame(x = c(1L, 2L, 1L), y = rep(NA_integer_, 3))
  df2 <- data_frame(x = c(2L, 1L, 1L), y = c(1L, NA, NA))

  res <- vec_locate_matches(df1, df2, condition = c("==", "=="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, 2L, 3L))

  expect_identical(
    vec_locate_matches(df1, df2, condition = c("<=", ">=")),
    vec_locate_matches(df1, df2, condition = c("==", "=="))
  )

  res <- vec_locate_matches(df1, df2, condition = c("<", ">"))

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(df1, df2, condition = c("<=", ">"), incomplete = "compare")

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(df1, df2, condition = c("<=", ">"), incomplete = "match")

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, 2L, 3L))
})

test_that("missing values being 'match'ed hands off correctly to next column", {
  df1 <- data_frame(x = c(NA, NA, 1L, 2L, NA), y = c(2, 3, 0, 1, NA))
  df2 <- data_frame(x = c(NA, NA, NA, 3L), y = c(2, 1, NA, 0))

  res <- vec_locate_matches(df1, df2, condition = c("<", ">"), incomplete = "match")

  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 4L, 5L))
  expect_identical(res$haystack, c(2L, 1L, 2L, NA, 4L, 3L))
})

test_that("integer needles can't match NAs in the haystack", {
  # At the C level, 1L > NA_integer_ (INT_MIN),
  # but we are careful to work around this
  res <- vec_locate_matches(1L, c(1L, NA, 2L, NA), condition = ">=")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(1L, c(1L, NA, 2L, NA), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("double needles can't match NAs or NaNs in the haystack", {
  # At the C level, our helpers assumg NA and NaN are the smallest values,
  # so we are careful to avoid including them with >= and > conditions
  res <- vec_locate_matches(1, c(1, NA, 2, NaN), condition = ">=")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(1, c(1, NA, 2, NaN), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("NA and NaN match correctly with non-equi conditions and `nan_distinct`", {
  res <- vec_locate_matches(c(NA, NaN), c(1L, NA, 2L, NaN), condition = "<=", nan_distinct = TRUE)

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(c(NA, NaN), c(1L, NA, 2L, NaN), condition = "<", nan_distinct = TRUE)

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))

  res <- vec_locate_matches(c(NA, NaN), c(1L, NA, 2L, NaN), condition = "<", nan_distinct = TRUE, incomplete = "match")

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(c(NA, NaN), c(1L, NA, 2L, NaN), condition = ">=", nan_distinct = FALSE)

  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(2L, 4L, 2L, 4L))

  res <- vec_locate_matches(c(NA, NaN), c(1L, NA, 2L, NaN), condition = ">", nan_distinct = FALSE, incomplete = "match")

  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(2L, 4L, 2L, 4L))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `incomplete`

test_that("can handle incomplete needles with `incomplete = <integer>`", {
  x <- c(1L, NA, 2L)
  y <- c(NA, 1L, 1L)

  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, NA))

  res <- vec_locate_matches(x, y, condition = "<=", incomplete = 0L)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, 0L, NA))

  res <- vec_locate_matches(x, y, condition = ">=", incomplete = -1L)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, -1L, 2L, 3L))
})

test_that("can drop incomplete needle rows with `incomplete = 'drop'", {
  x <- c(1L, NA, 2L)
  y <- c(NA, 1L, 1L)

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "drop")
  expect_identical(res$needles, c(1L, 1L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

test_that("if `incomplete = <integer>`, an NA in any column results in the value", {
  df1 <- data_frame(x = c(1L, NA, 2L, 1L, 1L), y = c(2L, 2L, NA, 1L, 1L))
  df2 <- data_frame(x = c(1L, 1L, 2L), y = c(1L, 1L, NA))

  res <- vec_locate_matches(df1, df2, condition = c("==", "=="), incomplete = NA)

  expect_identical(res$needles, c(1L, 2L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(NA, NA, NA, 1L, 2L, 1L, 2L))

  res <- vec_locate_matches(df1, df2, condition = c(">=", ">="), incomplete = NA)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(1L, 2L, NA, NA, 1L, 2L, 1L, 2L))
})

test_that("`incomplete = <integer> / 'drop'` still handles NAs in future columns when an earlier column has no matches", {
  df1 <- data_frame(x = c(1, 1, 2, 3), y = c(1, NA, NA, 4))
  df2 <- data_frame(x = c(1, 3), y = c(1, 5))

  # The 2 in row 3 of df1 has no match, but the NA in the 2nd column still propagates
  res <- vec_locate_matches(df1, df2, incomplete = NA, no_match = -1L)

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(1L, NA, NA, -1L))

  res <- vec_locate_matches(df1, df2, incomplete = "drop", no_match = -1L)

  expect_identical(res$needles, c(1L, 4L))
  expect_identical(res$haystack, c(1L, -1L))

  # The 1 in row 1 and 2 of df1 have no match, but the NA in row 2 of the 2nd column propagates
  res <- vec_locate_matches(df1, df2, incomplete = NA, no_match = -1L, condition = ">")

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(-1L, NA, NA, 1L))

  res <- vec_locate_matches(df1, df2, incomplete = "drop", no_match = -1L, condition = ">")

  expect_identical(res$needles, c(1L, 4L))
  expect_identical(res$haystack, c(-1L, 1L))
})

test_that("`incomplete` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(NA, 1, incomplete = "error")))
    (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo")))
    (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo", error_call = call("fn"))))
  })
})

test_that("`incomplete` error is classed", {
  expect_error(vec_locate_matches(NA, 1, incomplete = "error"), class = "vctrs_error_matches_incomplete")
})

test_that("`incomplete` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, incomplete = 1.5)))
    (expect_error(vec_locate_matches(1, 2, incomplete = c("match", "drop"))))
    (expect_error(vec_locate_matches(1, 2, incomplete = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(1, 2, incomplete = "x", error_call = call("fn"))))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `condition`

test_that("multiple matches from a non-equi condition are returned in first appearance order", {
  res <- vec_locate_matches(0L, c(1L, 0L, -1L, 0L), condition = "<=")

  expect_identical(res$needles, rep(1L, 3))
  expect_identical(res$haystack, c(1L, 2L, 4L))

  # Checking equi for good measure
  res <- vec_locate_matches(0L, c(1L, 0L, -1L, 0L), condition = "==")

  expect_identical(res$needles, rep(1L, 2))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("multiple matches from a non-equi condition are returned in first appearance order when the matches are in different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 2:1, y = 1:2)

  res <- vec_locate_matches(df, df2, condition = c("<=", "<="))

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))
})

test_that("`condition` is validated", {
  expect_error(vec_locate_matches(1, 2, condition = 1), "`condition` must be a character vector")
  expect_error(vec_locate_matches(1, 2, condition = "x"), 'must only contain "==", ">", ">=", "<", or "<="')
  expect_error(vec_locate_matches(1, 2, condition = c("==", "==")), "must be length 1, or the same length as the number of columns of the input")
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `multiple`

test_that("can get all matches", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "all")

  expect_identical(x$needles, c(1L, 1L, 2L, 2L))
  expect_identical(x$haystack, c(1L, 3L, 2L, 4L))
})

test_that("can get first match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "first")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

test_that("can get last match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "last")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 3:4)
})

test_that("can get any match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "any")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

test_that("duplicate needles match the same haystack locations", {
  x <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")

  expect_identical(x$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(x$haystack, c(1L, 3L, 2L, 1L, 3L, 2L))
})

test_that("correctly gets all matches when they come from different nesting containers", {
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )
  haystack <- data_frame(
    a = c(6, 5),
    b = c(6, 7)
  )

  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "all"),
    data_frame(needles = c(1L, 1L, 2L), haystack = c(1L, 2L, NA))
  )
})

test_that("correctly gets first/last/any match when they come from different nesting containers", {
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )
  haystack <- data_frame(
    a = c(6, 5, 0),
    b = c(6, 7, 1)
  )

  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "first"),
    data_frame(needles = c(1L, 2L), haystack = c(1L, NA))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "last"),
    data_frame(needles = c(1L, 2L), haystack = c(2L, NA))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "any"),
    data_frame(needles = c(1L, 2L), haystack = c(2L, NA))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "first", remaining = NA_integer_),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(1L, NA, 2L, 3L))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "last", remaining = NA_integer_),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(2L, NA, 1L, 3L))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "any", remaining = NA_integer_),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(2L, NA, 1L, 3L))
  )
})

test_that("`multiple` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, multiple = 1.5)))
    (expect_error(vec_locate_matches(1, 2, multiple = c("first", "last"))))
    (expect_error(vec_locate_matches(1, 2, multiple = "x")))
    # Uses internal error
    (expect_error(vec_locate_matches(1, 2, multiple = "x", error_call = call("fn"))))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `multiple` (deprecated)

test_that("`multiple` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo")))
    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", error_call = call("fn"))))
    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error", needles_arg = "foo", haystack_arg = "bar")))
  })
})

test_that("`multiple` can warn informatively", {
  expect_snapshot({
    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo")))
    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", error_call = call("fn"))))
    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning", needles_arg = "foo", haystack_arg = "bar")))
  })
})

test_that("warning falls back to 'all'", {
  expect_warning(
    result <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "warning"),
    class = "vctrs_warning_matches_multiple"
  )

  expect_identical(
    result,
    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
  )
})

test_that("errors on multiple matches that come from different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 1:2, y = 2:1)

  expect_snapshot(error = TRUE, {
    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error")
  })
})

test_that("errors when a match from a different nesting container is processed early on", {
  # Row 1 has 2 matches
  # Row 2 has 0 matches
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )

  # Rows 1 and 2 end up in different nesting containers
  haystack <- data_frame(
    a = c(5, 6),
    b = c(7, 6)
  )

  # needles[1,] records the haystack[1,] match first, which is in the 1st
  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
  # multiple matches)
  expect_snapshot(error = TRUE, {
    vec_locate_matches(needles, haystack, condition = "<", multiple = "error")
  })
})

test_that("`multiple = 'error'` doesn't error errneously on the last observation", {
  expect_error(res <- vec_locate_matches(1:2, 1:2, multiple = "error"), NA)
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)
})

test_that("`multiple = 'error' / 'warning'` throw correctly when combined with `relationship`", {
  x <- c(1, 2, 2)
  y <- c(2, 1, 2)

  # `multiple` error technically fires first
  expect_snapshot({
    (expect_error(vec_locate_matches(x, y, relationship = "one-to-one", multiple = "error")))
  })

  # Works when warning is also requested
  expect_snapshot({
    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
  })
  # Both warnings are thrown if applicable
  expect_snapshot({
    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
  })
  # Both warning and error are thrown if applicable
  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y, relationship = "one-to-one", multiple = "warning")
  })

  x <- c(1, 2)
  y <- c(2, 1, 2)

  expect_snapshot({
    (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "error")))
  })
  # Only `multiple` warning is applicable here
  expect_snapshot({
    vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `relationship`

test_that("`relationship` handles one-to-one case", {
  # No error
  expect_identical(
    vec_locate_matches(1:2, 2:1, relationship = "one-to-one"),
    vec_locate_matches(1:2, 2:1)
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "one-to-one"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(c(2, 1), c(1, 1), relationship = "one-to-one")))
    (expect_error(vec_locate_matches(c(1, 1), c(1, 2), relationship = "one-to-one")))
  })
})

test_that("`relationship` handles one-to-many case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "one-to-many"),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "one-to-many"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(c(1, 2, 2), c(2, 1), relationship = "one-to-many")))
  })
})

test_that("`relationship` handles many-to-one case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-one"),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "many-to-one"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-one")))
  })
})

test_that("`relationship` handles many-to-many case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # No error
  expect_identical(
    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # No error
  expect_identical(
    vec_locate_matches(c(1, 1, 2), c(1, 2, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "many-to-many"),
    vec_locate_matches(1:2, 3:4)
  )
})

test_that("`relationship` handles warn-many-to-many case", {
  # No warning
  expect_identical(
    expect_silent(
      vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "warn-many-to-many")
    ),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # No warning
  expect_identical(
    expect_silent(
      vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "warn-many-to-many")
    ),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    expect_silent(
      vec_locate_matches(1:2, 3:4, relationship = "warn-many-to-many")
    ),
    vec_locate_matches(1:2, 3:4)
  )

  # Specifically designed to ensure we test both:
  # - Finding multiple `needles` matches before multiple `haystack` matches
  # - Finding multiple `haystack` matches before multiple `needles` matches
  expect_snapshot({
    (expect_warning(vec_locate_matches(c(1, 2, 1), c(1, 2, 2), relationship = "warn-many-to-many")))
    (expect_warning(vec_locate_matches(c(1, 1, 2), c(2, 2, 1), relationship = "warn-many-to-many")))
  })
})

test_that("`relationship` considers `incomplete` matches as possible multiple matches", {
  x <- c(1, NA, NaN)
  y <- c(NA, 1)

  expect_snapshot({
    (expect_error(vec_locate_matches(x, y, relationship = "one-to-many")))
  })

  # No error
  expect_identical(
    vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA),
    vec_locate_matches(x, y, incomplete = NA)
  )

  # No error
  expect_identical(
    vec_locate_matches(x, y, relationship = "one-to-many", nan_distinct = TRUE),
    vec_locate_matches(x, y, nan_distinct = TRUE)
  )
})

test_that("`relationship` errors on multiple matches that come from different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 1:2, y = 2:1)

  expect_snapshot({
    (expect_error(vec_locate_matches(df, df2, condition = c("<=", "<="), relationship = "many-to-one")))
  })
})

test_that("`relationship` errors when a match from a different nesting container is processed early on", {
  # Row 1 has 2 matches
  # Row 2 has 0 matches
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )

  # Rows 1 and 2 end up in different nesting containers
  haystack <- data_frame(
    a = c(5, 6),
    b = c(7, 6)
  )

  # needles[1,] records the haystack[1,] match first, which is in the 1st
  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
  # multiple matches)
  expect_snapshot({
    (expect_error(vec_locate_matches(needles, haystack, condition = "<", relationship = "many-to-one")))
  })
})

test_that("`relationship` doesn't error errneously on the last observation", {
  expect_error(res <- vec_locate_matches(1:2, 1:2, relationship = "many-to-one"), NA)
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)
})

test_that("`relationship` doesn't error if `multiple` removes multiple matches", {
  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "any", relationship = "one-to-one")
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(1L, NA))

  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "first", relationship = "one-to-one")
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(1L, NA))

  out <- vec_locate_matches(c(1, 2), c(1, 1), multiple = "last", relationship = "one-to-one")
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(2L, NA))
})

test_that("`relationship` can still detect problematic `haystack` relationships when `multiple = first/last` are used", {
  expect_snapshot({
    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-one")))
    (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first", relationship = "one-to-many")))
  })
})

test_that("`relationship` and `remaining` work properly together", {
  expect_snapshot({
    out <- vec_locate_matches(
      c(1, 2, 2),
      c(2, 3, 1, 1, 4),
      relationship = "warn-many-to-many",
      remaining = NA_integer_
    )
  })
  expect_identical(out$needles, c(1L, 1L, 2L, 3L, NA, NA))
  expect_identical(out$haystack, c(3L, 4L, 1L, 1L, 2L, 5L))
})

test_that("`relationship` errors if `condition` creates multiple matches", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, c(1, 2), condition = "<=", relationship = "many-to-one")))
  })
})

test_that("`relationship` doesn't error if `filter` removes multiple matches", {
  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "min", relationship = "many-to-one")
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 1L)

  out <- vec_locate_matches(1, c(1, 2), condition = "<=", filter = "max", relationship = "many-to-one")
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)
})

test_that("`relationship` still errors if `filter` hasn't removed all multiple matches", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, c(1, 2, 1), condition = "<=", filter = "min", relationship = "many-to-one")))
  })

  # But not here
  out <- vec_locate_matches(c(1, 1), c(1, 2, 1), condition = "<=", filter = "max", relationship = "many-to-one")
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(2L, 2L))
})

test_that("`relationship` errors when we have >1 size 1 matches across containers (tidyverse/dplyr#6835)", {
  # Carefully designed to ensure we get 2 nested containment groups that split
  # up the rows of `y`, but each of the nested containment groups contain exactly
  # 1 match, so `size_match` in `expand_compact_indices()` won't ever be >1
  x <- data_frame(a = 1L, b = 5L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))

  expect_snapshot(error = TRUE, {
    vec_locate_matches(
      x,
      y,
      condition = c("<=", ">="),
      filter = c("none", "none"),
      relationship = "one-to-one"
    )
  })
})

test_that("`relationship` doesn't error when the first match from a different container gets filtered out (tidyverse/dplyr#6835)", {
  # Carefully designed to ensure we get 2 nested containment groups that split
  # up the rows of `y`. Row 1 (processed first) doesn't hold the minimum `b`
  # value, so it gets filtered out. Row 2 is in the "extra" matches section
  # but is actually the first (and only) real match, so we don't want to error
  # on it.
  x <- data_frame(a = 1L, b = 5L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))

  out <- vec_locate_matches(
    x,
    y,
    condition = c("<=", ">="),
    filter = c("none", "min"),
    relationship = "one-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)

  # Similar to the above example, but with a `max` filter. Row 1 doesn't hold
  # the max `c` value so it is filtered out even though it is a `>=` match.
  x <- data_frame(a = 1L, b = 5L, c = 3L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L), c = c(1L, 2L))

  out <- vec_locate_matches(
    x,
    y,
    condition = c("<=", ">=", ">="),
    filter = c("none", "none", "max"),
    relationship = "one-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)
})

test_that("`relationship` errors respect argument tags and error call", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "many-to-one", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
  })
})

test_that("`relationship` warnings respect argument tags and error call", {
  expect_snapshot({
    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", needles_arg = "foo", error_call = call("fn"))))
    (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many", haystack_arg = "bar", error_call = call("fn"))))
  })
})

test_that("`relationship` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, relationship = 1.5)))
    (expect_error(vec_locate_matches(1, 2, relationship = c("one-to-one", "one-to-many"))))
    (expect_error(vec_locate_matches(1, 2, relationship = "x")))
    # Uses internal error
    (expect_error(vec_locate_matches(1, 2, relationship = "x", error_call = call("fn"))))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `no_match`

test_that("can control `no_match`", {
  x <- vec_locate_matches(1:3, 1L)
  expect_identical(x$haystack, c(1L, NA, NA))

  x <- vec_locate_matches(1:3, 1L, no_match = 0L)
  expect_identical(x$haystack, c(1L, 0L, 0L))
})

test_that("can drop unmatched needles", {
  x <- vec_locate_matches(1:3, 2L, no_match = "drop")
  expect_identical(x$needles, 2L)
  expect_identical(x$haystack, 1L)
})

test_that("can drop unmatched missings when `incomplete = 'match'`", {
  x <- vec_locate_matches(c(NaN, 2, NA), 2, no_match = "drop")
  expect_identical(x$needles, 2L)
  expect_identical(x$haystack, 1L)

  x <- vec_locate_matches(c(NaN, 2, NA), NA, no_match = "drop", nan_distinct = FALSE)
  expect_identical(x$needles, c(1L, 3L))
  expect_identical(x$haystack, c(1L, 1L))

  x <- vec_locate_matches(c(NaN, 2, NA), NA, no_match = "drop", nan_distinct = TRUE)
  expect_identical(x$needles, 3L)
  expect_identical(x$haystack, 1L)
})

test_that("can differentiate between `no_match` and `incomplete`", {
  res <- vec_locate_matches(c(1, NA), 2, incomplete = NA, no_match = -1L)

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(-1L, NA))
})

test_that("`no_match` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, no_match = "error")))
    (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo")))
    (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo", error_call = call("fn"))))
    (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo", haystack_arg = "bar")))
  })
})

test_that("`no_match = 'error'` doesn't error on handled incomplete values", {
  res <- vec_locate_matches(c(NA, NaN, NA, 1), c(NA, 1), incomplete = NA, no_match = "error")

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(rep(NA, 3), 2L))
})

test_that("`no_match = 'drop'` doesn't drop handled incomplete values", {
  res <- vec_locate_matches(c(NA, NaN, NA, 1), c(NA, 1), incomplete = NA, no_match = "drop")

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(rep(NA, 3), 2L))
})

test_that("errors with the right location on unmatched needles when different nesting containers are present", {
  df <- data_frame(x = 2:1, y = 2:1)
  df2 <- data_frame(x = 1:2, y = 2:1)

  # i.e. should be location 2
  expect_snapshot(
    (expect_error(vec_locate_matches(df, df2, condition = ">=", no_match = "error")))
  )
})

test_that("`no_match` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, no_match = 1.5)))
    (expect_error(vec_locate_matches(1, 2, no_match = c(1L, 2L))))
    (expect_error(vec_locate_matches(1, 2, no_match = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(1, 2, no_match = "x", error_call = call("fn"))))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `remaining`

test_that("`remaining` can retain `haystack` values that `needles` didn't match", {
  res <- vec_locate_matches(1, 0:2, remaining = NA)
  expect_identical(res$needles, c(1L, NA, NA))
  expect_identical(res$haystack, c(2L, 1L, 3L))

  res <- vec_locate_matches(1, 0:2, remaining = NA, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))

  res <- vec_locate_matches(1, 0:2, remaining = NA, condition = "<")
  expect_identical(res$needles, c(1L, NA, NA))
  expect_identical(res$haystack, c(3L, 1L, 2L))
})

test_that("`incomplete` affects `needles` but not `haystack`", {
  # Matches NA to NA, so nothing remaining
  res <- vec_locate_matches(c(1, NA), c(NA, 1), incomplete = "compare", remaining = NA)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 1L))

  # Matches NA to NA, so nothing remaining
  res <- vec_locate_matches(c(1, NA), c(NA, 1), incomplete = "match", remaining = NA)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 1L))

  # Doesn't match NA to NA, so `haystack` is left with remaining values
  res <- vec_locate_matches(c(1, NA), c(NA, 1), condition = "<", incomplete = "compare", remaining = NA)
  expect_identical(res$needles, c(1L, 2L, NA, NA))
  expect_identical(res$haystack, c(NA, NA, 1L, 2L))

  # Matches NA to NA, so only remaining value is for `1`
  res <- vec_locate_matches(c(1, NA), c(NA, 1), condition = "<", incomplete = "match", remaining = NA)
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(NA, 1L, 2L))

  # `needles` NA value is propagated, so `haystack` is left with a remaining value
  res <- vec_locate_matches(c(1, NA), c(NA, 1), incomplete = NA, remaining = NA)
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(2L, NA, 1L))

  # `needles` NA value is dropped, so `haystack` is left with a remaining value
  res <- vec_locate_matches(c(1, NA), c(NA, 1), incomplete = "drop", remaining = NA)
  expect_identical(res$needles, c(1L, NA))
  expect_identical(res$haystack, c(2L, 1L))
})

test_that("`remaining` combined with `multiple = 'first/last'` treats non-first/last matches as remaining", {
  x <- c(1, 2)
  y <- c(1, 2, 2)

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "first")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "last")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 3L, 2L))

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "any")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))
})

test_that("`remaining` combined with the haystack reordering retains appearance order", {
  x <- data_frame(a = 1, b = 4)
  y <- data_frame(a = c(2, 1, 0), b = c(2, 1, 0))

  # Appearance order for the haystack locations
  res <- vec_locate_matches(x, y, condition = c("<=", ">="))
  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))

  # Retain that appearance order of the matches, with remaining values appended
  res <- vec_locate_matches(x, y, condition = c("<=", ">="), remaining = NA)
  expect_identical(res$needles, c(1L, 1L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))
})

test_that("`remaining` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, remaining = "error")))
    (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo")))
    (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo", error_call = call("fn"))))
    (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo", haystack_arg = "bar")))
  })
})

test_that("`remaining` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, remaining = 1.5)))
    (expect_error(vec_locate_matches(1, 2, remaining = c(1L, 2L))))
    (expect_error(vec_locate_matches(1, 2, remaining = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(1, 2, remaining = "x", error_call = call("fn"))))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - filter

test_that("simple `filter`s work", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 3, 0)

  res <- vec_locate_matches(needles, haystack, condition = "<", filter = "max")
  expect_identical(res$haystack, c(3L, 3L, NA))

  res <- vec_locate_matches(needles, haystack, condition = "<", filter = "min")
  expect_identical(res$haystack, c(1L, 3L, NA))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$haystack, c(2L, 1L, 3L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$haystack, c(4L, 4L, 4L))
})

test_that("haystack duplicates are preserved", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 2, 3, 0, 1, 0)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 6L, 1L, 3L, 4L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(5L, 7L, 5L, 7L, 5L, 7L))
})

test_that("haystack duplicates can be controlled by `multiple`", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 2, 3, 0, 1, 0)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", multiple = "first")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(2L, 1L, 4L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", multiple = "last")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(6L, 3L, 4L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", multiple = "any")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(2L, 1L, 4L))
})

test_that("`filter` works when valid matches are in different nesting containers", {
  needles <- data_frame(x = 0L, y = 1L, z = 2L)
  haystack <- data_frame(x = c(1L, 2L, 1L, 0L), y = c(2L, 1L, 2L, 3L), z = c(3L, 3L, 2L, 2L))

  info <- compute_nesting_container_info(haystack, c("<=", "<=", "<="))
  haystack_order <- info[[1]]
  container_ids <- info[[2]]

  # Rows 1 and 2 of haystack are in different nesting containers, but
  # both have the "max" filter value of `z=3` so both should be in the result.
  # Row 4 is in its own container, so it will be considered the "max"
  # of its group, but it is less than rows 1 and 2 so it will ultimately be
  # filtered out.
  expect_identical(container_ids, c(1L, 2L, 1L, 0L))
  expect_identical(haystack_order, c(4L, 3L, 1L, 2L))

  res <- vec_locate_matches(needles, haystack, condition = c("<=", "<=", "<="), filter = c("none", "none", "max"))
  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))

  res <- vec_locate_matches(needles, haystack, condition = c("<=", "<=", "<="), filter = c("none", "none", "max"), multiple = "first")
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(needles, haystack, condition = c("<=", "<=", "<="), filter = c("none", "none", "max"), multiple = "last")
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(needles, haystack, condition = c("<=", "<=", "<="), filter = c("none", "none", "max"), multiple = "any")
  expect_identical(res$haystack, 1L)
})

test_that("single filter is applied to all columns", {
  needles <- data_frame(x = 5L, y = 8L, z = 4L)
  haystack <- data_frame(x = c(1L, 3L, 2L, 2L), y = c(1L, 3L, 2L, 3L), z = c(1L, 2L, 3L, 3L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$haystack, 1L)
})

test_that("different `filter`s can be used per column", {
  needles <- data_frame(x = c(0, 2, 1, 1), y = c(2, 0, 0, 4))
  haystack <- data_frame(x = c(2, 2, 2, 1, 1), y = c(1, 1, 2, 2, 1))

  res <- vec_locate_matches(needles, haystack, condition = c(">=", "<"), filter = c("max", "min"))
  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 4L))
  expect_identical(res$haystack, c(NA, 1L, 2L, 5L, NA))
})

test_that("`filter` works with incomplete values", {
  needles <- c(1, NA, 4, NA)
  haystack <- c(NA, 1, NA, 1, 3)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", incomplete = "compare")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 3L, 5L, 1L, 3L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", incomplete = "compare", multiple = "first")
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(2L, 1L, 5L, 1L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", incomplete = "compare", multiple = "any")
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(2L, 1L, 5L, 1L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max", incomplete = NA)
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L))
  expect_identical(res$haystack, c(2L, 4L, NA, 5L, NA))
})

test_that("`filter` works with mixed NA and NaN", {
  needles <- c(1, NA, 4, NaN)
  haystack <- c(NA, 1, NaN, 1, 3)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min", incomplete = "compare", nan_distinct = FALSE)
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 3L, 2L, 4L, 1L, 3L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min", incomplete = "compare", nan_distinct = TRUE)
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 2L, 4L, 3L))
})

test_that("`filter` is validated", {
  expect_error(vec_locate_matches(1, 2, filter = 1.5), "character vector")
  expect_error(vec_locate_matches(1, 2, filter = "x"), 'one of "none", "min", or "max"')
  expect_error(vec_locate_matches(1, 2, filter = c("min", "max")), "length 1, or the same length as")
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - edge cases

test_that("zero row `needles` results in zero row data frame output", {
  res <- vec_locate_matches(integer(), 1:3)

  expect_identical(res$needles, integer())
  expect_identical(res$haystack, integer())

  res <- vec_locate_matches(integer(), 1:3, condition = "<")

  expect_identical(res$needles, integer())
  expect_identical(res$haystack, integer())
})

test_that("zero row `haystack` results in no-matches for all needles", {
  res <- vec_locate_matches(1:3, integer())

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(1:3, integer(), condition = "<")

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))
})

test_that("zero row `haystack` still allows needle incomplete handling", {
  res <- vec_locate_matches(c(1, NA), integer(), incomplete = NA, no_match = 0L)

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(0L, NA))

  res <- vec_locate_matches(c(1, NA), integer(), incomplete = NA, no_match = 0L, condition = "<")

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(0L, NA))
})

test_that("zero column data frames are not allowed", {
  expect_error(
    vec_locate_matches(data_frame(.size = 2L), data_frame(.size = 2L)),
    "at least 1 column"
  )
})

test_that("zero column input still checks `condition` correctness", {
  x <- data_frame(.size = 2)
  y <- data_frame(.size = 3)

  expect_error(
    vec_locate_matches(x, y, condition = c("==", "<=")),
    "length 1, or the same length as the number of columns"
  )
})

test_that("`multiple = 'first'/'last'` returns the first/last by appearance", {
  x <- c(1, 2, 3)
  y <- c(2, 1, 0)

  res <- vec_locate_matches(x, y, condition = ">=", multiple = "first")
  expect_identical(res$haystack, c(2L, 1L, 1L))

  res <- vec_locate_matches(x, y, condition = ">=", multiple = "last")
  expect_identical(res$haystack, c(3L, 3L, 3L))
})

test_that("NA adjustment of `>` and `>=` conditions is protected from empty haystack", {
  res <- vec_locate_matches(1L, integer(), condition = ">")
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("potential overflow on large output size is caught informatively", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  expect_snapshot({
    (expect_error(vec_locate_matches(1:1e7, 1:1e7, condition = ">=")))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - logicals

test_that("isn't confused by unspecified logical vectors", {
  x <- vec_locate_matches(logical(), NA)
  expect_identical(x$needles, integer())
  expect_identical(x$haystack, integer())

  x <- vec_locate_matches(NA, logical())
  expect_identical(x$needles, 1L)
  expect_identical(x$haystack, NA_integer_)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - integers

test_that("can match in increasing order", {
  x <- vec_locate_matches(1:2, 1:3)
  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - doubles

test_that("can match doubles", {
  x <- vec_locate_matches(c(1, 2, 5), c(2, 2, 3, 1))

  expect_identical(x$needles, c(1L, 2L, 2L, 3L))
  expect_identical(x$haystack, c(4L, 1L, 2L, NA))
})

test_that("can match Inf and -Inf with all conditions", {
  x <- c(Inf, -Inf)
  y <- c(-Inf, 0, Inf)

  res <- vec_locate_matches(x, y, condition = "==")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(3L, 1L))

  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 2L, 3L))

  res <- vec_locate_matches(x, y, condition = "<=")
  expect_identical(res$needles, c(1L, 2L, 2L, 2L))
  expect_identical(res$haystack, c(3L, 1L, 2L, 3L))

  res <- vec_locate_matches(x, y, condition = ">")
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, y, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, 3L, 1L))
})

test_that("NA and NaN don't match numbers with equality conditions", {
  expect_identical(vec_locate_matches(1, NA_real_)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(1, NaN)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(NA_real_, 1)$haystack, NA_integer_)
  expect_identical(vec_locate_matches(NaN, 1)$haystack, NA_integer_)
})

test_that("NA and NaN are the same by default", {
  res <- vec_locate_matches(NA_real_, NaN)
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(NaN, NA_real_)
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(c(NaN, NA, NaN), c(NA, NaN, NA), condition = "==")
  expect_identical(res$needles, rep(c(1L, 2L, 3L), each = 3))
  expect_identical(res$haystack, rep(c(1L, 2L, 3L), times = 3))

  res <- vec_locate_matches(
    c(NaN, NA, NaN),
    c(NA, NaN, NA),
    condition = "==",
    multiple = "first"
  )
  expect_identical(res$needles, c(1L, 2L, 3L))
  expect_identical(res$haystack, c(1L, 1L, 1L))
})

test_that("NA and NaN are distinct if requested", {
  res <- vec_locate_matches(
    c(NaN, NA, NaN),
    c(NA, NaN, NA),
    condition = "==",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))

  res <- vec_locate_matches(
    c(NaN, NA, NaN),
    c(NA, NaN, NA),
    condition = "==",
    multiple = "first",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, c(1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 2L))
})

test_that("NA and NaN match each other in non-equi conditions by default", {
  res <- vec_locate_matches(
    c(NaN, NA, NaN, 1),
    c(NA, NaN, NA),
    condition = ">=",
    nan_distinct = FALSE
  )
  expect_identical(res$needles, c(rep(c(1L, 2L, 3L), each = 3), 4L))
  expect_identical(res$haystack, c(rep(c(1L, 2L, 3L), times = 3), NA))

  res <- vec_locate_matches(
    c(NaN, NA, NaN, 1),
    c(NA, NaN, NA),
    condition = "<=",
    nan_distinct = FALSE
  )
  expect_identical(res$needles, c(rep(c(1L, 2L, 3L), each = 3), 4L))
  expect_identical(res$haystack, c(rep(c(1L, 2L, 3L), times = 3), NA))
})

test_that("NA and NaN never match each other in non-equi conditions if treated as distinct", {
  res <- vec_locate_matches(
    c(NaN, NA, NaN),
    c(NA, NaN, NA),
    condition = ">=",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))

  res <- vec_locate_matches(
    c(NaN, NA, NaN),
    c(NA, NaN, NA),
    condition = "<=",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, c(1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 1L, 3L, 2L))
})

test_that("NA and NaN are both considered incomplete no matter the value of `nan_distinct`", {
  res <- vec_locate_matches(
    c(NA, NaN),
    c(NA, NaN),
    incomplete = NA,
    nan_distinct = FALSE
  )
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))

  res <- vec_locate_matches(
    c(NA, NaN),
    c(NA, NaN),
    incomplete = NA,
    nan_distinct = TRUE
  )
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - complex

test_that("complex can be matched", {
  x <- complex(real = 1, imaginary = c(1, 2))
  y <- complex(real = 1, imaginary = c(1, 1, 3))
  z <- complex(real = 2, imaginary = 1)

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, z)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

test_that("complex order lexicographically", {
  x <- complex(real = 1, imaginary = c(1, 2, 5))
  y <- complex(real = 1, imaginary = c(1, 4, 3))

  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, 2L, 3L, NA))
})

test_that("complex incomplete values match correctly", {
  x <- complex(real = c(NA, NA, NaN, NaN), imaginary = c(NA, NaN, NA, NaN))
  y <- complex(real = c(NA, NA, NaN, NaN), imaginary = c(NA, NaN, NA, NaN))

  # Missings can match, and all missing values should be treated equally
  res <- vec_locate_matches(
    x,
    y,
    condition = "==",
    incomplete = "compare",
    nan_distinct = FALSE
  )
  expect_identical(res$needles, rep(1:4, each = 4))
  expect_identical(res$haystack, rep(1:4, times = 4))

  res <- vec_locate_matches(
    x,
    y,
    condition = "==",
    incomplete = "match",
    nan_distinct = FALSE
  )
  expect_identical(res$needles, rep(1:4, each = 4))
  expect_identical(res$haystack, rep(1:4, times = 4))

  # Missings can match, but all combinations are different
  res <- vec_locate_matches(
    x,
    y,
    condition = "==",
    incomplete = "compare",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, 1:4)

  res <- vec_locate_matches(
    x,
    y,
    condition = "==",
    incomplete = "match",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, 1:4)

  # Missings don't match
  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, rep(NA_integer_, 4))

  # Missings don't match, but are never considered no-matches
  expect_identical(
    vec_locate_matches(
      x,
      y,
      condition = "==",
      incomplete = NA,
      no_match = "error"
    ),
    vec_locate_matches(x, y, condition = "==", incomplete = NA)
  )
})

test_that("complex missing values are always grouped together (#1403)", {
  # Unlike data frames and rcrd types, for complex vectors if either element
  # is missing then the whole observation is normalised to have both components
  # be missing. This means `1+NAi` matches `2+NAi`. It also matches `2+NaNi`
  # unless `nan_distinct = TRUE`.
  x <- complex(real = c(1, 1, 2, 2, 2), imaginary = c(NA, 1, NA, 2, NaN))
  y <- x[-1]

  res <- vec_locate_matches(x, y, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 2L, 4L, 1L, 3L, 2L, 4L))
})

test_that("behavior with complex missing values matches base R", {
  x <- complex(real = c(1, 1, 2, 2, 2), imaginary = c(NA, 1, NA, 2, NaN))

  expect_identical(
    vec_locate_matches(x, x, nan_distinct = TRUE, multiple = "first")$haystack,
    match(x, x)
  )
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - characters

test_that("character ordering is done in the C locale", {
  x <- c("a", "A")
  y <- c("a", "A", "b", "B")

  # a < b, but a > A and a > B
  res <- vec_locate_matches(x, y, condition = "<")
  expect_identical(res$needles, c(1L, 2L, 2L, 2L))
  expect_identical(res$haystack, c(3L, 1L, 3L, 4L))
})

test_that("`chr_proxy_collate` can affect the matching process", {
  x <- c("a", "A")
  y <- c("a", "A")

  res <- vec_locate_matches(x, y, condition = "==")
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)

  res <- vec_locate_matches(x, y, condition = "==", chr_proxy_collate = tolower)
  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(1L, 2L, 1L, 2L))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - lists

test_that("lists can be matched", {
  x <- list(1, 2, 1, NULL)
  y <- list(1, 1, 3, NULL)

  res <- vec_locate_matches(x, y)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, 2L, NA, 1L, 2L, 4L))
})

test_that("list incompleteness is detected", {
  res <- vec_locate_matches(list(NULL), list(NULL), incomplete = NA)

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("list ordering is by first appearance in `needles` (so non-equi joins don't make much sense)", {
  x <- list(3, 2, 1, NULL)
  y <- list(1, 3, 1, 3)

  res <- vec_locate_matches(x, y, condition = ">")

  # x[1] appears first, so it isn't greater than anything
  # x[2] is greater than x[1] (when x[1] is in y)
  # and so on...
  # NULL still doesn't match anything
  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(NA, 2L, 4L, 2L, 4L, NA))

  # With data frame columns containing list-columns
  df1 <- data_frame(col = data_frame(x = x))
  df2 <- data_frame(col = data_frame(x = y))

  expect_identical(vec_locate_matches(x, y, condition = ">"), res)
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - data frame

test_that("can match with 1 column data frames", {
  df1 <- data_frame(x = c(1L, 3L, 1L, 3L))
  df2 <- data_frame(x = c(1L, 3L, 1L))

  expect_identical(
    vec_locate_matches(df1, df2),
    vec_locate_matches(df1$x, df2$x)
  )
})

test_that("can match with >1 column data frames", {
  df1 <- data_frame(x = c(1L, 3L, 1L, 3L), y = c(1L, 4L, 1L, 2L))
  df2 <- data_frame(x = c(1L, 3L, 1L), y = c(1L, 2L, 1L))

  res <- vec_locate_matches(df1, df2, condition = c("==", "=="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, 3L, NA, 1L, 3L, 2L))
})

test_that("can match with df-cols of varying types", {
  y <- c(1L, 1L)

  expect_needles <- c(1L, 2L)
  expect_haystack <- c(NA, 1L)

  df1 <- data_frame(x = data_frame(x = c(2L, 1L), y = y))
  df2 <- data_frame(x = data_frame(x = c(1L, 3L), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c(2, 1), y = y))
  df2 <- data_frame(x = data_frame(x = c(1, 3), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c(TRUE, FALSE), y = y))
  df2 <- data_frame(x = data_frame(x = c(FALSE, NA), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(x = data_frame(x = c("x", "y"), y = y))
  df2 <- data_frame(x = data_frame(x = c("y", "z"), y = y))

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)

  df1 <- data_frame(
    x = data_frame(x = complex(real = c(1, 2), imaginary = c(2, 1)), y = y)
  )
  df2 <- data_frame(
    x = data_frame(x = complex(real = c(2, 3), imaginary = c(1, 1)), y = y)
  )

  res <- vec_locate_matches(df1, df2)
  expect_identical(res$needles, expect_needles)
  expect_identical(res$haystack, expect_haystack)
})

test_that("ensure that matching works if outer runs are present (i.e. `==` comes before non-equi condition)", {
  df1 <- data_frame(x = c(1, 2, 1, 1), y = c(2, 2, 3, 2))
  df2 <- data_frame(x = c(1, 1), y = c(2, 3))

  res <- vec_locate_matches(df1, df2, condition = c("==", "<="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(1L, 2L, NA, 2L, 1L, 2L))

  df1$z <- c(1L, 2L, 1L, 3L)
  df2$z <- c(5L, 2L)

  res <- vec_locate_matches(df1, df2, condition = c("==", "==", "<"))

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(1L, NA, 2L, 1L))

  res <- vec_locate_matches(df1, df2, condition = c("==", ">=", "<"))

  expect_identical(res$needles, c(1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(1L, NA, 1L, 2L, 1L))
})

test_that("df-cols propagate an NA if any columns are incomplete", {
  df <- data_frame(x = 1, y = data_frame(x = c(1, 1, NA), y = c(1, NA, 2)))

  res <- vec_locate_matches(df, df, incomplete = "compare")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, 1:3)

  res <- vec_locate_matches(df, df, incomplete = "match")
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, 1:3)

  # 2nd and 3rd rows aren't fully complete
  res <- vec_locate_matches(df, df, incomplete = NA)
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(1L, NA, NA))

  res <- vec_locate_matches(df, df, incomplete = "drop")
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)
})

test_that("df-cols aren't flattened, so `condition` is applied jointly on the df-col columns", {
  x <- data_frame(a = 1L, b = data_frame(x = 3L, y = 4L))
  y <- data_frame(a = 1L, b = data_frame(x = 2L, y = 5L))

  # In particular `x$b[1,] > y$b[1,]` because `3 > 4` and that breaks the tie
  # before any values of the `x$b$y` column are checked
  res <- vec_locate_matches(x, y, condition = c("==", ">"))
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)
})

test_that("must have at least 1 column to match", {
  expect_snapshot(error = TRUE, {
    vec_locate_matches(data_frame(), data_frame())
  })
  expect_snapshot(error = TRUE, {
    vec_locate_matches(data_frame(), data_frame(), error_call = call("foo"))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - rcrd

test_that("rcrd types can be matched", {
  x <- new_rcrd(list(x = c(1L, 3L), y = c(1L, 4L)))
  y <- new_rcrd(list(x = c(1L, 2L), y = c(1L, 5L)))

  res <- vec_locate_matches(x, y, condition = "<=")
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  # In particular: `(3, 4) > (2, 5)` since the first elt breaks the tie
  res <- vec_locate_matches(x, y, condition = ">")
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 1L, 2L))
})

test_that("rcrd type matching works with rcrd-cols", {
  x <- data_frame(
    a = c(1L, 1L),
    b = new_rcrd(list(x = c(1L, 3L), y = c(1L, 4L)))
  )
  y <- data_frame(
    a = c(1L, 1L),
    b = new_rcrd(list(x = c(1L, 2L), y = c(1L, 5L)))
  )

  res <- vec_locate_matches(x, y, condition = c("==", "<="))
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 2L, NA))

  res <- vec_locate_matches(x, y, condition = c("==", ">"))
  expect_identical(res$needles, c(1L, 2L, 2L))
  expect_identical(res$haystack, c(NA, 1L, 2L))
})

test_that("rcrd type incompleteness is handled correctly", {
  x <- new_rcrd(list(x = c(1L, NA), y = c(NA_integer_, NA_integer_)))
  y <- new_rcrd(list(x = c(1L, 2L, NA), y = c(NA, 5L, NA)))

  # When `incomplete = "compare"`, the types of incompleteness still must
  # match exactly to have a match. i.e. (x=1L, y=NA) doesn't match (x=NA, y=1L).
  # This is the same as the rule for data frames.
  res <- vec_locate_matches(x, y, condition = "==", incomplete = "compare")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(1L, 3L))

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "match")
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(1L, 3L))

  # If any field contains NA, the entire observation is incomplete.
  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - S3

test_that("S3 types with order proxies that depend on the data are combined before the proxy is taken", {
  # i.e. `bignum:::vec_proxy_order.bignum_biginteger()`

  x <- structure(c(5L, 1L), class = "foo")
  y <- structure(c(8L, 5L), class = "foo")

  local_methods(
    vec_proxy_order.foo = function(x, ...) {
      rank(unclass(x))
    }
  )

  # Can't take the order proxies separately because they are the same!
  expect_identical(vec_proxy_order(x), vec_proxy_order(y))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, NA))

  x_df <- data_frame(a = x, b = x)
  y_df <- data_frame(a = y, b = y)

  res <- vec_locate_matches(x_df, y_df)
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, NA))
})

test_that("Works with base R S3 types we support natively", {
  x <- new_factor(c(1L, 2L), levels = c("x", "y"))
  y <- new_factor(c(3L, 1L, 1L), levels = c("x", "y", "z"))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- new_ordered(c(1L, 2L), levels = c("x", "y"))
  y <- new_ordered(c(2L, 1L, 1L), levels = c("x", "y"))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, 1L))

  x <- new_date(c(1, 2))
  y <- new_date(c(3, 1, 1))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- new_datetime(c(1, 2))
  y <- new_datetime(c(3, 1, 1))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))

  x <- as.POSIXlt(new_datetime(c(1, 2)))
  y <- as.POSIXlt(new_datetime(c(3, 1, 1)))
  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

test_that("Works with classed data frame columns", {
  x_col <- new_data_frame(list(a = c(1L, 2L), b = c(2, 3)), class = "foo")
  y_col <- new_data_frame(
    list(a = c(1L, 1L, 1L), b = c(2, 4, 2)),
    class = "foo"
  )

  x <- new_data_frame(list(c = c(1L, 1L), d = x_col))
  y <- new_data_frame(list(c = c(1L, 1L, 1L), d = y_col))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(1L, 3L, NA))
})

test_that("AsIs types are combined before order proxies are taken (#1557)", {
  x <- I(list(5, 1))
  y <- I(list(8, 5, 5))

  res <- vec_locate_matches(x, y)
  expect_identical(res$needles, c(1L, 1L, 2L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - ptype2 / casting

test_that("common type of `needles` and `haystack` is taken", {
  x <- 1
  y <- "a"

  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y)
  })
  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y, needles_arg = "x", error_call = call("foo"))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - missing values

test_that("integer missing values match with `==`, `>=`, and `<=` when `incomplete = 'compare'", {
  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "==")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<=")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">=")

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = "<")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)

  res <- vec_locate_matches(NA_integer_, c(1L, NA, 2L, NA), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("integer missing values can match with any condition when `incomplete = 'match'`", {
  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "==",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<=",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = ">=",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = ">",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("integer missing values report all matches even with a `filter`", {
  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<=",
    filter = "min"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<",
    filter = "min",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = ">=",
    filter = "max"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = ">",
    filter = "max",
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("integer missing value matches can be limited by `multiple`", {
  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<=",
    multiple = "first"
  )

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<=",
    multiple = "last"
  )

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 4L)

  res <- vec_locate_matches(
    NA_integer_,
    c(1L, NA, 2L, NA),
    condition = "<=",
    multiple = "any"
  )

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 2L)
})

test_that("missing values match within columns", {
  df1 <- data_frame(x = c(1L, 2L, 1L), y = rep(NA_integer_, 3))
  df2 <- data_frame(x = c(2L, 1L, 1L), y = c(1L, NA, NA))

  res <- vec_locate_matches(df1, df2, condition = c("==", "=="))

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, 2L, 3L))

  expect_identical(
    vec_locate_matches(df1, df2, condition = c("<=", ">=")),
    vec_locate_matches(df1, df2, condition = c("==", "=="))
  )

  res <- vec_locate_matches(df1, df2, condition = c("<", ">"))

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(
    df1,
    df2,
    condition = c("<=", ">"),
    incomplete = "compare"
  )

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(
    df1,
    df2,
    condition = c("<=", ">"),
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, 2L, 3L))
})

test_that("missing values being 'match'ed hands off correctly to next column", {
  df1 <- data_frame(x = c(NA, NA, 1L, 2L, NA), y = c(2, 3, 0, 1, NA))
  df2 <- data_frame(x = c(NA, NA, NA, 3L), y = c(2, 1, NA, 0))

  res <- vec_locate_matches(
    df1,
    df2,
    condition = c("<", ">"),
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 4L, 5L))
  expect_identical(res$haystack, c(2L, 1L, 2L, NA, 4L, 3L))
})

test_that("integer needles can't match NAs in the haystack", {
  # At the C level, 1L > NA_integer_ (INT_MIN),
  # but we are careful to work around this
  res <- vec_locate_matches(1L, c(1L, NA, 2L, NA), condition = ">=")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(1L, c(1L, NA, 2L, NA), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("double needles can't match NAs or NaNs in the haystack", {
  # At the C level, our helpers assumg NA and NaN are the smallest values,
  # so we are careful to avoid including them with >= and > conditions
  res <- vec_locate_matches(1, c(1, NA, 2, NaN), condition = ">=")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(1, c(1, NA, 2, NaN), condition = ">")

  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("NA and NaN match correctly with non-equi conditions and `nan_distinct`", {
  res <- vec_locate_matches(
    c(NA, NaN),
    c(1L, NA, 2L, NaN),
    condition = "<=",
    nan_distinct = TRUE
  )

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    c(NA, NaN),
    c(1L, NA, 2L, NaN),
    condition = "<",
    nan_distinct = TRUE
  )

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(NA_integer_, NA_integer_))

  res <- vec_locate_matches(
    c(NA, NaN),
    c(1L, NA, 2L, NaN),
    condition = "<",
    nan_distinct = TRUE,
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 4L))

  res <- vec_locate_matches(
    c(NA, NaN),
    c(1L, NA, 2L, NaN),
    condition = ">=",
    nan_distinct = FALSE
  )

  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(2L, 4L, 2L, 4L))

  res <- vec_locate_matches(
    c(NA, NaN),
    c(1L, NA, 2L, NaN),
    condition = ">",
    nan_distinct = FALSE,
    incomplete = "match"
  )

  expect_identical(res$needles, c(1L, 1L, 2L, 2L))
  expect_identical(res$haystack, c(2L, 4L, 2L, 4L))
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `incomplete`

test_that("can handle incomplete needles with `incomplete = <integer>`", {
  x <- c(1L, NA, 2L)
  y <- c(NA, 1L, 1L)

  res <- vec_locate_matches(x, y, condition = "==", incomplete = NA)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA, NA))

  res <- vec_locate_matches(x, y, condition = "<=", incomplete = 0L)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 3L, 0L, NA))

  res <- vec_locate_matches(x, y, condition = ">=", incomplete = -1L)

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(2L, 3L, -1L, 2L, 3L))
})

test_that("can drop incomplete needle rows with `incomplete = 'drop'", {
  x <- c(1L, NA, 2L)
  y <- c(NA, 1L, 1L)

  res <- vec_locate_matches(x, y, condition = "==", incomplete = "drop")
  expect_identical(res$needles, c(1L, 1L, 3L))
  expect_identical(res$haystack, c(2L, 3L, NA))
})

test_that("if `incomplete = <integer>`, an NA in any column results in the value", {
  df1 <- data_frame(x = c(1L, NA, 2L, 1L, 1L), y = c(2L, 2L, NA, 1L, 1L))
  df2 <- data_frame(x = c(1L, 1L, 2L), y = c(1L, 1L, NA))

  res <- vec_locate_matches(
    df1,
    df2,
    condition = c("==", "=="),
    incomplete = NA
  )

  expect_identical(res$needles, c(1L, 2L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(NA, NA, NA, 1L, 2L, 1L, 2L))

  res <- vec_locate_matches(
    df1,
    df2,
    condition = c(">=", ">="),
    incomplete = NA
  )

  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L, 4L, 5L, 5L))
  expect_identical(res$haystack, c(1L, 2L, NA, NA, 1L, 2L, 1L, 2L))
})

test_that("`incomplete = <integer> / 'drop'` still handles NAs in future columns when an earlier column has no matches", {
  df1 <- data_frame(x = c(1, 1, 2, 3), y = c(1, NA, NA, 4))
  df2 <- data_frame(x = c(1, 3), y = c(1, 5))

  # The 2 in row 3 of df1 has no match, but the NA in the 2nd column still propagates
  res <- vec_locate_matches(df1, df2, incomplete = NA, no_match = -1L)

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(1L, NA, NA, -1L))

  res <- vec_locate_matches(df1, df2, incomplete = "drop", no_match = -1L)

  expect_identical(res$needles, c(1L, 4L))
  expect_identical(res$haystack, c(1L, -1L))

  # The 1 in row 1 and 2 of df1 have no match, but the NA in row 2 of the 2nd column propagates
  res <- vec_locate_matches(
    df1,
    df2,
    incomplete = NA,
    no_match = -1L,
    condition = ">"
  )

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(-1L, NA, NA, 1L))

  res <- vec_locate_matches(
    df1,
    df2,
    incomplete = "drop",
    no_match = -1L,
    condition = ">"
  )

  expect_identical(res$needles, c(1L, 4L))
  expect_identical(res$haystack, c(-1L, 1L))
})

test_that("`incomplete` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(NA, 1, incomplete = "error")))
    (expect_error(vec_locate_matches(
      NA,
      1,
      incomplete = "error",
      needles_arg = "foo"
    )))
    (expect_error(vec_locate_matches(
      NA,
      1,
      incomplete = "error",
      needles_arg = "foo",
      error_call = call("fn")
    )))
  })
})

test_that("`incomplete` error is classed", {
  expect_error(
    vec_locate_matches(NA, 1, incomplete = "error"),
    class = "vctrs_error_matches_incomplete"
  )
})

test_that("`incomplete` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, incomplete = 1.5)))
    (expect_error(vec_locate_matches(1, 2, incomplete = c("match", "drop"))))
    (expect_error(vec_locate_matches(1, 2, incomplete = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(
      1,
      2,
      incomplete = "x",
      error_call = call("fn")
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `condition`

test_that("multiple matches from a non-equi condition are returned in first appearance order", {
  res <- vec_locate_matches(0L, c(1L, 0L, -1L, 0L), condition = "<=")

  expect_identical(res$needles, rep(1L, 3))
  expect_identical(res$haystack, c(1L, 2L, 4L))

  # Checking equi for good measure
  res <- vec_locate_matches(0L, c(1L, 0L, -1L, 0L), condition = "==")

  expect_identical(res$needles, rep(1L, 2))
  expect_identical(res$haystack, c(2L, 4L))
})

test_that("multiple matches from a non-equi condition are returned in first appearance order when the matches are in different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 2:1, y = 1:2)

  res <- vec_locate_matches(df, df2, condition = c("<=", "<="))

  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))
})

test_that("`condition` is validated", {
  expect_error(
    vec_locate_matches(1, 2, condition = 1),
    "`condition` must be a character vector"
  )
  expect_error(
    vec_locate_matches(1, 2, condition = "x"),
    'must only contain "==", ">", ">=", "<", or "<="'
  )
  expect_error(
    vec_locate_matches(1, 2, condition = c("==", "==")),
    "must be length 1, or the same length as the number of columns of the input"
  )
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `multiple`

test_that("can get all matches", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "all")

  expect_identical(x$needles, c(1L, 1L, 2L, 2L))
  expect_identical(x$haystack, c(1L, 3L, 2L, 4L))
})

test_that("can get first match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "first")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

test_that("can get last match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "last")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 3:4)
})

test_that("can get any match", {
  x <- vec_locate_matches(c(1L, 3L), c(1L, 3L, 1L, 3L), multiple = "any")

  expect_identical(x$needles, 1:2)
  expect_identical(x$haystack, 1:2)
})

test_that("duplicate needles match the same haystack locations", {
  x <- vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")

  expect_identical(x$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(x$haystack, c(1L, 3L, 2L, 1L, 3L, 2L))
})

test_that("correctly gets all matches when they come from different nesting containers", {
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )
  haystack <- data_frame(
    a = c(6, 5),
    b = c(6, 7)
  )

  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "all"),
    data_frame(needles = c(1L, 1L, 2L), haystack = c(1L, 2L, NA))
  )
})

test_that("correctly gets first/last/any match when they come from different nesting containers", {
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )
  haystack <- data_frame(
    a = c(6, 5, 0),
    b = c(6, 7, 1)
  )

  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "first"),
    data_frame(needles = c(1L, 2L), haystack = c(1L, NA))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "last"),
    data_frame(needles = c(1L, 2L), haystack = c(2L, NA))
  )
  expect_identical(
    vec_locate_matches(needles, haystack, condition = "<", multiple = "any"),
    data_frame(needles = c(1L, 2L), haystack = c(2L, NA))
  )
  expect_identical(
    vec_locate_matches(
      needles,
      haystack,
      condition = "<",
      multiple = "first",
      remaining = NA_integer_
    ),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(1L, NA, 2L, 3L))
  )
  expect_identical(
    vec_locate_matches(
      needles,
      haystack,
      condition = "<",
      multiple = "last",
      remaining = NA_integer_
    ),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(2L, NA, 1L, 3L))
  )
  expect_identical(
    vec_locate_matches(
      needles,
      haystack,
      condition = "<",
      multiple = "any",
      remaining = NA_integer_
    ),
    data_frame(needles = c(1L, 2L, NA, NA), haystack = c(2L, NA, 1L, 3L))
  )
})

test_that("`multiple` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, multiple = 1.5)))
    (expect_error(vec_locate_matches(1, 2, multiple = c("first", "last"))))
    (expect_error(vec_locate_matches(1, 2, multiple = "x")))
    # Uses internal error
    (expect_error(vec_locate_matches(
      1,
      2,
      multiple = "x",
      error_call = call("fn")
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `multiple` (deprecated)

test_that("`multiple` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
    (expect_error(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "error",
      needles_arg = "foo"
    )))
    (expect_error(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "error",
      needles_arg = "foo",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "error",
      needles_arg = "foo",
      haystack_arg = "bar"
    )))
  })
})

test_that("`multiple` can warn informatively", {
  expect_snapshot({
    (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
    (expect_warning(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "warning",
      needles_arg = "foo"
    )))
    (expect_warning(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "warning",
      needles_arg = "foo",
      error_call = call("fn")
    )))
    (expect_warning(vec_locate_matches(
      1L,
      c(1L, 1L),
      multiple = "warning",
      needles_arg = "foo",
      haystack_arg = "bar"
    )))
  })
})

test_that("warning falls back to 'all'", {
  expect_warning(
    result <- vec_locate_matches(
      c(1L, 3L, 1L, 3L),
      c(1L, 3L, 1L),
      multiple = "warning"
    ),
    class = "vctrs_warning_matches_multiple"
  )

  expect_identical(
    result,
    vec_locate_matches(c(1L, 3L, 1L, 3L), c(1L, 3L, 1L), multiple = "all")
  )
})

test_that("errors on multiple matches that come from different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 1:2, y = 2:1)

  expect_snapshot(error = TRUE, {
    vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error")
  })
})

test_that("errors when a match from a different nesting container is processed early on", {
  # Row 1 has 2 matches
  # Row 2 has 0 matches
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )

  # Rows 1 and 2 end up in different nesting containers
  haystack <- data_frame(
    a = c(5, 6),
    b = c(7, 6)
  )

  # needles[1,] records the haystack[1,] match first, which is in the 1st
  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
  # multiple matches)
  expect_snapshot(error = TRUE, {
    vec_locate_matches(needles, haystack, condition = "<", multiple = "error")
  })
})

test_that("`multiple = 'error'` doesn't error errneously on the last observation", {
  expect_error(res <- vec_locate_matches(1:2, 1:2, multiple = "error"), NA)
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)
})

test_that("`multiple = 'error' / 'warning'` throw correctly when combined with `relationship`", {
  x <- c(1, 2, 2)
  y <- c(2, 1, 2)

  # `multiple` error technically fires first
  expect_snapshot({
    (expect_error(vec_locate_matches(
      x,
      y,
      relationship = "one-to-one",
      multiple = "error"
    )))
  })

  # Works when warning is also requested
  expect_snapshot({
    (expect_error(vec_locate_matches(
      x,
      y,
      relationship = "warn-many-to-many",
      multiple = "error"
    )))
  })
  # Both warnings are thrown if applicable
  expect_snapshot({
    vec_locate_matches(
      x,
      y,
      relationship = "warn-many-to-many",
      multiple = "warning"
    )
  })
  # Both warning and error are thrown if applicable
  expect_snapshot(error = TRUE, {
    vec_locate_matches(x, y, relationship = "one-to-one", multiple = "warning")
  })

  x <- c(1, 2)
  y <- c(2, 1, 2)

  expect_snapshot({
    (expect_error(vec_locate_matches(
      x,
      y,
      relationship = "warn-many-to-many",
      multiple = "error"
    )))
  })
  # Only `multiple` warning is applicable here
  expect_snapshot({
    vec_locate_matches(
      x,
      y,
      relationship = "warn-many-to-many",
      multiple = "warning"
    )
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `relationship`

test_that("`relationship` handles one-to-one case", {
  # No error
  expect_identical(
    vec_locate_matches(1:2, 2:1, relationship = "one-to-one"),
    vec_locate_matches(1:2, 2:1)
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "one-to-one"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(
      c(2, 1),
      c(1, 1),
      relationship = "one-to-one"
    )))
    (expect_error(vec_locate_matches(
      c(1, 1),
      c(1, 2),
      relationship = "one-to-one"
    )))
  })
})

test_that("`relationship` handles one-to-many case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "one-to-many"),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "one-to-many"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(
      c(1, 2, 2),
      c(2, 1),
      relationship = "one-to-many"
    )))
  })
})

test_that("`relationship` handles many-to-one case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-one"),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "many-to-one"),
    vec_locate_matches(1:2, 3:4)
  )

  expect_snapshot({
    (expect_error(vec_locate_matches(
      c(1, 2),
      c(1, 2, 2),
      relationship = "many-to-one"
    )))
  })
})

test_that("`relationship` handles many-to-many case", {
  # No error
  expect_identical(
    vec_locate_matches(c(1, 2, 2), c(1, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # No error
  expect_identical(
    vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # No error
  expect_identical(
    vec_locate_matches(c(1, 1, 2), c(1, 2, 2), relationship = "many-to-many"),
    vec_locate_matches(c(1, 1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    vec_locate_matches(1:2, 3:4, relationship = "many-to-many"),
    vec_locate_matches(1:2, 3:4)
  )
})

test_that("`relationship` handles warn-many-to-many case", {
  # No warning
  expect_identical(
    expect_silent(
      vec_locate_matches(
        c(1, 2, 2),
        c(1, 2),
        relationship = "warn-many-to-many"
      )
    ),
    vec_locate_matches(c(1, 2, 2), c(1, 2))
  )

  # No warning
  expect_identical(
    expect_silent(
      vec_locate_matches(
        c(1, 2),
        c(1, 2, 2),
        relationship = "warn-many-to-many"
      )
    ),
    vec_locate_matches(c(1, 2), c(1, 2, 2))
  )

  # Doesn't care about the zero match case
  expect_identical(
    expect_silent(
      vec_locate_matches(1:2, 3:4, relationship = "warn-many-to-many")
    ),
    vec_locate_matches(1:2, 3:4)
  )

  # Specifically designed to ensure we test both:
  # - Finding multiple `needles` matches before multiple `haystack` matches
  # - Finding multiple `haystack` matches before multiple `needles` matches
  expect_snapshot({
    (expect_warning(vec_locate_matches(
      c(1, 2, 1),
      c(1, 2, 2),
      relationship = "warn-many-to-many"
    )))
    (expect_warning(vec_locate_matches(
      c(1, 1, 2),
      c(2, 2, 1),
      relationship = "warn-many-to-many"
    )))
  })
})

test_that("`relationship` considers `incomplete` matches as possible multiple matches", {
  x <- c(1, NA, NaN)
  y <- c(NA, 1)

  expect_snapshot({
    (expect_error(vec_locate_matches(x, y, relationship = "one-to-many")))
  })

  # No error
  expect_identical(
    vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA),
    vec_locate_matches(x, y, incomplete = NA)
  )

  # No error
  expect_identical(
    vec_locate_matches(x, y, relationship = "one-to-many", nan_distinct = TRUE),
    vec_locate_matches(x, y, nan_distinct = TRUE)
  )
})

test_that("`relationship` errors on multiple matches that come from different nesting containers", {
  df <- data_frame(x = 0, y = 0)
  df2 <- data_frame(x = 1:2, y = 2:1)

  expect_snapshot({
    (expect_error(vec_locate_matches(
      df,
      df2,
      condition = c("<=", "<="),
      relationship = "many-to-one"
    )))
  })
})

test_that("`relationship` errors when a match from a different nesting container is processed early on", {
  # Row 1 has 2 matches
  # Row 2 has 0 matches
  needles <- data_frame(
    a = c(1, 8),
    b = c(2, 9)
  )

  # Rows 1 and 2 end up in different nesting containers
  haystack <- data_frame(
    a = c(5, 6),
    b = c(7, 6)
  )

  # needles[1,] records the haystack[1,] match first, which is in the 1st
  # value of `loc_first_match_o_haystack`, then records the haystack[3,] match
  # which is in the 3rd value of `loc_first_match_o_haystack` even though it
  # is processed 2nd (i.e. we need to use `loc` rather than `i` when detecting
  # multiple matches)
  expect_snapshot({
    (expect_error(vec_locate_matches(
      needles,
      haystack,
      condition = "<",
      relationship = "many-to-one"
    )))
  })
})

test_that("`relationship` doesn't error errneously on the last observation", {
  expect_error(
    res <- vec_locate_matches(1:2, 1:2, relationship = "many-to-one"),
    NA
  )
  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, 1:2)
})

test_that("`relationship` doesn't error if `multiple` removes multiple matches", {
  out <- vec_locate_matches(
    c(1, 2),
    c(1, 1),
    multiple = "any",
    relationship = "one-to-one"
  )
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(1L, NA))

  out <- vec_locate_matches(
    c(1, 2),
    c(1, 1),
    multiple = "first",
    relationship = "one-to-one"
  )
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(1L, NA))

  out <- vec_locate_matches(
    c(1, 2),
    c(1, 1),
    multiple = "last",
    relationship = "one-to-one"
  )
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(2L, NA))
})

test_that("`relationship` can still detect problematic `haystack` relationships when `multiple = first/last` are used", {
  expect_snapshot({
    (expect_error(vec_locate_matches(
      c(3, 1, 1),
      c(2, 1, 3, 3),
      multiple = "first",
      relationship = "one-to-one"
    )))
    (expect_error(vec_locate_matches(
      c(3, 1, 1),
      c(2, 1, 3, 3),
      multiple = "first",
      relationship = "one-to-many"
    )))
  })
})

test_that("`relationship` and `remaining` work properly together", {
  expect_snapshot({
    out <- vec_locate_matches(
      c(1, 2, 2),
      c(2, 3, 1, 1, 4),
      relationship = "warn-many-to-many",
      remaining = NA_integer_
    )
  })
  expect_identical(out$needles, c(1L, 1L, 2L, 3L, NA, NA))
  expect_identical(out$haystack, c(3L, 4L, 1L, 1L, 2L, 5L))
})

test_that("`relationship` errors if `condition` creates multiple matches", {
  expect_snapshot({
    (expect_error(vec_locate_matches(
      1,
      c(1, 2),
      condition = "<=",
      relationship = "many-to-one"
    )))
  })
})

test_that("`relationship` doesn't error if `filter` removes multiple matches", {
  out <- vec_locate_matches(
    1,
    c(1, 2),
    condition = "<=",
    filter = "min",
    relationship = "many-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 1L)

  out <- vec_locate_matches(
    1,
    c(1, 2),
    condition = "<=",
    filter = "max",
    relationship = "many-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)
})

test_that("`relationship` still errors if `filter` hasn't removed all multiple matches", {
  expect_snapshot({
    (expect_error(vec_locate_matches(
      1,
      c(1, 2, 1),
      condition = "<=",
      filter = "min",
      relationship = "many-to-one"
    )))
  })

  # But not here
  out <- vec_locate_matches(
    c(1, 1),
    c(1, 2, 1),
    condition = "<=",
    filter = "max",
    relationship = "many-to-one"
  )
  expect_identical(out$needles, c(1L, 2L))
  expect_identical(out$haystack, c(2L, 2L))
})

test_that("`relationship` errors when we have >1 size 1 matches across containers (tidyverse/dplyr#6835)", {
  # Carefully designed to ensure we get 2 nested containment groups that split
  # up the rows of `y`, but each of the nested containment groups contain exactly
  # 1 match, so `size_match` in `expand_compact_indices()` won't ever be >1
  x <- data_frame(a = 1L, b = 5L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))

  expect_snapshot(error = TRUE, {
    vec_locate_matches(
      x,
      y,
      condition = c("<=", ">="),
      filter = c("none", "none"),
      relationship = "one-to-one"
    )
  })
})

test_that("`relationship` doesn't error when the first match from a different container gets filtered out (tidyverse/dplyr#6835)", {
  # Carefully designed to ensure we get 2 nested containment groups that split
  # up the rows of `y`. Row 1 (processed first) doesn't hold the minimum `b`
  # value, so it gets filtered out. Row 2 is in the "extra" matches section
  # but is actually the first (and only) real match, so we don't want to error
  # on it.
  x <- data_frame(a = 1L, b = 5L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L))

  out <- vec_locate_matches(
    x,
    y,
    condition = c("<=", ">="),
    filter = c("none", "min"),
    relationship = "one-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)

  # Similar to the above example, but with a `max` filter. Row 1 doesn't hold
  # the max `c` value so it is filtered out even though it is a `>=` match.
  x <- data_frame(a = 1L, b = 5L, c = 3L)
  y <- data_frame(a = c(1L, 2L), b = c(4L, 3L), c = c(1L, 2L))

  out <- vec_locate_matches(
    x,
    y,
    condition = c("<=", ">=", ">="),
    filter = c("none", "none", "max"),
    relationship = "one-to-one"
  )
  expect_identical(out$needles, 1L)
  expect_identical(out$haystack, 2L)
})

test_that("`relationship` errors respect argument tags and error call", {
  expect_snapshot({
    (expect_error(vec_locate_matches(
      1L,
      c(1L, 1L),
      relationship = "one-to-one",
      needles_arg = "foo",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      c(1L, 1L),
      1L,
      relationship = "one-to-one",
      needles_arg = "foo",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      c(1L, 1L),
      1L,
      relationship = "one-to-many",
      needles_arg = "foo",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      1L,
      c(1L, 1L),
      relationship = "many-to-one",
      needles_arg = "foo",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
  })
})

test_that("`relationship` warnings respect argument tags and error call", {
  expect_snapshot({
    (expect_warning(vec_locate_matches(
      c(1L, 1L),
      c(1L, 1L),
      relationship = "warn-many-to-many",
      needles_arg = "foo",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
    (expect_warning(vec_locate_matches(
      c(1L, 1L),
      c(1L, 1L),
      relationship = "warn-many-to-many",
      needles_arg = "foo",
      error_call = call("fn")
    )))
    (expect_warning(vec_locate_matches(
      c(1L, 1L),
      c(1L, 1L),
      relationship = "warn-many-to-many",
      haystack_arg = "bar",
      error_call = call("fn")
    )))
  })
})

test_that("`relationship` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, relationship = 1.5)))
    (expect_error(vec_locate_matches(
      1,
      2,
      relationship = c("one-to-one", "one-to-many")
    )))
    (expect_error(vec_locate_matches(1, 2, relationship = "x")))
    # Uses internal error
    (expect_error(vec_locate_matches(
      1,
      2,
      relationship = "x",
      error_call = call("fn")
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `no_match`

test_that("can control `no_match`", {
  x <- vec_locate_matches(1:3, 1L)
  expect_identical(x$haystack, c(1L, NA, NA))

  x <- vec_locate_matches(1:3, 1L, no_match = 0L)
  expect_identical(x$haystack, c(1L, 0L, 0L))
})

test_that("can drop unmatched needles", {
  x <- vec_locate_matches(1:3, 2L, no_match = "drop")
  expect_identical(x$needles, 2L)
  expect_identical(x$haystack, 1L)
})

test_that("can drop unmatched missings when `incomplete = 'match'`", {
  x <- vec_locate_matches(c(NaN, 2, NA), 2, no_match = "drop")
  expect_identical(x$needles, 2L)
  expect_identical(x$haystack, 1L)

  x <- vec_locate_matches(
    c(NaN, 2, NA),
    NA,
    no_match = "drop",
    nan_distinct = FALSE
  )
  expect_identical(x$needles, c(1L, 3L))
  expect_identical(x$haystack, c(1L, 1L))

  x <- vec_locate_matches(
    c(NaN, 2, NA),
    NA,
    no_match = "drop",
    nan_distinct = TRUE
  )
  expect_identical(x$needles, 3L)
  expect_identical(x$haystack, 1L)
})

test_that("can differentiate between `no_match` and `incomplete`", {
  res <- vec_locate_matches(c(1, NA), 2, incomplete = NA, no_match = -1L)

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(-1L, NA))
})

test_that("`no_match` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, no_match = "error")))
    (expect_error(vec_locate_matches(
      1,
      2,
      no_match = "error",
      needles_arg = "foo"
    )))
    (expect_error(vec_locate_matches(
      1,
      2,
      no_match = "error",
      needles_arg = "foo",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      1,
      2,
      no_match = "error",
      needles_arg = "foo",
      haystack_arg = "bar"
    )))
  })
})

test_that("`no_match = 'error'` doesn't error on handled incomplete values", {
  res <- vec_locate_matches(
    c(NA, NaN, NA, 1),
    c(NA, 1),
    incomplete = NA,
    no_match = "error"
  )

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(rep(NA, 3), 2L))
})

test_that("`no_match = 'drop'` doesn't drop handled incomplete values", {
  res <- vec_locate_matches(
    c(NA, NaN, NA, 1),
    c(NA, 1),
    incomplete = NA,
    no_match = "drop"
  )

  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(rep(NA, 3), 2L))
})

test_that("errors with the right location on unmatched needles when different nesting containers are present", {
  df <- data_frame(x = 2:1, y = 2:1)
  df2 <- data_frame(x = 1:2, y = 2:1)

  # i.e. should be location 2
  expect_snapshot(
    (expect_error(vec_locate_matches(
      df,
      df2,
      condition = ">=",
      no_match = "error"
    )))
  )
})

test_that("`no_match` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, no_match = 1.5)))
    (expect_error(vec_locate_matches(1, 2, no_match = c(1L, 2L))))
    (expect_error(vec_locate_matches(1, 2, no_match = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(
      1,
      2,
      no_match = "x",
      error_call = call("fn")
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - `remaining`

test_that("`remaining` can retain `haystack` values that `needles` didn't match", {
  res <- vec_locate_matches(1, 0:2, remaining = NA)
  expect_identical(res$needles, c(1L, NA, NA))
  expect_identical(res$haystack, c(2L, 1L, 3L))

  res <- vec_locate_matches(1, 0:2, remaining = NA, condition = ">=")
  expect_identical(res$needles, c(1L, 1L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))

  res <- vec_locate_matches(1, 0:2, remaining = NA, condition = "<")
  expect_identical(res$needles, c(1L, NA, NA))
  expect_identical(res$haystack, c(3L, 1L, 2L))
})

test_that("`incomplete` affects `needles` but not `haystack`", {
  # Matches NA to NA, so nothing remaining
  res <- vec_locate_matches(
    c(1, NA),
    c(NA, 1),
    incomplete = "compare",
    remaining = NA
  )
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 1L))

  # Matches NA to NA, so nothing remaining
  res <- vec_locate_matches(
    c(1, NA),
    c(NA, 1),
    incomplete = "match",
    remaining = NA
  )
  expect_identical(res$needles, c(1L, 2L))
  expect_identical(res$haystack, c(2L, 1L))

  # Doesn't match NA to NA, so `haystack` is left with remaining values
  res <- vec_locate_matches(
    c(1, NA),
    c(NA, 1),
    condition = "<",
    incomplete = "compare",
    remaining = NA
  )
  expect_identical(res$needles, c(1L, 2L, NA, NA))
  expect_identical(res$haystack, c(NA, NA, 1L, 2L))

  # Matches NA to NA, so only remaining value is for `1`
  res <- vec_locate_matches(
    c(1, NA),
    c(NA, 1),
    condition = "<",
    incomplete = "match",
    remaining = NA
  )
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(NA, 1L, 2L))

  # `needles` NA value is propagated, so `haystack` is left with a remaining value
  res <- vec_locate_matches(c(1, NA), c(NA, 1), incomplete = NA, remaining = NA)
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(2L, NA, 1L))

  # `needles` NA value is dropped, so `haystack` is left with a remaining value
  res <- vec_locate_matches(
    c(1, NA),
    c(NA, 1),
    incomplete = "drop",
    remaining = NA
  )
  expect_identical(res$needles, c(1L, NA))
  expect_identical(res$haystack, c(2L, 1L))
})

test_that("`remaining` combined with `multiple = 'first/last'` treats non-first/last matches as remaining", {
  x <- c(1, 2)
  y <- c(1, 2, 2)

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "first")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "last")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 3L, 2L))

  res <- vec_locate_matches(x, y, remaining = NA, multiple = "any")
  expect_identical(res$needles, c(1L, 2L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))
})

test_that("`remaining` combined with the haystack reordering retains appearance order", {
  x <- data_frame(a = 1, b = 4)
  y <- data_frame(a = c(2, 1, 0), b = c(2, 1, 0))

  # Appearance order for the haystack locations
  res <- vec_locate_matches(x, y, condition = c("<=", ">="))
  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))

  # Retain that appearance order of the matches, with remaining values appended
  res <- vec_locate_matches(x, y, condition = c("<=", ">="), remaining = NA)
  expect_identical(res$needles, c(1L, 1L, NA))
  expect_identical(res$haystack, c(1L, 2L, 3L))
})

test_that("`remaining` can error informatively", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, remaining = "error")))
    (expect_error(vec_locate_matches(
      1,
      2,
      remaining = "error",
      needles_arg = "foo"
    )))
    (expect_error(vec_locate_matches(
      1,
      2,
      remaining = "error",
      needles_arg = "foo",
      error_call = call("fn")
    )))
    (expect_error(vec_locate_matches(
      1,
      2,
      remaining = "error",
      needles_arg = "foo",
      haystack_arg = "bar"
    )))
  })
})

test_that("`remaining` is validated", {
  expect_snapshot({
    (expect_error(vec_locate_matches(1, 2, remaining = 1.5)))
    (expect_error(vec_locate_matches(1, 2, remaining = c(1L, 2L))))
    (expect_error(vec_locate_matches(1, 2, remaining = "x")))
    # Uses internal call
    (expect_error(vec_locate_matches(
      1,
      2,
      remaining = "x",
      error_call = call("fn")
    )))
  })
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - filter

test_that("simple `filter`s work", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 3, 0)

  res <- vec_locate_matches(needles, haystack, condition = "<", filter = "max")
  expect_identical(res$haystack, c(3L, 3L, NA))

  res <- vec_locate_matches(needles, haystack, condition = "<", filter = "min")
  expect_identical(res$haystack, c(1L, 3L, NA))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$haystack, c(2L, 1L, 3L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$haystack, c(4L, 4L, 4L))
})

test_that("haystack duplicates are preserved", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 2, 3, 0, 1, 0)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L))
  expect_identical(res$haystack, c(2L, 6L, 1L, 3L, 4L))

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 3L))
  expect_identical(res$haystack, c(5L, 7L, 5L, 7L, 5L, 7L))
})

test_that("haystack duplicates can be controlled by `multiple`", {
  needles <- c(1, 2, 4)
  haystack <- c(2, 1, 2, 3, 0, 1, 0)

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    multiple = "first"
  )
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(2L, 1L, 4L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    multiple = "last"
  )
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(6L, 3L, 4L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    multiple = "any"
  )
  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, c(2L, 1L, 4L))
})

test_that("`filter` works when valid matches are in different nesting containers", {
  needles <- data_frame(x = 0L, y = 1L, z = 2L)
  haystack <- data_frame(
    x = c(1L, 2L, 1L, 0L),
    y = c(2L, 1L, 2L, 3L),
    z = c(3L, 3L, 2L, 2L)
  )

  info <- compute_nesting_container_info(haystack, c("<=", "<=", "<="))
  haystack_order <- info[[1]]
  container_ids <- info[[2]]

  # Rows 1 and 2 of haystack are in different nesting containers, but
  # both have the "max" filter value of `z=3` so both should be in the result.
  # Row 4 is in its own container, so it will be considered the "max"
  # of its group, but it is less than rows 1 and 2 so it will ultimately be
  # filtered out.
  expect_identical(container_ids, c(1L, 2L, 1L, 0L))
  expect_identical(haystack_order, c(4L, 3L, 1L, 2L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = c("<=", "<=", "<="),
    filter = c("none", "none", "max")
  )
  expect_identical(res$needles, c(1L, 1L))
  expect_identical(res$haystack, c(1L, 2L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = c("<=", "<=", "<="),
    filter = c("none", "none", "max"),
    multiple = "first"
  )
  expect_identical(res$haystack, 1L)

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = c("<=", "<=", "<="),
    filter = c("none", "none", "max"),
    multiple = "last"
  )
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = c("<=", "<=", "<="),
    filter = c("none", "none", "max"),
    multiple = "any"
  )
  expect_identical(res$haystack, 1L)
})

test_that("single filter is applied to all columns", {
  needles <- data_frame(x = 5L, y = 8L, z = 4L)
  haystack <- data_frame(
    x = c(1L, 3L, 2L, 2L),
    y = c(1L, 3L, 2L, 3L),
    z = c(1L, 2L, 3L, 3L)
  )

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "max")
  expect_identical(res$haystack, 2L)

  res <- vec_locate_matches(needles, haystack, condition = ">=", filter = "min")
  expect_identical(res$haystack, 1L)
})

test_that("different `filter`s can be used per column", {
  needles <- data_frame(x = c(0, 2, 1, 1), y = c(2, 0, 0, 4))
  haystack <- data_frame(x = c(2, 2, 2, 1, 1), y = c(1, 1, 2, 2, 1))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = c(">=", "<"),
    filter = c("max", "min")
  )
  expect_identical(res$needles, c(1L, 2L, 2L, 3L, 4L))
  expect_identical(res$haystack, c(NA, 1L, 2L, 5L, NA))
})

test_that("`filter` works with incomplete values", {
  needles <- c(1, NA, 4, NA)
  haystack <- c(NA, 1, NA, 1, 3)

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    incomplete = "compare"
  )
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 3L, 5L, 1L, 3L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    incomplete = "compare",
    multiple = "first"
  )
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(2L, 1L, 5L, 1L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    incomplete = "compare",
    multiple = "any"
  )
  expect_identical(res$needles, 1:4)
  expect_identical(res$haystack, c(2L, 1L, 5L, 1L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "max",
    incomplete = NA
  )
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 4L))
  expect_identical(res$haystack, c(2L, 4L, NA, 5L, NA))
})

test_that("`filter` works with mixed NA and NaN", {
  needles <- c(1, NA, 4, NaN)
  haystack <- c(NA, 1, NaN, 1, 3)

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "min",
    incomplete = "compare",
    nan_distinct = FALSE
  )
  expect_identical(res$needles, c(1L, 1L, 2L, 2L, 3L, 3L, 4L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 3L, 2L, 4L, 1L, 3L))

  res <- vec_locate_matches(
    needles,
    haystack,
    condition = ">=",
    filter = "min",
    incomplete = "compare",
    nan_distinct = TRUE
  )
  expect_identical(res$needles, c(1L, 1L, 2L, 3L, 3L, 4L))
  expect_identical(res$haystack, c(2L, 4L, 1L, 2L, 4L, 3L))
})

test_that("`filter` is validated", {
  expect_error(vec_locate_matches(1, 2, filter = 1.5), "character vector")
  expect_error(
    vec_locate_matches(1, 2, filter = "x"),
    'one of "none", "min", or "max"'
  )
  expect_error(
    vec_locate_matches(1, 2, filter = c("min", "max")),
    "length 1, or the same length as"
  )
})

# ------------------------------------------------------------------------------
# vec_locate_matches() - edge cases

test_that("zero row `needles` results in zero row data frame output", {
  res <- vec_locate_matches(integer(), 1:3)

  expect_identical(res$needles, integer())
  expect_identical(res$haystack, integer())

  res <- vec_locate_matches(integer(), 1:3, condition = "<")

  expect_identical(res$needles, integer())
  expect_identical(res$haystack, integer())
})

test_that("zero row `haystack` results in no-matches for all needles", {
  res <- vec_locate_matches(1:3, integer())

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))

  res <- vec_locate_matches(1:3, integer(), condition = "<")

  expect_identical(res$needles, 1:3)
  expect_identical(res$haystack, rep(NA_integer_, 3))
})

test_that("zero row `haystack` still allows needle incomplete handling", {
  res <- vec_locate_matches(c(1, NA), integer(), incomplete = NA, no_match = 0L)

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(0L, NA))

  res <- vec_locate_matches(
    c(1, NA),
    integer(),
    incomplete = NA,
    no_match = 0L,
    condition = "<"
  )

  expect_identical(res$needles, 1:2)
  expect_identical(res$haystack, c(0L, NA))
})

test_that("zero column data frames are not allowed", {
  expect_error(
    vec_locate_matches(data_frame(.size = 2L), data_frame(.size = 2L)),
    "at least 1 column"
  )
})

test_that("zero column input still checks `condition` correctness", {
  x <- data_frame(.size = 2)
  y <- data_frame(.size = 3)

  expect_error(
    vec_locate_matches(x, y, condition = c("==", "<=")),
    "length 1, or the same length as the number of columns"
  )
})

test_that("`multiple = 'first'/'last'` returns the first/last by appearance", {
  x <- c(1, 2, 3)
  y <- c(2, 1, 0)

  res <- vec_locate_matches(x, y, condition = ">=", multiple = "first")
  expect_identical(res$haystack, c(2L, 1L, 1L))

  res <- vec_locate_matches(x, y, condition = ">=", multiple = "last")
  expect_identical(res$haystack, c(3L, 3L, 3L))
})

test_that("NA adjustment of `>` and `>=` conditions is protected from empty haystack", {
  res <- vec_locate_matches(1L, integer(), condition = ">")
  expect_identical(res$needles, 1L)
  expect_identical(res$haystack, NA_integer_)
})

test_that("potential overflow on large output size is caught informatively", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  expect_snapshot({
    (expect_error(vec_locate_matches(1:1e7, 1:1e7, condition = ">=")))
  })
})

testthat_import_from <- function(ns, names, env = caller_env()) {
  skip_if_not_installed(ns)
  import_from(ns, names, env = env)
}

shaped_int <- function(...) {
  array(NA_integer_, c(...))
}

set_rownames_recursively <- function(x, i = NULL) {
  n <- vec_size(x)
  stopifnot(n <= length(letters))

  for (j in seq_along(x)) {
    if (is.data.frame(x[[j]])) {
      x[[j]] <- set_rownames_recursively(x[[j]], i = i)
    }
  }

  row.names(x) <- paste0(letters[seq_len(n)], i)
  x
}

expect_waldo_equal <- function(type, act, exp, info, ...) {
  comp <- waldo::compare(
    act$val,
    exp$val,
    ...,
    x_arg = "actual",
    y_arg = "expected"
  )
  expect(
    length(comp) == 0,
    sprintf(
      "`actual` (%s) not %s to `expected` (%s).\n\n%s",
      act$lab,
      type,
      exp$lab,
      paste0(comp, collapse = "\n\n")
    ),
    info = info
  )

  invisible(act$val)
}

expect_identical <- function(
  object,
  expected,
  info = NULL,
  label = NULL,
  expected.label = NULL,
  ...
) {
  act <- quasi_label(enquo(object), label, arg = "object")
  exp <- quasi_label(enquo(expected), expected.label, arg = "expected")

  expect_waldo_equal("identical", act, exp, info, ...)
}

expect_equal <- function(
  object,
  expected,
  ...,
  tolerance = .Machine$double.eps^0.5,
  info = NULL,
  label = NULL,
  expected.label = NULL
) {
  act <- quasi_label(enquo(object), label, arg = "object")
  exp <- quasi_label(enquo(expected), expected.label, arg = "expected")

  expect_waldo_equal("equal", act, exp, info, ..., tolerance = tolerance)
}

raw2 <- function(...) {
  as.raw(vec_c(..., .ptype = integer()))
}
cpl2 <- function(...) {
  # R 4.4.0 changed `as.complex(NA_real/integer/logical)` so that it always uses
  # a `0` in the imaginary slot. While this is reasonable, it is annoying for
  # comparison purposes in tests, where we typically propagate the `NA`. As of
  # rlang 1.1.1, `cpl()` inherits this behavior change so we have a custom version
  # here that works the same on all R versions.
  # https://github.com/wch/r-source/commit/1a2aea9ac3c216fea718f33f712764afc34f6ee8
  out <- list2(...)
  out <- as.complex(out)
  out[is.na(out)] <- complex(real = NA_real_, imaginary = NA_real_)
  out
}

#' Recode and replace values
#'
#' @description
#'
#' - `vec_recode_values()` constructs an entirely new vector by recoding the
#'   values from `x` specified in `from` to the corresponding values in `to`. If
#'   there are values in `x` not matched by `from`, then they are recoded to the
#'   `default` value.
#'
#' - `vec_replace_values()` updates an existing vector by replacing the values
#'   from `x` specified in `from` with the corresponding values in `to`. In this
#'   case, `to` must have the same type as `x` and values in `x` not matched by
#'   `from` pass through untouched.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector.
#'
#' @param from Values to locate in `x` and map to values in `to`.
#'
#'   Extra values present in `from` but not in `x` are ignored.
#'
#'   - If `from_as_list_of_vectors = FALSE`, `from` must be a single vector of
#'     any size, which will be [cast][vctrs::theory-faq-coercion] to the type of
#'     `x`.
#'
#'   - If `from_as_list_of_vectors = TRUE`, `from` must be a list of vectors of
#'     any size, which will individually be [cast][vctrs::theory-faq-coercion]
#'     to the type of `x`.
#'
#' @param to Values to map `from` to.
#'
#'   The common type of `to` and `default` will determine the type of the
#'   output, unless `ptype` is provided.
#'
#'   - If `to_as_list_of_vectors = FALSE`, `to` must be a single vector of size
#'     1 or the same size as `from`.
#'
#'   - If `to_as_list_of_vectors = TRUE`, `to` must be a list of vectors. The
#'     list itself must be size 1 or the same size as `from`. Each individual
#'     vector in the list must be size 1 or the same size as `x`.
#'
#' @param default Default value to use when there is a value present in `x`
#'   that is unmatched by a value in `from`.
#'
#'   By default, a missing value is used as the default value.
#'
#'   If supplied, `default` must be size 1 or the same size as `x`.
#'
#'   Can only be set when `unmatched = "default"`.
#'
#' @param unmatched Handling of unmatched locations.
#'
#'   One of:
#'
#'   - `"default"` to use `default` in unmatched locations.
#'
#'   - `"error"` to error when there are unmatched locations.
#'
#' @param from_as_list_of_vectors,to_as_list_of_vectors Boolean values
#'   determining whether to treat `from` and `to` as vectors or as lists of
#'   vectors. See their parameter descriptions for more details.
#'
#' @param x_arg,from_arg,to_arg,default_arg Argument names used in error
#'   messages.
#'
#' @param ptype An optional override for the output type, which is usually
#'   computed as the common type of `to` and `default`.
#'
#' @returns
#' A vector the same size as `x`.
#'
#' - For `vec_recode_values()`, the type of the output is computed as the common
#'   type of `to` and `default`, unless overridden by `ptype`. The names of the
#'   output come from the names of `to` and `default`.
#'
#' - For `vec_replace_values()`, the type of the output will have the same type
#'   as `x`. The names of the output will be the same as the names of `x`.
#'
#' @name vec-recode-and-replace
#'
#' @examples
#' x <- c(1, 2, 3, 1, 2, 4, NA, 5)
#'
#' # Imagine you have a pre-existing lookup table
#' likert <- data.frame(
#'   from = c(1, 2, 3, 4, 5),
#'   to = c(
#'     "Strongly disagree",
#'     "Disagree",
#'     "Neutral",
#'     "Agree",
#'     "Strongly agree"
#'   )
#' )
#' vec_recode_values(x, from = likert$from, to = likert$to)
#'
#' # If you don't map all of the values, a `default` is used
#' x <- c(1, 2, 3, 1, 2, 4, NA, 5, 6, 7)
#' vec_recode_values(x, from = likert$from, to = likert$to)
#' vec_recode_values(x, from = likert$from, to = likert$to, default = "Unknown")
#'
#' # If you want existing `NA`s to pass through, include a mapping for `NA` in
#' # your lookup table
#' likert <- data.frame(
#'   from = c(1, 2, 3, 4, 5, NA),
#'   to = c(
#'     "Strongly disagree",
#'     "Disagree",
#'     "Neutral",
#'     "Agree",
#'     "Strongly agree",
#'     NA
#'   )
#' )
#' vec_recode_values(x, from = likert$from, to = likert$to, default = "Unknown")
#'
#' # If you believe you've captured all of the cases, you can assert this with
#' # `unmatched = "error"`, which will error if you've missed any cases
#' # (including `NA`, which must be explicitly handled)
#' try(vec_recode_values(
#'   x,
#'   from = likert$from,
#'   to = likert$to,
#'   unmatched = "error"
#' ))
#'
#' if (require("tibble")) {
#'   # If you want to partially update `x`, retaining the type of `x` and
#'   # leaving values not covered by `from` alone, use `vec_replace_values()`
#'   universities <- c(
#'     "Duke",
#'     "Fake U",
#'     "Duke U",
#'     NA,
#'     "Chapel Hill",
#'     "UNC",
#'     NA,
#'     "Duke"
#'   )
#'
#'   standardize <- tibble::tribble(
#'     ~from, ~to,
#'     "Duke", "Duke University",
#'     "Duke U", "Duke University",
#'     "UNC", "UNC Chapel Hill",
#'     "Chapel Hill", "UNC Chapel Hill",
#'   )
#'   vec_replace_values(
#'     universities,
#'     from = standardize$from,
#'     to = standardize$to
#'   )
#'
#'   # In this case, you can use a more powerful feature of
#'   # `vec_replace_values()`, `from_as_list_of_vectors`, which allows you to
#'   # provide a list of `from` vectors that each match multiple `from` values
#'   # to a single `to` value. `tribble()` can help you create these!
#'   standardize <- tibble::tribble(
#'     ~from, ~to,
#'     c("Duke", "Duke U"), "Duke University",
#'     c("UNC", "Chapel Hill"), "UNC Chapel Hill",
#'   )
#'
#'   # Note how `from` is a list column
#'   standardize
#'
#'   vec_replace_values(
#'     universities,
#'     from = standardize$from,
#'     to = standardize$to,
#'     from_as_list_of_vectors = TRUE
#'   )
#'
#'   # `vec_replace_values()` is also a useful way to map from or to `NA`
#'   vec_replace_values(universities, from = NA, to = "Unknown")
#'   vec_replace_values(universities, from = "Fake U", to = NA)
#' }
NULL

#' @rdname vec-recode-and-replace
#' @export
vec_recode_values <- function(
  x,
  ...,
  from,
  to,
  default = NULL,
  unmatched = "default",
  from_as_list_of_vectors = FALSE,
  to_as_list_of_vectors = FALSE,
  ptype = NULL,
  x_arg = "x",
  from_arg = "from",
  to_arg = "to",
  default_arg = "default",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_recode_values,
    x,
    from,
    to,
    default,
    unmatched,
    from_as_list_of_vectors,
    to_as_list_of_vectors,
    ptype,
    environment()
  )
}

#' @rdname vec-recode-and-replace
#' @export
vec_replace_values <- function(
  x,
  ...,
  from,
  to,
  from_as_list_of_vectors = FALSE,
  to_as_list_of_vectors = FALSE,
  x_arg = "x",
  from_arg = "from",
  to_arg = "to",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_replace_values,
    x,
    from,
    to,
    from_as_list_of_vectors,
    to_as_list_of_vectors,
    environment()
  )
}

test_that("from_as_list_of_vectors = TRUE / to_as_list_of_vectors = TRUE", {
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = list(c(2, 3), c(4, 1)),
      to = list(
        c("a", "b", "c"),
        c("d", "e", "f")
      ),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c("d", "b", "c")
  )
})

test_that("from_as_list_of_vectors = TRUE / to_as_list_of_vectors = TRUE (optimized to `from_as_list_of_vectors = FALSE`, `to_as_list_of_vectors = FALSE`)", {
  # First optimized to `to_as_list_of_vectors = FALSE`
  # Then further optimized to `from_as_list_of_vectors = FALSE`
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = list(c(2, 3), c(4, 1)),
      to = list(
        "a",
        "b"
      ),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c("b", "a", "a")
  )
})

test_that("from_as_list_of_vectors = TRUE / to_as_list_of_vectors = FALSE (always optimized to `from_as_list_of_vectors = FALSE`)", {
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = list(c(2, 3), c(4, 1)),
      to = c("a", "b"),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = FALSE
    ),
    c("b", "a", "a")
  )
})

test_that("from_as_list_of_vectors = FALSE / to_as_list_of_vectors = TRUE", {
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = c(2, 1),
      to = list(
        c("a", "b", "c"),
        c("d", "e", "f")
      ),
      from_as_list_of_vectors = FALSE,
      to_as_list_of_vectors = TRUE
    ),
    c("d", "b", NA)
  )
})

test_that("from_as_list_of_vectors = FALSE / to_as_list_of_vectors = TRUE (optimized to `to_as_list_of_vectors = FALSE`)", {
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = c(2, 1),
      to = list(
        "a",
        "b"
      ),
      from_as_list_of_vectors = FALSE,
      to_as_list_of_vectors = TRUE
    ),
    c("b", "a", NA)
  )
})

test_that("from_as_list_of_vectors = FALSE / to_as_list_of_vectors = FALSE", {
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3),
      from = c(2, 1),
      to = c("a", "b"),
      from_as_list_of_vectors = FALSE,
      to_as_list_of_vectors = FALSE
    ),
    c("b", "a", NA)
  )
})

test_that("can treat list input as a vector (i.e. not as a container of vectors)", {
  # This is why we have `from_as_list_of_vectors` and `to_as_list_of_vectors` as arguments
  expect_identical(
    vec_recode_values(
      x = list(1, 2, 3),
      from = list(2, 5, 1),
      to = list("x", 1:2, 1:3),
    ),
    list(1:3, "x", NULL)
  )
})

test_that("`to` names are kept during `from_as_list_of_vectors, !to_as_list_of_vectors` optimization recycling", {
  expect_identical(
    vec_recode_values(
      x = c(2, 1, 4, 5),
      from = list(1:2, 3:4, 5:6),
      to = c(a = "x", b = "y", c = "z"),
      from_as_list_of_vectors = TRUE
    ),
    c(a = "x", a = "x", b = "y", c = "z")
  )
})

test_that("`to` list names are dropped with `to_as_list_of_vectors`", {
  # With `to_as_list_of_vectors` optimization
  expect_identical(
    vec_recode_values(
      x = c(1, 1),
      from = 1,
      to = list(a = "x"),
      to_as_list_of_vectors = TRUE
    ),
    c("x", "x")
  )
  # Without `to_as_list_of_vectors` optimization
  expect_identical(
    vec_recode_values(
      x = c(1, 1),
      from = 1,
      to = list(a = c("x", "y")),
      to_as_list_of_vectors = TRUE
    ),
    c("x", "y")
  )
})

test_that("`to` is recycled to `from_size` of `to_as_list_of_vectors`", {
  # With `to_as_list_of_vectors`
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1, 4),
      from = c(1, 2),
      to = list(2),
      to_as_list_of_vectors = TRUE
    ),
    c(2, 2, NA, 2, NA)
  )
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1, 4),
      from = c(1, 2),
      to = list(c(2, 3, 4, 5, 6)),
      to_as_list_of_vectors = TRUE
    ),
    c(2, 3, NA, 5, NA)
  )
  # This doesn't make too much sense for a user to provide, but is consistent
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1, 4),
      from = list(c(1, 2), 4),
      to = list(2),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c(2, 2, NA, 2, 2)
  )
  # This doesn't make too much sense for a user to provide, but is consistent
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1, 4),
      from = list(c(1, 2), 4),
      to = list(c(2, 3, 4, 5, 6)),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c(2, 3, NA, 5, 6)
  )

  # Without `to_as_list_of_vectors`
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1),
      from = c(1, 2),
      to = 2
    ),
    c(2, 2, NA, 2)
  )
  expect_identical(
    vec_recode_values(
      x = c(1, 2, 3, 1, 4),
      from = list(c(1, 2), 4),
      to = 2,
      from_as_list_of_vectors = TRUE
    ),
    c(2, 2, NA, 2, 2)
  )
})

test_that("`from` list names are dropped with `from_as_list_of_vectors`", {
  # With `from_as_list_of_vectors, !to_as_list_of_vectors` optimization
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = list(a = 1, b = c(2, 3, 4)),
      to = c("x", "y"),
      from_as_list_of_vectors = TRUE
    ),
    c("x", "y")
  )
  # Without `from_as_list_of_vectors, !to_as_list_of_vectors` optimization
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = list(a = 1, b = c(2, 3, 4)),
      to = list(x = c("x1", "x2"), y = c("y1", "y2")),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c("x1", "y2")
  )
})

test_that("`vec_replace_values()` retains names of `x`", {
  # Mimicking `[<-` and `base::replace()`.
  # Note how `vec_recode_values()` "creates a new vector",
  # so it pulls the names from `to` and `default`.
  expect_identical(
    vec_replace_values(
      x = c(a = 1, b = 2, c = 0),
      from = c(2, 1),
      to = c(x = 3, y = 4)
    ),
    c(a = 4, b = 3, c = 0)
  )
  expect_identical(
    vec_recode_values(
      x = c(a = 1, b = 2, c = 0),
      from = c(2, 1),
      to = c(x = 3, y = 4)
    ),
    c(y = 4, x = 3, NA)
  )
})

test_that("`unmatched` errors are correct", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(c(1, 2), from = 1, to = 0, unmatched = "error")
  })

  expect_snapshot(error = TRUE, {
    # `NA` must be matched!
    vec_recode_values(c(1, NA), from = 1, to = 0, unmatched = "error")
  })

  expect_snapshot(error = TRUE, {
    # Many locations
    vec_recode_values(1:100, from = 1, to = 0, unmatched = "error")
  })
})

test_that("`x` and `from` common type errors are correct", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = "a", to = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = list("a"),
      to = 1,
      from_as_list_of_vectors = TRUE
    )
  })
})

test_that("`to` and `default` `ptype` errors are correct when it is inferred", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1:2,
      to = list(1, "x"),
      to_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1:2,
      to = list(1, 2),
      default = "x",
      to_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1:2, to = 1, default = "x")
  })
})

test_that("`to` and `default` `ptype` errors are correct when it is user supplied", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = 1, ptype = foobar())
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = 1, ptype = character())
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1,
      to = list(a = 1),
      ptype = character(),
      to_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = "x", default = 1, ptype = character())
  })
})

test_that("`to` size is validated", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1:5, from = 1, to = 2:3)
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1:5,
      from = list(1),
      to = 2:3,
      from_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1:5,
      from = 1,
      to = list(2, 3),
      to_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1:5,
      from = list(1),
      to = list(2, 3),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1:5,
      from = 1,
      to = list(a = 2:3),
      to_as_list_of_vectors = TRUE
    )
  })
})

test_that("`default` size is validated", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1:5, from = 1, to = 2, default = 1:2)
  })
})

test_that("`x` must be a vector", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(foobar(), from = 1, to = 2, x_arg = ".x")
  })
})

test_that("`from` must be a vector or list of vectors", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = foobar(), to = 2, from_arg = ".from")
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1,
      to = 2,
      from_as_list_of_vectors = TRUE,
      from_arg = ".from"
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = list(a = foobar()),
      to = 2,
      from_as_list_of_vectors = TRUE,
      from_arg = ".from"
    )
  })
})

test_that("`to` must be a vector or list of vectors", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = foobar(), to_arg = ".to")
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1,
      to = 2,
      to_as_list_of_vectors = TRUE,
      to_arg = ".to"
    )
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1,
      to = list(a = foobar()),
      to_as_list_of_vectors = TRUE,
      to_arg = ".to"
    )
  })
})

test_that("`default` must be a vector", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(
      1,
      from = 1,
      to = 2,
      default = foobar(),
      default_arg = ".default"
    )
  })
})

test_that("`from_as_list_of_vectors` and `to_as_list_of_vectors` are validated", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = 1, from_as_list_of_vectors = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = 1, to_as_list_of_vectors = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_replace_values(1, from = 1, to = 1, from_as_list_of_vectors = "x")
  })
  expect_snapshot(error = TRUE, {
    vec_replace_values(1, from = 1, to = 1, to_as_list_of_vectors = "x")
  })
})

test_that("`unmatched` is validated", {
  expect_snapshot(error = TRUE, {
    vec_recode_values(1, from = 1, to = 1, unmatched = "e")
  })
})

test_that("proof that `ptype` finalization is important", {
  # Imagine you have an input logical vector you are remapping
  # and it happens to only have `NA`s
  x <- c(NA, NA)
  from <- NA
  to <- FALSE

  # If no `ptype` finalization happened, then `ptype = x` would result in
  # `unspecified` being the output type and these would error
  expect_identical(
    vec_recode_values(x, from = from, to = to, default = x, ptype = x),
    c(FALSE, FALSE)
  )
  expect_identical(
    vec_replace_values(x, from = from, to = to),
    c(FALSE, FALSE)
  )
})

test_that("common `ptype` of `to` isn't finalized until `default` has been included", {
  # If the common type of `to` is finalized early, we get `logical`, which can't
  # combine with `default`'s `character` type
  expect_identical(
    vec_recode_values(
      x = "a",
      from = "a",
      to = list(NA),
      default = "x",
      to_as_list_of_vectors = TRUE
    ),
    NA_character_
  )
  expect_identical(
    vec_recode_values(
      x = "a",
      from = "b",
      to = list(NA),
      default = "x",
      to_as_list_of_vectors = TRUE
    ),
    "x"
  )
})

test_that("extraneous `to` attributes don't end up on the final output", {
  x <- c(1, 2, 3)

  # TODO: Ideally the attributes wouldn't show up on the output, but
  # `list_combine()` doesn't clear them because `vec_ptype(to)` retains
  # them for some reason
  # https://github.com/r-lib/vctrs/issues/2025
  from <- c(2, 3)
  to <- structure(c(0, -1), foo = "bar")
  expect_identical(
    vec_recode_values(x, from = from, to = to),
    structure(c(NA, 0, -1), foo = "bar")
  )

  # TODO: Ideally the attributes wouldn't show up on the output, but
  # `list_combine()` doesn't clear them because `vec_ptype(to)` retains
  # them for some reason
  # https://github.com/r-lib/vctrs/issues/2025
  from <- 2
  to <- list(
    structure(c(0, -1, -2), foo = "bar")
  )
  expect_identical(
    vec_recode_values(x, from = from, to = to, to_as_list_of_vectors = TRUE),
    structure(c(NA, -1, NA), foo = "bar")
  )

  # Note that as soon as you force a `ptype2` computation, the attributes
  # disappear anyways, suggesting an inconsistency

  # `ptype2` forced by `default`
  from <- c(2, 3)
  to <- structure(c(0, -1), foo = "bar")
  expect_identical(
    vec_recode_values(x, from = from, to = to, default = NA_real_),
    c(NA, 0, -1)
  )

  # `ptype2` forced by multiple `to` values
  from <- c(2, 3)
  to <- list(
    structure(c(0, -1, -2), foo = "bar"),
    c(-3, -4, -5)
  )
  expect_identical(
    vec_recode_values(x, from = from, to = to, to_as_list_of_vectors = TRUE),
    c(NA, -1, -5)
  )
})

test_that("extraneous `x` attributes don't end up on the final output", {
  # TODO: Ideally the attributes wouldn't show up on the output, but
  # `list_combine()` doesn't clear them because `vec_ptype(x)` retains
  # them for some reason
  # https://github.com/r-lib/vctrs/issues/2025
  x <- structure(1, foo = "bar")

  expect_identical(
    vec_replace_values(x, from = 1, to = 2),
    structure(2, foo = "bar")
    # 2
  )
})

test_that("first `from` wins when there are overlaps", {
  # Same as `vec_case_when()`
  expect_identical(
    vec_recode_values(
      x = 1,
      from = c(1, 1),
      to = c("a", "b")
    ),
    "a"
  )
  expect_identical(
    vec_recode_values(
      x = 1,
      from = list(c(1, 3), c(1, 2)),
      to = c("a", "b"),
      from_as_list_of_vectors = TRUE
    ),
    "a"
  )
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = c(1, 1),
      to = list(c("a", "b"), c("c", "d")),
      to_as_list_of_vectors = TRUE
    ),
    c("a", NA)
  )
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = list(c(1, 3), c(1, 2)),
      to = list(c("a", "b"), c("c", "d")),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c("a", "d")
  )
})

test_that("works when `from` is a list of size 1 elements and `to` doesn't simplify", {
  # This is a case where we don't actually build the `from_map` because the
  # `from` size doesn't change as it flattens
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = list(1, 2),
      to = list(c("a", "b"), c("c", "d")),
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    c("a", "d")
  )
})

test_that("works when `to` is a length >1 vector and every element of `x` is matched by `from`", {
  # This is an optimized case where we directly use the index provided by
  # `vec_match()` to slice `to` with
  expect_identical(
    vec_recode_values(
      x = c(1, 2),
      from = c(2, 1),
      to = c("a", "b")
    ),
    c("b", "a")
  )
})

test_that("data frames - vector `from`, vector `to`", {
  x <- data_frame(a = 1:3, b = 3:5)
  from <- data_frame(a = int(3, 1), b = c(5, 3))

  # Recycling
  to <- data_frame(c = "a", d = "y")
  expect_identical(
    vec_recode_values(x, from = from, to = to),
    data_frame(c = c("a", NA, "a"), d = c("y", NA, "y"))
  )

  to <- data_frame(c = c("a", "b"), d = c("x", "y"))
  expect_identical(
    vec_recode_values(x, from = from, to = to),
    data_frame(c = c("b", NA, "a"), d = c("y", NA, "x"))
  )
  # List `from`, vector `to`

  # List `from`, list `to`
})

test_that("data frames - vector `from`, list `to`", {
  x <- data_frame(a = 1:3, b = 3:5)
  from <- data_frame(a = int(3, 1), b = c(5, 3))

  # Recycling to `from` size and `x` size
  to <- list(
    data_frame(c = "a", d = "y")
  )
  expect_identical(
    vec_recode_values(x, from = from, to = to, to_as_list_of_vectors = TRUE),
    data_frame(c = c("a", NA, "a"), d = c("y", NA, "y"))
  )

  # Recycling to `from` size
  to <- list(
    data_frame(c = c("a", "b", "c"), d = c("x", "y", "z"))
  )
  expect_identical(
    vec_recode_values(x, from = from, to = to, to_as_list_of_vectors = TRUE),
    data_frame(c = c("a", NA, "c"), d = c("x", NA, "z"))
  )

  # Recycling to `x` size
  to <- list(
    data_frame(c = "a", d = "x"),
    data_frame(c = "b", d = "y")
  )
  expect_identical(
    vec_recode_values(x, from = from, to = to, to_as_list_of_vectors = TRUE),
    data_frame(c = c("b", NA, "a"), d = c("y", NA, "x"))
  )
})

test_that("data frames - list `from`, vector `to`", {
  x <- data_frame(a = 1:3, b = 3:5)

  # Recycling `to` to `from` size (this would be strange)
  from <- list(
    data_frame(a = 1L, b = 3L),
    data_frame(a = 3:4, b = 5:6)
  )
  to <- data_frame(c = "a", d = "x")
  expect_identical(
    vec_recode_values(x, from = from, to = to, from_as_list_of_vectors = TRUE),
    data_frame(c = c("a", NA, "a"), d = c("x", NA, "x"))
  )

  from <- list(
    data_frame(a = 1L, b = 3L),
    data_frame(a = 3:4, b = 5:6)
  )
  to <- data_frame(c = c("a", "b"), d = c("x", "y"))
  expect_identical(
    vec_recode_values(x, from = from, to = to, from_as_list_of_vectors = TRUE),
    data_frame(c = c("a", NA, "b"), d = c("x", NA, "y"))
  )
})

test_that("data frames - list `from`, list `to`", {
  x <- data_frame(a = 1:3, b = 3:5)

  # Recycling `to` to `from` size (this would be strange)
  # Recycling `to` elements to `x` size
  from <- list(
    data_frame(a = 1L, b = 3L),
    data_frame(a = 3:4, b = 5:6)
  )
  to <- list(
    data_frame(c = "a", d = "x")
  )
  expect_identical(
    vec_recode_values(
      x,
      from = from,
      to = to,
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    data_frame(c = c("a", NA, "a"), d = c("x", NA, "x"))
  )

  # Recycling `to` elements to `x` size
  from <- list(
    data_frame(a = 1L, b = 3L),
    data_frame(a = 3:4, b = 5:6)
  )
  to <- list(
    data_frame(c = "a", d = "x"),
    data_frame(c = "b", d = "y")
  )
  expect_identical(
    vec_recode_values(
      x,
      from = from,
      to = to,
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    data_frame(c = c("a", NA, "b"), d = c("x", NA, "y"))
  )

  from <- list(
    data_frame(a = 1L, b = 3L),
    data_frame(a = 3:4, b = 5:6)
  )
  to <- list(
    data_frame(c = c("a", "b", "c"), d = c("x", "y", "z")),
    data_frame(c = c("aa", "bb", "cc"), d = c("xx", "yy", "zz"))
  )
  expect_identical(
    vec_recode_values(
      x,
      from = from,
      to = to,
      from_as_list_of_vectors = TRUE,
      to_as_list_of_vectors = TRUE
    ),
    data_frame(c = c("a", NA, "cc"), d = c("x", NA, "zz"))
  )
})

#' Compute ranks
#'
#' `vec_rank()` computes the sample ranks of a vector. For data frames, ranks
#' are computed along the rows, using all columns after the first to break
#' ties.
#'
#' @details
#' Unlike [base::rank()], when `incomplete = "rank"` all missing values are
#' given the same rank, rather than an increasing sequence of ranks. When
#' `nan_distinct = FALSE`, `NaN` values are given the same rank as `NA`,
#' otherwise they are given a rank that differentiates them from `NA`.
#'
#' Like [vec_order_radix()], ordering is done in the C-locale. This can affect
#' the ranks of character vectors, especially regarding how uppercase and
#' lowercase letters are ranked. See the documentation of [vec_order_radix()]
#' for more information.
#'
#' @inheritParams order-radix
#' @inheritParams rlang::args_dots_empty
#'
#' @param ties Ranking of duplicate values.
#'   - `"min"`: Use the current rank for all duplicates. The next non-duplicate
#'   value will have a rank incremented by the number of duplicates present.
#'
#'   - `"max"`: Use the current rank `+ n_duplicates - 1` for all duplicates.
#'   The next non-duplicate value will have a rank incremented by the number of
#'   duplicates present.
#'
#'   - `"sequential"`: Use an increasing sequence of ranks starting at the
#'   current rank, applied to duplicates in order of appearance.
#'
#'   - `"dense"`: Use the current rank for all duplicates. The next
#'   non-duplicate value will have a rank incremented by `1`, effectively
#'   removing any gaps in the ranking.
#'
#' @param incomplete Ranking of missing and [incomplete][vec_detect_complete]
#'   observations.
#'
#'   - `"rank"`: Rank incomplete observations normally. Missing values within
#'   incomplete observations will be affected by `na_value` and `nan_distinct`.
#'
#'   - `"na"`: Don't rank incomplete observations at all. Instead, they are
#'   given a rank of `NA`. In this case, `na_value` and `nan_distinct` have
#'   no effect.
#'
#' @section Dependencies:
#'
#' - [vec_order_radix()]
#' - [vec_slice()]
#'
#' @export
#' @examples
#' x <- c(5L, 6L, 3L, 3L, 5L, 3L)
#'
#' vec_rank(x, ties = "min")
#' vec_rank(x, ties = "max")
#'
#' # Sequential ranks use an increasing sequence for duplicates
#' vec_rank(x, ties = "sequential")
#'
#' # Dense ranks remove gaps between distinct values,
#' # even if there are duplicates
#' vec_rank(x, ties = "dense")
#'
#' y <- c(NA, x, NA, NaN)
#'
#' # Incomplete values match other incomplete values by default, and their
#' # overall position can be adjusted with `na_value`
#' vec_rank(y, na_value = "largest")
#' vec_rank(y, na_value = "smallest")
#'
#' # NaN can be ranked separately from NA if required
#' vec_rank(y, nan_distinct = TRUE)
#'
#' # Rank in descending order. Since missing values are the largest value,
#' # they are given a rank of `1` when ranking in descending order.
#' vec_rank(y, direction = "desc", na_value = "largest")
#'
#' # Give incomplete values a rank of `NA` by setting `incomplete = "na"`
#' vec_rank(y, incomplete = "na")
#'
#' # Can also rank data frames, using columns after the first to break ties
#' z <- c(2L, 3L, 4L, 4L, 5L, 2L)
#' df <- data_frame(x = x, z = z)
#' df
#'
#' vec_rank(df)
vec_rank <- function(
  x,
  ...,
  ties = c("min", "max", "sequential", "dense"),
  incomplete = c("rank", "na"),
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
) {
  check_dots_empty0(...)

  ties <- arg_match0(ties, c("min", "max", "sequential", "dense"), "ties")
  incomplete <- arg_match0(incomplete, c("rank", "na"), "incomplete")

  .Call(
    vctrs_rank,
    x,
    ties,
    incomplete,
    direction,
    na_value,
    nan_distinct,
    chr_proxy_collate
  )
}

#' Is a vector empty
#'
#' @description
#'
#' `r lifecycle::badge("defunct")`
#'
#' This function is defunct, please use [vec_is_empty()].
#'
#' @param x An object.
#'
#' @keywords internal
#' @export
vec_empty <- function(x) {
  # Defunct: 2019-06
  lifecycle::deprecate_stop(
    when = "0.2.0",
    what = "vec_empty()",
    with = "vec_is_empty()"
  )
}

#' Deprecated type functions
#'
#' @description
#'
#' `r lifecycle::badge("deprecated")`
#'
#' These functions have been renamed:
#'
#' * `vec_type()` => [vec_ptype()]
#' * `vec_type2()` => [vec_ptype2()]
#' * `vec_type_common()` => [vec_ptype_common()]
#'
#' @param x,y,...,.ptype Arguments for deprecated functions.
#'
#' @keywords internal
#' @export
vec_type <- function(x) {
  # Deprecated: 2019-06
  lifecycle::deprecate_warn(
    when = "0.2.0",
    what = "vec_type()",
    with = "vec_ptype()",
    always = TRUE
  )
  vec_ptype(x)
}
#' @rdname vec_type
#' @export
vec_type_common <- function(..., .ptype = NULL) {
  # Deprecated: 2019-06
  lifecycle::deprecate_warn(
    when = "0.2.0",
    what = "vec_type_common()",
    with = "vec_ptype_common()",
    always = TRUE
  )
  vec_ptype_common(..., .ptype = .ptype)
}
#' @rdname vec_type
#' @export
vec_type2 <- function(x, y, ...) {
  # Deprecated: 2019-06
  lifecycle::deprecate_warn(
    when = "0.2.0",
    what = "vec_type2()",
    with = "vec_ptype2()",
    always = TRUE
  )
  vec_ptype2(x, y, ...)
}

#' Convert to an index vector
#'
#' @description
#'
#' `r lifecycle::badge("deprecated")`
#'
#' `vec_as_index()` has been renamed to [vec_as_location()] and is
#' deprecated as of vctrs 0.2.2.
#'
#' @inheritParams vec_as_location
#'
#' @keywords internal
#' @export
vec_as_index <- function(i, n, names = NULL) {
  # Soft-deprecated: 2020-01
  lifecycle::deprecate_soft(
    when = "0.2.2",
    what = "vec_as_index()",
    with = "vec_as_location()"
  )
  n <- vec_cast(n, integer())
  vec_check_size(n, size = 1L)
  i <- vec_as_subscript(i)

  # Picked up from the environment at the C level
  arg <- NULL

  .Call(
    ffi_as_location,
    i = i,
    n = n,
    names = names,
    loc_negative = "invert",
    loc_oob = "error",
    loc_zero = "remove",
    missing = "propagate",
    env = environment()
  )
}

#' Expand the length of a vector
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `vec_repeat()` has been replaced with [vec_rep()] and [vec_rep_each()] and is
#' deprecated as of vctrs 0.3.0.
#'
#' @param x A vector.
#' @param each Number of times to repeat each element of `x`.
#' @param times Number of times to repeat the whole vector of `x`.
#' @return A vector the same type as `x` with size `vec_size(x) * times * each`.
#' @keywords internal
#' @export
vec_repeat <- function(x, each = 1L, times = 1L) {
  # Soft-deprecated: 2020-03
  lifecycle::deprecate_soft(
    when = "0.3.0",
    what = "vec_repeat()",
    with = I("either `vec_rep()` or `vec_rep_each()`")
  )

  vec_check_size(each, size = 1L)
  vec_check_size(times, size = 1L)

  idx <- rep(vec_seq_along(x), times = times, each = each)
  vec_slice(x, idx)
}

#' Chopping
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `vec_unchop()` has been renamed to [list_unchop()] and is deprecated as of
#' vctrs 0.5.0.
#'
#' @inheritParams list_unchop
#' @inherit list_unchop return
#'
#' @keywords internal
#' @export
vec_unchop <- function(
  x,
  indices = NULL,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c("minimal", "unique", "check_unique", "universal")
) {
  # Soft-deprecated: 2022-09
  lifecycle::deprecate_soft("0.5.0", "vec_unchop()", "list_unchop()")

  list_unchop(
    x = x,
    indices = indices,
    ptype = ptype,
    name_spec = name_spec,
    name_repair = name_repair
  )
}

#' Missing values
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' `vec_equal_na()` has been renamed to [vec_detect_missing()] and is deprecated
#' as of vctrs 0.5.0.
#'
#' @inheritParams vec_detect_missing
#'
#' @return
#' A logical vector the same size as `x`.
#'
#' @keywords internal
#' @export
vec_equal_na <- function(x) {
  # Soft-deprecated: 2022-09
  lifecycle::deprecate_soft("0.5.0", "vec_equal_na()", "vec_detect_missing()")
  vec_detect_missing(x)
}

#' List checks
#'
#' @description
#' `r lifecycle::badge("deprecated")`
#'
#' These functions have been deprecated as of vctrs 0.6.0.
#'
#' - `vec_is_list()` has been renamed to [obj_is_list()].
#' - `vec_check_list()` has been renamed to [obj_check_list()].
#'
#' @inheritParams obj_is_list
#'
#' @keywords internal
#' @export
vec_is_list <- function(x) {
  # Silently-deprecated: 2023-03
  # lifecycle::deprecate_soft("0.6.0", "vec_is_list()", "obj_is_list()")
  obj_is_list(x)
}

#' @rdname vec_is_list
#' @export
vec_check_list <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
  # Silently-deprecated: 2023-03
  # lifecycle::deprecate_soft("0.6.0", "vec_check_list()", "obj_check_list()")
  obj_check_list(x, ..., arg = arg, call = call)
}

test_that("vec_as_index() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(vec_as_index(-2, 10), vec_as_location(-2, 10))
  expect_identical(
    vec_as_index("cyl", length(mtcars), names(mtcars)),
    vec_as_location("cyl", length(mtcars), names(mtcars))
  )
})

test_that("vec_repeat() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(vec_repeat(1:2, times = 2), vec_rep(1:2, 2))
  expect_identical(vec_repeat(1:2, each = 2), vec_rep_each(1:2, 2))
})

test_that("vec_unchop() is soft-deprecated", {
  local_options(lifecycle_verbosity = "warning")
  expect_snapshot(vec_unchop(list(1), indices = list(1)))
})

test_that("vec_unchop() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(
    vec_unchop(list(1L, 2:3), indices = list(2, c(3, 1))),
    c(3L, 1L, 2L)
  )
})

test_that("vec_equal_na() is soft-deprecated", {
  local_options(lifecycle_verbosity = "warning")
  expect_snapshot(vec_equal_na(c(1, NA)))
})

test_that("vec_equal_na() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(
    vec_equal_na(c(1, NA, 2, NA)),
    c(FALSE, TRUE, FALSE, TRUE)
  )
})

test_that("vec_is_list() still works", {
  expect_false(vec_is_list(1))
  expect_true(vec_is_list(list()))
})

test_that("vec_check_list() still works", {
  my_check <- function(x) vec_check_list(x)

  expect_snapshot(error = TRUE, {
    vec_check_list(1)
  })
  expect_snapshot(error = TRUE, {
    my_check(1)
  })
})

test_that("vec_as_index() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(vec_as_index(-2, 10), vec_as_location(-2, 10))
  expect_identical(
    vec_as_index("cyl", length(mtcars), names(mtcars)),
    vec_as_location("cyl", length(mtcars), names(mtcars))
  )
})

test_that("vec_repeat() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(vec_repeat(1:2, times = 2), vec_rep(1:2, 2))
  expect_identical(vec_repeat(1:2, each = 2), vec_rep_each(1:2, 2))
})

test_that("vec_unchop() is soft-deprecated", {
  local_options(lifecycle_verbosity = "warning")
  expect_snapshot(vec_unchop(list(1), indices = list(1)))
})

test_that("vec_unchop() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(
    vec_unchop(list(1L, 2:3), indices = list(2, c(3, 1))),
    c(3L, 1L, 2L)
  )
})

test_that("vec_equal_na() is soft-deprecated", {
  local_options(lifecycle_verbosity = "warning")
  expect_snapshot(vec_equal_na(c(1, NA)))
})

test_that("vec_equal_na() still works", {
  local_options(lifecycle_verbosity = "quiet")
  expect_identical(
    vec_equal_na(c(1, NA, 2, NA)),
    c(FALSE, TRUE, FALSE, TRUE)
  )
})

test_that("vec_is_list() still works", {
  expect_false(vec_is_list(1))
  expect_true(vec_is_list(list()))
})

test_that("vec_check_list() still works", {
  my_check <- function(x) vec_check_list(x)

  expect_snapshot(error = TRUE, {
    vec_check_list(1)
  })
  expect_snapshot(error = TRUE, {
    my_check(1)
  })
})

#' Interleave many vectors into one vector
#'
#' @description
#' `vec_interleave()` combines multiple vectors together, much like [vec_c()],
#' but does so in such a way that the elements of each vector are interleaved
#' together.
#'
#' It is a more efficient equivalent to the following usage of `vec_c()`:
#'
#' ```
#' vec_interleave(x, y) == vec_c(x[1], y[1], x[2], y[2], ..., x[n], y[n])
#' ```
#'
#' @section Dependencies:
#'
#' ## vctrs dependencies
#'
#' - [list_combine()]
#'
#' @inheritParams vec_c
#'
#' @param ... Vectors to interleave.
#'
#' @param .size The expected size of each vector. If not provided, computed
#'   automatically by [vec_size_common()]. Each vector will be
#'   [recycled][theory-faq-recycling] to this size.
#'
#' @param .ptype The expected type of each vector. If not provided, computed
#'   automatically by [vec_ptype_common()]. Each vector will be
#'   [cast][theory-faq-coercion] to this type.
#'
#' @export
#' @examples
#' # The most common case is to interleave two vectors
#' vec_interleave(1:3, 4:6)
#'
#' # But you aren't restricted to just two
#' vec_interleave(1:3, 4:6, 7:9, 10:12)
#'
#' # You can also interleave data frames
#' x <- data_frame(x = 1:2, y = c("a", "b"))
#' y <- data_frame(x = 3:4, y = c("c", "d"))
#'
#' vec_interleave(x, y)
#'
#' # `.size` can be used to recycle size 1 elements before interleaving
#' vec_interleave(1, 2, .size = 3)
#'
#' # `.ptype` can be used to enforce a particular type
#' typeof(vec_interleave(1, 2, .ptype = integer()))
vec_interleave <- function(
  ...,
  .size = NULL,
  .ptype = NULL,
  .name_spec = NULL,
  .name_repair = c(
    "minimal",
    "unique",
    "check_unique",
    "universal",
    "unique_quiet",
    "universal_quiet"
  ),
  .error_call = current_env()
) {
  list_interleave(
    x = list2(...),
    size = .size,
    ptype = .ptype,
    name_spec = .name_spec,
    name_repair = .name_repair,
    x_arg = "",
    error_call = .error_call
  )
}

# It's sometimes more convenient to supply a list, plus you get access to
# `x_arg` for better error messages than you get from `vec_interleave(!!!x)`.
# We could consider exporting this alongside `vec_interleave()`.
list_interleave <- function(
  x,
  ...,
  size = NULL,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c(
    "minimal",
    "unique",
    "check_unique",
    "universal",
    "unique_quiet",
    "universal_quiet"
  ),
  x_arg = caller_arg(x),
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_list_interleave,
    x,
    size,
    ptype,
    name_spec,
    name_repair,
    environment()
  )
}

test_that("interleaving is working as expected", {
  expect_identical(
    vec_interleave(1:3, 4:6),
    c(1L, 4L, 2L, 5L, 3L, 6L)
  )
  expect_identical(
    vec_interleave(1:3, 4:6, 7:9),
    c(1L, 4L, 7L, 2L, 5L, 8L, 3L, 6L, 9L)
  )
})

test_that("data frames can be interleaved", {
  x <- data_frame(x = 1:2, y = c("a", "b"))
  y <- data_frame(x = 3:4, y = c("c", "d"))

  expect_identical(
    vec_interleave(x, y),
    vec_slice(vec_c(x, y), c(1, 3, 2, 4))
  )
})

test_that("works with `NULL` inputs", {
  expect_identical(
    vec_interleave(1:3, NULL, 4:6),
    vec_interleave(1:3, 4:6)
  )
})

test_that("allows for name repair", {
  x <- c(x = 1)

  expect_identical(
    vec_interleave(x, x),
    c(x = 1, x = 1)
  )

  expect_snapshot(vec_interleave(x, x, .name_repair = "unique"))
})

test_that("can repair names quietly", {
  local_name_repair_verbose()

  expect_snapshot({
    res_unique <- vec_interleave(
      c(x = 1),
      c(x = 2),
      .name_repair = "unique_quiet"
    )
    res_universal <- vec_interleave(
      c("if" = 1),
      c("in" = 2),
      .name_repair = "universal_quiet"
    )
  })
  expect_named(res_unique, c("x...1", "x...2"))
  expect_named(res_universal, c(".if", ".in"))
})

test_that("works with name specs", {
  x <- c(x = 1)
  y <- 1

  expect_named(
    vec_interleave(x = x, y = y, .name_spec = "{outer}_{inner}"),
    c("x_x", "y")
  )
})

test_that("recycles inputs", {
  expect_identical(
    vec_interleave(1:3, NA),
    c(1L, NA, 2L, NA, 3L, NA)
  )
  expect_identical(
    vec_interleave(integer(), NA),
    integer()
  )
})

test_that("works with no inputs", {
  # Purposefully returns `unspecified()`, which is the more useful result
  # for generic programming against this
  expect_identical(vec_interleave(), unspecified())
  expect_identical(vec_interleave(NULL), unspecified())

  # `.size` affects the size of each element, thus it doesn't affect the output
  # when there are 0 elements
  expect_identical(vec_interleave(.size = 2), unspecified())
  expect_identical(vec_interleave(NULL, .size = 2), unspecified())
})

test_that("works with length zero input", {
  expect_identical(vec_interleave(integer(), integer()), integer())
})

test_that("respects `.ptype`", {
  expect_identical(vec_interleave(.ptype = character()), character())
  expect_identical(vec_interleave(NULL, .ptype = character()), character())

  expect_identical(vec_interleave(1L, 2L, .ptype = numeric()), c(1, 2))
})

test_that("reports type errors", {
  expect_snapshot(error = TRUE, {
    vec_interleave(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1, "x", .error_call = quote(foo()))
  })

  expect_snapshot(error = TRUE, {
    vec_interleave(1, "x", .ptype = double())
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1, "x", .ptype = double(), .error_call = quote(foo()))
  })

  # Index is right even with `NULL`!
  expect_snapshot(error = TRUE, {
    vec_interleave(1, NULL, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1, NULL, "x", .ptype = double())
  })
})

test_that("respects `.size`", {
  # Correctly does not report an error here
  expect_identical(
    vec_interleave(1:2, 3:4, .size = 2),
    c(1L, 3L, 2L, 4L)
  )

  # Useful for recycling to a known element size
  # in the case of all size 1 elements
  expect_identical(
    vec_interleave(1, 2, .size = 2),
    c(1, 2, 1, 2)
  )
})

test_that("reports recycling errors", {
  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, 1:3)
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, 1:3, .error_call = quote(foo()))
  })

  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, 3:4, .size = 3)
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, 3:4, .size = 3, .error_call = quote(foo()))
  })

  # Index is right even with `NULL`!
  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, NULL, 1:3)
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1:2, NULL, 1:3, .size = 2)
  })
})

test_that("reports scalar errors", {
  expect_snapshot(error = TRUE, {
    vec_interleave(lm(1 ~ 1))
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(lm(1 ~ 1), .error_call = quote(foo()))
  })

  # Index is right even with `NULL`!
  expect_snapshot(error = TRUE, {
    vec_interleave(1, NULL, lm(1 ~ 1))
  })
  expect_snapshot(error = TRUE, {
    vec_interleave(1, NULL, lm(1 ~ 1), .error_call = quote(foo()))
  })
})

test_that("`list_interleave()` checks for a list", {
  expect_snapshot(error = TRUE, {
    list_interleave(1)
  })
})

#' Find the prototype of a set of vectors
#'
#' @description
#' - `vec_ptype()` returns the [unfinalised][vec_ptype_finalise] prototype of a
#'   single vector.
#'
#' - `vec_ptype_common()` returns the common type of multiple vectors. By
#'   default, this is [finalised][vec_ptype_finalise] for immediate usage, but
#'   can optionally be left unfinalised for advanced common type determination.
#'
#' - `vec_ptype_show()` nicely prints the common type of any number of inputs,
#'   and is designed for interactive exploration.
#'
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector
#'
#' @param ... For `vec_ptype()`, these dots are for future extensions and must
#'   be empty.
#'
#'   For `vec_ptype_common()` and `vec_ptype_show()`, vector inputs.
#'
#' @param x_arg Argument name for `x`. This is used in error messages to inform
#'   the user about the locations of incompatible types.
#'
#' @param .ptype If `NULL`, the default, the output type is determined by
#'   computing the common type across all elements of `...`.
#'
#'   Alternatively, you can supply `.ptype` to give the output known type.
#'   If `getOption("vctrs.no_guessing")` is `TRUE` you must supply this value:
#'   this is a convenient way to make production code demand fixed types.
#'
#' @param .finalise Should `vec_ptype_common()` [finalise][vec_ptype_finalise]
#'   its output?
#'
#'   - If `TRUE`, [vec_ptype_finalise()] is called on the final `ptype` before
#'     it is returned. Practically this has the effect of converting any
#'     types from [unspecified] to logical.
#'
#'   - If `FALSE`, [unspecified] types are left unfinalised, which can be useful
#'     for advanced cases where you combine one common type result with another
#'     type via [vec_ptype2()]. Note that you must manually call
#'     [vec_ptype_finalise()] on the final `ptype` before supplying it to any
#'     other vctrs functions.
#'
#' @return `vec_ptype()` and `vec_ptype_common()` return a prototype
#'   (a size-0 vector).
#'
#' @section `vec_ptype()`:
#'
#' `vec_ptype()` returns [size][vec_size] 0 vectors potentially
#' containing attributes but no data. Generally, this is just
#' `vec_slice(x, 0L)`, but some inputs require special
#' handling.
#'
#' * While you can't slice `NULL`, the prototype of `NULL` is
#'   itself. This is because we treat `NULL` as an identity value in
#'   the `vec_ptype2()` monoid.
#'
#' * The prototype of logical vectors that only contain missing values
#'   is the special [unspecified] type, which can be coerced to any
#'   other 1d type. This allows bare `NA`s to represent missing values
#'   for any 1d vector type. [Finalising][vec_ptype_finalise] this type
#'   converts it from unspecified back to logical.
#'
#' See [internal-faq-ptype2-identity] for more information about
#' identity values.
#'
#' `vec_ptype()` is a _performance_ generic. It is not necessary to implement it
#' because the default method will work for any vctrs type. However the default
#' method builds around other vctrs primitives like `vec_slice()` which incurs
#' performance costs. If your class has a static prototype, you might consider
#' implementing a custom `vec_ptype()` method that returns a constant. This will
#' improve the performance of your class in many cases ([common
#' type][vec_ptype2] imputation in particular).
#'
#' Because it may contain unspecified vectors, the prototype returned by
#' `vec_ptype()` is said to be __unfinalised__. Call [vec_ptype_finalise()] to
#' finalise it.
#'
#' @section `vec_ptype_common()`:
#'
#' `vec_ptype_common()` first finds the prototype of each input, then
#' successively calls [vec_ptype2()] to find a common type. It returns a
#' [finalised][vec_ptype_finalise] prototype by default, but can optionally be
#' left unfinalised for advanced common type determination.
#'
#' @section Dependencies of `vec_ptype()`:
#' - [vec_slice()] for returning an empty slice
#'
#' @section Dependencies of `vec_ptype_common()`:
#' - [vec_ptype2()]
#' - [vec_ptype_finalise()]
#'
#' @export
#' @examples
#' # Unknown types ------------------------------------------
#' vec_ptype_show()
#' vec_ptype_show(NULL)
#'
#' # Vectors ------------------------------------------------
#' vec_ptype_show(1:10)
#' vec_ptype_show(letters)
#' vec_ptype_show(TRUE)
#'
#' vec_ptype_show(Sys.Date())
#' vec_ptype_show(Sys.time())
#' vec_ptype_show(factor("a"))
#' vec_ptype_show(ordered("a"))
#'
#' # Matrices -----------------------------------------------
#' # The prototype of a matrix includes the number of columns
#' vec_ptype_show(array(1, dim = c(1, 2)))
#' vec_ptype_show(array("x", dim = c(1, 2)))
#'
#' # Data frames --------------------------------------------
#' # The prototype of a data frame includes the prototype of
#' # every column
#' vec_ptype_show(iris)
#'
#' # The prototype of multiple data frames includes the prototype
#' # of every column that in any data frame
#' vec_ptype_show(
#'   data.frame(x = TRUE),
#'   data.frame(y = 2),
#'   data.frame(z = "a")
#' )
#'
#' # Finalisation -------------------------------------------
#'
#' # `vec_ptype()` and `vec_ptype2()` return unfinalised ptypes so that they
#' # can be coerced to any other type
#' vec_ptype(NA)
#' vec_ptype2(NA, NA)
#'
#' # By default `vec_ptype_common()` finalises so that you can use its result
#' # directly in other vctrs functions
#' vec_ptype_common(NA, NA)
#'
#' # You can opt out of finalisation to make it work like `vec_ptype()` and
#' # `vec_ptype2()` with `.finalise = FALSE`, but don't forget that you must
#' # call `vec_ptype_finalise()` manually if you do so!
#' vec_ptype_common(NA, NA, .finalise = FALSE)
#' vec_ptype_finalise(vec_ptype_common(NA, NA, .finalise = FALSE))
#'
#' # This can be useful in rare scenarios, like including a separate `default`
#' # argument in the ptype computation
#' xs <- list(NA, NA)
#' default <- "a"
#' try(vec_ptype2(vec_ptype_common(!!!xs), default))
#' vec_ptype2(vec_ptype_common(!!!xs, .finalise = FALSE), default)
vec_ptype <- function(x, ..., x_arg = "", call = caller_env()) {
  check_dots_empty0(...)
  return(.Call(ffi_ptype, x, x_arg, environment()))
  UseMethod("vec_ptype")
}

#' @export
#' @rdname vec_ptype
vec_ptype_common <- function(
  ...,
  .ptype = NULL,
  .finalise = TRUE,
  .arg = "",
  .call = caller_env()
) {
  .External2(ffi_ptype_common, list2(...), .ptype, .finalise)
}

vec_ptype_common_params <- function(
  ...,
  .ptype = NULL,
  .finalise = TRUE,
  .fallback_opts = fallback_opts(),
  .arg = "",
  .call = caller_env()
) {
  .External2(
    ffi_ptype_common_params,
    list2(...),
    .ptype,
    .finalise,
    .fallback_opts
  )
}

vec_ptype_common_fallback <- function(
  ...,
  .ptype = NULL,
  .arg = "",
  .call = caller_env()
) {
  vec_ptype_common_params(
    ...,
    .ptype = .ptype,
    .fallback_opts = enabled_fallback_opts(),
    .arg = .arg,
    .call = .call
  )
}

#' @export
#' @rdname vec_ptype
vec_ptype_show <- function(...) {
  args <- compact(list2(...))
  n <- length(args)
  if (n == 0) {
    cat_line("Prototype: NULL")
  } else if (n == 1) {
    cat_line("Prototype: ", vec_ptype_full(args[[1]]))
  } else {
    in_types <- map(args, vec_ptype)
    out_types <- vector("list", length(in_types))
    out_types[[1]] <- in_types[[1]]
    for (i in seq2(2, n)) {
      out_types[[i]] <- vec_ptype2(out_types[[i - 1]], in_types[[i]])
    }

    in_full <- paste0("<", map_chr(in_types, vec_ptype_full), ">")
    out_full <- paste0("<", map_chr(out_types, vec_ptype_full), ">")

    out <- cbind(
      n = paste0(seq(0, n - 1), ". "),
      lhs = c("", out_full[-n]),
      comma = " , ",
      rhs = in_full,
      equals = " = ",
      res = c(in_full[[1]], out_full[-1])
    )
    out <- t(apply(out, 1, pad_height))
    out <- apply(out, 2, pad_width)

    out[, "lhs"] <- parens(out[, "lhs"])
    out[, "rhs"] <- parens(out[, "rhs"], FALSE)

    lines <- strsplit(out, "\n")
    dim(lines) <- dim(out)

    steps <- apply(lines, 1, function(x) do.call(cbind, x))
    if (is.list(steps)) {
      step_lines <- unlist(lapply(steps, function(x) {
        apply(x, 1, paste0, collapse = "")
      }))
    } else {
      step_lines <- apply(steps, 2, paste0, collapse = "")
    }

    cat_line("Prototype: ", out_full[[n]])
    cat_line(step_lines)
  }

  invisible()
}

vec_typeof <- function(x) {
  .Call(vctrs_typeof, x, TRUE)
}
vec_typeof_bare <- function(x) {
  .Call(vctrs_typeof, x, FALSE)
}

vec_type_info <- function(x) {
  .Call(ffi_type_info, x)
}
vec_proxy_info <- function(x) {
  .Call(ffi_proxy_info, x)
}

test_that("vec_ptype() is a no-op for NULL", {
  expect_null(vec_ptype(NULL))
})

test_that("vec_ptype() is a no-op for partial types", {
  expect_identical(vec_ptype(partial_factor("x")), partial_factor("x"))
  expect_identical(vec_ptype(partial_frame(x = 1)), partial_frame(x = 1))
})

test_that("vec_ptype() errors on scalars", {
  expect_error(vec_ptype(quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype(quote(fn())), class = "vctrs_error_scalar_type")
})

test_that(".ptype argument overrides others", {
  expect_equal(vec_ptype_common(.ptype = 1:10), numeric())
})

test_that(".ptype required in strict mode", {
  old <- options(vctrs.no_guessing = TRUE)
  on.exit(options(old))

  expect_error(vec_ptype_common(), "strict mode")
})

test_that("can feed ptype into itself", {
  expect_equal(vec_ptype_common(vec_ptype_common(1:10)), numeric())
})

test_that("finalised prototypes created from under specified inputs", {
  expect_equal(vec_ptype_common(), NULL)
  expect_equal(vec_ptype_common(NULL), NULL)

  expect_equal(vec_ptype_common(NA), logical())
  expect_equal(vec_ptype_common(NA, NULL), logical())
  expect_equal(vec_ptype_common(NULL, NA), logical())
})

test_that("finalised prototypes created from under specified data frame cols", {
  df <- data.frame(x = NA)
  expect_equal(vec_ptype_common(df)$x, logical())
})

test_that("non-missing logical get correct type", {
  expect_equal(vec_ptype_common(TRUE), logical())
})

test_that("output tests", {
  expect_snapshot(vec_ptype_show())
  expect_snapshot(vec_ptype_show(integer()))
  expect_snapshot(vec_ptype_show(integer(), double()))
  expect_snapshot(vec_ptype_show(logical(), integer(), double()))
})

test_that("vec_ptype_common() handles matrices", {
  m <- matrix(1:4, nrow = 2)
  expect_identical(vec_ptype_common(m, m), matrix(int(), ncol = 2))
})

test_that("vec_ptype_common() includes index in argument tag", {
  df1 <- tibble(x = tibble(y = tibble(z = 1)))
  df2 <- tibble(x = tibble(y = tibble(z = "a")))

  # Create a column name too large for default buffer
  nm <- str_dup("foobarfoobar", 10)
  large_df1 <- set_names(df1, nm)
  large_df2 <- set_names(df2, nm)

  expect_snapshot(error = TRUE, vec_ptype_common(df1, df2))
  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, df2))
  expect_snapshot(error = TRUE, vec_ptype_common(large_df1, large_df2))

  # Names
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, bar = "foo"))
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo"))
  expect_snapshot(error = TRUE, vec_ptype_common(foo = df1, bar = df2))
  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, bar = df2))

  # One splice box
  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, "foo")))
  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, 2), "foo"))
  expect_snapshot(error = TRUE, vec_ptype_common(1, !!!list(TRUE, FALSE), "foo"))

  # One named splice box
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo"))
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo")))
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = "foo")))
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr"))

  # Two splice boxes in next and current
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr")))
})

test_that("proxied types are have s3 bare type", {
  for (x in proxied_empty_types) {
    expect_identical(vec_typeof_bare(x), "s3")
  }
})

test_that("vec_ptype() preserves attributes of unproxied structures", {
  expect_identical(vec_ptype(foobar(dbl(1))), foobar(dbl()))
})

test_that("vec_ptype() errors on scalar lists", {
  expect_error(vec_ptype(foobar(list())), class = "vctrs_error_scalar_type")
})

test_that("can retrieve type info", {
  exp <- list(type = "integer", proxy_method = NULL)
  expect_identical(vec_type_info(1:3), exp)

  exp <- list(type = "s3", proxy_method = NULL)
  expect_identical(vec_type_info(~foo), exp)

  x <- as.POSIXlt(new_datetime(0))
  exp <- list(type = "s3", proxy_method = vec_proxy.POSIXlt)
  expect_identical(vec_type_info(x), exp)
})

test_that("can retrieve proxy info", {
  exp <- list(type = "integer", proxy_method = NULL, proxy = 1:3)
  expect_identical(vec_proxy_info(1:3), exp)

  exp <- list(type = "scalar", proxy_method = NULL, proxy = ~foo)
  expect_identical(vec_proxy_info(~foo), exp)

  x <- as.POSIXlt(new_datetime(0))
  proxy <- new_data_frame(unclass(x))
  exp <- list(type = "dataframe", proxy_method = vec_proxy.POSIXlt, proxy = proxy)
  expect_identical(vec_proxy_info(x), exp)
})

test_that("class_type() detects classes", {
  expect_identical(class_type(list()), "none")
  expect_identical(class_type(foobar(list())), "unknown")
  expect_identical(class_type(structure(list(), class = "list")), "list")
  expect_identical(class_type(subclass(structure(list(), class = "list"))), "list")
  expect_identical(class_type(I(subclass(structure(list(), class = "list")))), "list")

  expect_identical(class_type(I(list())), "bare_asis")
  expect_identical(class_type(I(1)), "bare_asis")

  expect_identical(class_type(data.frame()), "bare_data_frame")
  expect_identical(class_type(tibble::tibble()), "bare_tibble")
  expect_identical(class_type(subclass(data.frame())), "data_frame")

  expect_identical(class_type(new_factor()), "bare_factor")
  expect_identical(class_type(new_ordered()), "bare_ordered")
  expect_identical(class_type(subclass(new_factor())), "unknown")
  expect_identical(class_type(subclass(new_ordered())), "unknown")


  expect_identical(class_type(new_date()), "bare_date")
  expect_identical(class_type(new_datetime()), "bare_posixct")
  expect_identical(class_type(as.POSIXlt(new_date())), "bare_posixlt")
  expect_identical(class_type(subclass(new_date())), "unknown")
  expect_identical(class_type(subclass(new_datetime())), "unknown")

  expect_identical(class_type(NA), "none")
  expect_identical(class_type(foobar()), "unknown")
})

test_that("vec_ptype() handles class-less yet OBJECT gremlins", {
  gremlin <- stats::model.frame(freeny)
  expect_error(vec_ptype(gremlin), NA)
  expect_error(vec_c(gremlin), NA)
  expect_error(vec_init(gremlin), NA)
  expect_error(vec_slice(gremlin, 1), NA)
})

test_that("explicit list subclasses are vectors", {
  list_subclass <- function(x) {
    structure(x, class = c("custom_subclass", "list"))
  }

  x <- list_subclass(list())
  expect_true(vec_is(x))

  df <- data.frame(x = 1:2)
  df$z <- list_subclass(list(1, 2))

  expect_identical(vec_slice(df, 1)$z, list_subclass(list(1)))
})

test_that("the type of a classed data frame with an unspecified column retains unspecifiedness", {
  df1 <- subclass(data_frame(x = 1, y = NA))
  df2 <- subclass(data_frame(x = 1, y = unspecified(1)))
  expect <- subclass(data_frame(x = numeric(), y = unspecified()))

  expect_identical(vec_ptype(df1), expect)
  expect_identical(vec_ptype(df2), expect)
})

test_that("vec_ptype() methods can be written", {
  local_methods(
    vec_ptype.vctrs_foobar = function(x, ...) "dispatch"
  )
  expect_identical(vec_ptype(foobar()), "dispatch")
})

test_that("vec_ptype_finalise() works with NULL", {
  expect_identical(vec_ptype_finalise(NULL), NULL)
})

test_that("vec_ptype_finalise() works recursively over bare data frames", {
  df <- new_data_frame(list(x = numeric(), y = unspecified(), z = partial_factor()))
  expect <- data_frame(x = numeric(), y = logical(), z = factor())

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() works recursively over classed data frames", {
  df <- new_data_frame(list(x = numeric(), y = unspecified(), z = partial_factor()))
  df <- subclass(df)
  expect <- subclass(data_frame(x = numeric(), y = logical(), z = factor()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() can handle data frame columns", {
  df <- data_frame(x = numeric(), y = data_frame(z = unspecified()))
  expect <- data_frame(x = numeric(), y = data_frame(z = logical()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() requires vector types", {
  expect_error(vec_ptype_finalise(quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype_finalise(foobar()), class = "vctrs_error_scalar_type")
})

# This might change in the future if we decide that prototypes don't
# have names
test_that("vec_ptype() preserves type of names and row names", {
  expect_identical(vec_ptype(c(foo = 1)), named(dbl()))
  expect_identical(vec_ptype(mtcars), mtcars[0, ])
  expect_identical(vec_ptype(foobar(mtcars)), foobar(mtcars[0, ]))
})

test_that("vec_ptype_common() handles spliced names consistently (#1570)", {
  args1 <- list(a = "foo", b = "bar")
  args2 <- list(y = NULL, z = 1)

  y_name <- "y"
  z_name <- "z"

  expect_snapshot(error = TRUE, {
    vec_ptype_common(
      a = "foo",
      b = "bar",
      y = NULL,
      z = 1
    )

    vec_ptype_common(
      !!!args1,
      !!!args2
    )

    vec_ptype_common(
      !!!args1,
      "{y_name}" := NULL,
      "{z_name}" := 1
    )
  })
})

test_that("Casting to named argument mentions 'match type <foo>'", {
  expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo", to_arg = "bar"))
  expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo"))
})

# vec_cast() ---------------------------------------------------------------

test_that("new classes are uncoercible by default", {
  x <- structure(1:10, class = "vctrs_nonexistant")
  expect_error(vec_cast(1, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(x, 1), class = "vctrs_error_incompatible_type")
})

test_that("casting requires vectors", {
  expect_error(vec_cast(NULL, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(NA, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(list(), quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), NA), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), list()), class = "vctrs_error_scalar_type")
  expect_error(
    vec_cast(quote(name), quote(name)),
    class = "vctrs_error_scalar_type"
  )
})

test_that("dimensionality matches output", {
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))

  x <- matrix(1, nrow = 2, ncol = 2)
  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_type")
})

test_that("empty input to vec_cast_common() returns list()", {
  expect_equal(vec_cast_common(), list())
  expect_equal(vec_cast_common(NULL, NULL), list(NULL, NULL))
})

test_that("identical structures can be cast to each other", {
  expect_identical(vec_cast(foobar("foo"), foobar("bar")), foobar("foo"))
})

test_that("cast common preserves names", {
  expect_identical(vec_cast_common(foo = 1, bar = 2L), list(foo = 1, bar = 2))
})

test_that("cast errors create helpful messages (#57, #225)", {
  # Lossy cast
  expect_snapshot(error = TRUE, vec_cast(1.5, 10L))

  # Incompatible cast
  expect_snapshot(error = TRUE, vec_cast(factor("foo"), 10))

  # Nested data frames - Lossy cast
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = 1.5))
    y <- tibble(a = tibble(b = 10L))
    vec_cast(x, y)
  })

  # Nested data frames - Incompatible cast
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = factor("foo")))
    y <- tibble(a = tibble(b = 10))
    vec_cast(x, y)
  })

  # Nested data frames - Common cast error
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = factor("foo")))
    y <- tibble(a = tibble(b = 10))
    vec_cast_common(x, y)
  })
})

test_that("unspecified can be cast to shaped vectors", {
  x <- matrix(letters[1:4], 2)
  expect_identical(vec_cast(NA, x), matrix(chr(NA, NA), 1))

  x <- foobar(c(1:4))
  dim(x) <- c(2, 2)
  out <- vec_cast(NA, x)

  exp <- foobar(int(c(NA, NA)))
  dim(exp) <- c(1, 2)
  expect_identical(out, exp)
})

test_that("vec_cast() falls back to base class even when casting to non-base type", {
  expect_equal(vec_cast(foobar(mtcars), mtcars), mtcars)
  expect_equal(vec_cast(mtcars, foobar(mtcars)), mtcars)
})

test_that("vec_cast() only attempts to fall back if `to` is a data frame (#1568)", {
  expect_snapshot({
    (expect_error(
      vec_cast(foobar(mtcars), 1),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_cast() evaluates x_arg and to_arg lazily", {
  expect_silent(vec_cast(TRUE, logical(), x_arg = print("oof")))
  expect_silent(vec_cast(TRUE, logical(), to_arg = print("oof")))
})


# Conditions --------------------------------------------------------------

test_that("can suppress cast errors selectively", {
  f <- function() vec_cast(factor("a"), to = factor("b"))
  expect_error(regexp = NA, allow_lossy_cast(f()))
  expect_error(regexp = NA, allow_lossy_cast(f(), x_ptype = factor("a")))
  expect_error(regexp = NA, allow_lossy_cast(f(), to_ptype = factor("b")))
  expect_error(
    regexp = NA,
    allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("b"))
  )
  expect_error(
    allow_lossy_cast(f(), x_ptype = factor("c")),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    allow_lossy_cast(f(), x_ptype = factor("b"), to_ptype = factor("a")),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("c")),
    class = "vctrs_error_cast_lossy"
  )
})

test_that("can signal deprecation warnings for lossy casts", {
  local_options(lifecycle_verbosity = "warning")

  lossy_cast <- function() {
    maybe_lossy_cast(
      TRUE,
      factor("foo"),
      factor("bar"),
      lossy = TRUE,
      .deprecation = TRUE,
      x_arg = "x",
      to_arg = "to"
    )
  }

  expect_snapshot({
    (expect_warning(expect_true(lossy_cast())))
  })
  expect_warning(regexp = NA, expect_true(allow_lossy_cast(lossy_cast())))
  expect_warning(
    regexp = NA,
    expect_true(allow_lossy_cast(lossy_cast(), factor("foo"), factor("bar")))
  )
  expect_warning(expect_true(allow_lossy_cast(
    lossy_cast(),
    factor("bar"),
    double()
  )))
})


# vec_cast_common() -------------------------------------------------------

test_that("vec_ptype_common() optionally falls back to base class", {
  x <- foobar(NA, foo = 1)
  y <- foobaz(NA, bar = 2)

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  expect_error(
    vec_ptype_common_params(x, y, .fallback_opts = enabled_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype_common_params(
      x_df,
      y_df,
      .fallback_opts = enabled_fallback_opts()
    ),
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    vec_ptype_common_params(x, y, .fallback_opts = enabled_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype_common_params(
      x_df,
      y_df,
      .fallback_opts = enabled_fallback_opts()
    ),
    class = "vctrs_error_incompatible_type"
  )

  class(y) <- c("foo", class(x))
  y_df <- data_frame(x = y)

  common_sentinel <- vec_ptype_common_params(
    x,
    y,
    .fallback_opts = enabled_fallback_opts()
  )
  expect_true(is_common_class_fallback(common_sentinel))
  expect_identical(fallback_class(common_sentinel), "vctrs_foobar")

  common_sentinel <- vec_ptype_common_params(
    x_df,
    y_df,
    .fallback_opts = enabled_fallback_opts()
  )
  expect_true(is_common_class_fallback(common_sentinel$x))
  expect_identical(fallback_class(common_sentinel$x), "vctrs_foobar")

  common <- vec_cast_common_opts(x = x, y = y, .opts = enabled_fallback_opts())
  expect_identical(common, list(x = x, y = y))

  common <- vec_cast_common_opts(
    x = x_df,
    y = y_df,
    .opts = enabled_fallback_opts()
  )
  expect_identical(common, list(x = x_df, y = y_df))
})

test_that("vec_ptype_common_fallback() collects common type", {
  x <- foobar(1, foo = 1, class = c("quux", "baz"))
  y <- foobar(2, bar = 2, class = "baz")

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  out <- vec_ptype_common_fallback(x, y)
  expect_identical(typeof(out), "double")
  expect_true(is_common_class_fallback(out))
  expect_identical(fallback_class(out), c("baz", "vctrs_foobar"))

  out <- vec_ptype_common_fallback(x_df, y_df)
  expect_identical(typeof(out$x), "double")
  expect_true(is_common_class_fallback(out$x))
  expect_identical(fallback_class(out$x), c("baz", "vctrs_foobar"))

  # Different base types can't fall back to common class
  z <- foobar(3L, baz = 3)
  expect_error(
    vec_ptype_common_fallback(x, z),
    class = "vctrs_error_incompatible_type"
  )

  z_df <- data_frame(x = z)
  expect_error(
    vec_ptype_common_fallback(x_df, z_df),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("fallback sentinel is returned with unspecified inputs", {
  fallback <- vec_ptype_common_fallback(foobar(1), foobar(1))
  expect_identical(vec_ptype_common_fallback(NA, foobar(1)), fallback)
  expect_identical(vec_ptype_common_fallback(foobar(1), NA), fallback)
})

test_that("vec_ptype_common() supports subclasses of list", {
  x <- structure(list(1), class = c("vctrs_foo", "list"))
  y <- structure(list(2), class = c("bar", "vctrs_foo", "list"))

  expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type")

  out <- with_methods(
    c.vctrs_foo = function(...) quux(NextMethod()),
    vec_c(x, y)
  )
  expect_identical(out, quux(list(1, 2)))
})

test_that("vec_cast_common_fallback() works with tibbles", {
  x <- foobar("foo")
  df <- data_frame(x = x)
  tib <- tibble(x = x)

  exp <- list(tib, tib)

  expect_identical(vec_cast_common_fallback(tib, tib), exp)
  expect_identical(vec_cast_common_fallback(tib, df), exp)
  expect_identical(vec_cast_common_fallback(df, tib), exp)
})

test_that("can call `vec_cast()` from C (#1666)", {
  fn <- inject(function(x, i) .Call(!!ffi_exp_vec_cast, x, i))
  environment(fn) <- ns_env("utils")

  x <- array(1, dim = c(1, 1))
  y <- array(2, dim = c(2, 2))

  expect_equal(fn(x, y), vec_cast(x, y))
})

test_that("df-fallback for cast is not sensitive to attributes order", {
  x <- structure(
    list(col = ""),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame"),
    row.names = c(NA, -1L),
    foo = "foo",
    bar = "bar"
  )
  ptype <- structure(
    list(col = character(0)),
    foo = "foo",
    bar = "bar",
    row.names = integer(0),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame")
  )

  expect_identical(vec_cast(x, ptype), x)
})

test_that("bare-type fallback for df-cast works", {
  # NOTE: Not sure why this was necessary. The cubble and yamlet
  # packages fail without this.
  local_methods(
    c.vctrs_foobaz = function(...) quux(NextMethod())
  )

  df <- data_frame(x = 1, y = foobaz("foo"))
  gdf <- dplyr::new_grouped_df(
    df,
    data_frame(x = 1, .rows = list(1L)),
    class = "vctrs_foobar"
  )

  expect_error(vec_rbind(gdf, gdf), NA)
})

test_that("can cast to unspecified `NA` with `vec_cast()` and `vec_cast_common()` (#2099)", {
  # In the `vec_cast()` case no `vec_ptype()` call is made, which means that no
  # finalization step is required. In the `vec_cast_common()` case, the
  # underlying call to `vec_ptype_common()` calls `vec_ptype(NA)` but also
  # finalizes that to `logical()` on the way out, so this still works.
  expect_identical(vec_cast(TRUE, to = NA), TRUE)
  expect_identical(vec_cast_common(TRUE, .to = NA), list(TRUE))

  # `vec_cast()` itself does not call `vec_ptype()` and does not finalize,
  # so this stays <unspecified> and the cast fails
  # (this behavior is questionable, but is very much an edge case)
  expect_snapshot(error = TRUE, {
    vec_cast(TRUE, to = unspecified(1))
  })

  # `vec_cast_common()` calls `vec_ptype_common()`, which always finalises,
  # so this technically works (but again, it is an edge case)
  expect_identical(vec_cast_common(TRUE, .to = unspecified(1)), list(TRUE))
})

# Golden tests -------------------------------------------------------

test_that("casting performs expected allocations", {
  expect_snapshot({
    # No allocations when shape doesn't change (#2006)
    x <- matrix(rep(1L, 1e2), ncol = 2)
    with_memory_prof(vec_cast(x, x))

    # One allocation when type changes
    x <- matrix(rep(1L, 1e2), ncol = 2)
    y <- matrix(rep(1, 1e2), ncol = 2)
    with_memory_prof(vec_cast(x, y))
  })
})

test_that("vec_ptype() is a no-op for NULL", {
  expect_null(vec_ptype(NULL))
})

test_that("vec_ptype() errors on scalars", {
  expect_error(vec_ptype(quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype(quote(fn())), class = "vctrs_error_scalar_type")
})

test_that(".ptype argument overrides others", {
  expect_equal(vec_ptype_common(.ptype = 1:10), numeric())
})

test_that(".ptype required in strict mode", {
  old <- options(vctrs.no_guessing = TRUE)
  on.exit(options(old))

  expect_error(vec_ptype_common(), "strict mode")
})

test_that("can feed ptype into itself", {
  expect_equal(vec_ptype_common(vec_ptype_common(1:10)), numeric())
})

test_that("finalised prototypes created from under specified inputs", {
  expect_equal(vec_ptype_common(), NULL)
  expect_equal(vec_ptype_common(NULL), NULL)

  expect_equal(vec_ptype_common(NA), logical())
  expect_equal(vec_ptype_common(NA, NULL), logical())
  expect_equal(vec_ptype_common(NULL, NA), logical())
})

test_that("finalised prototypes created from under specified data frame cols", {
  df <- data.frame(x = NA)
  expect_equal(vec_ptype_common(df)$x, logical())
})

test_that("non-missing logical get correct type", {
  expect_equal(vec_ptype_common(TRUE), logical())
})

test_that("output tests", {
  expect_snapshot(vec_ptype_show())
  expect_snapshot(vec_ptype_show(integer()))
  expect_snapshot(vec_ptype_show(integer(), double()))
  expect_snapshot(vec_ptype_show(logical(), integer(), double()))
})

test_that("vec_ptype_common() handles matrices", {
  m <- matrix(1:4, nrow = 2)
  expect_identical(vec_ptype_common(m, m), matrix(int(), ncol = 2))
})

test_that("vec_ptype_common() doesn't mutate input", {
  x <- list(a = 1L, b = 2)
  expect_identical(vec_ptype_common(!!!x), numeric())
  expect_identical(x, list(a = 1L, b = 2))
})

test_that("vec_ptype_common() includes index in argument tag", {
  df1 <- tibble(x = tibble(y = tibble(z = 1)))
  df2 <- tibble(x = tibble(y = tibble(z = "a")))

  # Create a column name too large for default buffer
  nm <- str_dup("foobarfoobar", 10)
  large_df1 <- set_names(df1, nm)
  large_df2 <- set_names(df2, nm)

  expect_snapshot(error = TRUE, vec_ptype_common(df1, df2))
  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, df2))
  expect_snapshot(error = TRUE, vec_ptype_common(large_df1, large_df2))

  # Names
  expect_snapshot(error = TRUE, vec_ptype_common(foo = TRUE, bar = "foo"))
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo")
  )
  expect_snapshot(error = TRUE, vec_ptype_common(foo = df1, bar = df2))
  expect_snapshot(error = TRUE, vec_ptype_common(df1, df1, bar = df2))

  # One splice box
  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, "foo")))
  expect_snapshot(error = TRUE, vec_ptype_common(TRUE, !!!list(1, 2), "foo"))
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(1, !!!list(TRUE, FALSE), "foo")
  )

  # One named splice box
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo")
  )
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo"))
  )
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, !!!list(bar = "foo"))
  )
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr")
  )

  # Two splice boxes in next and current
  expect_snapshot(
    error = TRUE,
    vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr"))
  )
})

test_that("proxied types are have s3 bare type", {
  for (x in proxied_empty_types) {
    expect_identical(vec_typeof_bare(x), "s3")
  }
})

test_that("vec_ptype() preserves attributes of unproxied structures", {
  expect_identical(vec_ptype(foobar(dbl(1))), foobar(dbl()))
})

test_that("vec_ptype() errors on scalar lists", {
  expect_error(vec_ptype(foobar(list())), class = "vctrs_error_scalar_type")
})

test_that("can retrieve type info", {
  exp <- list(type = "integer", had_proxy_method = FALSE)
  expect_identical(vec_type_info(1:3), exp)

  exp <- list(type = "s3", had_proxy_method = FALSE)
  expect_identical(vec_type_info(~foo), exp)

  x <- as.POSIXlt(new_datetime(0))
  exp <- list(type = "s3", had_proxy_method = TRUE)
  expect_identical(vec_type_info(x), exp)
})

test_that("can retrieve proxy info", {
  exp <- list(type = "integer", had_proxy_method = FALSE, proxy = 1:3)
  expect_identical(vec_proxy_info(1:3), exp)

  exp <- list(type = "scalar", had_proxy_method = FALSE, proxy = ~foo)
  expect_identical(vec_proxy_info(~foo), exp)

  x <- as.POSIXlt(new_datetime(0))
  proxy <- new_data_frame(unclass(x))
  exp <- list(
    type = "dataframe",
    had_proxy_method = TRUE,
    proxy = proxy
  )
  expect_identical(vec_proxy_info(x), exp)
})

test_that("class_type() detects classes", {
  expect_identical(class_type(list()), "none")
  expect_identical(class_type(foobar(list())), "unknown")
  expect_identical(class_type(structure(list(), class = "list")), "list")
  expect_identical(
    class_type(subclass(structure(list(), class = "list"))),
    "list"
  )
  expect_identical(
    class_type(I(subclass(structure(list(), class = "list")))),
    "list"
  )

  expect_identical(class_type(I(list())), "bare_asis")
  expect_identical(class_type(I(1)), "bare_asis")

  expect_identical(class_type(data.frame()), "bare_data_frame")
  expect_identical(class_type(tibble::tibble()), "bare_tibble")
  expect_identical(class_type(subclass(data.frame())), "data_frame")

  expect_identical(class_type(new_factor()), "bare_factor")
  expect_identical(class_type(new_ordered()), "bare_ordered")
  expect_identical(class_type(subclass(new_factor())), "unknown")
  expect_identical(class_type(subclass(new_ordered())), "unknown")

  expect_identical(class_type(new_date()), "bare_date")
  expect_identical(class_type(new_datetime()), "bare_posixct")
  expect_identical(class_type(as.POSIXlt(new_date())), "bare_posixlt")
  expect_identical(class_type(subclass(new_date())), "unknown")
  expect_identical(class_type(subclass(new_datetime())), "unknown")

  expect_identical(class_type(NA), "none")
  expect_identical(class_type(foobar()), "unknown")
})

test_that("vec_ptype() handles class-less yet OBJECT gremlins", {
  gremlin <- stats::model.frame(freeny)
  expect_error(vec_ptype(gremlin), NA)
  expect_error(vec_c(gremlin), NA)
  expect_error(vec_init(gremlin), NA)
  expect_error(vec_slice(gremlin, 1), NA)
})

test_that("explicit list subclasses are vectors", {
  list_subclass <- function(x) {
    structure(x, class = c("custom_subclass", "list"))
  }

  x <- list_subclass(list())
  expect_true(vec_is(x))

  df <- data.frame(x = 1:2)
  df$z <- list_subclass(list(1, 2))

  expect_identical(vec_slice(df, 1)$z, list_subclass(list(1)))
})

test_that("the type of a classed data frame with an unspecified column retains unspecifiedness", {
  df1 <- subclass(data_frame(x = 1, y = NA))
  df2 <- subclass(data_frame(x = 1, y = unspecified(1)))
  expect <- subclass(data_frame(x = numeric(), y = unspecified()))

  expect_identical(vec_ptype(df1), expect)
  expect_identical(vec_ptype(df2), expect)
})

test_that("vec_ptype() methods can be written", {
  local_methods(
    vec_ptype.vctrs_foobar = function(x, ...) "dispatch"
  )
  expect_identical(vec_ptype(foobar()), "dispatch")
})

test_that("vec_ptype_finalise() works with NULL", {
  expect_identical(vec_ptype_finalise(NULL), NULL)
})

test_that("vec_ptype_finalise() works recursively over bare data frames", {
  df <- new_data_frame(list(
    x = numeric(),
    y = unspecified()
  ))
  expect <- data_frame(x = numeric(), y = logical())

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() works recursively over classed data frames", {
  df <- new_data_frame(list(
    x = numeric(),
    y = unspecified()
  ))
  df <- subclass(df)
  expect <- subclass(data_frame(x = numeric(), y = logical()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() can handle data frame columns", {
  df <- data_frame(x = numeric(), y = data_frame(z = unspecified()))
  expect <- data_frame(x = numeric(), y = data_frame(z = logical()))

  expect_identical(vec_ptype_finalise(df), expect)
})

test_that("vec_ptype_finalise() requires vector types", {
  expect_error(
    vec_ptype_finalise(quote(name)),
    class = "vctrs_error_scalar_type"
  )
  expect_error(vec_ptype_finalise(foobar()), class = "vctrs_error_scalar_type")
})

# This might change in the future if we decide that prototypes don't
# have names
test_that("vec_ptype() preserves type of names and row names", {
  expect_identical(vec_ptype(c(foo = 1)), named(dbl()))
  expect_identical(vec_ptype(mtcars), mtcars[0, ])
  expect_identical(vec_ptype(foobar(mtcars)), foobar(mtcars[0, ]))
})

test_that("vec_ptype_common() handles spliced names consistently (#1570)", {
  args1 <- list(a = "foo", b = "bar")
  args2 <- list(y = NULL, z = 1)

  y_name <- "y"
  z_name <- "z"

  expect_snapshot(error = TRUE, {
    vec_ptype_common(
      a = "foo",
      b = "bar",
      y = NULL,
      z = 1
    )

    vec_ptype_common(
      !!!args1,
      !!!args2
    )

    vec_ptype_common(
      !!!args1,
      "{y_name}" := NULL,
      "{z_name}" := 1
    )
  })
})

test_that("vec_ptype() and vec_ptype2() don't finalize their output", {
  expect_identical(vec_ptype(NA), unspecified())
  expect_identical(vec_ptype2(NA, NA), unspecified())
  expect_identical(vec_ptype2(NA, NULL), unspecified())
  expect_identical(vec_ptype2(NULL, NA), unspecified())

  expect_identical(vec_ptype(unspecified()), unspecified())
  expect_identical(vec_ptype2(unspecified(), unspecified()), unspecified())
  expect_identical(vec_ptype2(unspecified(), NULL), unspecified())
  expect_identical(vec_ptype2(NULL, unspecified()), unspecified())
})

test_that("vec_ptype_common() always finalizes its output (#2099)", {
  expect_identical(vec_ptype_common(NA), logical())
  expect_identical(vec_ptype_common(NA, NA), logical())
  expect_identical(vec_ptype_common(unspecified(1)), logical())
  expect_identical(vec_ptype_common(unspecified(1), unspecified(1)), logical())

  # Even explicit `.ptype`s
  expect_identical(
    vec_ptype_common(.ptype = NA),
    logical()
  )
  expect_identical(
    vec_ptype_common(.ptype = unspecified(1)),
    logical()
  )
})

test_that("vec_ptype_common() lets you opt out of ptype finalization (#2094)", {
  expect_identical(
    vec_ptype_common(NA, .finalise = FALSE),
    unspecified()
  )
  expect_identical(
    vec_ptype_common(NA, NA, .finalise = FALSE),
    unspecified()
  )
  expect_identical(
    vec_ptype_common(unspecified(1), .finalise = FALSE),
    unspecified()
  )
  expect_identical(
    vec_ptype_common(unspecified(1), unspecified(1), .finalise = FALSE),
    unspecified()
  )

  # Works for explicit `.ptype` too
  expect_identical(
    vec_ptype_common(.ptype = NA, .finalise = FALSE),
    unspecified()
  )
  expect_identical(
    vec_ptype_common(.ptype = unspecified(1), .finalise = FALSE),
    unspecified()
  )
})

test_that("vec_ptype_common_params() lets you opt out of ptype finalization", {
  expect_identical(
    vec_ptype_common_params(NA, .finalise = FALSE),
    unspecified()
  )
  # Works for explicit `.ptype` too
  expect_identical(
    vec_ptype_common_params(.ptype = NA, .finalise = FALSE),
    unspecified()
  )
})

test_that("`.finalise` is validated", {
  expect_snapshot(error = TRUE, {
    vec_ptype_common(.finalise = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_ptype_common_params(.finalise = 1)
  })
})

#' Tools for accessing the fields of a record.
#'
#' A [rcrd] behaves like a vector, so `length()`, `names()`, and `$` can
#' not provide access to the fields of the underlying list. These helpers do:
#' `fields()` is equivalent to `names()`; `n_fields()` is equivalent to
#' `length()`; `field()` is equivalent to `$`.
#'
#' @param x A [rcrd], i.e. a list of equal length vectors with unique names.
#' @keywords internal
#' @export
#' @examples
#' x <- new_rcrd(list(x = 1:3, y = 3:1, z = letters[1:3]))
#' n_fields(x)
#' fields(x)
#'
#' field(x, "y")
#' field(x, "y") <- runif(3)
#' field(x, "y")
fields <- function(x) {
  .Call(vctrs_fields, x)
}

#' @export
#' @rdname fields
n_fields <- function(x) {
  .Call(vctrs_n_fields, x)
}

#' @export
#' @rdname fields
field <- function(x, i) {
  .Call(vctrs_field_get, x, i)
}

#' @export
#' @rdname fields
`field<-` <- function(x, i, value) {
  .Call(vctrs_field_set, x, i, value)
}

# constructor and accessors -----------------------------------------------

test_that("can construct and access components", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_equal(length(r), 1)
  expect_equal(n_fields(r), 2)

  expect_equal(names(r), NULL)
  expect_equal(fields(r), c("x", "y"))

  expect_error(r$x, class = "vctrs_error_unsupported")
  expect_equal(field(r, "x"), 1)
})

test_that("requires format method", {
  x <- new_rcrd(list(x = 1))
  expect_error(format(x), class = "vctrs_error_unimplemented")
})

test_that("vec_proxy() transforms records to data frames", {
  expect_identical(
    vec_proxy(new_rcrd(list(a = "1"))),
    new_data_frame(list(a = "1"))
  )
})

test_that("equality, comparison, and order proxies are recursive and fall through (#1503, #1664)", {
  base <- new_rcrd(list(a = 1), class = "custom")
  x <- new_rcrd(list(x = base))

  expect_identical(vec_proxy_equal(x), 1)
  expect_identical(vec_proxy_compare(x), 1)
  expect_identical(vec_proxy_order(x), 1)

  local_methods(vec_proxy_equal.custom = function(x, ...) rep("equal", length(x)))

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "equal")
  expect_identical(vec_proxy_order(x), "equal")

  local_methods(vec_proxy_compare.custom = function(x, ...) rep("compare", length(x)))

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "compare")
  expect_identical(vec_proxy_order(x), "compare")

  local_methods(vec_proxy_order.custom = function(x, ...) rep("order", length(x)))

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "compare")
  expect_identical(vec_proxy_order(x), "order")

  y <- new_rcrd(list(a = 1), class = "custom2")
  local_methods(vec_proxy_compare.custom2 = function(x, ...) rep("compare2", length(x)))

  z <- data_frame(x = x, y = y)

  # Each column falls back independently
  expect_identical(vec_proxy_equal(z), data_frame(x = "equal", y = 1))
  expect_identical(vec_proxy_compare(z), data_frame(x = "compare", y = "compare2"))
  expect_identical(vec_proxy_order(z), data_frame(x = "order", y = "compare2"))
})

# base methods ------------------------------------------------------------

test_that("has no names", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_null(names(x))
  expect_null(vec_names(x))
})

test_that("removing names with `NULL` is a no-op (#1419)", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_identical(`names<-`(x, NULL), x)
  expect_identical(vec_set_names(x, NULL), x)
})

test_that("setting character names is an error (#1419)", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_error(`names<-`(x, "x"), "Can't assign names")
  expect_error(vec_set_names(x, "x"), "Can't assign names")
})

test_that("na.omit() works and retains metadata (#1413)", {
  x <- new_rcrd(list(a = c(1, 1, NA, NA), b = c(1, NA, 1, NA)))
  result <- na.omit(x)

  expect <- vec_slice(x, 1:3)
  attr(expect, "na.action") <- structure(4L, class = "omit")

  expect_identical(result, expect)
})

test_that("na.fail() works", {
  # Only considered missing if all fields are missing
  x <- new_rcrd(list(a = c(1, 1, NA), b = c(1, NA, 1)))
  expect_identical(na.fail(x), x)

  x <- new_rcrd(list(a = c(1, 1, NA, NA), b = c(1, NA, 1, NA)))
  expect_snapshot(error = TRUE, na.fail(x))
})

# coercion ----------------------------------------------------------------

test_that("can't cast list to rcrd", {
  l <- list(
    new_rcrd(list(a = "1", b = 3L)),
    new_rcrd(list(b = "4", a = 2))
  )
  expect_error(
    vec_cast(l, new_rcrd(list(a = 1L, b = 2L))),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can recast rcrd from list", {
  r <- new_rcrd(list(x = integer(), y = numeric()))

  expect_equal(
    vec_restore(list(x = 1L, y = 1), r),
    new_rcrd(list(x = 1L, y = 1))
  )
})

test_that("can't cast rcrd to list", {
  r <- new_rcrd(list(x = 1:2, y = 2:3))
  expect_error(vec_cast(r, list()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(r, list()), class = "vctrs_error_incompatible_type")
})

test_that("default casts are implemented correctly", {
  r <- new_rcrd(list(x = 1, y = 1))

  expect_error(vec_cast(1, r), class = "vctrs_error_incompatible_type")
  expect_equal(vec_cast(NULL, r), NULL)
})

test_that("can't cast incompatible rcrd", {
  expect_error(
    vec_cast(
      new_rcrd(list(a = "1", b = 3L)),
      new_rcrd(list(a = "1"))
    ),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    vec_cast(
      new_rcrd(list(a = "1", b = 3L)),
      new_rcrd(list(a = "1", c = 3L))
    ),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    vec_cast(
      new_rcrd(list(a = "a", b = 3L)),
      new_rcrd(list(a = 1, b = 3L))
    ),
    class = "vctrs_error_incompatible_type"
  )
})

# input validation --------------------------------------------------------

test_that("must be list of equal length vectors", {
  expect_error(new_rcrd(list()), "list of length 1")
  expect_error(new_rcrd(list(x = environment())), class = "vctrs_error_scalar_type")
  expect_error(new_rcrd(list(x = 1:2, y = 1:3)), "same size")
})

test_that("names must be unique", {
  expect_error(new_rcrd(list(1, 2)), class = "vctrs_error_names_cannot_be_empty")
  expect_error(new_rcrd(list(x = 1, 2)), class = "vctrs_error_names_cannot_be_empty")
  expect_error(new_rcrd(list(x = 1, x = 2)), class = "vctrs_error_names_must_be_unique")
  expect_error(new_rcrd(setNames(list(1, 2), "x")), "can't return `NA`")
})

test_that("subset assignment throws error", {
  x <- new_rcrd(list(x = 1))
  expect_error(
    x$y <- 2,
    class = "vctrs_error_unsupported"
  )
})

test_that("can supply data frame as fields", {
  expect_identical(
    new_rcrd(list(x = 1)),
    new_rcrd(tibble(x = 1))
  )
})

test_that("fields are not recycled", {
  expect_error(
    new_rcrd(list(x = 1, y = 1:2)),
    "must be the same size"
  )
})


# tuple class ----------------------------------------------------------
# use simple class to test essential features of rcrds

test_that("print and str use format", {
  local_tuple_methods()
  r <- tuple(1, 1:100)

  expect_snapshot(r)
  expect_snapshot(str(r[1:10]))
  expect_snapshot(str(list(list(list(r, 1:100)))))
})

test_that("subsetting methods applied to each field", {
  local_tuple_methods()
  x <- tuple(1:2, 1)

  expect_equal(x[1], tuple(1, 1))
  expect_equal(x[[1]], tuple(1, 1))

  expect_equal(rep(tuple(1, 1), 2), tuple(c(1, 1), 1))

  length(x) <- 1
  expect_equal(x, tuple(1, 1))
})

test_that("subset assignment modifies each field", {
  local_tuple_methods()
  x <- tuple(c(1, 1), c(2, 2))

  expect_error(x[[]] <- tuple(), "missing")

  x[[1]] <- tuple(3, 3)
  expect_equal(x, tuple(c(3, 1), c(3, 2)))

  x[1] <- tuple(4, 4)
  expect_equal(x, tuple(c(4, 1), c(4, 2)))
})

test_that("subset assignment recycles", {
  local_tuple_methods()

  x <- tuple(c(1, 1), c(2, 2))
  x[1:2] <- tuple(1, 1)
  expect_equal(x, tuple(c(1, 1), c(1, 1)))

  x[] <- tuple(2, 2)
  expect_equal(x, tuple(c(2, 2), c(2, 2)))
})

test_that("can sort rcrd", {
  local_tuple_methods()
  x <- tuple(c(1, 2, 1), c(3, 1, 2))
  expect_equal(xtfrm(x), c(2, 3, 1))
  expect_equal(order(x), c(3, 1, 2))
  expect_equal(sort(x), tuple(c(1, 1, 2), c(2, 3, 1)))
})

test_that("can use dictionary methods on a rcrd", {
  local_tuple_methods()
  x <- tuple(c(1, 2, 1), c(3, 1, 3))
  expect_equal(unique(x), x[1:2])
  expect_equal(duplicated(x), c(FALSE, FALSE, TRUE))
  expect_equal(anyDuplicated(x), TRUE)
})

test_that("cannot round trip through list", {
  local_tuple_methods()
  t <- tuple(1:2, 3:4)

  # Used to be allowed
  expect_error(vec_cast(t, list()), class = "vctrs_error_incompatible_type")
})

test_that("can convert to list using as.list() or vec_chop() (#1113)", {
  local_tuple_methods()
  t <- tuple(1:2, 3:4)

  expect <- list(tuple(1L, 3L), tuple(2L, 4L))

  expect_identical(as.list(t), expect)
  expect_identical(vec_chop(t), expect)
})

test_that("dangerous methods marked as unimplemented", {
  local_tuple_methods()
  t <- tuple()

  expect_error(mean(t), class = "vctrs_error_unsupported")
  expect_error(abs(t), class = "vctrs_error_unsupported")
  expect_error(is.finite(t), class = "vctrs_error_unsupported")
  expect_error(is.nan(t), class = "vctrs_error_unsupported")
})


# slicing -----------------------------------------------------------------

test_that("dots are an error (#1295)", {
  foo <- new_rcrd(list(foo = "foo"))
  expect_snapshot(error = TRUE, foo[1, 2])
})

test_that("records are restored after slicing the proxy", {
  expect_identical(new_rcrd(list(x = 1:2))[1], new_rcrd(list(x = 1L)))
})

test_that("can slice with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:2)))

  out <- vec_slice(x, 2)
  expect_identical(
    out,
    new_rcrd(data_frame(x = data_frame(y = 2L)))
  )
  expect_identical(
    x[2],
    out
  )
  expect_identical(
    x[[2]],
    out
  )
})

test_that("can rep with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:2)))

  expect_identical(
    rep(x, length.out = 4),
    vec_slice(x, c(1:2, 1:2))
  )
})

test_that("can assign with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:3)))
  y <- new_rcrd(data_frame(x = data_frame(y = FALSE)))
  exp <- new_rcrd(data_frame(x = data_frame(y = c(1L, 2L, 0L))))

  expect_identical(vec_assign(x, 3, y), exp)

  out <- x
  out[[3]] <- y
  expect_identical(out, exp)
})

test_that("can resize with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:3)))

  length(x) <- 2
  expect_identical(x, new_rcrd(data_frame(x = data_frame(y = 1:2))))

  length(x) <- 4
  expect_identical(x, new_rcrd(data_frame(x = data_frame(y = c(1:2, NA, NA)))))
})

test_that("`[[` preserves type of record fields (#1205)", {
  x <- new_rcrd(list(x = 1:3, a = list(1, 2:3, 4:6)))
  expect_identical(field(x[3], "a"), list(4:6))
  expect_identical(field(x[[3]], "a"), list(4:6))
})

test_that("n_fields captures number of fields", {
  r <- new_rcrd(list(x = 1, y = 2))
  expect_equal(n_fields(r), 2)
})


# get ---------------------------------------------------------------------

test_that("can extract valid field", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_equal(field(r, "x"), 1)
  expect_equal(field(r, 1L), 1)
})

test_that("can extract field even if encoding is different", {
  x1 <- "fa\u00e7ile"
  skip_if_not(Encoding(x1) == "UTF-8")

  x2 <- iconv(x1, from = "UTF-8", to = "latin1")
  skip_if_not(Encoding(x2) == "latin1")

  r <- new_rcrd(setNames(list(1), x1))
  expect_equal(field(r, x1), 1)
  expect_equal(field(r, x2), 1)
})

test_that("invalid indices throw error", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_error(field(r, "z"), "Invalid index")
  expect_error(field(r, NA_character_), "Invalid index")
  expect_error(field(r, ""), "Invalid index")
  expect_error(field(r, letters), "Invalid index")

  expect_error(field(r, 0L), "Invalid index")
  expect_error(field(r, NA_integer_), "Invalid index")

  expect_error(field(r, 0), "Invalid index")
  expect_error(field(r, NA_real_), "Invalid index")
  expect_error(field(r, Inf), "Invalid index")

  expect_error(field(r, mean), "Invalid index")

})

test_that("corrupt rcrd throws error", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_error(field(1:10, 1L), "Corrupt rcrd")
  expect_error(field(list(), 1L), "Corrupt rcrd")

  expect_error(field(list(1), "x"), "Corrupt x")
  expect_error(field(setNames(list(1, 1), "y"), "x"), "Corrupt x")
})


# set ---------------------------------------------------------------------

test_that("field<- modifies a copy", {
  r1 <- new_rcrd(list(x = 1, y = 2))
  r2 <- r1

  field(r1, "x") <- 3
  expect_equal(field(r1, "x"), 3)
  expect_equal(field(r2, "x"), 1)
})

test_that("field<- checks inputs", {
  x <- list()
  expect_error(field(x, "x") <- 1, "Corrupt rcrd")

  r <- new_rcrd(list(x = 1))
  expect_error(field(r, "x") <- 1:3, "Invalid value")
  expect_error(field(r, "x") <- environment(), "Invalid value")
})

test_that("field<- respects size, not length (#450)", {
  r1 <- new_rcrd(list(df = new_data_frame(n = 2L)))
  new_df <- data.frame(x = 1:2)

  field(r1, 'df') <- new_df
  expect_equal(field(r1, "df"), new_df)

  expect_error(field(r1, 'df') <- new_data_frame(n = 3L), "Invalid value")
})

test_that("n_fields captures number of fields", {
  r <- new_rcrd(list(x = 1, y = 2))
  expect_equal(n_fields(r), 2)
})


# get ---------------------------------------------------------------------

test_that("can extract valid field", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_equal(field(r, "x"), 1)
  expect_equal(field(r, 1L), 1)
})

test_that("can extract field even if encoding is different", {
  x1 <- "fa\u00e7ile"
  skip_if_not(Encoding(x1) == "UTF-8")

  x2 <- iconv(x1, from = "UTF-8", to = "latin1")
  skip_if_not(Encoding(x2) == "latin1")

  r <- new_rcrd(setNames(list(1), x1))
  expect_equal(field(r, x1), 1)
  expect_equal(field(r, x2), 1)
})

test_that("invalid indices throw error", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_error(field(r, "z"), "Invalid index")
  expect_error(field(r, NA_character_), "Invalid index")
  expect_error(field(r, ""), "Invalid index")
  expect_error(field(r, letters), "Invalid index")

  expect_error(field(r, 0L), "Invalid index")
  expect_error(field(r, NA_integer_), "Invalid index")

  expect_error(field(r, 0), "Invalid index")
  expect_error(field(r, NA_real_), "Invalid index")
  expect_error(field(r, Inf), "Invalid index")

  expect_error(field(r, mean), "Invalid index")
})

test_that("corrupt rcrd throws error", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_error(field(1:10, 1L), "Corrupt rcrd")
  expect_error(field(list(), 1L), "Corrupt rcrd")

  expect_error(field(list(1), "x"), "Corrupt x")
  expect_error(field(setNames(list(1, 1), "y"), "x"), "Corrupt x")
})


# set ---------------------------------------------------------------------

test_that("field<- modifies a copy", {
  r1 <- new_rcrd(list(x = 1, y = 2))
  r2 <- r1

  field(r1, "x") <- 3
  expect_equal(field(r1, "x"), 3)
  expect_equal(field(r2, "x"), 1)
})

test_that("field<- checks inputs", {
  x <- list()
  expect_error(field(x, "x") <- 1, "Corrupt rcrd")

  r <- new_rcrd(list(x = 1))
  expect_error(field(r, "x") <- 1:3, "Invalid value")
  expect_error(field(r, "x") <- environment(), "Invalid value")
})

test_that("field<- respects size, not length (#450)", {
  r1 <- new_rcrd(list(df = new_data_frame(n = 2L)))
  new_df <- data.frame(x = 1:2)

  field(r1, 'df') <- new_df
  expect_equal(field(r1, "df"), new_df)

  expect_error(field(r1, 'df') <- new_data_frame(n = 3L), "Invalid value")
})

# constructor and accessors -----------------------------------------------

test_that("can construct and access components", {
  r <- new_rcrd(list(x = 1, y = 2))

  expect_equal(length(r), 1)
  expect_equal(n_fields(r), 2)

  expect_equal(names(r), NULL)
  expect_equal(fields(r), c("x", "y"))

  expect_error(r$x, class = "vctrs_error_unsupported")
  expect_equal(field(r, "x"), 1)
})

test_that("requires format method", {
  x <- new_rcrd(list(x = 1))
  expect_error(format(x), class = "vctrs_error_unimplemented")
})

test_that("vec_proxy() transforms records to data frames", {
  expect_identical(
    vec_proxy(new_rcrd(list(a = "1"))),
    new_data_frame(list(a = "1"))
  )
})

test_that("equality, comparison, and order proxies are recursive and fall through (#1503, #1664)", {
  base <- new_rcrd(list(a = 1), class = "custom")
  x <- new_rcrd(list(x = base))

  expect_identical(vec_proxy_equal(x), 1)
  expect_identical(vec_proxy_compare(x), 1)
  expect_identical(vec_proxy_order(x), 1)

  local_methods(vec_proxy_equal.custom = function(x, ...) {
    rep("equal", length(x))
  })

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "equal")
  expect_identical(vec_proxy_order(x), "equal")

  local_methods(vec_proxy_compare.custom = function(x, ...) {
    rep("compare", length(x))
  })

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "compare")
  expect_identical(vec_proxy_order(x), "compare")

  local_methods(vec_proxy_order.custom = function(x, ...) {
    rep("order", length(x))
  })

  expect_identical(vec_proxy_equal(x), "equal")
  expect_identical(vec_proxy_compare(x), "compare")
  expect_identical(vec_proxy_order(x), "order")

  y <- new_rcrd(list(a = 1), class = "custom2")
  local_methods(vec_proxy_compare.custom2 = function(x, ...) {
    rep("compare2", length(x))
  })

  z <- data_frame(x = x, y = y)

  # Each column falls back independently
  expect_identical(vec_proxy_equal(z), data_frame(x = "equal", y = 1))
  expect_identical(
    vec_proxy_compare(z),
    data_frame(x = "compare", y = "compare2")
  )
  expect_identical(vec_proxy_order(z), data_frame(x = "order", y = "compare2"))
})

# base methods ------------------------------------------------------------

test_that("has no names", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_null(names(x))
  expect_null(vec_names(x))
})

test_that("removing names with `NULL` is a no-op (#1419)", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_identical(`names<-`(x, NULL), x)
  expect_identical(vec_set_names(x, NULL), x)
})

test_that("setting character names is an error (#1419)", {
  x <- new_rcrd(list(a = 1, b = 2L))

  expect_error(`names<-`(x, "x"), "Can't assign names")
  expect_error(vec_set_names(x, "x"), "Can't assign names")
})

test_that("na.omit() works and retains metadata (#1413)", {
  x <- new_rcrd(list(a = c(1, 1, NA, NA), b = c(1, NA, 1, NA)))
  result <- na.omit(x)

  expect <- vec_slice(x, 1:3)
  attr(expect, "na.action") <- structure(4L, class = "omit")

  expect_identical(result, expect)
})

test_that("na.fail() works", {
  # Only considered missing if all fields are missing
  x <- new_rcrd(list(a = c(1, 1, NA), b = c(1, NA, 1)))
  expect_identical(na.fail(x), x)

  x <- new_rcrd(list(a = c(1, 1, NA, NA), b = c(1, NA, 1, NA)))
  expect_snapshot(error = TRUE, na.fail(x))
})

# coercion ----------------------------------------------------------------

test_that("can't cast list to rcrd", {
  l <- list(
    new_rcrd(list(a = "1", b = 3L)),
    new_rcrd(list(b = "4", a = 2))
  )
  expect_error(
    vec_cast(l, new_rcrd(list(a = 1L, b = 2L))),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can recast rcrd from list", {
  r <- new_rcrd(list(x = integer(), y = numeric()))

  expect_equal(
    vec_restore(list(x = 1L, y = 1), r),
    new_rcrd(list(x = 1L, y = 1))
  )
})

test_that("can't cast rcrd to list", {
  r <- new_rcrd(list(x = 1:2, y = 2:3))
  expect_error(vec_cast(r, list()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(r, list()), class = "vctrs_error_incompatible_type")
})

test_that("default casts are implemented correctly", {
  r <- new_rcrd(list(x = 1, y = 1))

  expect_error(vec_cast(1, r), class = "vctrs_error_incompatible_type")
  expect_equal(vec_cast(NULL, r), NULL)
})

test_that("can't cast incompatible rcrd", {
  expect_error(
    vec_cast(
      new_rcrd(list(a = "1", b = 3L)),
      new_rcrd(list(a = "1"))
    ),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    vec_cast(
      new_rcrd(list(a = "1", b = 3L)),
      new_rcrd(list(a = "1", c = 3L))
    ),
    class = "vctrs_error_cast_lossy"
  )
  expect_error(
    vec_cast(
      new_rcrd(list(a = "a", b = 3L)),
      new_rcrd(list(a = 1, b = 3L))
    ),
    class = "vctrs_error_incompatible_type"
  )
})

# input validation --------------------------------------------------------

test_that("must be list of equal length vectors", {
  expect_error(new_rcrd(list()), "list of length 1")
  expect_error(
    new_rcrd(list(x = environment())),
    class = "vctrs_error_scalar_type"
  )
  expect_error(new_rcrd(list(x = 1:2, y = 1:3)), "same size")
})

test_that("names must be unique", {
  expect_error(
    new_rcrd(list(1, 2)),
    class = "vctrs_error_names_cannot_be_empty"
  )
  expect_error(
    new_rcrd(list(x = 1, 2)),
    class = "vctrs_error_names_cannot_be_empty"
  )
  expect_error(
    new_rcrd(list(x = 1, x = 2)),
    class = "vctrs_error_names_must_be_unique"
  )
  expect_error(new_rcrd(setNames(list(1, 2), "x")), "can't return `NA`")
})

test_that("subset assignment throws error", {
  x <- new_rcrd(list(x = 1))
  expect_error(
    x$y <- 2,
    class = "vctrs_error_unsupported"
  )
})

test_that("can supply data frame as fields", {
  expect_identical(
    new_rcrd(list(x = 1)),
    new_rcrd(tibble(x = 1))
  )
})

test_that("fields are not recycled", {
  expect_error(
    new_rcrd(list(x = 1, y = 1:2)),
    "must be the same size"
  )
})


# tuple class ----------------------------------------------------------
# use simple class to test essential features of rcrds

test_that("print and str use format", {
  local_tuple_methods()
  r <- tuple(1, 1:100)

  expect_snapshot(r)
  expect_snapshot(str(r[1:10]))
  expect_snapshot(str(list(list(list(r, 1:100)))))
})

test_that("subsetting methods applied to each field", {
  local_tuple_methods()
  x <- tuple(1:2, 1)

  expect_equal(x[1], tuple(1, 1))
  expect_equal(x[[1]], tuple(1, 1))

  expect_equal(rep(tuple(1, 1), 2), tuple(c(1, 1), 1))

  length(x) <- 1
  expect_equal(x, tuple(1, 1))
})

test_that("subset assignment modifies each field", {
  local_tuple_methods()
  x <- tuple(c(1, 1), c(2, 2))

  expect_error(x[[]] <- tuple(), "missing")

  x[[1]] <- tuple(3, 3)
  expect_equal(x, tuple(c(3, 1), c(3, 2)))

  x[1] <- tuple(4, 4)
  expect_equal(x, tuple(c(4, 1), c(4, 2)))
})

test_that("subset assignment recycles", {
  local_tuple_methods()

  x <- tuple(c(1, 1), c(2, 2))
  x[1:2] <- tuple(1, 1)
  expect_equal(x, tuple(c(1, 1), c(1, 1)))

  x[] <- tuple(2, 2)
  expect_equal(x, tuple(c(2, 2), c(2, 2)))
})

test_that("can sort rcrd", {
  local_tuple_methods()
  x <- tuple(c(1, 2, 1), c(3, 1, 2))
  expect_equal(xtfrm(x), c(2, 3, 1))
  expect_equal(order(x), c(3, 1, 2))
  expect_equal(sort(x), tuple(c(1, 1, 2), c(2, 3, 1)))
})

test_that("can use dictionary methods on a rcrd", {
  local_tuple_methods()
  x <- tuple(c(1, 2, 1), c(3, 1, 3))
  expect_equal(unique(x), x[1:2])
  expect_equal(duplicated(x), c(FALSE, FALSE, TRUE))
  expect_equal(anyDuplicated(x), TRUE)
})

test_that("cannot round trip through list", {
  local_tuple_methods()
  t <- tuple(1:2, 3:4)

  # Used to be allowed
  expect_error(vec_cast(t, list()), class = "vctrs_error_incompatible_type")
})

test_that("can convert to list using as.list() or vec_chop() (#1113)", {
  local_tuple_methods()
  t <- tuple(1:2, 3:4)

  expect <- list(tuple(1L, 3L), tuple(2L, 4L))

  expect_identical(as.list(t), expect)
  expect_identical(vec_chop(t), expect)
})

test_that("dangerous methods marked as unimplemented", {
  local_tuple_methods()
  t <- tuple()

  expect_error(mean(t), class = "vctrs_error_unsupported")
  expect_error(abs(t), class = "vctrs_error_unsupported")
  expect_error(is.finite(t), class = "vctrs_error_unsupported")
  expect_error(is.nan(t), class = "vctrs_error_unsupported")
})


# slicing -----------------------------------------------------------------

test_that("dots are an error (#1295)", {
  foo <- new_rcrd(list(foo = "foo"))
  expect_snapshot(error = TRUE, foo[1, 2])
})

test_that("records are restored after slicing the proxy", {
  expect_identical(new_rcrd(list(x = 1:2))[1], new_rcrd(list(x = 1L)))
})

test_that("can slice with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:2)))

  out <- vec_slice(x, 2)
  expect_identical(
    out,
    new_rcrd(data_frame(x = data_frame(y = 2L)))
  )
  expect_identical(
    x[2],
    out
  )
  expect_identical(
    x[[2]],
    out
  )
})

test_that("can rep with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:2)))

  expect_identical(
    rep(x, length.out = 4),
    vec_slice(x, c(1:2, 1:2))
  )
})

test_that("can assign with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:3)))
  y <- new_rcrd(data_frame(x = data_frame(y = FALSE)))
  exp <- new_rcrd(data_frame(x = data_frame(y = c(1L, 2L, 0L))))

  expect_identical(vec_assign(x, 3, y), exp)

  out <- x
  out[[3]] <- y
  expect_identical(out, exp)
})

test_that("can resize with df-cols fields", {
  x <- new_rcrd(data_frame(x = data_frame(y = 1:3)))

  length(x) <- 2
  expect_identical(x, new_rcrd(data_frame(x = data_frame(y = 1:2))))

  length(x) <- 4
  expect_identical(x, new_rcrd(data_frame(x = data_frame(y = c(1:2, NA, NA)))))
})

test_that("`[[` preserves type of record fields (#1205)", {
  x <- new_rcrd(list(x = 1:3, a = list(1, 2:3, 4:6)))
  expect_identical(field(x[3], "a"), list(4:6))
  expect_identical(field(x[[3]], "a"), list(4:6))
})

#' @description
#' `r lifecycle::badge("maturing")`
#'
#' Defines new notions of prototype and size that are
#' used to provide tools for consistent and well-founded type-coercion
#' and size-recycling, and are in turn connected to ideas of type- and
#' size-stability useful for analysing function interfaces.
#'
#' @keywords internal
#' @import rlang
#' @useDynLib vctrs, .registration = TRUE
"_PACKAGE"

release_extra_revdeps <- function() {
  # Extra revdeps to run before release.
  # Recognized by `usethis::use_release_issue()`.
  c("dplyr", "tidyr", "purrr")
}

# Helper for `list_combine()` tests that tests a `list_combine()` call across
# 3 variants:
#
# - Normal case
# - Homogenous fallback case (all S3 objects with the same class but no ptype2
#   method and no `c()` method. these end up using the "main" loop due to the
#   lack of `c()` method.)
# - `c()` fallback case (all S3 objects with the same class and a `c()` method.
#   these end up using the `base_c_invoke()` fallback path)
#
# It's important that all variants are fairly consistent
expect_identical_list_combine <- function(
  ...,
  x,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = "minimal",
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  expect
) {
  expect_something_list_combine(
    expect_identical,
    ...,
    x = x,
    indices = indices,
    size = size,
    default = default,
    unmatched = unmatched,
    multiple = multiple,
    slice_x = slice_x,
    ptype = ptype,
    name_spec = name_spec,
    name_repair = name_repair,
    x_arg = x_arg,
    indices_arg = indices_arg,
    default_arg = default_arg,
    expect = expect,
    foobar_expect = TRUE
  )
}

expect_named_list_combine <- function(
  ...,
  x,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = "minimal",
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  expect
) {
  expect_something_list_combine(
    expect_named,
    ...,
    x = x,
    indices = indices,
    size = size,
    default = default,
    unmatched = unmatched,
    multiple = multiple,
    slice_x = slice_x,
    ptype = ptype,
    name_spec = name_spec,
    name_repair = name_repair,
    x_arg = x_arg,
    indices_arg = indices_arg,
    default_arg = default_arg,
    expect = expect,
    foobar_expect = FALSE
  )
}

expect_snapshot_list_combine <- function(
  ...,
  x,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = "minimal",
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  error = FALSE
) {
  check_dots_empty0(...)

  expect_snapshot(error = error, {
    list_combine(
      x,
      indices = indices,
      size = size,
      default = default,
      unmatched = unmatched,
      multiple = multiple,
      slice_x = slice_x,
      ptype = ptype,
      name_spec = name_spec,
      name_repair = name_repair,
      x_arg = x_arg,
      indices_arg = indices_arg,
      default_arg = default_arg
    )
  })

  x_foobar <- lapply(x, function(elt) {
    if (is.null(elt)) {
      elt
    } else {
      foobar(elt)
    }
  })

  if (is.null(default)) {
    default_foobar <- NULL
  } else {
    default_foobar <- foobar(default)
  }

  # Homogeneous fallback
  expect_snapshot(error = error, {
    list_combine(
      x_foobar,
      indices = indices,
      size = size,
      default = default_foobar,
      unmatched = unmatched,
      multiple = multiple,
      slice_x = slice_x,
      ptype = ptype,
      name_spec = name_spec,
      name_repair = name_repair,
      x_arg = x_arg,
      indices_arg = indices_arg,
      default_arg = default_arg
    )
  })

  # So they show up differently in the snapshot
  x_foobar_c <- x_foobar
  default_foobar_c <- default_foobar

  # `c()` fallback
  with_c_foobar({
    expect_snapshot(error = error, {
      list_combine(
        x_foobar_c,
        indices = indices,
        size = size,
        default = default_foobar_c,
        unmatched = unmatched,
        multiple = multiple,
        slice_x = slice_x,
        ptype = ptype,
        name_spec = name_spec,
        name_repair = name_repair,
        x_arg = x_arg,
        indices_arg = indices_arg,
        default_arg = default_arg
      )
    })
  })
}

expect_something_list_combine <- function(
  expect_fn,
  ...,
  x,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = "minimal",
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  expect,
  foobar_expect = TRUE
) {
  check_dots_empty0(...)

  expect_fn(
    list_combine(
      x,
      indices = indices,
      size = size,
      default = default,
      unmatched = unmatched,
      multiple = multiple,
      slice_x = slice_x,
      ptype = ptype,
      name_spec = name_spec,
      name_repair = name_repair,
      x_arg = x_arg,
      indices_arg = indices_arg,
      default_arg = default_arg
    ),
    expected = expect
  )

  x_foobar <- lapply(x, function(elt) {
    if (is.null(elt)) {
      elt
    } else {
      foobar(elt)
    }
  })

  if (is.null(default)) {
    default_foobar <- NULL
  } else {
    default_foobar <- foobar(default)
  }

  if (foobar_expect) {
    expect_foobar <- foobar(expect)
  } else {
    expect_foobar <- expect
  }

  # Homogeneous fallback
  expect_fn(
    list_combine(
      x_foobar,
      indices = indices,
      size = size,
      default = default_foobar,
      unmatched = unmatched,
      multiple = multiple,
      slice_x = slice_x,
      ptype = ptype,
      name_spec = name_spec,
      name_repair = name_repair,
      x_arg = x_arg,
      indices_arg = indices_arg,
      default_arg = default_arg
    ),
    expected = expect_foobar
  )

  if (foobar_expect) {
    expect_foobar_c <- foobar_c(expect)
  } else {
    expect_foobar_c <- expect
  }

  # `c()` fallback
  with_c_foobar({
    expect_fn(
      list_combine(
        x_foobar,
        indices = indices,
        size = size,
        default = default_foobar,
        unmatched = unmatched,
        multiple = multiple,
        slice_x = slice_x,
        ptype = ptype,
        name_spec = name_spec,
        name_repair = name_repair,
        x_arg = x_arg,
        indices_arg = indices_arg,
        default_arg = default_arg
      ),
      expected = expect_foobar_c
    )
  })
}

test_that("basic `list_combine()` works", {
  values <- list(1:2, 3:4)
  indices <- list(c(4, 1), c(3, 2))
  size <- 4

  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    expect = c(2L, 4L, 3L, 1L)
  )
})

test_that("`list_combine()` works with homogeneous fallback in `default`", {
  # This errors
  expect_snapshot(error = TRUE, {
    list_combine(
      list(foobar(1), 1),
      indices = list(1, 2),
      size = 2
    )
  })

  # So this should too
  expect_snapshot(error = TRUE, {
    list_combine(
      list(foobar(1)),
      indices = list(1),
      size = 2,
      default = 1
    )
  })
})

test_that("`list_combine()` returns `unspecified` to retain `size` invariant", {
  expect_identical(
    list_combine(list(), indices = list(), size = 0),
    unspecified()
  )
  expect_identical(
    list_combine(list(), indices = list(), size = 1),
    unspecified(1)
  )
  expect_identical(
    list_combine(list(), indices = list(), size = 2),
    unspecified(2)
  )

  # With only `NULL` elements, still no known `ptype`
  expect_identical(
    list_combine(list(NULL), indices = list(integer()), size = 0),
    unspecified(0)
  )
  expect_identical(
    list_combine(list(NULL), indices = list(integer()), size = 2),
    unspecified(2)
  )
  expect_identical(
    list_combine(list(NULL), indices = list(1:2), size = 2),
    unspecified(2)
  )

  # Now ptype is known
  expect_identical(
    list_combine(
      list(NULL),
      indices = list(integer()),
      ptype = numeric(),
      size = 1
    ),
    vec_init(numeric(), n = 1)
  )
})

test_that("`NULL`s are removed after computing `default`'s index", {
  expect_identical_list_combine(
    x = list(1:2, NULL, 7:8),
    indices = list(1:2, 5:6, 7:8),
    size = 8,
    default = 0L,
    expect = int(1, 2, 0, 0, NA, NA, 7, 8)
  )
})

test_that("can combine factors", {
  fctr1 <- factor("z")
  fctr2 <- factor(c("x", "y"))

  x <- list(fctr1, fctr2)
  indices <- list(2, c(3, 1))

  # levels are in the order they are seen!
  expect <- factor(c("y", "z", "x"), levels = c("z", "x", "y"))

  expect_identical(list_combine(x, indices = indices, size = 3), expect)

  # With `default`, the `default` is added in as the last item
  default <- factor("w")
  expect <- factor(c("y", "z", "x"), levels = c("z", "x", "y", "w"))
  expect_identical(
    list_combine(x, indices = indices, size = 3, default = default),
    expect
  )
})

test_that("preserves names when inputs are cast to a common type (#1689)", {
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(1),
      ptype = integer(),
      size = 1
    ),
    "a"
  )
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(TRUE),
      ptype = integer(),
      size = 1
    ),
    "a"
  )

  # With `default`
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(1),
      default = c(b = 0),
      ptype = integer(),
      size = 2
    ),
    c("a", "b")
  )
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(c(TRUE, FALSE)),
      default = c(b = 0),
      ptype = integer(),
      size = 2
    ),
    c("a", "b")
  )

  # With name spec
  name_spec <- "{outer}_{inner}"
  expect_named(
    list_combine(
      list(foo = c(a = 1)),
      indices = list(1),
      size = 1,
      ptype = integer(),
      name_spec = name_spec
    ),
    "foo_a"
  )
  expect_named(
    list_combine(
      list(foo = c(a = 1)),
      indices = list(1),
      default = c(b = 0),
      ptype = integer(),
      name_spec = name_spec,
      size = 2
    ),
    c("foo_a", "b")
  )
  expect_named(
    list_combine(
      list(foo = c(a = 1)),
      indices = list(c(TRUE, FALSE)),
      default = c(b = 0),
      ptype = integer(),
      name_spec = name_spec,
      size = 2
    ),
    c("foo_a", "b")
  )

  # When `x` elements are recycled, names are also recycled
  x <- list(c(a = 1), c(b = 2))
  expect_named(
    list_combine(
      x,
      indices = list(1:2, 3:4),
      size = 4,
      ptype = integer()
    ),
    c("a", "a", "b", "b")
  )
  expect_named(
    list_combine(
      x,
      indices = list(
        c(TRUE, TRUE, FALSE, FALSE),
        c(FALSE, FALSE, TRUE, TRUE)
      ),
      size = 4,
      ptype = integer()
    ),
    c("a", "a", "b", "b")
  )

  # When `default` elements are recycled, names are also recycled
  expect_named(
    list_combine(
      list(c(a = 1), c(b = 2)),
      indices = list(1, 3),
      default = c(c = 0),
      ptype = integer(),
      size = 4
    ),
    c("a", "c", "b", "c")
  )
  expect_named(
    list_combine(
      list(c(a = 1), c(b = 2)),
      indices = list(
        c(TRUE, FALSE, FALSE, FALSE),
        c(FALSE, FALSE, TRUE, FALSE)
      ),
      default = c(c = 0),
      ptype = integer(),
      size = 4
    ),
    c("a", "c", "b", "c")
  )
})

test_that("not all inputs have to be named", {
  x <- list(c(a = 1), 2, c(c = 3))
  indices <- list(2, 1, 3)

  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 3,
    expect = set_names(c(2, 1, 3), c("", "a", "c"))
  )
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 4,
    default = 0,
    expect = set_names(c(2, 1, 3, 0), c("", "a", "c", ""))
  )
})

test_that("list_combine() keeps data frame row names", {
  df1 <- data.frame(x = 1:2, row.names = c("r1", "r2"))
  df2 <- data.frame(x = 3:4, row.names = c("r3", "r4"))
  x <- list(df1, df2)
  indices <- list(c(3, 1), c(2, 4))

  result <- list_combine(x, indices = indices, size = sum(lengths(indices)))
  expect <- c("r2", "r3", "r1", "r4")
  expect_identical(vec_names(result), expect)

  default <- data.frame(x = 0L, row.names = "d")
  result <- list_combine(x, indices = indices, size = 5, default = default)
  expect <- c("r2", "r3", "r1", "r4", "d")
  expect_identical(vec_names(result), expect)

  # With casting
  ptype <- data.frame(x = double())
  default <- data.frame(x = 0L, row.names = "d")
  result <- list_combine(
    x = x,
    indices = indices,
    size = 5,
    default = default,
    ptype = ptype
  )
  expect <- c("r2", "r3", "r1", "r4", "d")
  expect_identical(vec_names(result), expect)
  expect_type(result$x, "double")
})

test_that("df-col row names are repaired silently", {
  df1 <- data_frame(x = new_data_frame(list(a = 1), row.names = "inner"))
  df2 <- data_frame(x = new_data_frame(list(a = 2), row.names = "inner"))
  x <- list(df1, df2)
  indices <- list(1, 2)
  expect_silent({
    result <- list_combine(x, indices = indices, size = 2)
  })
  expect_identical(vec_names(result$x), c("inner...1", "inner...2"))

  default <- data_frame(x = new_data_frame(list(a = 0), row.names = "inner"))
  expect_silent({
    result <- list_combine(x, indices = indices, size = 3, default = default)
  })
  expect_identical(
    vec_names(result$x),
    c("inner...1", "inner...2", "inner...3")
  )
})

test_that("name repair is respected and happens after ordering according to `indices`", {
  local_name_repair_quiet()

  x <- list(c(a = 1), c(a = 2))

  indices <- list(2, 1)
  expect_named(list_combine(x, indices = indices, size = 2), c("a", "a"))
  expect_named(
    list_combine(x, indices = indices, size = 2, name_repair = "unique"),
    c("a...1", "a...2")
  )

  # With `default`
  indices <- list(3, 1)
  default <- c(a = 0)
  expect_named(
    list_combine(x, indices = indices, size = 3, default = default),
    c("a", "a", "a")
  )
  expect_named(
    list_combine(
      x,
      indices = indices,
      size = 3,
      default = default,
      name_repair = "unique"
    ),
    c("a...1", "a...2", "a...3")
  )
})

test_that("list_combine() works with simple homogeneous foreign S3 classes", {
  expect_identical(
    list_combine(
      list(foobar(1), foobar(2)),
      indices = list(1, 2),
      size = 2
    ),
    vec_c(foobar(c(1, 2)))
  )

  # With `default`
  expect_identical(
    list_combine(
      list(foobar(1), foobar(2)),
      indices = list(3, 1),
      size = 4,
      default = foobar(0)
    ),
    vec_c(foobar(c(2, 0, 1, 0)))
  )
})

test_that("list_combine() fails with complex foreign S3 classes", {
  expect_snapshot(error = TRUE, {
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    list_combine(list(x, y), indices = list(1, 2), size = 2)
  })
  expect_snapshot(error = TRUE, {
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    list_combine(
      list(x, y),
      indices = list(1, 2),
      size = 2,
      error_call = call("foo"),
      x_arg = "arg"
    )
  })

  # Want this to mention `default` by name
  expect_snapshot(error = TRUE, {
    x <- structure(foobar(1), attr_foo = "foo")
    default <- structure(foobar(2), attr_foo = "bar")
    list_combine(
      list(x),
      indices = list(1),
      size = 2,
      default = default
    )
  })
  expect_snapshot(error = TRUE, {
    x <- structure(foobar(1), attr_foo = "foo")
    default <- structure(foobar(2), attr_foo = "bar")
    list_combine(
      list(x),
      indices = list(1),
      size = 2,
      default = default,
      default_arg = "d"
    )
  })
})

test_that("list_combine() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(c(1L, 2L), name = "Joe")
    jane <- .Counts(3L, name = "Jane")
    (expect_error(
      list_combine(list(joe, jane), indices = list(1:2, 3), size = 3),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      list_combine(
        list(joe, jane),
        indices = list(1:2, 3),
        size = 3,
        error_call = call("foo"),
        x_arg = "arg"
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_combine() falls back to c() if S3 method is available", {
  # Check off-by-one error
  expect_error(
    list_combine(
      list(foobar(1), "", foobar(2)),
      indices = list(1, 2, 3),
      size = 3
    ),
    class = "vctrs_error_incompatible_type"
  )

  # Fallback when the class implements `c()`
  method_foobar <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    foobar(res)
  }

  local_methods(
    c.vctrs_foobar = method_foobar
  )
  expect_identical(
    list_combine(list(foobar(1), foobar(2)), indices = list(1, 2), size = 2),
    foobar(c(1, 2))
  )
  expect_identical(
    list_combine(list(foobar(1), foobar(2)), indices = list(2, 1), size = 2),
    foobar(c(2, 1))
  )
  expect_identical(
    list_combine(
      list(NULL, foobar(1), NULL, foobar(2)),
      indices = list(integer(), 1, integer(), 2),
      size = 2
    ),
    foobar(c(1, 2))
  )

  # OOB error is respected
  expect_error(
    list_combine(
      list(foobar(1), foobar(2)),
      indices = list(1, 3),
      size = 2
    ),
    class = "vctrs_error_subscript_oob"
  )

  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(1, 3), 1),
      size = 3
    ),
    foobar(c(3, NA, 2))
  )
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(2, NA), NA),
      size = 3
    ),
    foobar(c(NA, 1, NA))
  )

  # Respects `default`
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(1, 3), 1),
      default = foobar(0),
      size = 4
    ),
    foobar(c(3, 0, 2, 0))
  )
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(1, 3), 1),
      default = foobar(c(4, 5, 6, 7)),
      size = 4
    ),
    foobar(c(3, 5, 2, 7))
  )

  # Names are kept
  expect_identical(
    list_combine(
      list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))),
      indices = list(c(2, 1), 3),
      size = 3
    ),
    foobar(c(y = 2, x = 1, x = 1))
  )
  expect_identical(
    list_combine(
      list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))),
      indices = list(c(2, 1), 3),
      size = 5,
      default = foobar(c(default = 0))
    ),
    foobar(c(y = 2, x = 1, x = 1, default = 0, default = 0))
  )

  # Recycles to the size of index
  expect_identical(
    list_combine(
      list(foobar(1), foobar(2)),
      indices = list(c(1, 3), 2),
      size = 3
    ),
    foobar(c(1, 2, 1))
  )
  expect_identical(
    list_combine(
      list(foobar(1), foobar(2)),
      indices = list(c(1, 2), integer()),
      size = 2
    ),
    foobar(c(1, 1))
  )
  expect_snapshot({
    (expect_error(
      list_combine(
        list(foobar(1), foobar(2)),
        indices = list(c(1, 3), integer()),
        size = 2
      ),
      class = "vctrs_error_subscript_oob"
    ))
  })
  expect_snapshot({
    x <- list(foobar(1:2))
    indices <- list(1:3)
    (expect_error(list_combine(x, indices = indices, size = 3)))
    (expect_error(list_combine(
      x,
      indices = indices,
      size = 3,
      x_arg = "arg",
      error_call = call("foo")
    )))
  })

  method_vctrs_c_fallback <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    structure(res, class = "vctrs_c_fallback")
  }

  # Registered fallback
  s3_register("base::c", "vctrs_c_fallback", method_vctrs_c_fallback)
  expect_identical(
    list_combine(
      list(
        structure(1, class = "vctrs_c_fallback"),
        structure(2, class = "vctrs_c_fallback")
      ),
      indices = list(2, 1),
      size = 2
    ),
    structure(c(2, 1), class = "vctrs_c_fallback")
  )

  # Don't fallback for S3 lists which are treated as scalars by default
  expect_error(
    list_combine(
      list(foobar(list(1)), foobar(list(2))),
      indices = list(1, 2),
      size = 2
    ),
    class = "vctrs_error_scalar_type"
  )
})

test_that("list_combine() falls back to c() if S3 method is available and respects `default`", {
  # Fallback when the class implements `c()`
  method_foobar <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    foobar(res)
  }

  local_methods(
    c.vctrs_foobar = method_foobar
  )

  # Respects `default`
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(1, 3), 1),
      default = foobar(0),
      size = 4
    ),
    foobar(c(3, 0, 2, 0))
  )
  expect_identical(
    list_combine(
      list(foobar(c(1, 2)), foobar(3)),
      indices = list(c(1, 3), 1),
      default = foobar(c(4, 5, 6, 7)),
      size = 4
    ),
    foobar(c(3, 5, 2, 7))
  )

  # Names are kept
  expect_identical(
    list_combine(
      list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))),
      indices = list(c(2, 1), 3),
      size = 3
    ),
    foobar(c(y = 2, x = 1, x = 1))
  )
  expect_identical(
    list_combine(
      list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))),
      indices = list(c(2, 1), 3),
      size = 5,
      default = foobar(c(default = 0))
    ),
    foobar(c(y = 2, x = 1, x = 1, default = 0, default = 0))
  )
})

test_that("list_combine() falls back for S4 classes with a registered c() method", {
  joe <- .Counts(c(1L, 2L), name = "Joe")
  jane <- .Counts(3L, name = "Jane")
  expect_snapshot({
    (expect_error(
      list_combine(list(joe, 1, jane), indices = list(c(1, 2), 3, 4), size = 4),
      class = "vctrs_error_incompatible_type"
    ))
  })
  local_c_counts()
  expect_identical(
    list_combine(list(joe, jane), indices = list(c(1, 3), 2), size = 3),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )
  expect_identical(
    list_combine(
      list(NULL, joe, jane),
      indices = list(integer(), c(1, 3), 2),
      size = 3
    ),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )
  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_combine(list(joe, jane), indices = list(c(1, 3), 1), size = 3),
    .Counts(c(3L, NA, 2L), name = "Dispatched")
  )
  expect_identical(
    list_combine(list(joe, jane), indices = list(c(2, NA), NA), size = 3),
    .Counts(c(NA, 1L, NA), name = "Dispatched")
  )

  # `default` is respected
  default <- .Counts(0L, name = "Unknown")

  expect_identical(
    list_combine(
      list(joe, jane),
      indices = list(c(1, 3), 1),
      default = default,
      size = 5
    ),
    .Counts(c(3L, 0L, 2L, 0L, 0L), name = "Dispatched")
  )
})

test_that("can ignore names in `list_combine()` by providing a `zap()` name-spec (#232)", {
  expect_snapshot({
    (expect_error(
      list_combine(
        list(a = c(b = 1:2)),
        indices = list(1:2),
        size = 2
      )
    ))
    (expect_error(
      list_combine(
        list(a = c(b = 1:2)),
        indices = list(1:2),
        size = 2,
        error_call = call("foo")
      )
    ))
  })

  expect_identical(
    list_combine(
      list(a = c(b = 1:2), b = 3L),
      indices = list(1:2, 3),
      size = 3,
      name_spec = zap()
    ),
    1:3
  )
  expect_identical(
    list_combine(
      list(a = c(foo = 1:2), b = c(bar = 3L)),
      indices = list(2:1, 3),
      size = 3,
      name_spec = zap()
    ),
    c(2L, 1L, 3L)
  )
  expect_identical(
    list_combine(
      list(a = c(foo = 1:2), b = c(bar = 3L)),
      indices = list(2:1, 3),
      default = c(baz = 0L),
      size = 4,
      name_spec = zap()
    ),
    c(2L, 1L, 3L, 0L)
  )

  expect_snapshot({
    x <- list(a = c(b = c("a", "b")), b = 3L)
    (expect_error(
      list_combine(
        x,
        indices = list(1:2, 3),
        size = 3,
        name_spec = zap()
      ),
      class = "vctrs_error_incompatible_type"
    ))

    x <- list(a = c(foo = 1:2), b = c(bar = ""))
    (expect_error(
      list_combine(
        x,
        indices = list(2:1, 3),
        size = 3,
        name_spec = zap()
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_combine() falls back to c() methods (#1120)", {
  expect_error(
    list_combine(
      list(foobar(1), foobar(2, class = "foo")),
      indices = list(1, 2),
      size = 2
    ),
    class = "vctrs_error_incompatible_type"
  )
  local_methods(
    c.vctrs_foobar = function(...) {
      out <- NextMethod()
      paste0(rep_along(out, "dispatched"), seq_along(out))
    }
  )
  # Homogeneous subclasses
  xs <- list(foobar(1), foobar(2, class = "foo"))
  expect_identical(
    list_combine(xs, indices = list(2, 1), size = 2),
    c("dispatched2", "dispatched1")
  )
  expect_identical(
    list_combine(xs, indices = list(3, 1), size = 4, default = foobar(0)),
    c("dispatched2", "dispatched3", "dispatched1", "dispatched4")
  )

  # Different subclasses
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )
  expect_identical(
    list_combine(xs, indices = list(c(2, 1), 3), size = 3),
    c("dispatched2", "dispatched1", "dispatched3")
  )
  expect_identical(
    list_combine(xs, indices = list(c(4, 1), 3), size = 5, default = foobar(0)),
    c("dispatched2", "dispatched4", "dispatched3", "dispatched1", "dispatched5")
  )
})

test_that("list_combine() fails if foreign classes are not homogeneous and there is no c() method", {
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )
  expect_error(
    list_combine(xs, indices = list(c(2, 1), 3), size = 3),
    class = "vctrs_error_incompatible_type"
  )

  x <- foobar(c(x = 1, y = 2), class = "foo")
  default <- foobar(c(x = 1), foo = 1)
  expect_snapshot(error = TRUE, {
    list_combine(
      list(x),
      indices = list(c(1, 2)),
      size = 3,
      default = default
    )
  })
})

test_that("recycling error indices are correct even with `NULL` removal", {
  # Should be element 3!
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, NULL, 7:8),
    indices = list(1:2, 5:6, 7:9),
    size = 9,
    default = 0L
  )
})

test_that("size 1 `default` is recycled correctly", {
  expect_identical_list_combine(
    x = list(1:2, 5L, 7:8),
    indices = list(1:2, 5, 7:8),
    size = 9,
    default = 0L,
    expect = int(1, 2, 0, 0, 5, 0, 7, 8, 0)
  )
})

test_that("`indices` is required!", {
  expect_error(list_combine(list(1, 2), size = 2))
})

test_that("`size` is required!", {
  expect_error(list_combine(list(1, 2), indices = list(1, 2)))
})

test_that("`x_arg` works", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1, "2"), indices = list(1, 2), size = 2, x_arg = "xs")
  })
  expect_snapshot(error = TRUE, {
    list_combine(list(1, 2), indices = list(1, 2, 3), size = 2, x_arg = "xs")
  })
})

test_that("`indices_arg` works", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1, 2), indices = 1, size = 2, indices_arg = "i")
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1, 2),
      indices = list(1, 2, 3),
      size = 2,
      indices_arg = "i"
    )
  })
})

test_that("`...` must be empty", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1, 2), indices = list(1, 2), size = 2, "foo")
  })
})

test_that("list_combine() `default` is inserted correctly", {
  xs <- list("a", "b")
  indices <- list(1, 3)

  expect_identical_list_combine(
    x = xs,
    indices = indices,
    size = 4,
    default = "c",
    expect = c("a", "c", "b", "c")
  )
})

test_that("list_combine() `default` is inserted correctly with data frames", {
  xs <- list(
    data.frame(a = 1:2, b = 1:2),
    data.frame(a = 3L, b = 3L)
  )
  indices <- list(1:2, 5)

  expect_identical(
    list_combine(
      x = xs,
      indices = indices,
      size = 6,
      default = data.frame(a = 0L, b = NA_integer_)
    ),
    data.frame(
      a = int(1, 2, 0, 0, 3, 0),
      b = int(1, 2, NA, NA, 3, NA)
    )
  )
})

test_that("list_combine() `default` is inserted correctly when there are duplicate indices", {
  xs <- list("a", "b", "c")
  indices <- list(1, 1, 3)

  expect_identical_list_combine(
    x = xs,
    indices = indices,
    size = 4,
    default = "d",
    expect = c("b", "d", "c", "d")
  )
})

test_that("list_combine() `default` is inserted correctly when it is the size of `size`", {
  xs <- list("2", "4")
  indices <- list(2, 4)

  default <- letters[1:5]

  expect_identical_list_combine(
    x = xs,
    indices = indices,
    size = 5,
    default = default,
    expect = c("a", "2", "c", "4", "e")
  )
})

test_that("list_combine() `default` is correctly not used when all spots are filled", {
  xs <- list("a", "b", "c")
  indices <- list(1, 2, 3)

  expect_identical_list_combine(
    x = xs,
    indices = indices,
    size = 3,
    default = "d",
    expect = c("a", "b", "c")
  )
})

test_that("list_combine() `default` names work correctly with `name_spec`", {
  xs <- list(x = c(a = "a"), y = c(b = "b"), z = c(c = "c"))
  indices <- list(1, 3, NA)

  expect_identical(
    list_combine(
      xs,
      indices = indices,
      size = 5,
      default = c(d = "d"),
      name_spec = "{outer}_{inner}"
    ),
    c(x_a = "a", d = "d", y_b = "b", d = "d", d = "d")
  )
})

test_that("list_combine() `size` type is validated", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1), indices = list(1), size = "x")
  })
})

test_that("list_combine() `indices` are validated against `size`", {
  # TODO: Not the best error message here
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1),
    indices = list(3),
    size = 2
  )
})

test_that("list_combine() `default` vector check is done", {
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1),
    indices = list(1),
    size = 1,
    default = lm(1 ~ 1),
    default_arg = "d"
  )
})

test_that("list_combine() `default` size check is done", {
  # Must be size 1, or same size as `size`
  expect_identical_list_combine(
    x = list(1),
    indices = list(2),
    size = 3,
    default = 0,
    default_arg = "d",
    expect = c(0, 1, 0)
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1L),
    indices = list(1),
    size = 1,
    default = 1:2,
    default_arg = "d"
  )
})

test_that("list_combine() `default` is taken into account when computing `ptype`", {
  expect_identical(
    list_combine(
      list(1L),
      indices = list(1),
      size = 2,
      default = 1.5,
      default_arg = "d"
    ),
    c(1, 1.5)
  )

  # `default` is not in output, but helps determine output type!
  expect_identical(
    list_combine(
      list(1L),
      indices = list(1),
      size = 1,
      default = 1.5,
      default_arg = "d"
    ),
    1
  )

  # Empty `xs` and `indices`, but `default` is provided and determines type,
  # which would otherwise be <unspecified>
  expect_identical_list_combine(
    x = list(),
    indices = list(),
    size = 2,
    default = 0,
    default_arg = "d",
    expect = c(0, 0)
  )

  # Computed `ptype` among `xs` isn't compatible with `default`
  expect_snapshot(error = TRUE, {
    list_combine(
      list(x = 1),
      indices = list(1),
      size = 2,
      default = "a",
      default_arg = "d"
    )
  })

  # Provided `ptype` isn't compatible with `default`
  expect_snapshot(error = TRUE, {
    list_combine(
      list(x = 1L),
      indices = list(1),
      size = 2,
      default = 1.5,
      default_arg = "d",
      ptype = integer()
    )
  })
})

test_that("list_combine() works with data frame `default` with fallback columns", {
  x <- list(
    data_frame(a = foobar(1:2), b = 3:4, c = foobar(5:6)),
    data_frame(a = foobar(3L), b = 5L, c = foobar(7L))
  )
  indices <- list(
    c(3, NA),
    2
  )
  default <- data_frame(a = foobar(0L), b = 0L, c = foobar(-1L))

  expect_identical(
    list_combine(
      x = x,
      indices = indices,
      size = 5,
      default = default
    ),
    data_frame(
      a = foobar(int(0, 3, 1, 0, 0)),
      b = int(0, 5, 3, 0, 0),
      c = foobar(int(-1, 7, 5, -1, -1))
    )
  )

  with_c_foobar({
    expect_identical(
      list_combine(
        x = x,
        indices = indices,
        size = 5,
        default = default
      ),
      data_frame(
        a = foobar_c(int(0, 3, 1, 0, 0)),
        b = int(0, 5, 3, 0, 0),
        c = foobar_c(int(-1, 7, 5, -1, -1))
      )
    )
  })
})

test_that("list_combine() `unmatched = 'error'` doesn't error when all locations are covered", {
  expect_identical_list_combine(
    x = list(1:3),
    indices = list(1:3),
    size = 3,
    unmatched = "error",
    expect = 1:3
  )
  expect_identical_list_combine(
    x = list(1:3),
    indices = list(c(TRUE, TRUE, TRUE)),
    size = 3,
    unmatched = "error",
    expect = 1:3
  )
})

test_that("list_combine() `unmatched = 'error'` doesn't error in the empty case", {
  expect_identical(
    list_combine(list(), indices = list(), size = 0, unmatched = "error"),
    unspecified()
  )
})

test_that("list_combine() `unmatched = 'error'` errors with unmatched `indices` when `size` is used", {
  expect_snapshot(error = TRUE, {
    # Duplicates result in unmatched locations
    list_combine(
      list(1, 1),
      indices = list(1, 1),
      size = 2,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # `NA` results in unmatched locations
    list_combine(
      list(1, 1),
      indices = list(1, NA),
      size = 2,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # `NA` results in unmatched locations
    list_combine(
      list(1:9, 1:9),
      indices = list(
        c(TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA),
        c(TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, NA, NA, NA)
      ),
      size = 9,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # Unused location
    list_combine(
      list(1, 3),
      indices = list(1, 3),
      size = 3,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # Unused location
    list_combine(
      list(1, 1),
      indices = list(c(TRUE, FALSE), c(TRUE, FALSE)),
      size = 2,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(),
      indices = list(),
      size = 2,
      unmatched = "error"
    )
  })
})

test_that("list_combine() `unmatched = 'error'` errors pluralize correctly", {
  expect_snapshot(error = TRUE, {
    # One location
    list_combine(
      list(1, 3),
      indices = list(1, 3),
      size = 3,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # Two locations
    list_combine(
      list(1, 3),
      indices = list(1, 3),
      size = 4,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    # Many locations
    list_combine(
      list(1, 3),
      indices = list(1, 3),
      size = 100,
      unmatched = "error"
    )
  })
})

test_that("list_combine() `unmatched = 'error'` error classes are as expected", {
  cnd <- catch_cnd(list_combine(
    list(1, 3),
    indices = list(1, 3),
    size = 3,
    unmatched = "error"
  ))
  expect_true(inherits_all(
    cnd,
    c("vctrs_error_combine_unmatched", "vctrs_error_combine")
  ))
})

test_that("list_combine() `unmatched = 'error'` can't be set when `default` is also set", {
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      default = 1,
      size = 1,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      default = 1,
      size = 1,
      unmatched = "error",
      default_arg = ".default",
      error_call = quote(foo())
    )
  })
})

test_that("list_combine() `unmatched` is validated", {
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      unmatched = "e"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      unmatched = c("a", "b")
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      unmatched = NA_character_
    )
  })

  # With error call
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      unmatched = "e",
      error_call = quote(foo())
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      unmatched = c("a", "b"),
      error_call = quote(foo())
    )
  })
})

test_that("list_combine() `multiple` is validated", {
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      multiple = "a"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      multiple = c("a", "b")
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      multiple = NA_character_
    )
  })

  # With error call
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      multiple = "a",
      error_call = quote(foo())
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      list(1),
      indices = list(1),
      size = 1,
      multiple = c("a", "b"),
      error_call = quote(foo())
    )
  })
})

test_that("`NA` indices are considered unmatched locations", {
  # The `NA` index is considered "unmatched". It's the same as providing a
  # logical index with an `NA` in it, like indices in this case of: list(c(TRUE,
  # FALSE, FALSE), c(FALSE, NA, TRUE)) We consider that `NA` to be unmatched,
  # and running vec_as_location on that gives the same index as this.
  expect_identical_list_combine(
    x = list(1L, 2:3),
    indices = list(1, c(NA, 3)),
    size = 3,
    default = 0L,
    expect = c(1L, 0L, 3L)
  )
  expect_identical_list_combine(
    x = list(1L, 2:3),
    indices = list(c(TRUE, FALSE, FALSE), c(FALSE, NA, TRUE)),
    size = 3,
    default = 0L,
    expect = c(1L, 0L, 3L)
  )

  # Which means this errors
  expect_snapshot(error = TRUE, {
    list_combine(
      x = list(1, 2:3),
      indices = list(1, c(NA, 3)),
      size = 3,
      unmatched = "error"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      x = list(1, 2:3),
      indices = list(c(TRUE, FALSE, FALSE), c(FALSE, NA, TRUE)),
      size = 3,
      unmatched = "error"
    )
  })
})

test_that("`indices` corresponding to `NULL` values are considered matched", {
  # These are considered "matched" but there is nothing to put there,
  # so `NA` ends up in the slot
  expect_identical_list_combine(
    x = list(1, NULL),
    indices = list(1, c(2, 3)),
    size = 3,
    default = 0,
    expect = c(1, NA, NA)
  )
  expect_identical_list_combine(
    x = list(1, NULL),
    indices = list(c(TRUE, FALSE, FALSE), c(FALSE, TRUE, TRUE)),
    size = 3,
    default = 0,
    expect = c(1, NA, NA)
  )

  # Which means this is not an error
  expect_identical_list_combine(
    x = list(1, NULL),
    indices = list(1, c(2, 3)),
    size = 3,
    unmatched = "error",
    expect = c(1, NA, NA)
  )
  expect_identical_list_combine(
    x = list(1, NULL),
    indices = list(c(TRUE, FALSE, FALSE), c(FALSE, TRUE, TRUE)),
    size = 3,
    unmatched = "error",
    expect = c(1, NA, NA)
  )
})

test_that("`x` must be a list", {
  expect_snapshot(error = TRUE, {
    list_combine(1, indices = list(1), size = 1)
  })
  expect_snapshot(error = TRUE, {
    list_combine(
      1,
      indices = list(1),
      size = 1,
      error_call = call("foo"),
      x_arg = "arg"
    )
  })
  expect_snapshot(error = TRUE, {
    list_combine(data.frame(x = 1), indices = list(1), size = 1)
  })
  expect_snapshot(error = TRUE, {
    list_combine(array(list(1)), indices = list(1), size = 1)
  })
})

test_that("`indices` must be a list", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1), indices = 1, size = 1)
  })
  expect_snapshot(error = TRUE, {
    list_combine(list(1), indices = 1, size = 1, error_call = call("foo"))
  })
  expect_snapshot(error = TRUE, {
    list_combine(list(1), indices = data.frame(x = 1), size = 1)
  })
})

test_that("`indices` has a restricted type", {
  expect_error(
    list_combine(list(1), indices = list("x"), size = 1),
    class = "vctrs_error_subscript_type"
  )
  expect_error(
    list_combine(list(1), indices = list(quote(name)), size = 1),
    class = "vctrs_error_subscript_type"
  )
})

test_that("`x` and `indices` must be lists of the same size", {
  expect_snapshot(error = TRUE, {
    list_combine(list(1, 2), indices = list(1), size = 1)
  })
})

test_that("can combine with an AsIs list (#1463)", {
  x <- I(list(1, 2))
  expect_identical(list_combine(x, indices = list(1, 2), size = 2), c(1, 2))
})

test_that("can combine empty vectors", {
  expect_identical(
    list_combine(
      list(),
      indices = list(),
      size = 0
    ),
    unspecified()
  )
  expect_identical(
    list_combine(
      list(),
      indices = list(),
      size = 0,
      multiple = "first"
    ),
    unspecified()
  )
  expect_identical(
    list_combine(
      list(),
      indices = list(),
      size = 0,
      ptype = numeric()
    ),
    numeric()
  )
  expect_identical(
    list_combine(
      list(),
      indices = list(),
      size = 0,
      ptype = numeric(),
      multiple = "first"
    ),
    numeric()
  )
})

test_that("can combine a list of NULL", {
  expect_identical(
    list_combine(list(NULL), indices = list(integer()), size = 0),
    unspecified(0)
  )
  expect_identical(
    list_combine(list(NULL), indices = list(integer()), size = 1),
    unspecified(1)
  )
  expect_identical(
    list_combine(
      list(NULL),
      indices = list(integer()),
      size = 0,
      ptype = numeric()
    ),
    numeric()
  )
  expect_identical(
    list_combine(
      list(NULL),
      indices = list(integer()),
      ptype = numeric(),
      size = 1
    ),
    vec_init(numeric(), n = 1)
  )
  expect_identical(
    list_combine(
      list(NULL, NULL),
      indices = list(integer(), integer()),
      size = 0,
      ptype = numeric()
    ),
    numeric()
  )
})

test_that("NULLs are ignored when combined with other vectors", {
  expect_identical_list_combine(
    x = list("a", NULL, "b"),
    indices = list(2, integer(), 1),
    size = 2,
    expect = c("b", "a")
  )
})

test_that("can use a `NULL` element with a corresponding index", {
  # Because of `size = 2`, we must return something with `size = 2`.
  # Our sized identity element is `unspecified(2)`.
  expect_identical(
    list_combine(list(NULL), indices = list(1:2), size = 2L),
    unspecified(2)
  )

  expect_identical(
    list_combine(list(NULL), indices = list(1:2), size = 2, ptype = integer()),
    c(NA_integer_, NA_integer_)
  )

  x <- list("a", NULL, c("b", "c"))
  indices <- list(3L, c(1L, 4L), c(2L, 5L))
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 5,
    expect = c(NA, "b", "a", NA, "c")
  )
})

test_that("can combine atomic vectors", {
  expect_identical_list_combine(
    x = list(1, 2),
    indices = list(2, 1),
    size = 2,
    expect = c(2, 1)
  )
  expect_identical_list_combine(
    x = list("a", "b"),
    indices = list(2, 1),
    size = 2,
    expect = c("b", "a")
  )
})

test_that("can combine lists", {
  x <- list(list("a", "b"), list("c"))
  indices <- list(c(2, 3), 1)

  expect_identical(
    list_combine(x, indices = indices, size = 3),
    list("c", "a", "b")
  )
})

test_that("can combine data frames", {
  df1 <- data_frame(x = 1:2)
  df2 <- data_frame(x = 3:4)

  x <- list(df1, df2)
  indices <- list(c(3, 1), c(2, 4))

  expect <- vec_slice(vec_c(df1, df2), vec_order(vec_c(!!!indices)))

  expect_identical(list_combine(x, indices = indices, size = 4), expect)
})

test_that("can fallback when combining matrices", {
  mat1 <- matrix(1:4, nrow = 2, ncol = 2)
  mat2 <- matrix(5:10, nrow = 3, ncol = 2)

  x <- list(mat1, mat2)
  indices <- list(c(4, 1), c(2, 3, 5))

  expect <- vec_slice(vec_c(mat1, mat2), vec_order(vec_c(!!!indices)))

  expect_identical(list_combine(x, indices = indices, size = 5), expect)
})

test_that("can fallback when combining matrices and using `default`", {
  mat1 <- matrix(1:4, nrow = 2, ncol = 2)
  mat2 <- matrix(5:10, nrow = 3, ncol = 2)

  x <- list(mat1, mat2)
  indices <- list(c(5, 1), c(2, 3, 7))

  default <- matrix(11:24, nrow = 7, ncol = 2)
  size <- 7

  expect <- vec_c(
    vec_slice(mat1, 2),
    vec_slice(mat2, 1),
    vec_slice(mat2, 2),
    vec_slice(default, 4),
    vec_slice(mat1, 1),
    vec_slice(default, 6),
    vec_slice(mat2, 3)
  )

  expect_identical(
    list_combine(x, indices = indices, size = size, default = default),
    expect
  )
})

test_that("can fallback when combining arrays of >2D", {
  arr1 <- array(1:8, c(2, 2, 2))
  arr2 <- matrix(9:10, c(1, 2))

  x <- list(arr1, arr2)
  indices <- list(c(3, 1), 2)

  expect <- vec_slice(vec_c(arr1, arr2), vec_order(vec_c(!!!indices)))

  expect_identical(list_combine(x, indices = indices, size = 3), expect)
})

test_that("common type failure after common class fallback reports the original class (#1981)", {
  int <- foobar(int(1))
  dbl <- foobar(dbl(1))

  # Works
  expect_identical(
    list_combine(
      list(int, int),
      indices = list(1, 2),
      size = 2
    ),
    foobar(int(1, 1))
  )

  # Failure with 1 and 2
  expect_snapshot(error = TRUE, {
    list_combine(
      list(int, dbl),
      indices = list(1, 2),
      size = 2
    )
  })

  # Failure with 1 and 3
  expect_snapshot(error = TRUE, {
    list_combine(
      list(int, int, dbl),
      indices = list(1, 2, 3),
      size = 3
    )
  })

  # Failure with 1 and `default`
  expect_snapshot(error = TRUE, {
    list_combine(
      list(int, int),
      indices = list(1, 2),
      size = 2,
      default = dbl
    )
  })
})

test_that("can combine with all size 0 elements and get the right ptype", {
  x <- list(integer(), integer())
  indices <- list(integer(), integer())
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 0,
    expect = integer()
  )
})

test_that("can combine with some size 0 elements", {
  x <- list(integer(), 1:2, integer())
  indices <- list(integer(), 2:1, integer())
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 2,
    expect = 2:1
  )
})

test_that("NULL is a valid index", {
  expect_identical_list_combine(
    x = list(1, 2),
    indices = list(NULL, 1),
    size = 1,
    expect = 2
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1, 2),
    indices = list(NULL, 2),
    size = 1
  )
})

test_that("combining recycles elements of x to the size of the index", {
  x <- list(1, 2)

  expect_identical_list_combine(
    x = x,
    indices = list(c(3, 4, 5), c(2, 1)),
    size = 5,
    expect = c(2, 2, 1, 1, 1)
  )
  expect_identical_list_combine(
    x = x,
    indices = list(
      c(FALSE, FALSE, TRUE, TRUE, TRUE),
      c(TRUE, TRUE, FALSE, FALSE, FALSE)
    ),
    size = 5,
    expect = c(2, 2, 1, 1, 1)
  )
  expect_identical_list_combine(
    x = x,
    indices = list(c(3, 4, 5), c(2, 1)),
    size = 5,
    slice_x = TRUE,
    expect = c(2, 2, 1, 1, 1)
  )
  expect_identical_list_combine(
    x = x,
    indices = list(
      c(FALSE, FALSE, TRUE, TRUE, TRUE),
      c(TRUE, TRUE, FALSE, FALSE, FALSE)
    ),
    size = 5,
    slice_x = TRUE,
    expect = c(2, 2, 1, 1, 1)
  )

  x <- list(1:2)

  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(1:3),
    size = 4
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(1:3),
    size = 4,
    x_arg = "arg"
  )

  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(1:3),
    size = 4,
    slice_x = TRUE
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(1:3),
    size = 4,
    slice_x = TRUE,
    x_arg = "arg"
  )

  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(c(TRUE, TRUE, TRUE, TRUE)),
    size = 4
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(c(TRUE, TRUE, TRUE, TRUE)),
    size = 4,
    x_arg = "arg"
  )

  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = list(c(TRUE, TRUE, TRUE, TRUE)),
    size = 4,
    slice_x = TRUE
  )
})

test_that("combining takes the common type", {
  x <- list(1, "a")
  indices <- list(1, 2)

  expect_snapshot_list_combine(
    error = TRUE,
    x = x,
    indices = indices,
    size = 2
  )

  x <- list(1, 2L)
  expect_identical(list_combine(x, indices = indices, size = 2), c(1, 2))
})

test_that("common type failure uses positional errors", {
  expect_snapshot({
    x <- list(1, a = "x", 2)

    # Looking for `x[[1]]` and `x$a`
    (expect_error(list_combine(x, indices = list(2, 1, 3), size = 3)))

    # Directed cast should also produce directional errors (#1690)
    (expect_error(list_combine(
      x,
      indices = list(2, 1, 3),
      size = 3,
      ptype = double()
    )))

    # Lossy cast
    y <- list(1, a = 2.5)
    (expect_error(list_combine(
      y,
      indices = list(2, 1),
      size = 2,
      ptype = integer()
    )))
  })
})

test_that("can specify a ptype to override common type", {
  indices <- list(1, 2)

  x <- list(1, 2L)
  expect_identical(
    list_combine(x, indices = indices, size = 2, ptype = integer()),
    c(1L, 2L)
  )

  x <- list(1.5, 2)
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_combine(x, indices = indices, size = 2, ptype = integer())
  })
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_combine(
      x,
      indices = indices,
      size = 2,
      ptype = integer(),
      error_call = call("foo"),
      x_arg = "arg"
    )
  })
})

test_that("common type is correctly computed with unspecified values and a `default` (#2094)", {
  expect_identical(
    list_combine(
      x = list(NA),
      indices = list(1),
      size = 1,
      default = "a"
    ),
    NA_character_
  )
  expect_identical(
    list_combine(
      x = list(NA),
      indices = list(1),
      size = 2,
      default = "a"
    ),
    c(NA, "a")
  )
})

test_that("outer names are kept", {
  x <- list(x = 1, y = 2)
  expect_named_list_combine(
    x = x,
    indices = list(2, 1),
    size = 2,
    expect = c("y", "x")
  )
})

test_that("outer names are recycled in the right order", {
  x <- list(x = 1, y = 2)

  expect_snapshot(error = TRUE, {
    list_combine(x, indices = list(c(1, 2), 3), size = 3)
  })
  expect_named(
    list_combine(
      x,
      indices = list(c(1, 3), 2),
      size = 3,
      name_spec = "{outer}_{inner}"
    ),
    c("x_1", "y", "x_2")
  )
  expect_named(
    list_combine(
      x,
      indices = list(c(3, 1), 2),
      size = 3,
      name_spec = "{outer}_{inner}"
    ),
    c("x_2", "y", "x_1")
  )
})

test_that("outer names can be merged with inner names", {
  x <- list(x = c(a = 1), y = c(b = 2))
  expect_named(
    list_combine(
      x,
      indices = list(2, 1),
      size = 2,
      name_spec = "{outer}_{inner}"
    ),
    c("y_b", "x_a")
  )
})

test_that("preserves names when inputs are cast to a common type (#1689)", {
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(1),
      size = 1,
      ptype = integer()
    ),
    "a"
  )

  # With `default`
  expect_named(
    list_combine(
      list(c(a = 1)),
      indices = list(1),
      default = c(b = 0),
      ptype = integer(),
      size = 2
    ),
    c("a", "b")
  )

  # With name spec
  name_spec <- "{outer}_{inner}"
  expect_named(
    list_combine(
      list(foo = c(a = 1)),
      ptype = integer(),
      name_spec = name_spec,
      indices = list(1),
      size = 1
    ),
    "foo_a"
  )
  expect_named(
    list_combine(
      list(foo = c(a = 1)),
      indices = list(1),
      default = c(b = 0),
      ptype = integer(),
      name_spec = name_spec,
      size = 2
    ),
    c("foo_a", "b")
  )

  # When `x` elements are recycled, names are also recycled
  x <- list(c(a = 1), c(b = 2))
  indices <- list(1:2, 3:4)
  expect_named(
    list_combine(x, indices = indices, size = 4, ptype = integer()),
    c("a", "a", "b", "b")
  )

  # When `default` elements are recycled, names are also recycled
  expect_named(
    list_combine(
      list(c(a = 1), c(b = 2)),
      indices = list(1, 3),
      default = c(c = 0),
      ptype = integer(),
      size = 4
    ),
    c("a", "c", "b", "c")
  )
})

test_that("individual data frame columns retain vector names", {
  df1 <- data_frame(x = c(a = 1, b = 2))
  df2 <- data_frame(x = c(c = 3))

  x <- list(df1, df2)
  indices <- list(c(1, 2), 3)

  result <- list_combine(x, indices = indices, size = 3)

  expect_named(result$x, c("a", "b", "c"))

  # Names should be identical to equivalent `vec_c()` call
  expect_identical(
    list_combine(x, indices = indices, size = 3),
    vec_c(!!!x)
  )
})

test_that("assigning to the same location twice means last wins", {
  x <- list(1:2, 3L)
  indices <- list(1:2, 1L)

  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 3,
    expect = c(3L, 2L, NA)
  )
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = 2,
    expect = c(3L, 2L)
  )
})

test_that("when assigning to the same location, names are continually overwritten (#2019)", {
  expect_identical_list_combine(
    x = list(
      c(a = 1, b = 2),
      c(c = 3, d = 4)
    ),
    indices = list(
      c(1, 2),
      c(1, 3)
    ),
    size = 3,
    multiple = "last",
    expect = c(c = 3, b = 2, d = 4)
  )
  expect_identical_list_combine(
    x = list(
      c(a = 1, b = 2),
      c(c = 3, d = 4)
    ),
    indices = list(
      c(1, 2),
      c(1, 3)
    ),
    size = 3,
    multiple = "first",
    expect = c(a = 1, b = 2, d = 4)
  )
})

test_that("df-cols: when assigning to the same location, names are continually overwritten (#2019)", {
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = c(a = 1, b = 2), y = c("foo", "bar")),
        data_frame(x = c(c = 0), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "last"
    ),
    data_frame(
      x = c(c = 0, b = 2),
      y = c("baz", "bar")
    )
  )
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = c(a = 1, b = 2), y = c("foo", "bar")),
        data_frame(x = c(c = 0), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "first"
    ),
    data_frame(
      x = c(a = 1, b = 2),
      y = c("foo", "bar")
    )
  )

  # Homogenous fallback
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
        data_frame(x = foobar(c(c = 0)), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "last"
    ),
    data_frame(
      x = foobar(c(c = 0, b = 2)),
      y = c("baz", "bar")
    )
  )
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
        data_frame(x = foobar(c(c = 0)), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "first"
    ),
    data_frame(
      x = foobar(c(a = 1, b = 2)),
      y = c("foo", "bar")
    )
  )

  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_combine(
        x = list(
          data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
          data_frame(x = foobar(c(c = 0)), y = "baz")
        ),
        indices = list(
          c(1, 2),
          1
        ),
        size = 2,
        multiple = "last"
      ),
      data_frame(
        x = foobar_c(c(c = 0, b = 2)),
        y = c("baz", "bar")
      )
    )
  })
  with_c_foobar({
    expect_identical(
      list_combine(
        x = list(
          data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
          data_frame(x = foobar(c(c = 0)), y = "baz")
        ),
        indices = list(
          c(1, 2),
          1
        ),
        size = 2,
        multiple = "first"
      ),
      data_frame(
        x = foobar_c(c(a = 1, b = 2)),
        y = c("foo", "bar")
      )
    )
  })
})

test_that("when assigning to the same location, names are cleared as needed (#2019)", {
  expect_identical_list_combine(
    x = list(
      c(a = 1, b = 2),
      c(3, 4)
    ),
    indices = list(
      c(1, 2),
      c(1, 3)
    ),
    size = 3,
    multiple = "last",
    expect = c(3, b = 2, 4)
  )
  expect_identical_list_combine(
    x = list(
      c(1, 2),
      c(c = 3, d = 4)
    ),
    indices = list(
      c(1, 2),
      c(1, 3)
    ),
    size = 3,
    multiple = "first",
    expect = c(1, 2, d = 4)
  )
})

test_that("df-cols: when assigning to the same location, names are cleared as needed (#2019)", {
  # - 1st element has names
  # - 2nd element doesn't have names, so `""` is used as the name to overwrite
  #   the names written when inserting the 1st element
  # - Reversed for `multiple = "first"`
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = c(a = 1, b = 2), y = c("foo", "bar")),
        data_frame(x = 0, y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "last"
    ),
    data_frame(
      x = set_names(c(0, 2), c("", "b")),
      y = c("baz", "bar")
    )
  )
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = c(1, 2), y = c("foo", "bar")),
        data_frame(x = c(c = 0), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "first"
    ),
    data_frame(
      x = set_names(c(1, 2), c("", "")),
      y = c("foo", "bar")
    )
  )

  # Homogenous fallback
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
        data_frame(x = foobar(0), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "last"
    ),
    data_frame(
      x = foobar(set_names(c(0, 2), c("", "b"))),
      y = c("baz", "bar")
    )
  )
  expect_identical(
    list_combine(
      x = list(
        data_frame(x = foobar(c(1, 2)), y = c("foo", "bar")),
        data_frame(x = foobar(c(c = 0)), y = "baz")
      ),
      indices = list(
        c(1, 2),
        1
      ),
      size = 2,
      multiple = "first"
    ),
    data_frame(
      x = foobar(set_names(c(1, 2), c("", ""))),
      y = c("foo", "bar")
    )
  )

  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_combine(
        x = list(
          data_frame(x = foobar(c(a = 1, b = 2)), y = c("foo", "bar")),
          data_frame(x = foobar(0), y = "baz")
        ),
        indices = list(
          c(1, 2),
          1
        ),
        size = 2,
        multiple = "last"
      ),
      data_frame(
        x = foobar_c(set_names(c(0, 2), c("", "b"))),
        y = c("baz", "bar")
      )
    )
  })
  with_c_foobar({
    expect_identical(
      list_combine(
        x = list(
          data_frame(x = foobar(c(1, 2)), y = c("foo", "bar")),
          data_frame(x = foobar(c(c = 0)), y = "baz")
        ),
        indices = list(
          c(1, 2),
          1
        ),
        size = 2,
        multiple = "first"
      ),
      data_frame(
        x = foobar_c(set_names(c(1, 2), c("", ""))),
        y = c("foo", "bar")
      )
    )
  })
})

test_that("index values are validated", {
  x <- list(1, 2)
  indices1 <- list(4, 1)
  indices2 <- list(c(1, 4), 2)
  indices3 <- list(c(1, 3, 4), 2)

  expect_error(
    list_combine(x, indices = indices1, size = 2),
    class = "vctrs_error_subscript_oob"
  )
  expect_error(
    list_combine(x, indices = indices2, size = 3),
    class = "vctrs_error_subscript_oob"
  )

  expect_identical(
    list_combine(x, indices = indices3, size = 4),
    c(1, 2, 1, 1)
  )
})

test_that("name repair is respected and happens after ordering according to `indices`", {
  local_name_repair_quiet()

  x <- list(c(a = 1), c(a = 2))

  indices <- list(2, 1)
  expect_named(list_combine(x, indices = indices, size = 2), c("a", "a"))
  expect_named(
    list_combine(x, indices = indices, size = 2, name_repair = "unique"),
    c("a...1", "a...2")
  )

  # With `default`
  indices <- list(3, 1)
  default <- c(a = 0)
  expect_named(
    list_combine(x, indices = indices, size = 3, default = default),
    c("a", "a", "a")
  )
  expect_named(
    list_combine(
      x,
      indices = indices,
      size = 3,
      default = default,
      name_repair = "unique"
    ),
    c("a...1", "a...2", "a...3")
  )
})

test_that("list_combine() can repair names quietly", {
  local_name_repair_verbose()

  x <- c(x = "a", x = "b", x = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_combine(
      vec_chop(x, indices = indices),
      indices = indices,
      size = 3,
      name_repair = "unique_quiet"
    )
  })
  expect_named(res, c("x...1", "x...2", "x...3"))

  x <- c("if" = "a", "in" = "b", "for" = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_combine(
      vec_chop(x, indices = indices),
      indices = indices,
      size = 3,
      name_repair = "universal_quiet"
    )
  })
  expect_named(res, c(".if", ".in", ".for"))
})

test_that("list_combine() errors on unsupported location values", {
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1, 2),
    indices = list(c(1, 2), 0),
    size = 3
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1),
    indices = list(-1),
    size = 1
  )
})

test_that("missing values propagate", {
  expect_identical_list_combine(
    x = list(1, 2),
    indices = list(c(NA_integer_, NA_integer_), c(NA_integer_, 3)),
    size = 4,
    expect = c(NA, NA, 2, NA)
  )
})

test_that("list_combine() fallback doesn't support `name_spec` or `ptype`", {
  expect_snapshot({
    foo <- structure(foobar(1), foo = "foo")
    bar <- structure(foobar(2), bar = "bar")
    (expect_error(
      with_c_foobar(list_combine(
        list(foo, bar),
        indices = list(1, 2),
        size = 2,
        name_spec = "{outer}_{inner}"
      )),
      "name specification"
    ))
    # With error call
    (expect_error(
      with_c_foobar(list_combine(
        list(foo, bar),
        indices = list(1, 2),
        size = 2,
        name_spec = "{outer}_{inner}",
        error_call = call("foo")
      )),
      "name specification"
    ))
    # Used to be an error about `ptype`
    x <- list(foobar(1))
    (expect_error(
      with_c_foobar(list_combine(x, indices = list(1), size = 1, ptype = "")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_combine() supports numeric S3 indices", {
  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_foobar")
    },
    vec_ptype2.vctrs_foobar.integer = function(x, y, ...) foobar(integer()),
    vec_cast.integer.vctrs_foobar = function(x, to, ...) vec_data(x)
  )

  expect_identical(
    list_combine(list(1), indices = list(foobar(1L)), size = 1),
    1
  )
})

test_that("list_combine() does not support non-numeric S3 indices", {
  expect_snapshot({
    (expect_error(
      list_combine(list(1), indices = list(factor("x")), size = 1),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      list_combine(list(1), indices = list(foobar(1L)), size = 1),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("list_combine() supports named `indices` (#2095)", {
  # Particularly in the fallback case where we have to `vec_c()` the `indices`

  # With outer names on `indices`:
  expect_identical_list_combine(
    x = list(c("a", "b")),
    indices = list(a = c(1, 2)),
    size = 2,
    expect = c("a", "b")
  )
  expect_identical_list_combine(
    x = list(c("a", "b", "c", "d")),
    indices = list(a = c(FALSE, TRUE, FALSE, TRUE)),
    size = 4,
    slice_x = TRUE,
    expect = c(NA, "b", NA, "d")
  )

  # With outer and inner names on `indices`:
  expect_identical_list_combine(
    x = list(c("a", "b"), c("c", "d")),
    indices = list(a = c(x = 1, y = 2), b = c(x = 3, y = 4)),
    size = 4,
    expect = c("a", "b", "c", "d")
  )
  expect_identical_list_combine(
    x = list(c("a", "b", "c", "d"), c("e", "f", "g", "h")),
    indices = list(
      a = c(w = FALSE, x = TRUE, y = FALSE, z = TRUE),
      b = c(w = FALSE, x = FALSE, y = TRUE, z = FALSE)
    ),
    size = 4,
    slice_x = TRUE,
    expect = c(NA, "b", "g", "d")
  )
})

test_that("`list_combine()` with `slice_x = FALSE`", {
  values <- list(1:2, 3:4)
  size <- 4

  indices <- list(
    c(TRUE, FALSE, FALSE, TRUE),
    c(FALSE, TRUE, TRUE, FALSE)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    expect = int(1, 3, 4, 2)
  )

  indices <- list(
    c(1, 4),
    c(2, 3)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    expect = int(1, 3, 4, 2)
  )
})

test_that("`list_combine()` with `slice_x = FALSE` / recycling", {
  values <- list(1L, 2L)
  size <- 4

  indices <- list(
    c(TRUE, FALSE, FALSE, TRUE),
    c(FALSE, TRUE, TRUE, FALSE)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    expect = int(1, 2, 2, 1)
  )

  indices <- list(
    c(1, 4),
    c(2, 3)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    expect = int(1, 2, 2, 1)
  )
})

test_that("`list_combine()` with `slice_x = TRUE`", {
  values <- list(1:4, 5:8)
  size <- 4

  indices <- list(
    c(TRUE, FALSE, FALSE, TRUE),
    c(FALSE, TRUE, TRUE, FALSE)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    slice_x = TRUE,
    expect = int(1, 6, 7, 4)
  )

  indices <- list(
    c(1, 4),
    c(2, 3)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    slice_x = TRUE,
    expect = int(1, 6, 7, 4)
  )
})

test_that("`list_combine()` with `slice_x = TRUE` / recycling", {
  values <- list(1L, 2L)
  size <- 4

  indices <- list(
    c(TRUE, FALSE, FALSE, TRUE),
    c(FALSE, TRUE, TRUE, FALSE)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    slice_x = TRUE,
    expect = int(1, 2, 2, 1)
  )

  indices <- list(
    c(1, 4),
    c(2, 3)
  )
  expect_identical_list_combine(
    x = values,
    indices = indices,
    size = size,
    slice_x = TRUE,
    expect = int(1, 2, 2, 1)
  )
})

test_that("`list_combine()` with logical `indices` checks `indices` size", {
  values <- list(1L, 2L)
  indices <- list(
    c(TRUE, FALSE, FALSE, TRUE),
    c(FALSE, TRUE, TRUE, FALSE)
  )
  size <- 5

  # This isn't the most obvious error but it is hard to know how to do better.
  # Ideally it would report a size error for `indices`, right now it falls
  # through to `list_as_locations()` which doesn't allow logical indices.
  expect_snapshot_list_combine(
    error = TRUE,
    x = values,
    indices = indices,
    size = size
  )
})

test_that("`multiple` can let first index win", {
  x <- list(
    1:3,
    4:6
  )
  indices <- list(
    c(1, 2, 3),
    c(2, 3, 4)
  )
  size <- 4

  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = size,
    multiple = "first",
    expect = int(1, 2, 3, 6)
  )
  expect_identical_list_combine(
    x = x,
    indices = indices,
    size = size,
    multiple = "last",
    expect = int(1, 4, 5, 6)
  )
})

test_that("`multiple` works with data frames", {
  x <- list(
    data_frame(a = 1:3, b = foobar(4:6), c = foobar(c(7, 8, 9))),
    data_frame(a = 4:6, b = foobar(7:9), c = foobar(c(10, 11, 12)))
  )
  indices <- list(
    c(1, 2, 3),
    c(2, 3, 4)
  )
  size <- 4

  # Normal and homogeneous fallback mixed
  expect_identical(
    list_combine(
      x = x,
      indices = indices,
      size = size,
      multiple = "first"
    ),
    data_frame(
      a = int(1, 2, 3, 6),
      b = foobar(int(4, 5, 6, 9)),
      c = foobar(dbl(7, 8, 9, 12))
    )
  )

  # Normal and `c()` fallback mixed
  with_c_foobar({
    expect_identical(
      list_combine(
        x = x,
        indices = indices,
        size = size,
        multiple = "first"
      ),
      data_frame(
        a = int(1, 2, 3, 6),
        b = foobar_c(int(4, 5, 6, 9)),
        c = foobar_c(dbl(7, 8, 9, 12))
      )
    )
  })
})

test_that("`multiple` works with data frame columns", {
  x <- list(
    data_frame(
      a = 1:3,
      b = data_frame(x = c("a", "b", "c"), y = foobar(4:6)),
      c = foobar(c(7, 8, 9))
    ),
    data_frame(
      a = 4:6,
      b = data_frame(x = c("d", "e", "f"), y = foobar(7:9)),
      c = foobar(c(10, 11, 12))
    )
  )
  indices <- list(
    c(1, 2, 3),
    c(2, 3, 4)
  )
  size <- 4

  # Normal and homogeneous fallback mixed
  expect_identical(
    list_combine(
      x = x,
      indices = indices,
      size = size,
      multiple = "first"
    ),
    data_frame(
      a = int(1, 2, 3, 6),
      b = data_frame(
        x = c("a", "b", "c", "f"),
        y = foobar(int(4, 5, 6, 9))
      ),
      c = foobar(dbl(7, 8, 9, 12))
    )
  )

  # Normal and `c()` fallback mixed
  with_c_foobar({
    expect_identical(
      list_combine(
        x = x,
        indices = indices,
        size = size,
        multiple = "first"
      ),
      data_frame(
        a = int(1, 2, 3, 6),
        b = data_frame(
          x = c("a", "b", "c", "f"),
          y = foobar_c(int(4, 5, 6, 9))
        ),
        c = foobar_c(dbl(7, 8, 9, 12))
      )
    )
  })
})

test_that("`multiple` shows correctly indexed errors", {
  # In fallback, reversal happens after recycling and slicing
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 3L),
    indices = list(1:3, 4),
    size = 4,
    multiple = "first"
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 3L),
    indices = list(1:3, 4),
    size = 4,
    multiple = "last"
  )

  # If there is only 1 issue in `x` sizes, they report consistently
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:4, 3:5),
    indices = list(1:3, 4),
    size = 4,
    slice_x = TRUE,
    multiple = "first"
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:4, 3:5),
    indices = list(1:3, 4),
    size = 4,
    slice_x = TRUE,
    multiple = "last"
  )

  # If there are multiple `x` issues, because we reverse the iteration
  # order in the main path in the `multiple = "first"` case, we end up
  # reporting the last problem first, while the fallback case still
  # reports the fist problem first. The indices in the error are correct
  # in both cases, so this inconsistency is allowed.
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 3:5),
    indices = list(1:3, 4),
    size = 4,
    slice_x = TRUE,
    multiple = "first"
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 3:5),
    indices = list(1:3, 4),
    size = 4,
    slice_x = TRUE,
    multiple = "last"
  )
})

test_that("`multiple` also applies to names", {
  expect_identical_list_combine(
    x = list(c(a = 1, b = 2), c(c = 3)),
    indices = list(1:2, 2),
    size = 2,
    multiple = "first",
    expect = c(a = 1, b = 2)
  )
  expect_identical_list_combine(
    x = list(c(a = 1, b = 2), c(c = 3)),
    indices = list(1:2, 2),
    size = 2,
    multiple = "last",
    expect = c(a = 1, c = 3)
  )
})

test_that("`multiple` doesn't affect `default`", {
  expect_identical_list_combine(
    x = list(1:2, 4L),
    indices = list(c(1, 4), 4),
    size = 5,
    multiple = "first",
    default = 0L,
    expect = int(1, 0, 0, 2, 0)
  )
  expect_identical_list_combine(
    x = list(1:2, 4L),
    indices = list(c(1, 4), 4),
    size = 5,
    multiple = "last",
    default = 0L,
    expect = int(1, 0, 0, 4, 0)
  )
})

test_that("`multiple` doesn't apply WITHIN a single index", {
  # You always get the last value within a single index vector.
  # That possibly makes `multiple = "first"` a little confusing,
  # but `multiple` is mostly useful with logical vectors anyways
  # (case_when() style approach), so it doesn't matter much.
  expect_identical_list_combine(
    x = list(1:2, 3:4),
    indices = list(c(1, 1), c(1, 1)),
    size = 1,
    multiple = "first",
    expect = 2L
  )
  expect_identical_list_combine(
    x = list(1:2, 3:4),
    indices = list(c(1, 1), c(1, 1)),
    size = 1,
    multiple = "last",
    expect = 4L
  )
})

test_that("`compact_seq()` work as `indices`", {
  expect_identical_list_combine(
    x = list(1:3, 4:5),
    indices = list(compact_seq(2, 3), compact_seq(0, 2)),
    size = 5,
    expect = int(4:5, 1:3)
  )
  expect_identical_list_combine(
    x = list(1:3, 4L),
    indices = list(compact_seq(2, 3), 1),
    size = 5,
    expect = int(4, NA, 1:3)
  )
})

test_that("`compact_seq()` `indices` work with `unmatched`", {
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 4:5),
    indices = list(compact_seq(3, 2), compact_seq(0, 2)),
    size = 5,
    unmatched = "error"
  )
  expect_snapshot_list_combine(
    error = TRUE,
    x = list(1:2, 4L),
    indices = list(compact_seq(3, 2), 1),
    size = 5,
    unmatched = "error"
  )
})

test_that("`compact_seq()` `indices` work with `default`", {
  expect_identical_list_combine(
    x = list(1:2, 4:5),
    indices = list(compact_seq(3, 2), compact_seq(0, 2)),
    size = 5,
    default = 0L,
    expect = int(4:5, 0L, 1:2)
  )
  expect_identical_list_combine(
    x = list(1:2, 4L),
    indices = list(compact_seq(3, 2), 1),
    size = 5,
    default = 0L,
    expect = int(4L, 0L, 0L, 1:2)
  )
})

#' Arithmetic operations
#'
#' This generic provides a common double dispatch mechanism for all infix
#' operators (`+`, `-`, `/`, `*`, `^`, `%%`, `%/%`, `!`, `&`, `|`). It is used
#' to power the default arithmetic and boolean operators for [vctr]s objects,
#' overcoming the limitations of the base [Ops] generic.
#'
#' `vec_arith_base()` is provided as a convenience for writing methods. It
#' recycles `x` and `y` to common length then calls the base operator with the
#' underlying [vec_data()].
#'
#' `vec_arith()` is also used in `diff.vctrs_vctr()` method via `-`.
#'
#' @param op An arithmetic operator as a string
#' @param x,y A pair of vectors. For `!`, unary `+` and unary `-`, `y` will be
#'   a sentinel object of class `MISSING`, as created by `MISSING()`.
#' @inheritParams rlang::args_dots_empty
#'
#' @seealso [stop_incompatible_op()] for signalling that an arithmetic
#'   operation is not permitted/supported.
#' @seealso See [vec_math()] for the equivalent for the unary mathematical
#'   functions.
#' @export
#' @keywords internal
#' @examples
#' d <- as.Date("2018-01-01")
#' dt <- as.POSIXct("2018-01-02 12:00")
#' t <- as.difftime(12, unit = "hours")
#'
#' vec_arith("-", dt, 1)
#' vec_arith("-", dt, t)
#' vec_arith("-", dt, d)
#'
#' vec_arith("+", dt, 86400)
#' vec_arith("+", dt, t)
#' vec_arith("+", t, t)
#'
#' vec_arith("/", t, t)
#' vec_arith("/", t, 2)
#'
#' vec_arith("*", t, 2)
vec_arith <- function(op, x, y, ...) {
  check_dots_empty0(...)
  UseMethod("vec_arith", x)
}

#' @export
#' @rdname vec_arith
vec_arith.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}

# Atomic vectors ----------------------------------------------------------

#' @rdname vec_arith
#' @export vec_arith.logical
#' @method vec_arith logical
#' @export
vec_arith.logical <- function(op, x, y, ...) UseMethod("vec_arith.logical", y)
#' @method vec_arith.logical default
#' @export
vec_arith.logical.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}
#' @method vec_arith.logical logical
#' @export
vec_arith.logical.logical <- function(op, x, y, ...) vec_arith_base(op, x, y)
#' @method vec_arith.logical numeric
#' @export
vec_arith.logical.numeric <- function(op, x, y, ...) vec_arith_base(op, x, y)

#' @rdname vec_arith
#' @export vec_arith.numeric
#' @method vec_arith numeric
#' @export
vec_arith.numeric <- function(op, x, y, ...) UseMethod("vec_arith.numeric", y)
#' @method vec_arith.numeric default
#' @export
vec_arith.numeric.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}
#' @method vec_arith.numeric logical
#' @export
vec_arith.numeric.logical <- function(op, x, y, ...) vec_arith_base(op, x, y)
#' @method vec_arith.numeric numeric
#' @export
vec_arith.numeric.numeric <- function(op, x, y, ...) vec_arith_base(op, x, y)

# Helpers -----------------------------------------------------------------

#' @export
#' @rdname vec_arith
vec_arith_base <- function(op, x, y) {
  args <- vec_recycle_common(x, y)

  op_fn <- getExportedValue("base", op)
  op_fn(vec_data(args[[1L]]), vec_data(args[[2L]]))
}

#' @export
#' @rdname vec_arith
MISSING <- function() {
  structure(list(), class = "MISSING")
}

.rando <- setClass(
  "vctrs_rando",
  contains = "numeric",
  slots = list(.Data = "numeric")
)
rando <- function(n = 0) {
  .rando(as.numeric(seq_len(n)))
}

as_rando <- function(x) {
  rando(length(x))
}

setMethod("[", "vctrs_rando", function(x, i, j, ..., drop = TRUE) {
  new_n <- length(vec_as_location(i, length(x@.Data), names(x@.Data)))
  rando(new_n)
})

.Counts <- methods::setClass(
  "vctrs_Counts",
  contains = "integer",
  slots = c(name = "character")
)

local_c_counts <- function(frame = caller_env()) {
  c_counts <- function(x, ...) {
    xs <- list(x, ...)

    xs_data <- lapply(xs, function(x) x@.Data)
    new_data <- do.call(c, xs_data)

    .Counts(new_data, name = "Dispatched")
  }

  local_s4_method(
    frame = frame,
    "c",
    methods::signature(x = "vctrs_Counts"),
    c_counts
  )
}


local_s4_method <- function(generic, signature, method, frame = caller_env()) {
  methods::setMethod(generic, signature, method)
  exit_expr <- call2(
    methods::removeMethod,
    generic,
    signature,
    where = topenv(frame)
  )
  local_exit(exit_expr, frame = frame)
}
with_s4_method <- function(generic, signature, method, expr) {
  local_s4_method(generic, signature, method)
  expr
}

local_exit <- function(expr, frame = caller_env()) {
  # We are at top-level when only one frame refers to the global environment
  if (is_reference(frame, global_env())) {
    is_global_frame <- sys.parents() == 0
    if (sum(is_global_frame) == 1) {
      abort("Can't add an exit event at top-level")
    }
  }

  # Inline everything so the call will succeed in any environment
  expr <- call2(on.exit, expr, add = TRUE)
  eval_bare(expr, frame)

  invisible(expr)
}

#' Combine a list of vectors
#'
#' @description
#' While `list_unchop()` is not deprecated, we now recommend that you use
#' either:
#'
#' - `list_combine(x, indices = indices, size = size)` over
#'   `list_unchop(x, indices = indices)`
#'
#' - `vec_c(!!!x)` over `list_unchop(x)`
#'
#' `list_unchop()` combines a list of vectors into a single vector, placing
#' elements in the output according to the locations specified by `indices`. It
#' is similar to [vec_c()], but gives greater control over how the elements are
#' combined.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams vec_c
#'
#' @param x A list
#'
#' @param indices A list of positive integer vectors specifying the
#'   locations to place elements of `x` in. Each element of `x` is recycled to
#'   the size of the corresponding index vector. The size of `indices` must
#'   match the size of `x`. If `NULL`, `x` is combined in the order it is
#'   provided in, which is equivalent to using [vec_c()].
#'
#' @param ptype If `NULL`, the default, the output type is determined by
#'   computing the common type across all elements of `x`. Alternatively, you
#'   can supply `ptype` to give the output a known type.
#'
#' @returns
#' A vector of type `vec_ptype_common(!!!x)`, or `ptype`, if specified. The size
#' is computed as `vec_size_common(!!!indices)` unless the indices are `NULL`,
#' in which case the size is `vec_size_common(!!!x)`.
#'
#' @section Dependencies:
#' - [vec_c()]
#'
#' @keywords internal
#'
#' @export
#' @examples
#' # If `indices` selects every value in `x` exactly once,
#' # in any order, then `list_unchop()` inverts `vec_chop()`
#' x <- c("a", "b", "c", "d")
#' indices <- list(2, c(3, 1), 4)
#' vec_chop(x, indices = indices)
#' list_unchop(vec_chop(x, indices = indices), indices = indices)
#'
#' # When unchopping, size 1 elements of `x` are recycled
#' # to the size of the corresponding index
#' list_unchop(list(1, 2:3), indices = list(c(1, 3, 5), c(2, 4)))
#'
#' # Names are retained, and outer names can be combined with inner
#' # names through the use of a `name_spec`
#' lst <- list(x = c(a = 1, b = 2), y = 1)
#' list_unchop(lst, indices = list(c(3, 2), c(1, 4)), name_spec = "{outer}_{inner}")
#'
#' # If you have a list of homogeneous vectors, sometimes it can be useful to
#' # unchop, apply a function to the flattened vector, and then rechop according
#' # to the original indices. This can be done efficiently with `list_sizes()`.
#' x <- list(c(1, 2, 1), c(3, 1), 5, double())
#' x_flat <- list_unchop(x)
#' x_flat <- x_flat + max(x_flat)
#' vec_chop(x_flat, sizes = list_sizes(x))
list_unchop <- function(
  x,
  ...,
  indices = NULL,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c(
    "minimal",
    "unique",
    "check_unique",
    "universal",
    "unique_quiet",
    "universal_quiet"
  ),
  error_arg = "x",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_list_unchop,
    x,
    indices,
    ptype,
    name_spec,
    name_repair,
    environment()
  )
}

#' Custom conditions for vctrs package
#'
#' These functions are called for their side effect of raising
#' errors and warnings.
#' These conditions have custom classes and structures to make
#' testing easier.
#'
#' @inheritParams rlang::args_error_context
#' @param x,y,to Vectors
#' @param ...,class Only use these fields when creating a subclass.
#' @param x_arg,y_arg,to_arg Argument names for `x`, `y`, and `to`. Used in
#'   error messages to inform the user about the locations of incompatible
#'   types.
#' @param action An option to customize the incompatible type message depending
#'   on the context. Errors thrown from [vec_ptype2()] use `"combine"` and
#'   those thrown from [vec_cast()] use `"convert"`.
#' @param details Any additional human readable details.
#' @param message An overriding message for the error. `details` and
#'   `message` are mutually exclusive, supplying both is an error.
#'
#' @examples
#'
#' # Most of the time, `maybe_lossy_cast()` returns its input normally:
#' maybe_lossy_cast(
#'   c("foo", "bar"),
#'   NA,
#'   "",
#'   lossy = c(FALSE, FALSE),
#'   x_arg = "",
#'   to_arg = ""
#' )
#'
#' # If `lossy` has any `TRUE`, an error is thrown:
#' try(maybe_lossy_cast(
#'   c("foo", "bar"),
#'   NA,
#'   "",
#'   lossy = c(FALSE, TRUE),
#'   x_arg = "",
#'   to_arg = ""
#' ))
#'
#' # Unless lossy casts are allowed:
#' allow_lossy_cast(
#'   maybe_lossy_cast(
#'     c("foo", "bar"),
#'     NA,
#'     "",
#'     lossy = c(FALSE, TRUE),
#'     x_arg = "",
#'     to_arg = ""
#'   )
#' )
#'
#' @keywords internal
#' @name vctrs-conditions
NULL

stop_vctrs <- function(
  message = NULL,
  class = NULL,
  ...,
  call = caller_env()
) {
  abort(
    message,
    class = c(class, "vctrs_error"),
    ...,
    call = call
  )
}
warn_vctrs <- function(
  message = NULL,
  class = NULL,
  ...,
  call = caller_env()
) {
  warn(
    message,
    class = c(class, "vctrs_warning"),
    ...,
    call = call
  )
}

stop_incompatible <- function(
  x,
  y,
  ...,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  stop_vctrs(
    message,
    class = c(class, "vctrs_error_incompatible"),
    x = x,
    y = y,
    details = details,
    ...,
    call = call
  )
}

#' @return
#' `stop_incompatible_*()` unconditionally raise an error of class
#' `"vctrs_error_incompatible_*"` and `"vctrs_error_incompatible"`.
#'
#' @rdname vctrs-conditions
#' @export
stop_incompatible_type <- function(
  x,
  y,
  ...,
  x_arg,
  y_arg,
  action = c("combine", "convert"),
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  obj_check_vector(x, arg = x_arg)
  obj_check_vector(y, arg = y_arg)

  action <- arg_match(action)

  message <- cnd_type_message(
    x,
    y,
    x_arg,
    y_arg,
    details,
    action,
    message,
    from_dispatch = match_from_dispatch(...)
  )

  subclass <- switch(
    action,
    combine = "vctrs_error_ptype2",
    convert = "vctrs_error_cast"
  )

  stop_incompatible(
    x,
    y,
    x_arg = x_arg,
    y_arg = y_arg,
    details = details,
    ...,
    message = message,
    class = c(class, subclass, "vctrs_error_incompatible_type"),
    call = call
  )
}

#' @rdname vctrs-conditions
#' @export
stop_incompatible_cast <- function(
  x,
  to,
  ...,
  x_arg,
  to_arg,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  stop_incompatible_type(
    x = x,
    y = to,
    to = to,
    ...,
    x_arg = x_arg,
    y_arg = to_arg,
    to_arg = to_arg,
    action = "convert",
    details = details,
    message = message,
    class = class,
    call = call
  )
}

stop_incompatible_shape <- function(
  x,
  y,
  x_size,
  y_size,
  axis,
  x_arg,
  y_arg,
  call = caller_env()
) {
  details <- format_error_bullets(c(
    x = glue::glue(
      "Incompatible sizes {x_size} and {y_size} along axis {axis}."
    )
  ))
  stop_incompatible_type(
    x,
    y,
    x_arg = x_arg,
    y_arg = y_arg,
    details = details,
    call = call
  )
}

type_actions <- c(
  "combine",
  "convert"
)

cnd_type_separator <- function(action) {
  if (identical(action, "combine")) {
    "and"
  } else if (identical(action, "convert")) {
    "to"
  } else {
    abort("Internal error: Unknown `action`.")
  }
}

cnd_type_message <- function(
  x,
  y,
  x_arg,
  y_arg,
  details,
  action,
  message,
  from_dispatch = FALSE,
  fallback = NULL
) {
  if (!is_null(message)) {
    if (!is_null(details)) {
      abort("Can't supply both `message` and `details`.")
    }
    return(message)
  }

  x_arg <- arg_as_string(x_arg)
  y_arg <- arg_as_string(y_arg)

  if (nzchar(x_arg)) {
    x_name <- paste0(" `", x_arg, "` ")
  } else {
    x_name <- " "
  }

  if (nzchar(y_arg)) {
    y_name <- paste0(" `", y_arg, "` ")
  } else {
    y_name <- " "
  }

  separator <- cnd_type_separator(action)

  if (is.data.frame(x) && is.data.frame(y)) {
    if (vec_is_coercible(new_data_frame(x), new_data_frame(y))) {
      x_type <- cnd_type_message_df_label(x)
      y_type <- cnd_type_message_df_label(y)
    } else {
      x_type <- vec_ptype_full(x)
      y_type <- vec_ptype_full(y)
    }
  } else {
    x_type <- cnd_type_message_type_label(x)
    y_type <- cnd_type_message_type_label(y)
  }

  converting <- action == "convert"

  # If we are here directly from dispatch, this means there is no
  # ptype2 method implemented and the is-same-class fallback has
  # failed because of diverging attributes. The author of the class
  # should implement a ptype2 method as documented in the FAQ
  # indicated below.
  if (from_dispatch && !converting && identical(class(x)[[1]], class(y)[[1]])) {
    details <- c(incompatible_attrib_bullets(), details)
    details <- format_error_bullets(details)
  }

  if (is_null(fallback)) {
    end <- "."
  } else {
    end <- glue::glue("; falling back to {fallback}.")
  }

  if (converting && nzchar(y_arg)) {
    header <- glue::glue(
      "Can't convert{x_name}<{x_type}> to match type of{y_name}<{y_type}>{end}"
    )
  } else {
    header <- glue::glue(
      "Can't {action}{x_name}<{x_type}> {separator}{y_name}<{y_type}>{end}"
    )
  }

  paste_line(header, details)
}

cnd_type_message_type_label <- function(x) {
  if (is.data.frame(x)) {
    class(x)[[1]]
  } else {
    vec_ptype_full(x)
  }
}
incompatible_attrib_bullets <- function() {
  c(
    x = "Some attributes are incompatible.",
    i = "The author of the class should implement vctrs methods.",
    i = "See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>."
  )
}

cnd_type_message_df_label <- function(x) {
  x <- class(x)[[1]]

  if (identical(x, "tbl_df")) {
    "tibble"
  } else {
    x
  }
}


#' @rdname vctrs-conditions
#' @export
stop_incompatible_op <- function(
  op,
  x,
  y,
  details = NULL,
  ...,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  message <- message %||%
    glue_lines(
      "<{vec_ptype_full(x)}> {op} <{vec_ptype_full(y)}> is not permitted",
      details
    )

  stop_incompatible(
    x,
    y,
    op = op,
    details = details,
    ...,
    message = message,
    class = c(class, "vctrs_error_incompatible_op"),
    call = call
  )
}

#' @rdname vctrs-conditions
#' @export
stop_incompatible_size <- function(
  x,
  y,
  x_size,
  y_size,
  ...,
  x_arg,
  y_arg,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  stop_incompatible(
    x,
    y,
    x_size = x_size,
    y_size = y_size,
    ...,
    x_arg = x_arg,
    y_arg = y_arg,
    details = details,
    message = message,
    class = c(class, "vctrs_error_incompatible_size"),
    call = call
  )
}

#' @export
cnd_header.vctrs_error_incompatible_size <- function(cnd, ...) {
  if (is_string(cnd$message) && nzchar(cnd$message)) {
    return(cnd$message)
  }

  x_size <- vec_cast(cnd$x_size, int())
  y_size <- vec_cast(cnd$y_size, int())

  stopifnot(
    length(x_size) == 1,
    length(y_size) == 1
  )

  x_arg <- arg_as_string(cnd$x_arg)
  y_arg <- arg_as_string(cnd$y_arg)

  if (nzchar(x_arg)) {
    x_tag <- glue::glue("`{x_arg}` (size {x_size})")
  } else {
    x_tag <- glue::glue("input of size {x_size}")
  }
  if (nzchar(y_arg)) {
    y_tag <- glue::glue("to match `{y_arg}` (size {y_size})")
  } else {
    y_tag <- glue::glue("to size {y_size}")
  }

  glue::glue("Can't recycle {x_tag} {y_tag}.")
}

#' @export
cnd_body.vctrs_error_incompatible_size <- function(cnd, ...) {
  cnd$details
}

#' Lossy cast error
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' By default, lossy casts are an error. Use `allow_lossy_cast()` to
#' silence these errors and continue with the partial results. In this
#' case the lost values are typically set to `NA` or to a lower value
#' resolution, depending on the type of cast.
#'
#' Lossy cast errors are thrown by `maybe_lossy_cast()`. Unlike
#' functions prefixed with `stop_`, `maybe_lossy_cast()` usually
#' returns a result. If a lossy cast is detected, it throws an error,
#' unless it's been wrapped in `allow_lossy_cast()`. In that case, it
#' returns the result silently.
#'
#' @inheritParams stop_incompatible_cast
#' @inheritParams vec_cast
#' @inheritParams rlang::args_error_context
#' @param result The result of a potentially lossy cast.
#' @param to Type to cast to.
#' @param lossy A logical vector indicating which elements of `result`
#'   were lossy.
#'
#'   Can also be a single `TRUE`, but note that `locations` picks up
#'   locations from this vector by default. In this case, supply your
#'   own location vector, possibly empty.
#' @param loss_type The kind of lossy cast to be mentioned in error
#'   messages. Can be loss of precision (for instance from double to
#'   integer) or loss of generality (from character to factor).
#' @param locations An optional integer vector giving the
#'   locations where `x` lost information.
#' @param .deprecation If `TRUE`, the error is downgraded to a
#'   deprecation warning. This is useful for transitioning your class
#'   to a stricter conversion scheme. The warning advises your users
#'   to wrap their code with `allow_lossy_cast()`.
#' @keywords internal
#' @export
maybe_lossy_cast <- function(
  result,
  x,
  to,
  lossy = NULL,
  locations = NULL,
  ...,
  loss_type = c("precision", "generality"),
  x_arg,
  to_arg,
  call = caller_env(),
  details = NULL,
  message = NULL,
  class = NULL,
  .deprecation = FALSE
) {
  if (!any(lossy)) {
    return(result)
  }
  if (.deprecation) {
    maybe_warn_deprecated_lossy_cast(x, to, loss_type, x_arg, to_arg)
    return(result)
  }

  locations <- locations %||% which(lossy)

  withRestarts(
    vctrs_restart_error_cast_lossy = function() result,
    stop_lossy_cast(
      x = x,
      to = to,
      result = result,
      locations = locations,
      ...,
      loss_type = loss_type,
      x_arg = x_arg,
      to_arg = to_arg,
      details = details,
      message = message,
      class = class,
      call = call
    )
  )
}
stop_lossy_cast <- function(
  x,
  to,
  result,
  locations = NULL,
  ...,
  loss_type,
  x_arg,
  to_arg,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
) {
  stop_incompatible_cast(
    x = x,
    to = to,
    result = result,
    locations = locations,
    ...,
    loss_type = loss_type,
    x_arg = x_arg,
    to_arg = to_arg,
    details = details,
    class = c(class, "vctrs_error_cast_lossy"),
    call = call
  )
}

#' @export
cnd_header.vctrs_error_cast_lossy <- function(cnd, ...) {
  x_label <- format_arg_label(vec_ptype_full(cnd$x), cnd$x_arg)
  to_label <- format_arg_label(vec_ptype_full(cnd$y), cnd$y_arg)
  loss_type <- loss_type(cnd$loss_type)
  glue::glue(
    "Can't convert from {x_label} to {to_label} due to loss of {loss_type}."
  )
}
#' @export
cnd_body.vctrs_error_cast_lossy <- function(cnd, ...) {
  if (length(cnd$locations)) {
    format_error_bullets(inline_list("Locations: ", cnd$locations))
  } else {
    character()
  }
}

loss_type <- function(x) {
  stopifnot(
    is_character(x),
    all(x %in% c("precision", "generality"))
  )
  x[[1]]
}

# Used in maybe_warn_deprecated_lossy_cast()
new_error_cast_lossy <- function(x, to, loss_type, x_arg = "", to_arg = "") {
  error_cnd(
    c("vctrs_error_cast_lossy", "vctrs_error_incompatible_type"),
    x = x,
    y = to,
    loss_type = loss_type,
    x_arg = x_arg,
    y_arg = to_arg
  )
}

#' @rdname vctrs-conditions
#' @param x_ptype,to_ptype Suppress only the casting errors where `x`
#'   or `to` match these [prototypes][vec_ptype].
#' @export
allow_lossy_cast <- function(expr, x_ptype = NULL, to_ptype = NULL) {
  withCallingHandlers(
    vctrs_error_cast_lossy = function(err) {
      if (!is_null(x_ptype) && !vec_is(err$x, x_ptype)) {
        return()
      }
      if (!is_null(to_ptype) && !vec_is(err$y, to_ptype)) {
        return()
      }

      invokeRestart("vctrs_restart_error_cast_lossy")
    },
    expr
  )
}

maybe_warn_deprecated_lossy_cast <- function(
  x,
  to,
  loss_type,
  x_arg,
  to_arg,
  user_env = caller_env(2)
) {
  # Returns `TRUE` if `allow_lossy_cast()` is on the stack and accepts
  # to handle the condition
  handled <- withRestarts(
    vctrs_restart_error_cast_lossy = function() TRUE,
    {
      # Signal fully formed condition but strip the error classes in
      # case someone is catching: This is not an abortive condition.
      cnd <- new_error_cast_lossy(
        x,
        to,
        loss_type = loss_type,
        x_arg = x_arg,
        to_arg = to_arg
      )

      class(cnd) <- setdiff(class(cnd), c("error", "rlang_error"))
      signalCondition(cnd)
      FALSE
    }
  )

  if (handled) {
    return(invisible())
  }

  from <- format_arg_label(vec_ptype_abbr(x), x_arg)
  to <- format_arg_label(vec_ptype_abbr(to), to_arg)

  lifecycle::deprecate_warn(
    when = "0.2.0",
    what = I("Coercion with lossy casts"),
    with = "allow_lossy_cast()",
    details = paste0(
      glue::glue(
        "We detected a lossy transformation from { from } to { to }. "
      ),
      "The result will contain lower-resolution values or missing values. ",
      "To suppress this warning, wrap your code with `allow_lossy_cast()`."
    ),
    always = TRUE,
    user_env = user_env
  )

  invisible()
}

stop_unsupported <- function(x, method, call = caller_env()) {
  msg <- glue::glue("`{method}.{class(x)[[1]]}()` not supported.")
  stop_vctrs(
    "vctrs_error_unsupported",
    message = msg,
    x = x,
    method = method,
    call = call
  )
}

stop_unimplemented <- function(x, method, call = caller_env()) {
  msg <- glue::glue("`{method}.{class(x)[[1]]}()` not implemented.")
  stop_vctrs(
    "vctrs_error_unimplemented",
    message = msg,
    x = x,
    method = method,
    call = call
  )
}

stop_scalar_type <- function(x, arg = NULL, call = caller_env()) {
  if (is_null(arg) || !nzchar(arg)) {
    arg <- "Input"
  } else {
    arg <- glue::backtick(arg)
  }

  message <- glue::glue("{arg} must be a vector, not {obj_type_friendly(x)}.")

  # Use the first detected issue, with a fallthrough to point to our scalar FAQ
  message <-
    with_incompatible_s3_list_bullets(message, x) %||%
    with_incompatible_data_frame_bullets(message, x) %||%
    with_scalar_faq_bullet(message)

  stop_vctrs(
    message,
    "vctrs_error_scalar_type",
    actual = x,
    call = call
  )
}

with_incompatible_s3_list_bullets <- function(message, x) {
  is_list_typeof <- typeof(x) == "list"

  classes <- class(x)
  doesnt_contain_explicit_list_class <- !any(classes == "list")
  doesnt_contain_data_frame_class <- !any(classes == "data.frame")

  # We also assume no `vec_proxy()` method exists, otherwise one would have
  # been invoked, avoiding the error
  is_incompatible_s3_list <-
    is_list_typeof &&
    doesnt_contain_explicit_list_class &&
    doesnt_contain_data_frame_class

  if (!is_incompatible_s3_list) {
    return(NULL)
  }

  c(
    message,
    x = cli::format_inline(paste(
      "Detected incompatible scalar S3 list.",
      "To be treated as a vector, the object must explicitly inherit from {.cls list}",
      "or should implement a {.fn vec_proxy} method.",
      "Class: {.cls {classes}}."
    )),
    i = "If this object comes from a package, please report this error to the package author.",
    i = cli::format_inline(paste(
      "Read our FAQ about",
      "{.topic [creating vector types](vctrs::howto_faq_fix_scalar_type_error)}",
      "to learn more."
    ))
  )
}

with_incompatible_data_frame_bullets <- function(message, x) {
  classes <- class(x)
  n_classes <- length(classes)

  contains_data_frame_class <- any(classes == "data.frame")

  if (n_classes == 0L) {
    # Edge case of `NULL` or `character()` classes
    last_class_is_not_data_frame <- TRUE
  } else {
    last_class_is_not_data_frame <- classes[n_classes] != "data.frame"
  }

  is_incompatible_data_frame <-
    contains_data_frame_class && last_class_is_not_data_frame

  if (!is_incompatible_data_frame) {
    return(NULL)
  }

  subclasses <- setdiff(classes, "data.frame")

  c(
    message,
    x = cli::format_inline(paste(
      "Detected incompatible data frame structure.",
      "A data frame is normally treated as a vector, but an incompatible class ordering was detected.",
      "To be compatible, the subclass {.cls {subclasses}} must come before {.cls data.frame}, not after.",
      "Class: {.cls {classes}}."
    )),
    i = "If this object comes from a package, please report this error to the package author.",
    i = cli::format_inline(paste(
      "Read our FAQ about",
      "{.topic [creating vector types](vctrs::howto_faq_fix_scalar_type_error)}",
      "to learn more."
    ))
  )
}

with_scalar_faq_bullet <- function(message) {
  c(
    message,
    i = cli::format_inline(paste(
      "Read our FAQ about {.topic [scalar types](vctrs::faq_error_scalar_type)}",
      "to learn more."
    ))
  )
}

stop_corrupt_factor_levels <- function(x, arg = "x", call = caller_env()) {
  msg <- glue::glue("`{arg}` is a corrupt factor with non-character levels")
  abort(msg, call = call)
}

stop_corrupt_ordered_levels <- function(x, arg = "x", call = caller_env()) {
  msg <- glue::glue(
    "`{arg}` is a corrupt ordered factor with non-character levels"
  )
  abort(msg, call = call)
}

stop_recycle_incompatible_size <- function(
  x_size,
  size,
  x_arg = "x",
  call = caller_env()
) {
  stop_vctrs(
    x_size = x_size,
    y_size = size,
    x_arg = x_arg,
    # FIXME: tibble is the only package that uses `vctrs_error_recycle_incompatible_size`
    class = c(
      "vctrs_error_incompatible_size",
      "vctrs_error_recycle_incompatible_size"
    ),
    call = call
  )
}


# Names -------------------------------------------------------------------

stop_names <- function(class = NULL, ..., call = caller_env()) {
  stop_vctrs(
    class = c(class, "vctrs_error_names"),
    ...,
    call = call
  )
}

stop_names_cannot_be_empty <- function(names, call = caller_env()) {
  stop_names(
    class = "vctrs_error_names_cannot_be_empty",
    names = names,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_names_cannot_be_empty <- function(cnd, ...) {
  "Names can't be empty."
}

#' @export
cnd_body.vctrs_error_names_cannot_be_empty <- function(cnd, ...) {
  locations <- detect_empty_names(cnd$names)

  if (length(locations) == 1) {
    bullet <- glue::glue("Empty name found at location {locations}.")
  } else {
    bullet <- glue::glue(
      "Empty names found at locations {ensure_full_stop(enumerate(locations))}"
    )
  }

  bullet <- c(x = bullet)
  format_error_bullets(bullet)
}

stop_names_cannot_be_dot_dot <- function(names, call = caller_env()) {
  stop_names(
    class = "vctrs_error_names_cannot_be_dot_dot",
    names = names,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_names_cannot_be_dot_dot <- function(cnd, ...) {
  "Names can't be of the form `...` or `..j`."
}

#' @export
cnd_body.vctrs_error_names_cannot_be_dot_dot <- function(cnd, ...) {
  names <- cnd$names

  locations <- detect_dot_dot(names)
  names <- names[locations]

  split <- vec_group_loc(names)

  info <- map2_chr(split$key, split$loc, make_names_loc_bullet)

  header <- "These names are invalid:"
  header <- c(x = header)
  header <- format_error_bullets(header)

  message <- bullets(info, header = header)
  message <- indent(message, 2)

  message
}

stop_names_must_be_unique <- function(names, arg = "", call = caller_env()) {
  stop_names(
    class = "vctrs_error_names_must_be_unique",
    arg = arg,
    names = names,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_names_must_be_unique <- function(cnd, ...) {
  "Names must be unique."
}

#' @export
cnd_body.vctrs_error_names_must_be_unique <- function(cnd, ...) {
  names <- cnd$names

  dups <- vec_group_loc(names)
  dup_indicator <- map_lgl(dups$loc, function(x) length(x) != 1L)
  dups <- vec_slice(dups, dup_indicator)

  header <- "These names are duplicated:"
  header <- c(x = header)
  header <- format_error_bullets(header)

  info <- map2_chr(dups$key, dups$loc, make_names_loc_bullet)

  message <- bullets(info, header = header)
  message <- indent(message, 2)

  arg <- arg_as_string(cnd$arg)
  if (arg != "") {
    hint <- c(
      i = glue::glue("Use argument `{cnd$arg}` to specify repair strategy.")
    )
    message <- c(message, format_error_bullets(hint))
  }

  message
}


make_names_loc_bullet <- function(x, loc) {
  if (length(loc) == 1) {
    glue::glue("{glue::double_quote(x)} at location {loc}.")
  } else {
    glue::glue(
      "{glue::double_quote(x)} at locations {ensure_full_stop(enumerate(loc))}"
    )
  }
}

enumerate <- function(x, max = 5L, allow_empty = FALSE) {
  n <- length(x)

  if (n == 0L && !allow_empty) {
    abort("Internal error: Enumeration can't be empty.")
  }
  if (n > max) {
    paste0(glue::glue_collapse(x[seq2(1, max)], ", "), ", etc.")
  } else {
    if (n == 2) {
      last <- " and "
    } else {
      last <- ", and "
    }
    glue::glue_collapse(x, ", ", last = last)
  }
}

ensure_full_stop <- function(x) {
  n <- nchar(x)
  if (substr(x, n, n) == ".") {
    x
  } else {
    paste0(x, ".")
  }
}


stop_native_implementation <- function(fn) {
  cli::cli_abort(
    c(
      "{.fn {fn}} is implemented at C level.",
      " " = "This R function is purely indicative and should never be called."
    ),
    .internal = TRUE
  )
}


# Helpers -----------------------------------------------------------------

glue_lines <- function(..., env = parent.frame()) {
  out <- map_chr(chr(...), glue::glue, .envir = env)
  paste(out, collapse = "\n")
}

format_arg_label <- function(type, arg = "") {
  type <- paste0("<", type, ">")
  if (nzchar(arg)) {
    paste0("`", arg, "` ", type)
  } else {
    type
  }
}

arg_backtick <- function(arg, or = "Input") {
  if (nzchar(arg)) {
    glue::backtick(arg)
  } else {
    or
  }
}

arg_as_string <- function(arg) {
  if (is_null(arg)) {
    ""
  } else if (is_string(arg)) {
    arg
  } else {
    as_label(arg)
  }
}
append_arg <- function(x, arg) {
  if (is_null(arg)) {
    return(x)
  }

  arg <- arg_as_string(arg)
  if (nzchar(arg)) {
    glue::glue("{x} `{arg}`")
  } else {
    x
  }
}

test_that("conditions inherit from `vctrs_error`", {
  expect_error(stop_incompatible(NULL, NULL), class = "vctrs_error")
  expect_error(stop_incompatible_type(NULL, NULL, x_arg = "x", y_arg = "y"), class = "vctrs_error")
  expect_error(stop_incompatible_cast(NULL, NULL, x_arg = "x", to_arg = "to"), class = "vctrs_error")
  expect_error(stop_incompatible_op("", NULL, NULL), class = "vctrs_error")
  expect_error(stop_incompatible_size(NULL, NULL, 0, 0, x_arg = "x", y_arg = "y"), class = "vctrs_error")
  expect_error(maybe_lossy_cast(NULL, NULL, NULL, TRUE, x_arg = "x", to_arg = "to"), class = "vctrs_error")
  expect_error(stop_unsupported("", ""), class = "vctrs_error")
  expect_error(stop_unimplemented("", ""), class = "vctrs_error")
  expect_error(stop_scalar_type(NULL), class = "vctrs_error")
  expect_error(stop_names(), class = "vctrs_error")
  expect_error(stop_names_cannot_be_empty(""), class = "vctrs_error")
  expect_error(stop_names_cannot_be_dot_dot("..1"), class = "vctrs_error")
  expect_error(stop_names_must_be_unique("x"), class = "vctrs_error")
})

test_that("incompatible cast throws an incompatible type error", {
  err <- expect_error(
    stop_incompatible_cast(1, 2, x_arg = "x", to_arg = "to"),
    class = "vctrs_error_incompatible_type"
  )

  expect_equal(err$x, 1)
  expect_equal(err$y, 2)
  expect_equal(err$x_arg, "x")
  expect_equal(err$y_arg, "to")

  # Convenience aliases
  expect_equal(err$to, err$y)
  expect_equal(err$to_arg, err$y_arg)
})

test_that("incompatible type error validates `action`", {
  expect_snapshot({
    (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = "c")))
    (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = 1)))
  })
})

test_that("can override arg in OOB conditions", {
  expect_snapshot({
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        NULL
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        quote(foo)
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        quote(foo(bar))
      ),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("scalar type errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_slice(foobar(list(1)), 1),
      class = "vctrs_error_scalar_type"
    ))
    (expect_error(
      stop_scalar_type(foobar(list(1)), arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
  })
})

test_that("empty names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("x", "", "y"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
    (expect_error(
      vec_as_names(c("x", "", "y", ""), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
    (expect_error(
      vec_as_names(rep("", 10), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
  })
})

test_that("dot dot names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("..1", "..1", "..1", "...", "z"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
    (expect_error(
      vec_as_names(c(rep("..1", 20), rep(c("..2", "..3", "..4", "...", "..5"), 2)), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
  })
})

test_that("unique names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("x", "x", "x", "y", "y", "z"), repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
    (expect_error(
      vec_as_names(c(rep("x", 20), rep(c("a", "b", "c", "d", "e"), 2)), repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
  })
})

test_that("can't supply both `message` and `details`", {
  expect_error(
    stop_incompatible_type(1, 2, message = "my message", x_arg = "x", y_arg = "y"),
    "my message",
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    stop_incompatible_type(1, 2, message = "my message", details = "my details", x_arg = "x", y_arg = "y"),
    "Can't supply both `message` and `details`."
  )
})

test_that("lossy cast errors are internal", {
  # Should not trigger testthat warnings about untested class
  expect_error(vec_cast(mtcars, mtcars[1:3]), "convert")
  expect_error(vec_cast(1.5, int()), "convert")
})

test_that("lossy cast from character to factor mentions loss of generality", {
  expect_snapshot({
    (expect_error(vec_cast("a", factor("b")), class = "vctrs_error_cast_lossy"))
  })
})

test_that("lossy cast `conditionMessage()` result matches `cnd_message()` (#1592)", {
  cnd <- catch_cnd(vec_cast(1.5, to = integer()))

  expect_identical(conditionMessage(cnd), cnd_message(cnd))

  expect_snapshot({
    cat(conditionMessage(cnd))
  })
})

test_that("ordered cast failures mention conversion", {
  expect_snapshot({
    (expect_error(
      vec_cast(ordered("x"), ordered("y")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("incompatible size errors", {
  expect_snapshot({
    (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "")))
    (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo), y_arg = "")))
    (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "bar")))
    (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo), y_arg = quote(bar))))
  })
})

test_that("conditions inherit from `vctrs_error`", {
  expect_error(stop_incompatible(NULL, NULL), class = "vctrs_error")
  expect_error(
    stop_incompatible_type(NULL, NULL, x_arg = "x", y_arg = "y"),
    class = "vctrs_error"
  )
  expect_error(
    stop_incompatible_cast(NULL, NULL, x_arg = "x", to_arg = "to"),
    class = "vctrs_error"
  )
  expect_error(stop_incompatible_op("", NULL, NULL), class = "vctrs_error")
  expect_error(
    stop_incompatible_size(NULL, NULL, 0, 0, x_arg = "x", y_arg = "y"),
    class = "vctrs_error"
  )
  expect_error(
    maybe_lossy_cast(NULL, NULL, NULL, TRUE, x_arg = "x", to_arg = "to"),
    class = "vctrs_error"
  )
  expect_error(stop_unsupported("", ""), class = "vctrs_error")
  expect_error(stop_unimplemented("", ""), class = "vctrs_error")
  expect_error(stop_scalar_type(NULL), class = "vctrs_error")
  expect_error(stop_names(), class = "vctrs_error")
  expect_error(stop_names_cannot_be_empty(""), class = "vctrs_error")
  expect_error(stop_names_cannot_be_dot_dot("..1"), class = "vctrs_error")
  expect_error(stop_names_must_be_unique("x"), class = "vctrs_error")
})

test_that("incompatible cast throws an incompatible type error", {
  err <- expect_error(
    stop_incompatible_cast(1, 2, x_arg = "x", to_arg = "to"),
    class = "vctrs_error_incompatible_type"
  )

  expect_equal(err$x, 1)
  expect_equal(err$y, 2)
  expect_equal(err$x_arg, "x")
  expect_equal(err$y_arg, "to")

  # Convenience aliases
  expect_equal(err$to, err$y)
  expect_equal(err$to_arg, err$y_arg)
})

test_that("incompatible type error validates `action`", {
  expect_snapshot({
    (expect_error(stop_incompatible_type(
      1,
      1,
      x_arg = "",
      y_arg = "",
      action = "c"
    )))
    (expect_error(stop_incompatible_type(
      1,
      1,
      x_arg = "",
      y_arg = "",
      action = 1
    )))
  })
})

test_that("can override arg in OOB conditions", {
  expect_snapshot({
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        NULL
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        quote(foo)
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_subscript_data(
        vec_slice(set_names(letters), "foo"),
        quote(foo(bar))
      ),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("scalar type errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_slice(foobar(list(1)), 1),
      class = "vctrs_error_scalar_type"
    ))
    (expect_error(
      stop_scalar_type(foobar(list(1)), arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
  })
})

test_that("empty names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("x", "", "y"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
    (expect_error(
      vec_as_names(c("x", "", "y", ""), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
    (expect_error(
      vec_as_names(rep("", 10), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"
    ))
  })
})

test_that("dot dot names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("..1", "..1", "..1", "...", "z"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
    (expect_error(
      vec_as_names(
        c(rep("..1", 20), rep(c("..2", "..3", "..4", "...", "..5"), 2)),
        repair = "check_unique"
      ),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
  })
})

test_that("unique names errors are informative", {
  expect_snapshot({
    (expect_error(
      vec_as_names(c("x", "x", "x", "y", "y", "z"), repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"
    ))
    (expect_error(
      vec_as_names(
        c(rep("x", 20), rep(c("a", "b", "c", "d", "e"), 2)),
        repair = "check_unique"
      ),
      class = "vctrs_error_names_must_be_unique"
    ))
  })
})

test_that("can't supply both `message` and `details`", {
  expect_error(
    stop_incompatible_type(
      1,
      2,
      message = "my message",
      x_arg = "x",
      y_arg = "y"
    ),
    "my message",
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    stop_incompatible_type(
      1,
      2,
      message = "my message",
      details = "my details",
      x_arg = "x",
      y_arg = "y"
    ),
    "Can't supply both `message` and `details`."
  )
})

test_that("lossy cast errors are internal", {
  # Should not trigger testthat warnings about untested class
  expect_error(vec_cast(mtcars, mtcars[1:3]), "convert")
  expect_error(vec_cast(1.5, int()), "convert")
})

test_that("lossy cast from character to factor mentions loss of generality", {
  expect_snapshot({
    (expect_error(vec_cast("a", factor("b")), class = "vctrs_error_cast_lossy"))
  })
})

test_that("lossy cast `conditionMessage()` result matches `cnd_message()` (#1592)", {
  cnd <- catch_cnd(vec_cast(1.5, to = integer()))

  expect_identical(conditionMessage(cnd), cnd_message(cnd))

  expect_snapshot({
    cat(conditionMessage(cnd))
  })
})

test_that("ordered cast failures mention conversion", {
  expect_snapshot({
    (expect_error(
      vec_cast(ordered("x"), ordered("y")),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("incompatible size errors", {
  expect_snapshot({
    (expect_error(stop_incompatible_size(
      1:2,
      3:5,
      2L,
      3L,
      x_arg = "",
      y_arg = ""
    )))
    (expect_error(stop_incompatible_size(
      1:2,
      3:5,
      2L,
      3L,
      x_arg = quote(foo),
      y_arg = ""
    )))
    (expect_error(stop_incompatible_size(
      1:2,
      3:5,
      2L,
      3L,
      x_arg = "",
      y_arg = "bar"
    )))
    (expect_error(stop_incompatible_size(
      1:2,
      3:5,
      2L,
      3L,
      x_arg = quote(foo),
      y_arg = quote(bar)
    )))
  })
})

#' Identify groups
#'
#' @description
#' * `vec_group_id()` returns an identifier for the group that each element of
#'   `x` falls in, constructed in the order that they appear. The number of
#'   groups is also returned as an attribute, `n`.
#'
#' * `vec_group_loc()` returns a data frame containing a `key` column with the
#'   unique groups, and a `loc` column with the locations of each group in `x`.
#'
#' * `vec_group_rle()` locates groups in `x` and returns them run length
#'   encoded in the order that they appear. The return value is a rcrd object
#'   with fields for the `group` identifiers and the run `length` of the
#'   corresponding group. The number of groups is also returned as an
#'   attribute, `n`.
#'
#' @param x A vector
#' @return
#'   * `vec_group_id()`: An integer vector with the same size as `x`.
#'   * `vec_group_loc()`: A two column data frame with size equal to
#'     `vec_size(vec_unique(x))`.
#'     * A `key` column of type `vec_ptype(x)`
#'     * A `loc` column of type list, with elements of type integer.
#'   * `vec_group_rle()`: A `vctrs_group_rle` rcrd object with two integer
#'     vector fields: `group` and `length`.
#'
#'   Note that when using `vec_group_loc()` for complex types, the default
#'   `data.frame` print method will be suboptimal, and you will want to coerce
#'   into a tibble to better understand the output.
#' @name vec_group
#'
#' @section Dependencies:
#' - [vec_proxy_equal()]
#'
#' @keywords internal
#' @examples
#' purrr <- c("p", "u", "r", "r", "r")
#' vec_group_id(purrr)
#' vec_group_rle(purrr)
#'
#' groups <- mtcars[c("vs", "am")]
#' vec_group_id(groups)
#'
#' group_rle <- vec_group_rle(groups)
#' group_rle
#'
#' # Access fields with `field()`
#' field(group_rle, "group")
#' field(group_rle, "length")
#'
#' # `vec_group_id()` is equivalent to
#' vec_match(groups, vec_unique(groups))
#'
#' vec_group_loc(mtcars$vs)
#' vec_group_loc(mtcars[c("vs", "am")])
#'
#' if (require("tibble")) {
#'   as_tibble(vec_group_loc(mtcars[c("vs", "am")]))
#' }
NULL

#' @rdname vec_group
#' @export
vec_group_id <- function(x) {
  .Call(vctrs_group_id, x)
}

#' @rdname vec_group
#' @export
vec_group_loc <- function(x) {
  .Call(vctrs_group_loc, x)
}

#' @rdname vec_group
#' @export
vec_group_rle <- function(x) {
  .Call(vctrs_group_rle, x)
}

#' @export
format.vctrs_group_rle <- function(x, ...) {
  group <- field(x, "group")
  length <- field(x, "length")
  paste0(group, "x", length)
}

#' @export
obj_print_header.vctrs_group_rle <- function(x, ...) {
  size <- vec_size(x)
  n <- attr(x, "n")
  cat_line("<", vec_ptype_full(x), "[", size, "][n = ", n, "]>")
  invisible(x)
}

# For testing
new_group_rle <- function(group, length, n) {
  stopifnot(is_integer(group))
  stopifnot(is_integer(length))
  stopifnot(is_integer(n))
  vec_check_size(n, size = 1L)

  if (vec_size(group) != vec_size(length)) {
    abort("`group` and `length` must have the same size.")
  }

  new_rcrd(
    list(group = group, length = length),
    n = n,
    class = "vctrs_group_rle"
  )
}

# group id ----------------------------------------------------------------

test_that("vec_group_id detects groups in order of appearance", {
  x <- c(2, 4, 2, 1, 4)
  expect <- structure(c(1L, 2L, 1L, 3L, 2L), n = 3L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works for size 0 input", {
  expect <- structure(integer(), n = 0L)
  expect_equal(vec_group_id(NULL), expect)
  expect_equal(vec_group_id(numeric()), expect)
})

test_that("vec_group_id works on base S3 objects", {
  x <- factor(c("x", "y", "x"))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)

  x <- new_date(c(0, 1, 0))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works row wise on data frames", {
  df <- data.frame(x = c(1, 2, 1, 1), y = c(2, 3, 2, 3))
  expect <- structure(c(1L, 2L, 1L, 3L), n = 3L)
  expect_equal(vec_group_id(df), expect)
})

test_that("vec_group_id works row wise on arrays", {
  x <- array(c(1, 1, 1, 2, 4, 2), c(3, 2))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works with different encodings", {
  expect <- structure(c(1L, 1L, 1L), n = 1L)
  expect_equal(vec_group_id(encodings()), expect)
})

test_that("vec_group_id takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1, 1), c(1, 1, 1, 2))
  # Compares on only the first field
  expect <- structure(c(1L, 2L, 1L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- structure(c(1L, 2L, 1L, 1L), n = 2L)

  expect_equal(vec_group_id(df), expect)
})

# group rle ---------------------------------------------------------------

test_that("vec_group_rle returns a `vctrs_group_rle` object", {
  expect_s3_class(vec_group_rle(1), "vctrs_group_rle")
})

test_that("vec_group_rle works with size 0 input", {
  expect <- new_group_rle(integer(), integer(), 0L)
  expect_equal(vec_group_rle(integer()), expect)
  expect_equal(vec_group_rle(NULL), expect)
})

test_that("vec_group_rle detects groups in order of appearance", {
  x <- c(2, 2, 3, 1, 1)
  expect <- new_group_rle(1:3, c(2L, 1L, 2L), 3L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle can refer to groups it has already seen", {
  x <- c(2, 3, 2)
  expect <- new_group_rle(c(1L, 2L, 1L), rep(1L, 3), 2L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle works on base S3 objects", {
  expect <- new_group_rle(c(1L, 2L, 1L, 3L), c(1L, 2L, 1L, 1L), 3L)

  x <- factor(c("x", "y", "y", "x", "z"))
  expect_equal(vec_group_rle(x), expect)

  x <- new_date(c(0, 1, 1, 0, 2))
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1, 1), c(1, 1, 1, 2))
  # Compares on only the first field
  expect <- new_group_rle(c(1L, 2L, 1L), c(1L, 1L, 2L), 2L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle works row wise on data frames", {
  df <- data.frame(x = c(1, 1, 2, 1), y = c(2, 2, 3, 2))
  expect <- new_group_rle(c(1L, 2L, 1L), c(2L, 1L, 1L), 2L)
  expect_equal(vec_group_rle(df), expect)
})

test_that("vec_group_rle takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- new_group_rle(c(1L, 2L, 1L), c(1L, 1L, 2L), 2L)

  expect_equal(vec_group_rle(df), expect)
})

test_that("can access fields", {
  x <- vec_group_rle(c(1, 1, 2))
  expect_equal(fields(x), c("group", "length"))
  expect_identical(field(x, "group"), c(1L, 2L))
  expect_identical(field(x, "length"), c(2L, 1L))
})

test_that("can access number of groups", {
  x <- vec_group_rle(c(1, 1, 2))
  expect_identical(attr(x, "n"), 2L)
})

test_that("print method is useful", {
  x <- new_group_rle(c(1L, 2L, 1L), c(3L, 2L, 1L), 2L)
  expect_snapshot(x)
})

# group loc --------------------------------------------------------------

test_that("can locate unique groups of an empty vector", {
  out <- vec_group_loc(integer())

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, integer())
  expect_equal(out$loc, list())
})

test_that("can locate unique groups of a data frame", {
  df <- data_frame(x = c(1, 1, 1, 2, 2), y = c("a", "a", "b", "a", "b"))
  out <- vec_group_loc(df)

  expect_equal(nrow(out), 4L)
  expect_equal(out$key, vec_unique(df))
})

test_that("can locate unique groups of a data frame with a list column", {
  df <- data_frame(x = list(1:2, 1:2, "a", 5.5, "a"))
  out <- vec_group_loc(df)

  expect_equal(nrow(out), 3L)
  expect_equal(out$key, vec_unique(df))
})

test_that("`x` must be a vector", {
  expect_error(vec_group_loc(environment()), class = "vctrs_error_scalar_type")
})

test_that("`key` column retains full type information", {
  x <- factor(letters[c(1, 2, 1)], levels = letters[1:3])
  out <- vec_group_loc(x)

  expect_equal(levels(out$key), levels(x))
})

test_that("vec_group_loc takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)
  expect_equal(vec_group_loc(x)$key, x[1:2])
  expect_equal(vec_group_loc(x)$loc, list(c(1L, 3L), 2L))

  x <- as.POSIXlt(new_datetime(c(1, 2, 1)))
  expect_equal(vec_group_loc(x)$key, x[1:2])
  expect_equal(vec_group_loc(x)$loc, list(c(1L, 3L), 2L))
})

test_that("vec_group_loc takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- data_frame(key = vec_slice(df, c(1, 2)), loc = list(c(1L, 3L, 4L), 2L))

  expect_equal(vec_group_loc(df), expect)
})

test_that("vec_group_loc works with different encodings", {
  encs <- encodings()
  expect_identical(nrow(vec_group_loc(encs)), 1L)
})

# group id ----------------------------------------------------------------

test_that("vec_group_id detects groups in order of appearance", {
  x <- c(2, 4, 2, 1, 4)
  expect <- structure(c(1L, 2L, 1L, 3L, 2L), n = 3L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works for size 0 input", {
  expect <- structure(integer(), n = 0L)
  expect_equal(vec_group_id(NULL), expect)
  expect_equal(vec_group_id(numeric()), expect)
})

test_that("vec_group_id works on base S3 objects", {
  x <- factor(c("x", "y", "x"))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)

  x <- new_date(c(0, 1, 0))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works row wise on data frames", {
  df <- data.frame(x = c(1, 2, 1, 1), y = c(2, 3, 2, 3))
  expect <- structure(c(1L, 2L, 1L, 3L), n = 3L)
  expect_equal(vec_group_id(df), expect)
})

test_that("vec_group_id works row wise on arrays", {
  x <- array(c(1, 1, 1, 2, 4, 2), c(3, 2))
  expect <- structure(c(1L, 2L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id works with different encodings", {
  expect <- structure(c(1L, 1L, 1L), n = 1L)
  expect_equal(vec_group_id(encodings()), expect)
})

test_that("vec_group_id takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1, 1), c(1, 1, 1, 2))
  # Compares on only the first field
  expect <- structure(c(1L, 2L, 1L, 1L), n = 2L)
  expect_equal(vec_group_id(x), expect)
})

test_that("vec_group_id takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- structure(c(1L, 2L, 1L, 1L), n = 2L)

  expect_equal(vec_group_id(df), expect)
})

# group rle ---------------------------------------------------------------

test_that("vec_group_rle returns a `vctrs_group_rle` object", {
  expect_s3_class(vec_group_rle(1), "vctrs_group_rle")
})

test_that("vec_group_rle works with size 0 input", {
  expect <- new_group_rle(integer(), integer(), 0L)
  expect_equal(vec_group_rle(integer()), expect)
  expect_equal(vec_group_rle(NULL), expect)
})

test_that("vec_group_rle detects groups in order of appearance", {
  x <- c(2, 2, 3, 1, 1)
  expect <- new_group_rle(1:3, c(2L, 1L, 2L), 3L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle can refer to groups it has already seen", {
  x <- c(2, 3, 2)
  expect <- new_group_rle(c(1L, 2L, 1L), rep(1L, 3), 2L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle works on base S3 objects", {
  expect <- new_group_rle(c(1L, 2L, 1L, 3L), c(1L, 2L, 1L, 1L), 3L)

  x <- factor(c("x", "y", "y", "x", "z"))
  expect_equal(vec_group_rle(x), expect)

  x <- new_date(c(0, 1, 1, 0, 2))
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1, 1), c(1, 1, 1, 2))
  # Compares on only the first field
  expect <- new_group_rle(c(1L, 2L, 1L), c(1L, 1L, 2L), 2L)
  expect_equal(vec_group_rle(x), expect)
})

test_that("vec_group_rle works row wise on data frames", {
  df <- data.frame(x = c(1, 1, 2, 1), y = c(2, 2, 3, 2))
  expect <- new_group_rle(c(1L, 2L, 1L), c(2L, 1L, 1L), 2L)
  expect_equal(vec_group_rle(df), expect)
})

test_that("vec_group_rle takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- new_group_rle(c(1L, 2L, 1L), c(1L, 1L, 2L), 2L)

  expect_equal(vec_group_rle(df), expect)
})

test_that("can access fields", {
  x <- vec_group_rle(c(1, 1, 2))
  expect_equal(fields(x), c("group", "length"))
  expect_identical(field(x, "group"), c(1L, 2L))
  expect_identical(field(x, "length"), c(2L, 1L))
})

test_that("can access number of groups", {
  x <- vec_group_rle(c(1, 1, 2))
  expect_identical(attr(x, "n"), 2L)
})

test_that("print method is useful", {
  x <- new_group_rle(c(1L, 2L, 1L), c(3L, 2L, 1L), 2L)
  expect_snapshot(x)
})

# group loc --------------------------------------------------------------

test_that("can locate unique groups of an empty vector", {
  out <- vec_group_loc(integer())

  expect_s3_class(out, "data.frame")
  expect_equal(out$key, integer())
  expect_equal(out$loc, list())
})

test_that("can locate unique groups of a data frame", {
  df <- data_frame(x = c(1, 1, 1, 2, 2), y = c("a", "a", "b", "a", "b"))
  out <- vec_group_loc(df)

  expect_equal(nrow(out), 4L)
  expect_equal(out$key, vec_unique(df))
})

test_that("can locate unique groups of a data frame with a list column", {
  df <- data_frame(x = list(1:2, 1:2, "a", 5.5, "a"))
  out <- vec_group_loc(df)

  expect_equal(nrow(out), 3L)
  expect_equal(out$key, vec_unique(df))
})

test_that("`x` must be a vector", {
  expect_error(vec_group_loc(environment()), class = "vctrs_error_scalar_type")
})

test_that("`key` column retains full type information", {
  x <- factor(letters[c(1, 2, 1)], levels = letters[1:3])
  out <- vec_group_loc(x)

  expect_equal(levels(out$key), levels(x))
})

test_that("vec_group_loc takes the equality proxy", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)
  expect_equal(vec_group_loc(x)$key, x[1:2])
  expect_equal(vec_group_loc(x)$loc, list(c(1L, 3L), 2L))

  x <- as.POSIXlt(new_datetime(c(1, 2, 1)))
  expect_equal(vec_group_loc(x)$key, x[1:2])
  expect_equal(vec_group_loc(x)$loc, list(c(1L, 3L), 2L))
})

test_that("vec_group_loc takes the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2, 1, 1), 1:4)
  df <- data_frame(x = x)

  expect <- data_frame(
    key = vec_slice(df, c(1, 2)),
    loc = list(c(1L, 3L, 4L), 2L)
  )

  expect_equal(vec_group_loc(df), expect)
})

test_that("vec_group_loc works with different encodings", {
  encs <- encodings()
  expect_identical(nrow(vec_group_loc(encs)), 1L)
})

#' Vector recycling
#'
#' `vec_recycle(x, size)` recycles a single vector to a given size.
#' `vec_recycle_common(...)` recycles multiple vectors to their common size. All
#' functions obey the [vctrs recycling rules][theory-faq-recycling], and will
#' throw an error if recycling is not possible. See [vec_size()] for the precise
#' definition of size.
#'
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector to recycle.
#' @param ... Depending on the function used:
#'   * For `vec_recycle_common()`, vectors to recycle.
#'   * For `vec_recycle()`, these dots should be empty.
#' @param size Desired output size.
#' @param .size Desired output size. If omitted,
#'   will use the common size from [vec_size_common()].
#' @param x_arg Argument name for `x`. These are used in error
#'   messages to inform the user about which argument has an
#'   incompatible size.
#'
#' @section Dependencies:
#' - [vec_slice()]
#'
#' @export
#' @examples
#' # Inputs with 1 observation are recycled
#' vec_recycle_common(1:5, 5)
#' vec_recycle_common(integer(), 5)
#' \dontrun{
#' vec_recycle_common(1:5, 1:2)
#' }
#'
#' # Data frames and matrices are recycled along their rows
#' vec_recycle_common(data.frame(x = 1), 1:5)
#' vec_recycle_common(array(1:2, c(1, 2)), 1:5)
#' vec_recycle_common(array(1:3, c(1, 3, 1)), 1:5)
vec_recycle <- function(x, size, ..., x_arg = "", call = caller_env()) {
  check_dots_empty0(...)
  .Call(ffi_recycle, x, size, environment())
}

#' @export
#' @rdname vec_recycle
vec_recycle_common <- function(
  ...,
  .size = NULL,
  .arg = "",
  .call = caller_env()
) {
  .External2(ffi_recycle_common, list2(...), .size)
}

test_that("vec_recycle_common() reports error context", {
  my_function <- function(...) vec_recycle_common(...)
  expect_snapshot({
    (expect_error(my_function(this_arg = 1:2, that_arg = int())))
    (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2)))
    (expect_error(my_function(this_arg = 1:2, that_arg = int(), .arg = "my_arg")))
    (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2, .arg = "my_arg")))
  })
})


# vec_recycle() -------------------------------------------------------------

test_that("vec_recycle recycles size 1 to any other size", {
  x <- 1
  x0 <- numeric()
  x2 <- c(x, x)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)
})

test_that("vec_recycle(): incompatible lengths get error messages", {
  x2 <- c(1, 2)

  expect_snapshot({
    (expect_error(vec_recycle(x2, 1), class = "vctrs_error_recycle_incompatible_size"))
  })
  expect_error(vec_recycle(x2, 0), class = "vctrs_error_recycle_incompatible_size")
  expect_error(vec_recycle(x2, 3), class = "vctrs_error_recycle_incompatible_size")
})

test_that("can recycle arrays", {
  x <- matrix(1:2, 1)
  x2 <- matrix(1:2, 2, 2, byrow = TRUE)
  x0 <- matrix(integer(), 0, 2)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)

  # List arrays
  data <- c(list(1), list(2))
  x <- matrix(data, 1)
  x2 <- matrix(data, 2, 2, byrow = TRUE)
  x0 <- matrix(list(), 0, 2)

  expect_equal(vec_recycle(x, 1), x)
  expect_equal(vec_recycle(x, 0), x0)
  expect_equal(vec_recycle(x, 2), x2)
})

test_that("vec_recycle() evaluates x_arg lazily", {
  expect_silent(vec_recycle(1L, 1L, x_arg = print("oof")))
})

# Empty -------------------------------------------------------------------

test_that("empty input returns empty list", {
  expect_equal(vec_recycle_common(), list())
})

# Vectors -----------------------------------------------------------------

test_that("NULL is idempotent", {
  expect_equal(vec_recycle_common(NULL, NULL), list(NULL, NULL))
  expect_equal(vec_recycle_common(1:5, NULL), list(1:5, NULL))
  expect_equal(vec_recycle_common(NULL, 1:5), list(NULL, 1:5))
})

test_that("equal lengths returned as is", {
  x <- 1:3

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x[1], x[1]), list(x[1], x[1]))
  expect_equal(vec_recycle_common(x[0], x[0]), list(x[0], x[0]))
})

test_that("vec_recycle_common recycles size 1 to any other size", {
  x1 <- 1
  x3 <- rep(1, 3)
  x0 <- numeric()

  expect_equal(vec_recycle_common(x1, x3), list(x3, x3))
  expect_equal(vec_recycle_common(x3, x1), list(x3, x3))
  expect_equal(vec_recycle_common(x1, x0), list(x0, x0))
})

test_that("vec_recycle_common(): incompatible lengths get error messages", {
  expect_snapshot({
    (expect_error(vec_recycle_common(1:2, 1:3), class = "vctrs_error_incompatible_size"))
  })
  expect_error(vec_recycle_common(1:3, 1:2), class = "vctrs_error_incompatible_size")
  expect_error(vec_recycle_common(numeric(), 1:2), class = "vctrs_error_incompatible_size")
})

# Matrices ----------------------------------------------------------------

test_that("can vec_recycle_common matrices", {
  x <- matrix(nrow = 4, ncol = 4)
  x1 <- x[1, , drop = FALSE]

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x1, x), list(x, x))
})

test_that("recycling matrices respects incompatible sizes", {
  x <- matrix(nrow = 4, ncol = 4)
  x2 <- x[1:2, , drop = FALSE]
  x0 <- x[0, , drop = FALSE]

  expect_snapshot({
    (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size"))
  })
  expect_error(vec_recycle_common(x0, x), class = "vctrs_error_incompatible_size")
})

test_that("can vec_recycle_common data frames", {
  x <- data.frame(a = rep(1, 3), b = rep(2, 3))
  x1 <- vec_slice(x, 1L)

  expect_equal(vec_recycle_common(x, x), list(x, x))
  expect_equal(vec_recycle_common(x1, x), list(x, x))
})

test_that("recycling data frames respects incompatible sizes", {
  x <- data.frame(a = rep(1, 3), b = rep(2, 3))
  x2 <- vec_slice(x, 1:2)
  x0 <- vec_slice(x, integer())

  expect_snapshot({
    (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size"))
  })
  expect_error(vec_recycle_common(x0, x), class = "vctrs_error_incompatible_size")
})

test_that("can vec_recycle_common matrix and data frame", {
  mt <- matrix(nrow = 2, ncol = 2)
  df <- data.frame(x = c(1, 1), y = c(2, 2))

  expect_equal(
    vec_recycle_common(vec_slice(mt, 1L), df),
    list(mt, df)
  )

  expect_equal(
    vec_recycle_common(mt, vec_slice(df, 1L)),
    list(mt, df)
  )
})

test_that("recycling data frames with matrices respects incompatible sizes", {
  mt <- matrix(nrow = 2, ncol = 2)
  df <- data.frame(x = c(1, 1), y = c(2, 2))

  expect_error(
    vec_recycle_common(vec_slice(mt, integer()), df),
    class = "vctrs_error_incompatible_size"
  )

  expect_error(
    vec_recycle_common(mt, vec_slice(df, 0L)),
    class = "vctrs_error_incompatible_size"
  )
})

test_that("recycling to size 1 has informative error", {
  expect_snapshot({
    (expect_error(vec_recycle(1:2, 1), class = "vctrs_error_recycle_incompatible_size"))
  })
})

test_that("incompatible recycling size has informative error", {
  expect_snapshot(error = TRUE, vec_recycle(1:2, 4))
  expect_snapshot(error = TRUE, vec_recycle(1:2, 4, x_arg = "foo"))
})

str_dup <- function(x, times) {
  paste0(rep(x, times = times), collapse = "")
}

indent <- function(x, n) {
  pad <- str_dup(" ", n)
  map_chr(x, gsub, pattern = "(\n+)", replacement = paste0("\\1", pad))
}

ones <- function(...) {
  array(1, dim = c(...))
}
ones_list <- function(...) {
  array(list(1), dim = c(...))
}

vec_coerce_bare <- function(x, type) {
  # FIXME! Unexported wrapper around Rf_coerceVector()
  coerce <- env_get(ns_env("rlang"), "vec_coerce")
  coerce(x, type)
}


# Matches the semantics of c() - based on experimenting with the output
# of c(), not reading the source code.
outer_names <- function(names, outer, n) {
  .Call(ffi_outer_names, names, outer, vec_cast(n, int()))
}

has_inner_names <- function(x) {
  !all(map_lgl(map(x, vec_names), is.null))
}

cat_line <- function(...) {
  cat(paste0(..., "\n", collapse = ""))
}


set_partition <- function(x, y) {
  list(
    both = intersect(x, y),
    only_x = setdiff(x, y),
    only_y = setdiff(y, x)
  )
}

all_equal <- function(x) all(x == x[[1]])

inline_list <- function(title, x, width = getOption("width"), quote = "") {
  label_width <- width - nchar(title)
  x <- glue::glue_collapse(
    encodeString(x, quote = quote),
    sep = ", ",
    width = label_width
  )
  paste0(title, x)
}

has_unique_names <- function(x) {
  nms <- names(x)

  if (length(nms) != length(x)) {
    return(FALSE)
  }

  if (any(is.na(nms) | nms == "")) {
    return(FALSE)
  }

  !anyDuplicated(nms)
}

compact <- function(x) {
  is_null <- map_lgl(x, is.null)
  x[!is_null]
}

paste_line <- function(...) {
  paste(chr(...), collapse = "\n")
}

# Experimental
result <- function(ok = NULL, err = NULL) {
  structure(
    list(ok = ok, err = err),
    class = "rlang_result"
  )
}
result_get <- function(x) {
  if (!is_null(x$err)) {
    cnd_signal(x$err)
  }
  x$ok
}

obj_type <- function(x) {
  if (vec_is(x)) {
    vec_ptype_full(x)
  } else if (is.object(x)) {
    paste(class(x), collapse = "/")
  } else if (is_function(x)) {
    "function"
  } else {
    typeof(x)
  }
}

new_opts <- function(x, opts, subclass = NULL, arg = NULL) {
  if (!all(x %in% opts)) {
    if (is_null(arg)) {
      arg <- "Argument"
    } else {
      arg <- glue::glue("`{arg}`")
    }
    opts <- encodeString(opts, quote = "\"")
    opts <- glue::glue_collapse(opts, sep = ", ", last = " or ")
    abort(glue::glue("{arg} must be one of {opts}."))
  }

  structure(
    set_names(opts %in% x, opts),
    class = c(subclass, "vctrs_opts")
  )
}

glue_data_bullets <- function(.data, ..., .env = caller_env()) {
  glue_data <- function(...) glue::glue_data(.data, ..., .envir = .env)
  format_error_bullets(map_chr(chr(...), glue_data))
}

unstructure <- function(x) {
  attributes(x) <- NULL
  x
}

# We almost never want `stringsAsFactors = TRUE`, and `FALSE` became
# the default in R 4.0.0. This wrapper ensures that our tests are compliant
# with versions of R before and after this change. Keeping it in `utils.R`
# rather than as a testthat helper ensures that it is sourced before any other
# testthat helpers.
data.frame <- function(..., stringsAsFactors = NULL) {
  stringsAsFactors <- stringsAsFactors %||% FALSE
  base::data.frame(..., stringsAsFactors = stringsAsFactors)
}

try_catch_callback <- function(data, cnd) {
  .Call(vctrs_try_catch_callback, data, cnd)
}

try_catch_hnd <- function(data) {
  function(cnd) {
    try_catch_callback(data, cnd)
  }
}
try_catch_impl <- function(data, ...) {
  tryCatch(
    try_catch_callback(data, NULL),
    ...
  )
}

ns_methods <- function(name) {
  ns_env(name)$.__S3MethodsTable__.
}

s3_find_method <- function(x, generic, ns = "base") {
  stopifnot(
    is_string(generic),
    is_string(ns)
  )
  table <- ns_methods(ns_env(ns))
  .Call(vctrs_s3_find_method, generic, x, table)
}
s3_get_method <- function(class, generic, ns = "base") {
  stopifnot(
    is_string(class),
    is_string(generic),
    is_string(ns)
  )
  table <- ns_methods(ns_env(ns))
  .Call(ffi_s3_get_method, generic, class, table)
}

s3_method_specific <- function(x, generic, ns = "base", default = TRUE) {
  classes <- class(x)[[1]]

  if (default) {
    classes <- c(classes, "default")
  }

  for (class in classes) {
    method <- s3_get_method(class, generic, ns = ns)
    if (!is_null(method)) {
      return(method)
    }
  }

  cli::cli_abort("Can't find {.fn {generic}} method for {.cls {class}}.")
}

df_has_base_subset <- function(x) {
  method <- s3_find_method(x, "[", ns = "base")
  is_null(method) || identical(method, `[.data.frame`)
}

last <- function(x) {
  x[[length(x)]]
}

# Find the longest common suffix of two vectors
vec_common_suffix <- function(x, y) {
  common <- vec_cast_common(x = x, y = y)
  x <- common$x
  y <- common$y

  x_size <- vec_size(x)
  y_size <- vec_size(y)
  n <- min(x_size, y_size)

  if (!n) {
    return(vec_slice(x, int()))
  }

  # Truncate the start of the vectors so they have equal size
  if (x_size < y_size) {
    y <- vec_slice(y, seq2(y_size - x_size + 1, y_size))
  } else if (y_size < x_size) {
    x <- vec_slice(x, seq2(x_size - y_size + 1, x_size))
  }

  # Find locations of unequal elements. Elements after the last
  # location are the common suffix.
  common <- vec_equal(x, y)
  i <- which(!common)

  # Slice the suffix after the last unequal element
  if (length(i)) {
    vec_slice(x, seq2(max(i) + 1, n))
  } else {
    x
  }
}

import_from <- function(ns, names, env = caller_env()) {
  objs <- env_get_list(ns_env(ns), names)
  env_bind(env, !!!objs)
}

fast_c <- function(x, y) {
  .Call(vctrs_fast_c, x, y)
}

# Based on r-lib/bench (itself based on gaborcsardi/prettyunits)
#' @export
format.vctrs_bytes <- function(
  x,
  scientific = FALSE,
  digits = 3,
  drop0trailing = TRUE,
  ...
) {
  nms <- names(x)

  bytes <- unclass(x)

  unit <- map_chr(x, find_unit, byte_units)
  res <- round(bytes / byte_units[unit], digits = digits)

  ## Zero bytes
  res[bytes == 0] <- 0
  unit[bytes == 0] <- "B"

  ## NA and NaN bytes
  res[is.na(bytes)] <- NA_real_
  res[is.nan(bytes)] <- NaN
  unit[is.na(bytes)] <- "" # Includes NaN as well

  # Append an extra B to each unit
  large_units <- unit %in% names(byte_units)[-1]
  unit[large_units] <- paste0(unit[large_units], "B")

  res <- format(
    res,
    scientific = scientific,
    digits = digits,
    drop0trailing = drop0trailing,
    ...
  )

  stats::setNames(paste0(res, unit), nms)
}
#' @export
print.vctrs_bytes <- function(x, ...) {
  print(format(x, ...), quote = FALSE)
}

tolerance <- sqrt(.Machine$double.eps)
find_unit <- function(x, units) {
  if (is.na(x) || is.nan(x) || x <= 0 || is.infinite(x)) {
    return(NA_character_)
  }
  epsilon <- 1 - (x * (1 / units))
  names(utils::tail(n = 1, which(epsilon < tolerance)))
}

byte_units <- c(
  'B' = 1,
  'K' = 1024,
  'M' = 1024^2,
  'G' = 1024^3,
  'T' = 1024^4,
  'P' = 1024^5,
  'E' = 1024^6,
  'Z' = 1024^7,
  'Y' = 1024^8
)

new_vctrs_bytes <- function(x) {
  structure(x, class = c("vctrs_bytes", "numeric"))
}

named <- function(x) {
  if (is_null(names(x))) {
    names(x) <- names2(x)
  }
  x
}

browser <- function(..., skipCalls = 0, frame = parent.frame()) {
  if (!identical(stdout(), getConnection(1))) {
    sink(getConnection(1))
    withr::defer(sink(), envir = frame)
  }

  # Calling `browser()` on exit avoids RStudio displaying the
  # `browser2()` location. We still need one `n` to get to the
  # expected place. Ideally `skipCalls` would not skip but exit the
  # contexts.
  on.exit(base::browser(..., skipCalls = skipCalls + 1))
}

vec_paste0 <- function(...) {
  args <- vec_recycle_common(...)
  exec(paste0, !!!args)
}

# On R <3.6.0, we used to have a special C version of this to avoid two copies
# from being made by `attributes(x) <- attrib`. Now that we require R >=4.0.0,
# we don't have the C utility anymore.
vec_set_attributes <- function(x, attrib) {
  attributes(x) <- attrib
  x
}

test_that("names preserved if outer name is missing", {
  x <- c("a", "z", "")

  expect_equal(outer_names(x, NULL, 3), x)
  expect_equal(outer_names(x, "", 3), x)
  expect_equal(outer_names(x, na_chr, 3), x)
})

test_that("outer name vectorised if needed", {
  expect_equal(outer_names(NULL, "x", 1L), c("x"))
  expect_equal(outer_names(NULL, "x", 2L), c("x1", "x2"))
})

test_that("outer and inner names are combined", {
  expect_equal(outer_names("x", "y", 1), c("y..x"))
})

test_that("options are created", {
  expect_identical(
    unclass(new_opts(c("a", "c"), letters[1:4])),
    c(a = TRUE, b = FALSE, c = TRUE, d = FALSE)
  )
})

test_that("can't supply unknown option", {
  expect_error(
    new_opts(c("a", "foo"), letters[1:4]),
    "Argument must be one of \"a\", \"b\", \"c\" or \"d\""
  )
  expect_error(
    new_opts(c("a", "foo"), letters[1:4], arg = "foo"),
    "`foo` must be one of \"a\", \"b\", \"c\" or \"d\""
  )
})

test_that("`has_dim()` doesn't partial match on the `dim` attribute (#948)", {
  x <- structure(1, dimB = 1)
  expect_false(has_dim(x))
})

test_that("df_has_base_subset() detects `[` methods", {
  expect_true(df_has_base_subset(foobar(mtcars)))

  out <- with_methods(
    `[.vctrs_foobar` = function(x, i, ...) structure(NextMethod(), dispatched = TRUE),
    df_has_base_subset(foobar(mtcars))
  )
  expect_false(out)
})

test_that("vec_common_suffix() finds common suffix", {
  x <- c("foo", "bar", "baz")
  y <- c("quux", "foo", "hop", "baz")
  expect_identical(vec_common_suffix(x, y), "baz")

  x <- c("foo", "bar", "baz")
  y <- c("quux", "foo", "bar", "baz")
  expect_identical(vec_common_suffix(x, y), x)

  x <- letters
  y <- chr()
  expect_identical(vec_common_suffix(x, y), chr())

  x <- data.frame(x = 1:3, y = c("foo", "bar", "baz"))
  y <- data.frame(x = 0:3, y = c("foo", "hop", "bar", "baz"))
  exp <- data.frame(x = 2:3, y = c("bar", "baz"))
  expect_identical(vec_common_suffix(x, y), exp)
})

test_that("fast_c() concatenates", {
  expect_identical(fast_c(character(), "foo"), "foo")
  expect_identical(fast_c("foo", character()), "foo")
  expect_identical(fast_c("foo", c("bar", "baz")), c("foo", "bar", "baz"))
  expect_identical(fast_c(c("bar", "baz"), "foo"), c("bar", "baz", "foo"))
})

test_that("names preserved if outer name is missing", {
  x <- c("a", "z", "")

  expect_equal(outer_names(x, NULL, 3), x)
  expect_equal(outer_names(x, "", 3), x)
  expect_equal(outer_names(x, na_chr, 3), x)
})

test_that("outer name vectorised if needed", {
  expect_equal(outer_names(NULL, "x", 1L), c("x"))
  expect_equal(outer_names(NULL, "x", 2L), c("x1", "x2"))
})

test_that("outer and inner names are combined", {
  expect_equal(outer_names("x", "y", 1), c("y..x"))
})

test_that("options are created", {
  expect_identical(
    unclass(new_opts(c("a", "c"), letters[1:4])),
    c(a = TRUE, b = FALSE, c = TRUE, d = FALSE)
  )
})

test_that("can't supply unknown option", {
  expect_error(
    new_opts(c("a", "foo"), letters[1:4]),
    "Argument must be one of \"a\", \"b\", \"c\" or \"d\""
  )
  expect_error(
    new_opts(c("a", "foo"), letters[1:4], arg = "foo"),
    "`foo` must be one of \"a\", \"b\", \"c\" or \"d\""
  )
})

test_that("`has_dim()` doesn't partial match on the `dim` attribute (#948)", {
  x <- structure(1, dimB = 1)
  expect_false(has_dim(x))
})

test_that("df_has_base_subset() detects `[` methods", {
  expect_true(df_has_base_subset(foobar(mtcars)))

  out <- with_methods(
    `[.vctrs_foobar` = function(x, i, ...) {
      structure(NextMethod(), dispatched = TRUE)
    },
    df_has_base_subset(foobar(mtcars))
  )
  expect_false(out)
})

test_that("vec_common_suffix() finds common suffix", {
  x <- c("foo", "bar", "baz")
  y <- c("quux", "foo", "hop", "baz")
  expect_identical(vec_common_suffix(x, y), "baz")

  x <- c("foo", "bar", "baz")
  y <- c("quux", "foo", "bar", "baz")
  expect_identical(vec_common_suffix(x, y), x)

  x <- letters
  y <- chr()
  expect_identical(vec_common_suffix(x, y), chr())

  x <- data.frame(x = 1:3, y = c("foo", "bar", "baz"))
  y <- data.frame(x = 0:3, y = c("foo", "hop", "bar", "baz"))
  exp <- data.frame(x = 2:3, y = c("bar", "baz"))
  expect_identical(vec_common_suffix(x, y), exp)
})

test_that("fast_c() concatenates", {
  expect_identical(fast_c(character(), "foo"), "foo")
  expect_identical(fast_c("foo", character()), "foo")
  expect_identical(fast_c("foo", c("bar", "baz")), c("foo", "bar", "baz"))
  expect_identical(fast_c(c("bar", "baz"), "foo"), c("bar", "baz", "foo"))
})

#' Chopping
#'
#' @description
#' `vec_chop()` provides an efficient method to repeatedly slice a vector. It
#' captures the pattern of `map(indices, vec_slice, x = x)`. When no indices
#' are supplied, it is generally equivalent to [as.list()].
#'
#' @inheritParams rlang::args_dots_empty
#'
#' @param x A vector
#'
#' @param indices A list of positive integer vectors to slice `x` with, or
#'   `NULL`. Can't be used if `sizes` is already specified. If both `indices`
#'   and `sizes` are `NULL`, `x` is split into its individual elements,
#'   equivalent to using an `indices` of `as.list(vec_seq_along(x))`.
#'
#' @param sizes An integer vector of non-negative sizes representing sequential
#'   indices to slice `x` with, or `NULL`. Can't be used if `indices` is already
#'   specified.
#'
#'   For example, `sizes = c(2, 4)` is equivalent to `indices = list(1:2, 3:6)`,
#'   but is typically faster.
#'
#'   `sum(sizes)` must be equal to `vec_size(x)`, i.e. `sizes` must completely
#'   partition `x`, but an individual size is allowed to be `0`.
#'
#' @returns
#' A list where each element has the same type as `x`. The size of the list is
#' equal to `vec_size(indices)`, `vec_size(sizes)`, or `vec_size(x)` depending
#' on whether or not `indices` or `sizes` is provided.
#'
#' @section Dependencies:
#' - [vec_slice()]
#'
#' @export
#' @examples
#' vec_chop(1:5)
#'
#' # These two are equivalent
#' vec_chop(1:5, indices = list(1:2, 3:5))
#' vec_chop(1:5, sizes = c(2, 3))
#'
#' # Can also be used on data frames
#' vec_chop(mtcars, indices = list(1:3, 4:6))
#'
#' # If you know your input is sorted and you'd like to split on the groups,
#' # `vec_run_sizes()` can be efficiently combined with `sizes`
#' df <- data_frame(
#'   g = c(2, 5, 5, 6, 6, 6, 6, 8, 9, 9),
#'   x = 1:10
#' )
#' vec_chop(df, sizes = vec_run_sizes(df$g))
#'
#' # If you have a list of homogeneous vectors, sometimes it can be useful to
#' # combine, apply a function to the flattened vector, and chop according
#' # to the original indices. This can be done efficiently with `list_sizes()`.
#' x <- list(c(1, 2, 1), c(3, 1), 5, double())
#' x_flat <- vec_c(!!!x)
#' x_flat <- x_flat + max(x_flat)
#' vec_chop(x_flat, sizes = list_sizes(x))
vec_chop <- function(x, ..., indices = NULL, sizes = NULL) {
  if (!missing(...)) {
    indices <- check_dots_chop(..., indices = indices)
  }
  .Call(ffi_vec_chop, x, indices, sizes)
}

check_dots_chop <- function(..., indices = NULL, call = caller_env()) {
  if (!is_null(indices)) {
    # Definitely can't supply both `indices` and `...`
    check_dots_empty0(..., call = call)
  }

  if (dots_n(...) != 1L) {
    # Backwards compatible case doesn't allow for length >1 `...`.
    # This must be an error case.
    check_dots_empty0(..., call = call)
  }

  # TODO: Soft-deprecate this after dplyr/tidyr have updated all `vec_chop()`
  # calls to be explicit about `indices =`

  # Assume this is an old style `vec_chop(x, indices)` call, before we
  # added the `...`
  indices <- list(...)[[1L]]

  indices
}

# Exposed for testing  (`starts` is 0-based)
vec_chop_seq <- function(x, starts, sizes, increasings = TRUE) {
  args <- vec_recycle_common(starts, sizes, increasings)
  .Call(ffi_vec_chop_seq, x, args[[1]], args[[2]], args[[3]])
}

#' Assemble attributes for data frame construction
#'
#' `new_data_frame()` constructs a new data frame from an existing list. It is
#' meant to be performant, and does not check the inputs for correctness in any
#' way. It is only safe to use after a call to [df_list()], which collects and
#' validates the columns used to construct the data frame.
#'
#' @seealso
#' [df_list()] for a way to safely construct a data frame's underlying
#' data structure from individual columns. This can be used to create a
#' named list for further use by `new_data_frame()`.
#'
#' @param x A named list of equal-length vectors. The lengths are not
#'   checked; it is responsibility of the caller to make sure they are
#'   equal.
#' @param n Number of rows. If `NULL`, will be computed from the length of
#'   the first element of `x`.
#' @param ...,class Additional arguments for creating subclasses.
#'
#'   The following attributes have special behavior:
#'   - `"names"` is preferred if provided, overriding existing names in `x`.
#'   - `"row.names"` is preferred if provided, overriding both `n` and the size
#'     implied by `x`.
#'
#' @export
#' @examples
#' new_data_frame(list(x = 1:10, y = 10:1))
new_data_frame <- function(x = list(), n = NULL, ..., class = NULL) {
  .External(ffi_new_data_frame, x, n, class, ...)
}
new_data_frame <- fn_inline_formals(new_data_frame, "x")

#' Collect columns for data frame construction
#'
#' `df_list()` constructs the data structure underlying a data
#' frame, a named list of equal-length vectors. It is often used in
#' combination with [new_data_frame()] to safely and consistently create
#' a helper function for data frame subclasses.
#'
#' @section Properties:
#'
#' - Inputs are [recycled][theory-faq-recycling] to a common size with
#'   [vec_recycle_common()].
#'
#' - With the exception of data frames, inputs are not modified in any way.
#'   Character vectors are never converted to factors, and lists are stored
#'   as-is for easy creation of list-columns.
#'
#' - Unnamed data frame inputs are automatically unpacked. Named data frame
#'   inputs are stored unmodified as data frame columns.
#'
#' - `NULL` inputs are completely ignored.
#'
#' - The dots are dynamic, allowing for splicing of lists with `!!!` and
#'   unquoting.
#'
#' @seealso
#' [new_data_frame()] for constructing data frame subclasses from a validated
#' input. [data_frame()] for a fast data frame creation helper.
#'
#' @inheritParams rlang::args_error_context
#'
#' @param ... Vectors of equal-length. When inputs are named, those names
#'   are used for names of the resulting list.
#' @param .size The common size of vectors supplied in `...`. If `NULL`, this
#'   will be computed as the common size of the inputs.
#' @param .unpack Should unnamed data frame inputs be unpacked? Defaults to
#'   `TRUE`.
#' @param .name_repair One of `"check_unique"`, `"unique"`, `"universal"`,
#'   `"minimal"`, `"unique_quiet"`, or `"universal_quiet"`. See [vec_as_names()]
#'   for the meaning of these options.
#'
#' @export
#' @examples
#' # `new_data_frame()` can be used to create custom data frame constructors
#' new_fancy_df <- function(x = list(), n = NULL, ..., class = NULL) {
#'   new_data_frame(x, n = n, ..., class = c(class, "fancy_df"))
#' }
#'
#' # Combine this constructor with `df_list()` to create a safe,
#' # consistent helper function for your data frame subclass
#' fancy_df <- function(...) {
#'   data <- df_list(...)
#'   new_fancy_df(data)
#' }
#'
#' df <- fancy_df(x = 1)
#' class(df)
df_list <- function(
  ...,
  .size = NULL,
  .unpack = TRUE,
  .name_repair = c(
    "check_unique",
    "unique",
    "universal",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  .error_call = current_env()
) {
  .Call(ffi_df_list, list2(...), .size, .unpack, .name_repair, environment())
}
df_list <- fn_inline_formals(df_list, ".name_repair")

#' Construct a data frame
#'
#' @description
#' `data_frame()` constructs a data frame. It is similar to
#' [base::data.frame()], but there are a few notable differences that make it
#' more in line with vctrs principles. The Properties section outlines these.
#'
#' @details
#' If no column names are supplied, `""` will be used as a default name for all
#' columns. This is applied before name repair occurs, so the default name
#' repair of `"check_unique"` will error if any unnamed inputs are supplied and
#' `"unique"` (or `"unique_quiet"`) will repair the empty string column names
#' appropriately. If the column names don't matter, use a `"minimal"` name
#' repair for convenience and performance.
#'
#' @inheritSection df_list Properties
#'
#' @seealso
#' [df_list()] for safely creating a data frame's underlying data structure from
#' individual columns. [new_data_frame()] for constructing the actual data
#' frame from that underlying data structure. Together, these can be useful
#' for developers when creating new data frame subclasses supporting
#' standard evaluation.
#'
#' @inheritParams rlang::args_error_context
#'
#' @param ... Vectors to become columns in the data frame. When inputs are
#'   named, those names are used for column names.
#' @param .size The number of rows in the data frame. If `NULL`, this will
#'   be computed as the common size of the inputs.
#' @param .name_repair One of `"check_unique"`, `"unique"`, `"universal"`,
#'   `"minimal"`, `"unique_quiet"`, or `"universal_quiet"`. See [vec_as_names()]
#'   for the meaning of these options.
#'
#' @export
#' @examples
#' data_frame(x = 1, y = 2)
#'
#' # Inputs are recycled using tidyverse recycling rules
#' data_frame(x = 1, y = 1:3)
#'
#' # Strings are never converted to factors
#' class(data_frame(x = "foo")$x)
#'
#' # List columns can be easily created
#' df <- data_frame(x = list(1:2, 2, 3:4), y = 3:1)
#'
#' # However, the base print method is suboptimal for displaying them,
#' # so it is recommended to convert them to tibble
#' if (rlang::is_installed("tibble")) {
#'   tibble::as_tibble(df)
#' }
#'
#' # Named data frame inputs create data frame columns
#' df <- data_frame(x = data_frame(y = 1:2, z = "a"))
#'
#' # The `x` column itself is another data frame
#' df$x
#'
#' # Again, it is recommended to convert these to tibbles for a better
#' # print method
#' if (rlang::is_installed("tibble")) {
#'   tibble::as_tibble(df)
#' }
#'
#' # Unnamed data frame input is automatically unpacked
#' data_frame(x = 1, data_frame(y = 1:2, z = "a"))
data_frame <- function(
  ...,
  .size = NULL,
  .name_repair = c(
    "check_unique",
    "unique",
    "universal",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  .error_call = current_env()
) {
  .Call(ffi_data_frame, list2(...), .size, .name_repair, environment())
}
data_frame <- fn_inline_formals(data_frame, ".name_repair")

#' @export
vec_ptype_abbr.data.frame <- function(x, ...) {
  "df"
}

# For testing
# Keep in sync with `enum vctrs_proxy_kind` in `vctrs.h`
df_proxy <- function(x, kind) {
  .Call(ffi_df_proxy, x, kind)
}
VCTRS_PROXY_KIND_equal <- 0L
VCTRS_PROXY_KIND_compare <- 1L
VCTRS_PROXY_KIND_order <- 2L

df_is_coercible <- function(x, y, opts) {
  vec_is_coercible(
    new_data_frame(x),
    new_data_frame(y),
    opts = opts
  )
}


# Coercion ----------------------------------------------------------------

#' Coercion between two data frames
#'
#' `df_ptype2()` and `df_cast()` are the two functions you need to
#' call from `vec_ptype2()` and `vec_cast()` methods for data frame
#' subclasses. See [?howto-faq-coercion-data-frame][howto-faq-coercion-data-frame].
#' Their main job is to determine the common type of two data frames,
#' adding and coercing columns as needed, or throwing an incompatible
#' type error when the columns are not compatible.
#'
#' @param x,y,to Subclasses of data frame.
#' @param ... If you call `df_ptype2()` or `df_cast()` from a
#'   `vec_ptype2()` or `vec_cast()` method, you must forward the dots
#'   passed to your method on to `df_ptype2()` or `df_cast()`.
#' @inheritParams vec_ptype2
#' @inheritParams vec_cast
#'
#' @return
#' * When `x` and `y` are not compatible, an error of class
#'   `vctrs_error_incompatible_type` is thrown.
#' * When `x` and `y` are compatible, `df_ptype2()` returns the common
#'   type as a bare data frame. `tib_ptype2()` returns the common type
#'   as a bare tibble.
#'
#' @export
df_ptype2 <- function(x, y, ..., x_arg = "", y_arg = "", call = caller_env()) {
  .Call(
    ffi_df_ptype2_opts,
    x,
    y,
    opts = match_fallback_opts(...),
    environment()
  )
}
#' @rdname df_ptype2
#' @export
df_cast <- function(x, to, ..., x_arg = "", to_arg = "", call = caller_env()) {
  .Call(
    ffi_df_cast_opts,
    x,
    to,
    opts = match_fallback_opts(...),
    environment()
  )
}

df_ptype2_opts <- function(
  x,
  y,
  ...,
  opts,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  .Call(ffi_df_ptype2_opts, x, y, opts = opts, environment())
}

df_cast_opts <- function(
  x,
  to,
  ...,
  opts = fallback_opts(),
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  .Call(
    ffi_df_cast_opts,
    x,
    to,
    opts,
    environment()
  )
}
df_cast_params <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  s3_fallback = NULL
) {
  opts <- fallback_opts(s3_fallback = s3_fallback)
  df_cast_opts(x, to, opts = opts, x_arg = x_arg, to_arg = to_arg)
}

#' vctrs methods for data frames
#'
#' These functions help the base data.frame class fit into the vctrs type system
#' by providing coercion and casting functions.
#'
#' @keywords internal
#' @name vctrs-data-frame
NULL

#' @rdname vctrs-data-frame
#' @export vec_ptype2.data.frame
#' @method vec_ptype2 data.frame
#' @export
vec_ptype2.data.frame <- function(x, y, ...) {
  UseMethod("vec_ptype2.data.frame")
}
#' @method vec_ptype2.data.frame data.frame
#' @export
vec_ptype2.data.frame.data.frame <- function(x, y, ...) {
  df_ptype2(x, y, ...)
}

# Fallback for data frame subclasses (#981)
vec_ptype2_df_fallback <- function(
  x,
  y,
  opts,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  vec_ptype2_params(
    as_base_df(x),
    as_base_df(y),
    s3_fallback = opts$s3_fallback,
    x_arg = x_arg,
    y_arg = y_arg,
    call = call
  )
}
as_base_df <- function(x) {
  if (inherits(x, "tbl_df")) {
    new_data_frame(x, class = c("tbl_df", "tbl"))
  } else {
    new_data_frame(x)
  }
}


# Cast --------------------------------------------------------------------

#' @rdname vctrs-data-frame
#' @export vec_cast.data.frame
#' @method vec_cast data.frame
#' @export
vec_cast.data.frame <- function(x, to, ...) {
  UseMethod("vec_cast.data.frame")
}
#' @export
#' @method vec_cast.data.frame data.frame
vec_cast.data.frame.data.frame <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = ""
) {
  df_cast(x, to, ..., x_arg = x_arg, to_arg = to_arg)
}

#' @export
vec_restore.data.frame <- function(x, to, ...) {
  .Call(ffi_vec_bare_df_restore, x, to)
}

# Helpers -----------------------------------------------------------------

df_size <- function(x) {
  .Call(vctrs_df_size, x)
}

df_lossy_cast <- function(
  out,
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  extra <- setdiff(names(x), names(to))

  maybe_lossy_cast(
    result = out,
    x = x,
    to = to,
    lossy = length(extra) > 0,
    locations = int(),
    x_arg = x_arg,
    to_arg = to_arg,
    call = call,
    details = inline_list("Dropped variables: ", extra, quote = "`"),
    class = "vctrs_error_cast_lossy_dropped"
  )
}

df_attrib <- function(x) {
  attributes(x)[c("row.names", "names")]
}
non_df_attrib <- function(x) {
  attrib <- attributes(x)
  attrib <- attrib[!names(attrib) %in% c("row.names", "names")]

  # Sort to allow comparison
  attrib[order(names(attrib))]
}

test_that("works with empty input", {
  x <- integer()

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("works with data frames with rows but no columns", {
  x <- new_data_frame(n = 2L)

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("vectors with all missing values are left unchanged", {
  x <- c(NA, NA, NA)

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("`NA_real_` and `NaN` are both considered missing", {
  expect_identical(
    vec_fill_missing(c(1, NA_real_, NaN)),
    c(1, 1, 1)
  )
})

test_that("missings are filled correctly", {
  x <- c(NA, 1, NA, 2, NA, NA)

  expect_identical(vec_fill_missing(x, "down"), c(NA, 1, 1, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "up"), c(1, 1, 2, 2, NA, NA))
  expect_identical(vec_fill_missing(x, "downup"), c(1, 1, 1, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "updown"), c(1, 1, 2, 2, 2, 2))
})

test_that("`max_fill` limits the sequential fill amount", {
  x <- c(NA, NA, 1, NA, NA, NA, 3, NA, NA)

  expect_identical(vec_fill_missing(x, "down", max_fill = 1), c(NA, NA, 1, 1, NA, NA, 3, 3, NA))
  expect_identical(vec_fill_missing(x, "downup", max_fill = 1), c(NA, 1, 1, 1, NA, NA, 3, 3, NA))
  expect_identical(vec_fill_missing(x, "down", max_fill = 2), c(NA, NA, 1, 1, 1, NA, 3, 3, 3))
  expect_identical(vec_fill_missing(x, "downup", max_fill = 2), c(1, 1, 1, 1, 1, NA, 3, 3, 3))
  expect_identical(vec_fill_missing(x, "up", max_fill = 1), c(NA, 1, 1, NA, NA, 3, 3, NA, NA))
  expect_identical(vec_fill_missing(x, "updown", max_fill = 1), c(NA, 1, 1, NA, NA, 3, 3, 3, NA))
  expect_identical(vec_fill_missing(x, "up", max_fill = 2), c(1, 1, 1, NA, 3, 3, 3, NA, NA))
  expect_identical(vec_fill_missing(x, "updown", max_fill = 2), c(1, 1, 1, NA, 3, 3, 3, 3, 3))
})

test_that("fills data frames", {
  df <- data_frame(x = c(NA, NA, NA, 2), y = c(NA, 1, NA, 3))

  expect_identical(vec_fill_missing(df, "down"), vec_slice(df, c(1, 2, 2, 4)))
  expect_identical(vec_fill_missing(df, "up"), vec_slice(df, c(2, 2, 4, 4)))
})

test_that("can fill rcrd types", {
  x <- new_rcrd(list(x = c(1, NA, NA), y = c(1, 2, NA)))

  expect_identical(vec_fill_missing(x, "down"), vec_slice(x, c(1, 2, 2)))
  expect_identical(vec_fill_missing(x, "up"), vec_slice(x, c(1, 2, 3)))
  expect_identical(vec_fill_missing(x, "updown"), vec_slice(x, c(1, 2, 2)))
})

test_that("validates `direction`", {
  expect_error(vec_fill_missing(1, 1), "`direction` must be one of")
  expect_error(vec_fill_missing(1, "foo"), "`direction` must be one of")
})

test_that("validates `max_fill`", {
  expect_error(vec_fill_missing(1, max_fill = -1), "`max_fill` must be")
  expect_error(vec_fill_missing(1, max_fill = c(1L, 2L)), "`max_fill` must be")
  expect_error(vec_fill_missing(1, max_fill = NA_integer_), "`max_fill` must be")
  expect_error(vec_fill_missing(1, max_fill = "x"), class = "vctrs_error_incompatible_type")
})

test_that("works with empty input", {
  x <- integer()

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("works with data frames with rows but no columns", {
  x <- new_data_frame(n = 2L)

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("vectors with all missing values are left unchanged", {
  x <- c(NA, NA, NA)

  expect_identical(vec_fill_missing(x, direction = "down"), x)
  expect_identical(vec_fill_missing(x, direction = "up"), x)
  expect_identical(vec_fill_missing(x, direction = "downup"), x)
  expect_identical(vec_fill_missing(x, direction = "updown"), x)
  expect_identical(vec_fill_missing(x, direction = "downup", max_fill = 1), x)
  expect_identical(vec_fill_missing(x, direction = "updown", max_fill = 1), x)
})

test_that("`NA_real_` and `NaN` are both considered missing", {
  expect_identical(
    vec_fill_missing(c(1, NA_real_, NaN)),
    c(1, 1, 1)
  )
})

test_that("missings are filled correctly", {
  x <- c(NA, 1, NA, 2, NA, NA)

  expect_identical(vec_fill_missing(x, "down"), c(NA, 1, 1, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "up"), c(1, 1, 2, 2, NA, NA))
  expect_identical(vec_fill_missing(x, "downup"), c(1, 1, 1, 2, 2, 2))
  expect_identical(vec_fill_missing(x, "updown"), c(1, 1, 2, 2, 2, 2))
})

test_that("`max_fill` limits the sequential fill amount", {
  x <- c(NA, NA, 1, NA, NA, NA, 3, NA, NA)

  expect_identical(
    vec_fill_missing(x, "down", max_fill = 1),
    c(NA, NA, 1, 1, NA, NA, 3, 3, NA)
  )
  expect_identical(
    vec_fill_missing(x, "downup", max_fill = 1),
    c(NA, 1, 1, 1, NA, NA, 3, 3, NA)
  )
  expect_identical(
    vec_fill_missing(x, "down", max_fill = 2),
    c(NA, NA, 1, 1, 1, NA, 3, 3, 3)
  )
  expect_identical(
    vec_fill_missing(x, "downup", max_fill = 2),
    c(1, 1, 1, 1, 1, NA, 3, 3, 3)
  )
  expect_identical(
    vec_fill_missing(x, "up", max_fill = 1),
    c(NA, 1, 1, NA, NA, 3, 3, NA, NA)
  )
  expect_identical(
    vec_fill_missing(x, "updown", max_fill = 1),
    c(NA, 1, 1, NA, NA, 3, 3, 3, NA)
  )
  expect_identical(
    vec_fill_missing(x, "up", max_fill = 2),
    c(1, 1, 1, NA, 3, 3, 3, NA, NA)
  )
  expect_identical(
    vec_fill_missing(x, "updown", max_fill = 2),
    c(1, 1, 1, NA, 3, 3, 3, 3, 3)
  )
})

test_that("fills data frames", {
  df <- data_frame(x = c(NA, NA, NA, 2), y = c(NA, 1, NA, 3))

  expect_identical(vec_fill_missing(df, "down"), vec_slice(df, c(1, 2, 2, 4)))
  expect_identical(vec_fill_missing(df, "up"), vec_slice(df, c(2, 2, 4, 4)))
})

test_that("can fill rcrd types", {
  x <- new_rcrd(list(x = c(1, NA, NA), y = c(1, 2, NA)))

  expect_identical(vec_fill_missing(x, "down"), vec_slice(x, c(1, 2, 2)))
  expect_identical(vec_fill_missing(x, "up"), vec_slice(x, c(1, 2, 3)))
  expect_identical(vec_fill_missing(x, "updown"), vec_slice(x, c(1, 2, 2)))
})

test_that("validates `direction`", {
  expect_error(vec_fill_missing(1, 1), "`direction` must be one of")
  expect_error(vec_fill_missing(1, "foo"), "`direction` must be one of")
})

test_that("validates `max_fill`", {
  expect_error(vec_fill_missing(1, max_fill = -1), "`max_fill` must be")
  expect_error(vec_fill_missing(1, max_fill = c(1L, 2L)), "`max_fill` must be")
  expect_error(
    vec_fill_missing(1, max_fill = NA_integer_),
    "`max_fill` must be"
  )
  expect_error(
    vec_fill_missing(1, max_fill = "x"),
    class = "vctrs_error_incompatible_type"
  )
})

chr_rle <- function(...) {
  new_chr_rle(c(...))
}

new_chr_rle <- function(x) {
  stopifnot(is.integer(x), is_named(x))
  .Call(vctrs_altrep_rle_Make, x)
}

chr_rle_is_materialized <- function(x) {
  .Call(vctrs_altrep_rle_is_materialized, x)
}

#' Combine many data frames into one data frame
#'
#' This pair of functions binds together data frames (and vectors), either
#' row-wise or column-wise. Row-binding creates a data frame with common type
#' across all arguments. Column-binding creates a data frame with common length
#' across all arguments.
#'
#' @section Invariants:
#'
#' All inputs are first converted to a data frame. The conversion for
#' 1d vectors depends on the direction of binding:
#'
#' * For `vec_rbind()`, each element of the vector becomes a column in
#'   a single row.
#' * For `vec_cbind()`, each element of the vector becomes a row in a
#'   single column.
#'
#' Once the inputs have all become data frames, the following
#' invariants are observed for row-binding:
#'
#' * `vec_size(vec_rbind(x, y)) == vec_size(x) + vec_size(y)`
#' * `vec_ptype(vec_rbind(x, y)) = vec_ptype_common(x, y)`
#'
#' Note that if an input is an empty vector, it is first converted to
#' a 1-row data frame with 0 columns. Despite being empty, its
#' effective size for the total number of rows is 1.
#'
#' For column-binding, the following invariants apply:
#'
#' * `vec_size(vec_cbind(x, y)) == vec_size_common(x, y)`
#' * `vec_ptype(vec_cbind(x, y)) == vec_cbind(vec_ptype(x), vec_ptype(x))`
#'
#' @inheritParams vec_c
#' @inheritParams rlang::args_error_context
#'
#' @param ... Data frames or vectors.
#'
#'   When the inputs are named:
#'   * `vec_rbind()` assigns names to row names unless `.names_to` is
#'     supplied. In that case the names are assigned in the column
#'     defined by `.names_to`.
#'   * `vec_cbind()` creates packed data frame columns with named
#'      inputs.
#'
#'   `NULL` inputs are silently ignored. Empty (e.g. zero row) inputs
#'   will not appear in the output, but will affect the derived `.ptype`.
#' @param .names_to This controls what to do with names on `...`:
#'
#'   * By default, names on `...` are [zapped][rlang::zap] and do not appear
#'     anywhere in the output.
#'
#'   * If a string, specifies a column where the names on `...` will be
#'     copied. These names are often useful to identify rows with
#'     their original input. If a column name is supplied and `...` is
#'     not named, an integer column is used instead.
#'
#'   * If `NULL`, the outer names on `...` are instead merged with inner
#'     row names on each element of `...` and are subject to `.name_spec`.
#' @param .name_repair One of `"unique"`, `"universal"`, `"check_unique"`,
#'   `"unique_quiet"`, or  `"universal_quiet"`. See [vec_as_names()] for the
#'   meaning of these options.
#'
#'   With `vec_rbind()`, the repair function is applied to all inputs
#'   separately. This is because `vec_rbind()` needs to align their
#'   columns before binding the rows, and thus needs all inputs to
#'   have unique names. On the other hand, `vec_cbind()` applies the
#'   repair function after all inputs have been concatenated together
#'   in a final data frame. Hence `vec_cbind()` allows the more
#'   permissive minimal names repair.
#'
#' @return A data frame, or subclass of data frame.
#'
#'   If `...` is a mix of different data frame subclasses, `vec_ptype2()`
#'   will be used to determine the output type. For `vec_rbind()`, this
#'   will determine the type of the container and the type of each column;
#'   for `vec_cbind()` it only determines the type of the output container.
#'   If there are no non-`NULL` inputs, the result will be `data.frame()`.
#'
#' @section Dependencies:
#'
#' ## vctrs dependencies
#'
#' - [vec_cast_common()]
#' - [vec_proxy()]
#' - [vec_init()]
#' - [vec_assign()]
#' - [vec_restore()]
#'
#'
#' ## base dependencies of `vec_rbind()`
#'
#' - [base::c()]
#'
#' If columns to combine inherit from a common class,
#' `vec_rbind()` falls back to `base::c()` if there exists a `c()`
#' method implemented for this class hierarchy.
#'
#' @seealso [vec_c()] for combining 1d vectors.
#' @examples
#' # row binding -----------------------------------------
#'
#' # common columns are coerced to common class
#' vec_rbind(
#'   data.frame(x = 1),
#'   data.frame(x = FALSE)
#' )
#'
#' # unique columns are filled with NAs
#' vec_rbind(
#'   data.frame(x = 1),
#'   data.frame(y = "x")
#' )
#'
#' # null inputs are ignored
#' vec_rbind(
#'   data.frame(x = 1),
#'   NULL,
#'   data.frame(x = 2)
#' )
#'
#' # bare vectors are treated as rows
#' vec_rbind(
#'   c(x = 1, y = 2),
#'   c(x = 3)
#' )
#'
#' # default names will be supplied if arguments are not named
#' vec_rbind(
#'   1:2,
#'   1:3,
#'   1:4
#' )
#'
#' # column binding --------------------------------------
#'
#' # each input is recycled to have common length
#' vec_cbind(
#'   data.frame(x = 1),
#'   data.frame(y = 1:3)
#' )
#'
#' # bare vectors are treated as columns
#' vec_cbind(
#'   data.frame(x = 1),
#'   y = letters[1:3]
#' )
#'
#' # if you supply a named data frame, it is packed in a single column
#' data <- vec_cbind(
#'   x = data.frame(a = 1, b = 2),
#'   y = 1
#' )
#' data
#'
#' # Packed data frames are nested in a single column. This makes it
#' # possible to access it through a single name:
#' data$x
#'
#' # since the base print method is suboptimal with packed data
#' # frames, it is recommended to use tibble to work with these:
#' if (rlang::is_installed("tibble")) {
#'   vec_cbind(x = tibble::tibble(a = 1, b = 2), y = 1)
#' }
#'
#' # duplicate names are flagged
#' vec_cbind(x = 1, x = 2)
#'
#' @name vec_bind
NULL

#' @export
#' @param .name_spec A name specification (as documented in [vec_c()]) for
#'   combining the outer names on `...` with the inner row names of each element
#'   of `...`. An outer name will only ever be provided when `.names_to` is set
#'   to `NULL`, which causes the outer name to be used as part of the row names
#'   rather than as a new column, but it can still be useful to hardcode this to
#'   either [rlang::zap()] to always ignore all names, or `"inner"` to always
#'   ignore outer names, regardless of `.names_to`.
#' @rdname vec_bind
vec_rbind <- function(
  ...,
  .ptype = NULL,
  .names_to = rlang::zap(),
  .name_repair = c(
    "unique",
    "universal",
    "check_unique",
    "unique_quiet",
    "universal_quiet"
  ),
  .name_spec = NULL,
  .error_call = current_env()
) {
  .External2(ffi_rbind, list2(...), .ptype, .names_to, .name_repair, .name_spec)
}
vec_rbind <- fn_inline_formals(vec_rbind, ".name_repair")

#' @export
#' @rdname vec_bind
#' @param .size If, `NULL`, the default, will determine the number of rows in
#'   `vec_cbind()` output by using the tidyverse [recycling
#'   rules][theory-faq-recycling].
#'
#'   Alternatively, specify the desired number of rows, and any inputs of length
#'   1 will be recycled appropriately.
vec_cbind <- function(
  ...,
  .ptype = NULL,
  .size = NULL,
  .name_repair = c(
    "unique",
    "universal",
    "check_unique",
    "minimal",
    "unique_quiet",
    "universal_quiet"
  ),
  .error_call = current_env()
) {
  .External2(ffi_cbind, list2(...), .ptype, .size, .name_repair)
}
vec_cbind <- fn_inline_formals(vec_cbind, ".name_repair")

as_df_row <- function(x, quiet = FALSE) {
  .Call(ffi_as_df_row, x, quiet, environment())
}
as_df_col <- function(x, outer_name) {
  .Call(ffi_as_df_col, x, outer_name, environment())
}

#' Frame prototype
#'
#' @description
#'
#' `r lifecycle::badge("experimental")`
#'
#' This is an experimental generic that returns zero-columns variants
#' of a data frame. It is needed for [vec_cbind()], to work around the
#' lack of colwise primitives in vctrs. Expect changes.
#'
#' @param x A data frame.
#' @inheritParams rlang::args_dots_empty
#'
#' @keywords internal
#' @export
vec_cbind_frame_ptype <- function(x, ...) {
  UseMethod("vec_cbind_frame_ptype")
}
#' @export
vec_cbind_frame_ptype.default <- function(x, ...) {
  x[0]
}

#' @export
vec_cbind_frame_ptype.sf <- function(x, ...) {
  data.frame()
}

# Standalone file: do not edit by hand
# Source: <https://github.com/r-lib/rlang/blob/main/R/standalone-linked-version.R>
# ----------------------------------------------------------------------
#
# ---
# repo: r-lib/rlang
# file: standalone-linked-version.R
# last-updated: 2022-05-26
# license: https://unlicense.org
# ---
#
# nocov start


check_linked_version <- local({

  # Keep in sync with standalone-downstream-deps.R
  howto_reinstall_msg <- function(pkg) {
    os <- tolower(Sys.info()[["sysname"]])

    if (os == "windows") {
      url <- "https://github.com/jennybc/what-they-forgot/issues/62"
      c(
        i = sprintf("Please update %s to the latest version.", pkg),
        i = sprintf("Updating packages on Windows requires precautions:\n  <%s>", url)
      )
    } else {
      c(
        i = sprintf("Please update %s with `install.packages(\"%s\")` and restart R.", pkg, pkg)
      )
    }
  }

  function(pkg, with_rlang = requireNamespace("rlang", quietly = TRUE)) {
    ver <- utils::packageVersion(pkg)

    ns <- asNamespace(pkg)
    linked_ver_ptr <- ns[[paste0(pkg, "_linked_version")]]
    if (is.null(linked_ver_ptr)) {
      linked_ver <- ""
    } else {
      # Construct call to avoid NOTE when argument to `.Call()` is not
      # statically analysable
      linked_ver <- do.call(".Call", list(linked_ver_ptr))
    }

    if (nzchar(linked_ver) && ver == linked_ver) {
      return(invisible(NULL))
    }

    header <- sprintf("The %s package is not properly installed.", pkg)

    if (nzchar(linked_ver)) {
      msg <- c(x = sprintf(
        "The DLL version (%s) does not correspond to the package version (%s).",
        linked_ver,
        ver
      ))
    } else {
      # Package does not have a version pointer. This happens when DLL
      # updating fails for the first version that includes the pointer.
      msg <- c(x = "The DLL version does not correspond to the package version.")
    }

    msg <- c(msg, howto_reinstall_msg(pkg))

    if (with_rlang) {
      msg <- paste(header, rlang::format_error_bullets(msg), sep = "\n")
      rlang::abort(msg)
    } else {
      msg <- paste(c(header, msg), collapse = "\n")
      stop(msg, call. = FALSE)
    }
  }
})


# nocov end

#' Missing values
#'
#' @description
#' - `vec_detect_missing()` returns a logical vector the same size as `x`. For
#' each element of `x`, it returns `TRUE` if the element is missing, and `FALSE`
#' otherwise.
#'
#' - `vec_any_missing()` returns a single `TRUE` or `FALSE` depending on whether
#' or not `x` has _any_ missing values.
#'
#' ## Differences with [is.na()]
#'
#' Data frame rows are only considered missing if every element in the row is
#' missing. Similarly, [record vector][new_rcrd()] elements are only considered
#' missing if every field in the record is missing. Put another way, rows with
#' _any_ missing values are considered [incomplete][vec_detect_complete()], but
#' only rows with _all_ missing values are considered missing.
#'
#' List elements are only considered missing if they are `NULL`.
#'
#' @param x A vector
#'
#' @return
#' - `vec_detect_missing()` returns a logical vector the same size as `x`.
#'
#' - `vec_any_missing()` returns a single `TRUE` or `FALSE`.
#'
#' @section Dependencies:
#' - [vec_proxy_equal()]
#'
#' @name missing
#' @seealso [vec_detect_complete()]
#'
#' @examples
#' x <- c(1, 2, NA, 4, NA)
#'
#' vec_detect_missing(x)
#' vec_any_missing(x)
#'
#' # Data frames are iterated over rowwise, and only report a row as missing
#' # if every element of that row is missing. If a row is only partially
#' # missing, it is said to be incomplete, but not missing.
#' y <- c("a", "b", NA, "d", "e")
#' df <- data_frame(x = x, y = y)
#'
#' df$missing <- vec_detect_missing(df)
#' df$incomplete <- !vec_detect_complete(df)
#' df
NULL

#' @rdname missing
#' @export
vec_detect_missing <- function(x) {
  .Call(ffi_vec_detect_missing, x)
}

#' @rdname missing
#' @export
vec_any_missing <- function(x) {
  .Call(ffi_vec_any_missing, x)
}

expect_lossy_cast <- function(expr) {
  cnd <- NULL

  out <- with_handlers(
    warning = calling(function(x) {
      cnd <<- x
      cnd_muffle(x)
    }),
    expr
  )
  expect_s3_class(cnd, "vctrs_warning_cast_lossy")

  out
}

#' FAQ - How is the compatibility of vector types decided?
#'
#' @includeRmd man/faq/user/faq-compatibility-types.Rmd description
#'
#' @name faq-compatibility-types
NULL


#' FAQ - Error/Warning: Some attributes are incompatible
#'
#' @description
#'
#' This error occurs when [vec_ptype2()] or [vec_cast()] are supplied
#' vectors of the same classes with different attributes. In this
#' case, vctrs doesn't know how to combine the inputs.
#'
#' To fix this error, the maintainer of the class should implement
#' self-to-self coercion methods for [vec_ptype2()] and [vec_cast()].
#'
#' @includeRmd man/faq/developer/links-coercion.Rmd
#'
#' @name faq-error-incompatible-attributes
NULL


#' FAQ - Error: Input must be a vector
#'
#' @includeRmd man/faq/user/faq-error-scalar-type.Rmd description
#'
#' @name faq-error-scalar-type
#' @aliases faq_error_scalar_type
NULL

coerces_to <- function(x, y, using = "strict") {
  type_max <- switch(
    using,
    strict = vec_ptype2,
    base_c = c,
    base_unlist = function(x, y) unlist(list(x, y)),
    base_modify = function(x, y) `[<-`(x, 2, value = y)
  )

  tryCatch(
    {
      type <- suppressWarnings(type_max(x, y))
      vec_ptype_full(type)
    },
    error = function(e) {
      NA_character_
    }
  )
}

maxtype_mat <- function(types, using = "strict") {
  names(types) <- map_chr(types, function(x) vec_ptype_full(vec_ptype(x)))

  grid <- expand.grid(x = types, y = types)
  grid$max <- map2_chr(grid$x, grid$y, coerces_to, using = using)

  matrix(
    grid$max,
    nrow = length(types),
    dimnames = list(names(types), names(types))
  )
}

# proxies -----------------------------------------------------------------

#' Comparison and order proxy
#'
#' @description
#' `vec_proxy_compare()` and `vec_proxy_order()` return proxy objects, i.e.
#' an atomic vector or data frame of atomic vectors.
#'
#' For [`vctrs_vctr`][vctr] objects:
#'
#' - `vec_proxy_compare()` determines the behavior of `<`, `>`, `>=`
#'   and `<=` (via [vec_compare()]); and [min()], [max()], [median()], and
#'   [quantile()].
#'
#' - `vec_proxy_order()` determines the behavior of `order()` and `sort()`
#'   (via `xtfrm()`).
#'
#' @details
#' The default method of `vec_proxy_compare()` assumes that all classes built
#' on top of atomic vectors or records are comparable. Internally the default
#' calls [vec_proxy_equal()]. If your class is not comparable, you will need
#' to provide a `vec_proxy_compare()` method that throws an error.
#'
#' The behavior of `vec_proxy_order()` is identical to `vec_proxy_compare()`,
#' with the exception of lists. Lists are not comparable, as comparing
#' elements of different types is undefined. However, to allow ordering of
#' data frames containing list-columns, the ordering proxy of a list is
#' generated as an integer vector that can be used to order list elements
#' by first appearance.
#'
#' If a class implements a `vec_proxy_compare()` method, it usually doesn't need
#' to provide a `vec_proxy_order()` method, because the latter is implemented
#' by forwarding to `vec_proxy_compare()` by default. Classes inheriting from
#' list are an exception: due to the default `vec_proxy_order()` implementation,
#' `vec_proxy_compare()` and `vec_proxy_order()` should be provided for such
#' classes (with identical implementations) to avoid mismatches between
#' comparison and sorting.
#'
#' @inheritSection vec_proxy_equal Data frames
#'
#' @param x A vector x.
#' @inheritParams rlang::args_dots_empty
#' @return A 1d atomic vector or a data frame.
#'
#' @section Dependencies:
#' - [vec_proxy_equal()] called by default in `vec_proxy_compare()`
#' - [vec_proxy_compare()] called by default in `vec_proxy_order()`
#'
#' @keywords internal
#' @export
#' @examples
#' # Lists are not comparable
#' x <- list(1:2, 1, 1:2, 3)
#' try(vec_compare(x, x))
#'
#' # But lists are orderable by first appearance to allow for
#' # ordering data frames with list-cols
#' df <- new_data_frame(list(x = x))
#' vec_sort(df)
vec_proxy_compare <- function(x, ...) {
  check_dots_empty0(...)
  return(.Call(vctrs_proxy_compare, x))
  UseMethod("vec_proxy_compare")
}
#' @export
vec_proxy_compare.default <- function(x, ...) {
  stop_native_implementation("vec_proxy_compare.default")
}

#' @rdname vec_proxy_compare
#' @export
vec_proxy_order <- function(x, ...) {
  check_dots_empty0(...)
  return(.Call(vctrs_proxy_order, x))
  UseMethod("vec_proxy_order")
}

#' @export
vec_proxy_order.default <- function(x, ...) {
  stop_native_implementation("vec_proxy_order.default")
}

# compare -----------------------------------------------------------------

#' Compare two vectors
#'
#' @section S3 dispatch:
#' `vec_compare()` is not generic for performance; instead it uses
#' [vec_proxy_compare()] to create a proxy that is used in the comparison.
#'
#' @param x,y Vectors with compatible types and lengths.
#' @param na_equal Should `NA` values be considered equal?
#' @param .ptype Override to optionally specify common type
#' @return An integer vector with values -1 for `x < y`, 0 if `x == y`,
#'    and 1 if `x > y`. If `na_equal` is `FALSE`, the result will be `NA`
#'    if either `x` or `y` is `NA`.
#'
#' @section Dependencies:
#' - [vec_cast_common()] with fallback
#' - [vec_recycle_common()]
#' - [vec_proxy_compare()]
#'
#' @export
#' @examples
#' vec_compare(c(TRUE, FALSE, NA), FALSE)
#' vec_compare(c(TRUE, FALSE, NA), FALSE, na_equal = TRUE)
#'
#' vec_compare(1:10, 5)
#' vec_compare(runif(10), 0.5)
#' vec_compare(letters[1:10], "d")
#'
#' df <- data.frame(x = c(1, 1, 1, 2), y = c(0, 1, 2, 1))
#' vec_compare(df, data.frame(x = 1, y = 1))
vec_compare <- function(x, y, na_equal = FALSE, .ptype = NULL) {
  obj_check_vector(x)
  obj_check_vector(y)
  check_bool(na_equal)

  args <- vec_recycle_common(x, y)
  args <- vec_cast_common_params(!!!args, .to = .ptype)

  .Call(
    ffi_vec_compare,
    vec_proxy_compare(args[[1]]),
    vec_proxy_compare(args[[2]]),
    na_equal
  )
}


# Helpers -----------------------------------------------------------------

# Used for testing
cmp <- function(x, y) (x > y) - (x < y)

test_that("`numeric_version` is a vector (#723)", {
  x <- numeric_version("0.1.0")
  y <- numeric_version("0.2.0")
  z <- c(x, y)

  expect_true(vec_is(x))

  expect_true(vec_equal(x, x))
  expect_false(vec_equal(x, y))
  expect_identical(vec_equal(y, z), c(FALSE, TRUE))

  expect_identical(vec_unique(z), z)
  expect_identical(vec_unique(c(y, z, x)), z[2:1])
})

test_that("`numeric_version` falls back to base methods", {
  x <- utils::packageVersion("rlang")
  y <- utils::packageVersion("vctrs")
  z <- c(x, y)

  # `z` is a `list-of`-like type but slicing 1 element returns the
  # atomic type. To implement this in vctrs we'd need to provide a way
  # of customising the "wrapper" type for size > 1 vectors.
  expect_identical(vec_slice(z, 1:2), z)
  expect_identical(vec_slice(z, 1), x)
  expect_identical(vec_slice(z, 2), y)

  expect_identical(vec_c(x, y), z)
})

test_that("`numeric_version` has an equality, comparison, and order proxy", {
  numeric_row <- function(...) {
    out <- list2(...)
    out <- map(out, as.integer)
    names(out) <- paste0("...", seq_len(8L))
    new_data_frame(out, n = 1L)
  }

  x <- numeric_version(c("1.2-3", "1.21.1", "3", "2.21.0.9000", "0.5.01"))

  expect <- vec_rbind(
    numeric_row(1, 2, 3, 0, 0, 0, 0, 0),
    numeric_row(1, 21, 1, 0, 0, 0, 0, 0),
    numeric_row(3, 0, 0, 0, 0, 0, 0, 0),
    numeric_row(2, 21, 0, 9000, 0, 0, 0, 0),
    numeric_row(0, 5, 1, 0, 0, 0, 0, 0)
  )

  expect_identical(vec_proxy_equal(x), expect)
  expect_identical(vec_proxy_compare(x), expect)
  expect_identical(vec_proxy_order(x), expect)
})

test_that("`numeric_version` proxy works with empty vectors", {
  x <- numeric_version(character())

  expect <- vec_rep(list(integer()), times = 8L)
  names(expect) <- paste0("...", seq_len(8L))
  expect <- new_data_frame(expect, n = 0L)

  expect_identical(vec_proxy_equal(x), expect)
})

test_that("`numeric_version` proxy handles pseudo-`NA`", {
  numeric_row <- function(...) {
    out <- list2(...)
    out <- map(out, as.integer)
    names(out) <- paste0("...", seq_len(8L))
    new_data_frame(out, n = 1L)
  }

  x <- numeric_version(c("1_1", "1.2", NA), strict = FALSE)

  expect <- vec_rbind(
    numeric_row(NA, NA, NA, NA, NA, NA, NA, NA),
    numeric_row(1, 2, 0, 0, 0, 0, 0, 0),
    numeric_row(NA, NA, NA, NA, NA, NA, NA, NA)
  )

  expect_identical(vec_proxy_equal(x), expect)
  expect_identical(vec_proxy_compare(x), expect)
  expect_identical(vec_proxy_order(x), expect)
})

test_that("`numeric_version` works with functions using the equality proxy", {
  x <- numeric_version(c("1.2-3", "1.21.1", "1_1", "0.5", "1.3"), strict = FALSE)
  y <- numeric_version(c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"), strict = FALSE)

  expect_identical(vec_unique(x), x)
  expect_identical(vec_unique(y), y[c(1, 3, 4)])

  expect_identical(vec_detect_missing(y), c(FALSE, FALSE, TRUE, FALSE, TRUE))

  expect_identical(vec_equal(x, y), c(FALSE, TRUE, NA, TRUE, NA))
  expect_identical(vec_equal(x, y, na_equal = TRUE), c(FALSE, TRUE, TRUE, TRUE, FALSE))
})

test_that("`numeric_version` works with functions using the comparison proxy", {
  x <- numeric_version(c("1.2-3", "1.21.1", "1_1", "0.5", "1.3"), strict = FALSE)
  y <- numeric_version(c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"), strict = FALSE)

  expect_identical(vec_compare(x, y), c(-1L, 0L, NA, 0L, NA))
  expect_identical(vec_compare(x, y, na_equal = TRUE), c(-1L, 0L, 0L, 0L, 1L))

  # Specifically related to base R taking a joint proxy in `Ops.numeric_version`
  x <- numeric_version("3.3")
  y <- numeric_version("3.21")
  # `.encode_numeric_version(x) < .encode_numeric_version(y)` == FALSE
  # `x < y` == TRUE
  expect_identical(vec_compare(x, y), -1L)
})

test_that("`numeric_version` works with functions using the order proxy (tidyverse/dplyr#6680)", {
  x <- numeric_version(c("1.2-3", "1.21.1", "1_1", "0.5", "1.30"), strict = FALSE)
  y <- numeric_version(c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"), strict = FALSE)

  expect_identical(vec_order(y), c(4L, 1L, 2L, 3L, 5L))
  expect_identical(vec_order_radix(y), c(4L, 1L, 2L, 3L, 5L))

  expect_identical(vec_order(y, na_value = "smallest"), c(3L, 5L, 4L, 1L, 2L))
  expect_identical(vec_order_radix(y, na_value = "smallest"), c(3L, 5L, 4L, 1L, 2L))

  expect_identical(
    vec_locate_matches(x, y),
    data_frame(
      needles = c(1L, 2L, 2L, 3L, 3L, 4L, 5L),
      haystack = c(NA, 1L, 2L, 3L, 5L, 4L, NA)
    )
  )
  expect_identical(
    vec_locate_matches(x, y, condition = "<"),
    data_frame(
      needles = c(1L, 1L, 2L, 3L, 4L, 4L, 5L),
      haystack = c(1L, 2L, NA, NA, 1L, 2L, NA)
    )
  )
})

test_that("`numeric_version` proxy can handle at most 8 components", {
  x <- numeric_version("1.2.3.4.5.6.7.8")
  expect_silent(vec_proxy_equal(x))

  x <- numeric_version("1.2.3.4.5.6.7.8.9")
  expect_snapshot(error = TRUE, {
    vec_proxy_equal(x)
  })
})

test_that("`numeric_version` can compare against components with 8 components", {
  x <- numeric_version("2.3.4.5.6.7.8.9")
  y <- c(x, numeric_version(c("1.1", "11.2", "2.1")))

  expect_identical(vec_compare(x, y), c(0L, 1L, -1L, 1L))
})

test_that("`package_version` and `R_system_version` use the `numeric_version` proxy", {
  x <- numeric_version("1.5.6")
  y <- package_version("1.5.6")
  z <- R_system_version("1.5.6")

  expect_identical(vec_proxy_equal(y), vec_proxy_equal(x))
  expect_identical(vec_proxy_equal(z), vec_proxy_equal(x))
})

test_that("can slice `ts` vectors", {
  x <- ts(1:3)
  expect_identical(vec_ptype(x), x[0])
  expect_identical(vec_slice(x, 2), x[2])
})

test_that("can concatenate `ts` vectors", {
  x <- ts(1:3)
  expect_identical(vec_c(x, x), c(x, x))

  df <- data_frame(x = x)
  expect_identical(vec_rbind(df, df), data_frame(x = c(x, x)))
})

test_that("`omit` class is numeric (#1160)", {
  x <- c(NA, 1:3, NA)
  omit <- attr(na.omit(x), "na.action")

  expect_identical(vec_ptype_common(omit, omit), structure(int(), class = "omit"))
  expect_identical(vec_ptype_common(1.5, omit), dbl())
  expect_identical(vec_ptype_common(omit, 1L), int())

  expect_identical(vec_cast_common(omit, omit), list(omit, omit))
  expect_identical(vec_cast_common(omit, 1L), list(unstructure(omit), 1L))
  expect_identical(vec_cast_common(1.5, omit), list(1.5, unstructure(as.double(omit))))

  expect_error(vec_cast(1L, omit), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1.0, omit), class = "vctrs_error_incompatible_type")

  expect_identical(vec_slice(omit, 1), structure(1L, class = "omit"))
  expect_identical(vec_c(omit, omit), structure(c(1L, 5L, 1L, 5L), class = "omit"))
  expect_identical(vec_c(omit, omit, 10L), c(1L, 5L, 1L, 5L, 10L))

  expect_identical(vec_slice(x, omit), x[omit])
})

test_that("`exclude` class is numeric (#1160)", {
  x <- c(NA, 1:3, NA)
  exc <- attr(na.exclude(x), "na.action")

  expect_identical(vec_ptype_common(exc, exc), structure(int(), class = "exclude"))
  expect_identical(vec_ptype_common(1.5, exc), dbl())
  expect_identical(vec_ptype_common(exc, 1L), int())

  expect_identical(vec_cast_common(exc, exc), list(exc, exc))
  expect_identical(vec_cast_common(exc, 1L), list(unstructure(exc), 1L))
  expect_identical(vec_cast_common(1.5, exc), list(1.5, unstructure(as.double(exc))))

  expect_error(vec_cast(1L, exc), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1.0, exc), class = "vctrs_error_incompatible_type")

  expect_identical(vec_slice(exc, 1), structure(1L, class = "exclude"))
  expect_identical(vec_c(exc, exc), structure(c(1L, 5L, 1L, 5L), class = "exclude"))
  expect_identical(vec_c(exc, exc, 10L), c(1L, 5L, 1L, 5L, 10L))

  expect_identical(vec_slice(x, exc), x[exc])
})

test_that("inputs must be vectors", {
  expect_error(vec_compare(NULL, 1), class = "vctrs_error_scalar_type")
  expect_error(vec_compare(1, NULL), class = "vctrs_error_scalar_type")
})

test_that("matches R ordering", {
  expect_same <- function(x, y) {
    expect_equal(vec_compare(!!x, !!y), cmp(!!x, !!y))
  }

  expect_same(c(NA, FALSE, TRUE), FALSE)
  expect_same(c(NA, -100L, 0L, 100L), 0L)
  expect_same(c(NA, -Inf, -100, 100, Inf), 0L)
  expect_same(c(NA, NaN, 0), NA)
  expect_same(c(NA, "a", "b", "c"), "b")
  expect_same(as.raw(2:5), as.raw(4))
})

test_that("NAs equal when requested", {
  expect_value <- function(x, y, val, .ptype = NULL) {
    expect_equal(vec_compare(!!x, !!y, .ptype = .ptype, na_equal = TRUE), !!val)
  }

  expect_value(NA, NA, 0L)
  expect_value(NA, FALSE, -1L)
  expect_value(FALSE, NA, 1L)

  expect_value(NA_integer_, NA_integer_, 0L)
  expect_value(NA_integer_, 0L, -1L)
  expect_value(0L, NA_integer_, 1L)

  expect_value(NA_character_, NA_character_, 0L)
  expect_value(NA_character_, "", -1L)
  expect_value("", NA_character_, 1L)

  expect_value(0, NA_real_, 1L)
  expect_value(0, NaN, 1L)
  expect_value(0, 0, 0L)
  expect_value(NA_real_, NA_real_, 0L)
  expect_value(NA_real_, NaN, 1L)
  expect_value(NA_real_, 0, -1L)
  expect_value(NaN, NA_real_, -1L)
  expect_value(NaN, NaN, 0L)
  expect_value(NaN, 0, -1L)
})

test_that("data frames are compared column by column", {
  df1 <- data.frame(x = c(1, 1, 1), y = c(-1, 0, 1))

  expect_equal(vec_compare(df1, df1[2, ]), c(-1, 0, 1))
  expect_equal(vec_compare(df1[1], df1[2, 1, drop = FALSE]), c(0, 0, 0))
  expect_equal(vec_compare(df1[2], df1[2, 2, drop = FALSE]), c(-1, 0, 1))
  expect_equal(vec_compare(df1[2:1], df1[2, 2:1]), c(-1, 0, 1))
})

test_that("can compare data frames with various types of columns", {
  x1 <- data_frame(x = 1, y = 2)
  y1 <- data_frame(x = 2, y = 1)

  x2 <- data_frame(x = "a")
  y2 <- data_frame(x = "b")

  x3 <- data_frame(x = FALSE)
  y3 <- data_frame(x = TRUE)

  x4 <- data_frame(x = 1L)
  y4 <- data_frame(x = 2L)

  expect_equal(vec_compare(x1, y1), -1)
  expect_equal(vec_compare(x2, y2), -1)
  expect_equal(vec_compare(x3, y3), -1)
  expect_equal(vec_compare(x4, y4), -1)
})

test_that("can compare data frames with data frame columns", {
  df1 <- data_frame(x = data_frame(a = 1))
  df2 <- data_frame(x = data_frame(a = 2))

  expect_equal(vec_compare(df1, df1), 0)
  expect_equal(vec_compare(df1, df2), -1)
})

test_that("can compare data frames with 0 columns", {
  x <- new_data_frame(n = 2L)
  expect_identical(vec_compare(x, x), c(0L, 0L))
})

test_that("C code doesn't crash with bad inputs", {
  df <- data.frame(x = c(1, 1, 1), y = c(-1, 0, 1))

  expect_error(.Call(ffi_vec_compare, df, df[1], TRUE), "not comparable")

  # Names are not checked, as `vec_cast_common()` should take care of the type.
  # So if `vec_cast_common()` is not called, or is improperly specified, then
  # this could result in false equality.
  expect_equal(.Call(ffi_vec_compare, df, setNames(df, c("x", "z")), TRUE), c(0, 0, 0))

  df1 <- new_data_frame(list(x = 1:3, y = c(1, 1, 1)))
  df2 <- new_data_frame(list(y = 1:2, x = 1:2))
  expect_error(.Call(ffi_vec_compare, df1, df2, TRUE), "must have the same types and lengths")
})

test_that("xtfrm.vctrs_vctr works for variety of base classes", {
  df <- data.frame(x = c(NA, 1, 1), y = c(1, 2, 1))
  # Internally uses `vec_rank()`, which propagates rows if not "complete"
  expect_equal(xtfrm.vctrs_vctr(df), c(NA, 2, 1))

  x <- c(2, 3, 1)
  expect_equal(xtfrm.vctrs_vctr(x), x)
  expect_equal(xtfrm.vctrs_vctr(letters[x]), x)
})

test_that("vec_proxy_order() orders list using order of appearance", {
  x <- 1:2
  y <- 2:4
  z <- "a"

  lst <- list(x, y, x, y, z)

  expect_identical(vec_proxy_order(lst), c(1L, 2L, 1L, 2L, 5L))
})

test_that("vec_compare() calls vec_proxy_compare()", {
  local_methods(
    vec_proxy_compare.vctrs_foobar = function(x, ...) rev(x),
    vec_ptype2.integer.vctrs_foobar = function(...) foobar(int()),
    vec_ptype2.vctrs_foobar = function(...) foobar(int()),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.integer = function(x, ...) x,
  )
  expect_identical(vec_compare(1:3, 1:3), int(0, 0, 0))
  expect_identical(vec_compare(1:3, foobar(1:3)), int(-1, 0, 1))
})

test_that("vec_proxy_compare() preserves data frames and vectors", {
  df <- data_frame(x = 1:2, y = c("a", "b"))
  expect_identical(vec_proxy_compare(df), df)

  x <- c(NA, "a", "b", "c")
  expect_identical(vec_proxy_compare(x), x)
})

test_that("vec_proxy_compare() handles data frame with a POSIXlt column", {
  df <- data.frame(times = 1:5, x = 1:5)
  df$times <- as.POSIXlt(seq.Date(as.Date("2019-12-30"), as.Date("2020-01-03"), by = "day"))

  df2 <- df
  df2$times <- vec_proxy_compare(df$times)

  expect_identical(
    vec_proxy_compare(df),
    vec_proxy_compare(df2)
  )
})

test_that("vec_proxy_compare.POSIXlt() correctly orders (#720)", {
  dates <- as.POSIXlt(seq.Date(as.Date("2019-12-30"), as.Date("2020-01-03"), by = "day"))
  expect_equal(vec_order(dates), 1:5)
})

test_that("vec_proxy_compare.POSIXlt() correctly orders around DST", {
  # 1am in EDT
  x <- as.POSIXlt("2020-11-01 01:00:00", tz = "America/New_York")

  # "falls back" to 1am again, but in EST
  y <- as.POSIXlt(x + 3600)

  expect_equal(vec_order(c(y, x)), c(2, 1))
})

test_that("vec_proxy_compare() flattens df-cols", {
  df_col <- data_frame(z = 3:4, w = 4:5)
  df <- data_frame(x = 1:2, y = df_col)

  expect <- data_frame(x = 1:2, z = 3:4, w = 4:5)

  expect_identical(vec_proxy_compare(df), expect)
})

test_that("vec_proxy_compare() unwraps 1 col dfs", {
  df <- data_frame(x = 1:2)

  expect_identical(vec_proxy_compare(df), 1:2)

  df_col <- data_frame(y = 1:2)
  df <- data_frame(x = df_col)

  expect_identical(vec_proxy_compare(df), 1:2)
})

test_that("vec_proxy_order() works on deeply nested lists", {
  df_col <- data_frame(z = list("b", "a", "b"))

  # Relaxed and unwrapped
  df1 <- data_frame(x = df_col)
  expect_identical(vec_proxy_order(df1), c(1L, 2L, 1L))

  df2 <- data_frame(x = df_col, y = 1:3)
  expect_identical(vec_proxy_order(df2), data_frame(x = c(1L, 2L, 1L), y = 1:3))
})

test_that("error is thrown when comparing complexes (#1655)", {
  expect_snapshot({
    (expect_error(vec_compare(complex(), complex())))
  })
})

test_that("error is thrown when comparing lists", {
  expect_error(vec_compare(list(), list()), class = "vctrs_error_unsupported")
  expect_error(.Call(ffi_vec_compare, list(), list(), FALSE), "Can't compare lists")
})

test_that("error is thrown when comparing data frames with list columns", {
  df <- data_frame(x = list())
  expect_error(vec_compare(df, df), class = "vctrs_error_unsupported")
  expect_error(.Call(ffi_vec_compare, df, df, FALSE), "Can't compare lists")
})

test_that("error is thrown when comparing scalars", {
  x <- new_sclr(x = 1)
  expect_error(vec_compare(x, x), class = "vctrs_error_scalar_type")
  expect_error(.Call(ffi_vec_compare, x, x, FALSE), class = "vctrs_error_scalar_type")
})

test_that("`na_equal` is validated", {
  expect_snapshot({
    (expect_error(vec_compare(1, 1, na_equal = 1)))
    (expect_error(vec_compare(1, 1, na_equal = c(TRUE, FALSE))))
  })
})

test_that("can compare equal strings with different encodings", {
  for (x_encoding in encodings()) {
    for (y_encoding in encodings()) {
      expect_equal(vec_compare(x_encoding, y_encoding), 0L)
    }
  }
})

test_that("can compare non-equal strings with different encodings", {
  x <- "x"
  y <- encodings()$latin1

  expect_equal(vec_compare(x, y), -1L)
})

test_that("comparison can be determined when strings have identical encodings", {
  encs <- encodings()

  for (enc in encs) {
    expect_equal(vec_compare(enc, enc), 0L)
  }
})

test_that("comparison is known to always fail with bytes", {
  enc <- encoding_bytes()
  error <- "translating strings with \"bytes\" encoding"
  expect_error(vec_compare(enc, enc), error)
})

test_that("comparison is known to fail when comparing bytes to other encodings", {
  error <- "translating strings with \"bytes\" encoding"

  for (enc in encodings()) {
    expect_error(vec_compare(encoding_bytes(), enc), error)
    expect_error(vec_compare(enc, encoding_bytes()), error)
  }
})

test_that("can compare unspecified", {
  expect_equal(vec_compare(NA, NA), NA_integer_)
  expect_equal(vec_compare(NA, NA, na_equal = TRUE), 0)
  expect_equal(vec_compare(c(NA, NA), unspecified(2)), c(NA_integer_, NA_integer_))
})

test_that("can't supply NA as `na_equal`", {
  expect_snapshot(error = TRUE, {
    vec_compare(NA, NA, na_equal = NA)
  })
})

test_that("vec_compare() silently falls back to base data frame", {
  expect_silent(expect_identical(
    vec_compare(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(0L, 32)
  ))
})

test_that("inputs must be vectors", {
  expect_error(vec_compare(NULL, 1), class = "vctrs_error_scalar_type")
  expect_error(vec_compare(1, NULL), class = "vctrs_error_scalar_type")
})

test_that("matches R ordering", {
  expect_same <- function(x, y) {
    expect_equal(vec_compare(!!x, !!y), cmp(!!x, !!y))
  }

  expect_same(c(NA, FALSE, TRUE), FALSE)
  expect_same(c(NA, -100L, 0L, 100L), 0L)
  expect_same(c(NA, -Inf, -100, 100, Inf), 0L)
  expect_same(c(NA, NaN, 0), NA)
  expect_same(c(NA, "a", "b", "c"), "b")
  expect_same(as.raw(2:5), as.raw(4))
})

test_that("NAs equal when requested", {
  expect_value <- function(x, y, val, .ptype = NULL) {
    expect_equal(vec_compare(!!x, !!y, .ptype = .ptype, na_equal = TRUE), !!val)
  }

  expect_value(NA, NA, 0L)
  expect_value(NA, FALSE, -1L)
  expect_value(FALSE, NA, 1L)

  expect_value(NA_integer_, NA_integer_, 0L)
  expect_value(NA_integer_, 0L, -1L)
  expect_value(0L, NA_integer_, 1L)

  expect_value(NA_character_, NA_character_, 0L)
  expect_value(NA_character_, "", -1L)
  expect_value("", NA_character_, 1L)

  expect_value(0, NA_real_, 1L)
  expect_value(0, NaN, 1L)
  expect_value(0, 0, 0L)
  expect_value(NA_real_, NA_real_, 0L)
  expect_value(NA_real_, NaN, 1L)
  expect_value(NA_real_, 0, -1L)
  expect_value(NaN, NA_real_, -1L)
  expect_value(NaN, NaN, 0L)
  expect_value(NaN, 0, -1L)
})

test_that("data frames are compared column by column", {
  df1 <- data.frame(x = c(1, 1, 1), y = c(-1, 0, 1))

  expect_equal(vec_compare(df1, df1[2, ]), c(-1, 0, 1))
  expect_equal(vec_compare(df1[1], df1[2, 1, drop = FALSE]), c(0, 0, 0))
  expect_equal(vec_compare(df1[2], df1[2, 2, drop = FALSE]), c(-1, 0, 1))
  expect_equal(vec_compare(df1[2:1], df1[2, 2:1]), c(-1, 0, 1))
})

test_that("can compare data frames with various types of columns", {
  x1 <- data_frame(x = 1, y = 2)
  y1 <- data_frame(x = 2, y = 1)

  x2 <- data_frame(x = "a")
  y2 <- data_frame(x = "b")

  x3 <- data_frame(x = FALSE)
  y3 <- data_frame(x = TRUE)

  x4 <- data_frame(x = 1L)
  y4 <- data_frame(x = 2L)

  expect_equal(vec_compare(x1, y1), -1)
  expect_equal(vec_compare(x2, y2), -1)
  expect_equal(vec_compare(x3, y3), -1)
  expect_equal(vec_compare(x4, y4), -1)
})

test_that("can compare data frames with data frame columns", {
  df1 <- data_frame(x = data_frame(a = 1))
  df2 <- data_frame(x = data_frame(a = 2))

  expect_equal(vec_compare(df1, df1), 0)
  expect_equal(vec_compare(df1, df2), -1)
})

test_that("can compare data frames with 0 columns", {
  x <- new_data_frame(n = 2L)
  expect_identical(vec_compare(x, x), c(0L, 0L))
})

test_that("C code doesn't crash with bad inputs", {
  df <- data.frame(x = c(1, 1, 1), y = c(-1, 0, 1))

  expect_error(.Call(ffi_vec_compare, df, df[1], TRUE), "not comparable")

  # Names are not checked, as `vec_cast_common()` should take care of the type.
  # So if `vec_cast_common()` is not called, or is improperly specified, then
  # this could result in false equality.
  expect_equal(
    .Call(ffi_vec_compare, df, setNames(df, c("x", "z")), TRUE),
    c(0, 0, 0)
  )

  df1 <- new_data_frame(list(x = 1:3, y = c(1, 1, 1)))
  df2 <- new_data_frame(list(y = 1:2, x = 1:2))
  expect_error(
    .Call(ffi_vec_compare, df1, df2, TRUE),
    "must have the same types and lengths"
  )
})

test_that("xtfrm.vctrs_vctr works for variety of base classes", {
  df <- data.frame(x = c(NA, 1, 1), y = c(1, 2, 1))
  # Internally uses `vec_rank()`, which propagates rows if not "complete"
  expect_equal(xtfrm.vctrs_vctr(df), c(NA, 2, 1))

  x <- c(2, 3, 1)
  expect_equal(xtfrm.vctrs_vctr(x), x)
  expect_equal(xtfrm.vctrs_vctr(letters[x]), x)
})

test_that("vec_proxy_order() orders list using order of appearance", {
  x <- 1:2
  y <- 2:4
  z <- "a"

  lst <- list(x, y, x, y, z)

  expect_identical(vec_proxy_order(lst), c(1L, 2L, 1L, 2L, 5L))
})

test_that("vec_compare() calls vec_proxy_compare()", {
  local_methods(
    vec_proxy_compare.vctrs_foobar = function(x, ...) rev(x),
    vec_ptype2.integer.vctrs_foobar = function(...) foobar(int()),
    vec_ptype2.vctrs_foobar = function(...) foobar(int()),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.integer = function(x, ...) x,
  )
  expect_identical(vec_compare(1:3, 1:3), int(0, 0, 0))
  expect_identical(vec_compare(1:3, foobar(1:3)), int(-1, 0, 1))
})

test_that("vec_proxy_compare() preserves data frames and vectors", {
  df <- data_frame(x = 1:2, y = c("a", "b"))
  expect_identical(vec_proxy_compare(df), df)

  x <- c(NA, "a", "b", "c")
  expect_identical(vec_proxy_compare(x), x)
})

test_that("vec_proxy_compare() handles data frame with a POSIXlt column", {
  df <- data.frame(times = 1:5, x = 1:5)
  df$times <- as.POSIXlt(seq.Date(
    as.Date("2019-12-30"),
    as.Date("2020-01-03"),
    by = "day"
  ))

  df2 <- df
  df2$times <- vec_proxy_compare(df$times)

  expect_identical(
    vec_proxy_compare(df),
    vec_proxy_compare(df2)
  )
})

test_that("vec_proxy_compare.POSIXlt() correctly orders (#720)", {
  dates <- as.POSIXlt(seq.Date(
    as.Date("2019-12-30"),
    as.Date("2020-01-03"),
    by = "day"
  ))
  expect_equal(vec_order(dates), 1:5)
})

test_that("vec_proxy_compare.POSIXlt() correctly orders around DST", {
  # 1am in EDT
  x <- as.POSIXlt("2020-11-01 01:00:00", tz = "America/New_York")

  # "falls back" to 1am again, but in EST
  y <- as.POSIXlt(x + 3600)

  expect_equal(vec_order(c(y, x)), c(2, 1))
})

test_that("vec_proxy_compare() flattens df-cols", {
  df_col <- data_frame(z = 3:4, w = 4:5)
  df <- data_frame(x = 1:2, y = df_col)

  expect <- data_frame(x = 1:2, z = 3:4, w = 4:5)

  expect_identical(vec_proxy_compare(df), expect)
})

test_that("vec_proxy_compare() unwraps 1 col dfs", {
  df <- data_frame(x = 1:2)

  expect_identical(vec_proxy_compare(df), 1:2)

  df_col <- data_frame(y = 1:2)
  df <- data_frame(x = df_col)

  expect_identical(vec_proxy_compare(df), 1:2)
})

test_that("vec_proxy_order() works on deeply nested lists", {
  df_col <- data_frame(z = list("b", "a", "b"))

  # Relaxed and unwrapped
  df1 <- data_frame(x = df_col)
  expect_identical(vec_proxy_order(df1), c(1L, 2L, 1L))

  df2 <- data_frame(x = df_col, y = 1:3)
  expect_identical(vec_proxy_order(df2), data_frame(x = c(1L, 2L, 1L), y = 1:3))
})

test_that("error is thrown when comparing complexes (#1655)", {
  expect_snapshot({
    (expect_error(vec_compare(complex(), complex())))
  })
})

test_that("error is thrown when comparing lists", {
  expect_error(vec_compare(list(), list()), class = "vctrs_error_unsupported")
  expect_error(
    .Call(ffi_vec_compare, list(), list(), FALSE),
    "Can't compare lists"
  )
})

test_that("error is thrown when comparing data frames with list columns", {
  df <- data_frame(x = list())
  expect_error(vec_compare(df, df), class = "vctrs_error_unsupported")
  expect_error(.Call(ffi_vec_compare, df, df, FALSE), "Can't compare lists")
})

test_that("error is thrown when comparing scalars", {
  x <- new_sclr(x = 1)
  expect_error(vec_compare(x, x), class = "vctrs_error_scalar_type")
  expect_error(
    .Call(ffi_vec_compare, x, x, FALSE),
    class = "vctrs_error_scalar_type"
  )
})

test_that("`na_equal` is validated", {
  expect_snapshot({
    (expect_error(vec_compare(1, 1, na_equal = 1)))
    (expect_error(vec_compare(1, 1, na_equal = c(TRUE, FALSE))))
  })
})

test_that("can compare equal strings with different encodings", {
  for (x_encoding in encodings()) {
    for (y_encoding in encodings()) {
      expect_equal(vec_compare(x_encoding, y_encoding), 0L)
    }
  }
})

test_that("can compare non-equal strings with different encodings", {
  x <- "x"
  y <- encodings()$latin1

  expect_equal(vec_compare(x, y), -1L)
})

test_that("comparison can be determined when strings have identical encodings", {
  encs <- encodings()

  for (enc in encs) {
    expect_equal(vec_compare(enc, enc), 0L)
  }
})

test_that("comparison is known to always fail with bytes", {
  enc <- encoding_bytes()
  error <- "translating strings with \"bytes\" encoding"
  expect_error(vec_compare(enc, enc), error)
})

test_that("comparison is known to fail when comparing bytes to other encodings", {
  error <- "translating strings with \"bytes\" encoding"

  for (enc in encodings()) {
    expect_error(vec_compare(encoding_bytes(), enc), error)
    expect_error(vec_compare(enc, encoding_bytes()), error)
  }
})

test_that("can compare unspecified", {
  expect_equal(vec_compare(NA, NA), NA_integer_)
  expect_equal(vec_compare(NA, NA, na_equal = TRUE), 0)
  expect_equal(
    vec_compare(c(NA, NA), unspecified(2)),
    c(NA_integer_, NA_integer_)
  )
})

test_that("can't supply NA as `na_equal`", {
  expect_snapshot(error = TRUE, {
    vec_compare(NA, NA, na_equal = NA)
  })
})

test_that("vec_compare() silently falls back to base data frame", {
  expect_silent(expect_identical(
    vec_compare(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(0L, 32)
  ))
})

test_that("`numeric_version` is a vector (#723)", {
  x <- numeric_version("0.1.0")
  y <- numeric_version("0.2.0")
  z <- c(x, y)

  expect_true(vec_is(x))

  expect_true(vec_equal(x, x))
  expect_false(vec_equal(x, y))
  expect_identical(vec_equal(y, z), c(FALSE, TRUE))

  expect_identical(vec_unique(z), z)
  expect_identical(vec_unique(c(y, z, x)), z[2:1])
})

test_that("`numeric_version` falls back to base methods", {
  x <- utils::packageVersion("rlang")
  y <- utils::packageVersion("vctrs")
  z <- c(x, y)

  # `z` is a `list-of`-like type but slicing 1 element returns the
  # atomic type. To implement this in vctrs we'd need to provide a way
  # of customising the "wrapper" type for size > 1 vectors.
  expect_identical(vec_slice(z, 1:2), z)
  expect_identical(vec_slice(z, 1), x)
  expect_identical(vec_slice(z, 2), y)

  expect_identical(vec_c(x, y), z)
})

test_that("`numeric_version` has an equality, comparison, and order proxy", {
  numeric_row <- function(...) {
    out <- list2(...)
    out <- map(out, as.integer)
    names(out) <- paste0("...", seq_len(8L))
    new_data_frame(out, n = 1L)
  }

  x <- numeric_version(c("1.2-3", "1.21.1", "3", "2.21.0.9000", "0.5.01"))

  expect <- vec_rbind(
    numeric_row(1, 2, 3, 0, 0, 0, 0, 0),
    numeric_row(1, 21, 1, 0, 0, 0, 0, 0),
    numeric_row(3, 0, 0, 0, 0, 0, 0, 0),
    numeric_row(2, 21, 0, 9000, 0, 0, 0, 0),
    numeric_row(0, 5, 1, 0, 0, 0, 0, 0)
  )

  expect_identical(vec_proxy_equal(x), expect)
  expect_identical(vec_proxy_compare(x), expect)
  expect_identical(vec_proxy_order(x), expect)
})

test_that("`numeric_version` proxy works with empty vectors", {
  x <- numeric_version(character())

  expect <- vec_rep(list(integer()), times = 8L)
  names(expect) <- paste0("...", seq_len(8L))
  expect <- new_data_frame(expect, n = 0L)

  expect_identical(vec_proxy_equal(x), expect)
})

test_that("`numeric_version` proxy handles pseudo-`NA`", {
  numeric_row <- function(...) {
    out <- list2(...)
    out <- map(out, as.integer)
    names(out) <- paste0("...", seq_len(8L))
    new_data_frame(out, n = 1L)
  }

  x <- numeric_version(c("1_1", "1.2", NA), strict = FALSE)

  expect <- vec_rbind(
    numeric_row(NA, NA, NA, NA, NA, NA, NA, NA),
    numeric_row(1, 2, 0, 0, 0, 0, 0, 0),
    numeric_row(NA, NA, NA, NA, NA, NA, NA, NA)
  )

  expect_identical(vec_proxy_equal(x), expect)
  expect_identical(vec_proxy_compare(x), expect)
  expect_identical(vec_proxy_order(x), expect)
})

test_that("`numeric_version` works with functions using the equality proxy", {
  x <- numeric_version(
    c("1.2-3", "1.21.1", "1_1", "0.5", "1.3"),
    strict = FALSE
  )
  y <- numeric_version(
    c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"),
    strict = FALSE
  )

  expect_identical(vec_unique(x), x)
  expect_identical(vec_unique(y), y[c(1, 3, 4)])

  expect_identical(vec_detect_missing(y), c(FALSE, FALSE, TRUE, FALSE, TRUE))

  expect_identical(vec_equal(x, y), c(FALSE, TRUE, NA, TRUE, NA))
  expect_identical(
    vec_equal(x, y, na_equal = TRUE),
    c(FALSE, TRUE, TRUE, TRUE, FALSE)
  )
})

test_that("`numeric_version` works with functions using the comparison proxy", {
  x <- numeric_version(
    c("1.2-3", "1.21.1", "1_1", "0.5", "1.3"),
    strict = FALSE
  )
  y <- numeric_version(
    c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"),
    strict = FALSE
  )

  expect_identical(vec_compare(x, y), c(-1L, 0L, NA, 0L, NA))
  expect_identical(vec_compare(x, y, na_equal = TRUE), c(-1L, 0L, 0L, 0L, 1L))

  # Specifically related to base R taking a joint proxy in `Ops.numeric_version`
  x <- numeric_version("3.3")
  y <- numeric_version("3.21")
  # `.encode_numeric_version(x) < .encode_numeric_version(y)` == FALSE
  # `x < y` == TRUE
  expect_identical(vec_compare(x, y), -1L)
})

test_that("`numeric_version` works with functions using the order proxy (tidyverse/dplyr#6680)", {
  x <- numeric_version(
    c("1.2-3", "1.21.1", "1_1", "0.5", "1.30"),
    strict = FALSE
  )
  y <- numeric_version(
    c("1.21.1", "1.21.1", "1_2", "0.05", "1_3"),
    strict = FALSE
  )

  expect_identical(vec_order(y), c(4L, 1L, 2L, 3L, 5L))
  expect_identical(vec_order_radix(y), c(4L, 1L, 2L, 3L, 5L))

  expect_identical(vec_order(y, na_value = "smallest"), c(3L, 5L, 4L, 1L, 2L))
  expect_identical(
    vec_order_radix(y, na_value = "smallest"),
    c(3L, 5L, 4L, 1L, 2L)
  )

  expect_identical(
    vec_locate_matches(x, y),
    data_frame(
      needles = c(1L, 2L, 2L, 3L, 3L, 4L, 5L),
      haystack = c(NA, 1L, 2L, 3L, 5L, 4L, NA)
    )
  )
  expect_identical(
    vec_locate_matches(x, y, condition = "<"),
    data_frame(
      needles = c(1L, 1L, 2L, 3L, 4L, 4L, 5L),
      haystack = c(1L, 2L, NA, NA, 1L, 2L, NA)
    )
  )
})

test_that("`numeric_version` proxy can handle at most 8 components", {
  x <- numeric_version("1.2.3.4.5.6.7.8")
  expect_silent(vec_proxy_equal(x))

  x <- numeric_version("1.2.3.4.5.6.7.8.9")
  expect_snapshot(error = TRUE, {
    vec_proxy_equal(x)
  })
})

test_that("`numeric_version` can compare against components with 8 components", {
  x <- numeric_version("2.3.4.5.6.7.8.9")
  y <- c(x, numeric_version(c("1.1", "11.2", "2.1")))

  expect_identical(vec_compare(x, y), c(0L, 1L, -1L, 1L))
})

test_that("`package_version` and `R_system_version` use the `numeric_version` proxy", {
  x <- numeric_version("1.5.6")
  y <- package_version("1.5.6")
  z <- R_system_version("1.5.6")

  expect_identical(vec_proxy_equal(y), vec_proxy_equal(x))
  expect_identical(vec_proxy_equal(z), vec_proxy_equal(x))
})

test_that("can slice `ts` vectors", {
  x <- ts(1:3)
  expect_identical(vec_ptype(x), x[0])
  expect_identical(vec_slice(x, 2), x[2])
})

test_that("can concatenate `ts` vectors", {
  x <- ts(1:3)
  expect_identical(vec_c(x, x), c(x, x))

  df <- data_frame(x = x)
  expect_identical(vec_rbind(df, df), data_frame(x = c(x, x)))
})

test_that("`omit` class is numeric (#1160)", {
  x <- c(NA, 1:3, NA)
  omit <- attr(na.omit(x), "na.action")

  expect_identical(
    vec_ptype_common(omit, omit),
    structure(int(), class = "omit")
  )
  expect_identical(vec_ptype_common(1.5, omit), dbl())
  expect_identical(vec_ptype_common(omit, 1L), int())

  expect_identical(vec_cast_common(omit, omit), list(omit, omit))
  expect_identical(vec_cast_common(omit, 1L), list(unstructure(omit), 1L))
  expect_identical(
    vec_cast_common(1.5, omit),
    list(1.5, unstructure(as.double(omit)))
  )

  expect_error(vec_cast(1L, omit), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1.0, omit), class = "vctrs_error_incompatible_type")

  expect_identical(vec_slice(omit, 1), structure(1L, class = "omit"))
  expect_identical(
    vec_c(omit, omit),
    structure(c(1L, 5L, 1L, 5L), class = "omit")
  )
  expect_identical(vec_c(omit, omit, 10L), c(1L, 5L, 1L, 5L, 10L))

  expect_identical(vec_slice(x, omit), x[omit])
})

test_that("`exclude` class is numeric (#1160)", {
  x <- c(NA, 1:3, NA)
  exc <- attr(na.exclude(x), "na.action")

  expect_identical(
    vec_ptype_common(exc, exc),
    structure(int(), class = "exclude")
  )
  expect_identical(vec_ptype_common(1.5, exc), dbl())
  expect_identical(vec_ptype_common(exc, 1L), int())

  expect_identical(vec_cast_common(exc, exc), list(exc, exc))
  expect_identical(vec_cast_common(exc, 1L), list(unstructure(exc), 1L))
  expect_identical(
    vec_cast_common(1.5, exc),
    list(1.5, unstructure(as.double(exc)))
  )

  expect_error(vec_cast(1L, exc), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1.0, exc), class = "vctrs_error_incompatible_type")

  expect_identical(vec_slice(exc, 1), structure(1L, class = "exclude"))
  expect_identical(
    vec_c(exc, exc),
    structure(c(1L, 5L, 1L, 5L), class = "exclude")
  )
  expect_identical(vec_c(exc, exc, 10L), c(1L, 5L, 1L, 5L, 10L))

  expect_identical(vec_slice(x, exc), x[exc])
})

# `numeric_version` from base ----------------------------------------

#' @export
vec_proxy.numeric_version <- function(x, ...) {
  x
}

#' @export
vec_proxy_equal.numeric_version <- function(x, ...) {
  proxy_equal_numeric_version(x)
}

# To generate data agnostic proxies of `<numeric_version>`, we enforce a
# restriction that each version can have at most 8 components. This allows us
# to `vec_compare()` them without needing a "joint" comparison proxy, unlike
# what `.encode_numeric_version()` returns.
proxy_equal_numeric_version <- function(x, error_call = caller_env()) {
  N_COMPONENTS <- 8L

  x <- unclass(x)

  size <- length(x)
  sizes <- lengths(x)

  if (length(sizes) != 0L) {
    max <- max(sizes)
  } else {
    max <- N_COMPONENTS
  }

  if (max > N_COMPONENTS) {
    cli::cli_abort(
      "`x` can't contain more than {N_COMPONENTS} version components.",
      call = error_call
    )
  }

  if (any(sizes != max)) {
    # Pad with zeros where needed to be able to transpose.
    # This is somewhat slow if required.
    pad_sizes <- max - sizes
    pad_needed <- which(pad_sizes != 0L)

    x[pad_needed] <- map2(
      x[pad_needed],
      pad_sizes[pad_needed],
      function(elt, pad_size) {
        c(elt, vec_rep(0L, times = pad_size))
      }
    )
  }

  # Transpose with combination of `vec_interleave()` and `vec_chop()`
  x <- vec_interleave(!!!x, .ptype = integer())
  out <- vec_chop(x, sizes = vec_rep(size, times = max))

  n_zeros <- N_COMPONENTS - max

  if (n_zeros != 0L) {
    # Pad columns of zeros out to `N_COMPONENTS` columns
    zero <- list(vec_rep(0L, times = size))
    out <- c(out, vec_rep(zero, times = n_zeros))
  }

  # Use a data frame as the proxy
  names(out) <- paste0("...", seq_len(N_COMPONENTS))
  out <- new_data_frame(out, n = size)

  # A `<numeric_version>` internally stored as `integer()` is considered the
  # `NA` value. We patch that in at the very end if needed. It is hard to create
  # so should be very uncommon.
  missing <- sizes == 0L

  if (any(missing)) {
    na <- vec_init(out)
    out <- vec_assign(out, missing, na)
  }

  out
}

# `omit` from base ---------------------------------------------------

#' @export
vec_proxy.omit <- function(x, ...) {
  x
}
#' @export
vec_restore.omit <- function(x, ...) {
  structure(x, class = "omit")
}

#' @export
vec_ptype2.omit.omit <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.integer.omit <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.omit.integer <- function(x, y, ...) {
  y
}
#' @export
vec_ptype2.double.omit <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.omit.double <- function(x, y, ...) {
  y
}

#' @export
vec_cast.omit.omit <- function(x, to, ...) {
  x
}
#' @export
vec_cast.integer.omit <- function(x, to, ...) {
  vec_cast(vec_data(x), to, ...)
}
#' @export
vec_cast.omit.integer <- function(x, to, ..., x_arg = "", to_arg = "") {
  stop_incompatible_cast(x, to, x_arg = x_arg, to_arg = to_arg)
}
#' @export
vec_cast.double.omit <- function(x, to, ...) {
  vec_cast(vec_data(x), to, ...)
}
#' @export
vec_cast.omit.double <- function(x, to, ..., x_arg = "", to_arg = "") {
  stop_incompatible_cast(x, to, x_arg = x_arg, to_arg = to_arg)
}


# `exclude` from base ------------------------------------------------

#' @export
vec_proxy.exclude <- function(x, ...) {
  x
}
#' @export
vec_restore.exclude <- function(x, ...) {
  structure(x, class = "exclude")
}

#' @export
vec_ptype2.exclude.exclude <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.integer.exclude <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.exclude.integer <- function(x, y, ...) {
  y
}
#' @export
vec_ptype2.double.exclude <- function(x, y, ...) {
  x
}
#' @export
vec_ptype2.exclude.double <- function(x, y, ...) {
  y
}

#' @export
vec_cast.exclude.exclude <- function(x, to, ...) {
  x
}
#' @export
vec_cast.integer.exclude <- function(x, to, ...) {
  vec_cast(vec_data(x), to, ...)
}
#' @export
vec_cast.exclude.integer <- function(x, to, ..., x_arg = "", to_arg = "") {
  stop_incompatible_cast(x, to, x_arg = x_arg, to_arg = to_arg)
}
#' @export
vec_cast.double.exclude <- function(x, to, ...) {
  vec_cast(vec_data(x), to, ...)
}
#' @export
vec_cast.exclude.double <- function(x, to, ..., x_arg = "", to_arg = "") {
  stop_incompatible_cast(x, to, x_arg = x_arg, to_arg = to_arg)
}

#' Complete
#'
#' @description
#' `vec_detect_complete()` detects "complete" observations. An observation is
#' considered complete if it is non-missing. For most vectors, this implies that
#' `vec_detect_complete(x) == !vec_detect_missing(x)`.
#'
#' For data frames and matrices, a row is only considered complete if all
#' elements of that row are non-missing. To compare, `!vec_detect_missing(x)`
#' detects rows that are partially complete (they have at least one non-missing
#' value).
#'
#' @details
#' A [record][new_rcrd] type vector is similar to a data frame, and is only
#' considered complete if all fields are non-missing.
#'
#' @param x A vector
#'
#' @return
#' A logical vector with the same size as `x`.
#'
#' @seealso [stats::complete.cases()]
#' @export
#' @examples
#' x <- c(1, 2, NA, 4, NA)
#'
#' # For most vectors, this is identical to `!vec_detect_missing(x)`
#' vec_detect_complete(x)
#' !vec_detect_missing(x)
#'
#' df <- data_frame(
#'   x = x,
#'   y = c("a", "b", NA, "d", "e")
#' )
#'
#' # This returns `TRUE` where all elements of the row are non-missing.
#' # Compare that with `!vec_detect_missing()`, which detects rows that have at
#' # least one non-missing value.
#' df2 <- df
#' df2$all_non_missing <- vec_detect_complete(df)
#' df2$any_non_missing <- !vec_detect_missing(df)
#' df2
vec_detect_complete <- function(x) {
  .Call(vctrs_detect_complete, x)
}

vec_slice_complete <- function(x) {
  .Call(vctrs_slice_complete, x)
}

vec_locate_complete <- function(x) {
  .Call(vctrs_locate_complete, x)
}

# vec_slice_complete -----------------------------------------------------------

test_that("can slice complete", {
  df <- data_frame(x = c(1, NA, 3), y = c(1, 2, NA))
  expect_identical(vec_slice_complete(df), vec_slice(df, 1))
})

test_that("vec_slice_complete() works with size 0 input", {
  expect_identical(vec_slice_complete(integer()), integer())
  expect_identical(vec_slice_complete(data.frame()), data.frame())
})

# vec_locate_complete ----------------------------------------------------------

test_that("vec_locate_complete() can locate complete", {
  df <- data_frame(x = c(1, NA, 3), y = c(1, 2, NA))
  expect_identical(vec_locate_complete(df), 1L)
})

test_that("vec_locate_complete() works with size 0 input", {
  expect_identical(vec_locate_complete(logical()), integer())
  expect_identical(vec_locate_complete(data.frame()), integer())
})

# vec_detect_complete ----------------------------------------------------------

test_that("works with size zero input", {
  expect_identical(vec_detect_complete(integer()), logical())
  expect_identical(vec_detect_complete(data.frame()), logical())
})

test_that("NA_real_ and NaN are both missing", {
  expect_identical(vec_detect_complete(c(NA_real_, NaN)), c(FALSE, FALSE))
})

test_that("works rowwise", {
  df <- data_frame(x = c(NA, 1, NA, 2), y = c(NA, NA, 2, 3))
  expect <- c(FALSE, FALSE, FALSE, TRUE)
  expect_identical(vec_detect_complete(df), expect)

  df <- data_frame(x = c(1, 1), y = c(2, 2), z = c(1, NA))
  expect <- c(TRUE, FALSE)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("works with data frames with rows but no columns", {
  expect_identical(vec_detect_complete(new_data_frame(n = 5L)), rep(TRUE, 5))
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, NA, 2, 2), b = c(1, 2, NA, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect <- c(TRUE, FALSE, FALSE, TRUE)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("works with various types", {
  expect <- c(TRUE, TRUE, FALSE, TRUE, FALSE)

  expect_identical(vec_detect_complete(c(TRUE, TRUE, NA, FALSE, NA)), expect)
  expect_identical(vec_detect_complete(c(1L, 1L, NA, 2L, NA)), expect)
  expect_identical(vec_detect_complete(c(1, 1, NA, 2, NA)), expect)
  expect_identical(vec_detect_complete(complex(real = c(1, 1, NA, 2, 2), imaginary = c(1, 1, 2, 2, NA))), expect)
  expect_identical(vec_detect_complete(c("a", "a", NA, "b", NA)), expect)
  expect_identical(vec_detect_complete(list(1, 1, NULL, 2, NULL)), expect)

  # No missing raw value
  expect_identical(vec_detect_complete(as.raw(c(1, 1, 2, 2, 3))), rep(TRUE, 5))
})

test_that("takes the equality proxy", {
  x <- as.POSIXlt(c(NA, 0), origin = "1970-01-01")
  df <- data_frame(a = 1:2, x = x)

  expect <- c(FALSE, TRUE)

  expect_identical(vec_detect_complete(x), expect)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("columns with a data frame proxy are incomplete if any columns of the proxy are incomplete (#1404)", {
  df <- data_frame(
    x = c(NA, 0, 1, 2, 3),
    y = new_rcrd(list(a = c(1, 1, 1, NA, NA), b = c(2, 2, 2, 2, NA))),
    z = new_rcrd(list(a = c(1, 1, NA, 1, 1), b = c(2, 2, NA, NA, 1)))
  )

  expect_identical(vec_detect_complete(df), c(FALSE, TRUE, FALSE, FALSE, FALSE))
})

test_that("can have rcrd fields of all types", {
  make_rcrd <- function(x) {
    new_rcrd(list(x = x))
  }

  expect <- c(TRUE, TRUE, FALSE, TRUE, FALSE)

  expect_identical(vec_detect_complete(make_rcrd(c(TRUE, TRUE, NA, FALSE, NA))), expect)
  expect_identical(vec_detect_complete(make_rcrd(c(1L, 1L, NA, 2L, NA))), expect)
  expect_identical(vec_detect_complete(make_rcrd(c(1, 1, NA, 2, NA))), expect)
  expect_identical(vec_detect_complete(make_rcrd(complex(real = c(1, 1, NA, 2, 2), imaginary = c(1, 1, 2, 2, NA)))), expect)
  expect_identical(vec_detect_complete(make_rcrd(c("a", "a", NA, "b", NA))), expect)
  expect_identical(vec_detect_complete(make_rcrd(list(1, 1, NULL, 2, NULL))), expect)

  # No missing raw value
  expect_identical(vec_detect_complete(make_rcrd(as.raw(c(1, 1, 2, 2, 3)))), rep(TRUE, 5))
})

test_that("works with arrays", {
  x <- array(c(1, 2, 3, NA), c(2, 2))
  y <- array(c(1:3, NA, 5:8), c(2, 2, 2))

  expect_identical(vec_detect_complete(x), c(TRUE, FALSE))
  expect_identical(vec_detect_complete(y), c(TRUE, FALSE))
})

# vec_slice_complete -----------------------------------------------------------

test_that("can slice complete", {
  df <- data_frame(x = c(1, NA, 3), y = c(1, 2, NA))
  expect_identical(vec_slice_complete(df), vec_slice(df, 1))
})

test_that("vec_slice_complete() works with size 0 input", {
  expect_identical(vec_slice_complete(integer()), integer())
  expect_identical(vec_slice_complete(data.frame()), data.frame())
})

# vec_locate_complete ----------------------------------------------------------

test_that("vec_locate_complete() can locate complete", {
  df <- data_frame(x = c(1, NA, 3), y = c(1, 2, NA))
  expect_identical(vec_locate_complete(df), 1L)
})

test_that("vec_locate_complete() works with size 0 input", {
  expect_identical(vec_locate_complete(logical()), integer())
  expect_identical(vec_locate_complete(data.frame()), integer())
})

# vec_detect_complete ----------------------------------------------------------

test_that("works with size zero input", {
  expect_identical(vec_detect_complete(integer()), logical())
  expect_identical(vec_detect_complete(data.frame()), logical())
})

test_that("NA_real_ and NaN are both missing", {
  expect_identical(vec_detect_complete(c(NA_real_, NaN)), c(FALSE, FALSE))
})

test_that("works rowwise", {
  df <- data_frame(x = c(NA, 1, NA, 2), y = c(NA, NA, 2, 3))
  expect <- c(FALSE, FALSE, FALSE, TRUE)
  expect_identical(vec_detect_complete(df), expect)

  df <- data_frame(x = c(1, 1), y = c(2, 2), z = c(1, NA))
  expect <- c(TRUE, FALSE)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("works with data frames with rows but no columns", {
  expect_identical(vec_detect_complete(new_data_frame(n = 5L)), rep(TRUE, 5))
})

test_that("works with data frame columns", {
  col <- data_frame(a = c(1, NA, 2, 2), b = c(1, 2, NA, 3))
  df <- data_frame(x = rep(1, 4), y = col)
  expect <- c(TRUE, FALSE, FALSE, TRUE)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("works with various types", {
  expect <- c(TRUE, TRUE, FALSE, TRUE, FALSE)

  expect_identical(vec_detect_complete(c(TRUE, TRUE, NA, FALSE, NA)), expect)
  expect_identical(vec_detect_complete(c(1L, 1L, NA, 2L, NA)), expect)
  expect_identical(vec_detect_complete(c(1, 1, NA, 2, NA)), expect)
  expect_identical(
    vec_detect_complete(complex(
      real = c(1, 1, NA, 2, 2),
      imaginary = c(1, 1, 2, 2, NA)
    )),
    expect
  )
  expect_identical(vec_detect_complete(c("a", "a", NA, "b", NA)), expect)
  expect_identical(vec_detect_complete(list(1, 1, NULL, 2, NULL)), expect)

  # No missing raw value
  expect_identical(vec_detect_complete(as.raw(c(1, 1, 2, 2, 3))), rep(TRUE, 5))
})

test_that("takes the equality proxy", {
  x <- as.POSIXlt(c(NA, 0), origin = "1970-01-01")
  df <- data_frame(a = 1:2, x = x)

  expect <- c(FALSE, TRUE)

  expect_identical(vec_detect_complete(x), expect)
  expect_identical(vec_detect_complete(df), expect)
})

test_that("columns with a data frame proxy are incomplete if any columns of the proxy are incomplete (#1404)", {
  df <- data_frame(
    x = c(NA, 0, 1, 2, 3),
    y = new_rcrd(list(a = c(1, 1, 1, NA, NA), b = c(2, 2, 2, 2, NA))),
    z = new_rcrd(list(a = c(1, 1, NA, 1, 1), b = c(2, 2, NA, NA, 1)))
  )

  expect_identical(vec_detect_complete(df), c(FALSE, TRUE, FALSE, FALSE, FALSE))
})

test_that("can have rcrd fields of all types", {
  make_rcrd <- function(x) {
    new_rcrd(list(x = x))
  }

  expect <- c(TRUE, TRUE, FALSE, TRUE, FALSE)

  expect_identical(
    vec_detect_complete(make_rcrd(c(TRUE, TRUE, NA, FALSE, NA))),
    expect
  )
  expect_identical(
    vec_detect_complete(make_rcrd(c(1L, 1L, NA, 2L, NA))),
    expect
  )
  expect_identical(vec_detect_complete(make_rcrd(c(1, 1, NA, 2, NA))), expect)
  expect_identical(
    vec_detect_complete(make_rcrd(complex(
      real = c(1, 1, NA, 2, 2),
      imaginary = c(1, 1, 2, 2, NA)
    ))),
    expect
  )
  expect_identical(
    vec_detect_complete(make_rcrd(c("a", "a", NA, "b", NA))),
    expect
  )
  expect_identical(
    vec_detect_complete(make_rcrd(list(1, 1, NULL, 2, NULL))),
    expect
  )

  # No missing raw value
  expect_identical(
    vec_detect_complete(make_rcrd(as.raw(c(1, 1, 2, 2, 3)))),
    rep(TRUE, 5)
  )
})

test_that("works with arrays", {
  x <- array(c(1, 2, 3, NA), c(2, 2))
  y <- array(c(1:3, NA, 5:8), c(2, 2, 2))

  expect_identical(vec_detect_complete(x), c(TRUE, FALSE))
  expect_identical(vec_detect_complete(y), c(TRUE, FALSE))
})

test_that("works with `NULL`", {
  # Consistent with `vec_detect_missing()`
  expect_identical(vec_detect_complete(NULL), logical())
})

test_that("catches `NULL` data frame columns", {
  df <- new_data_frame(list(x = integer(), y = NULL), n = 0L)

  expect_snapshot(error = TRUE, {
    vec_detect_complete(df)
  })
})

test_that("catches scalar objects", {
  expect_snapshot(error = TRUE, {
    vec_detect_complete(lm(1 ~ 1))
  })
})

# Imported at load-time in `sf_env`
st_crs = function(...) stop_sf()
st_precision = function(...) stop_sf()
st_as_sf = function(...) stop_sf()
stop_sf = function() abort("Internal error: Failed sf import.")

sf_deps = c(
  "st_crs",
  "st_precision",
  "st_as_sf"
)
sf_env = env()


# sf namespace
local(envir = sf_env, {
  # Registered at load-time (same for all other methods)
  vec_proxy_sf = function(x, ...) {
    x
  }

  vec_restore_sf = function(x, to, ...) {
    sfc_name = attr(to, "sf_column")
    crs = st_crs(to)
    prec = st_precision(to)

    st_as_sf(
      x,
      sf_column_name = sfc_name,
      crs = crs,
      precision = prec,
      stringsAsFactors = FALSE
    )
  }

  sf_ptype2 = function(x, y, ...) {
    data = vctrs::df_ptype2(x, y, ...)

    # Workaround for `c()` fallback sentinels. Must be fixed before
    # moving the methods downstream.
    opts <- match_fallback_opts(...)
    if (identical(opts$s3_fallback, S3_FALLBACK_true)) {
      return(data)
    }

    x_sf <- inherits(x, "sf")
    y_sf <- inherits(y, "sf")

    if (x_sf && y_sf) {
      # Take active geometry from left-hand side
      sfc_name = attr(x, "sf_column")

      # CRS and precision must match
      crs = common_crs(x, y)
      prec = common_prec(x, y)
    } else if (x_sf) {
      sfc_name = attr(x, "sf_column")
      crs = st_crs(x)
      prec = st_precision(x)
    } else if (y_sf) {
      sfc_name = attr(y, "sf_column")
      crs = st_crs(y)
      prec = st_precision(y)
    } else {
      stop("Internal error: Expected at least one `sf` input.")
    }

    st_as_sf(
      data,
      sf_column_name = sfc_name,
      crs = crs,
      precision = prec,
      stringsAsFactors = FALSE
    )
  }

  vec_ptype2_sf_sf = function(x, y, ...) {
    sf_ptype2(x, y, ...)
  }
  vec_ptype2_sf_data.frame = function(x, y, ...) {
    sf_ptype2(x, y, ...)
  }
  vec_ptype2_data.frame_sf = function(x, y, ...) {
    sf_ptype2(x, y, ...)
  }

  # Maybe we should not have these methods, but they are currently
  # required to avoid the base-df fallback
  vec_ptype2_sf_tbl_df = function(x, y, ...) {
    new_data_frame(sf_ptype2(x, y, ...))
  }
  vec_ptype2_tbl_df_sf = function(x, y, ...) {
    new_data_frame(sf_ptype2(x, y, ...))
  }

  sf_cast = function(x, to, ...) {
    data = vctrs::df_cast(x, to, ...)

    # Workaround for `c()` fallback sentinels. Must be fixed before
    # moving the methods downstream.
    opts <- match_fallback_opts(...)
    if (identical(opts$s3_fallback, S3_FALLBACK_true)) {
      return(data)
    }

    sfc_name = attr(to, "sf_column")
    crs = st_crs(to)
    prec = st_precision(to)

    st_as_sf(
      data,
      sf_column_name = sfc_name,
      crs = crs,
      precision = prec,
      stringsAsFactors = FALSE
    )
  }

  vec_cast_sf_sf = function(x, to, ...) {
    sf_cast(x, to, ...)
  }
  vec_cast_sf_data.frame = function(x, to, ...) {
    sf_cast(x, to, ...)
  }
  vec_cast_data.frame_sf = function(x, to, ...) {
    df_cast(x, to, ...)
  }

  vec_proxy_order_sfc <- function(x, ...) {
    # These are list columns, so they need to use the order-by-appearance proxy
    # that is defined by `vec_proxy_order.list()`
    x <- unstructure(x)
    vec_proxy_order(x)
  }

  # take conservative approach of requiring equal CRS and precision
  common_crs = function(x, y) {
    lhs = st_crs(x)
    rhs = st_crs(y)

    if (lhs != rhs) {
      stop("coordinate reference systems not equal: use st_transform() first?")
    }

    lhs
  }
  common_prec = function(x, y) {
    lhs = st_precision(x)
    rhs = st_precision(y)

    if (lhs != rhs) {
      stop("precisions not equal")
    }

    lhs
  }
}) # local(envir = sf_env)

env_bind(ns_env("vctrs"), !!!as.list(sf_env))

#' Cast a vector to a specified type
#'
#' @description
#'
#' `vec_cast()` provides directional conversions from one type of
#' vector to another. Along with [vec_ptype2()], this generic forms
#' the foundation of type coercions in vctrs.
#'
#' @includeRmd man/faq/developer/links-coercion.Rmd
#'
#' @inheritParams rlang::args_error_context
#' @param x Vectors to cast.
#' @param ... For `vec_cast_common()`, vectors to cast. For
#'   `vec_cast()`, `vec_cast_default()`, and `vec_restore()`, these
#'   dots are only for future extensions and should be empty.
#' @param to,.to Type to cast to. If `NULL`, `x` will be returned as is.
#' @param x_arg Argument name for `x`, used in error messages to
#'   inform the user about the locations of incompatible types
#'   (see [stop_incompatible_type()]).
#' @param to_arg Argument name `to` used in error messages to
#'   inform the user about the locations of incompatible types
#'   (see [stop_incompatible_type()]).
#' @return A vector the same length as `x` with the same type as `to`,
#'   or an error if the cast is not possible. An error is generated if
#'   information is lost when casting between compatible types (i.e. when
#'   there is no 1-to-1 mapping for a specific value).
#'
#' @section Dependencies of `vec_cast_common()`:
#'
#' ## vctrs dependencies
#'
#' - [vec_ptype2()]
#' - [vec_cast()]
#'
#'
#' ## base dependencies
#'
#' Some functions enable a base-class fallback for
#' `vec_cast_common()`. In that case the inputs are deemed compatible
#' when they have the same [base type][base::typeof] and inherit from
#' the same base class.
#'
#' @seealso Call [stop_incompatible_cast()] when you determine from the
#' attributes that an input can't be cast to the target type.
#' @export
#' @examples
#' # x is a double, but no information is lost
#' vec_cast(1, integer())
#'
#' # When information is lost the cast fails
#' try(vec_cast(c(1, 1.5), integer()))
#' try(vec_cast(c(1, 2), logical()))
#'
#' # You can suppress this error and get the partial results
#' allow_lossy_cast(vec_cast(c(1, 1.5), integer()))
#' allow_lossy_cast(vec_cast(c(1, 2), logical()))
#'
#' # By default this suppress all lossy cast errors without
#' # distinction, but you can be specific about what cast is allowed
#' # by supplying prototypes
#' allow_lossy_cast(vec_cast(c(1, 1.5), integer()), to_ptype = integer())
#' try(allow_lossy_cast(vec_cast(c(1, 2), logical()), to_ptype = integer()))
#'
#' # No sensible coercion is possible so an error is generated
#' try(vec_cast(1.5, factor("a")))
#'
#' # Cast to common type
#' vec_cast_common(factor("a"), factor(c("a", "b")))
vec_cast <- function(
  x,
  to,
  ...,
  x_arg = caller_arg(x),
  to_arg = "",
  call = caller_env()
) {
  if (!missing(...)) {
    check_ptype2_dots_empty(...)
  }
  return(.Call(ffi_cast, x, to, environment()))
  UseMethod("vec_cast", to)
}
vec_cast_dispatch <- function(x, to, ..., x_arg = "", to_arg = "") {
  UseMethod("vec_cast", to)
}

vec_cast_dispatch_native <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  .Call(
    ffi_cast_dispatch_native,
    x,
    to,
    match_fallback_opts(...),
    x_arg,
    to_arg,
    environment()
  )
}

#' @export
#' @rdname vec_cast
vec_cast_common <- function(..., .to = NULL, .arg = "", .call = caller_env()) {
  .External2(ffi_cast_common, list2(...), .to)
}
vec_cast_common_opts <- function(
  ...,
  .to = NULL,
  .opts = fallback_opts(),
  .arg = "",
  .call = caller_env()
) {
  .External2(ffi_cast_common_opts, list2(...), .to, .opts)
}
vec_cast_common_params <- function(
  ...,
  .to = NULL,
  .s3_fallback = NULL,
  .arg = "",
  .call = caller_env()
) {
  opts <- fallback_opts(
    s3_fallback = .s3_fallback
  )
  vec_cast_common_opts(
    ...,
    .to = .to,
    .opts = opts,
    .arg = .arg,
    .call = .call
  )
}
vec_cast_common_fallback <- function(
  ...,
  .to = NULL,
  .arg = "",
  .call = caller_env()
) {
  vec_cast_common_opts(
    ...,
    .to = .to,
    .opts = enabled_fallback_opts(),
    .arg = .arg,
    .call = .call
  )
}

#' @rdname vec_default_ptype2
#' @inheritParams vec_cast
#' @export
vec_default_cast <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  if (is_asis(x)) {
    return(vec_cast_from_asis(
      x,
      to,
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    ))
  }
  if (is_asis(to)) {
    return(vec_cast_to_asis(
      x,
      to,
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    ))
  }

  if (
    inherits(to, "vctrs_vctr") &&
      !inherits(to, c("vctrs_rcrd", "vctrs_list_of"))
  ) {
    return(vctr_cast(
      x,
      to,
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    ))
  }

  opts <- match_fallback_opts(...)

  if (is_common_class_fallback(to) && length(common_class_suffix(x, to))) {
    return(x)
  }

  # Data frames have special bare class and same type fallbacks
  if (is.data.frame(x) && is.data.frame(to)) {
    out <- df_cast_opts(
      x,
      to,
      ...,
      opts = opts,
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    )

    # Same-type fallback for data frames. If attributes of the empty
    # data frames are congruent, just reproduce these attributes. This
    # eschews any constraints on rows and cols that `[` and `[<-`
    # methods might have. If that is a problem, the class needs to
    # implement vctrs methods.
    if (identical(non_df_attrib(x), non_df_attrib(to))) {
      attributes(out) <- c(df_attrib(out), non_df_attrib(to))
      return(out)
    }

    # Bare-class fallback for data frames.
    # FIXME: Should we only allow it when target is a bare df?
    if (inherits(to, "tbl_df")) {
      out <- df_as_tibble(out)
    }
    return(out)
  }

  if (is_same_type(x, to)) {
    return(x)
  }

  withRestarts(
    stop_incompatible_cast(
      x,
      to,
      x_arg = x_arg,
      to_arg = to_arg,
      `vctrs:::from_dispatch` = match_from_dispatch(...),
      call = call
    ),
    vctrs_restart_cast = function(out) {
      out
    }
  )
}

is_bare_df <- function(x) {
  inherits_only(x, "data.frame") ||
    inherits_only(x, c("tbl_df", "tbl", "data.frame"))
}

test_that("Casting to named argument mentions 'match type <foo>'", {
  expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo", to_arg = "bar"))
  expect_snapshot(error = TRUE, vec_cast(1, "", x_arg = "foo"))
})

# vec_cast() ---------------------------------------------------------------

test_that("new classes are uncoercible by default", {
  x <- structure(1:10, class = "vctrs_nonexistant")
  expect_error(vec_cast(1, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(x, 1), class = "vctrs_error_incompatible_type")
})

test_that("casting requires vectors", {
  expect_error(vec_cast(NULL, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(NA, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(list(), quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), NA), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), list()), class = "vctrs_error_scalar_type")
  expect_error(vec_cast(quote(name), quote(name)), class = "vctrs_error_scalar_type")
})

test_that("casting between `NULL` and partial types is allowed", {
  expect_identical(vec_cast(NULL, partial_factor()), NULL)
  expect_identical(vec_cast(partial_factor(), NULL), partial_factor())
})

test_that("dimensionality matches output" ,{
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))

  x <- matrix(1, nrow = 2, ncol = 2)
  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_type")
})

test_that("empty input to vec_cast_common() returns list()", {
  expect_equal(vec_cast_common(), list())
  expect_equal(vec_cast_common(NULL, NULL), list(NULL, NULL))
})

test_that("identical structures can be cast to each other", {
  expect_identical(vec_cast(foobar("foo"), foobar("bar")), foobar("foo"))
})

test_that("cast common preserves names", {
  expect_identical(vec_cast_common(foo = 1, bar = 2L), list(foo = 1, bar = 2))
})

test_that("cast errors create helpful messages (#57, #225)", {
  # Lossy cast
  expect_snapshot(error = TRUE, vec_cast(1.5, 10L))

  # Incompatible cast
  expect_snapshot(error = TRUE, vec_cast(factor("foo"), 10))

  # Nested data frames - Lossy cast
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = 1.5))
    y <- tibble(a = tibble(b = 10L))
    vec_cast(x, y)
  })

  # Nested data frames - Incompatible cast
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = factor("foo")))
    y <- tibble(a = tibble(b = 10))
    vec_cast(x, y)
  })

  # Nested data frames - Common cast error
  expect_snapshot(error = TRUE, {
    x <- tibble(a = tibble(b = factor("foo")))
    y <- tibble(a = tibble(b = 10))
    vec_cast_common(x, y)
  })
})

test_that("unspecified can be cast to shaped vectors", {
  x <- matrix(letters[1:4], 2)
  expect_identical(vec_cast(NA, x), matrix(chr(NA, NA), 1))

  x <- foobar(c(1:4))
  dim(x) <- c(2, 2)
  out <- vec_cast(NA, x)

  exp <- foobar(int(c(NA, NA)))
  dim(exp) <- c(1, 2)
  expect_identical(out, exp)
})

test_that("vec_cast() falls back to base class even when casting to non-base type", {
  expect_equal(vec_cast(foobar(mtcars), mtcars), mtcars)
  expect_equal(vec_cast(mtcars, foobar(mtcars)), mtcars)
})

test_that("vec_cast() only attempts to fall back if `to` is a data frame (#1568)", {
  expect_snapshot({
    (expect_error(
      vec_cast(foobar(mtcars), 1),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_cast() evaluates x_arg and to_arg lazily", {
  expect_silent(vec_cast(TRUE, logical(), x_arg = print("oof")))
  expect_silent(vec_cast(TRUE, logical(), to_arg = print("oof")))
})


# Conditions --------------------------------------------------------------

test_that("can suppress cast errors selectively", {
  f <- function() vec_cast(factor("a"), to = factor("b"))
  expect_error(regexp = NA, allow_lossy_cast(f()))
  expect_error(regexp = NA, allow_lossy_cast(f(), x_ptype = factor("a")))
  expect_error(regexp = NA, allow_lossy_cast(f(), to_ptype = factor("b")))
  expect_error(regexp = NA, allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("b")))
  expect_error(allow_lossy_cast(f(), x_ptype = factor("c")), class = "vctrs_error_cast_lossy")
  expect_error(allow_lossy_cast(f(), x_ptype = factor("b"), to_ptype = factor("a")), class = "vctrs_error_cast_lossy")
  expect_error(allow_lossy_cast(f(), x_ptype = factor("a"), to_ptype = factor("c")), class = "vctrs_error_cast_lossy")
})

test_that("can signal deprecation warnings for lossy casts", {
  local_options(lifecycle_verbosity = "warning")

  lossy_cast <- function() {
    maybe_lossy_cast(
      TRUE,
      factor("foo"),
      factor("bar"),
      lossy = TRUE,
      .deprecation = TRUE,
      x_arg = "x",
      to_arg = "to"
    )
  }

  expect_snapshot({
    (expect_warning(expect_true(lossy_cast())))
  })
  expect_warning(regexp = NA, expect_true(allow_lossy_cast(lossy_cast())))
  expect_warning(regexp = NA, expect_true(allow_lossy_cast(lossy_cast(), factor("foo"), factor("bar"))))
  expect_warning(expect_true(allow_lossy_cast(lossy_cast(), factor("bar"), double())))
})


# vec_cast_common() -------------------------------------------------------

test_that("vec_ptype_common() optionally falls back to base class", {
  x <- foobar(NA, foo = 1)
  y <- foobaz(NA, bar = 2)

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  expect_error(
    vec_ptype_common_opts(x, y, .opts = full_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype_common_opts(x_df, y_df, .opts = full_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    vec_cast_common_opts(x, y, .opts = full_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast_common_opts(x_df, y_df, .opts = full_fallback_opts()),
    class = "vctrs_error_incompatible_type"
  )

  class(y) <- c("foo", class(x))
  y_df <- data_frame(x = y)

  common_sentinel <- vec_ptype_common_opts(x, y, .opts = full_fallback_opts())
  expect_true(is_common_class_fallback(common_sentinel))
  expect_identical(fallback_class(common_sentinel), "vctrs_foobar")

  common_sentinel <- vec_ptype_common_opts(x_df, y_df, .opts = full_fallback_opts())
  expect_true(is_common_class_fallback(common_sentinel$x))
  expect_identical(fallback_class(common_sentinel$x), "vctrs_foobar")

  common <- vec_cast_common_opts(x = x, y = y, .opts = full_fallback_opts())
  expect_identical(common, list(x = x, y = y))

  common <- vec_cast_common_opts(x = x_df, y = y_df, .opts = full_fallback_opts())
  expect_identical(common, list(x = x_df, y = y_df))
})

test_that("vec_ptype_common_fallback() collects common type", {
  x <- foobar(1, foo = 1, class = c("quux", "baz"))
  y <- foobar(2, bar = 2, class = "baz")

  x_df <- data_frame(x = x)
  y_df <- data_frame(x = y)

  out <- vec_ptype_common_fallback(x, y)
  expect_identical(typeof(out), "double")
  expect_true(is_common_class_fallback(out))
  expect_identical(fallback_class(out), c("baz", "vctrs_foobar"))

  out <- vec_ptype_common_fallback(x_df, y_df)
  expect_identical(typeof(out$x), "double")
  expect_true(is_common_class_fallback(out$x))
  expect_identical(fallback_class(out$x), c("baz", "vctrs_foobar"))

  # Different base types can't fall back to common class
  z <- foobar(3L, baz = 3)
  expect_error(
    vec_ptype_common_fallback(x, z),
    class = "vctrs_error_incompatible_type"
  )

  z_df <- data_frame(x = z)
  expect_error(
    vec_ptype_common_fallback(x_df, z_df),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("fallback sentinel is returned with unspecified inputs", {
  fallback <- vec_ptype_common_fallback(foobar(1), foobar(1))
  expect_identical(vec_ptype_common_fallback(NA, foobar(1)), fallback)
  expect_identical(vec_ptype_common_fallback(foobar(1), NA), fallback)
})

test_that("vec_ptype_common() supports subclasses of list", {
  x <- structure(list(1), class = c("vctrs_foo", "list"))
  y <- structure(list(2), class = c("bar", "vctrs_foo", "list"))

  expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type")

  out <- with_methods(
    c.vctrs_foo = function(...) quux(NextMethod()),
    vec_c(x, y)
  )
  expect_identical(out, quux(list(1, 2)))
})

test_that("vec_cast_common_fallback() works with tibbles", {
  x <- foobar("foo")
  df <- data_frame(x = x)
  tib <- tibble(x = x)

  exp <- list(tib, tib)

  expect_identical(vec_cast_common_fallback(tib, tib), exp)
  expect_identical(vec_cast_common_fallback(tib, df), exp)
  expect_identical(vec_cast_common_fallback(df, tib), exp)
})

test_that("can call `vec_cast()` from C (#1666)", {
  fn <- inject(function(x, i) .Call(!!ffi_exp_vec_cast, x, i))
  environment(fn) <- ns_env("utils")

  x <- array(1, dim = c(1, 1))
  y <- array(2, dim = c(2, 2))

  expect_equal(fn(x, y), vec_cast(x, y))
})

test_that("df-fallback for cast is not sensitive to attributes order", {
  x <- structure(
    list(col = ""),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame"),
    row.names = c(NA, -1L),
    foo = "foo",
    bar = "bar"
  )
  ptype <- structure(
    list(col = character(0)),
    foo = "foo",
    bar = "bar",
    row.names = integer(0),
    class = c("vctrs_foobar", "tbl_df", "tbl", "data.frame")
  )

  expect_identical(vec_cast(x, ptype), x)
})

test_that("bare-type fallback for df-cast works", {
  # NOTE: Not sure why this was necessary. The cubble and yamlet
  # packages fail without this.
  local_methods(
    c.vctrs_foobaz = function(...) quux(NextMethod())
  )

  df <- data_frame(x = 1, y = foobaz("foo"))
  gdf <- dplyr::new_grouped_df(
    df,
    data_frame(x = 1, .rows = list(1L)),
    class = "vctrs_foobar"
  )

  expect_error(vec_rbind(gdf, gdf), NA)
})

# Keep in sync with macros in `order.c`
GROUP_DATA_SIZE_DEFAULT <- 100000L
ORDER_INSERTION_BOUNDARY <- 128L
INT_ORDER_COUNTING_RANGE_BOUNDARY <- 100000L

# Force radix method for character comparisons
base_order <- function(x, na.last = TRUE, decreasing = FALSE) {
  if (is.data.frame(x)) {
    x <- unname(x)
  } else {
    x <- list(x)
  }

  args <- list(
    na.last = na.last,
    decreasing = decreasing,
    method = "radix"
  )

  args <- c(x, args)

  rlang::exec("order", !!!args)
}

#' vctr (vector) S3 class
#'
#' @description
#' This abstract class provides a set of useful default methods that makes it
#' considerably easier to get started with a new S3 vector class. See
#' `vignette("s3-vector")` to learn how to use it to create your own S3
#' vector classes.
#'
#' @details
#' List vctrs are special cases. When created through `new_vctr()`, the
#' resulting list vctr should always be recognized as a list by
#' `obj_is_list()`. Because of this, if `inherit_base_type` is `FALSE`
#' an error is thrown.
#'
#' @section Base methods:
#' The vctr class provides methods for many base generics using a smaller
#' set of generics defined by this package. Generally, you should think
#' carefully before overriding any of the methods that vctrs implements for
#' you as they've been carefully planned to be internally consistent.
#'
#' * `[[` and `[` use `NextMethod()` dispatch to the underlying base function,
#'    then restore attributes with `vec_restore()`.
#'    `rep()` and `length<-` work similarly.
#'
#' * `[[<-` and `[<-` cast `value` to same type as `x`, then call
#'   `NextMethod()`.
#'
#' * `as.logical()`, `as.integer()`, `as.numeric()`, `as.character()`,
#'   `as.Date()` and `as.POSIXct()` methods call `vec_cast()`.
#'   The `as.list()` method calls `[[` repeatedly, and the `as.data.frame()`
#'   method uses a standard technique to wrap a vector in a data frame.
#'
#' * `as.factor()`, `as.ordered()` and `as.difftime()` are not generic functions
#'   in base R, but have been reimplemented as generics in the `generics`
#'   package. `vctrs` extends these and calls `vec_cast()`. To inherit this
#'   behaviour in a package, import and re-export the generic of interest
#'   from `generics`.
#'
#' * `==`, `!=`, `unique()`, `anyDuplicated()`, and `is.na()` use
#'   [vec_proxy()].
#'
#' * `<`, `<=`, `>=`, `>`, `min()`, `max()`, `range()`, `median()`,
#'   `quantile()`, and `xtfrm()` methods use [vec_proxy_compare()].
#'
#' * `+`, `-`, `/`, `*`, `^`, `%%`, `%/%`, `!`, `&`, and `|` operators
#'   use [vec_arith()].
#'
#' * Mathematical operations including the Summary group generics (`prod()`,
#'   `sum()`, `any()`, `all()`), the Math group generics (`abs()`, `sign()`,
#'   etc), `mean()`, `is.nan()`, `is.finite()`, and `is.infinite()`
#'   use [vec_math()].
#'
#' * `dims()`, `dims<-`, `dimnames()`, `dimnames<-`, `levels()`, and
#'   `levels<-` methods throw errors.
#'
#' @param .data Foundation of class. Must be a vector
#' @param ... Name-value pairs defining attributes
#' @param class Name of subclass.
#' @param inherit_base_type `r lifecycle::badge("experimental")`
#'   A single logical, or `NULL`. Does this class extend the base type of
#'   `.data`? i.e. does the resulting object extend the behaviour of the
#'   underlying type? Defaults to `FALSE` for all types except lists, which
#'   are required to inherit from the base type.
#' @export
#' @keywords internal
#' @aliases vctr
new_vctr <- function(
  .data,
  ...,
  class = character(),
  inherit_base_type = NULL
) {
  if (!is_vector(.data)) {
    abort("`.data` must be a vector type.")
  }

  if (is_list(.data)) {
    if (is.data.frame(.data)) {
      abort("`.data` can't be a data frame.")
    }

    if (is.null(inherit_base_type)) {
      inherit_base_type <- TRUE
    } else if (is_false(inherit_base_type)) {
      abort("List `.data` must inherit from the base type.")
    }
  }

  # Default to `FALSE` in all cases except lists
  if (is.null(inherit_base_type)) {
    inherit_base_type <- FALSE
  }

  names <- names(.data)
  names <- names_repair_missing(names)

  class <- c(class, "vctrs_vctr", if (inherit_base_type) typeof(.data))
  attrib <- list(names = names, ..., class = class)

  vec_set_attributes(.data, attrib)
}

names_repair_missing <- function(x) {
  if (is.null(x)) {
    return(x)
  }

  if (vec_any_missing(x)) {
    # We never want to allow `NA_character_` names to slip through, but
    # erroring on them has caused issues. Instead, we repair them to the
    # empty string (#784).
    missing <- vec_detect_missing(x)
    x <- vec_assign(x, missing, "")
  }

  x
}

#' @export
vec_proxy.vctrs_vctr <- function(x, ...) {
  if (is_list(x)) {
    unclass(x)
  } else {
    x
  }
}

#' @export
vec_restore.vctrs_vctr <- function(x, to, ..., i = NULL) {
  if (typeof(x) != typeof(to)) {
    stop_incompatible_cast(x, to, x_arg = "", to_arg = "")
  }
  NextMethod()
}

#' @method vec_cast vctrs_vctr
#' @export
vec_cast.vctrs_vctr <- function(x, to, ...) {
  UseMethod("vec_cast.vctrs_vctr")
}

vctr_cast <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  # These are not strictly necessary, but make bootstrapping a new class
  # a bit simpler
  if (is.object(x)) {
    if (is_same_type(x, to)) {
      x
    } else {
      stop_incompatible_cast(
        x,
        to,
        x_arg = x_arg,
        to_arg = to_arg,
        call = call
      )
    }
  } else {
    # FIXME: `vec_restore()` should only be called on proxies
    vec_restore(x, to)
  }
}

#' @export
c.vctrs_vctr <- function(..., recursive = FALSE, use.names = TRUE) {
  if (!is_false(recursive)) {
    abort("`recursive` must be `FALSE` when concatenating vctrs classes.")
  }
  if (!is_true(use.names)) {
    abort("`use.names` must be `TRUE` when concatenating vctrs classes.")
  }
  vec_c(...)
}

# Printing ----------------------------------------------------------------

#' @export
print.vctrs_vctr <- function(x, ...) {
  obj_print(x, ...)
  invisible(x)
}

#' @export
str.vctrs_vctr <- function(object, ...) {
  obj_str(object, ...)
}

#' @export
format.vctrs_vctr <- function(x, ...) {
  format(vec_data(x), ...)
}

# Subsetting --------------------------------------------------------------

#' @export
`[.vctrs_vctr` <- function(x, i, ...) {
  vec_index(x, i, ...)
}

#' @export
`[[.vctrs_vctr` <- function(x, i, ...) {
  if (is.list(x)) {
    NextMethod()
  } else {
    vec_restore(NextMethod(), x)
  }
}

#' @export
`$.vctrs_vctr` <- function(x, i) {
  if (is.list(x)) {
    NextMethod()
  } else {
    vec_restore(NextMethod(), x)
  }
}

#' @export
rep.vctrs_vctr <- function(x, ...) {
  vec_restore(NextMethod(), x)
}

#' @export
`length<-.vctrs_vctr` <- function(x, value) {
  vec_restore(NextMethod(), x)
}

#' @export
diff.vctrs_vctr <- function(x, lag = 1L, differences = 1L, ...) {
  stopifnot(length(lag) == 1L, lag >= 1L)
  stopifnot(length(differences) == 1L, differences >= 1L)

  n <- vec_size(x)
  if (lag * differences >= n) {
    return(vec_slice(x, 0L))
  }

  out <- x
  for (i in seq_len(differences)) {
    n <- vec_size(out)
    lhs <- (1L + lag):n
    rhs <- 1L:(n - lag)

    out <- vec_slice(out, lhs) - vec_slice(out, rhs)
  }

  out
}


# Modification -------------------------------------------------------------

#' @export
`[[<-.vctrs_vctr` <- function(x, ..., value) {
  if (!is.list(x)) {
    value <- vec_cast(value, x)
  }
  NextMethod()
}

#' @export
`$<-.vctrs_vctr` <- function(x, i, value) {
  if (is.list(x)) {
    NextMethod()
  } else {
    # Default behaviour is to cast LHS to a list
    abort("$ operator is invalid for atomic vectors.")
  }
}

#' @export
`[<-.vctrs_vctr` <- function(x, i, value) {
  value <- vec_cast(value, x)
  NextMethod()
}

#' @export
`names<-.vctrs_vctr` <- function(x, value) {
  if (length(value) != 0 && length(value) != length(x)) {
    abort("`names()` must be the same length as x.")
  }

  value <- names_repair_missing(value)

  NextMethod()
}
# Coercion ----------------------------------------------------------------

#' @export
as.logical.vctrs_vctr <- function(x, ...) {
  vec_cast(x, logical())
}

#' @export
as.integer.vctrs_vctr <- function(x, ...) {
  vec_cast(x, integer())
}

#' @export
as.double.vctrs_vctr <- function(x, ...) {
  vec_cast(x, double())
}

#' @export
as.character.vctrs_vctr <- function(x, ...) {
  vec_cast(x, character())
}

#' @export
as.list.vctrs_vctr <- function(x, ...) {
  out <- vec_chop(x)

  if (obj_is_list(x)) {
    out <- lapply(out, `[[`, 1)
  }

  out
}

#' @export
as.Date.vctrs_vctr <- function(x, ...) {
  vec_cast(x, new_date())
}

#' @export
as.POSIXct.vctrs_vctr <- function(x, tz = "", ...) {
  vec_cast(x, new_datetime(tzone = tz))
}

#' @export
as.POSIXlt.vctrs_vctr <- function(x, tz = "", ...) {
  to <- as.POSIXlt(new_datetime(), tz = tz)
  vec_cast(x, to)
}

# Work around inconsistencies in as.data.frame()
as.data.frame2 <- function(x) {
  # Unclass to avoid dispatching on `as.data.frame()` methods that break size
  # invariants, like `as.data.frame.table()` (#913). This also prevents infinite
  # recursion with shaped vctrs in `as.data.frame.vctrs_vctr()`.
  x <- unclass(x)

  out <- as.data.frame(x)

  if (vec_dim_n(x) == 1) {
    # 1D arrays are not stripped from their dimensions
    out[[1]] <- as.vector(out[[1]])

    # 1D arrays are auto-labelled with substitute()
    names(out) <- "V1"
  }

  out
}

#' @export
as.data.frame.vctrs_vctr <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  nm = paste(deparse(substitute(x), width.cutoff = 500L), collapse = " ")
) {
  force(nm)

  if (has_dim(x)) {
    return(as.data.frame2(x))
  }

  cols <- list(x)
  if (!optional) {
    names(cols) <- nm
  }

  new_data_frame(cols, n = vec_size(x))
}

# Dynamically registered in .onLoad()
as.factor.vctrs_vctr <- function(x, levels = character(), ...) {
  vec_cast(x, new_factor(levels = levels))
}

# Dynamically registered in .onLoad()
as.ordered.vctrs_vctr <- function(x, levels = character(), ...) {
  vec_cast(x, new_ordered(levels = levels))
}

# Dynamically registered in .onLoad()
as.difftime.vctrs_vctr <- function(x, units = "secs", ...) {
  vec_cast(x, new_duration(units = units))
}

# Equality ----------------------------------------------------------------

#' @export
`==.vctrs_vctr` <- function(e1, e2) {
  vec_equal(e1, e2)
}

#' @export
`!=.vctrs_vctr` <- function(e1, e2) {
  !vec_equal(e1, e2)
}

#' @export
is.na.vctrs_vctr <- function(x) {
  vec_detect_missing(x)
}

#' @importFrom stats na.fail
#' @export
na.fail.vctrs_vctr <- function(object, ...) {
  if (vec_any_missing(object)) {
    # Return the same error as `na.fail.default()`
    abort("missing values in object")
  }

  object
}

#' @importFrom stats na.omit
#' @export
na.omit.vctrs_vctr <- function(object, ...) {
  na_remove(object, "omit")
}

#' @importFrom stats na.exclude
#' @export
na.exclude.vctrs_vctr <- function(object, ...) {
  na_remove(object, "exclude")
}

na_remove <- function(x, type) {
  # The only difference between `na.omit()` and `na.exclude()` is the class
  # of the `na.action` attribute

  if (!vec_any_missing(x)) {
    return(x)
  }

  # `na.omit/exclude()` attach the locations of the omitted values to the result
  missing <- vec_detect_missing(x)
  loc <- which(missing)

  names <- vec_names(x)
  if (!is_null(names)) {
    # `na.omit/exclude()` retain the original names, if applicable
    names <- vec_slice(names, loc)
    loc <- vec_set_names(loc, names)
  }

  attr(loc, "class") <- type

  out <- vec_slice(x, !missing)
  attr(out, "na.action") <- loc
  out
}

#' @export
anyNA.vctrs_vctr <- function(x, recursive = FALSE) {
  if (recursive && obj_is_list(x)) {
    any(map_lgl(x, anyNA, recursive = recursive))
  } else {
    any(is.na(x))
  }
}

#' @export
unique.vctrs_vctr <- function(x, incomparables = FALSE, ...) {
  vec_unique(x)
}

#' @export
duplicated.vctrs_vctr <- function(x, incomparables = FALSE, ...) {
  vec_duplicate_id(x) != seq_along(x)
}

#' @export
anyDuplicated.vctrs_vctr <- function(x, incomparables = FALSE, ...) {
  vec_duplicate_any(x)
}

# Comparison ----------------------------------------------------------------

#' @export
`<=.vctrs_vctr` <- function(e1, e2) {
  vec_compare(e1, e2) <= 0
}

#' @export
`<.vctrs_vctr` <- function(e1, e2) {
  vec_compare(e1, e2) < 0
}

#' @export
`>=.vctrs_vctr` <- function(e1, e2) {
  vec_compare(e1, e2) >= 0
}

#' @export
`>.vctrs_vctr` <- function(e1, e2) {
  vec_compare(e1, e2) > 0
}

#' @export
xtfrm.vctrs_vctr <- function(x) {
  proxy <- vec_proxy_order(x)
  type <- typeof(proxy)

  if (type == "logical") {
    proxy <- unstructure(proxy)
    proxy <- as.integer(proxy)
    return(proxy)
  }

  if (type %in% c("integer", "double")) {
    proxy <- unstructure(proxy)
    return(proxy)
  }

  vec_rank(proxy, ties = "dense", incomplete = "na")
}

#' @importFrom stats median
#' @export
median.vctrs_vctr <- function(x, ..., na.rm = FALSE) {
  # nocov start
  stop_unimplemented(x, "median")
  # nocov end
}

#' @importFrom stats quantile
#' @export
quantile.vctrs_vctr <- function(x, ..., type = 1, na.rm = FALSE) {
  # nocov start
  stop_unimplemented(x, "quantile")
  # nocov end
}

vec_cast_or_na <- function(x, to, ...) {
  tryCatch(
    vctrs_error_incompatible_type = function(...) vec_init(to, length(x)),
    vec_cast(x, to)
  )
}

#' @export
min.vctrs_vctr <- function(x, ..., na.rm = FALSE) {
  if (vec_is_empty(x)) {
    return(vec_cast_or_na(Inf, x))
  }

  # TODO: implement to do vec_arg_min()
  rank <- xtfrm(x)

  if (isTRUE(na.rm)) {
    idx <- which.min(rank)
    if (vec_is_empty(idx)) {
      return(vec_cast_or_na(Inf, x))
    }
  } else {
    idx <- which(vec_equal(rank, min(rank), na_equal = TRUE))
  }

  x[[idx[[1]]]]
}

#' @export
max.vctrs_vctr <- function(x, ..., na.rm = FALSE) {
  if (vec_is_empty(x)) {
    return(vec_cast_or_na(-Inf, x))
  }

  # TODO: implement to do vec_arg_max()
  rank <- xtfrm(x)

  if (isTRUE(na.rm)) {
    idx <- which.max(rank)
    if (vec_is_empty(idx)) {
      return(vec_cast_or_na(-Inf, x))
    }
  } else {
    idx <- which(vec_equal(rank, max(rank), na_equal = TRUE))
  }

  x[[idx[[1]]]]
}

#' @export
range.vctrs_vctr <- function(x, ..., na.rm = FALSE) {
  if (vec_is_empty(x)) {
    return(vec_cast_or_na(c(Inf, -Inf), x))
  }

  # Inline `min()` / `max()` to only call `xtfrm()` once
  rank <- xtfrm(x)

  if (isTRUE(na.rm)) {
    idx_min <- which.min(rank)
    idx_max <- which.max(rank)
    if (vec_is_empty(idx_min) && vec_is_empty(idx_max)) {
      return(vec_cast_or_na(c(Inf, -Inf), x))
    }
  } else {
    idx_min <- which(vec_equal(rank, min(rank), na_equal = TRUE))
    idx_max <- which(vec_equal(rank, max(rank), na_equal = TRUE))
  }

  c(x[[idx_min[[1]]]], x[[idx_max[[1]]]])
}

# Numeric -----------------------------------------------------------------

#' @export
Math.vctrs_vctr <- function(x, ...) {
  vec_math(.Generic, x, ...)
}

#' @export
Summary.vctrs_vctr <- function(..., na.rm = FALSE) {
  vec_math(.Generic, vec_c(...), na.rm = na.rm)
}

#' @export
mean.vctrs_vctr <- function(x, ..., na.rm = FALSE) {
  vec_math("mean", x, na.rm = na.rm)
}

#' @export
is.finite.vctrs_vctr <- function(x) {
  vec_math("is.finite", x)
}

#' @export
is.infinite.vctrs_vctr <- function(x) {
  vec_math("is.infinite", x)
}

#' @export
is.nan.vctrs_vctr <- function(x) {
  vec_math("is.nan", x)
}

# Arithmetic --------------------------------------------------------------

#' @export
`+.vctrs_vctr` <- function(e1, e2) {
  if (missing(e2)) {
    vec_arith("+", e1, MISSING())
  } else {
    vec_arith("+", e1, e2)
  }
}

#' @export
`-.vctrs_vctr` <- function(e1, e2) {
  if (missing(e2)) {
    vec_arith("-", e1, MISSING())
  } else {
    vec_arith("-", e1, e2)
  }
}

#' @export
`*.vctrs_vctr` <- function(e1, e2) {
  vec_arith("*", e1, e2)
}

#' @export
`/.vctrs_vctr` <- function(e1, e2) {
  vec_arith("/", e1, e2)
}

#' @export
`^.vctrs_vctr` <- function(e1, e2) {
  vec_arith("^", e1, e2)
}

#' @export
`%%.vctrs_vctr` <- function(e1, e2) {
  vec_arith("%%", e1, e2)
}

#' @export
`%/%.vctrs_vctr` <- function(e1, e2) {
  vec_arith("%/%", e1, e2)
}

#' @export
`!.vctrs_vctr` <- function(x) {
  vec_arith("!", x, MISSING())
}

#' @export
`&.vctrs_vctr` <- function(e1, e2) {
  vec_arith("&", e1, e2)
}

#' @export
`|.vctrs_vctr` <- function(e1, e2) {
  vec_arith("|", e1, e2)
}

# Unimplemented ------------------------------------------------------------

#' @export
summary.vctrs_vctr <- function(object, ...) {
  # nocov start
  stop_unimplemented(object, "summary")
  # nocov end
}

# Unsupported --------------------------------------------------------------

#' @export
`dim<-.vctrs_vctr` <- function(x, value) {
  stop_unsupported(x, "dim<-")
}

#' @export
`dimnames<-.vctrs_vctr` <- function(x, value) {
  stop_unsupported(x, "dimnames<-")
}

#' @export
levels.vctrs_vctr <- function(x) {
  NULL
}

#' @export
`levels<-.vctrs_vctr` <- function(x, value) {
  stop_unsupported(x, "levels<-")
}

#' @export
`t.vctrs_vctr` <- function(x) {
  stop_unsupported(x, "t")
}

#' @export
`is.na<-.vctrs_vctr` <- function(x, value) {
  vec_assign(x, value, vec_init(x))
}

# Helpers -----------------------------------------------------------------

# This simple class is used for testing as defining methods inside
# a test does not work (because the lexical scope is lost)
# nocov start
new_hidden <- function(x = double()) {
  stopifnot(is.numeric(x))
  new_vctr(vec_cast(x, double()), class = "hidden", inherit_base_type = FALSE)
}

#' @export
format.hidden <- function(x, ...) rep("xxx", length(x))

local_hidden <- function(frame = caller_env()) {
  local_bindings(
    .env = global_env(),
    .frame = frame,
    vec_ptype2.hidden.hidden = function(x, y, ...) new_hidden(),
    vec_ptype2.hidden.double = function(x, y, ...) new_hidden(),
    vec_ptype2.double.hidden = function(x, y, ...) new_hidden(),
    vec_ptype2.hidden.logical = function(x, y, ...) new_hidden(),
    vec_ptype2.logical.hidden = function(x, y, ...) new_hidden(),

    vec_cast.hidden.hidden = function(x, to, ...) x,
    vec_cast.hidden.double = function(x, to, ...) new_hidden(vec_data(x)),
    vec_cast.double.hidden = function(x, to, ...) vec_data(x),
    vec_cast.hidden.logical = function(x, to, ...) new_hidden(as.double(x)),
    vec_cast.logical.hidden = function(x, to, ...) as.logical(vec_data(x))
  )
}

# nocov end

test_that("constructor sets attributes", {
  x <- new_vctr(1:4, class = "x", x = 1)
  expect_equal(x, structure(1:4, class = c("x", "vctrs_vctr"), x = 1))

  x <- new_vctr(1:4, class = "x", x = 1, inherit_base_type = TRUE)
  expect_equal(x, structure(1:4, class = c("x", "vctrs_vctr", "integer"), x = 1))
})

test_that(".data must be a vector", {
  expect_error(new_vctr(mean), "vector type")
})

test_that("attributes other than names are ignored", {
  out <- new_vctr(structure(1, a = 1))
  expect_null(attributes(out)$a)
})

test_that("default format method is internal", {
  x <- new_vctr(1, class = "x")
  expect_equal(format(x), format(x))
})

test_that("vctr class is proxied", {
  expect_identical(vec_proxy(new_vctr(1:3)), new_vctr(1:3))
  expect_identical(vec_proxy(new_vctr(as.list(1:3))), unclass(new_vctr(as.list(1:3))))
  expect_true(vec_is(new_vctr(as.list(1:3))))
})

test_that("Can opt out of base type", {
  x <- new_vctr(1, class = "x", inherit_base_type = FALSE)
  expect_s3_class(x, c("x", "vctrs_vctr"), exact = TRUE)
})

test_that("base type is always set for lists", {
  expect_s3_class(new_vctr(list()), "list")
})

test_that("cannot opt out of the base type with lists", {
  expect_error(new_vctr(list(), inherit_base_type = FALSE), "must inherit from the base type")
})

test_that("data frames are not allowed", {
  expect_error(new_vctr(mtcars), "can't be a data frame")
})

test_that("attributes must be named", {
  expect_error(vec_set_attributes(1, list(1)), "must be named")
  expect_error(vec_set_attributes(1, list(y = 1, 2)), "2 does not")
})

test_that("can strip all attributes without adding new ones", {
  expect_equal(vec_set_attributes(structure(1, a = 1), NULL), 1)
})

test_that("`c.vctrs_vctr()` checks for default method arguments (#791)", {
  expect_error(c(new_vctr(1), recursive = TRUE), "`recursive` must be")
  expect_error(c(new_vctr(1), use.names = FALSE), "`use.names` must be")
})


# Cast/restore ------------------------------------------------------------

test_that("cast to NULL returns x", {
  x <- new_vctr(1, class = "x")
  expect_equal(vec_cast(NULL, x), NULL)
})

test_that("cast succeeds if attributes equal", {
  x1 <- new_vctr(1, class = "x", a = 1, b = 2)
  x2 <- new_vctr(2, class = "x", a = 1, b = 2)

  expect_equal(vec_cast(x1, x2), x1)
  expect_equal(vec_cast(x2, x1), x2)
})

test_that("and fails if attributes are different", {
  x1 <- new_vctr(1, class = "x", a = 1, b = 2)
  x2 <- new_vctr(2, class = "x", a = 2, b = 2)

  expect_error(vec_cast(x1, x2), class = "vctrs_error_incompatible_type")
})

test_that("restoring to atomic vector of same type preserves attributes", {
  x1 <- new_vctr(1, class = "x")
  x2 <- new_vctr(2, class = "x")

  expect_equal(vec_restore(2, x1), x2)
})

test_that("restoring to atomic vector of different type throws error", {
  x1 <- new_vctr(1, class = "x")

  expect_error(vec_restore("x", x1), class = "vctrs_error_incompatible_type")
})

test_that("base coercion methods mapped to vec_cast", {
  x <- new_vctr(1, inherit_base_type = FALSE)

  expect_error(as.logical(x), class = "vctrs_error_incompatible_type")
  expect_error(as.integer(x), class = "vctrs_error_incompatible_type")
  expect_error(as.logical(x), class = "vctrs_error_incompatible_type")
  expect_error(as.double(x), class = "vctrs_error_incompatible_type")
  expect_error(as.character(x), class = "vctrs_error_incompatible_type")
  expect_error(as.Date(x), class = "vctrs_error_incompatible_type")
  expect_error(as.POSIXct(x), class = "vctrs_error_incompatible_type")
  expect_error(as.POSIXlt(x), class = "vctrs_error_incompatible_type")

  expect_equal(as.list(x), list(x))
})

test_that("as.data.frame creates data frame", {
  x <- new_vctr(1:3)
  df <- as.data.frame(x)

  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 3)
  expect_named(df, "x")
})

test_that("as.data.frame on shaped vctrs doesn't bring along extra attributes", {
  x <- new_vctr(1:3, foo = "bar", dim = c(3L, 1L))
  df <- as.data.frame(x)
  expect_null(attr(df, "foo", exact = TRUE))
})

test_that("as.data.frame2() unclasses input to avoid dispatch on as.data.frame()", {
  x <- structure(1:2, dim = c(1L, 2L), dimnames = list("r1", c("c1", "c2")), class = "foo")
  expect <- data.frame(c1 = 1L, c2 = 2L, row.names = "r1")

  local_methods(as.data.frame.foo = function(x, ...) "dispatched!")

  expect_identical(as.data.frame2(x), expect)
})

test_that("as.list() chops vectors", {
  expect_identical(
    as.list(new_vctr(1:3)),
    list(new_vctr(1L), new_vctr(2L), new_vctr(3L))
  )

  x <- new_vctr(as.list(1:3))
  expect_identical(as.list(x), as.list(1:3))
})


# equality + comparison + arith + math ---------------------------------------

test_that("equality functions remapped", {
  x <- new_vctr(c(1, 1, NA), inherit_base_type = FALSE)

  expect_error(x == 1, class = "vctrs_error_incompatible_type")
  expect_error(x != 1, class = "vctrs_error_incompatible_type")
  expect_equal(is.na(x), c(FALSE, FALSE, TRUE))
  expect_true(anyNA(x))

  expect_equal(unique(x), new_vctr(c(1, NA), inherit_base_type = FALSE))
  expect_equal(duplicated(x), c(FALSE, TRUE, FALSE))
  expect_true(anyDuplicated(x))
})

test_that("is.na<-() supported", {
  x <- new_vctr(1:4)
  is.na(x) <- c(FALSE, FALSE, TRUE, NA)
  expect_identical(x, new_vctr(c(1:2, NA, 4L)))

  x <- new_vctr(1:4)
  is.na(x) <- TRUE
  expect_identical(x, new_vctr(rep(NA_integer_, 4)))

  x <- new_vctr(1:4)
  is.na(x) <- c(2, 3)
  expect_identical(x, new_vctr(c(1L, NA, NA, 4L)))

  names <- c("a", "b", "c", "d")
  x <- set_names(new_vctr(1:4), names)
  is.na(x) <- c("d", "b", "b")
  expect_identical(x, set_names(new_vctr(c(1L, NA, 3L, NA)), names))

  x <- new_vctr(1:4)
  expect_error(is.na(x) <- "x", "character names to index an unnamed vector")
  expect_error(is.na(x) <- 5, class = "vctrs_error_subscript_oob")
})

test_that("comparison functions remapped", {
  local_methods(
    vec_proxy_compare.bizzaro = function(x, ...) -vec_data(x)
  )

  x1 <- new_vctr(c(1, 2), class = "bizzaro")
  x2 <- new_vctr(2, class = "bizzaro")

  expect_equal(order(x1), c(2L, 1L))
  expect_equal(x1 < x2, c(FALSE, FALSE))
  expect_equal(x1 <= x2, c(FALSE, TRUE))
  expect_equal(x1 > x2, c(TRUE, FALSE))
  expect_equal(x1 >= x2, c(TRUE, TRUE))
})

test_that("operators remapped", {
  local_methods(
    vec_arith.bizzaro = function(op, x, y) 1L
  )
  x <- new_vctr(c(1, 2), class = "bizzaro")

  expect_equal(x + 1, 1L)
  expect_equal(x - 1, 1L)
  expect_equal(x * 1, 1L)
  expect_equal(x / 1, 1L)
  expect_equal(x ^ 1, 1L)
  expect_equal(x %% 1, 1L)
  expect_equal(x %/% 1, 1L)
  expect_equal(x & 1, 1L)
  expect_equal(x | 1, 1L)

  expect_equal(!x, 1L)
  expect_equal(+x, 1L)
  expect_equal(-x, 1L)
})

test_that("math functions overridden", {
  local_methods(
    vec_math.bizzaro = function(fn, x, ...) vec_math_base(fn, 2L)
  )
  x <- new_vctr(c(1, NA), class = "bizzaro")

  expect_equal(mean(x), 2L)
  expect_equal(sum(x), 2L)

  expect_equal(is.finite(x), TRUE)
  expect_equal(is.infinite(x), FALSE)
  expect_equal(is.nan(x), FALSE)
})

test_that("diff matches base R", {
  local_methods(
    vec_arith.vctrs_minus = function(op, x, y) vec_arith_base(op, x, y)
  )
  x1 <- cumsum(cumsum(1:10))
  x2 <- new_vctr(x1, class = "vctrs_minus")

  expect_equal(diff(x2), diff(x1))
  expect_equal(diff(x2, lag = 2L), diff(x1, lag = 2L))
  expect_equal(diff(x2, differences = 2L), diff(x1, differences = 2L))

  expect_equal(diff(x2, lag = 11), x2[0L])
  expect_equal(diff(x2, differences = 11), x2[0L])
})

test_that("na.omit() works and retains metadata", {
  x <- new_vctr(c(a = 1, b = NA, c = 2))
  result <- na.omit(x)

  expect <- vec_slice(x, c(1, 3))
  attr(expect, "na.action") <- structure(c(b = 2L), class = "omit")

  expect_identical(result, expect)
})

test_that("na.omit() returns its input unchanged if there are no missing values", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_identical(na.omit(x), x)
})

test_that("na.exclude() works and retains metadata", {
  x <- new_vctr(c(a = 1, b = NA, c = 2))
  result <- na.exclude(x)

  expect <- vec_slice(x, c(1, 3))
  attr(expect, "na.action") <- structure(c(b = 2L), class = "exclude")

  expect_identical(result, expect)
})

test_that("na.fail() works", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_identical(na.fail(x), x)

  x <- new_vctr(c(a = 1, b = NA, c = 2))
  expect_snapshot(error = TRUE, na.fail(x))
})

# names -------------------------------------------------------------------

test_that("`NA_character_` names are repaired to the empty string (#784)", {
  expect_named(new_vctr(set_names(1, NA_character_)), "")
  expect_named(new_vctr(set_names(1:2, c("a", NA))), c("a", ""))
})

test_that("the empty string is an allowed name (#784)", {
  expect_named(new_vctr(set_names(1, "")), "")
  expect_named(new_vctr(set_names(1:2, c("", "x"))), c("", "x"))
})

test_that("can not provide invalid names", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_error(names(x) <- "x", "length")
  expect_error(names(x) <- c("x", "y", "z"), "length")
  expect_error(names(x) <- NULL, NA)
})

test_that("can set names to the empty string (#784)", {
  x <- new_vctr(c(a = 1, b = 2))

  names(x) <- c("", "")
  expect_named(x, c("", ""))

  names(x) <- c("", "x")
  expect_named(x, c("", "x"))
})

test_that("setting names to `NA_character_` repairs to the empty string (#784)", {
  x <- new_vctr(1:2)

  names(x) <- c(NA_character_, NA_character_)
  expect_named(x, c("", ""))

  names(x) <- c("x", NA_character_)
  expect_named(x, c("x", ""))
})

test_that("can use [ and [[ with names", {
  local_methods(
    vec_ptype2.vctrs_vctr.double = function(...) dbl(),
    vec_ptype2.double.vctrs_vctr = function(...) dbl()
  )
  x <- new_vctr(c(a = 1, b = 2))

  expect_equal(x["b"], new_vctr(c(b = 2)))
  expect_equal(x[["b"]], new_vctr(2)) # [[ drops names

  x[["c"]] <- 3
  expect_equal(x[["c"]], new_vctr(3))
  x["d"] <- 4
  expect_equal(x[["d"]], new_vctr(4))
})

test_that("can use [ and [[ with names - list vctr", {
  local_methods(
    vec_ptype2.vctrs_vctr = function(...) list(),
    vec_ptype2.list.vctrs_vctr = function(...) list()
  )
  y <- new_vctr(list(a = 1, b = 2))
  y[["c"]] <- 3
  expect_equal(y[["c"]], 3)
  y["d"] <- list(4)
  expect_equal(y[["d"]], 4)
})

test_that("can use [[<- to replace n-dimensional elements", {
  local_methods(
    vec_restore.vctrs_mtrx = function(x, to, ...) x,
    vec_ptype2.double.vctrs_mtrx = function(...) dbl(),
    vec_ptype2.vctrs_mtrx = function(...) dbl()
  )
  x <- new_vctr(rep(1, times = 4), dim = c(2, 2), class = "vctrs_mtrx")
  x[[2, 2]] <- 4
  expect_equal(x[[2, 2]], 4)
})

test_that("subsetting preserves attributes", {
  x <- new_vctr(c(a = 1, b = 2))
  attr(x, "extra") <- TRUE

  y <- x[1]
  expect_equal(attr(x, "extra"), TRUE)
})

test_that("$ inherits from underlying vector", {
  x1 <- new_vctr(c(a = 1, b = 2))
  expect_error(x1$a, "atomic vectors")
  expect_error(x1$a <- 2, "atomic vectors")

  x2 <- new_vctr(list(a = 1, b = 2))
  expect_equal(x2$a, 1)
  x2$a <- 10
  expect_equal(x2$a, 10)
})


# unsupported/unimplemented operations --------------------------------------

test_that("can't touch protected attributes", {
  x <- new_vctr(1:4)

  expect_error(dim(x) <- c(2, 2), class = "vctrs_error_unsupported")
  expect_error(dimnames(x) <- list("x"), class = "vctrs_error_unsupported")
  expect_error(levels(x) <- "x", class = "vctrs_error_unsupported")

  # It is expected that unimplemented `levels()` returns `NULL`
  expect_null(levels(x))

  # But it's ok to set names to NULL; this happens at least in vec_c
  # and maybe elsewhere. We may need to back off on this level of
  # strictness in the future
  expect_error(names(x) <- NULL, NA)
})

test_that("summary is unimplemented", {
  x <- new_vctr(1:4)
  expect_error(summary(x), class = "vctrs_error_unimplemented")
})

# hidden class ------------------------------------------------------------
# We can't construct classes in test because the methods are not found
# when vctr generics call other generics. Instead we rely on a very simple
# class implemented in vctr.R

test_that("class preserved when subsetting", {
  h <- new_hidden(1:4)

  expect_s3_class(h, "hidden")
  expect_s3_class(h[1], "hidden")
  expect_s3_class(h[[1]], "hidden")
  expect_s3_class(rep(h[1], 2), "hidden")
  expect_s3_class(as.list(h)[[1]], "hidden")

  length(h) <- 3
  expect_s3_class(h, "hidden")
})

test_that("RHS cast when using subset assign", {
  local_hidden()
  h <- new_hidden(1)

  expect_error(h[[1]] <- "x", class = "vctrs_error_incompatible_type")
  expect_error(h[1] <- "x", class = "vctrs_error_incompatible_type")

  h[2] <- 1
  expect_equal(h, new_hidden(c(1, 1)))

  h[[2]] <- 2
  expect_equal(h, new_hidden(c(1, 2)))
})

test_that("c passes on to vec_c", {
  local_hidden()

  h <- new_hidden(1)

  expect_equal(c(h), h)
  expect_equal(c(h, NULL), h)

  expect_equal(c(h, 1), rep(h, 2))
  expect_equal(c(h, h), rep(h, 2))
})

test_that("rbind does not fail with an unclear message (#1186)", {
  # In general, vec_rbind() should be preferred. In many cases rbind() does
  # the right thing, this test exists to alert us if this changes in the future.
  skip_on_cran()

  local_hidden()

  h <- new_hidden(1)
  # A failure in levels() for vctrs_vctr classes was the underlying issue.
  expect_null(levels(h))

  df <- data_frame(h = h)

  expect_equal(rbind(df), df)
  expect_equal(rbind(df, NULL), df)

  expect_equal(rbind(df, data_frame(h = 1)), unrownames(df[c(1, 1), , drop = FALSE]))
  expect_equal(rbind(df, df), unrownames(df[c(1, 1), , drop = FALSE]))
  # An example where the result differs, to alert us if the rbind() contract
  # changes
  expect_equal(rbind(data_frame(h = 1), df), data_frame(h = c(1, 1)))
})

test_that("summaries preserve class", {
  h <- new_hidden(c(1, 2))

  expect_equal(sum(h), new_hidden(3))
  expect_equal(mean(h), new_hidden(1.5))
})

test_that("methods using vec_proxy_compare agree with base", {
  h <- new_hidden(c(1:10))
  h_na <- new_hidden(c(NA, 1:10))

  expect_agree <- function(f, x, na.rm = FALSE) {
    f <- enexpr(f)
    expect_equal(vec_data((!!f)(x, na.rm = na.rm)), (!!f)(vec_data(x), na.rm = na.rm))
  }

  expect_agree(min, h)
  expect_agree(max, h)
  expect_agree(range, h)

  expect_agree(min, h_na)
  expect_agree(max, h_na)
  expect_agree(range, h_na)

  expect_agree(min, h_na, na.rm = TRUE)
  expect_agree(max, h_na, na.rm = TRUE)
  expect_agree(range, h_na, na.rm = TRUE)
})

test_that("can put in data frame", {
  h <- new_hidden(1:4)

  expect_named(as.data.frame(h), "h")
  expect_named(data.frame(x = h), "x")
})

test_that("base coercions default to vec_cast", {
  local_hidden()
  h <- new_hidden(1)
  expect_error(as.character(h), class = "vctrs_error_incompatible_type")
  expect_error(as.integer(h), class = "vctrs_error_incompatible_type")
  expect_error(generics::as.factor(h), class = "vctrs_error_incompatible_type")
  expect_error(generics::as.ordered(h), class = "vctrs_error_incompatible_type")
  expect_error(generics::as.difftime(h), class = "vctrs_error_incompatible_type")
  expect_equal(as.logical(h), TRUE)
  expect_equal(as.double(h), 1)
})

test_that("default print and str methods are useful", {
  h <- new_hidden(1:4)
  expect_snapshot(h)
  expect_snapshot(h[0])
  expect_snapshot(str(h))
})

test_that("default print method shows names", {
  h <- new_hidden(c(A = 1, B = 2, C = 3))
  expect_snapshot(h)
})

test_that("can't transpose", {
  h <- new_hidden(1:4)
  expect_error(t(h), class = "vctrs_error_unsupported")
})

test_that("shaped vctrs can be cast to data frames", {
  x <- new_vctr(1:4, dim = 4)
  expect_identical(as.data.frame(x), data.frame(V1 = 1:4))

  x <- new_vctr(1:4, dim = c(2, 2))
  expect_identical(as.data.frame(x), data.frame(V1 = 1:2, V2 = 3:4))
})


# slicing -----------------------------------------------------------------

test_that("additional subscripts are handled (#269)", {
  new_2d <- function(.data, dim) {
    vctrs::new_vctr(.data, dim = dim, class = "vctrs_2d")
  }
  x <- new_2d(c(1, 2), dim = c(2L, 1L))

  expect_identical(x[1], new_2d(1, dim = c(1, 1)))
  expect_identical(x[1, 1], new_2d(1, dim = c(1, 1)))
  expect_identical(x[, 1], new_2d(c(1, 2), dim = c(2, 1)))
})


# summary generics --------------------------------------------------------

test_that("na.rm is forwarded to summary generics", {
  x <- new_vctr(dbl(1, 2, NA))

  expect_identical(mean(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(mean(x, na.rm = TRUE), new_vctr(1.5))

  expect_identical(min(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(min(x, na.rm = TRUE), new_vctr(1))

  expect_identical(max(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(max(x, na.rm = TRUE), new_vctr(2))

  x <- new_vctr(lgl(TRUE, NA))

  expect_identical(all(x, na.rm = FALSE), lgl(NA))
  expect_identical(all(x, na.rm = TRUE), TRUE)
})

test_that("Summary generics behave identically to base for empty vctrs (#88)", {
  expect_warning(
    expect_identical(
      new_vctr(max(numeric())),
      max(new_vctr(numeric()))
    )
  )

  expect_warning(
    expect_identical(
      new_vctr(min(numeric())),
      min(new_vctr(numeric()))
    )
  )

  expect_identical(
    new_vctr(range(1)),
    range(new_vctr(1))
  )

  expect_identical(
    new_vctr(prod(numeric())),
    prod(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(sum(numeric())),
    sum(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cummax(numeric())),
    cummax(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cummin(numeric())),
    cummin(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cumprod(numeric())),
    cumprod(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cumsum(numeric())),
    cumsum(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(mean(numeric())),
    mean(new_vctr(numeric()))
  )
})

test_that("generic predicates return logical vectors (#251)", {
  x <- new_vctr(c(1, 2))
  expect_identical(is.finite(x), c(TRUE, TRUE))
  expect_identical(is.infinite(x), c(FALSE, FALSE))
  expect_identical(is.nan(x), c(FALSE, FALSE))

  x <- new_vctr(TRUE)
  expect_identical(any(x), TRUE)
  expect_identical(all(x), TRUE)
})

test_that("xtfrm() converts logical types to integer", {
  expect_identical(xtfrm(new_vctr(c(TRUE, FALSE, NA), foo = "bar")), c(1L, 0L, NA))
})

test_that("xtfrm() unwraps integer and double atomic types", {
  expect_identical(xtfrm(new_vctr(1:3, foo = "bar")), 1:3)
  expect_identical(xtfrm(new_vctr(1:3 + 0, foo = "bar")), 1:3 + 0)
})

test_that("xtfrm() works with character subclass", {
  expect_identical(xtfrm(new_vctr(chr())), int())
})

test_that("xtfrm() maintains ties when falling through to vec_rank() (#1354)", {
  x <- new_vctr(c("F", "F", "M", "A", "M", "A"))
  expect_identical(xtfrm(x), c(2L, 2L, 3L, 1L, 3L, 1L))
})

test_that("xtfrm() propagates NAs when falling through to vec_rank()", {
  x <- new_vctr(c("F", NA))
  expect_identical(xtfrm(x), c(1L, NA))
})

test_that("xtfrm() uses C locale ordering with character proxies", {
  x <- new_vctr(c("A", "a", "B"))
  expect_identical(xtfrm(x), c(1L, 3L, 2L))
})

test_that("xtfrm() works on rcrd types", {
  x <- new_rcrd(list(x = c(1, 2, 1, NA), y = c(2, 1, 1, NA)))
  expect_identical(xtfrm(x), c(2L, 3L, 1L, NA))
})

test_that("Summary generics behave as expected if na.rm = TRUE and all values are NA (#1357)", {
  expect_identical(min(new_vctr(NA_real_), na.rm = TRUE), new_vctr(Inf))
  expect_identical(max(new_vctr(NA_real_), na.rm = TRUE), new_vctr(-Inf))
  expect_identical(range(new_vctr(NA_real_), na.rm = TRUE), new_vctr(c(Inf, -Inf)))

  expect_identical(min(new_vctr(NA_integer_), na.rm = TRUE), new_vctr(NA_integer_))
  expect_identical(max(new_vctr(NA_integer_), na.rm = TRUE), new_vctr(NA_integer_))
  expect_identical(range(new_vctr(NA_integer_), na.rm = TRUE), new_vctr(c(NA_integer_, NA_integer_)))

  expect_identical(min(new_vctr(NA_character_), na.rm = TRUE), new_vctr(NA_character_))
  expect_identical(max(new_vctr(NA_character_), na.rm = TRUE), new_vctr(NA_character_))
  expect_identical(range(new_vctr(NA_character_), na.rm = TRUE), new_vctr(c(NA_character_, NA_character_)))

  expect_identical(min(new_vctr(NA), na.rm = TRUE), new_vctr(NA))
  expect_identical(max(new_vctr(NA), na.rm = TRUE), new_vctr(NA))
  expect_identical(range(new_vctr(NA), na.rm = TRUE), new_vctr(c(NA, NA)))
})

test_that("Summary generics behave as expected for empty vctrs (#1357)", {
  expect_identical(min(new_vctr(numeric()), na.rm = TRUE), new_vctr(Inf))
  expect_identical(max(new_vctr(numeric()), na.rm = TRUE), new_vctr(-Inf))
  expect_identical(range(new_vctr(numeric()), na.rm = TRUE), new_vctr(c(Inf, -Inf)))

  expect_identical(min(new_vctr(integer()), na.rm = TRUE), new_vctr(NA_integer_))
  expect_identical(max(new_vctr(integer()), na.rm = TRUE), new_vctr(NA_integer_))
  expect_identical(range(new_vctr(integer()), na.rm = TRUE), new_vctr(c(NA_integer_, NA_integer_)))

  expect_identical(min(new_vctr(character()), na.rm = TRUE), new_vctr(NA_character_))
  expect_identical(max(new_vctr(character()), na.rm = TRUE), new_vctr(NA_character_))
  expect_identical(range(new_vctr(character()), na.rm = TRUE), new_vctr(c(NA_character_, NA_character_)))

  expect_identical(min(new_vctr(logical()), na.rm = TRUE), new_vctr(NA))
  expect_identical(max(new_vctr(logical()), na.rm = TRUE), new_vctr(NA))
  expect_identical(range(new_vctr(logical()), na.rm = TRUE), new_vctr(c(NA, NA)))
})

test_that("anyNA(recursive = TRUE) works with lists (#1278)", {
  x <- list_of(1:4, c(2, NA, 5))
  expect_false(anyNA(x))
  expect_true(anyNA(x, recursive = TRUE))

  x <- new_vctr(list(1:4, list(c(2, NA, 5))))
  expect_false(anyNA(x))
  expect_true(anyNA(x, recursive = TRUE))
})

test_that("constructor sets attributes", {
  x <- new_vctr(1:4, class = "x", x = 1)
  expect_equal(x, structure(1:4, class = c("x", "vctrs_vctr"), x = 1))

  x <- new_vctr(1:4, class = "x", x = 1, inherit_base_type = TRUE)
  expect_equal(
    x,
    structure(1:4, class = c("x", "vctrs_vctr", "integer"), x = 1)
  )
})

test_that(".data must be a vector", {
  expect_error(new_vctr(mean), "vector type")
})

test_that("attributes other than names are ignored", {
  out <- new_vctr(structure(1, a = 1))
  expect_null(attributes(out)$a)
})

test_that("default format method is internal", {
  x <- new_vctr(1, class = "x")
  expect_equal(format(x), format(x))
})

test_that("vctr class is proxied", {
  expect_identical(vec_proxy(new_vctr(1:3)), new_vctr(1:3))
  expect_identical(
    vec_proxy(new_vctr(as.list(1:3))),
    unclass(new_vctr(as.list(1:3)))
  )
  expect_true(vec_is(new_vctr(as.list(1:3))))
})

test_that("Can opt out of base type", {
  x <- new_vctr(1, class = "x", inherit_base_type = FALSE)
  expect_s3_class(x, c("x", "vctrs_vctr"), exact = TRUE)
})

test_that("base type is always set for lists", {
  expect_s3_class(new_vctr(list()), "list")
})

test_that("cannot opt out of the base type with lists", {
  expect_error(
    new_vctr(list(), inherit_base_type = FALSE),
    "must inherit from the base type"
  )
})

test_that("data frames are not allowed", {
  expect_error(new_vctr(mtcars), "can't be a data frame")
})

test_that("attributes must be named", {
  expect_error(vec_set_attributes(1, list(1)), "must be named")
  expect_error(vec_set_attributes(1, list(y = 1, 2)), "2 does not")
})

test_that("can strip all attributes without adding new ones", {
  expect_equal(vec_set_attributes(structure(1, a = 1), NULL), 1)
})

test_that("`c.vctrs_vctr()` checks for default method arguments (#791)", {
  expect_error(c(new_vctr(1), recursive = TRUE), "`recursive` must be")
  expect_error(c(new_vctr(1), use.names = FALSE), "`use.names` must be")
})


# Cast/restore ------------------------------------------------------------

test_that("cast to NULL returns x", {
  x <- new_vctr(1, class = "x")
  expect_equal(vec_cast(NULL, x), NULL)
})

test_that("cast succeeds if attributes equal", {
  x1 <- new_vctr(1, class = "x", a = 1, b = 2)
  x2 <- new_vctr(2, class = "x", a = 1, b = 2)

  expect_equal(vec_cast(x1, x2), x1)
  expect_equal(vec_cast(x2, x1), x2)
})

test_that("and fails if attributes are different", {
  x1 <- new_vctr(1, class = "x", a = 1, b = 2)
  x2 <- new_vctr(2, class = "x", a = 2, b = 2)

  expect_error(vec_cast(x1, x2), class = "vctrs_error_incompatible_type")
})

test_that("restoring to atomic vector of same type preserves attributes", {
  x1 <- new_vctr(1, class = "x")
  x2 <- new_vctr(2, class = "x")

  expect_equal(vec_restore(2, x1), x2)
})

test_that("restoring to atomic vector of different type throws error", {
  x1 <- new_vctr(1, class = "x")

  expect_error(vec_restore("x", x1), class = "vctrs_error_incompatible_type")
})

test_that("base coercion methods mapped to vec_cast", {
  x <- new_vctr(1, inherit_base_type = FALSE)

  expect_error(as.logical(x), class = "vctrs_error_incompatible_type")
  expect_error(as.integer(x), class = "vctrs_error_incompatible_type")
  expect_error(as.logical(x), class = "vctrs_error_incompatible_type")
  expect_error(as.double(x), class = "vctrs_error_incompatible_type")
  expect_error(as.character(x), class = "vctrs_error_incompatible_type")
  expect_error(as.Date(x), class = "vctrs_error_incompatible_type")
  expect_error(as.POSIXct(x), class = "vctrs_error_incompatible_type")
  expect_error(as.POSIXlt(x), class = "vctrs_error_incompatible_type")

  expect_equal(as.list(x), list(x))
})

test_that("as.data.frame creates data frame", {
  x <- new_vctr(1:3)
  df <- as.data.frame(x)

  expect_s3_class(df, "data.frame")
  expect_equal(nrow(df), 3)
  expect_named(df, "x")
})

test_that("as.data.frame on shaped vctrs doesn't bring along extra attributes", {
  x <- new_vctr(1:3, foo = "bar", dim = c(3L, 1L))
  df <- as.data.frame(x)
  expect_null(attr(df, "foo", exact = TRUE))
})

test_that("as.data.frame2() unclasses input to avoid dispatch on as.data.frame()", {
  x <- structure(
    1:2,
    dim = c(1L, 2L),
    dimnames = list("r1", c("c1", "c2")),
    class = "foo"
  )
  expect <- data.frame(c1 = 1L, c2 = 2L, row.names = "r1")

  local_methods(as.data.frame.foo = function(x, ...) "dispatched!")

  expect_identical(as.data.frame2(x), expect)
})

test_that("as.list() chops vectors", {
  expect_identical(
    as.list(new_vctr(1:3)),
    list(new_vctr(1L), new_vctr(2L), new_vctr(3L))
  )

  x <- new_vctr(as.list(1:3))
  expect_identical(as.list(x), as.list(1:3))
})


# equality + comparison + arith + math ---------------------------------------

test_that("equality functions remapped", {
  x <- new_vctr(c(1, 1, NA), inherit_base_type = FALSE)

  expect_error(x == 1, class = "vctrs_error_incompatible_type")
  expect_error(x != 1, class = "vctrs_error_incompatible_type")
  expect_equal(is.na(x), c(FALSE, FALSE, TRUE))
  expect_true(anyNA(x))

  expect_equal(unique(x), new_vctr(c(1, NA), inherit_base_type = FALSE))
  expect_equal(duplicated(x), c(FALSE, TRUE, FALSE))
  expect_true(anyDuplicated(x))
})

test_that("is.na<-() supported", {
  x <- new_vctr(1:4)
  is.na(x) <- c(FALSE, FALSE, TRUE, NA)
  expect_identical(x, new_vctr(c(1:2, NA, 4L)))

  x <- new_vctr(1:4)
  is.na(x) <- TRUE
  expect_identical(x, new_vctr(rep(NA_integer_, 4)))

  x <- new_vctr(1:4)
  is.na(x) <- c(2, 3)
  expect_identical(x, new_vctr(c(1L, NA, NA, 4L)))

  names <- c("a", "b", "c", "d")
  x <- set_names(new_vctr(1:4), names)
  is.na(x) <- c("d", "b", "b")
  expect_identical(x, set_names(new_vctr(c(1L, NA, 3L, NA)), names))

  x <- new_vctr(1:4)
  expect_error(is.na(x) <- "x", "character names to index an unnamed vector")
  expect_error(is.na(x) <- 5, class = "vctrs_error_subscript_oob")
})

test_that("comparison functions remapped", {
  local_methods(
    vec_proxy_compare.bizzaro = function(x, ...) -vec_data(x)
  )

  x1 <- new_vctr(c(1, 2), class = "bizzaro")
  x2 <- new_vctr(2, class = "bizzaro")

  expect_equal(order(x1), c(2L, 1L))
  expect_equal(x1 < x2, c(FALSE, FALSE))
  expect_equal(x1 <= x2, c(FALSE, TRUE))
  expect_equal(x1 > x2, c(TRUE, FALSE))
  expect_equal(x1 >= x2, c(TRUE, TRUE))
})

test_that("operators remapped", {
  local_methods(
    vec_arith.bizzaro = function(op, x, y) 1L
  )
  x <- new_vctr(c(1, 2), class = "bizzaro")

  expect_equal(x + 1, 1L)
  expect_equal(x - 1, 1L)
  expect_equal(x * 1, 1L)
  expect_equal(x / 1, 1L)
  expect_equal(x^1, 1L)
  expect_equal(x %% 1, 1L)
  expect_equal(x %/% 1, 1L)
  expect_equal(x & 1, 1L)
  expect_equal(x | 1, 1L)

  expect_equal(!x, 1L)
  expect_equal(+x, 1L)
  expect_equal(-x, 1L)
})

test_that("math functions overridden", {
  local_methods(
    vec_math.bizzaro = function(fn, x, ...) vec_math_base(fn, 2L)
  )
  x <- new_vctr(c(1, NA), class = "bizzaro")

  expect_equal(mean(x), 2L)
  expect_equal(sum(x), 2L)

  expect_equal(is.finite(x), TRUE)
  expect_equal(is.infinite(x), FALSE)
  expect_equal(is.nan(x), FALSE)
})

test_that("diff matches base R", {
  local_methods(
    vec_arith.vctrs_minus = function(op, x, y) vec_arith_base(op, x, y)
  )
  x1 <- cumsum(cumsum(1:10))
  x2 <- new_vctr(x1, class = "vctrs_minus")

  expect_equal(diff(x2), diff(x1))
  expect_equal(diff(x2, lag = 2L), diff(x1, lag = 2L))
  expect_equal(diff(x2, differences = 2L), diff(x1, differences = 2L))

  expect_equal(diff(x2, lag = 11), x2[0L])
  expect_equal(diff(x2, differences = 11), x2[0L])
})

test_that("na.omit() works and retains metadata", {
  x <- new_vctr(c(a = 1, b = NA, c = 2))
  result <- na.omit(x)

  expect <- vec_slice(x, c(1, 3))
  attr(expect, "na.action") <- structure(c(b = 2L), class = "omit")

  expect_identical(result, expect)
})

test_that("na.omit() returns its input unchanged if there are no missing values", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_identical(na.omit(x), x)
})

test_that("na.exclude() works and retains metadata", {
  x <- new_vctr(c(a = 1, b = NA, c = 2))
  result <- na.exclude(x)

  expect <- vec_slice(x, c(1, 3))
  attr(expect, "na.action") <- structure(c(b = 2L), class = "exclude")

  expect_identical(result, expect)
})

test_that("na.fail() works", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_identical(na.fail(x), x)

  x <- new_vctr(c(a = 1, b = NA, c = 2))
  expect_snapshot(error = TRUE, na.fail(x))
})

# names -------------------------------------------------------------------

test_that("`NA_character_` names are repaired to the empty string (#784)", {
  expect_named(new_vctr(set_names(1, NA_character_)), "")
  expect_named(new_vctr(set_names(1:2, c("a", NA))), c("a", ""))
})

test_that("the empty string is an allowed name (#784)", {
  expect_named(new_vctr(set_names(1, "")), "")
  expect_named(new_vctr(set_names(1:2, c("", "x"))), c("", "x"))
})

test_that("can not provide invalid names", {
  x <- new_vctr(c(a = 1, b = 2))
  expect_error(names(x) <- "x", "length")
  expect_error(names(x) <- c("x", "y", "z"), "length")
  expect_error(names(x) <- NULL, NA)
})

test_that("can set names to the empty string (#784)", {
  x <- new_vctr(c(a = 1, b = 2))

  names(x) <- c("", "")
  expect_named(x, c("", ""))

  names(x) <- c("", "x")
  expect_named(x, c("", "x"))
})

test_that("setting names to `NA_character_` repairs to the empty string (#784)", {
  x <- new_vctr(1:2)

  names(x) <- c(NA_character_, NA_character_)
  expect_named(x, c("", ""))

  names(x) <- c("x", NA_character_)
  expect_named(x, c("x", ""))
})

test_that("can use [ and [[ with names", {
  local_methods(
    vec_ptype2.vctrs_vctr.double = function(...) dbl(),
    vec_ptype2.double.vctrs_vctr = function(...) dbl()
  )
  x <- new_vctr(c(a = 1, b = 2))

  expect_equal(x["b"], new_vctr(c(b = 2)))
  expect_equal(x[["b"]], new_vctr(2)) # [[ drops names

  x[["c"]] <- 3
  expect_equal(x[["c"]], new_vctr(3))
  x["d"] <- 4
  expect_equal(x[["d"]], new_vctr(4))
})

test_that("can use [ and [[ with names - list vctr", {
  local_methods(
    vec_ptype2.vctrs_vctr = function(...) list(),
    vec_ptype2.list.vctrs_vctr = function(...) list()
  )
  y <- new_vctr(list(a = 1, b = 2))
  y[["c"]] <- 3
  expect_equal(y[["c"]], 3)
  y["d"] <- list(4)
  expect_equal(y[["d"]], 4)
})

test_that("can use [[<- to replace n-dimensional elements", {
  local_methods(
    vec_restore.vctrs_mtrx = function(x, to, ...) x,
    vec_ptype2.double.vctrs_mtrx = function(...) dbl(),
    vec_ptype2.vctrs_mtrx = function(...) dbl()
  )
  x <- new_vctr(rep(1, times = 4), dim = c(2, 2), class = "vctrs_mtrx")
  x[[2, 2]] <- 4
  expect_equal(x[[2, 2]], 4)
})

test_that("subsetting preserves attributes", {
  x <- new_vctr(c(a = 1, b = 2))
  attr(x, "extra") <- TRUE

  y <- x[1]
  expect_equal(attr(x, "extra"), TRUE)
})

test_that("$ inherits from underlying vector", {
  x1 <- new_vctr(c(a = 1, b = 2))
  expect_error(x1$a, "atomic vectors")
  expect_error(x1$a <- 2, "atomic vectors")

  x2 <- new_vctr(list(a = 1, b = 2))
  expect_equal(x2$a, 1)
  x2$a <- 10
  expect_equal(x2$a, 10)
})


# unsupported/unimplemented operations --------------------------------------

test_that("can't touch protected attributes", {
  x <- new_vctr(1:4)

  expect_error(dim(x) <- c(2, 2), class = "vctrs_error_unsupported")
  expect_error(dimnames(x) <- list("x"), class = "vctrs_error_unsupported")
  expect_error(levels(x) <- "x", class = "vctrs_error_unsupported")

  # It is expected that unimplemented `levels()` returns `NULL`
  expect_null(levels(x))

  # But it's ok to set names to NULL; this happens at least in vec_c
  # and maybe elsewhere. We may need to back off on this level of
  # strictness in the future
  expect_error(names(x) <- NULL, NA)
})

test_that("summary is unimplemented", {
  x <- new_vctr(1:4)
  expect_error(summary(x), class = "vctrs_error_unimplemented")
})

# hidden class ------------------------------------------------------------
# We can't construct classes in test because the methods are not found
# when vctr generics call other generics. Instead we rely on a very simple
# class implemented in vctr.R

test_that("class preserved when subsetting", {
  h <- new_hidden(1:4)

  expect_s3_class(h, "hidden")
  expect_s3_class(h[1], "hidden")
  expect_s3_class(h[[1]], "hidden")
  expect_s3_class(rep(h[1], 2), "hidden")
  expect_s3_class(as.list(h)[[1]], "hidden")

  length(h) <- 3
  expect_s3_class(h, "hidden")
})

test_that("RHS cast when using subset assign", {
  local_hidden()
  h <- new_hidden(1)

  expect_error(h[[1]] <- "x", class = "vctrs_error_incompatible_type")
  expect_error(h[1] <- "x", class = "vctrs_error_incompatible_type")

  h[2] <- 1
  expect_equal(h, new_hidden(c(1, 1)))

  h[[2]] <- 2
  expect_equal(h, new_hidden(c(1, 2)))
})

test_that("c passes on to vec_c", {
  local_hidden()

  h <- new_hidden(1)

  expect_equal(c(h), h)
  expect_equal(c(h, NULL), h)

  expect_equal(c(h, 1), rep(h, 2))
  expect_equal(c(h, h), rep(h, 2))
})

test_that("rbind does not fail with an unclear message (#1186)", {
  # In general, vec_rbind() should be preferred. In many cases rbind() does
  # the right thing, this test exists to alert us if this changes in the future.
  skip_on_cran()

  local_hidden()

  h <- new_hidden(1)
  # A failure in levels() for vctrs_vctr classes was the underlying issue.
  expect_null(levels(h))

  df <- data_frame(h = h)

  expect_equal(rbind(df), df)
  expect_equal(rbind(df, NULL), df)

  expect_equal(
    rbind(df, data_frame(h = 1)),
    unrownames(df[c(1, 1), , drop = FALSE])
  )
  expect_equal(rbind(df, df), unrownames(df[c(1, 1), , drop = FALSE]))
  # An example where the result differs, to alert us if the rbind() contract
  # changes
  expect_equal(rbind(data_frame(h = 1), df), data_frame(h = c(1, 1)))
})

test_that("summaries preserve class", {
  h <- new_hidden(c(1, 2))

  expect_equal(sum(h), new_hidden(3))
  expect_equal(mean(h), new_hidden(1.5))
})

test_that("methods using vec_proxy_compare agree with base", {
  h <- new_hidden(c(1:10))
  h_na <- new_hidden(c(NA, 1:10))

  expect_agree <- function(f, x, na.rm = FALSE) {
    f <- enexpr(f)
    expect_equal(
      vec_data((!!f)(x, na.rm = na.rm)),
      (!!f)(vec_data(x), na.rm = na.rm)
    )
  }

  expect_agree(min, h)
  expect_agree(max, h)
  expect_agree(range, h)

  expect_agree(min, h_na)
  expect_agree(max, h_na)
  expect_agree(range, h_na)

  expect_agree(min, h_na, na.rm = TRUE)
  expect_agree(max, h_na, na.rm = TRUE)
  expect_agree(range, h_na, na.rm = TRUE)
})

test_that("can put in data frame", {
  h <- new_hidden(1:4)

  expect_named(as.data.frame(h), "h")
  expect_named(data.frame(x = h), "x")
})

test_that("base coercions default to vec_cast", {
  local_hidden()
  h <- new_hidden(1)
  expect_error(as.character(h), class = "vctrs_error_incompatible_type")
  expect_error(as.integer(h), class = "vctrs_error_incompatible_type")
  expect_error(generics::as.factor(h), class = "vctrs_error_incompatible_type")
  expect_error(generics::as.ordered(h), class = "vctrs_error_incompatible_type")
  expect_error(
    generics::as.difftime(h),
    class = "vctrs_error_incompatible_type"
  )
  expect_equal(as.logical(h), TRUE)
  expect_equal(as.double(h), 1)
})

test_that("default print and str methods are useful", {
  h <- new_hidden(1:4)
  expect_snapshot(h)
  expect_snapshot(h[0])
  expect_snapshot(str(h))
})

test_that("default print method shows names", {
  h <- new_hidden(c(A = 1, B = 2, C = 3))
  expect_snapshot(h)
})

test_that("can't transpose", {
  h <- new_hidden(1:4)
  expect_error(t(h), class = "vctrs_error_unsupported")
})

test_that("shaped vctrs can be cast to data frames", {
  x <- new_vctr(1:4, dim = 4)
  expect_identical(as.data.frame(x), data.frame(V1 = 1:4))

  x <- new_vctr(1:4, dim = c(2, 2))
  expect_identical(as.data.frame(x), data.frame(V1 = 1:2, V2 = 3:4))
})


# slicing -----------------------------------------------------------------

test_that("additional subscripts are handled (#269)", {
  new_2d <- function(.data, dim) {
    vctrs::new_vctr(.data, dim = dim, class = "vctrs_2d")
  }
  x <- new_2d(c(1, 2), dim = c(2L, 1L))

  expect_identical(x[1], new_2d(1, dim = c(1, 1)))
  expect_identical(x[1, 1], new_2d(1, dim = c(1, 1)))
  expect_identical(x[, 1], new_2d(c(1, 2), dim = c(2, 1)))
})


# summary generics --------------------------------------------------------

test_that("na.rm is forwarded to summary generics", {
  x <- new_vctr(dbl(1, 2, NA))

  expect_identical(mean(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(mean(x, na.rm = TRUE), new_vctr(1.5))

  expect_identical(min(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(min(x, na.rm = TRUE), new_vctr(1))

  expect_identical(max(x, na.rm = FALSE), new_vctr(dbl(NA)))
  expect_identical(max(x, na.rm = TRUE), new_vctr(2))

  x <- new_vctr(lgl(TRUE, NA))

  expect_identical(all(x, na.rm = FALSE), lgl(NA))
  expect_identical(all(x, na.rm = TRUE), TRUE)
})

test_that("Summary generics behave identically to base for empty vctrs (#88)", {
  expect_warning(
    expect_identical(
      new_vctr(max(numeric())),
      max(new_vctr(numeric()))
    )
  )

  expect_warning(
    expect_identical(
      new_vctr(min(numeric())),
      min(new_vctr(numeric()))
    )
  )

  expect_identical(
    new_vctr(range(1)),
    range(new_vctr(1))
  )

  expect_identical(
    new_vctr(prod(numeric())),
    prod(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(sum(numeric())),
    sum(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cummax(numeric())),
    cummax(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cummin(numeric())),
    cummin(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cumprod(numeric())),
    cumprod(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(cumsum(numeric())),
    cumsum(new_vctr(numeric()))
  )

  expect_identical(
    new_vctr(mean(numeric())),
    mean(new_vctr(numeric()))
  )
})

test_that("generic predicates return logical vectors (#251)", {
  x <- new_vctr(c(1, 2))
  expect_identical(is.finite(x), c(TRUE, TRUE))
  expect_identical(is.infinite(x), c(FALSE, FALSE))
  expect_identical(is.nan(x), c(FALSE, FALSE))

  x <- new_vctr(TRUE)
  expect_identical(any(x), TRUE)
  expect_identical(all(x), TRUE)
})

test_that("xtfrm() converts logical types to integer", {
  expect_identical(
    xtfrm(new_vctr(c(TRUE, FALSE, NA), foo = "bar")),
    c(1L, 0L, NA)
  )
})

test_that("xtfrm() unwraps integer and double atomic types", {
  expect_identical(xtfrm(new_vctr(1:3, foo = "bar")), 1:3)
  expect_identical(xtfrm(new_vctr(1:3 + 0, foo = "bar")), 1:3 + 0)
})

test_that("xtfrm() works with character subclass", {
  expect_identical(xtfrm(new_vctr(chr())), int())
})

test_that("xtfrm() maintains ties when falling through to vec_rank() (#1354)", {
  x <- new_vctr(c("F", "F", "M", "A", "M", "A"))
  expect_identical(xtfrm(x), c(2L, 2L, 3L, 1L, 3L, 1L))
})

test_that("xtfrm() propagates NAs when falling through to vec_rank()", {
  x <- new_vctr(c("F", NA))
  expect_identical(xtfrm(x), c(1L, NA))
})

test_that("xtfrm() uses C locale ordering with character proxies", {
  x <- new_vctr(c("A", "a", "B"))
  expect_identical(xtfrm(x), c(1L, 3L, 2L))
})

test_that("xtfrm() works on rcrd types", {
  x <- new_rcrd(list(x = c(1, 2, 1, NA), y = c(2, 1, 1, NA)))
  expect_identical(xtfrm(x), c(2L, 3L, 1L, NA))
})

test_that("Summary generics behave as expected if na.rm = TRUE and all values are NA (#1357)", {
  expect_identical(min(new_vctr(NA_real_), na.rm = TRUE), new_vctr(Inf))
  expect_identical(max(new_vctr(NA_real_), na.rm = TRUE), new_vctr(-Inf))
  expect_identical(
    range(new_vctr(NA_real_), na.rm = TRUE),
    new_vctr(c(Inf, -Inf))
  )

  expect_identical(
    min(new_vctr(NA_integer_), na.rm = TRUE),
    new_vctr(NA_integer_)
  )
  expect_identical(
    max(new_vctr(NA_integer_), na.rm = TRUE),
    new_vctr(NA_integer_)
  )
  expect_identical(
    range(new_vctr(NA_integer_), na.rm = TRUE),
    new_vctr(c(NA_integer_, NA_integer_))
  )

  expect_identical(
    min(new_vctr(NA_character_), na.rm = TRUE),
    new_vctr(NA_character_)
  )
  expect_identical(
    max(new_vctr(NA_character_), na.rm = TRUE),
    new_vctr(NA_character_)
  )
  expect_identical(
    range(new_vctr(NA_character_), na.rm = TRUE),
    new_vctr(c(NA_character_, NA_character_))
  )

  expect_identical(min(new_vctr(NA), na.rm = TRUE), new_vctr(NA))
  expect_identical(max(new_vctr(NA), na.rm = TRUE), new_vctr(NA))
  expect_identical(range(new_vctr(NA), na.rm = TRUE), new_vctr(c(NA, NA)))
})

test_that("Summary generics behave as expected for empty vctrs (#1357)", {
  expect_identical(min(new_vctr(numeric()), na.rm = TRUE), new_vctr(Inf))
  expect_identical(max(new_vctr(numeric()), na.rm = TRUE), new_vctr(-Inf))
  expect_identical(
    range(new_vctr(numeric()), na.rm = TRUE),
    new_vctr(c(Inf, -Inf))
  )

  expect_identical(
    min(new_vctr(integer()), na.rm = TRUE),
    new_vctr(NA_integer_)
  )
  expect_identical(
    max(new_vctr(integer()), na.rm = TRUE),
    new_vctr(NA_integer_)
  )
  expect_identical(
    range(new_vctr(integer()), na.rm = TRUE),
    new_vctr(c(NA_integer_, NA_integer_))
  )

  expect_identical(
    min(new_vctr(character()), na.rm = TRUE),
    new_vctr(NA_character_)
  )
  expect_identical(
    max(new_vctr(character()), na.rm = TRUE),
    new_vctr(NA_character_)
  )
  expect_identical(
    range(new_vctr(character()), na.rm = TRUE),
    new_vctr(c(NA_character_, NA_character_))
  )

  expect_identical(min(new_vctr(logical()), na.rm = TRUE), new_vctr(NA))
  expect_identical(max(new_vctr(logical()), na.rm = TRUE), new_vctr(NA))
  expect_identical(
    range(new_vctr(logical()), na.rm = TRUE),
    new_vctr(c(NA, NA))
  )
})

test_that("anyNA(recursive = TRUE) works with lists (#1278)", {
  x <- list_of(1:4, c(2, NA, 5))
  expect_false(anyNA(x))
  expect_true(anyNA(x, recursive = TRUE))

  x <- new_vctr(list(1:4, list(c(2, NA, 5))))
  expect_false(anyNA(x))
  expect_true(anyNA(x, recursive = TRUE))
})

# The dimensionality of an matrix/array is partition into two parts:
# * the first dimension = the number of observations
# * all other dimensions = the shape parameter of the type
# These helpers work with the shape parameter

new_shape <- function(type, shape = integer()) {
  structure(type, dim = c(0L, shape))
}

vec_shaped_ptype <- function(ptype, x, y, ..., x_arg = "", y_arg = "") {
  check_dots_empty0(...)
  .Call(ffi_vec_shaped_ptype, ptype, x, y, environment())
}

vec_shape2 <- function(x, y, ..., x_arg = "", y_arg = "") {
  check_dots_empty0(...)
  .Call(ffi_vec_shape2, x, y, environment())
}

# Should take same signature as `vec_cast()`
shape_broadcast <- function(x, to, ..., x_arg, to_arg, call = caller_env()) {
  if (is.null(x) || is.null(to)) {
    return(x)
  }

  dim_x <- vec_dim(x)
  dim_to <- vec_dim(to)

  # Don't set dimensions for vectors
  if (length(dim_x) == 1L && length(dim_to) == 1L) {
    return(x)
  }

  if (length(dim_x) > length(dim_to)) {
    details <- sprintf(
      "Can't decrease dimensionality from %s to %s.",
      length(dim_x),
      length(dim_to)
    )
    stop_incompatible_cast(
      x,
      to,
      details = details,
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    )
  }

  dim_x <- n_dim2(dim_x, dim_to)$x
  dim_to[[1]] <- dim_x[[1]] # don't change number of observations
  ok <- dim_x == dim_to | dim_x == 1
  if (any(!ok)) {
    stop_incompatible_cast(
      x,
      to,
      details = "Non-recyclable dimensions.",
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    )
  }

  # Increase dimensionality if required
  if (vec_dim_n(x) != length(dim_x)) {
    dim(x) <- dim_x
  }

  recycle <- dim_x != dim_to

  # Avoid expensive subset
  if (all(!recycle)) {
    return(x)
  }

  indices <- rep(list(missing_arg()), length(dim_to))
  indices[recycle] <- map(dim_to[recycle], rep_len, x = 1L)

  eval_bare(expr(x[!!!indices, drop = FALSE]))
}

# Helpers -----------------------------------------------------------------

n_dim2 <- function(x, y) {
  nx <- length(x)
  ny <- length(y)

  if (nx == ny) {
    list(x = x, y = y)
  } else if (nx < ny) {
    list(x = c(x, rep(1L, ny - nx)), y = y)
  } else {
    list(x = x, y = c(y, rep(1L, nx - ny)))
  }
}

#' Create a vector of locations
#'
#' @description
#'
#' These helpers provide a means of standardizing common indexing
#' methods such as integer, character or logical indexing.
#'
#' * `vec_as_location()` accepts integer, character, or logical vectors
#'   of any size. The output is always an integer vector that is
#'   suitable for subsetting with `[` or [vec_slice()]. It might be a
#'   different size than the input because negative selections are
#'   transformed to positive ones and logical vectors are transformed
#'   to a vector of indices for the `TRUE` locations.
#'
#' * `vec_as_location2()` accepts a single number or string. It returns
#'   a single location as a integer vector of size 1. This is suitable
#'   for extracting with `[[`.
#'
#' * `num_as_location()` and `num_as_location2()` are specialized variants
#'   that have extra options for numeric indices.
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams rlang::args_dots_empty
#'
#' @param i An index vector to convert.
#'
#' @param n A single integer representing the total size of the
#'   object that `i` is meant to index into.
#'
#' @param names If `i` is a character vector, `names` should be a character
#'   vector that `i` will be matched against to construct the index. Otherwise,
#'   not used. The default value of `NULL` will result in an error
#'   if `i` is a character vector.
#'
#' @param missing How should missing `i` values be handled?
#'   - `"error"` throws an error.
#'   - `"propagate"` returns them as is.
#'   - `"remove"` removes them.
#'
#'   By default, vector subscripts propagate missing values but scalar
#'   subscripts error on them.
#'
#'   Propagated missing values can't be combined with negative indices when
#'   `negative = "invert"`, because they can't be meaningfully inverted.
#'
#' @param arg The argument name to be displayed in error messages.
#'
#' @return
#' - `vec_as_location()` and `num_as_location()` return an integer vector that
#'   can be used as an index in a subsetting operation.
#'
#' - `vec_as_location2()` and `num_as_location2()` return an integer of size 1
#'   that can be used a scalar index for extracting an element.
#'
#' @examples
#' x <- array(1:6, c(2, 3))
#' dimnames(x) <- list(c("r1", "r2"), c("c1", "c2", "c3"))
#'
#' # The most common use case validates row indices
#' vec_as_location(1, vec_size(x))
#'
#' # Negative indices can be used to index from the back
#' vec_as_location(-1, vec_size(x))
#'
#' # Character vectors can be used if `names` are provided
#' vec_as_location("r2", vec_size(x), rownames(x))
#'
#' # You can also construct an index for dimensions other than the first
#' vec_as_location(c("c2", "c1"), ncol(x), colnames(x))
#'
#' @keywords internal
#' @export
vec_as_location <- function(
  i,
  n,
  names = NULL,
  ...,
  missing = c("propagate", "remove", "error"),
  arg = caller_arg(i),
  call = caller_env()
) {
  check_dots_empty0(...)

  .Call(
    ffi_as_location,
    i = i,
    n = n,
    names = names,
    loc_negative = "invert",
    loc_oob = "error",
    loc_zero = "remove",
    missing = missing,
    frame = environment()
  )
}

#' @rdname vec_as_location
#'
#' @param negative How should negative `i` values be handled?
#'   - `"error"` throws an error.
#'   - `"ignore"` returns them as is.
#'   - `"invert"` returns the positive location generated by inverting the
#'     negative location. When inverting, positive and negative locations
#'     can't be mixed. This option is only applicable for `num_as_location()`.
#'
#' @param oob How should out-of-bounds `i` values be handled?
#'   - `"error"` throws an error.
#'   - `"remove"` removes both positive and negative out-of-bounds locations.
#'   - `"extend"` allows positive out-of-bounds locations if they directly
#'     follow the end of a vector. This can be used to implement extendable
#'     vectors, like `letters[1:30]`.
#'
#' @param zero How should zero `i` values be handled?
#'   - `"error"` throws an error.
#'   - `"remove"` removes them.
#'   - `"ignore"` returns them as is.
#'
#' @export
num_as_location <- function(
  i,
  n,
  ...,
  missing = c("propagate", "remove", "error"),
  negative = c("invert", "error", "ignore"),
  oob = c("error", "remove", "extend"),
  zero = c("remove", "error", "ignore"),
  arg = caller_arg(i),
  call = caller_env()
) {
  check_dots_empty0(...)

  if (is.object(i) || !(is_integer(i) || is_double(i))) {
    abort("`i` must be a numeric vector.")
  }
  .Call(
    ffi_as_location,
    i = i,
    n = n,
    names = NULL,
    loc_negative = negative,
    loc_oob = oob,
    loc_zero = zero,
    missing = missing,
    env = environment()
  )
}

#' @rdname vec_as_location
#' @export
vec_as_location2 <- function(
  i,
  n,
  names = NULL,
  ...,
  missing = c("error", "propagate"),
  arg = caller_arg(i),
  call = caller_env()
) {
  check_dots_empty0(...)
  result_get(vec_as_location2_result(
    i,
    n = n,
    names = names,
    negative = "error",
    missing = missing,
    arg = arg,
    call = call
  ))
}
#' @rdname vec_as_location
#' @export
num_as_location2 <- function(
  i,
  n,
  ...,
  negative = c("error", "ignore"),
  missing = c("error", "propagate"),
  arg = caller_arg(i),
  call = caller_env()
) {
  check_dots_empty0(...)

  if (!is_integer(i) && !is_double(i)) {
    abort("`i` must be a numeric vector.", call = call)
  }
  result_get(vec_as_location2_result(
    i,
    n = n,
    names = NULL,
    negative = negative,
    missing = missing,
    arg = arg,
    call = call
  ))
}

vec_as_location2_result <- function(i, n, names, missing, negative, arg, call) {
  allow_missing <- arg_match0(missing, c("error", "propagate")) == "propagate"
  allow_negative <- arg_match0(negative, c("error", "ignore")) == "ignore"

  result <- vec_as_subscript2_result(
    i = i,
    arg = arg,
    call = call
  )

  if (!is_null(result$err)) {
    parent <- result$err
    return(result(
      err = new_error_location2_type(
        i = i,
        subscript_arg = arg,
        body = parent$body,
        call = call
      )
    ))
  }

  # Locations must be size 1, can't be NA, and must be positive
  i <- result$ok

  if (length(i) != 1L) {
    return(result(
      err = new_error_location2_type(
        i = i,
        subscript_arg = arg,
        body = cnd_bullets_location2_need_scalar,
        call = call
      )
    ))
  }

  neg <- typeof(i) == "integer" && !is.na(i) && i < 0L
  if (allow_negative && neg) {
    i <- -i
  }

  if (is.na(i)) {
    if (!allow_missing && is.na(i)) {
      result <- result(
        err = new_error_location2_type(
          i = i,
          subscript_arg = arg,
          body = cnd_bullets_location2_need_present,
          call = call
        )
      )
    } else {
      result <- result(i)
    }
    return(result)
  }

  if (identical(i, 0L)) {
    return(result(
      err = new_error_location2_type(
        i = i,
        subscript_arg = arg,
        body = cnd_bullets_location2_need_positive,
        call = call
      )
    ))
  }

  if (!allow_negative && neg) {
    return(result(
      err = new_error_location2_type(
        i = i,
        subscript_arg = arg,
        body = cnd_bullets_location2_need_positive,
        call = call
      )
    ))
  }

  err <- NULL
  i <- tryCatch(
    vec_as_location(i, n, names = names, arg = arg, call = call),
    vctrs_error_subscript = function(err) {
      err[["subscript_scalar"]] <- TRUE
      err <<- err
      i
    }
  )
  if (!is_null(err)) {
    return(result(err = err))
  }

  if (neg) {
    i <- -i
  }

  result(i)
}


stop_location_negative_missing <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i,
    ...,
    body = cnd_body_vctrs_error_location_negative_missing,
    call = call
  ))
}
cnd_body_vctrs_error_location_negative_missing <- function(cnd, ...) {
  missing_loc <- which(is.na(cnd$i))
  arg <- append_arg("Subscript", cnd$subscript_arg)

  if (length(missing_loc) == 1) {
    loc <- glue::glue("{arg} has a missing value at location {missing_loc}.")
  } else {
    n_loc <- length(missing_loc)
    missing_loc <- ensure_full_stop(enumerate(missing_loc))
    loc <- glue::glue(
      "{arg} has {n_loc} missing values at locations {missing_loc}"
    )
  }
  format_error_bullets(c(
    x = "Negative locations can't have missing values.",
    i = loc
  ))
}

stop_location_negative_positive <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i,
    ...,
    body = cnd_body_vctrs_error_location_negative_positive,
    call = call
  ))
}
cnd_body_vctrs_error_location_negative_positive <- function(cnd, ...) {
  positive_loc <- which(cnd$i > 0)
  arg <- append_arg("Subscript", cnd$subscript_arg)

  if (length(positive_loc) == 1) {
    loc <- glue::glue("{arg} has a positive value at location {positive_loc}.")
  } else {
    n_loc <- length(positive_loc)
    positive_loc <- ensure_full_stop(enumerate(positive_loc))
    loc <- glue::glue(
      "{arg} has {n_loc} positive values at locations {positive_loc}"
    )
  }
  format_error_bullets(c(
    x = "Negative and positive locations can't be mixed.",
    i = loc
  ))
}


new_error_location2_type <- function(i, ..., class = NULL) {
  new_error_subscript2_type(
    class = class,
    i = i,
    numeric = "cast",
    character = "cast",
    ...
  )
}
cnd_bullets_location2_need_scalar <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
  format_error_bullets(c(
    x = glue::glue_data(cnd, "{subscript_arg} must be size 1, not {length(i)}.")
  ))
}
cnd_bullets_location2_need_present <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
  format_error_bullets(c(
    x = glue::glue_data(
      cnd,
      "{subscript_arg} must be a location, not {obj_type_friendly(i)}."
    )
  ))
}
cnd_bullets_location2_need_positive <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
  format_error_bullets(c(
    x = glue::glue_data(
      cnd,
      "{subscript_arg} must be a positive location, not {i}."
    )
  ))
}

stop_location_negative <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i,
    body = cnd_bullets_location_need_non_negative,
    ...,
    call = call
  ))
}
cnd_bullets_location_need_non_negative <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)
  format_error_bullets(c(
    x = glue::glue_data(
      cnd,
      "{subscript_arg} can't contain negative locations."
    )
  ))
}

stop_location_zero <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i,
    body = cnd_bullets_location_need_non_zero,
    ...,
    call = call
  ))
}
cnd_bullets_location_need_non_zero <- function(cnd, ...) {
  zero_loc <- which(cnd$i == 0)
  zero_loc_size <- length(zero_loc)
  arg <- append_arg("Subscript", cnd$subscript_arg)

  if (zero_loc_size == 1) {
    loc <- glue::glue("It has a `0` value at location {zero_loc}.")
  } else {
    zero_loc <- ensure_full_stop(enumerate(zero_loc))
    loc <- glue::glue(
      "It has {zero_loc_size} `0` values at locations {zero_loc}"
    )
  }
  format_error_bullets(c(
    x = glue::glue("{arg} can't contain `0` values."),
    i = loc
  ))
}

stop_subscript_missing <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i = i,
    body = cnd_bullets_subscript_missing,
    ...,
    call = call
  ))
}
cnd_bullets_subscript_missing <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)

  missing_loc <- which(is.na(cnd$i))
  if (length(missing_loc) == 1) {
    missing_line <- glue::glue(
      "It has a missing value at location {missing_loc}."
    )
  } else {
    missing_enum <- ensure_full_stop(enumerate(missing_loc))
    missing_line <- glue::glue(
      "It has missing values at locations {missing_enum}"
    )
  }

  format_error_bullets(c(
    x = glue::glue_data(cnd, "{subscript_arg} can't contain missing values."),
    x = missing_line
  ))
}

stop_subscript_empty <- function(i, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_type(
    i = i,
    body = cnd_bullets_subscript_empty,
    ...,
    call = call
  ))
}
cnd_bullets_subscript_empty <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Subscript", cnd$subscript_arg)

  loc <- which(cnd$i == "")
  if (length(loc) == 1) {
    line <- glue::glue("It has an empty string at location {loc}.")
  } else {
    enum <- ensure_full_stop(enumerate(loc))
    line <- glue::glue("It has an empty string at locations {enum}")
  }

  format_error_bullets(c(
    x = glue::glue_data(cnd, "{subscript_arg} can't contain the empty string."),
    x = line
  ))
}

stop_indicator_size <- function(i, n, ..., call = caller_env()) {
  cnd_signal(new_error_subscript_size(
    i,
    n = n,
    ...,
    body = cnd_body_vctrs_error_indicator_size,
    call = call
  ))
}
cnd_body_vctrs_error_indicator_size <- function(cnd, ...) {
  cnd$subscript_arg <- append_arg("Logical subscript", cnd$subscript_arg)
  glue_data_bullets(
    cnd,
    x = "{subscript_arg} must be size 1 or {n}, not {vec_size(i)}."
  )
}

stop_subscript_oob <- function(i, subscript_type, ..., call = caller_env()) {
  stop_subscript(
    class = "vctrs_error_subscript_oob",
    i = i,
    subscript_type = subscript_type,
    ...,
    call = call
  )
}

#' @export
cnd_header.vctrs_error_subscript_oob <- function(cnd, ...) {
  if (cnd_subscript_oob_non_consecutive(cnd)) {
    return(cnd_header_vctrs_error_subscript_oob_non_consecutive(cnd, ...))
  }

  elt <- cnd_subscript_element(cnd)
  action <- cnd_subscript_action(cnd)
  type <- cnd_subscript_type(cnd)

  if (action %in% c("rename", "relocate") || type == "character") {
    glue::glue("Can't {action} {elt[[2]]} that don't exist.")
  } else {
    glue::glue("Can't {action} {elt[[2]]} past the end.")
  }
}

#' @export
cnd_body.vctrs_error_subscript_oob <- function(cnd, ...) {
  switch(
    cnd_subscript_type(cnd),
    numeric = if (cnd_subscript_oob_non_consecutive(cnd)) {
      cnd_body_vctrs_error_subscript_oob_non_consecutive(cnd, ...)
    } else {
      cnd_body_vctrs_error_subscript_oob_location(cnd, ...)
    },
    character = cnd_body_vctrs_error_subscript_oob_name(cnd, ...),
    abort("Internal error: subscript type can't be `logical` for OOB errors.")
  )
}
cnd_body_vctrs_error_subscript_oob_location <- function(cnd, ...) {
  i <- cnd$i

  # In case of missing locations
  i <- i[!is.na(i)]

  if (cnd_subscript_action(cnd) == "negate") {
    # Only report negative indices
    i <- i[i < 0L]
  }

  # In case of negative indexing
  i <- abs(i)

  oob <- i[i > cnd$size]
  oob_enum <- vctrs_cli_vec(oob)

  n_loc <- length(oob)
  n <- cnd$size
  elt <- cnd_subscript_element_cli(n, cnd)

  # TODO: Switch to `format_inline()` and format bullets lazily through rlang
  cli::format_error(c(
    "i" = "{cli::qty(n_loc)} Location{?s} {oob_enum} do{?esn't/n't} exist.",
    "i" = "There {cli::qty(n)} {?is/are} only {elt}."
  ))
}
cnd_body_vctrs_error_subscript_oob_name <- function(cnd, ...) {
  elt <- cnd_subscript_element(cnd, capital = TRUE)
  oob <- cnd$i[!cnd$i %in% cnd$names]
  oob_enum <- enumerate(glue::backtick(oob))

  format_error_bullets(c(
    x = glue::glue(ngettext(
      length(oob),
      "{elt[[1]]} {oob_enum} doesn't exist.",
      "{elt[[2]]} {oob_enum} don't exist."
    ))
  ))
}

vctrs_cli_vec <- function(x, ..., vec_trunc = 5) {
  cli::cli_vec(as.character(x), list(..., vec_trunc = vec_trunc))
}

stop_location_oob_non_consecutive <- function(
  i,
  size,
  ...,
  call = caller_env()
) {
  stop_subscript_oob(
    i = i,
    size = size,
    subscript_type = "numeric",
    subscript_oob_non_consecutive = TRUE,
    ...,
    call = call
  )
}

cnd_header_vctrs_error_subscript_oob_non_consecutive <- function(cnd, ...) {
  action <- cnd_subscript_action(cnd)
  elt <- cnd_subscript_element(cnd)
  glue::glue(
    "Can't {action} {elt[[2]]} beyond the end with non-consecutive locations."
  )
}
cnd_body_vctrs_error_subscript_oob_non_consecutive <- function(cnd, ...) {
  i <- sort(cnd$i)
  i <- i[i > cnd$size]

  non_consecutive <- i[c(TRUE, diff(i) != 1L)]

  arg <- append_arg("Subscript", cnd$subscript_arg)
  if (length(non_consecutive) == 1) {
    x_line <- glue::glue(
      "{arg} contains non-consecutive location {non_consecutive}."
    )
  } else {
    non_consecutive <- ensure_full_stop(enumerate(non_consecutive))
    x_line <- glue::glue(
      "{arg} contains non-consecutive locations {non_consecutive}"
    )
  }

  glue_data_bullets(
    cnd,
    i = "Input has size {size}.",
    x = x_line
  )
}

cnd_subscript_oob_non_consecutive <- function(cnd) {
  out <- cnd$subscript_oob_non_consecutive %||% FALSE
  check_bool(out)
  out
}

test_that("vec_as_location2() returns a position", {
  expect_identical(vec_as_location2(2, 2L), 2L)
  expect_identical(vec_as_location2("foo", 2L, c("bar", "foo")), 2L)
  expect_identical(vec_as_location2("0", 4L, as.character(-1:2)), 2L)
})

test_that("vec_as_location2() requires integer or character inputs", {
  expect_snapshot({
    (expect_error(vec_as_location2(TRUE, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(mtcars, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(env(), 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(foobar(), 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(2.5, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(Inf, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(-Inf, 10L), class = "vctrs_error_subscript_type"))

    "Idem with custom `arg`"
    (expect_error(vec_as_location2(foobar(), 10L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(2.5, 3L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(with_tibble_rows(vec_as_location2(TRUE)), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location() requires integer, character, or logical inputs", {
  expect_snapshot({
    (expect_error(vec_as_location(mtcars, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(env(), 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(foobar(), 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(2.5, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(list(), 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(function() NULL, 10L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(Sys.Date(), 3L), class = "vctrs_error_subscript_type"))

    "Idem with custom `arg`"
    (expect_error(vec_as_location(env(), 10L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(foobar(), 10L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(2.5, 3L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location2() and vec_as_location() require integer- or character-like OO inputs", {
  expect_identical(vec_as_location2(factor("foo"), 2L, c("bar", "foo")), 2L)
  expect_identical(vec_as_location(factor("foo"), 2L, c("bar", "foo")), 2L)
  expect_error(vec_as_location2(foobar(1L), 10L), class = "vctrs_error_subscript_type")
  expect_error(vec_as_location(foobar(1L), 10L), class = "vctrs_error_subscript_type")

  # Define subtype of logical and integer
  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) UseMethod("vec_ptype2.vctrs_foobar"),
    vec_ptype2.vctrs_foobar.logical = function(x, y, ...) logical(),
    vec_ptype2.vctrs_foobar.integer = function(x, y, ...) integer(),
    vec_ptype2.logical.vctrs_foobar = function(x, y, ...) logical(),
    vec_ptype2.integer.vctrs_foobar = function(x, y, ...) integer(),
    vec_cast.vctrs_foobar = function(x, to, ...) UseMethod("vec_cast.vctrs_foobar"),
    vec_cast.vctrs_foobar.integer = function(x, to, ...) foobar(x),
    vec_cast.integer.vctrs_foobar = function(x, to, ...) vec_cast(unclass(x), int()),
    vec_cast.logical.vctrs_foobar = function(x, to, ...) vec_cast(unclass(x), lgl())
  )
  expect_error(vec_as_location2(foobar(TRUE), 10L), class = "vctrs_error_subscript_type")
  expect_identical(vec_as_location(foobar(TRUE), 10L), 1:10)
  expect_identical(vec_as_location(foobar(FALSE), 10L), int())
})

test_that("vec_as_location() and variants check for OOB elements (#1605)", {
  expect_snapshot({
    "Numeric indexing"
    (expect_error(vec_as_location(10L, 2L), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_as_location(-10L, 2L), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_as_location2(10L, 2L), class = "vctrs_error_subscript_oob"))

    "Character indexing"
    (expect_error(vec_as_location("foo", 1L, names = "bar"), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_as_location2("foo", 1L, names = "bar"), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_as_location2("foo", 1L, names = "bar", call = call("baz")), class = "vctrs_error_subscript_oob"))
  })

  expect_error(num_as_location(10L, 2L), class = "vctrs_error_subscript_oob")
  expect_error(num_as_location2(10L, 2L), class = "vctrs_error_subscript_oob")
})

test_that("vec_as_location() doesn't require `n` for character indexing", {
  expect_identical(vec_as_location("b", NULL, names = letters), 2L)
})

test_that("vec_as_location2() requires length 1 inputs", {
  expect_snapshot({
    (expect_error(vec_as_location2(1:2, 2L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(c("foo", "bar"), 2L, c("foo", "bar")), class = "vctrs_error_subscript_type"))

    "Idem with custom `arg`"
    (expect_error(vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(mtcars, 10L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location2() requires positive integers", {
  expect_snapshot({
    (expect_error(vec_as_location2(0, 2L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(-1, 2L), class = "vctrs_error_subscript_type"))

    "Idem with custom `arg`"
    (expect_error(vec_as_location2(0, 2L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location2() fails with NA", {
  expect_snapshot({
    (expect_error(vec_as_location2(na_int, 2L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location2(na_chr, 1L, names = "foo"), class = "vctrs_error_subscript_type"))

    "Idem with custom `arg`"
    (expect_error(vec_as_location2(na_int, 2L, arg = "foo", call = call("my_function")), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location2() doesn't allow lossy casts", {
  expect_error(vec_as_location2(2^31, 3L), class = "vctrs_error_subscript_type")

  # Lossy casts generate missing values, which are disallowed
  expect_error(allow_lossy_cast(vec_as_location2(2^31, 3L)), class = "vctrs_error_subscript_type")
})

test_that("all subscript errors inherit from `vctrs_error_subscript`", {
  expect_error(vec_as_location(100, 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location("foo", 2L, names = c("bar", "baz")), class = "vctrs_error_subscript")
  expect_error(vec_as_location(foobar(1L), 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location(1.5, 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location2(TRUE, 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location2(1.5, 2L), class = "vctrs_error_subscript")
})

test_that("all OOB errors inherit from `vctrs_error_subscript_oob`", {
  expect_error(vec_as_location(100, 2L), class = "vctrs_error_subscript_oob")
  expect_error(vec_as_location("foo", 2L, names = c("bar", "baz")), class = "vctrs_error_subscript_oob")
})

test_that("vec_as_location() preserves names if possible", {
  expect_identical(vec_as_location(c(a = 1L, b = 3L), 3L), c(a = 1L, b = 3L))
  expect_identical(vec_as_location(c(a = 1, b = 3), 3L), c(a = 1L, b = 3L))
  expect_identical(vec_as_location(c(a = "z", b = "y"), 26L, letters), c(a = 26L, b = 25L))

  expect_identical(vec_as_location(c(foo = TRUE, bar = FALSE, baz = TRUE), 3L), c(foo = 1L, baz = 3L))
  expect_identical(vec_as_location(c(foo = TRUE), 3L), c(foo = 1L, foo = 2L, foo = 3L))
  expect_identical(vec_as_location(c(foo = NA), 3L), c(foo = na_int, foo = na_int, foo = na_int))

  # Names of negative selections are dropped
  expect_identical(vec_as_location(c(a = -1L, b = -3L), 3L), 2L)
})

test_that("vec_as_location2() optionally allows missing values", {
  expect_identical(vec_as_location2(NA, 2L, missing = "propagate"), na_int)
  expect_error(vec_as_location2(NA, 2L, missing = "error"), class = "vctrs_error_subscript_type")
})

test_that("num_as_location2() optionally allows missing and negative locations", {
  expect_identical(num_as_location2(na_dbl, 2L, missing = "propagate"), na_int)
  expect_identical(num_as_location2(-1, 2L, negative = "ignore"), -1L)
  expect_error(num_as_location2(-3, 2L, negative = "ignore"), class = "vctrs_error_subscript_oob")
  expect_error(num_as_location2(0, 2L, negative = "ignore"), class = "vctrs_error_subscript_type")
})

test_that("num_as_location() optionally allows negative indices", {
  expect_identical(num_as_location(dbl(1, -1), 2L, negative = "ignore"), int(1L, -1L))
  expect_error(num_as_location(c(1, -10), 2L, negative = "ignore"), class = "vctrs_error_subscript_oob")
})

test_that("num_as_location() optionally forbids negative indices", {
  expect_snapshot({
    (expect_error(num_as_location(dbl(1, -1), 2L, negative = "error"), class = "vctrs_error_subscript_type"))
  })
  expect_error(num_as_location(c(1, -10), 2L, negative = "error"), class = "vctrs_error_subscript_type")
})

test_that("num_as_location() optionally ignores zero indices", {
  expect_identical(num_as_location(c(1, 0), 2L, zero = "ignore"), c(1L, 0L))
})

test_that("num_as_location() optionally forbids zero indices", {
  expect_snapshot({
    (expect_error(
      num_as_location(0L, 1L, zero = "error"),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location(c(0, 0, 0, 0, 0, 0), 1, zero = "error"),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() handles NULL", {
  expect_identical(
    vec_as_location(NULL, 10),
    vec_as_location(int(), 10),
  )
})

test_that("vec_as_location() checks for mix of negative and missing locations", {
  expect_snapshot({
    (expect_error(
      vec_as_location(-c(1L, NA), 30),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(-c(1L, rep(NA, 10)), 30),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() checks for mix of negative and positive locations", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(-1L, 1L), 30),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(-1L, rep(1L, 10)), 30),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("logical subscripts must match size of indexed vector", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(TRUE, FALSE), 3),
      class = "vctrs_error_subscript_size"
    ))
  })
})

test_that("character subscripts require named vectors", {
  expect_snapshot({
    (expect_error(vec_as_location(letters[1], 3), "unnamed vector"))
  })
})

test_that("arg is evaluated lazily (#1150)", {
  expect_silent(vec_as_location(1, 1, arg = { writeLines("oof"); "boo" }))
})

test_that("arg works for complex expressions (#1150)", {
  expect_error(vec_as_location(mean, 1, arg = paste0("foo", "bar")), "foobar")
})

test_that("can optionally extend beyond the end", {
  expect_error(num_as_location(1:5, 3), class = "vctrs_error_subscript_oob")

  expect_identical(num_as_location(1:5, 3, oob = "extend"), 1:5)
  expect_identical(num_as_location(4:5, 3, oob = "extend"), 4:5)

  expect_snapshot({
    (expect_error(
      num_as_location(3, 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1, 3), 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1:5, 7), 3, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1:5, 7, 1), 3, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      class = "vctrs_error_subscript_oob",
      num_as_location(c(1:5, 7, 1, 10), 3, oob = "extend")
    ))
  })
})

test_that("can extend beyond the end consecutively but non-monotonically (#1166)", {
  expect_identical(num_as_location(6:4, 3, oob = "extend"), 6:4)
  expect_identical(num_as_location(c(1:5, 7, 6), 3, oob = "extend"), c(1:5, 7L, 6L))
  expect_identical(num_as_location(c(1, NA, 4, 3), 2, oob = "extend"), c(1L, NA, 4L, 3L))
})

test_that("num_as_location() can optionally remove oob values (#1595)", {
  expect_identical(num_as_location(c(5, 3, 2, 4), 3, oob = "remove"), c(3L, 2L))
  expect_identical(num_as_location(c(-4, 5, 2, -1), 3, oob = "remove", negative = "ignore"), c(2L, -1L))
})

test_that("num_as_location() errors when inverting oob negatives unless `oob = 'remove'` (#1630)", {
  expect_snapshot(error = TRUE, {
    num_as_location(-4, 3, oob = "error", negative = "invert")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-4, 4, 5), 3, oob = "extend", negative = "invert")
  })
  expect_identical(num_as_location(-4, 3, oob = "remove", negative = "invert"), c(1L, 2L, 3L))
  expect_identical(num_as_location(c(-4, -2), 3, oob = "remove", negative = "invert"), c(1L, 3L))
})

test_that("num_as_location() generally drops zeros when inverting negatives (#1612)", {
  expect_identical(
    num_as_location(c(-3, 0, -1), n = 5L, negative = "invert", zero = "remove"),
    c(2L, 4L, 5L)
  )

  # Trying to "ignore" and retain the zeroes in the output doesn't make sense,
  # where would they be placed? Instead, think of the ignored zeros as being
  # inverted as well, they just don't correspond to any location after the
  # inversion so they aren't in the output.
  expect_identical(
    num_as_location(c(-3, 0, -1, 0), n = 5L, negative = "invert", zero = "ignore"),
    c(2L, 4L, 5L)
  )
})

test_that("num_as_location() errors on disallowed zeros when inverting negatives (#1612)", {
  expect_snapshot(error = TRUE, {
    num_as_location(c(0, -1), n = 2L, negative = "invert", zero = "error")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-1, 0), n = 2L, negative = "invert", zero = "error")
  })
})

test_that("num_as_location() with `oob = 'remove'` doesn't remove missings if they are being propagated", {
  expect_identical(num_as_location(NA_integer_, 1, oob = "remove"), NA_integer_)
})

test_that("num_as_location() with `oob = 'remove'` doesn't remove zeros if they are being ignored", {
  expect_identical(num_as_location(0, 1, oob = "remove", zero = "ignore"), 0L)
  expect_identical(num_as_location(0, 0, oob = "remove", zero = "ignore"), 0L)
})

test_that("num_as_location() with `oob = 'extend'` doesn't allow ignored oob negative values (#1614)", {
  # This is fine (ignored negative that is in bounds)
  expect_identical(num_as_location(c(-5L, 6L), 5L, oob = "extend", negative = "ignore"), c(-5L, 6L))

  expect_snapshot(error = TRUE, {
    # Ignored negatives aren't allowed to extend the vector
    num_as_location(-6L, 5L, oob = "extend", negative = "ignore")
  })
  expect_snapshot(error = TRUE, {
    # Ensure error only reports negative indices
    num_as_location(c(-7L, 6L), 5L, oob = "extend", negative = "ignore")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-7L, NA), 5L, oob = "extend", negative = "ignore")
  })
})

test_that("num_as_location() with `oob = 'error'` reports negative and positive oob values", {
  expect_snapshot(error = TRUE, {
    num_as_location(c(-6L, 7L), n = 5L, oob = "error", negative = "ignore")
  })
})

test_that("num_as_location() with `missing = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = NA, c = 2, d = NA)
  expect_named(num_as_location(x, n = 2, missing = "remove"), c("a", "c"))
})

test_that("num_as_location() with `zero = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = 0, c = 2, d = 0)
  expect_named(num_as_location(x, n = 2, zero = "remove"), c("a", "c"))
})

test_that("num_as_location() with `oob = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = 3, c = 2, d = 4)
  expect_named(num_as_location(x, n = 2, oob = "remove"), c("a", "c"))
})

test_that("num_as_location() with `negative = 'invert'` drops names (#1633)", {
  # The inputs don't map 1:1 to outputs
  x <- c(a = -1, b = -3)
  expect_named(num_as_location(x, n = 5), NULL)
})

test_that("missing values are supported in error formatters", {
  expect_snapshot({
    (expect_error(
      num_as_location(c(1, NA, 2, 3), 1),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1, NA, 3), 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("can disallow missing values", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(1, NA), 2, missing = "error"),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(1, NA, 2, NA), 2, missing = "error", arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(1, NA, 2, NA), 2, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(NA, 1, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(NA, 3, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(TRUE, NA, FALSE), 3, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(NA_character_, 2, missing = "error", names = c("x", "y"))),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can alter logical missing value handling (#1595)", {
  x <- c(a = TRUE, b = NA, c = FALSE, d = NA)

  expect_identical(
    vec_as_location(x, n = 4L, missing = "propagate"),
    c(a = 1L, b = NA, d = NA)
  )
  expect_identical(
    vec_as_location(x, n = 4L, missing = "remove"),
    c(a = 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 4L, missing = "error")
  })

  # Specifically test size 1 case, which has its own special path
  x <- c(a = NA)

  expect_identical(
    vec_as_location(x, n = 2L, missing = "propagate"),
    c(a = NA_integer_, a = NA_integer_)
  )
  expect_identical(
    vec_as_location(x, n = 2L, missing = "remove"),
    named(integer())
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 2L, missing = "error")
  })
})

test_that("can alter character missing value handling (#1595)", {
  x <- c(NA, "z", NA)
  names(x) <- c("a", "b", "c")
  names <- c("x", "z")

  expect_identical(
    vec_as_location(x, n = 2L, names = names, missing = "propagate"),
    set_names(c(NA, 2L, NA), names(x))
  )
  expect_identical(
    vec_as_location(x, n = 2L, names = names, missing = "remove"),
    set_names(2L, "b")
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 2L, names = names, missing = "error")
  })
})

test_that("can alter integer missing value handling (#1595)", {
  x <- c(NA, 1L, NA, 3L)
  names(x) <- c("a", "b", "c", "d")

  expect_identical(
    vec_as_location(x, n = 4L, missing = "propagate"),
    x
  )
  expect_identical(
    vec_as_location(x, n = 4L, missing = "remove"),
    c(b = 1L, d = 3L)
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 4L, missing = "error")
  })
})

test_that("can alter negative integer missing value handling (#1595)", {
  x <- c(-1L, NA, NA, -3L)

  expect_snapshot(error = TRUE, {
    num_as_location(x, n = 4L, missing = "propagate", negative = "invert")
  })
  expect_identical(
    num_as_location(x, n = 4L, missing = "remove", negative = "invert"),
    c(2L, 4L)
  )
  expect_snapshot(error = TRUE, {
    num_as_location(x, n = 4L, missing = "error", negative = "invert")
  })
})

test_that("missing value character indices never match missing value names (#1489)", {
  x <- NA_character_
  names <- NA_character_

  expect_identical(vec_as_location(x, n = 1L, names = names, missing = "propagate"), NA_integer_)
  expect_identical(vec_as_location(x, n = 1L, names = names, missing = "remove"), integer())
})

test_that("empty string character indices never match empty string names (#1489)", {
  names <- c("", "y")

  expect_snapshot(error = TRUE, {
    vec_as_location("", n = 2L, names = names)
  })
  expect_snapshot(error = TRUE, {
    vec_as_location(c("", "y", ""), n = 2L, names = names)
  })
})

test_that("scalar logical `FALSE` and `NA` cases don't modify a shared object (#1633)", {
  x <- vec_as_location(FALSE, n = 2)
  expect_identical(x, integer())

  y <- vec_as_location(c(a = FALSE), n = 2)
  expect_identical(y, named(integer()))
  # Still unnamed
  expect_identical(x, integer())


  x <- vec_as_location(NA, n = 2, missing = "remove")
  expect_identical(x, integer())

  y <- vec_as_location(c(a = FALSE), n = 2, missing = "remove")
  expect_identical(y, named(integer()))
  # Still unnamed
  expect_identical(x, integer())
})

test_that("can customise subscript type errors", {
  expect_snapshot({
    "With custom `arg`"
    (expect_error(
      num_as_location(-1, 2, negative = "error", arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location2(-1, 2, negative = "error", arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(0, 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(na_dbl, 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(c(1, 2), 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(TRUE, FALSE), 3, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_size"
    ))
    (expect_error(
      vec_as_location(c(-1, NA), 3, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(-1, 1), 3, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location(c(1, 4), 2, oob = "extend", arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(0, 1, zero = "error", arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))

    "With tibble columns"
    (expect_error(
      with_tibble_cols(num_as_location(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(num_as_location2(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(0, 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(na_dbl, 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(c(1, 2), 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(TRUE, FALSE), 3)),
      class = "vctrs_error_subscript_size"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(-1, NA), 3)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(-1, 1), 3)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(num_as_location(c(1, 4), 2, oob = "extend")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(num_as_location(0, 1, zero = "error")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can customise OOB errors", {
  expect_snapshot({
    (expect_error(
      vec_slice(set_names(letters), "foo"),
      class = "vctrs_error_subscript_oob"
    ))

    "With custom `arg`"
    (expect_error(
      vec_as_location(30, length(letters), arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location("foo", NULL, letters, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_oob"
    ))

    "With tibble columns"
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), "foo")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), -30)),
      class = "vctrs_error_subscript_oob"
    ))

    "With tibble rows"
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), 1:30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))

    "With tidyselect select"
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))

    "With tidyselect relocate"
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("num_as_location() requires non-S3 inputs", {
  expect_error(num_as_location(factor("foo"), 2), "must be a numeric vector")
})

test_that("vec_as_location() checks dimensionality", {
  expect_snapshot({
    (expect_error(vec_as_location(matrix(TRUE, nrow = 1), 3L), class = "vctrs_error_subscript_type"))
    (expect_error(vec_as_location(array(TRUE, dim = c(1, 1, 1)), 3L), class = "vctrs_error_subscript_type"))
    (expect_error(with_tibble_rows(vec_as_location(matrix(TRUE, nrow = 1), 3L)), class = "vctrs_error_subscript_type"))
  })
})

test_that("vec_as_location() works with vectors of dimensionality 1", {
  expect_identical(vec_as_location(array(TRUE, dim = 1), 3L), 1:3)
})

test_that("vec_as_location() UI", {
  expect_snapshot(error = TRUE, vec_as_location(1, 1L, missing = "bogus"))
})

test_that("num_as_location() UI", {
  expect_snapshot(error = TRUE, num_as_location(1, 1L, missing = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, negative = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, oob = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, zero = "bogus"))
})

test_that("vec_as_location2() UI", {
  expect_snapshot(error = TRUE, vec_as_location2(1, 1L, missing = "bogus"))
})

test_that("vec_as_location() evaluates arg lazily", {
  expect_silent(vec_as_location(1L, 1L, arg = print("oof")))
})

test_that("vec_as_location2() evaluates arg lazily", {
  expect_silent(vec_as_location2(1L, 1L, arg = print("oof")))
  expect_silent(vec_as_location2_result(1L, 1L, names = NULL, arg = print("oof"), missing = "error", negative = "error", call = NULL))
})

test_that("vec_as_location2() returns a position", {
  expect_identical(vec_as_location2(2, 2L), 2L)
  expect_identical(vec_as_location2("foo", 2L, c("bar", "foo")), 2L)
  expect_identical(vec_as_location2("0", 4L, as.character(-1:2)), 2L)
})

test_that("vec_as_location2() requires integer or character inputs", {
  expect_snapshot({
    (expect_error(
      vec_as_location2(TRUE, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(mtcars, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(env(), 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(foobar(), 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(2.5, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(Inf, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(-Inf, 10L),
      class = "vctrs_error_subscript_type"
    ))

    "Idem with custom `arg`"
    (expect_error(
      vec_as_location2(foobar(), 10L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(2.5, 3L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_rows(vec_as_location2(TRUE)),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() requires integer, character, or logical inputs", {
  expect_snapshot({
    (expect_error(
      vec_as_location(mtcars, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(env(), 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(foobar(), 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(2.5, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(list(), 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(function() NULL, 10L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(Sys.Date(), 3L),
      class = "vctrs_error_subscript_type"
    ))

    "Idem with custom `arg`"
    (expect_error(
      vec_as_location(env(), 10L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(foobar(), 10L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(2.5, 3L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location2() and vec_as_location() require integer- or character-like OO inputs", {
  expect_identical(vec_as_location2(factor("foo"), 2L, c("bar", "foo")), 2L)
  expect_identical(vec_as_location(factor("foo"), 2L, c("bar", "foo")), 2L)
  expect_error(
    vec_as_location2(foobar(1L), 10L),
    class = "vctrs_error_subscript_type"
  )
  expect_error(
    vec_as_location(foobar(1L), 10L),
    class = "vctrs_error_subscript_type"
  )

  # Define subtype of logical and integer
  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_foobar")
    },
    vec_ptype2.vctrs_foobar.logical = function(x, y, ...) logical(),
    vec_ptype2.vctrs_foobar.integer = function(x, y, ...) integer(),
    vec_ptype2.logical.vctrs_foobar = function(x, y, ...) logical(),
    vec_ptype2.integer.vctrs_foobar = function(x, y, ...) integer(),
    vec_cast.vctrs_foobar = function(x, to, ...) {
      UseMethod("vec_cast.vctrs_foobar")
    },
    vec_cast.vctrs_foobar.integer = function(x, to, ...) foobar(x),
    vec_cast.integer.vctrs_foobar = function(x, to, ...) {
      vec_cast(unclass(x), int())
    },
    vec_cast.logical.vctrs_foobar = function(x, to, ...) {
      vec_cast(unclass(x), lgl())
    }
  )
  expect_error(
    vec_as_location2(foobar(TRUE), 10L),
    class = "vctrs_error_subscript_type"
  )
  expect_identical(vec_as_location(foobar(TRUE), 10L), 1:10)
  expect_identical(vec_as_location(foobar(FALSE), 10L), int())
})

test_that("vec_as_location() and variants check for OOB elements (#1605)", {
  expect_snapshot({
    "Numeric indexing"
    (expect_error(
      vec_as_location(10L, 2L),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location(-10L, 2L),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location2(10L, 2L),
      class = "vctrs_error_subscript_oob"
    ))

    "Character indexing"
    (expect_error(
      vec_as_location("foo", 1L, names = "bar"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location2("foo", 1L, names = "bar"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location2("foo", 1L, names = "bar", call = call("baz")),
      class = "vctrs_error_subscript_oob"
    ))
  })

  expect_error(num_as_location(10L, 2L), class = "vctrs_error_subscript_oob")
  expect_error(num_as_location2(10L, 2L), class = "vctrs_error_subscript_oob")
})

test_that("vec_as_location() doesn't require `n` for character indexing", {
  expect_identical(vec_as_location("b", NULL, names = letters), 2L)
})

test_that("vec_as_location2() requires length 1 inputs", {
  expect_snapshot({
    (expect_error(
      vec_as_location2(1:2, 2L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(c("foo", "bar"), 2L, c("foo", "bar")),
      class = "vctrs_error_subscript_type"
    ))

    "Idem with custom `arg`"
    (expect_error(
      vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(mtcars, 10L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location2() requires positive integers", {
  expect_snapshot({
    (expect_error(
      vec_as_location2(0, 2L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(-1, 2L),
      class = "vctrs_error_subscript_type"
    ))

    "Idem with custom `arg`"
    (expect_error(
      vec_as_location2(0, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location2() fails with NA", {
  expect_snapshot({
    (expect_error(
      vec_as_location2(na_int, 2L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(na_chr, 1L, names = "foo"),
      class = "vctrs_error_subscript_type"
    ))

    "Idem with custom `arg`"
    (expect_error(
      vec_as_location2(na_int, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location2() doesn't allow lossy casts", {
  expect_error(vec_as_location2(2^31, 3L), class = "vctrs_error_subscript_type")

  # Lossy casts generate missing values, which are disallowed
  expect_error(
    allow_lossy_cast(vec_as_location2(2^31, 3L)),
    class = "vctrs_error_subscript_type"
  )
})

test_that("all subscript errors inherit from `vctrs_error_subscript`", {
  expect_error(vec_as_location(100, 2L), class = "vctrs_error_subscript")
  expect_error(
    vec_as_location("foo", 2L, names = c("bar", "baz")),
    class = "vctrs_error_subscript"
  )
  expect_error(vec_as_location(foobar(1L), 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location(1.5, 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location2(TRUE, 2L), class = "vctrs_error_subscript")
  expect_error(vec_as_location2(1.5, 2L), class = "vctrs_error_subscript")
})

test_that("all OOB errors inherit from `vctrs_error_subscript_oob`", {
  expect_error(vec_as_location(100, 2L), class = "vctrs_error_subscript_oob")
  expect_error(
    vec_as_location("foo", 2L, names = c("bar", "baz")),
    class = "vctrs_error_subscript_oob"
  )
})

test_that("vec_as_location() preserves names if possible", {
  expect_identical(vec_as_location(c(a = 1L, b = 3L), 3L), c(a = 1L, b = 3L))
  expect_identical(vec_as_location(c(a = 1, b = 3), 3L), c(a = 1L, b = 3L))
  expect_identical(
    vec_as_location(c(a = "z", b = "y"), 26L, letters),
    c(a = 26L, b = 25L)
  )

  expect_identical(
    vec_as_location(c(foo = TRUE, bar = FALSE, baz = TRUE), 3L),
    c(foo = 1L, baz = 3L)
  )
  expect_identical(
    vec_as_location(c(foo = TRUE), 3L),
    c(foo = 1L, foo = 2L, foo = 3L)
  )
  expect_identical(
    vec_as_location(c(foo = NA), 3L),
    c(foo = na_int, foo = na_int, foo = na_int)
  )

  # Names of negative selections are dropped
  expect_identical(vec_as_location(c(a = -1L, b = -3L), 3L), 2L)
})

test_that("vec_as_location2() optionally allows missing values", {
  expect_identical(vec_as_location2(NA, 2L, missing = "propagate"), na_int)
  expect_error(
    vec_as_location2(NA, 2L, missing = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("num_as_location2() optionally allows missing and negative locations", {
  expect_identical(num_as_location2(na_dbl, 2L, missing = "propagate"), na_int)
  expect_identical(num_as_location2(-1, 2L, negative = "ignore"), -1L)
  expect_error(
    num_as_location2(-3, 2L, negative = "ignore"),
    class = "vctrs_error_subscript_oob"
  )
  expect_error(
    num_as_location2(0, 2L, negative = "ignore"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("num_as_location() optionally allows negative indices", {
  expect_identical(
    num_as_location(dbl(1, -1), 2L, negative = "ignore"),
    int(1L, -1L)
  )
  expect_error(
    num_as_location(c(1, -10), 2L, negative = "ignore"),
    class = "vctrs_error_subscript_oob"
  )
})

test_that("num_as_location() optionally forbids negative indices", {
  expect_snapshot({
    (expect_error(
      num_as_location(dbl(1, -1), 2L, negative = "error"),
      class = "vctrs_error_subscript_type"
    ))
  })
  expect_error(
    num_as_location(c(1, -10), 2L, negative = "error"),
    class = "vctrs_error_subscript_type"
  )
})

test_that("num_as_location() optionally ignores zero indices", {
  expect_identical(num_as_location(c(1, 0), 2L, zero = "ignore"), c(1L, 0L))
})

test_that("num_as_location() optionally forbids zero indices", {
  expect_snapshot({
    (expect_error(
      num_as_location(0L, 1L, zero = "error"),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location(c(0, 0, 0, 0, 0, 0), 1, zero = "error"),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() handles NULL", {
  expect_identical(
    vec_as_location(NULL, 10),
    vec_as_location(int(), 10),
  )
})

test_that("vec_as_location() checks for mix of negative and missing locations", {
  expect_snapshot({
    (expect_error(
      vec_as_location(-c(1L, NA), 30),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(-c(1L, rep(NA, 10)), 30),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() checks for mix of negative and positive locations", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(-1L, 1L), 30),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(-1L, rep(1L, 10)), 30),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("logical subscripts must match size of indexed vector", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(TRUE, FALSE), 3),
      class = "vctrs_error_subscript_size"
    ))
  })
})

test_that("character subscripts require named vectors", {
  expect_snapshot({
    (expect_error(vec_as_location(letters[1], 3), "unnamed vector"))
  })
})

test_that("arg is evaluated lazily (#1150)", {
  expect_silent(vec_as_location(1, 1, arg = {
    writeLines("oof")
    "boo"
  }))
})

test_that("arg works for complex expressions (#1150)", {
  expect_error(vec_as_location(mean, 1, arg = paste0("foo", "bar")), "foobar")
})

test_that("can optionally extend beyond the end", {
  expect_error(num_as_location(1:5, 3), class = "vctrs_error_subscript_oob")

  expect_identical(num_as_location(1:5, 3, oob = "extend"), 1:5)
  expect_identical(num_as_location(4:5, 3, oob = "extend"), 4:5)

  expect_snapshot({
    (expect_error(
      num_as_location(3, 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1, 3), 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1:5, 7), 3, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1:5, 7, 1), 3, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      class = "vctrs_error_subscript_oob",
      num_as_location(c(1:5, 7, 1, 10), 3, oob = "extend")
    ))
  })
})

test_that("can extend beyond the end consecutively but non-monotonically (#1166)", {
  expect_identical(num_as_location(6:4, 3, oob = "extend"), 6:4)
  expect_identical(
    num_as_location(c(1:5, 7, 6), 3, oob = "extend"),
    c(1:5, 7L, 6L)
  )
  expect_identical(
    num_as_location(c(1, NA, 4, 3), 2, oob = "extend"),
    c(1L, NA, 4L, 3L)
  )
})

test_that("num_as_location() can optionally remove oob values (#1595)", {
  expect_identical(num_as_location(c(5, 3, 2, 4), 3, oob = "remove"), c(3L, 2L))
  expect_identical(
    num_as_location(c(-4, 5, 2, -1), 3, oob = "remove", negative = "ignore"),
    c(2L, -1L)
  )
})

test_that("num_as_location() errors when inverting oob negatives unless `oob = 'remove'` (#1630)", {
  expect_snapshot(error = TRUE, {
    num_as_location(-4, 3, oob = "error", negative = "invert")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-4, 4, 5), 3, oob = "extend", negative = "invert")
  })
  expect_identical(
    num_as_location(-4, 3, oob = "remove", negative = "invert"),
    c(1L, 2L, 3L)
  )
  expect_identical(
    num_as_location(c(-4, -2), 3, oob = "remove", negative = "invert"),
    c(1L, 3L)
  )
})

test_that("num_as_location() generally drops zeros when inverting negatives (#1612)", {
  expect_identical(
    num_as_location(c(-3, 0, -1), n = 5L, negative = "invert", zero = "remove"),
    c(2L, 4L, 5L)
  )

  # Trying to "ignore" and retain the zeroes in the output doesn't make sense,
  # where would they be placed? Instead, think of the ignored zeros as being
  # inverted as well, they just don't correspond to any location after the
  # inversion so they aren't in the output.
  expect_identical(
    num_as_location(
      c(-3, 0, -1, 0),
      n = 5L,
      negative = "invert",
      zero = "ignore"
    ),
    c(2L, 4L, 5L)
  )
})

test_that("num_as_location() errors on disallowed zeros when inverting negatives (#1612)", {
  expect_snapshot(error = TRUE, {
    num_as_location(c(0, -1), n = 2L, negative = "invert", zero = "error")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-1, 0), n = 2L, negative = "invert", zero = "error")
  })
})

test_that("num_as_location() with `oob = 'remove'` doesn't remove missings if they are being propagated", {
  expect_identical(num_as_location(NA_integer_, 1, oob = "remove"), NA_integer_)
})

test_that("num_as_location() with `oob = 'remove'` doesn't remove zeros if they are being ignored", {
  expect_identical(num_as_location(0, 1, oob = "remove", zero = "ignore"), 0L)
  expect_identical(num_as_location(0, 0, oob = "remove", zero = "ignore"), 0L)
})

test_that("num_as_location() with `oob = 'extend'` doesn't allow ignored oob negative values (#1614)", {
  # This is fine (ignored negative that is in bounds)
  expect_identical(
    num_as_location(c(-5L, 6L), 5L, oob = "extend", negative = "ignore"),
    c(-5L, 6L)
  )

  expect_snapshot(error = TRUE, {
    # Ignored negatives aren't allowed to extend the vector
    num_as_location(-6L, 5L, oob = "extend", negative = "ignore")
  })
  expect_snapshot(error = TRUE, {
    # Ensure error only reports negative indices
    num_as_location(c(-7L, 6L), 5L, oob = "extend", negative = "ignore")
  })
  expect_snapshot(error = TRUE, {
    num_as_location(c(-7L, NA), 5L, oob = "extend", negative = "ignore")
  })
})

test_that("num_as_location() with `oob = 'error'` reports negative and positive oob values", {
  expect_snapshot(error = TRUE, {
    num_as_location(c(-6L, 7L), n = 5L, oob = "error", negative = "ignore")
  })
})

test_that("num_as_location() with `missing = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = NA, c = 2, d = NA)
  expect_named(num_as_location(x, n = 2, missing = "remove"), c("a", "c"))
})

test_that("num_as_location() with `zero = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = 0, c = 2, d = 0)
  expect_named(num_as_location(x, n = 2, zero = "remove"), c("a", "c"))
})

test_that("num_as_location() with `oob = 'remove'` retains names (#1633)", {
  x <- c(a = 1, b = 3, c = 2, d = 4)
  expect_named(num_as_location(x, n = 2, oob = "remove"), c("a", "c"))
})

test_that("num_as_location() with `negative = 'invert'` drops names (#1633)", {
  # The inputs don't map 1:1 to outputs
  x <- c(a = -1, b = -3)
  expect_named(num_as_location(x, n = 5), NULL)
})

test_that("missing values are supported in error formatters", {
  expect_snapshot({
    (expect_error(
      num_as_location(c(1, NA, 2, 3), 1),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(c(1, NA, 3), 1, oob = "extend"),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("can disallow missing values", {
  expect_snapshot({
    (expect_error(
      vec_as_location(c(1, NA), 2, missing = "error"),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(
        c(1, NA, 2, NA),
        2,
        missing = "error",
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(1, NA, 2, NA), 2, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(NA, 1, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(NA, 3, missing = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(
        c(TRUE, NA, FALSE),
        3,
        missing = "error"
      )),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(
        NA_character_,
        2,
        missing = "error",
        names = c("x", "y")
      )),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can alter logical missing value handling (#1595)", {
  x <- c(a = TRUE, b = NA, c = FALSE, d = NA)

  expect_identical(
    vec_as_location(x, n = 4L, missing = "propagate"),
    c(a = 1L, b = NA, d = NA)
  )
  expect_identical(
    vec_as_location(x, n = 4L, missing = "remove"),
    c(a = 1L)
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 4L, missing = "error")
  })

  # Specifically test size 1 case, which has its own special path
  x <- c(a = NA)

  expect_identical(
    vec_as_location(x, n = 2L, missing = "propagate"),
    c(a = NA_integer_, a = NA_integer_)
  )
  expect_identical(
    vec_as_location(x, n = 2L, missing = "remove"),
    named(integer())
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 2L, missing = "error")
  })
})

test_that("can alter character missing value handling (#1595)", {
  x <- c(NA, "z", NA)
  names(x) <- c("a", "b", "c")
  names <- c("x", "z")

  expect_identical(
    vec_as_location(x, n = 2L, names = names, missing = "propagate"),
    set_names(c(NA, 2L, NA), names(x))
  )
  expect_identical(
    vec_as_location(x, n = 2L, names = names, missing = "remove"),
    set_names(2L, "b")
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 2L, names = names, missing = "error")
  })
})

test_that("can alter integer missing value handling (#1595)", {
  x <- c(NA, 1L, NA, 3L)
  names(x) <- c("a", "b", "c", "d")

  expect_identical(
    vec_as_location(x, n = 4L, missing = "propagate"),
    x
  )
  expect_identical(
    vec_as_location(x, n = 4L, missing = "remove"),
    c(b = 1L, d = 3L)
  )
  expect_snapshot(error = TRUE, {
    vec_as_location(x, n = 4L, missing = "error")
  })
})

test_that("can alter negative integer missing value handling (#1595)", {
  x <- c(-1L, NA, NA, -3L)

  expect_snapshot(error = TRUE, {
    num_as_location(x, n = 4L, missing = "propagate", negative = "invert")
  })
  expect_identical(
    num_as_location(x, n = 4L, missing = "remove", negative = "invert"),
    c(2L, 4L)
  )
  expect_snapshot(error = TRUE, {
    num_as_location(x, n = 4L, missing = "error", negative = "invert")
  })
})

test_that("missing value character indices never match missing value names (#1489)", {
  x <- NA_character_
  names <- NA_character_

  expect_identical(
    vec_as_location(x, n = 1L, names = names, missing = "propagate"),
    NA_integer_
  )
  expect_identical(
    vec_as_location(x, n = 1L, names = names, missing = "remove"),
    integer()
  )
})

test_that("empty string character indices never match empty string names (#1489)", {
  names <- c("", "y")

  expect_snapshot(error = TRUE, {
    vec_as_location("", n = 2L, names = names)
  })
  expect_snapshot(error = TRUE, {
    vec_as_location(c("", "y", ""), n = 2L, names = names)
  })
})

test_that("scalar logical `FALSE` and `NA` cases don't modify a shared object (#1633)", {
  x <- vec_as_location(FALSE, n = 2)
  expect_identical(x, integer())

  y <- vec_as_location(c(a = FALSE), n = 2)
  expect_identical(y, named(integer()))
  # Still unnamed
  expect_identical(x, integer())

  x <- vec_as_location(NA, n = 2, missing = "remove")
  expect_identical(x, integer())

  y <- vec_as_location(c(a = FALSE), n = 2, missing = "remove")
  expect_identical(y, named(integer()))
  # Still unnamed
  expect_identical(x, integer())
})

test_that("can customise subscript type errors", {
  expect_snapshot({
    "With custom `arg`"
    (expect_error(
      num_as_location(
        -1,
        2,
        negative = "error",
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location2(
        -1,
        2,
        negative = "error",
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(0, 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(na_dbl, 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location2(c(1, 2), 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(
        c(TRUE, FALSE),
        3,
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_size"
    ))
    (expect_error(
      vec_as_location(c(-1, NA), 3, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(c(-1, 1), 3, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      num_as_location(
        c(1, 4),
        2,
        oob = "extend",
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      num_as_location(
        0,
        1,
        zero = "error",
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_type"
    ))

    "With tibble columns"
    (expect_error(
      with_tibble_cols(num_as_location(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(num_as_location2(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(0, 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(na_dbl, 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location2(c(1, 2), 2)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(TRUE, FALSE), 3)),
      class = "vctrs_error_subscript_size"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(-1, NA), 3)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(vec_as_location(c(-1, 1), 3)),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_cols(num_as_location(c(1, 4), 2, oob = "extend")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(num_as_location(0, 1, zero = "error")),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can customise OOB errors", {
  expect_snapshot({
    (expect_error(
      vec_slice(set_names(letters), "foo"),
      class = "vctrs_error_subscript_oob"
    ))

    "With custom `arg`"
    (expect_error(
      vec_as_location(
        30,
        length(letters),
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_as_location(
        "foo",
        NULL,
        letters,
        arg = "foo",
        call = call("my_function")
      ),
      class = "vctrs_error_subscript_oob"
    ))

    "With tibble columns"
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), "foo")),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_cols(vec_slice(set_names(letters), -30)),
      class = "vctrs_error_subscript_oob"
    ))

    "With tibble rows"
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), 1:30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tibble_rows(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))

    "With tidyselect select"
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_select(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))

    "With tidyselect relocate"
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      with_tidyselect_relocate(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("num_as_location() requires non-S3 inputs", {
  expect_error(num_as_location(factor("foo"), 2), "must be a numeric vector")
})

test_that("vec_as_location() checks dimensionality", {
  expect_snapshot({
    (expect_error(
      vec_as_location(matrix(TRUE, nrow = 1), 3L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_as_location(array(TRUE, dim = c(1, 1, 1)), 3L),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      with_tibble_rows(vec_as_location(matrix(TRUE, nrow = 1), 3L)),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("vec_as_location() works with vectors of dimensionality 1", {
  expect_identical(vec_as_location(array(TRUE, dim = 1), 3L), 1:3)
})

test_that("vec_as_location() UI", {
  expect_snapshot(error = TRUE, vec_as_location(1, 1L, missing = "bogus"))
})

test_that("num_as_location() UI", {
  expect_snapshot(error = TRUE, num_as_location(1, 1L, missing = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, negative = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, oob = "bogus"))
  expect_snapshot(error = TRUE, num_as_location(1, 1L, zero = "bogus"))
})

test_that("vec_as_location2() UI", {
  expect_snapshot(error = TRUE, vec_as_location2(1, 1L, missing = "bogus"))
})

test_that("vec_as_location() evaluates arg lazily", {
  expect_silent(vec_as_location(1L, 1L, arg = print("oof")))
})

test_that("vec_as_location2() evaluates arg lazily", {
  expect_silent(vec_as_location2(1L, 1L, arg = print("oof")))
  expect_silent(vec_as_location2_result(
    1L,
    1L,
    names = NULL,
    arg = print("oof"),
    missing = "error",
    negative = "error",
    call = NULL
  ))
})

replace_from <- function(what, pkg, to = topenv(caller_env())) {
  if (what %in% getNamespaceExports(pkg)) {
    env <- ns_env(pkg)
  } else {
    env <- to
  }
  env_get(env, what, inherit = TRUE)
}

# nocov start

# Useful for micro-optimising default arguments requiring evaluation,
# such as `param = c("foo", "bar")`. Buys about 0.6us on my desktop.
fn_inline_formals <- function(fn, names) {
  stopifnot(typeof(fn) == "closure")

  fmls <- formals(fn)
  fmls[names] <- lapply(fmls[names], eval)

  formals(fn) <- fmls
  fn
}

# nocov end

#' @export
vec_proxy_equal.integer64 <- function(x, ...) {
  if (is.array(x)) {
    # Stopgap to convert arrays to data frames, then run them through
    # `vec_proxy_equal()` again, which will proxy each column
    x <- as_data_frame_from_array(x)
    x <- vec_proxy_equal(x)
    return(x)
  }

  integer64_proxy(x)
}

# Print -------------------------------------------------------------------

#' 64 bit integers
#'
#' A `integer64` is a 64 bits integer vector, implemented in the `bit64` package.
#'
#' These functions help the `integer64` class from `bit64` in to
#' the vctrs type system by providing coercion functions
#' and casting functions.
#'
#' @keywords internal
#' @rdname int64
#' @export
vec_ptype_full.integer64 <- function(x, ...) {
  "integer64"
}

#' @rdname int64
#' @export
vec_ptype_abbr.integer64 <- function(x, ...) {
  "int64"
}


# Coerce ------------------------------------------------------------------

#' @export
#' @rdname int64
#' @export vec_ptype2.integer64
#' @method vec_ptype2 integer64
vec_ptype2.integer64 <- function(x, y, ...) {
  UseMethod("vec_ptype2.integer64")
}

#' @method vec_ptype2.integer64 integer64
#' @export
vec_ptype2.integer64.integer64 <- function(x, y, ...) bit64::integer64()

#' @method vec_ptype2.integer64 integer
#' @export
vec_ptype2.integer64.integer <- function(x, y, ...) bit64::integer64()
#' @method vec_ptype2.integer integer64
#' @export
vec_ptype2.integer.integer64 <- function(x, y, ...) bit64::integer64()

#' @method vec_ptype2.integer64 logical
#' @export
vec_ptype2.integer64.logical <- function(x, y, ...) bit64::integer64()
#' @method vec_ptype2.logical integer64
#' @export
vec_ptype2.logical.integer64 <- function(x, y, ...) bit64::integer64()


# Cast --------------------------------------------------------------------

#' @export
#' @rdname int64
#' @export vec_cast.integer64
#' @method vec_cast integer64
vec_cast.integer64 <- function(x, to, ...) {
  UseMethod("vec_cast.integer64")
}

#' @export
#' @method vec_cast.integer64 integer64
vec_cast.integer64.integer64 <- function(x, to, ...) {
  x
}

#' @export
#' @method vec_cast.integer64 integer
vec_cast.integer64.integer <- function(x, to, ...) {
  bit64::as.integer64(x)
}

#' @export
#' @method vec_cast.integer integer64
vec_cast.integer.integer64 <- function(x, to, ...) {
  as.integer(x)
}

#' @export
#' @method vec_cast.integer64 logical
vec_cast.integer64.logical <- function(x, to, ...) {
  bit64::as.integer64(x)
}

#' @export
#' @method vec_cast.logical integer64
vec_cast.logical.integer64 <- function(x, to, ...) {
  as.logical(x)
}

#' @export
#' @method vec_cast.integer64 double
vec_cast.integer64.double <- function(x, to, ...) {
  bit64::as.integer64(x)
}

#' @export
#' @method vec_cast.double integer64
vec_cast.double.integer64 <- function(x, to, ...) {
  as.double(x)
}

# ------------------------------------------------------------------------------

integer64_proxy <- function(x) {
  .Call(vctrs_integer64_proxy, x)
}
integer64_restore <- function(x) {
  .Call(vctrs_integer64_restore, x)
}

# ------------------------------------------------------------------------------

as_data_frame_from_array <- function(x) {
  # Alternative to `as.data.frame.array()` that always strips 1-D arrays
  # of their dimensions. Unlike `as.data.frame2()`, it doesn't unclass the
  # input, which means that each column retains its original class.
  # This function doesn't attempt to keep the names of `x` at all.

  dim <- dim(x)
  n_dim <- length(dim)

  if (n_dim == 1) {
    # Treat 1-D arrays as 1 column matrices
    dim(x) <- c(dim, 1L)
    n_dim <- 2L
  }

  n_row <- dim[[1L]]
  n_col <- prod(dim[-1L])
  n_col_seq <- seq_len(n_col)

  dim(x) <- c(n_row, n_col)

  out <- vector("list", n_col)
  names(out) <- as_unique_names(rep("", n_col), quiet = TRUE)

  for (i in n_col_seq) {
    out[[i]] <- x[, i, drop = TRUE]
  }

  new_data_frame(out, n = n_row)
}

#' Combine many vectors into one vector
#'
#' Combine all arguments into a new vector of common type.
#'
#' @section Invariants:
#' * `vec_size(vec_c(x, y)) == vec_size(x) + vec_size(y)`
#' * `vec_ptype(vec_c(x, y)) == vec_ptype_common(x, y)`.
#'
#' @section Dependencies:
#'
#' ## vctrs dependencies
#'
#' - [vec_cast_common()] with fallback
#' - [vec_proxy()]
#' - [vec_restore()]
#'
#'
#' ## base dependencies
#'
#' - [base::c()]
#'
#' If inputs inherit from a common class hierarchy, `vec_c()` falls
#' back to `base::c()` if there exists a `c()` method implemented for
#' this class hierarchy.
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams vec_ptype_show
#' @inheritParams name_spec
#' @inheritParams vec_as_names
#'
#' @param ... Vectors to coerce.
#' @param .name_repair How to repair names, see `repair` options in
#'   [vec_as_names()].
#' @return A vector with class given by `.ptype`, and length equal to the
#'   sum of the `vec_size()` of the contents of `...`.
#'
#'   The vector will have names if the individual components have names
#'   (inner names) or if the arguments are named (outer names). If both
#'   inner and outer names are present, an error is thrown unless a
#'   `.name_spec` is provided.
#'
#' @seealso [vec_cbind()]/[vec_rbind()] for combining data frames by rows
#'   or columns.
#' @export
#' @examples
#' vec_c(FALSE, 1L, 1.5)
#'
#' # Date/times --------------------------
#' c(Sys.Date(), Sys.time())
#' c(Sys.time(), Sys.Date())
#'
#' vec_c(Sys.Date(), Sys.time())
#' vec_c(Sys.time(), Sys.Date())
#'
#' # Factors -----------------------------
#' c(factor("a"), factor("b"))
#' vec_c(factor("a"), factor("b"))
#'
#'
#' # By default, named inputs must be length 1:
#' vec_c(name = 1)
#' try(vec_c(name = 1:3))
#'
#' # Pass a name specification to work around this:
#' vec_c(name = 1:3, .name_spec = "{outer}_{inner}")
#'
#' # See `?name_spec` for more examples of name specifications.
vec_c <- function(
  ...,
  .ptype = NULL,
  .name_spec = NULL,
  .name_repair = c(
    "minimal",
    "unique",
    "check_unique",
    "universal",
    "unique_quiet",
    "universal_quiet"
  ),
  .error_arg = "",
  .error_call = current_env()
) {
  .External2(ffi_vec_c, list2(...), .ptype, .name_spec, .name_repair)
}
vec_c <- fn_inline_formals(vec_c, ".name_repair")

#' Split a vector into groups
#'
#' This is a generalisation of [split()] that can split by any type of vector,
#' not just factors. Instead of returning the keys in the character names,
#' the are returned in a separate parallel vector.
#'
#' @param x Vector to divide into groups.
#' @param by Vector whose unique values defines the groups.
#' @return A data frame with two columns and size equal to
#'   `vec_size(vec_unique(by))`. The `key` column has the same type as
#'   `by`, and the `val` column is a list containing elements of type
#'   `vec_ptype(x)`.
#'
#'   Note for complex types, the default `data.frame` print method will be
#'   suboptimal, and you will want to coerce into a tibble to better
#'   understand the output.
#' @export
#'
#' @section Dependencies:
#' - [vec_group_loc()]
#' - [vec_chop()]
#'
#' @examples
#' vec_split(mtcars$cyl, mtcars$vs)
#' vec_split(mtcars$cyl, mtcars[c("vs", "am")])
#'
#' if (require("tibble")) {
#'   as_tibble(vec_split(mtcars$cyl, mtcars[c("vs", "am")]))
#'   as_tibble(vec_split(mtcars, mtcars[c("vs", "am")]))
#' }
vec_split <- function(x, by) {
  .Call(vctrs_split, x, by)
}

#' Repeat a vector
#'
#' @description
#' - `vec_rep()` repeats an entire vector a set number of `times`.
#'
#' - `vec_rep_each()` repeats each element of a vector a set number of `times`.
#'
#' - `vec_unrep()` compresses a vector with repeated values. The repeated values
#'   are returned as a `key` alongside the number of `times` each key is
#'   repeated.
#'
#' @details
#' Using `vec_unrep()` and `vec_rep_each()` together is similar to using
#' [base::rle()] and [base::inverse.rle()]. The following invariant shows
#' the relationship between the two functions:
#'
#' ```
#' compressed <- vec_unrep(x)
#' identical(x, vec_rep_each(compressed$key, compressed$times))
#' ```
#'
#' There are two main differences between `vec_unrep()` and [base::rle()]:
#'
#' - `vec_unrep()` treats adjacent missing values as equivalent, while `rle()`
#'   treats them as different values.
#'
#' - `vec_unrep()` works along the size of `x`, while `rle()` works along its
#'   length. This means that `vec_unrep()` works on data frames by compressing
#'   repeated rows.
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams rlang::args_dots_empty
#' @param x A vector.
#' @param times
#'   For `vec_rep()`, a single integer for the number of times to repeat
#'   the entire vector.
#'
#'   For `vec_rep_each()`, an integer vector of the number of times to repeat
#'   each element of `x`. `times` will be [recycled][theory-faq-recycling] to
#'   the size of `x`.
#' @param x_arg,times_arg Argument names for errors.
#'
#' @return
#' For `vec_rep()`, a vector the same type as `x` with size
#' `vec_size(x) * times`.
#'
#' For `vec_rep_each()`, a vector the same type as `x` with size
#' `sum(vec_recycle(times, vec_size(x)))`.
#'
#' For `vec_unrep()`, a data frame with two columns, `key` and `times`. `key`
#' is a vector with the same type as `x`, and `times` is an integer vector.
#'
#' @section Dependencies:
#' - [vec_slice()]
#'
#' @name vec-rep
#' @examples
#' # Repeat the entire vector
#' vec_rep(1:2, 3)
#'
#' # Repeat within each vector
#' vec_rep_each(1:2, 3)
#' x <- vec_rep_each(1:2, c(3, 4))
#' x
#'
#' # After using `vec_rep_each()`, you can recover the original vector
#' # with `vec_unrep()`
#' vec_unrep(x)
#'
#' df <- data.frame(x = 1:2, y = 3:4)
#'
#' # `rep()` repeats columns of data frames, and returns lists
#' rep(df, each = 2)
#'
#' # `vec_rep()` and `vec_rep_each()` repeat rows, and return data frames
#' vec_rep(df, 2)
#' vec_rep_each(df, 2)
#'
#' # `rle()` treats adjacent missing values as different
#' y <- c(1, NA, NA, 2)
#' rle(y)
#'
#' # `vec_unrep()` treats them as equivalent
#' vec_unrep(y)
NULL

#' @rdname vec-rep
#' @export
vec_rep <- function(
  x,
  times,
  ...,
  error_call = current_env(),
  x_arg = "x",
  times_arg = "times"
) {
  check_dots_empty0(...)
  .Call(ffi_vec_rep, x, times, environment())
}

#' @rdname vec-rep
#' @export
vec_rep_each <- function(
  x,
  times,
  ...,
  error_call = current_env(),
  x_arg = "x",
  times_arg = "times"
) {
  check_dots_empty0(...)
  .Call(ffi_vec_rep_each, x, times, environment())
}

#' @rdname vec-rep
#' @export
vec_unrep <- function(x) {
  .Call(ffi_vec_unrep, x, environment())
}

test_that("expands the first column slowest by default", {
  x <- 1:4
  y <- 1:3
  z <- 1:2

  expect_identical(
    vec_expand_grid(x = x, y = y, z = z),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 6), times = 1),
      y = vec_rep(vec_rep_each(y, times = 2), times = 4),
      z = vec_rep(vec_rep_each(z, times = 1), times = 12)
    )
  )
})

test_that("can expand the first column fastest with `.vary`", {
  x <- 1:4
  y <- 1:3
  z <- 1:2

  expect_identical(
    vec_expand_grid(x = x, y = y, z = z, .vary = "fastest"),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 1), times = 6),
      y = vec_rep(vec_rep_each(y, times = 4), times = 2),
      z = vec_rep(vec_rep_each(z, times = 12), times = 1)
    )
  )
})

test_that("size 0 elements force a size 0 result", {
  expect_identical(
    vec_expand_grid(x = 1:3, y = integer(), z = 1:2),
    data_frame(x = integer(), y = integer(), z = integer())
  )

  expect_identical(
    vec_expand_grid(x = integer()),
    data_frame(x = integer())
  )
})

test_that("returns 1 row and 0 cols with no input", {
  # Because `prod(integer()) == 1L`
  expect_identical(vec_expand_grid(), data_frame(.size = 1L))
})

test_that("drops `NULL` values", {
  expect_identical(
    vec_expand_grid(NULL, NULL),
    vec_expand_grid()
  )

  # And that happens before all names checks
  expect_identical(
    vec_expand_grid(x = 1:2, x = NULL, y = 1:3, NULL),
    vec_expand_grid(x = 1:2, y = 1:3)
  )
})

test_that("works with data frame inputs", {
  x <- data_frame(a = 1:2, b = 2:3)
  y <- 1:3

  expect_identical(
    vec_expand_grid(x = x, y = y),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 3), times = 1),
      y = vec_rep(vec_rep_each(y, times = 1), times = 2),
    )
  )
})

test_that("`.name_repair` isn't affected by `.vary`", {
  expect <- vec_as_names(c("a", "b", "a", "z"), repair = "unique_quiet")

  expect_named(
    vec_expand_grid(a = 1, b = 2, a = 3, z = 4, .vary = "slowest", .name_repair = "unique_quiet"),
    expect
  )
  expect_named(
    vec_expand_grid(a = 1, b = 2, a = 3, z = 4, .vary = "fastest", .name_repair = "unique_quiet"),
    expect
  )
})

test_that("can use `.name_repair`", {
  expect_identical(
    vec_expand_grid(a = 1:2, a = 2:3, .name_repair = "minimal"),
    data_frame(a = c(1L, 1L, 2L, 2L), a = c(2L, 3L, 2L, 3L), .name_repair = "minimal")
  )
})

test_that("inputs must be named", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(1)
  })
  expect_snapshot(error = TRUE, {
    vec_expand_grid(x = 1, 2, y = 3)
  })
})

test_that("catches duplicate names by default", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(a = 1, a = 2)
  })
})

test_that("errors on non vectors and mentions the element name", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(y = environment())
  })
})

test_that("can adjust the `.error_call`", {
  my_expand_grid <- function() {
    vec_expand_grid(x = environment(), .error_call = current_env())
  }

  expect_snapshot(error = TRUE, {
    my_expand_grid()
  })
})

test_that("errors nicely when expansion results in a size larger than `R_len_t`", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  x <- seq_len((2^31 - 1) / 2)
  y <- 1:3

  expect_snapshot(error = TRUE, {
    vec_expand_grid(x = x, y = y)
  })
})

test_that("errors nicely when expansion results in a size larger than `R_xlen_t`", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  x <- seq_len(2^31 - 1)

  expect_snapshot(error = TRUE, transform = scrub_internal_error_line_number, {
    vec_expand_grid(x = x, y = x)
  })
})

test_that("validates `.vary`", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(.vary = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_expand_grid(.vary = "x")
  })
})

# ------------------------------------------------------------------------------
# vec_rep()

test_that("`vec_rep()` can repeat vectors", {
  expect_identical(vec_rep(1:2, 5), rep(1:2, 5))
  expect_identical(vec_rep(list(1, "x"), 5), rep(list(1, "x"), 5))
})

test_that("`vec_rep()` repeats data frames row wise", {
  x <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_rep(x, 2), vec_slice(x, c(1:2, 1:2)))
})

test_that("`vec_rep()` can repeat 0 `times`", {
  expect_identical(vec_rep(1, 0), numeric())
})

test_that("`vec_rep()` can repeat 1 `time`", {
  expect_identical(vec_rep(1:3, 1), 1:3)
})

test_that("`vec_rep()` can repeat `x` of size 1", {
  expect_identical(vec_rep(1, 2), c(1, 1))
})

test_that("`vec_rep()` errors on long vector output", {
  # Exact error message may be platform specific
  expect_error(vec_rep(1:2, .Machine$integer.max), "output size must be less than")
})

test_that("`vec_rep()` validates `times`", {
  expect_snapshot({
    (expect_error(my_vec_rep(1, "x"), class = "vctrs_error_incompatible_type"))
    (expect_error(my_vec_rep(1, c(1, 2))))
    (expect_error(my_vec_rep(1, -1)))
    (expect_error(my_vec_rep(1, NA_integer_)))
  })
})

# ------------------------------------------------------------------------------
# vec_rep_each()

test_that("`vec_rep_each()` can repeat each element of vectors", {
  expect_identical(vec_rep_each(1:2, 5), rep(1:2, each = 5))
  expect_identical(vec_rep_each(list(1, "x"), 5), rep(list(1, "x"), each = 5))
})

test_that("`vec_rep_each()` `times` is vectorized", {
  expect_identical(vec_rep_each(1:2, 1:2), rep(1:2, times = 1:2))
})

test_that("`vec_rep_each()` repeats data frames row wise", {
  x <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_rep_each(x, c(2, 1)), vec_slice(x, c(1, 1, 2)))
})

test_that("`vec_rep_each()` can repeat 0 `times`", {
  expect_identical(vec_rep_each(1:2, 0), integer())
})

test_that("`vec_rep_each()` finalizes type when repeating 0 times (#1673)", {
  expect_identical(vec_rep_each(NA, 0), logical())
})

test_that("`vec_rep_each()` retains names when repeating 0 times (#1673)", {
  x <- c(a = 1, b = 2)
  expect_identical(vec_rep_each(x, 0), named(numeric()))
})

test_that("`vec_rep_each()` can repeat 1 `time`", {
  expect_identical(vec_rep_each(1:2, 1), 1:2)
})

test_that("`vec_rep_each()` errors on long vector output", {
  # Exact error message may be platform specific
  expect_error(vec_rep_each(1:2, .Machine$integer.max), "output size must be less than")
})

test_that("`vec_rep_each()` validates `times`", {
  expect_snapshot({
    (expect_error(my_vec_rep_each(1, "x"), class = "vctrs_error_incompatible_type"))
    (expect_error(my_vec_rep_each(1, -1)))
    (expect_error(my_vec_rep_each(c(1, 2), c(1, -1))))
    (expect_error(my_vec_rep_each(1, NA_integer_)))
    (expect_error(my_vec_rep_each(c(1, 2), c(1, NA_integer_))))
  })
})

test_that("`vec_rep_each()` uses recyclying errors", {
  expect_snapshot({
    (expect_error(my_vec_rep_each(1:2, 1:3), class = "vctrs_error_recycle_incompatible_size"))
  })
})

# ------------------------------------------------------------------------------

test_that("`vec_rep()` validates `times`", {
  expect_snapshot(error = TRUE, my_vec_rep(1, "x"))
  expect_snapshot(error = TRUE, my_vec_rep(1, c(1, 2)))
  expect_snapshot(error = TRUE, my_vec_rep(1, -1))
  expect_snapshot(error = TRUE, my_vec_rep(1, NA_integer_))
})

test_that("`vec_rep_each()` validates `times`", {
  expect_snapshot(error = TRUE, my_vec_rep_each(1, "x"))
  expect_snapshot(error = TRUE, my_vec_rep_each(1, -1))
  expect_snapshot(error = TRUE, my_vec_rep_each(c(1, 2), c(1, -1)))
  expect_snapshot(error = TRUE, my_vec_rep_each(1, NA_integer_))
  expect_snapshot(error = TRUE, my_vec_rep_each(c(1, 2), c(1, NA_integer_)))
})

test_that("`vec_rep_each()` uses recyclying errors", {
  expect_snapshot(error = TRUE, my_vec_rep_each(1:2, 1:3))
})

# vec_unrep --------------------------------------------------------------------

test_that("can unrep a vector", {
  x <- c(1, 3, 3, 1, 5, 5, 6)

  expect <- data_frame(
    key = c(1, 3, 1, 5, 6),
    times = c(1L, 2L, 1L, 2L, 1L)
  )

  expect_identical(vec_unrep(x), expect)
})

test_that("can unrep a data frame", {
  df <- data_frame(
    x = c(1, 1, 2, 2, 2),
    y = c(1, 1, 1, 1, 2)
  )

  expect <- data_frame(
    key = vec_slice(df, c(1, 3, 5)),
    times = c(2L, 2L, 1L)
  )

  expect_identical(vec_unrep(df), expect)
})

test_that("works with size zero input", {
  expect_identical(vec_unrep(integer()), data_frame(key = integer(), times = integer()))
})

test_that("can roundtrip empty input", {
  x <- integer()
  compressed <- vec_unrep(x)
  expect_identical(vec_rep_each(compressed$key, compressed$times), x)

  x <- data_frame()
  compressed <- vec_unrep(x)
  expect_identical(vec_rep_each(compressed$key, compressed$times), x)
})

test_that("works with data frames with rows but no columns", {
  x <- data_frame(.size = 5)
  expect <- data_frame(key = data_frame(.size = 1L), times = 5L)
  expect_identical(vec_unrep(x), expect)
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_unrep(environment())
  })
})

test_that("expands the first column slowest by default", {
  x <- 1:4
  y <- 1:3
  z <- 1:2

  expect_identical(
    vec_expand_grid(x = x, y = y, z = z),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 6), times = 1),
      y = vec_rep(vec_rep_each(y, times = 2), times = 4),
      z = vec_rep(vec_rep_each(z, times = 1), times = 12)
    )
  )
})

test_that("can expand the first column fastest with `.vary`", {
  x <- 1:4
  y <- 1:3
  z <- 1:2

  expect_identical(
    vec_expand_grid(x = x, y = y, z = z, .vary = "fastest"),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 1), times = 6),
      y = vec_rep(vec_rep_each(y, times = 4), times = 2),
      z = vec_rep(vec_rep_each(z, times = 12), times = 1)
    )
  )
})

test_that("size 0 elements force a size 0 result", {
  expect_identical(
    vec_expand_grid(x = 1:3, y = integer(), z = 1:2),
    data_frame(x = integer(), y = integer(), z = integer())
  )

  expect_identical(
    vec_expand_grid(x = integer()),
    data_frame(x = integer())
  )
})

test_that("returns 1 row and 0 cols with no input", {
  # Because `prod(integer()) == 1L`
  expect_identical(vec_expand_grid(), data_frame(.size = 1L))
})

test_that("drops `NULL` values", {
  expect_identical(
    vec_expand_grid(NULL, NULL),
    vec_expand_grid()
  )

  # And that happens before all names checks
  expect_identical(
    vec_expand_grid(x = 1:2, x = NULL, y = 1:3, NULL),
    vec_expand_grid(x = 1:2, y = 1:3)
  )
})

test_that("works with data frame inputs", {
  x <- data_frame(a = 1:2, b = 2:3)
  y <- 1:3

  expect_identical(
    vec_expand_grid(x = x, y = y),
    data_frame(
      x = vec_rep(vec_rep_each(x, times = 3), times = 1),
      y = vec_rep(vec_rep_each(y, times = 1), times = 2),
    )
  )
})

test_that("`.name_repair` isn't affected by `.vary`", {
  expect <- vec_as_names(c("a", "b", "a", "z"), repair = "unique_quiet")

  expect_named(
    vec_expand_grid(
      a = 1,
      b = 2,
      a = 3,
      z = 4,
      .vary = "slowest",
      .name_repair = "unique_quiet"
    ),
    expect
  )
  expect_named(
    vec_expand_grid(
      a = 1,
      b = 2,
      a = 3,
      z = 4,
      .vary = "fastest",
      .name_repair = "unique_quiet"
    ),
    expect
  )
})

test_that("can use `.name_repair`", {
  expect_identical(
    vec_expand_grid(a = 1:2, a = 2:3, .name_repair = "minimal"),
    data_frame(
      a = c(1L, 1L, 2L, 2L),
      a = c(2L, 3L, 2L, 3L),
      .name_repair = "minimal"
    )
  )
})

test_that("inputs must be named", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(1)
  })
  expect_snapshot(error = TRUE, {
    vec_expand_grid(x = 1, 2, y = 3)
  })
})

test_that("catches duplicate names by default", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(a = 1, a = 2)
  })
})

test_that("errors on non vectors and mentions the element name", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(y = environment())
  })
})

test_that("can adjust the `.error_call`", {
  my_expand_grid <- function() {
    vec_expand_grid(x = environment(), .error_call = current_env())
  }

  expect_snapshot(error = TRUE, {
    my_expand_grid()
  })
})

test_that("errors nicely when expansion results in a size larger than `R_len_t`", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  x <- seq_len((2^31 - 1) / 2)
  y <- 1:3

  expect_snapshot(error = TRUE, {
    vec_expand_grid(x = x, y = y)
  })
})

test_that("errors nicely when expansion results in a size larger than `R_xlen_t`", {
  # Windows 32-bit doesn't support long vectors of this size, and the
  # intermediate `r_ssize` will be too large
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  x <- seq_len(2^31 - 1)

  expect_snapshot(error = TRUE, transform = scrub_internal_error_line_number, {
    vec_expand_grid(x = x, y = x)
  })
})

test_that("validates `.vary`", {
  expect_snapshot(error = TRUE, {
    vec_expand_grid(.vary = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_expand_grid(.vary = "x")
  })
})

# ------------------------------------------------------------------------------
# vec_rep()

test_that("`vec_rep()` can repeat vectors", {
  expect_identical(vec_rep(1:2, 5), rep(1:2, 5))
  expect_identical(vec_rep(list(1, "x"), 5), rep(list(1, "x"), 5))
})

test_that("`vec_rep()` repeats data frames row wise", {
  x <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_rep(x, 2), vec_slice(x, c(1:2, 1:2)))
})

test_that("`vec_rep()` can repeat 0 `times`", {
  expect_identical(vec_rep(1, 0), numeric())
})

test_that("`vec_rep()` can repeat 1 `time`", {
  expect_identical(vec_rep(1:3, 1), 1:3)
})

test_that("`vec_rep()` can repeat `x` of size 1", {
  expect_identical(vec_rep(1, 2), c(1, 1))
})

test_that("`vec_rep()` errors on long vector output", {
  # Exact error message may be platform specific
  expect_error(
    vec_rep(1:2, .Machine$integer.max),
    "output size must be less than"
  )
})

test_that("`vec_rep()` validates `times`", {
  expect_snapshot({
    (expect_error(my_vec_rep(1, "x"), class = "vctrs_error_incompatible_type"))
    (expect_error(my_vec_rep(1, c(1, 2))))
    (expect_error(my_vec_rep(1, -1)))
    (expect_error(my_vec_rep(1, NA_integer_)))
  })
})

# ------------------------------------------------------------------------------
# vec_rep_each()

test_that("`vec_rep_each()` can repeat each element of vectors", {
  expect_identical(vec_rep_each(1:2, 5), rep(1:2, each = 5))
  expect_identical(vec_rep_each(list(1, "x"), 5), rep(list(1, "x"), each = 5))
})

test_that("`vec_rep_each()` `times` is vectorized", {
  expect_identical(vec_rep_each(1:2, 1:2), rep(1:2, times = 1:2))
})

test_that("`vec_rep_each()` repeats data frames row wise", {
  x <- data.frame(x = 1:2, y = 3:4)
  expect_identical(vec_rep_each(x, c(2, 1)), vec_slice(x, c(1, 1, 2)))
})

test_that("`vec_rep_each()` can repeat 0 `times`", {
  expect_identical(vec_rep_each(1:2, 0), integer())
})

test_that("`vec_rep_each()` finalizes type when repeating 0 times (#1673)", {
  expect_identical(vec_rep_each(NA, 0), logical())
})

test_that("`vec_rep_each()` retains names when repeating 0 times (#1673)", {
  x <- c(a = 1, b = 2)
  expect_identical(vec_rep_each(x, 0), named(numeric()))
})

test_that("`vec_rep_each()` can repeat 1 `time`", {
  expect_identical(vec_rep_each(1:2, 1), 1:2)
})

test_that("`vec_rep_each()` errors on long vector output", {
  # Exact error message may be platform specific
  expect_error(
    vec_rep_each(1:2, .Machine$integer.max),
    "output size must be less than"
  )
})

test_that("`vec_rep_each()` validates `times`", {
  expect_snapshot({
    (expect_error(
      my_vec_rep_each(1, "x"),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(my_vec_rep_each(1, -1)))
    (expect_error(my_vec_rep_each(c(1, 2), c(1, -1))))
    (expect_error(my_vec_rep_each(1, NA_integer_)))
    (expect_error(my_vec_rep_each(c(1, 2), c(1, NA_integer_))))
  })
})

test_that("`vec_rep_each()` uses recyclying errors", {
  expect_snapshot({
    (expect_error(
      my_vec_rep_each(1:2, 1:3),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

# ------------------------------------------------------------------------------

test_that("`vec_rep()` validates `times`", {
  expect_snapshot(error = TRUE, my_vec_rep(1, "x"))
  expect_snapshot(error = TRUE, my_vec_rep(1, c(1, 2)))
  expect_snapshot(error = TRUE, my_vec_rep(1, -1))
  expect_snapshot(error = TRUE, my_vec_rep(1, NA_integer_))
})

test_that("`vec_rep_each()` validates `times`", {
  expect_snapshot(error = TRUE, my_vec_rep_each(1, "x"))
  expect_snapshot(error = TRUE, my_vec_rep_each(1, -1))
  expect_snapshot(error = TRUE, my_vec_rep_each(c(1, 2), c(1, -1)))
  expect_snapshot(error = TRUE, my_vec_rep_each(1, NA_integer_))
  expect_snapshot(error = TRUE, my_vec_rep_each(c(1, 2), c(1, NA_integer_)))
})

test_that("`vec_rep_each()` uses recyclying errors", {
  expect_snapshot(error = TRUE, my_vec_rep_each(1:2, 1:3))
})

# vec_unrep --------------------------------------------------------------------

test_that("can unrep a vector", {
  x <- c(1, 3, 3, 1, 5, 5, 6)

  expect <- data_frame(
    key = c(1, 3, 1, 5, 6),
    times = c(1L, 2L, 1L, 2L, 1L)
  )

  expect_identical(vec_unrep(x), expect)
})

test_that("can unrep a data frame", {
  df <- data_frame(
    x = c(1, 1, 2, 2, 2),
    y = c(1, 1, 1, 1, 2)
  )

  expect <- data_frame(
    key = vec_slice(df, c(1, 3, 5)),
    times = c(2L, 2L, 1L)
  )

  expect_identical(vec_unrep(df), expect)
})

test_that("works with size zero input", {
  expect_identical(
    vec_unrep(integer()),
    data_frame(key = integer(), times = integer())
  )
})

test_that("can roundtrip empty input", {
  x <- integer()
  compressed <- vec_unrep(x)
  expect_identical(vec_rep_each(compressed$key, compressed$times), x)

  x <- data_frame()
  compressed <- vec_unrep(x)
  expect_identical(vec_rep_each(compressed$key, compressed$times), x)
})

test_that("works with data frames with rows but no columns", {
  x <- data_frame(.size = 5)
  expect <- data_frame(key = data_frame(.size = 1L), times = 5L)
  expect_identical(vec_unrep(x), expect)
})

test_that("errors on scalars", {
  expect_snapshot(error = TRUE, {
    vec_unrep(environment())
  })
})

#' Date, date-time, and duration S3 classes
#'
#' * A `date` ([Date]) is a double vector. Its value represent the number
#'   of days since the Unix "epoch", 1970-01-01. It has no attributes.
#' * A `datetime` ([POSIXct] is a double vector. Its value represents the
#'   number of seconds since the Unix "Epoch", 1970-01-01. It has a single
#'   attribute: the timezone (`tzone`))
#' * A `duration` ([difftime])
#'
#' These function help the base `Date`, `POSIXct`, and `difftime` classes fit
#' into the vctrs type system by providing constructors, coercion functions,
#' and casting functions.
#'
#' @param x A double vector representing the number of days since UNIX
#'   epoch for `new_date()`, number of seconds since UNIX epoch for
#'   `new_datetime()`, and number of `units` for `new_duration()`.
#' @param tzone Time zone. A character vector of length 1. Either `""` for
#'   the local time zone, or a value from [OlsonNames()]
#' @param units Units of duration.
#' @export
#' @keywords internal
#' @examples
#' new_date(0)
#' new_datetime(0, tzone = "UTC")
#' new_duration(1, "hours")
new_date <- function(x = double()) {
  .Call(vctrs_new_date, x)
}

#' @export
#' @rdname new_date
new_datetime <- function(x = double(), tzone = "") {
  .Call(vctrs_new_datetime, x, tzone)
}

#' @export
#' @rdname new_date
new_duration <- function(
  x = double(),
  units = c("secs", "mins", "hours", "days", "weeks")
) {
  stopifnot(is.double(x))
  units <- arg_match0(units, c("secs", "mins", "hours", "days", "weeks"))

  structure(
    x,
    units = units,
    class = "difftime"
  )
}

#' @export
vec_proxy.Date <- function(x, ...) {
  date_validate(x)
}

#' @export
vec_proxy.POSIXct <- function(x, ...) {
  datetime_validate(x)
}

#' @export
vec_proxy.POSIXlt <- function(x, ...) {
  new_data_frame(unclass(x))
}
#' @export
vec_proxy_equal.POSIXlt <- function(x, ...) {
  x <- vec_cast(x, new_datetime(tzone = tzone(x)))
  vec_proxy_equal(x, ...)
}
#' @export
vec_proxy_compare.POSIXlt <- function(x, ...) {
  x <- vec_cast(x, new_datetime(tzone = tzone(x)))
  vec_proxy_compare(x)
}

#' @export
vec_restore.Date <- function(x, to, ...) {
  NextMethod()
}

#' @export
vec_restore.POSIXct <- function(x, to, ...) {
  NextMethod()
}

#' @export
vec_restore.POSIXlt <- function(x, to, ...) {
  NextMethod()
}

# Print ------------------------------------------------------------------

#' @export
vec_ptype_full.Date <- function(x, ...) {
  "date"
}

#' @export
vec_ptype_abbr.Date <- function(x, ...) {
  "date"
}

#' @export
vec_ptype_full.POSIXct <- function(x, ...) {
  tzone <- if (tzone_is_local(x)) "local" else tzone(x)
  paste0("datetime<", tzone, ">")
}

#' @export
vec_ptype_full.POSIXlt <- function(x, ...) {
  tzone <- if (tzone_is_local(x)) "local" else tzone(x)
  paste0("POSIXlt<", tzone, ">")
}

#' @export
vec_ptype_abbr.POSIXct <- function(x, ...) {
  "dttm"
}

#' @export
vec_ptype_abbr.POSIXlt <- function(x, ...) {
  "dttm"
}

#' @export
vec_ptype_full.difftime <- function(x, ...) {
  paste0("duration<", attr(x, "units"), ">")
}

#' @export
vec_ptype_abbr.difftime <- function(x, ...) {
  "drtn"
}

# Ptype -------------------------------------------------------------------

#' @export
vec_ptype.POSIXlt <- function(x, ...) {
  # `vec_ptype2()` pushes towards `POSIXct`. In theory, maybe `vec_ptype()`
  # should as well so that `vec_c(<POSIXlt>)` and `vec_c(<POSIXlt>, <POSIXlt>)`
  # are consistent and both return `POSIXct`. In practice this broke a number of
  # packages (datetimeoffset, slider, nanoarrow), and also generally means that
  # you wouldn't be able to provide a `POSIXlt` as a `ptype` argument to
  # anything and expect it to return a `POSIXlt` as output. So we've decided to
  # just leave this inconsistency. Getting `POSIXlt` attributes exactly right
  # is very tricky, and very R version dependent, so we just consistency align
  # with `[` here and in tests.
  x[0]
}

# Coerce ------------------------------------------------------------------

#' @rdname new_date
#' @export vec_ptype2.Date
#' @method vec_ptype2 Date
#' @export
vec_ptype2.Date <- function(x, y, ...) {
  UseMethod("vec_ptype2.Date")
}
#' @method vec_ptype2.Date Date
#' @export
vec_ptype2.Date.Date <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.Date.Date")
}
#' @method vec_ptype2.Date POSIXct
#' @export
vec_ptype2.Date.POSIXct <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.Date.POSIXct")
}
#' @method vec_ptype2.Date POSIXlt
#' @export
vec_ptype2.Date.POSIXlt <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.Date.POSIXlt")
}

#' @rdname new_date
#' @export vec_ptype2.POSIXct
#' @method vec_ptype2 POSIXct
#' @export
vec_ptype2.POSIXct <- function(x, y, ...) {
  UseMethod("vec_ptype2.POSIXct")
}
#' @method vec_ptype2.POSIXct POSIXct
#' @export
vec_ptype2.POSIXct.POSIXct <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXct.POSIXct")
}
#' @method vec_ptype2.POSIXct Date
#' @export
vec_ptype2.POSIXct.Date <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXct.Date")
}
#' @method vec_ptype2.POSIXct POSIXlt
#' @export
vec_ptype2.POSIXct.POSIXlt <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXct.POSIXlt")
}

#' @rdname new_date
#' @export vec_ptype2.POSIXlt
#' @method vec_ptype2 POSIXlt
#' @export
vec_ptype2.POSIXlt <- function(x, y, ...) {
  UseMethod("vec_ptype2.POSIXlt")
}
#' @method vec_ptype2.POSIXlt POSIXlt
#' @export
vec_ptype2.POSIXlt.POSIXlt <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXlt.POSIXlt")
}
#' @method vec_ptype2.POSIXlt Date
#' @export
vec_ptype2.POSIXlt.Date <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXlt.Date")
}
#' @method vec_ptype2.POSIXlt POSIXct
#' @export
vec_ptype2.POSIXlt.POSIXct <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.POSIXlt.POSIXct")
}

#' @rdname new_date
#' @export vec_ptype2.difftime
#' @method vec_ptype2 difftime
#' @export
vec_ptype2.difftime <- function(x, y, ...) UseMethod("vec_ptype2.difftime")
#' @method vec_ptype2.difftime difftime
#' @export
vec_ptype2.difftime.difftime <- function(x, y, ...) {
  new_duration(units = units_union(x, y))
}

# Cast --------------------------------------------------------------------

#' @rdname new_date
#' @export vec_cast.Date
#' @method vec_cast Date
#' @export
vec_cast.Date <- function(x, to, ...) {
  UseMethod("vec_cast.Date")
}
#' @export
#' @method vec_cast.Date Date
vec_cast.Date.Date <- function(x, to, ...) {
  stop_native_implementation("vec_cast.Date.Date")
}
#' @export
#' @method vec_cast.Date POSIXct
vec_cast.Date.POSIXct <- function(x, to, ...) {
  # TODO: Mark with `stop_native_implementation()` when we use lazy errors
  date_cast(x, to, ...)
}
#' @export
#' @method vec_cast.Date POSIXlt
vec_cast.Date.POSIXlt <- function(x, to, ...) {
  # TODO: Mark with `stop_native_implementation()` when we use lazy errors
  date_cast(x, to, ...)
}

# TODO: Remove when we have lazy errors
date_cast <- function(x, to, ..., x_arg = "", to_arg = "") {
  out <- as.Date(x, tz = tzone(x))

  x_ct <- as.POSIXct(x)
  out_ct <- as.POSIXct(as.character(out), tz = tzone(x))
  lossy <- abs(x_ct - out_ct) > 1e-9 & !is.na(x)

  maybe_lossy_cast(out, x, to, lossy, x_arg = x_arg, to_arg = to_arg)
}

#' @rdname new_date
#' @export vec_cast.POSIXct
#' @method vec_cast POSIXct
#' @export
vec_cast.POSIXct <- function(x, to, ...) {
  UseMethod("vec_cast.POSIXct")
}
#' @export
#' @method vec_cast.POSIXct Date
vec_cast.POSIXct.Date <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXct.Date")
}
#' @export
#' @method vec_cast.POSIXct POSIXlt
vec_cast.POSIXct.POSIXlt <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXct.POSIXlt")
}
#' @export
#' @method vec_cast.POSIXct POSIXct
vec_cast.POSIXct.POSIXct <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXct.POSIXct")
}

#' @rdname new_date
#' @export vec_cast.POSIXlt
#' @method vec_cast POSIXlt
#' @export
vec_cast.POSIXlt <- function(x, to, ...) {
  UseMethod("vec_cast.POSIXlt")
}
#' @export
#' @method vec_cast.POSIXlt Date
vec_cast.POSIXlt.Date <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXlt.Date")
}
#' @export
#' @method vec_cast.POSIXlt POSIXlt
vec_cast.POSIXlt.POSIXlt <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXlt.POSIXlt")
}
#' @export
#' @method vec_cast.POSIXlt POSIXct
vec_cast.POSIXlt.POSIXct <- function(x, to, ...) {
  stop_native_implementation("vec_cast.POSIXlt.POSIXct")
}


#' @rdname new_date
#' @export vec_cast.difftime
#' @method vec_cast difftime
#' @export
vec_cast.difftime <- function(x, to, ...) {
  UseMethod("vec_cast.difftime")
}
#' @export
#' @method vec_cast.difftime difftime
vec_cast.difftime.difftime <- function(x, to, ...) {
  if (identical(units(x), units(to))) {
    if (typeof(x) == "integer") {
      # Catch corrupt difftime objects (#1602)
      storage.mode(x) <- "double"
    }
    x
  } else {
    # Hack: I can't see any obvious way of changing the units
    origin <- as.POSIXct(0, origin = "1970-01-01")
    difftime(origin, origin - x, units = units(to))
  }
}


# Arithmetic --------------------------------------------------------------

#' @rdname new_date
#' @export vec_arith.Date
#' @method vec_arith Date
#' @export
vec_arith.Date <- function(op, x, y, ...) UseMethod("vec_arith.Date", y)
#' @rdname new_date
#' @export vec_arith.POSIXct
#' @method vec_arith POSIXct
#' @export
vec_arith.POSIXct <- function(op, x, y, ...) UseMethod("vec_arith.POSIXct", y)
#' @rdname new_date
#' @export vec_arith.POSIXlt
#' @method vec_arith POSIXlt
#' @export
vec_arith.POSIXlt <- function(op, x, y, ...) UseMethod("vec_arith.POSIXlt", y)
#' @rdname new_date
#' @export vec_arith.difftime
#' @method vec_arith difftime
#' @export
vec_arith.difftime <- function(op, x, y, ...) UseMethod("vec_arith.difftime", y)

#' @method vec_arith.Date default
#' @export
vec_arith.Date.default <- function(op, x, y, ...) stop_incompatible_op(op, x, y)
#' @method vec_arith.POSIXct default
#' @export
vec_arith.POSIXct.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}
#' @method vec_arith.POSIXlt default
#' @export
vec_arith.POSIXlt.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}
#' @method vec_arith.difftime default
#' @export
vec_arith.difftime.default <- function(op, x, y, ...) {
  stop_incompatible_op(op, x, y)
}

#' @method vec_arith.Date Date
#' @export
vec_arith.Date.Date <- function(op, x, y, ...) {
  switch(
    op,
    `-` = difftime(x, y, units = "days"),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.POSIXct POSIXct
#' @export
vec_arith.POSIXct.POSIXct <- function(op, x, y, ...) {
  switch(
    op,
    `-` = difftime(x, y, units = "secs"),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.POSIXlt POSIXlt
#' @export
vec_arith.POSIXlt.POSIXlt <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.POSIXct Date
#' @export
vec_arith.POSIXct.Date <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.Date POSIXct
#' @export
vec_arith.Date.POSIXct <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.POSIXlt Date
#' @export
vec_arith.POSIXlt.Date <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.Date POSIXlt
#' @export
vec_arith.Date.POSIXlt <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.POSIXlt POSIXct
#' @export
vec_arith.POSIXlt.POSIXct <- vec_arith.POSIXct.POSIXct
#' @method vec_arith.POSIXct POSIXlt
#' @export
vec_arith.POSIXct.POSIXlt <- vec_arith.POSIXct.POSIXct

#' @method vec_arith.Date numeric
#' @export
vec_arith.Date.numeric <- function(op, x, y, ...) {
  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, x, y), x),
    `-` = vec_restore(vec_arith_base(op, x, y), x),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.numeric Date
#' @export
vec_arith.numeric.Date <- function(op, x, y, ...) {
  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, x, y), y),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.POSIXct numeric
#' @export
vec_arith.POSIXct.numeric <- vec_arith.Date.numeric
#' @method vec_arith.numeric POSIXct
#' @export
vec_arith.numeric.POSIXct <- vec_arith.numeric.Date
#' @method vec_arith.POSIXlt numeric
#' @export
vec_arith.POSIXlt.numeric <- function(op, x, y, ...) {
  vec_arith.POSIXct.numeric(op, as.POSIXct(x), y, ...)
}
#' @method vec_arith.numeric POSIXlt
#' @export
vec_arith.numeric.POSIXlt <- function(op, x, y, ...) {
  vec_arith.numeric.POSIXct(op, x, as.POSIXct(y), ...)
}

#' @method vec_arith.Date difftime
#' @export
vec_arith.Date.difftime <- function(op, x, y, ...) {
  y <- vec_cast(y, new_duration(units = "days"))

  switch(
    op,
    `+` = ,
    `-` = vec_restore(vec_arith_base(op, x, lossy_floor(y, x)), x),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.difftime Date
#' @export
vec_arith.difftime.Date <- function(op, x, y, ...) {
  x <- vec_cast(x, new_duration(units = "days"))

  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, lossy_floor(x, y), y), y),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.POSIXct difftime
#' @export
vec_arith.POSIXct.difftime <- function(op, x, y, ...) {
  y <- vec_cast(y, new_duration(units = "secs"))

  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, x, y), x),
    `-` = vec_restore(vec_arith_base(op, x, y), x),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.difftime POSIXct
#' @export
vec_arith.difftime.POSIXct <- function(op, x, y, ...) {
  x <- vec_cast(x, new_duration(units = "secs"))

  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, x, y), y),
    stop_incompatible_op(op, x, y)
  )
}
#' @method vec_arith.POSIXlt difftime
#' @export
vec_arith.POSIXlt.difftime <- function(op, x, y, ...) {
  vec_arith.POSIXct.difftime(op, as.POSIXct(x), y, ...)
}
#' @method vec_arith.difftime POSIXlt
#' @export
vec_arith.difftime.POSIXlt <- function(op, x, y, ...) {
  vec_arith.difftime.POSIXct(op, x, as.POSIXct(y), ...)
}

#' @method vec_arith.difftime difftime
#' @export
vec_arith.difftime.difftime <- function(op, x, y, ...) {
  # Ensure x and y have same units
  args <- vec_cast_common(x, y)
  x <- args[[1L]]
  y <- args[[2L]]

  switch(
    op,
    `+` = vec_restore(vec_arith_base(op, x, y), x),
    `-` = vec_restore(vec_arith_base(op, x, y), x),
    `/` = vec_arith_base(op, x, y),
    `%/%` = vec_arith_base(op, x, y),
    `%%` = vec_arith_base(op, x, y),
    stop_incompatible_op(op, x, y)
  )
}

#' @method vec_arith.difftime MISSING
#' @export
vec_arith.difftime.MISSING <- function(op, x, y, ...) {
  switch(
    op,
    `-` = vec_restore(-vec_data(x), x),
    `+` = x,
    stop_incompatible_op(op, x, y)
  )
}

#' @method vec_arith.difftime numeric
#' @export
vec_arith.difftime.numeric <- function(op, x, y, ...) {
  vec_restore(vec_arith_base(op, x, y), x)
}
#' @method vec_arith.numeric difftime
#' @export
vec_arith.numeric.difftime <- function(op, x, y, ...) {
  switch(
    op,
    `/` = stop_incompatible_op(op, x, y),
    vec_restore(vec_arith_base(op, x, y), y)
  )
}

# Helpers -----------------------------------------------------------------

# The tz attribute for POSIXlt can have 3 components
# (time zone name, abbreviated name, abbreviated DST name)
tzone <- function(x) {
  attr(x, "tzone")[[1]] %||% ""
}

tzone_is_local <- function(x) {
  identical(tzone(x), "")
}

tzone_union <- function(x, y) {
  if (tzone_is_local(x)) {
    tzone(y)
  } else {
    tzone(x)
  }
}

units_union <- function(x, y) {
  if (identical(units(x), units(y))) {
    units(x)
  } else {
    "secs"
  }
}

date_validate <- function(x) {
  .Call(vctrs_date_validate, x)
}

datetime_validate <- function(x) {
  .Call(vctrs_datetime_validate, x)
}

# as.character.Date() calls format() which tries to guess a simplified format.
# Supplying a known format is faster and much more memory efficient.
date_as_character <- function(x) {
  format(x, format = "%Y-%m-%d")
}

# `as.POSIXlt.character()` tries multiple formats. Supplying
# a known format is much faster and more memory efficient.
chr_date_as_posixlt <- function(x, tzone) {
  as.POSIXlt.character(x, tz = tzone, format = "%Y-%m-%d")
}

# `as.POSIXct.default()` for characters goes through `as.POSIXlt.character()`
chr_date_as_posixct <- function(x, tzone) {
  out <- chr_date_as_posixlt(x, tzone)
  as.POSIXct.POSIXlt(out, tzone)
}

lossy_floor <- function(x, to, x_arg = "", to_arg = "") {
  x_floor <- floor(x)
  lossy <- x != x_floor
  maybe_lossy_cast(x_floor, x, to, lossy, x_arg = x_arg, to_arg = to_arg)
}

# Guarantees the presence of a `tzone` attribute
# by going through `as.POSIXlt.POSIXct()`.
# Useful for testing, since we always try to restore a `tzone`.
as_posixlt <- function(x, tz = "") {
  as.POSIXlt(as.POSIXct(x, tz))
}

# Math --------------------------------------------------------------------

#' @export
vec_math.Date <- function(.fn, .x, ...) {
  stop_unsupported(.x, .fn)
}
#' @export
vec_math.POSIXct <- function(.fn, .x, ...) {
  stop_unsupported(.x, .fn)
}
#' @export
vec_math.POSIXlt <- function(.fn, .x, ...) {
  stop_unsupported(.x, .fn)
}

test_that("date-times have informative types", {
  expect_identical(vec_ptype_abbr(Sys.Date()), "date")
  expect_identical(vec_ptype_full(Sys.Date()), "date")

  expect_identical(vec_ptype_abbr(Sys.time()), "dttm")
  expect_identical(vec_ptype_full(Sys.time()), "datetime<local>")

  expect_identical(vec_ptype_abbr(new_duration(10)), "drtn")
  expect_identical(vec_ptype_full(new_duration(10)), "duration<secs>")
})

test_that("vec_ptype() returns a double date for integer dates", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_ptype(x)))
})

test_that("dates and times are vectors", {
  expect_true(vec_is(Sys.Date()))
  expect_true(vec_is(as.POSIXct("2020-01-01")))
  expect_true(vec_is(as.POSIXlt("2020-01-01")))
})

test_that("vec_cast() converts POSIXct with int representation to double when converting zones", {
  x <- structure(integer(), class = c("POSIXct", "POSIXt"), tzone = "UTC")
  y <- structure(numeric(), class = c("POSIXct", "POSIXt"), tzone = "America/Los_Angeles")
  expect_true(is.double(vec_cast(x, y)))
})

test_that("vec_c() converts POSIXct with int representation to double representation (#540)", {
  time1 <- seq(as.POSIXct("2015-12-01", tz = "UTC"), length.out = 2, by = "days")
  time2 <- vec_c(time1)
  expect_true(is.double(time2))

  time3 <- vec_c(time1, time1)
  expect_true(is.double(time3))
})

test_that("vec_c() and vec_rbind() convert Dates with int representation to double representation (#396)", {
  x <- structure(0L, class = "Date")
  df <- data.frame(x = x)

  expect_true(is.double(vec_c(x)))
  expect_true(is.double(vec_c(x, x)))

  expect_true(is.double(vec_rbind(df)$x))
  expect_true(is.double(vec_rbind(df, df)$x))
})

test_that("vec_c() and vec_ptype() standardize missing `tzone` attributes (#561)", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_ptype(x), "tzone"), "")
  expect_identical(attr(vec_c(x, x), "tzone"), "")
})

# constructor -------------------------------------------------------------

test_that("can create a date", {
  expect_identical(new_date(), structure(double(), class = "Date"))
  expect_identical(new_date(0), structure(0, class = "Date"))
})

test_that("retains input names", {
  expect_named(new_date(c(x = 0)), "x")
})

test_that("drops attributes except names", {
  expect_identical(new_date(structure(1, foo = "bar")), new_date(1))
})

test_that("only allows doubles", {
  expect_error(new_date(1L), "must be a double vector")
  expect_error(new_date("x"), "must be a double vector")
})

test_that("can create a datetime", {
  expect_identical(new_datetime(), structure(double(), class = c("POSIXct", "POSIXt"), tzone = ""))
  expect_identical(new_datetime(0), structure(0, class = c("POSIXct", "POSIXt"), tzone = ""))
})

test_that("retains input names", {
  expect_named(new_datetime(c(x = 0)), "x")
})

test_that("drops attributes except names", {
  expect_identical(new_datetime(structure(1, foo = "bar")), new_datetime(1))
})

test_that("only allows doubles", {
  expect_error(new_datetime(1L), "must be a double vector")
  expect_error(new_datetime("x"), "must be a double vector")
})

test_that("tzone is allowed to be `NULL`", {
  expect_identical(new_datetime(tzone = NULL), new_datetime(tzone = ""))
})

test_that("tzone must be character or `NULL`", {
  expect_error(new_datetime(tzone = 1), "character vector or `NULL`")
})

# coerce ------------------------------------------------------------------

test_that("datetime coercions are symmetric and unchanging", {
  types <- list(
    new_date(),
    new_datetime(),
    new_datetime(tzone = "US/Central"),
    as.POSIXlt(character(), tz = "US/Central"),
    difftime(Sys.time() + 1000, Sys.time()),
    difftime(Sys.time() + 1, Sys.time())
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("tz comes from first non-empty", {
  # On the assumption that if you've set the time zone explicitly it
  # should win

  x <- as.POSIXct("2020-01-01")
  y <- as.POSIXct("2020-01-01", tz = "America/New_York")

  expect_identical(vec_ptype2(x, y), y[0])
  expect_identical(vec_ptype2(y, x), y[0])

  z <- as.POSIXct("2020-01-01", tz = "Pacific/Auckland")
  expect_identical(vec_ptype2(y, z), y[0])
  expect_identical(vec_ptype2(z, y), z[0])
})

test_that("POSIXlt always steered towards POSIXct", {
  dtc <- as.POSIXct("2020-01-01", tz = "UTC")
  dtl <- as.POSIXlt("2020-01-01", tz = "UTC")

  expect_identical(vec_ptype2(dtc, dtl), dtc[0])
  expect_identical(vec_ptype2(dtl, dtc), dtc[0])
  expect_identical(vec_ptype2(dtl, dtl), dtc[0])
})

test_that("vec_ptype2() on a POSIXlt with multiple time zones returns the first", {
  x <- as.POSIXlt(new_datetime(), tz = "Pacific/Auckland")

  expect_identical(attr(x, "tzone"), c("Pacific/Auckland", "NZST", "NZDT"))
  expect_identical(attr(vec_ptype2(x, new_date()), "tzone"), "Pacific/Auckland")
})

test_that("vec_ptype2(<Date>, NA) is symmetric (#687)", {
  date <- new_date()
  expect_identical(
    vec_ptype2(date, NA),
    vec_ptype2(NA, date)
  )
})

test_that("vec_ptype2(<POSIXt>, NA) is symmetric (#687)", {
  time <- Sys.time()
  expect_identical(
    vec_ptype2(time, NA),
    vec_ptype2(NA, time)
  )
})

test_that("vec_ptype2(<difftime>, NA) is symmetric (#687)", {
  dtime <- Sys.time() - Sys.time()
  expect_identical(
    vec_ptype2(dtime, NA),
    vec_ptype2(NA, dtime)
  )
})

test_that("vec_ptype2() standardizes duration storage type to double", {
  x <- structure(1L, units = "secs", class = "difftime")
  expect <- new_duration(double(), units = "secs")
  expect_identical(vec_ptype2(x, x), expect)
})

# cast: dates ---------------------------------------------------------------

test_that("safe casts work as expected", {
  date <- as.Date("2018-01-01")
  datetime_ct <- as.POSIXct(as.character(date))
  datetime_lt <- as.POSIXlt(datetime_ct)

  expect_identical(vec_cast(NULL, date), NULL)
  expect_identical(vec_cast(date, date), date)
  expect_identical(vec_cast(datetime_ct, date), date)
  expect_identical(vec_cast(datetime_lt, date), date)

  missing_date <- new_date(NA_real_)

  expect_identical(vec_cast(missing_date, missing_date), missing_date)
  expect_identical(vec_cast(as.POSIXct(missing_date), missing_date), missing_date)
  expect_identical(vec_cast(as.POSIXlt(missing_date), missing_date), missing_date)

  # These used to be allowed
  expect_error(vec_cast(17532, date), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast("2018-01-01", date), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(date), date), class = "vctrs_error_incompatible_type")
})

test_that("date - datetime cast can be roundtripped", {
  date <- as.Date("2018-01-01")
  datetime <- as.POSIXct("2018-01-01", tz = "America/New_York")

  expect_identical(vec_cast(vec_cast(date, datetime), date), date)
  expect_identical(vec_cast(vec_cast(datetime, date), datetime), datetime)
})

test_that("lossy casts generate error", {
  date <- as.Date("2018-01-01")
  datetime <- as.POSIXct(as.character(date)) + c(0, 3600)
  expect_lossy(vec_cast(datetime, date), vec_c(date, date), x = datetime, to = date)
})

test_that("invalid casts generate error", {
  date <- as.Date("2018-01-01")
  expect_error(vec_cast(integer(), date), class = "vctrs_error_incompatible_type")
})

test_that("can cast NA and unspecified to Date", {
  expect_identical(vec_cast(NA, new_date()), new_date(NA_real_))
  expect_identical(vec_cast(unspecified(2), new_date()), new_date(dbl(NA, NA)))
})

test_that("casting an integer date to another date returns a double date", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_cast(x, x)))
})

test_that("casting an integer POSIXct to a Date returns a double Date", {
  x <- .POSIXct(18000L, tz = "America/New_York")
  expect <- new_date(0)
  expect_identical(vec_cast(x, new_date()), expect)
})

# cast: datetimes -----------------------------------------------------------

test_that("safe casts work as expected", {
  datetime_c <- as.POSIXct("1970-02-01", tz = "UTC")
  datetime_l <- as.POSIXlt("1970-02-01", tz = "UTC")

  expect_identical(vec_cast(NULL, datetime_c), NULL)
  expect_identical(vec_cast(datetime_c, datetime_c), datetime_c)
  expect_identical(vec_cast(datetime_l, datetime_c), datetime_c)
  expect_identical(vec_cast(as.Date(datetime_c), datetime_c), datetime_c)

  expect_identical(vec_cast(NULL, datetime_l), NULL)
  expect_identical(vec_cast(datetime_c, datetime_l), datetime_l)
  expect_identical(vec_cast(datetime_l, datetime_l), datetime_l)
  expect_identical(vec_cast(as.Date(datetime_l), datetime_l), datetime_l)
  expect_error(vec_cast(raw(), datetime_l), class = "vctrs_error_incompatible_type")

  missing_c <- new_datetime(NA_real_, tzone = "UTC")
  missing_l <- as.POSIXlt(missing_c)

  expect_identical(vec_cast(missing_c, missing_c), missing_c)
  expect_identical(vec_cast(missing_l, missing_c), missing_c)
  expect_identical(vec_cast(as.Date(missing_c), missing_c), missing_c)

  expect_identical(vec_cast(missing_l, missing_l), missing_l)
  expect_identical(vec_cast(missing_c, missing_l), missing_l)
  expect_identical(vec_cast(as.Date(missing_l), missing_l), missing_l)

  # These used to be allowed
  expect_error(vec_cast(2678400, datetime_c), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast("1970-02-01", datetime_c), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(datetime_c), datetime_c), class = "vctrs_error_incompatible_type")

  expect_error(vec_cast(2678400, datetime_l), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast("1970-02-01", datetime_l), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(datetime_l), datetime_l), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  datetime <- as.POSIXct("1970-02-01", tz = "UTC")
  expect_error(vec_cast(integer(), datetime), class = "vctrs_error_incompatible_type")
})

test_that("dates become midnight in date-time tzone", {
  date1 <- as.Date("2010-01-01")
  datetime_c <- as.POSIXct(character(), tz = "Pacific/Auckland")
  datetime_l <- as.POSIXlt(character(), tz = "Pacific/Auckland")

  date2_c <- vec_cast(date1, datetime_c)
  expect_identical(tzone(date2_c), "Pacific/Auckland")
  expect_identical(format(date2_c, "%H:%M"), "00:00")

  date2_l <- vec_cast(date1, datetime_l)
  expect_identical(tzone(date2_l), "Pacific/Auckland")
  expect_identical(format(date2_l, "%H:%M"), "00:00")
})

test_that("can cast NA and unspecified to POSIXct and POSIXlt", {
  dtc <- as.POSIXct("2020-01-01")
  dtl <- as.POSIXlt("2020-01-01")
  expect_identical(vec_cast(NA, dtc), vec_init(dtc))
  expect_identical(vec_cast(NA, dtl), vec_init(dtl))
  expect_identical(vec_cast(unspecified(2), dtc), vec_init(dtc, 2))
  expect_identical(vec_cast(unspecified(2), dtl), vec_init(dtl, 2))
})

test_that("changing time zones retains the underlying moment in time", {
  x_ct <- as.POSIXct("2019-01-01", tz = "America/New_York")
  x_lt <- as.POSIXlt(x_ct)

  to_ct <- new_datetime(tzone = "America/Los_Angeles")
  to_lt <- as.POSIXlt(to_ct)

  expect_ct <- x_ct
  attr(expect_ct, "tzone") <- "America/Los_Angeles"
  expect_lt <- as.POSIXlt(expect_ct)

  expect_identical(vec_cast(x_ct, to_ct), expect_ct)
  expect_identical(vec_cast(x_ct, to_lt), expect_lt)
  expect_identical(vec_cast(x_lt, to_ct), expect_ct)
  expect_identical(vec_cast(x_lt, to_lt), expect_lt)
})

test_that("casting to date always retains the zoned year-month-day value", {
  x <- as.POSIXct("2019-01-01", tz = "Asia/Shanghai")
  expect_identical(vec_cast(x, new_date()), as.Date("2019-01-01"))
})

# cast: durations ------------------------------------------------------------

test_that("safe casts work as expected", {
  dt1 <- as.difftime(600, units = "secs")
  dt2 <- as.difftime(10, units = "mins")

  expect_identical(vec_cast(NULL, dt1), NULL)
  expect_identical(vec_cast(dt1, dt1), dt1)
  expect_identical(vec_cast(dt1, dt2), dt2)

  # These used to be allowed
  expect_error(vec_cast(600, dt1), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(dt1), dt1), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  dt <- as.difftime(600, units = "secs")
  expect_error(vec_cast(integer(), dt), class = "vctrs_error_incompatible_type")
})

test_that("can cast NA and unspecified to duration", {
  expect_identical(vec_cast(NA, new_duration()), new_duration(na_dbl))
  expect_identical(vec_cast(unspecified(2), new_duration()), new_duration(dbl(NA, NA)))
})

test_that("casting coerces corrupt integer storage durations to double (#1602)", {
  x <- structure(1L, units = "secs", class = "difftime")
  expect <- new_duration(1, units = "secs")

  expect_identical(vec_cast(x, x), expect)

  # Names are retained through the coercion
  names(x) <- "a"
  expect_named(vec_cast(x, x), "a")
})

# proxy/restore: dates ---------------------------------------------------

test_that("restoring an integer to an integer Date converts to double", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_restore(x, x)))
})

test_that("vec_proxy() returns a double for Dates with int representation", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_proxy(x)))
})

# proxy/restore: datetimes ------------------------------------------------

test_that("restoring an integer to an integer POSIXct converts to double", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_true(is.double(vec_restore(x, x)))
})

test_that("restoring to a POSIXct with no time zone standardizes to an empty string (#561)", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_restore(x, x), "tzone"), "")
})

test_that("restoring to a POSIXlt with no time zone standardizes to an empty string", {
  # Manually create a POSIXlt without a `tzone` attribute.
  # This is just:
  # `x <- as.POSIXlt("1970-01-01")`
  # which usually won't add a `tzone` attribute, but platforms where the local
  # time is UTC attach a `tzone` attribute automatically.
  x <- structure(
    list(
      sec = 0, min = 0L, hour = 0L, mday = 1L,
      mon = 0L, year = 70L, wday = 4L, yday = 0L,
      isdst = 0L, zone = "EST", gmtoff = NA_integer_
    ),
    class = c("POSIXlt", "POSIXt")
  )

  proxy <- vec_proxy(x)

  expect_identical(attr(vec_restore(proxy, x), "tzone"), "")
})

test_that("proxying a POSIXct with no time zone standardizes to an empty string", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_proxy(x), "tzone"), "")
})

test_that("vec_proxy() returns a double for POSIXct with int representation", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_true(is.double(vec_proxy(x)))
})

test_that("POSIXlt roundtrips through proxy and restore", {
  x <- as_posixlt("2020-01-03")
  out <- vec_restore(vec_proxy(x), x)
  expect_identical(out, x)
})

test_that("subclassed Dates / POSIXct / POSIXlt can be restored (#1015)", {
  x <- subclass(new_date(0))
  proxy <- vec_proxy(x)
  expect_identical(vec_restore(proxy, x), x)

  y <- subclass(new_datetime(0))
  proxy <- vec_proxy(y)
  expect_identical(vec_restore(proxy, y), y)

  z <- subclass(as.POSIXlt(new_datetime(0)))
  proxy <- vec_proxy(z)
  expect_identical(vec_restore(proxy, z), z)
})

# arithmetic --------------------------------------------------------------

test_that("default is error", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct("2018-01-02 12:00")
  lt <- as.POSIXlt(dt)
  t <- as.difftime(12, units = "hours")
  f <- factor("x")

  expect_error(vec_arith("+", d, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", dt, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", lt, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", t, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", dt, t), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", lt, t), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", d, t), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("!", t, MISSING()), class = "vctrs_error_incompatible_op")
})

test_that("date-time vs date-time", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct(d)
  lt <- as.POSIXlt(dt)

  expect_error(vec_arith("+", d, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, d), d - d)

  expect_error(vec_arith("+", dt, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, dt), dt - dt)

  expect_error(vec_arith("+", lt, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, lt), lt - lt)


  expect_error(vec_arith("+", d, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, dt), difftime(d, dt))

  expect_error(vec_arith("+", dt, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, d), difftime(dt, d))


  expect_error(vec_arith("+", d, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, lt), difftime(d, lt))

  expect_error(vec_arith("+", lt, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, d), difftime(lt, d))


  expect_error(vec_arith("+", dt, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, lt), difftime(dt, lt))

  expect_error(vec_arith("+", lt, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, dt), difftime(lt, dt))
})

test_that("date-time vs numeric", {
   d <- as.Date("2018-01-01")
   dt <- as.POSIXct("2018-01-01", tz = "America/New_York")
   lt <- as.POSIXlt(dt)

   expect_identical(vec_arith("+", d, 1), d + 1)
   expect_identical(vec_arith("+", 1, d), d + 1)
   expect_identical(vec_arith("-", d, 1), d - 1)
   expect_error(vec_arith("-", 1, d), class = "vctrs_error_incompatible_op")

   expect_identical(vec_arith("+", dt, 1), dt + 1)
   expect_identical(vec_arith("+", 1, dt), dt + 1)
   expect_identical(vec_arith("-", dt, 1), dt - 1)
   expect_error(vec_arith("-", 1, dt), class = "vctrs_error_incompatible_op")

   expect_identical(vec_arith("+", lt, 1), lt + 1)
   expect_identical(vec_arith("+", 1, lt), lt + 1)
   expect_identical(vec_arith("-", lt, 1), lt - 1)
   expect_error(vec_arith("-", 1, lt), class = "vctrs_error_incompatible_op")


   expect_error(vec_arith("*", 1, d), class = "vctrs_error_incompatible_op")
   expect_error(vec_arith("*", d, 1), class = "vctrs_error_incompatible_op")

   expect_error(vec_arith("*", 1, dt), class = "vctrs_error_incompatible_op")
   expect_error(vec_arith("*", dt, 1), class = "vctrs_error_incompatible_op")

   expect_error(vec_arith("*", 1, lt), class = "vctrs_error_incompatible_op")
   expect_error(vec_arith("*", lt, 1), class = "vctrs_error_incompatible_op")
})

test_that("POSIXlt + numeric = POSIXct", {
  lt <- as.POSIXlt("2018-01-01", tz = "America/New_York")
  expect_s3_class(vec_arith("+", lt, 1), "POSIXct")
  expect_s3_class(vec_arith("+", 1, lt), "POSIXct")
})

test_that("vec_arith() standardizes the `tzone` attribute", {
  dt <- structure(0, class = c("POSIXct", "POSIXt"))
  x <- vec_arith("+", dt, 1)
  expect_identical(attr(x, "tzone"), "")
})

test_that("date-time vs difftime", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct("2018-01-01", tz = "UTC")
  lt <- as.POSIXlt(dt)
  t <- as.difftime(1, units = "days")
  th <- as.difftime(c(1, 24), units = "hours")

  expect_identical(vec_arith("+", d, t), d + t)
  expect_identical(vec_arith("+", t, d), t + d)

  expect_identical(vec_arith("+", dt, t), dt + t)
  expect_identical(vec_arith("+", t, dt), t + dt)

  expect_identical(vec_arith("+", lt, t), lt + t)
  expect_identical(vec_arith("+", t, lt), t + lt)


  expect_lossy(vec_arith("+", d, th), d + th, x = t, to = d)
  expect_lossy(vec_arith("+", th, d), th + d, x = t, to = d)

  expect_identical(vec_arith("+", dt, th), dt + th)
  expect_identical(vec_arith("+", th, dt), th + dt)

  expect_identical(vec_arith("+", lt, th), lt + th)
  expect_identical(vec_arith("+", th, lt), th + lt)


  expect_identical(vec_arith("-", d, t), d - t)
  expect_error(vec_arith("-", t, d), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", dt, t), dt - t)
  expect_error(vec_arith("-", t, dt), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", lt, t), lt - t)
  expect_error(vec_arith("-", t, lt), class = "vctrs_error_incompatible_op")


  expect_lossy(vec_arith("-", d, th), d - th, x = t, to = d)
  expect_error(vec_arith("-", th, d), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", dt, th), dt - th)
  expect_error(vec_arith("-", th, dt), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", lt, th), lt - th)
  expect_error(vec_arith("-", th, lt), class = "vctrs_error_incompatible_op")
})

test_that("difftime vs difftime/numeric", {
  t <- as.difftime(12, units = "hours")

  expect_identical(vec_arith("-", t, MISSING()), -t)
  expect_identical(vec_arith("+", t, MISSING()), t)

  expect_identical(vec_arith("-", t, t), t - t)
  expect_identical(vec_arith("-", t, 1), t - 1)
  expect_identical(vec_arith("-", 1, t), 1 - t)

  expect_identical(vec_arith("+", t, t), 2 * t)
  expect_identical(vec_arith("+", t, 1), t + 1)
  expect_identical(vec_arith("+", 1, t), t + 1)

  expect_identical(vec_arith("*", 2, t), 2 * t)
  expect_identical(vec_arith("*", t, 2), 2 * t)
  expect_error(vec_arith("*", t, t), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("/", t, 2), t / 2)
  expect_error(vec_arith("/", 2, t), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("/", t, t), 1)
  expect_identical(vec_arith("%/%", t, t), 1)
  expect_identical(vec_arith("%%", t, t), 0)
})


# Math --------------------------------------------------------------------

test_that("date and date times don't support math", {
  expect_error(vec_math("sum", new_date()), class = "vctrs_error_unsupported")
  expect_error(vec_math("sum", new_datetime()), class = "vctrs_error_unsupported")
  expect_error(vec_math("sum", as.POSIXlt(new_datetime())), class = "vctrs_error_unsupported")
})

test_that("date-times have informative types", {
  expect_identical(vec_ptype_abbr(Sys.Date()), "date")
  expect_identical(vec_ptype_full(Sys.Date()), "date")

  expect_identical(vec_ptype_abbr(Sys.time()), "dttm")
  expect_identical(vec_ptype_full(Sys.time()), "datetime<local>")

  expect_identical(vec_ptype_abbr(new_duration(10)), "drtn")
  expect_identical(vec_ptype_full(new_duration(10)), "duration<secs>")
})

test_that("vec_ptype() returns a double date for integer dates", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_ptype(x)))
})

test_that("dates and times are vectors", {
  expect_true(vec_is(Sys.Date()))
  expect_true(vec_is(as.POSIXct("2020-01-01")))
  expect_true(vec_is(as.POSIXlt("2020-01-01")))
})

test_that("vec_cast() converts POSIXct with int representation to double when converting zones", {
  x <- structure(integer(), class = c("POSIXct", "POSIXt"), tzone = "UTC")
  y <- structure(
    numeric(),
    class = c("POSIXct", "POSIXt"),
    tzone = "America/Los_Angeles"
  )
  expect_true(is.double(vec_cast(x, y)))
})

test_that("vec_c() converts POSIXct with int representation to double representation (#540)", {
  time1 <- seq(
    as.POSIXct("2015-12-01", tz = "UTC"),
    length.out = 2,
    by = "days"
  )
  time2 <- vec_c(time1)
  expect_true(is.double(time2))

  time3 <- vec_c(time1, time1)
  expect_true(is.double(time3))
})

test_that("vec_c() and vec_rbind() convert Dates with int representation to double representation (#396)", {
  x <- structure(0L, class = "Date")
  df <- data.frame(x = x)

  expect_true(is.double(vec_c(x)))
  expect_true(is.double(vec_c(x, x)))

  expect_true(is.double(vec_rbind(df)$x))
  expect_true(is.double(vec_rbind(df, df)$x))
})

test_that("vec_c() and vec_ptype() standardize missing `tzone` attributes (#561)", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_ptype(x), "tzone"), "")
  expect_identical(attr(vec_c(x, x), "tzone"), "")
})

# constructor -------------------------------------------------------------

test_that("can create a date", {
  expect_identical(new_date(), structure(double(), class = "Date"))
  expect_identical(new_date(0), structure(0, class = "Date"))
})

test_that("retains input names", {
  expect_named(new_date(c(x = 0)), "x")
})

test_that("drops attributes except names", {
  expect_identical(new_date(structure(1, foo = "bar")), new_date(1))
})

test_that("only allows doubles", {
  expect_error(new_date(1L), "must be a double vector")
  expect_error(new_date("x"), "must be a double vector")
})

test_that("can create a datetime", {
  expect_identical(
    new_datetime(),
    structure(double(), class = c("POSIXct", "POSIXt"), tzone = "")
  )
  expect_identical(
    new_datetime(0),
    structure(0, class = c("POSIXct", "POSIXt"), tzone = "")
  )
})

test_that("retains input names", {
  expect_named(new_datetime(c(x = 0)), "x")
})

test_that("drops attributes except names", {
  expect_identical(new_datetime(structure(1, foo = "bar")), new_datetime(1))
})

test_that("only allows doubles", {
  expect_error(new_datetime(1L), "must be a double vector")
  expect_error(new_datetime("x"), "must be a double vector")
})

test_that("tzone is allowed to be `NULL`", {
  expect_identical(new_datetime(tzone = NULL), new_datetime(tzone = ""))
})

test_that("tzone must be character or `NULL`", {
  expect_error(new_datetime(tzone = 1), "character vector or `NULL`")
})

# coerce ------------------------------------------------------------------

test_that("datetime coercions are symmetric and unchanging", {
  types <- list(
    new_date(),
    new_datetime(),
    new_datetime(tzone = "US/Central"),
    as.POSIXlt(character(), tz = "US/Central"),
    difftime(Sys.time() + 1000, Sys.time()),
    difftime(Sys.time() + 1, Sys.time())
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("tz comes from first non-empty", {
  # On the assumption that if you've set the time zone explicitly it
  # should win

  x <- as.POSIXct("2020-01-01")
  y <- as.POSIXct("2020-01-01", tz = "America/New_York")

  expect_identical(vec_ptype2(x, y), y[0])
  expect_identical(vec_ptype2(y, x), y[0])

  z <- as.POSIXct("2020-01-01", tz = "Pacific/Auckland")
  expect_identical(vec_ptype2(y, z), y[0])
  expect_identical(vec_ptype2(z, y), z[0])
})

test_that("POSIXlt remains POSIXlt in vec_ptype()", {
  # See `vec_ptype.POSIXlt()` for details
  x <- as.POSIXlt("2020-01-01", tz = "UTC")
  exp <- x[0]
  expect_identical(vec_ptype(x), exp)

  x <- as.POSIXlt("2020-01-01", tz = "America/New_York")
  exp <- x[0]
  expect_identical(vec_ptype(x), exp)
})

test_that("POSIXlt always steered towards POSIXct in vec_ptype2()", {
  dtc <- as.POSIXct("2020-01-01", tz = "UTC")
  dtl <- as.POSIXlt("2020-01-01", tz = "UTC")

  expect_identical(vec_ptype2(dtc, dtl), dtc[0])
  expect_identical(vec_ptype2(dtl, dtc), dtc[0])
  expect_identical(vec_ptype2(dtl, dtl), dtc[0])
})

test_that("vec_ptype2() on a POSIXlt with multiple time zones returns the first", {
  x <- as.POSIXlt(new_datetime(), tz = "Pacific/Auckland")

  expect_identical(attr(x, "tzone"), c("Pacific/Auckland", "NZST", "NZDT"))
  expect_identical(attr(vec_ptype2(x, new_date()), "tzone"), "Pacific/Auckland")
})

test_that("vec_ptype2(<Date>, NA) is symmetric (#687)", {
  date <- new_date()
  expect_identical(
    vec_ptype2(date, NA),
    vec_ptype2(NA, date)
  )
})

test_that("vec_ptype2(<POSIXt>, NA) is symmetric (#687)", {
  time <- Sys.time()
  expect_identical(
    vec_ptype2(time, NA),
    vec_ptype2(NA, time)
  )
})

test_that("vec_ptype2(<difftime>, NA) is symmetric (#687)", {
  dtime <- Sys.time() - Sys.time()
  expect_identical(
    vec_ptype2(dtime, NA),
    vec_ptype2(NA, dtime)
  )
})

test_that("vec_ptype2() standardizes duration storage type to double", {
  x <- structure(1L, units = "secs", class = "difftime")
  expect <- new_duration(double(), units = "secs")
  expect_identical(vec_ptype2(x, x), expect)
})

test_that("vec_ptype_common() gives expected results with <POSIXlt>", {
  # See `vec_ptype.POSIXlt()` for details
  x <- as.POSIXlt("2020-01-01", tz = "UTC")

  # `vec_ptype(x)` returns `<POSIXlt>`
  exp <- x[0]
  expect_identical(vec_ptype_common(x), exp)
  expect_identical(vec_ptype_common(.ptype = x), exp)

  # `vec_ptype2(x, x)` returns `<POSIXct>`
  exp <- new_datetime(tzone = "UTC")
  expect_identical(vec_ptype_common(x, x), exp)

  # With <unspecified> input, still uses `vec_ptype(x)`, so returns `<POSIXlt>`
  exp <- x[0]
  expect_identical(vec_ptype_common(x, NA), exp)
  expect_identical(vec_ptype_common(NA, x), exp)
  expect_identical(vec_ptype_common(x, NULL), exp)
  expect_identical(vec_ptype_common(NULL, x), exp)
})

# cast: dates ---------------------------------------------------------------

test_that("safe casts work as expected", {
  date <- as.Date("2018-01-01")
  datetime_ct <- as.POSIXct(as.character(date))
  datetime_lt <- as.POSIXlt(datetime_ct)

  expect_identical(vec_cast(NULL, date), NULL)
  expect_identical(vec_cast(date, date), date)
  expect_identical(vec_cast(datetime_ct, date), date)
  expect_identical(vec_cast(datetime_lt, date), date)

  missing_date <- new_date(NA_real_)

  expect_identical(vec_cast(missing_date, missing_date), missing_date)
  expect_identical(
    vec_cast(as.POSIXct(missing_date), missing_date),
    missing_date
  )
  expect_identical(
    vec_cast(as.POSIXlt(missing_date), missing_date),
    missing_date
  )

  # These used to be allowed
  expect_error(vec_cast(17532, date), class = "vctrs_error_incompatible_type")
  expect_error(
    vec_cast("2018-01-01", date),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(date), date),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("date - datetime cast can be roundtripped", {
  date <- as.Date("2018-01-01")
  datetime <- as.POSIXct("2018-01-01", tz = "America/New_York")

  expect_identical(vec_cast(vec_cast(date, datetime), date), date)
  expect_identical(vec_cast(vec_cast(datetime, date), datetime), datetime)
})

test_that("lossy casts generate error", {
  date <- as.Date("2018-01-01")
  datetime <- as.POSIXct(as.character(date)) + c(0, 3600)
  expect_lossy(
    vec_cast(datetime, date),
    vec_c(date, date),
    x = datetime,
    to = date
  )
})

test_that("invalid casts generate error", {
  date <- as.Date("2018-01-01")
  expect_error(
    vec_cast(integer(), date),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can cast NA and unspecified to Date", {
  expect_identical(vec_cast(NA, new_date()), new_date(NA_real_))
  expect_identical(vec_cast(unspecified(2), new_date()), new_date(dbl(NA, NA)))
})

test_that("casting an integer date to another date returns a double date", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_cast(x, x)))
})

test_that("casting an integer POSIXct to a Date returns a double Date", {
  x <- .POSIXct(18000L, tz = "America/New_York")
  expect <- new_date(0)
  expect_identical(vec_cast(x, new_date()), expect)
})

# cast: datetimes -----------------------------------------------------------

test_that("safe casts work as expected", {
  datetime_c <- as.POSIXct("1970-02-01", tz = "UTC")
  datetime_l <- as.POSIXlt("1970-02-01", tz = "UTC")

  expect_identical(vec_cast(NULL, datetime_c), NULL)
  expect_identical(vec_cast(datetime_c, datetime_c), datetime_c)
  expect_identical(vec_cast(datetime_l, datetime_c), datetime_c)
  expect_identical(vec_cast(as.Date(datetime_c), datetime_c), datetime_c)

  expect_identical(vec_cast(NULL, datetime_l), NULL)
  expect_identical(vec_cast(datetime_c, datetime_l), datetime_l)
  expect_identical(vec_cast(datetime_l, datetime_l), datetime_l)
  expect_identical(vec_cast(as.Date(datetime_l), datetime_l), datetime_l)
  expect_error(
    vec_cast(raw(), datetime_l),
    class = "vctrs_error_incompatible_type"
  )

  missing_c <- new_datetime(NA_real_, tzone = "UTC")
  missing_l <- as.POSIXlt(missing_c)

  expect_identical(vec_cast(missing_c, missing_c), missing_c)
  expect_identical(vec_cast(missing_l, missing_c), missing_c)
  expect_identical(vec_cast(as.Date(missing_c), missing_c), missing_c)

  expect_identical(vec_cast(missing_l, missing_l), missing_l)
  expect_identical(vec_cast(missing_c, missing_l), missing_l)
  expect_identical(vec_cast(as.Date(missing_l), missing_l), missing_l)

  # These used to be allowed
  expect_error(
    vec_cast(2678400, datetime_c),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast("1970-02-01", datetime_c),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(datetime_c), datetime_c),
    class = "vctrs_error_incompatible_type"
  )

  expect_error(
    vec_cast(2678400, datetime_l),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast("1970-02-01", datetime_l),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(datetime_l), datetime_l),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  datetime <- as.POSIXct("1970-02-01", tz = "UTC")
  expect_error(
    vec_cast(integer(), datetime),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("dates become midnight in date-time tzone", {
  date1 <- as.Date("2010-01-01")
  datetime_c <- as.POSIXct(character(), tz = "Pacific/Auckland")
  datetime_l <- as.POSIXlt(character(), tz = "Pacific/Auckland")

  date2_c <- vec_cast(date1, datetime_c)
  expect_identical(tzone(date2_c), "Pacific/Auckland")
  expect_identical(format(date2_c, "%H:%M"), "00:00")

  date2_l <- vec_cast(date1, datetime_l)
  expect_identical(tzone(date2_l), "Pacific/Auckland")
  expect_identical(format(date2_l, "%H:%M"), "00:00")
})

test_that("can cast NA and unspecified to POSIXct and POSIXlt", {
  dtc <- as.POSIXct("2020-01-01")
  dtl <- as.POSIXlt("2020-01-01")
  expect_identical(vec_cast(NA, dtc), vec_init(dtc))
  expect_identical(vec_cast(NA, dtl), vec_init(dtl))
  expect_identical(vec_cast(unspecified(2), dtc), vec_init(dtc, 2))
  expect_identical(vec_cast(unspecified(2), dtl), vec_init(dtl, 2))
})

test_that("changing time zones retains the underlying moment in time", {
  x_ct <- as.POSIXct("2019-01-01", tz = "America/New_York")
  x_lt <- as.POSIXlt(x_ct)

  to_ct <- new_datetime(tzone = "America/Los_Angeles")
  to_lt <- as.POSIXlt(to_ct)

  expect_ct <- x_ct
  attr(expect_ct, "tzone") <- "America/Los_Angeles"
  expect_lt <- as.POSIXlt(expect_ct)

  expect_identical(vec_cast(x_ct, to_ct), expect_ct)
  expect_identical(vec_cast(x_ct, to_lt), expect_lt)
  expect_identical(vec_cast(x_lt, to_ct), expect_ct)
  expect_identical(vec_cast(x_lt, to_lt), expect_lt)
})

test_that("casting to date always retains the zoned year-month-day value", {
  x <- as.POSIXct("2019-01-01", tz = "Asia/Shanghai")
  expect_identical(vec_cast(x, new_date()), as.Date("2019-01-01"))
})

# cast: durations ------------------------------------------------------------

test_that("safe casts work as expected", {
  dt1 <- as.difftime(600, units = "secs")
  dt2 <- as.difftime(10, units = "mins")

  expect_identical(vec_cast(NULL, dt1), NULL)
  expect_identical(vec_cast(dt1, dt1), dt1)
  expect_identical(vec_cast(dt1, dt2), dt2)

  # These used to be allowed
  expect_error(vec_cast(600, dt1), class = "vctrs_error_incompatible_type")
  expect_error(
    vec_cast(list(dt1), dt1),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  dt <- as.difftime(600, units = "secs")
  expect_error(vec_cast(integer(), dt), class = "vctrs_error_incompatible_type")
})

test_that("can cast NA and unspecified to duration", {
  expect_identical(vec_cast(NA, new_duration()), new_duration(na_dbl))
  expect_identical(
    vec_cast(unspecified(2), new_duration()),
    new_duration(dbl(NA, NA))
  )
})

test_that("casting coerces corrupt integer storage durations to double (#1602)", {
  x <- structure(1L, units = "secs", class = "difftime")
  expect <- new_duration(1, units = "secs")

  expect_identical(vec_cast(x, x), expect)

  # Names are retained through the coercion
  names(x) <- "a"
  expect_named(vec_cast(x, x), "a")
})

# proxy/restore: dates ---------------------------------------------------

test_that("restoring an integer to an integer Date converts to double", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_restore(x, x)))
})

test_that("vec_proxy() returns a double for Dates with int representation", {
  x <- structure(0L, class = "Date")
  expect_true(is.double(vec_proxy(x)))
})

# proxy/restore: datetimes ------------------------------------------------

test_that("restoring an integer to an integer POSIXct converts to double", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_true(is.double(vec_restore(x, x)))
})

test_that("restoring to a POSIXct with no time zone standardizes to an empty string (#561)", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_restore(x, x), "tzone"), "")
})

test_that("restoring to a POSIXlt with no time zone standardizes to an empty string", {
  # Manually create a POSIXlt without a `tzone` attribute.
  # This is just:
  # `x <- as.POSIXlt("1970-01-01")`
  # which usually won't add a `tzone` attribute, but platforms where the local
  # time is UTC attach a `tzone` attribute automatically.
  x <- structure(
    list(
      sec = 0,
      min = 0L,
      hour = 0L,
      mday = 1L,
      mon = 0L,
      year = 70L,
      wday = 4L,
      yday = 0L,
      isdst = 0L,
      zone = "EST",
      gmtoff = NA_integer_
    ),
    class = c("POSIXlt", "POSIXt")
  )

  proxy <- vec_proxy(x)

  expect_identical(attr(vec_restore(proxy, x), "tzone"), "")
})

test_that("proxying a POSIXct with no time zone standardizes to an empty string", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_identical(attr(vec_proxy(x), "tzone"), "")
})

test_that("vec_proxy() returns a double for POSIXct with int representation", {
  x <- structure(0L, class = c("POSIXct", "POSIXt"))
  expect_true(is.double(vec_proxy(x)))
})

test_that("POSIXlt roundtrips through proxy and restore", {
  x <- as_posixlt("2020-01-03")
  out <- vec_restore(vec_proxy(x), x)
  expect_identical(out, x)
})

test_that("subclassed Dates / POSIXct / POSIXlt can be restored (#1015)", {
  x <- subclass(new_date(0))
  proxy <- vec_proxy(x)
  expect_identical(vec_restore(proxy, x), x)

  y <- subclass(new_datetime(0))
  proxy <- vec_proxy(y)
  expect_identical(vec_restore(proxy, y), y)

  z <- subclass(as.POSIXlt(new_datetime(0)))
  proxy <- vec_proxy(z)
  expect_identical(vec_restore(proxy, z), z)
})

# arithmetic --------------------------------------------------------------

test_that("default is error", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct("2018-01-02 12:00")
  lt <- as.POSIXlt(dt)
  t <- as.difftime(12, units = "hours")
  f <- factor("x")

  expect_error(vec_arith("+", d, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", dt, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", lt, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("+", t, f), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", dt, t), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", lt, t), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", d, t), class = "vctrs_error_incompatible_op")
  expect_error(
    vec_arith("!", t, MISSING()),
    class = "vctrs_error_incompatible_op"
  )
})

test_that("date-time vs date-time", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct(d)
  lt <- as.POSIXlt(dt)

  expect_error(vec_arith("+", d, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, d), d - d)

  expect_error(vec_arith("+", dt, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, dt), dt - dt)

  expect_error(vec_arith("+", lt, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, lt), lt - lt)

  expect_error(vec_arith("+", d, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, dt), difftime(d, dt))

  expect_error(vec_arith("+", dt, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, d), difftime(dt, d))

  expect_error(vec_arith("+", d, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", d, lt), difftime(d, lt))

  expect_error(vec_arith("+", lt, d), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, d), difftime(lt, d))

  expect_error(vec_arith("+", dt, lt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", dt, lt), difftime(dt, lt))

  expect_error(vec_arith("+", lt, dt), class = "vctrs_error_incompatible_op")
  expect_identical(vec_arith("-", lt, dt), difftime(lt, dt))
})

test_that("date-time vs numeric", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct("2018-01-01", tz = "America/New_York")
  lt <- as.POSIXlt(dt)

  expect_identical(vec_arith("+", d, 1), d + 1)
  expect_identical(vec_arith("+", 1, d), d + 1)
  expect_identical(vec_arith("-", d, 1), d - 1)
  expect_error(vec_arith("-", 1, d), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("+", dt, 1), dt + 1)
  expect_identical(vec_arith("+", 1, dt), dt + 1)
  expect_identical(vec_arith("-", dt, 1), dt - 1)
  expect_error(vec_arith("-", 1, dt), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("+", lt, 1), lt + 1)
  expect_identical(vec_arith("+", 1, lt), lt + 1)
  expect_identical(vec_arith("-", lt, 1), lt - 1)
  expect_error(vec_arith("-", 1, lt), class = "vctrs_error_incompatible_op")

  expect_error(vec_arith("*", 1, d), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", d, 1), class = "vctrs_error_incompatible_op")

  expect_error(vec_arith("*", 1, dt), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", dt, 1), class = "vctrs_error_incompatible_op")

  expect_error(vec_arith("*", 1, lt), class = "vctrs_error_incompatible_op")
  expect_error(vec_arith("*", lt, 1), class = "vctrs_error_incompatible_op")
})

test_that("POSIXlt + numeric = POSIXct", {
  lt <- as.POSIXlt("2018-01-01", tz = "America/New_York")
  expect_s3_class(vec_arith("+", lt, 1), "POSIXct")
  expect_s3_class(vec_arith("+", 1, lt), "POSIXct")
})

test_that("vec_arith() standardizes the `tzone` attribute", {
  dt <- structure(0, class = c("POSIXct", "POSIXt"))
  x <- vec_arith("+", dt, 1)
  expect_identical(attr(x, "tzone"), "")
})

test_that("date-time vs difftime", {
  d <- as.Date("2018-01-01")
  dt <- as.POSIXct("2018-01-01", tz = "UTC")
  lt <- as.POSIXlt(dt)
  t <- as.difftime(1, units = "days")
  th <- as.difftime(c(1, 24), units = "hours")

  expect_identical(vec_arith("+", d, t), d + t)
  expect_identical(vec_arith("+", t, d), t + d)

  expect_identical(vec_arith("+", dt, t), dt + t)
  expect_identical(vec_arith("+", t, dt), t + dt)

  expect_identical(vec_arith("+", lt, t), lt + t)
  expect_identical(vec_arith("+", t, lt), t + lt)

  expect_lossy(vec_arith("+", d, th), d + th, x = t, to = d)
  expect_lossy(vec_arith("+", th, d), th + d, x = t, to = d)

  expect_identical(vec_arith("+", dt, th), dt + th)
  expect_identical(vec_arith("+", th, dt), th + dt)

  expect_identical(vec_arith("+", lt, th), lt + th)
  expect_identical(vec_arith("+", th, lt), th + lt)

  expect_identical(vec_arith("-", d, t), d - t)
  expect_error(vec_arith("-", t, d), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", dt, t), dt - t)
  expect_error(vec_arith("-", t, dt), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", lt, t), lt - t)
  expect_error(vec_arith("-", t, lt), class = "vctrs_error_incompatible_op")

  expect_lossy(vec_arith("-", d, th), d - th, x = t, to = d)
  expect_error(vec_arith("-", th, d), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", dt, th), dt - th)
  expect_error(vec_arith("-", th, dt), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("-", lt, th), lt - th)
  expect_error(vec_arith("-", th, lt), class = "vctrs_error_incompatible_op")
})

test_that("difftime vs difftime/numeric", {
  t <- as.difftime(12, units = "hours")

  expect_identical(vec_arith("-", t, MISSING()), -t)
  expect_identical(vec_arith("+", t, MISSING()), t)

  expect_identical(vec_arith("-", t, t), t - t)
  expect_identical(vec_arith("-", t, 1), t - 1)
  expect_identical(vec_arith("-", 1, t), 1 - t)

  expect_identical(vec_arith("+", t, t), 2 * t)
  expect_identical(vec_arith("+", t, 1), t + 1)
  expect_identical(vec_arith("+", 1, t), t + 1)

  expect_identical(vec_arith("*", 2, t), 2 * t)
  expect_identical(vec_arith("*", t, 2), 2 * t)
  expect_error(vec_arith("*", t, t), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("/", t, 2), t / 2)
  expect_error(vec_arith("/", 2, t), class = "vctrs_error_incompatible_op")

  expect_identical(vec_arith("/", t, t), 1)
  expect_identical(vec_arith("%/%", t, t), 1)
  expect_identical(vec_arith("%%", t, t), 0)
})


# Math --------------------------------------------------------------------

test_that("date and date times don't support math", {
  expect_error(vec_math("sum", new_date()), class = "vctrs_error_unsupported")
  expect_error(
    vec_math("sum", new_datetime()),
    class = "vctrs_error_unsupported"
  )
  expect_error(
    vec_math("sum", as.POSIXlt(new_datetime())),
    class = "vctrs_error_unsupported"
  )
})

#' Set operations
#'
#' @description
#' - `vec_set_intersect()` returns all values in both `x` and `y`.
#'
#' - `vec_set_difference()` returns all values in `x` but not `y`. Note
#'   that this is an asymmetric set difference, meaning it is not commutative.
#'
#' - `vec_set_union()` returns all values in either `x` or `y`.
#'
#' - `vec_set_symmetric_difference()` returns all values in either `x` or `y`
#'   but not both. This is a commutative difference.
#'
#' Because these are _set_ operations, these functions only return unique values
#' from `x` and `y`, returned in the order they first appeared in the original
#' input. Names of `x` and `y` are retained on the result, but names are always
#' taken from `x` if the value appears in both inputs.
#'
#' These functions work similarly to [base::intersect()], [base::setdiff()], and
#' [base::union()], but don't strip attributes and can be used with data frames.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x,y A pair of vectors.
#'
#' @param ptype If `NULL`, the default, the output type is determined by
#'   computing the common type between `x` and `y`. If supplied, both `x` and
#'   `y` will be cast to this type.
#'
#' @param x_arg,y_arg Argument names for `x` and `y`. These are used in error
#'   messages.
#'
#' @returns
#' A vector of the common type of `x` and `y` (or `ptype`, if supplied)
#' containing the result of the corresponding set function.
#'
#' @details
#' Missing values are treated as equal to other missing values. For doubles and
#' complexes, `NaN` are equal to other `NaN`, but not to `NA`.
#'
#' @section Dependencies:
#'
#' ## `vec_set_intersect()`
#' - [vec_proxy_equal()]
#' - [vec_slice()]
#' - [vec_ptype2()]
#' - [vec_cast()]
#'
#' ## `vec_set_difference()`
#' - [vec_proxy_equal()]
#' - [vec_slice()]
#' - [vec_ptype2()]
#' - [vec_cast()]
#'
#' ## `vec_set_union()`
#' - [vec_proxy_equal()]
#' - [vec_slice()]
#' - [vec_ptype2()]
#' - [vec_cast()]
#' - [vec_c()]
#'
#' ## `vec_set_symmetric_difference()`
#' - [vec_proxy_equal()]
#' - [vec_slice()]
#' - [vec_ptype2()]
#' - [vec_cast()]
#' - [vec_c()]
#'
#' @name vec-set
#' @examples
#' x <- c(1, 2, 1, 4, 3)
#' y <- c(2, 5, 5, 1)
#'
#' # All unique values in both `x` and `y`.
#' # Duplicates in `x` and `y` are always removed.
#' vec_set_intersect(x, y)
#'
#' # All unique values in `x` but not `y`
#' vec_set_difference(x, y)
#'
#' # All unique values in either `x` or `y`
#' vec_set_union(x, y)
#'
#' # All unique values in either `x` or `y` but not both
#' vec_set_symmetric_difference(x, y)
#'
#' # These functions can also be used with data frames
#' x <- data_frame(
#'   a = c(2, 3, 2, 2),
#'   b = c("j", "k", "j", "l")
#' )
#' y <- data_frame(
#'   a = c(1, 2, 2, 2, 3),
#'   b = c("j", "l", "j", "l", "j")
#' )
#'
#' vec_set_intersect(x, y)
#' vec_set_difference(x, y)
#' vec_set_union(x, y)
#' vec_set_symmetric_difference(x, y)
#'
#' # Vector names don't affect set membership, but if you'd like to force
#' # them to, you can transform the vector into a two column data frame
#' x <- c(a = 1, b = 2, c = 2, d = 3)
#' y <- c(c = 2, b = 1, a = 3, d = 3)
#'
#' vec_set_intersect(x, y)
#'
#' x <- data_frame(name = names(x), value = unname(x))
#' y <- data_frame(name = names(y), value = unname(y))
#'
#' vec_set_intersect(x, y)
NULL

#' @rdname vec-set
#' @export
vec_set_intersect <- function(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(ffi_vec_set_intersect, x, y, ptype, environment())
}

#' @rdname vec-set
#' @export
vec_set_difference <- function(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(ffi_vec_set_difference, x, y, ptype, environment())
}

#' @rdname vec-set
#' @export
vec_set_union <- function(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(ffi_vec_set_union, x, y, ptype, environment())
}

#' @rdname vec-set
#' @export
vec_set_symmetric_difference <- function(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(ffi_vec_set_symmetric_difference, x, y, ptype, environment())
}

# vec_set_intersect -------------------------------------------------------

test_that("retains names of `x` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 2)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_intersect(x, y),
    c(a = 1, e = 2)
  )
})

test_that("returns elements in order they first appear in `x`", {
  expect_identical(vec_set_intersect(c(3, 1, 2, 3), c(2, 3)), c(3, 2))
})

test_that("returns unique elements", {
  expect_identical(vec_set_intersect(c(1, 2, 1), c(2, 2, 1)), c(1, 2))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(vec_set_intersect(encs$utf8, encs$latin1), encs$utf8)
  expect_identical(vec_set_intersect(encs$latin1, encs$utf8), encs$utf8)
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_intersect(c(NA_real_, 1), NA_real_), NA_real_)
  expect_identical(vec_set_intersect(c(1, NA_real_), NA_real_), NA_real_)

  expect_identical(vec_set_intersect(c(NA_real_, NaN), NaN), NaN)
  expect_identical(vec_set_intersect(c(NaN, NA_real_), NaN), NaN)
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_intersect(na, na), na)
  expect_identical(vec_set_intersect(na, na[2]), na[2])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_intersect(NA, NA), NA)
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_intersect(1L, c(2, 1)), 1)
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(vec_set_intersect(x, y), vec_slice(x, c(2, 4)))
  expect_identical(vec_set_intersect(y, x), vec_slice(y, c(1, 3)))
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(vec_set_intersect(x, y), vec_slice(x, c(2, 4)))
  expect_identical(vec_set_intersect(y, x), vec_slice(y, c(1, 3)))
})

# vec_set_difference ------------------------------------------------------

test_that("retains names of `x` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_difference(x, y),
    c(b = 4, e = 5)
  )
})

test_that("returns elements in order they first appear in `x`", {
  expect_identical(vec_set_difference(c(3, 1, 2, 3), 1), c(3, 2))
})

test_that("returns unique elements", {
  expect_identical(vec_set_difference(c(1, 2, 1, 4), c(4, 5)), c(1, 2))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  expect_identical(vec_set_difference(encs$utf8, encs$latin1), character())
  expect_identical(vec_set_difference(encs$latin1, encs$utf8), character())
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_difference(c(NA_real_, 1), NA_real_), 1)
  expect_identical(vec_set_difference(c(1, NA_real_), NA_real_), 1)

  expect_identical(vec_set_difference(c(NA_real_, NaN), NaN), NA_real_)
  expect_identical(vec_set_difference(c(NaN, NA_real_), NaN), NA_real_)
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_difference(na, na), complex())
  expect_identical(vec_set_difference(na, na[2]), na[c(1, 3, 4)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_difference(NA, NA), logical())
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_difference(c(3L, 1L), c(2, 1)), 3)
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(vec_set_difference(x, y), vec_slice(x, 1))
  expect_identical(vec_set_difference(y, x), vec_slice(y, c(2, 4)))
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(vec_set_difference(x, y), vec_slice(x, 1))
  expect_identical(vec_set_difference(y, x), vec_slice(y, c(2, 4)))
})

# vec_set_union -----------------------------------------------------------

test_that("retains names of `x` and `y` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_union(x, y),
    c(a = 1, b = 4, e = 5, w = 3, x = 2)
  )
})

test_that("does minimal name repair on duplicate names", {
  x <- c(a = 1)
  y <- c(a = 2)

  expect_named(vec_set_union(x, y), c("a", "a"))
})

test_that("returns elements in order they first appear in `x` and `y`", {
  expect_identical(vec_set_union(c(3, 1, 2, 3), c(4, 2, 5, 4)), c(3, 1, 2, 4, 5))
})

test_that("returns unique elements", {
  expect_identical(vec_set_union(c(1, 2, 1, 4), c(4, 5, 5)), c(1, 2, 4, 5))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(vec_set_union(encs$utf8, encs$latin1), encs$utf8)
  expect_identical(vec_set_union(encs$latin1, encs$utf8), encs$utf8)
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_union(c(NA_real_, 1), NA_real_), c(NA_real_, 1))
  expect_identical(vec_set_union(c(1, NA_real_), NA_real_), c(1, NA_real_))

  expect_identical(vec_set_union(NA_real_, NaN), c(NA_real_, NaN))
  expect_identical(vec_set_union(NaN, NA_real_), c(NaN, NA_real_))
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_union(na, na), na)
  expect_identical(vec_set_union(na[-2], na), na[c(1, 3, 4, 2)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_union(NA, NA), NA)
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_union(c(3L, 1L), c(2, 1)), c(3, 1, 2))
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(vec_set_union(x, y), vec_c(vec_slice(x, c(1, 2, 4)), vec_slice(y, c(2, 4))))
  expect_identical(vec_set_union(y, x), vec_c(vec_slice(y, c(1, 2, 3, 4)), vec_slice(x, 1)))
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(vec_set_union(x, y), vec_c(vec_slice(x, c(1, 2, 4)), vec_slice(y, c(2, 4))))
  expect_identical(vec_set_union(y, x), vec_c(vec_slice(y, c(1, 2, 3, 4)), vec_slice(x, 1)))
})

# vec_set_symmetric_difference --------------------------------------------

test_that("retains names of `x` and `y` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_symmetric_difference(x, y),
    c(b = 4, e = 5, w = 3, x = 2)
  )
})

test_that("returns elements in order they first appear in `x` and `y`", {
  expect_identical(vec_set_symmetric_difference(c(3, 1, 2, 3), c(4, 2, 5, 4)), c(3, 1, 4, 5))
})

test_that("returns unique elements", {
  expect_identical(vec_set_symmetric_difference(c(1, 2, 1, 4), c(4, 5, 5)), c(1, 2, 5))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(vec_set_symmetric_difference(encs$utf8, encs$latin1), character())
  expect_identical(vec_set_symmetric_difference(encs$latin1, encs$utf8), character())
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_symmetric_difference(c(NA_real_, 1), NA_real_), 1)
  expect_identical(vec_set_symmetric_difference(c(1, NA_real_), NA_real_), 1)

  expect_identical(vec_set_symmetric_difference(c(NaN, 1), NaN), 1)
  expect_identical(vec_set_symmetric_difference(c(1, NaN), NaN), 1)

  expect_identical(vec_set_symmetric_difference(NA_real_, NaN), c(NA_real_, NaN))
  expect_identical(vec_set_symmetric_difference(NaN, NA_real_), c(NaN, NA_real_))
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_symmetric_difference(na, na), complex())
  expect_identical(vec_set_symmetric_difference(na[-2], na[-4]), na[c(4, 2)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_symmetric_difference(NA, NA), logical())
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_symmetric_difference(c(3L, 1L), c(2, 1)), c(3, 2))
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(
    vec_set_symmetric_difference(x, y),
    vec_c(vec_slice(x, 1), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_symmetric_difference(y, x),
    vec_c(vec_slice(y, c(2, 4)), vec_slice(x, 1))
  )
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(
    vec_set_symmetric_difference(x, y),
    vec_c(vec_slice(x, 1), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_symmetric_difference(y, x),
    vec_c(vec_slice(y, c(2, 4)), vec_slice(x, 1))
  )
})

# common ------------------------------------------------------------------

test_that("works with package version columns of data frames (#1837)", {
  package_frame <- function(x) {
    data_frame(version = package_version(x))
  }

  x <- package_frame(c("4.0", "2.0"))
  y <- package_frame(c("1.0", "3.0" ,"4.0"))

  expect_identical(vec_set_intersect(x, y), package_frame("4.0"))
  expect_identical(vec_set_difference(x, y), package_frame("2.0"))
  expect_identical(vec_set_union(x, y), package_frame(c("4.0", "2.0", "1.0", "3.0")))
  expect_identical(vec_set_symmetric_difference(x, y), package_frame(c("2.0", "1.0", "3.0")))
})

test_that("errors nicely if common type can't be taken", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, "x")
  })
})

test_that("dots must be empty", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, 2, 3)
  })
})

test_that("`ptype` is respected", {
  expect_identical(vec_set_intersect(1, 1, ptype = integer()), 1L)
  expect_identical(vec_set_difference(1, 1, ptype = integer()), integer())
  expect_identical(vec_set_union(1, 2, ptype = integer()), c(1L, 2L))
  expect_identical(vec_set_symmetric_difference(1, 2, ptype = integer()), c(1L, 2L))

  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, 1.5, ptype = integer())
  })
})

test_that("`x_arg` and `y_arg` can be adjusted", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, "2", x_arg = "foo", y_arg = "bar")
  })

  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "2", x_arg = "", y_arg = "")
  })
})

test_that("`error_call` can be adjusted", {
  my_set_intersect <- function() {
    vec_set_intersect(1, "x", error_call = current_env())
  }
  my_set_difference <- function() {
    vec_set_difference(1, "x", error_call = current_env())
  }
  my_set_union <- function() {
    vec_set_union(1, "x", error_call = current_env())
  }
  my_set_symmetric_difference <- function() {
    vec_set_symmetric_difference(1, "x", error_call = current_env())
  }

  expect_snapshot(error = TRUE, {
    my_set_intersect()
  })
  expect_snapshot(error = TRUE, {
    my_set_difference()
  })
  expect_snapshot(error = TRUE, {
    my_set_union()
  })
  expect_snapshot(error = TRUE, {
    my_set_symmetric_difference()
  })
})

# vec_set_intersect -------------------------------------------------------

test_that("retains names of `x` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 2)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_intersect(x, y),
    c(a = 1, e = 2)
  )
})

test_that("returns elements in order they first appear in `x`", {
  expect_identical(vec_set_intersect(c(3, 1, 2, 3), c(2, 3)), c(3, 2))
})

test_that("returns unique elements", {
  expect_identical(vec_set_intersect(c(1, 2, 1), c(2, 2, 1)), c(1, 2))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(vec_set_intersect(encs$utf8, encs$latin1), encs$utf8)
  expect_identical(vec_set_intersect(encs$latin1, encs$utf8), encs$utf8)
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_intersect(c(NA_real_, 1), NA_real_), NA_real_)
  expect_identical(vec_set_intersect(c(1, NA_real_), NA_real_), NA_real_)

  expect_identical(vec_set_intersect(c(NA_real_, NaN), NaN), NaN)
  expect_identical(vec_set_intersect(c(NaN, NA_real_), NaN), NaN)
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_intersect(na, na), na)
  expect_identical(vec_set_intersect(na, na[2]), na[2])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_intersect(NA, NA), NA)
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_intersect(1L, c(2, 1)), 1)
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(vec_set_intersect(x, y), vec_slice(x, c(2, 4)))
  expect_identical(vec_set_intersect(y, x), vec_slice(y, c(1, 3)))
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(vec_set_intersect(x, y), vec_slice(x, c(2, 4)))
  expect_identical(vec_set_intersect(y, x), vec_slice(y, c(1, 3)))
})

# vec_set_difference ------------------------------------------------------

test_that("retains names of `x` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_difference(x, y),
    c(b = 4, e = 5)
  )
})

test_that("returns elements in order they first appear in `x`", {
  expect_identical(vec_set_difference(c(3, 1, 2, 3), 1), c(3, 2))
})

test_that("returns unique elements", {
  expect_identical(vec_set_difference(c(1, 2, 1, 4), c(4, 5)), c(1, 2))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  expect_identical(vec_set_difference(encs$utf8, encs$latin1), character())
  expect_identical(vec_set_difference(encs$latin1, encs$utf8), character())
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_difference(c(NA_real_, 1), NA_real_), 1)
  expect_identical(vec_set_difference(c(1, NA_real_), NA_real_), 1)

  expect_identical(vec_set_difference(c(NA_real_, NaN), NaN), NA_real_)
  expect_identical(vec_set_difference(c(NaN, NA_real_), NaN), NA_real_)
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_difference(na, na), complex())
  expect_identical(vec_set_difference(na, na[2]), na[c(1, 3, 4)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_difference(NA, NA), logical())
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_difference(c(3L, 1L), c(2, 1)), 3)
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(vec_set_difference(x, y), vec_slice(x, 1))
  expect_identical(vec_set_difference(y, x), vec_slice(y, c(2, 4)))
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(vec_set_difference(x, y), vec_slice(x, 1))
  expect_identical(vec_set_difference(y, x), vec_slice(y, c(2, 4)))
})

# vec_set_union -----------------------------------------------------------

test_that("retains names of `x` and `y` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_union(x, y),
    c(a = 1, b = 4, e = 5, w = 3, x = 2)
  )
})

test_that("does minimal name repair on duplicate names", {
  x <- c(a = 1)
  y <- c(a = 2)

  expect_named(vec_set_union(x, y), c("a", "a"))
})

test_that("returns elements in order they first appear in `x` and `y`", {
  expect_identical(
    vec_set_union(c(3, 1, 2, 3), c(4, 2, 5, 4)),
    c(3, 1, 2, 4, 5)
  )
})

test_that("returns unique elements", {
  expect_identical(vec_set_union(c(1, 2, 1, 4), c(4, 5, 5)), c(1, 2, 4, 5))
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(vec_set_union(encs$utf8, encs$latin1), encs$utf8)
  expect_identical(vec_set_union(encs$latin1, encs$utf8), encs$utf8)
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_union(c(NA_real_, 1), NA_real_), c(NA_real_, 1))
  expect_identical(vec_set_union(c(1, NA_real_), NA_real_), c(1, NA_real_))

  expect_identical(vec_set_union(NA_real_, NaN), c(NA_real_, NaN))
  expect_identical(vec_set_union(NaN, NA_real_), c(NaN, NA_real_))
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_union(na, na), na)
  expect_identical(vec_set_union(na[-2], na), na[c(1, 3, 4, 2)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_union(NA, NA), NA)
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_union(c(3L, 1L), c(2, 1)), c(3, 1, 2))
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(
    vec_set_union(x, y),
    vec_c(vec_slice(x, c(1, 2, 4)), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_union(y, x),
    vec_c(vec_slice(y, c(1, 2, 3, 4)), vec_slice(x, 1))
  )
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(
    vec_set_union(x, y),
    vec_c(vec_slice(x, c(1, 2, 4)), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_union(y, x),
    vec_c(vec_slice(y, c(1, 2, 3, 4)), vec_slice(x, 1))
  )
})

test_that("works on empty inputs", {
  expect_identical(vec_set_union(integer(), 12L), 12L)
  expect_identical(vec_set_union(12L, integer()), 12L)
})

# vec_set_symmetric_difference --------------------------------------------

test_that("retains names of `x` and `y` elements", {
  x <- c(a = 1, b = 4, c = 1, d = 4, e = 5)
  y <- c(w = 3, x = 2, y = 1, z = 2)

  expect_identical(
    vec_set_symmetric_difference(x, y),
    c(b = 4, e = 5, w = 3, x = 2)
  )
})

test_that("returns elements in order they first appear in `x` and `y`", {
  expect_identical(
    vec_set_symmetric_difference(c(3, 1, 2, 3), c(4, 2, 5, 4)),
    c(3, 1, 4, 5)
  )
})

test_that("returns unique elements", {
  expect_identical(
    vec_set_symmetric_difference(c(1, 2, 1, 4), c(4, 5, 5)),
    c(1, 2, 5)
  )
})

test_that("works with character vectors of different encodings", {
  encs <- encodings()
  # Always returns UTF-8
  expect_identical(
    vec_set_symmetric_difference(encs$utf8, encs$latin1),
    character()
  )
  expect_identical(
    vec_set_symmetric_difference(encs$latin1, encs$utf8),
    character()
  )
})

test_that("has consistency with `NA` values", {
  expect_identical(vec_set_symmetric_difference(c(NA_real_, 1), NA_real_), 1)
  expect_identical(vec_set_symmetric_difference(c(1, NA_real_), NA_real_), 1)

  expect_identical(vec_set_symmetric_difference(c(NaN, 1), NaN), 1)
  expect_identical(vec_set_symmetric_difference(c(1, NaN), NaN), 1)

  expect_identical(
    vec_set_symmetric_difference(NA_real_, NaN),
    c(NA_real_, NaN)
  )
  expect_identical(
    vec_set_symmetric_difference(NaN, NA_real_),
    c(NaN, NA_real_)
  )
})

test_that("works with complex missing values", {
  na <- complex(
    real = c(NA_real_, NA_real_, NaN, NaN),
    imaginary = c(NA_real_, NaN, NA_real_, NaN)
  )
  expect_identical(vec_set_symmetric_difference(na, na), complex())
  expect_identical(vec_set_symmetric_difference(na[-2], na[-4]), na[c(4, 2)])
})

test_that("works correctly with unspecified logical vectors", {
  expect_identical(vec_set_symmetric_difference(NA, NA), logical())
})

test_that("returns a vector of the common type", {
  expect_identical(vec_set_symmetric_difference(c(3L, 1L), c(2, 1)), c(3, 2))
})

test_that("works with data frames", {
  x <- data_frame(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  )
  y <- data_frame(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  )

  expect_identical(
    vec_set_symmetric_difference(x, y),
    vec_c(vec_slice(x, 1), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_symmetric_difference(y, x),
    vec_c(vec_slice(y, c(2, 4)), vec_slice(x, 1))
  )
})

test_that("works with rcrds", {
  x <- new_rcrd(list(
    a = c(1, 2, 1, 1),
    b = c("a", "b", "a", "d")
  ))
  y <- new_rcrd(list(
    a = c(2, 3, 1, 2),
    b = c("b", "b", "d", "d")
  ))

  expect_identical(
    vec_set_symmetric_difference(x, y),
    vec_c(vec_slice(x, 1), vec_slice(y, c(2, 4)))
  )
  expect_identical(
    vec_set_symmetric_difference(y, x),
    vec_c(vec_slice(y, c(2, 4)), vec_slice(x, 1))
  )
})

# common ------------------------------------------------------------------

test_that("works with package version columns of data frames (#1837)", {
  package_frame <- function(x) {
    data_frame(version = package_version(x))
  }

  x <- package_frame(c("4.0", "2.0"))
  y <- package_frame(c("1.0", "3.0", "4.0"))

  expect_identical(vec_set_intersect(x, y), package_frame("4.0"))
  expect_identical(vec_set_difference(x, y), package_frame("2.0"))
  expect_identical(
    vec_set_union(x, y),
    package_frame(c("4.0", "2.0", "1.0", "3.0"))
  )
  expect_identical(
    vec_set_symmetric_difference(x, y),
    package_frame(c("2.0", "1.0", "3.0"))
  )
})

test_that("errors nicely if common type can't be taken", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, "x")
  })
})

test_that("dots must be empty", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, 2, 3)
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, 2, 3)
  })
})

test_that("`ptype` is respected", {
  expect_identical(vec_set_intersect(1, 1, ptype = integer()), 1L)
  expect_identical(vec_set_difference(1, 1, ptype = integer()), integer())
  expect_identical(vec_set_union(1, 2, ptype = integer()), c(1L, 2L))
  expect_identical(
    vec_set_symmetric_difference(1, 2, ptype = integer()),
    c(1L, 2L)
  )

  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, 1.5, ptype = integer())
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, 1.5, ptype = integer())
  })
})

test_that("`x_arg` and `y_arg` can be adjusted", {
  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_difference(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_union(1, "2", x_arg = "foo", y_arg = "bar")
  })
  expect_snapshot(error = TRUE, {
    vec_set_symmetric_difference(1, "2", x_arg = "foo", y_arg = "bar")
  })

  expect_snapshot(error = TRUE, {
    vec_set_intersect(1, "2", x_arg = "", y_arg = "")
  })
})

test_that("`error_call` can be adjusted", {
  my_set_intersect <- function() {
    vec_set_intersect(1, "x", error_call = current_env())
  }
  my_set_difference <- function() {
    vec_set_difference(1, "x", error_call = current_env())
  }
  my_set_union <- function() {
    vec_set_union(1, "x", error_call = current_env())
  }
  my_set_symmetric_difference <- function() {
    vec_set_symmetric_difference(1, "x", error_call = current_env())
  }

  expect_snapshot(error = TRUE, {
    my_set_intersect()
  })
  expect_snapshot(error = TRUE, {
    my_set_difference()
  })
  expect_snapshot(error = TRUE, {
    my_set_union()
  })
  expect_snapshot(error = TRUE, {
    my_set_symmetric_difference()
  })
})

#' Recode and replace using logical conditions
#'
#' @description
#'
#' - `vec_case_when()` constructs an entirely new vector by recoding the `TRUE`
#'   `conditions` to their corresponding `values`. If there are locations not
#'   matched by `conditions`, then they are recoded to the `default` value.
#'
#' - `vec_replace_when()` updates an existing vector by replacing the values
#'   from `x` matched by the `TRUE` `conditions` with their corresponding
#'   `values`. In this case, each element of `values` must have the same type as
#'   `x` and locations not matched by `conditions` retain their original `x`
#'   value.
#'
#' `vec_case_when()` is often thought of as a way to vectorize multiple if-else
#' statements, and is an R equivalent of the SQL "searched" `CASE WHEN`
#' statement.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector.
#'
#' @param conditions A list of logical condition vectors.
#'
#'   For `vec_case_when()`, each vector should be the same size.
#'
#'   For `vec_replace_when()`, each vector should be the same size as `x`.
#'
#'   Where a value in `conditions` is `TRUE`, the corresponding value in
#'   `values` will be assigned to the result.
#'
#' @param values A list of vectors.
#'
#'   For `vec_case_when()`, each vector should be size 1 or the size implied by
#'   `conditions`. The common type of `values` and `default` determine the
#'   output type, unless overridden by `ptype`.
#'
#'   For `vec_replace_when()`, each vector should be size 1 or the same size
#'   as `x`. Each vector will be cast to the type of `x`.
#'
#' @param default Default value to use when `conditions` does not match every
#'   location in the output.
#'
#'   By default, a missing value is used as the default value.
#'
#'   If supplied, `default` must be size 1 or the size implied by `conditions`.
#'
#'   Can only be set when `unmatched = "default"`.
#'
#' @param unmatched Handling of unmatched locations.
#'
#'   One of:
#'
#'   - `"default"` to use `default` in unmatched locations.
#'
#'   - `"error"` to error when there are unmatched locations.
#'
#' @param ptype An optional override for the output type, which is usually
#'   computed as the common type of `values` and `default`.
#'
#' @param size An optional override for the output size, which is usually
#'   computed as the size of the first element of `conditions`.
#'
#'   Only useful for requiring a fixed size when `conditions` is an empty list.
#'
#' @param x_arg,conditions_arg,values_arg,default_arg Argument names used in
#'   error messages.
#'
#' @returns
#' A vector.
#'
#' - For `vec_case_when()`, the type of the output is computed as the common
#'   type of `values` and `default`, unless overridden by `ptype`. The names of
#'   the output come from the names of `values` and `default`. The size of the
#'   output comes from the implied size from `conditions`, unless overridden by
#'   `size`.
#'
#' - For `vec_replace_when()`, the type of the output will have the same type as
#'   `x`. The names of the output will be the same as the names of `x`. The size
#'   of the output will be the same size as `x`.
#'
#' @name vec-case-and-replace
#'
#' @examples
#' # Note how the first `TRUE` is used in the output.
#' # Also note how the `NA` falls through to `default`.
#' x <- seq(-2L, 2L, by = 1L)
#' x <- c(x, NA)
#' conditions <- list(
#'   x < 0,
#'   x < 1
#' )
#' values <- list(
#'   "<0",
#'   "<1"
#' )
#' vec_case_when(
#'   conditions,
#'   values,
#'   default = "other"
#' )
#'
#' # Missing values need to be handled with their own case
#' # if you want them to have a special value
#' conditions <- list(
#'   x < 0,
#'   x < 1,
#'   is.na(x)
#' )
#' values <- list(
#'   "<0",
#'   "<1",
#'   NA
#' )
#' vec_case_when(
#'   conditions,
#'   values,
#'   default = "other"
#' )
#'
#' # Both `values` and `default` are vectorized
#' values <- list(
#'   x * 5,
#'   x * 10,
#'   NA
#' )
#' vec_case_when(
#'   conditions,
#'   values,
#'   default = x * 100
#' )
#'
#' # Use `vec_replace_when()` if you need to update `x`, retaining
#' # all previous values in locations that you don't match
#' conditions <- list(
#'   x < 0,
#'   x < 1
#' )
#' values <- list(
#'   0,
#'   1
#' )
#' out <- vec_replace_when(
#'   x,
#'   conditions,
#'   values
#' )
#' out
#'
#' # Note how `vec_replace_when()` is type stable on `x`, we retain the
#' # integer type here even though `values` contained doubles
#' typeof(out)
#'
#' # `vec_case_when()` creates a new vector, so names come from `values`
#' # and `default`. `vec_replace_when()` modifies an existing vector, so
#' # names come from `x` no matter what, just like `[<-` and `base::replace()`
#' x <- c(a = 1, b = 2, c = 3)
#' conditions <- list(x == 1, x == 2)
#' values <- list(c(x = 0), c(y = -1))
#' vec_case_when(conditions, values)
#' vec_replace_when(x, conditions, values)
#'
#' # If you want to enforce that you've covered all of the locations in your
#' # `conditions`, use `unmatched = "error"` rather than providing a `default`
#' x <- c(0, 1, 2)
#' conditions <- list(x == 1, x == 2)
#' values <- list("a", "b")
#' try(vec_case_when(conditions, values, unmatched = "error"))
NULL

#' @rdname vec-case-and-replace
#' @export
vec_case_when <- function(
  conditions,
  values,
  ...,
  default = NULL,
  unmatched = "default",
  ptype = NULL,
  size = NULL,
  conditions_arg = "conditions",
  values_arg = "values",
  default_arg = "default",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_case_when,
    conditions,
    values,
    default,
    unmatched,
    ptype,
    size,
    environment()
  )
}

#' @rdname vec-case-and-replace
#' @export
vec_replace_when <- function(
  x,
  conditions,
  values,
  ...,
  x_arg = "x",
  conditions_arg = "conditions",
  values_arg = "values",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_replace_when,
    x,
    conditions,
    values,
    environment()
  )
}

test_that("works with data frames", {
  conditions <- list(
    c(FALSE, TRUE, FALSE, FALSE),
    c(TRUE, TRUE, FALSE, FALSE),
    c(FALSE, TRUE, FALSE, TRUE)
  )
  values <- list(
    data_frame(x = 1, y = 2),
    data_frame(x = 3, y = 4),
    data_frame(x = 3:6, y = 4:7)
  )

  out <- vec_case_when(conditions, values)

  expect_identical(
    out,
    data_frame(
      x = c(3, 1, NA, 6),
      y = c(4, 2, NA, 7)
    )
  )
})

test_that("first `TRUE` case wins", {
  conditions <- list(
    c(TRUE, FALSE),
    c(TRUE, TRUE),
    c(TRUE, TRUE)
  )
  values <- list(
    1,
    2,
    3
  )

  expect_identical(
    vec_case_when(conditions, values),
    c(1, 2)
  )
})

test_that("can replace missing values by catching with `is.na()`", {
  x <- c(1:3, NA)

  conditions <- list(
    x <= 1,
    x <= 2,
    is.na(x)
  )
  values <- list(
    1,
    2,
    0
  )

  expect_identical(
    vec_case_when(conditions, values),
    c(1, 2, NA, 0)
  )
})

test_that("Unused logical `NA` can still be cast to `values` ptype", {
  # Requires that casting happen before recycling, because it recycles
  # to size zero, resulting in a logical rather than an unspecified.
  expect_identical(
    vec_case_when(list(TRUE, FALSE), list("x", NA)),
    "x"
  )
  expect_identical(
    vec_case_when(list(FALSE, TRUE), list("x", NA)),
    NA_character_
  )
})

test_that("`conditions` inputs can be size zero", {
  expect_identical(
    vec_case_when(
      list(logical(), logical()),
      list(1, 2)
    ),
    numeric()
  )

  expect_snapshot(error = TRUE, {
    vec_case_when(list(logical()), list(1:2))
  })
})

test_that("retains inner names of `values` inputs", {
  value1 <- c(x = 1, y = 2)
  value2 <- c(z = 3, w = 4)

  out <- vec_case_when(
    list(c(TRUE, FALSE), c(TRUE, TRUE)),
    list(a = value1, b = value2)
  )

  expect_named(out, c("x", "w"))
})

test_that("outer names have no affect over the output names", {
  value1 <- c(1, 2)
  value2 <- c(3, 4)

  out <- vec_case_when(
    list(c(TRUE, FALSE), c(FALSE, TRUE)),
    list(x = value1, y = value2)
  )
  expect_named(out, NULL)
})

test_that("`values` are cast to their common type", {
  expect_identical(vec_case_when(list(FALSE, TRUE), list(1, 2L)), 2)
  expect_identical(vec_case_when(list(FALSE, TRUE), list(1, NA)), NA_real_)

  expect_snapshot(error = TRUE, {
    vec_case_when(list(FALSE, TRUE), list(1, "x"))
  })
})

test_that("`values` must be size 1 or same size as the `conditions`", {
  expect_identical(
    vec_case_when(
      list(c(TRUE, TRUE)),
      list(1)
    ),
    c(1, 1)
  )
  expect_identical(
    vec_case_when(
      list(c(TRUE, FALSE), c(TRUE, TRUE)),
      list(c(1, 2), c(3, 4))
    ),
    c(1, 4)
  )

  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(c(TRUE, FALSE, TRUE, TRUE)),
      list(1:3)
    )
  })
})

test_that("Unhandled `NA` are given a value of `default`", {
  expect_identical(
    vec_case_when(list(NA), list(1)),
    NA_real_
  )

  expect_identical(
    vec_case_when(list(NA), list(1), default = 2),
    2
  )

  expect_identical(
    vec_case_when(
      list(
        c(FALSE, NA, TRUE, FALSE),
        c(NA, FALSE, TRUE, FALSE)
      ),
      list(
        2,
        3
      ),
      default = 4
    ),
    c(4, 4, 2, 4)
  )
})

test_that("`NA` is overridden by any `TRUE` values", {
  x <- c(1, 2, NA, 3)
  expect <- c("one", "not_one", "missing", "not_one")

  # `TRUE` overriding before the `NA`
  conditions <- list(
    is.na(x),
    x == 1
  )
  values <- list(
    "missing",
    "one"
  )
  expect_identical(
    vec_case_when(
      conditions,
      values,
      default = "not_one"
    ),
    expect
  )

  # `TRUE` overriding after the `NA`
  conditions <- list(
    x == 1,
    is.na(x)
  )
  values <- list(
    "one",
    "missing"
  )
  expect_identical(
    vec_case_when(
      conditions,
      values,
      default = "not_one"
    ),
    expect
  )
})

test_that("works when there is a used `default` and no missing values", {
  expect_identical(
    vec_case_when(list(c(TRUE, FALSE)), list(1), default = 3:4),
    c(1, 4)
  )
})

test_that("works when there are missing values but no `default`", {
  expect_identical(vec_case_when(list(c(TRUE, NA)), list(1)), c(1, NA))
})

test_that("A `NULL` `default` fills in with missing values", {
  expect_identical(
    vec_case_when(list(c(TRUE, FALSE, FALSE)), list(1)),
    c(1, NA, NA)
  )
})

test_that("`default` fills in all unused slots", {
  expect_identical(
    vec_case_when(list(c(TRUE, FALSE, FALSE)), list(1), default = 2),
    c(1, 2, 2)
  )
})

test_that("`default` is initialized correctly in the logical / unspecified case", {
  # i.e. `vec_ptype(NA)` is unspecified but the result should be finalized to logical
  expect_identical(vec_case_when(list(FALSE), list(NA)), NA)
})

test_that("`default` can be vectorized, and is sliced to fit as needed", {
  out <- vec_case_when(
    list(
      c(FALSE, TRUE, FALSE, TRUE, FALSE),
      c(FALSE, TRUE, FALSE, FALSE, TRUE)
    ),
    list(
      1:5,
      6:10
    ),
    default = 11:15
  )

  expect_identical(out, c(11L, 2L, 13L, 4L, 10L))
})

test_that("`default` must be size 1 or same size as `conditions` (exact same as any other `values` input)", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(FALSE), list(1L), default = 2:3)
  })
})

test_that("`default` participates in common type determination (exact same as any other `values` input)", {
  expect_identical(vec_case_when(list(FALSE), list(1L), default = 2), 2)
})

test_that("`default` that is an unused logical `NA` can still be cast to `values` ptype", {
  # Requires that casting happen before recycling, because it recycles
  # to size zero, resulting in a logical rather than an unspecified.
  expect_identical(vec_case_when(list(TRUE), list("x"), default = NA), "x")
})

test_that("`default` type is used when all values are logical `NA` (#2094)", {
  expect_identical(
    vec_case_when(list(TRUE), list(NA), default = "a"),
    NA_character_
  )
  expect_identical(
    vec_case_when(list(c(TRUE, FALSE)), list(NA), default = "a"),
    c(NA_character_, "a")
  )
})

test_that("`default_arg` can be customized", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(FALSE), list(1L), default = 2:3, default_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(FALSE), list(1L), default = "x", default_arg = "foo")
  })
})

test_that("`conditions_arg` is validated", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list("x"), list(1), conditions_arg = 1)
  })
})

test_that("`values_arg` is validated", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(lm(1 ~ 1)), values_arg = 1)
  })
})

test_that("`default_arg` is validated", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(1), default = "x", default_arg = 1)
  })
})

test_that("`conditions` must all be the same size", {
  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(c(TRUE, FALSE), TRUE),
      list(1, 2)
    )
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(c(TRUE, FALSE), c(TRUE, FALSE, TRUE)),
      list(1, 2)
    )
  })
})

test_that("`conditions` must be logical (and aren't cast to logical!)", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(1), list(2))
  })

  # Make sure input numbering is right in the error message!
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE, 3.5), list(2, 4))
  })

  # `vec_as_location()` would not allow this either
  x <- structure(c(FALSE, TRUE), class = "my_logical")
  expect_snapshot(error = TRUE, {
    vec_case_when(list(x), list(1), default = 2)
  })
})

test_that("`conditions` are allowed to have attributes", {
  x <- structure(c(FALSE, TRUE), label = "foo")
  expect_identical(vec_case_when(list(x), list(1), default = 2), c(2, 1))
})

test_that("`conditions` can't be arrays (#6862)", {
  x <- array(TRUE, dim = c(3, 3))
  y <- c("a", "b", "c")

  expect_snapshot(error = TRUE, {
    vec_case_when(list(x), list(y))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(x), list(y), size = 3)
  })

  # Not even 1D arrays
  x <- array(TRUE, dim = 3)

  expect_snapshot(error = TRUE, {
    vec_case_when(list(x), list(y))
  })
})

test_that("`size` overrides the `conditions` sizes", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(1), size = 5)
  })

  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(c(TRUE, FALSE), c(TRUE, FALSE, TRUE)),
      list(1, 2),
      size = 2
    )
  })
})

test_that("0 `conditions` result depends on `size` and `default` and `ptype`", {
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list()
    ),
    unspecified()
  )

  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      size = 0
    ),
    unspecified()
  )
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      size = 2
    ),
    unspecified(2)
  )

  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      default = integer()
    ),
    integer()
  )
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      default = 1L
    ),
    integer()
  )
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      ptype = integer()
    ),
    integer()
  )

  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      size = 2L,
      default = 1L
    ),
    c(1L, 1L)
  )
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      size = 2L,
      default = 1:2
    ),
    c(1L, 2L)
  )
  expect_identical(
    vec_case_when(
      conditions = list(),
      values = list(),
      size = 2L,
      ptype = integer()
    ),
    c(NA_integer_, NA_integer_)
  )
})

test_that("`vec_replace_when()` with empty `conditions` is a no-op", {
  x <- 1:5

  expect_identical(
    vec_replace_when(x, conditions = list(), values = list()),
    x
  )
})

test_that("`ptype` overrides the `values` types", {
  expect_identical(
    vec_case_when(list(FALSE, TRUE), list(1, 2), ptype = integer()),
    2L
  )

  expect_snapshot(error = TRUE, {
    vec_case_when(list(FALSE, TRUE), list(1, 2), ptype = character())
  })
})

test_that("number of `conditions` and `values` must be the same", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list())
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE, TRUE), list(1))
  })
})

test_that("dots must be empty", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(1), 2)
  })
})

test_that("`conditions` must be a list", {
  expect_snapshot(error = TRUE, {
    vec_case_when(1, list(2))
  })
})

test_that("`values` must be a list", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), 1)
  })
})

test_that("named inputs show up in the error message", {
  expect_snapshot(error = TRUE, {
    vec_case_when(list(x = 1.5), list(1))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(x = 1.5), list(1), conditions_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(x = 1.5), list(1), conditions_arg = "")
  })

  expect_snapshot(error = TRUE, {
    vec_case_when(list(x = TRUE, y = c(TRUE, FALSE)), list(1, 2))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(x = TRUE, y = c(TRUE, FALSE)),
      list(1, 2),
      conditions_arg = "foo"
    )
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(
      list(x = TRUE, y = c(TRUE, FALSE)),
      list(1, 2),
      conditions_arg = ""
    )
  })

  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE, FALSE), list(1, x = "y"))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE, FALSE), list(1, x = "y"), values_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE, FALSE), list(1, x = "y"), values_arg = "")
  })

  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(NULL))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(x = NULL))
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(NULL), values_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(x = NULL), values_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(NULL), values_arg = "")
  })
  expect_snapshot(error = TRUE, {
    vec_case_when(list(TRUE), list(x = NULL), values_arg = "")
  })
})

test_that("proof that `ptype` finalization is important", {
  # Imagine you have an input logical vector you are remapping
  # and it happens to only have `NA`s
  x <- c(NA, NA)
  conditions <- list(x %in% NA)
  values <- list(FALSE)

  # If no `ptype` finalization happened, then `ptype = x` would result in
  # `unspecified` being the output type and these would error. `list_combine()`
  # now does `ptype` finalization when an explicit `ptype` is provided, so this
  # works.
  expect_identical(
    vec_case_when(conditions, values, default = x, ptype = x),
    c(FALSE, FALSE)
  )
  expect_identical(
    vec_replace_when(x, conditions, values),
    c(FALSE, FALSE)
  )
})

test_that("`unmatched` errors are correct", {
  conditions <- list(
    c(TRUE, FALSE, TRUE, FALSE, FALSE, NA, NA, NA, TRUE),
    c(FALSE, TRUE, TRUE, FALSE, NA, FALSE, NA, TRUE, NA)
  )
  values <- list(
    1,
    2
  )
  expect_snapshot(error = TRUE, {
    vec_case_when(conditions, values, unmatched = "error")
  })
})

#' Transpose a list of homogenous vectors
#'
#' @description
#' `list_of_transpose()` takes a list of homogenous vectors, transposes it, and
#' returns a new list of homogenous vectors. To perform a transpose, three
#' pieces of information are required:
#'
#' - The _list size_, from [`vec_size(x)`][vec_size()].
#'
#' - The _element size_, from [`list_of_size(x)`][list_of_size()].
#'
#' - The _element type_, from [`list_of_ptype(x)`][list_of_ptype()].
#'
#' Because all three of these are required, this function only works on fully
#' specified [list_of()]s, with both `size` and `ptype` specified.
#'
#' To predict the output from `list_of_transpose()`, swap the list size with the
#' element size. For example:
#'
#' - Input: `list_of<integer[3]>[2]`
#' - Output: `list_of<integer[2]>[3]`
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x A [list_of][list_of()] with both `size` and `ptype` specified.
#'
#' @param x_arg Argument name used in error messages.
#'
#' @returns
#' A `list_of` of size `list_of_size(x)`, with an element size of `vec_size(x)`
#' and an element type of `list_of_ptype(x)`.
#'
#' @export
#' @examples
#' # A form of `list_of()` that infers both ptype and size
#' list_of2 <- function(...) {
#'   list_of(..., .ptype = NULL, .size = NULL)
#' }
#'
#' # I: list_of<integer[2]>[3]
#' # O: list_of<integer[3]>[2]
#' list_of_transpose(list_of2(1:2, 3:4, 5:6))
#'
#' # With data frames
#' x <- data_frame(a = 1:2, b = letters[1:2])
#' y <- data_frame(a = 3:4, b = letters[3:4])
#' list_of_transpose(list_of2(x, y))
#'
#' # Size 1 elements are recycled
#' list_of_transpose(list_of2(1, 2:3, 4))
#'
#' # ---------------------------------------------------------------------------
#' # `NULL` handling
#'
#' # `NULL` values aren't allowed in `list_of_transpose()`
#' x <- list_of2(1:3, NULL, 5:7, NULL)
#' try(list_of_transpose(x))
#'
#' # Either drop them entirely or replace them up front before transposing
#'
#' x_dropped <- vec_slice(x, !vec_detect_missing(x))
#' x_dropped
#'
#' list_of_transpose(x_dropped)
#'
#' x_replaced <- vec_assign(x, vec_detect_missing(x), list(NA))
#' x_replaced
#'
#' list_of_transpose(x_replaced)
#'
#' # ---------------------------------------------------------------------------
#' # Reversibility
#'
#' # Because `list_of_transpose()` takes and returns fully specified list-ofs,
#' # it is fully reversible, even in the edge cases.
#' x <- list_of2(integer(), integer())
#'
#' # This returns a list of size 0
#' # I: list_of<integer[0]>[2]
#' # O: list_of<integer[2]>[0]
#' out <- list_of_transpose(x)
#' out
#'
#' # Even though there are no elements, we know the element size and type,
#' # so we can transpose a second time to recover `x`. This would not be
#' # possible if this function returned a bare `list()`, which would result
#' # in lost information.
#' # I: list_of<integer[2]>[0]
#' # O: list_of<integer[0]>[2]
#' list_of_transpose(out)
#'
#' # ---------------------------------------------------------------------------
#' # Padding
#'
#' # If you'd like to pad with a missing value rather than erroring,
#' # you might do something like this, which left-pads before conversion
#' # to list-of.
#' x <- list(1, 2:5, 6:7)
#'
#' sizes <- list_sizes(x)
#' size <- max(sizes)
#' index <- which(sizes != size)
#'
#' x[index] <- lapply(
#'   index,
#'   function(i) vec_c(rep(NA, times = size - sizes[[i]]), x[[i]])
#' )
#' x
#'
#' x <- as_list_of(x, .ptype = NULL, .size = NULL)
#'
#' list_of_transpose(x)
list_of_transpose <- function(
  x,
  ...,
  x_arg = caller_arg(x),
  error_call = current_env()
) {
  check_dots_empty0(...)

  check_list_of(x, arg = x_arg, call = error_call)

  if (vec_any_missing(x)) {
    abort(
      cli::format_inline(
        "{arg_backtick(x_arg)} can't contain `NULL` values."
      ),
      call = error_call
    )
  }

  size <- list_of_size0(x)
  ptype <- list_of_ptype0(x)

  if (is_null(size)) {
    abort(
      c(
        cli::format_inline(
          "{arg_backtick(x_arg)} must be a fully specified `<list_of>`."
        ),
        i = "`size` is not specified."
      ),
      call = error_call
    )
  }
  if (is_null(ptype)) {
    abort(
      c(
        cli::format_inline(
          "{arg_backtick(x_arg)} must be a fully specified `<list_of>`."
        ),
        i = "`ptype` is not specified."
      ),
      call = error_call
    )
  }

  x_size <- vec_size(x)
  sizes <- vec_rep(x_size, times = size)

  # Flatten pieces into one big vector
  out <- list_interleave(
    x,
    size = size,
    ptype = ptype,
    name_spec = "inner",
    x_arg = x_arg,
    error_call = error_call
  )

  # Chop the one big vector into transposed pieces of size `x_size`
  out <- vec_chop(out, sizes = sizes)

  new_list_of0(out, ptype = ptype, size = x_size)
}

#' Vector type as a string
#'
#' `vec_ptype_full()` displays the full type of the vector. `vec_ptype_abbr()`
#' provides an abbreviated summary suitable for use in a column heading.
#'
#' @section S3 dispatch:
#' The default method for `vec_ptype_full()` uses the first element of the
#' class vector. Override this method if your class has parameters that should
#' be prominently displayed.
#'
#' The default method for `vec_ptype_abbr()` [abbreviate()]s `vec_ptype_full()`
#' to 8 characters. You should almost always override, aiming for 4-6
#' characters where possible.
#'
#' These arguments are handled by the generic and not passed to methods:
#' * `prefix_named`
#' * `suffix_shape`
#'
#' @param x A vector.
#' @param prefix_named If `TRUE`, add a prefix for named vectors.
#' @param suffix_shape If `TRUE` (the default), append the shape of
#'   the vector.
#' @inheritParams rlang::args_dots_empty
#'
#' @keywords internal
#' @return A string.
#' @export
#' @examples
#' cat(vec_ptype_full(1:10))
#' cat(vec_ptype_full(iris))
#'
#' cat(vec_ptype_abbr(1:10))
vec_ptype_full <- function(x, ...) {
  check_dots_empty0(...)

  # Data frames and their subclasses have internal handling in the
  # default method to get the inner types format
  method <- s3_method_specific(x, "vec_ptype_full", ns = "vctrs")
  return(method(x, ...))

  UseMethod("vec_ptype_full")
}

#' @export
#' @rdname vec_ptype_full
vec_ptype_abbr <- function(x, ..., prefix_named = FALSE, suffix_shape = TRUE) {
  check_dots_empty0(...)

  method <- s3_method_specific(x, "vec_ptype_abbr", ns = "vctrs")
  abbr <- method(x, ...)

  named <- if ((prefix_named || is_bare_list(x)) && !is.null(vec_names(x))) {
    "named "
  }
  shape <- if (suffix_shape) vec_ptype_shape(x)
  abbr <- paste0(named, abbr, shape)

  return(abbr)
  UseMethod("vec_ptype_abbr")
}

#' @export
vec_ptype_full.NULL <- function(x, ...) "NULL"

#' @export
vec_ptype_abbr.NULL <- function(x, ...) "NULL"


# Default: base types and fallback for S3/S4 ------------------------------

#' @export
vec_ptype_full.default <- function(x, ...) {
  if (is.data.frame(x)) {
    vec_ptype_full_data_frame(x, ...)
  } else if (is.object(x)) {
    class(x)[[1]]
  } else if (is_vector(x)) {
    paste0(typeof(x), vec_ptype_shape(x))
  } else {
    abort("Not a vector.")
  }
}

#' @export
vec_ptype_abbr.default <- function(x, ...) {
  if (is.object(x)) {
    type <- class(x)[[1]]
  } else if (is_vector(x)) {
    type <- vec_ptype_abbr_bare(x, ...)
  } else {
    abort("Not a vector.")
  }
  unname(abbreviate(type, 8))
}

vec_ptype_full_data_frame <- function(x, ...) {
  if (length(x) == 0) {
    return(paste0(class(x)[[1]], "<>"))
  } else if (length(x) == 1) {
    return(paste0(
      class(x)[[1]],
      "<",
      names(x),
      ":",
      vec_ptype_full(x[[1]]),
      ">"
    ))
  }

  # Needs to handle recursion with indenting
  types <- map_chr(x, vec_ptype_full)
  needs_indent <- grepl("\n", types)
  types[needs_indent] <- map(types[needs_indent], function(x) {
    indent(paste0("\n", x), 4)
  })

  names <- paste0("  ", format(names(x)))

  paste0(
    class(x)[[1]],
    "<\n",
    paste0(names, ": ", types, collapse = "\n"),
    "\n>"
  )
}

vec_ptype_abbr_bare <- function(x, ...) {
  switch(
    typeof(x),
    list = "list",
    logical = "lgl",
    integer = "int",
    double = "dbl",
    character = "chr",
    complex = "cpl",
    list = "list",
    expression = "expr",
    raw = "raw",
    typeof(x)
  )
}


# Helpers -----------------------------------------------------------------

vec_ptype_shape <- function(x) {
  dim <- dim2(x)
  if (length(dim) == 1) {
    if (is_null(dim(x))) {
      ""
    } else {
      "[1d]"
    }
  } else {
    paste0("[,", paste(dim[-1], collapse = ","), "]")
  }
}

test_that("input must be a vector", {
  expect_error(vec_ptype_abbr(sum), "Not a vector")
  expect_error(vec_ptype_full(sum), "Not a vector")
})

test_that("NULL has method", {
  expect_equal(vec_ptype_abbr(NULL), "NULL")
  expect_equal(vec_ptype_full(NULL), "NULL")
})

test_that("non objects default to type + shape", {
  expect_equal(vec_ptype_abbr(ones(10)), "dbl[1d]")
  expect_equal(vec_ptype_abbr(ones(0, 10)), "dbl[,10]")
  expect_equal(vec_ptype_abbr(ones(10, 0)), "dbl[,0]")

  expect_equal(vec_ptype_full(ones(10)), "double[1d]")
  expect_equal(vec_ptype_full(ones(0, 10)), "double[,10]")
  expect_equal(vec_ptype_full(ones(10, 0)), "double[,0]")

})

test_that("non objects can omit shape", {
  expect_equal(vec_ptype_abbr(ones(10), suffix_shape = FALSE), "dbl")
  expect_equal(vec_ptype_abbr(ones(0, 10), suffix_shape = FALSE), "dbl")
  expect_equal(vec_ptype_abbr(ones(10, 0), suffix_shape = FALSE), "dbl")
})

test_that("objects default to first class", {
  x <- structure(1, class = c("foofy", "goofy"))
  expect_equal(vec_ptype_full(x), "foofy")
  expect_equal(vec_ptype_abbr(x), "foofy")
})

test_that("atomic vectors and arrays as expected", {
  expect_equal(vec_ptype_full(1:5), "integer")

  dbl_mat <- array(double(), c(0, 3))
  expect_equal(vec_ptype_full(dbl_mat), "double[,3]")
})

test_that("complex and factor as expected (#323)", {
  expect_equal(vec_ptype_abbr(0i), "cpl")
  expect_equal(vec_ptype_abbr(factor()), "fct")
})

test_that("named lists are always tagged (#322)", {
  expect_identical(vec_ptype_abbr(list(x = 1, y = 2)), "named list")
  expect_identical(vec_ptype_abbr(list(x = 1, y = 2), prefix_named = TRUE), "named list")
})

test_that("named atomics are tagged optionally (#781)", {
  expect_identical(vec_ptype_abbr(c(x = 1, y = 2), prefix_named = TRUE), "named dbl")
  expect_identical(vec_ptype_abbr(c(x = 1L, y = 2L), prefix_named = TRUE), "named int")
})

test_that("vec_ptype_abbr() adds named tag in case of row names", {
  expect_equal(
    vec_ptype_abbr(mtcars, prefix_named = TRUE),
    "named df[,11]"
  )

  mat <- matrix(1:4, 2)
  rownames(mat) <- c("foo", "bar")
  expect_equal(
    vec_ptype_abbr(mat, prefix_named = TRUE),
    "named int[,2]"
  )
})

test_that("vec_ptype_abbr() and vec_ptype_full() are not inherited (#1549)", {
  foobar <- foobar(class = c("vctrs_bar", "vctrs_foo"))

  local_methods(
    vec_ptype_abbr.vctrs_foo = function(...) "foo_abbr",
    vec_ptype_full.vctrs_foo = function(...) "foo_full"
  )
  expect_equal(
    vec_ptype_abbr(foobar),
    vec_ptype_abbr.default(foobar)
  )
  expect_equal(
    vec_ptype_full(foobar),
    "vctrs_bar"
  )

  local_methods(
    vec_ptype_abbr.vctrs_bar = function(...) "bar_abbr",
    vec_ptype_full.vctrs_bar = function(...) "bar_full"
  )
  expect_equal(
    vec_ptype_abbr(foobar),
    "bar_abbr"
  )
  expect_equal(
    vec_ptype_full(foobar),
    "bar_full"
  )
})

test_that("data.frames have good default abbr and full methods", {
  expect_snapshot({
    df <- foobar(data.frame(x = 1, y = "", z = TRUE))
    vec_ptype_abbr(df)
    vec_ptype_full(df)
  })
})

test_that("input must be a vector", {
  expect_error(vec_ptype_abbr(sum), "Not a vector")
  expect_error(vec_ptype_full(sum), "Not a vector")
})

test_that("NULL has method", {
  expect_equal(vec_ptype_abbr(NULL), "NULL")
  expect_equal(vec_ptype_full(NULL), "NULL")
})

test_that("non objects default to type + shape", {
  expect_equal(vec_ptype_abbr(ones(10)), "dbl[1d]")
  expect_equal(vec_ptype_abbr(ones(0, 10)), "dbl[,10]")
  expect_equal(vec_ptype_abbr(ones(10, 0)), "dbl[,0]")

  expect_equal(vec_ptype_full(ones(10)), "double[1d]")
  expect_equal(vec_ptype_full(ones(0, 10)), "double[,10]")
  expect_equal(vec_ptype_full(ones(10, 0)), "double[,0]")
})

test_that("non objects can omit shape", {
  expect_equal(vec_ptype_abbr(ones(10), suffix_shape = FALSE), "dbl")
  expect_equal(vec_ptype_abbr(ones(0, 10), suffix_shape = FALSE), "dbl")
  expect_equal(vec_ptype_abbr(ones(10, 0), suffix_shape = FALSE), "dbl")
})

test_that("objects default to first class", {
  x <- structure(1, class = c("foofy", "goofy"))
  expect_equal(vec_ptype_full(x), "foofy")
  expect_equal(vec_ptype_abbr(x), "foofy")
})

test_that("atomic vectors and arrays as expected", {
  expect_equal(vec_ptype_full(1:5), "integer")

  dbl_mat <- array(double(), c(0, 3))
  expect_equal(vec_ptype_full(dbl_mat), "double[,3]")
})

test_that("complex and factor as expected (#323)", {
  expect_equal(vec_ptype_abbr(0i), "cpl")
  expect_equal(vec_ptype_abbr(factor()), "fct")
})

test_that("named lists are always tagged (#322)", {
  expect_identical(vec_ptype_abbr(list(x = 1, y = 2)), "named list")
  expect_identical(
    vec_ptype_abbr(list(x = 1, y = 2), prefix_named = TRUE),
    "named list"
  )
})

test_that("named atomics are tagged optionally (#781)", {
  expect_identical(
    vec_ptype_abbr(c(x = 1, y = 2), prefix_named = TRUE),
    "named dbl"
  )
  expect_identical(
    vec_ptype_abbr(c(x = 1L, y = 2L), prefix_named = TRUE),
    "named int"
  )
})

test_that("vec_ptype_abbr() adds named tag in case of row names", {
  expect_equal(
    vec_ptype_abbr(mtcars, prefix_named = TRUE),
    "named df[,11]"
  )

  mat <- matrix(1:4, 2)
  rownames(mat) <- c("foo", "bar")
  expect_equal(
    vec_ptype_abbr(mat, prefix_named = TRUE),
    "named int[,2]"
  )
})

test_that("vec_ptype_abbr() and vec_ptype_full() are not inherited (#1549)", {
  foobar <- foobar(class = c("vctrs_bar", "vctrs_foo"))

  local_methods(
    vec_ptype_abbr.vctrs_foo = function(...) "foo_abbr",
    vec_ptype_full.vctrs_foo = function(...) "foo_full"
  )
  expect_equal(
    vec_ptype_abbr(foobar),
    vec_ptype_abbr.default(foobar)
  )
  expect_equal(
    vec_ptype_full(foobar),
    "vctrs_bar"
  )

  local_methods(
    vec_ptype_abbr.vctrs_bar = function(...) "bar_abbr",
    vec_ptype_full.vctrs_bar = function(...) "bar_full"
  )
  expect_equal(
    vec_ptype_abbr(foobar),
    "bar_abbr"
  )
  expect_equal(
    vec_ptype_full(foobar),
    "bar_full"
  )
})

test_that("data.frames have good default abbr and full methods", {
  expect_snapshot({
    df <- foobar(data.frame(x = 1, y = "", z = TRUE))
    vec_ptype_abbr(df)
    vec_ptype_full(df)
  })
})

#' Count unique values in a vector
#'
#' Count the number of unique values in a vector. `vec_count()` has two
#' important differences to `table()`: it returns a data frame, and when
#' given multiple inputs (as a data frame), it only counts combinations that
#' appear in the input.
#'
#' @param x A vector (including a data frame).
#' @param sort One of "count", "key", "location", or "none".
#'  * "count", the default, puts most frequent values at top
#'  * "key", orders by the output key column (i.e. unique values of `x`)
#'  * "location", orders by location where key first seen. This is useful
#'     if you want to match the counts up to other unique/duplicated functions.
#'  * "none", leaves unordered. This is not guaranteed to produce the same
#'    ordering across R sessions, but is the fastest method.
#' @return A data frame with columns `key` (same type as `x`) and
#'   `count` (an integer vector).
#'
#' @section Dependencies:
#' - [vec_proxy_equal()]
#' - [vec_slice()]
#' - [vec_order()]
#' @export
#' @examples
#' vec_count(mtcars$vs)
#' vec_count(iris$Species)
#'
#' # If you count a data frame you'll get a data frame
#' # column in the output
#' str(vec_count(mtcars[c("vs", "am")]))
#'
#' # Sorting ---------------------------------------
#'
#' x <- letters[rpois(100, 6)]
#' # default is to sort by frequency
#' vec_count(x)
#'
#' # by can sort by key
#' vec_count(x, sort = "key")
#'
#' # or location of first value
#' vec_count(x, sort = "location")
#' head(x)
#'
#' # or not at all
#' vec_count(x, sort = "none")
vec_count <- function(x, sort = c("count", "key", "location", "none")) {
  sort <- arg_match0(sort, c("count", "key", "location", "none"))

  # Returns info pair giving index of first occurrence value and count
  info <- vec_count_impl(x)

  # Sorting based on rearranging `info`
  if (sort == "location") {
    loc <- vec_order(info$loc)
    info <- vec_slice(info, loc)
  } else if (sort == "count") {
    # Order by descending count, but ascending original location.
    # This retains stable ordering in case of ties in the `count`.
    # Need `vec_order_radix()` to handle different `direction`s.
    loc <- vec_order_radix(
      info[c("count", "loc")],
      direction = c("desc", "asc")
    )
    info <- vec_slice(info, loc)
  }

  out <- data_frame(
    key = vec_slice(x, info$loc),
    count = info$count
  )

  # Sorting based on rearranging `out`
  if (sort == "key") {
    loc <- vec_order(out$key)
    out <- vec_slice(out, loc)
  }

  out
}

vec_count_impl <- function(x) {
  .Call(vctrs_count, x)
}

# Duplicates --------------------------------------------------------------

#' Find duplicated values
#'
#' * `vec_duplicate_any()`: detects the presence of duplicated values,
#'   similar to [anyDuplicated()].
#' * `vec_duplicate_detect()`: returns a logical vector describing if each
#'   element of the vector is duplicated elsewhere. Unlike [duplicated()], it
#'   reports all duplicated values, not just the second and subsequent
#'   repetitions.
#' * `vec_duplicate_id()`: returns an integer vector giving the location of
#'   the first occurrence of the value.
#'
#' @section Missing values:
#' In most cases, missing values are not considered to be equal, i.e.
#' `NA == NA` is not `TRUE`. This behaviour would be unappealing here,
#' so these functions consider all `NAs` to be equal. (Similarly,
#' all `NaN` are also considered to be equal.)
#'
#' @param x A vector (including a data frame).
#' @return
#'   * `vec_duplicate_any()`: a logical vector of length 1.
#'   * `vec_duplicate_detect()`: a logical vector the same length as `x`.
#'   * `vec_duplicate_id()`: an integer vector the same length as `x`.
#'
#' @section Dependencies:
#' - [vec_proxy_equal()]
#'
#' @seealso [vec_unique()] for functions that work with the dual of duplicated
#'   values: unique values.
#' @name vec_duplicate
#' @examples
#' vec_duplicate_any(1:10)
#' vec_duplicate_any(c(1, 1:10))
#'
#' x <- c(10, 10, 20, 30, 30, 40)
#' vec_duplicate_detect(x)
#' # Note that `duplicated()` doesn't consider the first instance to
#' # be a duplicate
#' duplicated(x)
#'
#' # Identify elements of a vector by the location of the first element that
#' # they're equal to:
#' vec_duplicate_id(x)
#' # Location of the unique values:
#' vec_unique_loc(x)
#' # Equivalent to `duplicated()`:
#' vec_duplicate_id(x) == seq_along(x)
NULL

#' @rdname vec_duplicate
#' @export
vec_duplicate_any <- function(x) {
  .Call(vctrs_duplicated_any, x)
}

#' @rdname vec_duplicate
#' @export
vec_duplicate_detect <- function(x) {
  .Call(vctrs_duplicated, x)
}

#' @rdname vec_duplicate
#' @export
vec_duplicate_id <- function(x) {
  .Call(vctrs_id, x)
}

# Unique values -----------------------------------------------------------

#' Find and count unique values
#'
#' * `vec_unique()`: the unique values. Equivalent to [unique()].
#' * `vec_unique_loc()`: the locations of the unique values.
#' * `vec_unique_count()`: the number of unique values.
#'
#' @inherit vec_duplicate sections
#' @param x A vector (including a data frame).
#' @return
#' * `vec_unique()`: a vector the same type as `x` containing only unique
#'    values.
#' * `vec_unique_loc()`: an integer vector, giving locations of unique values.
#' * `vec_unique_count()`: an integer vector of length 1, giving the
#'   number of unique values.
#' @seealso [vec_duplicate] for functions that work with the dual of
#'   unique values: duplicated values.
#'
#' @section Dependencies:
#' - [vec_proxy_equal()]
#'
#' @export
#' @examples
#' x <- rpois(100, 8)
#' vec_unique(x)
#' vec_unique_loc(x)
#' vec_unique_count(x)
#'
#' # `vec_unique()` returns values in the order that encounters them
#' # use sort = "location" to match to the result of `vec_count()`
#' head(vec_unique(x))
#' head(vec_count(x, sort = "location"))
#'
#' # Normally missing values are not considered to be equal
#' NA == NA
#'
#' # But they are for the purposes of considering uniqueness
#' vec_unique(c(NA, NA, NA, NA, 1, 2, 1))
vec_unique <- function(x) {
  vec_slice(x, vec_unique_loc(x))
}

#' @rdname vec_unique
#' @export
vec_unique_loc <- function(x) {
  .Call(vctrs_unique_loc, x)
}

#' @rdname vec_unique
#' @export
vec_unique_count <- function(x) {
  .Call(vctrs_n_distinct, x)
}


# Matching ----------------------------------------------------------------

#' Find matching observations across vectors
#'
#' `vec_in()` returns a logical vector based on whether `needle` is found in
#' haystack. `vec_match()` returns an integer vector giving location of
#' `needle` in `haystack`, or `NA` if it's not found.
#'
#' `vec_in()` is equivalent to [base::%in%]; `vec_match()` is equivalent to
#' [base::match()].
#'
#' @section Missing values:
#' In most cases places in R, missing values are not considered to be equal,
#' i.e. `NA == NA` is not `TRUE`. The exception is in matching functions
#' like [base::match()] and [merge()], where an `NA` will match another `NA`.
#' By `vec_match()` and `vec_in()` will match `NA`s; but you can control
#' this behaviour with the `na_equal` argument.
#'
#' @param needles,haystack Vector of `needles` to search for in vector haystack.
#'   `haystack` should usually be unique; if not `vec_match()` will only
#'   return the location of the first match.
#'
#'   `needles` and `haystack` are coerced to the same type prior to
#'   comparison.
#' @inheritParams rlang::args_dots_empty
#' @param na_equal If `TRUE`, missing values in `needles` can be
#'   matched to missing values in `haystack`. If `FALSE`, they
#'   propagate, missing values in `needles` are represented as `NA` in
#'   the return value.
#' @param needles_arg,haystack_arg Argument tags for `needles` and
#'   `haystack` used in error messages.
#' @return A vector the same length as `needles`. `vec_in()` returns a
#'   logical vector; `vec_match()` returns an integer vector.
#'
#' @section Dependencies:
#' - [vec_cast_common()] with fallback
#' - [vec_proxy_equal()]
#'
#' @export
#' @examples
#' hadley <- strsplit("hadley", "")[[1]]
#' vec_match(hadley, letters)
#'
#' vowels <- c("a", "e", "i", "o", "u")
#' vec_match(hadley, vowels)
#' vec_in(hadley, vowels)
#'
#' # Only the first index of duplicates is returned
#' vec_match(c("a", "b"), c("a", "b", "a", "b"))
vec_match <- function(
  needles,
  haystack,
  ...,
  na_equal = TRUE,
  needles_arg = "",
  haystack_arg = ""
) {
  check_dots_empty0(...)
  .Call(vctrs_match, needles, haystack, na_equal, environment())
}

#' @export
#' @rdname vec_match
vec_in <- function(
  needles,
  haystack,
  ...,
  na_equal = TRUE,
  needles_arg = "",
  haystack_arg = ""
) {
  check_dots_empty0(...)
  .Call(vctrs_in, needles, haystack, na_equal, environment())
}

# counting ----------------------------------------------------------------

test_that("vec_count counts number observations", {
  x <- vec_count(rep(1:3, 1:3), sort = "key")
  expect_equal(x, data.frame(key = 1:3, count = 1:3))
})

test_that("vec_count(sort = 'count') uses a stable sort when there are ties (#1588)", {
  x <- c("a", "b", "b", "a", "d")

  expect_identical(
    vec_count(x, sort = "count"),
    data_frame(key = c("a", "b", "d"), count = c(2L, 2L, 1L))
  )
})

test_that("vec_count works with matrices", {
  x <- matrix(c(1, 1, 1, 2, 2, 1), c(3, 2))

  out <- vec_count(x)
  exp <- data_frame(key = c(NA, NA), count = int(2L, 1L))
  exp$key <- vec_slice(x, c(1, 3))

  expect_identical(out, exp)
})

test_that("vec_count works with arrays", {
  x <- array(c(rep(1, 3), rep(2, 3)), dim = c(3, 2, 1))
  expect <- data.frame(key = NA, count = 3)
  expect$key <- vec_slice(x, 1L)
  expect_equal(vec_count(x), expect)
})

test_that("vec_count works for zero-length input", {
  x <- vec_count(integer(), sort = "none")
  expect_equal(x, data.frame(key = integer(), count = integer()))
})

test_that("vec_count works with different encodings", {
  x <- vec_count(encodings())
  expect_equal(x, new_data_frame(list(key = encodings()[1], count = 3L)))
})

test_that("vec_count recursively takes the equality proxy", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)
  expect <- data_frame(key = vec_slice(df, c(1, 3)), count = c(2L, 1L))

  expect_equal(vec_count(df), expect)
})

# duplicates and uniques --------------------------------------------------

test_that("vec_duplicated reports on duplicates regardless of position", {
  x <- c(1, 1, 2, 3, 4, 4)
  expect_equal(vec_duplicate_detect(x), c(TRUE, TRUE, FALSE, FALSE, TRUE, TRUE))
})

test_that("vec_duplicate_any returns single TRUE/FALSE", {
  expect_false(vec_duplicate_any(c(1:10)))
  expect_true(vec_duplicate_any(c(1:10, 1)))
})

test_that("vec_duplicate_id gives position of first found", {
  x <- c(1, 2, 3, 1, 4)
  expect_equal(vec_duplicate_id(x), c(1, 2, 3, 1, 5))
})

test_that("vec_unique matches unique", {
  x <- sample(100, 1000, replace = TRUE)
  expect_equal(vec_unique(x), unique(x))
  expect_equal(vec_unique(c("x", "x")), "x")
})

test_that("vec_unique matches unique for matrices", {
  x <- matrix(c(1, 1, 2, 2), ncol = 2)
  expect_equal(vec_unique(x), unique(x))
})

test_that("vec_unique_count matches length + unique", {
  x <- sample(100, 1000, replace = TRUE)
  expect_equal(vec_unique_count(x), length(unique(x)))
})

test_that("also works for data frames", {
  df <- data.frame(x = 1:3, y = letters[3:1], stringsAsFactors = FALSE)
  idx <- c(1L, 1L, 1L, 2L, 2L, 3L)
  df2 <- df[idx, , drop = FALSE]
  rownames(df2) <- NULL

  expect_equal(vec_duplicate_detect(df2), vec_duplicate_detect(idx))
  expect_equal(vec_unique(df2), vec_slice(df, vec_unique(idx)))

  count <- vec_count(df2, sort = "key")
  expect_equal(count$key, df)
  expect_equal(count$count, vec_count(idx)$count)

  exp <- tibble(x = c(1, 1, 2), y = c(1, 2, 3))
  expect_identical(vec_unique(vec_slice(exp, c(1, 1, 2, 3))), exp)
})

test_that("vec_unique() handles matrices (#327)", {
  x <- matrix(c(1, 2, 3, 4), c(2, 2))
  y <- matrix(c(1, 2, 3, 5), c(2, 2))
  expect_identical(vec_unique(list(x, x)), list(x))
  expect_identical(vec_unique(list(x, y)), list(x, y))

  x <- matrix(c(1, 2, 1, 1, 2, 1), nrow = 3)
  expect_identical(vec_unique(x), vec_slice(x, 1:2))
})

test_that("vec_unique() works with 1D arrays", {
  # 1D arrays are dispatched to `as.data.frame.vector()` which
  # currently does not strip dimensions. This caused an infinite
  # recursion.
  expect_identical(vec_unique(array(1:2)), array(1:2))

  x <- new_vctr(c(1, 1, 1, 2, 1, 2), dim = c(3, 2))
  expect_identical(vec_unique(x), new_vctr(c(1, 1, 2, 1), dim = c(2, 2)))
})

test_that("unique functions take the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)

  expect_true(vec_in(tuple(2, 100), x))
  expect_identical(vec_match(tuple(2, 100), x), 2L)
})

test_that("unique functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)

  expect_equal(vec_unique(df), vec_slice(df, c(1, 3)))
  expect_equal(vec_unique_count(df), 2L)
  expect_equal(vec_unique_loc(df), c(1, 3))
})

test_that("duplicate functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)

  expect_equal(vec_duplicate_any(df), TRUE)
  expect_equal(vec_duplicate_detect(df), c(TRUE, TRUE, FALSE))
  expect_equal(vec_duplicate_id(df), c(1, 1, 3))
})

test_that("unique functions treat positive and negative 0 as equivalent (#637)", {
  expect_equal(vec_unique(c(0, -0)), 0)
  expect_equal(vec_unique_count(c(0, -0)), 1)
  expect_equal(vec_unique_loc(c(0, -0)), 1)
})

test_that("unique functions work with different encodings", {
  encs <- encodings()

  expect_equal(vec_unique(encs), encs[1])
  expect_equal(vec_unique_count(encs), 1L)
  expect_equal(vec_unique_loc(encs), 1L)
})

test_that("unique functions can handle scalar types in lists", {
  x <- list(a ~ b, a ~ b, a ~ c)
  expect_equal(vec_unique(x), vec_slice(x, c(1, 3)))

  x <- list(call("x"), call("y"), call("x"))
  expect_equal(vec_unique(x), vec_slice(x, c(1, 2)))
})

test_that("duplicate functions works with different encodings", {
  encs <- encodings()

  expect_equal(vec_duplicate_id(encs), rep(1, 3))
  expect_equal(vec_duplicate_detect(encs), rep(TRUE, 3))
  expect_equal(vec_duplicate_any(encs), TRUE)
})

test_that("vec_unique() returns differently encoded strings in the order they appear", {
  encs <- encodings()
  x <- c(encs$unknown, encs$utf8)
  y <- c(encs$utf8, encs$unknown)

  expect_equal_encoding(vec_unique(x), encs$unknown)
  expect_equal_encoding(vec_unique(y), encs$utf8)
})

test_that("vec_unique() works on lists containing expressions", {
  x <- list(expression(x), expression(y), expression(x))
  expect_equal(vec_unique(x), x[1:2])
})

test_that("vec_unique() works with glm objects (#643)", {
  # class(model$family$initialize) == "expression"
  model <- glm(mpg ~ wt, data = mtcars)
  expect_equal(vec_unique(list(model, model)), list(model))
})

test_that("can take the unique locations of dfs with list-cols", {
  df <- tibble(x = list(1, 2, 1, 3), y = list(1, 2, 1, 3))
  expect_identical(vec_unique_loc(df), c(1L, 2L, 4L))
})

# matching ----------------------------------------------------------------

test_that("vec_match() matches match()", {
  n <- c(1:3, NA)
  h <- c(4, 2, 1, NA)
  expect_equal(vec_match(n, h), match(n, h))

  expect_equal(vec_match(1.5, c(2, 1.5, NA)), match(1.5, c(2, 1.5, NA)))
  expect_equal(vec_match("x", "x"), match("x", "x"))
})

test_that("vec_match() and vec_in() check types", {
  expect_snapshot({
    df1 <- data_frame(x = data_frame(foo = 1))
    df2 <- data_frame(x = data_frame(foo = ""))

    (expect_error(vec_match(df1, df2), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_match(df1, df2, needles_arg = "n", haystack_arg = "h"), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_in(df1, df2), class = "vctrs_error_incompatible_type"))
    (expect_error(vec_in(df1, df2, needles_arg = "n", haystack_arg = "h"), class = "vctrs_error_incompatible_type"))
  })
})

test_that("vec_in() matches %in%", {
  n <- c(1:3, NA)
  h <- c(4, 2, 1, NA)

  expect_equal(vec_in(n, h), n %in% h)
})

test_that("can opt out of NA matching", {
  n <- c(1, NA)
  h <- c(1:3, NA)

  expect_equal(vec_in(n, h, na_equal = FALSE), c(TRUE, NA))
})

test_that("vec_match works with empty data frame", {
  out <- vec_match(
    new_data_frame(n = 3L),
    new_data_frame(n = 0L)
  )
  expect_equal(out, vec_init(integer(), 3))
})

test_that("matching functions take the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)

  expect_identical(vec_unique_loc(x), 1:2)
  expect_identical(unique(x), tuple(c(1, 2), 1:2))

  expect_true(vec_duplicate_any(x))
  expect_identical(vec_duplicate_id(x), c(1L, 2L, 1L))
  expect_identical(vec_unique_count(x), 2L)

  expect_identical(vec_duplicate_detect(x), c(TRUE, FALSE, TRUE))
})

test_that("can take the unique loc of 1d arrays (#461)", {
  x <- array(c(1, 1, 2, 2, 3))
  y <- array(c(1, 1, 2, 2, 3), dimnames = list(NULL))
  expect_identical(vctrs::vec_unique_loc(x), int(1, 3, 5))
  expect_identical(vctrs::vec_unique_loc(y), int(1, 3, 5))

  z <- array(c(1, 1, 2, 2, 3, 4), c(3, 2))
  expect_silent(expect_identical(vctrs::vec_unique_loc(y), int(1, 3, 5)))
})

test_that("matching functions work with different encodings", {
  encs <- encodings()

  expect_equal(vec_match(encs, encs[1]), rep(1, 3))
  expect_equal(vec_in(encs, encs[1]), rep(TRUE, 3))
})

test_that("matching functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2), 1:2)
  df <- data_frame(x = x)

  y <- tuple(c(2, 3), c(3, 3))
  df2 <- data_frame(x = y)

  expect_equal(vec_match(df, df2), c(NA, 1))
  expect_equal(vec_in(df, df2), c(FALSE, TRUE))
})

test_that("can propagate missing values while matching", {
  exp <- c(NA, 3L, NA, 1L)
  expect_identical(vec_match(lgl(NA, TRUE, NA, FALSE), lgl(FALSE, NA, TRUE), na_equal = FALSE), exp)
  expect_identical(vec_match(int(NA, 1L, NA, 2L), int(2L, NA, 1L), na_equal = FALSE), exp)
  expect_identical(vec_match(dbl(NA, 1, NA, 2), dbl(2, NA, 1), na_equal = FALSE), exp)
  expect_identical(vec_match(cpl(NA, 1, NA, 2), cpl(2, NA, 1), na_equal = FALSE), exp)
  expect_identical(vec_match(chr(NA, "1", NA, "2"), chr("2", NA, "1"), na_equal = FALSE), exp)
  expect_identical(vec_match(list(NULL, 1, NULL, 2), list(2, NULL, 1), na_equal = FALSE), exp)

  # No missing values for raw vectors
  expect_identical(vec_match(raw2(0, 1, 0, 2), raw2(2, 0, 1), na_equal = FALSE), c(2L, 3L, 2L, 1L))
})

test_that("can propagate missingness of incomplete rcrd observations (#1386)", {
  x <- new_rcrd(list(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA)))
  expect_identical(vec_match(x, x, na_equal = FALSE), c(1L, NA, NA, NA))

  # Matches `vec_detect_complete()` results
  expect_identical(vec_detect_complete(x), c(TRUE, FALSE, FALSE, FALSE))
})

test_that("can propagate NaN as a missing value (#1252)", {
  expect_identical(vec_match(dbl(NaN, NA), c(NaN, NA), na_equal = FALSE), int(NA, NA))
  expect_identical(vec_in(dbl(NaN, NA), c(NaN, NA), na_equal = FALSE), lgl(NA, NA))
})

test_that("missing values are propagated across columns", {
  for (na_value in list(NA, na_int, na_dbl, na_cpl, na_chr, list(NULL))) {
    df <- data_frame(x = 1, y = data_frame(foo = 2, bar = na_value), z = 3)
    expect_identical(vec_match(df, df), 1L)
    expect_identical(vec_match(df, df, na_equal = FALSE), na_int)
  }
})

test_that("can't supply NA as `na_equal`", {
  expect_error(vec_match(NA, NA, na_equal = NA), "single `TRUE` or `FALSE`")
})

test_that("vec_match() and vec_in() silently fall back to base data frame", {
  expect_silent(expect_identical(
    vec_match(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    1:32
  ))
  expect_silent(expect_identical(
    vec_in(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(TRUE, 32)
  ))
})

test_that("vec_in() evaluates arg lazily", {
  expect_silent(vec_in(1L, 1L, needles_arg = print("oof")))
  expect_silent(vec_in(1L, 1L, haystack_arg = print("oof")))
})

test_that("vec_match() evaluates arg lazily", {
  expect_silent(vec_match(1L, 1L, needles_arg = print("oof")))
  expect_silent(vec_match(1L, 1L, haystack_arg = print("oof")))
})

# counting ----------------------------------------------------------------

test_that("vec_count counts number observations", {
  x <- vec_count(rep(1:3, 1:3), sort = "key")
  expect_equal(x, data.frame(key = 1:3, count = 1:3))
})

test_that("vec_count(sort = 'count') uses a stable sort when there are ties (#1588)", {
  x <- c("a", "b", "b", "a", "d")

  expect_identical(
    vec_count(x, sort = "count"),
    data_frame(key = c("a", "b", "d"), count = c(2L, 2L, 1L))
  )
})

test_that("vec_count works with matrices", {
  x <- matrix(c(1, 1, 1, 2, 2, 1), c(3, 2))

  out <- vec_count(x)
  exp <- data_frame(key = c(NA, NA), count = int(2L, 1L))
  exp$key <- vec_slice(x, c(1, 3))

  expect_identical(out, exp)
})

test_that("vec_count works with arrays", {
  x <- array(c(rep(1, 3), rep(2, 3)), dim = c(3, 2, 1))
  expect <- data.frame(key = NA, count = 3)
  expect$key <- vec_slice(x, 1L)
  expect_equal(vec_count(x), expect)
})

test_that("vec_count works for zero-length input", {
  x <- vec_count(integer(), sort = "none")
  expect_equal(x, data.frame(key = integer(), count = integer()))
})

test_that("vec_count works with different encodings", {
  x <- vec_count(encodings())
  expect_equal(x, new_data_frame(list(key = encodings()[1], count = 3L)))
})

test_that("vec_count recursively takes the equality proxy", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)
  expect <- data_frame(key = vec_slice(df, c(1, 3)), count = c(2L, 1L))

  expect_equal(vec_count(df), expect)
})

# duplicates and uniques --------------------------------------------------

test_that("vec_duplicated reports on duplicates regardless of position", {
  x <- c(1, 1, 2, 3, 4, 4)
  expect_equal(vec_duplicate_detect(x), c(TRUE, TRUE, FALSE, FALSE, TRUE, TRUE))
})

test_that("vec_duplicate_any returns single TRUE/FALSE", {
  expect_false(vec_duplicate_any(c(1:10)))
  expect_true(vec_duplicate_any(c(1:10, 1)))
})

test_that("vec_duplicate_id gives position of first found", {
  x <- c(1, 2, 3, 1, 4)
  expect_equal(vec_duplicate_id(x), c(1, 2, 3, 1, 5))
})

test_that("vec_unique matches unique", {
  x <- sample(100, 1000, replace = TRUE)
  expect_equal(vec_unique(x), unique(x))
  expect_equal(vec_unique(c("x", "x")), "x")
})

test_that("vec_unique matches unique for matrices", {
  x <- matrix(c(1, 1, 2, 2), ncol = 2)
  expect_equal(vec_unique(x), unique(x))
})

test_that("vec_unique_count matches length + unique", {
  x <- sample(100, 1000, replace = TRUE)
  expect_equal(vec_unique_count(x), length(unique(x)))
})

test_that("also works for data frames", {
  df <- data.frame(x = 1:3, y = letters[3:1], stringsAsFactors = FALSE)
  idx <- c(1L, 1L, 1L, 2L, 2L, 3L)
  df2 <- df[idx, , drop = FALSE]
  rownames(df2) <- NULL

  expect_equal(vec_duplicate_detect(df2), vec_duplicate_detect(idx))
  expect_equal(vec_unique(df2), vec_slice(df, vec_unique(idx)))

  count <- vec_count(df2, sort = "key")
  expect_equal(count$key, df)
  expect_equal(count$count, vec_count(idx)$count)

  exp <- tibble(x = c(1, 1, 2), y = c(1, 2, 3))
  expect_identical(vec_unique(vec_slice(exp, c(1, 1, 2, 3))), exp)
})

test_that("vec_unique() handles matrices (#327)", {
  x <- matrix(c(1, 2, 3, 4), c(2, 2))
  y <- matrix(c(1, 2, 3, 5), c(2, 2))
  expect_identical(vec_unique(list(x, x)), list(x))
  expect_identical(vec_unique(list(x, y)), list(x, y))

  x <- matrix(c(1, 2, 1, 1, 2, 1), nrow = 3)
  expect_identical(vec_unique(x), vec_slice(x, 1:2))
})

test_that("vec_unique() works with 1D arrays", {
  # 1D arrays are dispatched to `as.data.frame.vector()` which
  # currently does not strip dimensions. This caused an infinite
  # recursion.
  expect_identical(vec_unique(array(1:2)), array(1:2))

  x <- new_vctr(c(1, 1, 1, 2, 1, 2), dim = c(3, 2))
  expect_identical(vec_unique(x), new_vctr(c(1, 1, 2, 1), dim = c(2, 2)))
})

test_that("unique functions take the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)

  expect_true(vec_in(tuple(2, 100), x))
  expect_identical(vec_match(tuple(2, 100), x), 2L)
})

test_that("unique functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)

  expect_equal(vec_unique(df), vec_slice(df, c(1, 3)))
  expect_equal(vec_unique_count(df), 2L)
  expect_equal(vec_unique_loc(df), c(1, 3))
})

test_that("duplicate functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 1, 2), 1:3)
  df <- data_frame(x = x)

  expect_equal(vec_duplicate_any(df), TRUE)
  expect_equal(vec_duplicate_detect(df), c(TRUE, TRUE, FALSE))
  expect_equal(vec_duplicate_id(df), c(1, 1, 3))
})

test_that("unique functions treat positive and negative 0 as equivalent (#637)", {
  expect_equal(vec_unique(c(0, -0)), 0)
  expect_equal(vec_unique_count(c(0, -0)), 1)
  expect_equal(vec_unique_loc(c(0, -0)), 1)
})

test_that("unique functions work with different encodings", {
  encs <- encodings()

  expect_equal(vec_unique(encs), encs[1])
  expect_equal(vec_unique_count(encs), 1L)
  expect_equal(vec_unique_loc(encs), 1L)
})

test_that("unique functions can handle scalar types in lists", {
  x <- list(a ~ b, a ~ b, a ~ c)
  expect_equal(vec_unique(x), vec_slice(x, c(1, 3)))

  x <- list(call("x"), call("y"), call("x"))
  expect_equal(vec_unique(x), vec_slice(x, c(1, 2)))
})

test_that("duplicate functions works with different encodings", {
  encs <- encodings()

  expect_equal(vec_duplicate_id(encs), rep(1, 3))
  expect_equal(vec_duplicate_detect(encs), rep(TRUE, 3))
  expect_equal(vec_duplicate_any(encs), TRUE)
})

test_that("vec_unique() returns differently encoded strings in the order they appear", {
  encs <- encodings()
  x <- c(encs$unknown, encs$utf8)
  y <- c(encs$utf8, encs$unknown)

  expect_equal_encoding(vec_unique(x), encs$unknown)
  expect_equal_encoding(vec_unique(y), encs$utf8)
})

test_that("vec_unique() works on lists containing expressions", {
  x <- list(expression(x), expression(y), expression(x))
  expect_equal(vec_unique(x), x[1:2])
})

test_that("vec_unique() works with glm objects (#643)", {
  # class(model$family$initialize) == "expression"
  model <- glm(mpg ~ wt, data = mtcars)
  expect_equal(vec_unique(list(model, model)), list(model))
})

test_that("can take the unique locations of dfs with list-cols", {
  df <- tibble(x = list(1, 2, 1, 3), y = list(1, 2, 1, 3))
  expect_identical(vec_unique_loc(df), c(1L, 2L, 4L))
})

# matching ----------------------------------------------------------------

test_that("vec_match() matches match()", {
  n <- c(1:3, NA)
  h <- c(4, 2, 1, NA)
  expect_equal(vec_match(n, h), match(n, h))

  expect_equal(vec_match(1.5, c(2, 1.5, NA)), match(1.5, c(2, 1.5, NA)))
  expect_equal(vec_match("x", "x"), match("x", "x"))
})

test_that("vec_match() and vec_in() check types", {
  expect_snapshot({
    df1 <- data_frame(x = data_frame(foo = 1))
    df2 <- data_frame(x = data_frame(foo = ""))

    (expect_error(vec_match(df1, df2), class = "vctrs_error_incompatible_type"))
    (expect_error(
      vec_match(df1, df2, needles_arg = "n", haystack_arg = "h"),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(vec_in(df1, df2), class = "vctrs_error_incompatible_type"))
    (expect_error(
      vec_in(df1, df2, needles_arg = "n", haystack_arg = "h"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("vec_in() matches %in%", {
  n <- c(1:3, NA)
  h <- c(4, 2, 1, NA)

  expect_equal(vec_in(n, h), n %in% h)
})

test_that("can opt out of NA matching", {
  n <- c(1, NA)
  h <- c(1:3, NA)

  expect_equal(vec_in(n, h, na_equal = FALSE), c(TRUE, NA))
})

test_that("vec_match works with empty data frame", {
  out <- vec_match(
    new_data_frame(n = 3L),
    new_data_frame(n = 0L)
  )
  expect_equal(out, vec_init(integer(), 3))
})

test_that("matching functions take the equality proxy (#375)", {
  local_comparable_tuple()
  x <- tuple(c(1, 2, 1), 1:3)

  expect_identical(vec_unique_loc(x), 1:2)
  expect_identical(unique(x), tuple(c(1, 2), 1:2))

  expect_true(vec_duplicate_any(x))
  expect_identical(vec_duplicate_id(x), c(1L, 2L, 1L))
  expect_identical(vec_unique_count(x), 2L)

  expect_identical(vec_duplicate_detect(x), c(TRUE, FALSE, TRUE))
})

test_that("can take the unique loc of 1d arrays (#461)", {
  x <- array(c(1, 1, 2, 2, 3))
  y <- array(c(1, 1, 2, 2, 3), dimnames = list(NULL))
  expect_identical(vctrs::vec_unique_loc(x), int(1, 3, 5))
  expect_identical(vctrs::vec_unique_loc(y), int(1, 3, 5))

  z <- array(c(1, 1, 2, 2, 3, 4), c(3, 2))
  expect_silent(expect_identical(vctrs::vec_unique_loc(y), int(1, 3, 5)))
})

test_that("matching functions work with different encodings", {
  encs <- encodings()

  expect_equal(vec_match(encs, encs[1]), rep(1, 3))
  expect_equal(vec_in(encs, encs[1]), rep(TRUE, 3))
})

test_that("matching functions take the equality proxy recursively", {
  local_comparable_tuple()

  x <- tuple(c(1, 2), 1:2)
  df <- data_frame(x = x)

  y <- tuple(c(2, 3), c(3, 3))
  df2 <- data_frame(x = y)

  expect_equal(vec_match(df, df2), c(NA, 1))
  expect_equal(vec_in(df, df2), c(FALSE, TRUE))
})

test_that("can propagate missing values while matching", {
  exp <- c(NA, 3L, NA, 1L)
  expect_identical(
    vec_match(lgl(NA, TRUE, NA, FALSE), lgl(FALSE, NA, TRUE), na_equal = FALSE),
    exp
  )
  expect_identical(
    vec_match(int(NA, 1L, NA, 2L), int(2L, NA, 1L), na_equal = FALSE),
    exp
  )
  expect_identical(
    vec_match(dbl(NA, 1, NA, 2), dbl(2, NA, 1), na_equal = FALSE),
    exp
  )
  expect_identical(
    vec_match(cpl(NA, 1, NA, 2), cpl(2, NA, 1), na_equal = FALSE),
    exp
  )
  expect_identical(
    vec_match(chr(NA, "1", NA, "2"), chr("2", NA, "1"), na_equal = FALSE),
    exp
  )
  expect_identical(
    vec_match(list(NULL, 1, NULL, 2), list(2, NULL, 1), na_equal = FALSE),
    exp
  )

  # No missing values for raw vectors
  expect_identical(
    vec_match(raw2(0, 1, 0, 2), raw2(2, 0, 1), na_equal = FALSE),
    c(2L, 3L, 2L, 1L)
  )
})

test_that("can propagate missingness of incomplete rcrd observations (#1386)", {
  x <- new_rcrd(list(x = c(1, 1, NA, NA), y = c(1, NA, 1, NA)))
  expect_identical(vec_match(x, x, na_equal = FALSE), c(1L, NA, NA, NA))

  # Matches `vec_detect_complete()` results
  expect_identical(vec_detect_complete(x), c(TRUE, FALSE, FALSE, FALSE))
})

test_that("can propagate NaN as a missing value (#1252)", {
  expect_identical(
    vec_match(dbl(NaN, NA), c(NaN, NA), na_equal = FALSE),
    int(NA, NA)
  )
  expect_identical(
    vec_in(dbl(NaN, NA), c(NaN, NA), na_equal = FALSE),
    lgl(NA, NA)
  )
})

test_that("missing values are propagated across columns", {
  for (na_value in list(NA, na_int, na_dbl, na_cpl, na_chr, list(NULL))) {
    df <- data_frame(x = 1, y = data_frame(foo = 2, bar = na_value), z = 3)
    expect_identical(vec_match(df, df), 1L)
    expect_identical(vec_match(df, df, na_equal = FALSE), na_int)
  }
})

test_that("can't supply NA as `na_equal`", {
  expect_error(vec_match(NA, NA, na_equal = NA), "single `TRUE` or `FALSE`")
})

test_that("vec_match() and vec_in() silently fall back to base data frame", {
  expect_silent(expect_identical(
    vec_match(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    1:32
  ))
  expect_silent(expect_identical(
    vec_in(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(TRUE, 32)
  ))
})

test_that("vec_in() evaluates arg lazily", {
  expect_silent(vec_in(1L, 1L, needles_arg = print("oof")))
  expect_silent(vec_in(1L, 1L, haystack_arg = print("oof")))
})

test_that("vec_match() evaluates arg lazily", {
  expect_silent(vec_match(1L, 1L, needles_arg = print("oof")))
  expect_silent(vec_match(1L, 1L, haystack_arg = print("oof")))
})

#' Construct a list of homogenous vectors
#'
#' @description
#' A `list_of` is a restricted version of a list that adds constraints on the
#' list elements.
#'
#' - `list_of(.ptype = )` restricts the _type_ of each element.
#'
#'   - `.ptype = <type>` asserts that each element has type `<type>`.
#'
#'   - `.ptype = NULL` infers the type from the original set of elements, or
#'     errors if no vector inputs were provided.
#'
#'   - `.ptype = rlang::zap()` doesn't restrict the type.
#'
#' - `list_of(.size = )` restricts the _size_ of each element.
#'
#'   - `.size = <size>` asserts that each element has size `<size>`.
#'
#'   - `.size = NULL` infers the size from the original set of elements, or
#'     errors if no vector inputs were provided.
#'
#'   - `.size = rlang::zap()` doesn't restrict the size.
#'
#' The default behavior infers the element type and doesn't restrict the size.
#'
#' Both `.ptype` and `.size` may be specified to restrict both the size and
#' type of the list elements. You cannot set both of these to `rlang::zap()`,
#' as that would be the same as a bare `list()` with no restrictions.
#'
#' Modifying a `list_of` with `$<-`, `[<-`, and `[[<-` preserves the constraints
#' by coercing and recycling all input items.
#'
#' @param ... For `list_of()`, vectors to include in the list.
#'
#'   For other methods, these dots must be empty.
#'
#' @param x For `as_list_of()`, a vector to be coerced to list_of.
#'
#'   For `is_list_of()`, an object to test.
#'
#' @param y,to Arguments to `vec_ptype2()` and `vec_cast()`.
#'
#' @param .ptype The type to restrict each list element to. One of:
#'
#'   - A prototype like `integer()` or `double()`.
#'
#'   - `NULL`, to infer the type from `...`. If no vector inputs are provided,
#'     an error is thrown.
#'
#'   - [rlang::zap()] to avoid placing any restrictions on the type.
#'
#' @param .size The size to restrict each list element to. One of:
#'
#'   - A scalar integer size.
#'
#'   - `NULL`, to infer the size from `...`. If no vector inputs are provided,
#'     an error is thrown.
#'
#'   - [rlang::zap()] to avoid placing any restrictions on the size.
#'
#' @export
#' @examples
#' # Restrict the type, but not the size
#' x <- list_of(1:3, 5:6, 10:15)
#' x
#'
#' if (requireNamespace("tibble", quietly = TRUE)) {
#'   # As a column in a tibble
#'   tibble::tibble(x = x)
#' }
#'
#' # Coercion happens during assignment
#' x[1] <- list(4)
#' typeof(x[[1]])
#'
#' try(x[1] <- list(4.5))
#'
#' # Restrict the size, but not the type
#' x <- list_of(1, 2:3, .ptype = rlang::zap(), .size = 2)
#' x
#'
#' # Recycling happens during assignment
#' x[1] <- list(4)
#' x
#'
#' try(x[1] <- list(3:6))
#'
#' # Restricting both size and type
#' x <- list_of(1L, 2:3, .ptype = integer(), .size = 2)
#' x
#'
#' # Setting an element to `NULL`
#' x[2] <- list(NULL)
#' x
#'
#' # Note that using `NULL` shortens the list, like a base R list
#' x[2] <- NULL
#' x
#'
#' # Combining a list_of with a list results in a list
#' vec_c(list_of(1), list(2, "x"))
#'
#' # Combining a list_of with another list_of tries to find a common element
#' # type and common element size, but will remove the constraint if that
#' # fails
#' x <- list_of(1, .ptype = double())
#' y <- list_of(c("a", "b"), .ptype = character(), .size = 2)
#' z <- list_of(c("c", "d", "e"), .ptype = character(), .size = 3)
#'
#' # Falls back to a list
#' vec_c(x, y)
#'
#' # Falls back to a `list_of<character>` with no size restriction
#' vec_c(y, z)
list_of <- function(..., .ptype = NULL, .size = zap()) {
  args <- list2(...)
  list_as_list_of(args, ptype = .ptype, size = .size)
}

#' @export
#' @rdname list_of
as_list_of <- function(x, ...) {
  UseMethod("as_list_of")
}

#' @export
as_list_of.vctrs_list_of <- function(x, ...) {
  x
}

#' @export
as_list_of.list <- function(x, ..., .ptype = NULL, .size = zap()) {
  list_as_list_of(x, ptype = .ptype, size = .size)
}

#' Create list_of subclass
#'
#' @param x A list
#' @param ptype The prototype which every element of `x` belongs to. If `NULL`,
#'   the prototype is not specified.
#' @param size The size which every element of `x` has. If `NULL`, the size is
#'   not specified.
#' @param ... Additional attributes used by subclass
#' @param class Optional subclass name
#' @keywords internal
#' @export
new_list_of <- function(
  x = list(),
  ptype = logical(),
  size = NULL,
  ...,
  class = character()
) {
  obj_check_list(x)

  has_ptype <- !is_null(ptype)
  has_size <- !is_null(size)

  if (!has_ptype && !has_size) {
    abort("Must specify at least one of `ptype` or `size`.")
  }

  if (has_ptype) {
    ptype <- vec_ptype(ptype, x_arg = "ptype")
    ptype <- vec_ptype_finalise(ptype)
  }

  if (has_size) {
    check_number_whole(size, min = 0)
    size <- vec_cast(size, integer())
  }

  new_list_of0(x = x, ptype = ptype, size = size, ..., class = class)
}

new_list_of0 <- function(x, ptype, size, ..., class = character()) {
  new_vctr(
    x,
    ...,
    ptype = ptype,
    size = size,
    class = c(class, "vctrs_list_of")
  )
}

list_of_unstructure <- function(x) {
  attr(x, "ptype") <- NULL
  attr(x, "size") <- NULL
  attr(x, "class") <- NULL
  x
}

#' `list_of` attributes
#'
#' @description
#' - `list_of_ptype()` returns the `ptype` required by the `list_of`.
#'   If no `ptype` is required, then `NULL` is returned.
#'
#' - `list_of_size()` returns the `size` required by the `list_of`.
#'   If no `size` is required, then `NULL` is returned.
#'
#' @param x A [list_of][list_of()].
#'
#' @name list-of-attributes
#'
#' @examples
#' x <- list_of(1, 2)
#' list_of_ptype(x)
#' list_of_size(x)
#'
#' x <- list_of(.ptype = integer(), .size = 5)
#' list_of_ptype(x)
#' list_of_size(x)
NULL

#' @rdname list-of-attributes
#' @export
list_of_ptype <- function(x) {
  check_list_of(x)
  list_of_ptype0(x)
}

#' @rdname list-of-attributes
#' @export
list_of_size <- function(x) {
  check_list_of(x)
  list_of_size0(x)
}

list_of_ptype0 <- function(x) {
  attr(x, "ptype", exact = TRUE)
}

list_of_size0 <- function(x) {
  attr(x, "size", exact = TRUE)
}

#' @export
#' @rdname list_of
is_list_of <- function(x) {
  inherits(x, "vctrs_list_of")
}

check_list_of <- function(x, ..., arg = caller_arg(x), call = caller_env()) {
  if (is_list_of(x)) {
    return(invisible(NULL))
  }

  stop_input_type(
    x,
    "a `<list_of>`",
    ...,
    arg = arg,
    call = call
  )
}

#' @export
vec_proxy.vctrs_list_of <- function(x, ...) {
  unclass(x)
}

# Formatting --------------------------------------------------------------

#' @export
obj_print_data.vctrs_list_of <- function(x, ...) {
  if (length(x) == 0) {
    return()
  }

  print(vec_data(x))
}

#' @export
format.vctrs_list_of <- function(x, ...) {
  format.default(x)
}

#' @export
vec_ptype_full.vctrs_list_of <- function(x, ...) {
  size <- list_of_size0(x)
  if (is_null(size)) {
    size <- ""
  } else {
    size <- paste0("[", size, "]")
  }

  ptype <- list_of_ptype0(x)
  if (is_null(ptype)) {
    ptype <- "any"
  } else {
    ptype <- vec_ptype_full(ptype)
  }

  ptype <- paste0(ptype, size)

  if (grepl("\n", ptype)) {
    ptype <- paste0(indent(paste0("\n", ptype), 2), "\n")
  }

  paste0("list_of<", ptype, ">")
}

#' @export
vec_ptype_abbr.vctrs_list_of <- function(x, ...) {
  size <- list_of_size0(x)
  if (is_null(size)) {
    size <- ""
  } else {
    size <- paste0("[", size, "]")
  }

  ptype <- list_of_ptype0(x)
  if (is_null(ptype)) {
    ptype <- "any"
  } else {
    ptype <- vec_ptype_abbr(ptype)
  }

  ptype <- paste0(ptype, size)

  paste0("list<", ptype, ">")
}

# vctr methods ------------------------------------------------------------

#' @export
as.list.vctrs_list_of <- function(x, ...) {
  list_of_unstructure(x)
}

#' @export
as.character.vctrs_list_of <- function(x, ...) {
  # For compatibility with the RStudio Viewer. See tidyverse/tidyr#654.
  map_chr(x, function(elt) paste0("<", vec_ptype_abbr(elt), ">"))
}

#' @export
`[[.vctrs_list_of` <- function(x, i, ...) {
  .Call(vctrs_list_get, x, i)
}

#' @export
`$.vctrs_list_of` <- function(x, i, ...) {
  .Call(vctrs_list_get, x, i)
}

#' @export
`[<-.vctrs_list_of` <- function(x, i, value) {
  if (is_null(value)) {
    return(NextMethod())
  }

  if (!obj_is_list(value)) {
    # Ideally the user provides a list, but if `value` is not a list, we chop
    # it. This matches list semantics where this works:
    #
    # ```
    # x <- list(1, 2, 3)
    # x[1:2] <- c(4, 5)
    # ```
    value <- vec_chop(value)
  }

  ptype <- list_of_ptype0(x)
  if (!is_null(ptype)) {
    value <- map(value, vec_cast, to = ptype)
  }

  size <- list_of_size0(x)
  if (!is_null(size)) {
    value <- map(value, vec_recycle, size = size)
  }

  value <- new_list_of0(value, ptype = ptype, size = size)

  NextMethod()
}

#' @export
`[[<-.vctrs_list_of` <- function(x, i, value) {
  if (is_null(value)) {
    return(NextMethod())
  }

  ptype <- list_of_ptype0(x)
  if (!is_null(ptype)) {
    value <- vec_cast(value, ptype)
  }

  size <- list_of_size0(x)
  if (!is_null(size)) {
    value <- vec_recycle(value, size)
  }

  NextMethod()
}

#' @export
`$<-.vctrs_list_of` <- function(x, i, value) {
  if (is_null(value)) {
    return(NextMethod())
  }

  ptype <- list_of_ptype0(x)
  if (!is_null(ptype)) {
    value <- vec_cast(value, ptype)
  }

  size <- list_of_size0(x)
  if (!is_null(size)) {
    value <- vec_recycle(value, size)
  }

  NextMethod()
}

# Type system -------------------------------------------------------------

#' @rdname list_of
#' @inheritParams vec_ptype2
#' @export vec_ptype2.vctrs_list_of
#' @method vec_ptype2 vctrs_list_of
#' @export
vec_ptype2.vctrs_list_of <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.vctrs_list_of")
}

#' @method vec_ptype2.vctrs_list_of vctrs_list_of
#' @export
vec_ptype2.vctrs_list_of.vctrs_list_of <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = ""
) {
  x_ptype <- list_of_ptype0(x)
  y_ptype <- list_of_ptype0(y)

  x_size <- list_of_size0(x)
  y_size <- list_of_size0(y)

  if (identical(x_ptype, y_ptype) && identical(x_size, y_size)) {
    return(x)
  }

  # Common type always goes towards more lenient type
  #
  # Element type:
  # - If either `x_ptype` or `y_ptype` are `NULL`, fall back to `NULL`
  # - If both `x_ptype` and `y_ptype` are specified, try common type but fall
  #   back to `NULL`
  #
  # Element size:
  # - If either `x_size` or `y_size` are `NULL`, fall back to `NULL`
  # - If both `x_ptype` and `y_ptype` are specified, try common size but fall
  #   back to `NULL`
  #
  # If both `ptype` and `size` are `NULL` after this, return bare `list()`,
  # otherwise return `list_of()` with appropriate restrictions. Note that with
  # this set up we may fail a ptype2 determination but pass a size2
  # determination and still return a list-of.
  if (is_null(x_ptype) || is_null(y_ptype)) {
    ptype <- NULL
  } else {
    ptype <- tryCatch(
      vec_ptype2(x_ptype, y_ptype),
      vctrs_error_incompatible_type = function(cnd) NULL
    )
  }

  if (is_null(x_size) || is_null(y_size)) {
    size <- NULL
  } else {
    # No `vec_size2()`. This uses ALTREP to be efficient.
    size <- tryCatch(
      vec_size_common(seq_len(x_size), seq_len(y_size)),
      vctrs_error_incompatible_size = function(cnd) NULL
    )
  }

  if (is_null(ptype) && is_null(size)) {
    list()
  } else {
    new_list_of0(x = list(), ptype = ptype, size = size)
  }
}

#' @export
vec_ptype2.list.vctrs_list_of <- function(x, y, ...) {
  list()
}
#' @export
vec_ptype2.vctrs_list_of.list <- function(x, y, ...) {
  list()
}

#' @rdname list_of
#' @export vec_cast.vctrs_list_of
#' @method vec_cast vctrs_list_of
#' @export
vec_cast.vctrs_list_of <- function(x, to, ...) {
  UseMethod("vec_cast.vctrs_list_of")
}

#' @export
#' @method vec_cast.vctrs_list_of vctrs_list_of
vec_cast.vctrs_list_of.vctrs_list_of <- function(
  x,
  to,
  ...,
  call = caller_env()
) {
  x_ptype <- list_of_ptype0(x)
  to_ptype <- list_of_ptype0(to)

  x_size <- list_of_size0(x)
  to_size <- list_of_size0(to)

  if (identical(x_ptype, to_ptype) && identical(x_size, to_size)) {
    # FIXME: Suboptimal check for "same type", but should be good enough for the
    # common case of unchopping a list of identically generated list-ofs (#875).
    # Would be fixed by https://github.com/r-lib/vctrs/issues/1688.
    return(x)
  }

  x <- list_of_unstructure(x)

  ptype <- to_ptype %||% zap()
  size <- to_size %||% zap()

  list_as_list_of(
    x = x,
    ptype = ptype,
    size = size,
    error_call = call
  )
}

#' @export
vec_cast.list.vctrs_list_of <- function(x, to, ...) {
  list_of_unstructure(x)
}
#' @export
vec_cast.vctrs_list_of.list <- function(x, to, ..., call = caller_env()) {
  ptype <- list_of_ptype0(to) %||% zap()
  size <- list_of_size0(to) %||% zap()
  list_as_list_of(x, ptype = ptype, size = size, error_call = call)
}

# Helpers -----------------------------------------------------------------

list_as_list_of <- function(x, ptype, size, error_call = caller_env()) {
  free_ptype <- is_zap(ptype)
  free_size <- is_zap(size)

  if (free_ptype && free_size) {
    abort("Can't set both `ptype` and `size` to `<zap>`.", call = error_call)
  }

  if (free_ptype) {
    # Not locked
    ptype <- NULL
  } else {
    # Lock to a type or die trying
    ptype <- vec_ptype_common(
      !!!x,
      .ptype = ptype,
      .call = error_call
    )
    if (is_null(ptype)) {
      abort("Can't find common type for elements of `x`.", call = error_call)
    }
    x <- vec_cast_common(!!!x, .to = ptype, .call = error_call)
  }

  if (free_size) {
    # Not locked
    size <- NULL
  } else {
    # Lock to a size or die trying
    size <- vec_size_common(
      !!!x,
      .size = size,
      .absent = -1L,
      .call = error_call
    )
    if (size == -1L) {
      abort("Can't find common size for elements of `x`.", call = error_call)
    }
    x <- vec_recycle_common(!!!x, .size = size, .call = error_call)
  }

  new_list_of0(x = x, ptype = ptype, size = size)
}

test_that("list_of inherits from list", {
  x1 <- list_of(1, 1)
  expect_s3_class(x1, "list")
})

test_that("list_of works like list", {
  x1 <- list_of(1, 1)
  expect_type(x1, "list")
  expect_s3_class(x1, "vctrs_list_of")
  expect_equal(attr(x1, "ptype"), double())

  x2 <- list_of(1, 1, .ptype = integer())
  expect_equal(attr(x1, "ptype"), integer())

  x3 <- as_list_of(list(1, 1))
  expect_equal(x3, x1)

  x4 <- list_of(a = 1, b = 2)
  expect_equal(x4$b, 2)
  expect_error(x4$c, "Invalid index", fixed = TRUE)
})

test_that("list_of errors if it can't find common type", {
  expect_error(list_of(1, "a"), class = "vctrs_error_incompatible_type")
  expect_error(list_of(), "find common type")
})

test_that("can use as_list_of to change type", {
  x1 <- list_of(1)
  expect_equal(as_list_of(x1), x1)

  x2 <- as_list_of(x1, .ptype = integer())
  expect_identical(x2[[1]], 1L)
})

test_that("is_list_of as expected", {
  expect_false(is_list_of(list(1)))
  expect_true(is_list_of(list_of(1)))
})

test_that("print method gives human friendly output", {
  skip_on_cran() # Depends on tibble
  expect_snapshot(list_of(1, 2:3))
  expect_snapshot(tibble::tibble(x = list_of(1, 2:3)))
})

test_that("str method is reasonably correct", {
  x <- list_of(1, 2:3)

  expect_snapshot(str(x))
  expect_snapshot(str(list(list(x, y = 2:1))))

  expect_snapshot(str(x[0]))
  expect_snapshot(str(list(list(x[0], y = 2:1))))
})

test_that("constructor requires list input", {
  expect_error(new_list_of(1), "must be a list")
  expect_error(new_list_of(mtcars), "must be a list")
})

test_that("constructor requires size 0 ptype", {
  expect_error(new_list_of(ptype = 1), "must have size 0")
})

test_that("can combine a mix of named and unnamed list-ofs (#784)", {
  a <- new_list_of(list(x = 1L), ptype = integer())
  b <- new_list_of(list(2L), ptype = integer())

  expect <- new_list_of(list(x = 1L, 2L), ptype = integer())

  expect_identical(vec_c(a, b), expect)
})

# Subsetting --------------------------------------------------------------

test_that("[ preserves type", {
  x <- list_of(1)
  expect_equal(x[1], x)
})

test_that("[<-, [[<- and $<- coerce their input", {
  x <- list_of(x = 1, y = 1, z = 1, w = 1)

  x[1] <- list(FALSE)
  expect_identical(x, list_of(x = 0, y = 1, z = 1, w = 1))

  x[[2]] <- FALSE
  expect_identical(x, list_of(x = 0, y = 0, z = 1, w = 1))

  x$z <- FALSE
  expect_identical(x, list_of(x = 0, y = 0, z = 0, w = 1))

  x[3:4] <- c(TRUE, FALSE)
  expect_identical(x, list_of(x = 0, y = 0, z = 1, w = 0))

  x[[2]] <- NULL
  expect_equal(x, list_of(x = 0, y = NULL, z = 1, w = 0))

  expect_error(x[[2]] <- list(20), class = "vctrs_error_incompatible_type")
  expect_error(x$y <- list(20), class = "vctrs_error_incompatible_type")

  x[3:4] <- list(NULL)
  expect_equal(x, list_of(x = 0, y = NULL, z = NULL, w = NULL))

  expect_equal(is.na(x), c(FALSE, TRUE, TRUE, TRUE))
})

test_that("assingment can increase size of vector", {
  x <- list_of(x = 1)
  x[[2]] <- 2
  x$z <- 3
  x[4:5] <- c(4,5)

  expect_length(x, 5)
})


# Type system -------------------------------------------------------------

test_that("list_of() are vectors", {
  expect_true(obj_is_vector(list_of(1)))
  expect_true(vec_is(list_of(1)))
})

test_that("list coercions are symmetric and unchanging", {
  types <- list(
    list(),
    list_of(.ptype = integer()),
    list_of(.ptype = double()),
    list_of(.ptype = character())
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("max<list_of<a>, list_of<b>> is list_of<max<a, b>>", {
  r_int <- list_of(.ptype = integer())
  r_dbl <- list_of(.ptype = double())

  expect_equal(vec_ptype_common(r_int, r_int), r_int)
  expect_equal(vec_ptype_common(r_int, r_dbl), r_int)
})

test_that("can cast to self type", {
  x <- list_of(1)
  expect_identical(vec_cast(x, x), x)
})

test_that("can cast between different list_of types", {
  x <- list_of(1, 2)
  to <- list_of(.ptype = integer())
  expect_identical(vec_cast(x, to), list_of(1L, 2L))
})

test_that("list_of casting retains outer names", {
  x <- list_of(x = 1, 2, z = 3)
  to <- list_of(.ptype = integer())
  expect_named(vec_cast(x, to), c("x", "", "z"))
})

test_that("safe casts work as expected", {
  x <- list_of(1)
  expect_equal(vec_cast(NULL, x), NULL)
  expect_equal(vec_cast(NA, x), list_of(NULL, .ptype = double()))

  expect_identical(vec_cast(list(1), x), list_of(1))
  expect_identical(vec_cast(list(TRUE), x), list_of(1))
  expect_identical(vec_cast(x, list()), list(1))
  expect_identical(vec_cast(x, list()), list(1))

  expect_error(
    vec_cast(list_of(1), list_of("")),
    class = "vctrs_error_incompatible_type"
  )

  # These used to be allowed
  expect_error(vec_cast(1L, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1, x), class = "vctrs_error_incompatible_type")
})

test_that("error call is passed to inner cast methods", {
  fn1 <- function() vec_cast(list_of(1), list_of(""))
  fn2 <- function() vec_cast(list(1), list_of(""))
  expect_snapshot({
    (expect_error(fn1()))
    (expect_error(fn2()))
  })
})

test_that("lossy casts generate warning (no longer the case)", {
  # This used to be a lossy cast warning
  expect_error(
    vec_cast(list(c(1.5, 1), 1L), to = list_of(1L)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(factor("a"), list_of(1)), class = "vctrs_error_incompatible_type")
})

test_that("list_of() has as.character() method (tidyverse/tidyr#654)", {
  exp <- rep(paste0("<", vec_ptype_abbr(mtcars), ">"), 2)
  expect_identical(as.character(list_of(mtcars, mtcars)), exp)
})

test_that("vec_ptype2(<list_of<>>, NA) is symmetric (#687)", {
  lof <- list_of(1, 2, 3)
  expect_identical(vec_ptype2(lof, NA), vec_ptype(lof))
  expect_identical(vec_ptype2(NA, lof), vec_ptype(lof))
})

test_that("list_of() coerces to list() and list_of() (#1701)", {
  expect_equal(vec_ptype_common(list_of(1), list()), list())
  expect_equal(vec_cast_common(list_of(1), list()), list(list(1), list()))

  expect_equal(vec_ptype_common(list_of(1), list("")), list())
  expect_equal(vec_cast_common(list_of(1), list("")), list(list(1), list("")))

  expect_equal(
    vec_ptype_common(list_of(1), list_of("")),
    list()
  )
  expect_equal(
    vec_ptype_common(list_of(1), list(), list_of("")),
    list()
  )
})

test_that("can concatenate list and list-of (#1161)", {
  expect_equal(
    vec_c(list(1), list_of(2)),
    list(1, 2)
  )
  expect_equal(
    vec_c(list(""), list_of(2)),
    list("", 2)
  )
})

test_that("transposes vectors", {
  expect_identical(
    list_of_transpose(list_of2(1:2, 3:4, 5:6)),
    list_of2(c(1L, 3L, 5L), c(2L, 4L, 6L))
  )
})

test_that("transposes data frames", {
  expect_identical(
    list_of_transpose(list_of2(
      data_frame(a = 1:3, b = letters[1:3]),
      data_frame(a = 4:6, b = letters[4:6])
    )),
    list_of2(
      data_frame(a = c(1L, 4L), b = letters[c(1L, 4L)]),
      data_frame(a = c(2L, 5L), b = letters[c(2L, 5L)]),
      data_frame(a = c(3L, 6L), b = letters[c(3L, 6L)])
    )
  )
})

test_that("empty `x` is fully reversible", {
  x <- list_of(.ptype = integer(), .size = 0)
  out <- list_of_transpose(x)
  expect_identical(
    out,
    list_of(.ptype = integer(), .size = 0)
  )
  expect_identical(list_of_transpose(out), x)

  x <- list_of(.ptype = integer(), .size = 2)
  out <- list_of_transpose(x)
  expect_identical(
    out,
    list_of(integer(), integer(), .ptype = integer(), .size = 0)
  )
  expect_identical(list_of_transpose(out), x)
})

test_that("retains only inner names", {
  # I don't think we should expose `name_spec`, we've hard coded it to `"inner"`
  # for now. What would this even do with outer names? Exposing `name_spec` for
  # the interleave step would allow making names of `a_w` and `b_y` via a glue
  # spec, which feels weird and not useful.
  x <- list_of2(a = c(w = 1, x = 2), b = c(y = 3, z = 4))

  expect_identical(
    list_of_transpose(x),
    list_of2(
      c(w = 1, y = 3),
      c(x = 2, z = 4)
    )
  )

  # Silent repair of duplicate data frame row names
  x <- list_of2(
    data.frame(a = 1, row.names = "x"),
    data.frame(a = 2, row.names = "x")
  )

  expect_silent({
    expect_identical(
      list_of_transpose(x),
      list_of2(data.frame(a = c(1, 2), row.names = c("x...1", "x...2")))
    )
  })
})

test_that("`x` is validated", {
  expect_snapshot(error = TRUE, {
    list_of_transpose(1)
  })
  expect_snapshot(error = TRUE, {
    list_of_transpose(1, x_arg = "x", error_call = quote(foo()))
  })
})

test_that("`x` must be a fully specified list of", {
  expect_snapshot(error = TRUE, {
    x <- list_of(.ptype = integer(), .size = zap())
    list_of_transpose(x)
  })
  expect_snapshot(error = TRUE, {
    x <- list_of(.ptype = zap(), .size = 1)
    list_of_transpose(x)
  })
})

test_that("`...` must be empty", {
  expect_snapshot(error = TRUE, {
    list_of_transpose(list_of2(1), 2)
  })
})

test_that("doesn't allow `NULL` elements", {
  # These would break the invariants around the size of the output relative
  # to the size of the input if we just dropped them. We require that the user
  # drop them entirely or replace them up front. The `[<-` method for list-of
  # ensures that the type and size is maintained for whatever they replace with.
  expect_snapshot(error = TRUE, {
    list_of_transpose(list_of2(1:4, NULL, 5:8))
  })
})

test_that("`x` being a list subclass can't affect the transposition", {
  x <- new_list_of(
    list(1, 2),
    ptype = double(),
    size = 1L,
    class = "my_list"
  )

  expect_identical(
    list_of_transpose(x),
    list_of2(c(1, 2))
  )
})

# ------------------------------------------------------------------------------
# new_list_of

test_that("works without any arguments", {
  # Default is logical type, no size restriction
  expect_identical(
    new_list_of(),
    new_list_of(x = list(), ptype = logical(), size = NULL)
  )
})

test_that("constructor requires list input", {
  expect_snapshot(error = TRUE, {
    new_list_of(1)
  })
  expect_snapshot(error = TRUE, {
    new_list_of(mtcars)
  })
})

test_that("must lock at least one of ptype or size", {
  expect_snapshot(error = TRUE, {
    new_list_of(ptype = NULL, size = NULL)
  })
})

test_that("can lock ptype, size, or both", {
  x <- new_list_of(ptype = integer(), size = NULL)
  expect_identical(list_of_ptype(x), integer())
  expect_null(list_of_size(x))

  x <- new_list_of(ptype = NULL, size = 5L)
  expect_null(list_of_ptype(x))
  expect_identical(list_of_size(x), 5L)

  x <- new_list_of(ptype = integer(), size = 5L)
  expect_identical(list_of_ptype(x), integer())
  expect_identical(list_of_size(x), 5L)
})

test_that("validates `ptype`", {
  expect_snapshot(error = TRUE, {
    new_list_of(ptype = lm(1 ~ 1))
  })

  x <- new_list_of(ptype = 1:5)
  expect_identical(list_of_ptype(x), integer())
})

test_that("finalizes `ptype`", {
  x <- new_list_of(ptype = unspecified())
  expect_identical(list_of_ptype(x), logical())
})

test_that("validates `size`", {
  expect_snapshot(error = TRUE, {
    new_list_of(size = 1.1)
  })
  expect_snapshot(error = TRUE, {
    new_list_of(size = 1:2)
  })
  expect_snapshot(error = TRUE, {
    new_list_of(size = -5)
  })
})

test_that("has vctrs classes", {
  expect_identical(
    class(new_list_of()),
    c("vctrs_list_of", "vctrs_vctr", "list")
  )
})

test_that("can add extra class", {
  expect_s3_class(new_list_of(class = "foo_list_of"), "foo_list_of")
})

test_that("can add extra attributes", {
  x <- new_list_of(foo = "bar")
  expect_identical(attr(x, "foo"), "bar")
})

# ------------------------------------------------------------------------------
# is_list_of

test_that("is_list_of works as expected", {
  expect_false(is_list_of(list(1)))
  expect_true(is_list_of(list_of(1)))
})

test_that("can check for list of", {
  expect_snapshot(error = TRUE, {
    check_list_of(1)
  })
})

# ------------------------------------------------------------------------------
# list_of

test_that("default behavior infers ptype and doesn't lock size", {
  x <- list_of(1)
  expect_identical(list_of_ptype(x), double())
  expect_null(list_of_size(x))
})

test_that("errors if can't determine type", {
  expect_snapshot(error = TRUE, {
    list_of(.ptype = NULL)
  })
  expect_snapshot(error = TRUE, {
    list_of(1, "a")
  })
})

test_that("errors if can't determine size", {
  expect_snapshot(error = TRUE, {
    list_of(.ptype = zap(), .size = NULL)
  })
  expect_snapshot(error = TRUE, {
    list_of(1:2, 3:5, .ptype = zap(), .size = NULL)
  })
})

test_that("can specify ptype, size, or both", {
  x <- list_of(.ptype = integer(), .size = zap())
  expect_identical(list_of_ptype(x), integer())
  expect_null(list_of_size(x))

  x <- list_of(.ptype = zap(), .size = 5L)
  expect_null(list_of_ptype(x))
  expect_identical(list_of_size(x), 5L)

  x <- list_of(.ptype = integer(), .size = 5L)
  expect_identical(list_of_ptype(x), integer())
  expect_identical(list_of_size(x), 5L)
})

# ------------------------------------------------------------------------------
# as_list_of

test_that("can convert from list to list-of", {
  x <- list(1)
  expect_identical(as_list_of(x), list_of(1))
})

test_that("default behavior infers ptype and doesn't lock size", {
  x <- as_list_of(list(1))
  expect_identical(list_of_ptype(x), double())
  expect_null(list_of_size(x))
})

test_that("errors if can't determine type", {
  expect_snapshot(error = TRUE, {
    as_list_of(list())
  })
})

test_that("errors if can't determine size", {
  expect_snapshot(error = TRUE, {
    as_list_of(list(), .ptype = integer(), .size = NULL)
  })
})

test_that("can specify ptype, size, or both", {
  x <- as_list_of(list(), .ptype = integer(), .size = zap())
  expect_identical(list_of_ptype(x), integer())
  expect_null(list_of_size(x))

  x <- as_list_of(list(), .ptype = zap(), .size = 5L)
  expect_null(list_of_ptype(x))
  expect_identical(list_of_size(x), 5L)

  x <- as_list_of(list(), .ptype = integer(), .size = 5L)
  expect_identical(list_of_ptype(x), integer())
  expect_identical(list_of_size(x), 5L)
})

test_that("as_list_of on list_of is a no-op", {
  x <- list_of(1)
  expect_identical(as_list_of(x), x)

  # Used to be able to do this in `as_list_of.vctrs_list_of`, but we determined
  # you should just go through `list_of()` again.
  expect_identical(as_list_of(x, .ptype = integer()), x)
})

# ------------------------------------------------------------------------------
# list_of_size

test_that("`list_of_size()` returns the `size`", {
  x <- list_of(1:2, .size = 2, .ptype = zap())
  expect_identical(list_of_size(x), 2L)

  x <- list_of(1:2, .size = zap(), .ptype = integer())
  expect_null(list_of_size(x))
})

test_that("`list_of_size()` validates `x`", {
  expect_snapshot(error = TRUE, {
    list_of_size(list(1))
  })
})

# ------------------------------------------------------------------------------
# list_of_ptype

test_that("`list_of_ptype()` returns the `size`", {
  x <- list_of(1:2, .ptype = integer(), .size = zap())
  expect_identical(list_of_ptype(x), integer())

  x <- list_of(1:2, .ptype = zap(), .size = 2L)
  expect_null(list_of_ptype(x))
})

test_that("`list_of_ptype()` validates `x`", {
  expect_snapshot(error = TRUE, {
    list_of_ptype(list(1))
  })
})

# ------------------------------------------------------------------------------
# as.list()

test_that("can convert to base list", {
  x <- list_of(1)
  expect_identical(as.list(x), list(1))
})

# ------------------------------------------------------------------------------
# as.character()

test_that("list_of() has as.character() method (tidyverse/tidyr#654)", {
  exp <- rep(paste0("<", vec_ptype_abbr(mtcars), ">"), 2)
  expect_identical(as.character(list_of(mtcars, mtcars)), exp)
})

# ------------------------------------------------------------------------------
# Formatting

test_that("can print empty list-of", {
  expect_snapshot(list_of(.ptype = integer(), .size = 5L))
})

test_that("print method gives human friendly output", {
  # Just ptype
  x <- list_of(1, 2:3, .ptype = double(), .size = zap())
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))

  # Just size
  x <- list_of(1, 2:3, .ptype = zap(), .size = 2L)
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))

  # Both ptype and size
  x <- list_of(1, 2:3, .ptype = double(), .size = 2L)
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))
})

test_that("print method gives human friendly output for multi line types", {
  ptype <- data_frame(x = integer(), y = double(), z = character())

  # Just ptype
  x <- list_of(.ptype = ptype, .size = zap())
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))

  # Just size
  x <- list_of(.ptype = zap(), .size = 2L)
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))

  # Both ptype and size
  x <- list_of(.ptype = ptype, .size = 2L)
  expect_snapshot(cat(vec_ptype_full(x)))
  expect_snapshot(cat(vec_ptype_abbr(x)))
})

test_that("str method is reasonably correct", {
  x <- list_of(1, 2:3)

  expect_snapshot(str(x))
  expect_snapshot(str(list(list(x, y = 2:1))))

  expect_snapshot(str(x[0]))
  expect_snapshot(str(list(list(x[0], y = 2:1))))
})

# ------------------------------------------------------------------------------
# Subsetting

test_that("[ preserves type", {
  x <- list_of(1)
  expect_identical(x[1], x)

  x <- list_of(1, .ptype = double(), .size = 1)
  expect_identical(x[1], x)
})

test_that("[[ works", {
  x <- list_of(a = 1, b = 2)

  expect_identical(x[[1]], 1)
  expect_identical(x[["b"]], 2)

  expect_snapshot(error = TRUE, {
    x[[3]]
  })
  expect_snapshot(error = TRUE, {
    x[["c"]]
  })
})

test_that("$ works", {
  x <- list_of(a = 1, b = 2)

  expect_identical(x$b, 2)

  expect_snapshot(error = TRUE, {
    x$c
  })
})

test_that("[<- coerces and recycles", {
  # Just type
  x <- list_of(a = 1, b = 2, .ptype = double(), .size = zap())

  x[1] <- list(NULL)
  expect_identical(
    x,
    list_of(a = NULL, b = 2, .ptype = double(), .size = zap())
  )

  x[1] <- list(3)
  expect_identical(x[[1]], 3)

  # Casts automatically
  x[1] <- list(4:5)
  expect_identical(x[[1]], c(4, 5))

  expect_snapshot(error = TRUE, {
    x[1] <- list("5")
  })

  # Just size
  x <- list_of(a = 1:2, b = c("c", "d"), .ptype = zap(), .size = 2L)

  x[1] <- list(NULL)
  expect_identical(
    x,
    list_of(a = NULL, b = c("c", "d"), .ptype = zap(), .size = 2L)
  )

  x[1] <- list(c(5, 6))
  expect_identical(x[[1]], c(5, 6))

  # Recycles automatically
  x[1] <- list(7)
  expect_identical(x[[1]], c(7, 7))

  expect_snapshot(error = TRUE, {
    x[1] <- list(c(1, 2, 3))
  })
})

test_that("[[<- coerces and recycles", {
  # Just type
  x <- list_of(a = 1, b = 2, .ptype = double(), .size = zap())

  x[[1]] <- 3
  expect_identical(x[[1]], 3)

  # Casts automatically
  x[[1]] <- 4:5
  expect_identical(x[[1]], c(4, 5))

  expect_snapshot(error = TRUE, {
    x[[1]] <- "5"
  })

  # Just size
  x <- list_of(a = 1:2, b = c("c", "d"), .ptype = zap(), .size = 2L)

  x[[1]] <- c(5, 6)
  expect_identical(x[[1]], c(5, 6))

  # Recycles automatically
  x[[1]] <- 7
  expect_identical(x[[1]], c(7, 7))

  expect_snapshot(error = TRUE, {
    x[[1]] <- c(1, 2, 3)
  })
})

test_that("$<- coerces and recycles", {
  # Just type
  x <- list_of(a = 1, b = 2, .ptype = double(), .size = zap())

  x$a <- 3
  expect_identical(x$a, 3)

  # Casts automatically
  x$a <- 4:5
  expect_identical(x$a, c(4, 5))

  expect_snapshot(error = TRUE, {
    x$a <- "5"
  })

  # Just size
  x <- list_of(a = 1:2, b = c("c", "d"), .ptype = zap(), .size = 2L)

  x$a <- c(5, 6)
  expect_identical(x$a, c(5, 6))

  # Recycles automatically
  x$a <- 7
  expect_identical(x$a, c(7, 7))

  expect_snapshot(error = TRUE, {
    x$a <- c(1, 2, 3)
  })
})

test_that("[<- chops non list `value`s", {
  # Chops into `list(4, 5)` before casting or recycling
  x <- list_of(a = 1, b = 2, c = 3)
  x[2:3] <- c(4L, 5L)
  expect_identical(x, list_of(a = 1, b = 4, c = 5))

  # Chops into `list(7, 8)` before casting or recycling
  x <- list_of(a = 1:2, b = 2:3, c = 3:4, .size = 2L)
  x[2:3] <- c(7, 8)
  expect_identical(
    x,
    list_of(a = 1:2, b = c(7L, 7L), c = c(8L, 8L), .size = 2L)
  )
})

test_that("[<- can shorten with `NULL` (#2112)", {
  x <- list_of(a = 1, b = 2, c = 3)
  x[2] <- NULL
  expect_identical(x, list_of(a = 1, c = 3))

  x <- list_of(a = 1, b = 2, c = 3)
  x[2:3] <- NULL
  expect_identical(x, list_of(a = 1))

  # Like base lists
  x <- list(a = 1, b = 2, c = 3)
  x[2] <- NULL
  expect_identical(x, list(a = 1, c = 3))
})

test_that("[[<- can shorten with `NULL` (#2112)", {
  x <- list_of(a = 1, b = 2)
  x[[2]] <- NULL
  expect_identical(x, list_of(a = 1))

  x <- list_of(a = 1, b = 2)
  x[["b"]] <- NULL
  expect_identical(x, list_of(a = 1))

  # Like base lists
  x <- list(a = 1, b = 2)
  x[[2]] <- NULL
  expect_identical(x, list(a = 1))
})

test_that("$<- can shorten with `NULL` (#2112)", {
  x <- list_of(a = 1, b = 2)
  x$b <- NULL
  expect_identical(x, list_of(a = 1))

  # Like base lists
  x <- list(a = 1, b = 2)
  x$b <- NULL
  expect_identical(x, list(a = 1))
})

test_that("assignment can increase size of vector", {
  x <- list_of(x = 1)
  x[[2]] <- 2
  x$z <- 3
  x[4:5] <- c(4, 5)

  expect_length(x, 5)
})

# ------------------------------------------------------------------------------
# Type system

test_that("list_of() are vectors", {
  expect_true(obj_is_vector(list_of(1)))
  expect_true(vec_is(list_of(1)))
})

test_that("ptype2: list + list_of", {
  expect_identical(vec_ptype2(list(), new_list_of()), list())
  expect_identical(vec_ptype2(new_list_of(), list()), list())
})

test_that("ptype2: list_of + list_of", {
  # Self-self
  x <- list_of(.ptype = integer(), .size = zap())
  expect_identical(vec_ptype2(x, x), x)

  x <- list_of(.ptype = zap(), .size = 2L)
  expect_identical(vec_ptype2(x, x), x)

  x <- list_of(.ptype = integer(), .size = 2L)
  expect_identical(vec_ptype2(x, x), x)

  # All `size` variants with one fixed `ptype`
  x <- list_of(.ptype = integer(), .size = zap())
  y <- list_of(.ptype = zap(), .size = 2L)
  expect_identical(vec_ptype2(x, y), list())
  expect_identical(vec_ptype2(y, x), list())

  x <- list_of(.ptype = integer(), .size = 2L)
  y <- list_of(.ptype = zap(), .size = 2L)
  expect_identical(vec_ptype2(x, y), list_of(.ptype = zap(), .size = 2L))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = zap(), .size = 2L))

  x <- list_of(.ptype = integer(), .size = 1L)
  y <- list_of(.ptype = zap(), .size = 2L)
  expect_identical(vec_ptype2(x, y), list_of(.ptype = zap(), .size = 2L))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = zap(), .size = 2L))

  x <- list_of(.ptype = integer(), .size = 2L)
  y <- list_of(.ptype = zap(), .size = 3L)
  expect_identical(vec_ptype2(x, y), list())
  expect_identical(vec_ptype2(y, x), list())

  # All `ptype` variants with one fixed `size`
  x <- list_of(.ptype = zap(), .size = 1L)
  y <- list_of(.ptype = integer(), .size = zap())
  expect_identical(vec_ptype2(x, y), list())
  expect_identical(vec_ptype2(y, x), list())

  x <- list_of(.ptype = integer(), .size = 1L)
  y <- list_of(.ptype = integer(), .size = zap())
  expect_identical(vec_ptype2(x, y), list_of(.ptype = integer(), .size = zap()))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = integer(), .size = zap()))

  x <- list_of(.ptype = integer(), .size = 1L)
  y <- list_of(.ptype = double(), .size = zap())
  expect_identical(vec_ptype2(x, y), list_of(.ptype = double(), .size = zap()))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = double(), .size = zap()))

  x <- list_of(.ptype = integer(), .size = 1L)
  y <- list_of(.ptype = character(), .size = zap())
  expect_identical(vec_ptype2(x, y), list())
  expect_identical(vec_ptype2(y, x), list())

  # All variants with both `size` and `ptype` fixed
  x <- list_of(.ptype = integer(), .size = 1L)
  y <- list_of(.ptype = integer(), .size = 1L)
  expect_identical(vec_ptype2(x, y), x)
  expect_identical(vec_ptype2(y, x), x)

  x <- list_of(.ptype = integer(), .size = 2L)
  y <- list_of(.ptype = double(), .size = 1L)
  expect_identical(vec_ptype2(x, y), list_of(.ptype = double(), .size = 2L))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = double(), .size = 2L))

  x <- list_of(.ptype = integer(), .size = 2L)
  y <- list_of(.ptype = double(), .size = 3L)
  expect_identical(vec_ptype2(x, y), list_of(.ptype = double(), .size = zap()))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = double(), .size = zap()))

  x <- list_of(.ptype = integer(), .size = 2L)
  y <- list_of(.ptype = character(), .size = 1L)
  expect_identical(vec_ptype2(x, y), list_of(.ptype = zap(), .size = 2L))
  expect_identical(vec_ptype2(y, x), list_of(.ptype = zap(), .size = 2L))
})

test_that("cast: list_of to list", {
  x <- list_of(1, .ptype = double(), .size = zap())
  expect_identical(vec_cast(x, list()), list(1))

  x <- list_of(1, .ptype = zap(), .size = 1L)
  expect_identical(vec_cast(x, list()), list(1))

  x <- list_of(1, .ptype = double(), .size = 1L)
  expect_identical(vec_cast(x, list()), list(1))
})

test_that("cast: list to list_of", {
  # Just type
  to <- list_of(1, .ptype = double(), .size = zap())
  expect_identical(
    vec_cast(list(), to),
    list_of(.ptype = double(), .size = zap())
  )
  expect_snapshot(error = TRUE, {
    vec_cast(list("x"), to)
  })

  # Just size
  to <- list_of(1:2, .ptype = zap(), .size = 2L)
  expect_identical(
    vec_cast(list(), to),
    list_of(.ptype = zap(), .size = 2L)
  )
  expect_identical(
    vec_cast(list(1, "x"), to),
    list_of(c(1, 1), c("x", "x"), .ptype = zap(), .size = 2L)
  )
  expect_snapshot(error = TRUE, {
    vec_cast(list(1:3), to)
  })

  # Both type and size
  to <- list_of(1:2, .ptype = integer(), .size = 2L)
  expect_identical(
    vec_cast(list(), to),
    list_of(.ptype = integer(), .size = 2L)
  )
  expect_identical(
    vec_cast(list(1, 2:3), to),
    list_of(c(1L, 1L), 2:3, .ptype = integer(), .size = 2L)
  )
  expect_snapshot(error = TRUE, {
    vec_cast(list(1:3), to)
  })
  expect_snapshot(error = TRUE, {
    vec_cast(list("x"), to)
  })
})

test_that("cast: list_of to list_of", {
  # Starts just type restricted
  x <- list_of(1, 2)

  # To a different type restricted
  to <- list_of(.ptype = integer(), .size = zap())
  expect_identical(vec_cast(x, to), list_of(1L, 2L))

  to <- list_of(.ptype = character(), .size = zap())
  expect_snapshot(error = TRUE, {
    vec_cast(x, to)
  })

  # To a size restricted instead
  to <- list_of(.ptype = zap(), .size = 1L)
  expect_identical(vec_cast(x, to), list_of(1, 2, .ptype = zap(), .size = 1L))

  to <- list_of(.ptype = zap(), .size = 2L)
  expect_identical(
    vec_cast(x, to),
    list_of(c(1, 1), c(2, 2), .ptype = zap(), .size = 2L)
  )

  y <- list_of(1:2, 2:3)
  to <- list_of(.ptype = zap(), .size = 3L)
  expect_snapshot(error = TRUE, {
    vec_cast(y, to)
  })

  # To both type and size restricted
  to <- list_of(.ptype = integer(), .size = 1L)
  expect_identical(
    vec_cast(x, to),
    list_of(1L, 2L, .ptype = integer(), .size = 1L)
  )

  to <- list_of(.ptype = integer(), .size = 2L)
  expect_identical(
    vec_cast(x, to),
    list_of(c(1L, 1L), c(2L, 2L), .ptype = integer(), .size = 2L)
  )
})

test_that("list coercions are symmetric and unchanging", {
  types <- list(
    list(),
    list_of(.ptype = integer()),
    list_of(.ptype = double()),
    list_of(.ptype = character())
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))

  local_options(width = 200)
  expect_snapshot(print(mat))
})

test_that("max<list_of<a>, list_of<b>> is list_of<max<a, b>>", {
  r_int <- list_of(.ptype = integer())
  r_dbl <- list_of(.ptype = double())

  expect_identical(vec_ptype_common(r_int, r_int), r_int)
  expect_identical(vec_ptype_common(r_dbl, r_dbl), r_dbl)
  expect_identical(vec_ptype_common(r_int, r_dbl), r_dbl)
  expect_identical(vec_ptype_common(r_dbl, r_int), r_dbl)

  r_one <- list_of(.ptype = zap(), .size = 1L)
  r_two <- list_of(.ptype = zap(), .size = 2L)

  expect_identical(vec_ptype_common(r_one, r_one), r_one)
  expect_identical(vec_ptype_common(r_two, r_two), r_two)
  expect_identical(vec_ptype_common(r_one, r_two), r_two)
  expect_identical(vec_ptype_common(r_two, r_one), r_two)
})

test_that("can cast to self type", {
  x <- list_of(1, .ptype = double(), .size = zap())
  expect_identical(vec_cast(x, x), x)

  x <- list_of(c(1, 2), .ptype = zap(), .size = 2L)
  expect_identical(vec_cast(x, x), x)

  x <- list_of(c(1, 2), .ptype = double(), .size = 2L)
  expect_identical(vec_cast(x, x), x)
})

test_that("list_of casting retains outer names", {
  x <- list_of(x = 1, 2, z = 3)

  to <- list_of(.ptype = integer())
  expect_named(vec_cast(x, to), c("x", "", "z"))

  to <- list_of(.ptype = zap(), .size = 2L)
  expect_named(vec_cast(x, to), c("x", "", "z"))
})

test_that("safe casts work as expected", {
  x <- list_of(1)
  expect_equal(vec_cast(NULL, x), NULL)
  expect_equal(vec_cast(NA, x), list_of(NULL, .ptype = double()))

  expect_identical(vec_cast(list(1), x), list_of(1))
  expect_identical(vec_cast(list(TRUE), x), list_of(1))
  expect_identical(vec_cast(x, list()), list(1))
  expect_identical(vec_cast(x, list()), list(1))

  expect_error(
    vec_cast(list_of(1), list_of("")),
    class = "vctrs_error_incompatible_type"
  )

  # These used to be allowed
  expect_error(vec_cast(1L, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(1, x), class = "vctrs_error_incompatible_type")
})

test_that("error call is passed to inner cast methods", {
  fn1 <- function() vec_cast(list_of(1), list_of(""))
  fn2 <- function() vec_cast(list(1), list_of(""))
  expect_snapshot({
    (expect_error(fn1()))
    (expect_error(fn2()))
  })
})

test_that("lossy casts generate warning (no longer the case)", {
  # This used to be a lossy cast warning
  expect_error(
    vec_cast(list(c(1.5, 1), 1L), to = list_of(1L)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(factor("a"), list_of(1)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype2(<list_of<>>, NA) is symmetric (#687)", {
  lof <- list_of(1, 2, 3, .ptype = double(), .size = zap())
  expect_identical(vec_ptype2(lof, NA), vec_ptype(lof))
  expect_identical(vec_ptype2(NA, lof), vec_ptype(lof))

  lof <- list_of(1, 2, 3, .ptype = zap(), .size = 1L)
  expect_identical(vec_ptype2(lof, NA), vec_ptype(lof))
  expect_identical(vec_ptype2(NA, lof), vec_ptype(lof))
})

test_that("list_of() coerces to list() and list_of() (#1701)", {
  expect_equal(vec_ptype_common(list_of(1), list()), list())
  expect_equal(vec_cast_common(list_of(1), list()), list(list(1), list()))

  expect_equal(vec_ptype_common(list_of(1), list("")), list())
  expect_equal(vec_cast_common(list_of(1), list("")), list(list(1), list("")))

  # Fallback on common type failure
  expect_equal(
    vec_ptype_common(list_of(1), list_of("")),
    list()
  )
  expect_equal(
    vec_ptype_common(list_of(1), list(), list_of("")),
    list()
  )

  # Fallback on common size failure
  expect_equal(
    vec_ptype_common(
      list_of(1:2, .ptype = zap(), .size = 2L),
      list_of(1:3, .ptype = zap(), .size = 3L)
    ),
    list()
  )
  expect_equal(
    vec_ptype_common(
      list_of(1:2, .ptype = zap(), .size = 2L),
      list(),
      list_of(1:3, .ptype = zap(), .size = 3L)
    ),
    list()
  )
})

test_that("can concatenate list and list-of (#1161)", {
  # With fixed type
  expect_equal(
    vec_c(list(1), list_of(2)),
    list(1, 2)
  )
  expect_equal(
    vec_c(list(""), list_of(2)),
    list("", 2)
  )

  # With fixed size
  expect_equal(
    vec_c(list(1), list_of(2:3, .ptype = zap(), .size = 2)),
    list(1, 2:3)
  )
  expect_equal(
    vec_c(list(""), list_of(2:3, .ptype = zap(), .size = 2)),
    list("", 2:3)
  )
})

test_that("can combine a mix of named and unnamed list-ofs (#784)", {
  a <- new_list_of(list(x = 1L), ptype = integer())
  b <- new_list_of(list(2L), ptype = integer())

  expect <- new_list_of(list(x = 1L, 2L), ptype = integer())

  expect_identical(vec_c(a, b), expect)
})

#' Find the common type for a pair of vectors
#'
#' @description
#'
#' `vec_ptype2()` defines the coercion hierarchy for a set of related
#' vector types. Along with [vec_cast()], this generic forms the
#' foundation of type coercions in vctrs.
#'
#' `vec_ptype2()` is relevant when you are implementing vctrs methods
#' for your class, but it should not usually be called directly. If
#' you need to find the common type of a set of inputs, call
#' [vec_ptype_common()] instead. This function supports multiple
#' inputs and [finalises][vec_ptype_finalise] the common type.
#'
#' @includeRmd man/faq/developer/links-coercion.Rmd
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#' @param x,y Vector types.
#' @param x_arg,y_arg Argument names for `x` and `y`. These are used
#'   in error messages to inform the user about the locations of
#'   incompatible types (see [stop_incompatible_type()]).
#'
#' @seealso [stop_incompatible_type()] when you determine from the
#'   attributes that an input can't be cast to the target type.
#'
#' @section Dependencies:
#' - [vec_ptype()] is applied to `x` and `y`
#'
#' @export
vec_ptype2 <- function(
  x,
  y,
  ...,
  x_arg = caller_arg(x),
  y_arg = caller_arg(y),
  call = caller_env()
) {
  if (!missing(...)) {
    check_ptype2_dots_empty(...)
    return(vec_ptype2_opts(
      x,
      y,
      opts = match_fallback_opts(...),
      x_arg = x_arg,
      y_arg = y_arg,
      call = call
    ))
  }
  return(.Call(ffi_ptype2, x, y, environment()))
  UseMethod("vec_ptype2")
}
vec_ptype2_dispatch_s3 <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  UseMethod("vec_ptype2")
}

vec_ptype2_dispatch_native <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  fallback_opts <- match_fallback_opts(...)
  .Call(
    ffi_ptype2_dispatch_native,
    x,
    y,
    fallback_opts,
    frame = environment()
  )
}

#' Default cast and ptype2 methods
#'
#' @description
#'
#' These functions are automatically called when no [vec_ptype2()] or
#' [vec_cast()] method is implemented for a pair of types.
#'
#' * They apply special handling if one of the inputs is of type
#'   `AsIs` or `sfc`.
#'
#' * They attempt a number of fallbacks in cases where it would be too
#'   inconvenient to be strict:
#'
#'   - If the class and attributes are the same they are considered
#'     compatible. `vec_default_cast()` returns `x` in this case.
#'
#'   - In case of incompatible data frame classes, they fall back to
#'     `data.frame`. If an incompatible subclass of tibble is
#'     involved, they fall back to `tbl_df`.
#'
#' * Otherwise, an error is thrown with [stop_incompatible_type()] or
#'   [stop_incompatible_cast()].
#'
#' @keywords internal
#' @export
vec_default_ptype2 <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  if (is_asis(x)) {
    return(vec_ptype2_asis_left(
      x,
      y,
      x_arg = x_arg,
      y_arg = y_arg,
      call = call
    ))
  }
  if (is_asis(y)) {
    return(vec_ptype2_asis_right(
      x,
      y,
      x_arg = x_arg,
      y_arg = y_arg,
      call = call
    ))
  }

  opts <- match_fallback_opts(...)

  if (opts$s3_fallback && can_fall_back_2(x, y)) {
    common <- common_class_suffix(x, y)
    if (length(common)) {
      return(new_common_class_fallback(x, common))
    }
  }

  if (is.data.frame(x) && is.data.frame(y)) {
    out <- vec_ptype2_df_fallback(
      x,
      y,
      opts,
      x_arg = x_arg,
      y_arg = y_arg,
      call = call
    )

    if (identical(non_df_attrib(x), non_df_attrib(y))) {
      attributes(out) <- c(df_attrib(out), non_df_attrib(x))
    }

    return(out)
  }

  if (is_same_type(x, y)) {
    return(vec_ptype(x, x_arg = x_arg))
  }

  if (opts$s3_fallback) {
    # Undo common class fallback class for error messages
    if (is_common_class_fallback(x)) {
      x <- fallback_class_remove(x)
    }
    if (is_common_class_fallback(y)) {
      y <- fallback_class_remove(y)
    }
  }

  # The from-dispatch parameter is set only when called from our S3
  # dispatch mechanism, when no method is found to dispatch to. It
  # indicates whether the error message should provide advice about
  # diverging attributes.
  withRestarts(
    stop_incompatible_type(
      x,
      y,
      x_arg = x_arg,
      y_arg = y_arg,
      `vctrs:::from_dispatch` = match_from_dispatch(...),
      call = call
    ),
    vctrs_restart_ptype2 = function(ptype) {
      ptype
    }
  )
}

# This wrapper for `stop_incompatible_type()` matches error context
# arguments. It is useful to pass ptype2 arguments through dots
# without risking unknown arguments getting stored as condition fields.
vec_incompatible_ptype2 <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  stop_incompatible_type(
    x,
    y,
    x_arg = x_arg,
    y_arg = y_arg,
    call = call
  )
}

# We can't check for a proxy or ptype2 method to determine whether a
# class is foreign, because we implement these generics for many base
# classes and we still need to allow base fallbacks with subclasses.
can_fall_back_2 <- function(x, y) {
  if (!identical(typeof(x), typeof(y))) {
    return(FALSE)
  }

  if (!can_fall_back(x) || !can_fall_back(y)) {
    return(FALSE)
  }

  TRUE
}

can_fall_back <- function(x) {
  UseMethod("can_fall_back")
}

#' @export
can_fall_back.vctrs_vctr <- function(x) {
  # Work around bad interaction when `c()` method calls back into `vec_c()`
  FALSE
}
#' @export
can_fall_back.ts <- function(x) {
  # Work around bug with hard-coded `tsp` attribute in Rf_setAttrib()
  FALSE
}
#' @export
can_fall_back.data.frame <- function(x) {
  # The `c()` fallback is only for 1D vectors
  FALSE
}

#' @export
`can_fall_back.vctrs:::common_class_fallback` <- function(x) {
  TRUE
}

#' @export
can_fall_back.default <- function(x) {
  # Don't fall back for classes that directly implement a proxy.
  #
  # NOTE: That's suboptimal. For instance this forces us to override
  # `can_fall_back()` for `vctrs_vctr` to avoid recursing into
  # `vec_c()` through `c()`. Maybe we want to avoid falling back for
  # any vector that inherits a `vec_proxy()` method implemented
  # _outside_ of vctrs, i.e. not for a base class?
  is_null(s3_get_method(class(x)[[1]], "vec_proxy", ns = "vctrs"))
}

new_common_class_fallback <- function(x, fallback_class) {
  structure(
    vec_ptype(x),
    class = "vctrs:::common_class_fallback",
    fallback_class = fallback_class
  )
}
#' @export
`vec_proxy.vctrs:::common_class_fallback` <- function(x, ...) {
  x
}

is_common_class_fallback <- function(x) {
  inherits(x, "vctrs:::common_class_fallback")
}
common_class_suffix <- function(x, y) {
  vec_common_suffix(fallback_class(x), fallback_class(y))
}
fallback_class <- function(x) {
  if (is_common_class_fallback(x)) {
    attr(x, "fallback_class")
  } else {
    class(x)
  }
}
fallback_class_remove <- function(x) {
  class(x) <- attr(x, "fallback_class", exact = TRUE)
  attr(x, "fallback_class") <- NULL
  x
}

check_ptype2_dots_empty <- function(
  ...,
  `vctrs:::from_dispatch`,
  `vctrs:::s3_fallback`
) {
  check_dots_empty0(...)
}
match_fallback_opts <- function(..., `vctrs:::s3_fallback` = NULL) {
  fallback_opts(
    s3_fallback = `vctrs:::s3_fallback`
  )
}
match_from_dispatch <- function(..., `vctrs:::from_dispatch` = FALSE) {
  `vctrs:::from_dispatch`
}

fallback_opts <- function(s3_fallback = NULL) {
  # Order is important for the C side
  list(
    s3_fallback = s3_fallback %||% S3_FALLBACK_false
  )
}

enabled_fallback_opts <- function() {
  fallback_opts(
    s3_fallback = S3_FALLBACK_true
  )
}

vec_ptype2_opts <- function(
  x,
  y,
  ...,
  opts,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  .Call(ffi_ptype2_opts, x, y, opts, environment())
}
vec_ptype2_params <- function(
  x,
  y,
  ...,
  s3_fallback = NULL,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  opts <- fallback_opts(
    s3_fallback = s3_fallback
  )
  vec_ptype2_opts(
    x,
    y,
    opts = opts,
    x_arg = x_arg,
    y_arg = y_arg,
    call = call
  )
}

vec_ptype2_no_fallback <- function(
  x,
  y,
  ...,
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  opts <- fallback_opts(
    s3_fallback = S3_FALLBACK_false
  )
  vec_ptype2_opts(
    x,
    y,
    ...,
    ,
    opts = opts,
    x_arg = x_arg,
    y_arg = y_arg,
    call = call
  )
}

S3_FALLBACK_false <- 0L
S3_FALLBACK_true <- 1L


vec_typeof2 <- function(x, y) {
  .Call(ffi_typeof2, x, y)
}

vec_typeof2_s3 <- function(x, y) {
  .Call(ffi_typeof2_s3, x, y)
}

# https://github.com/r-lib/vctrs/issues/571
vec_is_coercible <- function(
  x,
  y,
  ...,
  opts = fallback_opts(),
  x_arg = "",
  y_arg = "",
  call = caller_env()
) {
  check_dots_empty0(...)

  .Call(
    ffi_is_coercible,
    x,
    y,
    opts,
    environment()
  )
}

vec_is_subtype <- function(x, super, ..., x_arg = "", super_arg = "") {
  tryCatch(
    vctrs_error_incompatible_type = function(...) FALSE,
    {
      common <- vctrs::vec_ptype2(
        x,
        super,
        ...,
        x_arg = x_arg,
        y_arg = super_arg
      )
      vec_is(common, super)
    }
  )
}

vec_implements_ptype2 <- function(x) {
  .Call(vctrs_implements_ptype2, x)
}

test_that("base coercions are symmetric and unchanging", {
  types <- list(
    logical(),
    integer(),
    double(),
    character(),
    raw(),
    list()
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))
  expect_snapshot(mat)
})

test_that("new classes are uncoercible by default", {
  x <- structure(1:10, class = "vctrs_nonexistant")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
})

test_that("vec_typeof2() returns common type", {
  nms <- names(base_empty_types)

  for (i in seq_along(base_empty_types)) {
    this <- nms[[i]]

    for (j in seq_along(base_empty_types)) {
      that <- nms[[j]]

      if (i <= j) {
        exp <- paste0("VCTRS_TYPE2_", this, "_", that)
      } else {
        exp <- paste0("VCTRS_TYPE2_", that, "_", this)
      }
      out <- vec_typeof2(base_empty_types[[this]], base_empty_types[[that]])

      expect_identical(out, exp)
    }
  }
})

test_that("vec_typeof2_s3() returns common type", {
  all_base_empty_types <- c(base_empty_types, base_s3_empty_types)

  nms_s3 <- names(base_s3_empty_types)
  nms <- names(all_base_empty_types)

  for (i in seq_along(all_base_empty_types)) {
    this <- nms[[i]]

    for (j in seq_along(all_base_empty_types)) {
      that <- nms[[j]]

      # Skip when we have two non s3 objects
      if (!(this %in% nms_s3) & !(that %in% nms_s3)) {
        next
      }

      if (i <= j) {
        exp <- paste0("VCTRS_TYPE2_S3_", this, "_", that)
      } else {
        exp <- paste0("VCTRS_TYPE2_S3_", that, "_", this)
      }
      out <- vec_typeof2_s3(all_base_empty_types[[this]], all_base_empty_types[[that]])

      expect_identical(out, exp)
    }
  }
})

test_that("vec_ptype2() dispatches when inputs have shape", {
  expect_identical(dim(vec_ptype2(int(), matrix(nrow = 3, ncol = 4))), c(0L, 4L))
  expect_identical(dim(vec_ptype2(matrix("", nrow = 3), c("", "", ""))), c(0L, 1L))
})

test_that("vec_ptype2() requires vectors", {
  expect_error(vec_ptype2(NULL, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(NA, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(list(), quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), NA), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), list()), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), quote(name)), class = "vctrs_error_scalar_type")
})

test_that("vec_ptype2() with unspecified requires vectors", {
  expect_error(vec_ptype2(unspecified(), quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), unspecified()), class = "vctrs_error_scalar_type")
})

test_that("vec_ptype2() forwards argument tag", {
  expect_error(vec_ptype2(quote(name), list(), x_arg = "foo"), "`foo`", class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(list(), quote(name), y_arg = "foo"), "`foo`", class = "vctrs_error_scalar_type")
})

test_that("stop_incompatible_type() checks for scalars", {
  expect_error(stop_incompatible_type(NA, foobar(), x_arg = "x", y_arg = "y"), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype_common(NA, foobar()), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype_common(foobar(), list()), class = "vctrs_error_scalar_type")
})

test_that("vec_ptype2() methods forward args to stop_incompatible_type()", {
  expect_args(new_hidden(), lgl(), x_arg = "foo", y_arg = "bar")
  expect_args(lgl(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(int(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(dbl(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(chr(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(list(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(new_rcrd(list(x = NA)), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(data.frame(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(Sys.Date(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(as.difftime(1, units = "hours"), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(factor(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(ordered(""), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(ordered(""), factor(), x_arg = "foo", y_arg = "bar")
  expect_args(bit64::as.integer64(1), new_hidden(), x_arg = "foo", y_arg = "bar")
})

test_that("vec_ptype2() data frame methods builds argument tags", {
  # Bare objects
  expect_snapshot(error = TRUE, {
    vec_ptype2("foo", 10)
  })

  # Nested dataframes
  expect_snapshot(error = TRUE, {
    df1 <- tibble(x = tibble(y = tibble(z = 1)))
    df2 <- tibble(x = tibble(y = tibble(z = "a")))
    vec_ptype2(df1, df2)
  })
})

test_that("stop_incompatible_type() can be called without argument tags", {
  expect_error(stop_incompatible_type(1, 2, x_arg = "", y_arg = ""), "<double> and <double>", class = "vctrs_error_incompatible_type")
})

test_that("vec_ptype2() returns empty prototype when other input is NULL", {
  expect_identical(vec_ptype2(1:5, NULL), int())
  expect_identical(vec_ptype2(NULL, 1:5), int())
})

test_that("Subclasses of data.frame dispatch to `vec_ptype2()` methods", {
  local_methods(
    vec_ptype2.quuxframe = function(x, y, ...) UseMethod("vec_ptype2.quuxframe"),
    vec_ptype2.quuxframe.data.frame = function(x, y, ...) "dispatched!",
    vec_ptype2.data.frame.quuxframe = function(x, y, ...) "dispatched!"
  )

  quux <- structure(data.frame(), class = c("quuxframe", "data.frame"))

  expect_identical(vec_ptype2(quux, mtcars), "dispatched!")
  expect_identical(vec_ptype2(mtcars, quux), "dispatched!")

  quux <- structure(data.frame(), class = c("quuxframe", "tbl_df", "data.frame"))

  expect_identical(vec_ptype2(quux, mtcars), "dispatched!")
  expect_identical(vec_ptype2(mtcars, quux), "dispatched!")
})

test_that("Subclasses of `tbl_df` have `tbl_df` common type (#481)", {
  quux <- foobar(tibble())

  expect_identical(
    vec_ptype_common(quux, tibble()),
    tibble()
  )
  expect_identical(
    vec_ptype_common(tibble(), quux),
    tibble()
  )
})

test_that("Column name encodings are handled correctly in the common type (#553)", {
  encs <- encodings()

  data <- list(chr())

  df_utf8 <- tibble::as_tibble(set_names(data, encs$utf8))
  df_unknown <- tibble::as_tibble(set_names(data, encs$unknown))

  expect_identical(vec_ptype2(df_utf8, df_unknown), df_utf8)
})

test_that("vec_is_subtype() determines subtyping relationship", {
  expect_true(vec_is_subtype(lgl(), int()))
  expect_false(vec_is_subtype(int(), lgl()))

  expect_false(vec_is_subtype(lgl(), chr()))
  expect_false(vec_is_subtype(chr(), lgl()))

  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) UseMethod("vec_ptype2.vctrs_foobar"),
    vec_ptype2.vctrs_foobar.logical = function(x, y, ...) logical(),
    vec_ptype2.logical.vctrs_foobar = function(x, y, ...) logical()
  )
  expect_true(vec_is_subtype(foobar(TRUE), lgl()))
  expect_false(vec_is_subtype(lgl(), foobar(TRUE)))
})

test_that("can override scalar vector error message for base scalar types", {
  expect_snapshot({
    (expect_error(vec_ptype2(NULL, quote(x), y_arg = "foo"), class = "vctrs_error_scalar_type"))
    (expect_error(vec_ptype2(quote(x), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type"))
  })
})

test_that("can override scalar vector error message for S3 types", {
  expect_snapshot({
    (expect_error(vec_ptype2(NULL, foobar(), y_arg = "foo"), class = "vctrs_error_scalar_type"))
    (expect_error(vec_ptype2(foobar(), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type"))
  })
})

test_that("ptype2 and cast errors when same class fallback is impossible are informative", {
  expect_snapshot({
    (expect_error(
      vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE)),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_ptype2(foobar(1, bar = TRUE), foobar(2, baz = TRUE)),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("Incompatible attributes bullets are not show when methods are implemented", {
  expect_snapshot({
    with_foobar_cast <- function(expr ) {
      with_methods(
        vec_cast.vctrs_foobar = function(...) NULL,
        vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) vec_default_cast(x, to, ...),
        expr
      )
    }
    with_foobar_ptype2 <- function(expr ) {
      with_methods(
        vec_ptype2.vctrs_foobar = function(...) NULL,
        vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) vec_default_ptype2(x, y, ...),
        expr
      )
    }

    (expect_error(
      with_foobar_cast(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE))),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      with_foobar_ptype2(vec_ptype2(foobar(1, bar = TRUE), foobar(2, baz = TRUE))),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("attributes no longer play a role in bare data frame fallback", {
  foobar_bud <- foobar(mtcars, bud = TRUE)
  foobar_boo <- foobar(mtcars, boo = TRUE)

  expect_equal(
    vec_ptype2(foobar_bud, foobar_boo),
    vec_slice(unrownames(mtcars), 0)
  )
  expect_equal(
    vec_ptype2(foobar(mtcars), foobaz(mtcars)),
    vec_slice(unrownames(mtcars), 0)
  )
})

test_that("vec_ptype2() methods get prototypes", {
  x <- NULL
  y <- NULL

  local_methods(vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
    x <<- x
    y <<- y
    NULL
  })

  vec_ptype2(foobar(1:3), foobar(letters))
  expect_identical(x, foobar(int()))
  expect_identical(y, foobar(chr()))

  vec_ptype2(foobar(mtcars), foobar(iris))
  expect_identical(x, foobar(mtcars[0, , drop = FALSE]))
  expect_identical(y, foobar(iris[0, , drop = FALSE]))
})

test_that("vec_ptype2() allows vec_ptype() to return another type", {
  out <- with_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) unstructure(x),
    vec_ptype2(foobar(1), foobar(2))
  )
  expect_identical(out, dbl())
})

test_that("vec_ptype2() evaluates x_arg and y_arg lazily", {
  expect_silent(vec_ptype2(1L, 1L, x_arg = print("oof")))
  expect_silent(vec_ptype2(1L, 1L, y_arg = print("oof")))
})

test_that("can restart ptype2 errors", {
  x <- data_frame(x = ordered(c("a", "b", "c")))
  y <- data_frame(x = ordered(c("A", "B", "C")))

  exp <- c("a", "b", "c", "A", "B", "C")
  exp <- factor(exp, exp)

  expect_error(vec_rbind(x, y), class = "vctrs_error_incompatible_type")

  expect_equal(
    with_ordered_restart(vec_rbind(x, y)),
    data_frame(x = exp)
  )

  z <- data_frame(x = chr())

  expect_equal(
    with_ordered_restart(vec_ptype_common(x, y)),
    data_frame(x = exp[0])
  )
  expect_equal(
    with_ordered_restart(vec_ptype_common(x, y, z)),
    data_frame(x = chr())
  )

  expect_equal(
    with_ordered_restart(vec_cast_common(x, y)),
    list(
      data_frame(x = factor(c("a", "b", "c"), levels(exp))),
      data_frame(x = factor(c("A", "B", "C"), levels(exp)))
    )
  )
  expect_equal(
    with_ordered_restart(vec_cast_common(x, y, z)),
    list(
      data_frame(x = c("a", "b", "c")),
      data_frame(x = c("A", "B", "C")),
      data_frame(x = chr())
    )
  )

  # Factor case
  y <- data_frame(x = factor(c("A", "B", "C")))
  expect_equal(
    with_ordered_restart(vec_rbind(x, y)),
    data_frame(x = exp)
  )
})

test_that("subclasses of tibble are compatible", {
  tib <- foobar(tibble(x = 1))
  ptype <- foobar(tibble(x = dbl()))

  expect_equal(vec_ptype_common(tib), ptype)
  expect_equal(vec_ptype_common(tib, tib), ptype)
})

test_that("base coercions are symmetric and unchanging", {
  types <- list(
    logical(),
    integer(),
    double(),
    character(),
    raw(),
    list()
  )
  mat <- maxtype_mat(types)

  expect_true(isSymmetric(mat))
  expect_snapshot(mat)
})

test_that("new classes are uncoercible by default", {
  x <- structure(1:10, class = "vctrs_nonexistant")
  expect_error(vec_ptype2(1, x), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(x, 1), class = "vctrs_error_incompatible_type")
})

test_that("vec_typeof2() returns common type", {
  nms <- names(base_empty_types)

  for (i in seq_along(base_empty_types)) {
    this <- nms[[i]]

    for (j in seq_along(base_empty_types)) {
      that <- nms[[j]]

      if (i <= j) {
        exp <- paste0("VCTRS_TYPE2_", this, "_", that)
      } else {
        exp <- paste0("VCTRS_TYPE2_", that, "_", this)
      }
      out <- vec_typeof2(base_empty_types[[this]], base_empty_types[[that]])

      expect_identical(out, exp)
    }
  }
})

test_that("vec_typeof2_s3() returns common type", {
  all_base_empty_types <- c(base_empty_types, base_s3_empty_types)

  nms_s3 <- names(base_s3_empty_types)
  nms <- names(all_base_empty_types)

  for (i in seq_along(all_base_empty_types)) {
    this <- nms[[i]]

    for (j in seq_along(all_base_empty_types)) {
      that <- nms[[j]]

      # Skip when we have two non s3 objects
      if (!(this %in% nms_s3) & !(that %in% nms_s3)) {
        next
      }

      if (i <= j) {
        exp <- paste0("VCTRS_TYPE2_S3_", this, "_", that)
      } else {
        exp <- paste0("VCTRS_TYPE2_S3_", that, "_", this)
      }
      out <- vec_typeof2_s3(
        all_base_empty_types[[this]],
        all_base_empty_types[[that]]
      )

      expect_identical(out, exp)
    }
  }
})

test_that("vec_ptype2() dispatches when inputs have shape", {
  expect_identical(
    dim(vec_ptype2(int(), matrix(nrow = 3, ncol = 4))),
    c(0L, 4L)
  )
  expect_identical(
    dim(vec_ptype2(matrix("", nrow = 3), c("", "", ""))),
    c(0L, 1L)
  )
})

test_that("vec_ptype2() requires vectors", {
  expect_error(vec_ptype2(NULL, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(NA, quote(name)), class = "vctrs_error_scalar_type")
  expect_error(
    vec_ptype2(list(), quote(name)),
    class = "vctrs_error_scalar_type"
  )
  expect_error(vec_ptype2(quote(name), NULL), class = "vctrs_error_scalar_type")
  expect_error(vec_ptype2(quote(name), NA), class = "vctrs_error_scalar_type")
  expect_error(
    vec_ptype2(quote(name), list()),
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype2(quote(name), quote(name)),
    class = "vctrs_error_scalar_type"
  )
})

test_that("vec_ptype2() with unspecified requires vectors", {
  expect_error(
    vec_ptype2(unspecified(), quote(name)),
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype2(quote(name), unspecified()),
    class = "vctrs_error_scalar_type"
  )
})

test_that("vec_ptype2() forwards argument tag", {
  expect_error(
    vec_ptype2(quote(name), list(), x_arg = "foo"),
    "`foo`",
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype2(list(), quote(name), y_arg = "foo"),
    "`foo`",
    class = "vctrs_error_scalar_type"
  )
})

test_that("stop_incompatible_type() checks for scalars", {
  expect_error(
    stop_incompatible_type(NA, foobar(), x_arg = "x", y_arg = "y"),
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype_common(NA, foobar()),
    class = "vctrs_error_scalar_type"
  )
  expect_error(
    vec_ptype_common(foobar(), list()),
    class = "vctrs_error_scalar_type"
  )
})

test_that("vec_ptype2() methods forward args to stop_incompatible_type()", {
  expect_args(new_hidden(), lgl(), x_arg = "foo", y_arg = "bar")
  expect_args(lgl(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(int(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(dbl(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(chr(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(list(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(
    new_rcrd(list(x = NA)),
    new_hidden(),
    x_arg = "foo",
    y_arg = "bar"
  )
  expect_args(data.frame(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(Sys.Date(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(
    as.difftime(1, units = "hours"),
    new_hidden(),
    x_arg = "foo",
    y_arg = "bar"
  )
  expect_args(factor(), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(ordered(""), new_hidden(), x_arg = "foo", y_arg = "bar")
  expect_args(ordered(""), factor(), x_arg = "foo", y_arg = "bar")
  expect_args(
    bit64::as.integer64(1),
    new_hidden(),
    x_arg = "foo",
    y_arg = "bar"
  )
})

test_that("vec_ptype2() data frame methods builds argument tags", {
  # Bare objects
  expect_snapshot(error = TRUE, {
    vec_ptype2("foo", 10)
  })

  # Nested dataframes
  expect_snapshot(error = TRUE, {
    df1 <- tibble(x = tibble(y = tibble(z = 1)))
    df2 <- tibble(x = tibble(y = tibble(z = "a")))
    vec_ptype2(df1, df2)
  })
})

test_that("stop_incompatible_type() can be called without argument tags", {
  expect_error(
    stop_incompatible_type(1, 2, x_arg = "", y_arg = ""),
    "<double> and <double>",
    class = "vctrs_error_incompatible_type"
  )
})

test_that("vec_ptype2() returns empty prototype when other input is NULL", {
  expect_identical(vec_ptype2(1:5, NULL), int())
  expect_identical(vec_ptype2(NULL, 1:5), int())
})

test_that("Subclasses of data.frame dispatch to `vec_ptype2()` methods", {
  local_methods(
    vec_ptype2.quuxframe = function(x, y, ...) {
      UseMethod("vec_ptype2.quuxframe")
    },
    vec_ptype2.quuxframe.data.frame = function(x, y, ...) "dispatched!",
    vec_ptype2.data.frame.quuxframe = function(x, y, ...) "dispatched!"
  )

  quux <- structure(data.frame(), class = c("quuxframe", "data.frame"))

  expect_identical(vec_ptype2(quux, mtcars), "dispatched!")
  expect_identical(vec_ptype2(mtcars, quux), "dispatched!")

  quux <- structure(
    data.frame(),
    class = c("quuxframe", "tbl_df", "data.frame")
  )

  expect_identical(vec_ptype2(quux, mtcars), "dispatched!")
  expect_identical(vec_ptype2(mtcars, quux), "dispatched!")
})

test_that("Subclasses of `tbl_df` have `tbl_df` common type (#481)", {
  quux <- foobar(tibble())

  expect_identical(
    vec_ptype_common(quux, tibble()),
    tibble()
  )
  expect_identical(
    vec_ptype_common(tibble(), quux),
    tibble()
  )
})

test_that("Column name encodings are handled correctly in the common type (#553)", {
  encs <- encodings()

  data <- list(chr())

  df_utf8 <- tibble::as_tibble(set_names(data, encs$utf8))
  df_unknown <- tibble::as_tibble(set_names(data, encs$unknown))

  expect_identical(vec_ptype2(df_utf8, df_unknown), df_utf8)
})

test_that("vec_is_subtype() determines subtyping relationship", {
  expect_true(vec_is_subtype(lgl(), int()))
  expect_false(vec_is_subtype(int(), lgl()))

  expect_false(vec_is_subtype(lgl(), chr()))
  expect_false(vec_is_subtype(chr(), lgl()))

  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_foobar")
    },
    vec_ptype2.vctrs_foobar.logical = function(x, y, ...) logical(),
    vec_ptype2.logical.vctrs_foobar = function(x, y, ...) logical()
  )
  expect_true(vec_is_subtype(foobar(TRUE), lgl()))
  expect_false(vec_is_subtype(lgl(), foobar(TRUE)))
})

test_that("can override scalar vector error message for base scalar types", {
  expect_snapshot({
    (expect_error(
      vec_ptype2(NULL, quote(x), y_arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
    (expect_error(
      vec_ptype2(quote(x), NULL, x_arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
  })
})

test_that("can override scalar vector error message for S3 types", {
  expect_snapshot({
    (expect_error(
      vec_ptype2(NULL, foobar(), y_arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
    (expect_error(
      vec_ptype2(foobar(), NULL, x_arg = "foo"),
      class = "vctrs_error_scalar_type"
    ))
  })
})

test_that("ptype2 and cast errors when same class fallback is impossible are informative", {
  expect_snapshot({
    (expect_error(
      vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE)),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_ptype2(foobar(1, bar = TRUE), foobar(2, baz = TRUE)),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("Incompatible attributes bullets are not show when methods are implemented", {
  expect_snapshot({
    with_foobar_cast <- function(expr) {
      with_methods(
        vec_cast.vctrs_foobar = function(...) NULL,
        vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
          vec_default_cast(x, to, ...)
        },
        expr
      )
    }
    with_foobar_ptype2 <- function(expr) {
      with_methods(
        vec_ptype2.vctrs_foobar = function(...) NULL,
        vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
          vec_default_ptype2(x, y, ...)
        },
        expr
      )
    }

    (expect_error(
      with_foobar_cast(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE))),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      with_foobar_ptype2(vec_ptype2(
        foobar(1, bar = TRUE),
        foobar(2, baz = TRUE)
      )),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("attributes no longer play a role in bare data frame fallback", {
  foobar_bud <- foobar(mtcars, bud = TRUE)
  foobar_boo <- foobar(mtcars, boo = TRUE)

  expect_equal(
    vec_ptype2(foobar_bud, foobar_boo),
    vec_slice(unrownames(mtcars), 0)
  )
  expect_equal(
    vec_ptype2(foobar(mtcars), foobaz(mtcars)),
    vec_slice(unrownames(mtcars), 0)
  )
})

test_that("vec_ptype2() methods get prototypes", {
  x <- NULL
  y <- NULL

  local_methods(vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
    x <<- x
    y <<- y
    NULL
  })

  vec_ptype2(foobar(1:3), foobar(letters))
  expect_identical(x, foobar(int()))
  expect_identical(y, foobar(chr()))

  vec_ptype2(foobar(mtcars), foobar(iris))
  expect_identical(x, foobar(mtcars[0, , drop = FALSE]))
  expect_identical(y, foobar(iris[0, , drop = FALSE]))
})

test_that("vec_ptype2() allows vec_ptype() to return another type", {
  out <- with_methods(
    vec_restore.vctrs_foobar = function(x, to, ...) unstructure(x),
    vec_ptype2(foobar(1), foobar(2))
  )
  expect_identical(out, dbl())
})

test_that("vec_ptype2() evaluates x_arg and y_arg lazily", {
  expect_silent(vec_ptype2(1L, 1L, x_arg = print("oof")))
  expect_silent(vec_ptype2(1L, 1L, y_arg = print("oof")))
})

test_that("can restart ptype2 errors", {
  x <- data_frame(x = ordered(c("a", "b", "c")))
  y <- data_frame(x = ordered(c("A", "B", "C")))

  exp <- c("a", "b", "c", "A", "B", "C")
  exp <- factor(exp, exp)

  expect_error(vec_rbind(x, y), class = "vctrs_error_incompatible_type")

  expect_equal(
    with_ordered_restart(vec_rbind(x, y)),
    data_frame(x = exp)
  )

  z <- data_frame(x = chr())

  expect_equal(
    with_ordered_restart(vec_ptype_common(x, y)),
    data_frame(x = exp[0])
  )
  expect_equal(
    with_ordered_restart(vec_ptype_common(x, y, z)),
    data_frame(x = chr())
  )

  expect_equal(
    with_ordered_restart(vec_cast_common(x, y)),
    list(
      data_frame(x = factor(c("a", "b", "c"), levels(exp))),
      data_frame(x = factor(c("A", "B", "C"), levels(exp)))
    )
  )
  expect_equal(
    with_ordered_restart(vec_cast_common(x, y, z)),
    list(
      data_frame(x = c("a", "b", "c")),
      data_frame(x = c("A", "B", "C")),
      data_frame(x = chr())
    )
  )

  # Factor case
  y <- data_frame(x = factor(c("A", "B", "C")))
  expect_equal(
    with_ordered_restart(vec_rbind(x, y)),
    data_frame(x = exp)
  )
})

test_that("subclasses of tibble are compatible", {
  tib <- foobar(tibble(x = 1))
  ptype <- foobar(tibble(x = dbl()))

  expect_equal(vec_ptype_common(tib), ptype)
  expect_equal(vec_ptype_common(tib, tib), ptype)
})

test_that("error indexing is correct with unspecifieds", {
  expect_snapshot(error = TRUE, {
    vec_ptype_common(1, NA, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_ptype_common(NA, 1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_ptype_common(NA, NA, 1, "x")
  })
  expect_snapshot(error = TRUE, {
    vec_ptype_common(1, NA, NA, 1, "x")
  })
})

#' Actual vector dimensions
#'
#' @description
#' * `vec_dim_n()` gives the dimensionality (i.e. number of dimensions)
#' * `vec_dim()` returns the size of each dimension
#'
#' These functions access the raw `"dim"` attribute of the object
#' and do not dispatch over the [dim()] generic.
#'
#' @details
#' Unlike base R, we treat vectors with `NULL` dimensions as 1d. This
#' simplifies the type system by eliding a special case. Compared to the base R
#' equivalent, `vec_dim()` returns [length()], not `NULL`, when `x` is 1d.
#'
#' @seealso
#' `dim2()`, a variant of [dim()] that returns [length()] if an object
#' doesn't have dimensions.
#'
#' @param x A vector
#' @noRd
#' @examples
#' # Compared to base R
#' x <- 1:5
#' dim(x)
#' vec_dim(x)
NULL

# FIXME: Should `vec_dim()` return the size instead of the length?
vec_dim <- function(x) {
  .Call(vctrs_dim, x)
}
vec_dim_n <- function(x) {
  .Call(vctrs_dim_n, x)
}
has_dim <- function(x) {
  .Call(vctrs_has_dim, x)
}

#' Perceived vector dimensions
#'
#' @description
#' `dim2()` is a variant of [dim()] that returns [vec_size()] if an object
#' doesn't have dimensions.
#'
#' @details
#' Unlike base R, we treat vectors with `NULL` dimensions as 1d. This
#' simplifies the type system by eliding a special case. Compared to the base R
#' equivalent, `dim2()` returns [length()], not `NULL`, when `x` is 1d.
#'
#' @seealso
#' `vec_dim()`, a variant that never dispatches over the [dim()] generic.
#'
#' @param x A vector
#' @noRd
#' @examples
#' # Compared to base R
#' x <- 1:5
#' dim(x)
#' vec_dim(x)
dim2 <- function(x) {
  dim(x) %||% length(x)
}

# vec_dim -----------------------------------------------------------------

test_that("dim is dimensions", {
  expect_equal(vec_dim(array(dim = c(1))), c(1))
  expect_equal(vec_dim(array(dim = c(1, 1))), c(1, 1))
  expect_equal(vec_dim(array(dim = c(1, 1, 1))), c(1, 1, 1))
})


test_that("dim_n is number of dimensions", {
  expect_equal(vec_dim_n(array(dim = c(1))), 1)
  expect_equal(vec_dim_n(array(dim = c(1, 1))), 2)
  expect_equal(vec_dim_n(array(dim = c(1, 1, 1))), 3)
})

test_that("vector and 1-d array are equivalent", {
  x1 <- 1:5
  x2 <- array(x1)

  expect_equal(vec_dim(x1), 5)
  expect_equal(vec_dim(x2), 5)

  expect_equal(vec_size(x1), 5)
  expect_equal(vec_size(x2), 5)
})

# vec_dim -----------------------------------------------------------------

test_that("dim is dimensions", {
  expect_equal(vec_dim(array(dim = c(1))), c(1))
  expect_equal(vec_dim(array(dim = c(1, 1))), c(1, 1))
  expect_equal(vec_dim(array(dim = c(1, 1, 1))), c(1, 1, 1))
})


test_that("dim_n is number of dimensions", {
  expect_equal(vec_dim_n(array(dim = c(1))), 1)
  expect_equal(vec_dim_n(array(dim = c(1, 1))), 2)
  expect_equal(vec_dim_n(array(dim = c(1, 1, 1))), 3)
})

test_that("vector and 1-d array are equivalent", {
  x1 <- 1:5
  x2 <- array(x1)

  expect_equal(vec_dim(x1), 5)
  expect_equal(vec_dim(x2), 5)

  expect_equal(vec_size(x1), 5)
  expect_equal(vec_size(x2), 5)
})

with_subscript_data <- function(
  expr,
  subscript_arg,
  subscript_elt = NULL,
  subscript_action = NULL
) {
  local_options(rlang_force_unhandled_error = TRUE)
  tryCatch(
    expr,
    vctrs_error_subscript = function(cnd) {
      cnd$subscript_arg <- subscript_arg
      cnd$subscript_elt <- subscript_elt
      cnd$subscript_action <- subscript_action
      cnd_signal(cnd)
    }
  )
}

with_tibble_cols <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "rename"
  )
}
with_tibble_rows <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "row",
    subscript_action = "remove"
  )
}
with_dm_tables <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "table",
    subscript_action = "extract"
  )
}
with_tidyselect_select <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "select"
  )
}
with_tidyselect_relocate <- function(expr) {
  with_subscript_data(
    expr,
    subscript_arg = quote(foo(bar)),
    subscript_elt = "column",
    subscript_action = "relocate"
  )
}

my_vec_rep <- function(my_x, my_times) {
  vec_rep(
    my_x,
    my_times,
    error_call = current_env(),
    x_arg = "my_x",
    times_arg = "my_times"
  )
}

my_vec_rep_each <- function(my_x, my_times) {
  vec_rep_each(
    my_x,
    my_times,
    error_call = current_env(),
    x_arg = "my_x",
    times_arg = "my_times"
  )
}

my_vec_as_names <- function(
  my_names,
  ...,
  my_repair = "minimal",
  my_quiet = FALSE
) {
  vec_as_names(
    my_names,
    repair = my_repair,
    repair_arg = "my_repair",
    quiet = my_quiet
  )
}

#' @rdname df_ptype2
#' @export
tib_ptype2 <- function(x, y, ..., x_arg = "", y_arg = "", call = caller_env()) {
  .Call(
    ffi_tib_ptype2,
    x = x,
    y = y,
    x_arg = x_arg,
    y_arg = y_arg,
    frame = environment()
  )
}
#' @rdname df_ptype2
#' @export
tib_cast <- function(x, to, ..., x_arg = "", to_arg = "", call = caller_env()) {
  .Call(
    ffi_tib_cast,
    x = x,
    to = to,
    x_arg = x_arg,
    to_arg = to_arg,
    frame = environment()
  )
}

df_as_tibble <- function(df) {
  class(df) <- c("tbl_df", "tbl", "data.frame")
  df
}

# Conditionally registered in .onLoad()

vec_ptype2_tbl_df_tbl_df <- function(x, y, ...) {
  vec_ptype2_dispatch_native(x, y, ...)
}
vec_ptype2_tbl_df_data.frame <- function(x, y, ...) {
  vec_ptype2_dispatch_native(x, y, ...)
}
vec_ptype2_data.frame_tbl_df <- function(x, y, ...) {
  vec_ptype2_dispatch_native(x, y, ...)
}

vec_cast_tbl_df_tbl_df <- function(x, to, ...) {
  vec_cast_dispatch_native(x, to, ...)
}
vec_cast_data.frame_tbl_df <- function(x, to, ...) {
  vec_cast_dispatch_native(x, to, ...)
}
vec_cast_tbl_df_data.frame <- function(x, to, ...) {
  vec_cast_dispatch_native(x, to, ...)
}

#' AsIs S3 class
#'
#' These functions help the base AsIs class fit into the vctrs type system
#' by providing coercion and casting functions.
#'
#' @keywords internal
#' @name as-is
NULL

# ------------------------------------------------------------------------------
# Printing

#' @export
vec_ptype_full.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  paste0("I<", vec_ptype_full(x), ">")
}

#' @export
vec_ptype_abbr.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  paste0("I<", vec_ptype_abbr(x), ">")
}

# ------------------------------------------------------------------------------
# Proxy / restore

# Arises with base df ctor: `data.frame(x = I(list(1, 2:3)))`

#' @export
vec_proxy.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  vec_proxy(x)
}

#' @export
vec_restore.AsIs <- function(x, to, ...) {
  asis_restore(x)
}

#' @export
vec_proxy_equal.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  vec_proxy_equal(x)
}

#' @export
vec_proxy_compare.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  vec_proxy_compare(x)
}

#' @export
vec_proxy_order.AsIs <- function(x, ...) {
  x <- asis_strip(x)
  vec_proxy_order(x)
}

# ------------------------------------------------------------------------------
# Coercion

#' @rdname as-is
#' @export vec_ptype2.AsIs
#' @method vec_ptype2 AsIs
#' @export
vec_ptype2.AsIs <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.AsIs")
}
#' @method vec_ptype2.AsIs AsIs
#' @export
vec_ptype2.AsIs.AsIs <- function(x, y, ..., x_arg = "", y_arg = "") {
  x <- asis_strip(x)
  y <- asis_strip(y)
  vec_ptype2_asis(x, y, ..., x_arg = x_arg, y_arg = y_arg)
}

vec_ptype2_asis_left <- function(x, y, ...) {
  x <- asis_strip(x)
  vec_ptype2_asis(x, y, ...)
}
vec_ptype2_asis_right <- function(x, y, ...) {
  y <- asis_strip(y)
  vec_ptype2_asis(x, y, ...)
}
vec_ptype2_asis <- function(x, y, ...) {
  out <- vec_ptype2(x, y, ...)
  asis_restore(out)
}

# ------------------------------------------------------------------------------
# Casting

vec_cast_from_asis <- function(x, to, ..., call = caller_env()) {
  x <- asis_strip(x)
  vec_cast(x, to, ..., call = call)
}

vec_cast_to_asis <- function(x, to, ..., call = caller_env()) {
  to <- asis_strip(to)
  out <- vec_cast(x, to, ..., call = call)
  asis_restore(out)
}

# ------------------------------------------------------------------------------

is_asis <- function(x) {
  inherits(x, "AsIs")
}

asis_strip <- function(x) {
  class(x) <- setdiff(class(x), "AsIs")
  x
}

asis_restore <- function(x) {
  # Using `oldClass()` here to return `NULL` for atomics
  # so that their implicit class isn't added
  class(x) <- c("AsIs", oldClass(x))
  x
}

#' Number of observations
#'
#' @description
#'
#' `vec_size(x)` returns the size of a vector. `vec_is_empty()`
#' returns `TRUE` if the size is zero, `FALSE` otherwise.
#'
#' The size is distinct from the [length()] of a vector because it
#' generalises to the "number of observations" for 2d structures,
#' i.e. it's the number of rows in matrix or a data frame.  This
#' definition has the important property that every column of a data
#' frame (even data frame and matrix columns) have the same size.
#' `vec_size_common(...)` returns the common size of multiple vectors.
#'
#' `list_sizes()` returns an integer vector containing the size of each element
#' of a list. It is nearly equivalent to, but faster than,
#' `map_int(x, vec_size)`, with the exception that `list_sizes()` will
#' error on non-list inputs, as defined by [obj_is_list()]. `list_sizes()` is
#' to `vec_size()` as [lengths()] is to [length()].
#'
#' @seealso [vec_slice()] for a variation of `[` compatible with `vec_size()`,
#'   and [vec_recycle()] to [recycle][theory-faq-recycling] vectors to common
#'   length.
#' @section Invariants:
#' * `vec_size(dataframe)` == `vec_size(dataframe[[i]])`
#' * `vec_size(matrix)` == `vec_size(matrix[, i, drop = FALSE])`
#' * `vec_size(vec_c(x, y))` == `vec_size(x)` + `vec_size(y)`
#'
#' @inheritParams rlang::args_error_context
#'
#' @param x,... Vector inputs or `NULL`.
#' @param .size If `NULL`, the default, the output size is determined by
#'   recycling the lengths of all elements of `...`. Alternatively, you can
#'   supply `.size` to force a known size; in this case, `x` and `...` are
#'   ignored.
#' @param .absent The size used when no input is provided, or when all input
#' is `NULL`. If left as `NULL` when no input is supplied, an error is thrown.
#' @return An integer (or double for long vectors).
#'
#'   `vec_size_common()` returns `.absent` if all inputs are `NULL` or
#'   absent, `0L` by default.
#'
#'
#' @details
#'
#' There is no vctrs helper that retrieves the number of columns: as this
#' is a property of the [type][vec_ptype_show()].
#'
#' `vec_size()` is equivalent to `NROW()` but has a name that is easier to
#' pronounce, and throws an error when passed non-vector inputs.
#'
#'
#' @section The size of NULL:
#'
#' The size of `NULL` is hard-coded to `0L` in `vec_size()`.
#' `vec_size_common()` returns `.absent` when all inputs are `NULL`
#' (if only some inputs are `NULL`, they are simply ignored).
#'
#' A default size of 0 makes sense because sizes are most often
#' queried in order to compute a total size while assembling a
#' collection of vectors. Since we treat `NULL` as an absent input by
#' principle, we return the identity of sizes under addition to
#' reflect that an absent input doesn't take up any size.
#'
#' Note that other defaults might make sense under different
#' circumstances. For instance, a default size of 1 makes sense for
#' finding the common size because 1 is the identity of the recycling
#' rules.
#'
#' @section Dependencies:
#' - [vec_proxy()]
#'
#' @export
#' @examples
#' vec_size(1:100)
#' vec_size(mtcars)
#' vec_size(array(dim = c(3, 5, 10)))
#'
#' vec_size_common(1:10, 1:10)
#' vec_size_common(1:10, 1)
#' vec_size_common(integer(), 1)
#'
#' list_sizes(list("a", 1:5, letters))
vec_size <- function(x) {
  .Call(ffi_size, x, environment())
}

#' @export
#' @rdname vec_size
vec_size_common <- function(
  ...,
  .size = NULL,
  .absent = 0L,
  .arg = "",
  .call = caller_env()
) {
  .External2(ffi_size_common, list2(...), .size, .absent)
}

#' @rdname vec_size
#' @export
list_sizes <- function(x) {
  .Call(ffi_list_sizes, x, environment())
}

#' @rdname vec_size
#' @export
vec_is_empty <- function(x) {
  vec_size(x) == 0L
}

#' Default value for empty vectors
#'
#' Use this inline operator when you need to provide a default value for
#' empty (as defined by [vec_is_empty()]) vectors.
#'
#' @param x A vector
#' @param y Value to use if `x` is empty. To preserve type-stability, should
#'   be the same type as `x`.
#' @rdname op-empty-default
#' @export
#' @examples
#' 1:10 %0% 5
#' integer() %0% 5
`%0%` <- function(x, y) {
  if (vec_is_empty(x)) y else x
}


# sequences -------------------------------------------------------------------

#' Useful sequences
#'
#' `vec_seq_along()` is equivalent to [seq_along()] but uses size, not length.
#' `vec_init_along()` creates a vector of missing values with size matching
#' an existing object.
#'
#' @param x,y Vectors
#' @return
#' * `vec_seq_along()` an integer vector with the same size as `x`.
#' * `vec_init_along()` a vector with the same type as `x` and the same size
#'   as `y`.
#' @export
#' @examples
#' vec_seq_along(mtcars)
#' vec_init_along(head(mtcars))
vec_seq_along <- function(x) {
  seq_len(vec_size(x))
}

#' @export
#' @rdname vec_seq_along
vec_init_along <- function(x, y = x) {
  vec_slice(x, rep_len(NA_integer_, vec_size(y)))
}

vec_as_short_length <- function(n, arg = caller_arg(n), call = caller_env()) {
  .Call(ffi_as_short_length, n, environment())
}

test_that("vec_as_short_length() checks inputs", {
  expect_equal(vec_as_short_length(0), 0)
  expect_equal(vec_as_short_length(1L), 1)

  my_function <- function(my_arg) vec_as_short_length(my_arg)
  expect_snapshot({
    (expect_error(my_function(-1)))
    (expect_error(my_function(1:2)))
    (expect_error(my_function(1.5)))
    (expect_error(my_function(NA)))
    (expect_error(my_function(na_int)))
    (expect_error(my_function("foo")))
    (expect_error(my_function(foobar(1:2))))
    (expect_error(my_function(.Machine$double.xmax)))
  })
})

test_that("vec_as_short_length() has a special error about long vector support", {
  # In particular, skips on 32-bit Windows where `r_ssize == int`
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  my_function <- function(my_arg) vec_as_short_length(my_arg)
  expect_snapshot({
    (expect_error(my_function(.Machine$integer.max + 1)))
  })
})

# vec_size -----------------------------------------------------------------

test_that("vec_size must be called with vector", {
  expect_error(vec_size(mean), class = "vctrs_error_scalar_type")
})

test_that("length is number of rows", {
  expect_equal(vec_size(integer()), 0)
  expect_equal(vec_size(array(integer())), 0)

  expect_equal(vec_size(1:2), 2)
  expect_equal(vec_size(array(dim = 2)), 2)

  expect_equal(vec_size(matrix(nrow = 2, ncol = 3)), 2)
  expect_equal(vec_size(array(dim = c(2, 1, 5))), 2)
})

test_that("length of record is number of rows, not fields", {
  r <- new_rcrd(list(x = 1:10))
  expect_equal(vec_size(r), 10)
})

test_that("handles three types of data frame rownames", {
  df1 <- df2 <- df3 <- data.frame(x = 1:3)
  rownames(df1) <- NULL
  rownames(df2) <- 3:1
  rownames(df3) <- letters[1:3]

  expect_equal(vec_size(df1), 3)
  expect_equal(vec_size(df2), 3)
  expect_equal(vec_size(df3), 3)
})

test_that("handles positive short row names (#220)", {
  data <- structure(mtcars, row.names = c(NA, 32))
  expect_identical(vec_size(data), 32L)
})

test_that("size is proxied", {
  local_env_proxy()
  expect_size(new_proxy(1:3), 3)
  expect_size(new_proxy(list(1, 2, 3)), 3)
  expect_size(new_proxy(foobar(list(1, 2, 3))), 3)
})

test_that("`NULL` has size zero", {
  expect_identical(vec_size(NULL), 0L)
})

test_that("can take the size of unspecified objects", {
  expect_size(NA, 1)
  expect_size(c(NA, NA), 2)
  expect_size(unspecified(2), 2)
})

# vec_size_common ---------------------------------------------------------

test_that("vec_size_common() checks inputs", {
  expect_snapshot({
    (expect_error(vec_size_common(.size = "foo")))
    (expect_error(vec_size_common(.size = 1:2)))
  })
})

test_that("vec_size_common() mentions `arg` in errors", {
  my_function <- function(...) vec_size_common(..., .arg = "my_arg")
  expect_snapshot({
    (expect_error(my_function(this_arg = 1:2, that_arg = int())))
  })
})

test_that("vec_size_common with no input is 0L unless `.absent` is provided", {
  expect_identical(vec_size_common(), 0L)
  expect_identical(vec_size_common(NULL), 0L)
  expect_equal(vec_size_common(.absent = na_int), na_int)
})

test_that("`.absent` must be supplied when `...` is empty", {
  expect_snapshot({
    (expect_error(vec_size_common(.absent = NULL)))
  })
})

test_that("`.absent` must be a length 1 integer if provided", {
  expect_snapshot({
    (expect_error(vec_size_common(.absent = 1), "must be a single integer"))
    (expect_error(vec_size_common(.absent = c(1L, 2L)), "must be a single integer"))
  })
})

test_that("`NULL` is treated as the absence of input", {
  expect_equal(vec_size_common(1:5, NULL), vec_size_common(1:5))
})

test_that("size 1 is overshadowed by any other size", {
  expect_equal(vec_size_common(1, integer()), 0)
  expect_equal(vec_size_common(1, 1:5), 5)
})

test_that("if not size 1, sizes must be identical", {
  expect_equal(vec_size_common(integer(), integer()), 0)
  expect_error(vec_size_common(1:2, integer()), class = "vctrs_error_incompatible_size")
  expect_error(vec_size_common(1:2, 1:3), class = "vctrs_error_incompatible_size")
})

test_that("argument tags are forwarded", {
  expect_snapshot(error = TRUE, vec_size_common(1:2, 1, 1:4))
  expect_snapshot(error = TRUE, vec_size_common(foo = 1:2, 1, bar = 1:4))
})

test_that("can pass size", {
  expect_identical(vec_size_common(1:2, 1:3, .size = 5L), 5L)
})

test_that("provided size is cast to an integer", {
  expect_identical(vec_size_common(.size = 1), 1L)
})

# list_sizes --------------------------------------------------------------

test_that("only lists are allowed", {
  expect_error(list_sizes(mtcars), "must be a list")
  expect_error(list_sizes(1), "must be a list")
})

test_that("computes element sizes", {
  expect_identical(list_sizes(list(1, 1:3, c("a", "b"))), c(1L, 3L, 2L))
})

test_that("retains list names", {
  x <- list(1, x = 2, a = 3)
  expect_named(list_sizes(x), c("", "x", "a"))

  x <- list_of(y = 1, x = 2, a = 3)
  expect_named(list_sizes(x), c("y", "x", "a"))
})

test_that("retains names of empty lists", {
  x <- structure(list(), names = character())
  expect_named(list_sizes(x), character())
})

# sequences ---------------------------------------------------------------

test_that("vec_seq_along returns size-0 output for size-0 input", {
  expect_equal(vec_seq_along(character()), integer())
  expect_equal(vec_seq_along(data.frame()), integer())
})

test_that("vec_init_along can be called with single argument", {
  expect_equal(vec_init_along(1:3), rep(NA_integer_, 3))
})

# %0% --------------------------------------------------------------------

test_that("uses y when x is empty", {
  expect_equal(1 %0% 2, 1)
  expect_equal(1[0] %0% 2, 2)
})

test_that("failing common type reports correct error call", {
  my_function <- function() vec_ptype2(2, chr())
  expect_snapshot((expect_error(my_function())))
})

test_that("failing cast reports correct error call", {
  my_function <- function() vec_cast(2, chr())
  expect_snapshot((expect_error(my_function())))

  df1 <- data_frame(x = TRUE, y = TRUE)
  df2 <- data_frame(y = "1")
  my_function <- function(lhs, rhs) vec_cast(lhs, rhs)
  expect_snapshot((expect_error(my_function(df1, df2))))

  df1 <- data_frame(y = TRUE)
  df2 <- data_frame(y = "1")
  expect_snapshot((expect_error(my_function(df1, df2))))
})

test_that("lossy cast reports correct error call", {
  my_function <- function() vec_cast(2, lgl())
  expect_snapshot((expect_error(my_function())))
})

test_that("failing common size reports correct error call", {
  my_function <- function() vec_recycle(1:2, 10)
  expect_snapshot((expect_error(my_function())))

  # FIXME
  my_function <- function() vec_size_common(1:2, 1:10)
  expect_snapshot((expect_error(my_function())))
})

test_that("unsupported error reports correct error call", {
  x <- new_vctr(1:2)

  my_function <- function() dim(x) <- 1:2
  expect_snapshot((expect_error(my_function())))

  my_function <- function() median(x)
  expect_snapshot((expect_error(my_function())))
})

test_that("scalar error reports correct error call", {
  my_function <- function() obj_check_vector(foobar())
  expect_snapshot((expect_error(my_function())))
})

test_that("size error reports correct error call", {
  my_function <- function() vec_check_size(1:2, size = 1)
  expect_snapshot((expect_error(my_function())))
})

test_that("bare casts report correct error call", {
  my_function <- function() vec_cast(1.5, int())
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_cast(1.5, lgl())
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_cast(2L, lgl())
  expect_snapshot((expect_error(my_function())))

  # Passing call to `shape_broadcast()`
  my_function <- function() vec_cast(matrix(TRUE), dbl())
  expect_snapshot((expect_error(my_function())))
})

test_that("base S3 casts report correct error call", {
  my_function <- function() vec_cast("a", factor("b"))
  expect_snapshot((expect_error(my_function())))
})

test_that("names validation reports correct error call", {
  my_function <- function() vec_as_names(c("x", "", "y"), repair = "check_unique")
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_as_names("...", repair = "check_unique", repair_arg = "repair")
  expect_snapshot((expect_error(my_function())))
})

test_that("subscript validation reports correct error calls", {
  my_function <- function() vctrs::num_as_location(1, 1L, missing = "bogus")
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vctrs::vec_as_location(10, 2)
  expect_snapshot((expect_error(my_function())))

  my_function <- function(my_arg) vec_as_location(my_arg, 2)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vctrs::vec_as_subscript(my_arg)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vctrs::vec_as_location(my_arg, 2)
  expect_snapshot((expect_error(my_function(list()))))

  my_function <- function(my_arg) vec_as_location(1, my_arg)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vec_as_location(my_arg, 1, missing = "error")
  expect_snapshot((expect_error(my_function(NA))))
})

test_that("`vec_ptype()` reports correct error call", {
  my_function <- function(my_arg) vec_ptype(my_arg)
  expect_snapshot({
    (expect_error(my_function(env())))
    (expect_error(my_function(foobar(list()))))
  })
})

test_that("`vec_slice()` uses `error_call`", {
  my_function <- function(x, i) vec_slice(x, i, error_call = current_env())
  expect_snapshot({
    (expect_error(my_function(env(), 1)))
    (expect_error(my_function(1, 2)))
  })
})

test_that("vec_slice() reports self in error context", {
  expect_snapshot({
    (expect_error(vec_slice(foobar(list()), 1)))
    (expect_error(vec_slice(list(), env())))
  })
})

test_that("list_sizes() reports error context", {
  expect_snapshot({
    (expect_error(list_sizes(foobar(list()))))
    (expect_error(list_sizes(list(env()))))
    (expect_error(list_sizes(list(1, 2, env()))))
    (expect_error(list_sizes(list(1, 2, foo = env()))))
  })
})

test_that("vec_size() reports error context", {
  expect_snapshot({
    (expect_error(vec_size(env())))
  })
})

test_that("vec_cast_common() reports error context", {
  my_function <- function(...) vec_cast_common(...)
  expect_snapshot((expect_error(my_function(my_arg = 1.5, .to = int()))))
  expect_snapshot((expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg"))))
  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg"))))
  expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))

  x <- data.frame(x = "a")
  y <- data.frame(x = 1, y = 2)
  expect_snapshot((expect_error(my_function(this_arg = x, that_arg = y))))
})

test_that("vec_ptype_common() reports error context", {
  my_function <- function(...) vec_ptype_common(...)
  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo"))))
  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg"))))
  expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))
})

test_that("failing common type reports correct error call", {
  my_function <- function() vec_ptype2(2, chr())
  expect_snapshot((expect_error(my_function())))
})

test_that("failing cast reports correct error call", {
  my_function <- function() vec_cast(2, chr())
  expect_snapshot((expect_error(my_function())))

  df1 <- data_frame(x = TRUE, y = TRUE)
  df2 <- data_frame(y = "1")
  my_function <- function(lhs, rhs) vec_cast(lhs, rhs)
  expect_snapshot((expect_error(my_function(df1, df2))))

  df1 <- data_frame(y = TRUE)
  df2 <- data_frame(y = "1")
  expect_snapshot((expect_error(my_function(df1, df2))))
})

test_that("lossy cast reports correct error call", {
  my_function <- function() vec_cast(2, lgl())
  expect_snapshot((expect_error(my_function())))
})

test_that("failing common size reports correct error call", {
  my_function <- function() vec_recycle(1:2, 10)
  expect_snapshot((expect_error(my_function())))

  # FIXME
  my_function <- function() vec_size_common(1:2, 1:10)
  expect_snapshot((expect_error(my_function())))
})

test_that("unsupported error reports correct error call", {
  x <- new_vctr(1:2)

  my_function <- function() dim(x) <- 1:2
  expect_snapshot((expect_error(my_function())))

  my_function <- function() median(x)
  expect_snapshot((expect_error(my_function())))
})

test_that("scalar error reports correct error call", {
  my_function <- function() obj_check_vector(foobar())
  expect_snapshot((expect_error(my_function())))
})

test_that("size error reports correct error call", {
  my_function <- function() vec_check_size(1:2, size = 1)
  expect_snapshot((expect_error(my_function())))
})

test_that("bare casts report correct error call", {
  my_function <- function() vec_cast(1.5, int())
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_cast(1.5, lgl())
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vec_cast(2L, lgl())
  expect_snapshot((expect_error(my_function())))

  # Passing call to `shape_broadcast()`
  my_function <- function() vec_cast(matrix(TRUE), dbl())
  expect_snapshot((expect_error(my_function())))
})

test_that("base S3 casts report correct error call", {
  my_function <- function() vec_cast("a", factor("b"))
  expect_snapshot((expect_error(my_function())))
})

test_that("names validation reports correct error call", {
  my_function <- function() {
    vec_as_names(c("x", "", "y"), repair = "check_unique")
  }
  expect_snapshot((expect_error(my_function())))

  my_function <- function() {
    vec_as_names(c("x", "x"), repair = "check_unique", repair_arg = "repair")
  }
  expect_snapshot((expect_error(my_function())))

  my_function <- function() {
    vec_as_names("...", repair = "check_unique", repair_arg = "repair")
  }
  expect_snapshot((expect_error(my_function())))
})

test_that("subscript validation reports correct error calls", {
  my_function <- function() vctrs::num_as_location(1, 1L, missing = "bogus")
  expect_snapshot((expect_error(my_function())))

  my_function <- function() vctrs::vec_as_location(10, 2)
  expect_snapshot((expect_error(my_function())))

  my_function <- function(my_arg) vec_as_location(my_arg, 2)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vctrs::vec_as_subscript(my_arg)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vctrs::vec_as_location(my_arg, 2)
  expect_snapshot((expect_error(my_function(list()))))

  my_function <- function(my_arg) vec_as_location(1, my_arg)
  expect_snapshot((expect_error(my_function(1.5))))

  my_function <- function(my_arg) vec_as_location(my_arg, 1, missing = "error")
  expect_snapshot((expect_error(my_function(NA))))
})

test_that("`vec_ptype()` reports correct error call", {
  my_function <- function(my_arg) vec_ptype(my_arg)
  expect_snapshot({
    (expect_error(my_function(env())))
    (expect_error(my_function(foobar(list()))))
  })
})

test_that("`vec_slice()` uses `error_call`", {
  my_function <- function(x, i) vec_slice(x, i, error_call = current_env())
  expect_snapshot({
    (expect_error(my_function(env(), 1)))
    (expect_error(my_function(1, 2)))
  })
})

test_that("vec_slice() reports self in error context", {
  expect_snapshot({
    (expect_error(vec_slice(foobar(list()), 1)))
    (expect_error(vec_slice(list(), env())))
  })
})

test_that("list_sizes() reports error context", {
  expect_snapshot({
    (expect_error(list_sizes(foobar(list()))))
    (expect_error(list_sizes(list(env()))))
    (expect_error(list_sizes(list(1, 2, env()))))
    (expect_error(list_sizes(list(1, 2, foo = env()))))
  })
})

test_that("vec_size() reports error context", {
  expect_snapshot({
    (expect_error(vec_size(env())))
  })
})

test_that("vec_cast_common() reports error context", {
  my_function <- function(...) vec_cast_common(...)
  expect_snapshot((expect_error(my_function(my_arg = 1.5, .to = int()))))
  expect_snapshot(
    (expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg")))
  )
  expect_snapshot(
    (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
  )
  expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))

  x <- data.frame(x = "a")
  y <- data.frame(x = 1, y = 2)
  expect_snapshot((expect_error(my_function(this_arg = x, that_arg = y))))
})

test_that("vec_ptype_common() reports error context", {
  my_function <- function(...) vec_ptype_common(...)
  expect_snapshot((expect_error(my_function(this_arg = 1, that_arg = "foo"))))
  expect_snapshot(
    (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
  )
  expect_snapshot((expect_error(my_function(1, "foo", .arg = "my_arg"))))
})

test_that("vec_as_short_length() checks inputs", {
  expect_equal(vec_as_short_length(0), 0)
  expect_equal(vec_as_short_length(1L), 1)

  my_function <- function(my_arg) vec_as_short_length(my_arg)
  expect_snapshot({
    (expect_error(my_function(-1)))
    (expect_error(my_function(1:2)))
    (expect_error(my_function(1.5)))
    (expect_error(my_function(NA)))
    (expect_error(my_function(na_int)))
    (expect_error(my_function("foo")))
    (expect_error(my_function(foobar(1:2))))
    (expect_error(my_function(.Machine$double.xmax)))
  })
})

test_that("vec_as_short_length() has a special error about long vector support", {
  # In particular, skips on 32-bit Windows where `r_ssize == int`
  skip_if(.Machine$sizeof.pointer < 8L, message = "No long vector support")

  my_function <- function(my_arg) vec_as_short_length(my_arg)
  expect_snapshot({
    (expect_error(my_function(.Machine$integer.max + 1)))
  })
})

# vec_size -----------------------------------------------------------------

test_that("vec_size must be called with vector", {
  expect_error(vec_size(mean), class = "vctrs_error_scalar_type")
})

test_that("length is number of rows", {
  expect_equal(vec_size(integer()), 0)
  expect_equal(vec_size(array(integer())), 0)

  expect_equal(vec_size(1:2), 2)
  expect_equal(vec_size(array(dim = 2)), 2)

  expect_equal(vec_size(matrix(nrow = 2, ncol = 3)), 2)
  expect_equal(vec_size(array(dim = c(2, 1, 5))), 2)
})

test_that("length of record is number of rows, not fields", {
  r <- new_rcrd(list(x = 1:10))
  expect_equal(vec_size(r), 10)
})

test_that("handles three types of data frame rownames", {
  df1 <- df2 <- df3 <- data.frame(x = 1:3)
  rownames(df1) <- NULL
  rownames(df2) <- 3:1
  rownames(df3) <- letters[1:3]

  expect_equal(vec_size(df1), 3)
  expect_equal(vec_size(df2), 3)
  expect_equal(vec_size(df3), 3)
})

test_that("handles positive short row names (#220)", {
  data <- structure(mtcars, row.names = c(NA, 32))
  expect_identical(vec_size(data), 32L)
})

test_that("size is proxied", {
  local_env_proxy()
  expect_size(new_proxy(1:3), 3)
  expect_size(new_proxy(list(1, 2, 3)), 3)
  expect_size(new_proxy(foobar(list(1, 2, 3))), 3)
})

test_that("`NULL` has size zero", {
  expect_identical(vec_size(NULL), 0L)
})

test_that("can take the size of unspecified objects", {
  expect_size(NA, 1)
  expect_size(c(NA, NA), 2)
  expect_size(unspecified(2), 2)
})

# vec_size_common ---------------------------------------------------------

test_that("vec_size_common() checks inputs", {
  expect_snapshot({
    (expect_error(vec_size_common(.size = "foo")))
    (expect_error(vec_size_common(.size = 1:2)))
  })
})

test_that("vec_size_common() mentions `arg` in errors", {
  my_function <- function(...) vec_size_common(..., .arg = "my_arg")
  expect_snapshot({
    (expect_error(my_function(this_arg = 1:2, that_arg = int())))
  })
})

test_that("vec_size_common with no input is 0L unless `.absent` is provided", {
  expect_identical(vec_size_common(), 0L)
  expect_identical(vec_size_common(NULL), 0L)
  expect_equal(vec_size_common(.absent = na_int), na_int)
})

test_that("`.absent` must be supplied when `...` is empty", {
  expect_snapshot({
    (expect_error(vec_size_common(.absent = NULL)))
  })
})

test_that("`.absent` must be a length 1 integer if provided", {
  expect_snapshot({
    (expect_error(vec_size_common(.absent = 1), "must be a single integer"))
    (expect_error(
      vec_size_common(.absent = c(1L, 2L)),
      "must be a single integer"
    ))
  })
})

test_that("`NULL` is treated as the absence of input", {
  expect_equal(vec_size_common(1:5, NULL), vec_size_common(1:5))
})

test_that("size 1 is overshadowed by any other size", {
  expect_equal(vec_size_common(1, integer()), 0)
  expect_equal(vec_size_common(1, 1:5), 5)
})

test_that("if not size 1, sizes must be identical", {
  expect_equal(vec_size_common(integer(), integer()), 0)
  expect_error(
    vec_size_common(1:2, integer()),
    class = "vctrs_error_incompatible_size"
  )
  expect_error(
    vec_size_common(1:2, 1:3),
    class = "vctrs_error_incompatible_size"
  )
})

test_that("argument tags are forwarded", {
  expect_snapshot(error = TRUE, vec_size_common(1:2, 1, 1:4))
  expect_snapshot(error = TRUE, vec_size_common(foo = 1:2, 1, bar = 1:4))
})

test_that("can pass size", {
  expect_identical(vec_size_common(1:2, 1:3, .size = 5L), 5L)
})

test_that("provided size is cast to an integer", {
  expect_identical(vec_size_common(.size = 1), 1L)
})

test_that("doesn't mutate the input", {
  x <- list(a = 1, b = 2:3)
  expect_identical(vec_size_common(!!!x), 2L)
  expect_identical(x, list(a = 1, b = 2:3))
})

# list_sizes --------------------------------------------------------------

test_that("only lists are allowed", {
  expect_error(list_sizes(mtcars), "must be a list")
  expect_error(list_sizes(1), "must be a list")
})

test_that("computes element sizes", {
  expect_identical(list_sizes(list(1, 1:3, c("a", "b"))), c(1L, 3L, 2L))
})

test_that("retains list names", {
  x <- list(1, x = 2, a = 3)
  expect_named(list_sizes(x), c("", "x", "a"))

  x <- list_of(y = 1, x = 2, a = 3)
  expect_named(list_sizes(x), c("y", "x", "a"))
})

test_that("retains names of empty lists", {
  x <- structure(list(), names = character())
  expect_named(list_sizes(x), character())
})

# sequences ---------------------------------------------------------------

test_that("vec_seq_along returns size-0 output for size-0 input", {
  expect_equal(vec_seq_along(character()), integer())
  expect_equal(vec_seq_along(data.frame()), integer())
})

test_that("vec_init_along can be called with single argument", {
  expect_equal(vec_init_along(1:3), rep(NA_integer_, 3))
})

# %0% --------------------------------------------------------------------

test_that("uses y when x is empty", {
  expect_equal(1 %0% 2, 1)
  expect_equal(1[0] %0% 2, 2)
})

parens <- function(x, left = TRUE) {
  x_lines <- strsplit(x, "\n")
  x_lines <- map(x_lines, paren, left = left)
  map_chr(x_lines, paste0, collapse = "\n")
}

paren <- function(x, left = TRUE) {
  if (length(x) <= 1) {
    if (left) {
      paste0("( ", x)
    } else {
      paste0(x, " )")
    }
  } else {
    if (left) {
      paste0(c("\u250c ", rep("\u2502 ", length(x) - 2), "\u2514 "), x)
    } else {
      paste0(format(x), c(" \u2510", rep(" \u2502", length(x) - 2), " \u2518"))
    }
  }
}

pad_height <- function(x) {
  pad <- function(x, n) c(x, rep("", n - length(x)))

  lines <- strsplit(x, "\n")
  height <- max(map_int(lines, length))
  lines <- map(lines, pad, height)
  map_chr(lines, paste0, "\n", collapse = "")
}

pad_width <- function(x) {
  lines <- strsplit(x, "\n", fixed = TRUE)

  # fix up strsplit bug
  n <- map_int(lines, length)
  lines[n == 0] <- ""

  width <- max(unlist(map(lines, nchar)))
  lines <- map(lines, format, width = width)
  map_chr(lines, paste, collapse = "\n")
}

str_backtick <- function(x) {
  paste0("`", x, "`")
}
str_is_multiline <- function(x) {
  grepl("\n", x)
}

#' Create a data frame from all combinations of the inputs
#'
#' @description
#' `vec_expand_grid()` creates a new data frame by creating a grid of all
#' possible combinations of the input vectors. It is inspired by
#' [expand.grid()]. Compared with `expand.grid()`, it:
#'
#' - Produces sorted output by default by varying the first column the slowest,
#'   rather than the fastest. Control this with `.vary`.
#'
#' - Never converts strings to factors.
#'
#' - Does not add additional attributes.
#'
#' - Drops `NULL` inputs.
#'
#' - Can expand any vector type, including data frames and [records][new_rcrd].
#'
#' @details
#' If any input is empty (i.e. size 0), then the result will have 0 rows.
#'
#' If no inputs are provided, the result is a 1 row data frame with 0 columns.
#' This is consistent with the fact that `prod()` with no inputs returns `1`.
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams df_list
#'
#' @param ... Name-value pairs. The name will become the column name in the
#'   resulting data frame.
#'
#' @param .vary One of:
#'
#'   - `"slowest"` to vary the first column slowest. This produces sorted
#'     output and is generally the most useful.
#'
#'   - `"fastest"` to vary the first column fastest. This matches the behavior
#'     of [expand.grid()].
#'
#' @returns
#' A data frame with as many columns as there are inputs in `...` and as many
#' rows as the [prod()] of the sizes of the inputs.
#'
#' @export
#' @examples
#' vec_expand_grid(x = 1:2, y = 1:3)
#'
#' # Use `.vary` to match `expand.grid()`:
#' vec_expand_grid(x = 1:2, y = 1:3, .vary = "fastest")
#'
#' # Can also expand data frames
#' vec_expand_grid(
#'   x = data_frame(a = 1:2, b = 3:4),
#'   y = 1:4
#' )
vec_expand_grid <- function(
  ...,
  .vary = "slowest",
  .name_repair = "check_unique",
  .error_call = current_env()
) {
  .vary <- arg_match0(
    arg = .vary,
    values = c("slowest", "fastest"),
    error_call = .error_call
  )

  .Call(ffi_vec_expand_grid, list2(...), .vary, .name_repair, environment())
}

new_ctor <- function(base_class) {
  function(x = list(), ..., class = NULL) {
    if (inherits(x, "tbl_df")) {
      tibble::new_tibble(x, class = c(class, base_class), nrow = nrow(x))
    } else if (is.data.frame(x)) {
      structure(x, class = c(class, base_class, "data.frame"), ...)
    } else {
      structure(x, class = c(class, base_class), ...)
    }
  }
}

foobar <- new_ctor("vctrs_foobar")
foobar_c <- new_ctor("vctrs_foobar_c")
foobaz <- new_ctor("vctrs_foobaz")
quux <- new_ctor("vctrs_quux")

expect_foobar <- function(x) expect_s3_class({{ x }}, "vctrs_foobar")
expect_foobar_c <- function(x) expect_s3_class({{ x }}, "vctrs_foobar_c")
expect_foobaz <- function(x) expect_s3_class({{ x }}, "vctrs_foobaz")
expect_quux <- function(x) expect_s3_class({{ x }}, "vctrs_quux")

local_c_foobar <- function(frame = caller_env()) {
  local_methods(
    c.vctrs_foobar = function(...) foobar_c(NextMethod()),
    .frame = frame
  )
}
with_c_foobar <- function(expr) {
  local_c_foobar()
  expr
}

unrownames <- function(x) {
  row.names(x) <- NULL
  x
}

local_methods <- function(..., .frame = caller_env()) {
  local_bindings(..., .env = global_env(), .frame = .frame)
}
with_methods <- function(.expr, ...) {
  local_methods(...)
  .expr
}

local_proxy <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_proxy.vctrs_proxy = function(x, ...) proxy_deref(x),
    vec_restore.vctrs_proxy = function(x, to, ...) new_proxy(x),

    vec_ptype2.vctrs_proxy = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_proxy")
    },
    vec_ptype2.vctrs_proxy.vctrs_proxy = function(x, y, ...) {
      new_proxy(vec_ptype(proxy_deref(x)))
    },

    vec_cast.vctrs_proxy = function(x, to, ...) {
      UseMethod("vec_cast.vctrs_proxy")
    },
    vec_cast.vctrs_proxy.vctrs_proxy = function(x, to, ...) x
  )
}

new_proxy <- function(x) {
  structure(list(env(x = x)), class = "vctrs_proxy")
}
proxy_deref <- function(x, ...) {
  x[[1]]$x
}
local_env_proxy <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_proxy.vctrs_proxy = proxy_deref,
    vec_restore.vctrs_proxy = function(x, ...) new_proxy(x),
    vec_cast.vctrs_proxy = function(x, to, ...) {
      UseMethod("vec_cast.vctrs_proxy")
    },
    vec_cast.vctrs_proxy.vctrs_proxy = function(x, to, ...) x,
    vec_ptype2.vctrs_proxy = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_proxy")
    },
    vec_ptype2.vctrs_proxy.vctrs_proxy = function(x, y, ...) {
      new_proxy(proxy_deref(x)[0])
    }
  )
}

local_no_stringsAsFactors <- function(frame = caller_env()) {
  local_options(.frame = frame, stringsAsFactors = FALSE)
}

tibble <- function(...) {
  tibble::tibble(...)
}

local_foobar_proxy <- function(frame = caller_env()) {
  local_methods(.frame = frame, vec_proxy.vctrs_foobar = function(x, ...) x)
}

subclass <- function(x) {
  class(x) <- c("vctrs_foo", "vctrs_foobar", class(x))
  x
}


# Subclass promoted to logical
new_lgl_subtype <- function(x) {
  stopifnot(is_logical(x))
  structure(x, class = "vctrs_lgl_subtype")
}
local_lgl_subtype <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_ptype2.vctrs_lgl_subtype = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_lgl_subtype")
    },
    vec_ptype2.vctrs_lgl_subtype.vctrs_lgl_subtype = function(x, y, ...) x,
    vec_ptype2.vctrs_lgl_subtype.logical = function(x, y, ...) y,
    vec_ptype2.logical.vctrs_lgl_subtype = function(x, y, ...) x,

    vec_cast.vctrs_lgl_subtype = function(x, to, ...) {
      UseMethod("vec_cast.vctrs_lgl_subtype")
    },
    vec_cast.vctrs_lgl_subtype.vctrs_lgl_subtype = function(x, to, ...) x,
    vec_cast.vctrs_lgl_subtype.logical = function(x, to, ...) {
      new_lgl_subtype(x)
    },
    vec_cast.logical.vctrs_lgl_subtype = function(x, to, ...) unstructure(x)
  )
}
with_lgl_subtype <- function(expr) {
  local_lgl_subtype()
  expr
}

# Logical promoted to subclass
new_lgl_supertype <- function(x) {
  stopifnot(is_logical(x))
  structure(x, class = "vctrs_lgl_supertype")
}
local_lgl_supertype <- function(frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_ptype2.vctrs_lgl_supertype = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_lgl_supertype")
    },
    vec_ptype2.vctrs_lgl_supertype.vctrs_lgl_supertype = function(x, y, ...) x,
    vec_ptype2.vctrs_lgl_supertype.logical = function(x, y, ...) x,
    vec_ptype2.logical.vctrs_lgl_supertype = function(x, y, ...) y,

    vec_cast.vctrs_lgl_supertype = function(x, to, ...) {
      UseMethod("vec_cast.vctrs_lgl_supertype")
    },
    vec_cast.vctrs_lgl_supertype.vctrs_lgl_supertype = function(x, to, ...) x,
    vec_cast.vctrs_lgl_supertype.logical = function(x, to, ...) {
      new_lgl_subtype(x)
    },
    vec_cast.logical.vctrs_lgl_supertype = function(x, to, ...) unstructure(x)
  )
}
with_lgl_supertype <- function(expr) {
  local_lgl_supertype()
  expr
}

foobar_df_ptype2 <- function(x, y, ...) {
  foobar(df_ptype2(x, y, ...))
}
foobar_df_cast <- function(x, y, ...) {
  foobar(df_cast(x, y, ...))
}
local_foobar_df_methods <- function(expr, frame = caller_env()) {
  local_methods(
    .frame = frame,
    vec_ptype2.vctrs_foobar.vctrs_foobar = foobar_df_ptype2,
    vec_ptype2.data.frame.vctrs_foobar = foobar_df_ptype2,
    vec_ptype2.vctrs_foobar.data.frame = foobar_df_ptype2,
    vec_cast.vctrs_foobar.vctrs_foobar = foobar_df_cast,
    vec_cast.data.frame.vctrs_foobar = foobar_df_cast,
    vec_cast.vctrs_foobar.data.frame = foobar_df_cast
  )
}
with_foobar_df_methods <- function(expr) {
  local_foobar_df_methods()
  expr
}

set_tibble <- function(x) {
  base <- class(x)[-length(class(x))]
  class(x) <- c(base, "tbl_df", "tbl", "data.frame")
  x
}

test_that("`x` must be a list", {
  expect_snapshot(error = TRUE, {
    list_unchop(1, indices = list(1))
  })
  expect_snapshot(error = TRUE, {
    list_unchop(
      1,
      indices = list(1),
      error_call = call("foo"),
      error_arg = "arg"
    )
  })
  expect_snapshot(error = TRUE, {
    list_unchop(data.frame(x = 1), indices = list(1))
  })
})

test_that("`indices` must be a list", {
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = 1)
  })
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = 1, error_call = call("foo"))
  })
  expect_snapshot(error = TRUE, {
    list_unchop(list(1), indices = data.frame(x = 1))
  })
})

test_that("`indices` must be a list of integers", {
  expect_error(
    list_unchop(list(1), indices = list("x")),
    class = "vctrs_error_subscript_type"
  )
  expect_error(
    list_unchop(list(1), indices = list(TRUE)),
    class = "vctrs_error_subscript_type"
  )
  expect_error(
    list_unchop(list(1), indices = list(quote(name))),
    class = "vctrs_error_subscript_type"
  )
})

test_that("`x` and `indices` must be lists of the same size", {
  expect_snapshot(error = TRUE, {
    list_unchop(list(1, 2), indices = list(1))
  })
})

test_that("can unchop with an AsIs list (#1463)", {
  x <- I(list(1, 2))
  expect_identical(list_unchop(x), c(1, 2))
})

test_that("can unchop empty vectors", {
  expect_null(list_unchop(list()))
  expect_null(list_unchop(list(), indices = list()))
  expect_identical(
    list_unchop(list(), indices = list(), ptype = numeric()),
    numeric()
  )
})

test_that("can unchop a list of NULL", {
  expect_null(list_unchop(list(NULL), indices = list(integer())))
  expect_identical(
    list_unchop(list(NULL), indices = list(integer()), ptype = numeric()),
    numeric()
  )
  expect_identical(
    list_unchop(
      list(NULL, NULL),
      indices = list(integer(), integer()),
      ptype = numeric()
    ),
    numeric()
  )
})

test_that("NULLs are ignored when unchopped with other vectors", {
  expect_identical(
    list_unchop(list("a", NULL, "b")),
    c("a", "b")
  )
  expect_identical(
    list_unchop(list("a", NULL, "b"), indices = list(2, integer(), 1)),
    c("b", "a")
  )

  # Homogeneous fallback
  expect_identical(
    list_unchop(list(foobar("a"), NULL, foobar("b"))),
    foobar(c("a", "b"))
  )
  expect_identical(
    list_unchop(
      list(foobar("a"), NULL, foobar("b")),
      indices = list(2, integer(), 1)
    ),
    foobar(c("b", "a"))
  )
  expect_identical(
    list_unchop(
      list(foobar("a"), NULL, foobar("b")),
      indices = list(2, 3, 1)
    ),
    foobar(c("b", "a", NA))
  )

  # Homoegeneous fallback (`NULL` at front)
  expect_identical(
    list_unchop(list(NULL, foobar("a"), foobar("b"))),
    foobar(c("a", "b"))
  )
  expect_identical(
    list_unchop(
      list(NULL, foobar("a"), foobar("b")),
      indices = list(integer(), 2, 1)
    ),
    foobar(c("b", "a"))
  )
  expect_identical(
    list_unchop(
      list(NULL, foobar("a"), foobar("b")),
      indices = list(3, 2, 1)
    ),
    foobar(c("b", "a", NA))
  )

  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_unchop(list(foobar("a"), NULL, foobar("b"))),
      foobar_c(c("a", "b"))
    )
    expect_identical(
      list_unchop(
        list(foobar("a"), NULL, foobar("b")),
        indices = list(2, integer(), 1)
      ),
      foobar_c(c("b", "a"))
    )
    expect_identical(
      list_unchop(
        list(foobar("a"), NULL, foobar("b")),
        indices = list(2, 3, 1)
      ),
      foobar_c(c("b", "a", NA))
    )
  })

  # `c()` fallback (`NULL` at front)
  with_c_foobar({
    expect_identical(
      list_unchop(list(NULL, foobar("a"), foobar("b"))),
      foobar_c(c("a", "b"))
    )
    expect_identical(
      list_unchop(
        list(NULL, foobar("a"), foobar("b")),
        indices = list(integer(), 2, 1)
      ),
      foobar_c(c("b", "a"))
    )
    expect_identical(
      list_unchop(
        list(NULL, foobar("a"), foobar("b")),
        indices = list(3, 2, 1)
      ),
      foobar_c(c("b", "a", NA))
    )
  })
})

test_that("can use a `NULL` element with a corresponding index", {
  # We've determined this is the behavior we are locked to in `list_unchop()`,
  # but in `list_combine()` we return `unspecified(2)` here because the user
  # also specifies a `size`, making this less ambiguous.
  expect_null(list_unchop(list(NULL), indices = list(1:2)))

  expect_identical(
    list_unchop(list(NULL), indices = list(1:2), ptype = integer()),
    c(NA_integer_, NA_integer_)
  )

  x <- list("a", NULL, c("b", "c"))
  indices <- list(3L, c(1L, 4L), c(2L, 5L))
  expect_identical(list_unchop(x, indices = indices), c(NA, "b", "a", NA, "c"))
})

test_that("can unchop atomic vectors", {
  expect_identical(list_unchop(list(1, 2), indices = list(2, 1)), c(2, 1))
  expect_identical(
    list_unchop(list("a", "b"), indices = list(2, 1)),
    c("b", "a")
  )
})

test_that("can unchop lists", {
  x <- list(list("a", "b"), list("c"))
  indices <- list(c(2, 3), 1)

  expect_identical(list_unchop(x, indices = indices), list("c", "a", "b"))
})

test_that("NA is logical if no other types intervene", {
  expect_identical(
    list_unchop(list(logical()), indices = list(integer())),
    logical()
  )
  expect_identical(
    list_unchop(list(NA), indices = list(1)),
    NA
  )
  expect_identical(
    list_unchop(list(NA, NA), indices = list(1, 2)),
    c(NA, NA)
  )
})

test_that("can unchop data frames of 1 column", {
  indices <- list(c(3, 1), c(2, 4))

  values <- list(
    data_frame(x = 1:2),
    data_frame(x = 3:4)
  )
  expect_identical(
    list_unchop(values, indices = indices),
    data_frame(x = int(2, 3, 1, 4))
  )

  # Homogeneous fallback (#1975)
  values <- list(
    data_frame(x = foobar(1:2)),
    data_frame(x = foobar(3:4))
  )
  expect_identical(
    list_unchop(values, indices = indices),
    data_frame(x = foobar(int(2, 3, 1, 4)))
  )

  # `c()` fallback
  with_c_foobar({
    values <- list(
      data_frame(x = foobar(1:2)),
      data_frame(x = foobar(3:4))
    )
    expect_identical(
      list_unchop(values, indices = indices),
      data_frame(x = foobar_c(int(2, 3, 1, 4)))
    )
  })
})

test_that("can unchop data frames of >1 column", {
  indices <- list(c(3, 1), c(2, 4))

  values <- list(
    data_frame(x = 1:2, y = letters[1:2], z = c(1, 2)),
    data_frame(x = 3:4, y = letters[3:4], z = c(3, 4))
  )
  expect_identical(
    list_unchop(values, indices = indices),
    data_frame(
      x = int(2, 3, 1, 4),
      y = letters[c(2, 3, 1, 4)],
      z = dbl(2, 3, 1, 4),
    )
  )

  # Homogeneous fallback (#1975)
  # Mix of fallback and non-fallback columns
  values <- list(
    data_frame(x = foobar(1:2), y = foobar(letters[1:2]), z = c(1, 2)),
    data_frame(x = foobar(3:4), y = foobar(letters[3:4]), z = c(3, 4))
  )
  expect_identical(
    list_unchop(values, indices = indices),
    data_frame(
      x = foobar(int(2, 3, 1, 4)),
      y = foobar(letters[c(2, 3, 1, 4)]),
      z = dbl(2, 3, 1, 4)
    )
  )

  # `c()` fallback
  # Mix of fallback and non-fallback columns
  with_c_foobar({
    values <- list(
      data_frame(x = foobar(1:2), y = foobar(letters[1:2]), z = c(1, 2)),
      data_frame(x = foobar(3:4), y = foobar(letters[3:4]), z = c(3, 4))
    )
    expect_identical(
      list_unchop(values, indices = indices),
      data_frame(
        x = foobar_c(int(2, 3, 1, 4)),
        y = foobar_c(letters[c(2, 3, 1, 4)]),
        z = dbl(2, 3, 1, 4)
      )
    )
  })
})

test_that("can unchop factors", {
  fctr1 <- factor("z")
  fctr2 <- factor(c("x", "y"))

  x <- list(fctr1, fctr2)
  indices <- list(2, c(3, 1))

  # levels are in the order they are seen!
  expect <- factor(c("y", "z", "x"), levels = c("z", "x", "y"))

  expect_identical(list_unchop(x, indices = indices), expect)
})

test_that("can fallback when unchopping matrices", {
  mat1 <- matrix(1:4, nrow = 2, ncol = 2)
  mat2 <- matrix(5:10, nrow = 3, ncol = 2)

  x <- list(mat1, mat2)
  indices <- list(c(4, 1), c(2, 3, 5))

  expect <- vec_slice(vec_c(mat1, mat2), vec_order(vec_c(!!!indices)))

  expect_identical(list_unchop(x, indices = indices), expect)
  expect_identical(list_unchop(x), vec_c(mat1, mat2))
})

test_that("can fallback when unchopping arrays of >2D", {
  arr1 <- array(1:8, c(2, 2, 2))
  arr2 <- matrix(9:10, c(1, 2))

  x <- list(arr1, arr2)
  indices <- list(c(3, 1), 2)

  expect <- vec_slice(vec_c(arr1, arr2), vec_order(vec_c(!!!indices)))

  expect_identical(list_unchop(x, indices = indices), expect)
  expect_identical(list_unchop(x), vec_c(arr1, arr2))
})

test_that("can unchop with all size 0 elements and get the right ptype", {
  indices <- list(integer(), integer())

  x <- list(integer(), integer())
  expect_identical(list_unchop(x, indices = indices), integer())

  # Homogeneous fallback
  x <- list(foobar(integer()), foobar(integer()))
  expect_identical(list_unchop(x, indices = indices), foobar(integer()))

  # `c()` fallback
  with_c_foobar({
    x <- list(foobar(integer()), foobar(integer()))
    expect_identical(list_unchop(x, indices = indices), foobar_c(integer()))
  })
})

test_that("can unchop with some size 0 elements", {
  x <- list(integer(), 1:2, integer())
  indices <- list(integer(), 2:1, integer())
  expect_identical(list_unchop(x, indices = indices), 2:1)
})

test_that("`NULL` is a valid index", {
  expect_identical(
    list_unchop(list(1, 2), indices = list(NULL, 1)),
    2
  )
  expect_snapshot(error = TRUE, {
    list_unchop(list(1, 2), indices = list(NULL, 2))
  })

  # Homogeneous fallback
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 1)),
    foobar(2)
  )
  expect_snapshot(error = TRUE, {
    list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 2))
  })

  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 1)),
      foobar_c(2)
    )
    expect_snapshot(error = TRUE, {
      list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 2))
    })
  })
})

test_that("unchopping recycles elements of x to the size of the index", {
  indices <- list(c(3, 4, 5), c(2, 1))

  expect_identical(
    list_unchop(list(1, 2), indices = indices),
    c(2, 2, 1, 1, 1)
  )
  # Homogeneous fallback
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = indices),
    foobar(c(2, 2, 1, 1, 1))
  )
  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_unchop(list(foobar(1), foobar(2)), indices = indices),
      foobar_c(c(2, 2, 1, 1, 1))
    )
  })

  indices <- list(1:3)

  expect_snapshot(error = TRUE, {
    list_unchop(list(1:2), indices = indices)
  })
  expect_snapshot(error = TRUE, {
    list_unchop(
      list(1:2),
      indices = indices,
      error_call = call("foo"),
      error_arg = "arg"
    )
  })

  # Homogeneous fallback
  expect_snapshot(error = TRUE, {
    list_unchop(list(foobar(1:2)), indices = indices)
  })
  expect_snapshot(error = TRUE, {
    list_unchop(
      list(foobar(1:2)),
      indices = indices,
      error_call = call("foo"),
      error_arg = "arg"
    )
  })

  # `c()` fallback
  with_c_foobar({
    expect_snapshot(error = TRUE, {
      list_unchop(list(foobar(1:2)), indices = indices)
    })
    expect_snapshot(error = TRUE, {
      list_unchop(
        list(foobar(1:2)),
        indices = indices,
        error_call = call("foo"),
        error_arg = "arg"
      )
    })
  })
})

test_that("unchopping takes the common type", {
  x <- list(1, "a")
  indices <- list(1, 2)

  expect_snapshot({
    (expect_error(
      list_unchop(x, indices = indices),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      list_unchop(
        x,
        indices = indices,
        error_call = call("foo"),
        error_arg = "arg"
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })

  x <- list(1, 2L)

  expect_type(list_unchop(x, indices = indices), "double")
})

test_that("common type failure uses positional errors", {
  expect_snapshot({
    x <- list(1, a = "x", 2)

    # Looking for `x[[1]]` and `x$a`
    (expect_error(list_unchop(x)))
    (expect_error(list_unchop(x, indices = list(2, 1, 3))))

    # Directed cast should also produce directional errors (#1690)
    (expect_error(list_unchop(x, ptype = double())))
    (expect_error(list_unchop(x, indices = list(2, 1, 3), ptype = double())))

    # Lossy cast
    y <- list(1, a = 2.5)
    (expect_error(list_unchop(y, ptype = integer())))
    (expect_error(list_unchop(y, indices = list(2, 1), ptype = integer())))
  })
})

test_that("can specify a ptype to override common type", {
  indices <- list(1, 2)

  x <- list(1, 2L)
  expect_identical(
    list_unchop(x, indices = indices, ptype = integer()),
    c(1L, 2L)
  )

  x <- list(1.5, 2)
  expect_snapshot({
    (expect_error(list_unchop(x, indices = indices, ptype = integer())))
    (expect_error(list_unchop(
      x,
      indices = indices,
      ptype = integer(),
      error_call = call("foo"),
      error_arg = "arg"
    )))
  })
})

test_that("leaving `indices = NULL` unchops sequentially", {
  x <- list(1:2, 3:5, 6L)
  expect_identical(list_unchop(x), 1:6)

  # Homogeneous fallback
  x <- list(foobar(1:2), foobar(3:5), foobar(6L))
  expect_identical(list_unchop(x), foobar(1:6))

  # `c()` fallback
  with_c_foobar({
    x <- list(foobar(1:2), foobar(3:5), foobar(6L))
    expect_identical(list_unchop(x), foobar_c(1:6))
  })
})

test_that("outer names are kept", {
  x <- list(x = 1, y = 2)
  expect_named(list_unchop(x), c("x", "y"))
  expect_named(list_unchop(x, indices = list(2, 1)), c("y", "x"))

  # Homogeneous fallback
  x <- list(x = foobar(1), y = foobar(2))
  expect_named(list_unchop(x), c("x", "y"))
  expect_named(list_unchop(x, indices = list(2, 1)), c("y", "x"))

  # `c()` fallback (dependent on `c()` implementation)
  with_c_foobar({
    x <- list(x = foobar(1), y = foobar(2))
    expect_identical(list_unchop(x), foobar_c(c(x = 1, y = 2)))
    expect_named(list_unchop(x), c("x", "y"))
    expect_named(list_unchop(x, indices = list(2, 1)), c("y", "x"))
  })
})

test_that("outer names are recycled in the right order", {
  x <- list(x = 1, y = 2)
  expect_error(list_unchop(x, indices = list(c(1, 2), 3)), "Can't merge")
  expect_named(
    list_unchop(x, indices = list(c(1, 3), 2), name_spec = "{outer}_{inner}"),
    c("x_1", "y", "x_2")
  )
  expect_named(
    list_unchop(x, indices = list(c(3, 1), 2), name_spec = "{outer}_{inner}"),
    c("x_2", "y", "x_1")
  )
})

test_that("outer names can be merged with inner names", {
  x <- list(x = c(a = 1), y = c(b = 2))
  expect_error(list_unchop(x), "Can't merge")
  expect_named(list_unchop(x, name_spec = "{outer}_{inner}"), c("x_a", "y_b"))
  expect_named(
    list_unchop(x, indices = list(2, 1), name_spec = "{outer}_{inner}"),
    c("y_b", "x_a")
  )
})

test_that("preserves names when inputs are cast to a common type (#1689)", {
  expect_named(list_unchop(list(c(a = 1)), ptype = integer()), "a")
  expect_named(
    list_unchop(list(c(a = 1)), ptype = integer(), indices = list(1)),
    "a"
  )

  # With name spec
  name_spec <- "{outer}_{inner}"
  expect_named(
    list_unchop(list(foo = c(a = 1)), ptype = integer(), name_spec = name_spec),
    "foo_a"
  )
  expect_named(
    list_unchop(
      list(foo = c(a = 1)),
      ptype = integer(),
      name_spec = name_spec,
      indices = list(1)
    ),
    "foo_a"
  )

  # When `x` elements are recycled, names are also recycled
  x <- list(c(a = 1), c(b = 2))
  indices <- list(1:2, 3:4)
  expect_named(
    list_unchop(x, indices = indices, ptype = integer()),
    c("a", "a", "b", "b")
  )

  expect_named(
    list_unchop(
      list(foo = c(a = 1)),
      ptype = integer(),
      name_spec = "inner"
    ),
    "a"
  )
  expect_named(
    list_unchop(
      list(foo = c(a = 1)),
      ptype = integer(),
      name_spec = "inner",
      indices = list(1)
    ),
    "a"
  )
})

test_that("not all inputs have to be named", {
  x <- list(c(a = 1), 2, c(c = 3))
  indices <- list(2, 1, 3)
  expect_named(list_unchop(x, indices = indices), c("", "a", "c"))

  # Homoegenous fallback
  x <- list(foobar(c(a = 1)), foobar(2), foobar(c(c = 3)))
  indices <- list(2, 1, 3)
  expect_named(list_unchop(x, indices = indices), c("", "a", "c"))

  # `c()` fallback
  with_c_foobar({
    x <- list(foobar(c(a = 1)), foobar(2), foobar(c(c = 3)))
    indices <- list(2, 1, 3)
    out <- list_unchop(x, indices = indices)
    expect_foobar_c(out)
    expect_named(out, c("", "a", "c"))
  })
})

test_that("list_unchop() keeps data frame row names", {
  df1 <- data.frame(x = 1:2, row.names = c("r1", "r2"))
  df2 <- data.frame(x = 3:4, row.names = c("r3", "r4"))

  x <- list(df1, df2)
  indices <- list(c(3, 1), c(2, 4))

  result <- list_unchop(x, indices = indices)
  expect <- c("r2", "r3", "r1", "r4")

  expect_identical(vec_names(result), expect)
})

test_that("individual data frame columns retain vector names", {
  df1 <- data_frame(x = c(a = 1, b = 2))
  df2 <- data_frame(x = c(c = 3))

  x <- list(df1, df2)
  indices <- list(c(1, 2), 3)

  result <- list_unchop(x, indices = indices)

  expect_named(result$x, c("a", "b", "c"))

  # Names should be identical to equivalent `vec_c()` call
  expect_identical(list_unchop(x, indices = indices), vec_c(!!!x))
})

test_that("df-col row names are repaired silently", {
  df1 <- data_frame(x = new_data_frame(list(a = 1), row.names = "inner"))
  df2 <- data_frame(x = new_data_frame(list(a = 2), row.names = "inner"))

  x <- list(df1, df2)
  indices <- list(1, 2)

  expect_silent({
    result <- list_unchop(x, indices = indices)
  })

  expect_identical(vec_names(result$x), c("inner...1", "inner...2"))
})

test_that("monitoring - can technically assign to the same location twice", {
  indices <- list(1:2, 1L)

  x <- list(1:2, 3L)
  expect_identical(
    list_unchop(x, indices = indices),
    c(3L, 2L, NA)
  )

  # Homogeneous fallback
  x <- list(foobar(1:2), foobar(3L))
  expect_identical(
    list_unchop(x, indices = indices),
    foobar(c(3L, 2L, NA))
  )

  # `c()` fallback
  with_c_foobar({
    x <- list(foobar(1:2), foobar(3L))
    expect_identical(
      list_unchop(x, indices = indices),
      foobar_c(c(3L, 2L, NA))
    )
  })
})

test_that("index values are validated", {
  x <- list(1, 2)
  indices1 <- list(4, 1)
  indices2 <- list(c(1, 4), 2)
  indices3 <- list(c(1, 3, 4), 2)

  expect_error(
    list_unchop(x, indices = indices1),
    class = "vctrs_error_subscript_oob"
  )
  expect_error(
    list_unchop(x, indices = indices2),
    class = "vctrs_error_subscript_oob"
  )

  expect_identical(list_unchop(x, indices = indices3), c(1, 2, 1, 1))
})

test_that("name repair is respected and happens after ordering according to `indices`", {
  local_name_repair_quiet()

  x <- list(c(a = 1), c(a = 2))
  indices <- list(2, 1)

  expect_named(list_unchop(x, indices = indices), c("a", "a"))
  expect_named(
    list_unchop(x, indices = indices, name_repair = "unique"),
    c("a...1", "a...2")
  )
})

test_that("list_unchop() can repair names quietly", {
  local_name_repair_verbose()

  x <- c(x = "a", x = "b", x = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_unchop(
      vec_chop(x, indices = indices),
      indices = indices,
      name_repair = "unique_quiet"
    )
  })
  expect_named(res, c("x...1", "x...2", "x...3"))

  x <- c("if" = "a", "in" = "b", "for" = "c")
  indices <- list(2, c(3, 1))
  expect_snapshot({
    res <- list_unchop(
      vec_chop(x, indices = indices),
      indices = indices,
      name_repair = "universal_quiet"
    )
  })
  expect_named(res, c(".if", ".in", ".for"))
})

test_that("list_unchop() errors on unsupported location values", {
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_unchop(list(1, 2), indices = list(c(1, 2), 0))
  })
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_unchop(list(1), indices = list(-1))
  })

  # Homogeneous fallback
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), 0))
  })
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    list_unchop(list(foobar(1)), indices = list(-1))
  })

  # `c()` fallback
  with_c_foobar({
    expect_snapshot(error = TRUE, cnd_class = TRUE, {
      list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), 0))
    })
    expect_snapshot(error = TRUE, cnd_class = TRUE, {
      list_unchop(list(foobar(1)), indices = list(-1))
    })
  })
})

test_that("missing values propagate", {
  indices <- list(c(NA_integer_, NA_integer_), c(NA_integer_, 3))

  expect_identical(
    list_unchop(
      list(1, 2),
      indices = indices
    ),
    c(NA, NA, 2, NA)
  )

  # Homogenous fallback
  expect_identical(
    list_unchop(
      list(foobar(1), foobar(2)),
      indices = indices
    ),
    foobar(c(NA, NA, 2, NA))
  )

  # `c()` fallback
  with_c_foobar({
    expect_identical(
      list_unchop(
        list(foobar(1), foobar(2)),
        indices = indices
      ),
      foobar_c(c(NA, NA, 2, NA))
    )
  })
})

test_that("list_unchop() works with simple homogeneous foreign S3 classes", {
  values <- list(
    foobar(1:2),
    foobar(3:4)
  )
  indices <- list(c(1, 3), c(2, 4))
  expect_identical(
    list_unchop(values, indices = indices),
    foobar(int(1, 3, 2, 4))
  )

  # And in data frame columns (#1975)
  values <- list(
    data_frame(x = foobar(1:2)),
    data_frame(x = foobar(3:4))
  )
  indices <- list(c(1, 3), c(2, 4))
  expect_identical(
    list_unchop(values, indices = indices),
    data_frame(x = foobar(int(1, 3, 2, 4)))
  )
})

test_that("list_unchop() fails with complex foreign S3 classes", {
  expect_snapshot({
    x <- structure(foobar(1), attr_foo = "foo")
    y <- structure(foobar(2), attr_bar = "bar")
    (expect_error(
      list_unchop(list(x, y)),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      list_unchop(list(x, y), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_unchop() fails with complex foreign S4 classes", {
  expect_snapshot({
    joe <- .Counts(c(1L, 2L), name = "Joe")
    jane <- .Counts(3L, name = "Jane")
    (expect_error(
      list_unchop(list(joe, jane)),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      list_unchop(list(joe, jane), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("list_unchop() falls back to c() if S3 method is available", {
  # Check off-by-one error
  expect_error(
    list_unchop(list(foobar(1), "", foobar(2)), indices = list(1, 2, 3)),
    class = "vctrs_error_incompatible_type"
  )

  # Fallback when the class implements `c()`
  local_c_foobar()

  expect_identical(
    list_unchop(list(foobar(1), foobar(2))),
    foobar_c(c(1, 2))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(1, 2)),
    foobar_c(c(1, 2))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(2, 1)),
    foobar_c(c(2, 1))
  )
  expect_identical(
    list_unchop(list(NULL, foobar(1), NULL, foobar(2))),
    foobar_c(c(1, 2))
  )

  # OOB error is respected
  expect_error(
    list_unchop(list(foobar(1), foobar(2)), indices = list(1, 3)),
    class = "vctrs_error_subscript_oob"
  )

  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_unchop(list(foobar(c(1, 2)), foobar(3)), indices = list(c(1, 3), 1)),
    foobar_c(c(3, NA, 2))
  )
  expect_identical(
    list_unchop(list(foobar(c(1, 2)), foobar(3)), indices = list(c(2, NA), NA)),
    foobar_c(c(NA, 1, NA))
  )

  # Names are kept
  expect_identical(
    list_unchop(
      list(foobar(c(x = 1, y = 2)), foobar(c(x = 1))),
      indices = list(c(2, 1), 3)
    ),
    foobar_c(c(y = 2, x = 1, x = 1))
  )

  # Recycles to the size of index
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 3), 2)),
    foobar_c(c(1, 2, 1))
  )
  expect_identical(
    list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), integer())),
    foobar_c(c(1, 1))
  )
  expect_snapshot({
    (expect_error(
      list_unchop(
        list(foobar(1), foobar(2)),
        indices = list(c(1, 3), integer())
      ),
      class = "vctrs_error_subscript_oob"
    ))
  })
  expect_snapshot({
    x <- list(foobar(1:2))
    indices <- list(1:3)
    (expect_error(list_unchop(x, indices = indices)))
    (expect_error(list_unchop(
      x,
      indices = indices,
      error_arg = "arg",
      error_call = call("foo")
    )))
  })

  method_vctrs_c_fallback <- function(...) {
    xs <- list(...)
    xs <- map(xs, unclass)
    res <- exec("c", !!!xs)
    structure(res, class = "vctrs_c_fallback")
  }

  # Registered fallback
  s3_register("base::c", "vctrs_c_fallback", method_vctrs_c_fallback)
  expect_identical(
    list_unchop(
      list(
        structure(1, class = "vctrs_c_fallback"),
        structure(2, class = "vctrs_c_fallback")
      ),
      indices = list(2, 1)
    ),
    structure(c(2, 1), class = "vctrs_c_fallback")
  )

  # Don't fallback for S3 lists which are treated as scalars by default
  expect_error(
    list_unchop(list(foobar(list(1)), foobar(list(2)))),
    class = "vctrs_error_scalar_type"
  )
})

test_that("list_unchop() falls back even when ptype is supplied", {
  expect_foobar(
    list_unchop(list(foobar(1), foobar(2)), ptype = foobar(dbl()))
  )

  with_c_quux <- function(expr) {
    with_methods(expr, c.vctrs_foobar = function(...) quux(NextMethod()))
  }

  with_c_quux({
    expect_quux(
      list_unchop(
        list(foobar(1), foobar(2)),
        indices = list(1, 2),
        ptype = foobar(dbl())
      )
    )
  })

  with_c_quux({
    expect_quux(
      list_unchop(
        list(foobar(1, foo = TRUE), foobar(2, bar = TRUE)),
        indices = list(1, 2),
        ptype = foobar(dbl())
      )
    )
  })
})

test_that("list_unchop() falls back for S4 classes with a registered c() method", {
  joe <- .Counts(c(1L, 2L), name = "Joe")
  jane <- .Counts(3L, name = "Jane")

  expect_snapshot({
    (expect_error(
      list_unchop(list(joe, 1, jane), indices = list(c(1, 2), 3, 4)),
      class = "vctrs_error_incompatible_type"
    ))
  })

  local_c_counts()

  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(1, 3), 2)),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )

  expect_identical(
    list_unchop(list(NULL, joe, jane), indices = list(integer(), c(1, 3), 2)),
    .Counts(c(1L, 3L, 2L), name = "Dispatched")
  )

  # Unassigned locations results in missing values.
  # Repeated assignment uses the last assigned value.
  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(1, 3), 1)),
    .Counts(c(3L, NA, 2L), name = "Dispatched")
  )
  expect_identical(
    list_unchop(list(joe, jane), indices = list(c(2, NA), NA)),
    .Counts(c(NA, 1L, NA), name = "Dispatched")
  )
})

test_that("list_unchop() fallback doesn't support (most) `name_spec` or `ptype`", {
  local_c_foobar()

  foo <- structure(foobar(1), foo = "foo")
  bar <- structure(foobar(2), bar = "bar")

  expect_snapshot(error = TRUE, {
    list_unchop(
      list(foo, bar),
      indices = list(1, 2),
      name_spec = "{outer}_{inner}"
    )
  })
  expect_snapshot(error = TRUE, {
    list_unchop(
      list(foo, bar),
      indices = list(1, 2),
      name_spec = "{outer}_{inner}",
      error_call = call("foo")
    )
  })

  # Used to be an error about `ptype`
  x <- list(foobar(1))
  expect_snapshot(error = TRUE, {
    list_unchop(x, indices = list(1), ptype = "")
  })
})

test_that("list_unchop() fallback does support `name_spec = 'inner'`", {
  # Because of how useful it is, and how easy it is to implement!
  expect_identical(
    with_c_foobar(list_unchop(
      list(foobar(1), foobar(2)),
      indices = list(1, 2),
      name_spec = "inner"
    )),
    foobar_c(c(1, 2))
  )
  expect_identical(
    with_c_foobar(list_unchop(
      list(x = foobar(1), y = foobar(2)),
      indices = list(1, 2),
      name_spec = "inner"
    )),
    foobar_c(c(1, 2))
  )
  expect_identical(
    with_c_foobar(list_unchop(
      list(
        x = foobar(c(a = 1)),
        y = foobar(c(b = 2)),
        z = foobar(3)
      ),
      indices = list(1, 2, 3),
      name_spec = "inner"
    )),
    foobar_c(c(a = 1, b = 2, 3))
  )
})

test_that("list_unchop() supports numeric S3 indices", {
  local_methods(
    vec_ptype2.vctrs_foobar = function(x, y, ...) {
      UseMethod("vec_ptype2.vctrs_foobar")
    },
    vec_ptype2.vctrs_foobar.integer = function(x, y, ...) foobar(integer()),
    vec_cast.integer.vctrs_foobar = function(x, to, ...) vec_data(x)
  )

  expect_identical(list_unchop(list(1), indices = list(foobar(1L))), 1)
})

test_that("list_unchop() does not support non-numeric S3 indices", {
  expect_snapshot({
    (expect_error(
      list_unchop(list(1), indices = list(factor("x"))),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      list_unchop(list(1), indices = list(foobar(1L))),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can ignore names in `list_unchop()` by providing a `zap()` name-spec (#232)", {
  expect_snapshot({
    (expect_error(list_unchop(list(a = c(b = 1:2)))))
    (expect_error(list_unchop(list(a = c(b = 1:2)), error_call = call("foo"))))
  })

  expect_identical(
    list_unchop(list(a = c(b = 1:2), b = 3L), name_spec = zap()),
    1:3
  )
  expect_identical(
    list_unchop(
      list(a = c(foo = 1:2), b = c(bar = 3L)),
      indices = list(2:1, 3),
      name_spec = zap()
    ),
    c(2L, 1L, 3L)
  )

  expect_snapshot({
    x <- list(a = c(b = letters), b = 3L)
    (expect_error(
      list_unchop(x, name_spec = zap()),
      class = "vctrs_error_incompatible_type"
    ))

    x <- list(a = c(foo = 1:2), b = c(bar = ""))
    (expect_error(
      list_unchop(
        x,
        indices = list(2:1, 3),
        name_spec = zap()
      ),
      class = "vctrs_error_incompatible_type"
    ))
  })
})

test_that("can ignore outer names in `list_unchop()` by providing a 'inner' name-spec (#1988)", {
  expect_identical(
    list_unchop(
      list(x = c(a = 1, 2), y = c(3, b = 4)),
      indices = list(c(3, 1), c(2, 4)),
      name_spec = "inner"
    ),
    c(2, 3, a = 1, b = 4)
  )

  # Importantly, outer names are still used in error messages!
  expect_snapshot(error = TRUE, {
    list_unchop(
      list(x = c(a = 1), y = c(b = "2")),
      indices = list(1, 2),
      name_spec = "inner"
    )
  })
})

test_that("list_unchop() falls back to c() methods (#1120)", {
  expect_error(
    list_unchop(list(foobar(1), foobar(2, class = "foo"))),
    class = "vctrs_error_incompatible_type"
  )

  local_methods(
    c.vctrs_foobar = function(...) {
      out <- NextMethod()
      paste0(rep_along(out, "dispatched"), seq_along(out))
    }
  )

  # Homogeneous subclasses
  xs <- list(foobar(1), foobar(2, class = "foo"))

  expect_identical(
    list_unchop(xs),
    c("dispatched1", "dispatched2")
  )
  expect_identical(
    list_unchop(xs, indices = list(2, 1)),
    c("dispatched2", "dispatched1")
  )

  # Different subclasses
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )

  expect_identical(
    list_unchop(xs),
    c("dispatched1", "dispatched2", "dispatched3")
  )
  expect_identical(
    list_unchop(xs, indices = list(c(2, 1), 3)),
    c("dispatched2", "dispatched1", "dispatched3")
  )
})

test_that("list_unchop() fails if foreign classes are not homogeneous and there is no c() method", {
  xs <- list(
    foobar(c(x = 1, y = 2), class = "foo"),
    foobar(c(x = 1), foo = 1)
  )
  expect_error(
    list_unchop(xs),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    list_unchop(xs, indices = list(c(2, 1), 3)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("calls cast method even with empty objects", {
  # https://github.com/paleolimbot/wk/issues/230

  # There is a common type, but the cast method is intended
  # to fail here for this test
  local_methods(
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
      x
    },
    vec_cast.vctrs_foobar.default = function(x, to, ...) {
      vec_default_cast(x, to)
    }
  )

  expect_snapshot(error = TRUE, {
    list_unchop(
      list(
        foobar(integer()),
        foobar(integer(), foo = "bar")
      ),
      indices = list(
        integer(),
        integer()
      )
    )
  })
})

test_that("Size 1 unspecified `NA` that isn't used doesn't error (#1989)", {
  # Works because we don't actually recycle `NA` to size 0 in the loop, we
  # just check that it can recycle. If we recycled to `logical()`, it would
  # no longer look unspecified and an error would be thrown instead.
  expect_identical(
    list_unchop(
      list("x", NA),
      indices = list(1L, integer())
    ),
    "x"
  )
  expect_identical(
    list_unchop(
      list("x", NA),
      indices = list(integer(), 1L)
    ),
    NA_character_
  )
})

test_that("list_unchop() and vec_c() are consistent-ish regarding `size` and empty inputs (#1980)", {
  x <- list()
  indices <- list()

  # These should be consistent and return `NULL` when no inputs are provided.
  # We treat this roughly equivalent to `unspecified(0)`.
  expect_identical(vec_c(), NULL)
  expect_identical(vec_c(), list_unchop(x, indices = indices))

  # These should be consistent and return `NULL` when no inputs are provided.
  # We treat this roughly equivalent to `unspecified(0)`.
  expect_identical(vec_c(NULL), NULL)
  expect_identical(
    vec_c(NULL),
    list_unchop(list(NULL), indices = list(integer()))
  )

  # This is ambiguous but we let this return `NULL` as well.
  # `list_combine()` doesn't have this ambiguity
  expect_null(list_unchop(list(NULL), indices = list(1:2)))
  expect_identical(
    list_combine(list(NULL), indices = list(1:2), size = 2),
    unspecified(2)
  )

  # These should be consistent and return size 0 `ptype`
  expect_identical(vec_c(.ptype = integer()), integer())
  expect_identical(
    vec_c(.ptype = integer()),
    list_unchop(x, indices = indices, ptype = integer())
  )
})

new_sclr <- function(..., class = character()) {
  fields <- list(...)
  stopifnot(has_unique_names(fields))

  structure(
    list(...),
    class = c(class, "vctrs_sclr")
  )
}

# Subsetting --------------------------------------------------------------

#' @export
`[[.vctrs_sclr` <- function(x, i, ...) {
  .Call(vctrs_list_get, x, i)
}

#' @export
`$.vctrs_sclr` <- function(x, i, ...) {
  .Call(vctrs_list_get, x, i)
}

#' @export
`[[<-.vctrs_sclr` <- function(x, i, value) {
  .Call(vctrs_list_set, x, i, value)
}

#' @export
`$<-.vctrs_sclr` <- function(x, i, value) {
  .Call(vctrs_list_set, x, i, value)
}

# Shared niceties ---------------------------------------------------------

#' @export
print.vctrs_sclr <- function(x, ...) {
  obj_print(x, ...)
  invisible(x)
}

#' @export
as.list.vctrs_sclr <- function(x, ...) {
  vec_set_attributes(x, list(names = names(x)))
}

#' @export
as.data.frame.vctrs_sclr <- function(
  x,
  row.names = NULL,
  optional = FALSE,
  ...,
  nm = paste(deparse(substitute(x), width.cutoff = 500L), collapse = " ")
) {
  force(nm)
  cols <- list(list(x))
  if (!optional) {
    names(cols) <- nm
  }

  new_data_frame(cols, n = 1L)
}

# Vector behaviours -------------------------------------------------------

#' @export
`[.vctrs_sclr` <- function(x, ...) {
  stop_unsupported(x, "[")
}

#' @export
`[<-.vctrs_sclr` <- function(x, ..., value) {
  stop_unsupported(x, "[<-")
}

#' @export
c.vctrs_sclr <- function(...) {
  stop_unsupported(..1, "c")
}

#' @export
Math.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, .Generic)
}

#' @export
Ops.vctrs_sclr <- function(e1, e2) {
  stop_unsupported(e1, .Generic)
}

#' @export
Complex.vctrs_sclr <- function(z) {
  stop_unsupported(z, .Generic)
}

#' @export
Summary.vctrs_sclr <- function(..., na.rm = TRUE) {
  stop_unsupported(..1, .Generic)
}

#' @export
`names<-.vctrs_sclr` <- function(x, value) {
  stop_unsupported(x, "names<-")
}

#' @export
xtfrm.vctrs_sclr <- function(x) {
  stop_unsupported(x, "xtfrm")
}

#' @export
`dim<-.vctrs_sclr` <- function(x, value) {
  stop_unsupported(x, "dim<-")
}

#' @export
`dimnames<-.vctrs_sclr` <- function(x, value) {
  stop_unsupported(x, "dimnames<-")
}

#' @export
levels.vctrs_sclr <- function(x) {
  stop_unsupported(x, "levels")
}

#' @export
`levels<-.vctrs_sclr` <- function(x, value) {
  stop_unsupported(x, "levels<-")
}

#' @export
`t.vctrs_sclr` <- function(x) {
  stop_unsupported(x, "t")
}

#' @export
`is.na<-.vctrs_sclr` <- function(x, value) {
  stop_unsupported(x, "is.na<-")
}

#' @export
unique.vctrs_sclr <- function(x, incomparables = FALSE, ...) {
  stop_unsupported(x, "unique")
}

#' @export
duplicated.vctrs_sclr <- function(x, incomparables = FALSE, ...) {
  stop_unsupported(x, "unique")
}

#' @export
anyDuplicated.vctrs_sclr <- function(x, incomparables = FALSE, ...) {
  stop_unsupported(x, "unique")
}

#' @export
as.logical.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, "as.logical")
}

#' @export
as.integer.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, "as.integer")
}

#' @export
as.double.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, "as.double")
}

#' @export
as.character.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, "as.character")
}

#' @export
as.Date.vctrs_sclr <- function(x, ...) {
  stop_unsupported(x, "as.Date")
}

#' @export
as.POSIXct.vctrs_sclr <- function(x, tz = "", ...) {
  stop_unsupported(x, "as.POSIXct")
}


# Unimplemented -----------------------------------------------------------

#' @export
summary.vctrs_sclr <- function(object, ...) {
  # nocov start
  stop_unimplemented(object, "summary")
  # nocov end
}

#' Retrieve and repair names
#'
#' @description
#'
#' `vec_as_names()` takes a character vector of names and repairs it
#' according to the `repair` argument. It is the r-lib and tidyverse
#' equivalent of [base::make.names()].
#'
#' vctrs deals with a few levels of name repair:
#'
#' * `minimal` names exist. The `names` attribute is not `NULL`. The
#'   name of an unnamed element is `""` and never `NA`. For instance,
#'   `vec_as_names()` always returns minimal names and data frames
#'   created by the tibble package have names that are, at least,
#'   `minimal`.
#'
#' * `unique` names are `minimal`, have no duplicates, and can be used
#'   where a variable name is expected. Empty names, `...`, and
#'   `..` followed by a sequence of digits are banned.
#'
#'   - All columns can be accessed by name via `df[["name"]]` and
#'     ``df$`name` `` and ``with(df, `name`)``.
#'
#' * `universal` names are `unique` and syntactic (see Details for
#'   more).
#'
#'   - Names work everywhere, without quoting: `df$name` and `with(df,
#'     name)` and `lm(name1 ~ name2, data = df)` and
#'     `dplyr::select(df, name)` all work.
#'
#' `universal` implies `unique`, `unique` implies `minimal`. These
#' levels are nested.
#'
#'
#' @inheritParams rlang::args_error_context
#' @inheritParams rlang::args_dots_empty
#'
#' @param names A character vector.

#' @param repair Either a string or a function. If a string, it must be one of
#'   `"check_unique"`, `"minimal"`, `"unique"`, `"universal"`, `"unique_quiet"`,
#'   or `"universal_quiet"`. If a function, it is invoked with a vector of
#'   minimal names and must return minimal names, otherwise an error is thrown.
#'
#'   * Minimal names are never `NULL` or `NA`. When an element doesn't
#'     have a name, its minimal name is an empty string.
#'
#'   * Unique names are unique. A suffix is appended to duplicate
#'     names to make them unique.
#'
#'   * Universal names are unique and syntactic, meaning that you can
#'     safely use the names as variables without causing a syntax
#'     error.
#'
#'   The `"check_unique"` option doesn't perform any name repair.
#'   Instead, an error is raised if the names don't suit the
#'   `"unique"` criteria.
#'
#'   The options `"unique_quiet"` and `"universal_quiet"` are here to help the
#'   user who calls this function indirectly, via another function which exposes
#'   `repair` but not `quiet`. Specifying `repair = "unique_quiet"` is like
#'   specifying `repair = "unique", quiet = TRUE`. When the `"*_quiet"` options
#'   are used, any setting of `quiet` is silently overridden.
#' @param repair_arg If specified and `repair = "check_unique"`, any errors
#'   will include a hint to set the `repair_arg`.
#' @param quiet By default, the user is informed of any renaming
#'   caused by repairing the names. This only concerns unique and
#'   universal repairing. Set `quiet` to `TRUE` to silence the
#'   messages.
#'
#'   Users can silence the name repair messages by setting the
#'   `"rlib_name_repair_verbosity"` global option to `"quiet"`.
#'
#' @section `minimal` names:
#'
#' `minimal` names exist. The `names` attribute is not `NULL`. The
#' name of an unnamed element is `""` and never `NA`.
#'
#' Examples:
#'
#' ```
#' Original names of a vector with length 3: NULL
#'                            minimal names: "" "" ""
#'
#'                           Original names: "x" NA
#'                            minimal names: "x" ""
#' ```
#'
#'
#' @section `unique` names:
#'
#' `unique` names are `minimal`, have no duplicates, and can be used
#'  (possibly with backticks) in contexts where a variable is
#'  expected. Empty names, `...`, and `..` followed by a sequence of
#'  digits are banned. If a data frame has `unique` names, you can
#'  index it by name, and also access the columns by name. In
#'  particular, `df[["name"]]` and `` df$`name` `` and also ``with(df,
#'  `name`)`` always work.
#'
#' There are many ways to make names `unique`. We append a suffix of the form
#' `...j` to any name that is `""` or a duplicate, where `j` is the position.
#' We also change `..#` and `...` to `...#`.
#'
#' Example:
#'
#' ```
#' Original names:     ""     "x"     "" "y"     "x"  "..2"  "..."
#'   unique names: "...1" "x...2" "...3" "y" "x...5" "...6" "...7"
#' ```
#'
#' Pre-existing suffixes of the form `...j` are always stripped, prior
#' to making names `unique`, i.e. reconstructing the suffixes. If this
#' interacts poorly with your names, you should take control of name
#' repair.
#'
#'
#' @section `universal` names:
#'
#' `universal` names are `unique` and syntactic, meaning they:
#'
#'   * Are never empty (inherited from `unique`).
#'   * Have no duplicates (inherited from `unique`).
#'   * Are not `...`. Do not have the form `..i`, where `i` is a
#'     number (inherited from `unique`).
#'   * Consist of letters, numbers, and the dot `.` or underscore `_`
#'     characters.
#'   * Start with a letter or start with the dot `.` not followed by a
#'     number.
#'   * Are not a [reserved] word, e.g., `if` or `function` or `TRUE`.
#'
#' If a vector has `universal` names, variable names can be used
#' "as is" in code. They work well with nonstandard evaluation, e.g.,
#' `df$name` works.
#'
#' vctrs has a different method of making names syntactic than
#' [base::make.names()]. In general, vctrs prepends one or more dots
#' `.` until the name is syntactic.
#'
#' Examples:
#'
#' ```
#'  Original names:     ""     "x"    NA      "x"
#' universal names: "...1" "x...2" "...3" "x...4"
#'
#'   Original names: "(y)"  "_z"  ".2fa"  "FALSE"
#'  universal names: ".y." "._z" "..2fa" ".FALSE"
#' ```
#'
#' @seealso [rlang::names2()] returns the names of an object, after
#'   making them `minimal`.
#' @examples
#' # By default, `vec_as_names()` returns minimal names:
#' vec_as_names(c(NA, NA, "foo"))
#'
#' # You can make them unique:
#' vec_as_names(c(NA, NA, "foo"), repair = "unique")
#'
#' # Universal repairing fixes any non-syntactic name:
#' vec_as_names(c("_foo", "+"), repair = "universal")
#' @export
vec_as_names <- function(
  names,
  ...,
  repair = c(
    "minimal",
    "unique",
    "universal",
    "check_unique",
    "unique_quiet",
    "universal_quiet"
  ),
  repair_arg = NULL,
  quiet = FALSE,
  call = caller_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_as_names,
    names,
    repair,
    quiet,
    environment()
  )
}

# TODO! Error calls
validate_name_repair_arg <- function(repair) {
  .Call(vctrs_validate_name_repair_arg, repair)
}
validate_minimal_names <- function(names, n = NULL) {
  .Call(vctrs_validate_minimal_names, names, n)
}
validate_unique <- function(names, arg = "", n = NULL, call = caller_env()) {
  validate_minimal_names(names, n)

  empty_names <- detect_empty_names(names)
  if (has_length(empty_names)) {
    stop_names_cannot_be_empty(names, call = call)
  }

  dot_dot_name <- detect_dot_dot(names)
  if (has_length(dot_dot_name)) {
    stop_names_cannot_be_dot_dot(names, call = call)
  }

  if (anyDuplicated(names)) {
    stop_names_must_be_unique(names, arg, call = call)
  }

  invisible(names)
}
detect_empty_names <- function(names) {
  which(names == "")
}
detect_dot_dot <- function(names) {
  grep("^[.][.](?:[.]|[1-9][0-9]*)$", names)
}

#' Get or set the names of a vector
#'
#' @description
#' These functions work like [rlang::names2()], [names()] and [names<-()],
#' except that they return or modify the the rowwise names of the vector. These are:
#' * The usual `names()` for atomic vectors and lists
#' * The row names for data frames and matrices
#' * The names of the first dimension for arrays
#' Rowwise names are size consistent: the length of the names always equals
#' [vec_size()].
#'
#' `vec_names2()` returns the repaired names from a vector, even if it is unnamed.
#' See [vec_as_names()] for details on name repair.
#'
#' `vec_names()` is a bare-bones version that returns `NULL` if the vector is
#' unnamed.
#'
#' `vec_set_names()` sets the names or removes them.
#'
#' @param x A vector with names
#' @param names A character vector, or `NULL`.
#' @inheritParams vec_as_names
#'
#' @return
#'   `vec_names2()` returns the names of `x`, repaired.
#'   `vec_names()` returns the names of `x` or `NULL` if unnamed.
#'   `vec_set_names()` returns `x` with names updated.
#'
#' @name vec_names
#' @export
#' @examples
#' vec_names2(1:3)
#' vec_names2(1:3, repair = "unique")
#' vec_names2(c(a = 1, b = 2))
#'
#' # `vec_names()` consistently returns the rowwise names of data frames and arrays:
#' vec_names(data.frame(a = 1, b = 2))
#' names(data.frame(a = 1, b = 2))
#' vec_names(mtcars)
#' names(mtcars)
#' vec_names(Titanic)
#' names(Titanic)
#'
#' vec_set_names(1:3, letters[1:3])
#' vec_set_names(data.frame(a = 1:3), letters[1:3])
vec_names2 <- function(
  x,
  ...,
  repair = c(
    "minimal",
    "unique",
    "universal",
    "check_unique",
    "unique_quiet",
    "universal_quiet"
  ),
  quiet = FALSE
) {
  check_dots_empty0(...)
  repair <- validate_name_repair_arg(repair)

  if (is_function(repair)) {
    names <- minimal_names(x)
    new_names <- validate_minimal_names(repair(names), n = length(names))

    if (!quiet) {
      describe_repair(names, new_names)
    }

    return(new_names)
  }

  switch(
    repair,
    minimal = minimal_names(x),
    unique = unique_names(x, quiet = quiet),
    universal = as_universal_names(minimal_names(x), quiet = quiet),
    check_unique = validate_unique(minimal_names(x)),
    unique_quiet = unique_names(x, quiet = TRUE),
    universal_quiet = as_universal_names(minimal_names(x), quiet = TRUE)
  )
}
vec_repair_names <- function(
  x,
  repair = c(
    "minimal",
    "unique",
    "universal",
    "check_unique",
    "unique_quiet",
    "universal_quiet"
  ),
  ...,
  quiet = FALSE
) {
  if (is.data.frame(x)) {
    x
  } else {
    vec_set_names(x, vec_names2(x, ..., repair = repair, quiet = quiet))
  }
}

minimal_names <- function(x) {
  .Call(ffi_minimal_names, x)
}
unique_names <- function(x, quiet = FALSE) {
  .Call(ffi_unique_names, x, quiet)
}

#' @rdname vec_names
#' @export
vec_names <- function(x) {
  .Call(vctrs_names, x)
}

as_minimal_names <- function(names) {
  .Call(ffi_as_minimal_names, names)
}
as_unique_names <- function(names, quiet = FALSE) {
  .Call(vctrs_as_unique_names, names, quiet)
}
as_universal_names <- function(names, quiet = FALSE) {
  new_names <- names
  new_names[] <- ""

  naked_names <- strip_pos(two_to_three_dots(names))
  empty <- naked_names %in% c("", "...")

  new_names[!empty] <- make_syntactic(naked_names[!empty])

  needs_suffix <- empty | vec_duplicate_detect(new_names)
  new_names <- append_pos(new_names, needs_suffix = needs_suffix)

  if (!quiet) {
    describe_repair(names, new_names)
  }

  new_names
}

two_to_three_dots <- function(names) {
  sub("(^[.][.][1-9][0-9]*$)", ".\\1", names)
}
append_pos <- function(names, needs_suffix) {
  need_append_pos <- which(needs_suffix)
  names[need_append_pos] <- paste0(
    names[need_append_pos],
    "...",
    need_append_pos
  )
  names
}
strip_pos <- function(names) {
  rx <- "([.][.][.][1-9][0-9]*)+$"
  gsub(rx, "", names) %|% ""
}

# Makes each individual name syntactic but does not enforce unique-ness
make_syntactic <- function(names) {
  names[is.na(names)] <- ""
  names[names == ""] <- "."
  names[names == "..."] <- "...."
  names <- sub("^_", "._", names)

  new_names <- make.names(names)

  X_prefix <- grepl("^X", new_names) & !grepl("^X", names)
  new_names[X_prefix] <- sub("^X", "", new_names[X_prefix])

  dot_suffix <- which(new_names == paste0(names, "."))
  new_names[dot_suffix] <- sub("^(.*)[.]$", ".\\1", new_names[dot_suffix])
  # Illegal characters have been replaced with '.' via make.names()
  # however, we have:
  #   * Declined its addition of 'X' prefixes.
  #   * Turned its '.' suffixes to '.' prefixes.

  regex <- paste0(
    "^(?<leading_dots>[.]{0,2})",
    "(?<numbers>[0-9]*)",
    "(?<leftovers>[^0-9]?.*$)"
  )

  re <- re_match(new_names, pattern = regex)
  needs_dots <- which(re$numbers != "")
  needs_third_dot <- (re$leftovers[needs_dots] == "")
  re$leading_dots[needs_dots] <- ifelse(needs_third_dot, "...", "..")
  new_names <- paste0(re$leading_dots, re$numbers, re$leftovers)

  new_names
}

# From rematch2, except we don't add tbl_df or tbl classes to the return value
re_match <- function(text, pattern, perl = TRUE, ...) {
  stopifnot(
    is.character(pattern),
    length(pattern) == 1,
    !is.na(pattern)
  )
  text <- as.character(text)

  match <- regexpr(pattern, text, perl = perl, ...)

  start <- as.vector(match)
  length <- attr(match, "match.length")
  end <- start + length - 1L

  matchstr <- substring(text, start, end)
  matchstr[start == -1] <- NA_character_

  res <- data.frame(
    stringsAsFactors = FALSE,
    .text = text,
    .match = matchstr
  )

  if (!is.null(attr(match, "capture.start"))) {
    gstart <- attr(match, "capture.start")
    glength <- attr(match, "capture.length")
    gend <- gstart + glength - 1L

    groupstr <- substring(text, gstart, gend)
    groupstr[gstart == -1] <- NA_character_
    dim(groupstr) <- dim(gstart)

    res <- cbind(groupstr, res, stringsAsFactors = FALSE)
  }

  names(res) <- c(attr(match, "capture.names"), ".text", ".match")
  res
}


describe_repair <- function(orig_names, names) {
  names_inform_repair(orig_names, names)
}

bullets <- function(..., header = NULL) {
  problems <- c(...)
  MAX_BULLETS <- 6L
  if (length(problems) >= MAX_BULLETS) {
    n_more <- length(problems) - MAX_BULLETS + 1L
    problems[[MAX_BULLETS]] <- "..."
    length(problems) <- MAX_BULLETS
  }

  info <- paste0("* ", problems, collapse = "\n")

  if (!is.null(header)) {
    info <- paste0(header, "\n", info)
  }

  info
}

# Used in names.c
set_rownames_dispatch <- function(x, names) {
  rownames(x) <- names
  x
}

# Used in names.c
set_names_dispatch <- function(x, names) {
  names(x) <- names
  x
}

#' @rdname vec_names
#' @export
vec_set_names <- function(x, names) {
  .Call(ffi_vec_set_names, x, names)
}

#' Repair names with legacy method
#'
#' This standardises names with the legacy approach that was used in
#' tidyverse packages (such as tibble, tidyr, and readxl) before
#' [vec_as_names()] was implemented. This tool is meant to help
#' transitioning to the new name repairing standard and will be
#' deprecated and removed from the package some time in the future.
#'
#' @inheritParams vec_as_names
#' @param prefix,sep Prefix and separator for repaired names.
#'
#' @examples
#' if (rlang::is_installed("tibble")) {
#'
#' library(tibble)
#'
#' # Names repair is turned off by default in tibble:
#' try(tibble(a = 1, a = 2))
#'
#' # You can turn it on by supplying a repair method:
#' tibble(a = 1, a = 2, .name_repair = "universal")
#'
#' # If you prefer the legacy method, use `vec_as_names_legacy()`:
#' tibble(a = 1, a = 2, .name_repair = vec_as_names_legacy)
#'
#' }
#' @keywords internal
#' @export
vec_as_names_legacy <- function(names, prefix = "V", sep = "") {
  if (length(names) == 0) {
    return(character())
  }

  blank <- names == ""
  names[!blank] <- make.unique(names[!blank], sep = sep)

  new_nms <- setdiff(paste(prefix, seq_along(names), sep = sep), names)
  names[blank] <- new_nms[seq_len(sum(blank))]

  names
}


#' Name specifications
#'
#' @description
#'
#' A name specification describes how to combine an inner and outer
#' names. This sort of name combination arises when concatenating
#' vectors or flattening lists. There are two possible cases:
#'
#' * Named vector:
#'
#'   ```
#'   vec_c(outer = c(inner1 = 1, inner2 = 2))
#'   ```
#'
#' * Unnamed vector:
#'
#'   ```
#'   vec_c(outer = 1:2)
#'   ```
#'
#' In r-lib and tidyverse packages, these cases are errors by default,
#' because there's no behaviour that works well for every case.
#' Instead, you can provide a name specification that describes how to
#' combine the inner and outer names of inputs. Name specifications
#' can refer to:
#'
#' * `outer`: The external name recycled to the size of the input
#'   vector.
#'
#' * `inner`: Either the names of the input vector, or a sequence of
#'   integer from 1 to the size of the vector if it is unnamed.
#'
#' @param name_spec,.name_spec A name specification for combining
#'   inner and outer names. This is relevant for inputs passed with a
#'   name, when these inputs are themselves named, like `outer =
#'   c(inner = 1)`, or when they have length greater than 1: `outer =
#'   1:2`. By default, these cases trigger an error. You can resolve
#'   the error by providing a specification that describes how to
#'   combine the names or the indices of the inner vector with the
#'   name of the input. This specification can be:
#'
#'   * A function of two arguments. The outer name is passed as a
#'     string to the first argument, and the inner names or positions
#'     are passed as second argument.
#'
#'   * An anonymous function as a purrr-style formula.
#'
#'   * A glue specification of the form `"{outer}_{inner}"`.
#'
#'   * `"inner"`, in which case outer names are ignored, and inner
#'     names are used if they exist. Note that outer names may still
#'     be used to provide informative error messages.
#'
#'   * An [rlang::zap()] object, in which case both outer and inner
#'     names are ignored and the result is unnamed.
#'
#'   See the [name specification topic][name_spec].
#'
#' @examples
#' # By default, named inputs must be length 1:
#' vec_c(name = 1)         # ok
#' try(vec_c(name = 1:3))  # bad
#'
#' # They also can't have internal names, even if scalar:
#' try(vec_c(name = c(internal = 1)))  # bad
#'
#' # Pass a name specification to work around this. A specification
#' # can be a glue string referring to `outer` and `inner`:
#' vec_c(name = 1:3, other = 4:5, .name_spec = "{outer}")
#' vec_c(name = 1:3, other = 4:5, .name_spec = "{outer}_{inner}")
#'
#' # They can also be functions:
#' my_spec <- function(outer, inner) paste(outer, inner, sep = "_")
#' vec_c(name = 1:3, other = 4:5, .name_spec = my_spec)
#'
#' # Or purrr-style formulas for anonymous functions:
#' vec_c(name = 1:3, other = 4:5, .name_spec = ~ paste0(.x, .y))
#'
#' # Or the string `"inner"` to only use inner names
#' vec_c(name = 1:3, outer = 4:5, .name_spec = "inner")
#' vec_c(name = c(a = 1, b = 2, c = 3), outer = 4:5, .name_spec = "inner")
#' # This can be useful when you want outer names mentioned in error messages,
#' # but you don't want them interfering with the result
#' try(vec_c(x = c(a = 1), y = c(b = "2"), .name_spec = "inner"))
#'
#' # Or `rlang::zap()` to ignore both outer and inner names entirely
#' vec_c(name = c(a = 1, b = 2), outer = c(c = 3), .name_spec = rlang::zap())
#' @name name_spec
NULL

apply_name_spec <- function(name_spec, outer, inner, n = length(inner)) {
  .Call(ffi_apply_name_spec, name_spec, outer, inner, n)
}

glue_as_name_spec <- function(`_spec`) {
  function(inner, outer) {
    glue::glue(`_spec`)
  }
}

# Evaluate glue specs in a child of base for now
environment(glue_as_name_spec) <- baseenv()

local_name_repair_quiet()

# vec_names() ---------------------------------------------------------

test_that("vec_names() retrieves names", {
  expect_null(vec_names(letters))
  expect_identical(vec_names(set_names(letters)), letters)

  expect_identical(vec_names(mtcars), row.names(mtcars))
  expect_null(vec_names(unrownames(mtcars)))

  expect_identical(vec_names(Titanic), dimnames(Titanic)[[1]])

  x <- matrix(1L, dimnames = list("row", "col"))
  expect_identical(vec_names(x), dimnames(x)[[1]])
})

test_that("vec_names() dispatches", {
  local_methods(
    names.vctrs_foobar = function(x) "dispatched!"
  )
  expect_identical(vec_names(foobar()), "dispatched!")
})


# vec_names2() -------------------------------------------------------------

test_that("vec_names2() repairs names", {
  expect_identical(vec_names2(1:2), c("", ""))
  expect_identical(vec_names2(1:2, repair = "unique"), c("...1", "...2"))
  expect_identical(vec_names2(set_names(1:2, c("_foo", "_bar")), repair = "universal"), c("._foo", "._bar"))
})

test_that("vec_names2() treats data frames and arrays as vectors", {
  expect_identical(vec_names2(mtcars), row.names(mtcars))
  expect_identical(vec_names2(as.matrix(mtcars)), row.names(mtcars))

  df <- unrownames(mtcars)
  exp <- rep_len("", nrow(mtcars))
  expect_identical(vec_names2(df), exp)
  expect_identical(vec_names2(as.matrix(df)), exp)
})

test_that("vec_names2() accepts and checks repair function", {
  expect_identical(vec_names2(1:2, repair = function(nms) rep_along(nms, "foo")), c("foo", "foo"))
  expect_error(vec_names2(1:2, repair = function(nms) "foo"), "length 1 instead of length 2")
})

test_that("vec_names2() repairs names before invoking repair function", {
  x <- set_names(1:2, c(NA, NA))
  expect_identical(vec_names2(x, repair = identity), c("", ""))
})

test_that("vec_names2() result is correct for *_quiet repair", {
  expect_identical(vec_names2(1:2, repair = "unique"), vec_names2(1:2, repair = "unique_quiet"))
  expect_identical(vec_names2(1:2, repair = "universal"), vec_names2(1:2, repair = "universal_quiet"))
})

# vec_as_names() -----------------------------------------------------------

test_that("vec_as_names() requires character vector", {
  expect_error(vec_as_names(NULL), "`names` must be a character vector")
})

test_that("vec_as_names() validates `repair`", {
  expect_snapshot({
    (expect_error(my_vec_as_names("x", my_repair = "foo"), "can't be \"foo\""))
    (expect_error(my_vec_as_names(1, my_repair = 1), "string or a function"))
  })
})

test_that("vec_as_names() repairs names", {
  expect_identical(vec_as_names(chr(NA, NA)), c("", ""))
  expect_identical(vec_as_names(chr(NA, NA), repair = "unique"), c("...1", "...2"))
  expect_identical(vec_as_names(chr("_foo", "_bar"), repair = "universal"), c("._foo", "._bar"))
  expect_identical(vec_as_names(chr("a", "b"), repair = "check_unique"), c("a", "b"))
})

test_that("vec_as_names() checks unique names", {
  expect_snapshot({
    (expect_error(my_vec_as_names(chr(NA), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr(""), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("a", "a"), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("..1"), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("..."), my_repair = "check_unique")))
  })
})

test_that("vec_as_names() result is correct for *_quiet repair", {
  expect_identical(
    vec_as_names(chr("_foo", "_bar"), repair = "unique"),
    vec_as_names(chr("_foo", "_bar"), repair = "unique_quiet")
  )
  expect_identical(
    vec_as_names(chr("_foo", "_bar"), repair = "universal"),
    vec_as_names(chr("_foo", "_bar"), repair = "universal_quiet")
  )
})

test_that("vec_as_names() keeps the names of a named vector", {
  x_unnamed <- c(NA, "", "..1", "...2")
  x_names <- letters[1:4]
  x <- set_names(x_unnamed, x_names)

  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "minimal"), x_names),
    vec_as_names(x, repair = "minimal")
  )
  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "unique"), x_names),
    vec_as_names(x, repair = "unique")
  )
  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "universal"), x_names),
    vec_as_names(x, repair = "universal")
  )
})

test_that("vec_as_names() accepts and checks repair function", {
  f <- local({
    local_obj <- "foo"
    ~ rep_along(.x, local_obj)
  })
  expect_identical(vec_as_names(c("", ""), repair = f), c("foo", "foo"))
  expect_snapshot(error = TRUE, my_vec_as_names(c("", ""), my_repair = function(nms) "foo"))
})

test_that("vec_as_names() repairs names before invoking repair function", {
  expect_identical(vec_as_names(chr(NA, NA), repair = identity), c("", ""))
})

test_that("vec_as_names() is noisy by default", {
  local_name_repair_verbose()

  expect_snapshot({
    # Noisy name repair
    vec_as_names(c("x", "x"), repair = "unique")

    # Quiet name repair
    vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)

    # Hint at repair argument, if known
    (expect_error(
      my_vec_as_names(c("x", "x"), my_repair = "check_unique")
    ))

    # request quiet via name repair string, don't specify `quiet`
    vec_as_names(c("1", "1"), repair = "unique_quiet")
    vec_as_names(c("1", "1"), repair = "universal_quiet")

    # request quiet via name repair string, specify `quiet` = TRUE
    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = TRUE)
    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = TRUE)

    # request quiet via name repair string, specify `quiet` = FALSE
    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = FALSE)
    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = FALSE)
  })
})

test_that("validate_minimal_names() checks names", {
  expect_snapshot({
    (expect_error(validate_minimal_names(1), "must return a character vector"))
    (expect_error(validate_minimal_names(NULL), "can't return `NULL`"))
    (expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values"))
  })
})

test_that("validate_unique() checks unique names", {
  expect_snapshot({
    (expect_error(validate_unique(chr(NA)), "`NA`"))
    (expect_error(validate_unique(chr("")), class = "vctrs_error_names_cannot_be_empty"))
    (expect_error(validate_unique(chr("a", "a")), class = "vctrs_error_names_must_be_unique"))
    (expect_error(validate_unique(chr("..1")), class = "vctrs_error_names_cannot_be_dot_dot"))
    (expect_error(validate_unique(chr("...")), class = "vctrs_error_names_cannot_be_dot_dot"))
  })
})

test_that("vec_as_names_validate() validates repair arguments", {
  expect_identical(
    validate_name_repair_arg(c("unique", "check_unique")),
    "unique"
  )
  expect_identical(
    validate_name_repair_arg(~ toupper(.))(letters),
    LETTERS
  )
})

test_that("vec_as_names() is quiet when function is supplied (#1018)", {
  expect_silent(
    vctrs::vec_as_names(
      c("a", "b"),
      repair = function(x) paste0(x, "a"),
      quiet = FALSE
    )
  )
})

test_that("vec_as_names() evaluates repair_arg lazily", {
  expect_silent(vec_as_names(letters, repair_arg = print("oof")))
})


# vec_repair_names() -------------------------------------------------------

test_that("vec_repair_names() repairs names", {
  expect_identical(vec_repair_names(1:2), set_names(1:2, c("", "")))
  expect_identical(vec_repair_names(1:2, "unique"), set_names(1:2, c("...1", "...2")))
  expect_identical(vec_repair_names(set_names(1:2, c("_foo", "_bar")), "universal"), set_names(1:2, c("._foo", "._bar")))
})

test_that("vec_repair_names() handles data frames and arrays", {
  df <- data.frame(x = 1:2)
  expect_identical(vec_repair_names(df), df)
  expect_identical(row.names(vec_repair_names(as.matrix(df))), c("", ""))
  expect_identical(row.names(vec_repair_names(as.matrix(df), "unique")), c("...1", "...2"))
})

# vec_set_names() -----------------------------------------------------------

test_that("vec_set_names() sets atomic names", {
  x <- 1:2
  names <- c("x1", "x2")
  exp <- set_names(x, names)
  expect_equal(vec_set_names(x, names), exp)
})

test_that("vec_set_names() sets matrix/array names", {
  x <- matrix(1:2)
  names <- c("x1", "x2")
  exp <- x
  rownames(exp) <- names
  expect_equal(vec_set_names(x, names), exp)

  y <- array(1:4, dim = c(2, 1, 2))
  exp <- y
  rownames(exp) <- names
  expect_equal(vec_set_names(y, names), exp)
})

test_that("vec_set_names() doesn't alter names", {
  x <- matrix(1, dimnames = list(rows = "a", cols = "x"))
  vec_set_names(x, "y")
  expect_equal(vec_names2(x), "a")
  expect_equal(colnames(x), "x")
  vec_set_names(x, NULL)
  expect_equal(vec_names2(x), "a")
  expect_equal(colnames(x), "x")

  y <- array(1:4, dim = c(1, 2, 2), dimnames = list(rows = "a", one = 1:2, two = 1:2))
  vec_set_names(y, "y")
  expect_equal(vec_names2(y), "a")
  vec_set_names(y, NULL)
  expect_equal(vec_names2(y), "a")
})

test_that("vec_set_names() sets row names on data frames", {
  expect_identical(
    vec_set_names(data_frame(x = 1), "foo"),
    new_data_frame(list(x = 1), row.names = "foo")
  )

  expect_identical(
    vec_set_names(data_frame(x = 1:2), c("foo", "foo")),
    new_data_frame(list(x = 1:2), row.names = c("foo...1", "foo...2"))
  )
})

test_that("vec_set_names() correctly sets names on POSIXlt objects", {
  x <- as.POSIXlt(new_datetime(0))
  exp <- set_names(x, "a")
  expect_equal(vec_set_names(x, "a"), exp)
})

test_that("vec_set_names() falls back to `names<-` with proxied objects", {
  x <- structure(1, class = "foobar")
  exp <- set_names(x, "a")

  expect_equal(vec_set_names(x, "a"), exp)

  local_methods(`names<-.foobar` = function(x, value) "fallback!")
  expect_equal(vec_set_names(x, "a"), "fallback!")
})

test_that("vec_set_names() falls back to `rownames<-` with shaped proxied objects", {
  x <- structure(1:2, dim = c(2L, 1L), class = "foobar")
  names <- c("r1", "r2")
  exp <- x
  rownames(exp) <- names

  expect_equal(vec_set_names(x, names), exp)

  # `rownames<-` is not generic, but eventually calls `dimnames<-` which is
  local_methods(`dimnames<-.foobar` = function(x, value) "fallback!")
  expect_equal(vec_set_names(x, names), "fallback!")
})

test_that("vec_set_names() can set NULL names", {
  x <- 1:2
  expect_equal(vec_set_names(x, NULL), x)

  x_named <- set_names(x)
  expect_equal(vec_set_names(x_named, NULL), x)

  x_mat <- as.matrix(x)
  expect_equal(vec_set_names(x_mat, NULL), x_mat)

  x_mat_named <- x_mat
  rownames(x_mat_named) <- c("1", "2")
  exp <- matrix(x_mat, dimnames = list(NULL, NULL))
  expect_equal(vec_set_names(x_mat_named, NULL), exp)
})

test_that("vec_set_names() errors with bad `names`", {
  expect_snapshot({
    (expect_error(vec_set_names(1, 1), "character vector, not a double"))
    (expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2"))
  })
})

test_that("vec_names() and vec_set_names() work with 1-dimensional arrays", {
  x <- array(1:2, dimnames = list(c("a", "b")))
  expect_identical(vec_names(x), c("a", "b"))
  expect_identical(vec_names(vec_set_names(x, c("A", "B"))), c("A", "B"))
})

# minimal names -------------------------------------------------------------

test_that("minimal names are made from `n` when `name = NULL`", {
  expect_identical(minimal_names(1:2), c("", ""))
})

test_that("as_minimal_names() checks input", {
  expect_error(as_minimal_names(1:3), "must be a character vector")
})

test_that("minimal names have '' instead of NAs", {
  expect_identical(as_minimal_names(c("", NA, "", NA)), c("", "", "", ""))
})

test_that("repairing minimal names copes with NULL input names", {
  x <- 1:3
  x_named <- vec_repair_names(x)
  expect_equal(names(x_named), rep("", 3))
})

test_that("as_minimal_names() is idempotent", {
  x <- c("", "", NA)
  expect_identical(as_minimal_names(x), as_minimal_names(as_minimal_names(x)))
})

test_that("minimal_names() treats data frames and arrays as vectors", {
  expect_identical(minimal_names(mtcars), row.names(mtcars))
  expect_identical(minimal_names(as.matrix(mtcars)), row.names(mtcars))

  df <- unrownames(mtcars)
  exp <- rep_len("", nrow(mtcars))
  expect_identical(minimal_names(df), exp)
  expect_identical(minimal_names(as.matrix(df)), exp)
})

test_that("as_minimal_names() copies on write", {
  nms <- chr(NA, NA)
  as_minimal_names(nms)
  expect_identical(nms, chr(NA, NA))

  nms <- c("a", "b")
  out <- as_minimal_names(nms)
  expect_true(is_reference(nms, out))
})


# unique names -------------------------------------------------------------

test_that("unique_names() handles unnamed vectors", {
  expect_identical(unique_names(1:3), c("...1", "...2", "...3"))
})

test_that("as_unique_names() is a no-op when no repairs are needed", {
  x <- c("x", "y")
  out <- as_unique_names(x)
  expect_true(is_reference(out, x))
  expect_identical(out, c("x", "y"))
})

test_that("as_unique_names() eliminates emptiness and duplication", {
  x <- c("", "x", "y", "x")
  expect_identical(as_unique_names(x), c("...1", "x...2", "y", "x...4"))
})

test_that("as_unique_names(): solo empty or NA gets suffix", {
  expect_identical(as_unique_names(""), "...1")
  expect_identical(as_unique_names(NA_character_), "...1")
})

test_that("as_unique_names() treats ellipsis like empty string", {
  expect_identical(as_unique_names("..."), as_unique_names(""))
})

test_that("two_three_dots() does its job and no more", {
  x <- c(".", ".1", "...1", "..1a")
  expect_identical(two_to_three_dots(x), x)

  expect_identical(two_to_three_dots(c("..1", "..22")), c("...1", "...22"))
})

test_that("two dots then number treated like three dots then number", {
  expect_identical(as_unique_names("..2"), as_unique_names("...5"))
})

test_that("as_unique_names() strips positional suffixes, re-applies as needed", {
  x <- c("...20", "a...1", "b", "", "a...2...34")
  expect_identical(as_unique_names(x), c("...1", "a...2", "b", "...4", "a...5"))

  expect_identical(as_unique_names("a...1"), "a")
  expect_identical(as_unique_names(c("a...2", "a")), c("a...1", "a...2"))
  expect_identical(as_unique_names(c("a...3", "a", "a")), c("a...1", "a...2", "a...3"))
  expect_identical(as_unique_names(c("a...2", "a", "a")), c("a...1", "a...2", "a...3"))
  expect_identical(as_unique_names(c("a...2", "a...2", "a...2")), c("a...1", "a...2", "a...3"))
})

test_that("as_unique_names() is idempotent", {
  x <- c("...20", "a...1", "b", "", "a...2")
  expect_identical(as_unique_names(!!x), as_unique_names(as_unique_names(!!x)))
})

test_that("unique-ification has an 'algebraic'-y property", {
  ## inspired by, but different from, this guarantee about base::make.unique()
  ## make.unique(c(A, B)) == make.unique(c(make.unique(A), B))
  ## If A is already unique, then make.unique(c(A, B)) preserves A.

  ## I haven't formulated what we guarantee very well yet, but it's probably
  ## implicit in this test (?)

  x <- c("...20", "a...1", "b", "", "a...2", "d")
  y <- c("", "a...3", "b", "...3", "e")

  ## fix names on each, catenate, fix the whole
  z1 <- as_unique_names(
    c(
      as_unique_names(x), as_unique_names(y)
    )
  )

  ## fix names on x, catenate, fix the whole
  z2 <- as_unique_names(
    c(
      as_unique_names(x), y
    )
  )

  ## fix names on y, catenate, fix the whole
  z3 <- as_unique_names(
    c(
      x, as_unique_names(y)
    )
  )

  ## catenate, fix the whole
  z4 <- as_unique_names(
    c(
      x, y
    )
  )

  expect_identical(z1, z2)
  expect_identical(z1, z3)
  expect_identical(z1, z4)
})

test_that("unique_names() and as_unique_names() are verbose or silent", {
  local_name_repair_verbose()

  expect_snapshot(unique_names(1:2))
  expect_snapshot(as_unique_names(c("", "")))

  expect_message(regexp = NA, unique_names(1:2, quiet = TRUE))
  expect_message(regexp = NA, as_unique_names(c("", ""), quiet = TRUE))
})

test_that("names with only duplicates are repaired", {
  expect_identical(unique_names(list(x = NA, x = NA)), c("x...1", "x...2"))
})


# Universal names ----------------------------------------------------------

test_that("zero-length input", {
  expect_equal(as_universal_names(character()), character())
})

test_that("universal names are not changed", {
  expect_equal(as_universal_names(letters), letters)
})

test_that("as_universal_names() is idempotent", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  expect_identical(as_universal_names(x), as_universal_names(as_universal_names(x)))
})

test_that("dupes get a suffix", {
  expect_equal(
    as_universal_names(c("a", "b", "a", "c", "b")),
    c("a...1", "b...2", "a...3", "c", "b...5")
  )
})

test_that("as_universal_names(): solo empty or NA gets suffix", {
  expect_identical(as_universal_names(""), "...1")
  expect_identical(as_universal_names(NA_character_), "...1")
})

test_that("as_universal_names() treats ellipsis like empty string", {
  expect_identical(as_universal_names("..."), as_universal_names(""))
})

test_that("solo dot is unchanged", {
  expect_equal(as_universal_names("."), ".")
})

test_that("dot, dot gets suffix", {
  expect_equal(as_universal_names(c(".", ".")), c("....1", "....2"))
})

test_that("dot-dot, dot-dot gets suffix", {
  expect_equal(as_universal_names(c("..", "..")), c(".....1", ".....2"))
})

test_that("empty, dot becomes suffix, dot", {
  expect_equal(as_universal_names(c("", ".")), c("...1", "."))
})

test_that("empty, empty, dot becomes suffix, suffix, dot", {
  expect_equal(as_universal_names(c("", "", ".")), c("...1", "...2", "."))
})

test_that("dot, dot, empty becomes suffix, suffix, suffix", {
  expect_equal(as_universal_names(c(".", ".", "")), c("....1", "....2", "...3"))
})

test_that("dot, empty, dot becomes suffix, suffix, suffix", {
  expect_equal(as_universal_names(c(".", "", ".")), c("....1", "...2", "....3"))
})

test_that("empty, dot, empty becomes suffix, dot, suffix", {
  expect_equal(as_universal_names(c("", ".", "")), c("...1", ".", "...3"))
})

test_that("'...j' gets stripped then names are modified", {
  expect_equal(as_universal_names(c("...6", "...1...2")), c("...1", "...2"))
  expect_equal(as_universal_names("if...2"), ".if")
})

test_that("complicated inputs", {
  expect_equal(
    as_universal_names(c("", ".", NA, "if...4", "if", "if...8", "for", "if){]1")),
    c("...1", ".", "...3", ".if...4", ".if...5", ".if...6", ".for", "if...1")
  )
})

test_that("message", {
  local_name_repair_verbose()
  expect_snapshot(as_universal_names(c("a b", "b c")))
})

test_that("quiet", {
  expect_message(
    as_universal_names("", quiet = TRUE),
    NA
  )
})

test_that("unique then universal is universal, with shuffling", {
  x <- c("", ".2", "..3", "...4", "....5", ".....6", "......7", "...")
  expect_identical(as_universal_names(as_unique_names(x)), as_universal_names(x))

  x2 <- x[c(7L, 4L, 3L, 6L, 5L, 1L, 2L, 8L)]
  expect_identical(as_universal_names(as_unique_names(x2)), as_universal_names(x2))

  x3 <- x[c(3L, 2L, 4L, 6L, 8L, 1L, 5L, 7L)]
  expect_identical(as_universal_names(as_unique_names(x3)), as_universal_names(x3))
})

test_that("zero-length inputs given character names", {
  out <- vec_repair_names(character(), "universal")
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  out <- vec_repair_names(1:3, "universal")
  expect_equal(names(out), c("...1", "...2", "...3"))
})

test_that("messages by default", {
  local_name_repair_verbose()
  expect_snapshot(vec_repair_names(set_names(1, "a:b"), "universal"))
  expect_snapshot(vec_repair_names(set_names(1, "a:b"), ~ make.names(.)))
})

test_that("quiet = TRUE", {
  expect_message(vec_repair_names(set_names(1, ""), "universal", quiet = TRUE), NA)
})

test_that("non-universal names", {
  out <- vec_repair_names(set_names(1, "a b"), "universal")
  expect_equal(names(out), "a.b")

  expect_equal(as_universal_names("a b"), "a.b")
})


# make_syntactic() ---------------------------------------------------------

test_that("make_syntactic(): empty or NA", {
  expect_syntactic(
      c("", NA_character_),
      c(".", ".")
  )
})

test_that("make_syntactic(): reserved words", {
  expect_syntactic(
    c("if", "TRUE", "Inf", "NA_real_", "normal"),
    c(".if", ".TRUE", ".Inf", ".NA_real_", "normal")
  )
})

test_that("make_syntactic(): underscore", {
  expect_syntactic(
    c( "_",  "_1",  "_a}"),
    c("._", "._1", "._a.")
  )
})

test_that("make_syntactic(): dots", {
  expect_syntactic(
    c(".", "..",  "...", "...."),
    c(".", "..", "....", "....")
  )
})

test_that("make_syntactic(): number", {
  expect_syntactic(
      c(   "0",    "1",    "22",    "333"),
      c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): number then character", {
  expect_syntactic(
    c(  "0a",   "1b",   "22c",   "333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): number then non-character", {
  expect_syntactic(
    c(  "0)",   "1&",   "22*",   "333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot then number", {
  expect_syntactic(
    c(  ".0",   ".1",   ".22",   ".333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot then number then character", {
  expect_syntactic(
    c( ".0a",  ".1b",  ".22c",  ".333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot then number then non-character", {
  expect_syntactic(
    c( ".0)",  ".1&",  ".22*",  ".333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot dot then number", {
  expect_syntactic(
    c( "..0",  "..1",  "..22",  "..333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot then number", {
  expect_syntactic(
    c("...0", "...1", "...22", "...333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot dot then number", {
  expect_syntactic(
    c("....0", "....1", "....22", "....333"),
    c("....0", "....1", "....22", "....333")
  )
})

test_that("make_syntactic(): dot dot dot dot dot then number", {
  expect_syntactic(
    c(".....0", ".....1", ".....22", ".....333"),
    c(".....0", ".....1", ".....22", ".....333")
  )
})

test_that("make_syntactic(): dot dot then number then character", {
  expect_syntactic(
    c("..0a", "..1b", "..22c", "..333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot dot then number then non-character", {
  expect_syntactic(
    c("..0)", "..1&", "..22*", "..333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

# Duplication --------------------------------------------------------------

test_that("Minimal name repair duplicates if needed", {
  x1 <- NA_character_

  x3 <- c(x1, x1)

  # Called to check absence of side effect
  vec_as_names(x3, repair = "minimal")

  expect_identical(x3, c(NA_character_, NA_character_))
})

test_that("Unique name repair duplicates if needed", {
  x1 <- "fa\u00e7ile"

  x3 <- c(x1, x1)

  # Called to check absence of side effect
  vec_as_names(x3, repair = "unique")

  expect_identical(x3, c("fa\u00e7ile", "fa\u00e7ile"))
})

# Encoding -------------------------------------------------------------

test_that("Name repair works with non-UTF-8 names", {
  x1 <- "fa\u00e7ile"
  skip_if_not(Encoding(x1) == "UTF-8")

  x2 <- iconv(x1, from = "UTF-8", to = "latin1")
  skip_if_not(Encoding(x2) == "latin1")

  x3 <- c(x2, x2)
  expect_equal(vec_as_names(x3, repair = "unique"), paste0(x3, "...", 1:2))
})


# Conditions -----------------------------------------------------------

test_that("names cannot be empty", {
  expect_error_cnd(
    stop_names_cannot_be_empty(c("", "")),
    class = c("vctrs_error_names_cannot_be_empty", "vctrs_error_names", "vctrs_error"),
    message = "Names can't be empty.",
    names = c("", "")
  )
})

test_that("names cannot be dot dot", {
  expect_error_cnd(
    stop_names_cannot_be_dot_dot(c("..1", "..2")),
    class = c("vctrs_error_names_cannot_be_dot_dot", "vctrs_error_names", "vctrs_error"),
    message = "Names can't be of the form `...` or `..j`.",
    names = c("..1", "..2")
  )
})

test_that("names must be unique", {
  expect_error_cnd(
    stop_names_must_be_unique(c("x", "y", "y", "x")),
    class = c("vctrs_error_names_must_be_unique", "vctrs_error_names", "vctrs_error"),
    message = "Names must be unique.",
    names = c("x", "y", "y", "x")
  )
})


# Legacy repair --------------------------------------------------------

test_that("vec_as_names_legacy() works", {
  expect_identical(vec_as_names_legacy(chr()), chr())
  expect_identical(vec_as_names_legacy(c("a", "a", "", "")), c("a", "a1", "V1", "V2"))
  expect_identical(vec_as_names_legacy(c("a", "a", "", ""), sep = "_"), c("a", "a_1", "V_1", "V_2"))
  expect_identical(vec_as_names_legacy(c("a", "a", "", ""), prefix = "foo"), c("a", "a1", "foo1", "foo2"))
  expect_identical(vec_as_names_legacy(c("a", "a", "", ""), prefix = "foo", sep = "_"), c("a", "a_1", "foo_1", "foo_2"))

  # From tibble
  expect_identical(vec_as_names_legacy(c("x", "x")), c("x", "x1"))
  expect_identical(vec_as_names_legacy(c("", "")), c("V1", "V2"))
  expect_identical(vec_as_names_legacy(c("", "V1")), c("V2", "V1"))
  expect_identical(vec_as_names_legacy(c("", "V", "V")), c("V2", "V", "V1"))
})


# Name specification ---------------------------------------------------

test_that("NULL name specs works with scalars", {
  expect_identical(apply_name_spec(NULL, "foo", NULL, 1L), "foo")
  expect_named(vec_c(foo = 1), "foo")

  expect_identical(apply_name_spec(NULL, "foo", chr(), 0L), chr())
  expect_equal(vec_c(foo = dbl()), set_names(dbl(), ""))
  expect_named(vec_c(foo = set_names(dbl())), chr())
  expect_named(vec_c(foo = set_names(dbl()), bar = set_names(dbl())), chr())

  expect_error(apply_name_spec(NULL, "foo", c("a", "b")), "vector of length > 1")
  expect_error(apply_name_spec(NULL, "foo", NULL, 2L), "vector of length > 1")

  expect_snapshot({
    (expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1"))
    (expect_error(vec_c(foo = 1:2), "vector of length > 1"))
    (expect_error(vec_c(x = c(xx = 1)), "named vector"))
  })
})

test_that("function name spec is applied", {
  spec <- function(outer, inner) {
    sep <- if (is_character(inner)) "_" else ":"
    paste0(outer, sep, inner)
  }

  expect_identical(apply_name_spec(spec, "foo", NULL, 1L), "foo")
  expect_named(vec_c(foo = 1, .name_spec = spec), "foo")

  expect_identical(apply_name_spec(spec, "foo", c("a", "b")), c("foo_a", "foo_b"))
  expect_named(vec_c(foo = c(a = 1, b = 2), .name_spec = spec), c("foo_a", "foo_b"))

  expect_identical(apply_name_spec(spec, "foo", NULL, 2L), c("foo:1", "foo:2"))
  expect_named(vec_c(foo = 1:2, .name_spec = spec), c("foo:1", "foo:2"))
})

test_that("can pass lambda formula as name spec", {
  expect_named(vec_c(foo = c(a = 1, b = 2), .name_spec = ~ paste(.x, .y, sep = "_")), c("foo_a", "foo_b"))
  expect_error(vec_c(foo = c(a = 1, b = 2), .name_spec = env()), "Can't convert `.name_spec`", fixed = TRUE)
})

test_that("can pass glue string as name spec", {
  expect_named(vec_c(foo = c(a = 1, b = 2), .name_spec = "{outer}_{inner}"), c("foo_a", "foo_b"))
  expect_named(vec_c(foo = 1:2, .name_spec = "{outer}_{inner}"), c("foo_1", "foo_2"))
  expect_error(vec_c(foo = c(a = 1, b = 2), .name_spec = c("a", "b")), "single string")
})

test_that("`outer` is recycled before name spec is invoked", {
  expect_identical(vec_c(outer = 1:2, .name_spec = "{outer}"), c(outer = 1L, outer = 2L))
})

test_that("apply_name_spec() recycles return value not arguments (#1099)", {
  out <- unstructure(apply_name_spec("foo", "outer", c("a", "b", "c")))
  expect_identical(out, c("foo", "foo", "foo"))

  inner <- NULL
  outer <- NULL
  spec <- function(outer, inner) {
    inner <<- inner
    outer <<- outer
  }

  apply_name_spec(spec, "outer", c("a", "b", "c"))
  expect_identical(inner, c("a", "b", "c"))
  expect_identical(outer, "outer")
})

test_that("r_chr_paste_prefix() works", {
  nms <- c("foo", "bar")

  expect_equal(
    .Call(ffi_chr_paste_prefix, nms, "baz", "."),
    c("baz.foo", "baz.bar")
  )

  # Greater than `VCTRS_PASTE_BUFFER_MAX_SIZE`
  long_prefix <- strrep("a", 5000)

  expect_equal(
    .Call(ffi_chr_paste_prefix, nms, long_prefix, "."),
    paste0(long_prefix, ".", nms)
  )
})

test_that("vec_as_names() uses internal error if `repair_arg` is not supplied", {
  expect_snapshot({
    (expect_error(vec_as_names("", repair = "foobar", call = quote(tilt()))))
    (expect_error(vec_as_names("", repair = env(), call = quote(tilt()))))
  })
})

local_name_repair_quiet()

# vec_names() ---------------------------------------------------------

test_that("vec_names() retrieves names", {
  expect_null(vec_names(letters))
  expect_identical(vec_names(set_names(letters)), letters)

  expect_identical(vec_names(mtcars), row.names(mtcars))
  expect_null(vec_names(unrownames(mtcars)))

  expect_identical(vec_names(Titanic), dimnames(Titanic)[[1]])

  x <- matrix(1L, dimnames = list("row", "col"))
  expect_identical(vec_names(x), dimnames(x)[[1]])
})

test_that("vec_names() dispatches", {
  local_methods(
    names.vctrs_foobar = function(x) "dispatched!"
  )
  expect_identical(vec_names(foobar()), "dispatched!")
})


# vec_names2() -------------------------------------------------------------

test_that("vec_names2() repairs names", {
  expect_identical(vec_names2(1:2), c("", ""))
  expect_identical(vec_names2(1:2, repair = "unique"), c("...1", "...2"))
  expect_identical(
    vec_names2(set_names(1:2, c("_foo", "_bar")), repair = "universal"),
    c("._foo", "._bar")
  )
})

test_that("vec_names2() treats data frames and arrays as vectors", {
  expect_identical(vec_names2(mtcars), row.names(mtcars))
  expect_identical(vec_names2(as.matrix(mtcars)), row.names(mtcars))

  df <- unrownames(mtcars)
  exp <- rep_len("", nrow(mtcars))
  expect_identical(vec_names2(df), exp)
  expect_identical(vec_names2(as.matrix(df)), exp)
})

test_that("vec_names2() accepts and checks repair function", {
  expect_identical(
    vec_names2(1:2, repair = function(nms) rep_along(nms, "foo")),
    c("foo", "foo")
  )
  expect_error(
    vec_names2(1:2, repair = function(nms) "foo"),
    "length 1 instead of length 2"
  )
})

test_that("vec_names2() repairs names before invoking repair function", {
  x <- set_names(1:2, c(NA, NA))
  expect_identical(vec_names2(x, repair = identity), c("", ""))
})

test_that("vec_names2() result is correct for *_quiet repair", {
  expect_identical(
    vec_names2(1:2, repair = "unique"),
    vec_names2(1:2, repair = "unique_quiet")
  )
  expect_identical(
    vec_names2(1:2, repair = "universal"),
    vec_names2(1:2, repair = "universal_quiet")
  )
})

# vec_as_names() -----------------------------------------------------------

test_that("vec_as_names() requires character vector", {
  expect_error(vec_as_names(NULL), "`names` must be a character vector")
})

test_that("vec_as_names() validates `repair`", {
  expect_snapshot({
    (expect_error(my_vec_as_names("x", my_repair = "foo"), "can't be \"foo\""))
    (expect_error(my_vec_as_names(1, my_repair = 1), "string or a function"))
  })
})

test_that("vec_as_names() repairs names", {
  expect_identical(vec_as_names(chr(NA, NA)), c("", ""))
  expect_identical(
    vec_as_names(chr(NA, NA), repair = "unique"),
    c("...1", "...2")
  )
  expect_identical(
    vec_as_names(chr("_foo", "_bar"), repair = "universal"),
    c("._foo", "._bar")
  )
  expect_identical(
    vec_as_names(chr("a", "b"), repair = "check_unique"),
    c("a", "b")
  )
})

test_that("vec_as_names() checks unique names", {
  expect_snapshot({
    (expect_error(my_vec_as_names(chr(NA), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr(""), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("a", "a"), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("..1"), my_repair = "check_unique")))
    (expect_error(my_vec_as_names(chr("..."), my_repair = "check_unique")))
  })
})

test_that("vec_as_names() result is correct for *_quiet repair", {
  expect_identical(
    vec_as_names(chr("_foo", "_bar"), repair = "unique"),
    vec_as_names(chr("_foo", "_bar"), repair = "unique_quiet")
  )
  expect_identical(
    vec_as_names(chr("_foo", "_bar"), repair = "universal"),
    vec_as_names(chr("_foo", "_bar"), repair = "universal_quiet")
  )
})

test_that("vec_as_names() keeps the names of a named vector", {
  x_unnamed <- c(NA, "", "..1", "...2")
  x_names <- letters[1:4]
  x <- set_names(x_unnamed, x_names)

  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "minimal"), x_names),
    vec_as_names(x, repair = "minimal")
  )
  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "unique"), x_names),
    vec_as_names(x, repair = "unique")
  )
  expect_identical(
    set_names(vec_as_names(x_unnamed, repair = "universal"), x_names),
    vec_as_names(x, repair = "universal")
  )
})

test_that("vec_as_names() accepts and checks repair function", {
  f <- local({
    local_obj <- "foo"
    ~ rep_along(.x, local_obj)
  })
  expect_identical(vec_as_names(c("", ""), repair = f), c("foo", "foo"))
  expect_snapshot(
    error = TRUE,
    my_vec_as_names(c("", ""), my_repair = function(nms) "foo")
  )
})

test_that("vec_as_names() repairs names before invoking repair function", {
  expect_identical(vec_as_names(chr(NA, NA), repair = identity), c("", ""))
})

test_that("vec_as_names() is noisy by default", {
  local_name_repair_verbose()

  expect_snapshot({
    # Noisy name repair
    vec_as_names(c("x", "x"), repair = "unique")

    # Quiet name repair
    vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)

    # Hint at repair argument, if known
    (expect_error(
      my_vec_as_names(c("x", "x"), my_repair = "check_unique")
    ))

    # request quiet via name repair string, don't specify `quiet`
    vec_as_names(c("1", "1"), repair = "unique_quiet")
    vec_as_names(c("1", "1"), repair = "universal_quiet")

    # request quiet via name repair string, specify `quiet` = TRUE
    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = TRUE)
    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = TRUE)

    # request quiet via name repair string, specify `quiet` = FALSE
    vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = FALSE)
    vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = FALSE)
  })
})

test_that("validate_minimal_names() checks names", {
  expect_snapshot({
    (expect_error(validate_minimal_names(1), "must return a character vector"))
    (expect_error(validate_minimal_names(NULL), "can't return `NULL`"))
    (expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values"))
  })
})

test_that("validate_unique() checks unique names", {
  expect_snapshot({
    (expect_error(validate_unique(chr(NA)), "`NA`"))
    (expect_error(
      validate_unique(chr("")),
      class = "vctrs_error_names_cannot_be_empty"
    ))
    (expect_error(
      validate_unique(chr("a", "a")),
      class = "vctrs_error_names_must_be_unique"
    ))
    (expect_error(
      validate_unique(chr("..1")),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
    (expect_error(
      validate_unique(chr("...")),
      class = "vctrs_error_names_cannot_be_dot_dot"
    ))
  })
})

test_that("vec_as_names_validate() validates repair arguments", {
  expect_identical(
    validate_name_repair_arg(c("unique", "check_unique")),
    "unique"
  )
  expect_identical(
    validate_name_repair_arg(~ toupper(.))(letters),
    LETTERS
  )
})

test_that("vec_as_names() is quiet when function is supplied (#1018)", {
  expect_silent(
    vctrs::vec_as_names(
      c("a", "b"),
      repair = function(x) paste0(x, "a"),
      quiet = FALSE
    )
  )
})

test_that("vec_as_names() evaluates repair_arg lazily", {
  expect_silent(vec_as_names(letters, repair_arg = print("oof")))
})


# vec_repair_names() -------------------------------------------------------

test_that("vec_repair_names() repairs names", {
  expect_identical(vec_repair_names(1:2), set_names(1:2, c("", "")))
  expect_identical(
    vec_repair_names(1:2, "unique"),
    set_names(1:2, c("...1", "...2"))
  )
  expect_identical(
    vec_repair_names(set_names(1:2, c("_foo", "_bar")), "universal"),
    set_names(1:2, c("._foo", "._bar"))
  )
})

test_that("vec_repair_names() handles data frames and arrays", {
  df <- data.frame(x = 1:2)
  expect_identical(vec_repair_names(df), df)
  expect_identical(row.names(vec_repair_names(as.matrix(df))), c("", ""))
  expect_identical(
    row.names(vec_repair_names(as.matrix(df), "unique")),
    c("...1", "...2")
  )
})

# vec_set_names() -----------------------------------------------------------

test_that("vec_set_names() sets atomic names", {
  x <- 1:2
  names <- c("x1", "x2")
  exp <- set_names(x, names)
  expect_equal(vec_set_names(x, names), exp)
})

test_that("vec_set_names() sets matrix/array names", {
  x <- matrix(1:2)
  names <- c("x1", "x2")
  exp <- x
  rownames(exp) <- names
  expect_equal(vec_set_names(x, names), exp)

  y <- array(1:4, dim = c(2, 1, 2))
  exp <- y
  rownames(exp) <- names
  expect_equal(vec_set_names(y, names), exp)
})

test_that("vec_set_names() doesn't alter names", {
  x <- matrix(1, dimnames = list(rows = "a", cols = "x"))
  vec_set_names(x, "y")
  expect_equal(vec_names2(x), "a")
  expect_equal(colnames(x), "x")
  vec_set_names(x, NULL)
  expect_equal(vec_names2(x), "a")
  expect_equal(colnames(x), "x")

  y <- array(
    1:4,
    dim = c(1, 2, 2),
    dimnames = list(rows = "a", one = 1:2, two = 1:2)
  )
  vec_set_names(y, "y")
  expect_equal(vec_names2(y), "a")
  vec_set_names(y, NULL)
  expect_equal(vec_names2(y), "a")
})

test_that("vec_set_names() sets row names on data frames", {
  expect_identical(
    vec_set_names(data_frame(x = 1), "foo"),
    new_data_frame(list(x = 1), row.names = "foo")
  )

  expect_identical(
    vec_set_names(data_frame(x = 1:2), c("foo", "foo")),
    new_data_frame(list(x = 1:2), row.names = c("foo...1", "foo...2"))
  )
})

test_that("vec_set_names() correctly sets names on POSIXlt objects", {
  x <- as.POSIXlt(new_datetime(0))
  exp <- set_names(x, "a")
  expect_equal(vec_set_names(x, "a"), exp)
})

test_that("vec_set_names() falls back to `names<-` with proxied objects", {
  x <- structure(1, class = "foobar")
  exp <- set_names(x, "a")

  expect_equal(vec_set_names(x, "a"), exp)

  local_methods(`names<-.foobar` = function(x, value) "fallback!")
  expect_equal(vec_set_names(x, "a"), "fallback!")
})

test_that("vec_set_names() falls back to `rownames<-` with shaped proxied objects", {
  x <- structure(1:2, dim = c(2L, 1L), class = "foobar")
  names <- c("r1", "r2")
  exp <- x
  rownames(exp) <- names

  expect_equal(vec_set_names(x, names), exp)

  # `rownames<-` is not generic, but eventually calls `dimnames<-` which is
  local_methods(`dimnames<-.foobar` = function(x, value) "fallback!")
  expect_equal(vec_set_names(x, names), "fallback!")
})

test_that("vec_set_names() can set NULL names", {
  x <- 1:2
  expect_equal(vec_set_names(x, NULL), x)

  x_named <- set_names(x)
  expect_equal(vec_set_names(x_named, NULL), x)

  x_mat <- as.matrix(x)
  expect_equal(vec_set_names(x_mat, NULL), x_mat)

  x_mat_named <- x_mat
  rownames(x_mat_named) <- c("1", "2")
  exp <- matrix(x_mat, dimnames = list(NULL, NULL))
  expect_equal(vec_set_names(x_mat_named, NULL), exp)
})

test_that("vec_set_names() errors with bad `names`", {
  expect_snapshot({
    (expect_error(vec_set_names(1, 1), "character vector, not a double"))
    (expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2"))
  })
})

test_that("vec_names() and vec_set_names() work with 1-dimensional arrays", {
  x <- array(1:2, dimnames = list(c("a", "b")))
  expect_identical(vec_names(x), c("a", "b"))
  expect_identical(vec_names(vec_set_names(x, c("A", "B"))), c("A", "B"))
})

test_that("vec_set_names() is consistent with `names<-` regarding `NULL` inputs", {
  # See also https://github.com/tidyverse/purrr/pull/1224

  # Can "clear" names on `NULL`
  expect_identical(`names<-`(NULL, NULL), NULL)
  expect_identical(vec_set_names(NULL, NULL), NULL)

  # But trying to make a "named `NULL`" is an error
  # (Don't capture the message, these are base R errors)
  expect_error(`names<-`(NULL, character()))
  expect_error(vec_set_names(NULL, character()))

  # This is more obviously an error, because the size of the names doesn't match
  expect_error(`names<-`(NULL, "x"))
  expect_error(vec_set_names(NULL, "x"))
})

# minimal names -------------------------------------------------------------

test_that("minimal names are made from `n` when `name = NULL`", {
  expect_identical(minimal_names(1:2), c("", ""))
})

test_that("as_minimal_names() checks input", {
  expect_error(as_minimal_names(1:3), "must be a character vector")
})

test_that("minimal names have '' instead of NAs", {
  expect_identical(as_minimal_names(c("", NA, "", NA)), c("", "", "", ""))
})

test_that("repairing minimal names copes with NULL input names", {
  x <- 1:3
  x_named <- vec_repair_names(x)
  expect_equal(names(x_named), rep("", 3))
})

test_that("as_minimal_names() is idempotent", {
  x <- c("", "", NA)
  expect_identical(as_minimal_names(x), as_minimal_names(as_minimal_names(x)))
})

test_that("minimal_names() treats data frames and arrays as vectors", {
  expect_identical(minimal_names(mtcars), row.names(mtcars))
  expect_identical(minimal_names(as.matrix(mtcars)), row.names(mtcars))

  df <- unrownames(mtcars)
  exp <- rep_len("", nrow(mtcars))
  expect_identical(minimal_names(df), exp)
  expect_identical(minimal_names(as.matrix(df)), exp)
})

test_that("as_minimal_names() copies on write", {
  nms <- chr(NA, NA)
  as_minimal_names(nms)
  expect_identical(nms, chr(NA, NA))

  nms <- c("a", "b")
  out <- as_minimal_names(nms)
  expect_true(is_reference(nms, out))
})


# unique names -------------------------------------------------------------

test_that("unique_names() handles unnamed vectors", {
  expect_identical(unique_names(1:3), c("...1", "...2", "...3"))
})

test_that("as_unique_names() is a no-op when no repairs are needed", {
  x <- c("x", "y")
  out <- as_unique_names(x)
  expect_true(is_reference(out, x))
  expect_identical(out, c("x", "y"))
})

test_that("as_unique_names() eliminates emptiness and duplication", {
  x <- c("", "x", "y", "x")
  expect_identical(as_unique_names(x), c("...1", "x...2", "y", "x...4"))
})

test_that("as_unique_names(): solo empty or NA gets suffix", {
  expect_identical(as_unique_names(""), "...1")
  expect_identical(as_unique_names(NA_character_), "...1")
})

test_that("as_unique_names() treats ellipsis like empty string", {
  expect_identical(as_unique_names("..."), as_unique_names(""))
})

test_that("two_three_dots() does its job and no more", {
  x <- c(".", ".1", "...1", "..1a")
  expect_identical(two_to_three_dots(x), x)

  expect_identical(two_to_three_dots(c("..1", "..22")), c("...1", "...22"))
})

test_that("two dots then number treated like three dots then number", {
  expect_identical(as_unique_names("..2"), as_unique_names("...5"))
})

test_that("as_unique_names() strips positional suffixes, re-applies as needed", {
  x <- c("...20", "a...1", "b", "", "a...2...34")
  expect_identical(as_unique_names(x), c("...1", "a...2", "b", "...4", "a...5"))

  expect_identical(as_unique_names("a...1"), "a")
  expect_identical(as_unique_names(c("a...2", "a")), c("a...1", "a...2"))
  expect_identical(
    as_unique_names(c("a...3", "a", "a")),
    c("a...1", "a...2", "a...3")
  )
  expect_identical(
    as_unique_names(c("a...2", "a", "a")),
    c("a...1", "a...2", "a...3")
  )
  expect_identical(
    as_unique_names(c("a...2", "a...2", "a...2")),
    c("a...1", "a...2", "a...3")
  )
})

test_that("as_unique_names() is idempotent", {
  x <- c("...20", "a...1", "b", "", "a...2")
  expect_identical(as_unique_names(!!x), as_unique_names(as_unique_names(!!x)))
})

test_that("unique-ification has an 'algebraic'-y property", {
  ## inspired by, but different from, this guarantee about base::make.unique()
  ## make.unique(c(A, B)) == make.unique(c(make.unique(A), B))
  ## If A is already unique, then make.unique(c(A, B)) preserves A.

  ## I haven't formulated what we guarantee very well yet, but it's probably
  ## implicit in this test (?)

  x <- c("...20", "a...1", "b", "", "a...2", "d")
  y <- c("", "a...3", "b", "...3", "e")

  ## fix names on each, catenate, fix the whole
  z1 <- as_unique_names(
    c(
      as_unique_names(x),
      as_unique_names(y)
    )
  )

  ## fix names on x, catenate, fix the whole
  z2 <- as_unique_names(
    c(
      as_unique_names(x),
      y
    )
  )

  ## fix names on y, catenate, fix the whole
  z3 <- as_unique_names(
    c(
      x,
      as_unique_names(y)
    )
  )

  ## catenate, fix the whole
  z4 <- as_unique_names(
    c(
      x,
      y
    )
  )

  expect_identical(z1, z2)
  expect_identical(z1, z3)
  expect_identical(z1, z4)
})

test_that("unique_names() and as_unique_names() are verbose or silent", {
  local_name_repair_verbose()

  expect_snapshot(unique_names(1:2))
  expect_snapshot(as_unique_names(c("", "")))

  expect_message(regexp = NA, unique_names(1:2, quiet = TRUE))
  expect_message(regexp = NA, as_unique_names(c("", ""), quiet = TRUE))
})

test_that("names with only duplicates are repaired", {
  expect_identical(unique_names(list(x = NA, x = NA)), c("x...1", "x...2"))
})


# Universal names ----------------------------------------------------------

test_that("zero-length input", {
  expect_equal(as_universal_names(character()), character())
})

test_that("universal names are not changed", {
  expect_equal(as_universal_names(letters), letters)
})

test_that("as_universal_names() is idempotent", {
  x <- c(NA, "", "x", "x", "a1:", "_x_y}")
  expect_identical(
    as_universal_names(x),
    as_universal_names(as_universal_names(x))
  )
})

test_that("dupes get a suffix", {
  expect_equal(
    as_universal_names(c("a", "b", "a", "c", "b")),
    c("a...1", "b...2", "a...3", "c", "b...5")
  )
})

test_that("as_universal_names(): solo empty or NA gets suffix", {
  expect_identical(as_universal_names(""), "...1")
  expect_identical(as_universal_names(NA_character_), "...1")
})

test_that("as_universal_names() treats ellipsis like empty string", {
  expect_identical(as_universal_names("..."), as_universal_names(""))
})

test_that("solo dot is unchanged", {
  expect_equal(as_universal_names("."), ".")
})

test_that("dot, dot gets suffix", {
  expect_equal(as_universal_names(c(".", ".")), c("....1", "....2"))
})

test_that("dot-dot, dot-dot gets suffix", {
  expect_equal(as_universal_names(c("..", "..")), c(".....1", ".....2"))
})

test_that("empty, dot becomes suffix, dot", {
  expect_equal(as_universal_names(c("", ".")), c("...1", "."))
})

test_that("empty, empty, dot becomes suffix, suffix, dot", {
  expect_equal(as_universal_names(c("", "", ".")), c("...1", "...2", "."))
})

test_that("dot, dot, empty becomes suffix, suffix, suffix", {
  expect_equal(as_universal_names(c(".", ".", "")), c("....1", "....2", "...3"))
})

test_that("dot, empty, dot becomes suffix, suffix, suffix", {
  expect_equal(as_universal_names(c(".", "", ".")), c("....1", "...2", "....3"))
})

test_that("empty, dot, empty becomes suffix, dot, suffix", {
  expect_equal(as_universal_names(c("", ".", "")), c("...1", ".", "...3"))
})

test_that("'...j' gets stripped then names are modified", {
  expect_equal(as_universal_names(c("...6", "...1...2")), c("...1", "...2"))
  expect_equal(as_universal_names("if...2"), ".if")
})

test_that("complicated inputs", {
  expect_equal(
    as_universal_names(c(
      "",
      ".",
      NA,
      "if...4",
      "if",
      "if...8",
      "for",
      "if){]1"
    )),
    c("...1", ".", "...3", ".if...4", ".if...5", ".if...6", ".for", "if...1")
  )
})

test_that("message", {
  local_name_repair_verbose()
  expect_snapshot(as_universal_names(c("a b", "b c")))
})

test_that("quiet", {
  expect_message(
    as_universal_names("", quiet = TRUE),
    NA
  )
})

test_that("unique then universal is universal, with shuffling", {
  x <- c("", ".2", "..3", "...4", "....5", ".....6", "......7", "...")
  expect_identical(
    as_universal_names(as_unique_names(x)),
    as_universal_names(x)
  )

  x2 <- x[c(7L, 4L, 3L, 6L, 5L, 1L, 2L, 8L)]
  expect_identical(
    as_universal_names(as_unique_names(x2)),
    as_universal_names(x2)
  )

  x3 <- x[c(3L, 2L, 4L, 6L, 8L, 1L, 5L, 7L)]
  expect_identical(
    as_universal_names(as_unique_names(x3)),
    as_universal_names(x3)
  )
})

test_that("zero-length inputs given character names", {
  out <- vec_repair_names(character(), "universal")
  expect_equal(names(out), character())
})

test_that("unnamed input gives uniquely named output", {
  out <- vec_repair_names(1:3, "universal")
  expect_equal(names(out), c("...1", "...2", "...3"))
})

test_that("messages by default", {
  local_name_repair_verbose()
  expect_snapshot(vec_repair_names(set_names(1, "a:b"), "universal"))
  expect_snapshot(vec_repair_names(set_names(1, "a:b"), ~ make.names(.)))
})

test_that("quiet = TRUE", {
  expect_message(
    vec_repair_names(set_names(1, ""), "universal", quiet = TRUE),
    NA
  )
})

test_that("non-universal names", {
  out <- vec_repair_names(set_names(1, "a b"), "universal")
  expect_equal(names(out), "a.b")

  expect_equal(as_universal_names("a b"), "a.b")
})


# make_syntactic() ---------------------------------------------------------

test_that("make_syntactic(): empty or NA", {
  expect_syntactic(
    c("", NA_character_),
    c(".", ".")
  )
})

test_that("make_syntactic(): reserved words", {
  expect_syntactic(
    c("if", "TRUE", "Inf", "NA_real_", "normal"),
    c(".if", ".TRUE", ".Inf", ".NA_real_", "normal")
  )
})

test_that("make_syntactic(): underscore", {
  expect_syntactic(
    c("_", "_1", "_a}"),
    c("._", "._1", "._a.")
  )
})

test_that("make_syntactic(): dots", {
  expect_syntactic(
    c(".", "..", "...", "...."),
    c(".", "..", "....", "....")
  )
})

test_that("make_syntactic(): number", {
  expect_syntactic(
    c("0", "1", "22", "333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): number then character", {
  expect_syntactic(
    c("0a", "1b", "22c", "333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): number then non-character", {
  expect_syntactic(
    c("0)", "1&", "22*", "333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot then number", {
  expect_syntactic(
    c(".0", ".1", ".22", ".333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot then number then character", {
  expect_syntactic(
    c(".0a", ".1b", ".22c", ".333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot then number then non-character", {
  expect_syntactic(
    c(".0)", ".1&", ".22*", ".333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

test_that("make_syntactic(): dot dot then number", {
  expect_syntactic(
    c("..0", "..1", "..22", "..333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot then number", {
  expect_syntactic(
    c("...0", "...1", "...22", "...333"),
    c("...0", "...1", "...22", "...333")
  )
})

test_that("make_syntactic(): dot dot dot dot then number", {
  expect_syntactic(
    c("....0", "....1", "....22", "....333"),
    c("....0", "....1", "....22", "....333")
  )
})

test_that("make_syntactic(): dot dot dot dot dot then number", {
  expect_syntactic(
    c(".....0", ".....1", ".....22", ".....333"),
    c(".....0", ".....1", ".....22", ".....333")
  )
})

test_that("make_syntactic(): dot dot then number then character", {
  expect_syntactic(
    c("..0a", "..1b", "..22c", "..333d"),
    c("..0a", "..1b", "..22c", "..333d")
  )
})

test_that("make_syntactic(): dot dot then number then non-character", {
  expect_syntactic(
    c("..0)", "..1&", "..22*", "..333@"),
    c("..0.", "..1.", "..22.", "..333.")
  )
})

# Duplication --------------------------------------------------------------

test_that("Minimal name repair duplicates if needed", {
  x1 <- NA_character_

  x3 <- c(x1, x1)

  # Called to check absence of side effect
  vec_as_names(x3, repair = "minimal")

  expect_identical(x3, c(NA_character_, NA_character_))
})

test_that("Unique name repair duplicates if needed", {
  x1 <- "fa\u00e7ile"

  x3 <- c(x1, x1)

  # Called to check absence of side effect
  vec_as_names(x3, repair = "unique")

  expect_identical(x3, c("fa\u00e7ile", "fa\u00e7ile"))
})

# Encoding -------------------------------------------------------------

test_that("Name repair works with non-UTF-8 names", {
  x1 <- "fa\u00e7ile"
  skip_if_not(Encoding(x1) == "UTF-8")

  x2 <- iconv(x1, from = "UTF-8", to = "latin1")
  skip_if_not(Encoding(x2) == "latin1")

  x3 <- c(x2, x2)
  expect_equal(vec_as_names(x3, repair = "unique"), paste0(x3, "...", 1:2))
})


# Conditions -----------------------------------------------------------

test_that("names cannot be empty", {
  expect_error_cnd(
    stop_names_cannot_be_empty(c("", "")),
    class = c(
      "vctrs_error_names_cannot_be_empty",
      "vctrs_error_names",
      "vctrs_error"
    ),
    message = "Names can't be empty.",
    names = c("", "")
  )
})

test_that("names cannot be dot dot", {
  expect_error_cnd(
    stop_names_cannot_be_dot_dot(c("..1", "..2")),
    class = c(
      "vctrs_error_names_cannot_be_dot_dot",
      "vctrs_error_names",
      "vctrs_error"
    ),
    message = "Names can't be of the form `...` or `..j`.",
    names = c("..1", "..2")
  )
})

test_that("names must be unique", {
  expect_error_cnd(
    stop_names_must_be_unique(c("x", "y", "y", "x")),
    class = c(
      "vctrs_error_names_must_be_unique",
      "vctrs_error_names",
      "vctrs_error"
    ),
    message = "Names must be unique.",
    names = c("x", "y", "y", "x")
  )
})


# Legacy repair --------------------------------------------------------

test_that("vec_as_names_legacy() works", {
  expect_identical(vec_as_names_legacy(chr()), chr())
  expect_identical(
    vec_as_names_legacy(c("a", "a", "", "")),
    c("a", "a1", "V1", "V2")
  )
  expect_identical(
    vec_as_names_legacy(c("a", "a", "", ""), sep = "_"),
    c("a", "a_1", "V_1", "V_2")
  )
  expect_identical(
    vec_as_names_legacy(c("a", "a", "", ""), prefix = "foo"),
    c("a", "a1", "foo1", "foo2")
  )
  expect_identical(
    vec_as_names_legacy(c("a", "a", "", ""), prefix = "foo", sep = "_"),
    c("a", "a_1", "foo_1", "foo_2")
  )

  # From tibble
  expect_identical(vec_as_names_legacy(c("x", "x")), c("x", "x1"))
  expect_identical(vec_as_names_legacy(c("", "")), c("V1", "V2"))
  expect_identical(vec_as_names_legacy(c("", "V1")), c("V2", "V1"))
  expect_identical(vec_as_names_legacy(c("", "V", "V")), c("V2", "V", "V1"))
})


# Name specification ---------------------------------------------------

test_that("NULL name specs works with scalars", {
  expect_identical(apply_name_spec(NULL, "foo", NULL, 1L), "foo")
  expect_named(vec_c(foo = 1), "foo")

  expect_identical(apply_name_spec(NULL, "foo", chr(), 0L), chr())
  expect_equal(vec_c(foo = dbl()), set_names(dbl(), ""))
  expect_named(vec_c(foo = set_names(dbl())), chr())
  expect_named(vec_c(foo = set_names(dbl()), bar = set_names(dbl())), chr())

  expect_error(
    apply_name_spec(NULL, "foo", c("a", "b")),
    "vector of length > 1"
  )
  expect_error(apply_name_spec(NULL, "foo", NULL, 2L), "vector of length > 1")

  expect_snapshot({
    (expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1"))
    (expect_error(vec_c(foo = 1:2), "vector of length > 1"))
    (expect_error(vec_c(x = c(xx = 1)), "named vector"))
  })
})

test_that("function name spec is applied", {
  spec <- function(outer, inner) {
    sep <- if (is_character(inner)) "_" else ":"
    paste0(outer, sep, inner)
  }

  expect_identical(apply_name_spec(spec, "foo", NULL, 1L), "foo")
  expect_named(vec_c(foo = 1, .name_spec = spec), "foo")

  expect_identical(
    apply_name_spec(spec, "foo", c("a", "b")),
    c("foo_a", "foo_b")
  )
  expect_named(
    vec_c(foo = c(a = 1, b = 2), .name_spec = spec),
    c("foo_a", "foo_b")
  )

  expect_identical(apply_name_spec(spec, "foo", NULL, 2L), c("foo:1", "foo:2"))
  expect_named(vec_c(foo = 1:2, .name_spec = spec), c("foo:1", "foo:2"))
})

test_that("can pass lambda formula as name spec", {
  expect_named(
    vec_c(foo = c(a = 1, b = 2), .name_spec = ~ paste(.x, .y, sep = "_")),
    c("foo_a", "foo_b")
  )
  expect_error(
    vec_c(foo = c(a = 1, b = 2), .name_spec = env()),
    "Can't convert `.name_spec`",
    fixed = TRUE
  )
})

test_that("can pass glue string as name spec", {
  expect_named(
    vec_c(foo = c(a = 1, b = 2), .name_spec = "{outer}_{inner}"),
    c("foo_a", "foo_b")
  )
  expect_named(
    vec_c(foo = 1:2, .name_spec = "{outer}_{inner}"),
    c("foo_1", "foo_2")
  )
  expect_error(
    vec_c(foo = c(a = 1, b = 2), .name_spec = c("a", "b")),
    "single string"
  )
})

test_that("can pass 'inner' string as name spec", {
  expect_named(vec_c(foo = c(a = 1, b = 2), .name_spec = "inner"), c("a", "b"))
  expect_named(vec_c(foo = 1:2, .name_spec = "inner"), NULL)
  expect_named(
    list_unchop(
      list(outer = c(a = 1)),
      indices = list(1:2),
      name_spec = "inner"
    ),
    c("a", "a")
  )
})

test_that("`outer` is recycled after name spec is invoked in functions that use `apply_name_spec()`", {
  expect_identical(
    vec_c(outer = 1:2, .name_spec = "{outer}"),
    c(outer = 1L, outer = 2L)
  )
  expect_identical(
    vec_rbind(
      outer = data_frame(x = 1:2),
      .names_to = NULL,
      .name_spec = "{outer}"
    ),
    new_data_frame(list(x = 1:2), row.names = c("outer...1", "outer...2"))
  )
  expect_identical(
    list_unchop(
      list(outer = c(a = 1)),
      indices = list(1:2),
      name_spec = "{outer}"
    ),
    c(outer = 1, outer = 1)
  )
  expect_identical(
    list_unchop(
      list(outer = c(a = 1)),
      indices = list(1:2),
      name_spec = "{outer}_{inner}"
    ),
    c(outer_a = 1, outer_a = 1)
  )

  # Note that `apply_name_spec()` itself doesn't recycle, it expects the caller
  # to do so
  expect_identical(
    unstructure(apply_name_spec("{outer}", "outer", NULL, n = 2L)),
    "outer"
  )
})

test_that("apply_name_spec() doesn't recycle inputs (#1099)", {
  # We used to recycle the output for the caller, but now we check that the
  # output is recyclable and just return it even if it is size 1, expecting the
  # caller to be able to handle it, possibly efficiently with `chr_assign()`.

  out <- unstructure(apply_name_spec("foo", "outer", c("a", "b", "c")))
  expect_identical(out, "foo")

  inner <- NULL
  outer <- NULL
  spec <- function(outer, inner) {
    inner <<- inner
    outer <<- outer
  }

  apply_name_spec(spec, "outer", c("a", "b", "c"))
  expect_identical(inner, c("a", "b", "c"))
  expect_identical(outer, "outer")

  apply_name_spec(spec, "outer", "a", n = 3L)
  expect_identical(inner, "a")
  expect_identical(outer, "outer")
})

test_that("apply_name_spec() checks recyclability of output", {
  # These are recyclable
  expect_identical(
    apply_name_spec(function(...) c("a", "b"), "outer", "inner", n = 2L),
    c("a", "b")
  )
  expect_identical(
    apply_name_spec(function(...) "a", "outer", "inner", n = 2L),
    "a"
  )

  # This is not
  expect_snapshot(error = TRUE, {
    apply_name_spec(function(...) c("a", "b", "c"), "outer", "inner", n = 2L)
  })
})

test_that("r_chr_paste_prefix() works", {
  nms <- c("foo", "bar")

  expect_equal(
    .Call(ffi_chr_paste_prefix, nms, "baz", "."),
    c("baz.foo", "baz.bar")
  )

  # Greater than `VCTRS_PASTE_BUFFER_MAX_SIZE`
  long_prefix <- strrep("a", 5000)

  expect_equal(
    .Call(ffi_chr_paste_prefix, nms, long_prefix, "."),
    paste0(long_prefix, ".", nms)
  )
})

test_that("vec_as_names() uses internal error if `repair_arg` is not supplied", {
  expect_snapshot({
    (expect_error(vec_as_names("", repair = "foobar", call = quote(tilt()))))
    (expect_error(vec_as_names("", repair = env(), call = quote(tilt()))))
  })
})

#' Equality proxy
#'
#' Returns a proxy object (i.e. an atomic vector or data frame of atomic
#' vectors). For [vctr]s, this determines the behaviour of `==` and
#' `!=` (via [vec_equal()]); [unique()], [duplicated()] (via
#' [vec_unique()] and [vec_duplicate_detect()]); [is.na()] and [anyNA()]
#' (via [vec_detect_missing()]).
#'
#' The default method calls [vec_proxy()], as the default underlying
#' vector data should be equal-able in most cases. If your class is
#' not equal-able, provide a `vec_proxy_equal()` method that throws an
#' error.
#'
#' @section Data frames:
#' If the proxy for `x` is a data frame, the proxy function is automatically
#' recursively applied on all columns as well. After applying the proxy
#' recursively, if there are any data frame columns present in the proxy, then
#' they are unpacked. Finally, if the resulting data frame only has a single
#' column, then it is unwrapped and a vector is returned as the proxy.
#'
#' @param x A vector x.
#' @inheritParams rlang::args_dots_empty
#'
#' @return A 1d atomic vector or a data frame.
#' @keywords internal
#'
#' @section Dependencies:
#' - [vec_proxy()] called by default
#'
#' @export
vec_proxy_equal <- function(x, ...) {
  check_dots_empty0(...)
  return(.Call(vctrs_proxy_equal, x))
  UseMethod("vec_proxy_equal")
}
#' @export
vec_proxy_equal.default <- function(x, ...) {
  stop_native_implementation("vec_proxy_equal.default")
}

#' Equality
#'
#' @description
#' `vec_equal()` tests if two vectors are equal.
#'
#' @details
#' Attributes of `x` and `y` are considered equal if they have the same names
#' and values, even if the attribute ordering is different. This reflects the
#' idea that attributes are treated as a map rather than an ordered list.
#'
#' @inheritParams vec_compare
#' @return A logical vector the same size as the common size of `x` and `y`.
#'   Will only contain `NA`s if `na_equal` is `FALSE`.
#'
#' @section Dependencies:
#' - [vec_cast_common()] with fallback
#' - [vec_recycle_common()]
#' - [vec_proxy_equal()]
#'
#' @seealso [vec_detect_missing()]
#'
#' @export
#' @examples
#' vec_equal(c(TRUE, FALSE, NA), FALSE)
#' vec_equal(c(TRUE, FALSE, NA), FALSE, na_equal = TRUE)
#'
#' vec_equal(5, 1:10)
#' vec_equal("d", letters[1:10])
#'
#' df <- data.frame(x = c(1, 1, 2, 1), y = c(1, 2, 1, NA))
#' vec_equal(df, data.frame(x = 1, y = 2))
vec_equal <- function(x, y, na_equal = FALSE, .ptype = NULL) {
  .Call(ffi_vec_equal, x, y, na_equal, .ptype, environment())
}

obj_equal <- function(x, y) {
  .Call(ffi_obj_equal, x, y)
}

# vectorised --------------------------------------------------------------

test_that("throws error for unsuported type", {
  expect_error(.Call(vctrs_equal, expression(x), expression(x), TRUE), class = "vctrs_error_scalar_type")
})

test_that("C wrapper throws error if length or type doesn't match", {
  expect_error(.Call(vctrs_equal, 1:2, 1L, TRUE), "same types and lengths")
  expect_error(.Call(vctrs_equal, 1, 1L, TRUE), "same types and lengths")
})

test_that("correct behaviour for basic vectors", {
  expect_equal(vec_equal(c(TRUE, FALSE), TRUE), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1L, 2L), 1L), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1, 2), 1), c(TRUE, FALSE))
  expect_equal(vec_equal(c("1", "2"), "1"), c(TRUE, FALSE))
  expect_equal(vec_equal(as.raw(1:2), as.raw(1L)), c(TRUE, FALSE))
  expect_equal(vec_equal(list(1:3, 1:2), list(1:3)), c(TRUE, FALSE))
  expect_equal(vec_equal(list(1:3, 1.5), list(1:3)), c(TRUE, FALSE))
  expect_equal(vec_equal(list(as.raw(1:3), as.raw(1.5)), list(as.raw(1:3))), c(TRUE, FALSE))
  expect_equal(vec_equal(list(1+1i, 1+0i), list(1+1i)), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1, 2) + 1i, 1+1i), c(TRUE, FALSE))
})

test_that("NAs are equal", {
  expect_true(vec_equal(NA, NA, na_equal = TRUE))
  expect_true(vec_equal(NA_integer_, NA_integer_, na_equal = TRUE))
  expect_true(vec_equal(NA_real_, NA_real_, na_equal = TRUE))
  expect_true(vec_equal(NA_character_, NA_character_, na_equal = TRUE))
  expect_true(vec_equal(list(NULL), list(NULL), na_equal = TRUE))
})

test_that("double special values", {
  expect_equal(vec_equal(c(NaN, NA), NaN, na_equal = TRUE), c(TRUE, FALSE))
  expect_equal(vec_equal(c(NA, NaN), NA, na_equal = TRUE), c(TRUE, FALSE))
  expect_true(vec_equal(Inf, Inf))
  expect_true(vec_equal(-Inf, -Inf))
})

test_that("`list(NULL)` is considered a missing value (#653)", {
  expect_equal(vec_equal(list(NULL), list(NULL)), NA)
  expect_equal(vec_equal(list(NULL), list(1)), NA)
})

test_that("can compare data frames", {
  df <- data.frame(x = 1:2, y = letters[2:1], stringsAsFactors = FALSE)
  expect_equal(vec_equal(df, df[1, ]), c(TRUE, FALSE))
})

test_that("can compare data frames with various types of columns", {
  x1 <- data_frame(x = 1, y = 2)
  y1 <- data_frame(x = 2, y = 1)

  x2 <- data_frame(x = "a")
  y2 <- data_frame(x = "b")

  x3 <- data_frame(x = FALSE)
  y3 <- data_frame(x = TRUE)

  x4 <- data_frame(x = 1L)
  y4 <- data_frame(x = 2L)

  x5 <- data_frame(x = as.raw(0))
  y5 <- data_frame(x = as.raw(1))

  x6 <- data_frame(x = 1+0i)
  y6 <- data_frame(x = 1+1i)

  expect_false(vec_equal(x1, y1))
  expect_false(vec_equal(x2, y2))
  expect_false(vec_equal(x3, y3))
  expect_false(vec_equal(x4, y4))
  expect_false(vec_equal(x5, y5))
  expect_false(vec_equal(x6, y6))
})

test_that("can compare data frames with data frame columns", {
  df1 <- data_frame(x = data_frame(a = 1))
  df2 <- data_frame(x = data_frame(a = 2))

  expect_true(vec_equal(df1, df1))
  expect_false(vec_equal(df1, df2))
})

test_that("can compare data frames with list columns", {
  df1 <- data_frame(x = list(a = 1, b = 2), y = c(1, 1))
  df2 <- data_frame(x = list(a = 0, b = 2), y = c(1, 1))

  expect_equal(vec_equal(df1, df2), c(FALSE, TRUE))
})

test_that("data frames must have same size and columns", {
  expect_error(.Call(vctrs_equal,
    data.frame(x = 1),
    data.frame(x = 1, y = 2),
    TRUE
  ),
    "must have same types and lengths"
  )

  expect_error(.Call(vctrs_equal,
    data.frame(x = 1, y = 2, z = 2),
    data.frame(x = 1, y = 2),
    TRUE
    ),
    "must have the same number of columns"
  )

  # Names are not checked, as `vec_cast_common()` should take care of the type.
  # So if `vec_cast_common()` is not called, or is improperly specified, then
  # this could result in false equality.
  expect_true(.Call(vctrs_equal,
    data.frame(x = 1),
    data.frame(y = 1),
    TRUE
  ))

  expect_error(.Call(vctrs_equal,
    data.frame(x = 1:2, y = 3:4),
    data.frame(x = 1, y = 2),
    TRUE
    ),
    "must have same types and lengths"
  )

  expect_false(.Call(vctrs_equal,
    data.frame(x = 1),
    data.frame(x = 2),
    TRUE
  ))

  expect_false(.Call(vctrs_equal,
    list(data.frame(x = 1)),
    list(10),
    TRUE
  ))

})

test_that("can compare data frames with 0 columns", {
  x <- new_data_frame(n = 1L)
  expect_true(vec_equal(x, x))
})

test_that("can compare lists of scalars (#643)", {
  lst <- list(new_sclr(x = 1))
  expect_true(vec_equal(lst, lst))

  # NA does not propagate
  lst <- list(new_sclr(y = NA))
  expect_true(vec_equal(lst, lst))

  df <- data.frame(x = c(1, 4, 3), y = c(2, 8, 9))
  model <- lm(y ~ x, df)
  lst <- list(model)
  expect_true(vec_equal(lst, lst))
})

test_that("can determine equality of strings with different encodings (#553)", {
  for (x_encoding in encodings()) {
    for (y_encoding in encodings()) {
      expect_equal(vec_equal(x_encoding, y_encoding), TRUE)
      expect_equal(vec_equal(x_encoding, y_encoding), x_encoding == y_encoding)
    }
  }
})

test_that("equality can be determined when strings have identical encodings", {
  encs <- encodings()

  for (enc in encs) {
    expect_true(vec_equal(enc, enc))
    expect_equal(vec_equal(enc, enc), enc == enc)
  }
})

test_that("equality is known to always fail with bytes", {
  enc <- encoding_bytes()
  error <- "translating strings with \"bytes\" encoding"
  expect_error(vec_equal(enc, enc), error)
})

test_that("equality is known to fail when comparing bytes to other encodings", {
  error <- "translating strings with \"bytes\" encoding"

  for (enc in encodings()) {
    expect_error(vec_equal(encoding_bytes(), enc), error)
    expect_error(vec_equal(enc, encoding_bytes()), error)
  }
})

test_that("`na_equal` is validated", {
  expect_snapshot(error = TRUE, {
    vec_equal(1, 1, na_equal = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_equal(1, 1, na_equal = c(TRUE, FALSE))
  })
})

test_that("can compare lists of expressions", {
  x <- list(expression(x), expression(y))
  y <- list(expression(x))

  expect_equal(vec_equal(x, y), c(TRUE, FALSE))
})

test_that("vec_equal() silently falls back to base data frame", {
  expect_silent(expect_identical(
    vec_equal(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(TRUE, 32)
  ))
})


# object ------------------------------------------------------------------

test_that("can compare NULL",{
  expect_true(obj_equal(NULL, NULL))
})

test_that("can compare objects with reference semantics", {
  expect_true(obj_equal(globalenv(), globalenv()))
  expect_false(obj_equal(globalenv(), environment()))

  expect_true(obj_equal(quote(x), quote(x)))
  expect_false(obj_equal(quote(x), quote(y)))
})

test_that("can compare pairlists", {
  expect_true(obj_equal(quote(x + y), quote(x + y)))
  expect_true(obj_equal(pairlist(x = 1, y = 2), pairlist(x = 1, y = 2)))
})

test_that("can compare functions", {
  f1 <- function(x, y) x + y
  f2 <- function(x, y) x + y
  expect_false(obj_equal(f2, f1))

  attr(f1, "srcref") <- NULL
  attr(f2, "srcref") <- NULL
  expect_true(obj_equal(f2, f1))

  f3 <- f1
  formals(f3) <- alist(x = 1)
  expect_false(obj_equal(f3, f1))

  f4 <- f1
  body(f4) <- quote(x)
  expect_false(obj_equal(f4, f2))
})

test_that("not equal if different types or lengths", {
  expect_false(obj_equal(1, 2))
  expect_false(obj_equal(1:2, 1))
})

test_that("not equal if attributes not equal", {
  x1 <- structure(1:10, x = 1, y = 2)
  x2 <- structure(1:10, x = 1, y = 3)
  expect_false(obj_equal(x1, x2))
})

test_that("can compare expressions", {
  expect_true(obj_equal(expression(x), expression(x)))
  expect_false(obj_equal(expression(x), expression(y)))
})

# na ----------------------------------------------------------------------

test_that("NA propagate symmetrically (#204)", {
  exp <- c(NA, NA)

  expect_identical(vec_equal(c(TRUE, FALSE), NA), exp)
  expect_identical(vec_equal(1:2, NA), exp)
  expect_identical(vec_equal(c(1, 2), NA), exp)
  expect_identical(vec_equal(letters[1:2], NA), exp)

  expect_identical(vec_equal(NA, c(TRUE, FALSE)), exp)
  expect_identical(vec_equal(NA, 1:2), exp)
  expect_identical(vec_equal(NA, c(1, 2)), exp)
  expect_identical(vec_equal(NA, letters[1:2]), exp)
})

test_that("NA propagate from data frames columns", {
  x <- data.frame(x = 1:3)
  y <- data.frame(x = c(1L, NA, 2L))

  expect_identical(vec_equal(x, y), c(TRUE, NA, FALSE))
  expect_identical(vec_equal(y, x), c(TRUE, NA, FALSE))

  expect_identical(vec_equal(x, y, na_equal = TRUE), c(TRUE, FALSE, FALSE))
  expect_identical(vec_equal(y, x, na_equal = TRUE), c(TRUE, FALSE, FALSE))

  x <- data.frame(x = 1:3, y = 1:3)
  y <- data.frame(x = c(1L, NA, 2L), y = c(NA, 2L, 3L))

  expect_identical(vec_equal(x, y), c(NA, NA, FALSE))
  expect_identical(vec_equal(y, x), c(NA, NA, FALSE))

  expect_identical(vec_equal(x, y, na_equal = TRUE), c(FALSE, FALSE, FALSE))
  expect_identical(vec_equal(y, x, na_equal = TRUE), c(FALSE, FALSE, FALSE))
})

test_that("NA do not propagate from list components (#662)", {
  expect_true(obj_equal(NA, NA))
  expect_true(vec_equal(list(NA), list(NA)))
})

test_that("NA do not propagate from names when comparing objects", {
  x <- set_names(1:3, c("a", "b", NA))
  y <- set_names(1:3, c("a", NA, NA))

  expect_true(obj_equal(x, x))
  expect_false(obj_equal(x, y))

  expect_equal(vec_equal(list(x, x, y), list(x, y, y)), c(TRUE, FALSE, TRUE))
})

test_that("NA do not propagate from attributes", {
  x <- structure(1:3, foo = NA)
  y <- structure(1:3, foo = "")
  expect_true(obj_equal(x, x))
  expect_false(obj_equal(x, y))
})

test_that("NA do not propagate from function bodies or formals", {
  fn <- other <- function() NA
  body(other) <- TRUE

  expect_true(vec_equal(list(fn), list(fn)))
  expect_false(vec_equal(list(fn), list(other)))
  expect_true(obj_equal(fn, fn))
  expect_false(obj_equal(fn, other))

  fn <- other <- function(x = NA) NULL
  formals(other) <- list(x = NULL)

  expect_true(vec_equal(list(fn), list(fn)))
  expect_false(vec_equal(list(fn), list(other)))
})

test_that("can check equality of unspecified objects", {
  expect_equal(vec_equal(NA, NA), NA)
  expect_true(vec_equal(NA, NA, na_equal = TRUE))

  expect_equal(vec_equal(unspecified(1), unspecified(1)), NA)
  expect_true(vec_equal(unspecified(1), unspecified(1), na_equal = TRUE))

  expect_equal(vec_equal(NA, unspecified(1)), NA)
  expect_true(vec_equal(NA, unspecified(1), na_equal = TRUE))
})

test_that("can't supply NA as `na_equal`", {
  expect_snapshot(error = TRUE, {
    vec_equal(NA, NA, na_equal = NA)
  })
})


# proxy -------------------------------------------------------------------

test_that("vec_equal() takes vec_proxy() by default", {
  local_env_proxy()
  x <- new_proxy(1:3)
  y <- new_proxy(3:1)
  expect_identical(vec_equal(x, y), lgl(FALSE, TRUE, FALSE))
})

test_that("vec_equal() takes vec_proxy_equal() if implemented", {
  local_comparable_tuple()

  x <- tuple(1:3, 1:3)
  y <- tuple(1:3, 4:6)

  expect_identical(x == y, rep(TRUE, 3))
  expect_identical(vec_equal(x, y), rep(TRUE, 3))

  # Recursive case
  foo <- data_frame(x = x)
  bar <- data_frame(x = y)
  expect_identical(vec_equal(foo, bar), rep(TRUE, 3))
})

# vectorised --------------------------------------------------------------

test_that("throws error for unsuported type", {
  expect_snapshot(error = TRUE, cnd_class = TRUE, {
    vec_equal(expression(x), expression(x))
  })
})

test_that("correct behaviour for basic vectors", {
  expect_equal(vec_equal(c(TRUE, FALSE), TRUE), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1L, 2L), 1L), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1, 2), 1), c(TRUE, FALSE))
  expect_equal(vec_equal(c("1", "2"), "1"), c(TRUE, FALSE))
  expect_equal(vec_equal(as.raw(1:2), as.raw(1L)), c(TRUE, FALSE))
  expect_equal(vec_equal(list(1:3, 1:2), list(1:3)), c(TRUE, FALSE))
  expect_equal(vec_equal(list(1:3, 1.5), list(1:3)), c(TRUE, FALSE))
  expect_equal(
    vec_equal(list(as.raw(1:3), as.raw(1.5)), list(as.raw(1:3))),
    c(TRUE, FALSE)
  )
  expect_equal(vec_equal(list(1 + 1i, 1 + 0i), list(1 + 1i)), c(TRUE, FALSE))
  expect_equal(vec_equal(c(1, 2) + 1i, 1 + 1i), c(TRUE, FALSE))
})

test_that("NAs are equal", {
  expect_true(vec_equal(NA, NA, na_equal = TRUE))
  expect_true(vec_equal(NA_integer_, NA_integer_, na_equal = TRUE))
  expect_true(vec_equal(NA_real_, NA_real_, na_equal = TRUE))
  expect_true(vec_equal(NA_character_, NA_character_, na_equal = TRUE))
  expect_true(vec_equal(list(NULL), list(NULL), na_equal = TRUE))
})

test_that("double special values", {
  expect_equal(vec_equal(c(NaN, NA), NaN, na_equal = TRUE), c(TRUE, FALSE))
  expect_equal(vec_equal(c(NA, NaN), NA, na_equal = TRUE), c(TRUE, FALSE))
  expect_true(vec_equal(Inf, Inf))
  expect_true(vec_equal(-Inf, -Inf))
})

test_that("`list(NULL)` is considered a missing value (#653)", {
  expect_equal(vec_equal(list(NULL), list(NULL)), NA)
  expect_equal(vec_equal(list(NULL), list(1)), NA)
})

test_that("can compare data frames", {
  df <- data.frame(x = 1:2, y = letters[2:1], stringsAsFactors = FALSE)
  expect_equal(vec_equal(df, df[1, ]), c(TRUE, FALSE))
})

test_that("can compare data frames with various types of columns", {
  x1 <- data_frame(x = 1, y = 2)
  y1 <- data_frame(x = 2, y = 1)

  x2 <- data_frame(x = "a")
  y2 <- data_frame(x = "b")

  x3 <- data_frame(x = FALSE)
  y3 <- data_frame(x = TRUE)

  x4 <- data_frame(x = 1L)
  y4 <- data_frame(x = 2L)

  x5 <- data_frame(x = as.raw(0))
  y5 <- data_frame(x = as.raw(1))

  x6 <- data_frame(x = 1 + 0i)
  y6 <- data_frame(x = 1 + 1i)

  expect_false(vec_equal(x1, y1))
  expect_false(vec_equal(x2, y2))
  expect_false(vec_equal(x3, y3))
  expect_false(vec_equal(x4, y4))
  expect_false(vec_equal(x5, y5))
  expect_false(vec_equal(x6, y6))
})

test_that("can compare data frames with data frame columns", {
  df1 <- data_frame(x = data_frame(a = 1))
  df2 <- data_frame(x = data_frame(a = 2))

  expect_true(vec_equal(df1, df1))
  expect_false(vec_equal(df1, df2))
})

test_that("can compare data frames with list columns", {
  df1 <- data_frame(x = list(a = 1, b = 2), y = c(1, 1))
  df2 <- data_frame(x = list(a = 0, b = 2), y = c(1, 1))

  expect_equal(vec_equal(df1, df2), c(FALSE, TRUE))
})

test_that("data frames are cast to common type", {
  expect_identical(
    vec_equal(
      data.frame(x = 1),
      data.frame(x = 1, y = 2),
      na_equal = TRUE
    ),
    FALSE
  )
  expect_identical(
    vec_equal(
      data.frame(x = 1, y = 2, z = 2),
      data.frame(x = 1, y = 2),
      na_equal = TRUE
    ),
    FALSE
  )
  expect_identical(
    vec_equal(
      data.frame(x = 1),
      data.frame(y = 1),
      na_equal = TRUE
    ),
    FALSE
  )

  expect_identical(
    vec_equal(
      data.frame(x = 1),
      data.frame(x = 2)
    ),
    FALSE
  )
})

test_that("can compare data frames with 0 columns", {
  x <- new_data_frame(n = 1L)
  expect_true(vec_equal(x, x))
})

test_that("can compare lists of scalars (#643)", {
  lst <- list(new_sclr(x = 1))
  expect_true(vec_equal(lst, lst))

  # NA does not propagate
  lst <- list(new_sclr(y = NA))
  expect_true(vec_equal(lst, lst))

  df <- data.frame(x = c(1, 4, 3), y = c(2, 8, 9))
  model <- lm(y ~ x, df)
  lst <- list(model)
  expect_true(vec_equal(lst, lst))
})

test_that("can determine equality of strings with different encodings (#553)", {
  for (x_encoding in encodings()) {
    for (y_encoding in encodings()) {
      expect_equal(vec_equal(x_encoding, y_encoding), TRUE)
      expect_equal(vec_equal(x_encoding, y_encoding), x_encoding == y_encoding)
    }
  }
})

test_that("equality can be determined when strings have identical encodings", {
  encs <- encodings()

  for (enc in encs) {
    expect_true(vec_equal(enc, enc))
    expect_equal(vec_equal(enc, enc), enc == enc)
  }
})

test_that("equality is known to always fail with bytes", {
  enc <- encoding_bytes()
  error <- "translating strings with \"bytes\" encoding"
  expect_error(vec_equal(enc, enc), error)
})

test_that("equality is known to fail when comparing bytes to other encodings", {
  error <- "translating strings with \"bytes\" encoding"

  for (enc in encodings()) {
    expect_error(vec_equal(encoding_bytes(), enc), error)
    expect_error(vec_equal(enc, encoding_bytes()), error)
  }
})

test_that("`na_equal` is validated", {
  expect_snapshot(error = TRUE, {
    vec_equal(1, 1, na_equal = 1)
  })
  expect_snapshot(error = TRUE, {
    vec_equal(1, 1, na_equal = c(TRUE, FALSE))
  })
})

test_that("can compare lists of expressions", {
  x <- list(expression(x), expression(y))
  y <- list(expression(x))

  expect_equal(vec_equal(x, y), c(TRUE, FALSE))
})

test_that("vec_equal() silently falls back to base data frame", {
  expect_silent(expect_identical(
    vec_equal(foobar(mtcars), foobar(tibble::as_tibble(mtcars))),
    rep(TRUE, 32)
  ))
})

test_that("recycling works in all cases", {
  # Both size 1 is its own path. Both "recycle".
  x <- 1
  y <- 1
  expect_identical(vec_equal(x, y), TRUE)

  x <- 1
  y <- 1:2
  expect_identical(vec_equal(x, y), c(TRUE, FALSE))

  x <- 1:2
  y <- 1
  expect_identical(vec_equal(x, y), c(TRUE, FALSE))

  x <- 1:2
  y <- 1:2
  expect_identical(vec_equal(x, y), c(TRUE, TRUE))

  # Again, with data frames

  x <- data.frame(x = 1, y = 2)
  y <- data.frame(x = 1, y = 2)
  expect_identical(vec_equal(x, y), TRUE)

  x <- data.frame(x = 1, y = 2)
  y <- data.frame(x = 1:2, y = 2:3)
  expect_identical(vec_equal(x, y), c(TRUE, FALSE))

  x <- data.frame(x = 1:2, y = 2:3)
  y <- data.frame(x = 1, y = 2)
  expect_identical(vec_equal(x, y), c(TRUE, FALSE))

  x <- data.frame(x = 1:2, y = 2:3)
  y <- data.frame(x = 1:2, y = 2:3)
  expect_identical(vec_equal(x, y), c(TRUE, TRUE))
})

test_that("equal if attributes are in different order (treated like a map)", {
  x1 <- structure(1, foo = 1, bar = 2)
  x2 <- structure(1, bar = 2, foo = 1)
  expect_true(vec_equal(x1, x2))

  x1 <- list(structure(expression(x = 1), foo = 1, bar = 2))
  x2 <- list(structure(expression(x = 1), bar = 2, foo = 1))
  expect_true(vec_equal(x1, x2))

  x1 <- list(structure(pairlist(x = 1), foo = 1, bar = 2))
  x2 <- list(structure(pairlist(x = 1), bar = 2, foo = 1))
  expect_true(vec_equal(x1, x2))

  # Want the same srcref
  x0 <- function() {}
  x1 <- list(structure(x0, foo = 1, bar = 2))
  x2 <- list(structure(x0, bar = 2, foo = 1))
  expect_true(vec_equal(x1, x2))
})

# object ------------------------------------------------------------------

test_that("can compare NULL", {
  expect_true(obj_equal(NULL, NULL))
})

test_that("can compare objects with reference semantics", {
  expect_true(obj_equal(globalenv(), globalenv()))
  expect_false(obj_equal(globalenv(), environment()))

  expect_true(obj_equal(quote(x), quote(x)))
  expect_false(obj_equal(quote(x), quote(y)))
})

test_that("can compare pairlists", {
  expect_true(obj_equal(quote(x + y), quote(x + y)))
  expect_true(obj_equal(pairlist(x = 1, y = 2), pairlist(x = 1, y = 2)))
})

test_that("can compare functions", {
  f1 <- function(x, y) x + y
  f2 <- function(x, y) x + y
  expect_false(obj_equal(f2, f1))

  attr(f1, "srcref") <- NULL
  attr(f2, "srcref") <- NULL
  expect_true(obj_equal(f2, f1))

  f3 <- f1
  formals(f3) <- alist(x = 1)
  expect_false(obj_equal(f3, f1))

  f4 <- f1
  body(f4) <- quote(x)
  expect_false(obj_equal(f4, f2))
})

test_that("not equal if different types or lengths", {
  expect_false(obj_equal(1, 2))
  expect_false(obj_equal(1:2, 1))
})

test_that("not equal if attributes not equal", {
  x1 <- structure(1:10, x = 1, y = 2)
  x2 <- structure(1:10, x = 1, y = 3)
  expect_false(obj_equal(x1, x2))
})

test_that("not equal if only one has attributes", {
  x1 <- structure(1, foo = 1)
  x2 <- 1
  expect_false(obj_equal(x1, x2))
  expect_false(obj_equal(x2, x1))
})

test_that("not equal if attribute tag names are different", {
  x1 <- structure(1, foo = 2)
  x2 <- structure(1, bar = 2)
  expect_false(obj_equal(x1, x2))
})

test_that("not equal if one has more attributes than the other", {
  x1 <- structure(1, foo = 1)
  x2 <- structure(1, foo = 1, bar = 2)
  expect_false(obj_equal(x1, x2))
  expect_false(obj_equal(x2, x1))
})

test_that("equal if attributes are in different order (treated like a map)", {
  x1 <- structure(1, foo = 1, bar = 2)
  x2 <- structure(1, bar = 2, foo = 1)
  expect_true(obj_equal(x1, x2))

  x1 <- structure(expression(x = 1), foo = 1, bar = 2)
  x2 <- structure(expression(x = 1), bar = 2, foo = 1)
  expect_true(obj_equal(x1, x2))

  x1 <- structure(pairlist(x = 1), foo = 1, bar = 2)
  x2 <- structure(pairlist(x = 1), bar = 2, foo = 1)
  expect_true(obj_equal(x1, x2))

  # Want the same srcref
  x0 <- function() {}
  x1 <- structure(x0, foo = 1, bar = 2)
  x2 <- structure(x0, bar = 2, foo = 1)
  expect_true(obj_equal(x1, x2))
})

test_that("can compare expressions", {
  expect_true(obj_equal(expression(x), expression(x)))
  expect_false(obj_equal(expression(x), expression(y)))

  # Attributes on the expression vectors
  expect_true(obj_equal(
    vec_set_attributes(expression(x), list(foo = 1)),
    vec_set_attributes(expression(x), list(foo = 1))
  ))
  expect_false(obj_equal(
    vec_set_attributes(expression(x), list(foo = 1)),
    vec_set_attributes(expression(x), list(foo = 2))
  ))

  # Length check
  expect_true(obj_equal(expression(x, y), expression(x, y)))
  expect_false(obj_equal(expression(x, y), expression(x)))
})

test_that("language argument names are considered", {
  expect_false(obj_equal(
    call("fn", foo = 1),
    call("fn", bar = 1)
  ))
})

test_that("pairlist tags are considered", {
  expect_false(obj_equal(
    pairlist(foo = 1),
    pairlist(bar = 1)
  ))
})

test_that("attribute pairlist tags are considered", {
  expect_false(obj_equal(
    structure(1, foo = 1),
    structure(1, bar = 1)
  ))
})

# na ----------------------------------------------------------------------

test_that("NA propagate symmetrically (#204)", {
  exp <- c(NA, NA)

  expect_identical(vec_equal(c(TRUE, FALSE), NA), exp)
  expect_identical(vec_equal(1:2, NA), exp)
  expect_identical(vec_equal(c(1, 2), NA), exp)
  expect_identical(vec_equal(letters[1:2], NA), exp)

  expect_identical(vec_equal(NA, c(TRUE, FALSE)), exp)
  expect_identical(vec_equal(NA, 1:2), exp)
  expect_identical(vec_equal(NA, c(1, 2)), exp)
  expect_identical(vec_equal(NA, letters[1:2]), exp)
})

test_that("NA propagate from data frames columns", {
  x <- data.frame(x = 1:3)
  y <- data.frame(x = c(1L, NA, 2L))

  expect_identical(vec_equal(x, y), c(TRUE, NA, FALSE))
  expect_identical(vec_equal(y, x), c(TRUE, NA, FALSE))

  expect_identical(vec_equal(x, y, na_equal = TRUE), c(TRUE, FALSE, FALSE))
  expect_identical(vec_equal(y, x, na_equal = TRUE), c(TRUE, FALSE, FALSE))

  x <- data.frame(x = 1:3, y = 1:3)
  y <- data.frame(x = c(1L, NA, 2L), y = c(NA, 2L, 3L))

  expect_identical(vec_equal(x, y), c(NA, NA, FALSE))
  expect_identical(vec_equal(y, x), c(NA, NA, FALSE))

  expect_identical(vec_equal(x, y, na_equal = TRUE), c(FALSE, FALSE, FALSE))
  expect_identical(vec_equal(y, x, na_equal = TRUE), c(FALSE, FALSE, FALSE))
})

test_that("NA do not propagate from list components (#662)", {
  expect_true(obj_equal(NA, NA))
  expect_true(vec_equal(list(NA), list(NA)))
})

test_that("NA do not propagate from names when comparing objects", {
  x <- set_names(1:3, c("a", "b", NA))
  y <- set_names(1:3, c("a", NA, NA))

  expect_true(obj_equal(x, x))
  expect_false(obj_equal(x, y))

  expect_equal(vec_equal(list(x, x, y), list(x, y, y)), c(TRUE, FALSE, TRUE))
})

test_that("NA do not propagate from attributes", {
  x <- structure(1:3, foo = NA)
  y <- structure(1:3, foo = "")
  expect_true(obj_equal(x, x))
  expect_false(obj_equal(x, y))
})

test_that("NA do not propagate from function bodies or formals", {
  fn <- other <- function() NA
  body(other) <- TRUE

  expect_true(vec_equal(list(fn), list(fn)))
  expect_false(vec_equal(list(fn), list(other)))
  expect_true(obj_equal(fn, fn))
  expect_false(obj_equal(fn, other))

  fn <- other <- function(x = NA) NULL
  formals(other) <- list(x = NULL)

  expect_true(vec_equal(list(fn), list(fn)))
  expect_false(vec_equal(list(fn), list(other)))
})

test_that("can check equality of unspecified objects", {
  expect_equal(vec_equal(NA, NA), NA)
  expect_true(vec_equal(NA, NA, na_equal = TRUE))

  expect_equal(vec_equal(unspecified(1), unspecified(1)), NA)
  expect_true(vec_equal(unspecified(1), unspecified(1), na_equal = TRUE))

  expect_equal(vec_equal(NA, unspecified(1)), NA)
  expect_true(vec_equal(NA, unspecified(1), na_equal = TRUE))
})

test_that("can't supply NA as `na_equal`", {
  expect_snapshot(error = TRUE, {
    vec_equal(NA, NA, na_equal = NA)
  })
})


# proxy -------------------------------------------------------------------

test_that("vec_equal() takes vec_proxy() by default", {
  local_env_proxy()
  x <- new_proxy(1:3)
  y <- new_proxy(3:1)
  expect_identical(vec_equal(x, y), lgl(FALSE, TRUE, FALSE))
})

test_that("vec_equal() takes vec_proxy_equal() if implemented", {
  local_comparable_tuple()

  x <- tuple(1:3, 1:3)
  y <- tuple(1:3, 4:6)

  expect_identical(x == y, rep(TRUE, 3))
  expect_identical(vec_equal(x, y), rep(TRUE, 3))

  # Recursive case
  foo <- data_frame(x = x)
  bar <- data_frame(x = y)
  expect_identical(vec_equal(foo, bar), rep(TRUE, 3))
})

# All methods in this file are conditionally registered in .onLoad()

### `grouped_df` -----------------------------------------------------

group_intersect <- function(x, new) {
  intersect(dplyr::group_vars(x), names(new))
}

vec_restore_grouped_df <- function(x, to, ...) {
  vars <- group_intersect(to, x)
  drop <- dplyr::group_by_drop_default(to)
  dplyr::grouped_df(x, vars, drop = drop)
}


# `vec_ptype2()` -----------------------------------------------------

vec_ptype2_grouped_df_grouped_df <- function(x, y, ...) {
  gdf_ptype2(x, y, ...)
}

vec_ptype2_grouped_df_data.frame <- function(x, y, ...) {
  gdf_ptype2(x, y, ...)
}
vec_ptype2_data.frame_grouped_df <- function(x, y, ...) {
  gdf_ptype2(x, y, ...)
}

vec_ptype2_grouped_df_tbl_df <- function(x, y, ...) {
  gdf_ptype2(x, y, ...)
}
vec_ptype2_tbl_df_grouped_df <- function(x, y, ...) {
  gdf_ptype2(x, y, ...)
}

gdf_ptype2 <- function(x, y, ...) {
  common <- df_ptype2(x, y, ...)

  x_vars <- dplyr::group_vars(x)
  y_vars <- dplyr::group_vars(y)
  vars <- union(x_vars, y_vars)

  drop <- dplyr::group_by_drop_default(x) && dplyr::group_by_drop_default(y)

  dplyr::grouped_df(common, vars, drop = drop)
}


# `vec_cast()` -------------------------------------------------------

vec_cast_grouped_df_grouped_df <- function(x, to, ...) {
  gdf_cast(x, to, ...)
}

vec_cast_grouped_df_data.frame <- function(x, to, ...) {
  gdf_cast(x, to, ...)
}
vec_cast_data.frame_grouped_df <- function(x, to, ...) {
  df_cast(x, to, ...)
}

vec_cast_grouped_df_tbl_df <- function(x, to, ...) {
  gdf_cast(x, to, ...)
}
vec_cast_tbl_df_grouped_df <- function(x, to, ...) {
  tib_cast(x, to, ...)
}

gdf_cast <- function(x, to, ...) {
  df <- df_cast(x, to, ...)

  vars <- dplyr::group_vars(to)
  drop <- dplyr::group_by_drop_default(to)

  dplyr::grouped_df(df, vars, drop = drop)
}


### `rowwise` --------------------------------------------------------

vec_restore_rowwise_df <- function(x, to, ...) {
  dplyr::rowwise(x)
}


# `vec_ptype2()` -----------------------------------------------------

vec_ptype2_rowwise_df_rowwise_df <- function(x, y, ...) {
  rww_ptype2(x, y, ...)
}

vec_ptype2_rowwise_df_data.frame <- function(x, y, ...) {
  rww_ptype2(x, y, ...)
}
vec_ptype2_data.frame_rowwise_df <- function(x, y, ...) {
  rww_ptype2(x, y, ...)
}

vec_ptype2_rowwise_df_tbl_df <- function(x, y, ...) {
  rww_ptype2(x, y, ...)
}
vec_ptype2_tbl_df_rowwise_df <- function(x, y, ...) {
  rww_ptype2(x, y, ...)
}

rww_ptype2 <- function(x, y, ...) {
  dplyr::rowwise(df_ptype2(x, y, ...))
}


# `vec_cast()` -------------------------------------------------------

vec_cast_rowwise_df_rowwise_df <- function(x, to, ...) {
  rww_cast(x, to, ...)
}

vec_cast_rowwise_df_data.frame <- function(x, to, ...) {
  rww_cast(x, to, ...)
}
vec_cast_data.frame_rowwise_df <- function(x, to, ...) {
  df_cast(x, to, ...)
}

vec_cast_rowwise_df_tbl_df <- function(x, to, ...) {
  rww_cast(x, to, ...)
}
vec_cast_tbl_df_rowwise_df <- function(x, to, ...) {
  tib_cast(x, to, ...)
}

rww_cast <- function(x, to, ...) {
  dplyr::rowwise(df_cast(x, to, ...))
}

#' Factor/ordered factor S3 class
#'
#' A [factor] is an integer with attribute `levels`, a character vector. There
#' should be one level for each integer between 1 and `max(x)`.
#' An [ordered] factor has the same properties as a factor, but possesses
#' an extra class that marks levels as having a total ordering.
#'
#' These functions help the base factor and ordered factor classes fit in to
#' the vctrs type system by providing constructors, coercion functions,
#' and casting functions. `new_factor()` and `new_ordered()` are low-level
#' constructors - they only check that types, but not values, are valid, so
#' are for expert use only.
#'
#' @param x Integer values which index in to `levels`.
#' @param levels Character vector of labels.
#' @param ...,class Used to for subclasses.
#' @keywords internal
#' @export
new_factor <- function(
  x = integer(),
  levels = character(),
  ...,
  class = character()
) {
  stopifnot(is.integer(x))
  stopifnot(is.character(levels))

  structure(
    x,
    levels = levels,
    ...,
    class = c(class, "factor")
  )
}

#' @export
#' @rdname new_factor
new_ordered <- function(x = integer(), levels = character()) {
  new_factor(x = x, levels = levels, class = "ordered")
}

#' @export
vec_proxy.factor <- function(x, ...) {
  x
}

#' @export
vec_proxy.ordered <- function(x, ...) {
  x
}

#' @export
vec_restore.factor <- function(x, to, ...) {
  NextMethod()
}

#' @export
vec_restore.ordered <- function(x, to, ...) {
  NextMethod()
}

# Print -------------------------------------------------------------------

#' @export
vec_ptype_full.factor <- function(x, ...) {
  paste0("factor<", hash_label(levels(x)), ">", vec_ptype_shape(x))
}

#' @export
vec_ptype_abbr.factor <- function(x, ...) {
  "fct"
}

#' @export
vec_ptype_full.ordered <- function(x, ...) {
  paste0("ordered<", hash_label(levels(x)), ">", vec_ptype_shape(x))
}

#' @export
vec_ptype_abbr.ordered <- function(x, ...) {
  "ord"
}

# Coerce ------------------------------------------------------------------

#' @rdname new_factor
#' @export vec_ptype2.factor
#' @method vec_ptype2 factor
#' @export
vec_ptype2.factor <- function(x, y, ...) {
  UseMethod("vec_ptype2.factor")
}
#' @export
vec_ptype2.factor.factor <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.factor.factor")
}
#' @export
vec_ptype2.character.factor <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.character.factor")
}
#' @export
vec_ptype2.factor.character <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.factor.character")
}

#' @rdname new_factor
#' @export vec_ptype2.ordered
#' @method vec_ptype2 ordered
#' @export
vec_ptype2.ordered <- function(x, y, ...) {
  UseMethod("vec_ptype2.ordered")
}
#' @export
vec_ptype2.ordered.ordered <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.ordered.ordered")
}
#' @export
vec_ptype2.ordered.character <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.ordered.character")
}
#' @export
vec_ptype2.character.ordered <- function(x, y, ...) {
  stop_native_implementation("vec_ptype2.character.ordered")
}

#' @export
vec_ptype2.ordered.factor <- function(x, y, ...) {
  vec_default_ptype2(x, y, ...)
}
#' @export
vec_ptype2.factor.ordered <- function(x, y, ...) {
  vec_default_ptype2(x, y, ...)
}


# Cast --------------------------------------------------------------------

#' @rdname new_factor
#' @export vec_cast.factor
#' @method vec_cast factor
#' @export
vec_cast.factor <- function(x, to, ...) {
  UseMethod("vec_cast.factor")
}

fct_cast <- function(x, to, ..., call = caller_env()) {
  fct_cast_impl(x, to, ..., ordered = FALSE, call = call)
}

fct_cast_impl <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  ordered = FALSE,
  call = caller_env()
) {
  if (length(levels(to)) == 0L) {
    levels <- levels(x)
    if (is.null(levels)) {
      exclude <- NA
      levels <- unique(x)
    } else {
      exclude <- NULL
    }
    factor(
      as.character(x),
      levels = levels,
      ordered = ordered,
      exclude = exclude
    )
  } else {
    lossy <- !(x %in% levels(to) | is.na(x))
    out <- factor(
      x,
      levels = levels(to),
      ordered = ordered,
      exclude = NULL
    )
    maybe_lossy_cast(
      out,
      x,
      to,
      lossy,
      loss_type = "generality",
      x_arg = x_arg,
      to_arg = to_arg,
      call = call
    )
  }
}

#' @export
vec_cast.factor.factor <- function(x, to, ...) {
  fct_cast(x, to, ...)
}
#' @export
vec_cast.factor.character <- function(x, to, ...) {
  fct_cast(x, to, ...)
}
#' @export
vec_cast.character.factor <- function(x, to, ...) {
  stop_native_implementation("vec_cast.character.factor")
}

#' @rdname new_factor
#' @export vec_cast.ordered
#' @method vec_cast ordered
#' @export
vec_cast.ordered <- function(x, to, ...) {
  UseMethod("vec_cast.ordered")
}

ord_cast <- function(x, to, ..., call = caller_env()) {
  fct_cast_impl(x, to, ..., ordered = TRUE, call = call)
}

#' @export
vec_cast.ordered.ordered <- function(x, to, ...) {
  ord_cast(x, to, ...)
}

#' @export
vec_cast.ordered.character <- function(x, to, ...) {
  ord_cast(x, to, ...)
}
#' @export
vec_cast.character.ordered <- function(x, to, ...) {
  stop_native_implementation("vec_cast.character.ordered")
}

# Math and arithmetic -----------------------------------------------------

#' @export
vec_math.factor <- function(.fn, .x, ...) {
  stop_unsupported(.x, .fn)
}

#' @export
vec_arith.factor <- function(op, x, y, ...) {
  stop_unsupported(x, op)
}

# Helpers -----------------------------------------------------------------

hash_label <- function(x, length = 5) {
  if (length(x) == 0) {
    ""
  } else {
    # Can't use obj_hash() because it hashes the string pointers
    # for performance, so the values in the test change each time
    substr(rlang::hash(x), 1, length)
  }
}

levels_union <- function(x, y) {
  union(levels(x), levels(y))
}

#' Get or set observations in a vector
#'
#' This provides a common interface to extracting and modifying observations
#' for all vector types, regardless of dimensionality. They are analogs to `[`
#' and `[<-` that match [vec_size()] instead of `length()`.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param x A vector
#'
#' @param i An integer, character or logical vector specifying the locations or
#'   names of the observations to get/set. Specify `TRUE` to index all elements
#'   (as in `x[]`), or `NULL`, `FALSE` or `integer()` to index none (as in
#'   `x[NULL]`).
#'
#' @param value A vector of replacement values
#'
#'   `value` is cast to the type of `x`.
#'
#'   If `slice_value = FALSE`, `value` must be size 1 or the same size as `i`
#'   after `i` has been converted to a positive integer location vector with
#'   [vec_as_location()] (which may not be the same size as `i` originally).
#'
#'   If `slice_value = TRUE`, `value` must be size 1 or the same size as `x`.
#'
#' @param slice_value A boolean. If `TRUE`, the assignment proceeds as if you
#'   had provided `vec_slice(x, i) <- vec_slice(value, i)`, but is optimized to
#'   avoid materializing the slice of `value`.
#'
#' @param x_arg,value_arg Argument names for `x` and `value`. These are used
#'   in error messages to inform the user about the locations of
#'   incompatible types and sizes (see [stop_incompatible_type()] and
#'   [stop_incompatible_size()]).
#'
#' @return A vector of the same type as `x`.
#'
#' @section Genericity:
#'
#' Support for S3 objects depends on whether the object implements a
#' [vec_proxy()] method.
#'
#' * When a `vec_proxy()` method exists, the proxy is sliced or assigned to and
#'   `vec_restore()` is called on the result.
#'
#' * Otherwise, `vec_slice()` falls back to the base generic `[` and
#'   `vec_slice<-()` falls back to the base generic `[<-`.
#'
#' When `vec_slice<-()` falls back to `[<-`, it is expected that the subclass's
#' `[<-` method can handle the following subset of cases that base R's `[<-`
#' can also handle:
#'
#' * An `i` vector of positive integer positions (notably excluding `NA`).
#'
#' * A `value` vector of length 1 or length `length(i)`. If length 1, it
#'   should be recycled by the `[<-` method to the length of `i`.
#'
#' If your `[<-` method eventually calls base R's native `[<-` code, then these
#' cases will be handled for you.
#'
#' Note that S3 lists are treated as scalars by default, and will
#' cause an error if they don't implement a [vec_proxy()] method.
#'
#' @section Differences with base R subsetting:
#'
#' * `vec_slice()` only slices along one dimension. For
#'   two-dimensional types, the first dimension is subsetted.
#'
#' * `vec_slice()` preserves attributes by default.
#'
#' * `vec_slice<-()` is type-stable and always returns the same type
#'   as the LHS.
#'
#' @section Dependencies:
#'
#' ## vctrs dependencies
#'
#' - [vec_proxy()]
#' - [vec_restore()]
#'
#' ## base dependencies
#'
#' - \code{base::`[`}
#' - \code{base::`[<-`}
#'
#' @export
#' @keywords internal
#' @examples
#' x <- sample(10)
#' x
#' vec_slice(x, 1:3)
#'
#' # You can assign with the infix variant:
#' vec_slice(x, 2) <- 100
#' x
#'
#' # Or with the regular variant that doesn't modify the original input:
#' y <- vec_assign(x, 3, 500)
#' y
#' x
#'
#'
#' # Slicing objects of higher dimension:
#' vec_slice(mtcars, 1:3)
#'
#' # Type stability --------------------------------------------------
#'
#' # The assign variant is type stable. It always returns the same
#' # type as the input.
#' x <- 1:5
#' vec_slice(x, 2) <- 20.0
#'
#' # `x` is still an integer vector because the RHS was cast to the
#' # type of the LHS:
#' vec_ptype(x)
#'
#' # Compare to `[<-`:
#' x[2] <- 20.0
#' vec_ptype(x)
#'
#'
#' # Note that the types must be coercible for the cast to happen.
#' # For instance, you can cast a double vector of whole numbers to an
#' # integer vector:
#' vec_cast(1, integer())
#'
#' # But not fractional doubles:
#' try(vec_cast(1.5, integer()))
#'
#' # For this reason you can't assign fractional values in an integer
#' # vector:
#' x <- 1:3
#' try(vec_slice(x, 2) <- 1.5)
#'
#' # Slicing `value` -------------------------------------------------
#'
#' # Sometimes both `x` and `value` start from objects that are the same length,
#' # and you need to slice `value` by `i` before assigning it to `x`. This comes
#' # up when thinking about how `base::ifelse()` and `dplyr::case_when()` work.
#' condition <- c(TRUE, FALSE, TRUE, FALSE)
#' yes <- 1:4
#' no <- 5:8
#'
#' # Create an output container and fill it
#' out <- vec_init(integer(), 4)
#' out <- vec_assign(out, condition, vec_slice(yes, condition))
#' out <- vec_assign(out, !condition, vec_slice(no, !condition))
#' out
#'
#' # This is wasteful because you have to materialize the slices of `yes` and
#' # `no` before they can be assigned, and you also have to validate `condition`
#' # multiple times. Using `slice_value` internally performs
#' # `vec_slice(yes, condition)` and `vec_slice(no, !condition)` for you,
#' # but does so in a way that avoids the materialization.
#' out <- vec_init(integer(), 4)
#' out <- vec_assign(out, condition, yes, slice_value = TRUE)
#' out <- vec_assign(out, !condition, no, slice_value = TRUE)
#' out
vec_slice <- function(x, i, ..., error_call = current_env()) {
  check_dots_empty0(...)
  .Call(ffi_slice, x, i, environment())
}

# Called when `x` has dimensions
vec_slice_fallback <- function(x, i) {
  out <- unclass(vec_proxy(x))
  obj_check_vector(out)

  d <- vec_dim_n(out)
  if (d == 2) {
    out <- out[i, , drop = FALSE]
  } else {
    miss_args <- rep(list(missing_arg()), d - 1)
    out <- eval_bare(expr(out[i, !!!miss_args, drop = FALSE]))
  }

  vec_restore(out, x)
}

vec_slice_fallback_integer64 <- function(x, i) {
  d <- vec_dim_n(x)

  if (d == 2) {
    out <- x[i, , drop = FALSE]
  } else {
    miss_args <- rep(list(missing_arg()), d - 1)
    out <- eval_bare(expr(x[i, !!!miss_args, drop = FALSE]))
  }

  is_na <- is.na(i)

  if (!any(is_na)) {
    return(out)
  }

  if (d == 2) {
    out[is_na, ] <- bit64::NA_integer64_
  } else {
    eval_bare(expr(out[is_na, !!!miss_args] <- bit64::NA_integer64_))
  }

  out
}

# bit64::integer64() objects do not have support for `NA_integer_`
# slicing. This manually replaces the garbage values that are created
# any time a slice with `NA_integer_` is made.
vec_slice_dispatch_integer64 <- function(x, i) {
  out <- x[i]

  is_na <- is.na(i)

  if (!any(is_na)) {
    return(out)
  }

  out[is_na] <- bit64::NA_integer64_

  out
}

vec_slice_altrep <- function(x, i) {
  # We have already validated `i`, it is one of:
  # - Integer vector from `vec_as_location()`
  # - Integer vector from materializing a `compact_rep()`
  # - Integer vector from materializing a `compact_seq()`
  # - Logical vector from materializing a `compact_condition()`
  #   (which `VectorSubset()` will convert to an integer vector)

  # For the main case we care about (an ALTREP vector with an Extract_Subset
  # method, like vroom), `.subset()` will:
  # - Call `do_subset_dflt()` (bypassing S3 dispatch!)
  # - Call `VectorSubset()`
  # - Call `ExtractSubset()`
  # - Call `ALTVEC_EXTRACT_SUBSET()`
  # - If that returns `NULL`, i.e. if this ALTREP class has not implemented an
  #   ALTREP `Extract_Subset` method, then it will use the `Elt` method to
  #   subset
  .subset(x, i)
}


#' @rdname vec_slice
#' @export
`vec_slice<-` <- function(x, i, value) {
  x_arg <- "" # Substitution is `*tmp*`
  delayedAssign("value_arg", as_label(substitute(value)))
  slice_value <- FALSE
  .Call(ffi_assign, x, i, value, slice_value, environment())
}
#' @rdname vec_slice
#' @export
vec_assign <- function(
  x,
  i,
  value,
  ...,
  slice_value = FALSE,
  x_arg = "",
  value_arg = ""
) {
  check_dots_empty0(...)
  .Call(ffi_assign, x, i, value, slice_value, environment())
}

# Invariants for `[<-` methods:
#
# - `i` will contain positive integers
# - `i` will not contain `NA` (a base `[<-` issue gets in the way if we allow this)
# - `value` will be size 1 or size `length(i)`
#
# Given these invariants, the base `[<-` works the way we want it to in fallback
# methods, and we expect any subclasses to also uphold the same behavior as base
# `[<-` with these inputs. In other words, we don't expect subclasses to have
# vctrs subassign behavior, but we do expect them to match a subset of base R
# subassign behavior.
vec_assign_fallback <- function(x, i, value, slice_value, index_style) {
  if (index_style == "condition") {
    # Convert logical condition `i` to integer location `i`. Must use
    # `vec_as_location()` rather than `which()` because we want `NA` values to
    # propagate, which is consistent with how `value`'s size is checked
    # internally (i.e. when `slice_value = FALSE`, `value` was checked to have
    # size equal to the number of `TRUE` or `NA` values in `i`). Propagated
    # `NA`s are later dropped before calling `[<-` to work around a base `[<-`
    # issue, but we need them to slice both `i` and `value` consistently.
    i <- vec_as_location(i, n = vec_size(x), missing = "propagate")
  }

  if (slice_value && vec_size(value) != 1L) {
    # `value` has same size as `x` rather than same size as `i`.
    # We need to pre-slice it down to same size as `i` to match what `[<-` expects.
    # Effectively we are preemptively doing the RHS of this:
    # vec_slice(x, i) <- vec_slice(value, i)
    value <- vec_slice(value, i)
  }

  # Work around issue in base `[<-` that errors on `NA_integer_` in subassign
  # indices. Only do this as necessary, as `vec_slice()` will typically call
  # back to `[` when slicing `value` here, which is somewhat slow.
  if (vec_any_missing(i)) {
    existing <- !is.na(i)
    i <- vec_slice(i, existing)
    if (vec_size(value) != 1L) {
      value <- vec_slice(value, existing)
    }
  }

  d <- vec_dim_n(x)
  miss_args <- rep(list(missing_arg()), d - 1)
  eval_bare(expr(x[i, !!!miss_args] <- value))
  x
}

# `start` is 0-based
vec_assign_seq <- function(
  x,
  value,
  start,
  size,
  increasing = TRUE,
  slice_value = FALSE
) {
  .Call(ffi_assign_seq, x, value, start, size, increasing, slice_value)
}

# `start` is 0-based
compact_seq <- function(start, size, increasing = TRUE) {
  .Call(ffi_compact_seq, start, size, increasing)
}

vec_assign_compact_condition <- function(
  x,
  i,
  value,
  slice_value = FALSE
) {
  .Call(ffi_assign_compact_condition, x, i, value, slice_value)
}

as_compact_condition <- function(x) {
  .Call(ffi_as_compact_condition, x)
}

#' @param assign_names A boolean. If `TRUE`, will assign names from `value`
#'   onto `x` as well.
#'
#' @noRd
vec_assign_params <- function(
  x,
  i,
  value,
  ...,
  assign_names = FALSE,
  slice_value = FALSE,
  x_arg = "",
  value_arg = ""
) {
  check_dots_empty0(...)
  .Call(
    ffi_assign_params,
    x,
    i,
    value,
    assign_names,
    slice_value,
    environment()
  )
}

vec_remove <- function(x, i) {
  vec_slice(x, -vec_as_location(i, length(x), names(x)))
}

vec_index <- function(x, i, ...) {
  i <- maybe_missing(i, TRUE)
  out <- vec_slice(x, i)

  if (!dots_n(...)) {
    return(out)
  }

  # Need to unclass to avoid infinite recursion through `[`
  proxy <- vec_data(out)
  out <- proxy[, ..., drop = FALSE]

  vec_restore(out, x)
}

#' Initialize a vector
#'
#' @param x Template of vector to initialize.
#' @param n Desired size of result.
#' @export
#' @section Dependencies:
#' * vec_slice()
#' @examples
#' vec_init(1:10, 3)
#' vec_init(Sys.Date(), 5)
#'
#' # The "missing" value for a data frame is a row that is entirely missing
#' vec_init(mtcars, 2)
#'
#' # The "missing" value for a list is `NULL`
#' vec_init(list(), 3)
vec_init <- function(x, n = 1L) {
  .Call(ffi_init, x, n, environment())
}

# Exposed for testing (`start` is 0-based)
vec_slice_seq <- function(x, start, size, increasing = TRUE) {
  .Call(ffi_slice_seq, x, start, size, increasing)
}

# Exposed for testing (`i` is 1-based)
vec_slice_rep <- function(x, i, n) {
  .Call(ffi_slice_rep, x, i, n)
}

# Forwards arguments to `base::rep()`
base_vec_rep <- function(x, ...) {
  i <- rep(seq_len(vec_size(x)), ...)
  vec_slice(x, i)
}

# Emulates `length<-`
vec_size_assign <- function(x, n) {
  x_size <- vec_size(x)

  if (n > x_size) {
    i <- seq_len(x_size)
    i <- c(i, vec_init(int(), n - x_size))
  } else {
    i <- seq_len(n)
  }

  vec_slice(x, i)
}

test_that("ptype2 base methods are not inherited", {
  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
  for (ptype in ptypes) {
    x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
    expect_s3_class(vec_ptype2(x, x), "foobar")
    expect_error(vec_ptype2(x, ptype), class = "vctrs_error_incompatible_type")
    expect_error(vec_ptype2(ptype, x), class = "vctrs_error_incompatible_type")
  }
})

test_that("cast base methods are not inherited", {
  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
  for (ptype in ptypes) {
    x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
    expect_s3_class(vec_cast(ptype, x), "foobar")
    expect_error(vec_cast(x, ptype), class = "vctrs_error_incompatible_type")
  }
})

test_that("default cast allows objects with the same type", {
  x <- structure(1, class = c("foo", "double"))
  expect_equal(vec_cast(x, x), x)
})

# vec_shaped_ptype -------------------------------------------------------

test_that("array dimensions are preserved", {
  mat1 <- matrix(lgl(), nrow = 1, ncol = 1)
  mat2 <- matrix(lgl(), nrow = 2, ncol = 2)
  mat3 <- matrix(lgl(), nrow = 2, ncol = 3)

  expect_equal(vec_ptype2(mat1, mat1), matrix(lgl(), nrow = 0, ncol = 1))
  expect_equal(vec_ptype2(mat1, mat2), matrix(lgl(), nrow = 0, ncol = 2))
  expect_error(vec_ptype2(mat2, mat3), class = "vctrs_error_incompatible_type")
})

test_that("vec_shaped_ptype()", {
  int <- function(...) array(NA_integer_, c(...))

  expect_identical(vec_shaped_ptype(integer(), int(5), int(10)), new_shape(integer()))
  expect_identical(vec_shaped_ptype(integer(), int(5, 1), int(10, 1)), new_shape(integer(), 1))
  expect_identical(vec_shaped_ptype(integer(), int(5, 1, 2), int(10, 1, 2)), new_shape(integer(), 1:2))
})

test_that("vec_shaped_ptype() evaluates arg lazily", {
  expect_silent(vec_shaped_ptype(integer(), int(5), int(10), x_arg = print("oof")))
  expect_silent(vec_shaped_ptype(integer(), int(5), int(10), y_arg = print("oof")))
})

# vec_cast() --------------------------------------------------------------

# NULL

test_that("NULL is idempotent", {
  expect_equal(vec_cast(NULL, NULL), NULL)
  expect_equal(vec_cast(list(1:3), NULL), list(1:3))
})


# Logical

test_that("safe casts work as expected", {
  exp <- lgl(TRUE, FALSE)
  expect_equal(vec_cast(NULL, logical()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), logical()), exp)
  expect_equal(vec_cast(int(1L, 0L), logical()), exp)
  expect_equal(vec_cast(dbl(1, 0), logical()), exp)

  # These used to be allowed
  expect_error(vec_cast(chr("T", "F"), logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(chr("TRUE", "FALSE"), logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(chr("true", "false"), logical()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(1, 0), logical()), class = "vctrs_error_incompatible_type")
})

test_that("NA casts work as expected", {
  exp <- lgl(NA)
  to <- lgl()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(lgl(NA))
  to_mat <- matrix(lgl())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
})

test_that("lossy casts generate warning", {
  expect_lossy(vec_cast(int(2L, 1L), lgl()), lgl(TRUE, TRUE), x = int(),  to = lgl())
  expect_lossy(vec_cast(dbl(2, 1), lgl()), lgl(TRUE, TRUE), x = dbl(),  to = lgl())

  # These used to be allowed
  expect_error(vec_cast(chr("x", "TRUE"), lgl()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(chr("t", "T"), lgl()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(chr("f", "F"), lgl()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(c(TRUE, FALSE), TRUE), lgl()), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(factor("a"), logical()), class = "vctrs_error_incompatible_type")
})

test_that("dimensionality matches output" ,{
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))

  x <- matrix(1, nrow = 2, ncol = 2)
  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_type")
})

test_that("the common type of two `NA` vectors is unspecified", {
  expect_equal(vec_ptype2(NA, NA), unspecified())
})


# Integer

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, integer()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), integer()), int(1L, 0L))
  expect_equal(vec_cast(int(1L, 2L), integer()), int(1L, 2L))
  expect_equal(vec_cast(dbl(1, 2), integer()), int(1L, 2L))

  # These used to be allowed
  expect_error(vec_cast(chr("1", "2"), integer()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(1L, 2L), integer()), class = "vctrs_error_incompatible_type")
})

test_that("NA casts work as expected", {
  exp <- int(NA)
  to <- int()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(int(NA))
  to_mat <- matrix(int())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
})

test_that("lossy casts generate error", {
  expect_lossy(vec_cast(c(2.5, 2), int()),     int(2, 2), x = dbl(), to = int())
  expect_lossy(vec_cast(c(.Machine$integer.max + 1, 1), int()),  int(NA, 1L), x = dbl(), to = int())
  expect_lossy(vec_cast(c(-.Machine$integer.max - 1, 1), int()), int(NA, 1L), x = dbl(), to = int())

  # These used to be allowed
  expect_error(vec_cast(c("2.5", "2"), int()), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(factor("a"), integer()), class = "vctrs_error_incompatible_type")
})


# Double

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, double()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), double()), dbl(1, 0))
  expect_equal(vec_cast(int(1, 0), double()), dbl(1, 0))
  expect_equal(vec_cast(dbl(1, 1.5), double()), dbl(1, 1.5))

  # These used to be allowed
  expect_error(vec_cast(chr("1", "1.5"), double()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(1, 1.5), double()), class = "vctrs_error_incompatible_type")
})

test_that("NA casts work as expected", {
  exp <- dbl(NA)
  to <- dbl()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(dbl(NA))
  to_mat <- matrix(dbl())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(vec_cast(mat(chr(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(factor("a"), double()), class = "vctrs_error_incompatible_type")
})


# Complex

test_that("safe casts to complex works", {
  expect_identical(vec_cast(NULL, cpl()), NULL)
  expect_identical(vec_cast(lgl(TRUE, FALSE), cpl()), cpl(1, 0))
  expect_identical(vec_cast(int(1, 0), cpl()), cpl(1, 0))
  expect_identical(vec_cast(dbl(1, 1.5), cpl()), cpl(1, 1.5))

  # This used to be allowed
  expect_error(vec_cast(list(1, 1.5), cpl()), class = "vctrs_error_incompatible_type")
})

test_that("NA casts work as expected", {
  # This goes through a special path for <unspecified>
  expect_equal(vec_cast(lgl(NA), cpl()), NA_complex_)

  # TODO: Use our own cast routines here?
  # It isn't great that this logical `NA` cast returns a different `NA`
  # than the one above with just `lgl(NA)` (which is seen as unspecified). i.e.
  # check the `Im()` slot between the two in R >=4.4.0. We can fix this with our
  # own cast routines rather than using `vec_coerce_bare()`.
  expect_type(vec_cast(lgl(NA, TRUE), cpl()), "complex")
  expect_identical(is.na(vec_cast(lgl(NA, TRUE), cpl())), c(TRUE, FALSE))

  # TODO: Use our own cast routines here?
  # `as.complex(NA/NA_real_/NA_integer_)` and `Rf_CoerceVector(NA/NA_real_/NA_integer_)`
  # have gone back and forth about what they return in the `Im()` slot. In some
  # R versions they return `0` and in others they return `NA_real_`.
  # https://stat.ethz.ch/pipermail/r-devel/2023-April/082545.html
  # https://stat.ethz.ch/pipermail/r-devel/2023-September/082864.html
  # expect_equal(vec_cast(int(NA), cpl()), NA_complex_)
  expect_type(vec_cast(int(NA), cpl()), "complex")
  expect_identical(is.na(vec_cast(int(NA), cpl())), TRUE)

  # expect_equal(vec_cast(dbl(NA), cpl()), NA_complex_)
  expect_type(vec_cast(dbl(NA), cpl()), "complex")
  expect_identical(is.na(vec_cast(dbl(NA), cpl())), TRUE)

  # This used to be allowed
  expect_error(vec_cast(list(NA), cpl()), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(NA_complex_)
  to_mat <- matrix(cpl())

  # TODO: Use our own cast routines here?
  # `as.complex(NA/NA_real_/NA_integer_)` and `Rf_CoerceVector(NA/NA_real_/NA_integer_)`
  # have gone back and forth about what they return in the `Im()` slot. In some
  # R versions they return `0` and in others they return `NA_real_`.
  # https://stat.ethz.ch/pipermail/r-devel/2023-April/082545.html
  # https://stat.ethz.ch/pipermail/r-devel/2023-September/082864.html

  # expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(lgl(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(lgl(NA)), to_mat)), matrix(TRUE))

  # expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(int(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(int(NA)), to_mat)), matrix(TRUE))

  # expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(dbl(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(dbl(NA)), to_mat)), matrix(TRUE))

  # This used to be allowed
  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
})

test_that("complex is coercible to numeric types", {
  expect_identical(vec_ptype2(cpl(), NULL), cpl())
  expect_identical(vec_ptype2(NULL, cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), int()), cpl())
  expect_identical(vec_ptype2(int(), cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), dbl()), cpl())
  expect_identical(vec_ptype2(dbl(), cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), cpl()), cpl())

  expect_identical(vec_c(0, 1i), cpl(0i, 1i))
})

test_that("complex is not coercible to logical", {
  expect_error(vec_ptype2(cpl(), lgl()), class = "vctrs_error_incompatible_type")
  expect_error(vec_ptype2(lgl(), cpl()), class = "vctrs_error_incompatible_type")
})


# Character

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, character()), NULL)
  expect_equal(vec_cast(NA, character()), NA_character_)

  # These used to be allowed
  expect_error(vec_cast(lgl(TRUE, FALSE), character()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list("x", "y"), character()), class = "vctrs_error_incompatible_type")
})

test_that("NA casts work as expected", {
  exp <- chr(NA)
  to <- chr()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(chr(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(int(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(dbl(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(chr(NA))
  to_mat <- matrix(chr())

  expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(vec_cast(mat(lgl(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(int(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(dbl(NA)), to_mat), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(mat(list(NA)), to_mat), class = "vctrs_error_incompatible_type")
})

test_that("difftime does not get special treatment", {
  dt1 <- as.difftime(600, units = "secs")
  # This used to be allowed
  expect_error(vec_cast(dt1, character()), class = "vctrs_error_incompatible_type")
})


# Raw

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, raw()), NULL)

  # This used to be allowed
  expect_error(vec_cast(list(raw(1)), raw()), class = "vctrs_error_incompatible_type")
})

test_that("invalid casts generate error", {
  expect_error(vec_cast(raw(1), double()), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(double(1), raw()), class = "vctrs_error_incompatible_type")
})

test_that("can sort raw", {
  x <- as.raw(c(3, 1, 2, 4))
  expect_identical(vec_order(x), int(2, 3, 1, 4))
  expect_identical(x[vec_order(x)], as.raw(1:4))
})

test_that("raw has informative type summaries", {
  expect_equal(vec_ptype_abbr(raw()), "raw")
  expect_equal(vec_ptype_full(raw()), "raw")
})

test_that("can provide common type with raw", {
  local_methods(
    vec_ptype2.raw.vctrs_foobar = function(...) "dispatched-left",
    vec_ptype2.vctrs_foobar = function(...) NULL,
    vec_ptype2.vctrs_foobar.raw = function(...) "dispatched-right"
  )
  expect_identical(vec_ptype2(raw(), foobar("")), "dispatched-left")
  expect_identical(vec_ptype2(foobar(""), raw()), "dispatched-right")
})


# Lists

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, list()), NULL)
  expect_equal(vec_cast(NA, list()), list(NULL))
  expect_equal(vec_cast(list(1L, 2L), list()), list(1L, 2L))

  # This used to be allowed
  expect_error(vec_cast(1:2, list()), class = "vctrs_error_incompatible_type")
})

test_that("dimensionality matches to" ,{
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1L, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))
})

test_that("data frames are cast to list row wise (#639)", {
  x <- data.frame(x = 1:2, row.names = c("a", "b"))
  expect <- list(data.frame(x = 1L), data.frame(x = 2L))

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})

test_that("data frames can be cast to shaped lists", {
  to <- array(list(), dim = c(0, 2, 1))
  x <- data.frame(x = 1:2, y = 3:4)

  expect <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect <- array(expect, dim = c(2, 2, 1))

  # This used to be allowed
  expect_error(vec_cast(x, to), class = "vctrs_error_incompatible_type")
})

test_that("Casting atomic `NA` values to list results in a `NULL`", {
  x <- c(NA, 1)
  expect <- list(NULL, 1)

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})

test_that("Casting data frame `NA` rows to list results in a `NULL`", {
  x <- data.frame(x = c(NA, NA, 1), y = c(NA, 1, 2))
  expect <- list(NULL, vec_slice(x, 2), vec_slice(x, 3))

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})


# Unspecified

test_that("unspecified can be cast to bare methods", {
  for (x in vectors[-4]) {
    expect_identical(vec_cast(unspecified(3), x), vec_init(x, 3))
  }
})

test_that("vec_slice throws error with non-vector inputs", {
  expect_error(vec_slice(environment(), 1L), class = "vctrs_error_scalar_type")
})

test_that("vec_slice throws error with non-vector subscripts", {
  expect_snapshot({
    (expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_subscript_type"))
    (expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type"))
  })
})

test_that("can subset base vectors", {
  i <- 2:3
  expect_identical(vec_slice(lgl(1, 0, 1), i), lgl(0, 1))
  expect_identical(vec_slice(int(1, 2, 3), i), int(2, 3))
  expect_identical(vec_slice(dbl(1, 2, 3), i), dbl(2, 3))
  expect_identical(vec_slice(cpl(1, 2, 3), i), cpl(2, 3))
  expect_identical(vec_slice(chr("1", "2", "3"), i), chr("2", "3"))
  expect_identical(vec_slice(raw2(1, 2, 3), i), raw2(2, 3))
  expect_identical(vec_slice(list(1, 2, 3), i), list(2, 3))
})

test_that("can subset shaped base vectors", {
  i <- 2:3
  mat <- as.matrix
  expect_identical(vec_slice(mat(lgl(1, 0, 1)), i), mat(lgl(0, 1)))
  expect_identical(vec_slice(mat(int(1, 2, 3)), i), mat(int(2, 3)))
  expect_identical(vec_slice(mat(dbl(1, 2, 3)), i), mat(dbl(2, 3)))
  expect_identical(vec_slice(mat(cpl(1, 2, 3)), i), mat(cpl(2, 3)))
  expect_identical(vec_slice(mat(chr("1", "2", "3")), i), mat(chr("2", "3")))
  expect_identical(vec_slice(mat(raw2(1, 2, 3)), i), mat(raw2(2, 3)))
  expect_identical(vec_slice(mat(list(1, 2, 3)), i), mat(list(2, 3)))
})

test_that("can subset with missing indices", {
  for (i in list(int(2L, NA), lgl(FALSE, TRUE, NA))) {
    expect_identical(vec_slice(lgl(1, 0, 1), i), lgl(0, NA))
    expect_identical(vec_slice(int(1, 2, 3), i), int(2, NA))
    expect_identical(vec_slice(dbl(1, 2, 3), i), dbl(2, NA))
    expect_identical(vec_slice(cpl2(1, 2, 3), i), cpl2(2, NA))
    expect_identical(vec_slice(chr("1", "2", "3"), i), c("2", NA))
    expect_identical(vec_slice(raw2(1, 2, 3), i), raw2(2, 0))
    expect_identical(vec_slice(list(1, 2, 3), i), list(2, NULL))
  }
})

test_that("can subset with a recycled NA", {
  local_name_repair_quiet()

  expect_identical(vec_slice(1:3, NA), int(NA, NA, NA))
  expect_identical(vec_slice(new_vctr(1:3), NA), new_vctr(int(NA, NA, NA)))

  rownames <- rep_len("", nrow(mtcars))
  rownames <- vec_as_names(rownames, repair = "unique")
  expect_identical(vec_slice(mtcars, NA), structure(mtcars[NA, ], row.names = rownames))
})

test_that("can subset with a recycled TRUE", {
  expect_identical(vec_slice(1:3, TRUE), 1:3)
  expect_identical(vec_slice(mtcars, TRUE), mtcars)
  expect_identical(vec_slice(new_vctr(1:3), TRUE), new_vctr(1:3))
  expect_identical(vec_as_location(TRUE, 2), 1:2)
})

test_that("can subset with a recycled FALSE", {
  expect_identical(vec_slice(1:3, FALSE), int())
  expect_identical(vec_slice(mtcars, FALSE), mtcars[NULL, ])
  expect_identical(vec_slice(new_vctr(1:3), FALSE), new_vctr(integer()))
})

test_that("can't index beyond the end of a vector", {
  expect_snapshot({
    (expect_error(vec_slice(1:2, 3L), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_slice(1:2, -3L), class = "vctrs_error_subscript_oob"))
  })
})

test_that("slicing non existing elements fails", {
  expect_error(vec_as_location("foo", 1L, "f"), class = "vctrs_error_subscript_oob")
  expect_error(vec_slice(c(f = 1), "foo"), class = "vctrs_error_subscript_oob")
})

test_that("can subset object of any dimensionality", {
  x0 <- c(1, 1)
  x1 <- ones(2)
  x2 <- ones(2, 3)
  x3 <- ones(2, 3, 4)
  x4 <- ones(2, 3, 4, 5)

  expect_equal(vec_slice(x0, 1L), 1)
  expect_identical(vec_slice(x1, 1L), ones(1))
  expect_identical(vec_slice(x2, 1L), ones(1, 3))
  expect_identical(vec_slice(x3, 1L), ones(1, 3, 4))
  expect_identical(vec_slice(x4, 1L), ones(1, 3, 4, 5))
})

test_that("can subset using logical subscript", {
  x0 <- c(1, 1)

  expect_identical(vec_slice(x0, TRUE), x0)
  expect_identical(vec_slice(x0, c(TRUE, FALSE)), 1)

  expect_error(
    vec_slice(x0, c(TRUE, FALSE, TRUE)),
    class = "vctrs_error_subscript_size"
  )

  expect_error(
    vec_slice(x0, lgl()),
    class = "vctrs_error_subscript_size"
  )

  expect_error(
    vec_slice(mtcars, c(TRUE, FALSE)),
    class = "vctrs_error_subscript_size"
  )
})

test_that("can subset data frame columns", {
  df <- data.frame(x = 1:2)
  df$y <- data.frame(a = 2:1)

  expect_equal(vec_slice(df, 1L)$y, vec_slice(df$y, 1L))
})

test_that("can subset empty data frames", {
  df <- new_data_frame(n = 3L)
  expect_equal(vec_size(vec_slice(df, integer())), 0)
  expect_equal(vec_size(vec_slice(df, 1L)), 1)
  expect_equal(vec_size(vec_slice(df, 1:3)), 3)

  df$df <- df
  expect_equal(vec_size(vec_slice(df, integer())), 0)
  expect_equal(vec_size(vec_slice(df, 1L)), 1)
  expect_equal(vec_size(vec_slice(df, 1:3)), 3)
})

test_that("ignores NA in logical subsetting", {
  x <- c(NA, 1, 2)
  expect_equal(vec_slice(x, x > 0), c(NA, 1, 2))
})

test_that("ignores NA in integer subsetting", {
  expect_equal(vec_slice(0:2, c(NA, 2:3)), c(NA, 1, 2))
})

test_that("can't slice with missing argument", {
  expect_error(vec_slice(1:3))
  expect_error(vec_slice(mtcars))
  expect_error(vec_slice(new_vctr(1:3)))
})

test_that("can slice with NULL argument", {
  expect_identical(vec_slice(1:3, NULL), integer())
  expect_identical(vec_slice(iris, NULL), iris[0, ])
  expect_identical(vec_slice(new_vctr(1:3), NULL), new_vctr(integer()))
})

test_that("slicing unclassed structures preserves attributes", {
  x <- structure(1:3, foo = "bar")
  expect_identical(vec_slice(x, 1L), structure(1L, foo = "bar"))
})

test_that("can slice with negative indices", {
  expect_identical(vec_slice(1:3, -c(1L, 3L)), 2L)
  expect_identical(vec_slice(mtcars, -(1:30)), vec_slice(mtcars, 31:32))

  expect_error(vec_slice(1:3, -c(1L, NA)), class = "vctrs_error_subscript_type")
  expect_error(vec_slice(1:3, c(-1L, 1L)), class = "vctrs_error_subscript_type")
})

test_that("0 is ignored in negative indices", {
  expect_identical(vec_slice(1:3, c(-2L, 0L)), int(1L, 3L))
  expect_identical(vec_slice(1:3, c(0L, -2L)), int(1L, 3L))
})

test_that("0 is ignored in positive indices", {
  expect_identical(vec_slice(1:3, 0L), int())
  expect_identical(vec_slice(1:3, c(0L, 0L)), int())
  expect_identical(vec_slice(1:3, c(0L, 2L, 0L)), 2L)
})

test_that("can slice with double indices", {
  expect_identical(vec_slice(1:3, dbl(2, 3)), 2:3)
  expect_snapshot((expect_error(vec_as_location(2^31, 3L), class = "vctrs_error_subscript_type")))
})

test_that("can slice with symbols", {
  expect_identical(vec_as_location(quote(b), 26, letters), 2L)
})

test_that("can `vec_slice()` S3 objects without dispatch infloop", {
  expect_identical(new_vctr(1:3)[1], new_vctr(1L))
  expect_identical(new_vctr(as.list(1:3))[1], new_vctr(list(1L)))
})

test_that("can `vec_slice()` records", {
  out <- vec_slice(new_rcrd(list(a = 1L, b = 2L)), rep(1, 3))
  expect_size(out, 3)

  out <- vec_init(new_rcrd(list(a = 1L, b = 2L)), 2)
  expect_size(out, 2)
})

test_that("vec_restore() is called after proxied slicing", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) "dispatch"
  )
  expect_identical(vec_slice(foobar(1:3), 2), "dispatch")
})

test_that("vec_slice() is proxied", {
  local_proxy()
  x <- vec_slice(new_proxy(1:3), 2:3)
  expect_identical(proxy_deref(x), 2:3)
})

test_that("dimensions are preserved by vec_slice()", {
  # Fallback case
  x <- foobar(1:4)
  dim(x) <- c(2, 2)
  dimnames(x) <- list(a = c("foo", "bar"), b = c("quux", "hunoz"))

  out <- vec_slice(x, 1)
  exp <- foobar(
    c(1L, 3L),
    dim = c(1, 2),
    dimnames = list(a = "foo", b = c("quux", "hunoz")
  ))
  expect_identical(out, exp)


  # Native case
  attrib <- NULL

  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) attrib <<- attributes(x)
  )

  vec_slice(x, 1)

  exp <- list(dim = 1:2, dimnames = list(a = "foo", b = c("quux", "hunoz")))
  expect_identical(attrib, exp)
})

test_that("can slice shaped objects by name", {
  x <- matrix(1:2)

  expect_error(vec_slice(x, "foo"), "unnamed")

  dimnames(x) <- list(c("foo", "bar"))

  expect_equal(vec_slice(x, "foo"), vec_slice(x, 1L))
  expect_error(vec_slice(x, "baz"), class = "vctrs_error_subscript_oob")
})

test_that("vec_slice() unclasses input before calling `vec_restore()`", {
  oo <- NULL
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, ...) oo <<- is.object(x)
  )

  x <- foobar(1:4)
  dim(x) <- c(2, 2)

  vec_slice(x, 1)
  expect_false(oo)
})

test_that("can call `vec_slice()` from `[` methods with shaped objects without infloop", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) vec_slice(x, i)
  )

  x <- foobar(1:4)
  dim(x) <- c(2, 2)

  exp <- foobar(c(1L, 3L))
  dim(exp) <- c(1, 2)
  expect_identical(x[1], exp)
})

test_that("vec_slice() restores attributes on shaped S3 objects correctly", {
  x <- factor(c("a", "b", "c", "d", "e", "f"))
  dim(x) <- c(3, 2)

  expect <- factor(c("a", "c", "d", "f"), levels = levels(x))
  dim(expect) <- c(2, 2)

  expect_identical(vec_slice(x, c(1, 3)), expect)
})

test_that("vec_slice() falls back to `[` with S3 objects", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) "dispatched"
  )
  expect_identical(vec_slice(foobar(NA), 1), "dispatched")

  expect_error(vec_slice(foobar(list(NA)), 1), class = "vctrs_error_scalar_type")
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x
  )
  expect_identical(vec_slice(foobar(list(NA)), 1), foobar(list(NA)))
})

test_that("vec_slice() doesn't restore when attributes have already been restored", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) structure("dispatched", foo = "bar"),
    vec_restore.vctrs_foobar = function(...) stop("not called")
  )
  expect_error(vec_slice(foobar(NA), 1), NA)
})

test_that("vec_slice() doesn't restore when `[` method intentionally dropped attributes", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) unstructure(NextMethod()),
    vec_restore.vctrs_foobar = function(...) stop("not called")
  )
  expect_identical(vec_slice(foobar(NA), 1), NA)
})

test_that("can vec_slice() without inflooping when restore calls math generics", {
  local_methods(
    new_foobar = function(x) {
      new_vctr(as.double(x), class = "vctrs_foobar")
    },
    vec_restore.vctrs_foobar = function(x, ...) {
      abs(x)
      sum(x)
      mean(x)
      is.finite(x)
      is.infinite(x)
      is.nan(x)
      new_foobar(x)
    }
  )
  expect_identical(new_foobar(1:10)[1:2], new_foobar(1:2))
})

test_that("vec_restore() is called after slicing data frames", {
  local_methods(
    vec_restore.vctrs_tabble = function(...) "dispatched"
  )
  df <- structure(mtcars, class = c("vctrs_tabble", "data.frame"))
  expect_identical(vec_slice(df, 1), "dispatched")
})

test_that("additional subscripts are forwarded to `[`", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) vec_index(x, i, ...)
  )

  x <- foobar(c("foo", "bar", "quux", "hunoz"))
  dim(x) <- c(2, 2)

  exp <- foobar("quux")
  dim(exp) <- c(1, 1)

  expect_identical(x[1, 2], exp)
})

test_that("can use names to vec_slice() a named object", {
  x0 <- c(a = 1, b = 2)
  x1 <- c(a = 1, a = 2)

  expect_identical(vec_slice(x0, letters[1]), c(a = 1))
  expect_identical(vec_slice(x0, letters[2:1]), c(b = 2, a = 1))
  expect_identical(vec_slice(x1, letters[1]), c(a = 1))

  expect_error(vec_slice(x0, letters[3:1]), class = "vctrs_error_subscript_oob")
  expect_error(vec_slice(x1, letters[2]), class = "vctrs_error_subscript_oob")
})

test_that("can't use names to vec_slice() an unnamed object", {
  expect_error(
    vec_slice(1:3, letters[1]),
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
  expect_error(
    vec_slice(1:3, letters[25:27]),
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
})

test_that("can slice with missing character indices (#244)", {
  expect_identical(vec_as_location(na_chr, 2L, c("x", "")), na_int)
  expect_identical(vec_slice(c(x = 1), na_chr), set_names(na_dbl, ""))
  expect_identical(vec_slice(c(x = "foo"), na_chr), set_names(na_chr, ""))
})

test_that("can slice with numerics (#577)", {
  expect_identical(vec_as_location(1:2, 3), 1:2)
  expect_error(vec_as_location(1:2, 3.5), class = "vctrs_error_cast_lossy")
})

test_that("missing indices don't create NA names", {
  x <- set_names(letters)
  expect_identical(vec_slice(x, na_int), set_names(na_chr, ""))
  expect_identical(vec_slice(x, int(1, NA, 3, NA)), chr(a = "a", NA, c = "c", NA))

  # Preserves existing NA names
  x <- set_names(1:2, c(NA, "foo"))
  expect_identical(vec_slice(x, 1:2), x)
})

test_that("vec_slice() asserts vectorness (#301)", {
  expect_error(vec_slice(NULL, 1), class = "vctrs_error_scalar_type")
})

test_that("slicing an unspecified logical vector returns a logical vector", {
  expect_identical(vec_slice(NA, integer()), logical())
  expect_identical(vec_slice(NA, c(1, 1)), c(NA, NA))
})

test_that("slicing an unspecified() object returns an unspecified()", {
  expect_identical(vec_slice(unspecified(1), integer()), unspecified())
  expect_identical(vec_slice(unspecified(1), c(1, 1)), unspecified(2))
})


test_that("vec_slice() works with Altrep classes with custom extract methods", {
  skip_if(getRversion() < "3.5")

  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))

  idx <- c(9, 10, 11)
  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar"))
})

test_that("Unnamed vector with character subscript is caught", {
  expect_snapshot(error = TRUE, vec_slice(1:3, letters[1]))
})

test_that("Negative subscripts are checked", {
  expect_snapshot(error = TRUE, vec_slice(1:3, -c(1L, NA)))
  expect_snapshot(error = TRUE, vec_slice(1:3, c(-1L, 1L)))
})

test_that("oob error messages are properly constructed", {
  expect_snapshot(error = TRUE, vec_slice(c(bar = 1), "foo"))

  # Multiple OOB indices
  expect_snapshot(error = TRUE, vec_slice(letters, c(100, 1000)))
  expect_snapshot(error = TRUE, vec_slice(letters, c(1, 100:103, 2, 104:110)))
  expect_snapshot(error = TRUE, vec_slice(set_names(letters), c("foo", "bar")))
  expect_snapshot(error = TRUE, vec_slice(set_names(letters), toupper(letters)))
})

# vec_init ----------------------------------------------------------------

test_that("na of atomic vectors is as expected", {
  expect_equal(vec_init(TRUE), NA)
  expect_equal(vec_init(1L), NA_integer_)
  expect_equal(vec_init(1), NA_real_)
  expect_equal(vec_init("x"), NA_character_)
  expect_equal(vec_init(1i), NA_complex_)
})

test_that("na of factor preserves levels", {
  f1 <- factor("a", levels = c("a", "b"))
  f2 <- vec_init(f1)

  expect_equal(levels(f1), levels(f2))
})

test_that("na of POSIXct preserves tz", {
  dt1 <- as.POSIXct("2010-01-01", tz = "America/New_York")
  dt2 <- vec_init(dt1)
  expect_equal(attr(dt2, "tzone"), "America/New_York")
})

test_that("na of list is list(NULL)", {
  expect_equal(vec_init(list()), list(NULL))
})

test_that("na of array is 1d slice", {
  x1 <- array(1:12, c(2, 3, 4))
  x2 <- vec_init(x1)

  expect_equal(x2, array(NA_integer_, c(1, 3, 4)))
})

test_that("na of list-array is 1d slice", {
  x1 <- array(as.list(1:12), c(2, 3, 4))
  x2 <- vec_init(x1)

  expect_equal(x2, array(list(), c(1, 3, 4)))
})

test_that("vec_init() asserts vectorness (#301)", {
  expect_error(vec_init(NULL, 1L), class = "vctrs_error_scalar_type")
})

test_that("vec_init() works with Altrep classes", {
  skip_if(getRversion() < "3.5")

  x <- .Call(vctrs_altrep_rle_Make, c(foo = 1L, bar = 2L))

  expect_equal(vec_init(x, 2), rep(NA_character_, 2))
})

test_that("vec_init() validates `n`", {
  expect_snapshot({
    (expect_error(vec_init(1L, 1.5)))
    (expect_error(vec_init(1L, c(1, 2))))
    (expect_error(vec_init(1L, -1L)))
    (expect_error(vec_init(1L, NA)))
    (expect_error(vec_init(1L, NA_integer_)))
  })
})


# vec_slice + compact_rep -------------------------------------------------

# `i` is 1-based

test_that("names are repaired correctly with compact reps and `NA_integer_`", {
  x <- list(a = 1L, b = 2L)
  expect <- set_names(list(NULL, NULL), c("", ""))

  expect_equal(vec_slice_rep(x, NA_integer_, 2L), expect)
})

test_that("names are recycled correctly with compact reps", {
  expect_named(vec_slice_rep(c(x = 1L), 1L, 3L), c("x", "x", "x"))
})

test_that("vec_slice() with compact_reps work with Altrep classes", {
  skip_if(getRversion() < "3.5")

  x <- .Call(vctrs_altrep_rle_Make, c(foo = 10L, bar = 5L))

  expect_equal(vec_slice_rep(x, 10L, 3L), rep("foo", 3))
})

# vec_slice + compact_seq -------------------------------------------------

# `start` is 0-based

test_that("can subset base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  increasing <- TRUE
  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl(0, 1))
  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int(2, 3))
  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl(2, 3))
  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl(2, 3))
  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr("2", "3"))
  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2(2, 3))
  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list(2, 3))
})

test_that("can subset base vectors with decreasing compact seqs", {
  start <- 2L
  size <- 2L
  increasing <- FALSE
  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl(1, 0))
  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int(3, 2))
  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl(3, 2))
  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl(3, 2))
  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr("3", "2"))
  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2(3, 2))
  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list(3, 2))
})

test_that("can subset base vectors with size 0 compact seqs", {
  start <- 1L
  size <- 0L
  increasing <- TRUE
  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl())
  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int())
  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl())
  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl())
  expect_identical(vec_slice_seq(chr("1", "2", "3"), start, size, increasing), chr())
  expect_identical(vec_slice_seq(raw2(1, 2, 3), start, size, increasing), raw2())
  expect_identical(vec_slice_seq(list(1, 2, 3), start, size, increasing), list())
})

test_that("can subset shaped base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl(0, 1)))
  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int(2, 3)))
  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl(2, 3)))
  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl(2, 3)))
  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr("2", "3")))
  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2(2, 3)))
  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list(2, 3)))
})

test_that("can subset shaped base vectors with decreasing compact seqs", {
  start <- 2L
  size <- 2L
  increasing <- FALSE
  mat <- as.matrix
  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl(1, 0)))
  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int(3, 2)))
  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl(3, 2)))
  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl(3, 2)))
  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr("3", "2")))
  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2(3, 2)))
  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list(3, 2)))
})

test_that("can subset shaped base vectors with size 0 compact seqs", {
  start <- 1L
  size <- 0L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing), mat(lgl()))
  expect_identical(vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing), mat(int()))
  expect_identical(vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing), mat(dbl()))
  expect_identical(vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing), mat(cpl()))
  expect_identical(vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing), mat(chr()))
  expect_identical(vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing), mat(raw2()))
  expect_identical(vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing), mat(list()))
})

test_that("can subset object of any dimensionality with compact seqs", {
  x0 <- c(1, 1)
  x1 <- ones(2)
  x2 <- ones(2, 3)
  x3 <- ones(2, 3, 4)
  x4 <- ones(2, 3, 4, 5)

  expect_equal(vec_slice_seq(x0, 0L, 1L), 1)
  expect_identical(vec_slice_seq(x1, 0L, 1L), ones(1))
  expect_identical(vec_slice_seq(x2, 0L, 1L), ones(1, 3))
  expect_identical(vec_slice_seq(x3, 0L, 1L), ones(1, 3, 4))
  expect_identical(vec_slice_seq(x4, 0L, 1L), ones(1, 3, 4, 5))
})

test_that("can subset data frames with compact seqs", {
  df <- data_frame(x = 1:5, y = letters[1:5])
  expect_equal(vec_slice_seq(df, 0L, 0L), vec_slice(df, integer()))
  expect_equal(vec_slice_seq(df, 0L, 1L), vec_slice(df, 1L))
  expect_equal(vec_slice_seq(df, 0L, 3L), vec_slice(df, 1:3))
  expect_equal(vec_slice_seq(df, 2L, 3L, FALSE), vec_slice(df, 3:1))

  df$df <- df
  expect_equal(vec_slice_seq(df, 0L, 0L), vec_slice(df, integer()))
  expect_equal(vec_slice_seq(df, 0L, 1L), vec_slice(df, 1L))
  expect_equal(vec_slice_seq(df, 0L, 3L), vec_slice(df, 1:3))
  expect_equal(vec_slice_seq(df, 2L, 3L, FALSE), vec_slice(df, 3:1))
})

test_that("can subset S3 objects using the fallback method with compact seqs", {
  x <- factor(c("a", "b", "c", "d"))
  expect_equal(vec_slice_seq(x, 0L, 0L), vec_slice(x, integer()))
  expect_equal(vec_slice_seq(x, 0L, 1L), vec_slice(x, 1L))
  expect_equal(vec_slice_seq(x, 2L, 2L), vec_slice(x, 3:4))
  expect_equal(vec_slice_seq(x, 3L, 2L, FALSE), vec_slice(x, 4:3))
})

test_that("vec_slice() with compact_seqs work with Altrep classes", {
  skip_if(getRversion() < "3.5")

  x <- .Call(vctrs_altrep_rle_Make, c(foo = 2L, bar = 3L))

  expect_equal(vec_slice_seq(x, 1L, 3L), c("foo", "bar", "bar"))
})

test_that("vec_slice() handles symbols and OO objects", {
  expect_identical(vec_slice(c(a = 1, b = 2), quote(b)), c(b = 2))
  expect_identical(vec_slice(c(a = 1, b = 2), factor("b")), c(b = 2))
  expect_error(vec_slice(c(a = 1, b = 2), foobar("b")), class = "vctrs_error_subscript_type")
})

test_that("vec_init() handles names in columns", {
  expect_identical(
    vec_init(data_frame(x = c(a = 1, b = 2)))$x,
    named(na_dbl)
  )
  expect_identical(
    vec_init(data_frame(x = c(1, 2)))$x,
    na_dbl
  )
})

test_that("vec_slice() restores unrestored but named foreign classes", {
  x <- foobar(c(x = 1))

  expect_identical(vec_slice(x, 1), x)
  expect_identical(vec_chop(x), list(x))
  expect_identical(vec_chop(x, indices = list(1)), list(x))
  expect_identical(vec_ptype(x), foobar(named(dbl())))
  expect_identical(vec_ptype(x), foobar(named(dbl())))
  expect_identical(vec_ptype_common(x, x), foobar(named(dbl())))

  out <- vec_ptype_common_fallback(x, x)
  expect_true(is_common_class_fallback(out))
  expect_identical(fallback_class(out), "vctrs_foobar")
})

test_that("scalar type error is thrown when `vec_slice_unsafe()` is called directly (#1139)", {
  x <- foobar(as.list(1:3))
  expect_error(vec_slice_seq(x, 1L, 1L), class = "vctrs_error_scalar_type")
})

test_that("column sizes are checked before slicing (#552)", {
  x <- structure(list(a = 1, b = 2:3), row.names = 1:2, class = "data.frame")
  expect_error(
    vctrs::vec_slice(x, 2),
    "Column `a` (size 1) must match the data frame (size 2)",
    fixed = TRUE
  )
})

test_that("base_vec_rep() slices data frames with the base::rep() UI", {
  df <- data_frame(x = data_frame(y = 1:2))
  expect_identical(
    base_vec_rep(df, length.out = 4),
    vec_slice(df, c(1:2, 1:2))
  )
})

test_that("vec_size_assign() slices data frames with the base::rep() UI", {
  df <- data_frame(x = data_frame(y = 1:3))

  expect_identical(
    vec_size_assign(df, 2),
    vec_slice(df, 1:2)
  )

  expect_identical(
    vec_size_assign(df, 4),
    vec_slice(df, c(1:3, NA))
  )
})

test_that("ptype2 base methods are not inherited", {
  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
  for (ptype in ptypes) {
    x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
    expect_s3_class(vec_ptype2(x, x), "foobar")
    expect_error(vec_ptype2(x, ptype), class = "vctrs_error_incompatible_type")
    expect_error(vec_ptype2(ptype, x), class = "vctrs_error_incompatible_type")
  }
})

test_that("cast base methods are not inherited", {
  ptypes <- vec_remove(base_empty_types, c("null", "dataframe"))
  for (ptype in ptypes) {
    x <- new_vctr(ptype, class = "foobar", inherit_base_type = TRUE)
    expect_s3_class(vec_cast(ptype, x), "foobar")
    expect_error(vec_cast(x, ptype), class = "vctrs_error_incompatible_type")
  }
})

test_that("default cast allows objects with the same type", {
  x <- structure(1, class = c("foo", "double"))
  expect_equal(vec_cast(x, x), x)
})

# vec_shaped_ptype -------------------------------------------------------

test_that("array dimensions are preserved", {
  mat1 <- matrix(lgl(), nrow = 1, ncol = 1)
  mat2 <- matrix(lgl(), nrow = 2, ncol = 2)
  mat3 <- matrix(lgl(), nrow = 2, ncol = 3)

  expect_equal(vec_ptype2(mat1, mat1), matrix(lgl(), nrow = 0, ncol = 1))
  expect_equal(vec_ptype2(mat1, mat2), matrix(lgl(), nrow = 0, ncol = 2))
  expect_error(vec_ptype2(mat2, mat3), class = "vctrs_error_incompatible_type")
})

test_that("vec_shaped_ptype()", {
  int <- function(...) array(NA_integer_, c(...))

  expect_identical(
    vec_shaped_ptype(integer(), int(5), int(10)),
    new_shape(integer())
  )
  expect_identical(
    vec_shaped_ptype(integer(), int(5, 1), int(10, 1)),
    new_shape(integer(), 1)
  )
  expect_identical(
    vec_shaped_ptype(integer(), int(5, 1, 2), int(10, 1, 2)),
    new_shape(integer(), 1:2)
  )
})

test_that("vec_shaped_ptype() evaluates arg lazily", {
  expect_silent(vec_shaped_ptype(
    integer(),
    int(5),
    int(10),
    x_arg = print("oof")
  ))
  expect_silent(vec_shaped_ptype(
    integer(),
    int(5),
    int(10),
    y_arg = print("oof")
  ))
})

# vec_cast() --------------------------------------------------------------

# NULL

test_that("NULL is idempotent", {
  expect_equal(vec_cast(NULL, NULL), NULL)
  expect_equal(vec_cast(list(1:3), NULL), list(1:3))
})


# Logical

test_that("safe casts work as expected", {
  exp <- lgl(TRUE, FALSE)
  expect_equal(vec_cast(NULL, logical()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), logical()), exp)
  expect_equal(vec_cast(int(1L, 0L), logical()), exp)
  expect_equal(vec_cast(dbl(1, 0), logical()), exp)

  # These used to be allowed
  expect_error(
    vec_cast(chr("T", "F"), logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(chr("TRUE", "FALSE"), logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(chr("true", "false"), logical()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(1, 0), logical()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("NA casts work as expected", {
  exp <- lgl(NA)
  to <- lgl()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(lgl(NA))
  to_mat <- matrix(lgl())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(
    vec_cast(mat(chr(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(list(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("lossy casts generate warning", {
  expect_lossy(
    vec_cast(int(2L, 1L), lgl()),
    lgl(TRUE, TRUE),
    x = int(),
    to = lgl()
  )
  expect_lossy(
    vec_cast(dbl(2, 1), lgl()),
    lgl(TRUE, TRUE),
    x = dbl(),
    to = lgl()
  )

  # These used to be allowed
  expect_error(
    vec_cast(chr("x", "TRUE"), lgl()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(chr("t", "T"), lgl()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(chr("f", "F"), lgl()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(c(TRUE, FALSE), TRUE), lgl()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(factor("a"), logical()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("dimensionality matches output", {
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))

  x <- matrix(1, nrow = 2, ncol = 2)
  expect_error(vec_cast(x, logical()), class = "vctrs_error_incompatible_type")
})

test_that("the common type of two `NA` vectors is unspecified", {
  expect_equal(vec_ptype2(NA, NA), unspecified())
})


# Integer

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, integer()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), integer()), int(1L, 0L))
  expect_equal(vec_cast(int(1L, 2L), integer()), int(1L, 2L))
  expect_equal(vec_cast(dbl(1, 2), integer()), int(1L, 2L))

  # These used to be allowed
  expect_error(
    vec_cast(chr("1", "2"), integer()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(1L, 2L), integer()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("NA casts work as expected", {
  exp <- int(NA)
  to <- int()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(int(NA))
  to_mat <- matrix(int())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(
    vec_cast(mat(chr(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(list(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("lossy casts generate error", {
  expect_lossy(vec_cast(c(2.5, 2), int()), int(2, 2), x = dbl(), to = int())
  expect_lossy(
    vec_cast(c(.Machine$integer.max + 1, 1), int()),
    int(NA, 1L),
    x = dbl(),
    to = int()
  )
  expect_lossy(
    vec_cast(c(-.Machine$integer.max - 1, 1), int()),
    int(NA, 1L),
    x = dbl(),
    to = int()
  )

  # These used to be allowed
  expect_error(
    vec_cast(c("2.5", "2"), int()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(factor("a"), integer()),
    class = "vctrs_error_incompatible_type"
  )
})


# Double

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, double()), NULL)
  expect_equal(vec_cast(lgl(TRUE, FALSE), double()), dbl(1, 0))
  expect_equal(vec_cast(int(1, 0), double()), dbl(1, 0))
  expect_equal(vec_cast(dbl(1, 1.5), double()), dbl(1, 1.5))

  # These used to be allowed
  expect_error(
    vec_cast(chr("1", "1.5"), double()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list(1, 1.5), double()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("NA casts work as expected", {
  exp <- dbl(NA)
  to <- dbl()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(int(NA), to), exp)
  expect_equal(vec_cast(dbl(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(chr(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(dbl(NA))
  to_mat <- matrix(dbl())

  expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(
    vec_cast(mat(chr(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(list(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(factor("a"), double()),
    class = "vctrs_error_incompatible_type"
  )
})


# Complex

test_that("safe casts to complex works", {
  expect_identical(vec_cast(NULL, cpl()), NULL)
  expect_identical(vec_cast(lgl(TRUE, FALSE), cpl()), cpl(1, 0))
  expect_identical(vec_cast(int(1, 0), cpl()), cpl(1, 0))
  expect_identical(vec_cast(dbl(1, 1.5), cpl()), cpl(1, 1.5))

  # This used to be allowed
  expect_error(
    vec_cast(list(1, 1.5), cpl()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("NA casts work as expected", {
  # This goes through a special path for <unspecified>
  expect_equal(vec_cast(lgl(NA), cpl()), NA_complex_)

  # TODO: Use our own cast routines here?
  # It isn't great that this logical `NA` cast returns a different `NA`
  # than the one above with just `lgl(NA)` (which is seen as unspecified). i.e.
  # check the `Im()` slot between the two in R >=4.4.0. We can fix this with our
  # own cast routines rather than using `vec_coerce_bare()`.
  expect_type(vec_cast(lgl(NA, TRUE), cpl()), "complex")
  expect_identical(is.na(vec_cast(lgl(NA, TRUE), cpl())), c(TRUE, FALSE))

  # TODO: Use our own cast routines here?
  # `as.complex(NA/NA_real_/NA_integer_)` and `Rf_CoerceVector(NA/NA_real_/NA_integer_)`
  # have gone back and forth about what they return in the `Im()` slot. In some
  # R versions they return `0` and in others they return `NA_real_`.
  # https://stat.ethz.ch/pipermail/r-devel/2023-April/082545.html
  # https://stat.ethz.ch/pipermail/r-devel/2023-September/082864.html
  # expect_equal(vec_cast(int(NA), cpl()), NA_complex_)
  expect_type(vec_cast(int(NA), cpl()), "complex")
  expect_identical(is.na(vec_cast(int(NA), cpl())), TRUE)

  # expect_equal(vec_cast(dbl(NA), cpl()), NA_complex_)
  expect_type(vec_cast(dbl(NA), cpl()), "complex")
  expect_identical(is.na(vec_cast(dbl(NA), cpl())), TRUE)

  # This used to be allowed
  expect_error(
    vec_cast(list(NA), cpl()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(NA_complex_)
  to_mat <- matrix(cpl())

  # TODO: Use our own cast routines here?
  # `as.complex(NA/NA_real_/NA_integer_)` and `Rf_CoerceVector(NA/NA_real_/NA_integer_)`
  # have gone back and forth about what they return in the `Im()` slot. In some
  # R versions they return `0` and in others they return `NA_real_`.
  # https://stat.ethz.ch/pipermail/r-devel/2023-April/082545.html
  # https://stat.ethz.ch/pipermail/r-devel/2023-September/082864.html

  # expect_equal(vec_cast(mat(lgl(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(lgl(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(lgl(NA)), to_mat)), matrix(TRUE))

  # expect_equal(vec_cast(mat(int(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(int(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(int(NA)), to_mat)), matrix(TRUE))

  # expect_equal(vec_cast(mat(dbl(NA)), to_mat), exp_mat)
  expect_type(vec_cast(mat(dbl(NA)), to_mat), "complex")
  expect_identical(is.na(vec_cast(mat(dbl(NA)), to_mat)), matrix(TRUE))

  # This used to be allowed
  expect_error(
    vec_cast(mat(list(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("complex is coercible to numeric types", {
  expect_identical(vec_ptype2(cpl(), NULL), cpl())
  expect_identical(vec_ptype2(NULL, cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), int()), cpl())
  expect_identical(vec_ptype2(int(), cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), dbl()), cpl())
  expect_identical(vec_ptype2(dbl(), cpl()), cpl())

  expect_identical(vec_ptype2(cpl(), cpl()), cpl())

  expect_identical(vec_c(0, 1i), cpl(0i, 1i))
})

test_that("complex is not coercible to logical", {
  expect_error(
    vec_ptype2(cpl(), lgl()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_ptype2(lgl(), cpl()),
    class = "vctrs_error_incompatible_type"
  )
})


# Character

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, character()), NULL)
  expect_equal(vec_cast(NA, character()), NA_character_)

  # These used to be allowed
  expect_error(
    vec_cast(lgl(TRUE, FALSE), character()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(list("x", "y"), character()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("NA casts work as expected", {
  exp <- chr(NA)
  to <- chr()

  expect_equal(vec_cast(lgl(NA), to), exp)
  expect_equal(vec_cast(chr(NA), to), exp)

  # These used to be allowed
  expect_error(vec_cast(int(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(dbl(NA), to), class = "vctrs_error_incompatible_type")
  expect_error(vec_cast(list(NA), to), class = "vctrs_error_incompatible_type")
})

test_that("Shaped NA casts work as expected", {
  mat <- matrix
  exp_mat <- mat(chr(NA))
  to_mat <- matrix(chr())

  expect_equal(vec_cast(mat(chr(NA)), to_mat), exp_mat)

  # These used to be allowed
  expect_error(
    vec_cast(mat(lgl(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(int(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(dbl(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(mat(list(NA)), to_mat),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("difftime does not get special treatment", {
  dt1 <- as.difftime(600, units = "secs")
  # This used to be allowed
  expect_error(
    vec_cast(dt1, character()),
    class = "vctrs_error_incompatible_type"
  )
})


# Raw

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, raw()), NULL)

  # This used to be allowed
  expect_error(
    vec_cast(list(raw(1)), raw()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("invalid casts generate error", {
  expect_error(
    vec_cast(raw(1), double()),
    class = "vctrs_error_incompatible_type"
  )
  expect_error(
    vec_cast(double(1), raw()),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("can sort raw", {
  x <- as.raw(c(3, 1, 2, 4))
  expect_identical(vec_order(x), int(2, 3, 1, 4))
  expect_identical(x[vec_order(x)], as.raw(1:4))
})

test_that("raw has informative type summaries", {
  expect_equal(vec_ptype_abbr(raw()), "raw")
  expect_equal(vec_ptype_full(raw()), "raw")
})

test_that("can provide common type with raw", {
  local_methods(
    vec_ptype2.raw.vctrs_foobar = function(...) "dispatched-left",
    vec_ptype2.vctrs_foobar = function(...) NULL,
    vec_ptype2.vctrs_foobar.raw = function(...) "dispatched-right"
  )
  expect_identical(vec_ptype2(raw(), foobar("")), "dispatched-left")
  expect_identical(vec_ptype2(foobar(""), raw()), "dispatched-right")
})


# Lists

test_that("safe casts work as expected", {
  expect_equal(vec_cast(NULL, list()), NULL)
  expect_equal(vec_cast(NA, list()), list(NULL))
  expect_equal(vec_cast(list(1L, 2L), list()), list(1L, 2L))

  # This used to be allowed
  expect_error(vec_cast(1:2, list()), class = "vctrs_error_incompatible_type")
})

test_that("dimensionality matches to", {
  x1 <- matrix(TRUE, nrow = 1, ncol = 1)
  x2 <- matrix(1L, nrow = 0, ncol = 2)
  expect_dim(vec_cast(x1, x2), c(1, 2))
  expect_dim(vec_cast(TRUE, x2), c(1, 2))
})

test_that("data frames are cast to list row wise (#639)", {
  x <- data.frame(x = 1:2, row.names = c("a", "b"))
  expect <- list(data.frame(x = 1L), data.frame(x = 2L))

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})

test_that("data frames can be cast to shaped lists", {
  to <- array(list(), dim = c(0, 2, 1))
  x <- data.frame(x = 1:2, y = 3:4)

  expect <- list(vec_slice(x, 1), vec_slice(x, 2))
  expect <- array(expect, dim = c(2, 2, 1))

  # This used to be allowed
  expect_error(vec_cast(x, to), class = "vctrs_error_incompatible_type")
})

test_that("Casting atomic `NA` values to list results in a `NULL`", {
  x <- c(NA, 1)
  expect <- list(NULL, 1)

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})

test_that("Casting data frame `NA` rows to list results in a `NULL`", {
  x <- data.frame(x = c(NA, NA, 1), y = c(NA, 1, 2))
  expect <- list(NULL, vec_slice(x, 2), vec_slice(x, 3))

  # This used to be allowed
  expect_error(vec_cast(x, list()), class = "vctrs_error_incompatible_type")
})


# Unspecified

test_that("unspecified can be cast to bare methods", {
  for (x in vectors[-4]) {
    expect_identical(vec_cast(unspecified(3), x), vec_init(x, 3))
  }
})

test_that("slice-assign throws error with non-vector inputs", {
  x <- environment()
  expect_error(vec_slice(x, 1L) <- 1L, class = "vctrs_error_scalar_type")
})

test_that("slice-assign throws error with non-vector `value`", {
  x <- 1L
  expect_error(vec_slice(x, 1L) <- NULL, class = "vctrs_error_scalar_type")
  expect_error(
    vec_slice(x, 1L) <- environment(),
    class = "vctrs_error_scalar_type"
  )
})

test_that("assign throws error with non-vector `value`", {
  x <- 1L

  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, NULL)
  })
  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, NULL, slice_value = TRUE)
  })

  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, NULL, value_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, NULL, slice_value = TRUE, value_arg = "foo")
  })

  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, environment(), value_arg = "foo")
  })
  expect_snapshot(error = TRUE, {
    vec_assign(x, 1L, environment(), slice_value = TRUE, value_arg = "foo")
  })
})

test_that("can slice-assign NULL", {
  x <- NULL
  vec_slice(x, 1L) <- 1
  expect_identical(x, NULL)
})

test_that("can assign on NULL `x`", {
  x <- NULL
  expect_identical(vec_assign(x, TRUE, 1), NULL)
  expect_identical(vec_assign(x, TRUE, 1, slice_value = TRUE), NULL)
})

test_that("can slice-assign base vectors", {
  x <- rep(FALSE, 3)
  vec_slice(x, 2) <- TRUE
  expect_identical(x, lgl(FALSE, TRUE, FALSE))

  x <- rep(0L, 3)
  vec_slice(x, 2) <- 1L
  expect_identical(x, int(0L, 1L, 0L))

  x <- rep(0., 3)
  vec_slice(x, 2) <- 1
  expect_identical(x, dbl(0, 1, 0))

  x <- rep(0i, 3)
  vec_slice(x, 2) <- 1i
  expect_identical(x, cpl(0i, 1i, 0i))

  x <- rep("", 3)
  vec_slice(x, 2) <- "foo"
  expect_identical(x, chr("", "foo", ""))

  x <- as.raw(rep(0, 3))
  vec_slice(x, 2) <- as.raw(1)
  expect_identical(x, as.raw(c(0, 1, 0)))

  x <- rep(list(NULL), 3)
  vec_slice(x, 2) <- list(NA)
  expect_identical(x, list(NULL, NA, NULL))
})

test_that("can assign base vectors", {
  x <- rep(FALSE, 3)
  expect_identical(vec_assign(x, 2, TRUE), lgl(FALSE, TRUE, FALSE))
  expect_identical(x, rep(FALSE, 3))

  x <- rep(0L, 3)
  expect_identical(vec_assign(x, 2, 1L), int(0L, 1L, 0L))
  expect_identical(x, rep(0L, 3))

  x <- rep(0., 3)
  expect_identical(vec_assign(x, 2, 1), dbl(0, 1, 0))
  expect_identical(x, rep(0., 3))

  x <- rep(0i, 3)
  expect_identical(vec_assign(x, 2, 1i), cpl(0i, 1i, 0i))
  expect_identical(x, rep(0i, 3))

  x <- rep("", 3)
  expect_identical(vec_assign(x, 2, "foo"), chr("", "foo", ""))
  expect_identical(x, rep("", 3))

  x <- as.raw(rep(0, 3))
  expect_identical(vec_assign(x, 2, as.raw(1)), as.raw(c(0, 1, 0)))
  expect_identical(x, as.raw(rep(0, 3)))

  x <- rep(list(NULL), 3)
  expect_identical(vec_assign(x, 2, list(NA)), list(NULL, NA, NULL))
  expect_identical(x, rep(list(NULL), 3))
})

test_that("can assign base vectors with logical indices", {
  # Logical indices have their own optimized path so we test them specially

  condition <- c(TRUE, FALSE, NA, TRUE)

  x <- rep(FALSE, 4)
  value <- c(NA, TRUE, TRUE, TRUE)
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, lgl(NA, FALSE, FALSE, TRUE))

  x <- rep(0L, 4)
  value <- c(NA, 1L, 2L, 3L)
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, int(NA, 0L, 0L, 3L))

  x <- rep(0, 4)
  value <- c(NA, 1, 2, 3)
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, dbl(NA, 0, 0, 3))

  x <- rep(0i, 4)
  na <- complex(real = NA, imaginary = NA)
  value <- c(na, 1i, 2i, 3i)
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, c(na, 0i, 0i, 3i))

  x <- rep("", 4)
  value <- c(NA, "foo", "bar", "baz")
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, c(NA, "", "", "baz"))

  x <- as.raw(rep(0, 4))
  value <- as.raw(c(1, 2, 3, 4))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, as.raw(c(1, 0, 0, 4)))

  x <- rep(list(1), 4)
  value <- list(NA, 2, 3, 4)
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, list(NA, 1, 1, 4))
})

test_that("can assign base vectors with recycling of `value`", {
  x <- rep(FALSE, 3)
  expect_identical(vec_assign(x, c(3, 1), TRUE), lgl(TRUE, FALSE, TRUE))
  expect_identical(x, rep(FALSE, 3))

  x <- rep(0L, 3)
  expect_identical(vec_assign(x, c(3, 1), 1L), int(1L, 0L, 1L))
  expect_identical(x, rep(0L, 3))

  x <- rep(0., 3)
  expect_identical(vec_assign(x, c(3, 1), 1), dbl(1, 0, 1))
  expect_identical(x, rep(0., 3))

  x <- rep(0i, 3)
  expect_identical(vec_assign(x, c(3, 1), 1i), cpl(1i, 0i, 1i))
  expect_identical(x, rep(0i, 3))

  x <- rep("", 3)
  expect_identical(vec_assign(x, c(3, 1), "foo"), chr("foo", "", "foo"))
  expect_identical(x, rep("", 3))

  x <- as.raw(rep(0, 3))
  expect_identical(vec_assign(x, c(3, 1), as.raw(1)), as.raw(c(1, 0, 1)))
  expect_identical(x, as.raw(rep(0, 3)))

  x <- rep(list(NULL), 3)
  expect_identical(vec_assign(x, c(3, 1), list(NA)), list(NA, NULL, NA))
  expect_identical(x, rep(list(NULL), 3))
})

test_that("can assign base vectors with logical indices with recycled `value`", {
  # Logical indices have their own optimized path so we test them specially

  condition <- c(TRUE, FALSE, NA, TRUE)

  x <- rep(FALSE, 4)
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, lgl(NA, FALSE, FALSE, NA))

  x <- rep(0L, 4)
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, int(NA, 0L, 0L, NA))

  x <- rep(0, 4)
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, dbl(NA, 0, 0, NA))

  x <- rep(0i, 4)
  na <- complex(real = NA, imaginary = NA)
  value <- na
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, c(na, 0i, 0i, na))

  x <- rep("", 4)
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, c(NA, "", "", NA))

  x <- as.raw(rep(0, 4))
  value <- as.raw(1)
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, as.raw(c(1, 0, 0, 1)))

  x <- rep(list(1), 4)
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, list(NULL, 1, 1, NULL))
})

test_that("can assign shaped base vectors", {
  mat <- as.matrix

  x <- mat(rep(FALSE, 3))
  expect_identical(vec_assign(x, 2, TRUE), mat(lgl(FALSE, TRUE, FALSE)))
  expect_identical(x, mat(rep(FALSE, 3)))

  x <- mat(rep(0L, 3))
  expect_identical(vec_assign(x, 2, 1L), mat(int(0L, 1L, 0L)))
  expect_identical(x, mat(rep(0L, 3)))

  x <- mat(rep(0, 3))
  expect_identical(vec_assign(x, 2, 1), mat(dbl(0, 1, 0)))
  expect_identical(x, mat(rep(0, 3)))

  x <- mat(rep(0i, 3))
  expect_identical(vec_assign(x, 2, 1i), mat(cpl(0i, 1i, 0i)))
  expect_identical(x, mat(rep(0i, 3)))

  x <- mat(rep("", 3))
  expect_identical(vec_assign(x, 2, "foo"), mat(chr("", "foo", "")))
  expect_identical(x, mat(rep("", 3)))

  x <- mat(as.raw(rep(0, 3)))
  expect_identical(vec_assign(x, 2, as.raw(1)), mat(as.raw(c(0, 1, 0))))
  expect_identical(x, mat(as.raw(rep(0, 3))))

  mat <- as.matrix
  x <- mat(rep(list(NULL), 3))
  expect_identical(vec_assign(x, 2, list(NA)), mat(list(NULL, NA, NULL)))
  expect_identical(x, mat(rep(list(NULL), 3)))
})

test_that("can assign shaped base vectors with logical indices", {
  # Logical indices have their own optimized path so we test them specially

  mat <- as.matrix

  condition <- c(TRUE, FALSE, NA, TRUE)

  x <- mat(rep(FALSE, 4))
  value <- mat(c(NA, TRUE, TRUE, TRUE))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, FALSE, FALSE, TRUE)))

  x <- mat(rep(0L, 4))
  value <- mat(c(NA, 1L, 2L, 3L))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, 0L, 0L, 3L)))

  x <- mat(rep(0, 4))
  value <- mat(c(NA, 1, 2, 3))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, 0, 0, 3)))

  x <- mat(rep(0i, 4))
  value <- mat(c(1i, 2i, 3i, 4i))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(1i, 0i, 0i, 4i)))

  x <- mat(rep("", 4))
  value <- mat(c(NA, "foo", "bar", "baz"))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, "", "", "baz")))

  x <- mat(as.raw(rep(0, 4)))
  value <- mat(as.raw(c(1, 2, 3, 4)))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(as.raw(c(1, 0, 0, 4))))

  x <- mat(rep(list(1), 4))
  value <- mat(list(NA, 2, 3, 4))
  y <- vec_assign(x, condition, vec_slice(value, condition))
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(list(NA, 1, 1, 4)))
})

test_that("can assign shaped base vectors with recycling of `value`", {
  mat <- as.matrix

  x <- mat(rep(FALSE, 3))
  expect_identical(vec_assign(x, c(3, 1), TRUE), mat(lgl(TRUE, FALSE, TRUE)))
  expect_identical(x, mat(rep(FALSE, 3)))

  x <- mat(rep(0L, 3))
  expect_identical(vec_assign(x, c(3, 1), 1L), mat(int(1L, 0L, 1L)))
  expect_identical(x, mat(rep(0L, 3)))

  x <- mat(rep(0, 3))
  expect_identical(vec_assign(x, c(3, 1), 1), mat(dbl(1, 0, 1)))
  expect_identical(x, mat(rep(0, 3)))

  x <- mat(rep(0i, 3))
  expect_identical(vec_assign(x, c(3, 1), 1i), mat(cpl(1i, 0i, 1i)))
  expect_identical(x, mat(rep(0i, 3)))

  x <- mat(rep("", 3))
  expect_identical(vec_assign(x, c(3, 1), "foo"), mat(chr("foo", "", "foo")))
  expect_identical(x, mat(rep("", 3)))

  x <- mat(as.raw(rep(0, 3)))
  expect_identical(vec_assign(x, c(3, 1), as.raw(1)), mat(as.raw(c(1, 0, 1))))
  expect_identical(x, mat(as.raw(rep(0, 3))))

  mat <- as.matrix
  x <- mat(rep(list(NULL), 3))
  expect_identical(vec_assign(x, c(3, 1), list(NA)), mat(list(NA, NULL, NA)))
  expect_identical(x, mat(rep(list(NULL), 3)))
})

test_that("can assign shaped base vectors with logical indices with recycling of `value`", {
  # Logical indices have their own optimized path so we test them specially

  mat <- as.matrix

  condition <- c(TRUE, FALSE, NA, TRUE)

  x <- mat(rep(FALSE, 4))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, FALSE, FALSE, NA)))

  x <- mat(rep(0L, 4))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, 0L, 0L, NA)))

  x <- mat(rep(0, 4))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, 0, 0, NA)))

  x <- mat(rep(0i, 4))
  value <- NA
  na <- complex(real = NA, imaginary = NA)
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(na, 0i, 0i, na)))

  x <- mat(rep("", 4))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(c(NA, "", "", NA)))

  x <- mat(as.raw(rep(1, 4)))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(as.raw(c(0, 1, 1, 0))))

  x <- mat(rep(list(1), 4))
  value <- NA
  y <- vec_assign(x, condition, value)
  z <- vec_assign(x, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, mat(list(NULL, 1, 1, NULL)))
})

test_that("can assign object of any dimensionality", {
  x1 <- ones(3)
  x2 <- ones(3, 4)
  x3 <- ones(3, 4, 5)
  x4 <- ones(3, 4, 5, 6)

  expect_identical(vec_assign(x1, 1L, 2L), array(rep(c(2, 1, 1), 1), dim = 3))
  expect_identical(
    vec_assign(x2, 1L, 2L),
    array(rep(c(2, 1, 1), 3), dim = c(3, 4))
  )
  expect_identical(
    vec_assign(x3, 1L, 2L),
    array(rep(c(2, 1, 1), 12), dim = c(3, 4, 5))
  )
  expect_identical(
    vec_assign(x4, 1L, 2L),
    array(rep(c(2, 1, 1), 60), dim = c(3, 4, 5, 6))
  )

  # With recycling of `value`
  expect_identical(
    vec_assign(x1, c(3L, 2L), 2L),
    array(rep(c(1, 2, 2), 1), dim = 3)
  )
  expect_identical(
    vec_assign(x2, c(3L, 2L), 2L),
    array(rep(c(1, 2, 2), 3), dim = c(3, 4))
  )
  expect_identical(
    vec_assign(x3, c(3L, 2L), 2L),
    array(rep(c(1, 2, 2), 12), dim = c(3, 4, 5))
  )
  expect_identical(
    vec_assign(x4, c(3L, 2L), 2L),
    array(rep(c(1, 2, 2), 60), dim = c(3, 4, 5, 6))
  )
})

test_that("can assign object of any dimensionality with logical indices", {
  # Logical indices have their own optimized path so we test them specially

  # Non barrier (integer, double, logical, etc)
  x1 <- ones(4)
  x2 <- ones(4, 5)
  x3 <- ones(4, 5, 6)
  x4 <- ones(4, 5, 6, 7)

  # Barrier (list, character)
  x1_list <- ones_list(4)
  x2_list <- ones_list(4, 5)
  x3_list <- ones_list(4, 5, 6)
  x4_list <- ones_list(4, 5, 6, 7)

  condition <- c(TRUE, FALSE, NA, TRUE)

  # No recycling
  value <- c(2, 3, 4, 5)
  value_list <- as.list(value)

  y <- vec_assign(x1, condition, vec_slice(value, condition))
  z <- vec_assign(x1, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 5), 1), dim = dim(x1)))

  y <- vec_assign(x1_list, condition, vec_slice(value_list, condition))
  z <- vec_assign(x1_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 5), 1), dim = dim(x1_list)))

  y <- vec_assign(x2, condition, vec_slice(value, condition))
  z <- vec_assign(x2, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 5), 4), dim = dim(x2)))

  y <- vec_assign(x2_list, condition, vec_slice(value_list, condition))
  z <- vec_assign(x2_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 5), 4), dim = dim(x2_list)))

  y <- vec_assign(x3, condition, vec_slice(value, condition))
  z <- vec_assign(x3, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 5), 20), dim = dim(x3)))

  y <- vec_assign(x3_list, condition, vec_slice(value_list, condition))
  z <- vec_assign(x3_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 5), 20), dim = dim(x3_list)))

  y <- vec_assign(x4, condition, vec_slice(value, condition))
  z <- vec_assign(x4, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 5), 120), dim = dim(x4)))

  y <- vec_assign(x4_list, condition, vec_slice(value_list, condition))
  z <- vec_assign(x4_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 5), 120), dim = dim(x4_list)))

  # `value` recycling
  value <- 2
  value_list <- as.list(value)

  y <- vec_assign(x1, condition, value)
  z <- vec_assign(x1, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 2), 1), dim = dim(x1)))

  y <- vec_assign(x1_list, condition, value_list)
  z <- vec_assign(x1_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 2), 1), dim = dim(x1_list)))

  y <- vec_assign(x2, condition, value)
  z <- vec_assign(x2, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 2), 4), dim = dim(x2)))

  y <- vec_assign(x2_list, condition, value_list)
  z <- vec_assign(x2_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 2), 4), dim = dim(x2_list)))

  y <- vec_assign(x3, condition, value)
  z <- vec_assign(x3, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 2), 20), dim = dim(x3)))

  y <- vec_assign(x3_list, condition, value_list)
  z <- vec_assign(x3_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 2), 20), dim = dim(x3_list)))

  y <- vec_assign(x4, condition, value)
  z <- vec_assign(x4, condition, value, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(c(2, 1, 1, 2), 120), dim = dim(x4)))

  y <- vec_assign(x4_list, condition, value_list)
  z <- vec_assign(x4_list, condition, value_list, slice_value = TRUE)
  expect_identical(y, z)
  expect_identical(y, array(rep(list(2, 1, 1, 2), 120), dim = dim(x4_list)))
})

test_that("atomics can't be assigned in lists", {
  x <- list(NULL)
  expect_error(vec_slice(x, 1) <- 1, class = "vctrs_error_incompatible_type")
  expect_error(vec_assign(x, 1, 2), class = "vctrs_error_incompatible_type")

  expect_error(
    vec_slice(x, 1) <- "foo",
    class = "vctrs_error_incompatible_type"
  )
  expect_error(vec_assign(x, 1, "foo"), class = "vctrs_error_incompatible_type")
})

test_that("Unspecified `NA` vector can be assigned into lists", {
  x <- list(1, 2)
  vec_slice(x, 1) <- NA
  expect_identical(x, list(NULL, 2))
})

test_that("monitoring test - unspecified() can be assigned in lists", {
  x <- list(1, 2)
  expect_error(vec_slice(x, 1) <- unspecified(1), NA)
  expect_equal(x, list(NULL, 2))
})

test_that("can assign and slice-assign data frames", {
  df <- data.frame(x = 1:2)
  df$y <- data.frame(a = 2:1)

  orig <- duplicate(df, shallow = FALSE)

  other <- data.frame(x = 3)
  other$y <- data.frame(a = 3)

  exp <- data.frame(x = int(3, 2))
  exp$y <- data.frame(a = int(3, 1))

  expect_identical(vec_assign(df, 1, other), exp)
  expect_identical(df, orig)

  expect_identical(vec_assign(df, 1, other, slice_value = TRUE), exp)
  expect_identical(df, orig)

  vec_slice(df, 1) <- other
  expect_identical(df, exp)
})

test_that("can assign using logical index", {
  x <- c(2, 1)
  vec_slice(x, TRUE) <- 3
  expect_equal(x, c(3, 3))

  vec_slice(x, c(TRUE, FALSE)) <- 4
  expect_equal(x, c(4, 3))

  expect_snapshot({
    (expect_error(
      vec_assign(x, c(TRUE, FALSE, TRUE), 5),
      class = "vctrs_error_subscript_size"
    ))
  })
  expect_snapshot({
    (expect_error(
      vec_assign(x, c(TRUE, FALSE, TRUE), 5, slice_value = TRUE),
      class = "vctrs_error_subscript_size"
    ))
  })

  expect_snapshot({
    (expect_error(
      vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]),
      class = "vctrs_error_subscript_size"
    ))
  })
  expect_snapshot({
    (expect_error(
      vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ], slice_value = TRUE),
      class = "vctrs_error_subscript_size"
    ))
  })
})

test_that("assign `value` size depends on `slice_value`", {
  x <- c(1, 2, 3)

  # `value` size depends on number of `NA` or `TRUE` values in `i`
  expect_snapshot(error = TRUE, {
    vec_assign(x, c(TRUE, NA, FALSE), c(1, 2, 3))
  })

  # `value` size depends on size of `x`
  expect_snapshot(error = TRUE, {
    vec_assign(x, c(TRUE, NA, FALSE), c(1, 2), slice_value = TRUE)
  })
})

test_that("assign ignores NA in logical subsetting", {
  x <- c(NA, 1, 2)

  expect_equal(vec_assign(x, x > 0, 1), c(NA, 1, 1))
  expect_equal(vec_assign(x, x > 0, 1, slice_value = TRUE), c(NA, 1, 1))

  expect_equal(vec_assign(x, x > 0, c(NA, 2:1)), c(NA, 2, 1))
  expect_equal(
    vec_assign(x, x > 0, c(NA, 2:1), slice_value = TRUE),
    c(NA, 2, 1)
  )
})

test_that("assign with arrays ignores NA in logical subsetting", {
  mat <- as.matrix
  x <- c(NA, 1, 2)
  expect_equal(vec_assign(mat(x), x > 0, 1), mat(c(NA, 1, 1)))
  expect_equal(vec_assign(mat(x), x > 0, c(NA, 2:1)), mat(c(NA, 2, 1)))
})

test_that("assign ignores NA in integer subsetting", {
  x <- 0:2
  expect_equal(vec_assign(x, c(NA, 2:3), 1), c(0, 1, 1))
  expect_equal(vec_assign(x, c(NA, 2:3), c(NA, 2:1)), c(0, 2, 1))
})

test_that("assign with arrays ignores NA in integer subsetting", {
  mat <- as.matrix
  x <- mat(0:2)
  expect_equal(vec_assign(x, c(NA, 2:3), 1), mat(c(0, 1, 1)))
  expect_equal(vec_assign(x, c(NA, 2:3), c(NA, 2:1)), mat(c(0, 2, 1)))
})

test_that("can't modify subset with missing argument", {
  x <- 1:3
  expect_error(vec_slice(x, ) <- 2L)
})

test_that("can modify subset with recycled NA argument", {
  x <- 1:3
  vec_slice(x, NA) <- 2L
  expect_identical(x, 1:3)
})

test_that("can modify subset with recycled TRUE argument", {
  x <- 1:3
  vec_slice(x, TRUE) <- 2L
  expect_identical(x, rep(2L, 3))
})

test_that("can modify subset with recycled FALSE argument", {
  x <- 1:3
  vec_slice(x, FALSE) <- 2L
  expect_identical(x, 1:3)
})

test_that("can modify subset with NULL argument", {
  x <- 1:3
  vec_slice(x, NULL) <- 2L
  expect_identical(x, 1:3)

  x <- vec_assign(x, NULL, 2L)
  expect_identical(x, 1:3)

  x <- vec_assign(x, NULL, 2L, slice_value = TRUE)
  expect_identical(x, 1:3)
})

test_that("can slice-assign with missing indices", {
  # Atomic case
  x <- 1:3
  y <- 4:6
  test <- c(NA, TRUE, FALSE)
  vec_slice(x, test) <- vec_slice(y, test)
  expect_identical(x, int(1, 5, 3))

  # Barrier case
  x <- as.list(1:3)
  y <- as.list(4:6)
  test <- c(NA, TRUE, FALSE)
  vec_slice(x, test) <- vec_slice(y, test)
  expect_identical(x, as.list(int(1, 5, 3)))

  # Atomic array case
  x <- array(1:12, dim = c(3, 2, 2))
  y <- array(13:24, dim = c(3, 2, 2))
  test <- c(NA, TRUE, FALSE)
  vec_slice(x, test) <- vec_slice(y, test)
  expect <- array(
    int(1, 14, 3, 4, 17, 6, 7, 20, 9, 10, 23, 12),
    dim = c(3, 2, 2)
  )
  expect_identical(x, expect)

  # Barrier array case
  x <- array(as.list(1:12), dim = c(3, 2, 2))
  y <- array(as.list(13:24), dim = c(3, 2, 2))
  test <- c(NA, TRUE, FALSE)
  vec_slice(x, test) <- vec_slice(y, test)
  expect <- array(
    as.list(int(1, 14, 3, 4, 17, 6, 7, 20, 9, 10, 23, 12)),
    dim = c(3, 2, 2)
  )
  expect_identical(x, expect)
})

test_that("slice-assign checks vectorness", {
  x <- foobar(list(1))
  expect_error(vec_slice(x, 1) <- 10, class = "vctrs_error_scalar_type")
})

test_that("a coercible RHS is cast to LHS before assignment (#140)", {
  x <- 1:2
  expect_error(vec_slice(x, 1) <- "1", class = "vctrs_error_incompatible_type")

  x <- c("foo", "bar")
  expect_error(vec_slice(x, 1) <- 1, class = "vctrs_error_incompatible_type")

  x <- 1:2
  expect_error(vec_slice(x, 1) <- 3.5, class = "vctrs_error_cast_lossy")

  allow_lossy_cast(vec_slice(x, 1) <- 3.5)
  expect_identical(x, int(3, 2))

  x <- matrix(1:4, 2)
  vec_slice(x, 1) <- matrix(c(FALSE, FALSE), 1)
  expect_identical(x, matrix(int(0, 2, 0, 4), 2))
  expect_error(
    vec_assign(x, 1, matrix(c("", ""), 1)),
    class = "vctrs_error_incompatible_type"
  )
})

test_that("slice-assign takes the proxy", {
  local_proxy()

  x <- new_proxy(1:3)
  y <- new_proxy(20:21)

  vec_slice(x, 2:3) <- y

  expect_identical(proxy_deref(x), int(1, 20, 21))
})

test_that("can use names to assign with a named object", {
  x0 <- c(a = 1, b = 2, c = 3)
  x1 <- c(a = 1, a = 2, a = 3)

  vec_slice(x0, "b") <- 4
  expect_identical(x0, c(a = 1, b = 4, c = 3))

  # Only first is changed
  vec_slice(x1, "a") <- 4
  expect_identical(x1, c(a = 4, a = 2, a = 3))

  x <- c(a = 1, b = 2, c = 3)

  expect_identical(
    vec_assign(x, c("c", "a"), c(4, 5)),
    c(a = 5, b = 2, c = 4)
  )
  # Slices `value` by `i` after matching `i` against `x` names
  expect_identical(
    vec_assign(x, c("c", "a"), c(4, 5, 6), slice_value = TRUE),
    c(a = 4, b = 2, c = 6)
  )

  expect_snapshot(error = TRUE, {
    vec_assign(x, c("c", "a"), c(4, 5, 6))
  })
  expect_snapshot(error = TRUE, {
    vec_assign(x, c("c", "a"), c(4, 5), slice_value = TRUE)
  })
})

test_that("can't use names to assign with an unnamed object", {
  x0 <- 1:3

  expect_error(
    vec_slice(x0, letters[1]) <- 4L,
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
  expect_error(
    vec_assign(x0, letters[1], 4L, slice_value = TRUE),
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
  expect_error(
    vec_slice(x0, letters[25:27]) <- 5L,
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
})

test_that("slice-assign falls back to `[<-` when proxy is not implemented", {
  obj <- foobar(c("foo", "bar", "baz"))
  expect_error(
    vec_slice(obj, 1:2) <- TRUE,
    class = "vctrs_error_incompatible_type"
  )

  vec_slice(obj, 1:2) <- foobar("quux")

  vec_ptype2(foobar(""), foobar(""))
  vec_cast(foobar(""), foobar(""))
  #> Error: Can't cast <vctrs_foobar> to <vctrs_foobar>

  local_methods(
    `[<-.vctrs_foobar` = function(x, i, value) {
      x <- unclass(x)
      x[i] <- "dispatched"
      x
    },
    vec_ptype2.logical.vctrs_foobar = function(...) foobar(""),
    vec_ptype2.vctrs_foobar = function(...) foobar(""),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.logical = function(x, to, ...) {
      foobar(rep("", length(x)))
    }
  )

  obj <- foobar(c("foo", "bar", "baz"))

  obj2 <- vec_assign(obj, 1:2, TRUE)
  expect_identical(obj2, foobar(c("dispatched", "dispatched", "baz")))

  obj2 <- vec_assign(obj, c(TRUE, FALSE, TRUE), TRUE)
  expect_identical(obj2, foobar(c("dispatched", "bar", "dispatched")))

  # We handle `NA` for the end user
  obj2 <- vec_assign(obj, c(1, NA, 2), TRUE)
  expect_identical(obj2, foobar(c("dispatched", "dispatched", "baz")))
})

test_that("vec_assign() can always assign unspecified values into foreign vector types", {
  obj <- foobar(c("foo", "bar", "baz"))
  expect <- foobar(c(NA, "bar", "baz"))

  expect_identical(vec_assign(obj, 1, NA), expect)
  expect_identical(vec_assign(obj, 1, unspecified(1)), expect)

  expect_identical(
    vec_assign(obj, 1, c(NA, NA, NA), slice_value = TRUE),
    expect
  )
  expect_identical(
    vec_assign(obj, 1, unspecified(3), slice_value = TRUE),
    expect
  )
})

test_that("slice-assign casts to `to` before falling back to `[<-` (#443)", {
  called <- FALSE

  local_methods(
    vec_proxy.vctrs_proxy = proxy_deref,
    vec_ptype2.vctrs_proxy = function(...) NULL,
    vec_ptype2.vctrs_proxy.vctrs_foobar = function(...) new_proxy(NA),
    vec_cast.vctrs_foobar = function(...) NULL,
    vec_cast.vctrs_foobar.vctrs_proxy = function(x, ...) foobar(proxy_deref(x)),
    `[<-.vctrs_foobar` = function(x, i, value) {
      called <<- TRUE
      expect_identical(value, foobar(10))
    }
  )

  x <- foobar(1)
  y <- new_proxy(10)
  vec_slice(x, 1) <- y
  expect_true(called)
})

test_that("index and value are sliced before falling back", {
  # Works around a bug in base R `[<-` before we call it, in particular, related
  # to the fact that `[<-` doesn't allow `NA` in subassign indices, but we do.

  lhs <- foobar(int(1:5))

  # With location vector
  expect_identical(
    vec_assign(lhs, c(NA, 1), foobar(int(6:7))),
    foobar(int(7, 2:5))
  )

  # With location vector and `slice_value = TRUE`
  expect_identical(
    vec_assign(lhs, c(NA, 1), foobar(int(6:10)), slice_value = TRUE),
    foobar(int(6, 2:5))
  )

  # With condition vector
  expect_identical(
    vec_assign(lhs, c(NA, FALSE, TRUE, NA, TRUE), foobar(int(6:9))),
    foobar(int(1, 2, 7, 4, 9))
  )

  # With condition vector and `slice_value = TRUE`
  expect_identical(
    vec_assign(
      lhs,
      c(NA, FALSE, TRUE, NA, TRUE),
      foobar(int(6:10)),
      slice_value = TRUE
    ),
    foobar(int(1, 2, 8, 4, 10))
  )
})

test_that("size 1 value is expected to be handled by the `[<-` fallback", {
  # i.e., we don't pre recycle size 1 value to the size of the index because
  # we expect that most `[<-` fallbacks eventually call base `[<-`, which
  # recycles size 1 value efficiently. See `vec_assign_fallback()`.
  lhs <- foobar(1:4)
  rhs <- foobar(0L)
  exp <- foobar(c(1L, 0L, 3L, 0L))

  expect_identical(vec_assign(lhs, c(2L, 4L), rhs), exp)

  expect_identical(
    vec_assign(lhs, c(FALSE, TRUE, FALSE, TRUE), rhs),
    exp
  )
  expect_identical(
    vec_assign(lhs, c(FALSE, TRUE, FALSE, TRUE), rhs, slice_value = TRUE),
    exp
  )
})

test_that("can assign to data frame", {
  x <- data_frame(x = 1:3)
  y <- data_frame(x = 20)
  expect_identical(vec_assign(x, 2, y), data_frame(x = int(1, 20, 3)))
})

test_that("can assign to data frame with `slice_value`", {
  x <- data_frame(x = 1:4)
  y <- data_frame(x = 21:24)
  i <- c(TRUE, FALSE, NA, TRUE)
  expect_identical(
    vec_assign(x, i, y, slice_value = TRUE),
    data_frame(x = int(21, 2, 3, 24))
  )
})

test_that("can assign to a data frame with matrix columns (#625)", {
  df <- tibble(x = 1:2, y = matrix(1:4, nrow = 2))
  expect_identical(vec_assign(df, 2L, df[1, ]), vec_slice(df, c(1, 1)))
})

test_that("assigning to a factor doesn't produce corrupt levels (#853)", {
  x <- factor(c("a", NA), levels = c("a", "b"))
  value <- factor("b", levels = "b")

  res <- vec_assign(x, 2, value)
  expect_identical(res, factor(c("a", "b")))

  res <- vec_assign(x, 1:2, value)
  expect_identical(res, factor(c("b", "b"), levels = c("a", "b")))
})

test_that("can slice-assign unspecified vectors with default type2 method", {
  local_rational_class()
  x <- rational(1:2, 2:3)
  x[[1]] <- NA
  expect_identical(x, rational(c(NA, 2L), c(NA, 3L)))
})

test_that("`vec_assign()` evaluates arg lazily", {
  expect_silent(vec_assign(1L, 1L, 1L, x_arg = print("oof")))
  expect_silent(vec_assign(1L, 1L, 1L, value_arg = print("oof")))
})

test_that("`vec_assign()` requires recyclable value", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, 1:2, 1:3),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, 1:2, 1:2, slice_value = TRUE),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

test_that("logical subscripts must match size of indexed vector", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:2, c(TRUE, FALSE, TRUE), 5),
      class = "vctrs_error_subscript_size"
    ))
  })

  expect_snapshot(
    (expect_error(
      vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]),
      class = "vctrs_error_subscript_size"
    ))
  )
})

test_that("must assign existing elements", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, 5, 10),
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_assign(1:3, "foo", 10),
      "unnamed vector"
    ))
    (expect_error(
      vec_slice(letters, -100) <- "foo",
      class = "vctrs_error_subscript_oob"
    ))
    (expect_error(
      vec_assign(set_names(letters), "foo", "bar"),
      class = "vctrs_error_subscript_oob"
    ))
  })
})

test_that("must assign with proper negative locations", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:3, c(-1, 1), 1:2),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_assign(1:3, c(-1, NA), 1:2),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("`vec_assign()` error args can be overridden", {
  expect_snapshot({
    (expect_error(
      vec_assign(1:2, 1L, "x", x_arg = "foo", value_arg = "bar"),
      class = "vctrs_error_incompatible_type"
    ))
    (expect_error(
      vec_assign(1:2, 1L, 1:2, value_arg = "bar"),
      class = "vctrs_error_recycle_incompatible_size"
    ))
  })
})

test_that("names are not assigned by default", {
  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  vec_out <- c(a = 1L, b = 4L, c = 3L)
  expect_identical(
    vec_assign(vec_x, 2, vec_y),
    vec_out
  )

  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "FOO")
  df_out <- new_data_frame(list(x = c(1L, 4L, 3L)), row.names = letters[1:3])
  expect_identical(
    vec_assign(df_x, 2, df_y),
    df_out
  )

  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("FOO"))
  mat_out <- matrix(c(1L, 4L, 3L), dimnames = list(letters[1:3]))
  expect_identical(
    vec_assign(mat_x, 2, mat_y),
    mat_out
  )

  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y),
    row.names = c("quux")
  )
  nested_out <- new_data_frame(
    list(df = df_out, mat = mat_out, vec = vec_out),
    row.names = c("foo", "bar", "baz")
  )
  expect_identical(
    vec_assign(nested_x, 2, nested_y),
    nested_out
  )
})

test_that("can optionally assign names", {
  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L)
  vec_out <- c(a = 1L, FOO = 4L, c = 3L)
  expect_identical(
    vec_assign_params(vec_x, 2, vec_y, assign_names = TRUE),
    vec_out
  )

  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4L), row.names = "FOO")
  df_out <- new_data_frame(
    list(x = c(1L, 4L, 3L)),
    row.names = c("a", "FOO", "c")
  )
  expect_identical(
    vec_assign_params(df_x, 2, df_y, assign_names = TRUE),
    df_out
  )

  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4L, 1, dimnames = list("FOO"))
  mat_out <- matrix(c(1L, 4L, 3L), dimnames = list(c("a", "FOO", "c")))
  expect_identical(
    vec_assign_params(mat_x, 2, mat_y, assign_names = TRUE),
    mat_out
  )

  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y),
    row.names = c("quux")
  )
  nested_out <- new_data_frame(
    list(df = df_out, mat = mat_out, vec = vec_out),
    row.names = c("foo", "quux", "baz")
  )

  expect_identical(
    vec_assign_params(nested_x, 2, nested_y, assign_names = TRUE),
    nested_out
  )
})

test_that("can optionally assign names with `slice_value`", {
  vec_x <- set_names(1:3, letters[1:3])
  vec_y <- c(FOO = 4L, BAR = 5L, BAZ = 6L)
  vec_out <- c(a = 1L, BAR = 5L, BAZ = 6L)
  expect_identical(
    vec_assign_params(
      vec_x,
      c(FALSE, TRUE, TRUE),
      vec_y,
      assign_names = TRUE,
      slice_value = TRUE
    ),
    vec_out
  )

  df_x <- new_data_frame(list(x = 1:3), row.names = letters[1:3])
  df_y <- new_data_frame(list(x = 4:6), row.names = c("FOO", "BAR", "BAZ"))
  df_out <- new_data_frame(
    list(x = c(1L, 5L, 6L)),
    row.names = c("a", "BAR", "BAZ")
  )
  expect_identical(
    vec_assign_params(
      df_x,
      c(FALSE, TRUE, TRUE),
      df_y,
      assign_names = TRUE,
      slice_value = TRUE
    ),
    df_out
  )

  mat_x <- matrix(1:3, 3, dimnames = list(letters[1:3]))
  mat_y <- matrix(4:6, 3, dimnames = list(c("FOO", "BAR", "BAZ")))
  mat_out <- matrix(c(1L, 5L, 6L), dimnames = list(c("a", "BAR", "BAZ")))
  expect_identical(
    vec_assign_params(
      mat_x,
      c(FALSE, TRUE, TRUE),
      mat_y,
      assign_names = TRUE,
      slice_value = TRUE
    ),
    mat_out
  )

  nested_x <- new_data_frame(
    list(df = df_x, mat = mat_x, vec = vec_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(
    list(df = df_y, mat = mat_y, vec = vec_y),
    row.names = c(c("x", "y", "z"))
  )
  nested_out <- new_data_frame(
    list(df = df_out, mat = mat_out, vec = vec_out),
    row.names = c("foo", "y", "z")
  )

  expect_identical(
    vec_assign_params(
      nested_x,
      c(FALSE, TRUE, TRUE),
      nested_y,
      assign_names = TRUE,
      slice_value = TRUE
    ),
    nested_out
  )
})

test_that("can optionally assign names (OO case)", {
  # In case upstream attributes handling changes
  skip_on_cran()

  # `set_names()` must be on the inside, otherwise the POSIXlt object
  # gets a `balanced` attribute of `NA`
  oo_x <- as_posixlt(set_names(
    c("2020-01-01", "2020-01-02", "2020-01-03"),
    letters[1:3]
  ))
  oo_y <- as_posixlt(c(FOO = "2020-01-04"))
  oo_out <- as_posixlt(c(
    a = "2020-01-01",
    FOO = "2020-01-04",
    c = "2020-01-03"
  ))
  expect_identical(
    vec_assign_params(oo_x, 2, oo_y, assign_names = TRUE),
    oo_out
  )

  nested_x <- new_data_frame(
    list(oo = oo_x),
    row.names = c("foo", "bar", "baz")
  )
  nested_y <- new_data_frame(list(oo = oo_y), row.names = c("quux"))
  nested_out <- new_data_frame(
    list(oo = oo_out),
    row.names = c("foo", "quux", "baz")
  )

  expect_identical(
    vec_assign_params(nested_x, 2, nested_y, assign_names = TRUE),
    nested_out
  )
})

test_that("assigning names clears existing names even if the new value doesn't have any (#2019)", {
  x <- c(a = 1, b = 2)

  # Keeps existing names
  expect_identical(
    vec_assign_params(x, 1L, 0),
    c(a = 0, b = 2)
  )
  expect_identical(
    vec_assign_params(x, 1L, c(c = 0)),
    c(a = 0, b = 2)
  )

  # Clears or replaces names
  expect_identical(
    vec_assign_params(x, 1L, 0, assign_names = TRUE),
    set_names(c(0, 2), c("", "b"))
  )
  expect_identical(
    vec_assign_params(x, 1L, c(c = 0), assign_names = TRUE),
    set_names(c(0, 2), c("c", "b"))
  )
})

test_that("assignment requires that the value proxy is the same type as the output proxy", {
  x <- foobar(1)
  y <- foobar("a")

  local_foobar_proxy()
  local_methods(
    vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) x
  )

  expect_error(
    vec_assign(x, 1, y),
    "`double` incompatible with `value` proxy of type `character`"
  )
})

test_that("assignment allows a df `value`'s column to be a different type than its proxy (#1082)", {
  x <- new_data_frame(list(x = foobar(1)))
  y <- new_data_frame(list(x = foobar(2)))

  local_methods(
    # proxying foobar wraps it in a 1 col df
    vec_proxy.vctrs_foobar = function(x, ...) {
      attributes(x) <- NULL
      new_data_frame(list(vec = x))
    },
    # restoring extracts the column
    vec_restore.vctrs_foobar = function(x, to, ...) {
      foobar(x$vec)
    },
    vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) x
  )

  expect1 <- new_data_frame(list(x = foobar(c(1, 1))))
  expect2 <- new_data_frame(list(x = foobar(2)))

  expect_identical(vec_rbind(x, x), expect1)
  expect_identical(vec_assign(x, 1, y), expect2)
})

test_that("monitoring: assignment to a data frame with unshared columns doesn't overwrite (#986)", {
  # We now require R >= 4.0.0, so this test no longer needs the < 4.0.0
  # branches, but we keep it around for historical reference

  x <- new_df_unshared_col()
  value <- new_data_frame(list(x = 2))
  expect <- new_data_frame(list(x = 1L))

  # - On R < 4.0.0, the NAMED value of the column is 0.
  # - On R >= 4.0.0, the refcnt of the column is 1 from the call to
  #   `SET_VECTOR_ELT()` in `new_df_unshared_col()`.
  expect_false(maybe_shared_col(x, 1L))

  new <- vec_assign(x, 1, value)

  # On R < 4.0.0, `vec_assign()` shallow duplicates `x`, which recursively
  # bumps the NAMED-ness of each column of `x` to the max value of 7 by
  # calling `ENSURE_NAMEDMAX()` on it. So the columns of `x` are all considered
  # shared from that.

  # On R >= 4.0.0, references are tracked more precisely.
  # - `new_df_unshared_col()` calls `SET_VECTOR_ELT()` when setting the
  #   column into `x`, bumping the column's namedness to 1.
  # - Then, at the start of `df_assign()`, `x` is shallow duplicated and
  #   assigned to `out`. This calls `ENSURE_NAMEDMAX()` on each column,
  #   however this does nothing on R 4.0.0. The refcnt of each column is instead
  #   incremented by 1 by calls to `SET_VECTOR_ELT()` in `duplicate1()`.
  #   So now it is at 2.
  # - But then in `df_assign()` we use `SET_VECTOR_ELT()` on `out`, overwriting
  #   each column. This actually decrements the refcnt on the value that was
  #   in `out` before the column was overwritten. The column of `out` that it
  #   decrements the refcnt for is the same SEXP as that column in `x`, so now
  #   it is back to 1, and it is not considered shared.

  if (getRversion() >= "4.0.0") {
    expect_false(maybe_shared_col(x, 1L))
  } else {
    expect_true(maybe_shared_col(x, 1L))
  }

  # Expect no changes to `x`!
  expect_identical(x, expect)
})

test_that("monitoring: assignment to atomic vectors doesn't modify by reference", {
  x <- c(1, 2, 3)
  expect <- c(1, 2, 3)

  vec_assign(x, 2, 3)

  expect_identical(x, expect)
})

test_that("monitoring: assignment to POSIXlt doesn't modify by reference (#1951)", {
  original <- as.POSIXlt("2020-11-01")
  expect_original <- as.POSIXlt("2020-11-01")

  value <- as.POSIXlt("2020-12-02")

  expect <- as.POSIXlt("2020-12-02")
  actual <- vec_assign(original, 1, value)
  expect_identical(expect, actual)

  expect_identical(original, expect_original)
})

test_that("monitoring: assignment to `vctrs_rcrd` doesn't modify by reference (#1951)", {
  original <- new_rcrd(list(x = 1:5))
  expect_original <- new_rcrd(list(x = 1:5))

  value <- new_rcrd(list(x = 0L))

  expect <- new_rcrd(list(x = c(0L, 2:5)))
  actual <- vec_assign(original, 1, value)
  expect_identical(expect, actual)

  expect_identical(original, expect_original)
})

# vec_assign + compact_seq -------------------------------------------------

test_that("can assign base vectors with compact seqs", {
  # `start` is 0-based
  start <- 1L
  size <- 2L
  increasing <- TRUE

  x <- c(FALSE, FALSE, FALSE)
  value <- c(TRUE, NA, TRUE)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, c(FALSE, NA, TRUE))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- c(1L, 2L, 3L)
  value <- c(4L, 5L, 6L)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, c(1L, 5L, 6L))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- c(1, 2, 3)
  value <- c(4, 5, 6)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, c(1, 5, 6))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- c(1i, 2i, 3i)
  value <- c(4i, 5i, 6i)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, c(1i, 5i, 6i))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- c("1", "2", "3")
  value <- c("4", "5", "6")
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, c("1", "5", "6"))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- raw2(1, 2, 3)
  value <- raw2(4, 5, 6)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, raw2(1, 5, 6))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- list(1, 2, 3)
  value <- list(4, 5, 6)
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, list(1, 5, 6))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )
})

test_that("can assign shaped base vectors with compact seqs", {
  # `start` is 0-based
  start <- 1L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix

  x <- mat(c(FALSE, FALSE, FALSE))
  value <- mat(c(TRUE, NA, TRUE))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(c(FALSE, NA, TRUE)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(c(1L, 2L, 3L))
  value <- mat(c(4L, 5L, 6L))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(c(1L, 5L, 6L)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(c(1, 2, 3))
  value <- mat(c(4, 5, 6))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(c(1, 5, 6)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(c(1i, 2i, 3i))
  value <- mat(c(4i, 5i, 6i))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(c(1i, 5i, 6i)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(c("1", "2", "3"))
  value <- mat(c("4", "5", "6"))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(c("1", "5", "6")))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(raw2(1, 2, 3))
  value <- mat(raw2(4, 5, 6))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(raw2(1, 5, 6)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )

  x <- mat(list(1, 2, 3))
  value <- mat(list(4, 5, 6))
  value_sliced <- vec_slice_seq(value, start, size, increasing)
  y <- vec_assign_seq(x, value_sliced, start, size, increasing)
  expect_identical(y, mat(list(1, 5, 6)))
  expect_identical(
    vec_assign_seq(x, value, start, size, increasing, slice_value = TRUE),
    y
  )
})

test_that("can assign shaped base vectors with compact seqs and recycled `value`", {
  # `start` is 0-based
  start <- 1L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(
    vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing),
    mat(lgl(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing),
    mat(int(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing),
    mat(dbl(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(cpl2(1, 2, 3)), NA, start, size, increasing),
    mat(cpl2(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing),
    mat(chr("1", NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing),
    mat(raw2(1, 1, 1))
  )
  expect_identical(
    vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing),
    mat(list(1, NULL, NULL))
  )
})

test_that("can assign shaped base vectors with decreasing compact seqs and recycled `value`", {
  # `start` is 0-based
  start <- 2L
  size <- 2L
  increasing <- FALSE
  mat <- as.matrix
  expect_identical(
    vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing),
    mat(lgl(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing),
    mat(int(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing),
    mat(dbl(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(cpl2(1, 2, 3)), NA, start, size, increasing),
    mat(cpl2(1, NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing),
    mat(chr("1", NA, NA))
  )
  expect_identical(
    vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing),
    mat(raw2(1, 1, 1))
  )
  expect_identical(
    vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing),
    mat(list(1, NULL, NULL))
  )
})

test_that("can assign shaped base vectors with size 0 compact seqs", {
  # `start` is 0-based
  start <- 1L
  size <- 0L
  increasing <- TRUE
  mat <- as.matrix

  expect_identical(
    vec_assign_seq(mat(lgl(1, 0, 1)), NA, start, size, increasing),
    mat(mat(lgl(1, 0, 1)))
  )
  expect_identical(
    vec_assign_seq(mat(int(1, 2, 3)), NA, start, size, increasing),
    mat(int(1, 2, 3))
  )
  expect_identical(
    vec_assign_seq(mat(dbl(1, 2, 3)), NA, start, size, increasing),
    mat(dbl(1, 2, 3))
  )
  expect_identical(
    vec_assign_seq(mat(cpl(1, 2, 3)), NA, start, size, increasing),
    mat(cpl(1, 2, 3))
  )
  expect_identical(
    vec_assign_seq(mat(chr("1", "2", "3")), NA, start, size, increasing),
    mat(chr("1", "2", "3"))
  )
  expect_identical(
    vec_assign_seq(mat(raw2(1, 2, 3)), raw2(1), start, size, increasing),
    mat(raw2(1, 2, 3))
  )
  expect_identical(
    vec_assign_seq(mat(list(1, 2, 3)), NA, start, size, increasing),
    mat(list(1, 2, 3))
  )
})

test_that("can assign object of any dimensionality with compact seqs", {
  x1 <- ones(3)
  x2 <- ones(3, 4)
  x3 <- ones(3, 4, 5)
  x4 <- ones(3, 4, 5, 6)

  # `start` is 0-based
  start <- 0L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix

  expect_identical(
    vec_assign_seq(x1, 2, start, size, increasing),
    array(rep(c(2, 2, 1), 1), dim = 3)
  )
  expect_identical(
    vec_assign_seq(x2, 2, start, size, increasing),
    array(rep(c(2, 2, 1), 4), dim = c(3, 4))
  )
  expect_identical(
    vec_assign_seq(x3, 2, start, size, increasing),
    array(rep(c(2, 2, 1), 20), dim = c(3, 4, 5))
  )
  expect_identical(
    vec_assign_seq(x4, 2, start, size, increasing),
    array(rep(c(2, 2, 1), 120), dim = c(3, 4, 5, 6))
  )
})

# vec_assign + compact_condition -----------------------------------------------

test_that("can assign base vectors with compact conditions", {
  i <- c(FALSE, TRUE, TRUE)
  i_compact <- as_compact_condition(i)

  x <- c(FALSE, FALSE, FALSE)
  value <- c(TRUE, NA, TRUE)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, c(FALSE, NA, TRUE))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- c(1L, 2L, 3L)
  value <- c(4L, 5L, 6L)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, c(1L, 5L, 6L))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- c(1, 2, 3)
  value <- c(4, 5, 6)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, c(1, 5, 6))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- c(1i, 2i, 3i)
  value <- c(4i, 5i, 6i)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, c(1i, 5i, 6i))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- c("1", "2", "3")
  value <- c("4", "5", "6")
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, c("1", "5", "6"))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- raw2(1, 2, 3)
  value <- raw2(4, 5, 6)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, raw2(1, 5, 6))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- list(1, 2, 3)
  value <- list(4, 5, 6)
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, list(1, 5, 6))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )
})

test_that("can assign shaped base vectors with compact conditions", {
  i <- c(FALSE, TRUE, TRUE)
  i_compact <- as_compact_condition(i)

  mat <- as.matrix

  x <- mat(c(FALSE, FALSE, FALSE))
  value <- mat(c(TRUE, NA, TRUE))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(c(FALSE, NA, TRUE)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(c(1L, 2L, 3L))
  value <- mat(c(4L, 5L, 6L))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(c(1L, 5L, 6L)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(c(1, 2, 3))
  value <- mat(c(4, 5, 6))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(c(1, 5, 6)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(c(1i, 2i, 3i))
  value <- mat(c(4i, 5i, 6i))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(c(1i, 5i, 6i)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(c("1", "2", "3"))
  value <- mat(c("4", "5", "6"))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(c("1", "5", "6")))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(raw2(1, 2, 3))
  value <- mat(raw2(4, 5, 6))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(raw2(1, 5, 6)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )

  x <- mat(list(1, 2, 3))
  value <- mat(list(4, 5, 6))
  value_sliced <- vec_slice(value, i)
  y <- vec_assign_compact_condition(x, i_compact, value_sliced)
  expect_identical(y, mat(list(1, 5, 6)))
  expect_identical(
    vec_assign_compact_condition(x, i_compact, value, slice_value = TRUE),
    y
  )
})

test_that("can assign shaped base vectors with compact conditions and recycled `value`", {
  i <- as_compact_condition(c(FALSE, TRUE, TRUE))
  mat <- as.matrix

  expect_identical(
    vec_assign_compact_condition(mat(lgl(1, 0, 1)), i, NA),
    mat(lgl(1, NA, NA))
  )
  expect_identical(
    vec_assign_compact_condition(mat(int(1, 2, 3)), i, NA),
    mat(int(1, NA, NA))
  )
  expect_identical(
    vec_assign_compact_condition(mat(dbl(1, 2, 3)), i, NA),
    mat(dbl(1, NA, NA))
  )
  expect_identical(
    vec_assign_compact_condition(mat(cpl2(1, 2, 3)), i, NA),
    mat(cpl2(1, NA, NA))
  )
  expect_identical(
    vec_assign_compact_condition(mat(chr("1", "2", "3")), i, NA),
    mat(chr("1", NA, NA))
  )
  expect_identical(
    vec_assign_compact_condition(mat(raw2(1, 2, 3)), i, raw2(1)),
    mat(raw2(1, 1, 1))
  )
  expect_identical(
    vec_assign_compact_condition(mat(list(1, 2, 3)), i, NA),
    mat(list(1, NULL, NULL))
  )
})

test_that("can assign shaped base vectors with size 0 compact conditions", {
  i <- as_compact_condition(logical())
  mat <- as.matrix

  expect_identical(
    vec_assign_compact_condition(mat(lgl(1, 0, 1)), i, NA),
    mat(mat(lgl(1, 0, 1)))
  )
  expect_identical(
    vec_assign_compact_condition(mat(int(1, 2, 3)), i, NA),
    mat(int(1, 2, 3))
  )
  expect_identical(
    vec_assign_compact_condition(mat(dbl(1, 2, 3)), i, NA),
    mat(dbl(1, 2, 3))
  )
  expect_identical(
    vec_assign_compact_condition(mat(cpl(1, 2, 3)), i, NA),
    mat(cpl(1, 2, 3))
  )
  expect_identical(
    vec_assign_compact_condition(mat(chr("1", "2", "3")), i, NA),
    mat(chr("1", "2", "3"))
  )
  expect_identical(
    vec_assign_compact_condition(mat(raw2(1, 2, 3)), i, raw2(1)),
    mat(raw2(1, 2, 3))
  )
  expect_identical(
    vec_assign_compact_condition(mat(list(1, 2, 3)), i, NA),
    mat(list(1, 2, 3))
  )
})

test_that("can assign object of any dimensionality with compact conditions", {
  x1 <- ones(3)
  x2 <- ones(3, 4)
  x3 <- ones(3, 4, 5)
  x4 <- ones(3, 4, 5, 6)

  i <- as_compact_condition(c(TRUE, TRUE, FALSE))

  expect_identical(
    vec_assign_compact_condition(x1, i, 2),
    array(rep(c(2, 2, 1), 1), dim = 3)
  )
  expect_identical(
    vec_assign_compact_condition(x2, i, 2),
    array(rep(c(2, 2, 1), 4), dim = c(3, 4))
  )
  expect_identical(
    vec_assign_compact_condition(x3, i, 2),
    array(rep(c(2, 2, 1), 20), dim = c(3, 4, 5))
  )
  expect_identical(
    vec_assign_compact_condition(x4, i, 2),
    array(rep(c(2, 2, 1), 120), dim = c(3, 4, 5, 6))
  )
})

test_that("vec_slice throws error with non-vector inputs", {
  expect_error(vec_slice(environment(), 1L), class = "vctrs_error_scalar_type")
})

test_that("vec_slice throws error with non-vector subscripts", {
  expect_snapshot({
    (expect_error(
      vec_slice(1:3, Sys.Date()),
      class = "vctrs_error_subscript_type"
    ))
    (expect_error(
      vec_slice(1:3, matrix(TRUE, nrow = 1)),
      class = "vctrs_error_subscript_type"
    ))
  })
})

test_that("can subset base vectors", {
  i <- 2:3
  expect_identical(vec_slice(lgl(1, 0, 1), i), lgl(0, 1))
  expect_identical(vec_slice(int(1, 2, 3), i), int(2, 3))
  expect_identical(vec_slice(dbl(1, 2, 3), i), dbl(2, 3))
  expect_identical(vec_slice(cpl(1, 2, 3), i), cpl(2, 3))
  expect_identical(vec_slice(chr("1", "2", "3"), i), chr("2", "3"))
  expect_identical(vec_slice(raw2(1, 2, 3), i), raw2(2, 3))
  expect_identical(vec_slice(list(1, 2, 3), i), list(2, 3))
})

test_that("can subset shaped base vectors", {
  i <- 2:3
  mat <- as.matrix
  expect_identical(vec_slice(mat(lgl(1, 0, 1)), i), mat(lgl(0, 1)))
  expect_identical(vec_slice(mat(int(1, 2, 3)), i), mat(int(2, 3)))
  expect_identical(vec_slice(mat(dbl(1, 2, 3)), i), mat(dbl(2, 3)))
  expect_identical(vec_slice(mat(cpl(1, 2, 3)), i), mat(cpl(2, 3)))
  expect_identical(vec_slice(mat(chr("1", "2", "3")), i), mat(chr("2", "3")))
  expect_identical(vec_slice(mat(raw2(1, 2, 3)), i), mat(raw2(2, 3)))
  expect_identical(vec_slice(mat(list(1, 2, 3)), i), mat(list(2, 3)))
})

test_that("can subset with missing indices", {
  for (i in list(int(2L, NA), lgl(FALSE, TRUE, NA))) {
    expect_identical(vec_slice(lgl(1, 0, 1), i), lgl(0, NA))
    expect_identical(vec_slice(int(1, 2, 3), i), int(2, NA))
    expect_identical(vec_slice(dbl(1, 2, 3), i), dbl(2, NA))
    expect_identical(vec_slice(cpl2(1, 2, 3), i), cpl2(2, NA))
    expect_identical(vec_slice(chr("1", "2", "3"), i), c("2", NA))
    expect_identical(vec_slice(raw2(1, 2, 3), i), raw2(2, 0))
    expect_identical(vec_slice(list(1, 2, 3), i), list(2, NULL))
  }
})

test_that("can subset with a recycled NA", {
  local_name_repair_quiet()

  expect_identical(vec_slice(1:3, NA), int(NA, NA, NA))
  expect_identical(vec_slice(new_vctr(1:3), NA), new_vctr(int(NA, NA, NA)))

  rownames <- rep_len("", nrow(mtcars))
  rownames <- vec_as_names(rownames, repair = "unique")
  expect_identical(
    vec_slice(mtcars, NA),
    structure(mtcars[NA, ], row.names = rownames)
  )
})

test_that("can subset with a recycled TRUE", {
  expect_identical(vec_slice(1:3, TRUE), 1:3)
  expect_identical(vec_slice(mtcars, TRUE), mtcars)
  expect_identical(vec_slice(new_vctr(1:3), TRUE), new_vctr(1:3))
  expect_identical(vec_as_location(TRUE, 2), 1:2)
})

test_that("can subset with a recycled FALSE", {
  expect_identical(vec_slice(1:3, FALSE), int())
  expect_identical(vec_slice(mtcars, FALSE), mtcars[NULL, ])
  expect_identical(vec_slice(new_vctr(1:3), FALSE), new_vctr(integer()))
})

test_that("can't index beyond the end of a vector", {
  expect_snapshot({
    (expect_error(vec_slice(1:2, 3L), class = "vctrs_error_subscript_oob"))
    (expect_error(vec_slice(1:2, -3L), class = "vctrs_error_subscript_oob"))
  })
})

test_that("slicing non existing elements fails", {
  expect_error(
    vec_as_location("foo", 1L, "f"),
    class = "vctrs_error_subscript_oob"
  )
  expect_error(vec_slice(c(f = 1), "foo"), class = "vctrs_error_subscript_oob")
})

test_that("can subset object of any dimensionality", {
  x0 <- c(1, 1)
  x1 <- ones(2)
  x2 <- ones(2, 3)
  x3 <- ones(2, 3, 4)
  x4 <- ones(2, 3, 4, 5)

  expect_equal(vec_slice(x0, 1L), 1)
  expect_identical(vec_slice(x1, 1L), ones(1))
  expect_identical(vec_slice(x2, 1L), ones(1, 3))
  expect_identical(vec_slice(x3, 1L), ones(1, 3, 4))
  expect_identical(vec_slice(x4, 1L), ones(1, 3, 4, 5))
})

test_that("can subset using logical subscript", {
  x0 <- c(1, 1)

  expect_identical(vec_slice(x0, TRUE), x0)
  expect_identical(vec_slice(x0, c(TRUE, FALSE)), 1)

  expect_error(
    vec_slice(x0, c(TRUE, FALSE, TRUE)),
    class = "vctrs_error_subscript_size"
  )

  expect_error(
    vec_slice(x0, lgl()),
    class = "vctrs_error_subscript_size"
  )

  expect_error(
    vec_slice(mtcars, c(TRUE, FALSE)),
    class = "vctrs_error_subscript_size"
  )
})

test_that("can subset data frame columns", {
  df <- data.frame(x = 1:2)
  df$y <- data.frame(a = 2:1)

  expect_equal(vec_slice(df, 1L)$y, vec_slice(df$y, 1L))
})

test_that("can subset empty data frames", {
  df <- new_data_frame(n = 3L)
  expect_equal(vec_size(vec_slice(df, integer())), 0)
  expect_equal(vec_size(vec_slice(df, 1L)), 1)
  expect_equal(vec_size(vec_slice(df, 1:3)), 3)

  df$df <- df
  expect_equal(vec_size(vec_slice(df, integer())), 0)
  expect_equal(vec_size(vec_slice(df, 1L)), 1)
  expect_equal(vec_size(vec_slice(df, 1:3)), 3)
})

test_that("ignores NA in logical subsetting", {
  x <- c(NA, 1, 2)
  expect_equal(vec_slice(x, x > 0), c(NA, 1, 2))
})

test_that("ignores NA in integer subsetting", {
  expect_equal(vec_slice(0:2, c(NA, 2:3)), c(NA, 1, 2))
})

test_that("can't slice with missing argument", {
  expect_error(vec_slice(1:3))
  expect_error(vec_slice(mtcars))
  expect_error(vec_slice(new_vctr(1:3)))
})

test_that("can slice with NULL argument", {
  expect_identical(vec_slice(1:3, NULL), integer())
  expect_identical(vec_slice(iris, NULL), iris[0, ])
  expect_identical(vec_slice(new_vctr(1:3), NULL), new_vctr(integer()))
})

test_that("slicing unclassed structures preserves attributes", {
  x <- structure(1:3, foo = "bar")
  expect_identical(vec_slice(x, 1L), structure(1L, foo = "bar"))
})

test_that("can slice with negative indices", {
  expect_identical(vec_slice(1:3, -c(1L, 3L)), 2L)
  expect_identical(vec_slice(mtcars, -(1:30)), vec_slice(mtcars, 31:32))

  expect_error(vec_slice(1:3, -c(1L, NA)), class = "vctrs_error_subscript_type")
  expect_error(vec_slice(1:3, c(-1L, 1L)), class = "vctrs_error_subscript_type")
})

test_that("0 is ignored in negative indices", {
  expect_identical(vec_slice(1:3, c(-2L, 0L)), int(1L, 3L))
  expect_identical(vec_slice(1:3, c(0L, -2L)), int(1L, 3L))
})

test_that("0 is ignored in positive indices", {
  expect_identical(vec_slice(1:3, 0L), int())
  expect_identical(vec_slice(1:3, c(0L, 0L)), int())
  expect_identical(vec_slice(1:3, c(0L, 2L, 0L)), 2L)
})

test_that("can slice with double indices", {
  expect_identical(vec_slice(1:3, dbl(2, 3)), 2:3)
  expect_snapshot(
    (expect_error(
      vec_as_location(2^31, 3L),
      class = "vctrs_error_subscript_type"
    ))
  )
})

test_that("can slice with symbols", {
  expect_identical(vec_as_location(quote(b), 26, letters), 2L)
})

test_that("can `vec_slice()` S3 objects without dispatch infloop", {
  expect_identical(new_vctr(1:3)[1], new_vctr(1L))
  expect_identical(new_vctr(as.list(1:3))[1], new_vctr(list(1L)))
})

test_that("can `vec_slice()` records", {
  out <- vec_slice(new_rcrd(list(a = 1L, b = 2L)), rep(1, 3))
  expect_size(out, 3)

  out <- vec_init(new_rcrd(list(a = 1L, b = 2L)), 2)
  expect_size(out, 2)
})

test_that("vec_restore() is called after proxied slicing", {
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) "dispatch"
  )
  expect_identical(vec_slice(foobar(1:3), 2), "dispatch")
})

test_that("vec_slice() is proxied", {
  local_proxy()
  x <- vec_slice(new_proxy(1:3), 2:3)
  expect_identical(proxy_deref(x), 2:3)
})

test_that("dimensions are preserved by vec_slice()", {
  # Fallback case
  x <- foobar(1:4)
  dim(x) <- c(2, 2)
  dimnames(x) <- list(a = c("foo", "bar"), b = c("quux", "hunoz"))

  out <- vec_slice(x, 1)
  exp <- foobar(
    c(1L, 3L),
    dim = c(1, 2),
    dimnames = list(a = "foo", b = c("quux", "hunoz"))
  )
  expect_identical(out, exp)

  # Native case
  attrib <- NULL

  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, to, ...) attrib <<- attributes(x)
  )

  vec_slice(x, 1)

  exp <- list(dim = 1:2, dimnames = list(a = "foo", b = c("quux", "hunoz")))
  expect_identical(attrib, exp)
})

test_that("can slice shaped objects by name", {
  x <- matrix(1:2)

  expect_error(vec_slice(x, "foo"), "unnamed")

  dimnames(x) <- list(c("foo", "bar"))

  expect_equal(vec_slice(x, "foo"), vec_slice(x, 1L))
  expect_error(vec_slice(x, "baz"), class = "vctrs_error_subscript_oob")
})

test_that("vec_slice() unclasses input before calling `vec_restore()`", {
  oo <- NULL
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x,
    vec_restore.vctrs_foobar = function(x, ...) oo <<- is.object(x)
  )

  x <- foobar(1:4)
  dim(x) <- c(2, 2)

  vec_slice(x, 1)
  expect_false(oo)
})

test_that("can call `vec_slice()` from `[` methods with shaped objects without infloop", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) vec_slice(x, i)
  )

  x <- foobar(1:4)
  dim(x) <- c(2, 2)

  exp <- foobar(c(1L, 3L))
  dim(exp) <- c(1, 2)
  expect_identical(x[1], exp)
})

test_that("vec_slice() restores attributes on shaped S3 objects correctly", {
  x <- factor(c("a", "b", "c", "d", "e", "f"))
  dim(x) <- c(3, 2)

  expect <- factor(c("a", "c", "d", "f"), levels = levels(x))
  dim(expect) <- c(2, 2)

  expect_identical(vec_slice(x, c(1, 3)), expect)
})

test_that("vec_slice() falls back to `[` with S3 objects", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) "dispatched"
  )
  expect_identical(vec_slice(foobar(NA), 1), "dispatched")

  expect_error(
    vec_slice(foobar(list(NA)), 1),
    class = "vctrs_error_scalar_type"
  )
  local_methods(
    vec_proxy.vctrs_foobar = function(x, ...) x
  )
  expect_identical(vec_slice(foobar(list(NA)), 1), foobar(list(NA)))
})

test_that("vec_slice() doesn't restore when attributes have already been restored", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) structure("dispatched", foo = "bar"),
    vec_restore.vctrs_foobar = function(...) stop("not called")
  )
  expect_error(vec_slice(foobar(NA), 1), NA)
})

test_that("vec_slice() doesn't restore when `[` method intentionally dropped attributes", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) unstructure(NextMethod()),
    vec_restore.vctrs_foobar = function(...) stop("not called")
  )
  expect_identical(vec_slice(foobar(NA), 1), NA)
})

test_that("can vec_slice() without inflooping when restore calls math generics", {
  local_methods(
    new_foobar = function(x) {
      new_vctr(as.double(x), class = "vctrs_foobar")
    },
    vec_restore.vctrs_foobar = function(x, ...) {
      abs(x)
      sum(x)
      mean(x)
      is.finite(x)
      is.infinite(x)
      is.nan(x)
      new_foobar(x)
    }
  )
  expect_identical(new_foobar(1:10)[1:2], new_foobar(1:2))
})

test_that("vec_restore() is called after slicing data frames", {
  local_methods(
    vec_restore.vctrs_tabble = function(...) "dispatched"
  )
  df <- structure(mtcars, class = c("vctrs_tabble", "data.frame"))
  expect_identical(vec_slice(df, 1), "dispatched")
})

test_that("additional subscripts are forwarded to `[`", {
  local_methods(
    `[.vctrs_foobar` = function(x, i, ...) vec_index(x, i, ...)
  )

  x <- foobar(c("foo", "bar", "quux", "hunoz"))
  dim(x) <- c(2, 2)

  exp <- foobar("quux")
  dim(exp) <- c(1, 1)

  expect_identical(x[1, 2], exp)
})

test_that("can use names to vec_slice() a named object", {
  x0 <- c(a = 1, b = 2)
  x1 <- c(a = 1, a = 2)

  expect_identical(vec_slice(x0, letters[1]), c(a = 1))
  expect_identical(vec_slice(x0, letters[2:1]), c(b = 2, a = 1))
  expect_identical(vec_slice(x1, letters[1]), c(a = 1))

  expect_error(vec_slice(x0, letters[3:1]), class = "vctrs_error_subscript_oob")
  expect_error(vec_slice(x1, letters[2]), class = "vctrs_error_subscript_oob")
})

test_that("can't use names to vec_slice() an unnamed object", {
  expect_error(
    vec_slice(1:3, letters[1]),
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
  expect_error(
    vec_slice(1:3, letters[25:27]),
    "Can't use character names to index an unnamed vector.",
    fixed = TRUE
  )
})

test_that("can slice with missing character indices (#244)", {
  expect_identical(vec_as_location(na_chr, 2L, c("x", "")), na_int)
  expect_identical(vec_slice(c(x = 1), na_chr), set_names(na_dbl, ""))
  expect_identical(vec_slice(c(x = "foo"), na_chr), set_names(na_chr, ""))
})

test_that("can slice with numerics (#577)", {
  expect_identical(vec_as_location(1:2, 3), 1:2)
  expect_error(vec_as_location(1:2, 3.5), class = "vctrs_error_cast_lossy")
})

test_that("missing indices don't create NA names", {
  x <- set_names(letters)
  expect_identical(vec_slice(x, na_int), set_names(na_chr, ""))
  expect_identical(
    vec_slice(x, int(1, NA, 3, NA)),
    chr(a = "a", NA, c = "c", NA)
  )

  # Preserves existing NA names
  x <- set_names(1:2, c(NA, "foo"))
  expect_identical(vec_slice(x, 1:2), x)
})

test_that("vec_slice() asserts vectorness (#301)", {
  expect_error(vec_slice(NULL, 1), class = "vctrs_error_scalar_type")
})

test_that("slicing an unspecified logical vector returns a logical vector", {
  expect_identical(vec_slice(NA, integer()), logical())
  expect_identical(vec_slice(NA, c(1, 1)), c(NA, NA))
})

test_that("slicing an unspecified() object returns an unspecified()", {
  expect_identical(vec_slice(unspecified(1), integer()), unspecified())
  expect_identical(vec_slice(unspecified(1), c(1, 1)), unspecified(2))
})


test_that("vec_slice() works with Altrep classes with custom extract methods", {
  x <- chr_rle(foo = 10L, bar = 5L)
  expect_false(chr_rle_is_materialized(x))

  idx <- c(9, 10, 11)
  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar"))
  expect_false(chr_rle_is_materialized(x))

  # With zero
  idx <- c(0, 1)
  expect_equal(vec_slice(x, idx), "foo")
  expect_false(chr_rle_is_materialized(x))

  # With integer missing values
  idx <- c(0, NA, 2, NA)
  expect_equal(vec_slice(x, idx), c(NA, "foo", NA))
  expect_false(chr_rle_is_materialized(x))

  # With logical condition index
  idx <- c(TRUE, rep(FALSE, 8), TRUE, rep(FALSE, 2), TRUE, TRUE, TRUE)
  expect_equal(vec_slice(x, idx), c("foo", "foo", "bar", "bar", "bar"))
  expect_false(chr_rle_is_materialized(x))

  # Everything
  idx <- TRUE
  expect_equal(vec_slice(x, idx), c(rep("foo", 10), rep("bar", 5)))
  expect_false(chr_rle_is_materialized(x))

  # Nothing
  idx <- FALSE
  expect_equal(vec_slice(x, idx), character())
  expect_false(chr_rle_is_materialized(x))

  # Whole vector of missing values
  idx <- NA
  expect_equal(vec_slice(x, idx), rep(NA_character_, 15))
  expect_false(chr_rle_is_materialized(x))

  # Just 1 missing value
  idx <- NA_integer_
  expect_equal(vec_slice(x, idx), rep(NA_character_, 1))
  expect_false(chr_rle_is_materialized(x))

  # OOB
  idx <- 16
  expect_snapshot(error = TRUE, {
    vec_slice(x, idx)
  })
})

test_that("Unnamed vector with character subscript is caught", {
  expect_snapshot(error = TRUE, vec_slice(1:3, letters[1]))
})

test_that("Negative subscripts are checked", {
  expect_snapshot(error = TRUE, vec_slice(1:3, -c(1L, NA)))
  expect_snapshot(error = TRUE, vec_slice(1:3, c(-1L, 1L)))
})

test_that("oob error messages are properly constructed", {
  expect_snapshot(error = TRUE, vec_slice(c(bar = 1), "foo"))

  # Multiple OOB indices
  expect_snapshot(error = TRUE, vec_slice(letters, c(100, 1000)))
  expect_snapshot(error = TRUE, vec_slice(letters, c(1, 100:103, 2, 104:110)))
  expect_snapshot(error = TRUE, vec_slice(set_names(letters), c("foo", "bar")))
  expect_snapshot(error = TRUE, vec_slice(set_names(letters), toupper(letters)))
})

# vec_init ----------------------------------------------------------------

test_that("na of atomic vectors is as expected", {
  expect_equal(vec_init(TRUE), NA)
  expect_equal(vec_init(1L), NA_integer_)
  expect_equal(vec_init(1), NA_real_)
  expect_equal(vec_init("x"), NA_character_)
  expect_equal(vec_init(1i), NA_complex_)
})

test_that("na of factor preserves levels", {
  f1 <- factor("a", levels = c("a", "b"))
  f2 <- vec_init(f1)

  expect_equal(levels(f1), levels(f2))
})

test_that("na of POSIXct preserves tz", {
  dt1 <- as.POSIXct("2010-01-01", tz = "America/New_York")
  dt2 <- vec_init(dt1)
  expect_equal(attr(dt2, "tzone"), "America/New_York")
})

test_that("na of list is list(NULL)", {
  expect_equal(vec_init(list()), list(NULL))
})

test_that("na of array is 1d slice", {
  x1 <- array(1:12, c(2, 3, 4))
  x2 <- vec_init(x1)

  expect_equal(x2, array(NA_integer_, c(1, 3, 4)))
})

test_that("na of list-array is 1d slice", {
  x1 <- array(as.list(1:12), c(2, 3, 4))
  x2 <- vec_init(x1)

  expect_equal(x2, array(list(), c(1, 3, 4)))
})

test_that("vec_init() asserts vectorness (#301)", {
  expect_error(vec_init(NULL, 1L), class = "vctrs_error_scalar_type")
})

test_that("vec_init() works with Altrep classes", {
  x <- chr_rle(foo = 1L, bar = 2L)
  expect_false(chr_rle_is_materialized(x))
  expect_equal(vec_init(x, 2), rep(NA_character_, 2))
  expect_false(chr_rle_is_materialized(x))
})

test_that("vec_init() validates `n`", {
  expect_snapshot({
    (expect_error(vec_init(1L, 1.5)))
    (expect_error(vec_init(1L, c(1, 2))))
    (expect_error(vec_init(1L, -1L)))
    (expect_error(vec_init(1L, NA)))
    (expect_error(vec_init(1L, NA_integer_)))
  })
})


# vec_slice + compact_rep -------------------------------------------------

# `i` is 1-based

test_that("names are repaired correctly with compact reps and `NA_integer_`", {
  x <- list(a = 1L, b = 2L)
  expect <- set_names(list(NULL, NULL), c("", ""))

  expect_equal(vec_slice_rep(x, NA_integer_, 2L), expect)
})

test_that("names are recycled correctly with compact reps", {
  expect_named(vec_slice_rep(c(x = 1L), 1L, 3L), c("x", "x", "x"))
})

test_that("vec_slice() with compact_reps work with Altrep classes", {
  x <- chr_rle(foo = 10L, bar = 5L)
  expect_false(chr_rle_is_materialized(x))
  expect_equal(vec_slice_rep(x, 10L, 3L), rep("foo", 3))
  expect_false(chr_rle_is_materialized(x))
})

# vec_slice + compact_seq -------------------------------------------------

# `start` is 0-based

test_that("can subset base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  increasing <- TRUE
  expect_identical(
    vec_slice_seq(lgl(1, 0, 1), start, size, increasing),
    lgl(0, 1)
  )
  expect_identical(
    vec_slice_seq(int(1, 2, 3), start, size, increasing),
    int(2, 3)
  )
  expect_identical(
    vec_slice_seq(dbl(1, 2, 3), start, size, increasing),
    dbl(2, 3)
  )
  expect_identical(
    vec_slice_seq(cpl(1, 2, 3), start, size, increasing),
    cpl(2, 3)
  )
  expect_identical(
    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
    chr("2", "3")
  )
  expect_identical(
    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
    raw2(2, 3)
  )
  expect_identical(
    vec_slice_seq(list(1, 2, 3), start, size, increasing),
    list(2, 3)
  )
})

test_that("can subset base vectors with decreasing compact seqs", {
  start <- 2L
  size <- 2L
  increasing <- FALSE
  expect_identical(
    vec_slice_seq(lgl(1, 0, 1), start, size, increasing),
    lgl(1, 0)
  )
  expect_identical(
    vec_slice_seq(int(1, 2, 3), start, size, increasing),
    int(3, 2)
  )
  expect_identical(
    vec_slice_seq(dbl(1, 2, 3), start, size, increasing),
    dbl(3, 2)
  )
  expect_identical(
    vec_slice_seq(cpl(1, 2, 3), start, size, increasing),
    cpl(3, 2)
  )
  expect_identical(
    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
    chr("3", "2")
  )
  expect_identical(
    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
    raw2(3, 2)
  )
  expect_identical(
    vec_slice_seq(list(1, 2, 3), start, size, increasing),
    list(3, 2)
  )
})

test_that("can subset base vectors with size 0 compact seqs", {
  start <- 1L
  size <- 0L
  increasing <- TRUE
  expect_identical(vec_slice_seq(lgl(1, 0, 1), start, size, increasing), lgl())
  expect_identical(vec_slice_seq(int(1, 2, 3), start, size, increasing), int())
  expect_identical(vec_slice_seq(dbl(1, 2, 3), start, size, increasing), dbl())
  expect_identical(vec_slice_seq(cpl(1, 2, 3), start, size, increasing), cpl())
  expect_identical(
    vec_slice_seq(chr("1", "2", "3"), start, size, increasing),
    chr()
  )
  expect_identical(
    vec_slice_seq(raw2(1, 2, 3), start, size, increasing),
    raw2()
  )
  expect_identical(
    vec_slice_seq(list(1, 2, 3), start, size, increasing),
    list()
  )
})

test_that("can subset shaped base vectors with compact seqs", {
  start <- 1L
  size <- 2L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(
    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
    mat(lgl(0, 1))
  )
  expect_identical(
    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
    mat(int(2, 3))
  )
  expect_identical(
    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
    mat(dbl(2, 3))
  )
  expect_identical(
    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
    mat(cpl(2, 3))
  )
  expect_identical(
    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
    mat(chr("2", "3"))
  )
  expect_identical(
    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
    mat(raw2(2, 3))
  )
  expect_identical(
    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
    mat(list(2, 3))
  )
})

test_that("can subset shaped base vectors with decreasing compact seqs", {
  start <- 2L
  size <- 2L
  increasing <- FALSE
  mat <- as.matrix
  expect_identical(
    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
    mat(lgl(1, 0))
  )
  expect_identical(
    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
    mat(int(3, 2))
  )
  expect_identical(
    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
    mat(dbl(3, 2))
  )
  expect_identical(
    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
    mat(cpl(3, 2))
  )
  expect_identical(
    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
    mat(chr("3", "2"))
  )
  expect_identical(
    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
    mat(raw2(3, 2))
  )
  expect_identical(
    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
    mat(list(3, 2))
  )
})

test_that("can subset shaped base vectors with size 0 compact seqs", {
  start <- 1L
  size <- 0L
  increasing <- TRUE
  mat <- as.matrix
  expect_identical(
    vec_slice_seq(mat(lgl(1, 0, 1)), start, size, increasing),
    mat(lgl())
  )
  expect_identical(
    vec_slice_seq(mat(int(1, 2, 3)), start, size, increasing),
    mat(int())
  )
  expect_identical(
    vec_slice_seq(mat(dbl(1, 2, 3)), start, size, increasing),
    mat(dbl())
  )
  expect_identical(
    vec_slice_seq(mat(cpl(1, 2, 3)), start, size, increasing),
    mat(cpl())
  )
  expect_identical(
    vec_slice_seq(mat(chr("1", "2", "3")), start, size, increasing),
    mat(chr())
  )
  expect_identical(
    vec_slice_seq(mat(raw2(1, 2, 3)), start, size, increasing),
    mat(raw2())
  )
  expect_identical(
    vec_slice_seq(mat(list(1, 2, 3)), start, size, increasing),
    mat(list())
  )
})

test_that("can subset object of any dimensionality with compact seqs", {
  x0 <- c(1, 1)
  x1 <- ones(2)
  x2 <- ones(2, 3)
  x3 <- ones(2, 3, 4)
  x4 <- ones(2, 3, 4, 5)

  expect_equal(vec_slice_seq(x0, 0L, 1L), 1)
  expect_identical(vec_slice_seq(x1, 0L, 1L), ones(1))
  expect_identical(vec_slice_seq(x2, 0L, 1L), ones(1, 3))
  expect_identical(vec_slice_seq(x3, 0L, 1L), ones(1, 3, 4))
  expect_identical(vec_slice_seq(x4, 0L, 1L), ones(1, 3, 4, 5))
})

test_that("can subset data frames with compact seqs", {
  df <- data_frame(x = 1:5, y = letters[1:5])
  expect_equal(vec_slice_seq(df, 0L, 0L), vec_slice(df, integer()))
  expect_equal(vec_slice_seq(df, 0L, 1L), vec_slice(df, 1L))
  expect_equal(vec_slice_seq(df, 0L, 3L), vec_slice(df, 1:3))
  expect_equal(vec_slice_seq(df, 2L, 3L, FALSE), vec_slice(df, 3:1))

  df$df <- df
  expect_equal(vec_slice_seq(df, 0L, 0L), vec_slice(df, integer()))
  expect_equal(vec_slice_seq(df, 0L, 1L), vec_slice(df, 1L))
  expect_equal(vec_slice_seq(df, 0L, 3L), vec_slice(df, 1:3))
  expect_equal(vec_slice_seq(df, 2L, 3L, FALSE), vec_slice(df, 3:1))
})

test_that("can subset S3 objects using the fallback method with compact seqs", {
  x <- factor(c("a", "b", "c", "d"))
  expect_equal(vec_slice_seq(x, 0L, 0L), vec_slice(x, integer()))
  expect_equal(vec_slice_seq(x, 0L, 1L), vec_slice(x, 1L))
  expect_equal(vec_slice_seq(x, 2L, 2L), vec_slice(x, 3:4))
  expect_equal(vec_slice_seq(x, 3L, 2L, FALSE), vec_slice(x, 4:3))
})

test_that("vec_slice() with compact_seqs work with Altrep classes", {
  x <- chr_rle(foo = 2L, bar = 3L)
  expect_false(chr_rle_is_materialized(x))
  expect_equal(vec_slice_seq(x, 1L, 3L), c("foo", "bar", "bar"))
  expect_false(chr_rle_is_materialized(x))
})

test_that("vec_slice() handles symbols and OO objects", {
  expect_identical(vec_slice(c(a = 1, b = 2), quote(b)), c(b = 2))
  expect_identical(vec_slice(c(a = 1, b = 2), factor("b")), c(b = 2))
  expect_error(
    vec_slice(c(a = 1, b = 2), foobar("b")),
    class = "vctrs_error_subscript_type"
  )
})

test_that("vec_init() handles names in columns", {
  expect_identical(
    vec_init(data_frame(x = c(a = 1, b = 2)))$x,
    named(na_dbl)
  )
  expect_identical(
    vec_init(data_frame(x = c(1, 2)))$x,
    na_dbl
  )
})

test_that("vec_slice() restores unrestored but named foreign classes", {
  x <- foobar(c(x = 1))

  expect_identical(vec_slice(x, 1), x)
  expect_identical(vec_chop(x), list(x))
  expect_identical(vec_chop(x, indices = list(1)), list(x))
  expect_identical(vec_ptype(x), foobar(named(dbl())))
  expect_identical(vec_ptype(x), foobar(named(dbl())))
  expect_identical(vec_ptype_common(x, x), foobar(named(dbl())))

  out <- vec_ptype_common_fallback(x, x)
  expect_true(is_common_class_fallback(out))
  expect_identical(fallback_class(out), "vctrs_foobar")
})

test_that("scalar type error is thrown when `vec_slice_unsafe()` is called directly (#1139)", {
  x <- foobar(as.list(1:3))
  expect_error(vec_slice_seq(x, 1L, 1L), class = "vctrs_error_scalar_type")
})

test_that("column sizes are checked before slicing (#552)", {
  x <- structure(list(a = 1, b = 2:3), row.names = 1:2, class = "data.frame")
  expect_error(
    vctrs::vec_slice(x, 2),
    "Column `a` (size 1) must match the data frame (size 2)",
    fixed = TRUE
  )
})

test_that("base_vec_rep() slices data frames with the base::rep() UI", {
  df <- data_frame(x = data_frame(y = 1:2))
  expect_identical(
    base_vec_rep(df, length.out = 4),
    vec_slice(df, c(1:2, 1:2))
  )
})

test_that("vec_size_assign() slices data frames with the base::rep() UI", {
  df <- data_frame(x = data_frame(y = 1:3))

  expect_identical(
    vec_size_assign(df, 2),
    vec_slice(df, 1:2)
  )

  expect_identical(
    vec_size_assign(df, 4),
    vec_slice(df, c(1:3, NA))
  )
})

#' FAQ - Is my class compatible with vctrs?
#'
#' @includeRmd man/faq/developer/reference-compatibility.Rmd description
#'
#' @name reference-faq-compatibility
NULL

#' FAQ - How does coercion work in vctrs?
#'
#' @includeRmd man/faq/developer/theory-coercion.Rmd description
#'
#' @name theory-faq-coercion
NULL

# Also see the `redirects:` section in `_pkgdown.yml`
# for `vector_recycling_rules.html`

#' FAQ - How does recycling work in vctrs and the tidyverse?
#'
#' @includeRmd man/faq/developer/theory-recycling.Rmd description
#'
#' @name theory-faq-recycling
#' @aliases vector_recycling_rules
NULL

#' FAQ - How to implement ptype2 and cast methods?
#'
#' @includeRmd man/faq/developer/howto-coercion.Rmd description
#'
#' @name howto-faq-coercion
NULL

#' FAQ - How to implement ptype2 and cast methods? (Data frames)
#'
#' @includeRmd man/faq/developer/howto-coercion-data-frame.Rmd description
#'
#' @name howto-faq-coercion-data-frame
NULL

#' FAQ - Why isn't my class treated as a vector?
#'
#' @includeRmd man/faq/developer/howto-faq-fix-scalar-type-error.Rmd description
#'
#' @name howto-faq-fix-scalar-type-error
#' @aliases howto_faq_fix_scalar_type_error
NULL

delayedAssign("as.data.table", {
  if (is_installed("data.table")) {
    env_get(ns_env("data.table"), "as.data.table")
  } else {
    function(...) abort("`data.table` must be installed.")
  }
})

dt_ptype2 <- function(x, y, ...) {
  as.data.table(df_ptype2(x, y, ...))
}
dt_cast <- function(x, to, ...) {
  as.data.table(df_cast(x, to, ...))
}

#' @export
vec_ptype2.data.table.data.table <- function(x, y, ...) {
  dt_ptype2(x, y, ...)
}
#' @export
vec_ptype2.data.table.data.frame <- function(x, y, ...) {
  dt_ptype2(x, y, ...)
}
#' @export
vec_ptype2.data.frame.data.table <- function(x, y, ...) {
  dt_ptype2(x, y, ...)
}

#' @export
vec_cast.data.table.data.table <- function(x, to, ...) {
  dt_cast(x, to, ...)
}
#' @export
vec_cast.data.table.data.frame <- function(x, to, ...) {
  dt_cast(x, to, ...)
}
#' @export
vec_cast.data.frame.data.table <- function(x, to, ...) {
  df_cast(x, to, ...)
}

#' @export
vec_ptype_abbr.data.table <- function(x, ...) {
  "dt"
}

# Proxy method for <IDate>
#
# `vec_proxy.Date()` coerces integer storage dates to double. If we don't
# intercept that, an IDate will also be converted to double storage, but that
# results in a corrupt IDate. Since we provide some methods for <data.table>, we
# also provide this one.
#
# Notably we don't provide `vec_ptype2.Date.IDate` methods. We don't think it is
# our place to provide methods for that, especially since we'd likely prefer
# <Date> as the more general common type, but `c.IDate` in data.table prefers
# <IDate>, so we'd probably just add conflicting behavior.
vec_proxy_IDate <- function(x, ...) {
  if (typeof(x) != "integer") {
    type <- typeof(x)
    cli::cli_abort(
      "Corrupt <IDate>. Expected integer storage, not {type} storage."
    )
  }

  x
}

# Restore method for <IDate>
#
# Follow `vec_restore.Date` and pass through to `vec_restore.default` which
# uses standard restore behavior
vec_restore_IDate <- function(x, to, ...) {
  NextMethod()
}

#' Vectorized if-else
#'
#' @description
#' `vec_if_else()` is a vectorized [if-else][if]. Compared to the base R
#' equivalent, [ifelse()], this function allows you to handle missing values in
#' the `condition` with `missing` and always takes `true`, `false`, and
#' `missing` into account when determining what the output type should be.
#'
#' @inheritParams rlang::args_dots_empty
#' @inheritParams rlang::args_error_context
#'
#' @param condition A logical vector.
#'
#' @param true,false Vectors to use for `TRUE` and `FALSE` values of
#'   `condition`.
#'
#'   Both `true` and `false` will be [recycled][theory-faq-recycling]
#'   to the size of `condition`.
#'
#'   `true`, `false`, and `missing` (if used) will be cast to their common type.
#'
#' @param missing If not `NULL`, will be used as the value for `NA` values of
#'   `condition`. Follows the same size and type rules as `true` and `false`.
#'
#' @param ptype An optional prototype declaring the desired output type. If
#'   supplied, this overrides the common type of `true`, `false`, and `missing`.
#'
#' @param condition_arg,true_arg,false_arg,missing_arg Argument names used in
#'   error messages.
#'
#' @returns
#' A vector with the same size as `condition` and the same type as the common
#' type of `true`, `false`, and `missing`.
#'
#' Where `condition` is `TRUE`, the matching values from `true`, where it is
#' `FALSE`, the matching values from `false`, and where it is `NA`, the matching
#' values from `missing`, if provided, otherwise a missing value will be used.
#'
#' @export
#' @examples
#' x <- c(-5:5, NA)
#' vec_if_else(x < 0, NA, x)
#'
#' # Explicitly handle `NA` values in the `condition` with `missing`
#' vec_if_else(x < 0, "negative", "positive", missing = "missing")
#'
#' # Unlike `ifelse()`, `vec_if_else()` preserves types
#' x <- factor(sample(letters[1:5], 10, replace = TRUE))
#' ifelse(x %in% c("a", "b", "c"), x, NA)
#' vec_if_else(x %in% c("a", "b", "c"), x, NA)
#'
#' # `vec_if_else()` also works with data frames
#' condition <- c(TRUE, FALSE, NA, TRUE)
#' true <- data_frame(x = 1:4, y = 5:8)
#' false <- data_frame(x = 9:12, y = 13:16)
#' vec_if_else(condition, true, false)
vec_if_else <- function(
  condition,
  true,
  false,
  ...,
  missing = NULL,
  ptype = NULL,
  condition_arg = "condition",
  true_arg = "true",
  false_arg = "false",
  missing_arg = "missing",
  error_call = current_env()
) {
  check_dots_empty0(...)
  .Call(
    ffi_vec_if_else,
    condition,
    true,
    false,
    missing,
    ptype,
    environment()
  )
}

# Type2 -------------------------------------------------------------------

# Left generics -----------------------------------------------------------

#' @rdname vec_ptype2
#' @export vec_ptype2.logical
#' @method vec_ptype2 logical
#' @export
vec_ptype2.logical <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.logical")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.integer
#' @method vec_ptype2 integer
#' @export
vec_ptype2.integer <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.integer")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.double
#' @method vec_ptype2 double
#' @export
vec_ptype2.double <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.double")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.complex
#' @method vec_ptype2 complex
#' @export
vec_ptype2.complex <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.complex")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.character
#' @method vec_ptype2 character
#' @export
vec_ptype2.character <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.character")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.raw
#' @method vec_ptype2 raw
#' @export
vec_ptype2.raw <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.raw")
}
#' @rdname vec_ptype2
#' @export vec_ptype2.list
#' @method vec_ptype2 list
#' @export
vec_ptype2.list <- function(x, y, ..., x_arg = "", y_arg = "") {
  UseMethod("vec_ptype2.list")
}


# Numeric-ish

#' @method vec_ptype2.logical logical
#' @export
vec_ptype2.logical.logical <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.logical.logical")
}

#' @export
#' @method vec_ptype2.integer integer
vec_ptype2.integer.integer <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.integer.integer")
}
#' @export
#' @method vec_ptype2.logical integer
vec_ptype2.logical.integer <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.logical.integer")
}
#' @export
#' @method vec_ptype2.integer logical
vec_ptype2.integer.logical <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.integer.logical")
}

#' @export
#' @method vec_ptype2.double double
vec_ptype2.double.double <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.double.double")
}
#' @export
#' @method vec_ptype2.logical double
vec_ptype2.logical.double <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.logical.double")
}
#' @export
#' @method vec_ptype2.double logical
vec_ptype2.double.logical <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.double.logical")
}
#' @export
#' @method vec_ptype2.integer double
vec_ptype2.integer.double <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.integer.double")
}
#' @export
#' @method vec_ptype2.double integer
vec_ptype2.double.integer <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.double.integer")
}

#' @export
#' @method vec_ptype2.complex complex
vec_ptype2.complex.complex <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.complex.complex")
}
#' @export
#' @method vec_ptype2.integer complex
vec_ptype2.integer.complex <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.integer.complex")
}
#' @export
#' @method vec_ptype2.complex integer
vec_ptype2.complex.integer <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.complex.integer")
}
#' @export
#' @method vec_ptype2.double complex
vec_ptype2.double.complex <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.double.complex")
}
#' @export
#' @method vec_ptype2.complex double
vec_ptype2.complex.double <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.complex.double")
}


# Character

#' @method vec_ptype2.character character
#' @export
vec_ptype2.character.character <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.character.character")
}


# Raw

#' @export
#' @method vec_ptype2.raw raw
vec_ptype2.raw.raw <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.raw.raw")
}


# Lists

#' @method vec_ptype2.list list
#' @export
vec_ptype2.list.list <- function(x, y, ..., x_arg = "", y_arg = "") {
  stop_native_implementation("vec_ptype2.list.list")
}


# Cast --------------------------------------------------------------------

# These methods for base types are handled at the C level unless
# inputs have shape or have lossy casts

#' @export
#' @rdname vec_cast
#' @export vec_cast.logical
#' @method vec_cast logical
vec_cast.logical <- function(x, to, ...) {
  UseMethod("vec_cast.logical")
}
#' @export
#' @method vec_cast.logical logical
vec_cast.logical.logical <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}
#' @export
#' @method vec_cast.logical integer
vec_cast.logical.integer <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  out <- vec_coerce_bare(x, "logical")
  out <- shape_broadcast(
    out,
    to,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
  lossy <- !x %in% c(0L, 1L, NA_integer_)
  maybe_lossy_cast(
    out,
    x,
    to,
    lossy,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
}
#' @export
#' @method vec_cast.logical double
vec_cast.logical.double <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  out <- vec_coerce_bare(x, "logical")
  out <- shape_broadcast(
    out,
    to,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
  lossy <- !x %in% c(0, 1, NA_real_)
  maybe_lossy_cast(
    out,
    x,
    to,
    lossy,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
}

#' @export
#' @rdname vec_cast
#' @export vec_cast.integer
#' @method vec_cast integer
vec_cast.integer <- function(x, to, ...) {
  UseMethod("vec_cast.integer")
}
#' @export
#' @method vec_cast.integer logical
vec_cast.integer.logical <- function(x, to, ...) {
  x <- vec_coerce_bare(x, "integer")
  shape_broadcast(x, to, ...)
}
#' @export
#' @method vec_cast.integer integer
vec_cast.integer.integer <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}
#' @export
#' @method vec_cast.integer double
vec_cast.integer.double <- function(
  x,
  to,
  ...,
  x_arg = "",
  to_arg = "",
  call = caller_env()
) {
  out <- suppressWarnings(vec_coerce_bare(x, "integer"))
  x_na <- is.na(x)
  lossy <- (out != x & !x_na) | xor(x_na, is.na(out))

  out <- shape_broadcast(
    out,
    to,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
  maybe_lossy_cast(
    out,
    x,
    to,
    lossy,
    x_arg = x_arg,
    to_arg = to_arg,
    call = call
  )
}

#' @export
#' @rdname vec_cast
#' @export vec_cast.double
#' @method vec_cast double
vec_cast.double <- function(x, to, ...) {
  UseMethod("vec_cast.double")
}
#' @export
#' @method vec_cast.double logical
vec_cast.double.logical <- function(x, to, ...) {
  x <- vec_coerce_bare(x, "double")
  shape_broadcast(x, to, ...)
}
#' @export
#' @method vec_cast.double integer
vec_cast.double.integer <- vec_cast.double.logical
#' @export
#' @method vec_cast.double double
vec_cast.double.double <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}

#' @export
#' @rdname vec_cast
#' @export vec_cast.complex
#' @method vec_cast complex
vec_cast.complex <- function(x, to, ...) {
  UseMethod("vec_cast.complex")
}
#' @export
#' @method vec_cast.complex logical
vec_cast.complex.logical <- function(x, to, ...) {
  x <- vec_coerce_bare(x, "complex")
  shape_broadcast(x, to, ...)
}
#' @export
#' @method vec_cast.complex integer
vec_cast.complex.integer <- vec_cast.complex.logical
#' @export
#' @method vec_cast.complex double
vec_cast.complex.double <- vec_cast.complex.logical
#' @export
#' @method vec_cast.complex complex
vec_cast.complex.complex <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}

#' @export
#' @rdname vec_cast
#' @export vec_cast.raw
#' @method vec_cast raw
vec_cast.raw <- function(x, to, ...) {
  UseMethod("vec_cast.raw")
}
#' @export
#' @method vec_cast.raw raw
vec_cast.raw.raw <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}

#' @export
#' @rdname vec_cast
#' @export vec_cast.character
#' @method vec_cast character
vec_cast.character <- function(x, to, ...) {
  UseMethod("vec_cast.character")
}
#' @export
#' @method vec_cast.character character
vec_cast.character.character <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}

#' @rdname vec_cast
#' @export vec_cast.list
#' @method vec_cast list
#' @export
vec_cast.list <- function(x, to, ...) {
  UseMethod("vec_cast.list")
}
#' @export
#' @method vec_cast.list list
vec_cast.list.list <- function(x, to, ...) {
  shape_broadcast(x, to, ...)
}

# equal --------------------------------------------------------------

#' @export
vec_proxy_equal.array <- function(x, ...) {
  # The conversion to data frame is only a stopgap, in the long
  # term, we'll hash arrays natively. Note that hashing functions
  # similarly convert to data frames.
  x <- as.data.frame(x)
  vec_proxy_equal(x)
}

# compare ------------------------------------------------------------

#' @export
vec_proxy_compare.raw <- function(x, ...) {
  # because:
  # order(as.raw(1:3))
  # #> Error in order(as.raw(1:3)): unimplemented type 'raw' in 'orderVector1'
  as.integer(x)
}

#' @export
vec_proxy_compare.list <- function(x, ...) {
  stop_unsupported(x, "vec_proxy_compare")
}

#' @export
vec_proxy_compare.array <- function(x, ...) {
  # The conversion to data frame is only a stopgap, in the long
  # term, we'll hash arrays natively. Note that hashing functions
  # similarly convert to data frames.
  x <- as.data.frame(x)
  vec_proxy_compare(x)
}

# order ------------------------------------------------------------

#' @export
vec_proxy_order.raw <- function(x, ...) {
  # Can't rely on fallthrough behavior to `vec_proxy_compare()` because this
  # isn't an S3 object. Have to call it manually.
  vec_proxy_compare(x)
}

#' @export
vec_proxy_order.list <- function(x, ...) {
  # Order lists by first appearance.
  # This allows list elements to be grouped in `vec_order()`.
  # Have to separately ensure missing values are propagated.
  out <- vec_duplicate_id(x)

  if (vec_any_missing(x)) {
    missing <- vec_detect_missing(x)
    out <- vec_assign(out, missing, NA_integer_)
  }

  out
}

#' @export
vec_proxy_order.array <- function(x, ...) {
  # The conversion to data frame is only a stopgap, in the long
  # term, we'll hash arrays natively. Note that hashing functions
  # similarly convert to data frames.
  x <- as.data.frame(x)
  vec_proxy_order(x)
}

#' Drop empty elements from a list
#'
#' `list_drop_empty()` removes empty elements from a list. This includes `NULL`
#' elements along with empty vectors, like `integer(0)`. This is equivalent to,
#' but faster than, `vec_slice(x, list_sizes(x) != 0L)`.
#'
#' @section Dependencies:
#' - [vec_slice()]
#'
#' @param x A list.
#'
#' @export
#' @examples
#' x <- list(1, NULL, integer(), 2)
#' list_drop_empty(x)
list_drop_empty <- function(x) {
  .Call(vctrs_list_drop_empty, x)
}

#' Lazy character vector
#'
#' `new_lazy_character()` takes a function with no arguments which must return
#' a character vector of arbitrary length. The function will be evaluated
#' exactly once whenever any properties of the character vector are required
#' (including the length or any vector elements).
#'
#' A "real" production level implementation might work more like
#' `carrier::crate()`, where the function is isolated and users must explicitly
#' provide any data required to evaluate the function, since the time of
#' evaluation is unknown.
#'
#' As of June 2023, running `x <- new_lazy_character(~ c("x", "y"))` in the
#' RStudio console will call the ALTREP length method, which materializes the
#' object. Doing this in a terminal session running R does not, so it is an
#' RStudio issue. This doesn't affect tests run within a `test_that()` block.
#'
#' @param fn A function with no arguments returning a character vector.
#'
#' @noRd
new_lazy_character <- function(fn) {
  fn <- as_function(fn)
  .Call(ffi_altrep_new_lazy_character, fn)
}

lazy_character_is_materialized <- function(x) {
  .Call(ffi_altrep_lazy_character_is_materialized, x)
}

---
title: "Printing vectors nicely in tibbles"
author: "Kirill Müller, Hadley Wickham"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Printing vectors nicely in tibbles}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(collapse = TRUE, comment = "#>")
```

You can get basic control over how a vector is printed in a tibble by providing a `format()` method.
If you want greater control, you need to understand how printing works.
The presentation of a column in a tibble is controlled by two S3 generics:

* `vctrs::vec_ptype_abbr()` determines what goes into the column header.
* `pillar::pillar_shaft()` determines what goes into the body, or the shaft, of the column.

Technically a [*pillar*](https://en.wikipedia.org/wiki/Column#Nomenclature) is composed of a *shaft* (decorated with an *ornament*), with a *capital* above and a *base* below.
Multiple pillars form a *colonnade*, which can be stacked in multiple *tiers*.
This is the motivation behind the names in our API.

This short vignette shows the basics of column styling using a `"latlon"` vector.
The vignette imagines the code is in a package, so that you can see the roxygen2 commands you'll need to create documentation and the `NAMESPACE` file.
In this vignette, we'll attach pillar and vctrs:

```{r setup}
library(vctrs)
library(pillar)
```

You don't need to do this in a package.
Instead, you'll need to _import_ the packages by then to the `Imports:` section of your `DESCRIPTION`.
The following helper does this for you:

```{r, eval = FALSE}
usethis::use_package("vctrs")
usethis::use_package("pillar")
```

## Prerequisites

To illustrate the basic ideas we're going to create a `"latlon"` class that encodes geographic coordinates in a record.
We'll pretend that this code lives in a package called earth.
For simplicity, the values are printed as degrees and minutes only.
By using `vctrs_rcrd()`, we already get the infrastructure to make this class fully compatible with data frames for free.
See `vignette("s3-vector", package = "vctrs")` for details on the record data type.

```{r}
#' @export
latlon <- function(lat, lon) {
  new_rcrd(list(lat = lat, lon = lon), class = "earth_latlon")
}

#' @export
format.earth_latlon <- function(x, ..., formatter = deg_min) {
  x_valid <- which(!is.na(x))

  lat <- field(x, "lat")[x_valid]
  lon <- field(x, "lon")[x_valid]

  ret <- rep(NA_character_, vec_size(x))
  ret[x_valid] <- paste0(formatter(lat, "lat"), " ", formatter(lon, "lon"))
  # It's important to keep NA in the vector!
  ret
}

deg_min <- function(x, direction) {
  pm <- if (direction == "lat") c("N", "S") else c("E", "W")

  sign <- sign(x)
  x <- abs(x)
  deg <- trunc(x)
  x <- x - deg
  min <- round(x * 60)

  # Ensure the columns are always the same width so they line up nicely
  ret <- sprintf("%d°%.2d'%s", deg, min, ifelse(sign >= 0, pm[[1]], pm[[2]]))
  format(ret, justify = "right")
}

latlon(c(32.71, 2.95), c(-117.17, 1.67))
```

## Using in a tibble

Columns of this class can be used in a tibble right away because we've made a class using the vctrs infrastructure and have provided a `format()` method:

```{r}
library(tibble)

loc <- latlon(
  c(28.3411783, 32.7102978, 30.2622356, 37.7859102, 28.5, NA),
  c(-81.5480348, -117.1704058, -97.7403327, -122.4131357, -81.4, NA)
)

data <- tibble(venue = "rstudio::conf", year = 2017:2022, loc = loc)

data
```

This output is ok, but we could improve it by:

1. Using a more description type abbreviation than `<erth_ltl>`.

1. Using a dash of colour to highlight the most important parts of the value.

1. Providing a narrower view when horizontal space is at a premium.

The following sections show how to enhance the rendering.

## Fixing the data type

Instead of `<erth_ltl>` we'd prefer to use `<latlon>`.
We can do that by implementing the `vec_ptype_abbr()` method, which should return a string that can be used in a column header.
For your own classes, strive for an evocative abbreviation that's under 6 characters.

```{r}
#' @export
vec_ptype_abbr.earth_latlon <- function(x) {
  "latlon"
}

data
```


## Custom rendering

The `format()` method is used by default for rendering.
For custom formatting you need to implement the `pillar_shaft()` method.
This function should always return a pillar shaft object, created by `new_pillar_shaft_simple()` or similar.
`new_pillar_shaft_simple()` accepts ANSI escape codes for colouring, and pillar includes some built in styles like `style_subtle()`.
We can use subtle style for the degree and minute separators to make the data more obvious.

First we define a degree formatter that makes use of `style_subtle()`:

```{r}
deg_min_color <- function(x, direction) {
  pm <- if (direction == "lat") c("N", "S") else c("E", "W")

  sign <- sign(x)
  x <- abs(x)
  deg <- trunc(x)
  x <- x - deg
  rad <- round(x * 60)
  ret <- sprintf(
    "%d%s%.2d%s%s",
    deg,
    pillar::style_subtle("°"),
    rad,
    pillar::style_subtle("'"),
    pm[ifelse(sign >= 0, 1, 2)]
  )
  format(ret, justify = "right")
}
```

And then we pass that to our `format()` method:

```{r}
#' @importFrom pillar pillar_shaft
#' @export
pillar_shaft.earth_latlon <- function(x, ...) {
  out <- format(x, formatter = deg_min_color)
  pillar::new_pillar_shaft_simple(out, align = "right")
}
```

Currently, ANSI escapes are not rendered in vignettes, so this result doesn't look any different, but if you run the code yourself you'll see an improved display.

```{r}
data
```

As well as the functions in pillar, the [cli](https://cli.r-lib.org/) package provides a variety of tools for styling text.

## Truncation

Tibbles can automatically compacts columns when there's no enough horizontal space to display everything:

```{r}
print(data, width = 30)
```

Currently the latlon class isn't ever compacted because we haven't specified a minimum width when constructing the shaft.
Let's fix that and re-print the data:

```{r}
#' @importFrom pillar pillar_shaft
#' @export
pillar_shaft.earth_latlon <- function(x, ...) {
  out <- format(x)
  pillar::new_pillar_shaft_simple(out, align = "right", min_width = 10)
}

print(data, width = 30)
```
## Adaptive rendering

Truncation may be useful for character data, but for lat-lon data it'd be nicer to show full degrees and remove the minutes.
We'll first write a function that does this:

```{r}
deg <- function(x, direction) {
  pm <- if (direction == "lat") c("N", "S") else c("E", "W")

  sign <- sign(x)
  x <- abs(x)
  deg <- round(x)

  ret <- sprintf("%d°%s", deg, pm[ifelse(sign >= 0, 1, 2)])
  format(ret, justify = "right")
}
```

Then use it as part of more sophisticated implementation of the `pillar_shaft()` method:

```{r}
#' @importFrom pillar pillar_shaft
#' @export
pillar_shaft.earth_latlon <- function(x, ...) {
  deg <- format(x, formatter = deg)
  deg_min <- format(x)

  pillar::new_pillar_shaft(
    list(deg = deg, deg_min = deg_min),
    width = pillar::get_max_extent(deg_min),
    min_width = pillar::get_max_extent(deg),
    class = "pillar_shaft_latlon"
  )
}
```

Now the `pillar_shaft()` method returns an object of class `"pillar_shaft_latlon"` created by `new_pillar_shaft()`.
This object contains the necessary information to render the values, and also minimum and maximum width values.
For simplicity, both formats are pre-rendered, and the minimum and maximum widths are computed from there.
(`get_max_extent()` is a helper that computes the maximum display width occupied by the values in a character vector.)

All that's left to do is to implement a `format()` method for our new `"pillar_shaft_latlon"` class.
This method will be called with a `width` argument, which then determines which of the formats to choose.
The formatting of our choice is passed to the `new_ornament()` function:

```{r}
#' @export
format.pillar_shaft_latlon <- function(x, width, ...) {
  if (get_max_extent(x$deg_min) <= width) {
    ornament <- x$deg_min
  } else {
    ornament <- x$deg
  }

  pillar::new_ornament(ornament, align = "right")
}

data
print(data, width = 30)
```

## Testing

If you want to test the output of your code, you can compare it with a known state recorded in a text file.  The `testthat::expect_snapshot()` function offers an easy way to test output-generating functions. It takes care about details such as Unicode, ANSI escapes, and output width. Furthermore it won't make the tests fail on CRAN. This is important because your output may rely on details out of your control, which should be fixed eventually but should not lead to your package being removed from CRAN.

Use this testthat expectation in one of your test files to create a snapshot test:

```{r eval = FALSE}
expect_snapshot(pillar_shaft(data$loc))
```

See <https://testthat.r-lib.org/articles/snapshotting.html> for more information.

---
title: "Prototypes and sizes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Prototypes and sizes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

Rather than using `class()` and `length()`, vctrs has notions of prototype (`vec_ptype_show()`) and size (`vec_size()`). This vignette discusses the motivation for why these alternatives are necessary and connects their definitions to type coercion and the recycling rules.

Size and prototype are motivated by thinking about the optimal behaviour for `c()` and `rbind()`, particularly inspired by data frames with columns that are matrices or data frames.

```{r}
library(vctrs)
```

## Prototype

The idea of a prototype is to capture the metadata associated with a vector without capturing any data. Unfortunately, the `class()` of an object is inadequate for this purpose:

* The `class()` doesn't include attributes. Attributes are important because,
  for example, they store the levels of a factor and the timezone of a 
  `POSIXct`. You cannot combine two factors or two `POSIXct`s without
  thinking about the attributes.
  
* The `class()` of a matrix is "matrix" and doesn't include the type of the
  underlying vector or the dimensionality.
  
Instead, vctrs takes advantage of R's vectorised nature and uses a __prototype__, a 0-observation slice of the vector (this is basically `x[0]` but with some subtleties we'll come back to later). This is a miniature version of the vector that contains all of the attributes but none of the data. 

Conveniently, you can create many prototypes using existing base functions (e.g, `double()` and `factor(levels = c("a", "b"))`). vctrs provides a few helpers (e.g. `new_date()`, `new_datetime()`, and `new_duration()`) where the equivalents in base R are missing.

### Base prototypes

`vec_ptype()` creates a prototype from an existing object. However, many base vectors have uninformative printing methods for 0-length subsets, so vctrs also provides `vec_ptype_show()`, which prints the prototype in a friendly way (and returns nothing).

Using `vec_ptype_show()` allows us to see the prototypes base R classes:

*  Atomic vectors have no attributes and just display the underlying `typeof()`:

    ```{r}
    vec_ptype_show(FALSE)
    vec_ptype_show(1L)
    vec_ptype_show(2.5)
    vec_ptype_show("three")
    vec_ptype_show(list(1, 2, 3))
    ```

*  The prototype of matrices and arrays include the base type and the 
   dimensions after the first:

    ```{r}
    vec_ptype_show(array(logical(), c(2, 3)))
    vec_ptype_show(array(integer(), c(2, 3, 4)))
    vec_ptype_show(array(character(), c(2, 3, 4, 5)))
    ```

*  The prototype of a factor includes its levels. Levels are a character vector,
   which can be arbitrarily long, so the prototype just shows a hash. If the 
   hash of two factors is equal, it's highly likely that their levels are also 
   equal.

    ```{r}
    vec_ptype_show(factor("a"))
    vec_ptype_show(ordered("b"))
    ```
    
    While `vec_ptype_show()` prints only the hash, the prototype object itself does
    contain all levels:
    
    ```{r}
    vec_ptype(factor("a"))
    ```
    
*   Base R has three key date time classes: dates, date-times (`POSIXct`),
    and durations (`difftime)`. Date-times have a timezone, and durations have 
    a unit.

    ```{r}
    vec_ptype_show(Sys.Date())
    vec_ptype_show(Sys.time())
    vec_ptype_show(as.difftime(10, units = "mins"))
    ```

*   Data frames have the most complex prototype: the prototype of a data frame
    is the name and prototype of each column:
    
    ```{r}
    vec_ptype_show(data.frame(a = FALSE, b = 1L, c = 2.5, d = "x"))
    ```

    Data frames can have columns that are themselves data frames, making this
    a "recursive" type:
    
    ```{r}
    df <- data.frame(x = FALSE)
    df$y <- data.frame(a = 1L, b = 2.5)
    vec_ptype_show(df)
    ```

### Coercing to common type

It's often important to combine vectors with multiple types. vctrs provides a consistent set of rules for coercion, via `vec_ptype_common()`. `vec_ptype_common()` possesses the following invariants:

* `class(vec_ptype_common(x, y))` equals `class(vec_ptype_common(y, x))`.

* `class(vec_ptype_common(x, vec_ptype_common(y, z))` equals 
  `class(vec_ptype_common(vec_ptype_common(x, y), z))`.
  
* `vec_ptype_common(x, NULL) == vec_ptype(x)`.

i.e., `vec_ptype_common()` is both commutative and associative (with respect to class) and has an identity element, `NULL`; i.e., it's a __commutative monoid__. This means the underlying implementation is quite simple: we can find the common type of any number of objects by progressively finding the common type of pairs of objects.

Like with `vec_ptype()`, the easiest way to explore `vec_ptype_common()` is with `vec_ptype_show()`: when given multiple inputs, it will print their common prototype. (In other words: program with `vec_ptype_common()` but play with `vec_ptype_show()`.)

*   The common type of atomic vectors is computed very similar to the rules of base 
    R, except that we do not coerce to character automatically:
    
    ```{r, error = TRUE}
    vec_ptype_show(logical(), integer(), double())
    
    vec_ptype_show(logical(), character())
    ```

*   Matrices and arrays are automatically broadcast to higher dimensions:

    ```{r}
    vec_ptype_show(
      array(1, c(0, 1)), 
      array(1, c(0, 2))
    )
    
    vec_ptype_show(
      array(1, c(0, 1)), 
      array(1, c(0, 3)),
      array(1, c(0, 3, 4)),
      array(1, c(0, 3, 4, 5))
    )
    ```
    
    Provided that the dimensions follow the vctrs recycling rules:
    
    ```{r, error = TRUE}
    vec_ptype_show(
      array(1, c(0, 2)), 
      array(1, c(0, 3))
    )
    ```

*   Factors combine levels in the order in which they appear.

    ```{r}
    fa <- factor("a")
    fb <- factor("b")
    
    levels(vec_ptype_common(fa, fb))
    levels(vec_ptype_common(fb, fa))
    ```

*   Combining a date and date-time yields a date-time:

    ```{r}
    vec_ptype_show(new_date(), new_datetime())
    ```

    When combining two date times, the timezone is taken from the first input:

    ```{r}
    vec_ptype_show(
      new_datetime(tzone = "US/Central"), 
      new_datetime(tzone = "Pacific/Auckland")
    )
    ```

    Unless it's the local timezone, in which case any explicit time zone will
    win:

    ```{r}
    vec_ptype_show(
      new_datetime(tzone = ""), 
      new_datetime(tzone = ""), 
      new_datetime(tzone = "Pacific/Auckland")
    )
    ```

*   The common type of two data frames is the common type of each column that 
    occurs in both data frames:
    
    ```{r}
    vec_ptype_show(
      data.frame(x = FALSE), 
      data.frame(x = 1L),
      data.frame(x = 2.5)
    )
    ```
    
    And the union of the columns that only occur in one:

    ```{r}
    vec_ptype_show(data.frame(x = 1, y = 1), data.frame(y = 1, z = 1))
    ```

    Note that new columns are added on the right-hand side. This is consistent
    with the way that factor levels and time zones are handled.

### Casting to specified type

`vec_ptype_common()` finds the common type of a set of vector. Typically, however, what you want is a set of vectors coerced to that common type. That's the job of `vec_cast_common()`:

```{r}
str(vec_cast_common(
  FALSE, 
  1:5, 
  2.5
))

str(vec_cast_common(
  factor("x"), 
  factor("y")
))

str(vec_cast_common(
  data.frame(x = 1),
  data.frame(y = 1:2)
))
```

Alternatively, you can cast to a specific prototype using `vec_cast()`:

```{r, error = TRUE}
# Cast succeeds
vec_cast(c(1, 2), integer())

# Cast fails
vec_cast(c(1.5, 2.5), factor("a"))
```

If a cast is possible in general (i.e., double -> integer), but information is lost for a specific input (e.g. 1.5 -> 1), it will generate an error.

```{r, error = TRUE}
vec_cast(c(1.5, 2), integer())
```

You can suppress the lossy cast errors with `allow_lossy_cast()`:

```{r}
allow_lossy_cast(
  vec_cast(c(1.5, 2), integer())
)
```

This will suppress all lossy cast errors. Supply prototypes if you want to be specific about the type of lossy cast allowed:

```{r}
allow_lossy_cast(
  vec_cast(c(1.5, 2), integer()),
  x_ptype = double(),
  to_ptype = integer()
)
```

The set of casts should not be more permissive than the set of coercions. This is not enforced but it is expected from classes to follow the rule and keep the coercion ecosystem sound.


## Size

`vec_size()` was motivated by the need to have an invariant that describes the number of "observations" in a data structure. This is particularly important for data frames, as it's useful to have some function such that `f(data.frame(x))` equals `f(x)`. No base function has this property:

* `length(data.frame(x))` equals `1` because the length of a data frame 
   is the number of columns.

* `nrow(data.frame(x))` does not equal `nrow(x)` because `nrow()` of a 
  vector is `NULL`.

* `NROW(data.frame(x))` equals `NROW(x)` for vector `x`, so is almost what
  we want. But because `NROW()` is defined in terms of `length()`, it returns
  a value for every object, even types that can't go in a data frame, e.g.
  `data.frame(mean)` errors even though `NROW(mean)` is `1`.

We define `vec_size()` as follows:

* It is the length of 1d vectors.
* It is the number of rows of data frames, matrices, and arrays.
* It throws error for non vectors.

Given `vec_size()`, we can give a precise definition of a data frame: a data frame is a list of vectors where every vector has the same size. This has the desirable property of trivially supporting matrix and data frame columns.

### Slicing

`vec_slice()` is to `vec_size()` as `[` is to `length()`; i.e., it allows you to select observations regardless of the dimensionality of the underlying object. `vec_slice(x, i)` is equivalent to:

* `x[i]` when `x` is a vector.
* `x[i, , drop = FALSE]` when `x` is a data frame.
* `x[i, , , drop = FALSE]` when `x` is a 3d array.

```{r}
x <- sample(1:10)
df <- data.frame(x = x)

vec_slice(x, 5:6)
vec_slice(df, 5:6)
```

`vec_slice(data.frame(x), i)` equals `data.frame(vec_slice(x, i))` (modulo variable and row names).

Prototypes are generated with `vec_slice(x, 0L)`; given a prototype, you can initialize a vector of given size (filled with `NA`s) with `vec_init()`.

### Common sizes: recycling rules

Closely related to the definition of size are the __recycling rules__. The recycling rules determine the size of the output when two vectors of different sizes are combined. In vctrs, the recycling rules are encoded in `vec_size_common()`, which gives the common size of a set of vectors:

```{r}
vec_size_common(1:3, 1:3, 1:3)
vec_size_common(1:10, 1)
vec_size_common(integer(), 1)
```

vctrs obeys a stricter set of recycling rules than base R. Vectors of size 1 are recycled to any other size. All other size combinations will generate an error. This strictness prevents common mistakes like `dest == c("IAH", "HOU"))`, at the cost of occasionally requiring an explicit calls to `rep()`.

```{r, echo = FALSE, fig.cap="Summary of vctrs recycling rules. X indicates an error"}
knitr::include_graphics("../man/figures/sizes-recycling.png", dpi = 300)
```

You can apply the recycling rules in two ways:

*   If you have a vector and desired size, use `vec_recycle()`:

    ```{r}
    vec_recycle(1:3, 3)
    vec_recycle(1, 10)
    ```
    
*  If you have multiple vectors and you want to recycle them to the same
   size, use `vec_recycle_common()`:

    ```{r}
    vec_recycle_common(1:3, 1:3)
    vec_recycle_common(1:10, 1)
    ```

## Appendix: recycling in base R

The recycling rules in base R are described in [The R Language Definition](https://cran.r-project.org/doc/manuals/r-release/R-lang.html#Recycling-rules) but are not implemented in a single function and thus are not applied consistently. Here, I give a brief overview of their most common realisation, as well as showing some of the exceptions.

Generally, in base R, when a pair of vectors is not the same length, the shorter vector is recycled to the same length as the longer:

```{r}
rep(1, 6) + 1
rep(1, 6) + 1:2
rep(1, 6) + 1:3
```

If the length of the longer vector is not an integer multiple of the length of the shorter, you usually get a warning:

```{r}
invisible(pmax(1:2, 1:3))
invisible(1:2 + 1:3)
invisible(cbind(1:2, 1:3))
```

But some functions recycle silently:

```{r}
length(atan2(1:3, 1:2))
length(paste(1:3, 1:2))
length(ifelse(1:3, 1:2, 1:2))
```

And `data.frame()` throws an error:

```{r, error = TRUE}
data.frame(1:2, 1:3)
```

The R language definition states that "any arithmetic operation involving a zero-length vector has a zero-length result". But outside of arithmetic, this rule is not consistently followed:

```{r, error = TRUE}
# length-0 output
1:2 + integer()
atan2(1:2, integer())
pmax(1:2, integer())

# dropped
cbind(1:2, integer())

# recycled to length of first
ifelse(rep(TRUE, 4), integer(), character())

# preserved-ish
paste(1:2, integer())

# Errors
data.frame(1:2, integer())
```

[vctrs](README.md) / VctrsDatabase

Defined in: [index.d.ts:85](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L85)

A persistent vector database backed by an HNSW index.

Stores vectors with string IDs and optional JSON metadata.
Supports cosine, euclidean (L2), and dot-product distance metrics.
Uses memory-mapped I/O for fast loading and a write-ahead log for
crash safety.

## Example

```typescript
// Create a new database
const db = new VctrsDatabase("./my_db", 384, "cosine");
db.add("doc1", embedding, { title: "Hello" });
const results = db.search(queryVector, 5);
db.save();

// Open an existing database (auto-detects dim and metric)
const db2 = new VctrsDatabase("./my_db");
```

## Constructors

### Constructor

```ts
new VctrsDatabase(
   path, 
   dim?, 
   metric?, 
   hnswM?, 
   efConstruction?, 
   quantize?): VctrsDatabase;
```

Defined in: [index.d.ts:104](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L104)

Open or create a database.

When opening an existing database, `dim` and `metric` are
auto-detected — pass `null` or omit them.

#### Parameters

| Parameter | Type | Description |
| ------ | ------ | ------ |
| `path` | `string` | Directory path for the database files. |
| `dim?` | `number` | Vector dimensionality (e.g. 384). Required for new databases. |
| `metric?` | `string` | Distance metric: "cosine" (default), "euclidean"/"l2", or "dot"/"dot_product". |
| `hnswM?` | `number` | Max edges per HNSW node (default 16). Higher improves recall at the cost of memory. |
| `efConstruction?` | `number` | Build-time search width (default 200). Higher improves index quality at the cost of build time. |
| `quantize?` | `boolean` | Enable SQ8 scalar quantization for faster search. |

#### Returns

`VctrsDatabase`

#### Throws

If opening an existing database without `dim` and it doesn't exist,
  or if `metric` is invalid.

## Accessors

### deletedCount

#### Get Signature

```ts
get deletedCount(): number;
```

Defined in: [index.d.ts:300](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L300)

Number of deleted slots not yet reclaimed by `compact()`.

##### Returns

`number`

***

### dim

#### Get Signature

```ts
get dim(): number;
```

Defined in: [index.d.ts:309](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L309)

The vector dimensionality of this database.

##### Returns

`number`

***

### length

#### Get Signature

```ts
get length(): number;
```

Defined in: [index.d.ts:306](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L306)

Number of active vectors in the database.

##### Returns

`number`

***

### metric

#### Get Signature

```ts
get metric(): string;
```

Defined in: [index.d.ts:312](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L312)

The distance metric: "cosine", "euclidean", or "dot_product".

##### Returns

`string`

***

### quantizedSearch

#### Get Signature

```ts
get quantizedSearch(): boolean;
```

Defined in: [index.d.ts:297](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L297)

Whether quantized search is currently enabled.

##### Returns

`boolean`

***

### totalSlots

#### Get Signature

```ts
get totalSlots(): number;
```

Defined in: [index.d.ts:303](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L303)

Total allocated slots (active + deleted).

##### Returns

`number`

## Methods

### add()

```ts
add(
   id, 
   vector, 
   metadata?): void;
```

Defined in: [index.d.ts:123](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L123)

Add a vector with a unique string ID.

#### Parameters

| Parameter | Type | Description |
| ------ | ------ | ------ |
| `id` | `string` | Unique identifier. Throws if the ID already exists (use `upsert()` to insert-or-update). |
| `vector` | `number`\[\] | The embedding vector. Must match the database's dimensionality. |
| `metadata?` | `Record`\<`string`, `any`\> | Optional JSON-serializable metadata object. |

#### Returns

`void`

#### Throws

If the ID already exists or dimension doesn't match.

***

### addAsync()

```ts
addAsync(
   id, 
   vector, 
metadata?): Promise<void>;
```

Defined in: [index.d.ts:317](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L317)

Async version of `add()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |
| `vector` | `number`\[\] |
| `metadata?` | `Record`\<`string`, `any`\> |

#### Returns

`Promise`\<`void`\>

***

### addMany()

```ts
addMany(
   ids, 
   vectors, 
   metadatas?): void;
```

Defined in: [index.d.ts:142](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L142)

Batch insert multiple vectors.

#### Parameters

| Parameter | Type | Description |
| ------ | ------ | ------ |
| `ids` | `string`\[\] | Array of unique identifiers. |
| `vectors` | `number`\[\]\[\] | Array of embedding vectors. |
| `metadatas?` | `Record`\<`string`, `any`\>\[\] | Optional array of metadata objects. |

#### Returns

`void`

#### Throws

If any ID already exists, lengths mismatch, or dimensions don't match.

***

### addManyAsync()

```ts
addManyAsync(
   ids, 
   vectors, 
metadatas?): Promise<void>;
```

Defined in: [index.d.ts:323](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L323)

Async version of `addMany()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `ids` | `string`\[\] |
| `vectors` | `number`\[\]\[\] |
| `metadatas?` | `Record`\<`string`, `any`\>\[\] |

#### Returns

`Promise`\<`void`\>

***

### close()

```ts
close(): void;
```

Defined in: [index.d.ts:271](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L271)

Close the database and save to disk. Alias for `save()`.

#### Returns

`void`

#### Throws

On I/O errors.

***

### compact()

```ts
compact(): void;
```

Defined in: [index.d.ts:282](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L282)

Rebuild the index with only live vectors, reclaiming deleted slots.
Call after many deletions to reduce memory and disk usage.

#### Returns

`void`

#### Throws

On internal errors.

***

### contains()

```ts
contains(id): boolean;
```

Defined in: [index.d.ts:251](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L251)

Check if a vector ID exists in the database.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |

#### Returns

`boolean`

***

### count()

```ts
count(whereFilter?): number;
```

Defined in: [index.d.ts:216](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L216)

Count vectors matching a filter, or all vectors if no filter.

Uses the inverted metadata index for fast counting with equality
and `$in` filters.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `whereFilter?` | [`WhereFilter`](TypeAlias.WhereFilter.md) |

#### Returns

`number`

***

### delete()

```ts
delete(id): boolean;
```

Defined in: [index.d.ts:234](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L234)

Delete a vector by ID.

The slot is marked as deleted but not reclaimed until `compact()`
is called.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |

#### Returns

`boolean`

`true` if found and deleted, `false` if not found.

#### Throws

On internal errors (e.g. WAL write failure).

***

### deleteAsync()

```ts
deleteAsync(id): Promise<boolean>;
```

Defined in: [index.d.ts:358](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L358)

Async version of `delete()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |

#### Returns

`Promise`\<`boolean`\>

***

### disableQuantizedSearch()

```ts
disableQuantizedSearch(): void;
```

Defined in: [index.d.ts:294](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L294)

Disable quantized search and use full-precision vectors.

#### Returns

`void`

***

### enableQuantizedSearch()

```ts
enableQuantizedSearch(): void;
```

Defined in: [index.d.ts:291](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L291)

Enable SQ8 quantized search for faster HNSW traversal.

Quantized vectors use 4x less memory for distance comparisons
during graph traversal, with full-precision re-ranking of final
candidates.

#### Returns

`void`

***

### get()

```ts
get(id): GetResult;
```

Defined in: [index.d.ts:223](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L223)

Retrieve a vector and its metadata by ID.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |

#### Returns

[`GetResult`](Interface.GetResult.md)

#### Throws

If the ID is not found.

***

### ids()

```ts
ids(): string\[\];
```

Defined in: [index.d.ts:254](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L254)

Return an array of all vector IDs in the database.

#### Returns

`string`\[\]

***

### save()

```ts
save(): void;
```

Defined in: [index.d.ts:264](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L264)

Persist the database to disk.

Writes the HNSW graph, metadata, and vectors, then truncates the
write-ahead log.

#### Returns

`void`

#### Throws

On I/O errors (e.g. disk full, permission denied).

***

### saveAsync()

```ts
saveAsync(): Promise<void>;
```

Defined in: [index.d.ts:355](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L355)

Async version of `save()`.

#### Returns

`Promise`\<`void`\>

***

### search()

```ts
search(
   vector, 
   k?, 
   efSearch?, 
   whereFilter?, 
   maxDistance?): SearchResult\[\];
```

Defined in: [index.d.ts:186](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L186)

Find the k nearest neighbors to a query vector.

#### Parameters

| Parameter | Type | Description |
| ------ | ------ | ------ |
| `vector` | `number`\[\] | The query embedding. |
| `k?` | `number` | Number of results to return (default 10). |
| `efSearch?` | `number` | HNSW search-time width. Higher improves recall. |
| `whereFilter?` | [`WhereFilter`](TypeAlias.WhereFilter.md) | Optional metadata filter. See [WhereFilter](TypeAlias.WhereFilter.md). |
| `maxDistance?` | `number` | Optional distance threshold — discard results beyond this distance. |

#### Returns

[`SearchResult`](Interface.SearchResult.md)\[\]

Array of SearchResult objects sorted by distance (ascending).

#### Throws

If the vector dimension doesn't match.

#### Example

```typescript
// Basic search
const results = db.search(queryVec, 5);

// With metadata filter
const results = db.search(queryVec, 10, null, { category: "science" });

// With distance threshold
const results = db.search(queryVec, 10, null, null, 0.5);
```

***

### searchAsync()

```ts
searchAsync(
   vector, 
   k?, 
   efSearch?, 
   whereFilter?, 
maxDistance?): Promise<SearchResult\[\]>;
```

Defined in: [index.d.ts:337](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L337)

Async version of `search()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `vector` | `number`\[\] |
| `k?` | `number` |
| `efSearch?` | `number` |
| `whereFilter?` | [`WhereFilter`](TypeAlias.WhereFilter.md) |
| `maxDistance?` | `number` |

#### Returns

`Promise`\<[`SearchResult`](Interface.SearchResult.md)[]\>

***

### searchMany()

```ts
searchMany(
   vectors, 
   k?, 
   efSearch?, 
   whereFilter?, 
   maxDistance?): SearchResult\[\]\[\];
```

Defined in: [index.d.ts:202](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L202)

Search multiple queries in parallel.

Significantly faster than calling `search()` in a loop.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `vectors` | `number`\[\]\[\] |
| `k?` | `number` |
| `efSearch?` | `number` |
| `whereFilter?` | [`WhereFilter`](TypeAlias.WhereFilter.md) |
| `maxDistance?` | `number` |

#### Returns

[`SearchResult`](Interface.SearchResult.md)\[\]\[\]

Array of result arrays, one per query.

#### Throws

If any vector dimension doesn't match.

***

### searchManyAsync()

```ts
searchManyAsync(
   vectors, 
   k?, 
   efSearch?, 
   whereFilter?, 
maxDistance?): Promise<SearchResult\[\]\[\]>;
```

Defined in: [index.d.ts:346](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L346)

Async version of `searchMany()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `vectors` | `number`\[\]\[\] |
| `k?` | `number` |
| `efSearch?` | `number` |
| `whereFilter?` | [`WhereFilter`](TypeAlias.WhereFilter.md) |
| `maxDistance?` | `number` |

#### Returns

`Promise`\<[`SearchResult`](Interface.SearchResult.md)\[\]\[\]\>

***

### stats()

```ts
stats(): VctrsStats;
```

Defined in: [index.d.ts:274](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L274)

Get graph-level statistics for diagnostics.

#### Returns

[`VctrsStats`](Interface.VctrsStats.md)

***

### update()

```ts
update(
   id, 
   vector?, 
   metadata?): void;
```

Defined in: [index.d.ts:244](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L244)

Update a vector's embedding and/or metadata in-place.

#### Parameters

| Parameter | Type | Description |
| ------ | ------ | ------ |
| `id` | `string` | The vector's unique identifier. |
| `vector?` | `number`\[\] | New embedding (or null to keep existing). |
| `metadata?` | `Record`\<`string`, `any`\> | New metadata (or null to keep existing). |

#### Returns

`void`

#### Throws

If the ID is not found or the vector dimension doesn't match.

***

### upsert()

```ts
upsert(
   id, 
   vector, 
   metadata?): void;
```

Defined in: [index.d.ts:132](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L132)

Insert a vector, or update it if the ID already exists.

Same as `add()` but overwrites existing entries instead of throwing.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |
| `vector` | `number`\[\] |
| `metadata?` | `Record`\<`string`, `any`\> |

#### Returns

`void`

#### Throws

If the vector dimension doesn't match.

***

### upsertAsync()

```ts
upsertAsync(
   id, 
   vector, 
metadata?): Promise<void>;
```

Defined in: [index.d.ts:320](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L320)

Async version of `upsert()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `id` | `string` |
| `vector` | `number`\[\] |
| `metadata?` | `Record`\<`string`, `any`\> |

#### Returns

`Promise`\<`void`\>

***

### upsertMany()

```ts
upsertMany(
   ids, 
   vectors, 
   metadatas?): void;
```

Defined in: [index.d.ts:156](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L156)

Batch upsert — inserts new vectors, updates existing ones.

More efficient than calling `upsert()` in a loop because new
vectors are batch-inserted together.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `ids` | `string`\[\] |
| `vectors` | `number`\[\]\[\] |
| `metadatas?` | `Record`\<`string`, `any`\>\[\] |

#### Returns

`void`

#### Throws

If lengths mismatch or dimensions don't match.

***

### upsertManyAsync()

```ts
upsertManyAsync(
   ids, 
   vectors, 
metadatas?): Promise<void>;
```

Defined in: [index.d.ts:330](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L330)

Async version of `upsertMany()`.

#### Parameters

| Parameter | Type |
| ------ | ------ |
| `ids` | `string`\[\] |
| `vectors` | `number`\[\]\[\] |
| `metadatas?` | `Record`\<`string`, `any`\>\[\] |

#### Returns

`Promise`\<`void`\>

# Python API Reference

## Database

::: vctrs.Database
    options:
      show_source: false

## Client

::: vctrs.Client
    options:
      show_source: false

## AsyncDatabase

::: vctrs.AsyncDatabase
    options:
      show_source: false

## SearchResult

::: vctrs.SearchResult
    options:
      show_source: false

# WebAssembly

vctrs compiles to WebAssembly, so you can run vector search directly in the browser. No server required.

The WASM build is ~220KB and runs entirely in-memory. It shares the same Rust core as the Python and Node.js bindings.

## Install

### npm

```bash
npm install @yang-29/vctrs-wasm
```

```javascript
import init, { VctrsDatabase } from "@yang-29/vctrs-wasm";

await init();
const db = new VctrsDatabase(384, "cosine");
```

### CDN (no bundler)

```html
<script type="module">
import init, { VctrsDatabase } from "https://unpkg.com/@yang-29/vctrs-wasm/vctrs_wasm.js";

await init();
const db = new VctrsDatabase(384, "cosine");
</script>
```

### Self-hosted

Build from source with [wasm-pack](https://rustwasm.github.io/wasm-pack/):

```bash
wasm-pack build wasm --target web --out-dir pkg
```

This produces a `wasm/pkg/` directory you can serve or bundle.

## Quick Start

```javascript
import init, { VctrsDatabase } from "@yang-29/vctrs-wasm";

await init();

// Create a database
const db = new VctrsDatabase(4, "cosine");

// Add vectors with metadata
db.add("cat", new Float32Array([1, 0, 0, 0]), { type: "animal" });
db.add("dog", new Float32Array([0.9, 0.1, 0, 0]), { type: "animal" });
db.add("car", new Float32Array([0, 0, 1, 0]), { type: "vehicle" });

// Search
const results = db.search(new Float32Array([1, 0, 0, 0]), 2);
for (const r of results) {
  console.log(r.id, r.distance, r.metadata);
  r.free(); // free WASM memory
}

// Clean up
db.free();
```

## API Reference

### `new VctrsDatabase(dim, metric)`

Create a new in-memory vector database.

| Parameter | Type | Description |
|-----------|------|-------------|
| `dim` | `number` | Vector dimensionality |
| `metric` | `string` | `"cosine"`, `"euclidean"` / `"l2"`, or `"dot"` / `"dot_product"` |

### `db.add(id, vector, metadata)`

Add a vector with a unique string ID.

| Parameter | Type | Description |
|-----------|------|-------------|
| `id` | `string` | Unique identifier |
| `vector` | `Float32Array` | Vector of length `dim` |
| `metadata` | `object \| null` | Optional JSON metadata |

Throws if `id` already exists. Use `upsert` to insert-or-update.

### `db.upsert(id, vector, metadata)`

Insert a new vector or update an existing one.

### `db.search(vector, k, filter?, maxDistance?)`

Find the `k` nearest neighbors. Returns an array of `SearchResult` objects.

| Parameter | Type | Description |
|-----------|------|-------------|
| `vector` | `Float32Array` | Query vector of length `dim` |
| `k` | `number` | Number of results |
| `filter` | `object \| null` | Optional metadata filter (same syntax as Python/Node) |
| `maxDistance` | `number \| null` | Optional distance threshold |

### `db.addMany(ids, vectors, dim, metadatas?)`

Batch add multiple vectors. `vectors` is a flat `Float32Array` of length `ids.length * dim`.

| Parameter | Type | Description |
|-----------|------|-------------|
| `ids` | `string[]` | Array of unique IDs |
| `vectors` | `Float32Array` | Flat array of all vectors concatenated |
| `dim` | `number` | Vector dimensionality |
| `metadatas` | `object[] \| null` | Optional array of metadata objects |

### `db.get(id)`

Retrieve a vector and its metadata. Returns `{ vector: Float32Array, metadata: object | null }`.

### `db.ids()`

Return all vector IDs as an array of strings.

### `db.count(filter?)`

Count vectors, optionally with a metadata filter.

### `db.delete(id)`

Delete a vector by ID. Returns `true` if found and deleted.

### `db.contains(id)`

Check if a vector ID exists. Returns `boolean`.

### `db.length` (getter)

Number of vectors in the database.

### `db.dim` (getter)

Vector dimensionality.

### `db.export_json()`

Export all vectors and metadata as a JSON string.

```javascript
const json = db.export_json();
// Save to file, localStorage, IndexedDB, etc.
localStorage.setItem("vectors_backup", json);
```

### `db.import_json(json)`

Import vectors from a JSON string (upsert semantics).

```javascript
const json = localStorage.getItem("vectors_backup");
db.import_json(json);
```

### `SearchResult`

| Property | Type | Description |
|----------|------|-------------|
| `.id` | `string` | Vector ID |
| `.distance` | `number` | Distance to query (lower = more similar for cosine/euclidean) |
| `.metadata` | `object \| null` | Stored metadata |

!!! warning "Memory management"
    WASM objects are not garbage collected automatically. Call `.free()` on `SearchResult` objects and the `VctrsDatabase` when done to avoid memory leaks.

    ```javascript
    const results = db.search(query, 10);
    for (const r of results) {
      console.log(r.id);
      r.free();
    }
    db.free();
    ```

## Differences from Python / Node.js

The WASM build runs entirely in-memory. It does not support:

- **Persistence** — no `save()` / `load()`. Data is lost on page refresh. Use `export_json()` / `import_json()` with localStorage or IndexedDB for client-side persistence.
- **Async** — all operations are synchronous (but fast).
- **BLAS acceleration** — uses pure-Rust math (still fast for moderate dataset sizes).
- **Parallel search** — no `searchMany()` (WASM is single-threaded).

## Performance

Benchmarks from Chrome on Apple M-series (single-threaded WASM):

| Operation | Time |
|-----------|------|
| Insert 1,000 vectors (384d) | ~900ms |
| Search k=10 (1,000 vectors, 384d) | ~0.5ms |
| WASM binary size | 220KB |

For datasets under ~10,000 vectors, WASM performance is suitable for interactive use. For larger datasets, use the Python or Node.js bindings.

## Use Cases

- **Client-side semantic search** — search embeddings without a backend
- **Offline-first apps** — works without network after loading
- **Prototyping** — test vector search logic in the browser console
- **Edge/serverless** — run in Cloudflare Workers, Deno Deploy, etc.

# Node.js API Reference

Auto-generated from TypeScript definitions.

## Classes

| Class | Description |
| ------ | ------ |
| [VctrsDatabase](typedoc/Class.VctrsDatabase.md) | A persistent vector database backed by an HNSW index. |

## Interfaces

| Interface | Description |
| ------ | ------ |
| [GetResult](typedoc/Interface.GetResult.md) | A vector and its metadata, returned by `get()`. |
| [SearchResult](typedoc/Interface.SearchResult.md) | A single search result. |
| [VctrsStats](typedoc/Interface.VctrsStats.md) | Graph-level statistics returned by `stats()`. |

## Type Aliases

| Type Alias | Description |
| ------ | ------ |
| [WhereFilter](typedoc/TypeAlias.WhereFilter.md) | Metadata filter for search queries. |

# vctrs

A fast embedded vector database. Rust core with Python, Node.js, and WebAssembly bindings.

Vector search as a library, not a service. No server, no config.

## Install

=== "Python"

    ```bash
    pip install vctrs
    ```

=== "Node.js"

    ```bash
    npm install @yang-29/vctrs
    ```

=== "WebAssembly"

    ```bash
    npm install @yang-29/vctrs-wasm
    ```

=== "Rust"

    ```toml
    [dependencies]
    vctrs-core = "0.2"
    ```

## Quick Start

=== "Python"

    ```python
    from vctrs import Database

    db = Database("./mydb", dim=384, metric="cosine")

    db.add("doc1", vector, {"title": "hello"})
    results = db.search(query_vector, k=10)

    for r in results:
        print(r.id, r.distance, r.metadata)
    ```

=== "Node.js"

    ```typescript
    import { VctrsDatabase } from "@yang-29/vctrs";

    const db = new VctrsDatabase("./mydb", 384, "cosine");

    db.add("doc1", vector, { title: "hello" });
    const results = db.search(queryVector, 10);

    for (const r of results) {
      console.log(r.id, r.distance, r.metadata);
    }
    ```

=== "WebAssembly"

    ```javascript
    import init, { VctrsDatabase } from "@yang-29/vctrs-wasm";

    await init();
    const db = new VctrsDatabase(384, "cosine");

    db.add("doc1", new Float32Array(vector), { title: "hello" });
    const results = db.search(new Float32Array(queryVector), 10);

    for (const r of results) {
      console.log(r.id, r.distance, r.metadata);
      r.free();
    }
    ```

=== "Rust"

    ```rust
    use vctrs_core::db::Database;
    use vctrs_core::distance::Metric;

    let db = Database::open_or_create("./mydb", 384, Metric::Cosine)?;

    db.add("doc1", embedding, Some(json!({"title": "hello"})))?;
    let results = db.search(&query, 10, None, None, None)?;

    for r in &results {
        println!("{} {:.4}", r.id, r.distance);
    }
    ```

## Why vctrs

ChromaDB, Pinecone, and Qdrant are databases you run. vctrs is a library you embed.

- **Embedded** — no server, no config, no network round trips
- **Fast** — 6-7x faster than ChromaDB, matches raw numpy
- **Lightweight** — single native binary, no 200MB dependency tree
- **Crash-safe** — write-ahead log with CRC32 checksums
- **Instant loads** — memory-mapped vectors, zero-copy on startup

## Performance

10,000 vectors, 384 dimensions, cosine, Apple M-series:

| | vctrs | ChromaDB | numpy |
|---|---|---|---|
| Search k=10 | **0.14ms** | 0.93ms | 0.16ms |
| Insert 10k | **518ms** | 2367ms | — |
| Load from disk | **1.2ms** | 1.6ms | — |

## Filter Operators

| Operator | Description | Example |
|----------|-------------|---------|
| `$eq` (default) | Equals | `{ field: "value" }` |
| `$ne` | Not equals | `{ field: { $ne: "value" } }` |
| `$in` | In list | `{ field: { $in: ["a", "b"] } }` |
| `$gt` | Greater than | `{ field: { $gt: 10 } }` |
| `$gte` | Greater or equal | `{ field: { $gte: 10 } }` |
| `$lt` | Less than | `{ field: { $lt: 20 } }` |
| `$lte` | Less or equal | `{ field: { $lte: 20 } }` |

Multiple keys in a filter are ANDed together.

## Collections

Manage multiple isolated indexes in one directory:

=== "Python"

    ```python
    from vctrs import Client

    client = Client("./data")
    movies = client.create_collection("movies", dim=384)
    docs = client.get_or_create_collection("docs", dim=768, metric="dot")

    client.list_collections()  # → ["docs", "movies"]
    ```

=== "Node.js"

    ```typescript
    import { VctrsClient } from "@yang-29/vctrs";

    const client = new VctrsClient("./data");
    const movies = client.createCollection("movies", 384);
    const docs = client.getOrCreateCollection("docs", 768, "dot");

    client.listCollections();  // → ["docs", "movies"]
    ```

=== "Rust"

    ```rust
    use vctrs_core::client::Client;

    let client = Client::new("./data")?;
    let movies = client.create_collection("movies", 384, Metric::Cosine)?;
    ```

## Export / Import

Backup and restore databases as JSON:

=== "Python"

    ```python
    db.export_json("backup.json", pretty=True)
    db.import_json("backup.json")  # upsert semantics
    ```

=== "Node.js"

    ```typescript
    db.exportJson("backup.json", true);
    db.importJson("backup.json");
    ```

=== "WebAssembly"

    ```javascript
    const json = db.export_json();     // → JSON string
    db.import_json(jsonString);        // upsert semantics
    ```

[vctrs](README.md) / GetResult

Defined in: [index.d.ts:20](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L20)

A vector and its metadata, returned by `get()`.

## Properties

| Property | Type | Description | Defined in |
| ------ | ------ | ------ | ------ |
| <a id="metadata"></a> `metadata?` | `Record`\<`string`, `any`\> | The metadata object attached to this vector, or undefined. | [index.d.ts:24](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L24) |
| <a id="vector"></a> `vector` | `number`\[\] | The stored embedding vector. | [index.d.ts:22](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L22) |

[vctrs](README.md) / WhereFilter

```ts
type WhereFilter = Record<string, any>;
```

Defined in: [index.d.ts:37](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L37)

Metadata filter for search queries.

Supports equality, operators, and compound filters:
- Equality: `{ field: "value" }`
- Not-equal: `{ field: { $ne: "value" } }`
- In-set: `{ field: { $in: ["a", "b"] } }`
- Numeric ranges: `{ field: { $gt: 10, $lte: 20 } }`
- Compound (AND): `{ f1: "v1", f2: { $gt: 5 } }`

[vctrs](README.md) / VctrsStats

Defined in: [index.d.ts:40](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L40)

Graph-level statistics returned by `stats()`.

## Properties

| Property | Type | Description | Defined in |
| ------ | ------ | ------ | ------ |
| <a id="avgdegreelayer0"></a> `avgDegreeLayer0` | `number` | Average degree (edges) on layer 0. | [index.d.ts:48](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L48) |
| <a id="maxdegreelayer0"></a> `maxDegreeLayer0` | `number` | Maximum degree on layer 0. | [index.d.ts:50](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L50) |
| <a id="memorygraphbytes"></a> `memoryGraphBytes` | `number` | Memory used by graph structure in bytes. | [index.d.ts:56](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L56) |
| <a id="memoryquantizedbytes"></a> `memoryQuantizedBytes` | `number` | Memory used by quantized vectors in bytes (0 if not quantized). | [index.d.ts:58](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L58) |
| <a id="memoryvectorsbytes"></a> `memoryVectorsBytes` | `number` | Memory used by vector storage in bytes. | [index.d.ts:54](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L54) |
| <a id="mindegreelayer0"></a> `minDegreeLayer0` | `number` | Minimum degree on layer 0. | [index.d.ts:52](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L52) |
| <a id="numdeleted"></a> `numDeleted` | `number` | Number of deleted vectors not yet reclaimed. | [index.d.ts:44](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L44) |
| <a id="numlayers"></a> `numLayers` | `number` | Number of HNSW layers. | [index.d.ts:46](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L46) |
| <a id="numvectors"></a> `numVectors` | `number` | Number of active (non-deleted) vectors. | [index.d.ts:42](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L42) |
| <a id="usesbruteforce"></a> `usesBruteForce` | `boolean` | Whether brute-force search is being used (small datasets). | [index.d.ts:60](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L60) |
| <a id="usesquantizedsearch"></a> `usesQuantizedSearch` | `boolean` | Whether quantized HNSW traversal is enabled. | [index.d.ts:62](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L62) |

# Interactive Demo

Try vctrs in the browser. Everything below runs client-side via WebAssembly — no server, no network requests.

<iframe src="../demo.html" width="100%" height="950" frameborder="0" style="border: 1px solid #e0e0e0; border-radius: 8px;"></iframe>

## Two modes

### Manual Vectors (default)
Pick a preset (Movies, Cities, or Foods) — each loads 20 items as 8-dimensional vectors where each dimension represents a quality. Click suggested searches to see how cosine similarity finds related items.

### Semantic Search (AI)
Click **Semantic Search (AI)** to enable real natural-language search powered by [mdbr-leaf-ir](https://huggingface.co/MongoDB/mdbr-leaf-ir) — the #1 ranked retrieval model under 100M params on MTEB — running entirely in your browser. The model (~30MB) is downloaded once and cached.

Load a preset (Sentences, Products, or Questions), then search with plain English — the model embeds your query into 768 dimensions and vctrs finds the closest matches.

!!! note "Everything runs locally"
    Both the vctrs WASM binary (~220KB) and the transformer model run entirely in your browser. No data is sent to any server.

Source: [`wasm/src/lib.rs`](https://github.com/yang-29/vctrs/blob/main/wasm/src/lib.rs)

# Python — Getting Started

## Install

```bash
pip install vctrs
```

## Basic Usage

```python
from vctrs import Database

# Create a new database (or open existing)
db = Database("./mydb", dim=384, metric="cosine")

# Add vectors
db.add("doc1", [0.1, 0.2, ...], {"title": "Hello"})

# Batch insert
db.add_many(ids, vectors, metadatas)

# Search
results = db.search(query_vector, k=10)
for r in results:
    print(r.id, r.distance, r.metadata)

# Filtered search
results = db.search(query, k=10, where_filter={"category": "science"})

# Distance threshold
results = db.search(query, k=10, max_distance=0.5)

# Save and close
db.save()
```

## Context Manager

```python
with Database("./mydb", dim=384) as db:
    db.add("doc1", vector)
    # auto-saves on exit
```

## Async Usage

```python
from vctrs import AsyncDatabase

async with AsyncDatabase("./mydb", dim=384) as db:
    await db.add("doc1", vector, {"title": "Hello"})
    results = await db.search(query_vector, k=10)
```

## Numpy Support

Vectors can be passed as Python lists or numpy arrays:

```python
import numpy as np

embedding = np.random.randn(384).astype(np.float32)
db.add("doc1", embedding)

# Batch with 2D numpy array
batch = np.random.randn(100, 384).astype(np.float32)
db.add_many([f"doc{i}" for i in range(100)], batch)
```

## Configuration

| Parameter | Default | Description |
|-----------|---------|-------------|
| `dim` | required | Vector dimensionality |
| `metric` | `"cosine"` | `"cosine"`, `"euclidean"`/`"l2"`, `"dot"`/`"dot_product"` |
| `m` | `16` | HNSW edges per node (higher = better recall, more memory) |
| `ef_construction` | `200` | Build-time search width (higher = better index quality) |
| `quantize` | `False` | Enable SQ8 scalar quantization |

## Collections

Manage multiple isolated indexes under one directory:

```python
from vctrs import Client

client = Client("./data")

# Create collections with different configs
movies = client.create_collection("movies", dim=384)
docs = client.create_collection("docs", dim=768, metric="dot")

# Use them like regular databases
movies.add("m1", vector, {"title": "Alien"})
movies.save()

# List, get, delete
client.list_collections()        # → ["docs", "movies"]
db = client.get_collection("movies")
db = client.get_or_create_collection("items", dim=384)
client.delete_collection("docs")
```

## Export / Import

Backup and restore databases as JSON:

```python
# Export to file
db.export_json("backup.json")
db.export_json("backup.json", pretty=True)  # human-readable

# Import into existing database (upsert semantics)
db.import_json("backup.json")
```

The JSON format includes dimension, metric, and all vectors with metadata.

# output tests

    Code
      vec_ptype_show()
    Output
      Prototype: NULL

---

    Code
      vec_ptype_show(integer())
    Output
      Prototype: integer

---

    Code
      vec_ptype_show(integer(), double())
    Output
      Prototype: <double>
      0. (           , <integer> ) = <integer>
      1. ( <integer> , <double>  ) = <double> 

---

    Code
      vec_ptype_show(logical(), integer(), double())
    Output
      Prototype: <double>
      0. (           , <logical> ) = <logical>
      1. ( <logical> , <integer> ) = <integer>
      2. ( <integer> , <double>  ) = <double> 

# vec_ptype_common() includes index in argument tag

    Code
      vec_ptype_common(df1, df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `..2$x$y$z` <character>.

---

    Code
      vec_ptype_common(df1, df1, df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `..3$x$y$z` <character>.

---

    Code
      vec_ptype_common(large_df1, large_df2)
    Condition
      Error:
      ! Can't combine `..1$foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar$y$z` <double> and `..2$foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar$y$z` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = df1, bar = df2)
    Condition
      Error:
      ! Can't combine `foo$x$y$z` <double> and `bar$x$y$z` <character>.

---

    Code
      vec_ptype_common(df1, df1, bar = df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `bar$x$y$z` <character>.

---

    Code
      vec_ptype_common(TRUE, !!!list(1, "foo"))
    Condition
      Error:
      ! Can't combine `..2` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(TRUE, !!!list(1, 2), "foo")
    Condition
      Error:
      ! Can't combine `..2` <double> and `..4` <character>.

---

    Code
      vec_ptype_common(1, !!!list(TRUE, FALSE), "foo")
    Condition
      Error:
      ! Can't combine `..1` <double> and `..4` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo"))
    Condition
      Error:
      ! Can't combine `foo` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = "foo"))
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `baz` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr"))
    Condition
      Error:
      ! Can't combine `foo` <logical> and `baz` <character>.

# vec_ptype_common() handles spliced names consistently (#1570)

    Code
      vec_ptype_common(a = "foo", b = "bar", y = NULL, z = 1)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.
    Code
      vec_ptype_common(!!!args1, !!!args2)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.
    Code
      vec_ptype_common(!!!args1, "{y_name}" := NULL, "{z_name}" := 1)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.

# obj_check_vector() errors on scalars

    Code
      obj_check_vector(quote(foo))
    Condition
      Error:
      ! `quote(foo)` must be a vector, not a symbol.

---

    Code
      obj_check_vector(foobar())
    Condition
      Error:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.

# obj_check_vector() error respects `arg` and `call`

    Code
      my_check_vector(foobar())
    Condition
      Error in `my_check_vector()`:
      ! `foo` must be a vector, not a <vctrs_foobar> object.

# assertion failures are explained

    Code
      vec_assert(lgl(), chr())
    Condition
      Error:
      ! `lgl()` must be a vector with type <character>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), factor())
    Condition
      Error:
      ! `lgl()` must be a vector with type <factor<>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), factor(levels = "foo"))
    Condition
      Error:
      ! `lgl()` must be a vector with type <factor<c1562>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(factor(levels = "bar"), factor(levels = "foo"))
    Condition
      Error:
      ! `factor(levels = "bar")` must be a vector with type <factor<c1562>>.
      Instead, it has type <factor<9f154>>.

---

    Code
      vec_assert(factor(), chr())
    Condition
      Error:
      ! `factor()` must be a vector with type <character>.
      Instead, it has type <factor<>>.

---

    Code
      vec_assert(lgl(), data.frame())
    Condition
      Error:
      ! `lgl()` must be a vector with type <data.frame<>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), data.frame(x = 1))
    Condition
      Error:
      ! `lgl()` must be a vector with type <data.frame<x:double>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), data.frame(x = 1, y = 2))
    Condition
      Error:
      ! `lgl()` must be a vector with type:
      
        <data.frame<
          x: double
          y: double
        >>
      
      Instead, it has type <logical>.

---

    Code
      vec_assert(data.frame(), chr())
    Condition
      Error:
      ! `data.frame()` must be a vector with type <character>.
      Instead, it has type <data.frame<>>.

---

    Code
      vec_assert(data.frame(x = 1), chr())
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type <character>.
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1), data.frame(x = "foo"))
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type <data.frame<x:character>>.
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1), data.frame(x = "foo", y = 2))
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type:
      
        <data.frame<
          x: character
          y: double
        >>
      
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1, y = 2), chr())
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type <character>.
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

---

    Code
      vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo"))
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type <data.frame<x:character>>.
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

---

    Code
      vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo", y = 2))
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type:
      
        <data.frame<
          x: character
          y: double
        >>
      
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

# vec_assert() validates `size` (#1470)

    Code
      (expect_error(vec_assert(1, size = c(2, 3))))
    Output
      <error/rlang_error>
      Error in `vec_assert()`:
      ! `size` must be length 1, not length 2.
    Code
      (expect_error(vec_assert(1, size = 1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_assert()`:
      ! Can't convert from `size` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(vec_assert(1, size = "x")))
    Output
      <error/vctrs_error_cast>
      Error in `vec_assert()`:
      ! Can't convert `size` <character> to <integer>.

# vec_check_size() errors on the wrong size

    Code
      vec_check_size(1:5, size = 1L)
    Condition
      Error:
      ! `1:5` must have size 1, not size 5.

---

    Code
      vec_check_size(1:5, size = 10L)
    Condition
      Error:
      ! `1:5` must have size 10, not size 5.

# vec_check_size() errors on scalars

    Code
      vec_check_size(quote(foo), size = 1L)
    Condition
      Error:
      ! `quote(foo)` must be a vector, not a symbol.

---

    Code
      vec_check_size(foobar(), size = 1L)
    Condition
      Error:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.

# vec_check_size() error respects `arg` and `call`

    Code
      my_check_size(1L, size = 5L)
    Condition
      Error in `my_check_size()`:
      ! `foo` must have size 5, not size 1.

---

    Code
      my_check_size(foobar(), size = 5L)
    Condition
      Error in `my_check_size()`:
      ! `foo` must be a vector, not a <vctrs_foobar> object.

# vec_check_size() validates `size`

    Code
      vec_check_size(1, size = "x")
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_size(1, size = c(1L, 2L))
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_size(1, size = 1.5)
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a whole number, not a decimal number.

# list_all_vectors() works

    Code
      (expect_error(list_all_vectors(env())))
    Output
      <error/rlang_error>
      Error in `list_all_vectors()`:
      ! `x` must be a list, not an environment.

# obj_check_list() works

    Code
      my_function <- (function(my_arg) obj_check_list(my_arg))
      (expect_error(my_function(env())))
    Output
      <error/rlang_error>
      Error in `my_function()`:
      ! `my_arg` must be a list, not an environment.

# obj_check_list() uses a special error when `arg` is the empty string (#1604)

    Code
      obj_check_list(1, arg = "")
    Condition
      Error:
      ! Input must be a list, not the number 1.

# obj_check_list() and list_check_all_vectors() work

    Code
      my_function <- (function(my_arg) list_check_all_vectors(my_arg))
      (expect_error(my_function(env())))
    Output
      <error/rlang_error>
      Error in `list_check_all_vectors()`:
      ! `x` must be a list, not an environment.
    Code
      (expect_error(my_function(list(1, env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg[[2]]` must be a vector, not an environment.
    Code
      (expect_error(my_function(list(1, name = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg$name` must be a vector, not an environment.
    Code
      (expect_error(my_function(list(1, foo = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg$foo` must be a vector, not an environment.

# list_check_all_size() works

    Code
      my_function <- (function(my_arg, size) list_check_all_size(my_arg, size))
      (expect_error(list_check_all_size(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error:
      ! `list(1:2, 1:3)[[2]]` must have size 2, not size 3.
    Code
      (expect_error(my_function(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `my_arg[[2]]` must have size 2, not size 3.
    Code
      (expect_error(my_function(list(NULL, 1:2), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `my_arg[[1]]` must have size 2, not size 0.

# list_all_size() and list_check_all_size() error on scalars

    Code
      (expect_error(list_all_size(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_all_size()`:
      ! `x[[1]]` must be a vector, not an environment.
    Code
      my_function <- (function(my_arg, size) list_check_all_size(my_arg, size))
      (expect_error(my_function(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg[[1]]` must be a vector, not an environment.

# list_all_size() and list_check_all_size() throw error using internal call on non-list input

    Code
      (expect_error(list_all_size(1, 2)))
    Output
      <error/rlang_error>
      Error in `list_all_size()`:
      ! `x` must be a list, not the number 1.
    Code
      (expect_error(list_check_all_size(1, 2, arg = "arg", call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `list_check_all_size()`:
      ! `x` must be a list, not the number 1.

# list_all_size() and list_check_all_size() validate `size`

    Code
      (expect_error(list_all_size(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_all_size()`:
      ! `size` must be a scalar integer or double.
    Code
      (expect_error(list_check_all_size(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_check_all_size()`:
      ! `size` must be a scalar integer or double.

# informative messages when 1d array doesn't match vector

    Code
      (expect_error(vec_assert(x, int())))
    Output
      <error/vctrs_error_assert_ptype>
      Error:
      ! `x` must be a vector with type <integer>.
      Instead, it has type <integer[1d]>.

[vctrs](README.md) / SearchResult

Defined in: [index.d.ts:7](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L7)

A single search result.

## Properties

| Property | Type | Description | Defined in |
| ------ | ------ | ------ | ------ |
| <a id="distance"></a> `distance` | `number` | Distance between the query and this vector. Lower is more similar for cosine and euclidean metrics. | [index.d.ts:14](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L14) |
| <a id="id"></a> `id` | `string` | The unique string identifier of the matched vector. | [index.d.ts:9](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L9) |
| <a id="metadata"></a> `metadata?` | `Record`\<`string`, `any`\> | The metadata object attached to this vector, or undefined. | [index.d.ts:16](https://github.com/yang-29/vctrs/blob/1ac8fc97d294178870f629cdcbed183d98989fb4/node/index.d.ts#L16) |

# cannot decrease dimensionality

    Code
      (expect_error(vec_cast(x, y), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `x` <table[,1,1]> to <table[,1]>.
      Can't decrease dimensionality from 3 to 2.

# Node.js — Getting Started

## Install

```bash
npm install @yang-29/vctrs
```

## Basic Usage

```typescript
import { VctrsDatabase } from "@yang-29/vctrs";

// Create a new database (or open existing)
const db = new VctrsDatabase("./mydb", 384, "cosine");

// Add vectors
db.add("doc1", [0.1, 0.2, ...], { title: "Hello" });

// Batch insert
db.addMany(ids, vectors, metadatas);

// Search
const results = db.search(queryVector, 10);
for (const r of results) {
  console.log(r.id, r.distance, r.metadata);
}

// Filtered search
const filtered = db.search(queryVec, 10, null, { category: "science" });

// Distance threshold
const close = db.search(queryVec, 10, null, null, 0.5);

// Save
db.save();
```

## Async Usage

All methods have async variants that run on the libuv thread pool:

```typescript
// Non-blocking — doesn't stall the event loop
await db.addAsync("doc1", vector, { title: "Hello" });
const results = await db.searchAsync(queryVector, 10);
await db.saveAsync();
```

## Opening Existing Databases

```typescript
// Auto-detects dim and metric from saved files
const db = new VctrsDatabase("./mydb");
console.log(db.dim, db.metric); // 384, "cosine"
```

## Configuration

| Parameter | Default | Description |
|-----------|---------|-------------|
| `dim` | required | Vector dimensionality |
| `metric` | `"cosine"` | `"cosine"`, `"euclidean"`/`"l2"`, `"dot"`/`"dot_product"` |
| `hnswM` | `16` | HNSW edges per node |
| `efConstruction` | `200` | Build-time search width |
| `quantize` | `false` | Enable SQ8 scalar quantization |

## Collections

Manage multiple isolated indexes under one directory:

```typescript
import { VctrsClient } from "@yang-29/vctrs";

const client = new VctrsClient("./data");

// Create collections with different configs
const movies = client.createCollection("movies", 384);
const docs = client.createCollection("docs", 768, "dot");

// Use them like regular databases
movies.add("m1", vector, { title: "Alien" });
movies.save();

// List, get, delete
client.listCollections();            // → ["docs", "movies"]
const db = client.getCollection("movies");
const items = client.getOrCreateCollection("items", 384);
client.deleteCollection("docs");
```

## Export / Import

Backup and restore databases as JSON:

```typescript
// Export to file
db.exportJson("backup.json");
db.exportJson("backup.json", true);  // pretty-print

// Import into existing database (upsert semantics)
db.importJson("backup.json");
```

The JSON format includes dimension, metric, and all vectors with metadata.

# print method gives human friendly output

    Code
      list_of(1, 2:3)
    Output
      <list_of<double>[2]>
      [[1]]
      [1] 1
      
      [[2]]
      [1] 2 3
      

---

    Code
      tibble::tibble(x = list_of(1, 2:3))
    Output
      # A tibble: 2 x 1
                  x
        <list<dbl>>
      1         [1]
      2         [2]

# str method is reasonably correct

    Code
      str(x)
    Output
      list<dbl> [1:2] 
      $ : num 1
      $ : num [1:2] 2 3
      @ ptype: num(0) 

---

    Code
      str(list(list(x, y = 2:1)))
    Output
      List of 1
       $ :List of 2
        ..$  : list<dbl> [1:2] 
        .. ..$ : num 1
        .. ..$ : num [1:2] 2 3
        .. ..@ ptype: num(0) 
        ..$ y: int [1:2] 2 1

---

    Code
      str(x[0])
    Output
      list<dbl> [1:0] 
       list()
      @ ptype: num(0) 

---

    Code
      str(list(list(x[0], y = 2:1)))
    Output
      List of 1
       $ :List of 2
        ..$  : list<dbl> [1:0] 
       list()
        .. ..@ ptype: num(0) 
        ..$ y: int [1:2] 2 1

# list coercions are symmetric and unchanging

    Code
      print(mat)
    Output
                         list   list_of<integer>   list_of<double>   list_of<character>  
      list               "list" "list"             "list"            "list"              
      list_of<integer>   "list" "list_of<integer>" "list_of<double>" "list"              
      list_of<double>    "list" "list_of<double>"  "list_of<double>" "list"              
      list_of<character> "list" "list"             "list"            "list_of<character>"

# error call is passed to inner cast methods

    Code
      (expect_error(fn1()))
    Output
      <error/vctrs_error_cast>
      Error in `fn1()`:
      ! Can't convert `..1` <double> to <character>.
    Code
      (expect_error(fn2()))
    Output
      <error/vctrs_error_cast>
      Error in `fn2()`:
      ! Can't convert `..1` <double> to <character>.

# failing common type reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `2` <double> and `chr()` <character>.

# failing cast reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `2` <double> to <character>.

---

    Code
      (expect_error(my_function(df1, df2)))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `lhs$y` <logical> to match type of `y` <character>.

---

    Code
      (expect_error(my_function(df1, df2)))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `lhs$y` <logical> to match type of `y` <character>.

# lossy cast reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `2` <double> to <logical> due to loss of precision.
      * Locations: 1

# failing common size reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle input of size 2 to size 10.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 10).

# unsupported error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_unsupported>
      Error in `dim<-`:
      ! `dim<-.vctrs_vctr()` not supported.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_unimplemented>
      Error in `median()`:
      ! `median.vctrs_vctr()` not implemented.

# scalar error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.

# size error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `1:2` must have size 1, not size 2.

# bare casts report correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `1.5` <double> to <logical> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `2L` <integer> to <logical> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `matrix(TRUE)` <double[,1]> to <double>.
      Can't decrease dimensionality from 2 to 1.

# base S3 casts report correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `"a"` <character> to <factor<9b7e3>> due to loss of generality.
      * Locations: 1

# names validation reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error in `my_function()`:
      ! Names can't be empty.
      x Empty name found at location 2.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_function()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
      i Use argument `repair` to specify repair strategy.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_function()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# subscript validation reports correct error calls

    Code
      (expect_error(my_function()))
    Output
      <error/rlang_error>
      Error in `vctrs::num_as_location()`:
      ! `missing` must be one of "propagate", "remove", or "error".

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements past the end.
      i Location 10 doesn't exist.
      i There are only 2 elements.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `my_arg`.
      x Can't convert from `my_arg` <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements.
      x Can't convert from <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(my_function(list())))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `my_arg`.
      x `my_arg` must be logical, numeric, or character, not an empty list.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_as_location()`:
      ! Can't convert from `n` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(NA)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has a missing value at location 1.

# `vec_ptype()` reports correct error call

    Code
      (expect_error(my_function(env())))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! Input must be a vector, not an environment.
    Code
      (expect_error(my_function(foobar(list()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! Input must be a vector, not a <vctrs_foobar> object.

# `vec_slice()` uses `error_call`

    Code
      (expect_error(my_function(env(), 1)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `x` must be a vector, not an environment.
    Code
      (expect_error(my_function(1, 2)))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# vec_slice() reports self in error context

    Code
      (expect_error(vec_slice(foobar(list()), 1)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_slice()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_slice(list(), env())))
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x `i` must be logical, numeric, or character, not an environment.

# list_sizes() reports error context

    Code
      (expect_error(list_sizes(foobar(list()))))
    Output
      <error/rlang_error>
      Error in `list_sizes()`:
      ! `x` must be a list, not a <vctrs_foobar> object.
    Code
      (expect_error(list_sizes(list(env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x[[1]]` must be a vector, not an environment.
    Code
      (expect_error(list_sizes(list(1, 2, env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x[[3]]` must be a vector, not an environment.
    Code
      (expect_error(list_sizes(list(1, 2, foo = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x$foo` must be a vector, not an environment.

# vec_size() reports error context

    Code
      (expect_error(vec_size(env())))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_size()`:
      ! `x` must be a vector, not an environment.

# vec_cast_common() reports error context

    Code
      (expect_error(my_function(my_arg = 1.5, .to = int())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `my_arg` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg")))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `my_arg$my_arg` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg$this_arg` <double> and `my_arg$that_arg` <character>.

---

    Code
      (expect_error(my_function(1, "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg[[1]]` <double> and `my_arg[[2]]` <character>.

---

    Code
      (expect_error(my_function(this_arg = x, that_arg = y)))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `this_arg$x` <character> and `that_arg$x` <double>.

# vec_ptype_common() reports error context

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `this_arg` <double> and `that_arg` <character>.

---

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg$this_arg` <double> and `my_arg$that_arg` <character>.

---

    Code
      (expect_error(my_function(1, "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg[[1]]` <double> and `my_arg[[2]]` <character>.

# Semantic Search

Search over documents using sentence embeddings.

=== "Python"

    ```python
    --8<-- "examples/semantic-search/search.py"
    ```

=== "TypeScript"

    ```typescript
    --8<-- "examples/semantic-search/search.ts"
    ```

## Run it

=== "Python"

    ```bash
    pip install vctrs sentence-transformers
    python examples/semantic-search/search.py
    ```

=== "TypeScript"

    ```bash
    npm install @yang-29/vctrs @xenova/transformers
    npx tsx examples/semantic-search/search.ts
    ```

# `na_equal` is validated

    Code
      vec_equal(1, 1, na_equal = 1)
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not the number 1.

---

    Code
      vec_equal(1, 1, na_equal = c(TRUE, FALSE))
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not a logical vector.

# can't supply NA as `na_equal`

    Code
      vec_equal(NA, NA, na_equal = NA)
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not `NA`.

# RAG (Retrieval-Augmented Generation)

Index documents, retrieve relevant context, generate an answer with an LLM.

=== "Python"

    ```python
    --8<-- "examples/rag/rag.py"
    ```

=== "TypeScript"

    ```typescript
    --8<-- "examples/rag/rag.ts"
    ```

## Run it

=== "Python"

    ```bash
    pip install vctrs sentence-transformers openai
    export OPENAI_API_KEY=sk-...
    python examples/rag/rag.py "What is HNSW?"
    ```

=== "TypeScript"

    ```bash
    npm install @yang-29/vctrs @xenova/transformers openai
    export OPENAI_API_KEY=sk-...
    npx tsx examples/rag/rag.ts "What is HNSW?"
    ```

# `vec_rep()` validates `times`

    Code
      (expect_error(my_vec_rep(1, "x"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `my_vec_rep()`:
      ! Can't convert `my_times` <character> to <integer>.
    Code
      (expect_error(my_vec_rep(1, c(1, 2))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` must be a single number.
    Code
      (expect_error(my_vec_rep(1, -1)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` must be a positive number.
    Code
      (expect_error(my_vec_rep(1, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` can't be missing.

---

    Code
      my_vec_rep(1, "x")
    Condition
      Error in `my_vec_rep()`:
      ! Can't convert `my_times` <character> to <integer>.

---

    Code
      my_vec_rep(1, c(1, 2))
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` must be a single number.

---

    Code
      my_vec_rep(1, -1)
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` must be a positive number.

---

    Code
      my_vec_rep(1, NA_integer_)
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` can't be missing.

# `vec_rep_each()` validates `times`

    Code
      (expect_error(my_vec_rep_each(1, "x"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `my_vec_rep_each()`:
      ! Can't convert `my_times` <character> to <integer>.
    Code
      (expect_error(my_vec_rep_each(1, -1)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 1 is negative.
    Code
      (expect_error(my_vec_rep_each(c(1, 2), c(1, -1))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 2 is negative.
    Code
      (expect_error(my_vec_rep_each(1, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 1 is missing.
    Code
      (expect_error(my_vec_rep_each(c(1, 2), c(1, NA_integer_))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 2 is missing.

---

    Code
      my_vec_rep_each(1, "x")
    Condition
      Error in `my_vec_rep_each()`:
      ! Can't convert `my_times` <character> to <integer>.

---

    Code
      my_vec_rep_each(1, -1)
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 1 is negative.

---

    Code
      my_vec_rep_each(c(1, 2), c(1, -1))
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 2 is negative.

---

    Code
      my_vec_rep_each(1, NA_integer_)
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 1 is missing.

---

    Code
      my_vec_rep_each(c(1, 2), c(1, NA_integer_))
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 2 is missing.

# `vec_rep_each()` uses recyclying errors

    Code
      (expect_error(my_vec_rep_each(1:2, 1:3), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_vec_rep_each()`:
      ! Can't recycle `my_times` (size 3) to size 2.

---

    Code
      my_vec_rep_each(1:2, 1:3)
    Condition
      Error in `my_vec_rep_each()`:
      ! Can't recycle `my_times` (size 3) to size 2.

# errors on scalars

    Code
      vec_unrep(environment())
    Condition
      Error in `vec_unrep()`:
      ! `x` must be a vector, not an environment.

# data.frames have good default abbr and full methods

    Code
      df <- foobar(data.frame(x = 1, y = "", z = TRUE))
      vec_ptype_abbr(df)
    Output
      [1] "vctrs_fb[,3]"
    Code
      vec_ptype_full(df)
    Output
      [1] "vctrs_foobar<\n  x: double\n  y: character\n  z: logical\n>"

# common type failure uses error call and error arg (#1641, #1692)

    Code
      vec_c("x", 1, .error_call = call("foo"), .error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <character> and `arg[[2]]` <double>.

---

    Code
      vec_c("x", .ptype = integer(), .error_call = call("foo"), .error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't convert `arg[[1]]` <character> to <integer>.

# common type failure uses positional errors

    Code
      (expect_error(vec_c(1, a = "x", 2)))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <double> and `a` <character>.
    Code
      (expect_error(vec_c(1, a = "x", 2, .ptype = double(), .error_arg = "arg")))
    Output
      <error/vctrs_error_cast>
      Error in `vec_c()`:
      ! Can't convert `arg$a` <character> to <double>.
    Code
      (expect_error(vec_c(1, a = 2.5, .ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_c()`:
      ! Can't convert from `a` <double> to <integer> due to loss of precision.
      * Locations: 1

# vec_c() includes index in argument tag

    Code
      vec_c(df1, df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `..1$x$y$z` <double> and `..2$x$y$z` <character>.

---

    Code
      vec_c(df1, df1, df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `..1$x$y$z` <double> and `..3$x$y$z` <character>.

---

    Code
      vec_c(foo = df1, bar = df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `foo$x$y$z` <double> and `bar$x$y$z` <character>.

# vec_c() can repair names quietly

    Code
      res_unique <- vec_c(x = TRUE, x = 0, .name_repair = "unique_quiet")
      res_universal <- vec_c(`if` = TRUE, `in` = 0, .name_repair = "universal_quiet")

# vec_c() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <vctrs_foobar> and `..2` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(vec_c(x, y, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_foobar> and `arg[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_c() fails with complex foreign S4 classes

    Code
      joe <- .Counts(c(1L, 2L), name = "Joe")
      jane <- .Counts(3L, name = "Jane")
      (expect_error(vec_c(joe, jane), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <vctrs_Counts> and `..2` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(vec_c(joe, jane, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_Counts> and `arg[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_c() fallback doesn't support `name_spec` or `ptype`

    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .name_spec = "{outer}_{inner}")),
      "name specification"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .ptype = "")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error in `vec_c()`:
      ! Can't convert `..1` <vctrs_foobar> to <character>.
    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .error_call = call(
        "foo"), .name_spec = "{outer}_{inner}"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.

# can ignore names in `vec_c()` by providing a `zap()` name-spec (#232)

    Code
      (expect_error(vec_c(a = c(b = letters), b = 1, .name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `a` <character> and `b` <double>.

# concatenation performs expected allocations

    Code
      ints <- rep(list(1L), 100)
      dbls <- rep(list(1), 100)
      # # `vec_c()` 
      # Integers
      with_memory_prof(vec_c_list(ints))
    Output
      [1] 1.96KB
    Code
      # Doubles
      with_memory_prof(vec_c_list(dbls))
    Output
      [1] 2.35KB
    Code
      # Integers to integer
      with_memory_prof(vec_c_list(ints, ptype = int()))
    Output
      [1] 1.7KB
    Code
      # Doubles to integer
      with_memory_prof(vec_c_list(dbls, ptype = int()))
    Output
      [1] 1.7KB
    Code
      # # `list_unchop()` 
      # Integers
      with_memory_prof(list_unchop(ints))
    Output
      [1] 1.13KB
    Code
      # Doubles
      with_memory_prof(list_unchop(dbls))
    Output
      [1] 1.52KB
    Code
      # Integers to integer
      with_memory_prof(list_unchop(ints, ptype = int()))
    Output
      [1] 896B
    Code
      # Doubles to integer
      with_memory_prof(list_unchop(dbls, ptype = int()))
    Output
      [1] 896B
    Code
      # # Concatenation with names
      # Named integers
      ints <- rep(list(set_names(1:3, letters[1:3])), 100)
      with_memory_prof(list_unchop(ints))
    Output
      [1] 4.3KB
    Code
      # Named matrices
      mat <- matrix(1:4, 2, dimnames = list(c("foo", "bar")))
      mats <- rep(list(mat), 100)
      with_memory_prof(list_unchop(mats))
    Output
      [1] 5.52KB
    Code
      # Data frame with named columns
      df <- data_frame(x = set_names(as.list(1:2), c("a", "b")), y = set_names(1:2, c(
        "A", "B")), z = data_frame(Z = set_names(1:2, c("Za", "Zb"))))
      dfs <- rep(list(df), 100)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 8.79KB
    Code
      # Data frame with rownames (non-repaired, non-recursive case)
      df <- data_frame(x = 1:2)
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 6.02KB
    Code
      # Data frame with rownames (repaired, non-recursive case)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 12.2KB
    Code
      # Data frame with rownames (non-repaired, recursive case) (#1217)
      df <- data_frame(x = 1:2, y = data_frame(x = 1:2))
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 11.4KB
    Code
      # Data frame with rownames (repaired, recursive case) (#1217)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 23.7KB
    Code
      # list-ofs (#1496)
      make_list_of <- (function(n) {
        df <- tibble::tibble(x = new_list_of(vec_chop(1:n), ptype = integer()))
        vec_chop(df)
      })
      with_memory_prof(list_unchop(make_list_of(1000)))
    Output
      [1] 103KB
    Code
      with_memory_prof(list_unchop(make_list_of(2000)))
    Output
      [1] 205KB
    Code
      with_memory_prof(list_unchop(make_list_of(4000)))
    Output
      [1] 408KB

# `missing` is validated

    Code
      (expect_error(vec_interval_locate_groups(1, 2, missing = "s")))
    Output
      <error/rlang_error>
      Error in `vec_interval_locate_groups()`:
      ! `missing` must be either "group" or "drop".

---

    Code
      (expect_error(vec_interval_locate_groups(1, 2, missing = c("group", "drop"))))
    Output
      <error/rlang_error>
      Error in `vec_interval_locate_groups()`:
      ! `missing` must be a string.

# common type is taken

    Code
      (expect_error(vec_interval_locate_groups(1, "x")))
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `start` <double> and `end` <character>.

---

    Code
      (expect_error(vec_interval_locate_containers(1, "x")))
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `start` <double> and `end` <character>.

# `lower` and `upper` can't contain missing values

    Code
      (expect_error(vec_interval_complement(1, 2, lower = NA)))
    Output
      <error/rlang_error>
      Error in `vec_interval_complement()`:
      ! `lower` can't contain missing values.
    Code
      (expect_error(vec_interval_complement(1, 2, upper = NA)))
    Output
      <error/rlang_error>
      Error in `vec_interval_complement()`:
      ! `upper` can't contain missing values.
    Code
      start <- data_frame(x = 1, y = 2)
      end <- data_frame(x = 1, y = 3)
      (expect_error(vec_interval_complement(start, end, lower = data_frame(x = 1, y = NA)))
      )
    Output
      <error/rlang_error>
      Error in `vec_interval_complement()`:
      ! `lower` can't contain missing values.
    Code
      (expect_error(vec_interval_complement(start, end, upper = data_frame(x = 1, y = NA)))
      )
    Output
      <error/rlang_error>
      Error in `vec_interval_complement()`:
      ! `upper` can't contain missing values.

# errors nicely if common type can't be taken

    Code
      vec_set_intersect(1, "x")
    Condition
      Error in `vec_set_intersect()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      vec_set_difference(1, "x")
    Condition
      Error in `vec_set_difference()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      vec_set_union(1, "x")
    Condition
      Error in `vec_set_union()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      vec_set_symmetric_difference(1, "x")
    Condition
      Error in `vec_set_symmetric_difference()`:
      ! Can't combine `x` <double> and `y` <character>.

# dots must be empty

    Code
      vec_set_intersect(1, 2, 3)
    Condition
      Error in `vec_set_intersect()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 3
      i Did you forget to name an argument?

---

    Code
      vec_set_difference(1, 2, 3)
    Condition
      Error in `vec_set_difference()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 3
      i Did you forget to name an argument?

---

    Code
      vec_set_union(1, 2, 3)
    Condition
      Error in `vec_set_union()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 3
      i Did you forget to name an argument?

---

    Code
      vec_set_symmetric_difference(1, 2, 3)
    Condition
      Error in `vec_set_symmetric_difference()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 3
      i Did you forget to name an argument?

# `ptype` is respected

    Code
      vec_set_intersect(1, 1.5, ptype = integer())
    Condition
      Error in `vec_set_intersect()`:
      ! Can't convert from `y` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_set_difference(1, 1.5, ptype = integer())
    Condition
      Error in `vec_set_difference()`:
      ! Can't convert from `y` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_set_union(1, 1.5, ptype = integer())
    Condition
      Error in `vec_set_union()`:
      ! Can't convert from `y` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_set_symmetric_difference(1, 1.5, ptype = integer())
    Condition
      Error in `vec_set_symmetric_difference()`:
      ! Can't convert from `y` <double> to <integer> due to loss of precision.
      * Locations: 1

# `x_arg` and `y_arg` can be adjusted

    Code
      vec_set_intersect(1, "2", x_arg = "foo", y_arg = "bar")
    Condition
      Error in `vec_set_intersect()`:
      ! Can't combine `foo` <double> and `bar` <character>.

---

    Code
      vec_set_difference(1, "2", x_arg = "foo", y_arg = "bar")
    Condition
      Error in `vec_set_difference()`:
      ! Can't combine `foo` <double> and `bar` <character>.

---

    Code
      vec_set_union(1, "2", x_arg = "foo", y_arg = "bar")
    Condition
      Error in `vec_set_union()`:
      ! Can't combine `foo` <double> and `bar` <character>.

---

    Code
      vec_set_symmetric_difference(1, "2", x_arg = "foo", y_arg = "bar")
    Condition
      Error in `vec_set_symmetric_difference()`:
      ! Can't combine `foo` <double> and `bar` <character>.

---

    Code
      vec_set_intersect(1, "2", x_arg = "", y_arg = "")
    Condition
      Error in `vec_set_intersect()`:
      ! Can't combine <double> and <character>.

# `error_call` can be adjusted

    Code
      my_set_intersect()
    Condition
      Error in `my_set_intersect()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      my_set_difference()
    Condition
      Error in `my_set_difference()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      my_set_union()
    Condition
      Error in `my_set_union()`:
      ! Can't combine `x` <double> and `y` <character>.

---

    Code
      my_set_symmetric_difference()
    Condition
      Error in `my_set_symmetric_difference()`:
      ! Can't combine `x` <double> and `y` <character>.

# error is thrown when comparing complexes (#1655)

    Code
      (expect_error(vec_compare(complex(), complex())))
    Output
      <error/rlang_error>
      Error in `vec_compare()`:
      ! Can't compare complexes.

# `na_equal` is validated

    Code
      (expect_error(vec_compare(1, 1, na_equal = 1)))
    Output
      <error/rlang_error>
      Error in `vec_compare()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not the number 1.
    Code
      (expect_error(vec_compare(1, 1, na_equal = c(TRUE, FALSE))))
    Output
      <error/rlang_error>
      Error in `vec_compare()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not a logical vector.

# can't supply NA as `na_equal`

    Code
      vec_compare(NA, NA, na_equal = NA)
    Condition
      Error in `vec_compare()`:
      ! `na_equal` must be `TRUE` or `FALSE`, not `NA`.

# vec_match() and vec_in() check types

    Code
      df1 <- data_frame(x = data_frame(foo = 1))
      df2 <- data_frame(x = data_frame(foo = ""))
      (expect_error(vec_match(df1, df2), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_match()`:
      ! Can't combine `x$foo` <double> and `x$foo` <character>.
    Code
      (expect_error(vec_match(df1, df2, needles_arg = "n", haystack_arg = "h"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_match()`:
      ! Can't combine `n$x$foo` <double> and `h$x$foo` <character>.
    Code
      (expect_error(vec_in(df1, df2), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_in()`:
      ! Can't combine `x$foo` <double> and `x$foo` <character>.
    Code
      (expect_error(vec_in(df1, df2, needles_arg = "n", haystack_arg = "h"), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_in()`:
      ! Can't combine `n$x$foo` <double> and `h$x$foo` <character>.

# `direction` is recycled right with array columns (#1753)

    Code
      vec_order_radix(df, direction = c("asc", "desc", "desc"))
    Condition
      Error:
      ! `direction` should have length 1 or length equal to the number of columns of `x` when `x` is a data frame.

# `na_value` is recycled right with array columns (#1753)

    Code
      vec_order_radix(df, direction = c("smallest", "largest", "largest"))
    Condition
      Error:
      ! `direction` must contain only "asc" or "desc".

# dots must be empty (#1647)

    Code
      vec_order(1, 2)
    Condition
      Error in `vec_order()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 2
      i Did you forget to name an argument?

---

    Code
      vec_sort(1, 2)
    Condition
      Error in `vec_sort()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 2
      i Did you forget to name an argument?

# `indices` are validated

    Code
      vec_chop(1, indices = 1)
    Condition
      Error:
      ! `indices` must be a list of index values, or `NULL`.

---

    Code
      (expect_error(vec_chop(1, indices = list(1.5)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Can't convert from <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(vec_chop(1, indices = list(2)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# `sizes` are validated

    Code
      vec_chop("a", sizes = "a")
    Condition
      Error:
      ! Can't convert `sizes` <character> to <integer>.

---

    Code
      vec_chop("a", sizes = 2)
    Condition
      Error:
      ! `sizes` can't contain sizes larger than 1.

---

    Code
      vec_chop("a", sizes = -1)
    Condition
      Error:
      ! `sizes` can't contain negative sizes.

---

    Code
      vec_chop("a", sizes = NA_integer_)
    Condition
      Error:
      ! `sizes` can't contain missing values.

---

    Code
      vec_chop("a", sizes = c(1, 1))
    Condition
      Error:
      ! `sizes` must sum to size 1, not size 2.

# can't use both `indices` and `sizes`

    Code
      vec_chop(1, indices = list(1), sizes = 1)
    Condition
      Error:
      ! Can't supply both `indices` and `sizes`.

# `vec_chop(x, indices)` backwards compatible behavior works

    Code
      vec_chop(1:2, 1)
    Condition
      Error:
      ! `indices` must be a list of index values, or `NULL`.

---

    Code
      vec_chop(1, list(1), sizes = 1)
    Condition
      Error:
      ! Can't supply both `indices` and `sizes`.

---

    Code
      vec_chop(1, list(1), 2)
    Condition
      Error in `vec_chop()`:
      ! `...` must be empty.
      x Problematic arguments:
      * ..1 = list(1)
      * ..2 = 2
      i Did you forget to name an argument?

---

    Code
      vec_chop(1, list(1), indices = list(1))
    Condition
      Error in `vec_chop()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = list(1)
      i Did you forget to name an argument?

# `x` must be a list

    Code
      list_unchop(1, indices = list(1))
    Condition
      Error in `list_unchop()`:
      ! `x` must be a list, not the number 1.

---

    Code
      list_unchop(1, indices = list(1), error_call = call("foo"), error_arg = "arg")
    Condition
      Error in `foo()`:
      ! `arg` must be a list, not the number 1.

---

    Code
      list_unchop(data.frame(x = 1), indices = list(1))
    Condition
      Error in `list_unchop()`:
      ! `x` must be a list, not a <data.frame> object.

# `indices` must be a list

    Code
      list_unchop(list(1), indices = 1)
    Condition
      Error in `list_unchop()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_unchop(list(1), indices = 1, error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_unchop(list(1), indices = data.frame(x = 1))
    Condition
      Error in `list_unchop()`:
      ! `indices` must be a list, not a <data.frame> object.

# unchopping recycles elements of x to the size of the index

    Code
      (expect_error(list_unchop(x, indices = indices)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.
    Code
      (expect_error(list_unchop(x, indices = indices, error_call = call("foo"),
      error_arg = "arg")))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

# unchopping takes the common type

    Code
      (expect_error(list_unchop(x, indices = indices), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x[[2]]` <character>.
    Code
      (expect_error(list_unchop(x, indices = indices, error_call = call("foo"),
      error_arg = "arg"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <double> and `arg[[2]]` <character>.

# common type failure uses positional errors

    Code
      x <- list(1, a = "x", 2)
      (expect_error(list_unchop(x)))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x$a` <character>.
    Code
      (expect_error(list_unchop(x, indices = list(2, 1, 3))))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x$a` <character>.
    Code
      (expect_error(list_unchop(x, ptype = double())))
    Output
      <error/vctrs_error_cast>
      Error in `list_unchop()`:
      ! Can't convert `x$a` <character> to <double>.
    Code
      (expect_error(list_unchop(x, indices = list(2, 1, 3), ptype = double())))
    Output
      <error/vctrs_error_cast>
      Error in `list_unchop()`:
      ! Can't convert `x$a` <character> to <double>.
    Code
      y <- list(1, a = 2.5)
      (expect_error(list_unchop(y, ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x$a` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(list_unchop(y, indices = list(2, 1), ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x$a` <double> to <integer> due to loss of precision.
      * Locations: 1

# can specify a ptype to override common type

    Code
      (expect_error(list_unchop(x, indices = indices, ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(list_unchop(x, indices = indices, ptype = integer(), error_call = call(
        "foo"), error_arg = "arg")))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `foo()`:
      ! Can't convert from `arg[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1

# list_unchop() can repair names quietly

    Code
      res <- list_unchop(vec_chop(x, indices = indices), indices = indices,
      name_repair = "unique_quiet")

---

    Code
      res <- list_unchop(vec_chop(x, indices = indices), indices = indices,
      name_repair = "universal_quiet")

# list_unchop() errors on unsupported location values

    Code
      (expect_error(list_unchop(list(1, 2), indices = list(c(1, 2), 0)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.
    Code
      (expect_error(list_unchop(list(1), indices = list(-1)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

# list_unchop() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(list_unchop(list(x, y)), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(list_unchop(list(x, y), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_foobar> and `arg[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_unchop() fails with complex foreign S4 classes

    Code
      joe <- .Counts(c(1L, 2L), name = "Joe")
      jane <- .Counts(3L, name = "Jane")
      (expect_error(list_unchop(list(joe, jane)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(list_unchop(list(joe, jane), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_Counts> and `arg[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_unchop() falls back to c() if S3 method is available

    Code
      (expect_error(list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 3),
      integer())), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

---

    Code
      x <- list(foobar(1:2))
      indices <- list(1:3)
      (expect_error(list_unchop(x, indices = indices)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.
    Code
      (expect_error(list_unchop(x, indices = indices, error_arg = "arg", error_call = call(
        "foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

# list_unchop() falls back for S4 classes with a registered c() method

    Code
      (expect_error(list_unchop(list(joe, 1, jane), indices = list(c(1, 2), 3, 4)),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <double>.

# list_unchop() fallback doesn't support `name_spec` or `ptype`

    Code
      foo <- structure(foobar(1), foo = "foo")
      bar <- structure(foobar(2), bar = "bar")
      (expect_error(with_c_foobar(list_unchop(list(foo, bar), name_spec = "{outer}_{inner}")),
      "name specification"))
    Output
      <error/rlang_error>
      Error in `list_unchop()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      (expect_error(with_c_foobar(list_unchop(list(foo, bar), name_spec = "{outer}_{inner}",
      error_call = call("foo"))), "name specification"))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      x <- list(foobar(1))
      (expect_error(with_c_foobar(list_unchop(x, ptype = "")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error in `list_unchop()`:
      ! Can't convert `x[[1]]` <vctrs_foobar> to <character>.

# list_unchop() does not support non-numeric S3 indices

    Code
      (expect_error(list_unchop(list(1), indices = list(factor("x"))), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not the string "x".
    Code
      (expect_error(list_unchop(list(1), indices = list(foobar(1L))), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a <vctrs_foobar> object.

# can ignore names in `list_unchop()` by providing a `zap()` name-spec (#232)

    Code
      (expect_error(list_unchop(list(a = c(b = 1:2)))))
    Output
      <error/rlang_error>
      Error in `list_unchop()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(list_unchop(list(a = c(b = 1:2)), error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `list_unchop()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.

---

    Code
      x <- list(a = c(b = letters), b = 3L)
      (expect_error(list_unchop(x, name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x$a` <character> and `x$b` <integer>.
    Code
      x <- list(a = c(foo = 1:2), b = c(bar = ""))
      (expect_error(list_unchop(x, indices = list(2:1, 3), name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x$a` <integer> and `x$b` <character>.

# incompatible type error validates `action`

    Code
      (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = "c"))
      )
    Output
      <error/rlang_error>
      Error in `stop_incompatible_type()`:
      ! `action` must be one of "combine" or "convert", not "c".
    Code
      (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = 1)))
    Output
      <error/rlang_error>
      Error in `stop_incompatible_type()`:
      ! `action` must be a character vector, not the number 1.

# can override arg in OOB conditions

    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), NULL),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), quote(
        foo)), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), quote(
        foo(bar))), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.

# scalar type errors are informative

    Code
      (expect_error(vec_slice(foobar(list(1)), 1), class = "vctrs_error_scalar_type"))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_slice()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
    Code
      (expect_error(stop_scalar_type(foobar(list(1)), arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.

# empty names errors are informative

    Code
      (expect_error(vec_as_names(c("x", "", "y"), repair = "check_unique"), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty name found at location 2.
    Code
      (expect_error(vec_as_names(c("x", "", "y", ""), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty names found at locations 2 and 4.
    Code
      (expect_error(vec_as_names(rep("", 10), repair = "check_unique"), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty names found at locations 1, 2, 3, 4, 5, etc.

# dot dot names errors are informative

    Code
      (expect_error(vec_as_names(c("..1", "..1", "..1", "...", "z"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_dot_dot"))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at locations 1, 2, and 3.
        * "..." at location 4.
    Code
      (expect_error(vec_as_names(c(rep("..1", 20), rep(c("..2", "..3", "..4", "...",
        "..5"), 2)), repair = "check_unique"), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at locations 1, 2, 3, 4, 5, etc.
        * "..2" at locations 21 and 26.
        * "..3" at locations 22 and 27.
        * "..4" at locations 23 and 28.
        * "..." at locations 24 and 29.
        * ...

# unique names errors are informative

    Code
      (expect_error(vec_as_names(c("x", "x", "x", "y", "y", "z"), repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1, 2, and 3.
        * "y" at locations 4 and 5.
    Code
      (expect_error(vec_as_names(c(rep("x", 20), rep(c("a", "b", "c", "d", "e"), 2)),
      repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1, 2, 3, 4, 5, etc.
        * "a" at locations 21 and 26.
        * "b" at locations 22 and 27.
        * "c" at locations 23 and 28.
        * "d" at locations 24 and 29.
        * ...

# lossy cast from character to factor mentions loss of generality

    Code
      (expect_error(vec_cast("a", factor("b")), class = "vctrs_error_cast_lossy"))
    Output
      <error/vctrs_error_cast_lossy>
      Error:
      ! Can't convert from `"a"` <character> to <factor<9b7e3>> due to loss of generality.
      * Locations: 1

# lossy cast `conditionMessage()` result matches `cnd_message()` (#1592)

    Code
      cat(conditionMessage(cnd))
    Output
      Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

# ordered cast failures mention conversion

    Code
      (expect_error(vec_cast(ordered("x"), ordered("y")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `ordered("x")` <ordered<bf275>> to <ordered<fd1ad>>.

# incompatible size errors

    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "")))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 3.
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo),
      y_arg = "")))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `foo` (size 2) to size 3.
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "bar"))
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to match `bar` (size 3).
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo),
      y_arg = quote(bar))))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `foo` (size 2) to match `bar` (size 3).

# can't cast vector to tibble

    Code
      local_error_call(call("my_function"))
      (expect_error(vec_ptype2(v, dt), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `v` <logical> and `dt` <tbl_df>.
    Code
      (expect_error(vec_ptype2(dt, v), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `dt` <tbl_df> and `v` <logical>.
    Code
      (expect_error(vec_cast(v, dt), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `v` <logical> to <tbl_df>.

# can use ptype2 and cast with tibble that has incorrect class vector

    Code
      local_error_call(call("my_function"))
      (expect_error(vec_cast(tib1, tib2), class = "vctrs_error_cast"))
    Output
      <error/vctrs_error_cast_lossy_dropped>
      Error in `my_function()`:
      ! Can't convert from `tib1` <tbl_df<x:double>> to <tbl_df<y:double>> due to loss of precision.
    Code
      (expect_error(vec_cast(tib1, data.frame(y = 2)), class = "vctrs_error_cast"))
    Output
      <error/vctrs_error_cast_lossy_dropped>
      Error in `my_function()`:
      ! Can't convert from `tib1` <tbl_df<x:double>> to <data.frame<y:double>> due to loss of precision.
    Code
      (expect_error(vec_cast(data.frame(x = 1), tib2), class = "vctrs_error_cast"))
    Output
      <error/vctrs_error_cast_lossy_dropped>
      Error in `my_function()`:
      ! Can't convert from `data.frame(x = 1)` <data.frame<x:double>> to <tbl_df<y:double>> due to loss of precision.

```{r, child = "../setup.Rmd", include = FALSE}
```

Recycling describes the concept of repeating elements of one vector to match the size of another. There are two rules that underlie the "tidyverse" recycling rules:

- Vectors of size 1 will be recycled to the size of any other vector

- Otherwise, all vectors must have the same size

# Examples

```{r, warning = FALSE, message = FALSE, include = FALSE}
library(tibble)
```

Vectors of size 1 are recycled to the size of any other vector:

```{r}
tibble(x = 1:3, y = 1L)
```

This includes vectors of size 0:

```{r}
tibble(x = integer(), y = 1L)
```

If vectors aren't size 1, they must all be the same size. Otherwise, an error is thrown:

```{r, error = TRUE}
tibble(x = 1:3, y = 4:7)
```

# vctrs backend

Packages in r-lib and the tidyverse generally use [vec_size_common()] and [vec_recycle_common()] as the backends for handling recycling rules.

- `vec_size_common()` returns the common size of multiple vectors, after applying the recycling rules

- `vec_recycle_common()` goes one step further, and actually recycles the vectors to their common size

```{r, error = TRUE}
vec_size_common(1:3, "x")

vec_recycle_common(1:3, "x")

vec_size_common(1:3, c("x", "y"))
```

# Base R recycling rules

The recycling rules described here are stricter than the ones generally used by base R, which are:

- If any vector is length 0, the output will be length 0

- Otherwise, the output will be length `max(length_x, length_y)`, and a warning will be thrown if the length of the longer vector is not an integer multiple of the length of the shorter vector.

We explore the base R rules in detail in `vignette("type-size")`.

# datetime coercions are symmetric and unchanging

    Code
      print(mat)
    Output
                           date                   datetime<local>        datetime<US/Central>   POSIXlt<US/Central>    duration<mins>   duration<secs>  
      date                 "date"                 "datetime<local>"      "datetime<US/Central>" "datetime<US/Central>" NA               NA              
      datetime<local>      "datetime<local>"      "datetime<local>"      "datetime<US/Central>" "datetime<US/Central>" NA               NA              
      datetime<US/Central> "datetime<US/Central>" "datetime<US/Central>" "datetime<US/Central>" "datetime<US/Central>" NA               NA              
      POSIXlt<US/Central>  "datetime<US/Central>" "datetime<US/Central>" "datetime<US/Central>" "datetime<US/Central>" NA               NA              
      duration<mins>       NA                     NA                     NA                     NA                     "duration<mins>" "duration<secs>"
      duration<secs>       NA                     NA                     NA                     NA                     "duration<secs>" "duration<secs>"

# data table has formatting methods

    Code
      dt <- data.table(x = 1, y = 2, z = 3)
      vec_ptype_abbr(dt)
    Output
      [1] "dt[,3]"
    Code
      vec_ptype_full(dt)
    Output
      [1] "data.table<\n  x: double\n  y: double\n  z: double\n>"

# na.fail() works

    Code
      na.fail(x)
    Condition
      Error in `na.fail()`:
      ! missing values in object

# default print and str methods are useful

    Code
      h
    Output
      <hidden[4]>
      [1] xxx xxx xxx xxx

---

    Code
      h[0]
    Output
      <hidden[0]>

---

    Code
      str(h)
    Output
       hidden [1:4] xxx, xxx, xxx, xxx

# default print method shows names

    Code
      h
    Output
      <hidden[3]>
        A   B   C 
      xxx xxx xxx 

# can customise subscript errors

    Code
      (expect_error(with_tibble_cols(vec_as_subscript(env())), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x `foo(bar)` must be logical, numeric, or character, not an environment.

---

    Code
      (expect_error(with_dm_tables(vec_as_subscript(env())), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract tables with `foo(bar)`.
      x `foo(bar)` must be logical, numeric, or character, not an environment.

# vec_as_subscript() checks dimensionality

    Code
      (expect_error(vec_as_subscript(matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be a simple vector, not a matrix.
    Code
      (expect_error(vec_as_subscript(array(TRUE, dim = c(1, 1, 1))), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be a simple vector, not an array.
    Code
      (expect_error(with_tibble_rows(vec_as_subscript(matrix(TRUE, nrow = 1))),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't remove rows with `foo(bar)`.
      x Subscript `foo(bar)` must be a simple vector, not a matrix.

# vec_as_subscript() forbids subscript types

    Code
      vec_as_subscript(1L, logical = "error", numeric = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be character, not the number 1.

---

    Code
      vec_as_subscript("foo", logical = "error", character = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not the string "foo".

---

    Code
      vec_as_subscript(TRUE, logical = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be numeric or character, not `TRUE`.

---

    Code
      vec_as_subscript("foo", character = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be logical or numeric, not the string "foo".

---

    Code
      vec_as_subscript(NULL, numeric = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be logical or character, not `NULL`.

---

    Code
      vec_as_subscript(quote(foo), character = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be logical or numeric, not a symbol.

# vec_as_subscript2() forbids subscript types

    Code
      vec_as_subscript2(1L, numeric = "error")
    Condition
      Error:
      ! Can't extract element.
      x Subscript must be character, not the number 1.

---

    Code
      vec_as_subscript2("foo", character = "error")
    Condition
      Error:
      ! Can't extract element.
      x Subscript must be numeric, not the string "foo".

---

    Code
      vec_as_subscript2(TRUE)
    Condition
      Error:
      ! Can't extract element.
      x Subscript must be numeric or character, not `TRUE`.

# vec_as_subscript2() retains the call when throwing vec_as_subscript() errors (#1605)

    Code
      vec_as_subscript2(1L, numeric = "error", call = call("foo"))
    Condition
      Error in `foo()`:
      ! Can't extract element.
      x Subscript must be character, not the number 1.

---

    Code
      vec_as_subscript2(1.5, call = call("foo"))
    Condition
      Error in `foo()`:
      ! Can't extract element.
      x Can't convert from <double> to <integer> due to loss of precision.

# vec_as_subscript2() retains the call when erroring on logical input (#1605)

    Code
      vec_as_subscript2(TRUE, call = call("foo"))
    Condition
      Error in `foo()`:
      ! Can't extract element.
      x Subscript must be numeric or character, not `TRUE`.

# `logical = 'cast'` is deprecated

    Code
      vec_as_subscript2(TRUE, logical = "cast")
    Condition
      Error in `vec_as_subscript2()`:
      ! `vctrs::vec_as_subscript2(logical = 'cast')` is deprecated.

---

    Code
      vec_as_subscript2(TRUE, logical = "error")
    Condition
      Error:
      ! Can't extract element.
      x Subscript must be numeric or character, not `TRUE`.

# lossy cast errors for scalar subscripts work (#1606)

    Code
      vec_as_subscript2(1.5)
    Condition
      Error:
      ! Can't extract element.
      x Can't convert from <double> to <integer> due to loss of precision.

# incompatible columns throws common type error

    Code
      (expect_error(vec_rbind(x_int, x_chr), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1$x` <integer> and `..2$x` <character>.
    Code
      (expect_error(vec_rbind(x_int, x_chr, .error_call = call("foo")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `..1$x` <integer> and `..2$x` <character>.
    Code
      (expect_error(vec_rbind(x_int, x_chr, .ptype = x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `foo()`:
      ! Can't convert `..1$x` <integer> to match type of `x` <character>.

# names are supplied if needed

    Code
      out <- vec_rbind(data_frame(...1 = 1), 1)
    Message
      New names:
      * `` -> `...1`

# can repair names in `vec_rbind()` (#229)

    Code
      (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"none"`.
      It must be one of `"unique"`, `"universal"`, or `"check_unique"`.
    Code
      (expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"minimal"`.
      It must be one of `"unique"`, `"universal"`, or `"check_unique"`.
    Code
      (expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_rbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.

# can repair names quietly

    Code
      res_unique <- vec_rbind(c(x = 1, x = 2), c(x = 3, x = 4), .name_repair = "unique_quiet")
      res_universal <- vec_rbind(c(`if` = 1, `in` = 2), c(`if` = 3, `for` = 4),
      .name_repair = "universal_quiet")

---

    Code
      res_unique <- vec_cbind(x = 1, x = 2, .name_repair = "unique_quiet")
      res_universal <- vec_cbind(`if` = 1, `in` = 2, .name_repair = "universal_quiet")

# vec_rbind() fails with arrays of dimensionality > 3

    Code
      (expect_error(vec_rbind(array(NA, c(1, 1, 1)))))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_rbind(array(NA, c(1, 1, 1)), .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't bind arrays.

# can assign row names in vec_rbind()

    Code
      (expect_error(vec_rbind(foo = df1, df2, .names_to = NULL), "specification"))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.

# vec_cbind() reports error context

    Code
      (expect_error(vec_cbind(foobar(list()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_cbind()`:
      ! `..1` must be a vector, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_cbind(foobar(list()), .error_call = call("foo"))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `foo()`:
      ! `..1` must be a vector, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_cbind(a = 1:2, b = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_cbind()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).
    Code
      (expect_error(vec_cbind(a = 1:2, b = int(), .error_call = call("foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).

# duplicate names are de-deduplicated

    Code
      (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
        x...1 x...2
      1     1     1
    Code
      (expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1",
        "x...2")))
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
        x...1 x...2
      1     1     1

# can repair names in `vec_cbind()` (#227)

    Code
      (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"),
      "can't be `\"none\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"none"`.
      It must be one of `"unique"`, `"universal"`, `"check_unique"`, or `"minimal"`.
    Code
      (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_cbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.

# can supply `.names_to` to `vec_rbind()` (#229)

    Code
      (expect_error(vec_rbind(.names_to = letters)))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.
    Code
      (expect_error(vec_rbind(.names_to = 10)))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.
    Code
      (expect_error(vec_rbind(.names_to = letters, .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.

# vec_cbind() fails with arrays of dimensionality > 3

    Code
      (expect_error(vec_cbind(a)))
    Output
      <error/rlang_error>
      Error in `vec_cbind()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_cbind(a, .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_cbind(x = a)))
    Output
      <error/rlang_error>
      Error in `vec_cbind()`:
      ! Can't bind arrays.

# vec_rbind() name repair messages are useful

    Code
      vec_rbind(1, 2)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(1, 2, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(1, 2, ...10 = 3)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
      3    3
    Code
      vec_rbind(1, 2, ...10 = 3, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
           ...1
      ...1    1
      ...2    2
      ...3    3
    Code
      vec_rbind(a = 1, b = 2)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(a = 1, b = 2, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      a    1
      b    2
    Code
      vec_rbind(c(a = 1), c(b = 2))
    Output
         a  b
      1  1 NA
      2 NA  2
    Code
      vec_rbind(c(a = 1), c(b = 2), .names_to = NULL)
    Output
         a  b
      1  1 NA
      2 NA  2

# vec_rbind() is silent when assigning duplicate row names of df-cols

    Code
      vec_rbind(df, df)
    Output
         mpg
      1 21.0
      2 21.0
      3 22.8
      4 21.0
      5 21.0
      6 22.8

---

    Code
      vec_rbind(mtcars[1:4, ], mtcars[1:3, ])
    Output
                         mpg cyl disp  hp drat    wt  qsec vs am gear carb
      Mazda RX4...1     21.0   6  160 110 3.90 2.620 16.46  0  1    4    4
      Mazda RX4 Wag...2 21.0   6  160 110 3.90 2.875 17.02  0  1    4    4
      Datsun 710...3    22.8   4  108  93 3.85 2.320 18.61  1  1    4    1
      Hornet 4 Drive    21.4   6  258 110 3.08 3.215 19.44  1  0    3    1
      Mazda RX4...5     21.0   6  160 110 3.90 2.620 16.46  0  1    4    4
      Mazda RX4 Wag...6 21.0   6  160 110 3.90 2.875 17.02  0  1    4    4
      Datsun 710...7    22.8   4  108  93 3.85 2.320 18.61  1  1    4    1

# vec_cbind() name repair messages are useful

    Code
      vec_cbind(1, 2)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
        ...1 ...2
      1    1    2
    Code
      vec_cbind(1, 2, ...10 = 3)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
      * `...10` -> `...3`
    Output
        ...1 ...2 ...3
      1    1    2    3
    Code
      vec_cbind(a = 1, b = 2)
    Output
        a b
      1 1 2
    Code
      vec_cbind(c(a = 1), c(b = 2))
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
        ...1 ...2
      1    1    2

# rbind repairs names of data frames (#704)

    Code
      (expect_error(vec_rbind(df, df, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_rbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
    Code
      (expect_error(vec_rbind(df, df, .name_repair = "check_unique", .error_call = call(
        "foo")), class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `foo()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.

# vec_rbind() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(vec_rbind(set_names(x, "x"), set_names(y, "x")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1` <vctrs_foobar> and `..2` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_rbind() fails with complex foreign S4 classes

    Code
      joe <- .Counts(1L, name = "Joe")
      jane <- .Counts(2L, name = "Jane")
      (expect_error(vec_rbind(set_names(joe, "x"), set_names(jane, "y")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1` <vctrs_Counts> and `..2` <vctrs:::common_class_fallback>.

# can't zap names when `.names_to` is supplied

    Code
      (expect_error(vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap())))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! Can't zap outer names when `.names_to` is supplied.
    Code
      (expect_error(vec_rbind(foo = c(x = 1), .names_to = "id", .name_spec = zap(),
      .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't zap outer names when `.names_to` is supplied.

# row-binding performs expected allocations

    Code
      ints <- rep(list(1L), 100)
      named_ints <- rep(list(set_names(1:3, letters[1:3])), 100)
      # Integers as rows
      suppressMessages(with_memory_prof(vec_rbind_list(ints)))
    Output
      [1] 2.79KB
    Code
      suppressMessages(with_memory_prof(vec_rbind_list(named_ints)))
    Output
      [1] 3.66KB
    Code
      # Data frame with named columns
      df <- data_frame(x = set_names(as.list(1:2), c("a", "b")), y = set_names(1:2, c(
        "A", "B")), z = data_frame(Z = set_names(1:2, c("Za", "Zb"))))
      dfs <- rep(list(df), 100)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 10.4KB
    Code
      # Data frame with rownames (non-repaired, non-recursive case)
      df <- data_frame(x = 1:2)
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 7.68KB
    Code
      # Data frame with rownames (repaired, non-recursive case)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 13.8KB
    Code
      # Data frame with rownames (non-repaired, recursive case) (#1217)
      df <- data_frame(x = 1:2, y = data_frame(x = 1:2))
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 13KB
    Code
      # Data frame with rownames (repaired, recursive case) (#1217)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 25.3KB

# Casting to named argument mentions 'match type <foo>'

    Code
      vec_cast(1, "", x_arg = "foo", to_arg = "bar")
    Condition
      Error:
      ! Can't convert `foo` <double> to match type of `bar` <character>.

---

    Code
      vec_cast(1, "", x_arg = "foo")
    Condition
      Error:
      ! Can't convert `foo` <double> to <character>.

# cast errors create helpful messages (#57, #225)

    Code
      vec_cast(1.5, 10L)
    Condition
      Error:
      ! Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_cast(factor("foo"), 10)
    Condition
      Error:
      ! Can't convert `factor("foo")` <factor<c1562>> to <double>.

---

    Code
      x <- tibble(a = tibble(b = 1.5))
      y <- tibble(a = tibble(b = 10L))
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert from `x$a$b` <double> to `a$b` <integer> due to loss of precision.
      * Locations: 1

---

    Code
      x <- tibble(a = tibble(b = factor("foo")))
      y <- tibble(a = tibble(b = 10))
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert `x$a$b` <factor<c1562>> to match type of `a$b` <double>.

---

    Code
      x <- tibble(a = tibble(b = factor("foo")))
      y <- tibble(a = tibble(b = 10))
      vec_cast_common(x, y)
    Condition
      Error:
      ! Can't combine `..1$a$b` <factor<c1562>> and `..2$a$b` <double>.

# vec_cast() only attempts to fall back if `to` is a data frame (#1568)

    Code
      (expect_error(vec_cast(foobar(mtcars), 1), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(mtcars)` <vctrs_foobar> to <double>.

# can signal deprecation warnings for lossy casts

    Code
      (expect_warning(expect_true(lossy_cast())))
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      Coercion with lossy casts was deprecated in vctrs 0.2.0.
      i Please use `allow_lossy_cast()` instead.
      i We detected a lossy transformation from `x` <fct> to `to` <fct>. The result will contain lower-resolution values or missing values. To suppress this warning, wrap your code with `allow_lossy_cast()`.

# has ok print method

    Code
      partial
    Output
      partial_factor<
        bf275 {partial}
      >

---

    Code
      both
    Output
      partial_factor<
        bf275 {partial}
        fd1ad
      >

---

    Code
      empty
    Output
      partial_factor<
      
      >

---

    Code
      learned
    Output
      partial_factor<
        fd1ad
      >

# vec_recycle_common() reports error context

    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `this_arg` (size 2) to match `that_arg` (size 0).
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `that_arg` (size 0) to size 2.
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .arg = "my_arg")))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg$this_arg` (size 2) to match `my_arg$that_arg` (size 0).
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2, .arg = "my_arg"))
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg$that_arg` (size 0) to size 2.

# vec_recycle(): incompatible lengths get error messages

    Code
      (expect_error(vec_recycle(x2, 1), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 1.

# vec_recycle_common(): incompatible lengths get error messages

    Code
      (expect_error(vec_recycle_common(1:2, 1:3), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

# recycling matrices respects incompatible sizes

    Code
      (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 4).

# recycling data frames respects incompatible sizes

    Code
      (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

# recycling to size 1 has informative error

    Code
      (expect_error(vec_recycle(1:2, 1), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 1.

# incompatible recycling size has informative error

    Code
      vec_recycle(1:2, 4)
    Condition
      Error:
      ! Can't recycle input of size 2 to size 4.

---

    Code
      vec_recycle(1:2, 4, x_arg = "foo")
    Condition
      Error:
      ! Can't recycle `foo` (size 2) to size 4.

# base coercions are symmetric and unchanging

    Code
      mat
    Output
                logical   integer   double   character   raw   list  
      logical   "logical" "integer" "double" NA          NA    NA    
      integer   "integer" "integer" "double" NA          NA    NA    
      double    "double"  "double"  "double" NA          NA    NA    
      character NA        NA        NA       "character" NA    NA    
      raw       NA        NA        NA       NA          "raw" NA    
      list      NA        NA        NA       NA          NA    "list"

# vec_ptype2() data frame methods builds argument tags

    Code
      vec_ptype2("foo", 10)
    Condition
      Error:
      ! Can't combine `"foo"` <character> and `10` <double>.

---

    Code
      df1 <- tibble(x = tibble(y = tibble(z = 1)))
      df2 <- tibble(x = tibble(y = tibble(z = "a")))
      vec_ptype2(df1, df2)
    Condition
      Error:
      ! Can't combine `df1$x$y$z` <double> and `df2$x$y$z` <character>.

# can override scalar vector error message for base scalar types

    Code
      (expect_error(vec_ptype2(NULL, quote(x), y_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a symbol.
    Code
      (expect_error(vec_ptype2(quote(x), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a symbol.

# can override scalar vector error message for S3 types

    Code
      (expect_error(vec_ptype2(NULL, foobar(), y_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_ptype2(foobar(), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.

# ptype2 and cast errors when same class fallback is impossible are informative

    Code
      (expect_error(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(1, bar = TRUE)` <vctrs_foobar> to <vctrs_foobar>.
    Code
      (expect_error(vec_ptype2(foobar(1, bar = TRUE), foobar(2, baz = TRUE)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `foobar(1, bar = TRUE)` <vctrs_foobar> and `foobar(2, baz = TRUE)` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# Incompatible attributes bullets are not show when methods are implemented

    Code
      with_foobar_cast <- (function(expr) {
        with_methods(vec_cast.vctrs_foobar = function(...) NULL,
        vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) vec_default_cast(x,
          to, ...), expr)
      })
      with_foobar_ptype2 <- (function(expr) {
        with_methods(vec_ptype2.vctrs_foobar = function(...) NULL,
        vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) vec_default_ptype2(
          x, y, ...), expr)
      })
      (expect_error(with_foobar_cast(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE))),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(1, bar = TRUE)` <vctrs_foobar> to <vctrs_foobar>.
    Code
      (expect_error(with_foobar_ptype2(vec_ptype2(foobar(1, bar = TRUE), foobar(2,
        baz = TRUE))), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `foobar(1, bar = TRUE)` <vctrs_foobar> and `foobar(2, baz = TRUE)` <vctrs_foobar>.

# vec_as_short_length() checks inputs

    Code
      (expect_error(my_function(-1)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a positive number or zero.
    Code
      (expect_error(my_function(1:2)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a single number, not an integer vector.
    Code
      (expect_error(my_function(1.5)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a whole number, not a fractional number.
    Code
      (expect_error(my_function(NA)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a single number, not `NA`.
    Code
      (expect_error(my_function(na_int)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a single number, not an integer `NA`.
    Code
      (expect_error(my_function("foo")))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a single number, not the string "foo".
    Code
      (expect_error(my_function(foobar(1:2))))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` must be a single number, not `NULL`.
    Code
      (expect_error(my_function(.Machine$double.xmax)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` is too large a number.

# vec_as_short_length() has a special error about long vector support

    Code
      (expect_error(my_function(.Machine$integer.max + 1)))
    Output
      <error/rlang_error>
      Error in `vec_as_short_length()`:
      ! `my_arg` is too large a number and long vectors are not supported.

# vec_size_common() checks inputs

    Code
      (expect_error(vec_size_common(.size = "foo")))
    Output
      <error/rlang_error>
      Error in `vec_size_common()`:
      ! `.size` must be a single number, not the string "foo".
    Code
      (expect_error(vec_size_common(.size = 1:2)))
    Output
      <error/rlang_error>
      Error in `vec_size_common()`:
      ! `.size` must be a single number, not an integer vector.

# vec_size_common() mentions `arg` in errors

    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg$this_arg` (size 2) to match `my_arg$that_arg` (size 0).

# `.absent` must be supplied when `...` is empty

    Code
      (expect_error(vec_size_common(.absent = NULL)))
    Output
      <error/rlang_error>
      Error in `vec_size_common()`:
      ! `.absent` must be supplied when `...` is empty.

# `.absent` must be a length 1 integer if provided

    Code
      (expect_error(vec_size_common(.absent = 1), "must be a single integer"))
    Output
      <error/rlang_error>
      Error in `vec_size_common()`:
      ! `.absent` must be a single integer.
    Code
      (expect_error(vec_size_common(.absent = c(1L, 2L)), "must be a single integer"))
    Output
      <error/rlang_error>
      Error in `vec_size_common()`:
      ! `.absent` must be a single integer.

# argument tags are forwarded

    Code
      vec_size_common(1:2, 1, 1:4)
    Condition
      Error:
      ! Can't recycle `..1` (size 2) to match `..3` (size 4).

---

    Code
      vec_size_common(foo = 1:2, 1, bar = 1:4)
    Condition
      Error:
      ! Can't recycle `foo` (size 2) to match `bar` (size 4).

# allows for name repair

    Code
      vec_interleave(x, x, .name_repair = "unique")
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
      x...1 x...2 
          1     1 

# can repair names quietly

    Code
      res_unique <- vec_interleave(c(x = 1), c(x = 2), .name_repair = "unique_quiet")
      res_universal <- vec_interleave(c(`if` = 1), c(`in` = 2), .name_repair = "universal_quiet")

# uses recycling errors

    Code
      vec_interleave(1:2, 1:3)
    Condition
      Error in `vec_interleave()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

# errors if the result would be a long vector

    Code
      vec_interleave_indices(3L, 1000000000L)
    Condition
      Error in `vec_interleave_indices()`:
      ! Long vectors are not yet supported in `vec_interleave()`. Result from interleaving would have size 3000000000, which is larger than the maximum supported size of 2^31 - 1.

# vec_unchop() is soft-deprecated

    Code
      vec_unchop(list(1), indices = list(1))
    Condition
      Warning:
      `vec_unchop()` was deprecated in vctrs 0.5.0.
      i Please use `list_unchop()` instead.
    Output
      [1] 1

# vec_equal_na() is soft-deprecated

    Code
      vec_equal_na(c(1, NA))
    Condition
      Warning:
      `vec_equal_na()` was deprecated in vctrs 0.5.0.
      i Please use `vec_detect_missing()` instead.
    Output
      [1] FALSE  TRUE

# vec_check_list() still works

    Code
      vec_check_list(1)
    Condition
      Error:
      ! `1` must be a list, not the number 1.

---

    Code
      my_check(1)
    Condition
      Error in `my_check()`:
      ! `x` must be a list, not the number 1.

# vec_as_location2() requires integer or character inputs

    Code
      (expect_error(vec_as_location2(TRUE, 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `TRUE`.
      x `TRUE` must be numeric or character, not `TRUE`.
    Code
      (expect_error(vec_as_location2(mtcars, 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `mtcars`.
      x `mtcars` must be numeric or character, not a <data.frame> object.
    Code
      (expect_error(vec_as_location2(env(), 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `env()`.
      x `env()` must be numeric or character, not an environment.
    Code
      (expect_error(vec_as_location2(foobar(), 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `foobar()`.
      x `foobar()` must be numeric or character, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_as_location2(2.5, 10L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `2.5`.
      x Can't convert from `2.5` <double> to <integer> due to loss of precision.
    Code
      (expect_error(vec_as_location2(Inf, 10L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `Inf`.
      x Can't convert from `Inf` <double> to <integer> due to loss of precision.
    Code
      (expect_error(vec_as_location2(-Inf, 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `-Inf`.
      x Can't convert from `-Inf` <double> to <integer> due to loss of precision.
    Code
      # Idem with custom `arg`
      (expect_error(vec_as_location2(foobar(), 10L, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x `foo` must be numeric or character, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_as_location2(2.5, 3L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Can't convert from `foo` <double> to <integer> due to loss of precision.
    Code
      (expect_error(with_tibble_rows(vec_as_location2(TRUE)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't remove row with `foo(bar)`.
      x `foo(bar)` must be numeric or character, not `TRUE`.

# vec_as_location() requires integer, character, or logical inputs

    Code
      (expect_error(vec_as_location(mtcars, 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `mtcars`.
      x `mtcars` must be logical, numeric, or character, not a <data.frame> object.
    Code
      (expect_error(vec_as_location(env(), 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `env()`.
      x `env()` must be logical, numeric, or character, not an environment.
    Code
      (expect_error(vec_as_location(foobar(), 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `foobar()`.
      x `foobar()` must be logical, numeric, or character, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_as_location(2.5, 10L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `2.5`.
      x Can't convert from `2.5` <double> to <integer> due to loss of precision.
    Code
      (expect_error(vec_as_location(list(), 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `list()`.
      x `list()` must be logical, numeric, or character, not an empty list.
    Code
      (expect_error(vec_as_location(function() NULL, 10L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `function() NULL`.
      x `function() NULL` must be logical, numeric, or character, not a function.
    Code
      (expect_error(vec_as_location(Sys.Date(), 3L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `Sys.Date()`.
      x `Sys.Date()` must be logical, numeric, or character, not a <Date> object.
    Code
      # Idem with custom `arg`
      (expect_error(vec_as_location(env(), 10L, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x `foo` must be logical, numeric, or character, not an environment.
    Code
      (expect_error(vec_as_location(foobar(), 10L, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x `foo` must be logical, numeric, or character, not a <vctrs_foobar> object.
    Code
      (expect_error(vec_as_location(2.5, 3L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Can't convert from `foo` <double> to <integer> due to loss of precision.

# vec_as_location() and variants check for OOB elements (#1605)

    Code
      # Numeric indexing
      (expect_error(vec_as_location(10L, 2L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 10 doesn't exist.
      i There are only 2 elements.
    Code
      (expect_error(vec_as_location(-10L, 2L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't negate elements past the end.
      i Location 10 doesn't exist.
      i There are only 2 elements.
    Code
      (expect_error(vec_as_location2(10L, 2L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't extract elements past the end.
      i Location 10 doesn't exist.
      i There are only 2 elements.
    Code
      # Character indexing
      (expect_error(vec_as_location("foo", 1L, names = "bar"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(vec_as_location2("foo", 1L, names = "bar"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't extract elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(vec_as_location2("foo", 1L, names = "bar", call = call("baz")),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `baz()`:
      ! Can't extract elements that don't exist.
      x Element `foo` doesn't exist.

# vec_as_location2() requires length 1 inputs

    Code
      (expect_error(vec_as_location2(1:2, 2L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `1:2`.
      x Subscript `1:2` must be size 1, not 2.
    Code
      (expect_error(vec_as_location2(c("foo", "bar"), 2L, c("foo", "bar")), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `c("foo", "bar")`.
      x Subscript `c("foo", "bar")` must be size 1, not 2.
    Code
      # Idem with custom `arg`
      (expect_error(vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be size 1, not 2.
    Code
      (expect_error(vec_as_location2(mtcars, 10L, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x `foo` must be numeric or character, not a <data.frame> object.
    Code
      (expect_error(vec_as_location2(1:2, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be size 1, not 2.

# vec_as_location2() requires positive integers

    Code
      (expect_error(vec_as_location2(0, 2L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `0`.
      x Subscript `0` must be a positive location, not 0.
    Code
      (expect_error(vec_as_location2(-1, 2L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `-1`.
      x Subscript `-1` must be a positive location, not -1.
    Code
      # Idem with custom `arg`
      (expect_error(vec_as_location2(0, 2L, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be a positive location, not 0.

# vec_as_location2() fails with NA

    Code
      (expect_error(vec_as_location2(na_int, 2L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `na_int`.
      x Subscript `na_int` must be a location, not an integer `NA`.
    Code
      (expect_error(vec_as_location2(na_chr, 1L, names = "foo"), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't extract element with `na_chr`.
      x Subscript `na_chr` must be a location, not a character `NA`.
    Code
      # Idem with custom `arg`
      (expect_error(vec_as_location2(na_int, 2L, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be a location, not an integer `NA`.

# num_as_location() optionally forbids negative indices

    Code
      (expect_error(num_as_location(dbl(1, -1), 2L, negative = "error"), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `dbl(1, -1)`.
      x Subscript `dbl(1, -1)` can't contain negative locations.

# num_as_location() optionally forbids zero indices

    Code
      (expect_error(num_as_location(0L, 1L, zero = "error"), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `0L`.
      x Subscript `0L` can't contain `0` values.
      i It has a `0` value at location 1.
    Code
      (expect_error(num_as_location(c(0, 0, 0, 0, 0, 0), 1, zero = "error"), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `c(0, 0, 0, 0, 0, 0)`.
      x Subscript `c(0, 0, 0, 0, 0, 0)` can't contain `0` values.
      i It has 6 `0` values at locations 1, 2, 3, 4, 5, etc.

# vec_as_location() checks for mix of negative and missing locations

    Code
      (expect_error(vec_as_location(-c(1L, NA), 30), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `-c(1L, NA)`.
      x Negative locations can't have missing values.
      i Subscript `-c(1L, NA)` has a missing value at location 2.
    Code
      (expect_error(vec_as_location(-c(1L, rep(NA, 10)), 30), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `-c(1L, rep(NA, 10))`.
      x Negative locations can't have missing values.
      i Subscript `-c(1L, rep(NA, 10))` has 10 missing values at locations 2, 3, 4, 5, 6, etc.

# vec_as_location() checks for mix of negative and positive locations

    Code
      (expect_error(vec_as_location(c(-1L, 1L), 30), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `c(-1L, 1L)`.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1L, 1L)` has a positive value at location 2.
    Code
      (expect_error(vec_as_location(c(-1L, rep(1L, 10)), 30), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `c(-1L, rep(1L, 10))`.
      x Negative and positive locations can't be mixed.
      i Subscript `c(-1L, rep(1L, 10))` has 10 positive values at locations 2, 3, 4, 5, 6, etc.

# logical subscripts must match size of indexed vector

    Code
      (expect_error(vec_as_location(c(TRUE, FALSE), 3), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't subset elements with `c(TRUE, FALSE)`.
      x Logical subscript `c(TRUE, FALSE)` must be size 1 or 3, not 2.

# character subscripts require named vectors

    Code
      (expect_error(vec_as_location(letters[1], 3), "unnamed vector"))
    Output
      <error/rlang_error>
      Error in `vec_as_location()`:
      ! Can't use character names to index an unnamed vector.

# can optionally extend beyond the end

    Code
      (expect_error(num_as_location(3, 1, oob = "extend"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 1.
      x Subscript `3` contains non-consecutive location 3.
    Code
      (expect_error(num_as_location(c(1, 3), 1, oob = "extend"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 1.
      x Subscript `c(1, 3)` contains non-consecutive location 3.
    Code
      (expect_error(num_as_location(c(1:5, 7), 3, oob = "extend"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 3.
      x Subscript `c(1:5, 7)` contains non-consecutive locations 4 and 7.
    Code
      (expect_error(num_as_location(c(1:5, 7, 1), 3, oob = "extend"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 3.
      x Subscript `c(1:5, 7, 1)` contains non-consecutive locations 4 and 7.
    Code
      (expect_error(class = "vctrs_error_subscript_oob", num_as_location(c(1:5, 7, 1,
      10), 3, oob = "extend")))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 3.
      x Subscript `c(1:5, 7, 1, 10)` contains non-consecutive locations 4, 7, and 10.

# num_as_location() errors when inverting oob negatives unless `oob = 'remove'` (#1630)

    Code
      num_as_location(-4, 3, oob = "error", negative = "invert")
    Condition
      Error:
      ! Can't negate elements past the end.
      i Location 4 doesn't exist.
      i There are only 3 elements.

---

    Code
      num_as_location(c(-4, 4, 5), 3, oob = "extend", negative = "invert")
    Condition
      Error:
      ! Can't negate elements past the end.
      i Location 4 doesn't exist.
      i There are only 3 elements.

# num_as_location() errors on disallowed zeros when inverting negatives (#1612)

    Code
      num_as_location(c(0, -1), n = 2L, negative = "invert", zero = "error")
    Condition
      Error:
      ! Can't subset elements with `c(0, -1)`.
      x Subscript `c(0, -1)` can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      num_as_location(c(-1, 0), n = 2L, negative = "invert", zero = "error")
    Condition
      Error:
      ! Can't subset elements with `c(-1, 0)`.
      x Subscript `c(-1, 0)` can't contain `0` values.
      i It has a `0` value at location 2.

# num_as_location() with `oob = 'extend'` doesn't allow ignored oob negative values (#1614)

    Code
      num_as_location(-6L, 5L, oob = "extend", negative = "ignore")
    Condition
      Error:
      ! Can't negate elements past the end.
      i Location 6 doesn't exist.
      i There are only 5 elements.

---

    Code
      num_as_location(c(-7L, 6L), 5L, oob = "extend", negative = "ignore")
    Condition
      Error:
      ! Can't negate elements past the end.
      i Location 7 doesn't exist.
      i There are only 5 elements.

---

    Code
      num_as_location(c(-7L, NA), 5L, oob = "extend", negative = "ignore")
    Condition
      Error:
      ! Can't negate elements past the end.
      i Location 7 doesn't exist.
      i There are only 5 elements.

# num_as_location() with `oob = 'error'` reports negative and positive oob values

    Code
      num_as_location(c(-6L, 7L), n = 5L, oob = "error", negative = "ignore")
    Condition
      Error:
      ! Can't subset elements past the end.
      i Locations 6 and 7 don't exist.
      i There are only 5 elements.

# missing values are supported in error formatters

    Code
      (expect_error(num_as_location(c(1, NA, 2, 3), 1), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Locations 2 and 3 don't exist.
      i There is only 1 element.
    Code
      (expect_error(num_as_location(c(1, NA, 3), 1, oob = "extend"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 1.
      x Subscript `c(1, NA, 3)` contains non-consecutive location 3.

# can disallow missing values

    Code
      (expect_error(vec_as_location(c(1, NA), 2, missing = "error"), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has a missing value at location 2.
    Code
      (expect_error(vec_as_location(c(1, NA, 2, NA), 2, missing = "error", arg = "foo",
      call = call("my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has missing values at locations 2 and 4.
    Code
      (expect_error(with_tibble_cols(vec_as_location(c(1, NA, 2, NA), 2, missing = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain missing values.
      x It has missing values at locations 2 and 4.
    Code
      (expect_error(with_tibble_cols(vec_as_location(NA, 1, missing = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain missing values.
      x It has a missing value at location 1.
    Code
      (expect_error(with_tibble_cols(vec_as_location(NA, 3, missing = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain missing values.
      x It has a missing value at location 1.
    Code
      (expect_error(with_tibble_cols(vec_as_location(c(TRUE, NA, FALSE), 3, missing = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain missing values.
      x It has a missing value at location 2.
    Code
      (expect_error(with_tibble_cols(vec_as_location(NA_character_, 2, missing = "error",
        names = c("x", "y"))), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain missing values.
      x It has a missing value at location 1.

# can alter logical missing value handling (#1595)

    Code
      vec_as_location(x, n = 4L, missing = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has missing values at locations 2 and 4.

---

    Code
      vec_as_location(x, n = 2L, missing = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has a missing value at location 1.

# can alter character missing value handling (#1595)

    Code
      vec_as_location(x, n = 2L, names = names, missing = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has missing values at locations 1 and 3.

# can alter integer missing value handling (#1595)

    Code
      vec_as_location(x, n = 4L, missing = "error")
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has missing values at locations 1 and 3.

# can alter negative integer missing value handling (#1595)

    Code
      num_as_location(x, n = 4L, missing = "propagate", negative = "invert")
    Condition
      Error:
      ! Can't subset elements with `x`.
      x Negative locations can't have missing values.
      i Subscript `x` has 2 missing values at locations 2 and 3.

---

    Code
      num_as_location(x, n = 4L, missing = "error", negative = "invert")
    Condition
      Error:
      ! Can't subset elements with `x`.
      x Negative locations can't have missing values.
      i Subscript `x` has 2 missing values at locations 2 and 3.

# empty string character indices never match empty string names (#1489)

    Code
      vec_as_location("", n = 2L, names = names)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain the empty string.
      x It has an empty string at location 1.

---

    Code
      vec_as_location(c("", "y", ""), n = 2L, names = names)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain the empty string.
      x It has an empty string at locations 1 and 3.

# can customise subscript type errors

    Code
      # With custom `arg`
      (expect_error(num_as_location(-1, 2, negative = "error", arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Subscript `foo` can't contain negative locations.
    Code
      (expect_error(num_as_location2(-1, 2, negative = "error", arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be a positive location, not -1.
    Code
      (expect_error(vec_as_location2(0, 2, arg = "foo", call = call("my_function")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be a positive location, not 0.
    Code
      (expect_error(vec_as_location2(na_dbl, 2, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be a location, not an integer `NA`.
    Code
      (expect_error(vec_as_location2(c(1, 2), 2, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't extract element with `foo`.
      x Subscript `foo` must be size 1, not 2.
    Code
      (expect_error(vec_as_location(c(TRUE, FALSE), 3, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_size"))
    Output
      <error/vctrs_error_subscript_size>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Logical subscript `foo` must be size 1 or 3, not 2.
    Code
      (expect_error(vec_as_location(c(-1, NA), 3, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Negative locations can't have missing values.
      i Subscript `foo` has a missing value at location 2.
    Code
      (expect_error(vec_as_location(c(-1, 1), 3, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Negative and positive locations can't be mixed.
      i Subscript `foo` has a positive value at location 2.
    Code
      (expect_error(num_as_location(c(1, 4), 2, oob = "extend", arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `foo` contains non-consecutive location 4.
    Code
      (expect_error(num_as_location(0, 1, zero = "error", arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `foo`.
      x Subscript `foo` can't contain `0` values.
      i It has a `0` value at location 1.
    Code
      # With tibble columns
      (expect_error(with_tibble_cols(num_as_location(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain negative locations.
    Code
      (expect_error(with_tibble_cols(num_as_location2(-1, 2, negative = "error")),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename column with `foo(bar)`.
      x Subscript `foo(bar)` must be a positive location, not -1.
    Code
      (expect_error(with_tibble_cols(vec_as_location2(0, 2)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename column with `foo(bar)`.
      x Subscript `foo(bar)` must be a positive location, not 0.
    Code
      (expect_error(with_tibble_cols(vec_as_location2(na_dbl, 2)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename column with `foo(bar)`.
      x Subscript `foo(bar)` must be a location, not an integer `NA`.
    Code
      (expect_error(with_tibble_cols(vec_as_location2(c(1, 2), 2)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename column with `foo(bar)`.
      x Subscript `foo(bar)` must be size 1, not 2.
    Code
      (expect_error(with_tibble_cols(vec_as_location(c(TRUE, FALSE), 3)), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Logical subscript `foo(bar)` must be size 1 or 3, not 2.
    Code
      (expect_error(with_tibble_cols(vec_as_location(c(-1, NA), 3)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Negative locations can't have missing values.
      i Subscript `foo(bar)` has a missing value at location 2.
    Code
      (expect_error(with_tibble_cols(vec_as_location(c(-1, 1), 3)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Negative and positive locations can't be mixed.
      i Subscript `foo(bar)` has a positive value at location 2.
    Code
      (expect_error(with_tibble_cols(num_as_location(c(1, 4), 2, oob = "extend")),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't rename columns beyond the end with non-consecutive locations.
      i Input has size 2.
      x Subscript `foo(bar)` contains non-consecutive location 4.
    Code
      (expect_error(with_tibble_cols(num_as_location(0, 1, zero = "error")), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't rename columns with `foo(bar)`.
      x Subscript `foo(bar)` can't contain `0` values.
      i It has a `0` value at location 1.

# can customise OOB errors

    Code
      (expect_error(vec_slice(set_names(letters), "foo"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      # With custom `arg`
      (expect_error(vec_as_location(30, length(letters), arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements past the end.
      i Location 30 doesn't exist.
      i There are only 26 elements.
    Code
      (expect_error(vec_as_location("foo", NULL, letters, arg = "foo", call = call(
        "my_function")), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      # With tibble columns
      (expect_error(with_tibble_cols(vec_slice(set_names(letters), "foo")), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't rename columns that don't exist.
      x Column `foo` doesn't exist.
    Code
      (expect_error(with_tibble_cols(vec_slice(set_names(letters), 30)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't rename columns that don't exist.
      i Location 30 doesn't exist.
      i There are only 26 columns.
    Code
      (expect_error(with_tibble_cols(vec_slice(set_names(letters), -30)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't rename columns that don't exist.
      i Location 30 doesn't exist.
      i There are only 26 columns.
    Code
      # With tibble rows
      (expect_error(with_tibble_rows(vec_slice(set_names(letters), c("foo", "bar"))),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't remove rows that don't exist.
      x Rows `foo` and `bar` don't exist.
    Code
      (expect_error(with_tibble_rows(vec_slice(set_names(letters), 1:30)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't remove rows past the end.
      i Locations 27, 28, 29, and 30 don't exist.
      i There are only 26 rows.
    Code
      (expect_error(with_tibble_rows(vec_slice(set_names(letters), -(1:30))), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't remove rows past the end.
      i Locations 27, 28, 29, and 30 don't exist.
      i There are only 26 rows.
    Code
      # With tidyselect select
      (expect_error(with_tidyselect_select(vec_slice(set_names(letters), c("foo",
        "bar"))), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't select columns that don't exist.
      x Columns `foo` and `bar` don't exist.
    Code
      (expect_error(with_tidyselect_select(vec_slice(set_names(letters), 30)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't select columns past the end.
      i Location 30 doesn't exist.
      i There are only 26 columns.
    Code
      (expect_error(with_tidyselect_select(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't select columns past the end.
      i Locations 27, 28, 29, and 30 don't exist.
      i There are only 26 columns.
    Code
      # With tidyselect relocate
      (expect_error(with_tidyselect_relocate(vec_slice(set_names(letters), c("foo",
        "bar"))), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't relocate columns that don't exist.
      x Columns `foo` and `bar` don't exist.
    Code
      (expect_error(with_tidyselect_relocate(vec_slice(set_names(letters), 30)),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't relocate columns that don't exist.
      i Location 30 doesn't exist.
      i There are only 26 columns.
    Code
      (expect_error(with_tidyselect_relocate(vec_slice(set_names(letters), -(1:30))),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't relocate columns that don't exist.
      i Locations 27, 28, 29, and 30 don't exist.
      i There are only 26 columns.

# vec_as_location() checks dimensionality

    Code
      (expect_error(vec_as_location(matrix(TRUE, nrow = 1), 3L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `matrix(TRUE, nrow = 1)`.
      x Subscript `matrix(TRUE, nrow = 1)` must be a simple vector, not a matrix.
    Code
      (expect_error(vec_as_location(array(TRUE, dim = c(1, 1, 1)), 3L), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `array(TRUE, dim = c(1, 1, 1))`.
      x Subscript `array(TRUE, dim = c(1, 1, 1))` must be a simple vector, not an array.
    Code
      (expect_error(with_tibble_rows(vec_as_location(matrix(TRUE, nrow = 1), 3L)),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't remove rows with `foo(bar)`.
      x Subscript `foo(bar)` must be a simple vector, not a matrix.

# vec_as_location() UI

    Code
      vec_as_location(1, 1L, missing = "bogus")
    Condition
      Error in `vec_as_location()`:
      ! `missing` must be one of "propagate", "remove", or "error".

# num_as_location() UI

    Code
      num_as_location(1, 1L, missing = "bogus")
    Condition
      Error in `num_as_location()`:
      ! `missing` must be one of "propagate", "remove", or "error".

---

    Code
      num_as_location(1, 1L, negative = "bogus")
    Condition
      Error in `num_as_location()`:
      ! `negative` must be one of "invert", "error", or "ignore".

---

    Code
      num_as_location(1, 1L, oob = "bogus")
    Condition
      Error in `num_as_location()`:
      ! `oob` must be one of "error", "remove", or "extend".

---

    Code
      num_as_location(1, 1L, zero = "bogus")
    Condition
      Error in `num_as_location()`:
      ! `zero` must be one of "remove", "error", or "ignore".

# vec_as_location2() UI

    Code
      vec_as_location2(1, 1L, missing = "bogus")
    Condition
      Error in `vec_as_location2_result()`:
      ! `missing` must be one of "error" or "propagate", not "bogus".

# AsIs objects throw ptype2 errors with their underlying types

    Code
      (expect_error(vec_ptype2(I(1), I("x")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `I(1)` <double> and `I("x")` <character>.

# AsIs objects throw cast errors with their underlying types

    Code
      (expect_error(vec_cast(I(1), I(factor("x"))), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `I(1)` <double> to <factor<bf275>>.

# errors on scalars

    Code
      vec_identify_runs(foobar())
    Condition
      Error in `vec_identify_runs()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.

---

    Code
      vec_run_sizes(foobar())
    Condition
      Error in `vec_run_sizes()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.

# vec_locate_run_bounds() validates `which`

    Code
      vec_locate_run_bounds(1, which = "x")
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `which` must be one of "start" or "end", not "x".

---

    Code
      vec_locate_run_bounds(1, which = 1)
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `which` must be a string or character vector.

---

    Code
      vec_locate_run_bounds(1, which = c("foo", "bar"))
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `arg` must be length 1 or a permutation of `c("start", "end")`.

# vec_detect_run_bounds() validates `which`

    Code
      vec_detect_run_bounds(1, which = "x")
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `which` must be one of "start" or "end", not "x".

---

    Code
      vec_detect_run_bounds(1, which = 1)
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `which` must be a string or character vector.

---

    Code
      vec_detect_run_bounds(1, which = c("foo", "bar"))
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `arg` must be length 1 or a permutation of `c("start", "end")`.

# `numeric_version` proxy can handle at most 8 components

    Code
      vec_proxy_equal(x)
    Condition
      Error in `vec_proxy_equal()`:
      ! `x` can't contain more than 8 version components.

# inputs must be named

    Code
      vec_expand_grid(1)
    Condition
      Error in `vec_expand_grid()`:
      ! All inputs must be named.

---

    Code
      vec_expand_grid(x = 1, 2, y = 3)
    Condition
      Error in `vec_expand_grid()`:
      ! All inputs must be named.

# catches duplicate names by default

    Code
      vec_expand_grid(a = 1, a = 2)
    Condition
      Error in `vec_expand_grid()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.

# errors on non vectors and mentions the element name

    Code
      vec_expand_grid(y = environment())
    Condition
      Error in `vec_expand_grid()`:
      ! `y` must be a vector, not an environment.

# can adjust the `.error_call`

    Code
      my_expand_grid()
    Condition
      Error in `my_expand_grid()`:
      ! `x` must be a vector, not an environment.

# errors nicely when expansion results in a size larger than `R_len_t`

    Code
      vec_expand_grid(x = x, y = y)
    Condition
      Error in `vec_expand_grid()`:
      ! Long vectors are not yet supported. Expansion results in an allocation larger than 2^31-1 elements. Attempted allocation size was 3221225469.

# errors nicely when expansion results in a size larger than `R_xlen_t`

    Code
      vec_expand_grid(x = x, y = x)
    Condition
      Error in `vec_expand_grid()`:
      ! Result too large for an `r_ssize`.
      i In file './rlang/c-utils.h' at line <scrubbed>.
      i This is an internal error that was detected in the vctrs package.
        Please report it at <https://github.com/r-lib/vctrs/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# validates `.vary`

    Code
      vec_expand_grid(.vary = 1)
    Condition
      Error in `vec_expand_grid()`:
      ! `.vary` must be a string or character vector.

---

    Code
      vec_expand_grid(.vary = "x")
    Condition
      Error in `vec_expand_grid()`:
      ! `.vary` must be one of "slowest" or "fastest", not "x".

# `ties` is validated

    Code
      vec_rank(1, ties = "foo")
    Condition
      Error in `vec_rank()`:
      ! `ties` must be one of "min", "max", "sequential", or "dense", not "foo".

---

    Code
      vec_rank(1, ties = 1)
    Condition
      Error in `vec_rank()`:
      ! `ties` must be a string or character vector.

# `incomplete` is validated

    Code
      vec_rank(1, incomplete = NA)
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be a string or character vector.

---

    Code
      vec_rank(1, incomplete = c(TRUE, FALSE))
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be a string or character vector.

---

    Code
      vec_rank(1, incomplete = "foo")
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be one of "rank" or "na", not "foo".

# must have at least 1 column to match

    Code
      vec_locate_matches(data_frame(), data_frame())
    Condition
      Error in `vec_locate_matches()`:
      ! Must have at least 1 column to match on.

---

    Code
      vec_locate_matches(data_frame(), data_frame(), error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! Must have at least 1 column to match on.

# common type of `needles` and `haystack` is taken

    Code
      vec_locate_matches(x, y)
    Condition
      Error in `vec_locate_matches()`:
      ! Can't combine `needles` <double> and `haystack` <character>.

---

    Code
      vec_locate_matches(x, y, needles_arg = "x", error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! Can't combine `x` <double> and `haystack` <character>.

# `incomplete` can error informatively

    Code
      (expect_error(vec_locate_matches(NA, 1, incomplete = "error")))
    Output
      <error/vctrs_error_matches_incomplete>
      Error in `vec_locate_matches()`:
      ! `needles` can't contain missing values.
      x Location 1 contains missing values.
    Code
      (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo"))
      )
    Output
      <error/vctrs_error_matches_incomplete>
      Error in `vec_locate_matches()`:
      ! `foo` can't contain missing values.
      x Location 1 contains missing values.
    Code
      (expect_error(vec_locate_matches(NA, 1, incomplete = "error", needles_arg = "foo",
        error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_incomplete>
      Error in `fn()`:
      ! `foo` can't contain missing values.
      x Location 1 contains missing values.

# `incomplete` is validated

    Code
      (expect_error(vec_locate_matches(1, 2, incomplete = 1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_locate_matches()`:
      ! Can't convert from `incomplete` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(vec_locate_matches(1, 2, incomplete = c("match", "drop"))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `incomplete` must be length 1, not length 2.
    Code
      (expect_error(vec_locate_matches(1, 2, incomplete = "x")))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `incomplete` must be one of: "compare", "match", "drop", or "error".
    Code
      (expect_error(vec_locate_matches(1, 2, incomplete = "x", error_call = call("fn")))
      )
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `incomplete` must be one of: "compare", "match", "drop", or "error".

# `multiple` is validated

    Code
      (expect_error(vec_locate_matches(1, 2, multiple = 1.5)))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `multiple` must be a string.
    Code
      (expect_error(vec_locate_matches(1, 2, multiple = c("first", "last"))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `multiple` must be a string.
    Code
      (expect_error(vec_locate_matches(1, 2, multiple = "x")))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `multiple` must be one of "all", "any", "first", or "last".
    Code
      (expect_error(vec_locate_matches(1, 2, multiple = "x", error_call = call("fn")))
      )
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `multiple` must be one of "all", "any", "first", or "last".

# `multiple` can error informatively

    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error")))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.
    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error",
      needles_arg = "foo")))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `foo` can match at most 1 value from `haystack`.
      x Location 1 of `foo` matches multiple values.
    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error",
      needles_arg = "foo", error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `fn()`:
      ! Each value of `foo` can match at most 1 value from `haystack`.
      x Location 1 of `foo` matches multiple values.
    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), multiple = "error",
      needles_arg = "foo", haystack_arg = "bar")))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `foo` can match at most 1 value from `bar`.
      x Location 1 of `foo` matches multiple values.

# `multiple` can warn informatively

    Code
      (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning")))
    Output
      <warning/vctrs_warning_matches_multiple>
      Warning in `vec_locate_matches()`:
      Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning",
      needles_arg = "foo")))
    Output
      <warning/vctrs_warning_matches_multiple>
      Warning in `vec_locate_matches()`:
      Each value of `foo` can match at most 1 value from `haystack`.
      x Location 1 of `foo` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning",
      needles_arg = "foo", error_call = call("fn"))))
    Output
      <warning/vctrs_warning_matches_multiple>
      Warning in `fn()`:
      Each value of `foo` can match at most 1 value from `haystack`.
      x Location 1 of `foo` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(1L, c(1L, 1L), multiple = "warning",
      needles_arg = "foo", haystack_arg = "bar")))
    Output
      <warning/vctrs_warning_matches_multiple>
      Warning in `vec_locate_matches()`:
      Each value of `foo` can match at most 1 value from `bar`.
      x Location 1 of `foo` matches multiple values.

# errors on multiple matches that come from different nesting containers

    Code
      vec_locate_matches(df, df2, condition = c("<=", "<="), multiple = "error")
    Condition
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# errors when a match from a different nesting container is processed early on

    Code
      vec_locate_matches(needles, haystack, condition = "<", multiple = "error")
    Condition
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `multiple = 'error' / 'warning'` throw correctly when combined with `relationship`

    Code
      (expect_error(vec_locate_matches(x, y, relationship = "one-to-one", multiple = "error"))
      )
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.

---

    Code
      (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many",
        multiple = "error")))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.

---

    Code
      vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
    Condition
      Warning in `vec_locate_matches()`:
      Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.
      Warning in `vec_locate_matches()`:
      Detected an unexpected many-to-many relationship between `needles` and `haystack`.
      x Location 2 of `needles` matches multiple values.
      x Location 1 of `haystack` matches multiple values.
    Output
        needles haystack
      1       1        2
      2       2        1
      3       2        3
      4       3        1
      5       3        3

---

    Code
      vec_locate_matches(x, y, relationship = "one-to-one", multiple = "warning")
    Condition
      Warning in `vec_locate_matches()`:
      Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.

---

    Code
      (expect_error(vec_locate_matches(x, y, relationship = "warn-many-to-many",
        multiple = "error")))
    Output
      <error/vctrs_error_matches_multiple>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.

---

    Code
      vec_locate_matches(x, y, relationship = "warn-many-to-many", multiple = "warning")
    Condition
      Warning in `vec_locate_matches()`:
      Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.
    Output
        needles haystack
      1       1        2
      2       2        1
      3       2        3

# `relationship` handles one-to-one case

    Code
      (expect_error(vec_locate_matches(c(2, 1), c(1, 1), relationship = "one-to-one"))
      )
    Output
      <error/vctrs_error_matches_relationship_one_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.
    Code
      (expect_error(vec_locate_matches(c(1, 1), c(1, 2), relationship = "one-to-one"))
      )
    Output
      <error/vctrs_error_matches_relationship_one_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` can match at most 1 value from `needles`.
      x Location 1 of `haystack` matches multiple values.

# `relationship` handles one-to-many case

    Code
      (expect_error(vec_locate_matches(c(1, 2, 2), c(2, 1), relationship = "one-to-many"))
      )
    Output
      <error/vctrs_error_matches_relationship_one_to_many>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` can match at most 1 value from `needles`.
      x Location 1 of `haystack` matches multiple values.

# `relationship` handles many-to-one case

    Code
      (expect_error(vec_locate_matches(c(1, 2), c(1, 2, 2), relationship = "many-to-one"))
      )
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 2 of `needles` matches multiple values.

# `relationship` handles warn-many-to-many case

    Code
      (expect_warning(vec_locate_matches(c(1, 2, 1), c(1, 2, 2), relationship = "warn-many-to-many"))
      )
    Output
      <warning/vctrs_warning_matches_relationship_many_to_many>
      Warning in `vec_locate_matches()`:
      Detected an unexpected many-to-many relationship between `needles` and `haystack`.
      x Location 2 of `needles` matches multiple values.
      x Location 1 of `haystack` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(c(1, 1, 2), c(2, 2, 1), relationship = "warn-many-to-many"))
      )
    Output
      <warning/vctrs_warning_matches_relationship_many_to_many>
      Warning in `vec_locate_matches()`:
      Detected an unexpected many-to-many relationship between `needles` and `haystack`.
      x Location 3 of `needles` matches multiple values.
      x Location 3 of `haystack` matches multiple values.

# `relationship` considers `incomplete` matches as possible multiple matches

    Code
      (expect_error(vec_locate_matches(x, y, relationship = "one-to-many")))
    Output
      <error/vctrs_error_matches_relationship_one_to_many>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` can match at most 1 value from `needles`.
      x Location 1 of `haystack` matches multiple values.

# `relationship` errors on multiple matches that come from different nesting containers

    Code
      (expect_error(vec_locate_matches(df, df2, condition = c("<=", "<="),
      relationship = "many-to-one")))
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `relationship` errors when a match from a different nesting container is processed early on

    Code
      (expect_error(vec_locate_matches(needles, haystack, condition = "<",
        relationship = "many-to-one")))
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `relationship` can still detect problematic `haystack` relationships when `multiple = first/last` are used

    Code
      (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first",
      relationship = "one-to-one")))
    Output
      <error/vctrs_error_matches_relationship_one_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` can match at most 1 value from `needles`.
      x Location 2 of `haystack` matches multiple values.
    Code
      (expect_error(vec_locate_matches(c(3, 1, 1), c(2, 1, 3, 3), multiple = "first",
      relationship = "one-to-many")))
    Output
      <error/vctrs_error_matches_relationship_one_to_many>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` can match at most 1 value from `needles`.
      x Location 2 of `haystack` matches multiple values.

# `relationship` and `remaining` work properly together

    Code
      out <- vec_locate_matches(c(1, 2, 2), c(2, 3, 1, 1, 4), relationship = "warn-many-to-many",
      remaining = NA_integer_)
    Condition
      Warning in `vec_locate_matches()`:
      Detected an unexpected many-to-many relationship between `needles` and `haystack`.
      x Location 1 of `needles` matches multiple values.
      x Location 1 of `haystack` matches multiple values.

# `relationship` errors if `condition` creates multiple matches

    Code
      (expect_error(vec_locate_matches(1, c(1, 2), condition = "<=", relationship = "many-to-one"))
      )
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `relationship` still errors if `filter` hasn't removed all multiple matches

    Code
      (expect_error(vec_locate_matches(1, c(1, 2, 1), condition = "<=", filter = "min",
      relationship = "many-to-one")))
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `relationship` errors when we have >1 size 1 matches across containers (tidyverse/dplyr#6835)

    Code
      vec_locate_matches(x, y, condition = c("<=", ">="), filter = c("none", "none"),
      relationship = "one-to-one")
    Condition
      Error in `vec_locate_matches()`:
      ! Each value of `needles` can match at most 1 value from `haystack`.
      x Location 1 of `needles` matches multiple values.

# `relationship` errors respect argument tags and error call

    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "one-to-one",
      needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_relationship_one_to_one>
      Error in `fn()`:
      ! Each value of `foo` can match at most 1 value from `bar`.
      x Location 1 of `foo` matches multiple values.
    Code
      (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-one",
      needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_relationship_one_to_one>
      Error in `fn()`:
      ! Each value of `bar` can match at most 1 value from `foo`.
      x Location 1 of `bar` matches multiple values.
    Code
      (expect_error(vec_locate_matches(c(1L, 1L), 1L, relationship = "one-to-many",
      needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_relationship_one_to_many>
      Error in `fn()`:
      ! Each value of `bar` can match at most 1 value from `foo`.
      x Location 1 of `bar` matches multiple values.
    Code
      (expect_error(vec_locate_matches(1L, c(1L, 1L), relationship = "many-to-one",
      needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_relationship_many_to_one>
      Error in `fn()`:
      ! Each value of `foo` can match at most 1 value from `bar`.
      x Location 1 of `foo` matches multiple values.

# `relationship` warnings respect argument tags and error call

    Code
      (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many",
      needles_arg = "foo", haystack_arg = "bar", error_call = call("fn"))))
    Output
      <warning/vctrs_warning_matches_relationship_many_to_many>
      Warning in `fn()`:
      Detected an unexpected many-to-many relationship between `foo` and `bar`.
      x Location 1 of `foo` matches multiple values.
      x Location 1 of `bar` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many",
      needles_arg = "foo", error_call = call("fn"))))
    Output
      <warning/vctrs_warning_matches_relationship_many_to_many>
      Warning in `fn()`:
      Detected an unexpected many-to-many relationship between `foo` and `haystack`.
      x Location 1 of `foo` matches multiple values.
      x Location 1 of `haystack` matches multiple values.
    Code
      (expect_warning(vec_locate_matches(c(1L, 1L), c(1L, 1L), relationship = "warn-many-to-many",
      haystack_arg = "bar", error_call = call("fn"))))
    Output
      <warning/vctrs_warning_matches_relationship_many_to_many>
      Warning in `fn()`:
      Detected an unexpected many-to-many relationship between `needles` and `bar`.
      x Location 1 of `needles` matches multiple values.
      x Location 1 of `bar` matches multiple values.

# `relationship` is validated

    Code
      (expect_error(vec_locate_matches(1, 2, relationship = 1.5)))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `relationship` must be a string.
    Code
      (expect_error(vec_locate_matches(1, 2, relationship = c("one-to-one",
        "one-to-many"))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `relationship` must be a string.
    Code
      (expect_error(vec_locate_matches(1, 2, relationship = "x")))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `relationship` must be one of "none", "one-to-one", "one-to-many", "many-to-one", "many-to-many", or "warn-many-to-many".
    Code
      (expect_error(vec_locate_matches(1, 2, relationship = "x", error_call = call(
        "fn"))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `relationship` must be one of "none", "one-to-one", "one-to-many", "many-to-one", "many-to-many", or "warn-many-to-many".

# `no_match` can error informatively

    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "error")))
    Output
      <error/vctrs_error_matches_nothing>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` must have a match in `haystack`.
      x Location 1 of `needles` does not have a match.
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo"))
      )
    Output
      <error/vctrs_error_matches_nothing>
      Error in `vec_locate_matches()`:
      ! Each value of `foo` must have a match in `haystack`.
      x Location 1 of `foo` does not have a match.
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo",
        error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_nothing>
      Error in `fn()`:
      ! Each value of `foo` must have a match in `haystack`.
      x Location 1 of `foo` does not have a match.
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "error", needles_arg = "foo",
        haystack_arg = "bar")))
    Output
      <error/vctrs_error_matches_nothing>
      Error in `vec_locate_matches()`:
      ! Each value of `foo` must have a match in `bar`.
      x Location 1 of `foo` does not have a match.

# errors with the right location on unmatched needles when different nesting containers are present

    Code
      (expect_error(vec_locate_matches(df, df2, condition = ">=", no_match = "error"))
      )
    Output
      <error/vctrs_error_matches_nothing>
      Error in `vec_locate_matches()`:
      ! Each value of `needles` must have a match in `haystack`.
      x Location 2 of `needles` does not have a match.

# `no_match` is validated

    Code
      (expect_error(vec_locate_matches(1, 2, no_match = 1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_locate_matches()`:
      ! Can't convert from `no_match` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = c(1L, 2L))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `no_match` must be length 1, not length 2.
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "x")))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `no_match` must be either "drop" or "error".
    Code
      (expect_error(vec_locate_matches(1, 2, no_match = "x", error_call = call("fn")))
      )
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `no_match` must be either "drop" or "error".

# `remaining` can error informatively

    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "error")))
    Output
      <error/vctrs_error_matches_remaining>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` must be matched by `needles`.
      x Location 1 of `haystack` was not matched.
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo"))
      )
    Output
      <error/vctrs_error_matches_remaining>
      Error in `vec_locate_matches()`:
      ! Each value of `haystack` must be matched by `foo`.
      x Location 1 of `haystack` was not matched.
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo",
        error_call = call("fn"))))
    Output
      <error/vctrs_error_matches_remaining>
      Error in `fn()`:
      ! Each value of `haystack` must be matched by `foo`.
      x Location 1 of `haystack` was not matched.
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "error", needles_arg = "foo",
        haystack_arg = "bar")))
    Output
      <error/vctrs_error_matches_remaining>
      Error in `vec_locate_matches()`:
      ! Each value of `bar` must be matched by `foo`.
      x Location 1 of `bar` was not matched.

# `remaining` is validated

    Code
      (expect_error(vec_locate_matches(1, 2, remaining = 1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_locate_matches()`:
      ! Can't convert from `remaining` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = c(1L, 2L))))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `remaining` must be length 1, not length 2.
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "x")))
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `remaining` must be either "drop" or "error".
    Code
      (expect_error(vec_locate_matches(1, 2, remaining = "x", error_call = call("fn")))
      )
    Output
      <error/rlang_error>
      Error in `vec_locate_matches()`:
      ! `remaining` must be either "drop" or "error".

# potential overflow on large output size is caught informatively

    Code
      (expect_error(vec_locate_matches(1:1e+07, 1:1e+07, condition = ">=")))
    Output
      <error/vctrs_error_matches_overflow>
      Error in `vec_locate_matches()`:
      ! Match procedure results in an allocation larger than 2^31-1 elements.
      i Attempted allocation size was 50000005000000.
      i This is an internal error that was detected in the vctrs package.
        Please report it at <https://github.com/r-lib/vctrs/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# data frames print nicely

    Code
      vec_ptype_show(mtcars)
    Output
      Prototype: data.frame<
        mpg : double
        cyl : double
        disp: double
        hp  : double
        drat: double
        wt  : double
        qsec: double
        vs  : double
        am  : double
        gear: double
        carb: double
      >

---

    Code
      vec_ptype_show(iris)
    Output
      Prototype: data.frame<
        Sepal.Length: double
        Sepal.Width : double
        Petal.Length: double
        Petal.Width : double
        Species     : factor<fb977>
      >

# embedded data frames print nicely

    Code
      vec_ptype_show(df)
    Output
      Prototype: data.frame<
        x: integer
        a: 
          data.frame<
            a: integer
            b: character
          >
        b: list_of<double>
        c: 
          list_of<
            data.frame<
              x: integer
              y: character
            >
          >
      >

# `x` must be a list

    Code
      (expect_error(new_data_frame(1), "`x` must be a list"))
    Output
      <error/rlang_error>
      Error:
      ! `x` must be a list

# if supplied, `n` must be an integer of size 1

    Code
      (expect_error(new_data_frame(n = c(1L, 2L)), "must be an integer of size 1"))
    Output
      <error/rlang_error>
      Error in `new_data_frame()`:
      ! `n` must be an integer of size 1.
    Code
      (expect_error(new_data_frame(n = "x"), "must be an integer of size 1"))
    Output
      <error/rlang_error>
      Error in `new_data_frame()`:
      ! `n` must be an integer of size 1.

# if supplied, `n` can't be negative or missing (#1477)

    Code
      (expect_error(new_data_frame(n = -1L)))
    Output
      <error/rlang_error>
      Error in `new_data_frame()`:
      ! `n` can't be negative.
    Code
      (expect_error(new_data_frame(n = NA_integer_)))
    Output
      <error/rlang_error>
      Error in `new_data_frame()`:
      ! `n` can't be missing.

# `class` must be a character vector

    Code
      (expect_error(new_data_frame(class = 1), "must be NULL or a character vector"))
    Output
      <error/rlang_error>
      Error:
      ! `class` must be NULL or a character vector

# data_frame() and df_list() report error context

    Code
      (expect_error(data_frame(a = 1, a = 1)))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `data_frame()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.
    Code
      (expect_error(data_frame(a = 1, a = 1, .error_call = call("foo"))))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `foo()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.
    Code
      (expect_error(data_frame(a = 1:2, b = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `data_frame()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).
    Code
      (expect_error(data_frame(a = 1:2, b = int(), .error_call = call("foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).
    Code
      (expect_error(df_list(a = 1, a = 1)))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `df_list()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.
    Code
      (expect_error(df_list(a = 1, a = 1, .error_call = call("foo"))))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `foo()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.
    Code
      (expect_error(df_list(a = 1:2, b = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `df_list()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).
    Code
      (expect_error(df_list(a = 1:2, b = int(), .error_call = call("foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).

# input is tidy recycled

    Code
      expect_error(data_frame(1:2, 1:3), class = "vctrs_error_incompatible_size")

# `.unpack` is validated

    Code
      df_list(.unpack = 1)
    Condition
      Error in `df_list()`:
      ! `.unpack` must be `TRUE` or `FALSE`.

---

    Code
      df_list(.unpack = c(TRUE, FALSE))
    Condition
      Error in `df_list()`:
      ! `.unpack` must be `TRUE` or `FALSE`.

# `.name_repair` can be quiet

    Code
      dfl_unique <- df_list(1, 2, .name_repair = "unique_quiet")
      dfl_universal <- df_list(`if` = 1, `in` = 2, .name_repair = "universal_quiet")
      df_unique <- data_frame(1, 2, .name_repair = "unique_quiet")
      df_universal <- data_frame(`if` = 1, `in` = 2, .name_repair = "universal_quiet")

# data frame fallback handles column types (#999)

    Code
      local_error_call(call("my_function"))
      (expect_error(vec_ptype2(df1, df3), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `df1$x` <double> and `df3$x` <character>.
    Code
      (expect_error(vec_ptype2(df3, df1), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `df3$x` <character> and `df1$x` <double>.
    Code
      (expect_error(vec_cast(df2, df1), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast_lossy_dropped>
      Error in `my_function()`:
      ! Can't convert from `df2` <vctrs_foobar<
        x: double
        y: double
      >> to <vctrs_foobar<x:double>> due to loss of precision.

```{r, child = "../setup.Rmd", include = FALSE}
```

```{r, include = FALSE}
old_warn_on_fallback <- options(`vctrs:::warn_on_fallback` = FALSE)
knitr_defer(options(old_warn_on_fallback))
```

This guide illustrates how to implement `vec_ptype2()` and `vec_cast()` methods for existing classes. Related topics:

- For an overview of how these generics work and their roles in vctrs, see [`?theory-faq-coercion`][theory-faq-coercion].

- For an example of implementing coercion methods for data frame subclasses, see [`?howto-faq-coercion-data-frame`][howto-faq-coercion-data-frame].

- For a tutorial about implementing vctrs classes from scratch, see `vignette("s3-vector")`


## The natural number class

We'll illustrate how to implement coercion methods with a simple class that represents natural numbers. In this scenario we have an existing class that already features a constructor and methods for `print()` and subset.

```{r}
#' @export
new_natural <- function(x) {
  if (is.numeric(x) || is.logical(x)) {
    stopifnot(is_whole(x))
    x <- as.integer(x)
  } else {
    stop("Can't construct natural from unknown type.")
  }
  structure(x, class = "my_natural")
}
is_whole <- function(x) {
  all(x %% 1 == 0 | is.na(x))
}

#' @export
print.my_natural <- function(x, ...) {
  cat("<natural>\n")
  x <- unclass(x)
  NextMethod()
}
#' @export
`[.my_natural` <- function(x, i, ...) {
  new_natural(NextMethod())
}
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("base::print", "my_natural")
knitr_local_registration("base::[", "my_natural")
```

```{r}
new_natural(1:3)
new_natural(c(1, NA))
```

## Roxygen workflow

```{r, child = "snippet-roxy-workflow.Rmd"}
```


## Implementing `vec_ptype2()`

### The self-self method

The first method to implement is the one that signals that your class is compatible with itself:

```{r}
#' @export
vec_ptype2.my_natural.my_natural <- function(x, y, ...) {
  x
}

vec_ptype2(new_natural(1), new_natural(2:3))
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("vctrs::vec_ptype2", "my_natural.my_natural")
```

`vec_ptype2()` implements a fallback to try and be compatible with simple classes, so it may seem that you don't need to implement the self-self coercion method. However, you must implement it explicitly because this is how vctrs knows that a class that is implementing vctrs methods (for instance this disable fallbacks to `base::c()`). Also, it makes your class a bit more efficient.


### The parent and children methods

Our natural number class is conceptually a parent of `<logical>` and a child of `<integer>`, but the class is not compatible with logical, integer, or double vectors yet:

```{r, error = TRUE}
vec_ptype2(TRUE, new_natural(2:3))

vec_ptype2(new_natural(1), 2:3)
```

We'll specify the twin methods for each of these classes, returning the richer class in each case.

```{r}
#' @export
vec_ptype2.my_natural.logical <- function(x, y, ...) {
  # The order of the classes in the method name follows the order of
  # the arguments in the function signature, so `x` is the natural
  # number and `y` is the logical
  x
}
#' @export
vec_ptype2.logical.my_natural <- function(x, y, ...) {
  # In this case `y` is the richer natural number
  y
}
```

Between a natural number and an integer, the latter is the richer class:

```{r}
#' @export
vec_ptype2.my_natural.integer <- function(x, y, ...) {
  y
}
#' @export
vec_ptype2.integer.my_natural <- function(x, y, ...) {
  x
}
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("vctrs::vec_ptype2", "my_natural.logical")
knitr_local_registration("vctrs::vec_ptype2", "my_natural.integer")
knitr_local_registration("vctrs::vec_ptype2", "integer.my_natural")
knitr_local_registration("vctrs::vec_ptype2", "logical.my_natural")
```

We no longer get common type errors for logical and integer:

```{r}
vec_ptype2(TRUE, new_natural(2:3))

vec_ptype2(new_natural(1), 2:3)
```

We are not done yet. Pairwise coercion methods must be implemented for all the connected nodes in the coercion hierarchy, which include double vectors further up. The coercion methods for grand-parent types must be implemented separately:

```{r}
#' @export
vec_ptype2.my_natural.double <- function(x, y, ...) {
  y
}
#' @export
vec_ptype2.double.my_natural <- function(x, y, ...) {
  x
}
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("vctrs::vec_ptype2", "my_natural.double")
knitr_local_registration("vctrs::vec_ptype2", "double.my_natural")
```



### Incompatible attributes

Most of the time, inputs are incompatible because they have different classes for which no `vec_ptype2()` method is implemented. More rarely, inputs could be incompatible because of their attributes. In that case incompatibility is signalled by calling `stop_incompatible_type()`.

In the following example, we implement a self-self ptype2 method for a hypothetical subclass of `<factor>` that has stricter combination semantics. The method throws an error when the levels of the two factors are not compatible.

```{r, eval = FALSE}
#' @export
vec_ptype2.my_strict_factor.my_strict_factor <- function(x, y, ..., x_arg = "", y_arg = "") {
  if (!setequal(levels(x), levels(y))) {
    stop_incompatible_type(x, y, x_arg = x_arg, y_arg = y_arg)
  }

  x
}
```

Note how the methods need to take `x_arg` and `y_arg` parameters and pass them on to `stop_incompatible_type()`. These argument tags help create more informative error messages when the common type determination is for a column of a data frame. They are part of the generic signature but can usually be left out if not used.


## Implementing `vec_cast()`

Corresponding `vec_cast()` methods must be implemented for all `vec_ptype2()` methods. The general pattern is to convert the argument `x` to the type of `to`. The methods should validate the values in `x` and make sure they conform to the values of `to`.

Please note that for historical reasons, the order of the classes in the method name is in reverse order of the arguments in the function signature. The first class represents `to`, whereas the second class represents `x`.

The self-self method is easy in this case, it just returns the target input:

```{r}
#' @export
vec_cast.my_natural.my_natural <- function(x, to, ...) {
  x
}
```

The other types need to be validated. We perform input validation in the `new_natural()` constructor, so that's a good fit for our `vec_cast()` implementations.

```{r}
#' @export
vec_cast.my_natural.logical <- function(x, to, ...) {
  # The order of the classes in the method name is in reverse order
  # of the arguments in the function signature, so `to` is the natural
  # number and `x` is the logical
  new_natural(x)
}
vec_cast.my_natural.integer <- function(x, to, ...) {
  new_natural(x)
}
vec_cast.my_natural.double <- function(x, to, ...) {
  new_natural(x)
}
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("vctrs::vec_cast", "my_natural.my_natural")
knitr_local_registration("vctrs::vec_cast", "my_natural.logical")
knitr_local_registration("vctrs::vec_cast", "my_natural.integer")
knitr_local_registration("vctrs::vec_cast", "my_natural.double")
```

With these methods, vctrs is now able to combine logical and natural vectors. It properly returns the richer type of the two, a natural vector:

```{r}
vec_c(TRUE, new_natural(1), FALSE)
```

Because we haven't implemented conversions _from_ natural, it still doesn't know how to combine natural with the richer integer and double types:

```{r, error = TRUE}
vec_c(new_natural(1), 10L)
vec_c(1.5, new_natural(1))
```

This is quick work which completes the implementation of coercion methods for vctrs:

```{r}
#' @export
vec_cast.logical.my_natural <- function(x, to, ...) {
  # In this case `to` is the logical and `x` is the natural number
  attributes(x) <- NULL
  as.logical(x)
}
#' @export
vec_cast.integer.my_natural <- function(x, to, ...) {
  attributes(x) <- NULL
  as.integer(x)
}
#' @export
vec_cast.double.my_natural <- function(x, to, ...) {
  attributes(x) <- NULL
  as.double(x)
}
```

```{r, include = FALSE}
# Necessary because includeRmd() evaluated in a child of global
knitr_local_registration("vctrs::vec_cast", "logical.my_natural")
knitr_local_registration("vctrs::vec_cast", "integer.my_natural")
knitr_local_registration("vctrs::vec_cast", "double.my_natural")
```

And we now get the expected combinations.

```{r}
vec_c(new_natural(1), 10L)

vec_c(1.5, new_natural(1))
```

# vec_as_names() validates `repair`

    Code
      (expect_error(my_vec_as_names("x", my_repair = "foo"), "can't be \"foo\""))
    Output
      <error/rlang_error>
      Error in `my_vec_as_names()`:
      ! `my_repair` can't be "foo". See `?vctrs::vec_as_names`.
    Code
      (expect_error(my_vec_as_names(1, my_repair = 1), "string or a function"))
    Output
      <error/rlang_error>
      Error in `my_vec_as_names()`:
      ! `my_repair` must be a string or a function. See `?vctrs::vec_as_names`.

# vec_as_names() checks unique names

    Code
      (expect_error(my_vec_as_names(chr(NA), my_repair = "check_unique")))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.
    Code
      (expect_error(my_vec_as_names(chr(""), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error in `my_vec_as_names()`:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      (expect_error(my_vec_as_names(chr("a", "a"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_vec_as_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `my_repair` to specify repair strategy.
    Code
      (expect_error(my_vec_as_names(chr("..1"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_vec_as_names()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      (expect_error(my_vec_as_names(chr("..."), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_vec_as_names()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# vec_as_names() accepts and checks repair function

    Code
      my_vec_as_names(c("", ""), my_repair = function(nms) "foo")
    Condition
      Error in `my_vec_as_names()`:
      ! Repaired names have length 1 instead of length 2.

# vec_as_names() is noisy by default

    Code
      vec_as_names(c("x", "x"), repair = "unique")
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
      [1] "x...1" "x...2"
    Code
      vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)
    Output
      [1] "x...1" "x...2"
    Code
      (expect_error(my_vec_as_names(c("x", "x"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_vec_as_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
      i Use argument `my_repair` to specify repair strategy.
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet")
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet")
    Output
      [1] "...1...1" "...1...2"
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = TRUE)
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = TRUE)
    Output
      [1] "...1...1" "...1...2"
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = FALSE)
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = FALSE)
    Output
      [1] "...1...1" "...1...2"

# validate_minimal_names() checks names

    Code
      (expect_error(validate_minimal_names(1), "must return a character vector"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions must return a character vector.
    Code
      (expect_error(validate_minimal_names(NULL), "can't return `NULL`"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NULL`.
    Code
      (expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.

# validate_unique() checks unique names

    Code
      (expect_error(validate_unique(chr(NA)), "`NA`"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.
    Code
      (expect_error(validate_unique(chr("")), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      (expect_error(validate_unique(chr("a", "a")), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      (expect_error(validate_unique(chr("..1")), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      (expect_error(validate_unique(chr("...")), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# vec_set_names() errors with bad `names`

    Code
      (expect_error(vec_set_names(1, 1), "character vector, not a double"))
    Output
      <error/rlang_error>
      Error in `vec_set_names()`:
      ! `names` must be a character vector, not a double.
    Code
      (expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2"))
    Output
      <error/rlang_error>
      Error in `vec_set_names()`:
      ! The size of `names`, 2, must be the same as the size of `x`, 1.

# unique_names() and as_unique_names() are verbose or silent

    Code
      unique_names(1:2)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
      [1] "...1" "...2"

---

    Code
      as_unique_names(c("", ""))
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
      [1] "...1" "...2"

# message

    Code
      as_universal_names(c("a b", "b c"))
    Message
      New names:
      * `a b` -> `a.b`
      * `b c` -> `b.c`
    Output
      [1] "a.b" "b.c"

# messages by default

    Code
      vec_repair_names(set_names(1, "a:b"), "universal")
    Message
      New names:
      * `a:b` -> `a.b`
    Output
      a.b 
        1 

---

    Code
      vec_repair_names(set_names(1, "a:b"), ~ make.names(.))
    Message
      New names:
      * `a:b` -> `a.b`
    Output
      a.b 
        1 

# NULL name specs works with scalars

    Code
      (expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(vec_c(foo = 1:2), "vector of length > 1"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(vec_c(x = c(xx = 1)), "named vector"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `x` with a named vector.
      Please supply a `.name_spec` specification.

# vec_as_names() uses internal error if `repair_arg` is not supplied

    Code
      (expect_error(vec_as_names("", repair = "foobar", call = quote(tilt()))))
    Output
      <error/rlang_error>
      Error in `vec_as_names()`:
      ! `repair` can't be "foobar". See `?vctrs::vec_as_names`.
    Code
      (expect_error(vec_as_names("", repair = env(), call = quote(tilt()))))
    Output
      <error/rlang_error>
      Error in `vec_as_names()`:
      ! `repair` must be a string or a function. See `?vctrs::vec_as_names`.

# incompatible shapes throw errors

    Code
      (expect_error(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine <integer[,0,5]> and <integer[,5,1]>.
      x Incompatible sizes 0 and 5 along axis 2.
    Code
      (expect_error(vec_shape2(shaped_int(1, 5, 0), shaped_int(1, 1, 5)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine <integer[,5,0]> and <integer[,1,5]>.
      x Incompatible sizes 0 and 5 along axis 3.

# can override error args

    Code
      (expect_error(vec_shape2(shaped_int(1, 0, 5), shaped_int(1, 5, 1), x_arg = "foo",
      y_arg = "bar"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `foo` <integer[,0,5]> and `bar` <integer[,5,1]>.
      x Incompatible sizes 0 and 5 along axis 2.

# can combine shaped native classes (#1290, #1329)

    Code
      vec_c(x, y)
    Condition
      Error:
      ! Can't combine `..1` <datetime<local>> and `..2` <datetime<local>>.
      x Incompatible sizes 2 and 3 along axis 2.

# factor casts support shape

    Code
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert `x` <factor<32af0>[,1]> to <factor<32af0>>.
      Can't decrease dimensionality from 2 to 1.

```{r, child = "../setup.Rmd", include = FALSE}
```

```{r, include = FALSE}
stopifnot(rlang::is_installed("dplyr"))
```

Two vectors are __compatible__ when you can safely:

- Combine them into one larger vector.
- Assign values from one of the vectors into the other vector.

Examples of compatible types are integer and double vectors. On the other hand, integer and character vectors are not compatible.


# Common type of multiple vectors

There are two possible outcomes when multiple vectors of different types are combined into a larger vector:

-  An incompatible type error is thrown because some of the types are not compatible:

   ```{r, error = TRUE}
   df1 <- data.frame(x = 1:3)
   df2 <- data.frame(x = "foo")
   dplyr::bind_rows(df1, df2)
   ```

-  The vectors are combined into a vector that has the common type of all inputs. In this example, the common type of integer and logical is integer:

   ```{r}
   df1 <- data.frame(x = 1:3)
   df2 <- data.frame(x = FALSE)
   dplyr::bind_rows(df1, df2)
   ```

In general, the common type is the _richer_ type, in other words the type that can represent the most values. Logical vectors are at the bottom of the hierarchy of numeric types because they can only represent two values (not counting missing values). Then come integer vectors, and then doubles. Here is the vctrs type hierarchy for the fundamental vectors:

\\figure{coerce.png}


# Type conversion and lossy cast errors

Type compatibility does not necessarily mean that you can __convert__ one type to the other type. That's because one of the types might support a larger set of possible values. For instance, integer and double vectors are compatible, but double vectors can't be converted to integer if they contain fractional values.

When vctrs can't convert a vector because the target type is not as rich as the source type, it throws a lossy cast error. Assigning a fractional number to an integer vector is a typical example of a lossy cast error:

```{r, error = TRUE}
int_vector <- 1:3
vec_assign(int_vector, 2, 0.001)
```


# How to make two vector classes compatible?

If you encounter two vector types that you think should be compatible, they might need to implement coercion methods. Reach out to the author(s) of the classes and ask them if it makes sense for their classes to be compatible.

These developer FAQ items provide guides for implementing coercion methods:

- For an example of implementing coercion methods for simple vectors, see [`?howto-faq-coercion`][howto-faq-coercion].

- For an example of implementing coercion methods for data frame subclasses, see [`?howto-faq-coercion-data-frame`][howto-faq-coercion-data-frame].

# factor/character coercions are symmetric and unchanging

    Code
      print(mat)
    Output
                ordered<>   factor<>    character  
      ordered<> "ordered<>" NA          "character"
      factor<>  NA          "factor<>"  "character"
      character "character" "character" "character"

# `vec_assign()` requires recyclable value

    Code
      (expect_error(vec_assign(1:3, 1:3, 1:2), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_assign()`:
      ! Can't recycle input of size 2 to size 3.

# logical subscripts must match size of indexed vector

    Code
      (expect_error(vec_assign(1:2, c(TRUE, FALSE, TRUE), 5), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 2, not 3.

---

    Code
      (expect_error(vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 32, not 2.

# must assign existing elements

    Code
      (expect_error(vec_assign(1:3, 5, 10), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't assign to elements past the end.
      i Location 5 doesn't exist.
      i There are only 3 elements.
    Code
      (expect_error(vec_assign(1:3, "foo", 10), "unnamed vector"))
    Output
      <error/rlang_error>
      Error in `vec_assign()`:
      ! Can't use character names to index an unnamed vector.
    Code
      (expect_error(vec_slice(letters, -100) <- "foo", class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't negate elements past the end.
      i Location 100 doesn't exist.
      i There are only 26 elements.
    Code
      (expect_error(vec_assign(set_names(letters), "foo", "bar"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't assign to elements that don't exist.
      x Element `foo` doesn't exist.

# must assign with proper negative locations

    Code
      (expect_error(vec_assign(1:3, c(-1, 1), 1:2), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't assign elements.
      x Negative and positive locations can't be mixed.
      i Subscript has a positive value at location 2.
    Code
      (expect_error(vec_assign(1:3, c(-1, NA), 1:2), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't assign elements.
      x Negative locations can't have missing values.
      i Subscript has a missing value at location 2.

# `vec_assign()` error args can be overridden

    Code
      (expect_error(vec_assign(1:2, 1L, "x", x_arg = "foo", value_arg = "bar"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `vec_assign()`:
      ! Can't convert `bar` <character> to match type of `foo` <integer>.
    Code
      (expect_error(vec_assign(1:2, 1L, 1:2, value_arg = "bar"), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_assign()`:
      ! Can't recycle `bar` (size 2) to size 1.

# vec_slice throws error with non-vector subscripts

    Code
      (expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x `i` must be logical, numeric, or character, not a <Date> object.
    Code
      (expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Subscript `i` must be a simple vector, not a matrix.

# can't index beyond the end of a vector

    Code
      (expect_error(vec_slice(1:2, 3L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.
    Code
      (expect_error(vec_slice(1:2, -3L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't negate elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

# can slice with double indices

    Code
      (expect_error(vec_as_location(2^31, 3L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `2^31`.
      x Can't convert from `2^31` <double> to <integer> due to loss of precision.

# Unnamed vector with character subscript is caught

    Code
      vec_slice(1:3, letters[1])
    Condition
      Error in `vec_slice()`:
      ! Can't use character names to index an unnamed vector.

# Negative subscripts are checked

    Code
      vec_slice(1:3, -c(1L, NA))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Negative locations can't have missing values.
      i Subscript `i` has a missing value at location 2.

---

    Code
      vec_slice(1:3, c(-1L, 1L))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Negative and positive locations can't be mixed.
      i Subscript `i` has a positive value at location 2.

# oob error messages are properly constructed

    Code
      vec_slice(c(bar = 1), "foo")
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.

---

    Code
      vec_slice(letters, c(100, 1000))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Locations 100 and 1000 don't exist.
      i There are only 26 elements.

---

    Code
      vec_slice(letters, c(1, 100:103, 2, 104:110))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Locations 100, 101, 102, ..., 109, and 110 don't exist.
      i There are only 26 elements.

---

    Code
      vec_slice(set_names(letters), c("foo", "bar"))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Elements `foo` and `bar` don't exist.

---

    Code
      vec_slice(set_names(letters), toupper(letters))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Elements `A`, `B`, `C`, `D`, `E`, etc. don't exist.

# vec_init() validates `n`

    Code
      (expect_error(vec_init(1L, 1.5)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a whole number, not a fractional number.
    Code
      (expect_error(vec_init(1L, c(1, 2))))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not a double vector.
    Code
      (expect_error(vec_init(1L, -1L)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a positive number or zero.
    Code
      (expect_error(vec_init(1L, NA)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not `NA`.
    Code
      (expect_error(vec_init(1L, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not an integer `NA`.

To implement methods for generics, first import the generics in your namespace and redocument:

```{r, eval = FALSE}
#' @importFrom vctrs vec_ptype2 vec_cast
NULL
```

Note that for each batches of methods that you add to your package, you need to export the methods and redocument immediately, even during development. Otherwise they won't be in scope when you run unit tests e.g. with testthat.

Implementing double dispatch methods is very similar to implementing regular S3 methods. In these examples we are using roxygen2 tags to register the methods, but you can also register the methods manually in your NAMESPACE file or lazily with `s3_register()`.

vctrs provides a framework for working with vector classes in a generic way. However, it implements several compatibility fallbacks to base R methods. In this reference you will find how vctrs tries to be compatible with your vector class, and what base methods you need to implement for compatibility.

If you're starting from scratch, we think you'll find it easier to start using [new_vctr()] as documented in `vignette("s3-vector")`. This guide is aimed for developers with existing vector classes.


## Aggregate operations with fallbacks

All vctrs operations are based on four primitive generics described in the next section. However there are many higher level operations. The most important ones implement fallbacks to base generics for maximum compatibility with existing classes.

- [vec_slice()] falls back to the base `[` generic if no [vec_proxy()] method is implemented. This way foreign classes  that do not implement [vec_restore()] can restore attributes based on the new subsetted contents.

- [vec_c()] and [vec_rbind()] now fall back to [base::c()] if the inputs have a common parent class with a `c()` method (only if they have no self-to-self `vec_ptype2()` method).

  vctrs works hard to make your `c()` method success in various situations (with `NULL` and `NA` inputs, even as first input which would normally prevent dispatch to your method). The main downside compared to using vctrs primitives is that you can't combine vectors of different classes since there is no extensible mechanism of coercion in `c()`, and it is less efficient in some cases.


## The vctrs primitives

Most functions in vctrs are aggregate operations: they call other vctrs functions which themselves call other vctrs functions. The dependencies of a vctrs functions are listed in the Dependencies section of its documentation page. Take a look at [vec_count()] for an example.

These dependencies form a tree whose leaves are the four vctrs primitives. Here is the diagram for `vec_count()`:

\\figure{vec-count-deps.png}


### The coercion generics

The coercion mechanism in vctrs is based on two generics:

- [vec_ptype2()]
- [vec_cast()]

See the [theory overview][theory-faq-coercion].

Two objects with the same class and the same attributes are always considered compatible by ptype2 and cast. If the attributes or classes differ, they throw an incompatible type error.

Coercion errors are the main source of incompatibility with vctrs. See the [howto guide][howto-faq-coercion] if you need to implement methods for these generics.


### The proxy and restoration generics

- [vec_proxy()]
- [vec_restore()]

These generics are essential for vctrs but mostly optional. `vec_proxy()` defaults to an [identity][identity] function and you normally don't need to implement it. The proxy a vector must be one of the atomic vector types, a list, or a data frame. By default, S3 lists that do not inherit from `"list"` do not have an identity proxy. In that case, you need to explicitly implement `vec_proxy()` or make your class inherit from list.

```{r, child = "../setup.Rmd", include = FALSE}
```

```{r, include = FALSE}
stopifnot(rlang::is_installed("dplyr"))
```

The tidyverse is a bit stricter than base R regarding what kind of objects are considered as vectors (see the [user FAQ][faq-error-scalar-type] about this topic). Sometimes vctrs won't treat your class as a vector when it should.


## Why isn't my list class considered a vector?

By default, S3 lists are not considered to be vectors by vctrs:

```{r}
my_list <- structure(list(), class = "my_class")

vctrs::vec_is(my_list)
```

To be treated as a vector, the class must either inherit from `"list"` explicitly:

```{r}
my_explicit_list <- structure(list(), class = c("my_class", "list"))
vctrs::vec_is(my_explicit_list)
```

Or it should implement a `vec_proxy()` method that returns its input if explicit inheritance is not possible or troublesome:

```{r}
#' @export
vec_proxy.my_class <- function(x, ...) x

vctrs::vec_is(my_list)
```

Note that explicit inheritance is the preferred way because this makes it possible for your class to dispatch on `list` methods of S3 generics:

```{r, error = TRUE}
my_generic <- function(x) UseMethod("my_generic")
my_generic.list <- function(x) "dispatched!"

my_generic(my_list)

my_generic(my_explicit_list)
```


## Why isn't my data frame class considered a vector?

The most likely explanation is that the data frame has not been
properly constructed.

However, if you get an "Input must be a vector" error with a data frame subclass, it probably means that the data frame has not been properly constructed. The main cause of these errors are data frames whose _base class_ is not `"data.frame"`:

```{r, error = TRUE}
my_df <- data.frame(x = 1)
class(my_df) <- c("data.frame", "my_class")

vctrs::obj_check_vector(my_df)
```

This is problematic as many tidyverse functions won't work properly:

```{r, error = TRUE}
dplyr::slice(my_df, 1)
```

It is generally not appropriate to declare your class to be a superclass of another class. We generally consider this undefined behaviour (UB). To fix these errors, you can simply change the construction of your data frame class so that `"data.frame"` is a base class, i.e. it should come last in the class vector:

```{r}
class(my_df) <- c("my_class", "data.frame")

vctrs::obj_check_vector(my_df)

dplyr::slice(my_df, 1)
```

```{r, child = "../setup.Rmd", include = FALSE}
```

This is an overview of the usage of `vec_ptype2()` and `vec_cast()` and their role in the vctrs coercion mechanism. Related topics:

- For an example of implementing coercion methods for simple vectors, see [`?howto-faq-coercion`][howto-faq-coercion].

- For an example of implementing coercion methods for data frame subclasses, see [`?howto-faq-coercion-data-frame`][howto-faq-coercion-data-frame].

- For a tutorial about implementing vctrs classes from scratch, see `vignette("s3-vector")`.


## Combination mechanism in vctrs

The coercion system in vctrs is designed to make combination of multiple inputs consistent and extensible. Combinations occur in many places, such as row-binding, joins, subset-assignment, or grouped summary functions that use the split-apply-combine strategy. For example:

```{r, error = TRUE}
vec_c(TRUE, 1)

vec_c("a", 1)

vec_rbind(
  data.frame(x = TRUE),
  data.frame(x = 1, y = 2)
)

vec_rbind(
  data.frame(x = "a"),
  data.frame(x = 1, y = 2)
)
```

One major goal of vctrs is to provide a central place for implementing the coercion methods that make generic combinations possible. The two relevant generics are `vec_ptype2()` and `vec_cast()`. They both take two arguments and perform __double dispatch__, meaning that a method is selected based on the classes of both inputs.

The general mechanism for combining multiple inputs is:

1. Find the common type of a set of inputs by reducing (as in `base::Reduce()` or `purrr::reduce()`) the `vec_ptype2()` binary function over the set.

2. Convert all inputs to the common type with `vec_cast()`.

3. Initialise the output vector as an instance of this common type with `vec_init()`.

4. Fill the output vector with the elements of the inputs using `vec_assign()`.

The last two steps may require `vec_proxy()` and `vec_restore()` implementations, unless the attributes of your class are constant and do not depend on the contents of the vector. We focus here on the first two steps, which require `vec_ptype2()` and `vec_cast()` implementations.


## `vec_ptype2()`

Methods for `vec_ptype2()` are passed two _prototypes_, i.e. two inputs emptied of their elements. They implement two behaviours:

* If the types of their inputs are compatible, indicate which of them is the richer type by returning it. If the types are of equal resolution, return any of the two.

* Throw an error with `stop_incompatible_type()` when it can be determined from the attributes that the types of the inputs are not compatible.


### Type compatibility

A type is __compatible__ with another type if the values it represents are a subset or a superset of the values of the other type. The notion of "value" is to be interpreted at a high level, in particular it is not the same as the memory representation. For example, factors are represented in memory with integers but their values are more related to character vectors than to round numbers:

```{r, error = TRUE}
# Two factors are compatible
vec_ptype2(factor("a"), factor("b"))

# Factors are compatible with a character
vec_ptype2(factor("a"), "b")

# But they are incompatible with integers
vec_ptype2(factor("a"), 1L)
```


### Richness of type

Richness of type is not a very precise notion. It can be about richer data (for instance a `double` vector covers more values than an integer vector), richer behaviour (a `data.table` has richer behaviour than a `data.frame`), or both. If you have trouble determining which one of the two types is richer, it probably means they shouldn't be automatically coercible.

Let's look again at what happens when we combine a factor and a character:

```{r}
vec_ptype2(factor("a"), "b")
```

The ptype2 method for `<character>` and `<factor<"a">>` returns `<character>` because the former is a richer type. The factor can only contain `"a"` strings, whereas the character can contain any strings. In this sense, factors are a _subset_ of character.

Note that another valid behaviour would be to throw an incompatible type error. This is what a strict factor implementation would do. We have decided to be laxer in vctrs because it is easy to inadvertently create factors instead of character vectors, especially with older versions of R where `stringsAsFactors` is still true by default.


### Consistency and symmetry on permutation

Each ptype2 method should strive to have exactly the same behaviour when the inputs are permuted. This is not always possible, for example factor levels are aggregated in order:

```{r}
vec_ptype2(factor(c("a", "c")), factor("b"))

vec_ptype2(factor("b"), factor(c("a", "c")))
```

In any case, permuting the input should not return a fundamentally different type or introduce an incompatible type error.


### Coercion hierarchy

The classes that you can coerce together form a coercion (or subtyping) hierarchy. Below is a schema of the hierarchy for the base types like integer and factor. In this diagram the directions of the arrows express which type is richer. They flow from the bottom (more constrained types) to the top (richer types).

\\figure{coerce.png}

A coercion hierarchy is distinct from the structural hierarchy implied by memory types and classes. For instance, in a structural hierarchy, factors are built on top of integers. But in the coercion hierarchy they are more related to character vectors. Similarly, subclasses are not necessarily coercible with their superclasses because the coercion and structural hierarchies are separate.


### Implementing a coercion hierarchy

As a class implementor, you have two options. The simplest is to create an entirely separate hierarchy. The date and date-time classes are an example of an S3-based hierarchy that is completely separate. Alternatively, you can integrate your class in an existing hierarchy, typically by adding parent nodes on top of the hierarchy (your class is richer), by adding children node at the root of the hierarchy (your class is more constrained), or by inserting a node in the tree.

These coercion hierarchies are _implicit_, in the sense that they are implied by the `vec_ptype2()` implementations. There is no structured way to create or modify a hierarchy, instead you need to implement the appropriate coercion methods for all the types in your hierarchy, and diligently return the richer type in each case. The `vec_ptype2()` implementations are not transitive nor inherited, so all pairwise methods between classes lying on a given path must be implemented manually. This is something we might make easier in the future.


## `vec_cast()`

The second generic, `vec_cast()`, is the one that looks at the data and actually performs the conversion. Because it has access to more information than `vec_ptype2()`, it may be stricter and cause an error in more cases. `vec_cast()` has three possible behaviours:

- Determine that the prototypes of the two inputs are not compatible. This must be decided in exactly the same way as for `vec_ptype2()`. Call `stop_incompatible_cast()` if you can determine from the attributes that the types are not compatible.

- Detect incompatible values. Usually this is because the target type is too restricted for the values supported by the input type. For example, a fractional number can't be converted to an integer. The method should throw an error in that case.

- Return the input vector converted to the target type if all values are compatible. Whereas `vec_ptype2()` must return the same type when the inputs are permuted, `vec_cast()` is _directional_. It always returns the type of the right-hand side, or dies trying.


## Double dispatch

The dispatch mechanism for `vec_ptype2()` and `vec_cast()` looks like S3 but is actually a custom mechanism. Compared to S3, it has the following differences:

* It dispatches on the classes of the first two inputs.

* There is no inheritance of ptype2 and cast methods. This is because the S3 class hierarchy is not necessarily the same as the coercion hierarchy.

* `NextMethod()` does not work. Parent methods must be called explicitly if necessary.

* The default method is hard-coded.


## Data frames

The determination of the common type of data frames with `vec_ptype2()` happens in three steps:

1. Match the columns of the two input data frames. If some columns don't exist, they are created and filled with adequately typed `NA` values.

2. Find the common type for each column by calling `vec_ptype2()` on each pair of matched columns.

3. Find the common data frame type. For example the common type of a grouped tibble and a tibble is a grouped tibble because the latter is the richer type. The common type of a data table and a data frame is a data table.

`vec_cast()` operates similarly. If a data frame is cast to a target type that has fewer columns, this is an error.

If you are implementing coercion methods for data frames, you will need to explicitly call the parent methods that perform the common type determination or the type conversion described above. These are exported as [df_ptype2()] and [df_cast()].


### Data frame fallbacks

Being too strict with data frame combinations would cause too much pain because there are many data frame subclasses in the wild that don't implement vctrs methods. We have decided to implement a special fallback behaviour for foreign data frames. Incompatible data frames fall back to a base data frame:

```{r}
df1 <- data.frame(x = 1)
df2 <- structure(df1, class = c("foreign_df", "data.frame"))

vec_rbind(df1, df2)
```

When a tibble is involved, we fall back to tibble:

```{r}
df3 <- tibble::as_tibble(df1)

vec_rbind(df1, df3)
```

These fallbacks are not ideal but they make sense because all data frames share a common data structure. This is not generally the case for vectors. For example factors and characters have different representations, and it is not possible to find a fallback time mechanically.

However this fallback has a big downside: implementing vctrs methods for your data frame subclass is a breaking behaviour change. The proper coercion behaviour for your data frame class should be specified as soon as possible to limit the consequences of changing the behaviour of your class in R scripts.

```{r, child = "../setup.Rmd", include = FALSE}
```

## Promotion monoid

Promotions (i.e. automatic coercions) should always transform inputs to their richer type to avoid losing values of precision. `vec_ptype2()` returns the _richer_ type of two vectors, or throws an incompatible type error if none of the two vector types include the other. For example, the richer type of integer and double is the latter because double covers a larger range of values than integer.

`vec_ptype2()` is a [monoid](https://en.wikipedia.org/wiki/Monoid) over vectors, which in practical terms means that it is a well behaved operation for [reduction](https://purrr.tidyverse.org/reference/reduce.html). Reduction is an important operation for promotions because that is how the richer type of multiple elements is computed. As a monoid, `vec_ptype2()` needs an identity element, i.e. a value that doesn't change the result of the reduction. vctrs has two identity values, `NULL` and __unspecified__ vectors.


## The `NULL` identity

As an identity element that shouldn't influence the determination of the common type of a set of vectors, `NULL` is promoted to any type:

```{r}
vec_ptype2(NULL, "")
vec_ptype2(1L, NULL)
```

The common type of `NULL` and `NULL` is the identity `NULL`:

```{r}
vec_ptype2(NULL, NULL)
```

This way the result of `vec_ptype2(NULL, NULL)` does not influence subsequent promotions:

```{r}
vec_ptype2(
  vec_ptype2(NULL, NULL),
  ""
)
```


## Unspecified vectors

In the vctrs coercion system, logical vectors of missing values are also automatically promoted to the type of any other vector, just like `NULL`. We call these vectors unspecified. The special coercion semantics of unspecified vectors serve two purposes:

1. It makes it possible to assign vectors of `NA` inside any type of vectors, even when they are not coercible with logical:

    ```{r}
    x <- letters[1:5]
    vec_assign(x, 1:2, c(NA, NA))
    ```

2. We can't put `NULL` in a data frame, so we need an identity element that behaves more like a vector. Logical vectors of `NA` seem a natural fit for this.

Unspecified vectors are thus promoted to any other type, just like `NULL`:

```{r}
vec_ptype2(NA, "")
vec_ptype2(1L, c(NA, NA))
```


## Finalising common types

vctrs has an internal vector type of class `vctrs_unspecified`. Users normally don't see such vectors in the wild, but they do come up when taking the common type of an unspecified vector with another identity value:

```{r}
vec_ptype2(NA, NA)
vec_ptype2(NA, NULL)
vec_ptype2(NULL, NA)
```

We can't return `NA` here because `vec_ptype2()` normally returns empty vectors. We also can't return `NULL` because unspecified vectors need to be recognised as logical vectors if they haven't been promoted at the end of the reduction.

```{r}
vec_ptype_finalise(vec_ptype2(NULL, NA))
```

See the output of `vec_ptype_common()` which performs the reduction and finalises the type, ready to be used by the caller:

```{r}
vec_ptype_common(NULL, NULL)
vec_ptype_common(NA, NULL)
```

`vec_ptype_finalise()` is an S3 generic, but the only time you should ever need to write an S3 method for it is if your class _wraps_ another vector in some way and needs special handling to propagate the default finalisation. For example, the ivs package contains an interval class that wraps `start` and `end` vectors of the same type and has a `vec_ptype_finalize()` method that finalises those wrapped vectors:

```{r, eval = FALSE}
vec_ptype_finalise.ivs_iv <- function(x, ...) {
  start <- unclass(x)[[1L]]
  ptype <- vec_ptype_finalise(start, ...)
  new_bare_iv(ptype, ptype)
}
```

This ensures that `vec_ptype_finalise(vec_ptype(ivs::iv(NA, NA)))` correctly finalises to `<iv<logical>>` rather than `<iv<unspecified>>`.

Note that data frames are already recursively finalised, so you don't need a `vec_ptype_finalise()` method for a data frame subclass.

# Implementing coercion methods

- For an overview of how these generics work and their roles in vctrs, see [`?theory-faq-coercion`][theory-faq-coercion].

- For an example of implementing coercion methods for simple vectors, see [`?howto-faq-coercion`][howto-faq-coercion].

- For an example of implementing coercion methods for data frame subclasses, see [`?howto-faq-coercion-data-frame`][howto-faq-coercion-data-frame].

- For a tutorial about implementing vctrs classes from scratch, see `vignette("s3-vector")`.

```{r, child = "../setup.Rmd", include = FALSE}
```

This error occurs when a function expects a vector and gets a scalar object instead. This commonly happens when some code attempts to assign a scalar object as column in a data frame:

```{r, error = TRUE}
fn <- function() NULL
tibble::tibble(x = fn)

fit <- lm(1:3 ~ 1)
tibble::tibble(x = fit)
```


# Vectorness in base R and in the tidyverse

In base R, almost everything is a vector or behaves like a vector. In the tidyverse we have chosen to be a bit stricter about what is considered a vector. The main question we ask ourselves to decide on the vectorness of a type is whether it makes sense to include that object as a column in a data frame.

The main difference is that S3 lists are considered vectors by base R but in the tidyverse that's not the case by default:

```{r, error = TRUE}
fit <- lm(1:3 ~ 1)

typeof(fit)
class(fit)

# S3 lists can be subset like a vector using base R:
fit[c(1, 4)]

# But not in vctrs
vctrs::vec_slice(fit, c(1, 4))
```

Defused function calls are another (more esoteric) example:

```{r, error = TRUE}
call <- quote(foo(bar = TRUE, baz = FALSE))
call

# They can be subset like a vector using base R:
call[1:2]
lapply(call, function(x) x)

# But not with vctrs:
vctrs::vec_slice(call, 1:2)
```


# I get a scalar type error but I think this is a bug

It's possible the author of the class needs to do some work to declare their class a vector. Consider reaching out to the author. We have written a [developer FAQ page][howto-faq-fix-scalar-type-error] to help them fix the issue.

# incompatible type error validates `action`

    Code
      (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = "c"))
      )
    Output
      <error/rlang_error>
      Error in `stop_incompatible_type()`:
      ! `action` must be one of "combine" or "convert", not "c".
    Code
      (expect_error(stop_incompatible_type(1, 1, x_arg = "", y_arg = "", action = 1)))
    Output
      <error/rlang_error>
      Error in `stop_incompatible_type()`:
      ! `action` must be a character vector, not the number 1.

# can override arg in OOB conditions

    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), NULL),
      class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), quote(
        foo)), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.
    Code
      (expect_error(with_subscript_data(vec_slice(set_names(letters), "foo"), quote(
        foo(bar))), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.

# scalar type errors are informative

    Code
      (expect_error(vec_slice(foobar(list(1)), 1), class = "vctrs_error_scalar_type"))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_slice()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
    Code
      (expect_error(stop_scalar_type(foobar(list(1)), arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# empty names errors are informative

    Code
      (expect_error(vec_as_names(c("x", "", "y"), repair = "check_unique"), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty name found at location 2.
    Code
      (expect_error(vec_as_names(c("x", "", "y", ""), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_empty"))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty names found at locations 2 and 4.
    Code
      (expect_error(vec_as_names(rep("", 10), repair = "check_unique"), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty names found at locations 1, 2, 3, 4, 5, etc.

# dot dot names errors are informative

    Code
      (expect_error(vec_as_names(c("..1", "..1", "..1", "...", "z"), repair = "check_unique"),
      class = "vctrs_error_names_cannot_be_dot_dot"))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at locations 1, 2, and 3.
        * "..." at location 4.
    Code
      (expect_error(vec_as_names(c(rep("..1", 20), rep(c("..2", "..3", "..4", "...",
        "..5"), 2)), repair = "check_unique"), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at locations 1, 2, 3, 4, 5, etc.
        * "..2" at locations 21 and 26.
        * "..3" at locations 22 and 27.
        * "..4" at locations 23 and 28.
        * "..." at locations 24 and 29.
        * ...

# unique names errors are informative

    Code
      (expect_error(vec_as_names(c("x", "x", "x", "y", "y", "z"), repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1, 2, and 3.
        * "y" at locations 4 and 5.
    Code
      (expect_error(vec_as_names(c(rep("x", 20), rep(c("a", "b", "c", "d", "e"), 2)),
      repair = "check_unique"), class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1, 2, 3, 4, 5, etc.
        * "a" at locations 21 and 26.
        * "b" at locations 22 and 27.
        * "c" at locations 23 and 28.
        * "d" at locations 24 and 29.
        * ...

# lossy cast from character to factor mentions loss of generality

    Code
      (expect_error(vec_cast("a", factor("b")), class = "vctrs_error_cast_lossy"))
    Output
      <error/vctrs_error_cast_lossy>
      Error:
      ! Can't convert from `"a"` <character> to <factor<9b7e3>> due to loss of generality.
      * Locations: 1

# lossy cast `conditionMessage()` result matches `cnd_message()` (#1592)

    Code
      cat(conditionMessage(cnd))
    Output
      Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

# ordered cast failures mention conversion

    Code
      (expect_error(vec_cast(ordered("x"), ordered("y")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `ordered("x")` <ordered<bf275>> to <ordered<fd1ad>>.

# incompatible size errors

    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "")))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 3.
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo),
      y_arg = "")))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `foo` (size 2) to size 3.
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = "", y_arg = "bar"))
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to match `bar` (size 3).
    Code
      (expect_error(stop_incompatible_size(1:2, 3:5, 2L, 3L, x_arg = quote(foo),
      y_arg = quote(bar))))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `foo` (size 2) to match `bar` (size 3).

# throws error for unsuported type

    Code
      vec_equal(expression(x), expression(x))
    Condition <vctrs_error_scalar_type>
      Error in `vec_equal()`:
      ! `x` must be a vector, not an expression vector.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# `na_equal` is validated

    Code
      vec_equal(1, 1, na_equal = 1)
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`.

---

    Code
      vec_equal(1, 1, na_equal = c(TRUE, FALSE))
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`.

# can't supply NA as `na_equal`

    Code
      vec_equal(NA, NA, na_equal = NA)
    Condition
      Error in `vec_equal()`:
      ! `na_equal` must be `TRUE` or `FALSE`.

# `conditions` inputs can be size zero

    Code
      vec_case_when(list(logical()), list(1:2))
    Condition
      Error in `vec_case_when()`:
      ! Can't recycle `values[[1]]` (size 2) to size 0.

# `values` are cast to their common type

    Code
      vec_case_when(list(FALSE, TRUE), list(1, "x"))
    Condition
      Error in `vec_case_when()`:
      ! Can't combine `values[[1]]` <double> and `values[[2]]` <character>.

# `values` must be size 1 or same size as the `conditions`

    Code
      vec_case_when(list(c(TRUE, FALSE, TRUE, TRUE)), list(1:3))
    Condition
      Error in `vec_case_when()`:
      ! Can't recycle `values[[1]]` (size 3) to size 4.

# `default` must be size 1 or same size as `conditions` (exact same as any other `values` input)

    Code
      vec_case_when(list(FALSE), list(1L), default = 2:3)
    Condition
      Error in `vec_case_when()`:
      ! Can't recycle `default` (size 2) to size 1.

# `default_arg` can be customized

    Code
      vec_case_when(list(FALSE), list(1L), default = 2:3, default_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! Can't recycle `foo` (size 2) to size 1.

---

    Code
      vec_case_when(list(FALSE), list(1L), default = "x", default_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! Can't combine <integer> and `foo` <character>.

# `conditions_arg` is validated

    Code
      vec_case_when(list("x"), list(1), conditions_arg = 1)
    Condition
      Error in `vec_case_when()`:
      ! `arg` must be a string.

# `values_arg` is validated

    Code
      vec_case_when(list(TRUE), list(lm(1 ~ 1)), values_arg = 1)
    Condition
      Error in `vec_case_when()`:
      ! `arg` must be a string.

# `default_arg` is validated

    Code
      vec_case_when(list(TRUE), list(1), default = "x", default_arg = 1)
    Condition
      Error in `vec_case_when()`:
      ! `arg` must be a string.

# `conditions` must all be the same size

    Code
      vec_case_when(list(c(TRUE, FALSE), TRUE), list(1, 2))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[2]]` must have size 2, not size 1.

---

    Code
      vec_case_when(list(c(TRUE, FALSE), c(TRUE, FALSE, TRUE)), list(1, 2))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[2]]` must have size 2, not size 3.

# `conditions` must be logical (and aren't cast to logical!)

    Code
      vec_case_when(list(1), list(2))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must be a logical vector, not the number 1.

---

    Code
      vec_case_when(list(TRUE, 3.5), list(2, 4))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[2]]` must be a logical vector, not the number 3.5.

---

    Code
      vec_case_when(list(x), list(1), default = 2)
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must be a logical vector, not a <my_logical> object.

# `conditions` can't be arrays (#6862)

    Code
      vec_case_when(list(x), list(y))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must be a logical vector, not a logical matrix.

---

    Code
      vec_case_when(list(x), list(y), size = 3)
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must be a logical vector, not a logical matrix.

---

    Code
      vec_case_when(list(x), list(y))
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must be a logical vector, not a logical 1D array.

# `size` overrides the `conditions` sizes

    Code
      vec_case_when(list(TRUE), list(1), size = 5)
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[1]]` must have size 5, not size 1.

---

    Code
      vec_case_when(list(c(TRUE, FALSE), c(TRUE, FALSE, TRUE)), list(1, 2), size = 2)
    Condition
      Error in `vec_case_when()`:
      ! `conditions[[2]]` must have size 2, not size 3.

# `ptype` overrides the `values` types

    Code
      vec_case_when(list(FALSE, TRUE), list(1, 2), ptype = character())
    Condition
      Error in `vec_case_when()`:
      ! Can't convert `values[[2]]` <double> to <character>.

# number of `conditions` and `values` must be the same

    Code
      vec_case_when(list(TRUE), list())
    Condition
      Error in `vec_case_when()`:
      ! `conditions` must have size 0, not size 1.

---

    Code
      vec_case_when(list(TRUE, TRUE), list(1))
    Condition
      Error in `vec_case_when()`:
      ! `conditions` must have size 1, not size 2.

# dots must be empty

    Code
      vec_case_when(list(TRUE), list(1), 2)
    Condition
      Error in `vec_case_when()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 2
      i Did you forget to name an argument?

# `conditions` must be a list

    Code
      vec_case_when(1, list(2))
    Condition
      Error in `vec_case_when()`:
      ! `conditions` must be a list, not the number 1.

# `values` must be a list

    Code
      vec_case_when(list(TRUE), 1)
    Condition
      Error in `vec_case_when()`:
      ! `values` must be a list, not the number 1.

# named inputs show up in the error message

    Code
      vec_case_when(list(x = 1.5), list(1))
    Condition
      Error in `vec_case_when()`:
      ! `conditions$x` must be a logical vector, not the number 1.5.

---

    Code
      vec_case_when(list(x = 1.5), list(1), conditions_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! `foo$x` must be a logical vector, not the number 1.5.

---

    Code
      vec_case_when(list(x = 1.5), list(1), conditions_arg = "")
    Condition
      Error in `vec_case_when()`:
      ! `x` must be a logical vector, not the number 1.5.

---

    Code
      vec_case_when(list(x = TRUE, y = c(TRUE, FALSE)), list(1, 2))
    Condition
      Error in `vec_case_when()`:
      ! `conditions$y` must have size 1, not size 2.

---

    Code
      vec_case_when(list(x = TRUE, y = c(TRUE, FALSE)), list(1, 2), conditions_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! `foo$y` must have size 1, not size 2.

---

    Code
      vec_case_when(list(x = TRUE, y = c(TRUE, FALSE)), list(1, 2), conditions_arg = "")
    Condition
      Error in `vec_case_when()`:
      ! `y` must have size 1, not size 2.

---

    Code
      vec_case_when(list(TRUE, FALSE), list(1, x = "y"))
    Condition
      Error in `vec_case_when()`:
      ! Can't combine `values[[1]]` <double> and `values$x` <character>.

---

    Code
      vec_case_when(list(TRUE, FALSE), list(1, x = "y"), values_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! Can't combine `foo[[1]]` <double> and `foo$x` <character>.

---

    Code
      vec_case_when(list(TRUE, FALSE), list(1, x = "y"), values_arg = "")
    Condition
      Error in `vec_case_when()`:
      ! Can't combine `..1` <double> and `x` <character>.

---

    Code
      vec_case_when(list(TRUE), list(NULL))
    Condition
      Error in `vec_case_when()`:
      ! `values[[1]]` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_case_when(list(TRUE), list(x = NULL))
    Condition
      Error in `vec_case_when()`:
      ! `values$x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_case_when(list(TRUE), list(NULL), values_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! `foo[[1]]` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_case_when(list(TRUE), list(x = NULL), values_arg = "foo")
    Condition
      Error in `vec_case_when()`:
      ! `foo$x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_case_when(list(TRUE), list(NULL), values_arg = "")
    Condition
      Error in `vec_case_when()`:
      ! `..1` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_case_when(list(TRUE), list(x = NULL), values_arg = "")
    Condition
      Error in `vec_case_when()`:
      ! `x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# `unmatched` errors are correct

    Code
      vec_case_when(conditions, values, unmatched = "error")
    Condition
      Error in `vec_case_when()`:
      ! Each location must be matched.
      x Locations 4, 5, 6, and 7 are unmatched.

# allows for name repair

    Code
      vec_interleave(x, x, .name_repair = "unique")
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
      x...1 x...2 
          1     1 

# can repair names quietly

    Code
      res_unique <- vec_interleave(c(x = 1), c(x = 2), .name_repair = "unique_quiet")
      res_universal <- vec_interleave(c(`if` = 1), c(`in` = 2), .name_repair = "universal_quiet")

# reports type errors

    Code
      vec_interleave(1, "x")
    Condition
      Error in `vec_interleave()`:
      ! Can't combine `..1` <double> and `..2` <character>.

---

    Code
      vec_interleave(1, "x", .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! Can't combine `..1` <double> and `..2` <character>.

---

    Code
      vec_interleave(1, "x", .ptype = double())
    Condition
      Error in `vec_interleave()`:
      ! Can't convert `..2` <character> to <double>.

---

    Code
      vec_interleave(1, "x", .ptype = double(), .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! Can't convert `..2` <character> to <double>.

---

    Code
      vec_interleave(1, NULL, "x")
    Condition
      Error in `vec_interleave()`:
      ! Can't combine `..1` <double> and `..3` <character>.

---

    Code
      vec_interleave(1, NULL, "x", .ptype = double())
    Condition
      Error in `vec_interleave()`:
      ! Can't convert `..3` <character> to <double>.

# reports recycling errors

    Code
      vec_interleave(1:2, 1:3)
    Condition
      Error in `vec_interleave()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

---

    Code
      vec_interleave(1:2, 1:3, .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

---

    Code
      vec_interleave(1:2, 3:4, .size = 3)
    Condition
      Error in `vec_interleave()`:
      ! Can't recycle `..1` (size 2) to size 3.

---

    Code
      vec_interleave(1:2, 3:4, .size = 3, .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! Can't recycle `..1` (size 2) to size 3.

---

    Code
      vec_interleave(1:2, NULL, 1:3)
    Condition
      Error in `vec_interleave()`:
      ! Can't recycle `..1` (size 2) to match `..3` (size 3).

---

    Code
      vec_interleave(1:2, NULL, 1:3, .size = 2)
    Condition
      Error in `vec_interleave()`:
      ! Can't recycle `..3` (size 3) to size 2.

# reports scalar errors

    Code
      vec_interleave(lm(1 ~ 1))
    Condition
      Error in `vec_interleave()`:
      ! `..1` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_interleave(lm(1 ~ 1), .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `..1` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_interleave(1, NULL, lm(1 ~ 1))
    Condition
      Error in `vec_interleave()`:
      ! `..3` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_interleave(1, NULL, lm(1 ~ 1), .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `..3` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `list_interleave()` checks for a list

    Code
      list_interleave(1)
    Condition
      Error in `list_interleave()`:
      ! `1` must be a list, not the number 1.

# errors on scalars

    Code
      vec_identify_runs(foobar())
    Condition
      Error in `vec_identify_runs()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_run_sizes(foobar())
    Condition
      Error in `vec_run_sizes()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# vec_locate_run_bounds() validates `which`

    Code
      vec_locate_run_bounds(1, which = "x")
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `which` must be one of "start" or "end", not "x".

---

    Code
      vec_locate_run_bounds(1, which = 1)
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `which` must be a string or character vector.

---

    Code
      vec_locate_run_bounds(1, which = c("foo", "bar"))
    Condition
      Error in `vec_locate_run_bounds()`:
      ! `arg` must be length 1 or a permutation of `c("start", "end")`.

# vec_detect_run_bounds() validates `which`

    Code
      vec_detect_run_bounds(1, which = "x")
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `which` must be one of "start" or "end", not "x".

---

    Code
      vec_detect_run_bounds(1, which = 1)
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `which` must be a string or character vector.

---

    Code
      vec_detect_run_bounds(1, which = c("foo", "bar"))
    Condition
      Error in `vec_detect_run_bounds()`:
      ! `arg` must be length 1 or a permutation of `c("start", "end")`.

# assign throws error with non-vector `value`

    Code
      vec_assign(x, 1L, NULL)
    Condition
      Error in `vec_assign()`:
      ! Input must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_assign(x, 1L, NULL, slice_value = TRUE)
    Condition
      Error in `vec_assign()`:
      ! Input must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_assign(x, 1L, NULL, value_arg = "foo")
    Condition
      Error in `vec_assign()`:
      ! `foo` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_assign(x, 1L, NULL, slice_value = TRUE, value_arg = "foo")
    Condition
      Error in `vec_assign()`:
      ! `foo` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_assign(x, 1L, environment(), value_arg = "foo")
    Condition
      Error in `vec_assign()`:
      ! `foo` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_assign(x, 1L, environment(), slice_value = TRUE, value_arg = "foo")
    Condition
      Error in `vec_assign()`:
      ! `foo` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# can assign using logical index

    Code
      (expect_error(vec_assign(x, c(TRUE, FALSE, TRUE), 5), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 2, not 3.

---

    Code
      (expect_error(vec_assign(x, c(TRUE, FALSE, TRUE), 5, slice_value = TRUE),
      class = "vctrs_error_subscript_size"))
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 2, not 3.

---

    Code
      (expect_error(vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 32, not 2.

---

    Code
      (expect_error(vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ], slice_value = TRUE),
      class = "vctrs_error_subscript_size"))
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 32, not 2.

# assign `value` size depends on `slice_value`

    Code
      vec_assign(x, c(TRUE, NA, FALSE), c(1, 2, 3))
    Condition
      Error in `vec_assign()`:
      ! Can't recycle input of size 3 to size 2.

---

    Code
      vec_assign(x, c(TRUE, NA, FALSE), c(1, 2), slice_value = TRUE)
    Condition
      Error in `vec_assign()`:
      ! Can't recycle input of size 2 to size 3.

# can use names to assign with a named object

    Code
      vec_assign(x, c("c", "a"), c(4, 5, 6))
    Condition
      Error in `vec_assign()`:
      ! Can't recycle input of size 3 to size 2.

---

    Code
      vec_assign(x, c("c", "a"), c(4, 5), slice_value = TRUE)
    Condition
      Error in `vec_assign()`:
      ! Can't recycle input of size 2 to size 3.

# `vec_assign()` requires recyclable value

    Code
      (expect_error(vec_assign(1:3, 1:2, 1:3), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_assign()`:
      ! Can't recycle input of size 3 to size 2.

---

    Code
      (expect_error(vec_assign(1:3, 1:2, 1:2, slice_value = TRUE), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_assign()`:
      ! Can't recycle input of size 2 to size 3.

# logical subscripts must match size of indexed vector

    Code
      (expect_error(vec_assign(1:2, c(TRUE, FALSE, TRUE), 5), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 2, not 3.

---

    Code
      (expect_error(vec_assign(mtcars, c(TRUE, FALSE), mtcars[1, ]), class = "vctrs_error_subscript_size")
      )
    Output
      <error/vctrs_error_subscript_size>
      Error:
      ! Can't assign elements.
      x Logical subscript must be size 1 or 32, not 2.

# must assign existing elements

    Code
      (expect_error(vec_assign(1:3, 5, 10), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't assign to elements past the end.
      i Location 5 doesn't exist.
      i There are only 3 elements.
    Code
      (expect_error(vec_assign(1:3, "foo", 10), "unnamed vector"))
    Output
      <error/rlang_error>
      Error in `vec_assign()`:
      ! Can't use character names to index an unnamed vector.
    Code
      (expect_error(vec_slice(letters, -100) <- "foo", class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't negate elements past the end.
      i Location 100 doesn't exist.
      i There are only 26 elements.
    Code
      (expect_error(vec_assign(set_names(letters), "foo", "bar"), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't assign to elements that don't exist.
      x Element `foo` doesn't exist.

# must assign with proper negative locations

    Code
      (expect_error(vec_assign(1:3, c(-1, 1), 1:2), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't assign elements.
      x Negative and positive locations can't be mixed.
      i Subscript has a positive value at location 2.
    Code
      (expect_error(vec_assign(1:3, c(-1, NA), 1:2), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't assign elements.
      x Negative locations can't have missing values.
      i Subscript has a missing value at location 2.

# `vec_assign()` error args can be overridden

    Code
      (expect_error(vec_assign(1:2, 1L, "x", x_arg = "foo", value_arg = "bar"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `vec_assign()`:
      ! Can't convert `bar` <character> to match type of `foo` <integer>.
    Code
      (expect_error(vec_assign(1:2, 1L, 1:2, value_arg = "bar"), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_assign()`:
      ! Can't recycle `bar` (size 2) to size 1.

# `unmatched` errors are correct

    Code
      vec_recode_values(c(1, 2), from = 1, to = 0, unmatched = "error")
    Condition
      Error in `vec_recode_values()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      vec_recode_values(c(1, NA), from = 1, to = 0, unmatched = "error")
    Condition
      Error in `vec_recode_values()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      vec_recode_values(1:100, from = 1, to = 0, unmatched = "error")
    Condition
      Error in `vec_recode_values()`:
      ! Each location must be matched.
      x Locations 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, ..., 99, and 100 are unmatched.

# `x` and `from` common type errors are correct

    Code
      vec_recode_values(1, from = "a", to = 1)
    Condition
      Error in `vec_recode_values()`:
      ! Can't convert `from` <character> to match type of `x` <double>.

---

    Code
      vec_recode_values(1, from = list("a"), to = 1, from_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't convert `from[[1]]` <character> to <double>.

# `to` and `default` `ptype` errors are correct when it is inferred

    Code
      vec_recode_values(1, from = 1:2, to = list(1, "x"), to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't combine `to[[1]]` <double> and `to[[2]]` <character>.

---

    Code
      vec_recode_values(1, from = 1:2, to = list(1, 2), default = "x",
      to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't combine `default` <character> and <double>.

---

    Code
      vec_recode_values(1, from = 1:2, to = 1, default = "x")
    Condition
      Error in `vec_recode_values()`:
      ! Can't combine `to` <double> and `default` <character>.

# `to` and `default` `ptype` errors are correct when it is user supplied

    Code
      vec_recode_values(1, from = 1, to = 1, ptype = foobar())
    Condition
      Error in `vec_recode_values()`:
      ! `ptype` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_recode_values(1, from = 1, to = 1, ptype = character())
    Condition
      Error in `vec_recode_values()`:
      ! Can't convert `to` <double> to <character>.

---

    Code
      vec_recode_values(1, from = 1, to = list(a = 1), ptype = character(),
      to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't convert `to$a` <double> to <character>.

---

    Code
      vec_recode_values(1, from = 1, to = "x", default = 1, ptype = character())
    Condition
      Error in `vec_recode_values()`:
      ! Can't convert `default` <double> to <character>.

# `to` size is validated

    Code
      vec_recode_values(1:5, from = 1, to = 2:3)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `to` (size 2) to size 1.

---

    Code
      vec_recode_values(1:5, from = list(1), to = 2:3, from_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `to` (size 2) to size 1.

---

    Code
      vec_recode_values(1:5, from = 1, to = list(2, 3), to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `to` (size 2) to size 1.

---

    Code
      vec_recode_values(1:5, from = list(1), to = list(2, 3),
      from_as_list_of_vectors = TRUE, to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `to` (size 2) to size 1.

---

    Code
      vec_recode_values(1:5, from = 1, to = list(a = 2:3), to_as_list_of_vectors = TRUE)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `to$a` (size 2) to size 5.

# `default` size is validated

    Code
      vec_recode_values(1:5, from = 1, to = 2, default = 1:2)
    Condition
      Error in `vec_recode_values()`:
      ! Can't recycle `default` (size 2) to size 5.

# `x` must be a vector

    Code
      vec_recode_values(foobar(), from = 1, to = 2, x_arg = ".x")
    Condition
      Error in `vec_recode_values()`:
      ! `.x` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `from` must be a vector or list of vectors

    Code
      vec_recode_values(1, from = foobar(), to = 2, from_arg = ".from")
    Condition
      Error in `vec_recode_values()`:
      ! `.from` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_recode_values(1, from = 1, to = 2, from_as_list_of_vectors = TRUE,
        from_arg = ".from")
    Condition
      Error in `vec_recode_values()`:
      ! `.from` must be a list, not the number 1.

---

    Code
      vec_recode_values(1, from = list(a = foobar()), to = 2,
      from_as_list_of_vectors = TRUE, from_arg = ".from")
    Condition
      Error in `vec_recode_values()`:
      ! `.from$a` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `to` must be a vector or list of vectors

    Code
      vec_recode_values(1, from = 1, to = foobar(), to_arg = ".to")
    Condition
      Error in `vec_recode_values()`:
      ! `.to` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_recode_values(1, from = 1, to = 2, to_as_list_of_vectors = TRUE, to_arg = ".to")
    Condition
      Error in `vec_recode_values()`:
      ! `.to` must be a list, not the number 2.

---

    Code
      vec_recode_values(1, from = 1, to = list(a = foobar()), to_as_list_of_vectors = TRUE,
      to_arg = ".to")
    Condition
      Error in `vec_recode_values()`:
      ! `.to$a` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `default` must be a vector

    Code
      vec_recode_values(1, from = 1, to = 2, default = foobar(), default_arg = ".default")
    Condition
      Error in `vec_recode_values()`:
      ! `.default` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `from_as_list_of_vectors` and `to_as_list_of_vectors` are validated

    Code
      vec_recode_values(1, from = 1, to = 1, from_as_list_of_vectors = "x")
    Condition
      Error in `vec_recode_values()`:
      ! `from_as_list_of_vectors` must be `TRUE` or `FALSE`.

---

    Code
      vec_recode_values(1, from = 1, to = 1, to_as_list_of_vectors = "x")
    Condition
      Error in `vec_recode_values()`:
      ! `to_as_list_of_vectors` must be `TRUE` or `FALSE`.

---

    Code
      vec_replace_values(1, from = 1, to = 1, from_as_list_of_vectors = "x")
    Condition
      Error in `vec_replace_values()`:
      ! `from_as_list_of_vectors` must be `TRUE` or `FALSE`.

---

    Code
      vec_replace_values(1, from = 1, to = 1, to_as_list_of_vectors = "x")
    Condition
      Error in `vec_replace_values()`:
      ! `to_as_list_of_vectors` must be `TRUE` or `FALSE`.

# `unmatched` is validated

    Code
      vec_recode_values(1, from = 1, to = 1, unmatched = "e")
    Condition
      Error in `vec_recode_values()`:
      ! `unmatched` must be either "default" or "error", not "e".

# no casting is done

    Code
      vec_pall(1)
    Condition
      Error in `vec_pall()`:
      ! `..1` must be a logical vector, not the number 1.

---

    Code
      vec_pany(1)
    Condition
      Error in `vec_pany()`:
      ! `..1` must be a logical vector, not the number 1.

---

    Code
      vec_pall(array(TRUE))
    Condition
      Error in `vec_pall()`:
      ! `..1` must be a logical vector, not a logical 1D array.

---

    Code
      vec_pany(array(TRUE))
    Condition
      Error in `vec_pany()`:
      ! `..1` must be a logical vector, not a logical 1D array.

---

    Code
      vec_pall(structure(TRUE, class = "foo"))
    Condition
      Error in `vec_pall()`:
      ! `..1` must be a logical vector, not a <foo> object.

---

    Code
      vec_pany(structure(TRUE, class = "foo"))
    Condition
      Error in `vec_pany()`:
      ! `..1` must be a logical vector, not a <foo> object.

# no recycling is done

    Code
      vec_pall(TRUE, c(TRUE, TRUE, TRUE))
    Condition
      Error in `vec_pall()`:
      ! `..2` must have size 1, not size 3.

---

    Code
      vec_pany(TRUE, c(TRUE, TRUE, TRUE))
    Condition
      Error in `vec_pany()`:
      ! `..2` must have size 1, not size 3.

---

    Code
      vec_pall(TRUE, c(TRUE, TRUE, TRUE), .size = 3L)
    Condition
      Error in `vec_pall()`:
      ! `..1` must have size 3, not size 1.

---

    Code
      vec_pany(TRUE, c(TRUE, TRUE, TRUE), .size = 3L)
    Condition
      Error in `vec_pany()`:
      ! `..1` must have size 3, not size 1.

# validates `.missing`

    Code
      vec_pall(.missing = c(TRUE, FALSE))
    Condition
      Error in `vec_pall()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

---

    Code
      vec_pany(.missing = c(TRUE, FALSE))
    Condition
      Error in `vec_pany()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

---

    Code
      vec_pall(.missing = 1)
    Condition
      Error in `vec_pall()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

---

    Code
      vec_pany(.missing = 1)
    Condition
      Error in `vec_pany()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

---

    Code
      vec_pall(.missing = NULL)
    Condition
      Error in `vec_pall()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

---

    Code
      vec_pany(.missing = NULL)
    Condition
      Error in `vec_pany()`:
      ! `.missing` must be `NA`, `FALSE`, or `TRUE`.

# validates `.size`

    Code
      vec_pall(.size = c(1, 2))
    Condition
      Error in `vec_pall()`:
      ! `.size` must be a scalar integer or double.

---

    Code
      vec_pany(.size = c(1, 2))
    Condition
      Error in `vec_pany()`:
      ! `.size` must be a scalar integer or double.

---

    Code
      vec_pall(.size = 1.5)
    Condition
      Error in `vec_pall()`:
      ! `.size` must be a whole number, not a decimal number.

---

    Code
      vec_pany(.size = 1.5)
    Condition
      Error in `vec_pany()`:
      ! `.size` must be a whole number, not a decimal number.

---

    Code
      vec_pall(.size = NA_integer_)
    Condition
      Error in `vec_pall()`:
      ! negative length vectors are not allowed

---

    Code
      vec_pany(.size = NA_integer_)
    Condition
      Error in `vec_pany()`:
      ! negative length vectors are not allowed

# names are used in errors

    Code
      vec_pall(1.5, .arg = "x")
    Condition
      Error in `vec_pall()`:
      ! `x[[1]]` must be a logical vector, not the number 1.5.

---

    Code
      vec_pall(a = 1.5, .arg = "x")
    Condition
      Error in `vec_pall()`:
      ! `x$a` must be a logical vector, not the number 1.5.

---

    Code
      x <- c(TRUE, FALSE)
      y <- logical()
      vec_pany(x, y)
    Condition
      Error in `vec_pany()`:
      ! `..2` must have size 2, not size 0.

---

    Code
      x <- c(TRUE, FALSE)
      y <- logical()
      vec_pany(a = x, b = y, .arg = "x", .error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `x$b` must have size 2, not size 0.

# `x` must not be `NULL` (#1823, #1967)

    Code
      vec_rank(NULL)
    Condition
      Error:
      ! `x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_rank(NULL, incomplete = "na")
    Condition
      Error:
      ! `x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_rank(NULL, ties = "sequential", incomplete = "na")
    Condition
      Error:
      ! `x` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# `ties` is validated

    Code
      vec_rank(1, ties = "foo")
    Condition
      Error in `vec_rank()`:
      ! `ties` must be one of "min", "max", "sequential", or "dense", not "foo".

---

    Code
      vec_rank(1, ties = 1)
    Condition
      Error in `vec_rank()`:
      ! `ties` must be a string or character vector.

# `incomplete` is validated

    Code
      vec_rank(1, incomplete = NA)
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be a string or character vector.

---

    Code
      vec_rank(1, incomplete = c(TRUE, FALSE))
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be a string or character vector.

---

    Code
      vec_rank(1, incomplete = "foo")
    Condition
      Error in `vec_rank()`:
      ! `incomplete` must be one of "rank" or "na", not "foo".

# obj_check_vector() errors on scalars

    Code
      obj_check_vector(quote(foo))
    Condition
      Error:
      ! `quote(foo)` must be a vector, not a symbol.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      obj_check_vector(foobar())
    Condition
      Error:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# obj_check_vector() error respects `arg` and `call`

    Code
      my_check_vector(foobar())
    Condition
      Error in `my_check_vector()`:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# obj_check_vector() error contains FAQ links and correct bullets

    Code
      obj_check_vector(x)
    Condition
      Error:
      ! `x` must be a vector, not an expression vector.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      obj_check_vector(x)
    Condition
      Error:
      ! `x` must be a vector, not a <my_list> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <my_list>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      obj_check_vector(x)
    Condition
      Error:
      ! `x` must be a vector, not a <data.frame> object.
      x Detected incompatible data frame structure. A data frame is normally treated as a vector, but an incompatible class ordering was detected. To be compatible, the subclass <my_df> must come before <data.frame>, not after. Class: <data.frame/my_df>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# assertion failures are explained

    Code
      vec_assert(lgl(), chr())
    Condition
      Error:
      ! `lgl()` must be a vector with type <character>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), factor())
    Condition
      Error:
      ! `lgl()` must be a vector with type <factor<>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), factor(levels = "foo"))
    Condition
      Error:
      ! `lgl()` must be a vector with type <factor<c1562>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(factor(levels = "bar"), factor(levels = "foo"))
    Condition
      Error:
      ! `factor(levels = "bar")` must be a vector with type <factor<c1562>>.
      Instead, it has type <factor<9f154>>.

---

    Code
      vec_assert(factor(), chr())
    Condition
      Error:
      ! `factor()` must be a vector with type <character>.
      Instead, it has type <factor<>>.

---

    Code
      vec_assert(lgl(), data.frame())
    Condition
      Error:
      ! `lgl()` must be a vector with type <data.frame<>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), data.frame(x = 1))
    Condition
      Error:
      ! `lgl()` must be a vector with type <data.frame<x:double>>.
      Instead, it has type <logical>.

---

    Code
      vec_assert(lgl(), data.frame(x = 1, y = 2))
    Condition
      Error:
      ! `lgl()` must be a vector with type:
      
        <data.frame<
          x: double
          y: double
        >>
      
      Instead, it has type <logical>.

---

    Code
      vec_assert(data.frame(), chr())
    Condition
      Error:
      ! `data.frame()` must be a vector with type <character>.
      Instead, it has type <data.frame<>>.

---

    Code
      vec_assert(data.frame(x = 1), chr())
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type <character>.
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1), data.frame(x = "foo"))
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type <data.frame<x:character>>.
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1), data.frame(x = "foo", y = 2))
    Condition
      Error:
      ! `data.frame(x = 1)` must be a vector with type:
      
        <data.frame<
          x: character
          y: double
        >>
      
      Instead, it has type <data.frame<x:double>>.

---

    Code
      vec_assert(data.frame(x = 1, y = 2), chr())
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type <character>.
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

---

    Code
      vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo"))
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type <data.frame<x:character>>.
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

---

    Code
      vec_assert(data.frame(x = 1, y = 2), data.frame(x = "foo", y = 2))
    Condition
      Error:
      ! `data.frame(x = 1, y = 2)` must be a vector with type:
      
        <data.frame<
          x: character
          y: double
        >>
      
      Instead, it has type:
      
        <data.frame<
          x: double
          y: double
        >>

# vec_assert() validates `size` (#1470)

    Code
      (expect_error(vec_assert(1, size = c(2, 3))))
    Output
      <error/rlang_error>
      Error in `vec_assert()`:
      ! `size` must be length 1, not length 2.
    Code
      (expect_error(vec_assert(1, size = 1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_assert()`:
      ! Can't convert from `size` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(vec_assert(1, size = "x")))
    Output
      <error/vctrs_error_cast>
      Error in `vec_assert()`:
      ! Can't convert `size` <character> to <integer>.

# vec_check_size() errors on the wrong size

    Code
      vec_check_size(1:5, size = 1L)
    Condition
      Error:
      ! `1:5` must have size 1, not size 5.

---

    Code
      vec_check_size(1:5, size = 10L)
    Condition
      Error:
      ! `1:5` must have size 10, not size 5.

# vec_check_size() errors on scalars

    Code
      vec_check_size(quote(foo), size = 1L)
    Condition
      Error:
      ! `quote(foo)` must be a vector, not a symbol.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_check_size(foobar(), size = 1L)
    Condition
      Error:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# vec_check_size() error respects `arg` and `call`

    Code
      my_check_size(1L, size = 5L)
    Condition
      Error in `my_check_size()`:
      ! `foo` must have size 5, not size 1.

---

    Code
      my_check_size(foobar(), size = 5L)
    Condition
      Error in `my_check_size()`:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# vec_check_size() validates `size`

    Code
      vec_check_size(1, size = "x")
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_size(1, size = c(1L, 2L))
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_size(1, size = 1.5)
    Condition
      Error in `vec_check_size()`:
      ! `size` must be a whole number, not a decimal number.

# vec_check_recyclable() errors on the wrong size

    Code
      vec_check_recyclable(1:5, size = 1L)
    Condition
      Error:
      ! Can't recycle `1:5` (size 5) to size 1.

---

    Code
      vec_check_recyclable(1:5, size = 10L)
    Condition
      Error:
      ! Can't recycle `1:5` (size 5) to size 10.

# vec_check_recyclable() errors on scalars

    Code
      vec_check_recyclable(quote(foo), size = 1L)
    Condition
      Error:
      ! `quote(foo)` must be a vector, not a symbol.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      vec_check_recyclable(foobar(), size = 1L)
    Condition
      Error:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# vec_check_recyclable() error respects `arg` and `call`

    Code
      my_check_recyclable(1:2, size = 5L)
    Condition
      Error in `my_check_recyclable()`:
      ! Can't recycle `foo` (size 2) to size 5.

---

    Code
      my_check_recyclable(foobar(), size = 5L)
    Condition
      Error in `my_check_recyclable()`:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# vec_check_recyclable() validates `size`

    Code
      vec_check_recyclable(1, size = "x")
    Condition
      Error in `vec_check_recyclable()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_recyclable(1, size = c(1L, 2L))
    Condition
      Error in `vec_check_recyclable()`:
      ! `size` must be a scalar integer or double.

---

    Code
      vec_check_recyclable(1, size = 1.5)
    Condition
      Error in `vec_check_recyclable()`:
      ! `size` must be a whole number, not a decimal number.

# list_all_vectors() works

    Code
      (expect_error(list_all_vectors(env())))
    Output
      <error/rlang_error>
      Error in `list_all_vectors()`:
      ! `x` must be a list, not an environment.

# obj_check_list() works

    Code
      my_function <- (function(my_arg) obj_check_list(my_arg))
      (expect_error(my_function(env())))
    Output
      <error/rlang_error>
      Error in `my_function()`:
      ! `my_arg` must be a list, not an environment.

# obj_check_list() uses a special error when `arg` is the empty string (#1604)

    Code
      obj_check_list(1, arg = "")
    Condition
      Error:
      ! Input must be a list, not the number 1.

# obj_check_list() and list_check_all_vectors() work

    Code
      my_function <- (function(my_arg) list_check_all_vectors(my_arg))
      (expect_error(my_function(env())))
    Output
      <error/rlang_error>
      Error in `list_check_all_vectors()`:
      ! `x` must be a list, not an environment.
    Code
      (expect_error(my_function(list(1, env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg[[2]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(my_function(list(1, name = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg$name` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(my_function(list(1, foo = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg$foo` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# list_check_all_size() works

    Code
      my_function <- (function(my_arg, size) list_check_all_size(my_arg, size))
      (expect_error(list_check_all_size(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error:
      ! `list(1:2, 1:3)[[2]]` must have size 2, not size 3.
    Code
      (expect_error(my_function(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `my_arg[[2]]` must have size 2, not size 3.
    Code
      (expect_error(my_function(list(NULL, 1:2), 2)))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `my_arg[[1]]` must have size 2, not size 0.

# list_check_all_recyclable() works

    Code
      my_function <- (function(my_arg, size) {
        list_check_all_recyclable(my_arg, size)
      })
      (expect_error(list_check_all_recyclable(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `list(1:2, 1:3)[[2]]` (size 3) to size 2.
    Code
      (expect_error(my_function(list(1:2, 1:3), 2)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg[[2]]` (size 3) to size 2.
    Code
      (expect_error(my_function(list(NULL, 1:2), 2)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg[[1]]` (size 0) to size 2.

# list_all_size() and list_check_all_size() error on scalars

    Code
      (expect_error(list_all_size(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_all_size()`:
      ! `x[[1]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      my_function <- (function(my_arg, size) list_check_all_size(my_arg, size))
      (expect_error(my_function(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg[[1]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# list_all_recyclable() and list_check_all_recyclable() error on scalars

    Code
      (expect_error(list_all_recyclable(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_all_recyclable()`:
      ! `x[[1]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      my_function <- (function(my_arg, size) {
        list_check_all_recyclable(my_arg, size)
      })
      (expect_error(my_function(x, 2)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `my_arg[[1]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# list_all_size() and list_check_all_size() throw error using internal call on non-list input

    Code
      (expect_error(list_all_size(1, 2)))
    Output
      <error/rlang_error>
      Error in `list_all_size()`:
      ! `x` must be a list, not the number 1.
    Code
      (expect_error(list_check_all_size(1, 2, arg = "arg", call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `list_check_all_size()`:
      ! `x` must be a list, not the number 1.

# list_all_size() and list_check_all_size() validate `size`

    Code
      (expect_error(list_all_size(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_all_size()`:
      ! `size` must be a scalar integer or double.
    Code
      (expect_error(list_check_all_size(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_check_all_size()`:
      ! `size` must be a scalar integer or double.

# list_all_recyclable() and list_check_all_recyclable() validate `size`

    Code
      (expect_error(list_all_recyclable(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_all_recyclable()`:
      ! `size` must be a scalar integer or double.
    Code
      (expect_error(list_check_all_recyclable(list(), size = "x")))
    Output
      <error/rlang_error>
      Error in `list_check_all_recyclable()`:
      ! `size` must be a scalar integer or double.

# list_check_all_size() works with `allow_null`

    Code
      list_check_all_size(x, size = 1)
    Condition
      Error:
      ! `x[[2]]` must have size 1, not size 0.

---

    Code
      list_check_all_size(x, size = 1)
    Condition
      Error:
      ! `x[[2]]` must have size 1, not size 0.

---

    Code
      list_check_all_size(x, size = 1, allow_null = TRUE)
    Condition
      Error:
      ! `x[[3]]` must have size 1, not size 2.

# list_check_all_vectors() works with `allow_null`

    Code
      list_check_all_vectors(x)
    Condition
      Error:
      ! `x[[2]]` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      list_check_all_vectors(x)
    Condition
      Error:
      ! `x[[2]]` must be a vector, not `NULL`.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

---

    Code
      list_check_all_vectors(x, allow_null = TRUE)
    Condition
      Error:
      ! `x[[3]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# list_check_all_recyclable() works with `allow_null`

    Code
      list_check_all_recyclable(x, size = 2)
    Condition
      Error:
      ! Can't recycle `x[[2]]` (size 0) to size 2.

---

    Code
      list_check_all_recyclable(x, size = 2)
    Condition
      Error:
      ! Can't recycle `x[[2]]` (size 0) to size 2.

---

    Code
      list_check_all_recyclable(x, size = 2, allow_null = TRUE)
    Condition
      Error:
      ! Can't recycle `x[[3]]` (size 3) to size 2.

# informative messages when 1d array doesn't match vector

    Code
      (expect_error(vec_assert(x, int())))
    Output
      <error/vctrs_error_assert_ptype>
      Error:
      ! `x` must be a vector with type <integer>.
      Instead, it has type <integer[1d]>.

# constructor requires list input

    Code
      new_list_of(1)
    Condition
      Error in `new_list_of()`:
      ! `x` must be a list, not the number 1.

---

    Code
      new_list_of(mtcars)
    Condition
      Error in `new_list_of()`:
      ! `x` must be a list, not a <data.frame> object.

# must lock at least one of ptype or size

    Code
      new_list_of(ptype = NULL, size = NULL)
    Condition
      Error in `new_list_of()`:
      ! Must specify at least one of `ptype` or `size`.

# validates `ptype`

    Code
      new_list_of(ptype = lm(1 ~ 1))
    Condition
      Error in `new_list_of()`:
      ! `ptype` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# validates `size`

    Code
      new_list_of(size = 1.1)
    Condition
      Error in `new_list_of()`:
      ! `size` must be a whole number, not the number 1.1.

---

    Code
      new_list_of(size = 1:2)
    Condition
      Error in `new_list_of()`:
      ! `size` must be a whole number, not an integer vector.

---

    Code
      new_list_of(size = -5)
    Condition
      Error in `new_list_of()`:
      ! `size` must be a whole number larger than or equal to 0, not the number -5.

# can check for list of

    Code
      check_list_of(1)
    Condition
      Error:
      ! `1` must be a `<list_of>`, not the number 1.

# errors if can't determine type

    Code
      list_of(.ptype = NULL)
    Condition
      Error in `list_of()`:
      ! Can't find common type for elements of `x`.

---

    Code
      list_of(1, "a")
    Condition
      Error in `list_of()`:
      ! Can't combine `..1` <double> and `..2` <character>.

---

    Code
      as_list_of(list())
    Condition
      Error in `as_list_of()`:
      ! Can't find common type for elements of `x`.

# errors if can't determine size

    Code
      list_of(.ptype = zap(), .size = NULL)
    Condition
      Error in `list_of()`:
      ! Can't find common size for elements of `x`.

---

    Code
      list_of(1:2, 3:5, .ptype = zap(), .size = NULL)
    Condition
      Error in `list_of()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

---

    Code
      as_list_of(list(), .ptype = integer(), .size = NULL)
    Condition
      Error in `as_list_of()`:
      ! Can't find common size for elements of `x`.

# `list_of_size()` validates `x`

    Code
      list_of_size(list(1))
    Condition
      Error in `list_of_size()`:
      ! `x` must be a `<list_of>`, not a list.

# `list_of_ptype()` validates `x`

    Code
      list_of_ptype(list(1))
    Condition
      Error in `list_of_ptype()`:
      ! `x` must be a `<list_of>`, not a list.

# can print empty list-of

    Code
      list_of(.ptype = integer(), .size = 5L)
    Output
      <list_of<integer[5]>[0]>

# print method gives human friendly output

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<double>

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<dbl>

---

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<any[2]>

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<any[2]>

---

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<double[2]>

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<dbl[2]>

# print method gives human friendly output for multi line types

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<
        data.frame<
          x: integer
          y: double
          z: character
        >
      >

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<df[,3]>

---

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<any[2]>

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<any[2]>

---

    Code
      cat(vec_ptype_full(x))
    Output
      list_of<
        data.frame<
          x: integer
          y: double
          z: character
        >[2]
      >

---

    Code
      cat(vec_ptype_abbr(x))
    Output
      list<df[,3][2]>

# str method is reasonably correct

    Code
      str(x)
    Output
      list<dbl> [1:2] 
      $ : num 1
      $ : num [1:2] 2 3
      @ ptype: num(0) 

---

    Code
      str(list(list(x, y = 2:1)))
    Output
      List of 1
       $ :List of 2
        ..$  : list<dbl> [1:2] 
        .. ..$ : num 1
        .. ..$ : num [1:2] 2 3
        .. ..@ ptype: num(0) 
        ..$ y: int [1:2] 2 1

---

    Code
      str(x[0])
    Output
      list<dbl> [1:0] 
       list()
      @ ptype: num(0) 

---

    Code
      str(list(list(x[0], y = 2:1)))
    Output
      List of 1
       $ :List of 2
        ..$  : list<dbl> [1:0] 
       list()
        .. ..@ ptype: num(0) 
        ..$ y: int [1:2] 2 1

# [[ works

    Code
      x[[3]]
    Condition
      Error:
      ! Invalid index: out of bounds

---

    Code
      x[["c"]]
    Condition
      Error:
      ! Invalid index: field name 'c' not found

# $ works

    Code
      x$c
    Condition
      Error:
      ! Invalid index: field name 'c' not found

# [<- coerces and recycles

    Code
      x[1] <- list("5")
    Condition
      Error in `lapply()`:
      ! Can't convert `X[[i]]` <character> to <double>.

---

    Code
      x[1] <- list(c(1, 2, 3))
    Condition
      Error in `lapply()`:
      ! Can't recycle input of size 3 to size 2.

# [[<- coerces and recycles

    Code
      x[[1]] <- "5"
    Condition
      Error in `[[<-`:
      ! Can't convert `value` <character> to <double>.

---

    Code
      x[[1]] <- c(1, 2, 3)
    Condition
      Error in `[[<-`:
      ! Can't recycle input of size 3 to size 2.

# $<- coerces and recycles

    Code
      x$a <- "5"
    Condition
      Error in `$<-`:
      ! Can't convert `value` <character> to <double>.

---

    Code
      x$a <- c(1, 2, 3)
    Condition
      Error in `$<-`:
      ! Can't recycle input of size 3 to size 2.

# cast: list to list_of

    Code
      vec_cast(list("x"), to)
    Condition
      Error:
      ! Can't convert `..1` <character> to <double>.

---

    Code
      vec_cast(list(1:3), to)
    Condition
      Error:
      ! Can't recycle `..1` (size 3) to size 2.

---

    Code
      vec_cast(list(1:3), to)
    Condition
      Error:
      ! Can't recycle `..1` (size 3) to size 2.

---

    Code
      vec_cast(list("x"), to)
    Condition
      Error:
      ! Can't convert `..1` <character> to <integer>.

# cast: list_of to list_of

    Code
      vec_cast(x, to)
    Condition
      Error:
      ! Can't convert `..1` <double> to <character>.

---

    Code
      vec_cast(y, to)
    Condition
      Error:
      ! Can't recycle `..1` (size 2) to size 3.

# list coercions are symmetric and unchanging

    Code
      print(mat)
    Output
                         list   list_of<integer>   list_of<double>   list_of<character>  
      list               "list" "list"             "list"            "list"              
      list_of<integer>   "list" "list_of<integer>" "list_of<double>" "list"              
      list_of<double>    "list" "list_of<double>"  "list_of<double>" "list"              
      list_of<character> "list" "list"             "list"            "list_of<character>"

# error call is passed to inner cast methods

    Code
      (expect_error(fn1()))
    Output
      <error/vctrs_error_cast>
      Error in `fn1()`:
      ! Can't convert `..1` <double> to <character>.
    Code
      (expect_error(fn2()))
    Output
      <error/vctrs_error_cast>
      Error in `fn2()`:
      ! Can't convert `..1` <double> to <character>.

# vec_slice throws error with non-vector subscripts

    Code
      (expect_error(vec_slice(1:3, Sys.Date()), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x `i` must be logical, numeric, or character, not a <Date> object.
    Code
      (expect_error(vec_slice(1:3, matrix(TRUE, nrow = 1)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Subscript `i` must be a simple vector, not a matrix.

# can't index beyond the end of a vector

    Code
      (expect_error(vec_slice(1:2, 3L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.
    Code
      (expect_error(vec_slice(1:2, -3L), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `vec_slice()`:
      ! Can't negate elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

# can slice with double indices

    Code
      (expect_error(vec_as_location(2^31, 3L), class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements with `2^31`.
      x Can't convert from `2^31` <double> to <integer> due to loss of precision.

# vec_slice() works with Altrep classes with custom extract methods

    Code
      vec_slice(x, idx)
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Location 16 doesn't exist.
      i There are only 15 elements.

# Unnamed vector with character subscript is caught

    Code
      vec_slice(1:3, letters[1])
    Condition
      Error in `vec_slice()`:
      ! Can't use character names to index an unnamed vector.

# Negative subscripts are checked

    Code
      vec_slice(1:3, -c(1L, NA))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Negative locations can't have missing values.
      i Subscript `i` has a missing value at location 2.

---

    Code
      vec_slice(1:3, c(-1L, 1L))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x Negative and positive locations can't be mixed.
      i Subscript `i` has a positive value at location 2.

# oob error messages are properly constructed

    Code
      vec_slice(c(bar = 1), "foo")
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Element `foo` doesn't exist.

---

    Code
      vec_slice(letters, c(100, 1000))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Locations 100 and 1000 don't exist.
      i There are only 26 elements.

---

    Code
      vec_slice(letters, c(1, 100:103, 2, 104:110))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements past the end.
      i Locations 100, 101, 102, ..., 109, and 110 don't exist.
      i There are only 26 elements.

---

    Code
      vec_slice(set_names(letters), c("foo", "bar"))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Elements `foo` and `bar` don't exist.

---

    Code
      vec_slice(set_names(letters), toupper(letters))
    Condition
      Error in `vec_slice()`:
      ! Can't subset elements that don't exist.
      x Elements `A`, `B`, `C`, `D`, `E`, etc. don't exist.

# vec_init() validates `n`

    Code
      (expect_error(vec_init(1L, 1.5)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a whole number, not a fractional number.
    Code
      (expect_error(vec_init(1L, c(1, 2))))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not a double vector.
    Code
      (expect_error(vec_init(1L, -1L)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a positive number or zero.
    Code
      (expect_error(vec_init(1L, NA)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not `NA`.
    Code
      (expect_error(vec_init(1L, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `vec_init()`:
      ! `n` must be a single number, not an integer `NA`.

# output tests

    Code
      vec_ptype_show()
    Output
      Prototype: NULL

---

    Code
      vec_ptype_show(integer())
    Output
      Prototype: integer

---

    Code
      vec_ptype_show(integer(), double())
    Output
      Prototype: <double>
      0. (           , <integer> ) = <integer>
      1. ( <integer> , <double>  ) = <double> 

---

    Code
      vec_ptype_show(logical(), integer(), double())
    Output
      Prototype: <double>
      0. (           , <logical> ) = <logical>
      1. ( <logical> , <integer> ) = <integer>
      2. ( <integer> , <double>  ) = <double> 

# vec_ptype_common() includes index in argument tag

    Code
      vec_ptype_common(df1, df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `..2$x$y$z` <character>.

---

    Code
      vec_ptype_common(df1, df1, df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `..3$x$y$z` <character>.

---

    Code
      vec_ptype_common(large_df1, large_df2)
    Condition
      Error:
      ! Can't combine `..1$foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar$y$z` <double> and `..2$foobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobarfoobar$y$z` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, baz = FALSE, bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = df1, bar = df2)
    Condition
      Error:
      ! Can't combine `foo$x$y$z` <double> and `bar$x$y$z` <character>.

---

    Code
      vec_ptype_common(df1, df1, bar = df2)
    Condition
      Error:
      ! Can't combine `..1$x$y$z` <double> and `bar$x$y$z` <character>.

---

    Code
      vec_ptype_common(TRUE, !!!list(1, "foo"))
    Condition
      Error:
      ! Can't combine `..2` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(TRUE, !!!list(1, 2), "foo")
    Condition
      Error:
      ! Can't combine `..2` <double> and `..4` <character>.

---

    Code
      vec_ptype_common(1, !!!list(TRUE, FALSE), "foo")
    Condition
      Error:
      ! Can't combine `..1` <double> and `..4` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(FALSE, FALSE), bar = "foo")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = 1, "foo"))
    Condition
      Error:
      ! Can't combine `bar` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = "foo"))
    Condition
      Error:
      ! Can't combine `foo` <logical> and `bar` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), baz = "chr")
    Condition
      Error:
      ! Can't combine `foo` <logical> and `baz` <character>.

---

    Code
      vec_ptype_common(foo = TRUE, !!!list(bar = FALSE), !!!list(baz = "chr"))
    Condition
      Error:
      ! Can't combine `foo` <logical> and `baz` <character>.

# vec_ptype_common() handles spliced names consistently (#1570)

    Code
      vec_ptype_common(a = "foo", b = "bar", y = NULL, z = 1)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.
    Code
      vec_ptype_common(!!!args1, !!!args2)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.
    Code
      vec_ptype_common(!!!args1, "{y_name}" := NULL, "{z_name}" := 1)
    Condition
      Error:
      ! Can't combine `a` <character> and `z` <double>.

# `.finalise` is validated

    Code
      vec_ptype_common(.finalise = 1)
    Condition
      Error in `vec_ptype_common()`:
      ! `.finalise` must be `TRUE` or `FALSE`.

---

    Code
      vec_ptype_common_params(.finalise = 1)
    Condition
      Error in `vec_ptype_common_params()`:
      ! `.finalise` must be `TRUE` or `FALSE`.

# catches `NULL` data frame columns

    Code
      vec_detect_complete(df)
    Condition
      Error in `vec_detect_complete()`:
      ! Unexpected `NULL` column found in a data frame.

# catches scalar objects

    Code
      vec_detect_complete(lm(1 ~ 1))
    Condition
      Error in `vec_size()`:
      ! `x` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# base coercions are symmetric and unchanging

    Code
      mat
    Output
                logical   integer   double   character   raw   list  
      logical   "logical" "integer" "double" NA          NA    NA    
      integer   "integer" "integer" "double" NA          NA    NA    
      double    "double"  "double"  "double" NA          NA    NA    
      character NA        NA        NA       "character" NA    NA    
      raw       NA        NA        NA       NA          "raw" NA    
      list      NA        NA        NA       NA          NA    "list"

# vec_ptype2() data frame methods builds argument tags

    Code
      vec_ptype2("foo", 10)
    Condition
      Error:
      ! Can't combine `"foo"` <character> and `10` <double>.

---

    Code
      df1 <- tibble(x = tibble(y = tibble(z = 1)))
      df2 <- tibble(x = tibble(y = tibble(z = "a")))
      vec_ptype2(df1, df2)
    Condition
      Error:
      ! Can't combine `df1$x$y$z` <double> and `df2$x$y$z` <character>.

# can override scalar vector error message for base scalar types

    Code
      (expect_error(vec_ptype2(NULL, quote(x), y_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a symbol.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(vec_ptype2(quote(x), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a symbol.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# can override scalar vector error message for S3 types

    Code
      (expect_error(vec_ptype2(NULL, foobar(), y_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
    Code
      (expect_error(vec_ptype2(foobar(), NULL, x_arg = "foo"), class = "vctrs_error_scalar_type")
      )
    Output
      <error/vctrs_error_scalar_type>
      Error:
      ! `foo` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# ptype2 and cast errors when same class fallback is impossible are informative

    Code
      (expect_error(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(1, bar = TRUE)` <vctrs_foobar> to <vctrs_foobar>.
    Code
      (expect_error(vec_ptype2(foobar(1, bar = TRUE), foobar(2, baz = TRUE)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `foobar(1, bar = TRUE)` <vctrs_foobar> and `foobar(2, baz = TRUE)` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# Incompatible attributes bullets are not show when methods are implemented

    Code
      with_foobar_cast <- (function(expr) {
        with_methods(vec_cast.vctrs_foobar = function(...) NULL,
        vec_cast.vctrs_foobar.vctrs_foobar = function(x, to, ...) {
          vec_default_cast(x, to, ...)
        }, expr)
      })
      with_foobar_ptype2 <- (function(expr) {
        with_methods(vec_ptype2.vctrs_foobar = function(...) NULL,
        vec_ptype2.vctrs_foobar.vctrs_foobar = function(x, y, ...) {
          vec_default_ptype2(x, y, ...)
        }, expr)
      })
      (expect_error(with_foobar_cast(vec_cast(foobar(1, bar = TRUE), foobar(2, baz = TRUE))),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(1, bar = TRUE)` <vctrs_foobar> to <vctrs_foobar>.
    Code
      (expect_error(with_foobar_ptype2(vec_ptype2(foobar(1, bar = TRUE), foobar(2,
        baz = TRUE))), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error:
      ! Can't combine `foobar(1, bar = TRUE)` <vctrs_foobar> and `foobar(2, baz = TRUE)` <vctrs_foobar>.

# error indexing is correct with unspecifieds

    Code
      vec_ptype_common(1, NA, "x")
    Condition
      Error:
      ! Can't combine `..1` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(NA, 1, "x")
    Condition
      Error:
      ! Can't combine `..2` <double> and `..3` <character>.

---

    Code
      vec_ptype_common(NA, NA, 1, "x")
    Condition
      Error:
      ! Can't combine `..3` <double> and `..4` <character>.

---

    Code
      vec_ptype_common(1, NA, NA, 1, "x")
    Condition
      Error:
      ! Can't combine `..1` <double> and `..5` <character>.

# vec_as_names() validates `repair`

    Code
      (expect_error(my_vec_as_names("x", my_repair = "foo"), "can't be \"foo\""))
    Output
      <error/rlang_error>
      Error in `my_vec_as_names()`:
      ! `my_repair` can't be "foo". See `?vctrs::vec_as_names`.
    Code
      (expect_error(my_vec_as_names(1, my_repair = 1), "string or a function"))
    Output
      <error/rlang_error>
      Error in `my_vec_as_names()`:
      ! `my_repair` must be a string or a function. See `?vctrs::vec_as_names`.

# vec_as_names() checks unique names

    Code
      (expect_error(my_vec_as_names(chr(NA), my_repair = "check_unique")))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.
    Code
      (expect_error(my_vec_as_names(chr(""), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error in `my_vec_as_names()`:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      (expect_error(my_vec_as_names(chr("a", "a"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_vec_as_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `my_repair` to specify repair strategy.
    Code
      (expect_error(my_vec_as_names(chr("..1"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_vec_as_names()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      (expect_error(my_vec_as_names(chr("..."), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_vec_as_names()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# vec_as_names() accepts and checks repair function

    Code
      my_vec_as_names(c("", ""), my_repair = function(nms) "foo")
    Condition
      Error in `my_vec_as_names()`:
      ! Repaired names have length 1 instead of length 2.

# vec_as_names() is noisy by default

    Code
      vec_as_names(c("x", "x"), repair = "unique")
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
      [1] "x...1" "x...2"
    Code
      vec_as_names(c("x", "x"), repair = "unique", quiet = TRUE)
    Output
      [1] "x...1" "x...2"
    Code
      (expect_error(my_vec_as_names(c("x", "x"), my_repair = "check_unique")))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_vec_as_names()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
      i Use argument `my_repair` to specify repair strategy.
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet")
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet")
    Output
      [1] "...1...1" "...1...2"
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = TRUE)
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = TRUE)
    Output
      [1] "...1...1" "...1...2"
    Code
      vec_as_names(c("1", "1"), repair = "unique_quiet", quiet = FALSE)
    Output
      [1] "1...1" "1...2"
    Code
      vec_as_names(c("1", "1"), repair = "universal_quiet", quiet = FALSE)
    Output
      [1] "...1...1" "...1...2"

# validate_minimal_names() checks names

    Code
      (expect_error(validate_minimal_names(1), "must return a character vector"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions must return a character vector.
    Code
      (expect_error(validate_minimal_names(NULL), "can't return `NULL`"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NULL`.
    Code
      (expect_error(validate_minimal_names(chr(NA)), "can't return `NA` values"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.

# validate_unique() checks unique names

    Code
      (expect_error(validate_unique(chr(NA)), "`NA`"))
    Output
      <error/rlang_error>
      Error:
      ! Names repair functions can't return `NA` values.
    Code
      (expect_error(validate_unique(chr("")), class = "vctrs_error_names_cannot_be_empty")
      )
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error:
      ! Names can't be empty.
      x Empty name found at location 1.
    Code
      (expect_error(validate_unique(chr("a", "a")), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
    Code
      (expect_error(validate_unique(chr("..1")), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..1" at location 1.
    Code
      (expect_error(validate_unique(chr("...")), class = "vctrs_error_names_cannot_be_dot_dot")
      )
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# vec_set_names() errors with bad `names`

    Code
      (expect_error(vec_set_names(1, 1), "character vector, not a double"))
    Output
      <error/rlang_error>
      Error in `vec_set_names()`:
      ! `names` must be a character vector, not a double.
    Code
      (expect_error(vec_set_names(1, c("x", "y")), "The size of `names`, 2"))
    Output
      <error/rlang_error>
      Error in `vec_set_names()`:
      ! The size of `names`, 2, must be the same as the size of `x`, 1.

# unique_names() and as_unique_names() are verbose or silent

    Code
      unique_names(1:2)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
      [1] "...1" "...2"

---

    Code
      as_unique_names(c("", ""))
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
      [1] "...1" "...2"

# message

    Code
      as_universal_names(c("a b", "b c"))
    Message
      New names:
      * `a b` -> `a.b`
      * `b c` -> `b.c`
    Output
      [1] "a.b" "b.c"

# messages by default

    Code
      vec_repair_names(set_names(1, "a:b"), "universal")
    Message
      New names:
      * `a:b` -> `a.b`
    Output
      a.b 
        1 

---

    Code
      vec_repair_names(set_names(1, "a:b"), ~ make.names(.))
    Message
      New names:
      * `a:b` -> `a.b`
    Output
      a.b 
        1 

# NULL name specs works with scalars

    Code
      (expect_error(vec_c(foo = c(a = 1, b = 2)), "vector of length > 1"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(vec_c(foo = 1:2), "vector of length > 1"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(vec_c(x = c(xx = 1)), "named vector"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't merge the outer name `x` with a named vector.
      Please supply a `.name_spec` specification.

# apply_name_spec() checks recyclability of output

    Code
      apply_name_spec(function(...) c("a", "b", "c"), "outer", "inner", n = 2L)
    Condition
      Error:
      ! Can't recycle input of size 3 to size 2.

# vec_as_names() uses internal error if `repair_arg` is not supplied

    Code
      (expect_error(vec_as_names("", repair = "foobar", call = quote(tilt()))))
    Output
      <error/rlang_error>
      Error in `vec_as_names()`:
      ! `repair` can't be "foobar". See `?vctrs::vec_as_names`.
    Code
      (expect_error(vec_as_names("", repair = env(), call = quote(tilt()))))
    Output
      <error/rlang_error>
      Error in `vec_as_names()`:
      ! `repair` must be a string or a function. See `?vctrs::vec_as_names`.

# `condition` must be a condition vector

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not the number 1.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not the number 1.

---

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not a <foo> object.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not a <foo> object.

---

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not a logical 1D array.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not a logical 1D array.

# `true`, `false`, and `missing` must be vectors

    Code
      vec_if_else(condition = TRUE, true = lm(1 ~ 1), false = 2, missing = 0)
    Condition
      Error in `vec_if_else()`:
      ! `true` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_if_else(condition = TRUE, true = 1, false = lm(1 ~ 1), missing = 0)
    Condition
      Error in `vec_if_else()`:
      ! `false` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_if_else(condition = TRUE, true = 1, false = 2, missing = lm(1 ~ 1))
    Condition
      Error in `vec_if_else()`:
      ! `missing` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `true`, `false`, and `missing` must recycle to size of `condition`

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `true` (size 2) to size 1.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `true` (size 2) to size 1.

---

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `false` (size 2) to size 1.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `false` (size 2) to size 1.

---

    Code
      vec_if_else(condition = condition, true = true, false = false, missing = missing,
        ptype = ptype, condition_arg = condition_arg, true_arg = true_arg, false_arg = false_arg,
        missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `missing` (size 2) to size 1.

---

    Code
      vec_if_else(condition = condition, true = true_vctr, false = false_vctr,
        missing = missing_vctr, ptype = ptype, condition_arg = condition_arg,
        true_arg = true_arg, false_arg = false_arg, missing_arg = missing_arg)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `missing` (size 2) to size 1.

# `ptype` overrides common type

    Code
      vec_if_else(condition = TRUE, true = 1.5, false = 2, missing = 0, ptype = integer())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert from `true` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_if_else(condition = TRUE, true = 1, false = 2.5, missing = 0, ptype = integer())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert from `false` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_if_else(condition = TRUE, true = 1, false = 2, missing = 0.5, ptype = integer())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert from `missing` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_if_else(condition = TRUE, true = 1, false = new_date(2), missing = new_date(
        0), ptype = new_date())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert `true` <double> to <date>.

---

    Code
      vec_if_else(condition = TRUE, true = new_date(1), false = 2, missing = new_date(
        0), ptype = new_date())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert `false` <double> to <date>.

---

    Code
      vec_if_else(condition = TRUE, true = new_date(1), false = new_date(2), missing = 0,
      ptype = new_date())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert `missing` <double> to <date>.

# takes the common type of `true` and `false` (tidyverse/dplyr#6243)

    Code
      vec_if_else(TRUE, 1, "x")
    Condition
      Error in `vec_if_else()`:
      ! Can't combine `true` <double> and `false` <character>.

---

    Code
      vec_if_else(TRUE, 1, "x", true_arg = "t", false_arg = "f", error_call = current_env())
    Condition
      Error:
      ! Can't combine `t` <double> and `f` <character>.

# includes `missing` in the common type computation if used

    Code
      vec_if_else(TRUE, 1, 2, missing = "x")
    Condition
      Error in `vec_if_else()`:
      ! Can't combine `true` <double> and `missing` <character>.

---

    Code
      vec_if_else(TRUE, 1L, 2, missing = "x")
    Condition
      Error in `vec_if_else()`:
      ! Can't combine `false` <double> and `missing` <character>.

---

    Code
      vec_if_else(TRUE, 1, 2L, missing = "x")
    Condition
      Error in `vec_if_else()`:
      ! Can't combine `true` <double> and `missing` <character>.

# `condition` must be logical (and isn't cast to logical!)

    Code
      vec_if_else(1:10, 1, 2)
    Condition
      Error in `vec_if_else()`:
      ! `condition` must be a logical vector, not an integer vector.

# `true`, `false`, and `missing` must recycle to the size of `condition`

    Code
      vec_if_else(x < 2, bad, x)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `true` (size 2) to size 3.

---

    Code
      vec_if_else(x < 2, x, bad)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `false` (size 2) to size 3.

---

    Code
      vec_if_else(x < 2, x, x, missing = bad)
    Condition
      Error in `vec_if_else()`:
      ! Can't recycle `missing` (size 2) to size 3.

# must have empty dots

    Code
      vec_if_else(TRUE, 1, 2, missing = 3, 4)
    Condition
      Error in `vec_if_else()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 4
      i Did you forget to name an argument?

# `ptype` overrides the common type

    Code
      vec_if_else(TRUE, 1L, 2.5, ptype = integer())
    Condition
      Error in `vec_if_else()`:
      ! Can't convert from `false` <double> to <integer> due to loss of precision.
      * Locations: 1

# vec_recycle_common() reports error context

    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `this_arg` (size 2) to match `that_arg` (size 0).
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `that_arg` (size 0) to size 2.
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .arg = "my_arg")))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg$this_arg` (size 2) to match `my_arg$that_arg` (size 0).
    Code
      (expect_error(my_function(this_arg = 1:2, that_arg = int(), .size = 2, .arg = "my_arg"))
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `my_arg$that_arg` (size 0) to size 2.

# vec_recycle(): incompatible lengths get error messages

    Code
      (expect_error(vec_recycle(x2, 1), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 1.

# recycling to size 1 has informative error

    Code
      (expect_error(vec_recycle(1:2, 1), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle input of size 2 to size 1.

# incompatible recycling size has informative error

    Code
      vec_recycle(1:2, 4)
    Condition
      Error:
      ! Can't recycle input of size 2 to size 4.

---

    Code
      vec_recycle(1:2, 4, x_arg = "foo")
    Condition
      Error:
      ! Can't recycle `foo` (size 2) to size 4.

# vec_recycle_common(): incompatible lengths get error messages

    Code
      (expect_error(vec_recycle_common(1:2, 1:3), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

# vec_recycle_common errors on scalars

    Code
      vec_recycle_common(1, lm(1 ~ 1))
    Condition
      Error:
      ! `..2` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      vec_recycle_common(1, NULL, lm(1 ~ 1))
    Condition
      Error:
      ! `..3` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# recycling matrices respects incompatible sizes

    Code
      (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 4).

# recycling data frames respects incompatible sizes

    Code
      (expect_error(vec_recycle_common(x2, x), class = "vctrs_error_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error:
      ! Can't recycle `..1` (size 2) to match `..2` (size 3).

# incompatible columns throws common type error

    Code
      (expect_error(vec_rbind(x_int, x_chr), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1$x` <integer> and `..2$x` <character>.
    Code
      (expect_error(vec_rbind(x_int, x_chr, .error_call = call("foo")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `..1$x` <integer> and `..2$x` <character>.
    Code
      (expect_error(vec_rbind(x_int, x_chr, .ptype = x_chr, .error_call = call("foo")),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `foo()`:
      ! Can't convert `..1$x` <integer> to match type of `x` <character>.

# names are supplied if needed

    Code
      out <- vec_rbind(data_frame(...1 = 1), 1)
    Message
      New names:
      * `` -> `...1`

# can repair names in `vec_rbind()` (#229)

    Code
      (expect_error(vec_rbind(.name_repair = "none"), "can't be `\"none\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"none"`.
      It must be one of `"unique"`, `"universal"`, or `"check_unique"`.
    Code
      (expect_error(vec_rbind(.name_repair = "minimal"), "can't be `\"minimal\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"minimal"`.
      It must be one of `"unique"`, `"universal"`, or `"check_unique"`.
    Code
      (expect_error(vec_rbind(list(a = 1, a = 2), .name_repair = "check_unique"),
      class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_rbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.

# can repair names quietly

    Code
      res_unique <- vec_rbind(c(x = 1, x = 2), c(x = 3, x = 4), .name_repair = "unique_quiet")
      res_universal <- vec_rbind(c(`if` = 1, `in` = 2), c(`if` = 3, `for` = 4),
      .name_repair = "universal_quiet")

---

    Code
      res_unique <- vec_cbind(x = 1, x = 2, .name_repair = "unique_quiet")
      res_universal <- vec_cbind(`if` = 1, `in` = 2, .name_repair = "universal_quiet")

# vec_rbind() fails with arrays of dimensionality > 3

    Code
      (expect_error(vec_rbind(array(NA, c(1, 1, 1)))))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_rbind(array(NA, c(1, 1, 1)), .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't bind arrays.

# can assign row names in vec_rbind()

    Code
      (expect_error(vec_rbind(foo = df1, df2, .names_to = NULL), "specification"))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! Can't merge the outer name `foo` with a vector of length > 1.
      Please supply a `.name_spec` specification.

# names of `...` are used for type and cast errors even when zapped

    Code
      vec_rbind(!!!xs)
    Condition
      Error in `vec_rbind()`:
      ! Can't combine `a$x` <double> and `b$x` <character>.

---

    Code
      vec_rbind(!!!xs, .ptype = data_frame(x = double()))
    Condition
      Error in `vec_rbind()`:
      ! Can't convert `b$x` <character> to match type of `x` <double>.

# vec_cbind() reports error context

    Code
      (expect_error(vec_cbind(foobar(list()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_cbind()`:
      ! `..1` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
    Code
      (expect_error(vec_cbind(foobar(list()), .error_call = call("foo"))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `foo()`:
      ! `..1` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
    Code
      (expect_error(vec_cbind(a = 1:2, b = int())))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `vec_cbind()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).
    Code
      (expect_error(vec_cbind(a = 1:2, b = int(), .error_call = call("foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `a` (size 2) to match `b` (size 0).

# duplicate names are de-deduplicated

    Code
      (expect_named(vec_cbind(x = 1, x = 1), c("x...1", "x...2")))
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
        x...1 x...2
      1     1     1
    Code
      (expect_named(vec_cbind(data.frame(x = 1), data.frame(x = 1)), c("x...1",
        "x...2")))
    Message
      New names:
      * `x` -> `x...1`
      * `x` -> `x...2`
    Output
        x...1 x...2
      1     1     1

# can repair names in `vec_cbind()` (#227)

    Code
      (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "none"),
      "can't be `\"none\"`"))
    Output
      <error/rlang_error>
      Error:
      ! `.name_repair` can't be `"none"`.
      It must be one of `"unique"`, `"universal"`, `"check_unique"`, or `"minimal"`.
    Code
      (expect_error(vec_cbind(a = 1, a = 2, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_cbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.

# can supply `.names_to` to `vec_rbind()` (#229)

    Code
      (expect_error(vec_rbind(data_frame(), .names_to = letters)))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.
    Code
      (expect_error(vec_rbind(data_frame(), .names_to = 10)))
    Output
      <error/rlang_error>
      Error in `vec_rbind()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.
    Code
      (expect_error(vec_rbind(data_frame(), .names_to = letters, .error_call = call(
        "foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! `.names_to` must be `NULL`, a string, or an `rlang::zap()` object.

# vec_cbind() fails with arrays of dimensionality > 3

    Code
      (expect_error(vec_cbind(a)))
    Output
      <error/rlang_error>
      Error in `vec_cbind()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_cbind(a, .error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't bind arrays.
    Code
      (expect_error(vec_cbind(x = a)))
    Output
      <error/rlang_error>
      Error in `vec_cbind()`:
      ! Can't bind arrays.

# vec_rbind() name repair messages are useful

    Code
      vec_rbind(1, 2)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(1, 2, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(1, 2, ...10 = 3)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
      3    3
    Code
      vec_rbind(1, 2, ...10 = 3, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
           ...1
      ...1    1
      ...2    2
      ...3    3
    Code
      vec_rbind(a = 1, b = 2)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      1    1
      2    2
    Code
      vec_rbind(a = 1, b = 2, .names_to = NULL)
    Message
      New names:
      * `` -> `...1`
      New names:
      * `` -> `...1`
    Output
        ...1
      a    1
      b    2
    Code
      vec_rbind(c(a = 1), c(b = 2))
    Output
         a  b
      1  1 NA
      2 NA  2
    Code
      vec_rbind(c(a = 1), c(b = 2), .names_to = NULL)
    Output
         a  b
      1  1 NA
      2 NA  2

# vec_rbind() is silent when assigning duplicate row names of df-cols

    Code
      vec_rbind(df, df)
    Output
         mpg
      1 21.0
      2 21.0
      3 22.8
      4 21.0
      5 21.0
      6 22.8

---

    Code
      vec_rbind(mtcars[1:4, ], mtcars[1:3, ])
    Output
                         mpg cyl disp  hp drat    wt  qsec vs am gear carb
      Mazda RX4...1     21.0   6  160 110 3.90 2.620 16.46  0  1    4    4
      Mazda RX4 Wag...2 21.0   6  160 110 3.90 2.875 17.02  0  1    4    4
      Datsun 710...3    22.8   4  108  93 3.85 2.320 18.61  1  1    4    1
      Hornet 4 Drive    21.4   6  258 110 3.08 3.215 19.44  1  0    3    1
      Mazda RX4...5     21.0   6  160 110 3.90 2.620 16.46  0  1    4    4
      Mazda RX4 Wag...6 21.0   6  160 110 3.90 2.875 17.02  0  1    4    4
      Datsun 710...7    22.8   4  108  93 3.85 2.320 18.61  1  1    4    1

# vec_cbind() name repair messages are useful

    Code
      vec_cbind(1, 2)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
        ...1 ...2
      1    1    2
    Code
      vec_cbind(1, 2, ...10 = 3)
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
      * `...10` -> `...3`
    Output
        ...1 ...2 ...3
      1    1    2    3
    Code
      vec_cbind(a = 1, b = 2)
    Output
        a b
      1 1 2
    Code
      vec_cbind(c(a = 1), c(b = 2))
    Message
      New names:
      * `` -> `...1`
      * `` -> `...2`
    Output
        ...1 ...2
      1    1    2

# rbind repairs names of data frames (#704)

    Code
      (expect_error(vec_rbind(df, df, .name_repair = "check_unique"), class = "vctrs_error_names_must_be_unique")
      )
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `vec_rbind()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
    Code
      (expect_error(vec_rbind(df, df, .name_repair = "check_unique", .error_call = call(
        "foo")), class = "vctrs_error_names_must_be_unique"))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `foo()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.

# vec_rbind() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(vec_rbind(set_names(x, "x"), set_names(y, "x")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1` <vctrs_foobar> and `..2` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_rbind() fails with complex foreign S4 classes

    Code
      joe <- .Counts(1L, name = "Joe")
      jane <- .Counts(2L, name = "Jane")
      (expect_error(vec_rbind(set_names(joe, "x"), set_names(jane, "y")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_rbind()`:
      ! Can't combine `..1` <vctrs_Counts> and `..2` <vctrs:::common_class_fallback>.

# row-binding performs expected allocations

    Code
      ints <- rep(list(1L), 100)
      named_ints <- rep(list(set_names(1:3, letters[1:3])), 100)
      # Integers as rows
      suppressMessages(with_memory_prof(vec_rbind_list(ints)))
    Output
      [1] 2.74KB
    Code
      suppressMessages(with_memory_prof(vec_rbind_list(named_ints)))
    Output
      [1] 3.62KB
    Code
      # Data frame with named columns
      df <- data_frame(x = set_names(as.list(1:2), c("a", "b")), y = set_names(1:2, c(
        "A", "B")), z = data_frame(Z = set_names(1:2, c("Za", "Zb"))))
      dfs <- rep(list(df), 100)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 10.4KB
    Code
      # Data frame with rownames (non-repaired, non-recursive case)
      df <- data_frame(x = 1:2)
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 7.63KB
    Code
      # Data frame with rownames (repaired, non-recursive case)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 13.8KB
    Code
      # Data frame with rownames (non-repaired, recursive case) (#1217)
      df <- data_frame(x = 1:2, y = data_frame(x = 1:2))
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 13KB
    Code
      # Data frame with rownames (repaired, recursive case) (#1217)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(vec_rbind_list(dfs))
    Output
      [1] 25.3KB

# can't get common type of Date and IDate

    Code
      vec_ptype2(x, y)
    Condition
      Error:
      ! Can't combine `x` <date> and `y` <IDate>.

---

    Code
      vec_ptype2(y, x)
    Condition
      Error:
      ! Can't combine `y` <IDate> and `x` <date>.

# can't cast Date to IDate

    Code
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert `x` <date> to <IDate>.

# can't cast IDate to Date

    Code
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert `x` <IDate> to <date>.

# common type failure uses error call and error arg (#1641, #1692)

    Code
      vec_c("x", 1, .error_call = call("foo"), .error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <character> and `arg[[2]]` <double>.

---

    Code
      vec_c("x", .ptype = integer(), .error_call = call("foo"), .error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't convert `arg[[1]]` <character> to <integer>.

# common type failure uses positional errors

    Code
      (expect_error(vec_c(1, a = "x", 2)))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <double> and `a` <character>.
    Code
      (expect_error(vec_c(1, a = "x", 2, .ptype = double(), .error_arg = "arg")))
    Output
      <error/vctrs_error_cast>
      Error in `vec_c()`:
      ! Can't convert `arg$a` <character> to <double>.
    Code
      (expect_error(vec_c(1, a = 2.5, .ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_c()`:
      ! Can't convert from `a` <double> to <integer> due to loss of precision.
      * Locations: 1

# vec_c() includes index in argument tag

    Code
      vec_c(df1, df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `..1$x$y$z` <double> and `..2$x$y$z` <character>.

---

    Code
      vec_c(df1, df1, df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `..1$x$y$z` <double> and `..3$x$y$z` <character>.

---

    Code
      vec_c(foo = df1, bar = df2)
    Condition
      Error in `vec_c()`:
      ! Can't combine `foo$x$y$z` <double> and `bar$x$y$z` <character>.

# vec_c() can repair names quietly

    Code
      res_unique <- vec_c(x = TRUE, x = 0, .name_repair = "unique_quiet")
      res_universal <- vec_c(`if` = TRUE, `in` = 0, .name_repair = "universal_quiet")

# vec_c() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(vec_c(x, y), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <vctrs_foobar> and `..2` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(vec_c(x, y, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_foobar> and `arg[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_c() fails with complex foreign S4 classes

    Code
      joe <- .Counts(c(1L, 2L), name = "Joe")
      jane <- .Counts(3L, name = "Jane")
      (expect_error(vec_c(joe, jane), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `..1` <vctrs_Counts> and `..2` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(vec_c(joe, jane, .error_call = call("foo"), .error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_Counts> and `arg[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# vec_c() fallback doesn't support (most) `name_spec` or `ptype`

    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .name_spec = "{outer}_{inner}")),
      "name specification"))
    Output
      <error/rlang_error>
      Error in `vec_c()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .ptype = "")), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error in `vec_c()`:
      ! Can't convert `..1` <vctrs_foobar> to <character>.
    Code
      (expect_error(with_c_foobar(vec_c(foobar(1), foobar(2), .error_call = call(
        "foo"), .name_spec = "{outer}_{inner}"))))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.

# can ignore names in `vec_c()` by providing a `zap()` name-spec (#232)

    Code
      (expect_error(vec_c(a = c(b = letters), b = 1, .name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `vec_c()`:
      ! Can't combine `a` <character> and `b` <double>.

# can ignore outer names in `vec_c()` by providing an 'inner' name-spec (#1988)

    Code
      vec_c(x = c(a = 1), y = c(b = "2"), .name_spec = "inner")
    Condition
      Error in `vec_c()`:
      ! Can't combine `x` <double> and `y` <character>.

# calls cast method even with empty objects

    Code
      vec_c(foobar(integer()), foobar(integer(), foo = "bar"))
    Condition
      Error in `vec_c()`:
      ! Can't convert `..2` <vctrs_foobar> to <vctrs_foobar>.

# concatenation performs expected allocations

    Code
      ints <- rep(list(1L), 100)
      dbls <- rep(list(1), 100)
      # # `vec_c()` 
      # Integers
      with_memory_prof(vec_c_list(ints))
    Output
      [1] 1.48KB
    Code
      # Doubles
      with_memory_prof(vec_c_list(dbls))
    Output
      [1] 1.87KB
    Code
      # Integers to integer
      with_memory_prof(vec_c_list(ints, ptype = int()))
    Output
      [1] 1.27KB
    Code
      # Doubles to integer
      with_memory_prof(vec_c_list(dbls, ptype = int()))
    Output
      [1] 1.27KB
    Code
      # # `list_unchop()` 
      # Integers
      with_memory_prof(list_unchop(ints))
    Output
      [1] 1.48KB
    Code
      # Doubles
      with_memory_prof(list_unchop(dbls))
    Output
      [1] 1.87KB
    Code
      # Integers to integer
      with_memory_prof(list_unchop(ints, ptype = int()))
    Output
      [1] 1.27KB
    Code
      # Doubles to integer
      with_memory_prof(list_unchop(dbls, ptype = int()))
    Output
      [1] 1.27KB
    Code
      # # Concatenation with names
      # Named integers
      ints <- rep(list(set_names(1:3, letters[1:3])), 100)
      with_memory_prof(list_unchop(ints))
    Output
      [1] 4.65KB
    Code
      # Named matrices
      mat <- matrix(1:4, 2, dimnames = list(c("foo", "bar")))
      mats <- rep(list(mat), 100)
      with_memory_prof(list_unchop(mats))
    Output
      [1] 4.26KB
    Code
      # Data frame with named columns
      df <- data_frame(x = set_names(as.list(1:2), c("a", "b")), y = set_names(1:2, c(
        "A", "B")), z = data_frame(Z = set_names(1:2, c("Za", "Zb"))))
      dfs <- rep(list(df), 100)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 9.13KB
    Code
      # Data frame with rownames (non-repaired, non-recursive case)
      df <- data_frame(x = 1:2)
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 6.37KB
    Code
      # Data frame with rownames (repaired, non-recursive case)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 12.5KB
    Code
      # Data frame with rownames (non-repaired, recursive case) (#1217)
      df <- data_frame(x = 1:2, y = data_frame(x = 1:2))
      dfs <- rep(list(df), 100)
      dfs <- map2(dfs, seq_along(dfs), set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 11.7KB
    Code
      # Data frame with rownames (repaired, recursive case) (#1217)
      dfs <- map(dfs, set_rownames_recursively)
      with_memory_prof(list_unchop(dfs))
    Output
      [1] 24KB
    Code
      # list-ofs (#1496)
      make_list_of <- (function(n) {
        df <- tibble::tibble(x = new_list_of(vec_chop(1:n), ptype = integer()))
        vec_chop(df)
      })
      with_memory_prof(list_unchop(make_list_of(1000)))
    Output
      [1] 59.7KB
    Code
      with_memory_prof(list_unchop(make_list_of(2000)))
    Output
      [1] 118KB
    Code
      with_memory_prof(list_unchop(make_list_of(4000)))
    Output
      [1] 235KB

# `indices` are validated

    Code
      vec_chop(1, indices = 1)
    Condition
      Error:
      ! `indices` must be a list of index values, or `NULL`.

---

    Code
      (expect_error(vec_chop(1, indices = list(1.5)), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Can't convert from <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(vec_chop(1, indices = list(2)), class = "vctrs_error_subscript_oob")
      )
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# `sizes` are validated

    Code
      vec_chop("a", sizes = "a")
    Condition
      Error:
      ! Can't convert `sizes` <character> to <integer>.

---

    Code
      vec_chop("a", sizes = 2)
    Condition
      Error:
      ! `sizes` can't contain sizes larger than 1.

---

    Code
      vec_chop("a", sizes = -1)
    Condition
      Error:
      ! `sizes` can't contain negative sizes.

---

    Code
      vec_chop("a", sizes = NA_integer_)
    Condition
      Error:
      ! `sizes` can't contain missing values.

---

    Code
      vec_chop("a", sizes = c(1, 1))
    Condition
      Error:
      ! `sizes` must sum to size 1, not size 2.

# can't use both `indices` and `sizes`

    Code
      vec_chop(1, indices = list(1), sizes = 1)
    Condition
      Error:
      ! Can't supply both `indices` and `sizes`.

# `vec_chop(x, indices)` backwards compatible behavior works

    Code
      vec_chop(1:2, 1)
    Condition
      Error:
      ! `indices` must be a list of index values, or `NULL`.

---

    Code
      vec_chop(1, list(1), sizes = 1)
    Condition
      Error:
      ! Can't supply both `indices` and `sizes`.

---

    Code
      vec_chop(1, list(1), 2)
    Condition
      Error in `vec_chop()`:
      ! `...` must be empty.
      x Problematic arguments:
      * ..1 = list(1)
      * ..2 = 2
      i Did you forget to name an argument?

---

    Code
      vec_chop(1, list(1), indices = list(1))
    Condition
      Error in `vec_chop()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = list(1)
      i Did you forget to name an argument?

# `vec_chop()` can't take `compact_seq()` indices directly

    Code
      vec_chop(1:2, indices = list(compact_seq(1, 2)))
    Condition
      Error in `vec_chop()`:
      ! `compact_seq` are not allowed.
      i In file 'slice-chop.c' at line <scrubbed>.
      i This is an internal error that was detected in the vctrs package.
        Please report it at <https://github.com/r-lib/vctrs/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# `x` is validated

    Code
      list_of_transpose(1)
    Condition
      Error in `list_of_transpose()`:
      ! `1` must be a `<list_of>`, not the number 1.

---

    Code
      list_of_transpose(1, x_arg = "x", error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `x` must be a `<list_of>`, not the number 1.

# `x` must be a fully specified list of

    Code
      x <- list_of(.ptype = integer(), .size = zap())
      list_of_transpose(x)
    Condition
      Error in `list_of_transpose()`:
      ! `x` must be a fully specified `<list_of>`.
      i `size` is not specified.

---

    Code
      x <- list_of(.ptype = zap(), .size = 1)
      list_of_transpose(x)
    Condition
      Error in `list_of_transpose()`:
      ! `x` must be a fully specified `<list_of>`.
      i `ptype` is not specified.

# `...` must be empty

    Code
      list_of_transpose(list_of2(1), 2)
    Condition
      Error in `list_of_transpose()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = 2
      i Did you forget to name an argument?

# doesn't allow `NULL` elements

    Code
      list_of_transpose(list_of2(1:4, NULL, 5:8))
    Condition
      Error in `list_of_transpose()`:
      ! `list_of2(1:4, NULL, 5:8)` can't contain `NULL` values.

# failing common type reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `2` <double> and `chr()` <character>.

# failing cast reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `2` <double> to <character>.

---

    Code
      (expect_error(my_function(df1, df2)))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `lhs$y` <logical> to match type of `y` <character>.

---

    Code
      (expect_error(my_function(df1, df2)))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `lhs$y` <logical> to match type of `y` <character>.

# lossy cast reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `2` <double> to <logical> due to loss of precision.
      * Locations: 1

# failing common size reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle input of size 2 to size 10.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_function()`:
      ! Can't recycle `..1` (size 2) to match `..2` (size 10).

# unsupported error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_unsupported>
      Error in `dim<-`:
      ! `dim<-.vctrs_vctr()` not supported.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_unimplemented>
      Error in `median()`:
      ! `median.vctrs_vctr()` not implemented.

# scalar error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `foobar()` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# size error reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_assert_size>
      Error in `my_function()`:
      ! `1:2` must have size 1, not size 2.

# bare casts report correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `1.5` <double> to <logical> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `2L` <integer> to <logical> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast>
      Error in `my_function()`:
      ! Can't convert `matrix(TRUE)` <double[,1]> to <double>.
      Can't decrease dimensionality from 2 to 1.

# base S3 casts report correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `"a"` <character> to <factor<9b7e3>> due to loss of generality.
      * Locations: 1

# names validation reports correct error call

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_cannot_be_empty>
      Error in `my_function()`:
      ! Names can't be empty.
      x Empty name found at location 2.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_must_be_unique>
      Error in `my_function()`:
      ! Names must be unique.
      x These names are duplicated:
        * "x" at locations 1 and 2.
      i Use argument `repair` to specify repair strategy.

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_names_cannot_be_dot_dot>
      Error in `my_function()`:
      ! Names can't be of the form `...` or `..j`.
      x These names are invalid:
        * "..." at location 1.

# subscript validation reports correct error calls

    Code
      (expect_error(my_function()))
    Output
      <error/rlang_error>
      Error in `vctrs::num_as_location()`:
      ! `missing` must be one of "propagate", "remove", or "error".

---

    Code
      (expect_error(my_function()))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements past the end.
      i Location 10 doesn't exist.
      i There are only 2 elements.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `my_arg`.
      x Can't convert from `my_arg` <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements.
      x Can't convert from <double> to <integer> due to loss of precision.

---

    Code
      (expect_error(my_function(list())))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements with `my_arg`.
      x `my_arg` must be logical, numeric, or character, not an empty list.

---

    Code
      (expect_error(my_function(1.5)))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `vec_as_location()`:
      ! Can't convert from `n` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(NA)))
    Output
      <error/vctrs_error_subscript_type>
      Error in `my_function()`:
      ! Can't subset elements.
      x Subscript can't contain missing values.
      x It has a missing value at location 1.

# `vec_ptype()` reports correct error call

    Code
      (expect_error(my_function(env())))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! Input must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(my_function(foobar(list()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! Input must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# `vec_slice()` uses `error_call`

    Code
      (expect_error(my_function(env(), 1)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `my_function()`:
      ! `x` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(my_function(1, 2)))
    Output
      <error/vctrs_error_subscript_oob>
      Error in `my_function()`:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# vec_slice() reports self in error context

    Code
      (expect_error(vec_slice(foobar(list()), 1)))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_slice()`:
      ! `x` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
    Code
      (expect_error(vec_slice(list(), env())))
    Output
      <error/vctrs_error_subscript_type>
      Error in `vec_slice()`:
      ! Can't subset elements with `i`.
      x `i` must be logical, numeric, or character, not an environment.

# list_sizes() reports error context

    Code
      (expect_error(list_sizes(foobar(list()))))
    Output
      <error/rlang_error>
      Error in `list_sizes()`:
      ! `x` must be a list, not a <vctrs_foobar> object.
    Code
      (expect_error(list_sizes(list(env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x[[1]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(list_sizes(list(1, 2, env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x[[3]]` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
    Code
      (expect_error(list_sizes(list(1, 2, foo = env()))))
    Output
      <error/vctrs_error_scalar_type>
      Error in `list_sizes()`:
      ! `x$foo` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# vec_size() reports error context

    Code
      (expect_error(vec_size(env())))
    Output
      <error/vctrs_error_scalar_type>
      Error in `vec_size()`:
      ! `x` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# vec_cast_common() reports error context

    Code
      (expect_error(my_function(my_arg = 1.5, .to = int())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `my_arg` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(my_arg = 1.5, .to = int(), .arg = "my_arg")))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `my_function()`:
      ! Can't convert from `my_arg$my_arg` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg$this_arg` <double> and `my_arg$that_arg` <character>.

---

    Code
      (expect_error(my_function(1, "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg[[1]]` <double> and `my_arg[[2]]` <character>.

---

    Code
      (expect_error(my_function(this_arg = x, that_arg = y)))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `this_arg$x` <character> and `that_arg$x` <double>.

# vec_ptype_common() reports error context

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `this_arg` <double> and `that_arg` <character>.

---

    Code
      (expect_error(my_function(this_arg = 1, that_arg = "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg$this_arg` <double> and `my_arg$that_arg` <character>.

---

    Code
      (expect_error(my_function(1, "foo", .arg = "my_arg")))
    Output
      <error/vctrs_error_ptype2>
      Error in `my_function()`:
      ! Can't combine `my_arg[[1]]` <double> and `my_arg[[2]]` <character>.

# `vec_rep()` validates `times`

    Code
      (expect_error(my_vec_rep(1, "x"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `my_vec_rep()`:
      ! Can't convert `my_times` <character> to <integer>.
    Code
      (expect_error(my_vec_rep(1, c(1, 2))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` must be a single number.
    Code
      (expect_error(my_vec_rep(1, -1)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` must be a positive number.
    Code
      (expect_error(my_vec_rep(1, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep()`:
      ! `my_times` can't be missing.

---

    Code
      my_vec_rep(1, "x")
    Condition
      Error in `my_vec_rep()`:
      ! Can't convert `my_times` <character> to <integer>.

---

    Code
      my_vec_rep(1, c(1, 2))
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` must be a single number.

---

    Code
      my_vec_rep(1, -1)
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` must be a positive number.

---

    Code
      my_vec_rep(1, NA_integer_)
    Condition
      Error in `my_vec_rep()`:
      ! `my_times` can't be missing.

# `vec_rep_each()` validates `times`

    Code
      (expect_error(my_vec_rep_each(1, "x"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `my_vec_rep_each()`:
      ! Can't convert `my_times` <character> to <integer>.
    Code
      (expect_error(my_vec_rep_each(1, -1)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 1 is negative.
    Code
      (expect_error(my_vec_rep_each(c(1, 2), c(1, -1))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 2 is negative.
    Code
      (expect_error(my_vec_rep_each(1, NA_integer_)))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 1 is missing.
    Code
      (expect_error(my_vec_rep_each(c(1, 2), c(1, NA_integer_))))
    Output
      <error/rlang_error>
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 2 is missing.

---

    Code
      my_vec_rep_each(1, "x")
    Condition
      Error in `my_vec_rep_each()`:
      ! Can't convert `my_times` <character> to <integer>.

---

    Code
      my_vec_rep_each(1, -1)
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 1 is negative.

---

    Code
      my_vec_rep_each(c(1, 2), c(1, -1))
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` must be a vector of positive numbers. Location 2 is negative.

---

    Code
      my_vec_rep_each(1, NA_integer_)
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 1 is missing.

---

    Code
      my_vec_rep_each(c(1, 2), c(1, NA_integer_))
    Condition
      Error in `my_vec_rep_each()`:
      ! `my_times` can't be missing. Location 2 is missing.

# `vec_rep_each()` uses recyclying errors

    Code
      (expect_error(my_vec_rep_each(1:2, 1:3), class = "vctrs_error_recycle_incompatible_size")
      )
    Output
      <error/vctrs_error_incompatible_size>
      Error in `my_vec_rep_each()`:
      ! Can't recycle `my_times` (size 3) to size 2.

---

    Code
      my_vec_rep_each(1:2, 1:3)
    Condition
      Error in `my_vec_rep_each()`:
      ! Can't recycle `my_times` (size 3) to size 2.

# errors on scalars

    Code
      vec_unrep(environment())
    Condition
      Error in `vec_unrep()`:
      ! `x` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# `list_combine()` works with homogeneous fallback in `default`

    Code
      list_combine(list(foobar(1), 1), indices = list(1, 2), size = 2)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <double>.

---

    Code
      list_combine(list(foobar(1)), indices = list(1), size = 2, default = 1)
    Condition
      Error in `list_combine()`:
      ! Can't combine <vctrs_foobar> and `default` <double>.

# list_combine() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      list_combine(list(x, y), indices = list(1, 2), size = 2)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      list_combine(list(x, y), indices = list(1, 2), size = 2, error_call = call(
        "foo"), x_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_foobar> and `arg[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      default <- structure(foobar(2), attr_foo = "bar")
      list_combine(list(x), indices = list(1), size = 2, default = default)
    Condition
      Error in `list_combine()`:
      ! Can't combine <vctrs_foobar> and `default` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      default <- structure(foobar(2), attr_foo = "bar")
      list_combine(list(x), indices = list(1), size = 2, default = default,
      default_arg = "d")
    Condition
      Error in `list_combine()`:
      ! Can't combine <vctrs_foobar> and `d` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_combine() fails with complex foreign S4 classes

    Code
      joe <- .Counts(c(1L, 2L), name = "Joe")
      jane <- .Counts(3L, name = "Jane")
      (expect_error(list_combine(list(joe, jane), indices = list(1:2, 3), size = 3),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(list_combine(list(joe, jane), indices = list(1:2, 3), size = 3,
      error_call = call("foo"), x_arg = "arg"), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_Counts> and `arg[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_combine() falls back to c() if S3 method is available

    Code
      (expect_error(list_combine(list(foobar(1), foobar(2)), indices = list(c(1, 3),
      integer()), size = 2), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

---

    Code
      x <- list(foobar(1:2))
      indices <- list(1:3)
      (expect_error(list_combine(x, indices = indices, size = 3)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.
    Code
      (expect_error(list_combine(x, indices = indices, size = 3, x_arg = "arg",
        error_call = call("foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

# list_combine() falls back for S4 classes with a registered c() method

    Code
      (expect_error(list_combine(list(joe, 1, jane), indices = list(c(1, 2), 3, 4),
      size = 4), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <double>.

# can ignore names in `list_combine()` by providing a `zap()` name-spec (#232)

    Code
      (expect_error(list_combine(list(a = c(b = 1:2)), indices = list(1:2), size = 2))
      )
    Output
      <error/rlang_error>
      Error in `list_combine()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(list_combine(list(a = c(b = 1:2)), indices = list(1:2), size = 2,
      error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `list_combine()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.

---

    Code
      x <- list(a = c(b = c("a", "b")), b = 3L)
      (expect_error(list_combine(x, indices = list(1:2, 3), size = 3, name_spec = zap()),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_combine()`:
      ! Can't combine `x$a` <character> and `x$b` <integer>.
    Code
      x <- list(a = c(foo = 1:2), b = c(bar = ""))
      (expect_error(list_combine(x, indices = list(2:1, 3), size = 3, name_spec = zap()),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_combine()`:
      ! Can't combine `x$a` <integer> and `x$b` <character>.

# list_combine() fails if foreign classes are not homogeneous and there is no c() method

    Code
      list_combine(list(x), indices = list(c(1, 2)), size = 3, default = default)
    Condition
      Error in `list_combine()`:
      ! Can't combine <foo> and `default` <vctrs_foobar>.

# recycling error indices are correct even with `NULL` removal

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[3]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[3]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[3]]` (size 2) to size 3.

# `x_arg` works

    Code
      list_combine(list(1, "2"), indices = list(1, 2), size = 2, x_arg = "xs")
    Condition
      Error in `list_combine()`:
      ! Can't combine `xs[[1]]` <double> and `xs[[2]]` <character>.

---

    Code
      list_combine(list(1, 2), indices = list(1, 2, 3), size = 2, x_arg = "xs")
    Condition
      Error in `list_combine()`:
      ! `indices` must have size 2, not size 3.

# `indices_arg` works

    Code
      list_combine(list(1, 2), indices = 1, size = 2, indices_arg = "i")
    Condition
      Error in `list_combine()`:
      ! `i` must be a list, not the number 1.

---

    Code
      list_combine(list(1, 2), indices = list(1, 2, 3), size = 2, indices_arg = "i")
    Condition
      Error in `list_combine()`:
      ! `i` must have size 2, not size 3.

# `...` must be empty

    Code
      list_combine(list(1, 2), indices = list(1, 2), size = 2, "foo")
    Condition
      Error in `list_combine()`:
      ! `...` must be empty.
      x Problematic argument:
      * ..1 = "foo"
      i Did you forget to name an argument?

# list_combine() `size` type is validated

    Code
      list_combine(list(1), indices = list(1), size = "x")
    Condition
      Error in `list_combine()`:
      ! `size` must be a scalar integer or double.

# list_combine() `indices` are validated against `size`

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

# list_combine() `default` vector check is done

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! `d` must be a vector, not a <lm> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! `d` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! `d` must be a vector, not a <vctrs_foobar> object.
      x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <vctrs_foobar>.
      i If this object comes from a package, please report this error to the package author.
      i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.

# list_combine() `default` size check is done

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `d` (size 2) to size 1.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `d` (size 2) to size 1.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `d` (size 2) to size 1.

# list_combine() `default` is taken into account when computing `ptype`

    Code
      list_combine(list(x = 1), indices = list(1), size = 2, default = "a",
      default_arg = "d")
    Condition
      Error in `list_combine()`:
      ! Can't combine <double> and `d` <character>.

---

    Code
      list_combine(list(x = 1L), indices = list(1), size = 2, default = 1.5,
      default_arg = "d", ptype = integer())
    Condition
      Error in `list_combine()`:
      ! Can't convert from `d` <double> to <integer> due to loss of precision.
      * Locations: 1

# list_combine() `unmatched = 'error'` errors with unmatched `indices` when `size` is used

    Code
      list_combine(list(1, 1), indices = list(1, 1), size = 2, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(list(1, 1), indices = list(1, NA), size = 2, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(list(1:9, 1:9), indices = list(c(TRUE, FALSE, NA, TRUE, FALSE, NA,
        TRUE, FALSE, NA), c(TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, NA, NA, NA)),
      size = 9, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 5, 6, 8, and 9 are unmatched.

---

    Code
      list_combine(list(1, 3), indices = list(1, 3), size = 3, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(list(1, 1), indices = list(c(TRUE, FALSE), c(TRUE, FALSE)), size = 2,
      unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(list(), indices = list(), size = 2, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 1 and 2 are unmatched.

# list_combine() `unmatched = 'error'` errors pluralize correctly

    Code
      list_combine(list(1, 3), indices = list(1, 3), size = 3, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(list(1, 3), indices = list(1, 3), size = 4, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 2 and 4 are unmatched.

---

    Code
      list_combine(list(1, 3), indices = list(1, 3), size = 100, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 2, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, ..., 99, and 100 are unmatched.

# list_combine() `unmatched = 'error'` can't be set when `default` is also set

    Code
      list_combine(list(1), indices = list(1), default = 1, size = 1, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Can't set `default` when `unmatched = "error"`.

---

    Code
      list_combine(list(1), indices = list(1), default = 1, size = 1, unmatched = "error",
      default_arg = ".default", error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! Can't set `.default` when `unmatched = "error"`.

# list_combine() `unmatched` is validated

    Code
      list_combine(list(1), indices = list(1), size = 1, unmatched = "e")
    Condition
      Error in `list_combine()`:
      ! `unmatched` must be either "default" or "error", not "e".

---

    Code
      list_combine(list(1), indices = list(1), size = 1, unmatched = c("a", "b"))
    Condition
      Error in `list_combine()`:
      ! `unmatched` must be a string, not a character vector.

---

    Code
      list_combine(list(1), indices = list(1), size = 1, unmatched = NA_character_)
    Condition
      Error in `list_combine()`:
      ! `unmatched` must be a string, not a character `NA`.

---

    Code
      list_combine(list(1), indices = list(1), size = 1, unmatched = "e", error_call = quote(
        foo()))
    Condition
      Error in `foo()`:
      ! `unmatched` must be either "default" or "error", not "e".

---

    Code
      list_combine(list(1), indices = list(1), size = 1, unmatched = c("a", "b"),
      error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `unmatched` must be a string, not a character vector.

# list_combine() `multiple` is validated

    Code
      list_combine(list(1), indices = list(1), size = 1, multiple = "a")
    Condition
      Error in `list_combine()`:
      ! `multiple` must be either "last" or "first", not "a".

---

    Code
      list_combine(list(1), indices = list(1), size = 1, multiple = c("a", "b"))
    Condition
      Error in `list_combine()`:
      ! `multiple` must be a string, not a character vector.

---

    Code
      list_combine(list(1), indices = list(1), size = 1, multiple = NA_character_)
    Condition
      Error in `list_combine()`:
      ! `multiple` must be a string, not a character `NA`.

---

    Code
      list_combine(list(1), indices = list(1), size = 1, multiple = "a", error_call = quote(
        foo()))
    Condition
      Error in `foo()`:
      ! `multiple` must be either "last" or "first", not "a".

---

    Code
      list_combine(list(1), indices = list(1), size = 1, multiple = c("a", "b"),
      error_call = quote(foo()))
    Condition
      Error in `foo()`:
      ! `multiple` must be a string, not a character vector.

# `NA` indices are considered unmatched locations

    Code
      list_combine(x = list(1, 2:3), indices = list(1, c(NA, 3)), size = 3,
      unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

---

    Code
      list_combine(x = list(1, 2:3), indices = list(c(TRUE, FALSE, FALSE), c(FALSE,
        NA, TRUE)), size = 3, unmatched = "error")
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 2 is unmatched.

# `x` must be a list

    Code
      list_combine(1, indices = list(1), size = 1)
    Condition
      Error in `list_combine()`:
      ! `x` must be a list, not the number 1.

---

    Code
      list_combine(1, indices = list(1), size = 1, error_call = call("foo"), x_arg = "arg")
    Condition
      Error in `foo()`:
      ! `arg` must be a list, not the number 1.

---

    Code
      list_combine(data.frame(x = 1), indices = list(1), size = 1)
    Condition
      Error in `list_combine()`:
      ! `x` must be a list, not a <data.frame> object.

---

    Code
      list_combine(array(list(1)), indices = list(1), size = 1)
    Condition
      Error in `list_combine()`:
      ! `x` must be a list, not a list 1D array.

# `indices` must be a list

    Code
      list_combine(list(1), indices = 1, size = 1)
    Condition
      Error in `list_combine()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_combine(list(1), indices = 1, size = 1, error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_combine(list(1), indices = data.frame(x = 1), size = 1)
    Condition
      Error in `list_combine()`:
      ! `indices` must be a list, not a <data.frame> object.

# `x` and `indices` must be lists of the same size

    Code
      list_combine(list(1, 2), indices = list(1), size = 1)
    Condition
      Error in `list_combine()`:
      ! `indices` must have size 2, not size 1.

# common type failure after common class fallback reports the original class (#1981)

    Code
      list_combine(list(int, dbl), indices = list(1, 2), size = 2)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      list_combine(list(int, int, dbl), indices = list(1, 2, 3), size = 3)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[3]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      list_combine(list(int, int), indices = list(1, 2), size = 2, default = dbl)
    Condition
      Error in `list_combine()`:
      ! Can't combine <vctrs_foobar> and `default` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# NULL is a valid index

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# combining recycles elements of x to the size of the index

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

# combining takes the common type

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <double> and `x[[2]]` <character>.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# common type failure uses positional errors

    Code
      x <- list(1, a = "x", 2)
      (expect_error(list_combine(x, indices = list(2, 1, 3), size = 3)))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_combine()`:
      ! Can't combine `x[[1]]` <double> and `x$a` <character>.
    Code
      (expect_error(list_combine(x, indices = list(2, 1, 3), size = 3, ptype = double()))
      )
    Output
      <error/vctrs_error_cast>
      Error in `list_combine()`:
      ! Can't convert `x$a` <character> to <double>.
    Code
      y <- list(1, a = 2.5)
      (expect_error(list_combine(y, indices = list(2, 1), size = 2, ptype = integer()))
      )
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_combine()`:
      ! Can't convert from `x$a` <double> to <integer> due to loss of precision.
      * Locations: 1

# can specify a ptype to override common type

    Code
      list_combine(x, indices = indices, size = 2, ptype = integer())
    Condition <vctrs_error_cast_lossy>
      Error in `list_combine()`:
      ! Can't convert from `x[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      list_combine(x, indices = indices, size = 2, ptype = integer(), error_call = call(
        "foo"), x_arg = "arg")
    Condition <vctrs_error_cast_lossy>
      Error in `foo()`:
      ! Can't convert from `arg[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1

# outer names are recycled in the right order

    Code
      list_combine(x, indices = list(c(1, 2), 3), size = 3)
    Condition
      Error in `list_combine()`:
      ! Can't merge the outer name `x` with a vector of length > 1.
      Please supply a `.name_spec` specification.

# list_combine() can repair names quietly

    Code
      res <- list_combine(vec_chop(x, indices = indices), indices = indices, size = 3,
      name_repair = "unique_quiet")

---

    Code
      res <- list_combine(vec_chop(x, indices = indices), indices = indices, size = 3,
      name_repair = "universal_quiet")

# list_combine() errors on unsupported location values

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

# list_combine() fallback doesn't support `name_spec` or `ptype`

    Code
      foo <- structure(foobar(1), foo = "foo")
      bar <- structure(foobar(2), bar = "bar")
      (expect_error(with_c_foobar(list_combine(list(foo, bar), indices = list(1, 2),
      size = 2, name_spec = "{outer}_{inner}")), "name specification"))
    Output
      <error/rlang_error>
      Error in `list_combine()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      (expect_error(with_c_foobar(list_combine(list(foo, bar), indices = list(1, 2),
      size = 2, name_spec = "{outer}_{inner}", error_call = call("foo"))),
      "name specification"))
    Output
      <error/rlang_error>
      Error in `foo()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.
    Code
      x <- list(foobar(1))
      (expect_error(with_c_foobar(list_combine(x, indices = list(1), size = 1, ptype = "")),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_cast>
      Error in `list_combine()`:
      ! Can't convert `x[[1]]` <vctrs_foobar> to <character>.

# list_combine() does not support non-numeric S3 indices

    Code
      (expect_error(list_combine(list(1), indices = list(factor("x")), size = 1),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not the string "x".
    Code
      (expect_error(list_combine(list(1), indices = list(foobar(1L)), size = 1),
      class = "vctrs_error_subscript_type"))
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a <vctrs_foobar> object.

# `list_combine()` with logical `indices` checks `indices` size

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a logical vector.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a logical vector.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a logical vector.

# `multiple` shows correctly indexed errors

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[2]]` (size 3) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Can't recycle `x[[1]]` (size 2) to size 4.

# `compact_seq()` `indices` work with `unmatched`

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 3 is unmatched.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 3 is unmatched.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Location 3 is unmatched.

---

    Code
      list_combine(x, indices = indices, size = size, default = default, unmatched = unmatched,
        multiple = multiple, slice_x = slice_x, ptype = ptype, name_spec = name_spec,
        name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 2 and 3 are unmatched.

---

    Code
      list_combine(x_foobar, indices = indices, size = size, default = default_foobar,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 2 and 3 are unmatched.

---

    Code
      list_combine(x_foobar_c, indices = indices, size = size, default = default_foobar_c,
        unmatched = unmatched, multiple = multiple, slice_x = slice_x, ptype = ptype,
        name_spec = name_spec, name_repair = name_repair, x_arg = x_arg, indices_arg = indices_arg,
        default_arg = default_arg)
    Condition
      Error in `list_combine()`:
      ! Each location must be matched.
      x Locations 2 and 3 are unmatched.

# inputs must be named

    Code
      vec_expand_grid(1)
    Condition
      Error in `vec_expand_grid()`:
      ! All inputs must be named.

---

    Code
      vec_expand_grid(x = 1, 2, y = 3)
    Condition
      Error in `vec_expand_grid()`:
      ! All inputs must be named.

# catches duplicate names by default

    Code
      vec_expand_grid(a = 1, a = 2)
    Condition
      Error in `vec_expand_grid()`:
      ! Names must be unique.
      x These names are duplicated:
        * "a" at locations 1 and 2.
      i Use argument `.name_repair` to specify repair strategy.

# errors on non vectors and mentions the element name

    Code
      vec_expand_grid(y = environment())
    Condition
      Error in `vec_expand_grid()`:
      ! `y` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# can adjust the `.error_call`

    Code
      my_expand_grid()
    Condition
      Error in `my_expand_grid()`:
      ! `x` must be a vector, not an environment.
      i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.

# errors nicely when expansion results in a size larger than `R_len_t`

    Code
      vec_expand_grid(x = x, y = y)
    Condition
      Error in `vec_expand_grid()`:
      ! Long vectors are not yet supported. Expansion results in an allocation larger than 2^31-1 elements. Attempted allocation size was 3221225469.

# errors nicely when expansion results in a size larger than `R_xlen_t`

    Code
      vec_expand_grid(x = x, y = x)
    Condition
      Error in `vec_expand_grid()`:
      ! Result too large for an `r_ssize`.
      i In file './rlang/c-utils.h' at line <scrubbed>.
      i This is an internal error that was detected in the vctrs package.
        Please report it at <https://github.com/r-lib/vctrs/issues> with a reprex (<https://tidyverse.org/help/>) and the full backtrace.

# validates `.vary`

    Code
      vec_expand_grid(.vary = 1)
    Condition
      Error in `vec_expand_grid()`:
      ! `.vary` must be a string or character vector.

---

    Code
      vec_expand_grid(.vary = "x")
    Condition
      Error in `vec_expand_grid()`:
      ! `.vary` must be one of "slowest" or "fastest", not "x".

# Casting to named argument mentions 'match type <foo>'

    Code
      vec_cast(1, "", x_arg = "foo", to_arg = "bar")
    Condition
      Error:
      ! Can't convert `foo` <double> to match type of `bar` <character>.

---

    Code
      vec_cast(1, "", x_arg = "foo")
    Condition
      Error:
      ! Can't convert `foo` <double> to <character>.

# cast errors create helpful messages (#57, #225)

    Code
      vec_cast(1.5, 10L)
    Condition
      Error:
      ! Can't convert from `1.5` <double> to <integer> due to loss of precision.
      * Locations: 1

---

    Code
      vec_cast(factor("foo"), 10)
    Condition
      Error:
      ! Can't convert `factor("foo")` <factor<c1562>> to <double>.

---

    Code
      x <- tibble(a = tibble(b = 1.5))
      y <- tibble(a = tibble(b = 10L))
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert from `x$a$b` <double> to `a$b` <integer> due to loss of precision.
      * Locations: 1

---

    Code
      x <- tibble(a = tibble(b = factor("foo")))
      y <- tibble(a = tibble(b = 10))
      vec_cast(x, y)
    Condition
      Error:
      ! Can't convert `x$a$b` <factor<c1562>> to match type of `a$b` <double>.

---

    Code
      x <- tibble(a = tibble(b = factor("foo")))
      y <- tibble(a = tibble(b = 10))
      vec_cast_common(x, y)
    Condition
      Error:
      ! Can't combine `..1$a$b` <factor<c1562>> and `..2$a$b` <double>.

# vec_cast() only attempts to fall back if `to` is a data frame (#1568)

    Code
      (expect_error(vec_cast(foobar(mtcars), 1), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_cast>
      Error:
      ! Can't convert `foobar(mtcars)` <vctrs_foobar> to <double>.

# can signal deprecation warnings for lossy casts

    Code
      (expect_warning(expect_true(lossy_cast())))
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      Coercion with lossy casts was deprecated in vctrs 0.2.0.
      i Please use `allow_lossy_cast()` instead.
      i We detected a lossy transformation from `x` <fct> to `to` <fct>. The result will contain lower-resolution values or missing values. To suppress this warning, wrap your code with `allow_lossy_cast()`.

# can cast to unspecified `NA` with `vec_cast()` and `vec_cast_common()` (#2099)

    Code
      vec_cast(TRUE, to = unspecified(1))
    Condition
      Error:
      ! Can't convert `TRUE` <logical> to <vctrs_unspecified>.

# casting performs expected allocations

    Code
      x <- matrix(rep(1L, 100), ncol = 2)
      with_memory_prof(vec_cast(x, x))
    Output
      [1] 0B
    Code
      x <- matrix(rep(1L, 100), ncol = 2)
      y <- matrix(rep(1, 100), ncol = 2)
      with_memory_prof(vec_cast(x, y))
    Output
      [1] 848B

# `x` must be a list

    Code
      list_unchop(1, indices = list(1))
    Condition
      Error in `list_unchop()`:
      ! `x` must be a list, not the number 1.

---

    Code
      list_unchop(1, indices = list(1), error_call = call("foo"), error_arg = "arg")
    Condition
      Error in `foo()`:
      ! `arg` must be a list, not the number 1.

---

    Code
      list_unchop(data.frame(x = 1), indices = list(1))
    Condition
      Error in `list_unchop()`:
      ! `x` must be a list, not a <data.frame> object.

# `indices` must be a list

    Code
      list_unchop(list(1), indices = 1)
    Condition
      Error in `list_unchop()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_unchop(list(1), indices = 1, error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! `indices` must be a list, not the number 1.

---

    Code
      list_unchop(list(1), indices = data.frame(x = 1))
    Condition
      Error in `list_unchop()`:
      ! `indices` must be a list, not a <data.frame> object.

# `x` and `indices` must be lists of the same size

    Code
      list_unchop(list(1, 2), indices = list(1))
    Condition
      Error in `list_unchop()`:
      ! `indices` must have size 2, not size 1.

# `NULL` is a valid index

    Code
      list_unchop(list(1, 2), indices = list(NULL, 2))
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

---

    Code
      list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 2))
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

---

    Code
      list_unchop(list(foobar(1), foobar(2)), indices = list(NULL, 2))
    Condition
      Error:
      ! Can't subset elements past the end.
      i Location 2 doesn't exist.
      i There is only 1 element.

# unchopping recycles elements of x to the size of the index

    Code
      list_unchop(list(1:2), indices = indices)
    Condition
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_unchop(list(1:2), indices = indices, error_call = call("foo"), error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

---

    Code
      list_unchop(list(foobar(1:2)), indices = indices)
    Condition
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_unchop(list(foobar(1:2)), indices = indices, error_call = call("foo"),
      error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

---

    Code
      list_unchop(list(foobar(1:2)), indices = indices)
    Condition
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.

---

    Code
      list_unchop(list(foobar(1:2)), indices = indices, error_call = call("foo"),
      error_arg = "arg")
    Condition
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

# unchopping takes the common type

    Code
      (expect_error(list_unchop(x, indices = indices), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x[[2]]` <character>.
    Code
      (expect_error(list_unchop(x, indices = indices, error_call = call("foo"),
      error_arg = "arg"), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <double> and `arg[[2]]` <character>.

# common type failure uses positional errors

    Code
      x <- list(1, a = "x", 2)
      (expect_error(list_unchop(x)))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x$a` <character>.
    Code
      (expect_error(list_unchop(x, indices = list(2, 1, 3))))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <double> and `x$a` <character>.
    Code
      (expect_error(list_unchop(x, ptype = double())))
    Output
      <error/vctrs_error_cast>
      Error in `list_unchop()`:
      ! Can't convert `x$a` <character> to <double>.
    Code
      (expect_error(list_unchop(x, indices = list(2, 1, 3), ptype = double())))
    Output
      <error/vctrs_error_cast>
      Error in `list_unchop()`:
      ! Can't convert `x$a` <character> to <double>.
    Code
      y <- list(1, a = 2.5)
      (expect_error(list_unchop(y, ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x$a` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(list_unchop(y, indices = list(2, 1), ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x$a` <double> to <integer> due to loss of precision.
      * Locations: 1

# can specify a ptype to override common type

    Code
      (expect_error(list_unchop(x, indices = indices, ptype = integer())))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `list_unchop()`:
      ! Can't convert from `x[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1
    Code
      (expect_error(list_unchop(x, indices = indices, ptype = integer(), error_call = call(
        "foo"), error_arg = "arg")))
    Output
      <error/vctrs_error_cast_lossy>
      Error in `foo()`:
      ! Can't convert from `arg[[1]]` <double> to <integer> due to loss of precision.
      * Locations: 1

# list_unchop() can repair names quietly

    Code
      res <- list_unchop(vec_chop(x, indices = indices), indices = indices,
      name_repair = "unique_quiet")

---

    Code
      res <- list_unchop(vec_chop(x, indices = indices), indices = indices,
      name_repair = "universal_quiet")

# list_unchop() errors on unsupported location values

    Code
      list_unchop(list(1, 2), indices = list(c(1, 2), 0))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_unchop(list(1), indices = list(-1))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

---

    Code
      list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), 0))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_unchop(list(foobar(1)), indices = list(-1))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

---

    Code
      list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 2), 0))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain `0` values.
      i It has a `0` value at location 1.

---

    Code
      list_unchop(list(foobar(1)), indices = list(-1))
    Condition <vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript can't contain negative locations.

# list_unchop() fails with complex foreign S3 classes

    Code
      x <- structure(foobar(1), attr_foo = "foo")
      y <- structure(foobar(2), attr_bar = "bar")
      (expect_error(list_unchop(list(x, y)), class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_foobar> and `x[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(list_unchop(list(x, y), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_foobar> and `arg[[2]]` <vctrs_foobar>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_unchop() fails with complex foreign S4 classes

    Code
      joe <- .Counts(c(1L, 2L), name = "Joe")
      jane <- .Counts(3L, name = "Jane")
      (expect_error(list_unchop(list(joe, jane)), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.
    Code
      (expect_error(list_unchop(list(joe, jane), error_call = call("foo"), error_arg = "arg"),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `foo()`:
      ! Can't combine `arg[[1]]` <vctrs_Counts> and `arg[[2]]` <vctrs_Counts>.
      x Some attributes are incompatible.
      i The author of the class should implement vctrs methods.
      i See <https://vctrs.r-lib.org/reference/faq-error-incompatible-attributes.html>.

# list_unchop() falls back to c() if S3 method is available

    Code
      (expect_error(list_unchop(list(foobar(1), foobar(2)), indices = list(c(1, 3),
      integer())), class = "vctrs_error_subscript_oob"))
    Output
      <error/vctrs_error_subscript_oob>
      Error:
      ! Can't subset elements past the end.
      i Location 3 doesn't exist.
      i There are only 2 elements.

---

    Code
      x <- list(foobar(1:2))
      indices <- list(1:3)
      (expect_error(list_unchop(x, indices = indices)))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `list_unchop()`:
      ! Can't recycle `x[[1]]` (size 2) to size 3.
    Code
      (expect_error(list_unchop(x, indices = indices, error_arg = "arg", error_call = call(
        "foo"))))
    Output
      <error/vctrs_error_incompatible_size>
      Error in `foo()`:
      ! Can't recycle `arg[[1]]` (size 2) to size 3.

# list_unchop() falls back for S4 classes with a registered c() method

    Code
      (expect_error(list_unchop(list(joe, 1, jane), indices = list(c(1, 2), 3, 4)),
      class = "vctrs_error_incompatible_type"))
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x[[1]]` <vctrs_Counts> and `x[[2]]` <double>.

# list_unchop() fallback doesn't support (most) `name_spec` or `ptype`

    Code
      list_unchop(list(foo, bar), indices = list(1, 2), name_spec = "{outer}_{inner}")
    Condition
      Error in `list_unchop()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.

---

    Code
      list_unchop(list(foo, bar), indices = list(1, 2), name_spec = "{outer}_{inner}",
      error_call = call("foo"))
    Condition
      Error in `foo()`:
      ! Can't use a name specification with non-vctrs types.
      vctrs methods must be implemented for class `vctrs_foobar`.
      See <https://vctrs.r-lib.org/articles/s3-vector.html>.

---

    Code
      list_unchop(x, indices = list(1), ptype = "")
    Condition
      Error in `list_unchop()`:
      ! Can't convert `x[[1]]` <vctrs_foobar> to <character>.

# list_unchop() does not support non-numeric S3 indices

    Code
      (expect_error(list_unchop(list(1), indices = list(factor("x"))), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not the string "x".
    Code
      (expect_error(list_unchop(list(1), indices = list(foobar(1L))), class = "vctrs_error_subscript_type")
      )
    Output
      <error/vctrs_error_subscript_type>
      Error:
      ! Can't subset elements.
      x Subscript must be numeric, not a <vctrs_foobar> object.

# can ignore names in `list_unchop()` by providing a `zap()` name-spec (#232)

    Code
      (expect_error(list_unchop(list(a = c(b = 1:2)))))
    Output
      <error/rlang_error>
      Error in `list_unchop()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.
    Code
      (expect_error(list_unchop(list(a = c(b = 1:2)), error_call = call("foo"))))
    Output
      <error/rlang_error>
      Error in `list_unchop()`:
      ! Can't merge the outer name `a` with a vector of length > 1.
      Please supply a `.name_spec` specification.

---

    Code
      x <- list(a = c(b = letters), b = 3L)
      (expect_error(list_unchop(x, name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x$a` <character> and `x$b` <integer>.
    Code
      x <- list(a = c(foo = 1:2), b = c(bar = ""))
      (expect_error(list_unchop(x, indices = list(2:1, 3), name_spec = zap()), class = "vctrs_error_incompatible_type")
      )
    Output
      <error/vctrs_error_ptype2>
      Error in `list_unchop()`:
      ! Can't combine `x$a` <integer> and `x$b` <character>.

# can ignore outer names in `list_unchop()` by providing a 'inner' name-spec (#1988)

    Code
      list_unchop(list(x = c(a = 1), y = c(b = "2")), indices = list(1, 2),
      name_spec = "inner")
    Condition
      Error in `list_unchop()`:
      ! Can't combine `x$x` <double> and `x$y` <character>.

# calls cast method even with empty objects

    Code
      list_unchop(list(foobar(integer()), foobar(integer(), foo = "bar")), indices = list(
        integer(), integer()))
    Condition
      Error in `list_unchop()`:
      ! Can't convert `x[[2]]` <vctrs_foobar> to <vctrs_foobar>.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cast.R, R/type-bare.R
\name{vec_cast}
\alias{vec_cast}
\alias{vec_cast_common}
\alias{vec_cast.logical}
\alias{vec_cast.integer}
\alias{vec_cast.double}
\alias{vec_cast.complex}
\alias{vec_cast.raw}
\alias{vec_cast.character}
\alias{vec_cast.list}
\title{Cast a vector to a specified type}
\usage{
vec_cast(x, to, ..., x_arg = caller_arg(x), to_arg = "", call = caller_env())

vec_cast_common(..., .to = NULL, .arg = "", .call = caller_env())

\method{vec_cast}{logical}(x, to, ...)

\method{vec_cast}{integer}(x, to, ...)

\method{vec_cast}{double}(x, to, ...)

\method{vec_cast}{complex}(x, to, ...)

\method{vec_cast}{raw}(x, to, ...)

\method{vec_cast}{character}(x, to, ...)

\method{vec_cast}{list}(x, to, ...)
}
\arguments{
\item{x}{Vectors to cast.}

\item{to, .to}{Type to cast to. If \code{NULL}, \code{x} will be returned as is.}

\item{...}{For \code{vec_cast_common()}, vectors to cast. For
\code{vec_cast()}, \code{vec_cast_default()}, and \code{vec_restore()}, these
dots are only for future extensions and should be empty.}

\item{x_arg}{Argument name for \code{x}, used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{to_arg}{Argument name \code{to} used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{call, .call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{.arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}
}
\value{
A vector the same length as \code{x} with the same type as \code{to},
or an error if the cast is not possible. An error is generated if
information is lost when casting between compatible types (i.e. when
there is no 1-to-1 mapping for a specific value).
}
\description{
\code{vec_cast()} provides directional conversions from one type of
vector to another. Along with \code{\link[=vec_ptype2]{vec_ptype2()}}, this generic forms
the foundation of type coercions in vctrs.
}
\section{Implementing coercion methods}{
\itemize{
\item For an overview of how these generics work and their roles in vctrs,
see \code{\link[=theory-faq-coercion]{?theory-faq-coercion}}.
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
\item For a tutorial about implementing vctrs classes from scratch, see
\code{vignette("s3-vector")}.
}
}

\section{Dependencies of \code{vec_cast_common()}}{

\subsection{vctrs dependencies}{
\itemize{
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
}
}

\subsection{base dependencies}{

Some functions enable a base-class fallback for
\code{vec_cast_common()}. In that case the inputs are deemed compatible
when they have the same \link[base:typeof]{base type} and inherit from
the same base class.
}
}

\examples{
# x is a double, but no information is lost
vec_cast(1, integer())

# When information is lost the cast fails
try(vec_cast(c(1, 1.5), integer()))
try(vec_cast(c(1, 2), logical()))

# You can suppress this error and get the partial results
allow_lossy_cast(vec_cast(c(1, 1.5), integer()))
allow_lossy_cast(vec_cast(c(1, 2), logical()))

# By default this suppress all lossy cast errors without
# distinction, but you can be specific about what cast is allowed
# by supplying prototypes
allow_lossy_cast(vec_cast(c(1, 1.5), integer()), to_ptype = integer())
try(allow_lossy_cast(vec_cast(c(1, 2), logical()), to_ptype = integer()))

# No sensible coercion is possible so an error is generated
try(vec_cast(1.5, factor("a")))

# Cast to common type
vec_cast_common(factor("a"), factor(c("a", "b")))
}
\seealso{
Call \code{\link[=stop_incompatible_cast]{stop_incompatible_cast()}} when you determine from the
attributes that an input can't be cast to the target type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_unchop}
\alias{vec_unchop}
\title{Chopping}
\usage{
vec_unchop(
  x,
  indices = NULL,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c("minimal", "unique", "check_unique", "universal")
)
}
\arguments{
\item{x}{A list}

\item{indices}{A list of positive integer vectors specifying the
locations to place elements of \code{x} in. Each element of \code{x} is recycled to
the size of the corresponding index vector. The size of \code{indices} must
match the size of \code{x}. If \code{NULL}, \code{x} is combined in the order it is
provided in, which is equivalent to using \code{\link[=vec_c]{vec_c()}}.}

\item{ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type across all elements of \code{x}. Alternatively, you
can supply \code{ptype} to give the output a known type.}

\item{name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}

\item{name_repair}{How to repair names, see \code{repair} options in
\code{\link[=vec_as_names]{vec_as_names()}}.}
}
\value{
A vector of type \code{vec_ptype_common(!!!x)}, or \code{ptype}, if specified. The size
is computed as \code{vec_size_common(!!!indices)} unless the indices are \code{NULL},
in which case the size is \code{vec_size_common(!!!x)}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{vec_unchop()} has been renamed to \code{\link[=list_unchop]{list_unchop()}} and is deprecated as of
vctrs 0.5.0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slice-chop.R
\name{vec_chop}
\alias{vec_chop}
\title{Chopping}
\usage{
vec_chop(x, ..., indices = NULL, sizes = NULL)
}
\arguments{
\item{x}{A vector}

\item{...}{These dots are for future extensions and must be empty.}

\item{indices}{A list of positive integer vectors to slice \code{x} with, or
\code{NULL}. Can't be used if \code{sizes} is already specified. If both \code{indices}
and \code{sizes} are \code{NULL}, \code{x} is split into its individual elements,
equivalent to using an \code{indices} of \code{as.list(vec_seq_along(x))}.}

\item{sizes}{An integer vector of non-negative sizes representing sequential
indices to slice \code{x} with, or \code{NULL}. Can't be used if \code{indices} is already
specified.

For example, \code{sizes = c(2, 4)} is equivalent to \code{indices = list(1:2, 3:6)},
but is typically faster.

\code{sum(sizes)} must be equal to \code{vec_size(x)}, i.e. \code{sizes} must completely
partition \code{x}, but an individual size is allowed to be \code{0}.}
}
\value{
A list where each element has the same type as \code{x}. The size of the list is
equal to \code{vec_size(indices)}, \code{vec_size(sizes)}, or \code{vec_size(x)} depending
on whether or not \code{indices} or \code{sizes} is provided.
}
\description{
\code{vec_chop()} provides an efficient method to repeatedly slice a vector. It
captures the pattern of \code{map(indices, vec_slice, x = x)}. When no indices
are supplied, it is generally equivalent to \code{\link[=as.list]{as.list()}}.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
vec_chop(1:5)

# These two are equivalent
vec_chop(1:5, indices = list(1:2, 3:5))
vec_chop(1:5, sizes = c(2, 3))

# Can also be used on data frames
vec_chop(mtcars, indices = list(1:3, 4:6))

# If you know your input is sorted and you'd like to split on the groups,
# `vec_run_sizes()` can be efficiently combined with `sizes`
df <- data_frame(
  g = c(2, 5, 5, 6, 6, 6, 6, 8, 9, 9),
  x = 1:10
)
vec_chop(df, sizes = vec_run_sizes(df$g))

# If you have a list of homogeneous vectors, sometimes it can be useful to
# combine, apply a function to the flattened vector, and chop according
# to the original indices. This can be done efficiently with `list_sizes()`.
x <- list(c(1, 2, 1), c(3, 1), 5, double())
x_flat <- vec_c(!!!x)
x_flat <- x_flat + max(x_flat)
vec_chop(x_flat, sizes = list_sizes(x))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_repeat}
\alias{vec_repeat}
\title{Expand the length of a vector}
\usage{
vec_repeat(x, each = 1L, times = 1L)
}
\arguments{
\item{x}{A vector.}

\item{each}{Number of times to repeat each element of \code{x}.}

\item{times}{Number of times to repeat the whole vector of \code{x}.}
}
\value{
A vector the same type as \code{x} with size \code{vec_size(x) * times * each}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{vec_repeat()} has been replaced with \code{\link[=vec_rep]{vec_rep()}} and \code{\link[=vec_rep_each]{vec_rep_each()}} and is
deprecated as of vctrs 0.3.0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/numeric.R
\name{vec_math}
\alias{vec_math}
\alias{vec_math_base}
\title{Mathematical operations}
\usage{
vec_math(.fn, .x, ...)

vec_math_base(.fn, .x, ...)
}
\arguments{
\item{.fn}{A mathematical function from the base package, as a string.}

\item{.x}{A vector.}

\item{...}{Additional arguments passed to \code{.fn}.}
}
\description{
This generic provides a common dispatch mechanism for all regular unary
mathematical functions. It is used as a common wrapper around many of the
Summary group generics, the Math group generics, and a handful of other
mathematical functions like \code{mean()} (but not \code{var()} or \code{sd()}).
}
\details{
\code{vec_math_base()} is provided as a convenience for writing methods. It
calls the base \code{.fn} on the underlying \code{\link[=vec_data]{vec_data()}}.
}
\section{Included functions}{

\itemize{
\item From the \link{Summary} group generic:
\code{prod()}, \code{sum()}, \code{any()}, \code{all()}.
\item From the \link{Math} group generic:
\code{abs()}, \code{sign()}, \code{sqrt()}, \code{ceiling()}, \code{floor()}, \code{trunc()}, \code{cummax()},
\code{cummin()}, \code{cumprod()}, \code{cumsum()}, \code{log()}, \code{log10()}, \code{log2()},
\code{log1p()}, \code{acos()}, \code{acosh()}, \code{asin()}, \code{asinh()}, \code{atan()}, \code{atanh()},
\code{exp()}, \code{expm1()}, \code{cos()}, \code{cosh()}, \code{cospi()}, \code{sin()}, \code{sinh()},
\code{sinpi()}, \code{tan()}, \code{tanh()}, \code{tanpi()}, \code{gamma()}, \code{lgamma()},
\code{digamma()}, \code{trigamma()}.
\item Additional generics: \code{mean()}, \code{is.nan()}, \code{is.finite()}, \code{is.infinite()}.
}

Note that \code{median()} is currently not implemented, and \code{sd()} and
\code{var()} are currently not generic and so do not support custom
classes.
}

\examples{
x <- new_vctr(c(1, 2.5, 10))
x

abs(x)
sum(x)
cumsum(x)
}
\seealso{
\code{\link[=vec_arith]{vec_arith()}} for the equivalent for the arithmetic infix operators.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recode.R
\name{vec-recode-and-replace}
\alias{vec-recode-and-replace}
\alias{vec_recode_values}
\alias{vec_replace_values}
\title{Recode and replace values}
\usage{
vec_recode_values(
  x,
  ...,
  from,
  to,
  default = NULL,
  unmatched = "default",
  from_as_list_of_vectors = FALSE,
  to_as_list_of_vectors = FALSE,
  ptype = NULL,
  x_arg = "x",
  from_arg = "from",
  to_arg = "to",
  default_arg = "default",
  error_call = current_env()
)

vec_replace_values(
  x,
  ...,
  from,
  to,
  from_as_list_of_vectors = FALSE,
  to_as_list_of_vectors = FALSE,
  x_arg = "x",
  from_arg = "from",
  to_arg = "to",
  error_call = current_env()
)
}
\arguments{
\item{x}{A vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{from}{Values to locate in \code{x} and map to values in \code{to}.

Extra values present in \code{from} but not in \code{x} are ignored.
\itemize{
\item If \code{from_as_list_of_vectors = FALSE}, \code{from} must be a single vector of
any size, which will be \link[=theory-faq-coercion]{cast} to the type of
\code{x}.
\item If \code{from_as_list_of_vectors = TRUE}, \code{from} must be a list of vectors of
any size, which will individually be \link[=theory-faq-coercion]{cast}
to the type of \code{x}.
}}

\item{to}{Values to map \code{from} to.

The common type of \code{to} and \code{default} will determine the type of the
output, unless \code{ptype} is provided.
\itemize{
\item If \code{to_as_list_of_vectors = FALSE}, \code{to} must be a single vector of size
1 or the same size as \code{from}.
\item If \code{to_as_list_of_vectors = TRUE}, \code{to} must be a list of vectors. The
list itself must be size 1 or the same size as \code{from}. Each individual
vector in the list must be size 1 or the same size as \code{x}.
}}

\item{default}{Default value to use when there is a value present in \code{x}
that is unmatched by a value in \code{from}.

By default, a missing value is used as the default value.

If supplied, \code{default} must be size 1 or the same size as \code{x}.

Can only be set when \code{unmatched = "default"}.}

\item{unmatched}{Handling of unmatched locations.

One of:
\itemize{
\item \code{"default"} to use \code{default} in unmatched locations.
\item \code{"error"} to error when there are unmatched locations.
}}

\item{from_as_list_of_vectors, to_as_list_of_vectors}{Boolean values
determining whether to treat \code{from} and \code{to} as vectors or as lists of
vectors. See their parameter descriptions for more details.}

\item{ptype}{An optional override for the output type, which is usually
computed as the common type of \code{to} and \code{default}.}

\item{x_arg, from_arg, to_arg, default_arg}{Argument names used in error
messages.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector the same size as \code{x}.
\itemize{
\item For \code{vec_recode_values()}, the type of the output is computed as the common
type of \code{to} and \code{default}, unless overridden by \code{ptype}. The names of the
output come from the names of \code{to} and \code{default}.
\item For \code{vec_replace_values()}, the type of the output will have the same type
as \code{x}. The names of the output will be the same as the names of \code{x}.
}
}
\description{
\itemize{
\item \code{vec_recode_values()} constructs an entirely new vector by recoding the
values from \code{x} specified in \code{from} to the corresponding values in \code{to}. If
there are values in \code{x} not matched by \code{from}, then they are recoded to the
\code{default} value.
\item \code{vec_replace_values()} updates an existing vector by replacing the values
from \code{x} specified in \code{from} with the corresponding values in \code{to}. In this
case, \code{to} must have the same type as \code{x} and values in \code{x} not matched by
\code{from} pass through untouched.
}
}
\examples{
x <- c(1, 2, 3, 1, 2, 4, NA, 5)

# Imagine you have a pre-existing lookup table
likert <- data.frame(
  from = c(1, 2, 3, 4, 5),
  to = c(
    "Strongly disagree",
    "Disagree",
    "Neutral",
    "Agree",
    "Strongly agree"
  )
)
vec_recode_values(x, from = likert$from, to = likert$to)

# If you don't map all of the values, a `default` is used
x <- c(1, 2, 3, 1, 2, 4, NA, 5, 6, 7)
vec_recode_values(x, from = likert$from, to = likert$to)
vec_recode_values(x, from = likert$from, to = likert$to, default = "Unknown")

# If you want existing `NA`s to pass through, include a mapping for `NA` in
# your lookup table
likert <- data.frame(
  from = c(1, 2, 3, 4, 5, NA),
  to = c(
    "Strongly disagree",
    "Disagree",
    "Neutral",
    "Agree",
    "Strongly agree",
    NA
  )
)
vec_recode_values(x, from = likert$from, to = likert$to, default = "Unknown")

# If you believe you've captured all of the cases, you can assert this with
# `unmatched = "error"`, which will error if you've missed any cases
# (including `NA`, which must be explicitly handled)
try(vec_recode_values(
  x,
  from = likert$from,
  to = likert$to,
  unmatched = "error"
))

if (require("tibble")) {
  # If you want to partially update `x`, retaining the type of `x` and
  # leaving values not covered by `from` alone, use `vec_replace_values()`
  universities <- c(
    "Duke",
    "Fake U",
    "Duke U",
    NA,
    "Chapel Hill",
    "UNC",
    NA,
    "Duke"
  )

  standardize <- tibble::tribble(
    ~from, ~to,
    "Duke", "Duke University",
    "Duke U", "Duke University",
    "UNC", "UNC Chapel Hill",
    "Chapel Hill", "UNC Chapel Hill",
  )
  vec_replace_values(
    universities,
    from = standardize$from,
    to = standardize$to
  )

  # In this case, you can use a more powerful feature of
  # `vec_replace_values()`, `from_as_list_of_vectors`, which allows you to
  # provide a list of `from` vectors that each match multiple `from` values
  # to a single `to` value. `tribble()` can help you create these!
  standardize <- tibble::tribble(
    ~from, ~to,
    c("Duke", "Duke U"), "Duke University",
    c("UNC", "Chapel Hill"), "UNC Chapel Hill",
  )

  # Note how `from` is a list column
  standardize

  vec_replace_values(
    universities,
    from = standardize$from,
    to = standardize$to,
    from_as_list_of_vectors = TRUE
  )

  # `vec_replace_values()` is also a useful way to map from or to `NA`
  vec_replace_values(universities, from = NA, to = "Unknown")
  vec_replace_values(universities, from = "Fake U", to = NA)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fill.R
\name{vec_fill_missing}
\alias{vec_fill_missing}
\title{Fill in missing values with the previous or following value}
\usage{
vec_fill_missing(
  x,
  direction = c("down", "up", "downup", "updown"),
  max_fill = NULL
)
}
\arguments{
\item{x}{A vector}

\item{direction}{Direction in which to fill missing values. Must be either
\code{"down"}, \code{"up"}, \code{"downup"}, or \code{"updown"}.}

\item{max_fill}{A single positive integer specifying the maximum number of
sequential missing values that will be filled. If \code{NULL}, there is
no limit.}
}
\description{
\code{vec_fill_missing()} fills gaps of missing values with the previous or
following non-missing value.
}
\examples{
x <- c(NA, NA, 1, NA, NA, NA, 3, NA, NA)

# Filling down replaces missing values with the previous non-missing value
vec_fill_missing(x, direction = "down")

# To also fill leading missing values, use `"downup"`
vec_fill_missing(x, direction = "downup")

# Limit the number of sequential missing values to fill with `max_fill`
vec_fill_missing(x, max_fill = 1)

# Data frames are filled rowwise. Rows are only considered missing
# if all elements of that row are missing.
y <- c(1, NA, 2, NA, NA, 3, 4, NA, 5)
df <- data_frame(x = x, y = y)
df

vec_fill_missing(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{vec_proxy_compare}
\alias{vec_proxy_compare}
\alias{vec_proxy_order}
\title{Comparison and order proxy}
\usage{
vec_proxy_compare(x, ...)

vec_proxy_order(x, ...)
}
\arguments{
\item{x}{A vector x.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A 1d atomic vector or a data frame.
}
\description{
\code{vec_proxy_compare()} and \code{vec_proxy_order()} return proxy objects, i.e.
an atomic vector or data frame of atomic vectors.

For \code{\link[=vctr]{vctrs_vctr}} objects:
\itemize{
\item \code{vec_proxy_compare()} determines the behavior of \code{<}, \code{>}, \code{>=}
and \code{<=} (via \code{\link[=vec_compare]{vec_compare()}}); and \code{\link[=min]{min()}}, \code{\link[=max]{max()}}, \code{\link[=median]{median()}}, and
\code{\link[=quantile]{quantile()}}.
\item \code{vec_proxy_order()} determines the behavior of \code{order()} and \code{sort()}
(via \code{xtfrm()}).
}
}
\details{
The default method of \code{vec_proxy_compare()} assumes that all classes built
on top of atomic vectors or records are comparable. Internally the default
calls \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}. If your class is not comparable, you will need
to provide a \code{vec_proxy_compare()} method that throws an error.

The behavior of \code{vec_proxy_order()} is identical to \code{vec_proxy_compare()},
with the exception of lists. Lists are not comparable, as comparing
elements of different types is undefined. However, to allow ordering of
data frames containing list-columns, the ordering proxy of a list is
generated as an integer vector that can be used to order list elements
by first appearance.

If a class implements a \code{vec_proxy_compare()} method, it usually doesn't need
to provide a \code{vec_proxy_order()} method, because the latter is implemented
by forwarding to \code{vec_proxy_compare()} by default. Classes inheriting from
list are an exception: due to the default \code{vec_proxy_order()} implementation,
\code{vec_proxy_compare()} and \code{vec_proxy_order()} should be provided for such
classes (with identical implementations) to avoid mismatches between
comparison and sorting.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}} called by default in \code{vec_proxy_compare()}
\item \code{\link[=vec_proxy_compare]{vec_proxy_compare()}} called by default in \code{vec_proxy_order()}
}
}

\section{Data frames}{

If the proxy for \code{x} is a data frame, the proxy function is automatically
recursively applied on all columns as well. After applying the proxy
recursively, if there are any data frame columns present in the proxy, then
they are unpacked. Finally, if the resulting data frame only has a single
column, then it is unwrapped and a vector is returned as the proxy.
}

\examples{
# Lists are not comparable
x <- list(1:2, 1, 1:2, 3)
try(vec_compare(x, x))

# But lists are orderable by first appearance to allow for
# ordering data frames with list-cols
df <- new_data_frame(list(x = x))
vec_sort(df)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/names.R
\name{vec_names}
\alias{vec_names}
\alias{vec_names2}
\alias{vec_set_names}
\title{Get or set the names of a vector}
\usage{
vec_names2(
  x,
  ...,
  repair = c("minimal", "unique", "universal", "check_unique", "unique_quiet",
    "universal_quiet"),
  quiet = FALSE
)

vec_names(x)

vec_set_names(x, names)
}
\arguments{
\item{x}{A vector with names}

\item{...}{These dots are for future extensions and must be empty.}

\item{repair}{Either a string or a function. If a string, it must be one of
\code{"check_unique"}, \code{"minimal"}, \code{"unique"}, \code{"universal"}, \code{"unique_quiet"},
or \code{"universal_quiet"}. If a function, it is invoked with a vector of
minimal names and must return minimal names, otherwise an error is thrown.
\itemize{
\item Minimal names are never \code{NULL} or \code{NA}. When an element doesn't
have a name, its minimal name is an empty string.
\item Unique names are unique. A suffix is appended to duplicate
names to make them unique.
\item Universal names are unique and syntactic, meaning that you can
safely use the names as variables without causing a syntax
error.
}

The \code{"check_unique"} option doesn't perform any name repair.
Instead, an error is raised if the names don't suit the
\code{"unique"} criteria.

The options \code{"unique_quiet"} and \code{"universal_quiet"} are here to help the
user who calls this function indirectly, via another function which exposes
\code{repair} but not \code{quiet}. Specifying \code{repair = "unique_quiet"} is like
specifying \verb{repair = "unique", quiet = TRUE}. When the \code{"*_quiet"} options
are used, any setting of \code{quiet} is silently overridden.}

\item{quiet}{By default, the user is informed of any renaming
caused by repairing the names. This only concerns unique and
universal repairing. Set \code{quiet} to \code{TRUE} to silence the
messages.

Users can silence the name repair messages by setting the
\code{"rlib_name_repair_verbosity"} global option to \code{"quiet"}.}

\item{names}{A character vector, or \code{NULL}.}
}
\value{
\code{vec_names2()} returns the names of \code{x}, repaired.
\code{vec_names()} returns the names of \code{x} or \code{NULL} if unnamed.
\code{vec_set_names()} returns \code{x} with names updated.
}
\description{
These functions work like \code{\link[rlang:names2]{rlang::names2()}}, \code{\link[=names]{names()}} and \code{\link[=names<-]{names<-()}},
except that they return or modify the the rowwise names of the vector. These are:
\itemize{
\item The usual \code{names()} for atomic vectors and lists
\item The row names for data frames and matrices
\item The names of the first dimension for arrays
Rowwise names are size consistent: the length of the names always equals
\code{\link[=vec_size]{vec_size()}}.
}

\code{vec_names2()} returns the repaired names from a vector, even if it is unnamed.
See \code{\link[=vec_as_names]{vec_as_names()}} for details on name repair.

\code{vec_names()} is a bare-bones version that returns \code{NULL} if the vector is
unnamed.

\code{vec_set_names()} sets the names or removes them.
}
\examples{
vec_names2(1:3)
vec_names2(1:3, repair = "unique")
vec_names2(c(a = 1, b = 2))

# `vec_names()` consistently returns the rowwise names of data frames and arrays:
vec_names(data.frame(a = 1, b = 2))
names(data.frame(a = 1, b = 2))
vec_names(mtcars)
names(mtcars)
vec_names(Titanic)
names(Titanic)

vec_set_names(1:3, letters[1:3])
vec_set_names(data.frame(a = 1:3), letters[1:3])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-bare.R, R/type2.R
\name{vec_ptype2.logical}
\alias{vec_ptype2.logical}
\alias{vec_ptype2.integer}
\alias{vec_ptype2.double}
\alias{vec_ptype2.complex}
\alias{vec_ptype2.character}
\alias{vec_ptype2.raw}
\alias{vec_ptype2.list}
\alias{vec_ptype2}
\title{Find the common type for a pair of vectors}
\usage{
\method{vec_ptype2}{logical}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{integer}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{double}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{complex}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{character}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{raw}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_ptype2}{list}(x, y, ..., x_arg = "", y_arg = "")

vec_ptype2(
  x,
  y,
  ...,
  x_arg = caller_arg(x),
  y_arg = caller_arg(y),
  call = caller_env()
)
}
\arguments{
\item{x, y}{Vector types.}

\item{...}{These dots are for future extensions and must be empty.}

\item{x_arg, y_arg}{Argument names for \code{x} and \code{y}. These are used
in error messages to inform the user about the locations of
incompatible types (see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\code{vec_ptype2()} defines the coercion hierarchy for a set of related
vector types. Along with \code{\link[=vec_cast]{vec_cast()}}, this generic forms the
foundation of type coercions in vctrs.

\code{vec_ptype2()} is relevant when you are implementing vctrs methods
for your class, but it should not usually be called directly. If
you need to find the common type of a set of inputs, call
\code{\link[=vec_ptype_common]{vec_ptype_common()}} instead. This function supports multiple
inputs and \link[=vec_ptype_finalise]{finalises} the common type.
}
\section{Implementing coercion methods}{
\itemize{
\item For an overview of how these generics work and their roles in vctrs,
see \code{\link[=theory-faq-coercion]{?theory-faq-coercion}}.
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
\item For a tutorial about implementing vctrs classes from scratch, see
\code{vignette("s3-vector")}.
}
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_ptype]{vec_ptype()}} is applied to \code{x} and \code{y}
}
}

\seealso{
\code{\link[=stop_incompatible_type]{stop_incompatible_type()}} when you determine from the
attributes that an input can't be cast to the target type.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/arith.R
\name{vec_arith}
\alias{vec_arith}
\alias{vec_arith.default}
\alias{vec_arith.logical}
\alias{vec_arith.numeric}
\alias{vec_arith_base}
\alias{MISSING}
\title{Arithmetic operations}
\usage{
vec_arith(op, x, y, ...)

\method{vec_arith}{default}(op, x, y, ...)

\method{vec_arith}{logical}(op, x, y, ...)

\method{vec_arith}{numeric}(op, x, y, ...)

vec_arith_base(op, x, y)

MISSING()
}
\arguments{
\item{op}{An arithmetic operator as a string}

\item{x, y}{A pair of vectors. For \code{!}, unary \code{+} and unary \code{-}, \code{y} will be
a sentinel object of class \code{MISSING}, as created by \code{MISSING()}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
This generic provides a common double dispatch mechanism for all infix
operators (\code{+}, \code{-}, \code{/}, \code{*}, \code{^}, \code{\%\%}, \code{\%/\%}, \code{!}, \code{&}, \code{|}). It is used
to power the default arithmetic and boolean operators for \link{vctr}s objects,
overcoming the limitations of the base \link{Ops} generic.
}
\details{
\code{vec_arith_base()} is provided as a convenience for writing methods. It
recycles \code{x} and \code{y} to common length then calls the base operator with the
underlying \code{\link[=vec_data]{vec_data()}}.

\code{vec_arith()} is also used in \code{diff.vctrs_vctr()} method via \code{-}.
}
\examples{
d <- as.Date("2018-01-01")
dt <- as.POSIXct("2018-01-02 12:00")
t <- as.difftime(12, unit = "hours")

vec_arith("-", dt, 1)
vec_arith("-", dt, t)
vec_arith("-", dt, d)

vec_arith("+", dt, 86400)
vec_arith("+", dt, t)
vec_arith("+", t, t)

vec_arith("/", t, t)
vec_arith("/", t, 2)

vec_arith("*", t, 2)
}
\seealso{
\code{\link[=stop_incompatible_op]{stop_incompatible_op()}} for signalling that an arithmetic
operation is not permitted/supported.

See \code{\link[=vec_math]{vec_math()}} for the equivalent for the unary mathematical
functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-list-of.R
\name{list_of}
\alias{list_of}
\alias{as_list_of}
\alias{is_list_of}
\alias{vec_ptype2.vctrs_list_of}
\alias{vec_cast.vctrs_list_of}
\title{Construct a list of homogenous vectors}
\usage{
list_of(..., .ptype = NULL, .size = zap())

as_list_of(x, ...)

is_list_of(x)

\method{vec_ptype2}{vctrs_list_of}(x, y, ..., x_arg = "", y_arg = "")

\method{vec_cast}{vctrs_list_of}(x, to, ...)
}
\arguments{
\item{...}{For \code{list_of()}, vectors to include in the list.

For other methods, these dots must be empty.}

\item{.ptype}{The type to restrict each list element to. One of:
\itemize{
\item A prototype like \code{integer()} or \code{double()}.
\item \code{NULL}, to infer the type from \code{...}. If no vector inputs are provided,
an error is thrown.
\item \code{\link[rlang:zap]{rlang::zap()}} to avoid placing any restrictions on the type.
}}

\item{.size}{The size to restrict each list element to. One of:
\itemize{
\item A scalar integer size.
\item \code{NULL}, to infer the size from \code{...}. If no vector inputs are provided,
an error is thrown.
\item \code{\link[rlang:zap]{rlang::zap()}} to avoid placing any restrictions on the size.
}}

\item{x}{For \code{as_list_of()}, a vector to be coerced to list_of.

For \code{is_list_of()}, an object to test.}

\item{y, to}{Arguments to \code{vec_ptype2()} and \code{vec_cast()}.}

\item{x_arg, y_arg}{Argument names for \code{x} and \code{y}. These are used
in error messages to inform the user about the locations of
incompatible types (see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}
}
\description{
A \code{list_of} is a restricted version of a list that adds constraints on the
list elements.
\itemize{
\item \code{list_of(.ptype = )} restricts the \emph{type} of each element.
\itemize{
\item \verb{.ptype = <type>} asserts that each element has type \verb{<type>}.
\item \code{.ptype = NULL} infers the type from the original set of elements, or
errors if no vector inputs were provided.
\item \code{.ptype = rlang::zap()} doesn't restrict the type.
}
\item \code{list_of(.size = )} restricts the \emph{size} of each element.
\itemize{
\item \verb{.size = <size>} asserts that each element has size \verb{<size>}.
\item \code{.size = NULL} infers the size from the original set of elements, or
errors if no vector inputs were provided.
\item \code{.size = rlang::zap()} doesn't restrict the size.
}
}

The default behavior infers the element type and doesn't restrict the size.

Both \code{.ptype} and \code{.size} may be specified to restrict both the size and
type of the list elements. You cannot set both of these to \code{rlang::zap()},
as that would be the same as a bare \code{list()} with no restrictions.

Modifying a \code{list_of} with \verb{$<-}, \verb{[<-}, and \verb{[[<-} preserves the constraints
by coercing and recycling all input items.
}
\examples{
# Restrict the type, but not the size
x <- list_of(1:3, 5:6, 10:15)
x

if (requireNamespace("tibble", quietly = TRUE)) {
  # As a column in a tibble
  tibble::tibble(x = x)
}

# Coercion happens during assignment
x[1] <- list(4)
typeof(x[[1]])

try(x[1] <- list(4.5))

# Restrict the size, but not the type
x <- list_of(1, 2:3, .ptype = rlang::zap(), .size = 2)
x

# Recycling happens during assignment
x[1] <- list(4)
x

try(x[1] <- list(3:6))

# Restricting both size and type
x <- list_of(1L, 2:3, .ptype = integer(), .size = 2)
x

# Setting an element to `NULL`
x[2] <- list(NULL)
x

# Note that using `NULL` shortens the list, like a base R list
x[2] <- NULL
x

# Combining a list_of with a list results in a list
vec_c(list_of(1), list(2, "x"))

# Combining a list_of with another list_of tries to find a common element
# type and common element size, but will remove the constraint if that
# fails
x <- list_of(1, .ptype = double())
y <- list_of(c("a", "b"), .ptype = character(), .size = 2)
z <- list_of(c("c", "d", "e"), .ptype = character(), .size = 3)

# Falls back to a list
vec_c(x, y)

# Falls back to a `list_of<character>` with no size restriction
vec_c(y, z)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-data-frame.R
\name{vctrs-data-frame}
\alias{vctrs-data-frame}
\alias{vec_ptype2.data.frame}
\alias{vec_cast.data.frame}
\title{vctrs methods for data frames}
\usage{
\method{vec_ptype2}{data.frame}(x, y, ...)

\method{vec_cast}{data.frame}(x, to, ...)
}
\description{
These functions help the base data.frame class fit into the vctrs type system
by providing coercion and casting functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subscript-loc.R
\name{vec_as_location}
\alias{vec_as_location}
\alias{num_as_location}
\alias{vec_as_location2}
\alias{num_as_location2}
\title{Create a vector of locations}
\usage{
vec_as_location(
  i,
  n,
  names = NULL,
  ...,
  missing = c("propagate", "remove", "error"),
  arg = caller_arg(i),
  call = caller_env()
)

num_as_location(
  i,
  n,
  ...,
  missing = c("propagate", "remove", "error"),
  negative = c("invert", "error", "ignore"),
  oob = c("error", "remove", "extend"),
  zero = c("remove", "error", "ignore"),
  arg = caller_arg(i),
  call = caller_env()
)

vec_as_location2(
  i,
  n,
  names = NULL,
  ...,
  missing = c("error", "propagate"),
  arg = caller_arg(i),
  call = caller_env()
)

num_as_location2(
  i,
  n,
  ...,
  negative = c("error", "ignore"),
  missing = c("error", "propagate"),
  arg = caller_arg(i),
  call = caller_env()
)
}
\arguments{
\item{i}{An index vector to convert.}

\item{n}{A single integer representing the total size of the
object that \code{i} is meant to index into.}

\item{names}{If \code{i} is a character vector, \code{names} should be a character
vector that \code{i} will be matched against to construct the index. Otherwise,
not used. The default value of \code{NULL} will result in an error
if \code{i} is a character vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{missing}{How should missing \code{i} values be handled?
\itemize{
\item \code{"error"} throws an error.
\item \code{"propagate"} returns them as is.
\item \code{"remove"} removes them.
}

By default, vector subscripts propagate missing values but scalar
subscripts error on them.

Propagated missing values can't be combined with negative indices when
\code{negative = "invert"}, because they can't be meaningfully inverted.}

\item{arg}{The argument name to be displayed in error messages.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{negative}{How should negative \code{i} values be handled?
\itemize{
\item \code{"error"} throws an error.
\item \code{"ignore"} returns them as is.
\item \code{"invert"} returns the positive location generated by inverting the
negative location. When inverting, positive and negative locations
can't be mixed. This option is only applicable for \code{num_as_location()}.
}}

\item{oob}{How should out-of-bounds \code{i} values be handled?
\itemize{
\item \code{"error"} throws an error.
\item \code{"remove"} removes both positive and negative out-of-bounds locations.
\item \code{"extend"} allows positive out-of-bounds locations if they directly
follow the end of a vector. This can be used to implement extendable
vectors, like \code{letters[1:30]}.
}}

\item{zero}{How should zero \code{i} values be handled?
\itemize{
\item \code{"error"} throws an error.
\item \code{"remove"} removes them.
\item \code{"ignore"} returns them as is.
}}
}
\value{
\itemize{
\item \code{vec_as_location()} and \code{num_as_location()} return an integer vector that
can be used as an index in a subsetting operation.
\item \code{vec_as_location2()} and \code{num_as_location2()} return an integer of size 1
that can be used a scalar index for extracting an element.
}
}
\description{
These helpers provide a means of standardizing common indexing
methods such as integer, character or logical indexing.
\itemize{
\item \code{vec_as_location()} accepts integer, character, or logical vectors
of any size. The output is always an integer vector that is
suitable for subsetting with \code{[} or \code{\link[=vec_slice]{vec_slice()}}. It might be a
different size than the input because negative selections are
transformed to positive ones and logical vectors are transformed
to a vector of indices for the \code{TRUE} locations.
\item \code{vec_as_location2()} accepts a single number or string. It returns
a single location as a integer vector of size 1. This is suitable
for extracting with \code{[[}.
\item \code{num_as_location()} and \code{num_as_location2()} are specialized variants
that have extra options for numeric indices.
}
}
\examples{
x <- array(1:6, c(2, 3))
dimnames(x) <- list(c("r1", "r2"), c("c1", "c2", "c3"))

# The most common use case validates row indices
vec_as_location(1, vec_size(x))

# Negative indices can be used to index from the back
vec_as_location(-1, vec_size(x))

# Character vectors can be used if `names` are provided
vec_as_location("r2", vec_size(x), rownames(x))

# You can also construct an index for dimensions other than the first
vec_as_location(c("c2", "c1"), ncol(x), colnames(x))

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{howto-faq-fix-scalar-type-error}
\alias{howto-faq-fix-scalar-type-error}
\alias{howto_faq_fix_scalar_type_error}
\title{FAQ - Why isn't my class treated as a vector?}
\description{
The tidyverse is a bit stricter than base R regarding what kind of
objects are considered as vectors (see the \link[=faq-error-scalar-type]{user FAQ} about this topic). Sometimes vctrs won’t
treat your class as a vector when it should.
\subsection{Why isn’t my list class considered a vector?}{

By default, S3 lists are not considered to be vectors by vctrs:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{my_list <- structure(list(), class = "my_class")

vctrs::vec_is(my_list)
#> [1] FALSE
}\if{html}{\out{</div>}}

To be treated as a vector, the class must either inherit from \code{"list"}
explicitly:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{my_explicit_list <- structure(list(), class = c("my_class", "list"))
vctrs::vec_is(my_explicit_list)
#> [1] TRUE
}\if{html}{\out{</div>}}

Or it should implement a \code{vec_proxy()} method that returns its input if
explicit inheritance is not possible or troublesome:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_proxy.my_class <- function(x, ...) x

vctrs::vec_is(my_list)
#> [1] FALSE
}\if{html}{\out{</div>}}

Note that explicit inheritance is the preferred way because this makes
it possible for your class to dispatch on \code{list} methods of S3 generics:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{my_generic <- function(x) UseMethod("my_generic")
my_generic.list <- function(x) "dispatched!"

my_generic(my_list)
#> Error in `UseMethod()`:
#> ! no applicable method for 'my_generic' applied to an object of class "my_class"

my_generic(my_explicit_list)
#> [1] "dispatched!"
}\if{html}{\out{</div>}}
}

\subsection{Why isn’t my data frame class considered a vector?}{

The most likely explanation is that the data frame has not been properly
constructed.

However, if you get an “Input must be a vector” error with a data frame
subclass, it probably means that the data frame has not been properly
constructed. The main cause of these errors are data frames whose \emph{base
class} is not \code{"data.frame"}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{my_df <- data.frame(x = 1)
class(my_df) <- c("data.frame", "my_class")

vctrs::obj_check_vector(my_df)
#> Error:
#> ! `my_df` must be a vector, not a <data.frame> object.
#> x Detected incompatible data frame structure. A data frame is normally treated as a vector, but an incompatible class ordering was detected. To be compatible, the subclass <my_class> must come before <data.frame>, not after. Class: <data.frame/my_class>.
#> i If this object comes from a package, please report this error to the package author.
#> i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
}\if{html}{\out{</div>}}

This is problematic as many tidyverse functions won’t work properly:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{dplyr::slice(my_df, 1)
#> Error in `vec_slice()`:
#> ! `x` must be a vector, not a <data.frame> object.
#> x Detected incompatible data frame structure. A data frame is normally treated as a vector, but an incompatible class ordering was detected. To be compatible, the subclass <my_class> must come before <data.frame>, not after. Class: <data.frame/my_class>.
#> i If this object comes from a package, please report this error to the package author.
#> i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
}\if{html}{\out{</div>}}

It is generally not appropriate to declare your class to be a superclass
of another class. We generally consider this undefined behaviour (UB).
To fix these errors, you can simply change the construction of your data
frame class so that \code{"data.frame"} is a base class, i.e. it should come
last in the class vector:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{class(my_df) <- c("my_class", "data.frame")

vctrs::obj_check_vector(my_df)

dplyr::slice(my_df, 1)
#>   x
#> 1 1
}\if{html}{\out{</div>}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{vec_locate_matches}
\alias{vec_locate_matches}
\title{Locate observations matching specified conditions}
\usage{
vec_locate_matches(
  needles,
  haystack,
  ...,
  condition = "==",
  filter = "none",
  incomplete = "compare",
  no_match = NA_integer_,
  remaining = "drop",
  multiple = "all",
  relationship = "none",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL,
  needles_arg = "needles",
  haystack_arg = "haystack",
  error_call = current_env()
)
}
\arguments{
\item{needles, haystack}{Vectors used for matching.
\itemize{
\item \code{needles} represents the vector to search for.
\item \code{haystack} represents the vector to search in.
}

Prior to comparison, \code{needles} and \code{haystack} are coerced to the same type.}

\item{...}{These dots are for future extensions and must be empty.}

\item{condition}{Condition controlling how \code{needles} should be compared
against \code{haystack} to identify a successful match.
\itemize{
\item One of: \code{"=="}, \code{">"}, \code{">="}, \code{"<"}, or \code{"<="}.
\item For data frames, a length \code{1} or \code{ncol(needles)} character vector
containing only the above options, specifying how matching is determined
for each column.
}}

\item{filter}{Filter to be applied to the matched results.
\itemize{
\item \code{"none"} doesn't apply any filter.
\item \code{"min"} returns only the minimum haystack value matching the current
needle.
\item \code{"max"} returns only the maximum haystack value matching the current
needle.
\item For data frames, a length \code{1} or \code{ncol(needles)} character vector
containing only the above options, specifying a filter to apply to
each column.
}

Filters don't have any effect on \code{"=="} conditions, but are useful for
computing "rolling" matches with other conditions.

A filter can return multiple haystack matches for a particular needle
if the maximum or minimum haystack value is duplicated in \code{haystack}. These
can be further controlled with \code{multiple}.}

\item{incomplete}{Handling of missing and \link[=vec_detect_complete]{incomplete}
values in \code{needles}.
\itemize{
\item \code{"compare"} uses \code{condition} to determine whether or not a missing value
in \code{needles} matches a missing value in \code{haystack}. If \code{condition} is
\code{==}, \code{>=}, or \code{<=}, then missing values will match.
\item \code{"match"} always allows missing values in \code{needles} to match missing
values in \code{haystack}, regardless of the \code{condition}.
\item \code{"drop"} drops incomplete values in \code{needles} from the result.
\item \code{"error"} throws an error if any \code{needles} are incomplete.
\item If a single integer is provided, this represents the value returned
in the \code{haystack} column for values of \code{needles} that are incomplete. If
\code{no_match = NA}, setting \code{incomplete = NA} forces incomplete values in
\code{needles} to be treated like unmatched values.
}

\code{nan_distinct} determines whether a \code{NA} is allowed to match a \code{NaN}.}

\item{no_match}{Handling of \code{needles} without a match.
\itemize{
\item \code{"drop"} drops \code{needles} with zero matches from the result.
\item \code{"error"} throws an error if any \code{needles} have zero matches.
\item If a single integer is provided, this represents the value returned in
the \code{haystack} column for values of \code{needles} that have zero matches. The
default represents an unmatched needle with \code{NA}.
}}

\item{remaining}{Handling of \code{haystack} values that \code{needles} never matched.
\itemize{
\item \code{"drop"} drops remaining \code{haystack} values from the result.
Typically, this is the desired behavior if you only care when \code{needles}
has a match.
\item \code{"error"} throws an error if there are any remaining \code{haystack}
values.
\item If a single integer is provided (often \code{NA}), this represents the value
returned in the \code{needles} column for the remaining \code{haystack} values
that \code{needles} never matched. Remaining \code{haystack} values are always
returned at the end of the result.
}}

\item{multiple}{Handling of \code{needles} with multiple matches. For each needle:
\itemize{
\item \code{"all"} returns all matches detected in \code{haystack}.
\item \code{"any"} returns any match detected in \code{haystack} with no guarantees on
which match will be returned. It is often faster than \code{"first"} and
\code{"last"} if you just need to detect if there is at least one match.
\item \code{"first"} returns the first match detected in \code{haystack}.
\item \code{"last"} returns the last match detected in \code{haystack}.
}}

\item{relationship}{Handling of the expected relationship between
\code{needles} and \code{haystack}. If the expectations chosen from the list below
are invalidated, an error is thrown.
\itemize{
\item \code{"none"} doesn't perform any relationship checks.
\item \code{"one-to-one"} expects:
\itemize{
\item Each value in \code{needles} matches at most 1 value in \code{haystack}.
\item Each value in \code{haystack} matches at most 1 value in \code{needles}.
}
\item \code{"one-to-many"} expects:
\itemize{
\item Each value in \code{needles} matches any number of values in \code{haystack}.
\item Each value in \code{haystack} matches at most 1 value in \code{needles}.
}
\item \code{"many-to-one"} expects:
\itemize{
\item Each value in \code{needles} matches at most 1 value in \code{haystack}.
\item Each value in \code{haystack} matches any number of values in \code{needles}.
}
\item \code{"many-to-many"} expects:
\itemize{
\item Each value in \code{needles} matches any number of values in \code{haystack}.
\item Each value in \code{haystack} matches any number of values in \code{needles}.
}

This performs no checks, and is identical to \code{"none"}, but is provided to
allow you to be explicit about this relationship if you know it exists.
\item \code{"warn-many-to-many"} doesn't assume there is any known relationship, but
will warn if \code{needles} and \code{haystack} have a many-to-many relationship
(which is typically unexpected), encouraging you to either take a closer
look at your inputs or make this relationship explicit by specifying
\code{"many-to-many"}.
}

\code{relationship} is applied after \code{filter} and \code{multiple} to allow potential
multiple matches to be filtered out first.

\code{relationship} doesn't handle cases where there are zero matches. For that,
see \code{no_match} and \code{remaining}.}

\item{nan_distinct}{A single logical specifying whether or not \code{NaN} should
be considered distinct from \code{NA} for double and complex vectors. If \code{TRUE},
\code{NaN} will always be ordered between \code{NA} and non-missing numbers.}

\item{chr_proxy_collate}{A function generating an alternate representation
of character vectors to use for collation, often used for locale-aware
ordering.
\itemize{
\item If \code{NULL}, no transformation is done.
\item Otherwise, this must be a function of one argument. If the input contains
a character vector, it will be passed to this function after it has been
translated to UTF-8. This function should return a character vector with
the same length as the input. The result should sort as expected in the
C-locale, regardless of encoding.
}

For data frames, \code{chr_proxy_collate} will be applied to all character
columns.

Common transformation functions include: \code{tolower()} for case-insensitive
ordering and \code{stringi::stri_sort_key()} for locale-aware ordering.}

\item{needles_arg, haystack_arg}{Argument tags for \code{needles} and \code{haystack}
used in error messages.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A two column data frame containing the locations of the matches.
\itemize{
\item \code{needles} is an integer vector containing the location of
the needle currently being matched.
\item \code{haystack} is an integer vector containing the location of the
corresponding match in the haystack for the current needle.
}
}
\description{
\code{vec_locate_matches()} is a more flexible version of \code{\link[=vec_match]{vec_match()}} used to
identify locations where each value of \code{needles} matches one or multiple
values in \code{haystack}. Unlike \code{vec_match()}, \code{vec_locate_matches()} returns
all matches by default, and can match on binary conditions other than
equality, such as \code{>}, \code{>=}, \code{<}, and \code{<=}.
}
\details{
\code{\link[=vec_match]{vec_match()}} is identical to (but often slightly faster than):

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_locate_matches(
  needles,
  haystack,
  condition = "==",
  multiple = "first",
  nan_distinct = TRUE
)
}\if{html}{\out{</div>}}

\code{vec_locate_matches()} is extremely similar to a SQL join between \code{needles}
and \code{haystack}, with the default being most similar to a left join.

Be very careful when specifying match \code{condition}s. If a condition is
misspecified, it is very easy to accidentally generate an exponentially
large number of matches.
}
\section{Dependencies of \code{vec_locate_matches()}}{

\itemize{
\item \code{\link[=vec_order_radix]{vec_order_radix()}}
\item \code{\link[=vec_detect_complete]{vec_detect_complete()}}
}
}

\examples{
x <- c(1, 2, NA, 3, NaN)
y <- c(2, 1, 4, NA, 1, 2, NaN)

# By default, for each value of `x`, all matching locations in `y` are
# returned
matches <- vec_locate_matches(x, y)
matches

# The result can be used to slice the inputs to align them
data_frame(
  x = vec_slice(x, matches$needles),
  y = vec_slice(y, matches$haystack)
)

# If multiple matches are present, control which is returned with `multiple`
vec_locate_matches(x, y, multiple = "first")
vec_locate_matches(x, y, multiple = "last")
vec_locate_matches(x, y, multiple = "any")

# Use `relationship` to add constraints and error on multiple matches if
# they aren't expected
try(vec_locate_matches(x, y, relationship = "one-to-one"))

# In this case, the `NA` in `y` matches two rows in `x`
try(vec_locate_matches(x, y, relationship = "one-to-many"))

# By default, `NA` is treated as being identical to `NaN`.
# Using `nan_distinct = TRUE` treats `NA` and `NaN` as different values, so
# `NA` can only match `NA`, and `NaN` can only match `NaN`.
vec_locate_matches(x, y, nan_distinct = TRUE)

# If you never want missing values to match, set `incomplete = NA` to return
# `NA` in the `haystack` column anytime there was an incomplete value
# in `needles`.
vec_locate_matches(x, y, incomplete = NA)

# Using `incomplete = NA` allows us to enforce the one-to-many relationship
# that we couldn't before
vec_locate_matches(x, y, relationship = "one-to-many", incomplete = NA)

# `no_match` allows you to specify the returned value for a needle with
# zero matches. Note that this is different from an incomplete value,
# so specifying `no_match` allows you to differentiate between incomplete
# values and unmatched values.
vec_locate_matches(x, y, incomplete = NA, no_match = 0L)

# If you want to require that every `needle` has at least 1 match, set
# `no_match` to `"error"`:
try(vec_locate_matches(x, y, incomplete = NA, no_match = "error"))

# By default, `vec_locate_matches()` detects equality between `needles` and
# `haystack`. Using `condition`, you can detect where an inequality holds
# true instead. For example, to find every location where `x[[i]] >= y`:
matches <- vec_locate_matches(x, y, condition = ">=")

data_frame(
  x = vec_slice(x, matches$needles),
  y = vec_slice(y, matches$haystack)
)

# You can limit which matches are returned with a `filter`. For example,
# with the above example you can filter the matches returned by `x[[i]] >= y`
# down to only the ones containing the maximum `y` value of those matches.
matches <- vec_locate_matches(x, y, condition = ">=", filter = "max")

# Here, the matches for the `3` needle value have been filtered down to
# only include the maximum haystack value of those matches, `2`. This is
# often referred to as a rolling join.
data_frame(
  x = vec_slice(x, matches$needles),
  y = vec_slice(y, matches$haystack)
)

# In the very rare case that you need to generate locations for a
# cross match, where every value of `x` is forced to match every
# value of `y` regardless of what the actual values are, you can
# replace `x` and `y` with integer vectors of the same size that contain
# a single value and match on those instead.
x_proxy <- vec_rep(1L, vec_size(x))
y_proxy <- vec_rep(1L, vec_size(y))
nrow(vec_locate_matches(x_proxy, y_proxy))
vec_size(x) * vec_size(y)

# By default, missing values will match other missing values when using
# `==`, `>=`, or `<=` conditions, but not when using `>` or `<` conditions.
# This is similar to how `vec_compare(x, y, na_equal = TRUE)` works.
x <- c(1, NA)
y <- c(NA, 2)

vec_locate_matches(x, y, condition = "<=")
vec_locate_matches(x, y, condition = "<")

# You can force missing values to match regardless of the `condition`
# by using `incomplete = "match"`
vec_locate_matches(x, y, condition = "<", incomplete = "match")

# You can also use data frames for `needles` and `haystack`. The
# `condition` will be recycled to the number of columns in `needles`, or
# you can specify varying conditions per column. In this example, we take
# a vector of date `values` and find all locations where each value is
# between lower and upper bounds specified by the `haystack`.
values <- as.Date("2019-01-01") + 0:9
needles <- data_frame(lower = values, upper = values)

set.seed(123)
lower <- as.Date("2019-01-01") + sample(10, 10, replace = TRUE)
upper <- lower + sample(3, 10, replace = TRUE)
haystack <- data_frame(lower = lower, upper = upper)

# (values >= lower) & (values <= upper)
matches <- vec_locate_matches(needles, haystack, condition = c(">=", "<="))

data_frame(
  lower = vec_slice(lower, matches$haystack),
  value = vec_slice(values, matches$needle),
  upper = vec_slice(upper, matches$haystack)
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_empty}
\alias{vec_empty}
\title{Is a vector empty}
\usage{
vec_empty(x)
}
\arguments{
\item{x}{An object.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#defunct}{\figure{lifecycle-defunct.svg}{options: alt='[Defunct]'}}}{\strong{[Defunct]}}

This function is defunct, please use \code{\link[=vec_is_empty]{vec_is_empty()}}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq.R
\name{faq-compatibility-types}
\alias{faq-compatibility-types}
\title{FAQ - How is the compatibility of vector types decided?}
\description{
Two vectors are \strong{compatible} when you can safely:
\itemize{
\item Combine them into one larger vector.
\item Assign values from one of the vectors into the other vector.
}

Examples of compatible types are integer and double vectors. On the
other hand, integer and character vectors are not compatible.
}
\section{Common type of multiple vectors}{
There are two possible outcomes when multiple vectors of different types
are combined into a larger vector:
\itemize{
\item An incompatible type error is thrown because some of the types are not
compatible:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{df1 <- data.frame(x = 1:3)
df2 <- data.frame(x = "foo")
dplyr::bind_rows(df1, df2)
#> Error in `dplyr::bind_rows()`:
#> ! Can't combine `..1$x` <integer> and `..2$x` <character>.
}\if{html}{\out{</div>}}
\item The vectors are combined into a vector that has the common type of all
inputs. In this example, the common type of integer and logical is
integer:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{df1 <- data.frame(x = 1:3)
df2 <- data.frame(x = FALSE)
dplyr::bind_rows(df1, df2)
#>   x
#> 1 1
#> 2 2
#> 3 3
#> 4 0
}\if{html}{\out{</div>}}
}

In general, the common type is the \emph{richer} type, in other words the
type that can represent the most values. Logical vectors are at the
bottom of the hierarchy of numeric types because they can only represent
two values (not counting missing values). Then come integer vectors, and
then doubles. Here is the vctrs type hierarchy for the fundamental
vectors:

\figure{coerce.png}
}

\section{Type conversion and lossy cast errors}{
Type compatibility does not necessarily mean that you can \strong{convert}
one type to the other type. That’s because one of the types might
support a larger set of possible values. For instance, integer and
double vectors are compatible, but double vectors can’t be converted to
integer if they contain fractional values.

When vctrs can’t convert a vector because the target type is not as rich
as the source type, it throws a lossy cast error. Assigning a fractional
number to an integer vector is a typical example of a lossy cast error:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{int_vector <- 1:3
vec_assign(int_vector, 2, 0.001)
#> Error in `vec_assign()`:
#> ! Can't convert from <double> to <integer> due to loss of precision.
#> * Locations: 1
}\if{html}{\out{</div>}}
}

\section{How to make two vector classes compatible?}{
If you encounter two vector types that you think should be compatible,
they might need to implement coercion methods. Reach out to the
author(s) of the classes and ask them if it makes sense for their
classes to be compatible.

These developer FAQ items provide guides for implementing coercion
methods:
\itemize{
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq.R
\name{faq-error-scalar-type}
\alias{faq-error-scalar-type}
\alias{faq_error_scalar_type}
\title{FAQ - Error: Input must be a vector}
\description{
This error occurs when a function expects a vector and gets a scalar
object instead. This commonly happens when some code attempts to assign
a scalar object as column in a data frame:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{fn <- function() NULL
tibble::tibble(x = fn)
#> Error in `tibble::tibble()`:
#> ! All columns in a tibble must be vectors.
#> x Column `x` is a function.

fit <- lm(1:3 ~ 1)
tibble::tibble(x = fit)
#> Error in `tibble::tibble()`:
#> ! All columns in a tibble must be vectors.
#> x Column `x` is a `lm` object.
}\if{html}{\out{</div>}}
}
\section{Vectorness in base R and in the tidyverse}{
In base R, almost everything is a vector or behaves like a vector. In
the tidyverse we have chosen to be a bit stricter about what is
considered a vector. The main question we ask ourselves to decide on the
vectorness of a type is whether it makes sense to include that object as
a column in a data frame.

The main difference is that S3 lists are considered vectors by base R
but in the tidyverse that’s not the case by default:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{fit <- lm(1:3 ~ 1)

typeof(fit)
#> [1] "list"
class(fit)
#> [1] "lm"

# S3 lists can be subset like a vector using base R:
fit[c(1, 4)]
#> $coefficients
#> (Intercept) 
#>           2 
#> 
#> $rank
#> [1] 1

# But not in vctrs
vctrs::vec_slice(fit, c(1, 4))
#> Error in `vctrs::vec_slice()`:
#> ! `x` must be a vector, not a <lm> object.
#> x Detected incompatible scalar S3 list. To be treated as a vector, the object must explicitly inherit from <list> or should implement a `vec_proxy()` method. Class: <lm>.
#> i If this object comes from a package, please report this error to the package author.
#> i Read our FAQ about creating vector types (`?vctrs::howto_faq_fix_scalar_type_error`) to learn more.
}\if{html}{\out{</div>}}

Defused function calls are another (more esoteric) example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{call <- quote(foo(bar = TRUE, baz = FALSE))
call
#> foo(bar = TRUE, baz = FALSE)

# They can be subset like a vector using base R:
call[1:2]
#> foo(bar = TRUE)
lapply(call, function(x) x)
#> [[1]]
#> foo
#> 
#> $bar
#> [1] TRUE
#> 
#> $baz
#> [1] FALSE

# But not with vctrs:
vctrs::vec_slice(call, 1:2)
#> Error in `vctrs::vec_slice()`:
#> ! `x` must be a vector, not a call.
#> i Read our FAQ about scalar types (`?vctrs::faq_error_scalar_type`) to learn more.
}\if{html}{\out{</div>}}
}

\section{I get a scalar type error but I think this is a bug}{
It’s possible the author of the class needs to do some work to declare
their class a vector. Consider reaching out to the author. We have
written a \link[=howto-faq-fix-scalar-type-error]{developer FAQ page} to
help them fix the issue.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{vec_assert}
\alias{vec_assert}
\alias{vec_is}
\title{Assert an argument has known prototype and/or size}
\usage{
vec_assert(
  x,
  ptype = NULL,
  size = NULL,
  arg = caller_arg(x),
  call = caller_env()
)

vec_is(x, ptype = NULL, size = NULL)
}
\arguments{
\item{x}{A vector argument to check.}

\item{ptype}{Prototype to compare against. If the prototype has a
class, its \code{\link[=vec_ptype]{vec_ptype()}} is compared to that of \code{x} with
\code{identical()}. Otherwise, its \code{\link[=typeof]{typeof()}} is compared to that of
\code{x} with \code{==}.}

\item{size}{A single integer size against which to compare.}

\item{arg}{Name of argument being checked. This is used in error
messages. The label of the expression passed as \code{x} is taken as
default.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
\code{vec_is()} returns \code{TRUE} or \code{FALSE}. \code{vec_assert()} either
throws a typed error (see section on error types) or returns \code{x},
invisibly.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#questioning}{\figure{lifecycle-questioning.svg}{options: alt='[Questioning]'}}}{\strong{[Questioning]}}
\itemize{
\item \code{vec_is()} is a predicate that checks if its input is a vector that
conforms to a prototype and/or a size.
\item \code{vec_assert()} throws an error when the input is not a vector or
doesn't conform.
}
}
\section{Error types}{


\code{vec_is()} never throws.
\code{vec_assert()} throws the following errors:
\itemize{
\item If the input is not a vector, an error of class
\code{"vctrs_error_scalar_type"} is raised.
\item If the prototype doesn't match, an error of class
\code{"vctrs_error_assert_ptype"} is raised.
\item If the size doesn't match, an error of class
\code{"vctrs_error_assert_size"} is raised.
}

Both errors inherit from \code{"vctrs_error_assert"}.
}

\section{Lifecycle}{


Both \code{vec_is()} and \code{vec_assert()} are questioning because their \code{ptype}
arguments have semantics that are challenging to define clearly and are
rarely useful.
\itemize{
\item Use \code{\link[=obj_is_vector]{obj_is_vector()}} or \code{\link[=obj_check_vector]{obj_check_vector()}} for vector checks
\item Use \code{\link[=vec_check_size]{vec_check_size()}} for size checks
\item Use \code{\link[=vec_cast]{vec_cast()}}, \code{\link[=inherits]{inherits()}}, or simple type predicates like
\code{\link[rlang:type-predicates]{rlang::is_logical()}} for specific type checks
}
}

\section{Vectors and scalars}{


Informally, a vector is a collection that makes sense to use as column in a
data frame. The following rules define whether or not \code{x} is considered a
vector.

If no \code{\link[=vec_proxy]{vec_proxy()}} method has been registered, \code{x} is a vector if:
\itemize{
\item The \link[=typeof]{base type} of the object is atomic: \code{"logical"}, \code{"integer"},
\code{"double"}, \code{"complex"}, \code{"character"}, or \code{"raw"}.
\item \code{x} is a list, as defined by \code{\link[=obj_is_list]{obj_is_list()}}.
\item \code{x} is a \link{data.frame}.
}

If a \code{vec_proxy()} method has been registered, \code{x} is a vector if:
\itemize{
\item The proxy satisfies one of the above conditions.
\item The base type of the proxy is \code{"list"}, regardless of its class. S3 lists
are thus treated as scalars unless they implement a \code{vec_proxy()} method.
}

Otherwise an object is treated as scalar and cannot be used as a vector.
In particular:
\itemize{
\item \code{NULL} is not a vector.
\item S3 lists like \code{lm} objects are treated as scalars by default.
\item Objects of type \link{expression} are not treated as vectors.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-rcrd.R
\name{new_rcrd}
\alias{new_rcrd}
\alias{ses}
\alias{rcrd}
\title{rcrd (record) S3 class}
\usage{
new_rcrd(fields, ..., class = character())
}
\arguments{
\item{fields}{A list or a data frame. Lists must be rectangular
(same sizes), and contain uniquely named vectors (at least
one). \code{fields} is validated with \code{\link[=df_list]{df_list()}} to ensure uniquely
named vectors.}

\item{...}{Additional attributes}

\item{class}{Name of subclass.}
}
\description{
The rcrd class extends \link{vctr}. A rcrd is composed of 1 or more \link{field}s,
which must be vectors of the same length. Is designed specifically for
classes that can naturally be decomposed into multiple vectors of the same
length, like \link{POSIXlt}, but where the organisation should be considered
an implementation detail invisible to the user (unlike a \link{data.frame}).
}
\details{
Record-style objects created with \code{\link[=new_rcrd]{new_rcrd()}} do not do much on their own.
For instance they do not have a default \code{\link[=format]{format()}} method, which means
printing the object causes an error. See \href{https://vctrs.r-lib.org/articles/s3-vector.html#record-style-objects}{Record-style objects}
for details on implementing methods for record vectors.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictionary.R
\name{vec_match}
\alias{vec_match}
\alias{vec_in}
\title{Find matching observations across vectors}
\usage{
vec_match(
  needles,
  haystack,
  ...,
  na_equal = TRUE,
  needles_arg = "",
  haystack_arg = ""
)

vec_in(
  needles,
  haystack,
  ...,
  na_equal = TRUE,
  needles_arg = "",
  haystack_arg = ""
)
}
\arguments{
\item{needles, haystack}{Vector of \code{needles} to search for in vector haystack.
\code{haystack} should usually be unique; if not \code{vec_match()} will only
return the location of the first match.

\code{needles} and \code{haystack} are coerced to the same type prior to
comparison.}

\item{...}{These dots are for future extensions and must be empty.}

\item{na_equal}{If \code{TRUE}, missing values in \code{needles} can be
matched to missing values in \code{haystack}. If \code{FALSE}, they
propagate, missing values in \code{needles} are represented as \code{NA} in
the return value.}

\item{needles_arg, haystack_arg}{Argument tags for \code{needles} and
\code{haystack} used in error messages.}
}
\value{
A vector the same length as \code{needles}. \code{vec_in()} returns a
logical vector; \code{vec_match()} returns an integer vector.
}
\description{
\code{vec_in()} returns a logical vector based on whether \code{needle} is found in
haystack. \code{vec_match()} returns an integer vector giving location of
\code{needle} in \code{haystack}, or \code{NA} if it's not found.
}
\details{
\code{vec_in()} is equivalent to \link[base:match]{base::\%in\%}; \code{vec_match()} is equivalent to
\code{\link[base:match]{base::match()}}.
}
\section{Missing values}{

In most cases places in R, missing values are not considered to be equal,
i.e. \code{NA == NA} is not \code{TRUE}. The exception is in matching functions
like \code{\link[base:match]{base::match()}} and \code{\link[=merge]{merge()}}, where an \code{NA} will match another \code{NA}.
By \code{vec_match()} and \code{vec_in()} will match \code{NA}s; but you can control
this behaviour with the \code{na_equal} argument.
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_cast_common]{vec_cast_common()}} with fallback
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\examples{
hadley <- strsplit("hadley", "")[[1]]
vec_match(hadley, letters)

vowels <- c("a", "e", "i", "o", "u")
vec_match(hadley, vowels)
vec_in(hadley, vowels)

# Only the first index of duplicates is returned
vec_match(c("a", "b"), c("a", "b", "a", "b"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-asis.R
\name{as-is}
\alias{as-is}
\alias{vec_ptype2.AsIs}
\title{AsIs S3 class}
\usage{
\method{vec_ptype2}{AsIs}(x, y, ..., x_arg = "", y_arg = "")
}
\description{
These functions help the base AsIs class fit into the vctrs type system
by providing coercion and casting functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_is_list}
\alias{vec_is_list}
\alias{vec_check_list}
\title{List checks}
\usage{
vec_is_list(x)

vec_check_list(x, ..., arg = caller_arg(x), call = caller_env())
}
\arguments{
\item{x}{For \verb{vec_*()} functions, an object. For \verb{list_*()} functions, a
list.}

\item{...}{These dots are for future extensions and must be empty.}

\item{arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

These functions have been deprecated as of vctrs 0.6.0.
\itemize{
\item \code{vec_is_list()} has been renamed to \code{\link[=obj_is_list]{obj_is_list()}}.
\item \code{vec_check_list()} has been renamed to \code{\link[=obj_check_list]{obj_check_list()}}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{reference-faq-compatibility}
\alias{reference-faq-compatibility}
\title{FAQ - Is my class compatible with vctrs?}
\description{
vctrs provides a framework for working with vector classes in a generic
way. However, it implements several compatibility fallbacks to base R
methods. In this reference you will find how vctrs tries to be
compatible with your vector class, and what base methods you need to
implement for compatibility.

If you’re starting from scratch, we think you’ll find it easier to start
using \code{\link[=new_vctr]{new_vctr()}} as documented in
\code{vignette("s3-vector")}. This guide is aimed for developers with
existing vector classes.
\subsection{Aggregate operations with fallbacks}{

All vctrs operations are based on four primitive generics described in
the next section. However there are many higher level operations. The
most important ones implement fallbacks to base generics for maximum
compatibility with existing classes.
\itemize{
\item \code{\link[=vec_slice]{vec_slice()}} falls back to the base \code{[} generic if no
\code{\link[=vec_proxy]{vec_proxy()}} method is implemented. This way foreign
classes that do not implement \code{\link[=vec_restore]{vec_restore()}} can
restore attributes based on the new subsetted contents.
\item \code{\link[=vec_c]{vec_c()}} and \code{\link[=vec_rbind]{vec_rbind()}} now fall back to
\code{\link[base:c]{base::c()}} if the inputs have a common parent class with
a \code{c()} method (only if they have no self-to-self \code{vec_ptype2()}
method).

vctrs works hard to make your \code{c()} method success in various
situations (with \code{NULL} and \code{NA} inputs, even as first input which
would normally prevent dispatch to your method). The main downside
compared to using vctrs primitives is that you can’t combine vectors
of different classes since there is no extensible mechanism of
coercion in \code{c()}, and it is less efficient in some cases.
}
}

\subsection{The vctrs primitives}{

Most functions in vctrs are aggregate operations: they call other vctrs
functions which themselves call other vctrs functions. The dependencies
of a vctrs functions are listed in the Dependencies section of its
documentation page. Take a look at \code{\link[=vec_count]{vec_count()}} for an
example.

These dependencies form a tree whose leaves are the four vctrs
primitives. Here is the diagram for \code{vec_count()}:

\figure{vec-count-deps.png}
\subsection{The coercion generics}{

The coercion mechanism in vctrs is based on two generics:
\itemize{
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
}

See the \link[=theory-faq-coercion]{theory overview}.

Two objects with the same class and the same attributes are always
considered compatible by ptype2 and cast. If the attributes or classes
differ, they throw an incompatible type error.

Coercion errors are the main source of incompatibility with vctrs. See
the \link[=howto-faq-coercion]{howto guide} if you need to implement methods
for these generics.
}

\subsection{The proxy and restoration generics}{
\itemize{
\item \code{\link[=vec_proxy]{vec_proxy()}}
\item \code{\link[=vec_restore]{vec_restore()}}
}

These generics are essential for vctrs but mostly optional.
\code{vec_proxy()} defaults to an \link{identity} function and you
normally don’t need to implement it. The proxy a vector must be one of
the atomic vector types, a list, or a data frame. By default, S3 lists
that do not inherit from \code{"list"} do not have an identity proxy. In that
case, you need to explicitly implement \code{vec_proxy()} or make your class
inherit from list.
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order.R
\name{order-radix}
\alias{order-radix}
\alias{vec_order_radix}
\alias{vec_sort_radix}
\title{Order and sort vectors}
\usage{
vec_order_radix(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
)

vec_sort_radix(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
)
}
\arguments{
\item{x}{A vector}

\item{...}{These dots are for future extensions and must be empty.}

\item{direction}{Direction to sort in.
\itemize{
\item A single \code{"asc"} or \code{"desc"} for ascending or descending order
respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"asc"} or \code{"desc"}, specifying the direction for each column.
}}

\item{na_value}{Ordering of missing values.
\itemize{
\item A single \code{"largest"} or \code{"smallest"} for ordering missing values as the
largest or smallest values respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"largest"} or \code{"smallest"}, specifying how missing values should
be ordered within each column.
}}

\item{nan_distinct}{A single logical specifying whether or not \code{NaN} should
be considered distinct from \code{NA} for double and complex vectors. If \code{TRUE},
\code{NaN} will always be ordered between \code{NA} and non-missing numbers.}

\item{chr_proxy_collate}{A function generating an alternate representation
of character vectors to use for collation, often used for locale-aware
ordering.
\itemize{
\item If \code{NULL}, no transformation is done.
\item Otherwise, this must be a function of one argument. If the input contains
a character vector, it will be passed to this function after it has been
translated to UTF-8. This function should return a character vector with
the same length as the input. The result should sort as expected in the
C-locale, regardless of encoding.
}

For data frames, \code{chr_proxy_collate} will be applied to all character
columns.

Common transformation functions include: \code{tolower()} for case-insensitive
ordering and \code{stringi::stri_sort_key()} for locale-aware ordering.}
}
\value{
\itemize{
\item \code{vec_order_radix()} an integer vector the same size as \code{x}.
\item \code{vec_sort_radix()} a vector with the same size and type as \code{x}.
}
}
\description{
\code{vec_order_radix()} computes the order of \code{x}. For data frames, the order is
computed along the rows by computing the order of the first column and
using subsequent columns to break ties.

\code{vec_sort_radix()} sorts \code{x}. It is equivalent to \code{vec_slice(x, vec_order_radix(x))}.
}
\section{Differences with \code{order()}}{


Unlike the \code{na.last} argument of \code{order()} which decides the positions of
missing values irrespective of the \code{decreasing} argument, the \code{na_value}
argument of \code{vec_order_radix()} interacts with \code{direction}. If missing values
are considered the largest value, they will appear last in ascending order,
and first in descending order.

Character vectors are ordered in the C-locale. This is different from
\code{base::order()}, which respects \code{base::Sys.setlocale()}. Sorting in a
consistent locale can produce more reproducible results between different
sessions and platforms, however, the results of sorting in the C-locale
can be surprising. For example, capital letters sort before lower case
letters. Sorting \code{c("b", "C", "a")} with \code{vec_sort_radix()} will return
\code{c("C", "a", "b")}, but with \code{base::order()} will return \code{c("a", "b", "C")}
unless \code{base::order(method = "radix")} is explicitly set, which also uses
the C-locale. While sorting with the C-locale can be useful for
algorithmic efficiency, in many real world uses it can be the cause of
data analysis mistakes. To balance these trade-offs, you can supply a
\code{chr_proxy_collate} function to transform character vectors into an
alternative representation that orders in the C-locale in a less surprising
way. For example, providing \code{\link[base:chartr]{base::tolower()}} as a transform will order the
original vector in a case-insensitive manner. Locale-aware ordering can be
achieved by providing \code{stringi::stri_sort_key()} as a transform, setting the
collation options as appropriate for your locale.

Character vectors are always translated to UTF-8 before ordering, and before
any transform is applied by \code{chr_proxy_collate}.

For complex vectors, if either the real or imaginary component is \code{NA} or
\code{NaN}, then the entire observation is considered missing.
}

\section{Dependencies of \code{vec_order_radix()}}{

\itemize{
\item \code{\link[=vec_proxy_order]{vec_proxy_order()}}
}
}

\section{Dependencies of \code{vec_sort_radix()}}{

\itemize{
\item \code{\link[=vec_order_radix]{vec_order_radix()}}
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
if (FALSE) {

x <- round(sample(runif(5), 9, replace = TRUE), 3)
x <- c(x, NA)

vec_order_radix(x)
vec_sort_radix(x)
vec_sort_radix(x, direction = "desc")

# Can also handle data frames
df <- data.frame(g = sample(2, 10, replace = TRUE), x = x)
vec_order_radix(df)
vec_sort_radix(df)
vec_sort_radix(df, direction = "desc")

# For data frames, `direction` and `na_value` are allowed to be vectors
# with length equal to the number of columns in the data frame
vec_sort_radix(
  df,
  direction = c("desc", "asc"),
  na_value = c("largest", "smallest")
)

# Character vectors are ordered in the C locale, which orders capital letters
# below lowercase ones
y <- c("B", "A", "a")
vec_sort_radix(y)

# To order in a case-insensitive manner, provide a `chr_proxy_collate`
# function that transforms the strings to all lowercase
vec_sort_radix(y, chr_proxy_collate = tolower)

}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/match.R
\name{internal-faq-matches-algorithm}
\alias{internal-faq-matches-algorithm}
\title{Internal FAQ - Implementation of \code{vec_locate_matches()}}
\description{
\code{vec_locate_matches()} is similar to \code{vec_match()}, but detects \emph{all} matches by default, and can match on conditions other than equality (like \code{>=} and \code{<}). There are also various other arguments to limit or adjust exactly which kinds of matches are returned. Here is an example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{x <- c("a", "b", "a", "c", "d")
y <- c("d", "b", "a", "d", "a", "e")

# For each value of `x`, find all matches in `y`
# - The "c" in `x` doesn't have a match, so it gets an NA location by default
# - The "e" in `y` isn't matched by anything in `x`, so it is dropped by default
vec_locate_matches(x, y)
#>   needles haystack
#> 1       1        3
#> 2       1        5
#> 3       2        2
#> 4       3        3
#> 5       3        5
#> 6       4       NA
#> 7       5        1
#> 8       5        4
}\if{html}{\out{</div>}}
}
\section{Algorithm description}{
\subsection{Overview and \code{==}}{

The simplest (approximate) way to think about the algorithm that \code{df_locate_matches_recurse()} uses is that it sorts both inputs, and then starts at the midpoint in \code{needles} and uses a binary search to find each needle in \code{haystack}. Since there might be multiple of the same needle, we find the location of the lower and upper duplicate of that needle to handle all duplicates of that needle at once. Similarly, if there are duplicates of a matching \code{haystack} value, we find the lower and upper duplicates of the match.

If the condition is \code{==}, that is pretty much all we have to do. For each needle, we then record 3 things: the location of the needle, the location of the lower match in the haystack, and the match size (i.e. \code{loc_upper_match - loc_lower_match + 1}). This later gets expanded in \code{expand_compact_indices()} into the actual output.

After recording the matches for a single needle, we perform the same procedure on the LHS and RHS of that needle (remember we started on the midpoint needle). i.e. from \verb{[1, loc_needle-1]} and \verb{[loc_needle+1, size_needles]}, again taking the midpoint of those two ranges, finding their respective needle in the haystack, recording matches, and continuing on to the next needle. This iteration proceeds until we run out of needles.

When we have a data frame with multiple columns, we add a layer of recursion to this. For the first column, we find the locations of the lower/upper duplicate of the current needle, and we find the locations of the lower/upper matches in the haystack. If we are on the final column in the data frame, we record the matches, otherwise we pass this information on to another call to \code{df_locate_matches_recurse()}, bumping the column index and using these refined lower/upper bounds as the starting bounds for the next column.

I think an example would be useful here, so below I step through this process for a few iterations:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# these are sorted already for simplicity
needles <- data_frame(x = c(1, 1, 2, 2, 2, 3), y = c(1, 2, 3, 4, 5, 3))
haystack <- data_frame(x = c(1, 1, 2, 2, 3), y = c(2, 3, 4, 4, 1))

needles
#>   x y
#> 1 1 1
#> 2 1 2
#> 3 2 3
#> 4 2 4
#> 5 2 5
#> 6 3 3

haystack
#>   x y
#> 1 1 2
#> 2 1 3
#> 3 2 4
#> 4 2 4
#> 5 3 1

## Column 1, iteration 1

# start at midpoint in needles
# this corresponds to x==2
loc_mid_needles <- 3L

# finding all x==2 values in needles gives us:
loc_lower_duplicate_needles <- 3L
loc_upper_duplicate_needles <- 5L

# finding matches in haystack give us:
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 4L

# compute LHS/RHS bounds for next needle
lhs_loc_lower_bound_needles <- 1L # original lower bound
lhs_loc_upper_bound_needles <- 2L # lower_duplicate-1

rhs_loc_lower_bound_needles <- 6L # upper_duplicate+1
rhs_loc_upper_bound_needles <- 6L # original upper bound

# We still have a 2nd column to check. So recurse and pass on the current
# duplicate and match bounds to start the 2nd column with.

## Column 2, iteration 1

# midpoint of [3, 5]
# value y==4
loc_mid_needles <- 4L

loc_lower_duplicate_needles <- 4L
loc_upper_duplicate_needles <- 4L

loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 4L

# last column, so record matches
# - this was location 4 in needles
# - lower match in haystack is at loc 3
# - match size is 2

# Now handle LHS and RHS of needle midpoint
lhs_loc_lower_bound_needles <- 3L # original lower bound
lhs_loc_upper_bound_needles <- 3L # lower_duplicate-1

rhs_loc_lower_bound_needles <- 5L # upper_duplicate+1
rhs_loc_upper_bound_needles <- 5L # original upper bound

## Column 2, iteration 2 (using LHS bounds)

# midpoint of [3,3]
# value of y==3
loc_mid_needles <- 3L

loc_lower_duplicate_needles <- 3L
loc_upper_duplicate_needles <- 3L

# no match! no y==3 in haystack for x==2
# lower-match will always end up > upper-match in this case
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 2L

# no LHS or RHS needle values to do, so we are done here

## Column 2, iteration 3 (using RHS bounds)

# same as above, range of [5,5], value of y==5, which has no match in haystack

## Column 1, iteration 2 (LHS of first x needle)

# Now we are done with the x needles from [3,5], so move on to the LHS and RHS
# of that. Here we would do the LHS:

# midpoint of [1,2]
loc_mid_needles <- 1L

# ...

## Column 1, iteration 3 (RHS of first x needle)

# midpoint of [6,6]
loc_mid_needles <- 6L

# ...
}\if{html}{\out{</div>}}

In the real code, rather than comparing the double values of the columns directly, we replace each column with pseudo "joint ranks" computed between the i-th column of \code{needles} and the i-th column of \code{haystack}. It is approximately like doing \code{vec_rank(vec_c(needles$x, haystack$x), type = "dense")}, then splitting the resulting ranks back up into their corresponding needle/haystack columns. This keeps the recursion code simpler, because we only have to worry about comparing integers.
}

\subsection{Non-equi conditions and containers}{

At this point we can talk about non-equi conditions like \code{<} or \code{>=}. The general idea is pretty simple, and just builds on the above algorithm. For example, start with the \code{x} column from needles/haystack above:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{needles$x
#> [1] 1 1 2 2 2 3

haystack$x
#> [1] 1 1 2 2 3
}\if{html}{\out{</div>}}

If we used a condition of \code{<=}, then we'd do everything the same as before:
\itemize{
\item Midpoint in needles is location 3, value \code{x==2}
\item Find lower/upper duplicates in needles, giving locations \verb{[3, 5]}
\item Find lower/upper \emph{exact} match in haystack, giving locations \verb{[3, 4]}
}

At this point, we need to "adjust" the \code{haystack} match bounds to account for the condition. Since \code{haystack} is ordered, our "rule" for \code{<=} is to keep the lower match location the same, but extend the upper match location to the upper bound, so we end up with \verb{[3, 5]}. We know we can extend the upper match location because every haystack value after the exact match should be less than the needle. Then we just record the matches and continue on normally.

This approach is really nice, because we only have to exactly match the \code{needle} in \code{haystack}. We don't have to compare each needle against every value in \code{haystack}, which would take a massive amount of time.

However, it gets slightly more complex with data frames with multiple columns. Let's go back to our original \code{needles} and \code{haystack} data frames and apply the condition \code{<=} to each column. Here is another worked example, which shows a case where our "rule" falls apart on the second column.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{needles
#>   x y
#> 1 1 1
#> 2 1 2
#> 3 2 3
#> 4 2 4
#> 5 2 5
#> 6 3 3

haystack
#>   x y
#> 1 1 2
#> 2 1 3
#> 3 2 4
#> 4 2 4
#> 5 3 1

# `condition = c("<=", "<=")`

## Column 1, iteration 1

# x == 2
loc_mid_needles <- 3L

loc_lower_duplicate_needles <- 3L
loc_upper_duplicate_needles <- 5L

# finding exact matches in haystack give us:
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 4L

# because haystack is ordered we know we can expand the upper bound automatically
# to include everything past the match. i.e. needle of x==2 must be less than
# the haystack value at loc 5, which we can check by seeing that it is x==3.
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 5L

## Column 2, iteration 1

# needles range of [3, 5]
# y == 4
loc_mid_needles <- 4L

loc_lower_duplicate_needles <- 4L
loc_upper_duplicate_needles <- 4L

# finding exact matches in haystack give us:
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 4L

# lets try using our rule, which tells us we should be able to extend the upper
# bound:
loc_lower_match_haystack <- 3L
loc_upper_match_haystack <- 5L

# but the haystack value of y at location 5 is y==1, which is not less than y==4
# in the needles! looks like our rule failed us.
}\if{html}{\out{</div>}}

If you read through the above example, you'll see that the rule didn't work here. The problem is that while \code{haystack} is ordered (by \code{vec_order()}s standards), each column isn't ordered \emph{independently} of the others. Instead, each column is ordered within the "group" created by previous columns. Concretely, \code{haystack} here has an ordered \code{x} column, but if you look at \code{haystack$y} by itself, it isn't ordered (because of that 1 at the end). That is what causes the rule to fail.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{haystack
#>   x y
#> 1 1 2
#> 2 1 3
#> 3 2 4
#> 4 2 4
#> 5 3 1
}\if{html}{\out{</div>}}

To fix this, we need to create haystack "containers" where the values within each container are all \emph{totally} ordered. For \code{haystack} that would create 2 containers and look like:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{haystack[1:4,]
#> # A tibble: 4 × 2
#>       x     y
#>   <dbl> <dbl>
#> 1     1     2
#> 2     1     3
#> 3     2     4
#> 4     2     4

haystack[5,]
#> # A tibble: 1 × 2
#>       x     y
#>   <dbl> <dbl>
#> 1     3     1
}\if{html}{\out{</div>}}

This is essentially what \code{computing_nesting_container_ids()} does. You can actually see these ids with the helper, \code{compute_nesting_container_info()}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{haystack2 <- haystack

# we really pass along the integer ranks, but in this case that is equivalent
# to converting our double columns to integers
haystack2$x <- as.integer(haystack2$x)
haystack2$y <- as.integer(haystack2$y)

info <- compute_nesting_container_info(haystack2, condition = c("<=", "<="))

# the ids are in the second slot.
# container ids break haystack into [1, 4] and [5, 5].
info[[2]]
#> [1] 0 0 0 0 1
}\if{html}{\out{</div>}}

So the idea is that for each needle, we look in each haystack container and find all the matches, then we aggregate all of the matches once at the end. \code{df_locate_matches_with_containers()} has the job of iterating over the containers.

Computing totally ordered containers can be expensive, but luckily it doesn't happen very often in normal usage.
\itemize{
\item If there are all \code{==} conditions, we don't need containers (i.e. any equi join)
\item If there is only 1 non-equi condition and no conditions after it, we don't need containers (i.e. most rolling joins)
\item Otherwise the typical case where we need containers is if we have something like \verb{date >= lower, date <= upper}. Even so, the computation cost generally scales with the number of columns in \code{haystack} you compute containers with (here 2), and it only really slows down around 4 columns or so, which I haven't ever seen a real life example of.
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-data-frame.R
\name{df_list}
\alias{df_list}
\title{Collect columns for data frame construction}
\usage{
df_list(
  ...,
  .size = NULL,
  .unpack = TRUE,
  .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet",
    "universal_quiet"),
  .error_call = current_env()
)
}
\arguments{
\item{...}{Vectors of equal-length. When inputs are named, those names
are used for names of the resulting list.}

\item{.size}{The common size of vectors supplied in \code{...}. If \code{NULL}, this
will be computed as the common size of the inputs.}

\item{.unpack}{Should unnamed data frame inputs be unpacked? Defaults to
\code{TRUE}.}

\item{.name_repair}{One of \code{"check_unique"}, \code{"unique"}, \code{"universal"},
\code{"minimal"}, \code{"unique_quiet"}, or \code{"universal_quiet"}. See \code{\link[=vec_as_names]{vec_as_names()}}
for the meaning of these options.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\code{df_list()} constructs the data structure underlying a data
frame, a named list of equal-length vectors. It is often used in
combination with \code{\link[=new_data_frame]{new_data_frame()}} to safely and consistently create
a helper function for data frame subclasses.
}
\section{Properties}{

\itemize{
\item Inputs are \link[=theory-faq-recycling]{recycled} to a common size with
\code{\link[=vec_recycle_common]{vec_recycle_common()}}.
\item With the exception of data frames, inputs are not modified in any way.
Character vectors are never converted to factors, and lists are stored
as-is for easy creation of list-columns.
\item Unnamed data frame inputs are automatically unpacked. Named data frame
inputs are stored unmodified as data frame columns.
\item \code{NULL} inputs are completely ignored.
\item The dots are dynamic, allowing for splicing of lists with \verb{!!!} and
unquoting.
}
}

\examples{
# `new_data_frame()` can be used to create custom data frame constructors
new_fancy_df <- function(x = list(), n = NULL, ..., class = NULL) {
  new_data_frame(x, n = n, ..., class = c(class, "fancy_df"))
}

# Combine this constructor with `df_list()` to create a safe,
# consistent helper function for your data frame subclass
fancy_df <- function(...) {
  data <- df_list(...)
  new_fancy_df(data)
}

df <- fancy_df(x = 1)
class(df)
}
\seealso{
\code{\link[=new_data_frame]{new_data_frame()}} for constructing data frame subclasses from a validated
input. \code{\link[=data_frame]{data_frame()}} for a fast data frame creation helper.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-date-time.R
\name{new_date}
\alias{new_date}
\alias{new_datetime}
\alias{new_duration}
\alias{vec_ptype2.Date}
\alias{vec_ptype2.POSIXct}
\alias{vec_ptype2.POSIXlt}
\alias{vec_ptype2.difftime}
\alias{vec_cast.Date}
\alias{vec_cast.POSIXct}
\alias{vec_cast.POSIXlt}
\alias{vec_cast.difftime}
\alias{vec_arith.Date}
\alias{vec_arith.POSIXct}
\alias{vec_arith.POSIXlt}
\alias{vec_arith.difftime}
\title{Date, date-time, and duration S3 classes}
\usage{
new_date(x = double())

new_datetime(x = double(), tzone = "")

new_duration(x = double(), units = c("secs", "mins", "hours", "days", "weeks"))

\method{vec_ptype2}{Date}(x, y, ...)

\method{vec_ptype2}{POSIXct}(x, y, ...)

\method{vec_ptype2}{POSIXlt}(x, y, ...)

\method{vec_ptype2}{difftime}(x, y, ...)

\method{vec_cast}{Date}(x, to, ...)

\method{vec_cast}{POSIXct}(x, to, ...)

\method{vec_cast}{POSIXlt}(x, to, ...)

\method{vec_cast}{difftime}(x, to, ...)

\method{vec_arith}{Date}(op, x, y, ...)

\method{vec_arith}{POSIXct}(op, x, y, ...)

\method{vec_arith}{POSIXlt}(op, x, y, ...)

\method{vec_arith}{difftime}(op, x, y, ...)
}
\arguments{
\item{x}{A double vector representing the number of days since UNIX
epoch for \code{new_date()}, number of seconds since UNIX epoch for
\code{new_datetime()}, and number of \code{units} for \code{new_duration()}.}

\item{tzone}{Time zone. A character vector of length 1. Either \code{""} for
the local time zone, or a value from \code{\link[=OlsonNames]{OlsonNames()}}}

\item{units}{Units of duration.}
}
\description{
\itemize{
\item A \code{date} (\link{Date}) is a double vector. Its value represent the number
of days since the Unix "epoch", 1970-01-01. It has no attributes.
\item A \code{datetime} (\link{POSIXct} is a double vector. Its value represents the
number of seconds since the Unix "Epoch", 1970-01-01. It has a single
attribute: the timezone (\code{tzone}))
\item A \code{duration} (\link{difftime})
}
}
\details{
These function help the base \code{Date}, \code{POSIXct}, and \code{difftime} classes fit
into the vctrs type system by providing constructors, coercion functions,
and casting functions.
}
\examples{
new_date(0)
new_datetime(0, tzone = "UTC")
new_duration(1, "hours")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{vec_proxy}
\alias{vec_proxy}
\alias{vec_restore}
\title{Proxy and restore}
\usage{
vec_proxy(x, ...)

vec_restore(x, to, ...)
}
\arguments{
\item{x}{A vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{to}{The original vector to restore to.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{vec_proxy()} returns the data structure containing the values of a
vector. This data structure is usually the vector itself. In this
case the proxy is the \link[base:identity]{identity function}, which is
the default \code{vec_proxy()} method.

Only experts should implement special \code{vec_proxy()} methods, for
these cases:
\itemize{
\item A vector has vectorised attributes, i.e. metadata for
each element of the vector. These \emph{record types} are implemented
in vctrs by returning a data frame in the proxy method. If you're
starting your class from scratch, consider deriving from the
\code{\link[=new_rcrd]{rcrd}} class. It implements the appropriate data
frame proxy and is generally the preferred way to create a record
class.
\item When you're implementing a vector on top of a non-vector type,
like an environment or an S4 object. This is currently only
partially supported.
\item S3 lists are considered scalars by default. This is the safe
choice for list objects such as returned by \code{stats::lm()}. To
declare that your S3 list class is a vector, you normally add
\code{"list"} to the right of your class vector. Explicit inheritance
from list is generally the preferred way to declare an S3 list in
R, for instance it makes it possible to dispatch on
\code{generic.list} S3 methods.

If you can't modify your class vector, you can implement an
identity proxy (i.e. a proxy method that just returns its input)
to let vctrs know this is a vector list and not a scalar.
}

\code{vec_restore()} is the inverse operation of \code{vec_proxy()}. It
should only be called on vector proxies.
\itemize{
\item It undoes the transformations of \code{vec_proxy()}.
\item It restores attributes and classes. These may be lost when the
memory values are manipulated. For example slicing a subset of a
vector's proxy causes a new proxy to be allocated.
}

By default vctrs restores all attributes and classes
automatically. You only need to implement a \code{vec_restore()} method
if your class has attributes that depend on the data.
}
\section{Proxying}{


You should only implement \code{vec_proxy()} when your type is designed
around a non-vector class. I.e. anything that is not either:
\itemize{
\item An atomic vector
\item A bare list
\item A data frame
}

In this case, implement \code{vec_proxy()} to return such a vector
class. The vctrs operations such as \code{\link[=vec_slice]{vec_slice()}} are applied on
the proxy and \code{vec_restore()} is called to restore the original
representation of your type.

The most common case where you need to implement \code{vec_proxy()} is
for S3 lists. In vctrs, S3 lists are treated as scalars by
default. This way we don't treat objects like model fits as
vectors. To prevent vctrs from treating your S3 list as a scalar,
unclass it in the \code{vec_proxy()} method. For instance, here is the
definition for \code{list_of}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_proxy.vctrs_list_of <- function(x) \{
  unclass(x)
\}
}\if{html}{\out{</div>}}

Another case where you need to implement a proxy is \link[=new_rcrd]{record types}. Record types should return a data frame, as in
the \code{POSIXlt} method:

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_proxy.POSIXlt <- function(x) \{
  new_data_frame(unclass(x))
\}
}\if{html}{\out{</div>}}

Note that you don't need to implement \code{vec_proxy()} when your class
inherits from \code{vctrs_vctr} or \code{vctrs_rcrd}.
}

\section{Restoring}{


A restore is a specialised type of cast, primarily used in
conjunction with \code{NextMethod()} or a C-level function that works on
the underlying data structure. A \code{vec_restore()} method can make
the following assumptions about \code{x}:
\itemize{
\item It has the correct type.
\item It has the correct names.
\item It has the correct \code{dim} and \code{dimnames} attributes.
\item It is unclassed. This way you can call vctrs generics with \code{x}
without triggering an infinite loop of restoration.
}

The length may be different (for example after \code{\link[=vec_slice]{vec_slice()}} has
been called), and all other attributes may have been lost. The
method should restore all attributes so that after restoration,
\code{vec_restore(vec_data(x), x)} yields \code{x}.

To understand the difference between \code{vec_cast()} and \code{vec_restore()}
think about factors: it doesn't make sense to cast an integer to a factor,
but if \code{NextMethod()} or another low-level function has stripped attributes,
you still need to be able to restore them.

The default method copies across all attributes so you only need to
provide your own method if your attributes require special care
(i.e. they are dependent on the data in some way). When implementing
your own method, bear in mind that many R users add attributes to track
additional metadata that is important to them, so you should preserve any
attributes that don't require special handling for your class.
}

\section{Dependencies}{

\itemize{
\item \code{x} must be a vector in the vctrs sense (see \code{\link[=vec_is]{vec_is()}})
\item By default the underlying data is returned as is (identity proxy)
}

All vector classes have a proxy, even those who don't implement any
vctrs methods. The exception is S3 lists that don't inherit from
\code{"list"} explicitly. These might have to implement an identity
proxy for compatibility with vctrs (see discussion above).
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slice.R
\name{vec_slice}
\alias{vec_slice}
\alias{vec_slice<-}
\alias{vec_assign}
\title{Get or set observations in a vector}
\usage{
vec_slice(x, i, ..., error_call = current_env())

vec_slice(x, i) <- value

vec_assign(x, i, value, ..., slice_value = FALSE, x_arg = "", value_arg = "")
}
\arguments{
\item{x}{A vector}

\item{i}{An integer, character or logical vector specifying the locations or
names of the observations to get/set. Specify \code{TRUE} to index all elements
(as in \code{x[]}), or \code{NULL}, \code{FALSE} or \code{integer()} to index none (as in
\code{x[NULL]}).}

\item{...}{These dots are for future extensions and must be empty.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{value}{A vector of replacement values

\code{value} is cast to the type of \code{x}.

If \code{slice_value = FALSE}, \code{value} must be size 1 or the same size as \code{i}
after \code{i} has been converted to a positive integer location vector with
\code{\link[=vec_as_location]{vec_as_location()}} (which may not be the same size as \code{i} originally).

If \code{slice_value = TRUE}, \code{value} must be size 1 or the same size as \code{x}.}

\item{slice_value}{A boolean. If \code{TRUE}, the assignment proceeds as if you
had provided \code{vec_slice(x, i) <- vec_slice(value, i)}, but is optimized to
avoid materializing the slice of \code{value}.}

\item{x_arg, value_arg}{Argument names for \code{x} and \code{value}. These are used
in error messages to inform the user about the locations of
incompatible types and sizes (see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}} and
\code{\link[=stop_incompatible_size]{stop_incompatible_size()}}).}
}
\value{
A vector of the same type as \code{x}.
}
\description{
This provides a common interface to extracting and modifying observations
for all vector types, regardless of dimensionality. They are analogs to \code{[}
and \verb{[<-} that match \code{\link[=vec_size]{vec_size()}} instead of \code{length()}.
}
\section{Genericity}{


Support for S3 objects depends on whether the object implements a
\code{\link[=vec_proxy]{vec_proxy()}} method.
\itemize{
\item When a \code{vec_proxy()} method exists, the proxy is sliced or assigned to and
\code{vec_restore()} is called on the result.
\item Otherwise, \code{vec_slice()} falls back to the base generic \code{[} and
\verb{vec_slice<-()} falls back to the base generic \verb{[<-}.
}

When \verb{vec_slice<-()} falls back to \verb{[<-}, it is expected that the subclass's
\verb{[<-} method can handle the following subset of cases that base R's \verb{[<-}
can also handle:
\itemize{
\item An \code{i} vector of positive integer positions (notably excluding \code{NA}).
\item A \code{value} vector of length 1 or length \code{length(i)}. If length 1, it
should be recycled by the \verb{[<-} method to the length of \code{i}.
}

If your \verb{[<-} method eventually calls base R's native \verb{[<-} code, then these
cases will be handled for you.

Note that S3 lists are treated as scalars by default, and will
cause an error if they don't implement a \code{\link[=vec_proxy]{vec_proxy()}} method.
}

\section{Differences with base R subsetting}{

\itemize{
\item \code{vec_slice()} only slices along one dimension. For
two-dimensional types, the first dimension is subsetted.
\item \code{vec_slice()} preserves attributes by default.
\item \verb{vec_slice<-()} is type-stable and always returns the same type
as the LHS.
}
}

\section{Dependencies}{

\subsection{vctrs dependencies}{
\itemize{
\item \code{\link[=vec_proxy]{vec_proxy()}}
\item \code{\link[=vec_restore]{vec_restore()}}
}
}

\subsection{base dependencies}{
\itemize{
\item \code{base::`[`}
\item \code{base::`[<-`}
}
}
}

\examples{
x <- sample(10)
x
vec_slice(x, 1:3)

# You can assign with the infix variant:
vec_slice(x, 2) <- 100
x

# Or with the regular variant that doesn't modify the original input:
y <- vec_assign(x, 3, 500)
y
x


# Slicing objects of higher dimension:
vec_slice(mtcars, 1:3)

# Type stability --------------------------------------------------

# The assign variant is type stable. It always returns the same
# type as the input.
x <- 1:5
vec_slice(x, 2) <- 20.0

# `x` is still an integer vector because the RHS was cast to the
# type of the LHS:
vec_ptype(x)

# Compare to `[<-`:
x[2] <- 20.0
vec_ptype(x)


# Note that the types must be coercible for the cast to happen.
# For instance, you can cast a double vector of whole numbers to an
# integer vector:
vec_cast(1, integer())

# But not fractional doubles:
try(vec_cast(1.5, integer()))

# For this reason you can't assign fractional values in an integer
# vector:
x <- 1:3
try(vec_slice(x, 2) <- 1.5)

# Slicing `value` -------------------------------------------------

# Sometimes both `x` and `value` start from objects that are the same length,
# and you need to slice `value` by `i` before assigning it to `x`. This comes
# up when thinking about how `base::ifelse()` and `dplyr::case_when()` work.
condition <- c(TRUE, FALSE, TRUE, FALSE)
yes <- 1:4
no <- 5:8

# Create an output container and fill it
out <- vec_init(integer(), 4)
out <- vec_assign(out, condition, vec_slice(yes, condition))
out <- vec_assign(out, !condition, vec_slice(no, !condition))
out

# This is wasteful because you have to materialize the slices of `yes` and
# `no` before they can be assigned, and you also have to validate `condition`
# multiple times. Using `slice_value` internally performs
# `vec_slice(yes, condition)` and `vec_slice(no, !condition)` for you,
# but does so in a way that avoids the materialization.
out <- vec_init(integer(), 4)
out <- vec_assign(out, condition, yes, slice_value = TRUE)
out <- vec_assign(out, !condition, no, slice_value = TRUE)
out
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print-str.R
\name{obj_print}
\alias{obj_print}
\alias{obj_print_header}
\alias{obj_print_data}
\alias{obj_print_footer}
\alias{obj_str}
\alias{obj_str_header}
\alias{obj_str_data}
\alias{obj_str_footer}
\title{\code{print()} and \code{str()} generics.}
\usage{
obj_print(x, ...)

obj_print_header(x, ...)

obj_print_data(x, ...)

obj_print_footer(x, ...)

obj_str(x, ...)

obj_str_header(x, ...)

obj_str_data(x, ...)

obj_str_footer(x, ...)
}
\arguments{
\item{x}{A vector}

\item{...}{Additional arguments passed on to methods. See \code{\link[=print]{print()}} and
\code{\link[=str]{str()}} for commonly used options}
}
\description{
These are constructed to be more easily extensible since you can override
the \verb{_header()}, \verb{_data()} or \verb{_footer()} components individually. The
default methods are built on top of \code{format()}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/complete.R
\name{vec_detect_complete}
\alias{vec_detect_complete}
\title{Complete}
\usage{
vec_detect_complete(x)
}
\arguments{
\item{x}{A vector}
}
\value{
A logical vector with the same size as \code{x}.
}
\description{
\code{vec_detect_complete()} detects "complete" observations. An observation is
considered complete if it is non-missing. For most vectors, this implies that
\code{vec_detect_complete(x) == !vec_detect_missing(x)}.

For data frames and matrices, a row is only considered complete if all
elements of that row are non-missing. To compare, \code{!vec_detect_missing(x)}
detects rows that are partially complete (they have at least one non-missing
value).
}
\details{
A \link[=new_rcrd]{record} type vector is similar to a data frame, and is only
considered complete if all fields are non-missing.
}
\examples{
x <- c(1, 2, NA, 4, NA)

# For most vectors, this is identical to `!vec_detect_missing(x)`
vec_detect_complete(x)
!vec_detect_missing(x)

df <- data_frame(
  x = x,
  y = c("a", "b", NA, "d", "e")
)

# This returns `TRUE` where all elements of the row are non-missing.
# Compare that with `!vec_detect_missing()`, which detects rows that have at
# least one non-missing value.
df2 <- df
df2$all_non_missing <- vec_detect_complete(df)
df2$any_non_missing <- !vec_detect_missing(df)
df2
}
\seealso{
\code{\link[stats:complete.cases]{stats::complete.cases()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-list-of.R
\name{new_list_of}
\alias{new_list_of}
\title{Create list_of subclass}
\usage{
new_list_of(
  x = list(),
  ptype = logical(),
  size = NULL,
  ...,
  class = character()
)
}
\arguments{
\item{x}{A list}

\item{ptype}{The prototype which every element of \code{x} belongs to. If \code{NULL},
the prototype is not specified.}

\item{size}{The size which every element of \code{x} has. If \code{NULL}, the size is
not specified.}

\item{...}{Additional attributes used by subclass}

\item{class}{Optional subclass name}
}
\description{
Create list_of subclass
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/names.R
\name{vec_as_names}
\alias{vec_as_names}
\title{Retrieve and repair names}
\usage{
vec_as_names(
  names,
  ...,
  repair = c("minimal", "unique", "universal", "check_unique", "unique_quiet",
    "universal_quiet"),
  repair_arg = NULL,
  quiet = FALSE,
  call = caller_env()
)
}
\arguments{
\item{names}{A character vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{repair}{Either a string or a function. If a string, it must be one of
\code{"check_unique"}, \code{"minimal"}, \code{"unique"}, \code{"universal"}, \code{"unique_quiet"},
or \code{"universal_quiet"}. If a function, it is invoked with a vector of
minimal names and must return minimal names, otherwise an error is thrown.
\itemize{
\item Minimal names are never \code{NULL} or \code{NA}. When an element doesn't
have a name, its minimal name is an empty string.
\item Unique names are unique. A suffix is appended to duplicate
names to make them unique.
\item Universal names are unique and syntactic, meaning that you can
safely use the names as variables without causing a syntax
error.
}

The \code{"check_unique"} option doesn't perform any name repair.
Instead, an error is raised if the names don't suit the
\code{"unique"} criteria.

The options \code{"unique_quiet"} and \code{"universal_quiet"} are here to help the
user who calls this function indirectly, via another function which exposes
\code{repair} but not \code{quiet}. Specifying \code{repair = "unique_quiet"} is like
specifying \verb{repair = "unique", quiet = TRUE}. When the \code{"*_quiet"} options
are used, any setting of \code{quiet} is silently overridden.}

\item{repair_arg}{If specified and \code{repair = "check_unique"}, any errors
will include a hint to set the \code{repair_arg}.}

\item{quiet}{By default, the user is informed of any renaming
caused by repairing the names. This only concerns unique and
universal repairing. Set \code{quiet} to \code{TRUE} to silence the
messages.

Users can silence the name repair messages by setting the
\code{"rlib_name_repair_verbosity"} global option to \code{"quiet"}.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\code{vec_as_names()} takes a character vector of names and repairs it
according to the \code{repair} argument. It is the r-lib and tidyverse
equivalent of \code{\link[base:make.names]{base::make.names()}}.

vctrs deals with a few levels of name repair:
\itemize{
\item \code{minimal} names exist. The \code{names} attribute is not \code{NULL}. The
name of an unnamed element is \code{""} and never \code{NA}. For instance,
\code{vec_as_names()} always returns minimal names and data frames
created by the tibble package have names that are, at least,
\code{minimal}.
\item \code{unique} names are \code{minimal}, have no duplicates, and can be used
where a variable name is expected. Empty names, \code{...}, and
\code{..} followed by a sequence of digits are banned.
\itemize{
\item All columns can be accessed by name via \code{df[["name"]]} and
\code{df$`name` } and \code{with(df, `name`)}.
}
\item \code{universal} names are \code{unique} and syntactic (see Details for
more).
\itemize{
\item Names work everywhere, without quoting: \code{df$name} and \code{with(df, name)} and \code{lm(name1 ~ name2, data = df)} and
\code{dplyr::select(df, name)} all work.
}
}

\code{universal} implies \code{unique}, \code{unique} implies \code{minimal}. These
levels are nested.
}
\section{\code{minimal} names}{


\code{minimal} names exist. The \code{names} attribute is not \code{NULL}. The
name of an unnamed element is \code{""} and never \code{NA}.

Examples:

\if{html}{\out{<div class="sourceCode">}}\preformatted{Original names of a vector with length 3: NULL
                           minimal names: "" "" ""

                          Original names: "x" NA
                           minimal names: "x" ""
}\if{html}{\out{</div>}}
}

\section{\code{unique} names}{


\code{unique} names are \code{minimal}, have no duplicates, and can be used
(possibly with backticks) in contexts where a variable is
expected. Empty names, \code{...}, and \code{..} followed by a sequence of
digits are banned. If a data frame has \code{unique} names, you can
index it by name, and also access the columns by name. In
particular, \code{df[["name"]]} and \code{df$`name`} and also \code{with(df, `name`)} always work.

There are many ways to make names \code{unique}. We append a suffix of the form
\code{...j} to any name that is \code{""} or a duplicate, where \code{j} is the position.
We also change \code{..#} and \code{...} to \code{...#}.

Example:

\if{html}{\out{<div class="sourceCode">}}\preformatted{Original names:     ""     "x"     "" "y"     "x"  "..2"  "..."
  unique names: "...1" "x...2" "...3" "y" "x...5" "...6" "...7"
}\if{html}{\out{</div>}}

Pre-existing suffixes of the form \code{...j} are always stripped, prior
to making names \code{unique}, i.e. reconstructing the suffixes. If this
interacts poorly with your names, you should take control of name
repair.
}

\section{\code{universal} names}{


\code{universal} names are \code{unique} and syntactic, meaning they:
\itemize{
\item Are never empty (inherited from \code{unique}).
\item Have no duplicates (inherited from \code{unique}).
\item Are not \code{...}. Do not have the form \code{..i}, where \code{i} is a
number (inherited from \code{unique}).
\item Consist of letters, numbers, and the dot \code{.} or underscore \verb{_}
characters.
\item Start with a letter or start with the dot \code{.} not followed by a
number.
\item Are not a \link{reserved} word, e.g., \code{if} or \code{function} or \code{TRUE}.
}

If a vector has \code{universal} names, variable names can be used
"as is" in code. They work well with nonstandard evaluation, e.g.,
\code{df$name} works.

vctrs has a different method of making names syntactic than
\code{\link[base:make.names]{base::make.names()}}. In general, vctrs prepends one or more dots
\code{.} until the name is syntactic.

Examples:

\if{html}{\out{<div class="sourceCode">}}\preformatted{ Original names:     ""     "x"    NA      "x"
universal names: "...1" "x...2" "...3" "x...4"

  Original names: "(y)"  "_z"  ".2fa"  "FALSE"
 universal names: ".y." "._z" "..2fa" ".FALSE"
}\if{html}{\out{</div>}}
}

\examples{
# By default, `vec_as_names()` returns minimal names:
vec_as_names(c(NA, NA, "foo"))

# You can make them unique:
vec_as_names(c(NA, NA, "foo"), repair = "unique")

# Universal repairing fixes any non-syntactic name:
vec_as_names(c("_foo", "+"), repair = "universal")
}
\seealso{
\code{\link[rlang:names2]{rlang::names2()}} returns the names of an object, after
making them \code{minimal}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recycle.R
\name{vec_recycle}
\alias{vec_recycle}
\alias{vec_recycle_common}
\title{Vector recycling}
\usage{
vec_recycle(x, size, ..., x_arg = "", call = caller_env())

vec_recycle_common(..., .size = NULL, .arg = "", .call = caller_env())
}
\arguments{
\item{x}{A vector to recycle.}

\item{size}{Desired output size.}

\item{...}{Depending on the function used:
\itemize{
\item For \code{vec_recycle_common()}, vectors to recycle.
\item For \code{vec_recycle()}, these dots should be empty.
}}

\item{x_arg}{Argument name for \code{x}. These are used in error
messages to inform the user about which argument has an
incompatible size.}

\item{call, .call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{.size}{Desired output size. If omitted,
will use the common size from \code{\link[=vec_size_common]{vec_size_common()}}.}

\item{.arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}
}
\description{
\code{vec_recycle(x, size)} recycles a single vector to a given size.
\code{vec_recycle_common(...)} recycles multiple vectors to their common size. All
functions obey the \link[=theory-faq-recycling]{vctrs recycling rules}, and will
throw an error if recycling is not possible. See \code{\link[=vec_size]{vec_size()}} for the precise
definition of size.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
# Inputs with 1 observation are recycled
vec_recycle_common(1:5, 5)
vec_recycle_common(integer(), 5)
\dontrun{
vec_recycle_common(1:5, 1:2)
}

# Data frames and matrices are recycled along their rows
vec_recycle_common(data.frame(x = 1), 1:5)
vec_recycle_common(array(1:2, c(1, 2)), 1:5)
vec_recycle_common(array(1:3, c(1, 3, 1)), 1:5)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cast.R, R/type2.R
\name{vec_default_cast}
\alias{vec_default_cast}
\alias{vec_default_ptype2}
\title{Default cast and ptype2 methods}
\usage{
vec_default_cast(x, to, ..., x_arg = "", to_arg = "", call = caller_env())

vec_default_ptype2(x, y, ..., x_arg = "", y_arg = "", call = caller_env())
}
\arguments{
\item{x}{Vectors to cast.}

\item{to}{Type to cast to. If \code{NULL}, \code{x} will be returned as is.}

\item{...}{For \code{vec_cast_common()}, vectors to cast. For
\code{vec_cast()}, \code{vec_cast_default()}, and \code{vec_restore()}, these
dots are only for future extensions and should be empty.}

\item{x_arg}{Argument name for \code{x}, used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{to_arg}{Argument name \code{to} used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
These functions are automatically called when no \code{\link[=vec_ptype2]{vec_ptype2()}} or
\code{\link[=vec_cast]{vec_cast()}} method is implemented for a pair of types.
\itemize{
\item They apply special handling if one of the inputs is of type
\code{AsIs} or \code{sfc}.
\item They attempt a number of fallbacks in cases where it would be too
inconvenient to be strict:
\itemize{
\item If the class and attributes are the same they are considered
compatible. \code{vec_default_cast()} returns \code{x} in this case.
\item In case of incompatible data frame classes, they fall back to
\code{data.frame}. If an incompatible subclass of tibble is
involved, they fall back to \code{tbl_df}.
}
\item Otherwise, an error is thrown with \code{\link[=stop_incompatible_type]{stop_incompatible_type()}} or
\code{\link[=stop_incompatible_cast]{stop_incompatible_cast()}}.
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equal.R
\name{vec_proxy_equal}
\alias{vec_proxy_equal}
\title{Equality proxy}
\usage{
vec_proxy_equal(x, ...)
}
\arguments{
\item{x}{A vector x.}

\item{...}{These dots are for future extensions and must be empty.}
}
\value{
A 1d atomic vector or a data frame.
}
\description{
Returns a proxy object (i.e. an atomic vector or data frame of atomic
vectors). For \link{vctr}s, this determines the behaviour of \code{==} and
\code{!=} (via \code{\link[=vec_equal]{vec_equal()}}); \code{\link[=unique]{unique()}}, \code{\link[=duplicated]{duplicated()}} (via
\code{\link[=vec_unique]{vec_unique()}} and \code{\link[=vec_duplicate_detect]{vec_duplicate_detect()}}); \code{\link[=is.na]{is.na()}} and \code{\link[=anyNA]{anyNA()}}
(via \code{\link[=vec_detect_missing]{vec_detect_missing()}}).
}
\details{
The default method calls \code{\link[=vec_proxy]{vec_proxy()}}, as the default underlying
vector data should be equal-able in most cases. If your class is
not equal-able, provide a \code{vec_proxy_equal()} method that throws an
error.
}
\section{Data frames}{

If the proxy for \code{x} is a data frame, the proxy function is automatically
recursively applied on all columns as well. After applying the proxy
recursively, if there are any data frame columns present in the proxy, then
they are unpacked. Finally, if the resulting data frame only has a single
column, then it is unwrapped and a vector is returned as the proxy.
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy]{vec_proxy()}} called by default
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_equal_na}
\alias{vec_equal_na}
\title{Missing values}
\usage{
vec_equal_na(x)
}
\arguments{
\item{x}{A vector}
}
\value{
A logical vector the same size as \code{x}.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{vec_equal_na()} has been renamed to \code{\link[=vec_detect_missing]{vec_detect_missing()}} and is deprecated
as of vctrs 0.5.0.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-list-of.R
\name{list-of-attributes}
\alias{list-of-attributes}
\alias{list_of_ptype}
\alias{list_of_size}
\title{\code{list_of} attributes}
\usage{
list_of_ptype(x)

list_of_size(x)
}
\arguments{
\item{x}{A \link[=list_of]{list_of}.}
}
\description{
\itemize{
\item \code{list_of_ptype()} returns the \code{ptype} required by the \code{list_of}.
If no \code{ptype} is required, then \code{NULL} is returned.
\item \code{list_of_size()} returns the \code{size} required by the \code{list_of}.
If no \code{size} is required, then \code{NULL} is returned.
}
}
\examples{
x <- list_of(1, 2)
list_of_ptype(x)
list_of_size(x)

x <- list_of(.ptype = integer(), .size = 5)
list_of_ptype(x)
list_of_size(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/names.R
\name{name_spec}
\alias{name_spec}
\title{Name specifications}
\arguments{
\item{name_spec, .name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}
}
\description{
A name specification describes how to combine an inner and outer
names. This sort of name combination arises when concatenating
vectors or flattening lists. There are two possible cases:
\itemize{
\item Named vector:

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_c(outer = c(inner1 = 1, inner2 = 2))
}\if{html}{\out{</div>}}
\item Unnamed vector:

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_c(outer = 1:2)
}\if{html}{\out{</div>}}
}

In r-lib and tidyverse packages, these cases are errors by default,
because there's no behaviour that works well for every case.
Instead, you can provide a name specification that describes how to
combine the inner and outer names of inputs. Name specifications
can refer to:
\itemize{
\item \code{outer}: The external name recycled to the size of the input
vector.
\item \code{inner}: Either the names of the input vector, or a sequence of
integer from 1 to the size of the vector if it is unnamed.
}
}
\examples{
# By default, named inputs must be length 1:
vec_c(name = 1)         # ok
try(vec_c(name = 1:3))  # bad

# They also can't have internal names, even if scalar:
try(vec_c(name = c(internal = 1)))  # bad

# Pass a name specification to work around this. A specification
# can be a glue string referring to `outer` and `inner`:
vec_c(name = 1:3, other = 4:5, .name_spec = "{outer}")
vec_c(name = 1:3, other = 4:5, .name_spec = "{outer}_{inner}")

# They can also be functions:
my_spec <- function(outer, inner) paste(outer, inner, sep = "_")
vec_c(name = 1:3, other = 4:5, .name_spec = my_spec)

# Or purrr-style formulas for anonymous functions:
vec_c(name = 1:3, other = 4:5, .name_spec = ~ paste0(.x, .y))

# Or the string `"inner"` to only use inner names
vec_c(name = 1:3, outer = 4:5, .name_spec = "inner")
vec_c(name = c(a = 1, b = 2, c = 3), outer = 4:5, .name_spec = "inner")
# This can be useful when you want outer names mentioned in error messages,
# but you don't want them interfering with the result
try(vec_c(x = c(a = 1), y = c(b = "2"), .name_spec = "inner"))

# Or `rlang::zap()` to ignore both outer and inner names entirely
vec_c(name = c(a = 1, b = 2), outer = c(c = 3), .name_spec = rlang::zap())
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/register-s3.R
\name{s3_register}
\alias{s3_register}
\title{Register a method for a suggested dependency}
\arguments{
\item{generic}{Name of the generic in the form \code{pkg::generic}.}

\item{class}{Name of the class}

\item{method}{Optionally, the implementation of the method. By default,
this will be found by looking for a function called \code{generic.class}
in the package environment.

Note that providing \code{method} can be dangerous if you use
devtools. When the namespace of the method is reloaded by
\code{devtools::load_all()}, the function will keep inheriting from
the old namespace. This might cause crashes because of dangling
\code{.Call()} pointers.}
}
\description{
Generally, the recommend way to register an S3 method is to use the
\code{S3Method()} namespace directive (often generated automatically by the
\verb{@export} roxygen2 tag). However, this technique requires that the generic
be in an imported package, and sometimes you want to suggest a package,
and only provide a method when that package is loaded. \code{s3_register()}
can be called from your package's \code{.onLoad()} to dynamically register
a method only if the generic's package is loaded.
}
\details{
For R 3.5.0 and later, \code{s3_register()} is also useful when demonstrating
class creation in a vignette, since method lookup no longer always involves
the lexical scope. For R 3.6.0 and later, you can achieve a similar effect
by using "delayed method registration", i.e. placing the following in your
\code{NAMESPACE} file:

\if{html}{\out{<div class="sourceCode">}}\preformatted{if (getRversion() >= "3.6.0") \{
  S3method(package::generic, class)
\}
}\if{html}{\out{</div>}}
}
\section{Usage in other packages}{

To avoid taking a dependency on vctrs, you copy the source of
\href{https://github.com/r-lib/vctrs/blob/main/R/register-s3.R}{\code{s3_register()}}
into your own package. It is licensed under the permissive
\href{https://choosealicense.com/licenses/unlicense/}{unlicense} to make it
crystal clear that we're happy for you to do this. There's no need to include
the license or even credit us when using this function.
}

\examples{
# A typical use case is to dynamically register tibble/pillar methods
# for your class. That way you avoid creating a hard dependency on packages
# that are not essential, while still providing finer control over
# printing when they are used.

.onLoad <- function(...) {
  s3_register("pillar::pillar_shaft", "vctrs_vctr")
  s3_register("tibble::type_sum", "vctrs_vctr")
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{obj_is_list}
\alias{obj_is_list}
\alias{obj_check_list}
\alias{list_all_vectors}
\alias{list_check_all_vectors}
\alias{list_all_size}
\alias{list_check_all_size}
\alias{list_all_recyclable}
\alias{list_check_all_recyclable}
\title{List checks}
\usage{
obj_is_list(x)

obj_check_list(x, ..., arg = caller_arg(x), call = caller_env())

list_all_vectors(x, ..., allow_null = FALSE)

list_check_all_vectors(
  x,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
)

list_all_size(x, size, ..., allow_null = FALSE)

list_check_all_size(
  x,
  size,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
)

list_all_recyclable(x, size, ..., allow_null = FALSE)

list_check_all_recyclable(
  x,
  size,
  ...,
  allow_null = FALSE,
  arg = caller_arg(x),
  call = caller_env()
)
}
\arguments{
\item{x}{For \verb{vec_*()} functions, an object. For \verb{list_*()} functions, a
list.}

\item{...}{These dots are for future extensions and must be empty.}

\item{arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{allow_null}{Whether \code{NULL} elements should be skipped over
automatically or not.}

\item{size}{The size to check each element for compatibility with.}
}
\description{
\itemize{
\item \code{obj_is_list()} tests if \code{x} is considered a list in the vctrs sense. It
returns \code{TRUE} if all of the following hold:
\itemize{
\item \code{x} must have list storage, i.e. \code{typeof(x)} returns \code{"list"}
\item \code{x} must not have a \code{dim} attribute
\item \code{x} must not have a \code{class} attribute, or must explicitly inherit from
\code{"list"} as the last class
}
\item \code{list_all_vectors()} takes a list and returns \code{TRUE} if all elements of
that list are vectors.
\item \code{list_all_size()} takes a list and returns \code{TRUE} if all elements of that
list have the same \code{size}.
\item \code{list_all_recyclable()} takes a list and returns \code{TRUE} if all elements of
that list can recycle to \code{size}.
\item \code{obj_check_list()}, \code{list_check_all_vectors()}, \code{list_check_all_size()},
and \code{list_check_all_recyclable()} use the above functions, but throw a
standardized and informative error if they return \code{FALSE}.
}
}
\details{
Notably, data frames and S3 record style classes like POSIXlt are not
considered lists.
}
\examples{
obj_is_list(list())
obj_is_list(list_of(1))
obj_is_list(data.frame())

list_all_vectors(list(1, mtcars))
list_all_vectors(list(1, environment()))

list_all_size(list(1:2, 2:3), 2)
list_all_size(list(1:2, 2:4), 2)

list_all_recyclable(list(1, 2:3), 2)
list_all_recyclable(list(1, 2:4), 2)

# `list_`-prefixed functions assume a list:
try(list_all_vectors(environment()))

# `NULL` elements are not considered vectors and generally have a size of 0
try(list_check_all_vectors(list(1, NULL, 2)))
try(list_check_all_size(list(1, NULL, 2), size = 1))

# However, it is often useful to perform upfront vector/size checks on a
# list, excluding `NULL`s, and then filter them out later on
list_check_all_vectors(list(1, NULL, 2), allow_null = TRUE)
list_check_all_size(list(1, NULL, 2), size = 1, allow_null = TRUE)

# Performing the checks before removing `NULL`s from the list ensures that
# any errors report the correct index. Note how the index is incorrect from a
# user's point of view if we filter out `NULL` too soon.
xs <- list(1, NULL, 2:3)
try(list_check_all_size(xs, size = 1, allow_null = TRUE))
xs <- vec_slice(xs, !vec_detect_missing(xs))
try(list_check_all_size(xs, size = 1))
}
\seealso{
\code{\link[=list_sizes]{list_sizes()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/size.R
\name{vec_seq_along}
\alias{vec_seq_along}
\alias{vec_init_along}
\title{Useful sequences}
\usage{
vec_seq_along(x)

vec_init_along(x, y = x)
}
\arguments{
\item{x, y}{Vectors}
}
\value{
\itemize{
\item \code{vec_seq_along()} an integer vector with the same size as \code{x}.
\item \code{vec_init_along()} a vector with the same type as \code{x} and the same size
as \code{y}.
}
}
\description{
\code{vec_seq_along()} is equivalent to \code{\link[=seq_along]{seq_along()}} but uses size, not length.
\code{vec_init_along()} creates a vector of missing values with size matching
an existing object.
}
\examples{
vec_seq_along(mtcars)
vec_init_along(head(mtcars))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-internal.R
\name{internal-faq-ptype2-identity}
\alias{internal-faq-ptype2-identity}
\title{Internal FAQ - \code{vec_ptype2()}, \code{NULL}, and unspecified vectors}
\description{
\subsection{Promotion monoid}{

Promotions (i.e. automatic coercions) should always transform inputs to
their richer type to avoid losing values of precision. \code{vec_ptype2()}
returns the \emph{richer} type of two vectors, or throws an incompatible type
error if none of the two vector types include the other. For example,
the richer type of integer and double is the latter because double
covers a larger range of values than integer.

\code{vec_ptype2()} is a \href{https://en.wikipedia.org/wiki/Monoid}{monoid} over
vectors, which in practical terms means that it is a well behaved
operation for
\href{https://purrr.tidyverse.org/reference/reduce.html}{reduction}.
Reduction is an important operation for promotions because that is how
the richer type of multiple elements is computed. As a monoid,
\code{vec_ptype2()} needs an identity element, i.e. a value that doesn’t
change the result of the reduction. vctrs has two identity values,
\code{NULL} and \strong{unspecified} vectors.
}

\subsection{The \code{NULL} identity}{

As an identity element that shouldn’t influence the determination of the
common type of a set of vectors, \code{NULL} is promoted to any type:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(NULL, "")
#> character(0)
vec_ptype2(1L, NULL)
#> integer(0)
}\if{html}{\out{</div>}}

The common type of \code{NULL} and \code{NULL} is the identity \code{NULL}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(NULL, NULL)
#> NULL
}\if{html}{\out{</div>}}

This way the result of \code{vec_ptype2(NULL, NULL)} does not influence
subsequent promotions:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(
  vec_ptype2(NULL, NULL),
  ""
)
#> character(0)
}\if{html}{\out{</div>}}
}

\subsection{Unspecified vectors}{

In the vctrs coercion system, logical vectors of missing values are also
automatically promoted to the type of any other vector, just like
\code{NULL}. We call these vectors unspecified. The special coercion
semantics of unspecified vectors serve two purposes:
\enumerate{
\item It makes it possible to assign vectors of \code{NA} inside any type of
vectors, even when they are not coercible with logical:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{x <- letters[1:5]
vec_assign(x, 1:2, c(NA, NA))
#> [1] NA  NA  "c" "d" "e"
}\if{html}{\out{</div>}}
\item We can’t put \code{NULL} in a data frame, so we need an identity element
that behaves more like a vector. Logical vectors of \code{NA} seem a
natural fit for this.
}

Unspecified vectors are thus promoted to any other type, just like
\code{NULL}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(NA, "")
#> character(0)
vec_ptype2(1L, c(NA, NA))
#> integer(0)
}\if{html}{\out{</div>}}
}

\subsection{Finalising common types}{

vctrs has an internal vector type of class \code{vctrs_unspecified}. Users
normally don’t see such vectors in the wild, but they do come up when
taking the common type of an unspecified vector with another identity
value:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(NA, NA)
#> <unspecified> [0]
vec_ptype2(NA, NULL)
#> <unspecified> [0]
vec_ptype2(NULL, NA)
#> <unspecified> [0]
}\if{html}{\out{</div>}}

We can’t return \code{NA} here because \code{vec_ptype2()} normally returns empty
vectors. We also can’t return \code{NULL} because unspecified vectors need to
be recognised as logical vectors if they haven’t been promoted at the
end of the reduction.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype_finalise(vec_ptype2(NULL, NA))
#> logical(0)
}\if{html}{\out{</div>}}

See the output of \code{vec_ptype_common()} which performs the reduction and
finalises the type, ready to be used by the caller:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype_common(NULL, NULL)
#> NULL
vec_ptype_common(NA, NULL)
#> logical(0)
}\if{html}{\out{</div>}}

\code{vec_ptype_finalise()} is an S3 generic, but the only time you should
ever need to write an S3 method for it is if your class \emph{wraps} another
vector in some way and needs special handling to propagate the default
finalisation. For example, the ivs package contains an interval class
that wraps \code{start} and \code{end} vectors of the same type and has a
\code{vec_ptype_finalize()} method that finalises those wrapped vectors:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype_finalise.ivs_iv <- function(x, ...) \{
  start <- unclass(x)[[1L]]
  ptype <- vec_ptype_finalise(start, ...)
  new_bare_iv(ptype, ptype)
\}
}\if{html}{\out{</div>}}

This ensures that \code{vec_ptype_finalise(vec_ptype(ivs::iv(NA, NA)))}
correctly finalises to \verb{<iv<logical>>} rather than \verb{<iv<unspecified>>}.

Note that data frames are already recursively finalised, so you don’t
need a \code{vec_ptype_finalise()} method for a data frame subclass.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictionary.R
\name{vec_count}
\alias{vec_count}
\title{Count unique values in a vector}
\usage{
vec_count(x, sort = c("count", "key", "location", "none"))
}
\arguments{
\item{x}{A vector (including a data frame).}

\item{sort}{One of "count", "key", "location", or "none".
\itemize{
\item "count", the default, puts most frequent values at top
\item "key", orders by the output key column (i.e. unique values of \code{x})
\item "location", orders by location where key first seen. This is useful
if you want to match the counts up to other unique/duplicated functions.
\item "none", leaves unordered. This is not guaranteed to produce the same
ordering across R sessions, but is the fastest method.
}}
}
\value{
A data frame with columns \code{key} (same type as \code{x}) and
\code{count} (an integer vector).
}
\description{
Count the number of unique values in a vector. \code{vec_count()} has two
important differences to \code{table()}: it returns a data frame, and when
given multiple inputs (as a data frame), it only counts combinations that
appear in the input.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
\item \code{\link[=vec_slice]{vec_slice()}}
\item \code{\link[=vec_order]{vec_order()}}
}
}

\examples{
vec_count(mtcars$vs)
vec_count(iris$Species)

# If you count a data frame you'll get a data frame
# column in the output
str(vec_count(mtcars[c("vs", "am")]))

# Sorting ---------------------------------------

x <- letters[rpois(100, 6)]
# default is to sort by frequency
vec_count(x)

# by can sort by key
vec_count(x, sort = "key")

# or location of first value
vec_count(x, sort = "location")
head(x)

# or not at all
vec_count(x, sort = "none")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-factor.R
\name{new_factor}
\alias{new_factor}
\alias{new_ordered}
\alias{vec_ptype2.factor}
\alias{vec_ptype2.ordered}
\alias{vec_cast.factor}
\alias{vec_cast.ordered}
\title{Factor/ordered factor S3 class}
\usage{
new_factor(x = integer(), levels = character(), ..., class = character())

new_ordered(x = integer(), levels = character())

\method{vec_ptype2}{factor}(x, y, ...)

\method{vec_ptype2}{ordered}(x, y, ...)

\method{vec_cast}{factor}(x, to, ...)

\method{vec_cast}{ordered}(x, to, ...)
}
\arguments{
\item{x}{Integer values which index in to \code{levels}.}

\item{levels}{Character vector of labels.}

\item{..., class}{Used to for subclasses.}
}
\description{
A \link{factor} is an integer with attribute \code{levels}, a character vector. There
should be one level for each integer between 1 and \code{max(x)}.
An \link{ordered} factor has the same properties as a factor, but possesses
an extra class that marks levels as having a total ordering.
}
\details{
These functions help the base factor and ordered factor classes fit in to
the vctrs type system by providing constructors, coercion functions,
and casting functions. \code{new_factor()} and \code{new_ordered()} are low-level
constructors - they only check that types, but not values, are valid, so
are for expert use only.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/group.R
\name{vec_group}
\alias{vec_group}
\alias{vec_group_id}
\alias{vec_group_loc}
\alias{vec_group_rle}
\title{Identify groups}
\usage{
vec_group_id(x)

vec_group_loc(x)

vec_group_rle(x)
}
\arguments{
\item{x}{A vector}
}
\value{
\itemize{
\item \code{vec_group_id()}: An integer vector with the same size as \code{x}.
\item \code{vec_group_loc()}: A two column data frame with size equal to
\code{vec_size(vec_unique(x))}.
\itemize{
\item A \code{key} column of type \code{vec_ptype(x)}
\item A \code{loc} column of type list, with elements of type integer.
}
\item \code{vec_group_rle()}: A \code{vctrs_group_rle} rcrd object with two integer
vector fields: \code{group} and \code{length}.
}

Note that when using \code{vec_group_loc()} for complex types, the default
\code{data.frame} print method will be suboptimal, and you will want to coerce
into a tibble to better understand the output.
}
\description{
\itemize{
\item \code{vec_group_id()} returns an identifier for the group that each element of
\code{x} falls in, constructed in the order that they appear. The number of
groups is also returned as an attribute, \code{n}.
\item \code{vec_group_loc()} returns a data frame containing a \code{key} column with the
unique groups, and a \code{loc} column with the locations of each group in \code{x}.
\item \code{vec_group_rle()} locates groups in \code{x} and returns them run length
encoded in the order that they appear. The return value is a rcrd object
with fields for the \code{group} identifiers and the run \code{length} of the
corresponding group. The number of groups is also returned as an
attribute, \code{n}.
}
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\examples{
purrr <- c("p", "u", "r", "r", "r")
vec_group_id(purrr)
vec_group_rle(purrr)

groups <- mtcars[c("vs", "am")]
vec_group_id(groups)

group_rle <- vec_group_rle(groups)
group_rle

# Access fields with `field()`
field(group_rle, "group")
field(group_rle, "length")

# `vec_group_id()` is equivalent to
vec_match(groups, vec_unique(groups))

vec_group_loc(mtcars$vs)
vec_group_loc(mtcars[c("vs", "am")])

if (require("tibble")) {
  as_tibble(vec_group_loc(mtcars[c("vs", "am")]))
}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-data-frame.R
\name{data_frame}
\alias{data_frame}
\title{Construct a data frame}
\usage{
data_frame(
  ...,
  .size = NULL,
  .name_repair = c("check_unique", "unique", "universal", "minimal", "unique_quiet",
    "universal_quiet"),
  .error_call = current_env()
)
}
\arguments{
\item{...}{Vectors to become columns in the data frame. When inputs are
named, those names are used for column names.}

\item{.size}{The number of rows in the data frame. If \code{NULL}, this will
be computed as the common size of the inputs.}

\item{.name_repair}{One of \code{"check_unique"}, \code{"unique"}, \code{"universal"},
\code{"minimal"}, \code{"unique_quiet"}, or \code{"universal_quiet"}. See \code{\link[=vec_as_names]{vec_as_names()}}
for the meaning of these options.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\code{data_frame()} constructs a data frame. It is similar to
\code{\link[base:data.frame]{base::data.frame()}}, but there are a few notable differences that make it
more in line with vctrs principles. The Properties section outlines these.
}
\details{
If no column names are supplied, \code{""} will be used as a default name for all
columns. This is applied before name repair occurs, so the default name
repair of \code{"check_unique"} will error if any unnamed inputs are supplied and
\code{"unique"} (or \code{"unique_quiet"}) will repair the empty string column names
appropriately. If the column names don't matter, use a \code{"minimal"} name
repair for convenience and performance.
}
\section{Properties}{

\itemize{
\item Inputs are \link[=theory-faq-recycling]{recycled} to a common size with
\code{\link[=vec_recycle_common]{vec_recycle_common()}}.
\item With the exception of data frames, inputs are not modified in any way.
Character vectors are never converted to factors, and lists are stored
as-is for easy creation of list-columns.
\item Unnamed data frame inputs are automatically unpacked. Named data frame
inputs are stored unmodified as data frame columns.
\item \code{NULL} inputs are completely ignored.
\item The dots are dynamic, allowing for splicing of lists with \verb{!!!} and
unquoting.
}
}

\examples{
data_frame(x = 1, y = 2)

# Inputs are recycled using tidyverse recycling rules
data_frame(x = 1, y = 1:3)

# Strings are never converted to factors
class(data_frame(x = "foo")$x)

# List columns can be easily created
df <- data_frame(x = list(1:2, 2, 3:4), y = 3:1)

# However, the base print method is suboptimal for displaying them,
# so it is recommended to convert them to tibble
if (rlang::is_installed("tibble")) {
  tibble::as_tibble(df)
}

# Named data frame inputs create data frame columns
df <- data_frame(x = data_frame(y = 1:2, z = "a"))

# The `x` column itself is another data frame
df$x

# Again, it is recommended to convert these to tibbles for a better
# print method
if (rlang::is_installed("tibble")) {
  tibble::as_tibble(df)
}

# Unnamed data frame input is automatically unpacked
data_frame(x = 1, data_frame(y = 1:2, z = "a"))
}
\seealso{
\code{\link[=df_list]{df_list()}} for safely creating a data frame's underlying data structure from
individual columns. \code{\link[=new_data_frame]{new_data_frame()}} for constructing the actual data
frame from that underlying data structure. Together, these can be useful
for developers when creating new data frame subclasses supporting
standard evaluation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ptype-abbr-full.R
\name{vec_ptype_full}
\alias{vec_ptype_full}
\alias{vec_ptype_abbr}
\title{Vector type as a string}
\usage{
vec_ptype_full(x, ...)

vec_ptype_abbr(x, ..., prefix_named = FALSE, suffix_shape = TRUE)
}
\arguments{
\item{x}{A vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{prefix_named}{If \code{TRUE}, add a prefix for named vectors.}

\item{suffix_shape}{If \code{TRUE} (the default), append the shape of
the vector.}
}
\value{
A string.
}
\description{
\code{vec_ptype_full()} displays the full type of the vector. \code{vec_ptype_abbr()}
provides an abbreviated summary suitable for use in a column heading.
}
\section{S3 dispatch}{

The default method for \code{vec_ptype_full()} uses the first element of the
class vector. Override this method if your class has parameters that should
be prominently displayed.

The default method for \code{vec_ptype_abbr()} \code{\link[=abbreviate]{abbreviate()}}s \code{vec_ptype_full()}
to 8 characters. You should almost always override, aiming for 4-6
characters where possible.

These arguments are handled by the generic and not passed to methods:
\itemize{
\item \code{prefix_named}
\item \code{suffix_shape}
}
}

\examples{
cat(vec_ptype_full(1:10))
cat(vec_ptype_full(iris))

cat(vec_ptype_abbr(1:10))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/proxy.R
\name{vec_data}
\alias{vec_data}
\title{Extract underlying data}
\usage{
vec_data(x)
}
\arguments{
\item{x}{A vector or object implementing \code{vec_proxy()}.}
}
\value{
The data underlying \code{x}, free from any attributes except the names.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Extract the data underlying an S3 vector object, i.e. the underlying
(named) atomic vector, data frame, or list.
}
\section{Difference with \code{vec_proxy()}}{

\itemize{
\item \code{vec_data()} returns unstructured data. The only attributes
preserved are names, dims, and dimnames.

Currently, due to the underlying memory architecture of R, this
creates a full copy of the data for atomic vectors.
\item \code{vec_proxy()} may return structured data. This generic is the
main customisation point for accessing memory values in vctrs,
along with \code{\link[=vec_restore]{vec_restore()}}.

Methods must return a vector type. Records and data frames will
be processed rowwise.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-table.R
\name{table}
\alias{table}
\title{Table S3 class}
\description{
These functions help the base table class fit into the vctrs type system
by providing coercion and casting functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare.R
\name{vec_compare}
\alias{vec_compare}
\title{Compare two vectors}
\usage{
vec_compare(x, y, na_equal = FALSE, .ptype = NULL)
}
\arguments{
\item{x, y}{Vectors with compatible types and lengths.}

\item{na_equal}{Should \code{NA} values be considered equal?}

\item{.ptype}{Override to optionally specify common type}
}
\value{
An integer vector with values -1 for \code{x < y}, 0 if \code{x == y},
and 1 if \code{x > y}. If \code{na_equal} is \code{FALSE}, the result will be \code{NA}
if either \code{x} or \code{y} is \code{NA}.
}
\description{
Compare two vectors
}
\section{S3 dispatch}{

\code{vec_compare()} is not generic for performance; instead it uses
\code{\link[=vec_proxy_compare]{vec_proxy_compare()}} to create a proxy that is used in the comparison.
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_cast_common]{vec_cast_common()}} with fallback
\item \code{\link[=vec_recycle_common]{vec_recycle_common()}}
\item \code{\link[=vec_proxy_compare]{vec_proxy_compare()}}
}
}

\examples{
vec_compare(c(TRUE, FALSE, NA), FALSE)
vec_compare(c(TRUE, FALSE, NA), FALSE, na_equal = TRUE)

vec_compare(1:10, 5)
vec_compare(runif(10), 0.5)
vec_compare(letters[1:10], "d")

df <- data.frame(x = c(1, 1, 1, 2), y = c(0, 1, 2, 1))
vec_compare(df, data.frame(x = 1, y = 1))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set.R
\name{vec-set}
\alias{vec-set}
\alias{vec_set_intersect}
\alias{vec_set_difference}
\alias{vec_set_union}
\alias{vec_set_symmetric_difference}
\title{Set operations}
\usage{
vec_set_intersect(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
)

vec_set_difference(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
)

vec_set_union(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
)

vec_set_symmetric_difference(
  x,
  y,
  ...,
  ptype = NULL,
  x_arg = "x",
  y_arg = "y",
  error_call = current_env()
)
}
\arguments{
\item{x, y}{A pair of vectors.}

\item{...}{These dots are for future extensions and must be empty.}

\item{ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type between \code{x} and \code{y}. If supplied, both \code{x} and
\code{y} will be cast to this type.}

\item{x_arg, y_arg}{Argument names for \code{x} and \code{y}. These are used in error
messages.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector of the common type of \code{x} and \code{y} (or \code{ptype}, if supplied)
containing the result of the corresponding set function.
}
\description{
\itemize{
\item \code{vec_set_intersect()} returns all values in both \code{x} and \code{y}.
\item \code{vec_set_difference()} returns all values in \code{x} but not \code{y}. Note
that this is an asymmetric set difference, meaning it is not commutative.
\item \code{vec_set_union()} returns all values in either \code{x} or \code{y}.
\item \code{vec_set_symmetric_difference()} returns all values in either \code{x} or \code{y}
but not both. This is a commutative difference.
}

Because these are \emph{set} operations, these functions only return unique values
from \code{x} and \code{y}, returned in the order they first appeared in the original
input. Names of \code{x} and \code{y} are retained on the result, but names are always
taken from \code{x} if the value appears in both inputs.

These functions work similarly to \code{\link[base:sets]{base::intersect()}}, \code{\link[base:sets]{base::setdiff()}}, and
\code{\link[base:sets]{base::union()}}, but don't strip attributes and can be used with data frames.
}
\details{
Missing values are treated as equal to other missing values. For doubles and
complexes, \code{NaN} are equal to other \code{NaN}, but not to \code{NA}.
}
\section{Dependencies}{

\subsection{\code{vec_set_intersect()}}{
\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
\item \code{\link[=vec_slice]{vec_slice()}}
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
}
}

\subsection{\code{vec_set_difference()}}{
\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
\item \code{\link[=vec_slice]{vec_slice()}}
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
}
}

\subsection{\code{vec_set_union()}}{
\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
\item \code{\link[=vec_slice]{vec_slice()}}
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
\item \code{\link[=vec_c]{vec_c()}}
}
}

\subsection{\code{vec_set_symmetric_difference()}}{
\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
\item \code{\link[=vec_slice]{vec_slice()}}
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_cast]{vec_cast()}}
\item \code{\link[=vec_c]{vec_c()}}
}
}
}

\examples{
x <- c(1, 2, 1, 4, 3)
y <- c(2, 5, 5, 1)

# All unique values in both `x` and `y`.
# Duplicates in `x` and `y` are always removed.
vec_set_intersect(x, y)

# All unique values in `x` but not `y`
vec_set_difference(x, y)

# All unique values in either `x` or `y`
vec_set_union(x, y)

# All unique values in either `x` or `y` but not both
vec_set_symmetric_difference(x, y)

# These functions can also be used with data frames
x <- data_frame(
  a = c(2, 3, 2, 2),
  b = c("j", "k", "j", "l")
)
y <- data_frame(
  a = c(1, 2, 2, 2, 3),
  b = c("j", "l", "j", "l", "j")
)

vec_set_intersect(x, y)
vec_set_difference(x, y)
vec_set_union(x, y)
vec_set_symmetric_difference(x, y)

# Vector names don't affect set membership, but if you'd like to force
# them to, you can transform the vector into a two column data frame
x <- c(a = 1, b = 2, c = 2, d = 3)
y <- c(c = 2, b = 1, a = 3, d = 3)

vec_set_intersect(x, y)

x <- data_frame(name = names(x), value = unname(x))
y <- data_frame(name = names(y), value = unname(y))

vec_set_intersect(x, y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictionary.R
\name{vec_duplicate}
\alias{vec_duplicate}
\alias{vec_duplicate_any}
\alias{vec_duplicate_detect}
\alias{vec_duplicate_id}
\title{Find duplicated values}
\usage{
vec_duplicate_any(x)

vec_duplicate_detect(x)

vec_duplicate_id(x)
}
\arguments{
\item{x}{A vector (including a data frame).}
}
\value{
\itemize{
\item \code{vec_duplicate_any()}: a logical vector of length 1.
\item \code{vec_duplicate_detect()}: a logical vector the same length as \code{x}.
\item \code{vec_duplicate_id()}: an integer vector the same length as \code{x}.
}
}
\description{
\itemize{
\item \code{vec_duplicate_any()}: detects the presence of duplicated values,
similar to \code{\link[=anyDuplicated]{anyDuplicated()}}.
\item \code{vec_duplicate_detect()}: returns a logical vector describing if each
element of the vector is duplicated elsewhere. Unlike \code{\link[=duplicated]{duplicated()}}, it
reports all duplicated values, not just the second and subsequent
repetitions.
\item \code{vec_duplicate_id()}: returns an integer vector giving the location of
the first occurrence of the value.
}
}
\section{Missing values}{

In most cases, missing values are not considered to be equal, i.e.
\code{NA == NA} is not \code{TRUE}. This behaviour would be unappealing here,
so these functions consider all \code{NAs} to be equal. (Similarly,
all \code{NaN} are also considered to be equal.)
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\examples{
vec_duplicate_any(1:10)
vec_duplicate_any(c(1, 1:10))

x <- c(10, 10, 20, 30, 30, 40)
vec_duplicate_detect(x)
# Note that `duplicated()` doesn't consider the first instance to
# be a duplicate
duplicated(x)

# Identify elements of a vector by the location of the first element that
# they're equal to:
vec_duplicate_id(x)
# Location of the unique values:
vec_unique_loc(x)
# Equivalent to `duplicated()`:
vec_duplicate_id(x) == seq_along(x)
}
\seealso{
\code{\link[=vec_unique]{vec_unique()}} for functions that work with the dual of duplicated
values: unique values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/assert.R
\name{vector-checks}
\alias{vector-checks}
\alias{obj_is_vector}
\alias{obj_check_vector}
\alias{vec_check_size}
\alias{vec_check_recyclable}
\title{Vector checks}
\usage{
obj_is_vector(x)

obj_check_vector(x, ..., arg = caller_arg(x), call = caller_env())

vec_check_size(x, size, ..., arg = caller_arg(x), call = caller_env())

vec_check_recyclable(x, size, ..., arg = caller_arg(x), call = caller_env())
}
\arguments{
\item{x}{For \verb{obj_*()} functions, an object. For \verb{vec_*()} functions, a
vector.}

\item{...}{These dots are for future extensions and must be empty.}

\item{arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{size}{The size to check for compatibility with.}
}
\value{
\itemize{
\item \code{obj_is_vector()} returns a single \code{TRUE} or \code{FALSE}.
\item \code{obj_check_vector()} returns \code{NULL} invisibly, or errors.
\item \code{vec_check_size()} returns \code{NULL} invisibly, or errors.
\item \code{vec_check_recyclable()} returns \code{NULL} invisibly, or errors.
}
}
\description{
\itemize{
\item \code{obj_is_vector()} tests if \code{x} is considered a vector in the vctrs sense.
See \emph{Vectors and scalars} below for the exact details.
\item \code{obj_check_vector()} uses \code{obj_is_vector()} and throws a standardized and
informative error if it returns \code{FALSE}.
\item \code{vec_check_size()} tests if \code{x} has size \code{size}, and throws an informative
error if it doesn't.
\item \code{vec_check_recyclable()} tests if \code{x} can recycle to size \code{size}, and
throws an informative error if it can't.
}
}
\section{Vectors and scalars}{


Informally, a vector is a collection that makes sense to use as column in a
data frame. The following rules define whether or not \code{x} is considered a
vector.

If no \code{\link[=vec_proxy]{vec_proxy()}} method has been registered, \code{x} is a vector if:
\itemize{
\item The \link[=typeof]{base type} of the object is atomic: \code{"logical"}, \code{"integer"},
\code{"double"}, \code{"complex"}, \code{"character"}, or \code{"raw"}.
\item \code{x} is a list, as defined by \code{\link[=obj_is_list]{obj_is_list()}}.
\item \code{x} is a \link{data.frame}.
}

If a \code{vec_proxy()} method has been registered, \code{x} is a vector if:
\itemize{
\item The proxy satisfies one of the above conditions.
\item The base type of the proxy is \code{"list"}, regardless of its class. S3 lists
are thus treated as scalars unless they implement a \code{vec_proxy()} method.
}

Otherwise an object is treated as scalar and cannot be used as a vector.
In particular:
\itemize{
\item \code{NULL} is not a vector.
\item S3 lists like \code{lm} objects are treated as scalars by default.
\item Objects of type \link{expression} are not treated as vectors.
}
}

\section{Technical limitations}{

\itemize{
\item Support for S4 vectors is currently limited to objects that inherit from an
atomic type.
\item Subclasses of \link{data.frame} that \emph{append} their class to the back of the
\code{"class"} attribute are not treated as vectors. If you inherit from an S3
class, always prepend your class to the front of the \code{"class"} attribute
for correct dispatch. This matches our general principle of allowing
subclasses but not mixins.
}
}

\examples{
obj_is_vector(1)

# Data frames are vectors
obj_is_vector(data_frame())

# Bare lists are vectors
obj_is_vector(list())

# S3 lists are vectors if they explicitly inherit from `"list"`
x <- structure(list(), class = c("my_list", "list"))
obj_is_list(x)
obj_is_vector(x)

# But if they don't explicitly inherit from `"list"`, they aren't
# automatically considered to be vectors. Instead, vctrs considers this
# to be a scalar object, like a linear model returned from `lm()`.
y <- structure(list(), class = "my_list")
obj_is_list(y)
obj_is_vector(y)

# `obj_check_vector()` throws an informative error if the input
# isn't a vector
try(obj_check_vector(y))

# `vec_check_size()` throws an informative error if the size of the
# input doesn't match `size`
vec_check_size(1:5, size = 5)
try(vec_check_size(1:5, size = 4))

# `vec_check_recyclable()` throws an informative error if the input can't
# recycle to size `size`
vec_check_recyclable(1:5, size = 5)
vec_check_recyclable(1, size = 5)
try(vec_check_recyclable(1:2, size = 5))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slice.R
\name{vec_init}
\alias{vec_init}
\title{Initialize a vector}
\usage{
vec_init(x, n = 1L)
}
\arguments{
\item{x}{Template of vector to initialize.}

\item{n}{Desired size of result.}
}
\description{
Initialize a vector
}
\section{Dependencies}{

\itemize{
\item vec_slice()
}
}

\examples{
vec_init(1:10, 3)
vec_init(Sys.Date(), 5)

# The "missing" value for a data frame is a row that is entirely missing
vec_init(mtcars, 2)

# The "missing" value for a list is `NULL`
vec_init(list(), 3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/case-when.R
\name{vec-case-and-replace}
\alias{vec-case-and-replace}
\alias{vec_case_when}
\alias{vec_replace_when}
\title{Recode and replace using logical conditions}
\usage{
vec_case_when(
  conditions,
  values,
  ...,
  default = NULL,
  unmatched = "default",
  ptype = NULL,
  size = NULL,
  conditions_arg = "conditions",
  values_arg = "values",
  default_arg = "default",
  error_call = current_env()
)

vec_replace_when(
  x,
  conditions,
  values,
  ...,
  x_arg = "x",
  conditions_arg = "conditions",
  values_arg = "values",
  error_call = current_env()
)
}
\arguments{
\item{conditions}{A list of logical condition vectors.

For \code{vec_case_when()}, each vector should be the same size.

For \code{vec_replace_when()}, each vector should be the same size as \code{x}.

Where a value in \code{conditions} is \code{TRUE}, the corresponding value in
\code{values} will be assigned to the result.}

\item{values}{A list of vectors.

For \code{vec_case_when()}, each vector should be size 1 or the size implied by
\code{conditions}. The common type of \code{values} and \code{default} determine the
output type, unless overridden by \code{ptype}.

For \code{vec_replace_when()}, each vector should be size 1 or the same size
as \code{x}. Each vector will be cast to the type of \code{x}.}

\item{...}{These dots are for future extensions and must be empty.}

\item{default}{Default value to use when \code{conditions} does not match every
location in the output.

By default, a missing value is used as the default value.

If supplied, \code{default} must be size 1 or the size implied by \code{conditions}.

Can only be set when \code{unmatched = "default"}.}

\item{unmatched}{Handling of unmatched locations.

One of:
\itemize{
\item \code{"default"} to use \code{default} in unmatched locations.
\item \code{"error"} to error when there are unmatched locations.
}}

\item{ptype}{An optional override for the output type, which is usually
computed as the common type of \code{values} and \code{default}.}

\item{size}{An optional override for the output size, which is usually
computed as the size of the first element of \code{conditions}.

Only useful for requiring a fixed size when \code{conditions} is an empty list.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{x}{A vector.}

\item{x_arg, conditions_arg, values_arg, default_arg}{Argument names used in
error messages.}
}
\value{
A vector.
\itemize{
\item For \code{vec_case_when()}, the type of the output is computed as the common
type of \code{values} and \code{default}, unless overridden by \code{ptype}. The names of
the output come from the names of \code{values} and \code{default}. The size of the
output comes from the implied size from \code{conditions}, unless overridden by
\code{size}.
\item For \code{vec_replace_when()}, the type of the output will have the same type as
\code{x}. The names of the output will be the same as the names of \code{x}. The size
of the output will be the same size as \code{x}.
}
}
\description{
\itemize{
\item \code{vec_case_when()} constructs an entirely new vector by recoding the \code{TRUE}
\code{conditions} to their corresponding \code{values}. If there are locations not
matched by \code{conditions}, then they are recoded to the \code{default} value.
\item \code{vec_replace_when()} updates an existing vector by replacing the values
from \code{x} matched by the \code{TRUE} \code{conditions} with their corresponding
\code{values}. In this case, each element of \code{values} must have the same type as
\code{x} and locations not matched by \code{conditions} retain their original \code{x}
value.
}

\code{vec_case_when()} is often thought of as a way to vectorize multiple if-else
statements, and is an R equivalent of the SQL "searched" \verb{CASE WHEN}
statement.
}
\examples{
# Note how the first `TRUE` is used in the output.
# Also note how the `NA` falls through to `default`.
x <- seq(-2L, 2L, by = 1L)
x <- c(x, NA)
conditions <- list(
  x < 0,
  x < 1
)
values <- list(
  "<0",
  "<1"
)
vec_case_when(
  conditions,
  values,
  default = "other"
)

# Missing values need to be handled with their own case
# if you want them to have a special value
conditions <- list(
  x < 0,
  x < 1,
  is.na(x)
)
values <- list(
  "<0",
  "<1",
  NA
)
vec_case_when(
  conditions,
  values,
  default = "other"
)

# Both `values` and `default` are vectorized
values <- list(
  x * 5,
  x * 10,
  NA
)
vec_case_when(
  conditions,
  values,
  default = x * 100
)

# Use `vec_replace_when()` if you need to update `x`, retaining
# all previous values in locations that you don't match
conditions <- list(
  x < 0,
  x < 1
)
values <- list(
  0,
  1
)
out <- vec_replace_when(
  x,
  conditions,
  values
)
out

# Note how `vec_replace_when()` is type stable on `x`, we retain the
# integer type here even though `values` contained doubles
typeof(out)

# `vec_case_when()` creates a new vector, so names come from `values`
# and `default`. `vec_replace_when()` modifies an existing vector, so
# names come from `x` no matter what, just like `[<-` and `base::replace()`
x <- c(a = 1, b = 2, c = 3)
conditions <- list(x == 1, x == 2)
values <- list(c(x = 0), c(y = -1))
vec_case_when(conditions, values)
vec_replace_when(x, conditions, values)

# If you want to enforce that you've covered all of the locations in your
# `conditions`, use `unmatched = "error"` rather than providing a `default`
x <- c(0, 1, 2)
conditions <- list(x == 1, x == 2)
values <- list("a", "b")
try(vec_case_when(conditions, values, unmatched = "error"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-data-frame.R, R/type-tibble.R
\name{df_ptype2}
\alias{df_ptype2}
\alias{df_cast}
\alias{tib_ptype2}
\alias{tib_cast}
\title{Coercion between two data frames}
\usage{
df_ptype2(x, y, ..., x_arg = "", y_arg = "", call = caller_env())

df_cast(x, to, ..., x_arg = "", to_arg = "", call = caller_env())

tib_ptype2(x, y, ..., x_arg = "", y_arg = "", call = caller_env())

tib_cast(x, to, ..., x_arg = "", to_arg = "", call = caller_env())
}
\arguments{
\item{x, y, to}{Subclasses of data frame.}

\item{...}{If you call \code{df_ptype2()} or \code{df_cast()} from a
\code{vec_ptype2()} or \code{vec_cast()} method, you must forward the dots
passed to your method on to \code{df_ptype2()} or \code{df_cast()}.}

\item{x_arg, y_arg}{Argument names for \code{x} and \code{y}. These are used
in error messages to inform the user about the locations of
incompatible types (see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{to_arg}{Argument name \code{to} used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}
}
\value{
\itemize{
\item When \code{x} and \code{y} are not compatible, an error of class
\code{vctrs_error_incompatible_type} is thrown.
\item When \code{x} and \code{y} are compatible, \code{df_ptype2()} returns the common
type as a bare data frame. \code{tib_ptype2()} returns the common type
as a bare tibble.
}
}
\description{
\code{df_ptype2()} and \code{df_cast()} are the two functions you need to
call from \code{vec_ptype2()} and \code{vec_cast()} methods for data frame
subclasses. See \link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}.
Their main job is to determine the common type of two data frames,
adding and coercing columns as needed, or throwing an incompatible
type error when the columns are not compatible.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/slice-interleave.R
\name{vec_interleave}
\alias{vec_interleave}
\title{Interleave many vectors into one vector}
\usage{
vec_interleave(
  ...,
  .size = NULL,
  .ptype = NULL,
  .name_spec = NULL,
  .name_repair = c("minimal", "unique", "check_unique", "universal", "unique_quiet",
    "universal_quiet"),
  .error_call = current_env()
)
}
\arguments{
\item{...}{Vectors to interleave.}

\item{.size}{The expected size of each vector. If not provided, computed
automatically by \code{\link[=vec_size_common]{vec_size_common()}}. Each vector will be
\link[=theory-faq-recycling]{recycled} to this size.}

\item{.ptype}{The expected type of each vector. If not provided, computed
automatically by \code{\link[=vec_ptype_common]{vec_ptype_common()}}. Each vector will be
\link[=theory-faq-coercion]{cast} to this type.}

\item{.name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}

\item{.name_repair}{How to repair names, see \code{repair} options in
\code{\link[=vec_as_names]{vec_as_names()}}.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\code{vec_interleave()} combines multiple vectors together, much like \code{\link[=vec_c]{vec_c()}},
but does so in such a way that the elements of each vector are interleaved
together.

It is a more efficient equivalent to the following usage of \code{vec_c()}:

\if{html}{\out{<div class="sourceCode">}}\preformatted{vec_interleave(x, y) == vec_c(x[1], y[1], x[2], y[2], ..., x[n], y[n])
}\if{html}{\out{</div>}}
}
\section{Dependencies}{

\subsection{vctrs dependencies}{
\itemize{
\item \code{\link[=list_combine]{list_combine()}}
}
}
}

\examples{
# The most common case is to interleave two vectors
vec_interleave(1:3, 4:6)

# But you aren't restricted to just two
vec_interleave(1:3, 4:6, 7:9, 10:12)

# You can also interleave data frames
x <- data_frame(x = 1:2, y = c("a", "b"))
y <- data_frame(x = 3:4, y = c("c", "d"))

vec_interleave(x, y)

# `.size` can be used to recycle size 1 elements before interleaving
vec_interleave(1, 2, .size = 3)

# `.ptype` can be used to enforce a particular type
typeof(vec_interleave(1, 2, .ptype = integer()))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/equal.R
\name{vec_equal}
\alias{vec_equal}
\title{Equality}
\usage{
vec_equal(x, y, na_equal = FALSE, .ptype = NULL)
}
\arguments{
\item{x, y}{Vectors with compatible types and lengths.}

\item{na_equal}{Should \code{NA} values be considered equal?}

\item{.ptype}{Override to optionally specify common type}
}
\value{
A logical vector the same size as the common size of \code{x} and \code{y}.
Will only contain \code{NA}s if \code{na_equal} is \code{FALSE}.
}
\description{
\code{vec_equal()} tests if two vectors are equal.
}
\details{
Attributes of \code{x} and \code{y} are considered equal if they have the same names
and values, even if the attribute ordering is different. This reflects the
idea that attributes are treated as a map rather than an ordered list.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_cast_common]{vec_cast_common()}} with fallback
\item \code{\link[=vec_recycle_common]{vec_recycle_common()}}
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\examples{
vec_equal(c(TRUE, FALSE, NA), FALSE)
vec_equal(c(TRUE, FALSE, NA), FALSE, na_equal = TRUE)

vec_equal(5, 1:10)
vec_equal("d", letters[1:10])

df <- data.frame(x = c(1, 1, 2, 1), y = c(1, 2, 1, NA))
vec_equal(df, data.frame(x = 1, y = 2))
}
\seealso{
\code{\link[=vec_detect_missing]{vec_detect_missing()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/names.R
\name{vec_as_names_legacy}
\alias{vec_as_names_legacy}
\title{Repair names with legacy method}
\usage{
vec_as_names_legacy(names, prefix = "V", sep = "")
}
\arguments{
\item{names}{A character vector.}

\item{prefix, sep}{Prefix and separator for repaired names.}
}
\description{
This standardises names with the legacy approach that was used in
tidyverse packages (such as tibble, tidyr, and readxl) before
\code{\link[=vec_as_names]{vec_as_names()}} was implemented. This tool is meant to help
transitioning to the new name repairing standard and will be
deprecated and removed from the package some time in the future.
}
\examples{
if (rlang::is_installed("tibble")) {

library(tibble)

# Names repair is turned off by default in tibble:
try(tibble(a = 1, a = 2))

# You can turn it on by supplying a repair method:
tibble(a = 1, a = 2, .name_repair = "universal")

# If you prefer the legacy method, use `vec_as_names_legacy()`:
tibble(a = 1, a = 2, .name_repair = vec_as_names_legacy)

}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/size.R
\name{\%0\%}
\alias{\%0\%}
\title{Default value for empty vectors}
\usage{
x \%0\% y
}
\arguments{
\item{x}{A vector}

\item{y}{Value to use if \code{x} is empty. To preserve type-stability, should
be the same type as \code{x}.}
}
\description{
Use this inline operator when you need to provide a default value for
empty (as defined by \code{\link[=vec_is_empty]{vec_is_empty()}}) vectors.
}
\examples{
1:10 \%0\% 5
integer() \%0\% 5
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/missing.R
\name{missing}
\alias{missing}
\alias{vec_detect_missing}
\alias{vec_any_missing}
\title{Missing values}
\usage{
vec_detect_missing(x)

vec_any_missing(x)
}
\arguments{
\item{x}{A vector}
}
\value{
\itemize{
\item \code{vec_detect_missing()} returns a logical vector the same size as \code{x}.
\item \code{vec_any_missing()} returns a single \code{TRUE} or \code{FALSE}.
}
}
\description{
\itemize{
\item \code{vec_detect_missing()} returns a logical vector the same size as \code{x}. For
each element of \code{x}, it returns \code{TRUE} if the element is missing, and \code{FALSE}
otherwise.
\item \code{vec_any_missing()} returns a single \code{TRUE} or \code{FALSE} depending on whether
or not \code{x} has \emph{any} missing values.
}
\subsection{Differences with \code{\link[=is.na]{is.na()}}}{

Data frame rows are only considered missing if every element in the row is
missing. Similarly, \link[=new_rcrd]{record vector} elements are only considered
missing if every field in the record is missing. Put another way, rows with
\emph{any} missing values are considered \link[=vec_detect_complete]{incomplete}, but
only rows with \emph{all} missing values are considered missing.

List elements are only considered missing if they are \code{NULL}.
}
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\examples{
x <- c(1, 2, NA, 4, NA)

vec_detect_missing(x)
vec_any_missing(x)

# Data frames are iterated over rowwise, and only report a row as missing
# if every element of that row is missing. If a row is only partially
# missing, it is said to be incomplete, but not missing.
y <- c("a", "b", NA, "d", "e")
df <- data_frame(x = x, y = y)

df$missing <- vec_detect_missing(df)
df$incomplete <- !vec_detect_complete(df)
df
}
\seealso{
\code{\link[=vec_detect_complete]{vec_detect_complete()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list-of-transpose.R
\name{list_of_transpose}
\alias{list_of_transpose}
\title{Transpose a list of homogenous vectors}
\usage{
list_of_transpose(x, ..., x_arg = caller_arg(x), error_call = current_env())
}
\arguments{
\item{x}{A \link[=list_of]{list_of} with both \code{size} and \code{ptype} specified.}

\item{...}{These dots are for future extensions and must be empty.}

\item{x_arg}{Argument name used in error messages.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A \code{list_of} of size \code{list_of_size(x)}, with an element size of \code{vec_size(x)}
and an element type of \code{list_of_ptype(x)}.
}
\description{
\code{list_of_transpose()} takes a list of homogenous vectors, transposes it, and
returns a new list of homogenous vectors. To perform a transpose, three
pieces of information are required:
\itemize{
\item The \emph{list size}, from \code{\link[=vec_size]{vec_size(x)}}.
\item The \emph{element size}, from \code{\link[=list_of_size]{list_of_size(x)}}.
\item The \emph{element type}, from \code{\link[=list_of_ptype]{list_of_ptype(x)}}.
}

Because all three of these are required, this function only works on fully
specified \code{\link[=list_of]{list_of()}}s, with both \code{size} and \code{ptype} specified.

To predict the output from \code{list_of_transpose()}, swap the list size with the
element size. For example:
\itemize{
\item Input: \verb{list_of<integer[3]>[2]}
\item Output: \verb{list_of<integer[2]>[3]}
}
}
\examples{
# A form of `list_of()` that infers both ptype and size
list_of2 <- function(...) {
  list_of(..., .ptype = NULL, .size = NULL)
}

# I: list_of<integer[2]>[3]
# O: list_of<integer[3]>[2]
list_of_transpose(list_of2(1:2, 3:4, 5:6))

# With data frames
x <- data_frame(a = 1:2, b = letters[1:2])
y <- data_frame(a = 3:4, b = letters[3:4])
list_of_transpose(list_of2(x, y))

# Size 1 elements are recycled
list_of_transpose(list_of2(1, 2:3, 4))

# ---------------------------------------------------------------------------
# `NULL` handling

# `NULL` values aren't allowed in `list_of_transpose()`
x <- list_of2(1:3, NULL, 5:7, NULL)
try(list_of_transpose(x))

# Either drop them entirely or replace them up front before transposing

x_dropped <- vec_slice(x, !vec_detect_missing(x))
x_dropped

list_of_transpose(x_dropped)

x_replaced <- vec_assign(x, vec_detect_missing(x), list(NA))
x_replaced

list_of_transpose(x_replaced)

# ---------------------------------------------------------------------------
# Reversibility

# Because `list_of_transpose()` takes and returns fully specified list-ofs,
# it is fully reversible, even in the edge cases.
x <- list_of2(integer(), integer())

# This returns a list of size 0
# I: list_of<integer[0]>[2]
# O: list_of<integer[2]>[0]
out <- list_of_transpose(x)
out

# Even though there are no elements, we know the element size and type,
# so we can transpose a second time to recover `x`. This would not be
# possible if this function returned a bare `list()`, which would result
# in lost information.
# I: list_of<integer[2]>[0]
# O: list_of<integer[0]>[2]
list_of_transpose(out)

# ---------------------------------------------------------------------------
# Padding

# If you'd like to pad with a missing value rather than erroring,
# you might do something like this, which left-pads before conversion
# to list-of.
x <- list(1, 2:5, 6:7)

sizes <- list_sizes(x)
size <- max(sizes)
index <- which(sizes != size)

x[index] <- lapply(
  index,
  function(i) vec_c(rep(NA, times = size - sizes[[i]]), x[[i]])
)
x

x <- as_list_of(x, .ptype = NULL, .size = NULL)

list_of_transpose(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-unspecified.R
\name{vctrs-unspecified}
\alias{vctrs-unspecified}
\alias{unspecified}
\alias{vec_ptype_finalise}
\title{Unspecified vectors and prototype finalisation}
\usage{
unspecified(n = 0)

vec_ptype_finalise(x, ...)
}
\arguments{
\item{n}{Length of vector}

\item{x}{A \code{ptype} to finalize, typically a result of \code{\link[=vec_ptype]{vec_ptype()}},
\code{\link[=vec_ptype2]{vec_ptype2()}}, or \code{\link[=vec_ptype_common]{vec_ptype_common(.finalise = FALSE)}}.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
\code{unspecified()} is the underlying type used to represent logical vectors that
only contain \code{NA}. These require special handling because we want to allow
logical \code{NA} to specify missingness that can be cast to any other type.

In vctrs, the \verb{<unspecified>} type is considered \emph{unfinalised} and is not
suitable for use in most vctrs functions that take a \code{ptype} argument, like
\code{\link[=vec_c]{vec_c()}}. The purpose of \code{vec_ptype_finalise()} is to finalise any
\verb{<unspecified>} types into \verb{<logical>} after common type determination
has been completed.

\code{\link[=vec_ptype]{vec_ptype()}} and \code{\link[=vec_ptype2]{vec_ptype2()}} return \emph{unfinalised} types, and will convert
a logical vector of \code{NA} into an empty \verb{<unspecified>} type that can combine
with any other type. It is unlikely that you will call these yourself, but,
if you do, you'll need to manually finalise with \code{vec_ptype_finalise()} to
take care of any \verb{<unspecified>} types.

\code{\link[=vec_ptype_common]{vec_ptype_common()}} uses both \code{\link[=vec_ptype]{vec_ptype()}} and \code{\link[=vec_ptype2]{vec_ptype2()}} to compute
the common type, but typically returns a \emph{finalised} type for immediate usage
in other vctrs functions. You can optionally skip finalisation by setting
\code{.finalise = FALSE}, in which case \code{vec_ptype_common()} can return
\verb{<unspecified>} and you'll need to manually call \code{vec_ptype_finalise()}
yourself.

\code{vec_ptype_finalise()} is an S3 generic, but it is extremely rare to need to
write an S3 method for this. Data frames (and data frame subclasses) are
already recursively finalised by the default method. The only time you may
need to write an S3 method for \code{vec_ptype_finalise()} is if your class
\emph{wraps} an arbitrary vector that has the potential to be a logical vector
containing only \code{NA}s. See \code{ivs::iv()} for an example of this, which wraps
arbitrary \code{start} and \code{end} vectors of the same type into a single interval
vector class.
}
\examples{
# Returns `unspecified()`
vec_ptype(NA)
vec_ptype(c(NA, NA))

# We've chosen to make this return `logical()`, but this is admittedly
# ambiguous, as it could be seen as "an empty vector of `NA`s" that could
# also be treated as unspecified.
vec_ptype(logical())

# These return `unspecified()`
vec_ptype2(NA, NA)
vec_ptype2(NA, NULL)
vec_ptype2(NULL, NA)

# An unspecified vector can combine with any other type
vec_ptype2(NA, "x")
vec_ptype2("x", NA)

# Same as using `unspecified()` directly
vec_ptype2(unspecified(1), "x")
vec_ptype2("x", unspecified(1))

# Finalising a ptype turns unspecified back to logical
vec_ptype(NA)
vec_ptype_finalise(vec_ptype(NA))

# This works recursively over data frames
df <- data_frame(x = NA, y = data_frame(z = NA))
vec_ptype_show(vec_ptype(df))
vec_ptype_show(vec_ptype_finalise(vec_ptype(df)))

# `vec_ptype_common()` finalises automatically rather than returning an
# unspecified type
vec_ptype_common(NA)
vec_ptype_common(NA, NA)
vec_ptype_show(vec_ptype_common(df))

# `vec_ptype_common()` lets you opt out of finalisation using `.finalise`
vec_ptype_common(NA, .finalise = FALSE)
vec_ptype_show(vec_ptype_common(df, .finalise = FALSE))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-package.R
\docType{package}
\name{vctrs-package}
\alias{vctrs}
\alias{vctrs-package}
\title{vctrs: Vector Helpers}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#maturing}{\figure{lifecycle-maturing.svg}{options: alt='[Maturing]'}}}{\strong{[Maturing]}}

Defines new notions of prototype and size that are
used to provide tools for consistent and well-founded type-coercion
and size-recycling, and are in turn connected to ideas of type- and
size-stability useful for analysing function interfaces.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://vctrs.r-lib.org/}
  \item \url{https://github.com/r-lib/vctrs}
  \item Report bugs at \url{https://github.com/r-lib/vctrs/issues}
}

}
\author{
\strong{Maintainer}: Davis Vaughan \email{davis@posit.co}

Authors:
\itemize{
  \item Hadley Wickham \email{hadley@posit.co}
  \item Lionel Henry \email{lionel@posit.co}
}

Other contributors:
\itemize{
  \item data.table team (Radix sort based on data.table's forder() and their contribution to R's order()) [copyright holder]
  \item Posit Software, PBC [copyright holder, funder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{theory-faq-recycling}
\alias{theory-faq-recycling}
\alias{vector_recycling_rules}
\title{FAQ - How does recycling work in vctrs and the tidyverse?}
\description{
Recycling describes the concept of repeating elements of one vector to
match the size of another. There are two rules that underlie the
“tidyverse” recycling rules:
\itemize{
\item Vectors of size 1 will be recycled to the size of any other vector
\item Otherwise, all vectors must have the same size
}
}
\section{Examples}{
Vectors of size 1 are recycled to the size of any other vector:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{tibble(x = 1:3, y = 1L)
#> # A tibble: 3 x 2
#>       x     y
#>   <int> <int>
#> 1     1     1
#> 2     2     1
#> 3     3     1
}\if{html}{\out{</div>}}

This includes vectors of size 0:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{tibble(x = integer(), y = 1L)
#> # A tibble: 0 x 2
#> # i 2 variables: x <int>, y <int>
}\if{html}{\out{</div>}}

If vectors aren’t size 1, they must all be the same size. Otherwise, an
error is thrown:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{tibble(x = 1:3, y = 4:7)
#> Error in `tibble()`:
#> ! Tibble columns must have compatible sizes.
#> * Size 3: Existing data.
#> * Size 4: Column `y`.
#> i Only values of size one are recycled.
}\if{html}{\out{</div>}}
}

\section{vctrs backend}{
Packages in r-lib and the tidyverse generally use
\code{\link[=vec_size_common]{vec_size_common()}} and
\code{\link[=vec_recycle_common]{vec_recycle_common()}} as the backends for
handling recycling rules.
\itemize{
\item \code{vec_size_common()} returns the common size of multiple vectors, after
applying the recycling rules
\item \code{vec_recycle_common()} goes one step further, and actually recycles
the vectors to their common size
}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_size_common(1:3, "x")
#> [1] 3

vec_recycle_common(1:3, "x")
#> [[1]]
#> [1] 1 2 3
#> 
#> [[2]]
#> [1] "x" "x" "x"

vec_size_common(1:3, c("x", "y"))
#> Error:
#> ! Can't recycle `..1` (size 3) to match `..2` (size 2).
}\if{html}{\out{</div>}}
}

\section{Base R recycling rules}{
The recycling rules described here are stricter than the ones generally
used by base R, which are:
\itemize{
\item If any vector is length 0, the output will be length 0
\item Otherwise, the output will be length \code{max(length_x, length_y)}, and a
warning will be thrown if the length of the longer vector is not an
integer multiple of the length of the shorter vector.
}

We explore the base R rules in detail in \code{vignette("type-size")}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type.R
\name{vec_ptype}
\alias{vec_ptype}
\alias{vec_ptype_common}
\alias{vec_ptype_show}
\title{Find the prototype of a set of vectors}
\usage{
vec_ptype(x, ..., x_arg = "", call = caller_env())

vec_ptype_common(
  ...,
  .ptype = NULL,
  .finalise = TRUE,
  .arg = "",
  .call = caller_env()
)

vec_ptype_show(...)
}
\arguments{
\item{x}{A vector}

\item{...}{For \code{vec_ptype()}, these dots are for future extensions and must
be empty.

For \code{vec_ptype_common()} and \code{vec_ptype_show()}, vector inputs.}

\item{x_arg}{Argument name for \code{x}. This is used in error messages to inform
the user about the locations of incompatible types.}

\item{call, .call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{.ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type across all elements of \code{...}.

Alternatively, you can supply \code{.ptype} to give the output known type.
If \code{getOption("vctrs.no_guessing")} is \code{TRUE} you must supply this value:
this is a convenient way to make production code demand fixed types.}

\item{.finalise}{Should \code{vec_ptype_common()} \link[=vec_ptype_finalise]{finalise}
its output?
\itemize{
\item If \code{TRUE}, \code{\link[=vec_ptype_finalise]{vec_ptype_finalise()}} is called on the final \code{ptype} before
it is returned. Practically this has the effect of converting any
types from \link{unspecified} to logical.
\item If \code{FALSE}, \link{unspecified} types are left unfinalised, which can be useful
for advanced cases where you combine one common type result with another
type via \code{\link[=vec_ptype2]{vec_ptype2()}}. Note that you must manually call
\code{\link[=vec_ptype_finalise]{vec_ptype_finalise()}} on the final \code{ptype} before supplying it to any
other vctrs functions.
}}

\item{.arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}
}
\value{
\code{vec_ptype()} and \code{vec_ptype_common()} return a prototype
(a size-0 vector).
}
\description{
\itemize{
\item \code{vec_ptype()} returns the \link[=vec_ptype_finalise]{unfinalised} prototype of a
single vector.
\item \code{vec_ptype_common()} returns the common type of multiple vectors. By
default, this is \link[=vec_ptype_finalise]{finalised} for immediate usage, but
can optionally be left unfinalised for advanced common type determination.
\item \code{vec_ptype_show()} nicely prints the common type of any number of inputs,
and is designed for interactive exploration.
}
}
\section{\code{vec_ptype()}}{


\code{vec_ptype()} returns \link[=vec_size]{size} 0 vectors potentially
containing attributes but no data. Generally, this is just
\code{vec_slice(x, 0L)}, but some inputs require special
handling.
\itemize{
\item While you can't slice \code{NULL}, the prototype of \code{NULL} is
itself. This is because we treat \code{NULL} as an identity value in
the \code{vec_ptype2()} monoid.
\item The prototype of logical vectors that only contain missing values
is the special \link{unspecified} type, which can be coerced to any
other 1d type. This allows bare \code{NA}s to represent missing values
for any 1d vector type. \link[=vec_ptype_finalise]{Finalising} this type
converts it from unspecified back to logical.
}

See \link{internal-faq-ptype2-identity} for more information about
identity values.

\code{vec_ptype()} is a \emph{performance} generic. It is not necessary to implement it
because the default method will work for any vctrs type. However the default
method builds around other vctrs primitives like \code{vec_slice()} which incurs
performance costs. If your class has a static prototype, you might consider
implementing a custom \code{vec_ptype()} method that returns a constant. This will
improve the performance of your class in many cases (\link[=vec_ptype2]{common type} imputation in particular).

Because it may contain unspecified vectors, the prototype returned by
\code{vec_ptype()} is said to be \strong{unfinalised}. Call \code{\link[=vec_ptype_finalise]{vec_ptype_finalise()}} to
finalise it.
}

\section{\code{vec_ptype_common()}}{


\code{vec_ptype_common()} first finds the prototype of each input, then
successively calls \code{\link[=vec_ptype2]{vec_ptype2()}} to find a common type. It returns a
\link[=vec_ptype_finalise]{finalised} prototype by default, but can optionally be
left unfinalised for advanced common type determination.
}

\section{Dependencies of \code{vec_ptype()}}{

\itemize{
\item \code{\link[=vec_slice]{vec_slice()}} for returning an empty slice
}
}

\section{Dependencies of \code{vec_ptype_common()}}{

\itemize{
\item \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{\link[=vec_ptype_finalise]{vec_ptype_finalise()}}
}
}

\examples{
# Unknown types ------------------------------------------
vec_ptype_show()
vec_ptype_show(NULL)

# Vectors ------------------------------------------------
vec_ptype_show(1:10)
vec_ptype_show(letters)
vec_ptype_show(TRUE)

vec_ptype_show(Sys.Date())
vec_ptype_show(Sys.time())
vec_ptype_show(factor("a"))
vec_ptype_show(ordered("a"))

# Matrices -----------------------------------------------
# The prototype of a matrix includes the number of columns
vec_ptype_show(array(1, dim = c(1, 2)))
vec_ptype_show(array("x", dim = c(1, 2)))

# Data frames --------------------------------------------
# The prototype of a data frame includes the prototype of
# every column
vec_ptype_show(iris)

# The prototype of multiple data frames includes the prototype
# of every column that in any data frame
vec_ptype_show(
  data.frame(x = TRUE),
  data.frame(y = 2),
  data.frame(z = "a")
)

# Finalisation -------------------------------------------

# `vec_ptype()` and `vec_ptype2()` return unfinalised ptypes so that they
# can be coerced to any other type
vec_ptype(NA)
vec_ptype2(NA, NA)

# By default `vec_ptype_common()` finalises so that you can use its result
# directly in other vctrs functions
vec_ptype_common(NA, NA)

# You can opt out of finalisation to make it work like `vec_ptype()` and
# `vec_ptype2()` with `.finalise = FALSE`, but don't forget that you must
# call `vec_ptype_finalise()` manually if you do so!
vec_ptype_common(NA, NA, .finalise = FALSE)
vec_ptype_finalise(vec_ptype_common(NA, NA, .finalise = FALSE))

# This can be useful in rare scenarios, like including a separate `default`
# argument in the ptype computation
xs <- list(NA, NA)
default <- "a"
try(vec_ptype2(vec_ptype_common(!!!xs), default))
vec_ptype2(vec_ptype_common(!!!xs, .finalise = FALSE), default)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-vctr.R
\name{new_vctr}
\alias{new_vctr}
\alias{vctr}
\title{vctr (vector) S3 class}
\usage{
new_vctr(.data, ..., class = character(), inherit_base_type = NULL)
}
\arguments{
\item{.data}{Foundation of class. Must be a vector}

\item{...}{Name-value pairs defining attributes}

\item{class}{Name of subclass.}

\item{inherit_base_type}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
A single logical, or \code{NULL}. Does this class extend the base type of
\code{.data}? i.e. does the resulting object extend the behaviour of the
underlying type? Defaults to \code{FALSE} for all types except lists, which
are required to inherit from the base type.}
}
\description{
This abstract class provides a set of useful default methods that makes it
considerably easier to get started with a new S3 vector class. See
\code{vignette("s3-vector")} to learn how to use it to create your own S3
vector classes.
}
\details{
List vctrs are special cases. When created through \code{new_vctr()}, the
resulting list vctr should always be recognized as a list by
\code{obj_is_list()}. Because of this, if \code{inherit_base_type} is \code{FALSE}
an error is thrown.
}
\section{Base methods}{

The vctr class provides methods for many base generics using a smaller
set of generics defined by this package. Generally, you should think
carefully before overriding any of the methods that vctrs implements for
you as they've been carefully planned to be internally consistent.
\itemize{
\item \code{[[} and \code{[} use \code{NextMethod()} dispatch to the underlying base function,
then restore attributes with \code{vec_restore()}.
\code{rep()} and \verb{length<-} work similarly.
\item \verb{[[<-} and \verb{[<-} cast \code{value} to same type as \code{x}, then call
\code{NextMethod()}.
\item \code{as.logical()}, \code{as.integer()}, \code{as.numeric()}, \code{as.character()},
\code{as.Date()} and \code{as.POSIXct()} methods call \code{vec_cast()}.
The \code{as.list()} method calls \code{[[} repeatedly, and the \code{as.data.frame()}
method uses a standard technique to wrap a vector in a data frame.
\item \code{as.factor()}, \code{as.ordered()} and \code{as.difftime()} are not generic functions
in base R, but have been reimplemented as generics in the \code{generics}
package. \code{vctrs} extends these and calls \code{vec_cast()}. To inherit this
behaviour in a package, import and re-export the generic of interest
from \code{generics}.
\item \code{==}, \code{!=}, \code{unique()}, \code{anyDuplicated()}, and \code{is.na()} use
\code{\link[=vec_proxy]{vec_proxy()}}.
\item \code{<}, \code{<=}, \code{>=}, \code{>}, \code{min()}, \code{max()}, \code{range()}, \code{median()},
\code{quantile()}, and \code{xtfrm()} methods use \code{\link[=vec_proxy_compare]{vec_proxy_compare()}}.
\item \code{+}, \code{-}, \code{/}, \code{*}, \code{^}, \code{\%\%}, \code{\%/\%}, \code{!}, \code{&}, and \code{|} operators
use \code{\link[=vec_arith]{vec_arith()}}.
\item Mathematical operations including the Summary group generics (\code{prod()},
\code{sum()}, \code{any()}, \code{all()}), the Math group generics (\code{abs()}, \code{sign()},
etc), \code{mean()}, \code{is.nan()}, \code{is.finite()}, and \code{is.infinite()}
use \code{\link[=vec_math]{vec_math()}}.
\item \code{dims()}, \verb{dims<-}, \code{dimnames()}, \verb{dimnames<-}, \code{levels()}, and
\verb{levels<-} methods throw errors.
}
}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/empty.R
\name{list_drop_empty}
\alias{list_drop_empty}
\title{Drop empty elements from a list}
\usage{
list_drop_empty(x)
}
\arguments{
\item{x}{A list.}
}
\description{
\code{list_drop_empty()} removes empty elements from a list. This includes \code{NULL}
elements along with empty vectors, like \code{integer(0)}. This is equivalent to,
but faster than, \code{vec_slice(x, list_sizes(x) != 0L)}.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
x <- list(1, NULL, integer(), 2)
list_drop_empty(x)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rank.R
\name{vec_rank}
\alias{vec_rank}
\title{Compute ranks}
\usage{
vec_rank(
  x,
  ...,
  ties = c("min", "max", "sequential", "dense"),
  incomplete = c("rank", "na"),
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
)
}
\arguments{
\item{x}{A vector}

\item{...}{These dots are for future extensions and must be empty.}

\item{ties}{Ranking of duplicate values.
\itemize{
\item \code{"min"}: Use the current rank for all duplicates. The next non-duplicate
value will have a rank incremented by the number of duplicates present.
\item \code{"max"}: Use the current rank \code{+ n_duplicates - 1} for all duplicates.
The next non-duplicate value will have a rank incremented by the number of
duplicates present.
\item \code{"sequential"}: Use an increasing sequence of ranks starting at the
current rank, applied to duplicates in order of appearance.
\item \code{"dense"}: Use the current rank for all duplicates. The next
non-duplicate value will have a rank incremented by \code{1}, effectively
removing any gaps in the ranking.
}}

\item{incomplete}{Ranking of missing and \link[=vec_detect_complete]{incomplete}
observations.
\itemize{
\item \code{"rank"}: Rank incomplete observations normally. Missing values within
incomplete observations will be affected by \code{na_value} and \code{nan_distinct}.
\item \code{"na"}: Don't rank incomplete observations at all. Instead, they are
given a rank of \code{NA}. In this case, \code{na_value} and \code{nan_distinct} have
no effect.
}}

\item{direction}{Direction to sort in.
\itemize{
\item A single \code{"asc"} or \code{"desc"} for ascending or descending order
respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"asc"} or \code{"desc"}, specifying the direction for each column.
}}

\item{na_value}{Ordering of missing values.
\itemize{
\item A single \code{"largest"} or \code{"smallest"} for ordering missing values as the
largest or smallest values respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"largest"} or \code{"smallest"}, specifying how missing values should
be ordered within each column.
}}

\item{nan_distinct}{A single logical specifying whether or not \code{NaN} should
be considered distinct from \code{NA} for double and complex vectors. If \code{TRUE},
\code{NaN} will always be ordered between \code{NA} and non-missing numbers.}

\item{chr_proxy_collate}{A function generating an alternate representation
of character vectors to use for collation, often used for locale-aware
ordering.
\itemize{
\item If \code{NULL}, no transformation is done.
\item Otherwise, this must be a function of one argument. If the input contains
a character vector, it will be passed to this function after it has been
translated to UTF-8. This function should return a character vector with
the same length as the input. The result should sort as expected in the
C-locale, regardless of encoding.
}

For data frames, \code{chr_proxy_collate} will be applied to all character
columns.

Common transformation functions include: \code{tolower()} for case-insensitive
ordering and \code{stringi::stri_sort_key()} for locale-aware ordering.}
}
\description{
\code{vec_rank()} computes the sample ranks of a vector. For data frames, ranks
are computed along the rows, using all columns after the first to break
ties.
}
\details{
Unlike \code{\link[base:rank]{base::rank()}}, when \code{incomplete = "rank"} all missing values are
given the same rank, rather than an increasing sequence of ranks. When
\code{nan_distinct = FALSE}, \code{NaN} values are given the same rank as \code{NA},
otherwise they are given a rank that differentiates them from \code{NA}.

Like \code{\link[=vec_order_radix]{vec_order_radix()}}, ordering is done in the C-locale. This can affect
the ranks of character vectors, especially regarding how uppercase and
lowercase letters are ranked. See the documentation of \code{\link[=vec_order_radix]{vec_order_radix()}}
for more information.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_order_radix]{vec_order_radix()}}
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
x <- c(5L, 6L, 3L, 3L, 5L, 3L)

vec_rank(x, ties = "min")
vec_rank(x, ties = "max")

# Sequential ranks use an increasing sequence for duplicates
vec_rank(x, ties = "sequential")

# Dense ranks remove gaps between distinct values,
# even if there are duplicates
vec_rank(x, ties = "dense")

y <- c(NA, x, NA, NaN)

# Incomplete values match other incomplete values by default, and their
# overall position can be adjusted with `na_value`
vec_rank(y, na_value = "largest")
vec_rank(y, na_value = "smallest")

# NaN can be ranked separately from NA if required
vec_rank(y, nan_distinct = TRUE)

# Rank in descending order. Since missing values are the largest value,
# they are given a rank of `1` when ranking in descending order.
vec_rank(y, direction = "desc", na_value = "largest")

# Give incomplete values a rank of `NA` by setting `incomplete = "na"`
vec_rank(y, incomplete = "na")

# Can also rank data frames, using columns after the first to break ties
z <- c(2L, 3L, 4L, 4L, 5L, 2L)
df <- data_frame(x = x, z = z)
df

vec_rank(df)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list-unchop.R
\name{list_unchop}
\alias{list_unchop}
\title{Combine a list of vectors}
\usage{
list_unchop(
  x,
  ...,
  indices = NULL,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c("minimal", "unique", "check_unique", "universal", "unique_quiet",
    "universal_quiet"),
  error_arg = "x",
  error_call = current_env()
)
}
\arguments{
\item{x}{A list}

\item{...}{These dots are for future extensions and must be empty.}

\item{indices}{A list of positive integer vectors specifying the
locations to place elements of \code{x} in. Each element of \code{x} is recycled to
the size of the corresponding index vector. The size of \code{indices} must
match the size of \code{x}. If \code{NULL}, \code{x} is combined in the order it is
provided in, which is equivalent to using \code{\link[=vec_c]{vec_c()}}.}

\item{ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type across all elements of \code{x}. Alternatively, you
can supply \code{ptype} to give the output a known type.}

\item{name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}

\item{name_repair}{How to repair names, see \code{repair} options in
\code{\link[=vec_as_names]{vec_as_names()}}.}

\item{error_arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector of type \code{vec_ptype_common(!!!x)}, or \code{ptype}, if specified. The size
is computed as \code{vec_size_common(!!!indices)} unless the indices are \code{NULL},
in which case the size is \code{vec_size_common(!!!x)}.
}
\description{
While \code{list_unchop()} is not deprecated, we now recommend that you use
either:
\itemize{
\item \code{list_combine(x, indices = indices, size = size)} over
\code{list_unchop(x, indices = indices)}
\item \code{vec_c(!!!x)} over \code{list_unchop(x)}
}

\code{list_unchop()} combines a list of vectors into a single vector, placing
elements in the output according to the locations specified by \code{indices}. It
is similar to \code{\link[=vec_c]{vec_c()}}, but gives greater control over how the elements are
combined.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_c]{vec_c()}}
}
}

\examples{
# If `indices` selects every value in `x` exactly once,
# in any order, then `list_unchop()` inverts `vec_chop()`
x <- c("a", "b", "c", "d")
indices <- list(2, c(3, 1), 4)
vec_chop(x, indices = indices)
list_unchop(vec_chop(x, indices = indices), indices = indices)

# When unchopping, size 1 elements of `x` are recycled
# to the size of the corresponding index
list_unchop(list(1, 2:3), indices = list(c(1, 3, 5), c(2, 4)))

# Names are retained, and outer names can be combined with inner
# names through the use of a `name_spec`
lst <- list(x = c(a = 1, b = 2), y = 1)
list_unchop(lst, indices = list(c(3, 2), c(1, 4)), name_spec = "{outer}_{inner}")

# If you have a list of homogeneous vectors, sometimes it can be useful to
# unchop, apply a function to the flattened vector, and then rechop according
# to the original indices. This can be done efficiently with `list_sizes()`.
x <- list(c(1, 2, 1), c(3, 1), 5, double())
x_flat <- list_unchop(x)
x_flat <- x_flat + max(x_flat)
vec_chop(x_flat, sizes = list_sizes(x))
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rep.R
\name{vec-rep}
\alias{vec-rep}
\alias{vec_rep}
\alias{vec_rep_each}
\alias{vec_unrep}
\title{Repeat a vector}
\usage{
vec_rep(
  x,
  times,
  ...,
  error_call = current_env(),
  x_arg = "x",
  times_arg = "times"
)

vec_rep_each(
  x,
  times,
  ...,
  error_call = current_env(),
  x_arg = "x",
  times_arg = "times"
)

vec_unrep(x)
}
\arguments{
\item{x}{A vector.}

\item{times}{For \code{vec_rep()}, a single integer for the number of times to repeat
the entire vector.

For \code{vec_rep_each()}, an integer vector of the number of times to repeat
each element of \code{x}. \code{times} will be \link[=theory-faq-recycling]{recycled} to
the size of \code{x}.}

\item{...}{These dots are for future extensions and must be empty.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{x_arg, times_arg}{Argument names for errors.}
}
\value{
For \code{vec_rep()}, a vector the same type as \code{x} with size
\code{vec_size(x) * times}.

For \code{vec_rep_each()}, a vector the same type as \code{x} with size
\code{sum(vec_recycle(times, vec_size(x)))}.

For \code{vec_unrep()}, a data frame with two columns, \code{key} and \code{times}. \code{key}
is a vector with the same type as \code{x}, and \code{times} is an integer vector.
}
\description{
\itemize{
\item \code{vec_rep()} repeats an entire vector a set number of \code{times}.
\item \code{vec_rep_each()} repeats each element of a vector a set number of \code{times}.
\item \code{vec_unrep()} compresses a vector with repeated values. The repeated values
are returned as a \code{key} alongside the number of \code{times} each key is
repeated.
}
}
\details{
Using \code{vec_unrep()} and \code{vec_rep_each()} together is similar to using
\code{\link[base:rle]{base::rle()}} and \code{\link[base:rle]{base::inverse.rle()}}. The following invariant shows
the relationship between the two functions:

\if{html}{\out{<div class="sourceCode">}}\preformatted{compressed <- vec_unrep(x)
identical(x, vec_rep_each(compressed$key, compressed$times))
}\if{html}{\out{</div>}}

There are two main differences between \code{vec_unrep()} and \code{\link[base:rle]{base::rle()}}:
\itemize{
\item \code{vec_unrep()} treats adjacent missing values as equivalent, while \code{rle()}
treats them as different values.
\item \code{vec_unrep()} works along the size of \code{x}, while \code{rle()} works along its
length. This means that \code{vec_unrep()} works on data frames by compressing
repeated rows.
}
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
# Repeat the entire vector
vec_rep(1:2, 3)

# Repeat within each vector
vec_rep_each(1:2, 3)
x <- vec_rep_each(1:2, c(3, 4))
x

# After using `vec_rep_each()`, you can recover the original vector
# with `vec_unrep()`
vec_unrep(x)

df <- data.frame(x = 1:2, y = 3:4)

# `rep()` repeats columns of data frames, and returns lists
rep(df, each = 2)

# `vec_rep()` and `vec_rep_each()` repeat rows, and return data frames
vec_rep(df, 2)
vec_rep_each(df, 2)

# `rle()` treats adjacent missing values as different
y <- c(1, NA, NA, 2)
rle(y)

# `vec_unrep()` treats them as equivalent
vec_unrep(y)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order.R
\name{vec_order}
\alias{vec_order}
\alias{vec_sort}
\title{Order and sort vectors}
\usage{
vec_order(
  x,
  ...,
  direction = c("asc", "desc"),
  na_value = c("largest", "smallest")
)

vec_sort(
  x,
  ...,
  direction = c("asc", "desc"),
  na_value = c("largest", "smallest")
)
}
\arguments{
\item{x}{A vector}

\item{...}{These dots are for future extensions and must be empty.}

\item{direction}{Direction to sort in. Defaults to \code{asc}ending.}

\item{na_value}{Should \code{NA}s be treated as the largest or smallest values?}
}
\value{
\itemize{
\item \code{vec_order()} an integer vector the same size as \code{x}.
\item \code{vec_sort()} a vector with the same size and type as \code{x}.
}
}
\description{
Order and sort vectors
}
\section{Differences with \code{order()}}{

Unlike the \code{na.last} argument of \code{order()} which decides the
positions of missing values irrespective of the \code{decreasing}
argument, the \code{na_value} argument of \code{vec_order()} interacts with
\code{direction}. If missing values are considered the largest value,
they will appear last in ascending order, and first in descending
order.
}

\section{Dependencies of \code{vec_order()}}{

\itemize{
\item \code{\link[=vec_proxy_order]{vec_proxy_order()}}
}
}

\section{Dependencies of \code{vec_sort()}}{

\itemize{
\item \code{\link[=vec_proxy_order]{vec_proxy_order()}}
\item \code{\link[=vec_order]{vec_order()}}
\item \code{\link[=vec_slice]{vec_slice()}}
}
}

\examples{
x <- round(c(runif(9), NA), 3)
vec_order(x)
vec_sort(x)
vec_sort(x, direction = "desc")

# Can also handle data frames
df <- data.frame(g = sample(2, 10, replace = TRUE), x = x)
vec_order(df)
vec_sort(df)
vec_sort(df, direction = "desc")

# Missing values interpreted as largest values are last when
# in increasing order:
vec_order(c(1, NA), na_value = "largest", direction = "asc")
vec_order(c(1, NA), na_value = "largest", direction = "desc")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list-combine.R
\name{list_combine}
\alias{list_combine}
\title{Combine a list of vectors}
\usage{
list_combine(
  x,
  ...,
  indices,
  size,
  default = NULL,
  unmatched = "default",
  multiple = "last",
  slice_x = FALSE,
  ptype = NULL,
  name_spec = NULL,
  name_repair = c("minimal", "unique", "check_unique", "universal", "unique_quiet",
    "universal_quiet"),
  x_arg = "x",
  indices_arg = "indices",
  default_arg = "default",
  error_call = current_env()
)
}
\arguments{
\item{x}{A list of vectors.

If \code{slice_x = FALSE}, each element must be size 1 or the same size as its
corresponding index in \code{indices} after that index has been converted to
a positive integer location vector with \code{\link[=vec_as_location]{vec_as_location()}}.

If \code{slice_x = TRUE}, each element must be size 1 or size \code{size}.}

\item{...}{These dots are for future extensions and must be empty.}

\item{indices}{A list of indices.

Indices can be provided in one of two forms:
\itemize{
\item Positive integer vectors of locations less than or equal to \code{size}. Each
vector can be any size.
\item Logical vectors of size \code{size} where \code{TRUE} denotes the location in the
output to assign to, and the location from the \code{x} element to pull from.
Both \code{NA} and \code{FALSE} are considered unmatched.
}

The size of \code{indices} must match the size of \code{x}.}

\item{size}{The output size.}

\item{default}{If \code{NULL}, a missing value is used for locations unmatched by
\code{indices}, otherwise the provided \code{default} is used.

If provided, \code{default} must be size 1 or size \code{size}.

Can only be set when \code{unmatched = "default"}.}

\item{unmatched}{Handling of locations in the output unmatched by \code{indices}.
One of:
\itemize{
\item \code{"default"} to use \code{default} in unmatched locations.
\item \code{"error"} to error when there are unmatched locations.
}}

\item{multiple}{Handling of locations in the output matched by multiple
\code{indices}.
\itemize{
\item \code{"last"} uses the value from the last matched index.
\item \code{"first"} uses the value from the first matched index.
}

Note that \code{multiple} only applies across \code{indices}. Within a single index
if there are overlapping locations, then the last will always win. This can
only occur with integer \code{indices}, as you can't overlap within an index
when using logical \code{indices}.}

\item{slice_x}{A boolean.

If \code{TRUE}, each element of \code{x} is sliced by its corresponding index from
\code{indices} before being assigned into the output, which is effectively the
same as \code{map2(list(x, indices), function(x, index) vec_slice(x, index))},
but is optimized to avoid materializing the slices.

See the \code{slice_value} argument of \code{\link[=vec_assign]{vec_assign()}} for more examples.}

\item{ptype}{If \code{NULL}, the output type is determined by computing the common
type across all elements of \code{x} and \code{default}. Alternatively, you can
supply \code{ptype} to give the output a known type.}

\item{name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}

\item{name_repair}{How to repair names, see \code{repair} options in
\code{\link[=vec_as_names]{vec_as_names()}}.}

\item{x_arg, indices_arg, default_arg}{An argument name as a string. This
argument will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector of type \code{vec_ptype_common(!!!x)}, or \code{ptype}, if specified.

The size of the output is determined by \code{size}.
}
\description{
\code{list_combine()} is a more powerful version of \code{\link[=vec_c]{vec_c()}}. While \code{vec_c()} is
used for sequential combination, \code{list_combine()} takes a list of \code{indices}
that specify where to place each element in the output.

If you have a list of vectors and just need to combine them sequentially,
you'll still want to use \code{vec_c(!!!x)}.
}
\examples{
# Combine a list of vectors using
# a list of `indices`
x <- list(
  1:3,
  4:6,
  7:8
)
indices <- list(
  c(1, 3, 7),
  c(8, 6, 5),
  c(2, 4)
)
list_combine(x, indices = indices, size = 8)

# Overlapping `indices` are allowed.
# The last match "wins" by default.
x <- list(
  1:3,
  4:6
)
indices <- list(
  c(1, 2, 3),
  c(1, 2, 6)
)
list_combine(x, indices = indices, size = 6)

# Use `multiple` to force the first match to win.
# This is similar to how `dplyr::case_when()` works.
list_combine(x, indices = indices, size = 6, multiple = "first")

# Works with data frames as well.
# Now how index 2 is not assigned to.
x <- list(
  data.frame(x = 1:2, y = c("a", "b")),
  data.frame(x = 3:4, y = c("c", "d"))
)
indices <- list(
  c(4, 1),
  c(3, NA)
)
list_combine(x, indices = indices, size = 4)

# You can use `size` to combine into a larger object than you have values for
list_combine(list(1:2, 4:5), indices = list(1:2, 4:5), size = 8)

# Additionally specifying `default` allows you to control the value used in
# unfilled locations
list_combine(
  list(1:2, 4:5),
  indices = list(1:2, 4:5),
  size = 8,
  default = 0L
)

# Alternatively, if you'd like to assert that you've covered all output
# locations through `indices`, set `unmatched = "error"`.
# Here, we've set the size to 5 but missed location 3:
try(list_combine(
  list(1:2, 4:5),
  indices = list(1:2, 4:5),
  size = 5,
  unmatched = "error"
))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditions.R
\name{maybe_lossy_cast}
\alias{maybe_lossy_cast}
\title{Lossy cast error}
\usage{
maybe_lossy_cast(
  result,
  x,
  to,
  lossy = NULL,
  locations = NULL,
  ...,
  loss_type = c("precision", "generality"),
  x_arg,
  to_arg,
  call = caller_env(),
  details = NULL,
  message = NULL,
  class = NULL,
  .deprecation = FALSE
)
}
\arguments{
\item{result}{The result of a potentially lossy cast.}

\item{x}{Vectors to cast.}

\item{to}{Type to cast to.}

\item{lossy}{A logical vector indicating which elements of \code{result}
were lossy.

Can also be a single \code{TRUE}, but note that \code{locations} picks up
locations from this vector by default. In this case, supply your
own location vector, possibly empty.}

\item{locations}{An optional integer vector giving the
locations where \code{x} lost information.}

\item{..., class}{Only use these fields when creating a subclass.}

\item{loss_type}{The kind of lossy cast to be mentioned in error
messages. Can be loss of precision (for instance from double to
integer) or loss of generality (from character to factor).}

\item{x_arg}{Argument name for \code{x}, used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{to_arg}{Argument name \code{to} used in error messages to
inform the user about the locations of incompatible types
(see \code{\link[=stop_incompatible_type]{stop_incompatible_type()}}).}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{details}{Any additional human readable details.}

\item{message}{An overriding message for the error. \code{details} and
\code{message} are mutually exclusive, supplying both is an error.}

\item{.deprecation}{If \code{TRUE}, the error is downgraded to a
deprecation warning. This is useful for transitioning your class
to a stricter conversion scheme. The warning advises your users
to wrap their code with \code{allow_lossy_cast()}.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

By default, lossy casts are an error. Use \code{allow_lossy_cast()} to
silence these errors and continue with the partial results. In this
case the lost values are typically set to \code{NA} or to a lower value
resolution, depending on the type of cast.

Lossy cast errors are thrown by \code{maybe_lossy_cast()}. Unlike
functions prefixed with \code{stop_}, \code{maybe_lossy_cast()} usually
returns a result. If a lossy cast is detected, it throws an error,
unless it's been wrapped in \code{allow_lossy_cast()}. In that case, it
returns the result silently.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expand.R
\name{vec_expand_grid}
\alias{vec_expand_grid}
\title{Create a data frame from all combinations of the inputs}
\usage{
vec_expand_grid(
  ...,
  .vary = "slowest",
  .name_repair = "check_unique",
  .error_call = current_env()
)
}
\arguments{
\item{...}{Name-value pairs. The name will become the column name in the
resulting data frame.}

\item{.vary}{One of:
\itemize{
\item \code{"slowest"} to vary the first column slowest. This produces sorted
output and is generally the most useful.
\item \code{"fastest"} to vary the first column fastest. This matches the behavior
of \code{\link[=expand.grid]{expand.grid()}}.
}}

\item{.name_repair}{One of \code{"check_unique"}, \code{"unique"}, \code{"universal"},
\code{"minimal"}, \code{"unique_quiet"}, or \code{"universal_quiet"}. See \code{\link[=vec_as_names]{vec_as_names()}}
for the meaning of these options.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A data frame with as many columns as there are inputs in \code{...} and as many
rows as the \code{\link[=prod]{prod()}} of the sizes of the inputs.
}
\description{
\code{vec_expand_grid()} creates a new data frame by creating a grid of all
possible combinations of the input vectors. It is inspired by
\code{\link[=expand.grid]{expand.grid()}}. Compared with \code{expand.grid()}, it:
\itemize{
\item Produces sorted output by default by varying the first column the slowest,
rather than the fastest. Control this with \code{.vary}.
\item Never converts strings to factors.
\item Does not add additional attributes.
\item Drops \code{NULL} inputs.
\item Can expand any vector type, including data frames and \link[=new_rcrd]{records}.
}
}
\details{
If any input is empty (i.e. size 0), then the result will have 0 rows.

If no inputs are provided, the result is a 1 row data frame with 0 columns.
This is consistent with the fact that \code{prod()} with no inputs returns \code{1}.
}
\examples{
vec_expand_grid(x = 1:2, y = 1:3)

# Use `.vary` to match `expand.grid()`:
vec_expand_grid(x = 1:2, y = 1:3, .vary = "fastest")

# Can also expand data frames
vec_expand_grid(
  x = data_frame(a = 1:2, b = 3:4),
  y = 1:4
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subscript.R
\name{vec_as_subscript}
\alias{vec_as_subscript}
\alias{vec_as_subscript2}
\title{Convert to a base subscript type}
\usage{
vec_as_subscript(
  i,
  ...,
  logical = c("cast", "error"),
  numeric = c("cast", "error"),
  character = c("cast", "error"),
  arg = NULL,
  call = caller_env()
)

vec_as_subscript2(
  i,
  ...,
  numeric = c("cast", "error"),
  character = c("cast", "error"),
  arg = NULL,
  call = caller_env()
)
}
\arguments{
\item{i}{An index vector to convert.}

\item{...}{These dots are for future extensions and must be empty.}

\item{logical, numeric, character}{How to handle logical, numeric,
and character subscripts.

If \code{"cast"} and the subscript is not one of the three base types
(logical, integer or character), the subscript is
\link[=vec_cast]{cast} to the relevant base type, e.g. factors are
coerced to character. \code{NULL} is treated as an empty integer
vector, and is thus coercible depending on the setting of
\code{numeric}. Symbols are treated as character vectors and thus
coercible depending on the setting of \code{character}.

If \code{"error"}, the subscript type is disallowed and triggers an
informative error.}

\item{arg}{The argument name to be displayed in error messages.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

Convert \code{i} to the base type expected by \code{\link[=vec_as_location]{vec_as_location()}} or
\code{\link[=vec_as_location2]{vec_as_location2()}}. The values of the subscript type are
not checked in any way (length, missingness, negative elements).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/if-else.R
\name{vec_if_else}
\alias{vec_if_else}
\title{Vectorized if-else}
\usage{
vec_if_else(
  condition,
  true,
  false,
  ...,
  missing = NULL,
  ptype = NULL,
  condition_arg = "condition",
  true_arg = "true",
  false_arg = "false",
  missing_arg = "missing",
  error_call = current_env()
)
}
\arguments{
\item{condition}{A logical vector.}

\item{true, false}{Vectors to use for \code{TRUE} and \code{FALSE} values of
\code{condition}.

Both \code{true} and \code{false} will be \link[=theory-faq-recycling]{recycled}
to the size of \code{condition}.

\code{true}, \code{false}, and \code{missing} (if used) will be cast to their common type.}

\item{...}{These dots are for future extensions and must be empty.}

\item{missing}{If not \code{NULL}, will be used as the value for \code{NA} values of
\code{condition}. Follows the same size and type rules as \code{true} and \code{false}.}

\item{ptype}{An optional prototype declaring the desired output type. If
supplied, this overrides the common type of \code{true}, \code{false}, and \code{missing}.}

\item{condition_arg, true_arg, false_arg, missing_arg}{Argument names used in
error messages.}

\item{error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector with the same size as \code{condition} and the same type as the common
type of \code{true}, \code{false}, and \code{missing}.

Where \code{condition} is \code{TRUE}, the matching values from \code{true}, where it is
\code{FALSE}, the matching values from \code{false}, and where it is \code{NA}, the matching
values from \code{missing}, if provided, otherwise a missing value will be used.
}
\description{
\code{vec_if_else()} is a vectorized \link[=if]{if-else}. Compared to the base R
equivalent, \code{\link[=ifelse]{ifelse()}}, this function allows you to handle missing values in
the \code{condition} with \code{missing} and always takes \code{true}, \code{false}, and
\code{missing} into account when determining what the output type should be.
}
\examples{
x <- c(-5:5, NA)
vec_if_else(x < 0, NA, x)

# Explicitly handle `NA` values in the `condition` with `missing`
vec_if_else(x < 0, "negative", "positive", missing = "missing")

# Unlike `ifelse()`, `vec_if_else()` preserves types
x <- factor(sample(letters[1:5], 10, replace = TRUE))
ifelse(x \%in\% c("a", "b", "c"), x, NA)
vec_if_else(x \%in\% c("a", "b", "c"), x, NA)

# `vec_if_else()` also works with data frames
condition <- c(TRUE, FALSE, NA, TRUE)
true <- data_frame(x = 1:4, y = 5:8)
false <- data_frame(x = 9:12, y = 13:16)
vec_if_else(condition, true, false)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq.R
\name{faq-error-incompatible-attributes}
\alias{faq-error-incompatible-attributes}
\title{FAQ - Error/Warning: Some attributes are incompatible}
\description{
This error occurs when \code{\link[=vec_ptype2]{vec_ptype2()}} or \code{\link[=vec_cast]{vec_cast()}} are supplied
vectors of the same classes with different attributes. In this
case, vctrs doesn't know how to combine the inputs.

To fix this error, the maintainer of the class should implement
self-to-self coercion methods for \code{\link[=vec_ptype2]{vec_ptype2()}} and \code{\link[=vec_cast]{vec_cast()}}.
}
\section{Implementing coercion methods}{
\itemize{
\item For an overview of how these generics work and their roles in vctrs,
see \code{\link[=theory-faq-coercion]{?theory-faq-coercion}}.
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
\item For a tutorial about implementing vctrs classes from scratch, see
\code{vignette("s3-vector")}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{howto-faq-coercion}
\alias{howto-faq-coercion}
\title{FAQ - How to implement ptype2 and cast methods?}
\description{
This guide illustrates how to implement \code{vec_ptype2()} and \code{vec_cast()}
methods for existing classes. Related topics:
\itemize{
\item For an overview of how these generics work and their roles in vctrs,
see \code{\link[=theory-faq-coercion]{?theory-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
\item For a tutorial about implementing vctrs classes from scratch, see
\code{vignette("s3-vector")}
}
\subsection{The natural number class}{

We’ll illustrate how to implement coercion methods with a simple class
that represents natural numbers. In this scenario we have an existing
class that already features a constructor and methods for \code{print()} and
subset.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
new_natural <- function(x) \{
  if (is.numeric(x) || is.logical(x)) \{
    stopifnot(is_whole(x))
    x <- as.integer(x)
  \} else \{
    stop("Can't construct natural from unknown type.")
  \}
  structure(x, class = "my_natural")
\}
is_whole <- function(x) \{
  all(x \%\% 1 == 0 | is.na(x))
\}

#' @export
print.my_natural <- function(x, ...) \{
  cat("<natural>\n")
  x <- unclass(x)
  NextMethod()
\}
#' @export
`[.my_natural` <- function(x, i, ...) \{
  new_natural(NextMethod())
\}
}\if{html}{\out{</div>}}

\if{html}{\out{<div class="sourceCode r">}}\preformatted{new_natural(1:3)
#> <natural>
#> [1] 1 2 3
new_natural(c(1, NA))
#> <natural>
#> [1]  1 NA
}\if{html}{\out{</div>}}
}

\subsection{Roxygen workflow}{

To implement methods for generics, first import the generics in your
namespace and redocument:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @importFrom vctrs vec_ptype2 vec_cast
NULL
}\if{html}{\out{</div>}}

Note that for each batches of methods that you add to your package, you
need to export the methods and redocument immediately, even during
development. Otherwise they won’t be in scope when you run unit tests
e.g. with testthat.

Implementing double dispatch methods is very similar to implementing
regular S3 methods. In these examples we are using roxygen2 tags to
register the methods, but you can also register the methods manually in
your NAMESPACE file or lazily with \code{s3_register()}.
}

\subsection{Implementing \code{vec_ptype2()}}{
\subsection{The self-self method}{

The first method to implement is the one that signals that your class is
compatible with itself:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_natural.my_natural <- function(x, y, ...) \{
  x
\}

vec_ptype2(new_natural(1), new_natural(2:3))
#> <natural>
#> integer(0)
}\if{html}{\out{</div>}}

\code{vec_ptype2()} implements a fallback to try and be compatible with
simple classes, so it may seem that you don’t need to implement the
self-self coercion method. However, you must implement it explicitly
because this is how vctrs knows that a class that is implementing vctrs
methods (for instance this disable fallbacks to \code{base::c()}). Also, it
makes your class a bit more efficient.
}

\subsection{The parent and children methods}{

Our natural number class is conceptually a parent of \verb{<logical>} and a
child of \verb{<integer>}, but the class is not compatible with logical,
integer, or double vectors yet:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(TRUE, new_natural(2:3))
#> Error:
#> ! Can't combine `TRUE` <logical> and `new_natural(2:3)` <my_natural>.

vec_ptype2(new_natural(1), 2:3)
#> Error:
#> ! Can't combine `new_natural(1)` <my_natural> and `2:3` <integer>.
}\if{html}{\out{</div>}}

We’ll specify the twin methods for each of these classes, returning the
richer class in each case.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_natural.logical <- function(x, y, ...) \{
  # The order of the classes in the method name follows the order of
  # the arguments in the function signature, so `x` is the natural
  # number and `y` is the logical
  x
\}
#' @export
vec_ptype2.logical.my_natural <- function(x, y, ...) \{
  # In this case `y` is the richer natural number
  y
\}
}\if{html}{\out{</div>}}

Between a natural number and an integer, the latter is the richer class:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_natural.integer <- function(x, y, ...) \{
  y
\}
#' @export
vec_ptype2.integer.my_natural <- function(x, y, ...) \{
  x
\}
}\if{html}{\out{</div>}}

We no longer get common type errors for logical and integer:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(TRUE, new_natural(2:3))
#> <natural>
#> integer(0)

vec_ptype2(new_natural(1), 2:3)
#> integer(0)
}\if{html}{\out{</div>}}

We are not done yet. Pairwise coercion methods must be implemented for
all the connected nodes in the coercion hierarchy, which include double
vectors further up. The coercion methods for grand-parent types must be
implemented separately:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_natural.double <- function(x, y, ...) \{
  y
\}
#' @export
vec_ptype2.double.my_natural <- function(x, y, ...) \{
  x
\}
}\if{html}{\out{</div>}}
}

\subsection{Incompatible attributes}{

Most of the time, inputs are incompatible because they have different
classes for which no \code{vec_ptype2()} method is implemented. More rarely,
inputs could be incompatible because of their attributes. In that case
incompatibility is signalled by calling \code{stop_incompatible_type()}.

In the following example, we implement a self-self ptype2 method for a
hypothetical subclass of \verb{<factor>} that has stricter combination
semantics. The method throws an error when the levels of the two factors
are not compatible.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_strict_factor.my_strict_factor <- function(x, y, ..., x_arg = "", y_arg = "") \{
  if (!setequal(levels(x), levels(y))) \{
    stop_incompatible_type(x, y, x_arg = x_arg, y_arg = y_arg)
  \}

  x
\}
}\if{html}{\out{</div>}}

Note how the methods need to take \code{x_arg} and \code{y_arg} parameters and
pass them on to \code{stop_incompatible_type()}. These argument tags help
create more informative error messages when the common type
determination is for a column of a data frame. They are part of the
generic signature but can usually be left out if not used.
}

}

\subsection{Implementing \code{vec_cast()}}{

Corresponding \code{vec_cast()} methods must be implemented for all
\code{vec_ptype2()} methods. The general pattern is to convert the argument
\code{x} to the type of \code{to}. The methods should validate the values in \code{x}
and make sure they conform to the values of \code{to}.

Please note that for historical reasons, the order of the classes in the
method name is in reverse order of the arguments in the function
signature. The first class represents \code{to}, whereas the second class
represents \code{x}.

The self-self method is easy in this case, it just returns the target
input:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_cast.my_natural.my_natural <- function(x, to, ...) \{
  x
\}
}\if{html}{\out{</div>}}

The other types need to be validated. We perform input validation in the
\code{new_natural()} constructor, so that’s a good fit for our \code{vec_cast()}
implementations.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_cast.my_natural.logical <- function(x, to, ...) \{
  # The order of the classes in the method name is in reverse order
  # of the arguments in the function signature, so `to` is the natural
  # number and `x` is the logical
  new_natural(x)
\}
vec_cast.my_natural.integer <- function(x, to, ...) \{
  new_natural(x)
\}
vec_cast.my_natural.double <- function(x, to, ...) \{
  new_natural(x)
\}
}\if{html}{\out{</div>}}

With these methods, vctrs is now able to combine logical and natural
vectors. It properly returns the richer type of the two, a natural
vector:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_c(TRUE, new_natural(1), FALSE)
#> <natural>
#> [1] 1 1 0
}\if{html}{\out{</div>}}

Because we haven’t implemented conversions \emph{from} natural, it still
doesn’t know how to combine natural with the richer integer and double
types:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_c(new_natural(1), 10L)
#> Error in `vec_c()`:
#> ! Can't convert `..1` <my_natural> to <integer>.
vec_c(1.5, new_natural(1))
#> Error in `vec_c()`:
#> ! Can't convert `..2` <my_natural> to <double>.
}\if{html}{\out{</div>}}

This is quick work which completes the implementation of coercion
methods for vctrs:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_cast.logical.my_natural <- function(x, to, ...) \{
  # In this case `to` is the logical and `x` is the natural number
  attributes(x) <- NULL
  as.logical(x)
\}
#' @export
vec_cast.integer.my_natural <- function(x, to, ...) \{
  attributes(x) <- NULL
  as.integer(x)
\}
#' @export
vec_cast.double.my_natural <- function(x, to, ...) \{
  attributes(x) <- NULL
  as.double(x)
\}
}\if{html}{\out{</div>}}

And we now get the expected combinations.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_c(new_natural(1), 10L)
#> [1]  1 10

vec_c(1.5, new_natural(1))
#> [1] 1.5 1.0
}\if{html}{\out{</div>}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/size.R
\name{vec_size}
\alias{vec_size}
\alias{vec_size_common}
\alias{list_sizes}
\alias{vec_is_empty}
\title{Number of observations}
\usage{
vec_size(x)

vec_size_common(
  ...,
  .size = NULL,
  .absent = 0L,
  .arg = "",
  .call = caller_env()
)

list_sizes(x)

vec_is_empty(x)
}
\arguments{
\item{x, ...}{Vector inputs or \code{NULL}.}

\item{.size}{If \code{NULL}, the default, the output size is determined by
recycling the lengths of all elements of \code{...}. Alternatively, you can
supply \code{.size} to force a known size; in this case, \code{x} and \code{...} are
ignored.}

\item{.absent}{The size used when no input is provided, or when all input
is \code{NULL}. If left as \code{NULL} when no input is supplied, an error is thrown.}

\item{.arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{.call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
An integer (or double for long vectors).

\code{vec_size_common()} returns \code{.absent} if all inputs are \code{NULL} or
absent, \code{0L} by default.
}
\description{
\code{vec_size(x)} returns the size of a vector. \code{vec_is_empty()}
returns \code{TRUE} if the size is zero, \code{FALSE} otherwise.

The size is distinct from the \code{\link[=length]{length()}} of a vector because it
generalises to the "number of observations" for 2d structures,
i.e. it's the number of rows in matrix or a data frame.  This
definition has the important property that every column of a data
frame (even data frame and matrix columns) have the same size.
\code{vec_size_common(...)} returns the common size of multiple vectors.

\code{list_sizes()} returns an integer vector containing the size of each element
of a list. It is nearly equivalent to, but faster than,
\code{map_int(x, vec_size)}, with the exception that \code{list_sizes()} will
error on non-list inputs, as defined by \code{\link[=obj_is_list]{obj_is_list()}}. \code{list_sizes()} is
to \code{vec_size()} as \code{\link[=lengths]{lengths()}} is to \code{\link[=length]{length()}}.
}
\details{
There is no vctrs helper that retrieves the number of columns: as this
is a property of the \link[=vec_ptype_show]{type}.

\code{vec_size()} is equivalent to \code{NROW()} but has a name that is easier to
pronounce, and throws an error when passed non-vector inputs.
}
\section{Invariants}{

\itemize{
\item \code{vec_size(dataframe)} == \code{vec_size(dataframe[[i]])}
\item \code{vec_size(matrix)} == \code{vec_size(matrix[, i, drop = FALSE])}
\item \code{vec_size(vec_c(x, y))} == \code{vec_size(x)} + \code{vec_size(y)}
}
}

\section{The size of NULL}{


The size of \code{NULL} is hard-coded to \code{0L} in \code{vec_size()}.
\code{vec_size_common()} returns \code{.absent} when all inputs are \code{NULL}
(if only some inputs are \code{NULL}, they are simply ignored).

A default size of 0 makes sense because sizes are most often
queried in order to compute a total size while assembling a
collection of vectors. Since we treat \code{NULL} as an absent input by
principle, we return the identity of sizes under addition to
reflect that an absent input doesn't take up any size.

Note that other defaults might make sense under different
circumstances. For instance, a default size of 1 makes sense for
finding the common size because 1 is the identity of the recycling
rules.
}

\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy]{vec_proxy()}}
}
}

\examples{
vec_size(1:100)
vec_size(mtcars)
vec_size(array(dim = c(3, 5, 10)))

vec_size_common(1:10, 1:10)
vec_size_common(1:10, 1)
vec_size_common(integer(), 1)

list_sizes(list("a", 1:5, letters))
}
\seealso{
\code{\link[=vec_slice]{vec_slice()}} for a variation of \code{[} compatible with \code{vec_size()},
and \code{\link[=vec_recycle]{vec_recycle()}} to \link[=theory-faq-recycling]{recycle} vectors to common
length.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind.R
\name{vec_bind}
\alias{vec_bind}
\alias{vec_rbind}
\alias{vec_cbind}
\title{Combine many data frames into one data frame}
\usage{
vec_rbind(
  ...,
  .ptype = NULL,
  .names_to = rlang::zap(),
  .name_repair = c("unique", "universal", "check_unique", "unique_quiet",
    "universal_quiet"),
  .name_spec = NULL,
  .error_call = current_env()
)

vec_cbind(
  ...,
  .ptype = NULL,
  .size = NULL,
  .name_repair = c("unique", "universal", "check_unique", "minimal", "unique_quiet",
    "universal_quiet"),
  .error_call = current_env()
)
}
\arguments{
\item{...}{Data frames or vectors.

When the inputs are named:
\itemize{
\item \code{vec_rbind()} assigns names to row names unless \code{.names_to} is
supplied. In that case the names are assigned in the column
defined by \code{.names_to}.
\item \code{vec_cbind()} creates packed data frame columns with named
inputs.
}

\code{NULL} inputs are silently ignored. Empty (e.g. zero row) inputs
will not appear in the output, but will affect the derived \code{.ptype}.}

\item{.ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type across all elements of \code{...}.

Alternatively, you can supply \code{.ptype} to give the output known type.
If \code{getOption("vctrs.no_guessing")} is \code{TRUE} you must supply this value:
this is a convenient way to make production code demand fixed types.}

\item{.names_to}{This controls what to do with names on \code{...}:
\itemize{
\item By default, names on \code{...} are \link[rlang:zap]{zapped} and do not appear
anywhere in the output.
\item If a string, specifies a column where the names on \code{...} will be
copied. These names are often useful to identify rows with
their original input. If a column name is supplied and \code{...} is
not named, an integer column is used instead.
\item If \code{NULL}, the outer names on \code{...} are instead merged with inner
row names on each element of \code{...} and are subject to \code{.name_spec}.
}}

\item{.name_repair}{One of \code{"unique"}, \code{"universal"}, \code{"check_unique"},
\code{"unique_quiet"}, or  \code{"universal_quiet"}. See \code{\link[=vec_as_names]{vec_as_names()}} for the
meaning of these options.

With \code{vec_rbind()}, the repair function is applied to all inputs
separately. This is because \code{vec_rbind()} needs to align their
columns before binding the rows, and thus needs all inputs to
have unique names. On the other hand, \code{vec_cbind()} applies the
repair function after all inputs have been concatenated together
in a final data frame. Hence \code{vec_cbind()} allows the more
permissive minimal names repair.}

\item{.name_spec}{A name specification (as documented in \code{\link[=vec_c]{vec_c()}}) for
combining the outer names on \code{...} with the inner row names of each element
of \code{...}. An outer name will only ever be provided when \code{.names_to} is set
to \code{NULL}, which causes the outer name to be used as part of the row names
rather than as a new column, but it can still be useful to hardcode this to
either \code{\link[rlang:zap]{rlang::zap()}} to always ignore all names, or \code{"inner"} to always
ignore outer names, regardless of \code{.names_to}.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{.size}{If, \code{NULL}, the default, will determine the number of rows in
\code{vec_cbind()} output by using the tidyverse \link[=theory-faq-recycling]{recycling rules}.

Alternatively, specify the desired number of rows, and any inputs of length
1 will be recycled appropriately.}
}
\value{
A data frame, or subclass of data frame.

If \code{...} is a mix of different data frame subclasses, \code{vec_ptype2()}
will be used to determine the output type. For \code{vec_rbind()}, this
will determine the type of the container and the type of each column;
for \code{vec_cbind()} it only determines the type of the output container.
If there are no non-\code{NULL} inputs, the result will be \code{data.frame()}.
}
\description{
This pair of functions binds together data frames (and vectors), either
row-wise or column-wise. Row-binding creates a data frame with common type
across all arguments. Column-binding creates a data frame with common length
across all arguments.
}
\section{Invariants}{


All inputs are first converted to a data frame. The conversion for
1d vectors depends on the direction of binding:
\itemize{
\item For \code{vec_rbind()}, each element of the vector becomes a column in
a single row.
\item For \code{vec_cbind()}, each element of the vector becomes a row in a
single column.
}

Once the inputs have all become data frames, the following
invariants are observed for row-binding:
\itemize{
\item \code{vec_size(vec_rbind(x, y)) == vec_size(x) + vec_size(y)}
\item \code{vec_ptype(vec_rbind(x, y)) = vec_ptype_common(x, y)}
}

Note that if an input is an empty vector, it is first converted to
a 1-row data frame with 0 columns. Despite being empty, its
effective size for the total number of rows is 1.

For column-binding, the following invariants apply:
\itemize{
\item \code{vec_size(vec_cbind(x, y)) == vec_size_common(x, y)}
\item \code{vec_ptype(vec_cbind(x, y)) == vec_cbind(vec_ptype(x), vec_ptype(x))}
}
}

\section{Dependencies}{

\subsection{vctrs dependencies}{
\itemize{
\item \code{\link[=vec_cast_common]{vec_cast_common()}}
\item \code{\link[=vec_proxy]{vec_proxy()}}
\item \code{\link[=vec_init]{vec_init()}}
\item \code{\link[=vec_assign]{vec_assign()}}
\item \code{\link[=vec_restore]{vec_restore()}}
}
}

\subsection{base dependencies of \code{vec_rbind()}}{
\itemize{
\item \code{\link[base:c]{base::c()}}
}

If columns to combine inherit from a common class,
\code{vec_rbind()} falls back to \code{base::c()} if there exists a \code{c()}
method implemented for this class hierarchy.
}
}

\examples{
# row binding -----------------------------------------

# common columns are coerced to common class
vec_rbind(
  data.frame(x = 1),
  data.frame(x = FALSE)
)

# unique columns are filled with NAs
vec_rbind(
  data.frame(x = 1),
  data.frame(y = "x")
)

# null inputs are ignored
vec_rbind(
  data.frame(x = 1),
  NULL,
  data.frame(x = 2)
)

# bare vectors are treated as rows
vec_rbind(
  c(x = 1, y = 2),
  c(x = 3)
)

# default names will be supplied if arguments are not named
vec_rbind(
  1:2,
  1:3,
  1:4
)

# column binding --------------------------------------

# each input is recycled to have common length
vec_cbind(
  data.frame(x = 1),
  data.frame(y = 1:3)
)

# bare vectors are treated as columns
vec_cbind(
  data.frame(x = 1),
  y = letters[1:3]
)

# if you supply a named data frame, it is packed in a single column
data <- vec_cbind(
  x = data.frame(a = 1, b = 2),
  y = 1
)
data

# Packed data frames are nested in a single column. This makes it
# possible to access it through a single name:
data$x

# since the base print method is suboptimal with packed data
# frames, it is recommended to use tibble to work with these:
if (rlang::is_installed("tibble")) {
  vec_cbind(x = tibble::tibble(a = 1, b = 2), y = 1)
}

# duplicate names are flagged
vec_cbind(x = 1, x = 2)

}
\seealso{
\code{\link[=vec_c]{vec_c()}} for combining 1d vectors.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split.R
\name{vec_split}
\alias{vec_split}
\title{Split a vector into groups}
\usage{
vec_split(x, by)
}
\arguments{
\item{x}{Vector to divide into groups.}

\item{by}{Vector whose unique values defines the groups.}
}
\value{
A data frame with two columns and size equal to
\code{vec_size(vec_unique(by))}. The \code{key} column has the same type as
\code{by}, and the \code{val} column is a list containing elements of type
\code{vec_ptype(x)}.

Note for complex types, the default \code{data.frame} print method will be
suboptimal, and you will want to coerce into a tibble to better
understand the output.
}
\description{
This is a generalisation of \code{\link[=split]{split()}} that can split by any type of vector,
not just factors. Instead of returning the keys in the character names,
the are returned in a separate parallel vector.
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_group_loc]{vec_group_loc()}}
\item \code{\link[=vec_chop]{vec_chop()}}
}
}

\examples{
vec_split(mtcars$cyl, mtcars$vs)
vec_split(mtcars$cyl, mtcars[c("vs", "am")])

if (require("tibble")) {
  as_tibble(vec_split(mtcars$cyl, mtcars[c("vs", "am")]))
  as_tibble(vec_split(mtcars, mtcars[c("vs", "am")]))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{howto-faq-coercion-data-frame}
\alias{howto-faq-coercion-data-frame}
\title{FAQ - How to implement ptype2 and cast methods? (Data frames)}
\description{
This guide provides a practical recipe for implementing \code{vec_ptype2()}
and \code{vec_cast()} methods for coercions of data frame subclasses. Related
topics:
\itemize{
\item For an overview of the coercion mechanism in vctrs, see
\code{\link[=theory-faq-coercion]{?theory-faq-coercion}}.
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
}

Coercion of data frames occurs when different data frame classes are
combined in some way. The two main methods of combination are currently
row-binding with \code{\link[=vec_rbind]{vec_rbind()}} and col-binding with
\code{\link[=vec_cbind]{vec_cbind()}} (which are in turn used by a number of
dplyr and tidyr functions). These functions take multiple data frame
inputs and automatically coerce them to their common type.

vctrs is generally strict about the kind of automatic coercions that are
performed when combining inputs. In the case of data frames we have
decided to be a bit less strict for convenience. Instead of throwing an
incompatible type error, we fall back to a base data frame or a tibble
if we don’t know how to combine two data frame subclasses. It is still a
good idea to specify the proper coercion behaviour for your data frame
subclasses as soon as possible.

We will see two examples in this guide. The first example is about a
data frame subclass that has no particular attributes to manage. In the
second example, we implement coercion methods for a tibble subclass that
includes potentially incompatible attributes.
\subsection{Roxygen workflow}{

To implement methods for generics, first import the generics in your
namespace and redocument:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @importFrom vctrs vec_ptype2 vec_cast
NULL
}\if{html}{\out{</div>}}

Note that for each batches of methods that you add to your package, you
need to export the methods and redocument immediately, even during
development. Otherwise they won’t be in scope when you run unit tests
e.g. with testthat.

Implementing double dispatch methods is very similar to implementing
regular S3 methods. In these examples we are using roxygen2 tags to
register the methods, but you can also register the methods manually in
your NAMESPACE file or lazily with \code{s3_register()}.
}

\subsection{Parent methods}{

Most of the common type determination should be performed by the parent
class. In vctrs, double dispatch is implemented in such a way that you
need to call the methods for the parent class manually. For
\code{vec_ptype2()} this means you need to call \code{df_ptype2()} (for data frame
subclasses) or \code{tib_ptype2()} (for tibble subclasses). Similarly,
\code{df_cast()} and \code{tib_cast()} are the workhorses for \code{vec_cast()} methods
of subtypes of \code{data.frame} and \code{tbl_df}. These functions take the union
of the columns in \code{x} and \code{y}, and ensure shared columns have the same
type.

These functions are much less strict than \code{vec_ptype2()} and
\code{vec_cast()} as they accept any subclass of data frame as input. They
always return a \code{data.frame} or a \code{tbl_df}. You will probably want to
write similar functions for your subclass to avoid repetition in your
code. You may want to export them as well if you are expecting other
people to derive from your class.
}

\subsection{A \code{data.table} example}{

This example is the actual implementation of vctrs coercion methods for
\code{data.table}. This is a simple example because we don’t have to keep
track of attributes for this class or manage incompatibilities. See the
tibble section for a more complicated example.

We first create the \code{dt_ptype2()} and \code{dt_cast()} helpers. They wrap
around the parent methods \code{df_ptype2()} and \code{df_cast()}, and transform
the common type or converted input to a data table. You may want to
export these helpers if you expect other packages to derive from your
data frame class.

These helpers should always return data tables. To this end we use the
conversion generic \code{as.data.table()}. Depending on the tools available
for the particular class at hand, a constructor might be appropriate as
well.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{dt_ptype2 <- function(x, y, ...) \{
  as.data.table(df_ptype2(x, y, ...))
\}
dt_cast <- function(x, to, ...) \{
  as.data.table(df_cast(x, to, ...))
\}
}\if{html}{\out{</div>}}

We start with the self-self method:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.data.table.data.table <- function(x, y, ...) \{
  dt_ptype2(x, y, ...)
\}
}\if{html}{\out{</div>}}

Between a data frame and a data table, we consider the richer type to be
data table. This decision is not based on the value coverage of each
data structures, but on the idea that data tables have richer behaviour.
Since data tables are the richer type, we call \code{dt_type2()} from the
\code{vec_ptype2()} method. It always returns a data table, no matter the
order of arguments:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.data.table.data.frame <- function(x, y, ...) \{
  dt_ptype2(x, y, ...)
\}
#' @export
vec_ptype2.data.frame.data.table <- function(x, y, ...) \{
  dt_ptype2(x, y, ...)
\}
}\if{html}{\out{</div>}}

The \code{vec_cast()} methods follow the same pattern, but note how the
method for coercing to data frame uses \code{df_cast()} rather than
\code{dt_cast()}.

Also, please note that for historical reasons, the order of the classes
in the method name is in reverse order of the arguments in the function
signature. The first class represents \code{to}, whereas the second class
represents \code{x}.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_cast.data.table.data.table <- function(x, to, ...) \{
  dt_cast(x, to, ...)
\}
#' @export
vec_cast.data.table.data.frame <- function(x, to, ...) \{
  # `x` is a data.frame to be converted to a data.table
  dt_cast(x, to, ...)
\}
#' @export
vec_cast.data.frame.data.table <- function(x, to, ...) \{
  # `x` is a data.table to be converted to a data.frame
  df_cast(x, to, ...)
\}
}\if{html}{\out{</div>}}

With these methods vctrs is now able to combine data tables with data
frames:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_cbind(data.frame(x = 1:3), data.table(y = "foo"))
#>        x      y
#>    <int> <char>
#> 1:     1    foo
#> 2:     2    foo
#> 3:     3    foo
}\if{html}{\out{</div>}}
}

\subsection{A tibble example}{

In this example we implement coercion methods for a tibble subclass that
carries a colour as a scalar metadata:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# User constructor
my_tibble <- function(colour = NULL, ...) \{
  new_my_tibble(tibble::tibble(...), colour = colour)
\}
# Developer constructor
new_my_tibble <- function(x, colour = NULL) \{
  stopifnot(is.data.frame(x))
  tibble::new_tibble(
    x,
    colour = colour,
    class = "my_tibble",
    nrow = nrow(x)
  )
\}

df_colour <- function(x) \{
  if (inherits(x, "my_tibble")) \{
    attr(x, "colour")
  \} else \{
    NULL
  \}
\}

#'@export
print.my_tibble <- function(x, ...) \{
  cat(sprintf("<\%s: \%s>\n", class(x)[[1]], df_colour(x)))
  cli::cat_line(format(x)[-1])
\}
}\if{html}{\out{</div>}}

This subclass is very simple. All it does is modify the header.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{red <- my_tibble("red", x = 1, y = 1:2)
red
#> <my_tibble: red>
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2

red[2]
#> <my_tibble: red>
#>       y
#>   <int>
#> 1     1
#> 2     2

green <- my_tibble("green", z = TRUE)
green
#> <my_tibble: green>
#>   z    
#>   <lgl>
#> 1 TRUE
}\if{html}{\out{</div>}}

Combinations do not work properly out of the box, instead vctrs falls
back to a bare tibble:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_rbind(red, tibble::tibble(x = 10:12))
#> # A tibble: 5 x 2
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3    10    NA
#> 4    11    NA
#> 5    12    NA
}\if{html}{\out{</div>}}

Instead of falling back to a data frame, we would like to return a
\verb{<my_tibble>} when combined with a data frame or a tibble. Because this
subclass has more metadata than normal data frames (it has a colour), it
is a \emph{supertype} of tibble and data frame, i.e. it is the richer type.
This is similar to how a grouped tibble is a more general type than a
tibble or a data frame. Conceptually, the latter are pinned to a single
constant group.

The coercion methods for data frames operate in two steps:
\itemize{
\item They check for compatible subclass attributes. In our case the tibble
colour has to be the same, or be undefined.
\item They call their parent methods, in this case
\code{\link[=tib_ptype2]{tib_ptype2()}} and \code{\link[=tib_cast]{tib_cast()}} because
we have a subclass of tibble. This eventually calls the data frame
methods \code{\link[=df_ptype2]{df_ptype2()}} and
\code{\link[=tib_ptype2]{tib_ptype2()}} which match the columns and their
types.
}

This process should usually be wrapped in two functions to avoid
repetition. Consider exporting these if you expect your class to be
derived by other subclasses.

We first implement a helper to determine if two data frames have
compatible colours. We use the \code{df_colour()} accessor which returns
\code{NULL} when the data frame colour is undefined.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{has_compatible_colours <- function(x, y) \{
  x_colour <- df_colour(x) \%||\% df_colour(y)
  y_colour <- df_colour(y) \%||\% x_colour
  identical(x_colour, y_colour)
\}
}\if{html}{\out{</div>}}

Next we implement the coercion helpers. If the colours are not
compatible, we call \code{stop_incompatible_cast()} or
\code{stop_incompatible_type()}. These strict coercion semantics are
justified because in this class colour is a \emph{data} attribute. If it were
a non essential \emph{detail} attribute, like the timezone in a datetime, we
would just standardise it to the value of the left-hand side.

In simpler cases (like the data.table example), these methods do not
need to take the arguments suffixed in \verb{_arg}. Here we do need to take
these arguments so we can pass them to the \code{stop_} functions when we
detect an incompatibility. They also should be passed to the parent
methods.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
my_tib_cast <- function(x, to, ..., x_arg = "", to_arg = "") \{
  out <- tib_cast(x, to, ..., x_arg = x_arg, to_arg = to_arg)

  if (!has_compatible_colours(x, to)) \{
    stop_incompatible_cast(
      x,
      to,
      x_arg = x_arg,
      to_arg = to_arg,
      details = "Can't combine colours."
    )
  \}

  colour <- df_colour(x) \%||\% df_colour(to)
  new_my_tibble(out, colour = colour)
\}
#' @export
my_tib_ptype2 <- function(x, y, ..., x_arg = "", y_arg = "") \{
  out <- tib_ptype2(x, y, ..., x_arg = x_arg, y_arg = y_arg)

  if (!has_compatible_colours(x, y)) \{
    stop_incompatible_type(
      x,
      y,
      x_arg = x_arg,
      y_arg = y_arg,
      details = "Can't combine colours."
    )
  \}

  colour <- df_colour(x) \%||\% df_colour(y)
  new_my_tibble(out, colour = colour)
\}
}\if{html}{\out{</div>}}

Let’s now implement the coercion methods, starting with the self-self
methods.

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_tibble.my_tibble <- function(x, y, ...) \{
  my_tib_ptype2(x, y, ...)
\}
#' @export
vec_cast.my_tibble.my_tibble <- function(x, to, ...) \{
  my_tib_cast(x, to, ...)
\}
}\if{html}{\out{</div>}}

We can now combine compatible instances of our class!

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_rbind(red, red)
#> <my_tibble: red>
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3     1     1
#> 4     1     2

vec_rbind(green, green)
#> <my_tibble: green>
#>   z    
#>   <lgl>
#> 1 TRUE 
#> 2 TRUE

vec_rbind(green, red)
#> Error in `my_tib_ptype2()`:
#> ! Can't combine `..1` <my_tibble> and `..2` <my_tibble>.
#> Can't combine colours.
}\if{html}{\out{</div>}}

The methods for combining our class with tibbles follow the same
pattern. For ptype2 we return our class in both cases because it is the
richer type:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_tibble.tbl_df <- function(x, y, ...) \{
  my_tib_ptype2(x, y, ...)
\}
#' @export
vec_ptype2.tbl_df.my_tibble <- function(x, y, ...) \{
  my_tib_ptype2(x, y, ...)
\}
}\if{html}{\out{</div>}}

For cast are careful about returning a tibble when casting to a tibble.
Note the call to \code{vctrs::tib_cast()}:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_cast.my_tibble.tbl_df <- function(x, to, ...) \{
  my_tib_cast(x, to, ...)
\}
#' @export
vec_cast.tbl_df.my_tibble <- function(x, to, ...) \{
  tib_cast(x, to, ...)
\}
}\if{html}{\out{</div>}}

From this point, we get correct combinations with tibbles:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_rbind(red, tibble::tibble(x = 10:12))
#> <my_tibble: red>
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3    10    NA
#> 4    11    NA
#> 5    12    NA
}\if{html}{\out{</div>}}

However we are not done yet. Because the coercion hierarchy is different
from the class hierarchy, there is no inheritance of coercion methods.
We’re not getting correct behaviour for data frames yet because we
haven’t explicitly specified the methods for this class:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_rbind(red, data.frame(x = 10:12))
#> # A tibble: 5 x 2
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3    10    NA
#> 4    11    NA
#> 5    12    NA
}\if{html}{\out{</div>}}

Let’s finish up the boiler plate:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{#' @export
vec_ptype2.my_tibble.data.frame <- function(x, y, ...) \{
  my_tib_ptype2(x, y, ...)
\}
#' @export
vec_ptype2.data.frame.my_tibble <- function(x, y, ...) \{
  my_tib_ptype2(x, y, ...)
\}

#' @export
vec_cast.my_tibble.data.frame <- function(x, to, ...) \{
  my_tib_cast(x, to, ...)
\}
#' @export
vec_cast.data.frame.my_tibble <- function(x, to, ...) \{
  df_cast(x, to, ...)
\}
}\if{html}{\out{</div>}}

This completes the implementation:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_rbind(red, data.frame(x = 10:12))
#> <my_tibble: red>
#>       x     y
#>   <dbl> <int>
#> 1     1     1
#> 2     1     2
#> 3    10    NA
#> 4    11    NA
#> 5    12    NA
}\if{html}{\out{</div>}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/order.R
\name{vec_locate_sorted_groups}
\alias{vec_locate_sorted_groups}
\title{Locate sorted groups}
\usage{
vec_locate_sorted_groups(
  x,
  ...,
  direction = "asc",
  na_value = "largest",
  nan_distinct = FALSE,
  chr_proxy_collate = NULL
)
}
\arguments{
\item{x}{A vector}

\item{...}{These dots are for future extensions and must be empty.}

\item{direction}{Direction to sort in.
\itemize{
\item A single \code{"asc"} or \code{"desc"} for ascending or descending order
respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"asc"} or \code{"desc"}, specifying the direction for each column.
}}

\item{na_value}{Ordering of missing values.
\itemize{
\item A single \code{"largest"} or \code{"smallest"} for ordering missing values as the
largest or smallest values respectively.
\item For data frames, a length \code{1} or \code{ncol(x)} character vector containing
only \code{"largest"} or \code{"smallest"}, specifying how missing values should
be ordered within each column.
}}

\item{nan_distinct}{A single logical specifying whether or not \code{NaN} should
be considered distinct from \code{NA} for double and complex vectors. If \code{TRUE},
\code{NaN} will always be ordered between \code{NA} and non-missing numbers.}

\item{chr_proxy_collate}{A function generating an alternate representation
of character vectors to use for collation, often used for locale-aware
ordering.
\itemize{
\item If \code{NULL}, no transformation is done.
\item Otherwise, this must be a function of one argument. If the input contains
a character vector, it will be passed to this function after it has been
translated to UTF-8. This function should return a character vector with
the same length as the input. The result should sort as expected in the
C-locale, regardless of encoding.
}

For data frames, \code{chr_proxy_collate} will be applied to all character
columns.

Common transformation functions include: \code{tolower()} for case-insensitive
ordering and \code{stringi::stri_sort_key()} for locale-aware ordering.}
}
\value{
A two column data frame with size equal to \code{vec_size(vec_unique(x))}.
\itemize{
\item A \code{key} column of type \code{vec_ptype(x)}.
\item A \code{loc} column of type list, with elements of type integer.
}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

\code{vec_locate_sorted_groups()} returns a data frame containing a \code{key} column
with sorted unique groups, and a \code{loc} column with the locations of each
group in \code{x}. It is similar to \code{\link[=vec_group_loc]{vec_group_loc()}}, except the groups are
returned sorted rather than by first appearance.
}
\details{
\code{vec_locate_sorted_groups(x)} is equivalent to, but faster than:

\if{html}{\out{<div class="sourceCode">}}\preformatted{info <- vec_group_loc(x)
vec_slice(info, vec_order(info$key))
}\if{html}{\out{</div>}}
}
\section{Dependencies of \code{vec_locate_sorted_groups()}}{

\itemize{
\item \code{\link[=vec_proxy_order]{vec_proxy_order()}}
}
}

\examples{
df <- data.frame(
  g = sample(2, 10, replace = TRUE),
  x = c(NA, sample(5, 9, replace = TRUE))
)

# `vec_locate_sorted_groups()` is similar to `vec_group_loc()`, except keys
# are returned ordered rather than by first appearance.
vec_locate_sorted_groups(df)

vec_group_loc(df)
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-integer64.R
\name{vec_ptype_full.integer64}
\alias{vec_ptype_full.integer64}
\alias{vec_ptype_abbr.integer64}
\alias{vec_ptype2.integer64}
\alias{vec_cast.integer64}
\title{64 bit integers}
\usage{
\method{vec_ptype_full}{integer64}(x, ...)

\method{vec_ptype_abbr}{integer64}(x, ...)

\method{vec_ptype2}{integer64}(x, y, ...)

\method{vec_cast}{integer64}(x, to, ...)
}
\description{
A \code{integer64} is a 64 bits integer vector, implemented in the \code{bit64} package.
}
\details{
These functions help the \code{integer64} class from \code{bit64} in to
the vctrs type system by providing coercion functions
and casting functions.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/conditions.R
\name{vctrs-conditions}
\alias{vctrs-conditions}
\alias{stop_incompatible_type}
\alias{stop_incompatible_cast}
\alias{stop_incompatible_op}
\alias{stop_incompatible_size}
\alias{allow_lossy_cast}
\title{Custom conditions for vctrs package}
\usage{
stop_incompatible_type(
  x,
  y,
  ...,
  x_arg,
  y_arg,
  action = c("combine", "convert"),
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
)

stop_incompatible_cast(
  x,
  to,
  ...,
  x_arg,
  to_arg,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
)

stop_incompatible_op(
  op,
  x,
  y,
  details = NULL,
  ...,
  message = NULL,
  class = NULL,
  call = caller_env()
)

stop_incompatible_size(
  x,
  y,
  x_size,
  y_size,
  ...,
  x_arg,
  y_arg,
  details = NULL,
  message = NULL,
  class = NULL,
  call = caller_env()
)

allow_lossy_cast(expr, x_ptype = NULL, to_ptype = NULL)
}
\arguments{
\item{x, y, to}{Vectors}

\item{..., class}{Only use these fields when creating a subclass.}

\item{x_arg, y_arg, to_arg}{Argument names for \code{x}, \code{y}, and \code{to}. Used in
error messages to inform the user about the locations of incompatible
types.}

\item{action}{An option to customize the incompatible type message depending
on the context. Errors thrown from \code{\link[=vec_ptype2]{vec_ptype2()}} use \code{"combine"} and
those thrown from \code{\link[=vec_cast]{vec_cast()}} use \code{"convert"}.}

\item{details}{Any additional human readable details.}

\item{message}{An overriding message for the error. \code{details} and
\code{message} are mutually exclusive, supplying both is an error.}

\item{call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}

\item{x_ptype, to_ptype}{Suppress only the casting errors where \code{x}
or \code{to} match these \link[=vec_ptype]{prototypes}.}
}
\value{
\verb{stop_incompatible_*()} unconditionally raise an error of class
\code{"vctrs_error_incompatible_*"} and \code{"vctrs_error_incompatible"}.
}
\description{
These functions are called for their side effect of raising
errors and warnings.
These conditions have custom classes and structures to make
testing easier.
}
\examples{

# Most of the time, `maybe_lossy_cast()` returns its input normally:
maybe_lossy_cast(
  c("foo", "bar"),
  NA,
  "",
  lossy = c(FALSE, FALSE),
  x_arg = "",
  to_arg = ""
)

# If `lossy` has any `TRUE`, an error is thrown:
try(maybe_lossy_cast(
  c("foo", "bar"),
  NA,
  "",
  lossy = c(FALSE, TRUE),
  x_arg = "",
  to_arg = ""
))

# Unless lossy casts are allowed:
allow_lossy_cast(
  maybe_lossy_cast(
    c("foo", "bar"),
    NA,
    "",
    lossy = c(FALSE, TRUE),
    x_arg = "",
    to_arg = ""
  )
)

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dictionary.R
\name{vec_unique}
\alias{vec_unique}
\alias{vec_unique_loc}
\alias{vec_unique_count}
\title{Find and count unique values}
\usage{
vec_unique(x)

vec_unique_loc(x)

vec_unique_count(x)
}
\arguments{
\item{x}{A vector (including a data frame).}
}
\value{
\itemize{
\item \code{vec_unique()}: a vector the same type as \code{x} containing only unique
values.
\item \code{vec_unique_loc()}: an integer vector, giving locations of unique values.
\item \code{vec_unique_count()}: an integer vector of length 1, giving the
number of unique values.
}
}
\description{
\itemize{
\item \code{vec_unique()}: the unique values. Equivalent to \code{\link[=unique]{unique()}}.
\item \code{vec_unique_loc()}: the locations of the unique values.
\item \code{vec_unique_count()}: the number of unique values.
}
}
\section{Dependencies}{

\itemize{
\item \code{\link[=vec_proxy_equal]{vec_proxy_equal()}}
}
}

\section{Missing values}{

In most cases, missing values are not considered to be equal, i.e.
\code{NA == NA} is not \code{TRUE}. This behaviour would be unappealing here,
so these functions consider all \code{NAs} to be equal. (Similarly,
all \code{NaN} are also considered to be equal.)
}

\examples{
x <- rpois(100, 8)
vec_unique(x)
vec_unique_loc(x)
vec_unique_count(x)

# `vec_unique()` returns values in the order that encounters them
# use sort = "location" to match to the result of `vec_count()`
head(vec_unique(x))
head(vec_count(x, sort = "location"))

# Normally missing values are not considered to be equal
NA == NA

# But they are for the purposes of considering uniqueness
vec_unique(c(NA, NA, NA, NA, 1, 2, 1))
}
\seealso{
\link{vec_duplicate} for functions that work with the dual of
unique values: duplicated values.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bind.R
\name{vec_cbind_frame_ptype}
\alias{vec_cbind_frame_ptype}
\title{Frame prototype}
\usage{
vec_cbind_frame_ptype(x, ...)
}
\arguments{
\item{x}{A data frame.}

\item{...}{These dots are for future extensions and must be empty.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}

This is an experimental generic that returns zero-columns variants
of a data frame. It is needed for \code{\link[=vec_cbind]{vec_cbind()}}, to work around the
lack of colwise primitives in vctrs. Expect changes.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/c.R
\name{vec_c}
\alias{vec_c}
\title{Combine many vectors into one vector}
\usage{
vec_c(
  ...,
  .ptype = NULL,
  .name_spec = NULL,
  .name_repair = c("minimal", "unique", "check_unique", "universal", "unique_quiet",
    "universal_quiet"),
  .error_arg = "",
  .error_call = current_env()
)
}
\arguments{
\item{...}{Vectors to coerce.}

\item{.ptype}{If \code{NULL}, the default, the output type is determined by
computing the common type across all elements of \code{...}.

Alternatively, you can supply \code{.ptype} to give the output known type.
If \code{getOption("vctrs.no_guessing")} is \code{TRUE} you must supply this value:
this is a convenient way to make production code demand fixed types.}

\item{.name_spec}{A name specification for combining
inner and outer names. This is relevant for inputs passed with a
name, when these inputs are themselves named, like \code{outer = c(inner = 1)}, or when they have length greater than 1: \code{outer = 1:2}. By default, these cases trigger an error. You can resolve
the error by providing a specification that describes how to
combine the names or the indices of the inner vector with the
name of the input. This specification can be:
\itemize{
\item A function of two arguments. The outer name is passed as a
string to the first argument, and the inner names or positions
are passed as second argument.
\item An anonymous function as a purrr-style formula.
\item A glue specification of the form \code{"{outer}_{inner}"}.
\item \code{"inner"}, in which case outer names are ignored, and inner
names are used if they exist. Note that outer names may still
be used to provide informative error messages.
\item An \code{\link[rlang:zap]{rlang::zap()}} object, in which case both outer and inner
names are ignored and the result is unnamed.
}

See the \link[=name_spec]{name specification topic}.}

\item{.name_repair}{How to repair names, see \code{repair} options in
\code{\link[=vec_as_names]{vec_as_names()}}.}

\item{.error_arg}{An argument name as a string. This argument
will be mentioned in error messages as the input that is at the
origin of a problem.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A vector with class given by \code{.ptype}, and length equal to the
sum of the \code{vec_size()} of the contents of \code{...}.

The vector will have names if the individual components have names
(inner names) or if the arguments are named (outer names). If both
inner and outer names are present, an error is thrown unless a
\code{.name_spec} is provided.
}
\description{
Combine all arguments into a new vector of common type.
}
\section{Invariants}{

\itemize{
\item \code{vec_size(vec_c(x, y)) == vec_size(x) + vec_size(y)}
\item \code{vec_ptype(vec_c(x, y)) == vec_ptype_common(x, y)}.
}
}

\section{Dependencies}{

\subsection{vctrs dependencies}{
\itemize{
\item \code{\link[=vec_cast_common]{vec_cast_common()}} with fallback
\item \code{\link[=vec_proxy]{vec_proxy()}}
\item \code{\link[=vec_restore]{vec_restore()}}
}
}

\subsection{base dependencies}{
\itemize{
\item \code{\link[base:c]{base::c()}}
}

If inputs inherit from a common class hierarchy, \code{vec_c()} falls
back to \code{base::c()} if there exists a \code{c()} method implemented for
this class hierarchy.
}
}

\examples{
vec_c(FALSE, 1L, 1.5)

# Date/times --------------------------
c(Sys.Date(), Sys.time())
c(Sys.time(), Sys.Date())

vec_c(Sys.Date(), Sys.time())
vec_c(Sys.time(), Sys.Date())

# Factors -----------------------------
c(factor("a"), factor("b"))
vec_c(factor("a"), factor("b"))


# By default, named inputs must be length 1:
vec_c(name = 1)
try(vec_c(name = 1:3))

# Pass a name specification to work around this:
vec_c(name = 1:3, .name_spec = "{outer}_{inner}")

# See `?name_spec` for more examples of name specifications.
}
\seealso{
\code{\link[=vec_cbind]{vec_cbind()}}/\code{\link[=vec_rbind]{vec_rbind()}} for combining data frames by rows
or columns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/type-data-frame.R
\name{new_data_frame}
\alias{new_data_frame}
\title{Assemble attributes for data frame construction}
\usage{
new_data_frame(x = list(), n = NULL, ..., class = NULL)
}
\arguments{
\item{x}{A named list of equal-length vectors. The lengths are not
checked; it is responsibility of the caller to make sure they are
equal.}

\item{n}{Number of rows. If \code{NULL}, will be computed from the length of
the first element of \code{x}.}

\item{..., class}{Additional arguments for creating subclasses.

The following attributes have special behavior:
\itemize{
\item \code{"names"} is preferred if provided, overriding existing names in \code{x}.
\item \code{"row.names"} is preferred if provided, overriding both \code{n} and the size
implied by \code{x}.
}}
}
\description{
\code{new_data_frame()} constructs a new data frame from an existing list. It is
meant to be performant, and does not check the inputs for correctness in any
way. It is only safe to use after a call to \code{\link[=df_list]{df_list()}}, which collects and
validates the columns used to construct the data frame.
}
\examples{
new_data_frame(list(x = 1:10, y = 10:1))
}
\seealso{
\code{\link[=df_list]{df_list()}} for a way to safely construct a data frame's underlying
data structure from individual columns. This can be used to create a
named list for further use by \code{new_data_frame()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/faq-developer.R
\name{theory-faq-coercion}
\alias{theory-faq-coercion}
\title{FAQ - How does coercion work in vctrs?}
\description{
This is an overview of the usage of \code{vec_ptype2()} and \code{vec_cast()} and
their role in the vctrs coercion mechanism. Related topics:
\itemize{
\item For an example of implementing coercion methods for simple vectors,
see \code{\link[=howto-faq-coercion]{?howto-faq-coercion}}.
\item For an example of implementing coercion methods for data frame
subclasses, see
\code{\link[=howto-faq-coercion-data-frame]{?howto-faq-coercion-data-frame}}.
\item For a tutorial about implementing vctrs classes from scratch, see
\code{vignette("s3-vector")}.
}
\subsection{Combination mechanism in vctrs}{

The coercion system in vctrs is designed to make combination of multiple
inputs consistent and extensible. Combinations occur in many places,
such as row-binding, joins, subset-assignment, or grouped summary
functions that use the split-apply-combine strategy. For example:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_c(TRUE, 1)
#> [1] 1 1

vec_c("a", 1)
#> Error in `vec_c()`:
#> ! Can't combine `..1` <character> and `..2` <double>.

vec_rbind(
  data.frame(x = TRUE),
  data.frame(x = 1, y = 2)
)
#>   x  y
#> 1 1 NA
#> 2 1  2

vec_rbind(
  data.frame(x = "a"),
  data.frame(x = 1, y = 2)
)
#> Error in `vec_rbind()`:
#> ! Can't combine `..1$x` <character> and `..2$x` <double>.
}\if{html}{\out{</div>}}

One major goal of vctrs is to provide a central place for implementing
the coercion methods that make generic combinations possible. The two
relevant generics are \code{vec_ptype2()} and \code{vec_cast()}. They both take
two arguments and perform \strong{double dispatch}, meaning that a method is
selected based on the classes of both inputs.

The general mechanism for combining multiple inputs is:
\enumerate{
\item Find the common type of a set of inputs by reducing (as in
\code{base::Reduce()} or \code{purrr::reduce()}) the \code{vec_ptype2()} binary
function over the set.
\item Convert all inputs to the common type with \code{vec_cast()}.
\item Initialise the output vector as an instance of this common type with
\code{vec_init()}.
\item Fill the output vector with the elements of the inputs using
\code{vec_assign()}.
}

The last two steps may require \code{vec_proxy()} and \code{vec_restore()}
implementations, unless the attributes of your class are constant and do
not depend on the contents of the vector. We focus here on the first two
steps, which require \code{vec_ptype2()} and \code{vec_cast()} implementations.
}

\subsection{\code{vec_ptype2()}}{

Methods for \code{vec_ptype2()} are passed two \emph{prototypes}, i.e. two inputs
emptied of their elements. They implement two behaviours:
\itemize{
\item If the types of their inputs are compatible, indicate which of them is
the richer type by returning it. If the types are of equal resolution,
return any of the two.
\item Throw an error with \code{stop_incompatible_type()} when it can be
determined from the attributes that the types of the inputs are not
compatible.
}
\subsection{Type compatibility}{

A type is \strong{compatible} with another type if the values it represents
are a subset or a superset of the values of the other type. The notion
of “value” is to be interpreted at a high level, in particular it is not
the same as the memory representation. For example, factors are
represented in memory with integers but their values are more related to
character vectors than to round numbers:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{# Two factors are compatible
vec_ptype2(factor("a"), factor("b"))
#> factor()
#> Levels: a b

# Factors are compatible with a character
vec_ptype2(factor("a"), "b")
#> character(0)

# But they are incompatible with integers
vec_ptype2(factor("a"), 1L)
#> Error:
#> ! Can't combine `factor("a")` <factor<4d52a>> and `1L` <integer>.
}\if{html}{\out{</div>}}
}

\subsection{Richness of type}{

Richness of type is not a very precise notion. It can be about richer
data (for instance a \code{double} vector covers more values than an integer
vector), richer behaviour (a \code{data.table} has richer behaviour than a
\code{data.frame}), or both. If you have trouble determining which one of the
two types is richer, it probably means they shouldn’t be automatically
coercible.

Let’s look again at what happens when we combine a factor and a
character:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(factor("a"), "b")
#> character(0)
}\if{html}{\out{</div>}}

The ptype2 method for \verb{<character>} and \verb{<factor<"a">>} returns
\verb{<character>} because the former is a richer type. The factor can only
contain \code{"a"} strings, whereas the character can contain any strings. In
this sense, factors are a \emph{subset} of character.

Note that another valid behaviour would be to throw an incompatible type
error. This is what a strict factor implementation would do. We have
decided to be laxer in vctrs because it is easy to inadvertently create
factors instead of character vectors, especially with older versions of
R where \code{stringsAsFactors} is still true by default.
}

\subsection{Consistency and symmetry on permutation}{

Each ptype2 method should strive to have exactly the same behaviour when
the inputs are permuted. This is not always possible, for example factor
levels are aggregated in order:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{vec_ptype2(factor(c("a", "c")), factor("b"))
#> factor()
#> Levels: a c b

vec_ptype2(factor("b"), factor(c("a", "c")))
#> factor()
#> Levels: b a c
}\if{html}{\out{</div>}}

In any case, permuting the input should not return a fundamentally
different type or introduce an incompatible type error.
}

\subsection{Coercion hierarchy}{

The classes that you can coerce together form a coercion (or subtyping)
hierarchy. Below is a schema of the hierarchy for the base types like
integer and factor. In this diagram the directions of the arrows express
which type is richer. They flow from the bottom (more constrained types)
to the top (richer types).

\figure{coerce.png}

A coercion hierarchy is distinct from the structural hierarchy implied
by memory types and classes. For instance, in a structural hierarchy,
factors are built on top of integers. But in the coercion hierarchy they
are more related to character vectors. Similarly, subclasses are not
necessarily coercible with their superclasses because the coercion and
structural hierarchies are separate.
}

\subsection{Implementing a coercion hierarchy}{

As a class implementor, you have two options. The simplest is to create
an entirely separate hierarchy. The date and date-time classes are an
example of an S3-based hierarchy that is completely separate.
Alternatively, you can integrate your class in an existing hierarchy,
typically by adding parent nodes on top of the hierarchy (your class is
richer), by adding children node at the root of the hierarchy (your
class is more constrained), or by inserting a node in the tree.

These coercion hierarchies are \emph{implicit}, in the sense that they are
implied by the \code{vec_ptype2()} implementations. There is no structured
way to create or modify a hierarchy, instead you need to implement the
appropriate coercion methods for all the types in your hierarchy, and
diligently return the richer type in each case. The \code{vec_ptype2()}
implementations are not transitive nor inherited, so all pairwise
methods between classes lying on a given path must be implemented
manually. This is something we might make easier in the future.
}

}

\subsection{\code{vec_cast()}}{

The second generic, \code{vec_cast()}, is the one that looks at the data and
actually performs the conversion. Because it has access to more
information than \code{vec_ptype2()}, it may be stricter and cause an error
in more cases. \code{vec_cast()} has three possible behaviours:
\itemize{
\item Determine that the prototypes of the two inputs are not compatible.
This must be decided in exactly the same way as for \code{vec_ptype2()}.
Call \code{stop_incompatible_cast()} if you can determine from the
attributes that the types are not compatible.
\item Detect incompatible values. Usually this is because the target type is
too restricted for the values supported by the input type. For
example, a fractional number can’t be converted to an integer. The
method should throw an error in that case.
\item Return the input vector converted to the target type if all values are
compatible. Whereas \code{vec_ptype2()} must return the same type when the
inputs are permuted, \code{vec_cast()} is \emph{directional}. It always returns
the type of the right-hand side, or dies trying.
}
}

\subsection{Double dispatch}{

The dispatch mechanism for \code{vec_ptype2()} and \code{vec_cast()} looks like S3
but is actually a custom mechanism. Compared to S3, it has the following
differences:
\itemize{
\item It dispatches on the classes of the first two inputs.
\item There is no inheritance of ptype2 and cast methods. This is because
the S3 class hierarchy is not necessarily the same as the coercion
hierarchy.
\item \code{NextMethod()} does not work. Parent methods must be called explicitly
if necessary.
\item The default method is hard-coded.
}
}

\subsection{Data frames}{

The determination of the common type of data frames with \code{vec_ptype2()}
happens in three steps:
\enumerate{
\item Match the columns of the two input data frames. If some columns
don’t exist, they are created and filled with adequately typed \code{NA}
values.
\item Find the common type for each column by calling \code{vec_ptype2()} on
each pair of matched columns.
\item Find the common data frame type. For example the common type of a
grouped tibble and a tibble is a grouped tibble because the latter
is the richer type. The common type of a data table and a data frame
is a data table.
}

\code{vec_cast()} operates similarly. If a data frame is cast to a target
type that has fewer columns, this is an error.

If you are implementing coercion methods for data frames, you will need
to explicitly call the parent methods that perform the common type
determination or the type conversion described above. These are exported
as \code{\link[=df_ptype2]{df_ptype2()}} and \code{\link[=df_cast]{df_cast()}}.
\subsection{Data frame fallbacks}{

Being too strict with data frame combinations would cause too much pain
because there are many data frame subclasses in the wild that don’t
implement vctrs methods. We have decided to implement a special fallback
behaviour for foreign data frames. Incompatible data frames fall back to
a base data frame:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{df1 <- data.frame(x = 1)
df2 <- structure(df1, class = c("foreign_df", "data.frame"))

vec_rbind(df1, df2)
#>   x
#> 1 1
#> 2 1
}\if{html}{\out{</div>}}

When a tibble is involved, we fall back to tibble:

\if{html}{\out{<div class="sourceCode r">}}\preformatted{df3 <- tibble::as_tibble(df1)

vec_rbind(df1, df3)
#> # A tibble: 2 x 1
#>       x
#>   <dbl>
#> 1     1
#> 2     1
}\if{html}{\out{</div>}}

These fallbacks are not ideal but they make sense because all data
frames share a common data structure. This is not generally the case for
vectors. For example factors and characters have different
representations, and it is not possible to find a fallback time
mechanically.

However this fallback has a big downside: implementing vctrs methods for
your data frame subclass is a breaking behaviour change. The proper
coercion behaviour for your data frame class should be specified as soon
as possible to limit the consequences of changing the behaviour of your
class in R scripts.
}

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fields.R
\name{fields}
\alias{fields}
\alias{n_fields}
\alias{field}
\alias{field<-}
\title{Tools for accessing the fields of a record.}
\usage{
fields(x)

n_fields(x)

field(x, i)

field(x, i) <- value
}
\arguments{
\item{x}{A \link{rcrd}, i.e. a list of equal length vectors with unique names.}
}
\description{
A \link{rcrd} behaves like a vector, so \code{length()}, \code{names()}, and \code{$} can
not provide access to the fields of the underlying list. These helpers do:
\code{fields()} is equivalent to \code{names()}; \code{n_fields()} is equivalent to
\code{length()}; \code{field()} is equivalent to \code{$}.
}
\examples{
x <- new_rcrd(list(x = 1:3, y = 3:1, z = letters[1:3]))
n_fields(x)
fields(x)

field(x, "y")
field(x, "y") <- runif(3)
field(x, "y")
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_as_index}
\alias{vec_as_index}
\title{Convert to an index vector}
\usage{
vec_as_index(i, n, names = NULL)
}
\arguments{
\item{i}{An index vector to convert.}

\item{n}{A single integer representing the total size of the
object that \code{i} is meant to index into.}

\item{names}{If \code{i} is a character vector, \code{names} should be a character
vector that \code{i} will be matched against to construct the index. Otherwise,
not used. The default value of \code{NULL} will result in an error
if \code{i} is a character vector.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

\code{vec_as_index()} has been renamed to \code{\link[=vec_as_location]{vec_as_location()}} and is
deprecated as of vctrs 0.2.2.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/parallel.R
\name{parallel-operators}
\alias{parallel-operators}
\alias{vec_pany}
\alias{vec_pall}
\title{Parallel \code{any()} and \code{all()}}
\usage{
vec_pany(
  ...,
  .missing = NA,
  .size = NULL,
  .arg = "",
  .error_call = current_env()
)

vec_pall(
  ...,
  .missing = NA,
  .size = NULL,
  .arg = "",
  .error_call = current_env()
)
}
\arguments{
\item{...}{Logical vectors of equal size.}

\item{.missing}{Value to use when a missing value is encountered. One of:
\itemize{
\item \code{NA} to propagate missing values. With this, missings are treated the
same way as \code{|} or \code{&}.
\item \code{FALSE} to treat missing values as \code{FALSE}.
\item \code{TRUE} to treat missing values as \code{TRUE}.
}}

\item{.size}{An optional output size. Only useful to specify if it is possible
for no inputs to be provided.}

\item{.arg}{Argument name used in error messages.}

\item{.error_call}{The execution environment of a currently
running function, e.g. \code{caller_env()}. The function will be
mentioned in error messages as the source of the error. See the
\code{call} argument of \code{\link[rlang:abort]{abort()}} for more information.}
}
\value{
A logical vector the same size as the vectors in \code{...}.
}
\description{
These functions are variants of \code{\link[=any]{any()}} and \code{\link[=all]{all()}} that work in parallel on
multiple inputs at once. They work similarly to how \code{\link[=pmin]{pmin()}} and \code{\link[=pmax]{pmax()}} are
parallel variants of \code{\link[=min]{min()}} and \code{\link[=max]{max()}}.
}
\details{
\code{vec_pany()} and \code{vec_pall()} are consistent with \code{\link[=any]{any()}} and \code{\link[=all]{all()}} when
there are no inputs to process in parallel:
\itemize{
\item \code{any()} returns \code{FALSE} with no inputs. Similarly, \code{vec_pany(.size = 1)}
returns \code{FALSE}.
\item \code{all()} returns \code{TRUE} with no inputs. Similarly, \code{vec_pall(.size = 1)}
returns \code{TRUE}.
}
}
\examples{
a <- c(TRUE, TRUE, TRUE, FALSE, FALSE, FALSE, NA, NA, NA)
b <- c(TRUE, FALSE, NA, TRUE, FALSE, NA, TRUE, FALSE, NA)

# Default behavior treats missings like `|` does
vec_pany(a, b)
a | b

# Default behavior treats missings like `&` does
vec_pall(a, b)
a & b

# Remove missings from the computation, like `na_rm = TRUE`
vec_pany(a, b, .missing = FALSE)
(a & !is.na(a)) | (b & !is.na(b))

vec_pall(a, b, .missing = TRUE)
(a | is.na(a)) & (b | is.na(b))

# `vec_pall()` can be used to implement a `dplyr::filter()` style API
df <- data_frame(id = seq_along(a), a = a, b = b)

keep_rows <- function(x, ...) {
  vec_slice(x, vec_pall(..., .missing = FALSE))
}
drop_rows <- function(x, ...) {
  vec_slice(x, !vec_pall(..., .missing = FALSE))
}

# "Keep / Drop the rows when both a and b are TRUE"
# These form complements of one another, even with `NA`s.
keep_rows(df, a, b)
drop_rows(df, a, b)

# Same empty behavior as `any()` and `all()`
vec_pany(.size = 1)
any()

vec_pall(.size = 1)
all()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/runs.R
\name{runs}
\alias{runs}
\alias{vec_identify_runs}
\alias{vec_run_sizes}
\title{Runs}
\usage{
vec_identify_runs(x)

vec_run_sizes(x)
}
\arguments{
\item{x}{A vector.}
}
\value{
\itemize{
\item For \code{vec_identify_runs()}, an integer vector with the same size as \code{x}. A
scalar integer attribute, \code{n}, is attached.
\item For \code{vec_run_sizes()}, an integer vector with size equal to the number of
runs in \code{x}.
}
}
\description{
\itemize{
\item \code{vec_identify_runs()} returns a vector of identifiers for the elements of
\code{x} that indicate which run of repeated values they fall in. The number of
runs is also returned as an attribute, \code{n}.
\item \code{vec_run_sizes()} returns an integer vector corresponding to the size of
each run. This is identical to the \code{times} column from \code{vec_unrep()}, but
is faster if you don't need the run keys.
\item \code{\link[=vec_unrep]{vec_unrep()}} is a generalized \code{\link[base:rle]{base::rle()}}. It is documented alongside
the "repeat" functions of \code{\link[=vec_rep]{vec_rep()}} and \code{\link[=vec_rep_each]{vec_rep_each()}}; look there for
more information.
}
}
\details{
Unlike \code{\link[base:rle]{base::rle()}}, adjacent missing values are considered identical when
constructing runs. For example, \code{vec_identify_runs(c(NA, NA))} will return
\code{c(1, 1)}, not \code{c(1, 2)}.
}
\examples{
x <- c("a", "z", "z", "c", "a", "a")

vec_identify_runs(x)
vec_run_sizes(x)
vec_unrep(x)

y <- c(1, 1, 1, 2, 2, 3)

# With multiple columns, the runs are constructed rowwise
df <- data_frame(
  x = x,
  y = y
)

vec_identify_runs(df)
vec_run_sizes(df)
vec_unrep(df)
}
\seealso{
\code{\link[=vec_unrep]{vec_unrep()}} for a generalized \code{\link[base:rle]{base::rle()}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vctrs-deprecated.R
\name{vec_type}
\alias{vec_type}
\alias{vec_type_common}
\alias{vec_type2}
\title{Deprecated type functions}
\usage{
vec_type(x)

vec_type_common(..., .ptype = NULL)

vec_type2(x, y, ...)
}
\arguments{
\item{x, y, ..., .ptype}{Arguments for deprecated functions.}
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}

These functions have been renamed:
\itemize{
\item \code{vec_type()} => \code{\link[=vec_ptype]{vec_ptype()}}
\item \code{vec_type2()} => \code{\link[=vec_ptype2]{vec_ptype2()}}
\item \code{vec_type_common()} => \code{\link[=vec_ptype_common]{vec_ptype_common()}}
}
}
\keyword{internal}
